/*!
*************************************************************************
**
** \file    abst_common_priv.c
**
** \brief   abstraction library source file
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#else
#include <linux/ioctl.h>
#endif
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "abst_common_priv.h"


//#define FAPEX_SYS_PRV_DEBUG

#ifdef FAPEX_SYS_PRV_DEBUG
#define FAPEX_SYS_PRV_DMSG(fmt,...)    printf(fmt,##__VA_ARGS__)
#else
#define FAPEX_SYS_PRV_DMSG(fmt,...)
#endif

/*!
*******************************************************************************
**
** \brief  open method.
**         This function is called with the systemcall "open".
**
** \param  id     The device identifier.
** \param  flags  This parameter indicate a open mode (read only/write only/read-write).<br>
**                This implementation supports only read-write mode.
** \param  argptr argument pointer
** \param  cmd    The ioctl command.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXOPENRET_T fapex_COMMON_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags,
                                 FAPEXOPENARG3_T argptr, uint32_t cmd)
{
    FAPEXOPENRET_T  ret;

#ifndef _LINUX_FAPI
    id = FAPEXSAL_Open(id, flags);
    if(id >= 0) {
        ret = FAPEXSAL_Ioctl(id, cmd, (FAPEXPLARG1_T)argptr);
        if(ret < 0) {
            FAPEXSAL_Release(id);
            return ret;
        }
    }
#else /* !_LINUX_FAPI */
    FAPEXOPENARG1_T  tmp_id = id;
    char pathname[32];

    sprintf(pathname, "/dev/fapi/fapi%03d", tmp_id);
    id = open(pathname, flags);
    if(id >= 0) {
        FAPEX_SYS_PRV_DMSG("open %d OK[%d]\n", tmp_id, id);

        ret = ioctl(id, cmd, argptr);
        if(ret < 0) {
            FAPEX_SYS_PRV_DMSG("open cmd error[%d] \"%s\"\n",
                               id, strerror(errno));
            close(id);
            return ret;
        }
    }
    else {
        FAPEX_SYS_PRV_DMSG("open error \"%s\"\n",
                           strerror(errno));
    }
#endif /* !_LINUX_FAPI */

    return id;
}

/*!
*******************************************************************************
**
** \brief  ioctl method.
**         This function is called with the systemcall "ioctl".
**
** \param  id     The identifier for this descriptor.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
** \retval 0  process OK
** \retval <0 process fails, its value is set to errno.
**
******************************************************************************/
FAPEXIOCRET_T fapex_COMMON_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg)
{
    FAPEXIOCRET_T  ret = 0;
    int32_t        sub_id;

#ifndef _LINUX_FAPI
    if(_IOC_NR(cmd) >= FAPEX_IOCCOM_NB_SUBID) {
        sub_id = FAPEXSAL_Open(((FAPEX_COMMON_OpenSubIdParamStrT*)(pArg))->sub_id,
                               (O_RDWR | O_SYNC));
        if(sub_id >= 0) {
            ret = FAPEXSAL_Ioctl(sub_id, cmd, pArg);
            if(ret < 0) {
                FAPEXSAL_Release(sub_id);
                sub_id = -1;
            }
        }
        ((FAPEX_COMMON_OpenSubIdParamStrT*)(pArg))->sub_id = sub_id;
    }
    else if(cmd == FAPEX_IOCCOM_CLOSE_SUBID) {
        ret = FAPEXSAL_Ioctl(id, cmd, pArg);
        if(ret == 0) {
            ret = FAPEXSAL_Release(id);
        }
    }
    else {
        ret = FAPEXSAL_Ioctl(id, cmd, pArg);
    }
#else /* !_LINUX_FAPI */
    if(_IOC_NR(cmd) >= FAPEX_IOCCOM_NB_SUBID) {
        char pathname[32];

        sprintf(pathname, "/dev/fapi/fapi%03d",
                ((FAPEX_COMMON_OpenSubIdParamStrT*)(pArg))->sub_id);

        sub_id = open(pathname, (O_RDWR | O_SYNC));
        if(sub_id >= 0) {
            FAPEX_SYS_PRV_DMSG("open %d OK[%d]\n",
                               ((FAPEX_COMMON_OpenSubIdParamStrT*)(pArg))->sub_id,
                               sub_id);
            ret = ioctl(sub_id, cmd, pArg);
            if(ret < 0) {
                FAPEX_SYS_PRV_DMSG("open_sub cmd error[%d] \"%s\"\n",
                                   sub_id, strerror(errno));
                close(sub_id);
                sub_id = -1;
            }
        }
        else {
            FAPEX_SYS_PRV_DMSG("sub open error \"%s\"\n",
                               strerror(errno));
        }
        ((FAPEX_COMMON_OpenSubIdParamStrT*)(pArg))->sub_id = sub_id;
    }
    else if(cmd == FAPEX_IOCCOM_CLOSE_SUBID) {
        ret = ioctl(id, (int)cmd, pArg);
        if(ret == 0) {
            ret = close(id);
        }
        else {
            FAPEX_SYS_PRV_DMSG("sub close error[%d] \"%s\"\n",
                               id, strerror(errno));
        }
    }
    else {
        ret = ioctl(id, (int)cmd, pArg);
        if(ret < 0) {
            FAPEX_SYS_PRV_DMSG("ioctl(%08x) error[%d] \"%s\"\n",
                               cmd, id, strerror(errno));
        }
    }
#endif /* !_LINUX_FAPI */

    return ret;
}
