/*!
*************************************************************************
**
** \file    fapex/abst_libaudec.h
**
** \brief   abstraction library header file (Audio Decoder driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBAUDEC_H
#define _FAPEX_ABST_LIBAUDEC_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/

/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex AUDEC error code base value */
#define FAPEX_AUDEC_ERR_BASE              (FAPEX_AUDEC_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_AUDEC_ERR_BAD_PARAMETER     (FAPEX_AUDEC_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define FAPEX_AUDEC_ERR_OUT_OF_MEMORY     (FAPEX_AUDEC_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)

/*! Invalid status. */
#define FAPEX_AUDEC_ERR_INVALID_STATUS    (FAPEX_AUDEC_ERR_BASE + FAPEX_ERR_INVALID_HANDLE)

/*! Semaphore could not be created. */
#define FAPEX_AUDEC_ERR_SEMAPHORE_CREATE  (FAPEX_AUDEC_ERR_BASE + FAPEX_ERR_SEMAPHORE_CREATE)

/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/

/*!
*************************************************************************
** \brief open parameters for the AUDEC driver
**
** \sa FAPEX_AUDEC_Open
*************************************************************************/
typedef struct {
    /*!
        input Device minor number<br>
        (#FAPEXDEV_TSD_SUB_MINOR_FIRST..#FAPEXDEV_TSD_SUB_MINOR_LAST).
    */
    int32_t     inputId;
    /*!
        Audio codec type
        \param #FAPEX_AUDEC_CODEC_TYPE_UNDEF  Audio decoder undefined.
        \param #FAPEX_AUDEC_CODEC_TYPE_AC3    Dolby Digital 5.1(AC-3) audio decoder.
        \param #FAPEX_AUDEC_CODEC_TYPE_AC3SPDIF  Dolby Digital 5.1(AC-3) to SPDIF copier/synchronizer.
        \param #FAPEX_AUDEC_CODEC_TYPE_DD  Dolby Digital 5.1(AC-3) audio decoder.
        \param #FAPEX_AUDEC_CODEC_TYPE_DDPDCV   Dolby Digital Plus 5.1(E-/AC-3) decoder/converter.
        \param #FAPEX_AUDEC_CODEC_TYPE_HEAAC   HE-AAC 5.1 audio decoder.
        \param #FAPEX_AUDEC_CODEC_TYPE_MPEG  MPEG audio decoder.
        \param #FAPEX_AUDEC_CODEC_TYPE_MIXER MIX audio decoder
        \param #FAPEX_AUDEC_CODEC_TYPE_PCMOUTPUT PCM/WAV Copier.
        \param #FAPEX_AUDEC_CODEC_TYPE_WMA  WMA stereo audio decoder
    */
    int32_t     codecType;

    /*! --------------------
        for mixer decoder only
        NOTE: inputId will be mixer main input device id(a pcm buffer AUOUT device) when codecType is FAPEX_AUDEC_CODEC_TYPE_MIXER
      ---------------------*/
      /*!
        mixer sub input device id(a pcm buffer AUOUT device)
        */
      int32_t subInputDevId;
} FAPEX_AUDEC_OpenParamStrT;

#define FAPEX_AUDEC_CODEC_TYPE_UNDEF    (-1)    //!< Audio decoder undefined.
#define FAPEX_AUDEC_CODEC_TYPE_AC3      (0)     //!< Dolby Digital 5.1(AC-3) audio decoder.
#define FAPEX_AUDEC_CODEC_TYPE_AC3SPDIF (1)     //!< Dolby Digital 5.1(AC-3) to SPDIF copier/synchronizer.
#define FAPEX_AUDEC_CODEC_TYPE_DD       (2)     //!< Dolby Digital 5.1(AC-3) audio decoder.
#define FAPEX_AUDEC_CODEC_TYPE_DDPDCV   (3)     //!< Dolby Digital Plus 5.1(E-/AC-3) decoder/converter.
#define FAPEX_AUDEC_CODEC_TYPE_HEAAC    (4)     //!< HE-AAC 5.1 audio decoder
#define FAPEX_AUDEC_CODEC_TYPE_MIXER    (5)     //!< PCM Mixer utility
#define FAPEX_AUDEC_CODEC_TYPE_MPEG     (6)     //!< MPEG stereo audio decoder
#define FAPEX_AUDEC_CODEC_PCMOUTPUT     (7)     //!< PCM/WAV Copier
#define FAPEX_AUDEC_CODEC_TYPE_WMA      (8)     //!< WMA stereo audio decoder.
#define FAPEX_AUDEC_CODEC_TYPE_CUSTOMER (100)   //!< CUSTOMER specific audio decoder.

/*!
*************************************************************************
** \brief output parameters for the AUDEC driver
**
** The related ioctl command type :
** \ref audec_cmd_set_output "FAPEX_IOCCOM_AUDEC_SET_OUTPUT"
**
** \sa FAPEX_AUDEC_Ioctl
*************************************************************************/
typedef struct {
    /*!
        output Device minor number<br>
        (#FAPEXDEV_AUOUT_MINOR_FIRST..#FAPEXDEV_AUOUT_MINOR_LAST).
    */
    int32_t     outputId;
    /*!
        PCM sample bits
    */
    uint32_t    sampleBits;

} FAPEX_AUDEC_OutputParamStrT;

/*!
*************************************************************************
** \brief codec specific information for the AUDEC driver
**
** The related ioctl command type :
** \ref audec_cmd_set_codec_params "FAPEX_IOCCOM_AUDEC_SET_CODEC_PARAMS"
**
** \sa FAPEX_AUDEC_Ioctl
*************************************************************************/
typedef struct {
    /*!
        pointer to parameter
    */
    void        *params;
    /*!
        parameter length
    */
    uint32_t    length;

} FAPEX_AUDEC_CodecSpecificParamStrT;

/*!
*************************************************************************
** \brief decoder capabilities for the AUDEC driver
**
** The related ioctl command type :
** \ref audec_cmd_get_capabilities "FAPEX_IOCCOM_AUDEC_GET_CAPABILITIES"
**
** \sa FAPEX_AUDEC_Ioctl
*************************************************************************/
typedef struct {
    uint32_t  aacDecode:    1; //!< AAC-LC decoder availability flag.
    uint32_t  ac3Decode:    1; //!< AC3 decoder availability flag.
    uint32_t  mpgDecode:    1; //!< MPEG1/2 Layer 1-2 decoder availability flag.
    uint32_t  wmaDecode:    1; //!< WMA decoder availability flag.
    uint32_t  ddp71Decode:  1; //!< DDP 7.1 decoder availability flag.
    uint32_t  mp3Decode:    1; //!< MP3 decoder availability flag.
    uint32_t  heaacDecode:  1; //!< HE-AAC decoder availability flag.
    uint32_t  ddpdcvDecode: 1; //!< DDP Decoder Converter availability flag.

} FAPEX_AUDEC_CapabilitiesParamStrT;

/*!
*************************************************************************
** \brief decoder status information for the AUDEC driver
**
** The related ioctl command type :
** \ref audec_cmd_get_status "FAPEX_IOCCOM_GET_STATUS"
**
** \sa FAPEX_AUDEC_Ioctl
*************************************************************************/
typedef struct {
    /*!
        module status<br>
        \param #FAPEX_STATUS_OPENED  opened status
        \param #FAPEX_STATUS_READY   ready starts
        \param #FAPEX_STATUS_RUNNING running status
    */
    int32_t     status;
    /*!
        firmware status
        \param #FAPEX_AUDEC_FIRM_STATUS_RUNNING     Decoder is normally running.
        \param #FAPEX_AUDEC_FIRM_STATUS_IDLE        Decoder is waiting for start (after Open or Stop).
        \param #FAPEX_AUDEC_FIRM_STATUS_WAIT_INPUT  Decoder gets no more input data.
        \param #FAPEX_AUDEC_FIRM_STATUS_WAIT_OUTPUT Decoder cannot write to audio output.
        \param #FAPEX_AUDEC_FIRM_STATUS_ABORT       Decoder aborted due to fatal error.
        \param #FAPEX_AUDEC_FIRM_STATUS_NOT_RESPOND Decoder doesn't respond at all (maybe hanging).
        \param #FAPEX_AUDEC_FIRM_STATUS_RETRY       Decoder is retrying to decode after errors.
    */
    int32_t     firmStatus;

} FAPEX_AUDEC_StatusStrT;

#define FAPEX_AUDEC_FIRM_STATUS_RUNNING      0   //!< Decoder is normally running.
#define FAPEX_AUDEC_FIRM_STATUS_IDLE         1   //!< Decoder is waiting for start (after Open or Stop).
#define FAPEX_AUDEC_FIRM_STATUS_WAIT_INPUT   2   //!< Decoder gets no more input data.
#define FAPEX_AUDEC_FIRM_STATUS_WAIT_OUTPUT  3   //!< Decoder cannot write to audio output.
#define FAPEX_AUDEC_FIRM_STATUS_ABORT        4   //!< Decoder aborted due to fatal error.
#define FAPEX_AUDEC_FIRM_STATUS_NOT_RESPOND  5   //!< Decoder doesn't respond at all (maybe hanging).
#define FAPEX_AUDEC_FIRM_STATUS_RETRY        6   //!< Decoder is retrying to decode after errors.
#define FAPEX_AUDEC_FIRM_STATUS_DEC_NOT_RESPOND 	7   //!< Decoder doesn't respond at all (maybe hanging).
#define FAPEX_AUDEC_FIRM_STATUS_WRITE_PIPE_FAILED 8   //!< write pipe failed, need restart


/*!
*************************************************************************
** \brief ioctl specific command for the AUDEC driver
*************************************************************************/
#define FAPEX_IOCCOM_AUDEC_OPEN              _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUDEC + 0, FAPEX_AUDEC_OpenParamStrT)
#define FAPEX_IOCCOM_AUDEC_GET_STATUS        _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUDEC + 1, FAPEX_AUDEC_StatusStrT)
#define FAPEX_IOCCOM_AUDEC_SET_OUTPUT        _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUDEC + 2, FAPEX_AUDEC_OutputParamStrT)
#define FAPEX_IOCCOM_AUDEC_SET_CODEC_PARAMS  _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUDEC + 3, FAPEX_AUDEC_CodecSpecificParamStrT)
#define FAPEX_IOCCOM_AUDEC_GET_CAPABILITIES  _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUDEC + 4, FAPEX_AUDEC_CapabilitiesParamStrT)
#define FAPEX_IOCCOM_AUDEC_SET_OUTPUT_DELAY  _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUDEC + 5, uint32_t)
#define FAPEX_IOCCOM_AUDEC_SET_PTS	  		 _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUDEC + 6, uint64_t)


/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
extern "C" {
#endif

FAPEXOPENRET_T FAPEX_AUDEC_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T  FAPEX_AUDEC_Close(FAPEXRELARG1_T id);
FAPEXIOCRET_T  FAPEX_AUDEC_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBAUDEC_H */

