/*!
*************************************************************************
**
** \file    fapex/abst_libclkpwr.h
**
** \brief   abstraction library header file (CLKPWR driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBCLKPWR_H
#define _FAPEX_ABST_LIBCLKPWR_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/

/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex CLKPWR error code base value */
#define FAPEX_CLKPWR_ERR_BASE             (FAPEX_CLKPWR_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_CLKPWR_ERR_BAD_PARAMETER    (FAPEX_CLKPWR_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define FAPEX_CLKPWR_ERR_OUT_OF_MEMORY    (FAPEX_CLKPWR_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)

/*
*************************************************************************
** watchdog value
*************************************************************************/
/* watchdog settings */
#define FAPEX_CLKPWR_WATCHDOG_MIN_VALUE     0x00001000 //!< minimum value
#define FAPEX_CLKPWR_WATCHDOG_MAX_VALUE     0x0FFFF000 //!< maximum value

/* basic clock power settings */
#define FAPEX_CLKPWR_ARM                    (0x1ULL <<  0)
#define FAPEX_CLKPWR_AUDIO                  (0x1ULL <<  1)
#define FAPEX_CLKPWR_AUDIO_PROCESSOR        (0x1ULL <<  2)
#define FAPEX_CLKPWR_AXI                    (0x1ULL <<  3) // on power down - hardware reset required
#define FAPEX_CLKPWR_DDR2_SDRAM             (0x1ULL <<  4) // on power down - hardware reset required
#define FAPEX_CLKPWR_DMA                    (0x1ULL <<  5) // on power down - hardware reset required
#define FAPEX_CLKPWR_ETH                    (0x1ULL <<  6)
#define FAPEX_CLKPWR_I2C                    (0x1ULL <<  7)
#define FAPEX_CLKPWR_ICC_APB_EFUSE          (0x1ULL <<  8)
#define FAPEX_CLKPWR_IRQ_CONTROL            (0x1ULL <<  9) // on power down - hardware reset required
#define FAPEX_CLKPWR_GPIO                   (0x1ULL << 10)
#define FAPEX_CLKPWR_SFLASH                 (0x1ULL << 11)
#define FAPEX_CLKPWR_SSP                    (0x1ULL << 12)
#define FAPEX_CLKPWR_TSD                    (0x1ULL << 13)
#define FAPEX_CLKPWR_UART_FPC               (0x1ULL << 14)
#define FAPEX_CLKPWR_UPI                    (0x1ULL << 15)
#define FAPEX_CLKPWR_USB                    (0x1ULL << 16)
#define FAPEX_CLKPWR_VIDEO                  (0x1ULL << 17)
#define FAPEX_CLKPWR_CRITICAL               (0x1ULL << 18) // RAM and ARM (system stop)
#define FAPEX_CLKPWR_HDMAC                  (0x1ULL << 19)
#define FAPEX_CLKPWR_SDIO                   (0x1ULL << 20)
#define FAPEX_CLKPWR_SATA                   (0x1ULL << 21)
#define FAPEX_CLKPWR_R2R                    (0x1ULL << 22)
#define FAPEX_CLKPWR_OSD1                   (0x1ULL << 23)
#define FAPEX_CLKPWR_OSD2                   (0x1ULL << 24)
#define FAPEX_CLKPWR_OSD34                  (0x1ULL << 25)
#define FAPEX_CLKPWR_CURSOR                 (0x1ULL << 26)
#define FAPEX_CLKPWR_SD_SCALER              (0x1ULL << 27)
#define FAPEX_CLKPWR_ADC                    (0x1ULL << 28)
#define FAPEX_CLKPWR_BLTE                   (0x1ULL << 29)
#define FAPEX_CLKPWR_HDMI                   (0x1ULL << 30)
#define FAPEX_CLKPWR_DAC0                   (0x1ULL << 31)
#define FAPEX_CLKPWR_DAC1                   (0x1ULL << 32)
#define FAPEX_CLKPWR_DAC2                   (0x1ULL << 33)
#define FAPEX_CLKPWR_DAC3                   (0x1ULL << 34)
#define FAPEX_CLKPWR_DAC4                   (0x1ULL << 35)
#define FAPEX_CLKPWR_DAC5                   (0x1ULL << 36)
#define FAPEX_CLKPWR_HD_RGB                 (0x1ULL << 37)


/* power down settings */
/*! \brief Disables audio (also processor), tsd, video and reduces ARM speed to 162 MHz. */
#define FAPEX_CLKPWR_BASIC_POWER_DOWN       0x00022007
/*! \brief Additionally disables... */
#define FAPEX_CLKPWR_GENERIC_POWER_DOWN     0x0003FFF7
/*! \brief Disables all allowed modules. */
#define FAPEX_CLKPWR_UNIVERSAL_POWER_DOWN   0x0003FFF7 // hardware reset required

/* wake up settings */
#define FAPEX_CLKPWR_BASIC_WAKE_UP          0x00022007
#define FAPEX_CLKPWR_GENERIC_WAKE_UP        0x0003FFFF
#define FAPEX_CLKPWR_UNIVERSAL_WAKE_UP      0x0001FFC3

/* PWM settings */
#define FAPEX_CLKPWR_PWM_MAX_VALUE          0x000003FF //!< maximum value

/* Deep power down wakeup sources. */
#define FAPEX_CLKPWR_WAKEUP_SRC_CEC          0x01
#define FAPEX_CLKPWR_WAKEUP_SRC_IR           0x02
#define FAPEX_CLKPWR_WAKEUP_SRC_PIN          0x04
#define FAPEX_CLKPWR_WAKEUP_SRC_TIMER        0x08
#define FAPEX_CLKPWR_WAKEUP_SRC_ALL          0x0F

/* CEC wakeup commands. */
#define FAPEX_CLKPWR_CEC_CMD_USER_1              0x80
#define FAPEX_CLKPWR_CEC_CMD_USER_0              0x40
#define FAPEX_CLKPWR_CEC_CMD_ACT_SRC             0x20
#define FAPEX_CLKPWR_CEC_CMD_SET_STREAM_PATH     0x10
#define FAPEX_CLKPWR_CEC_CMD_AUDIO_MODE_REQUEST  0x08
#define FAPEX_CLKPWR_CEC_CMD_DECK_CTRL_CMD       0x04
#define FAPEX_CLKPWR_CEC_CMD_PLAY_CMD            0x02
#define FAPEX_CLKPWR_CEC_CMD_USER_CTRL_CMD       0x01
#define FAPEX_CLKPWR_CEC_CMD_ALL                 0xff
#define FAPEX_CLKPWR_CEC_CMD_ALL_STD             0x3F
#define FAPEX_CLKPWR_CEC_CMD_NONE                0x00
    

/*
*************************************************************************
** CLKPWR status
*************************************************************************/
#define FAPEX_CLKPWR_RUNNING    0   //!< State after init
#define FAPEX_CLKPWR_STAND_BY   1   //!< Stand by mode

/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/

/*!
*************************************************************************
** \brief pwm parameters for the CLKPWR driver
**
** The related ioctl command type :
** \ref clkpwr_cmd_get_pwm "FAPEX_IOCCOM_CLKPWR_GET_PWM"
** \ref clkpwr_cmd_set_pwm "FAPEX_IOCCOM_CLKPWR_SET_PWM"
**
** \sa FAPEX_CLKPWR_Ioctl
*************************************************************************/
#define FAPEX_CLKPWR_PWM_INDEX_0     0
#define FAPEX_CLKPWR_PWM_INDEX_1     1
#define FAPEX_CLKPWR_PWM_INDEX_2     2
#define FAPEX_CLKPWR_PWM_INDEX_3     3

typedef struct {
    /*!
        Index of the PWM module.Must be in the range from 
    FAPEX_CLKPWR_PWM_INDEX_0 to FAPEX_CLKPWR_PWM_INDEX_3. 
    */
    uint32_t   index;
    /*!
        Desired maximum value of the PWM counter 
    */
    uint32_t   pwmMaximum;
    /*!
        Maximum value of the PWM counter with output 0
    */
    uint32_t   pwmSetting;

} FAPEX_CLKPWR_PwmParamStrT;

/*!
*************************************************************************
** \brief Deep Power Down Configuration.
**
** This struct contains configurable parameters for the deep power
** down mode. 
**
*************************************************************************
*/
typedef struct {

    /*! \brief Sources for wakeup, FAPEX_CLKPWR_WAKEUP_SRC_XXX. */
    uint32_t wakeupSrc;

    /*! \brief Main chip power on/off. */
    /*! Set to zero to power off the whole chip except the power island.
        Set to a non-zero value to switch off the clock of the main
        chip, but to keep it powered on. */
    int32_t mainPowerOn;

    /*! \brief Configuration for wakeup triggered by CEC. */
    struct {
       
       /*! \brief User defined CEC wakeup command 0, low part. */
       uint32_t userCmd0Low;
       
       /*! \brief User defined CEC wakeup command 0, high part. */
       uint32_t userCmd0High;
       
       /*! \brief User defined CEC wakeup command 1, low part. */
       uint32_t userCmd1Low;
       
       /*! \brief User defined CEC wakeup command 1, high part. */
       uint32_t userCmd1High;
       
       /*! \brief Set of wakeup commands, FAPEX_CLKPWR_CEC_CMD_XXX. */
       uint32_t cmdSet;
    }
        cec;
      
    /*! \brief Configuration for wakeup triggered by IR interface. */
    /*! \note  The IR interface has to be configured to an appropriate
               hardware mode if wakeup by the IR interface is desired. */ 
    struct {
        
        /*! \brief Wakeup pattern. */
        uint32_t trigger;
        
        /*! \brief Mask for wakeup pattern. */
        uint32_t mask;
    }
        ir;

    /*! \brief Configuration for wakeup triggered by timer. */
    /*! The clock of the wakeup timer counter is derived from the system clock
        by a 25 bit prescaler. When the 22bit wakeup timer counter reaches the
        configured value, a wakeup signal is generated. */
    struct {
        
        /*! \brief 25bit prescaler value. */
        uint32_t prescaler;
        
        /*! \brief 22bit counter value. */
        uint32_t counter;
    }
        timer;
      
} FAPEX_CLKPWR_DeepPowerDownCfgT;


/*!
*************************************************************************
** \brief ioctl specific command for the CLKPWR driver
*************************************************************************/
#define FAPEX_IOCCOM_CLKPWR_START           _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_CLKPWR + 0, uint64_t)
#define FAPEX_IOCCOM_CLKPWR_STOP            _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_CLKPWR + 1, uint64_t)
#define FAPEX_IOCCOM_CLKPWR_GET_STATUS      _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_CLKPWR + 2, int32_t)
#define FAPEX_IOCCOM_CLKPWR_WATCHDOG_START  _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_CLKPWR + 3, uint32_t)
#define FAPEX_IOCCOM_CLKPWR_WATCHDOG_STOP   FAPEX_IOCCOM_STOP
#define FAPEX_IOCCOM_CLKPWR_WATCHDOG_RESET  FAPEX_IOCCOM_FORCE_RESET
#define FAPEX_IOCCOM_CLKPWR_SET_PWM         _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_CLKPWR + 4, FAPEX_CLKPWR_PwmParamStrT)
#define FAPEX_IOCCOM_CLKPWR_GET_PWM         _IOWR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_CLKPWR + 5, FAPEX_CLKPWR_PwmParamStrT)
#define	FAPEX_IOCCOM_CLKPWR_SET_DEEPMODE    _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_CLKPWR + 6, FAPEX_CLKPWR_DeepPowerDownCfgT)
#define FAPEX_IOCCOM_CLKPWR_GET_WAKEUPSRC   _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_CLKPWR + 7, uint32_t)
/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
}
#endif

FAPEXOPENRET_T FAPEX_CLKPWR_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T  FAPEX_CLKPWR_Close(FAPEXRELARG1_T id);
FAPEXIOCRET_T  FAPEX_CLKPWR_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBCLKPWR_H */

