/*!
*************************************************************************
**
** \file    fapex/abst_libdma.h
**
** \brief   abstraction library header file (DMA driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBDMA_H
#define _FAPEX_ABST_LIBDMA_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

    /*
    ****************************************************************************
    ****************************************************************************
    **
    ** Crypto feature specific functions and data types
    **
    ****************************************************************************
    ****************************************************************************
    */
    #define FAPEX_DMA_CRYPTO_K_KEY       0 //!< key from the DMA_CRYPTO_K register
    #define FAPEX_DMA_CRYPTO_SESSION_KEY 1 //!< session key, not supported!

    /*!
    ****************************************************************************
    ** \brief Version of the DMA driver.
    **
    ** This number consists of a \b major version number stored in bits
    ** 16 to 31 (upper word) and a \b minor version number stored in bits
    ** 0 to 15 (lower word).
    ** - The \b major number has to be increased when a parameter change
    **   occurs for the existing driver's API after its first release.
    ** - The \b minor number has to be increased when functions are added to
    **   the existing driver's API after its first release.
    **
    ** \note This value has to be assigend to the \b version field of the
    **       FAPI_DMA_OpenParamsT structure.
    **
    ****************************************************************************
    */
    typedef enum {
        FAPEX_DMA_VERSION = (int32_t)0x00020000   //!< The current driver version
    } FAPIEX_DMA_VersionEnumT;

    /*!
    ****************************************************************************
    **
    ** \brief DMA channel configuration parameter structure
    **
    ** This structure contains all settings required to configure a
    ** single DMA channel for a specific feature, this structure is
    ** used within the DMA request data structure.
    **
    ****************************************************************************
    */
    typedef struct
    {
        uint32_t chConfig;
        uint32_t chLength;
        uint32_t chLladdr;
        uint32_t chRdaddr_EndianSwap;
        uint32_t chRdaddr_PeripheralAddress;
        uint32_t chRdaddr_OffsetAddress;
        uint32_t chRdline_Lines;
        uint32_t chRdline_LoopIncrement;
        uint32_t chRdinc_LineLength;
        uint32_t chRdinc_LineIncrement;
        uint32_t chRdlpaddr_LoopAddress;
        uint32_t chWraddr_EndianSwap;
        uint32_t chWraddr_PeripheralAddress;
        uint32_t chWraddr_OffsetAddress;
        uint32_t chWrline_Lines;
        uint32_t chWrline_LoopIncrement;
        uint32_t chWrinc_LineLength;
        uint32_t chWrinc_LineIncrement;
        uint32_t chWrlpaddr_LoopAddress;
    }
    FAPEX_DMA_ChannelConfigT;

    /*!
    ****************************************************************************
    **
    ** \brief DMA callback function signature
    **
    ** This is the callback function signature required by the DMA driver
    ** for internally used pre and post functions.
    **
    ** \param request The request
    **
    ** \note functions of this signature will be called in interrupt
    **       context
    **
    ****************************************************************************
    */
    struct FAPEX_DMA_RequestS;
    typedef struct FAPEX_DMA_RequestS FAPEX_DMA_RequestT;
    typedef void (*FAPEX_DMA_CallbackT)(FAPEX_DMA_RequestT* damRequestPtr);

    /*!
    ****************************************************************************
    **
    ** \brief DMA callback function signature for user callbacks
    **
    ** This is the callback function signature required by the DMA
    ** driver used for user specific optional completion callbacks.
    **
    ** \param status  The FD status of the current feature operation
    ** \param bytes   The real number of bytes transfered by the channel
    **
    ** \note functions of this signature will be called in interrupt
    **       context
    **
    ****************************************************************************
    */
    typedef void (*FAPEX_DMA_OptCallbackT)(int32_t status,uint32_t bytes);

    /*!
    ****************************************************************************
    **
    ** \brief DMA request parameter structure
    **
    ** This structure contains all settings required for a single DMA
    ** request.
    **
    ****************************************************************************
    */
    typedef struct FAPEX_DMA_RequestS
    {
        // /*
        // ** The version of the driver.
        // ** \ref FAPI_DMA_VersionEnumT "FAPI_DMA_VERSION".
        // */
        // FAPI_DMA_VersionEnumT version;

        /*!
        ** The DMA feature to be used, this field will be set by the
        ** function FAPI_DMA_RequestAllocate().
        **
        ** \note You should not modify this field manually.
        */
        uint32_t feature;

        /*!
        ** The status of the current DMA request, this field will be set
        ** by the DMA interrupt service routine.
        **
        ** \note You should not modify this field manually.
        */
        uint32_t status;

        /*!
        ** The current DMA channel the request is running on
        **
        ** \note You should not modify this field manually.
        */
        uint32_t channel;

        /*!
        ** The specific result status of the DMA operation
        ** will be set to FAPI_OK if ok or FAPI_DMA_ERR_xxx in case of
        ** error, this field will be set by the DMA interrupt service
        ** routine.
        **
        ** \note You should not modify this field manually.
        */
        int32_t result;

        /*!
        ** This flag decides whether to destroy the current request
        ** structure automatically or not, if 0 the caller should
        ** destroy the request structure somewhere in the optFinshCallback
        ** function.
        **
        ** \note You should not modify this field manually.
        */
        uint32_t selfDestroy;

        /*!
        ** The DMA channel configuration data.
        */
        FAPEX_DMA_ChannelConfigT channelConfig;

        /*!
        ** the callback function to be called just before the DMA channel
        ** will be enabled by the DMA fifo, in case of the ATA feature
        ** this may be a pointer to a function preparing the ATA block
        ** for DMA access and to send the required ATA command to the
        ** IDE command register.
        */
        FAPEX_DMA_CallbackT preEnableCallback;

        /*!
        ** the callback function to be called just after the DMA channel
        ** has been enabled by the DMA fifo, in case of the ATA feature
        ** this may be a pointer to a function preparing the ATA block
        ** for DMA access and to send the required ATA command to the
        ** IDE command register.
        */
        FAPEX_DMA_CallbackT postEnableCallback;

        /*!
        ** the callback function to be called after the DMA channel
        ** has finished its transfer, in case of the ATA feature
        ** this may be a pointer to a function disabling the ATA block
        ** from DMA access.
        */
        FAPEX_DMA_CallbackT postFinishCallback;

        /*!
        ** A generic handle to be passed to all callback functions.
        */
        FAPEX_SYS_HandleT optHandle;

        /*!
        ** A void pointer to transport any addition optional data, as DMA
        ** request structure will be passed to each callback function, the
        ** optional data pointer can be used for any kind of feature specific
        ** data.
        */
        void* optData;

        /*!
        ** The callback function to be called if the transfer is completed,
        ** either if done OK or because of error.
        **
        */
        FAPEX_DMA_OptCallbackT optFinishCallback;

        /*!
        ** the callback function to be called if a change is detected
        ** in case of the search-and-replace engine this is the SearchHit
        ** callback in case of the CCIR engine this is the SyncChange callback.
        **
        ** \note This function is called in interrupt context.
        */
        FAPEX_DMA_OptCallbackT optChangeCallback;

        /*!
        ** This flag decides whether to automatically restart the DMA
        ** channel after having received the finish condition, or not,
        ** this feature is useful only for the DMA capture features
        ** #FAPI_DMA_FEATURE_CCIR656_CAPTURE and #FAPI_DMA_FEATURE_AUDIO_CAPTURE,
        ** internally this requires also the use of DMA channel#0 as this is
        ** the only channel capable for the auto-restart feature.
        **
        ** \note You should not modify this field manually, this flag should
        **       be set only by the function dmaRequestSetAutoRestart
        **       which is called internally by the CCIR and AUDIO capture
        **       prepare functions.
        */
        uint32_t autoRestart;

        /*!
        ** This number is used to split a DMA buffer into several parts.
        ** Each split buffer is used for DMA in a linked-list operation.
        */
        uint32_t optSplitCount;

        /*!
        ** The number of split buffers which finished DMA transfer.
        **
        ** \note You should not modify this field manually.
        */
        uint32_t completeSplitCount;

        /*!
        ** the callback function to be called if the transfer to a splited
        ** buffer is completed.
        **
        ** \note This function is called in interrupt context
        */
        FAPEX_DMA_OptCallbackT optSplitCallback;
    }
    FAPEX_DMA_RequestS;


    /*!
    ****************************************************************************
    **
    ** \brief 3des crypto request parameters
    **
    ** This structure contains all settings required for a single DMA
    ** 3des crypto request.
    **
    ****************************************************************************
    */
    typedef struct
    {
        /*!
        ** Controls key generation
        ** 1 means generate session key
        ** 0 means means do not generate session key
        ** this entry is currently not supported and will be ignored
        */
        uint32_t genSessionKey;

        /*!
        ** Selects which key to be used, supported value is
        ** #FAPI_DMA_CRYPTO_K_KEY. #FAPI_DMA_CRYPTO_SESSION_KEY is not
        ** supported!
        */
        uint32_t keySelection;

        /*!
        ** Controls endianness
        ** 1 means little endian
        ** 0 means big endian
        */
        uint32_t littleEndian;

        /*!
        ** Selects the TDES mode
        ** 1 means CBC
        ** 0 means ECB (without initial vector)
        */
        uint32_t cbc;

        /*!
        ** Selects the encryption/decryption mode
        ** 1 means decrypt
        ** 0 means encrypt
        */
        uint32_t decrypt;
    }
    FAPEX_DMA_CryptoParamsT;

    typedef struct
    {
        // keys and initialization vectors
        uint32_t key1Msb;  // Key 1 55:28 (31:25)(23:17)(15:9)(7:1)
        uint32_t key1Lsb;  // Key 1 27:0  (31:25)(23:17)(15:9)(7:1)
        uint32_t key2Msb;  // Key 2 55:28 (31:25)(23:17)(15:9)(7:1)
        uint32_t key2Lsb;  // Key 2 27:0  (31:25)(23:17)(15:9)(7:1)
        uint32_t key3Msb;  // Key 3 55:28 (31:25)(23:17)(15:9)(7:1)
        uint32_t key3Lsb;  // Key 3 27:0  (31:25)(23:17)(15:9)(7:1)
        uint32_t ivMsb;    // Initialization vector 63:32
        uint32_t ivLsb;    // Initialization vector 31:0
    }
    FAPEX_DMA_CryptoKeysT;


    #ifdef __cplusplus
    extern "C" {
    #endif

    FAPEX_DMA_RequestT* FAPEX_DMA_StartCrypto(
                           volatile uint32_t*      sourceBuffer,
                           volatile uint32_t*      targetBuffer,
                           uint32_t                bufferBytes,
                           FAPEX_DMA_CryptoParamsT* cryptoParams,
                           FAPEX_DMA_CryptoKeysT*   cryptoKeys,
                           FAPEX_DMA_OptCallbackT   finishCallback );

    #ifdef __cplusplus
    }
    #endif

#endif
