/*!
*************************************************************************
**
** \file    fapex/abst_libfpc.h
**
** \brief   abstraction library header file (FPC driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBFPC_H
#define _FAPEX_ABST_LIBFPC_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/

/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex FPC error code base value */
#define FAPEX_FPC_ERR_BASE              (FAPEX_FPC_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_FPC_ERR_BAD_PARAMETER     (FAPEX_FPC_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define FAPEX_FPC_ERR_OUT_OF_MEMORY     (FAPEX_FPC_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)

/*! Timeout occured. */
#define FAPEX_FPC_ERR_TIMEOUT           (FAPEX_FPC_ERR_BASE + FAPEX_ERR_TIMEOUT)

/*! Invalid status. */
#define FAPEX_FPC_ERR_INVALID_STATUS    (FAPEX_FPC_ERR_BASE + FAPEX_ERR_INVALID_HANDLE)


/*!
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/

/*!
*************************************************************************
** \brief display segment parameters for the FPC driver
**
** The related ioctl command type :
** \ref fpc_cmd_set_display_segment "FAPEX_IOCCOM_FPC_SET_DISPLAY_SEGMENT"
**
** \sa FAPEX_FPC_Ioctl
*************************************************************************/
typedef struct {
    /*!
        character word data for Digit0..3
    */
    uint8_t  data[4];
    /*!
        dot bits (bit3 for Digit0, bit2 for Digit1, ..)
    */
    uint8_t  dot;
    /*!
        setting mask bits (bit3 for Digit0, bit2 for Digit1, ..)
    */
    uint8_t  mask;

} FAPEX_FPC_DispSegParamStrT;

/*!
*************************************************************************
** \brief segment Digit parameters for the FPC driver
**
** The related ioctl command type :
** \ref fpc_cmd_set_digit "FAPEX_IOCCOM_FPC_SET_DIGIT",
** \ref fpc_cmd_get_digit "FAPEX_IOCCOM_FPC_GET_DIGIT",
** \ref fpc_cmd_set_increment_digit "FAPEX_IOCCOM_FPC_SET_INCREMENT_DIGIT",
** \ref fpc_cmd_set_decrement_digit "FAPEX_IOCCOM_FPC_SET_DECREMENT_DIGIT"
**
** \sa FAPEX_FPC_Ioctl
*************************************************************************/
typedef struct {
    /*!
        Digit position (0..3)
    */
    uint32_t  position;
    /*!
        Digit value (0..9)
    */
    int32_t   value;
    /*!
        carry up/down flag (return value, 0:none / 0>:up / 0<:down)
    */
    int32_t   carry;

} FAPEX_FPC_DigitParamStrT;

/*!
*************************************************************************
** \brief segment Dot parameters for the FPC driver
**
** The related ioctl command type :
** \ref fpc_cmd_get_dot "FAPEX_IOCCOM_FPC_GET_DOT"
**
** \sa FAPEX_FPC_Ioctl
*************************************************************************/
typedef struct {
    /*!
        Dot position (0..3)
    */
    uint32_t  position;
    /*!
        Dot activ flag
    */
    int32_t   actFlag;

} FAPEX_FPC_DotParamStrT;

/*!
*************************************************************************
** \brief message parameters for the FPC driver
**
** The related ioctl command type :
** \ref fpc_cmd_set_scroll_text "FAPEX_IOCCOM_FPC_SET_SCROLL_TEXT"
**
** \sa FAPEX_FPC_Ioctl
*************************************************************************/
typedef struct {
    /*!
        pointer of message data
    */
    uint8_t   *message;
    /*!
        message size include last '\\0'
    */
    uint32_t  size;
    /*!
        Time delay between the scroll steps [msec]
    */
    int32_t   delay;

} FAPEX_FPC_MessageParamStrT;

/*!
*************************************************************************
** \brief button event type for the FPC driver
**
** The related ioctl command type :
** \ref fpc_cmd_get_button_notify "FAPEX_IOCCOM_FPC_GET_BUTTON_NOTIFY"
**
** \sa FAPEX_FPC_Ioctl
*************************************************************************/
#define FAPEX_FPC_EVENT_NONE     (0x0000)    //!< none event
#define FAPEX_FPC_EVENT_BUTTON1  (0x0001)    //!< button1 event
#define FAPEX_FPC_EVENT_BUTTON2  (0x0002)    //!< button2 event
#define FAPEX_FPC_EVENT_BUTTON3  (0x0004)    //!< button3 event
#define FAPEX_FPC_EVENT_BUTTON4  (0x0008)    //!< button4 event
#define FAPEX_FPC_EVENT_BUTTON5  (0x0010)    //!< button5 event
#define FAPEX_FPC_EVENT_BUTTON6  (0x0020)    //!< button6 event
#define FAPEX_FPC_EVENT_BUTTON7  (0x0040)    //!< button7 event
#define FAPEX_FPC_EVENT_BUTTON8  (0x0080)    //!< button8 event
#define FAPEX_FPC_EVENT_END      (0x8000)    //!< end event


/*!
*************************************************************************
** \brief ioctl specific command for the FPC driver
*************************************************************************/
#define FAPEX_IOCCOM_FPC_SET_BRIGHTNESS              _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FPC + 0, uint32_t)
#define FAPEX_IOCCOM_FPC_SET_CAHNGE_BRIGHTNESS       _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FPC + 1, int32_t)
#define FAPEX_IOCCOM_FPC_SET_DISPLAY_SEGMENT         _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FPC + 2, FAPEX_FPC_DispSegParamStrT)
#define FAPEX_IOCCOM_FPC_SET_DIGIT                   _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FPC + 3, FAPEX_FPC_DigitParamStrT)
#define FAPEX_IOCCOM_FPC_GET_DIGIT                   _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FPC + 3, FAPEX_FPC_DigitParamStrT)
#define FAPEX_IOCCOM_FPC_SET_INCREMENT_DIGIT         _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FPC + 4, FAPEX_FPC_DigitParamStrT)
#define FAPEX_IOCCOM_FPC_SET_DECREMENT_DIGIT         _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FPC + 5, FAPEX_FPC_DigitParamStrT)
#define FAPEX_IOCCOM_FPC_GET_DOT                     _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FPC + 6, FAPEX_FPC_DotParamStrT)
#define FAPEX_IOCCOM_FPC_SET_DOT_TOGGLE              _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FPC + 7, uint32_t)
#define FAPEX_IOCCOM_FPC_SET_INTEGER_VALUE           _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FPC + 8, int32_t)
#define FAPEX_IOCCOM_FPC_GET_INTEGER_VALUE           _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FPC + 8, int32_t)
#define FAPEX_IOCCOM_FPC_SET_INCREMENT_INTEGER_VALUE _IO (FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FPC + 9)
#define FAPEX_IOCCOM_FPC_SET_DECREMENT_INTEGER_VALUE _IO (FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FPC +10)
#define FAPEX_IOCCOM_FPC_SET_SCROLL_TEXT             _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FPC +11, FAPEX_FPC_MessageParamStrT)
#define FAPEX_IOCCOM_FPC_GET_BUTTON_NOTIFY           _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FPC +12, int32_t)


/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
extern "C" {
#endif

FAPEXOPENRET_T FAPEX_FPC_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T  FAPEX_FPC_Close(FAPEXRELARG1_T id);
FAPEXIOCRET_T  FAPEX_FPC_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBFPC_H */

