/*!
*************************************************************************
**
** \file    fapex/abst_libtsd.h
**
** \brief   abstraction library header file (TSD driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBTSD_H
#define _FAPEX_ABST_LIBTSD_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/
#define FAPEX_TSD0                   0    //!< TSD0 hardware block
#define FAPEX_TSD1                   1    //!< TSD1 hardware block
#define FAPEX_TSD2                   2    //!< TSD2 hardware block
#define FAPEX_TSD3                   3    //!< TSD3 hardware block

#define FAPEX_TSD_MAX                4    //!< Total of TSD HW blocks

#define FAPEX_TSD_CHANNEL_AUTODETECT -1    //!< Selects the first free channel
#define FAPEX_TS_DISABLE              9    //!< No TS input (feed via packet RAM)

#define FAPEX_TSD_IGNORE_BYTE        0x00000000 //!< Ignore provided section byte
#define FAPEX_TSD_FILTER_BYTE        0xFFFFFFFF //!< Filter exactly provided section byte
#define FAPEX_TSD_IGNORE_VERSION     0xFF000000 //!< Ignore the version when filtering
#define FAPEX_TSD_CHECK_VERSION      0x00FF0000 //!< Check the version when filtering

#define FAPEX_SYNC_INVALIDATE  0xFFFFFFFF       //!< Invalidate the PCR PID

/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex TSD error code base value */
#define FAPEX_TSD_ERR_BASE              (FAPEX_TSD_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_TSD_ERR_BAD_PARAMETER     (FAPEX_TSD_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define FAPEX_TSD_ERR_OUT_OF_MEMORY     (FAPEX_TSD_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)

/*! Timeout occured. */
#define FAPEX_TSD_ERR_TIMEOUT           (FAPEX_TSD_ERR_BASE + FAPEX_ERR_TIMEOUT)

/*! Invalid status. */
#define FAPEX_TSD_ERR_INVALID_STATUS    (FAPEX_TSD_ERR_BASE + FAPEX_ERR_INVALID_HANDLE)

/*! Semaphore could not be created. */
#define FAPEX_TSD_ERR_SEMAPHORE_CREATE  (FAPEX_TSD_ERR_BASE + FAPEX_ERR_SEMAPHORE_CREATE)


/*!
*************************************************************************
** section filter matching bytes
*************************************************************************/
#define FAPEX_TSD_FILTER_SIZE    (16)

/*!
*************************************************************************
** \brief Collection of sections that can be filtered by the TSD section
**        filter. The PID value needs to be set to one of these.
*************************************************************************
*/
typedef enum {
    FAPEX_TSD_PAT_SECTION_PID     = (int32_t)0x0000, //!< Program association section
    FAPEX_TSD_CAT_SECTION_PID     = (int32_t)0x0001, //!< Conditional access section
    FAPEX_TSD_TSDT_SECTION_PID    = (int32_t)0x0002, //!< Transport stream description section
    FAPEX_TSD_PMT_MIN_SECTION_PID = (int32_t)0x0010, //!< Transport stream program map section
    FAPEX_TSD_PMT_MAX_SECTION_PID = (int32_t)0x1FFE, //!< Transport stream program map section
    FAPEX_TSD_NIT_SECTION_PID     = (int32_t)0x0010, //!< Network information section
    FAPEX_TSD_SDT_SECTION_PID     = (int32_t)0x0011, //!< Service description table
    FAPEX_TSD_EIT_SECTION_PID     = (int32_t)0x0012, //!< Event information section
    FAPEX_TSD_TDT_SECTION_PID     = (int32_t)0x0014, //!< Time and date section
    FAPEX_TSD_BAT_SECTION_PID     = (int32_t)0x0011, //!< Bouquet association section
    FAPEX_TSD_RST_SECTION_PID     = (int32_t)0x0013, //!< Running status section
    FAPEX_TSD_TOT_SECTION_PID     = (int32_t)0x0014, //!< Time offset section
    FAPEX_TSD_ST_MIN_SECTION_PID  = (int32_t)0x0010, //!< Stuffing section
    FAPEX_TSD_ST_MAX_SECTION_PID  = (int32_t)0x0014, //!< Stuffing section
    FAPEX_TSD_DIT_SECTION_PID     = (int32_t)0x0000, //!< Disconuity information section
    FAPEX_TSD_SIT_SECTION_PID     = (int32_t)0x0000  //!< Selection information section

} FAPEX_TSD_SectionPidT;

/*!
*************************************************************************
** \brief Collection of tables that can be filtered by the TSD section
**        filter.
*************************************************************************
*/
typedef enum {
    FAPEX_TSD_PAT_TABLE          = (int32_t)0x00, //!< Program association section
    FAPEX_TSD_CAT_TABLE          = (int32_t)0x01, //!< Conditional access section
    FAPEX_TSD_PMT_TABLE          = (int32_t)0x02, //!< Transport stream program map section
    FAPEX_TSD_TSDT_TABLE         = (int32_t)0x03, //!< Transport stream description section
    FAPEX_TSD_NIT_ATS_TABLE      = (int32_t)0x40, //!< Network information section (actual transport stream)
    FAPEX_TSD_NIT_OTS_TABLE      = (int32_t)0x41, //!< Network information section (other transport stream)
    FAPEX_TSD_SDT_ATS_TABLE      = (int32_t)0x42, //!< Service description table (actual transport stream)
    FAPEX_TSD_SDT_OTS_TABLE      = (int32_t)0x46, //!< Service description table (other transport stream)
    FAPEX_TSD_EIT_ATS_PF_TABLE   = (int32_t)0x4E, //!< Event information section (actual transport stream / present following)
    FAPEX_TSD_EIT_OTS_PF_TABLE   = (int32_t)0x4F, //!< Event information section (other transport stream / present following)
    FAPEX_TSD_BAT_TABLE          = (int32_t)0x4A, //!< Bouquet association section
    FAPEX_TSD_TDT_TABLE          = (int32_t)0x70, //!< Time and date section
    FAPEX_TSD_RST_TABLE          = (int32_t)0x71, //!< Running status section
    FAPEX_TSD_ST_TABLE           = (int32_t)0x72, //!< Stuffing section
    FAPEX_TSD_TOT_TABLE          = (int32_t)0x73, //!< Time offset section
    FAPEX_TSD_DIT_TABLE          = (int32_t)0x7E, //!< Disconuity information section
    FAPEX_TSD_SIT_TABLE          = (int32_t)0x7F, //!< Selection information section
    FAPEX_TSD_USER_000_TABLE     = (int32_t)0x80, //!< User defined table 000
    FAPEX_TSD_USER_001_TABLE     = (int32_t)0x81, //!< User defined table 001
    FAPEX_TSD_USER_002_TABLE     = (int32_t)0x82, //!< User defined table 002
    FAPEX_TSD_USER_003_TABLE     = (int32_t)0x83, //!< User defined table 003
    FAPEX_TSD_USER_004_TABLE     = (int32_t)0x84, //!< User defined table 004
    FAPEX_TSD_USER_005_TABLE     = (int32_t)0x85, //!< User defined table 005
    FAPEX_TSD_USER_006_TABLE     = (int32_t)0x86, //!< User defined table 006
    FAPEX_TSD_USER_007_TABLE     = (int32_t)0x87, //!< User defined table 007
    FAPEX_TSD_USER_008_TABLE     = (int32_t)0x88, //!< User defined table 008
    FAPEX_TSD_USER_009_TABLE     = (int32_t)0x89, //!< User defined table 009
    FAPEX_TSD_USER_010_TABLE     = (int32_t)0x8A, //!< User defined table 010
    FAPEX_TSD_USER_011_TABLE     = (int32_t)0x8B, //!< User defined table 011
    FAPEX_TSD_USER_012_TABLE     = (int32_t)0x8C, //!< User defined table 012
    FAPEX_TSD_USER_013_TABLE     = (int32_t)0x8D, //!< User defined table 013
    FAPEX_TSD_USER_014_TABLE     = (int32_t)0x8E, //!< User defined table 014
    FAPEX_TSD_USER_015_TABLE     = (int32_t)0x8F, //!< User defined table 015
    FAPEX_TSD_USER_016_TABLE     = (int32_t)0x90, //!< User defined table 016
    FAPEX_TSD_USER_017_TABLE     = (int32_t)0x91, //!< User defined table 017
    FAPEX_TSD_USER_018_TABLE     = (int32_t)0x92, //!< User defined table 018
    FAPEX_TSD_USER_019_TABLE     = (int32_t)0x93, //!< User defined table 019
    FAPEX_TSD_USER_020_TABLE     = (int32_t)0x94, //!< User defined table 020
    FAPEX_TSD_USER_021_TABLE     = (int32_t)0x95, //!< User defined table 021
    FAPEX_TSD_USER_022_TABLE     = (int32_t)0x96, //!< User defined table 022
    FAPEX_TSD_USER_023_TABLE     = (int32_t)0x97, //!< User defined table 023
    FAPEX_TSD_USER_024_TABLE     = (int32_t)0x98, //!< User defined table 024
    FAPEX_TSD_USER_025_TABLE     = (int32_t)0x99, //!< User defined table 025
    FAPEX_TSD_USER_026_TABLE     = (int32_t)0x9A, //!< User defined table 026
    FAPEX_TSD_USER_027_TABLE     = (int32_t)0x9B, //!< User defined table 027
    FAPEX_TSD_USER_028_TABLE     = (int32_t)0x9C, //!< User defined table 028
    FAPEX_TSD_USER_029_TABLE     = (int32_t)0x9D, //!< User defined table 029
    FAPEX_TSD_USER_030_TABLE     = (int32_t)0x9E, //!< User defined table 030
    FAPEX_TSD_USER_031_TABLE     = (int32_t)0x9F, //!< User defined table 031
    FAPEX_TSD_USER_032_TABLE     = (int32_t)0xA0, //!< User defined table 032
    FAPEX_TSD_USER_033_TABLE     = (int32_t)0xA1, //!< User defined table 033
    FAPEX_TSD_USER_034_TABLE     = (int32_t)0xA2, //!< User defined table 034
    FAPEX_TSD_USER_035_TABLE     = (int32_t)0xA3, //!< User defined table 035
    FAPEX_TSD_USER_036_TABLE     = (int32_t)0xA4, //!< User defined table 036
    FAPEX_TSD_USER_037_TABLE     = (int32_t)0xA5, //!< User defined table 037
    FAPEX_TSD_USER_038_TABLE     = (int32_t)0xA6, //!< User defined table 038
    FAPEX_TSD_USER_039_TABLE     = (int32_t)0xA7, //!< User defined table 039
    FAPEX_TSD_USER_040_TABLE     = (int32_t)0xA8, //!< User defined table 040
    FAPEX_TSD_USER_041_TABLE     = (int32_t)0xA9, //!< User defined table 041
    FAPEX_TSD_USER_042_TABLE     = (int32_t)0xAA, //!< User defined table 042
    FAPEX_TSD_USER_043_TABLE     = (int32_t)0xAB, //!< User defined table 043
    FAPEX_TSD_USER_044_TABLE     = (int32_t)0xAC, //!< User defined table 044
    FAPEX_TSD_USER_045_TABLE     = (int32_t)0xAD, //!< User defined table 045
    FAPEX_TSD_USER_046_TABLE     = (int32_t)0xAE, //!< User defined table 046
    FAPEX_TSD_USER_047_TABLE     = (int32_t)0xAF, //!< User defined table 047
    FAPEX_TSD_USER_048_TABLE     = (int32_t)0xB0, //!< User defined table 048
    FAPEX_TSD_USER_049_TABLE     = (int32_t)0xB1, //!< User defined table 049
    FAPEX_TSD_USER_051_TABLE     = (int32_t)0xB2, //!< User defined table 051
    FAPEX_TSD_USER_052_TABLE     = (int32_t)0xB3, //!< User defined table 052
    FAPEX_TSD_USER_053_TABLE     = (int32_t)0xB4, //!< User defined table 053
    FAPEX_TSD_USER_054_TABLE     = (int32_t)0xB5, //!< User defined table 054
    FAPEX_TSD_USER_055_TABLE     = (int32_t)0xB6, //!< User defined table 055
    FAPEX_TSD_USER_056_TABLE     = (int32_t)0xB7, //!< User defined table 056
    FAPEX_TSD_USER_057_TABLE     = (int32_t)0xB8, //!< User defined table 057
    FAPEX_TSD_USER_058_TABLE     = (int32_t)0xB9, //!< User defined table 058
    FAPEX_TSD_USER_059_TABLE     = (int32_t)0xBA, //!< User defined table 059
    FAPEX_TSD_USER_060_TABLE     = (int32_t)0xBB, //!< User defined table 060
    FAPEX_TSD_USER_061_TABLE     = (int32_t)0xBC, //!< User defined table 061
    FAPEX_TSD_USER_062_TABLE     = (int32_t)0xBD, //!< User defined table 062
    FAPEX_TSD_USER_063_TABLE     = (int32_t)0xBE, //!< User defined table 063
    FAPEX_TSD_USER_064_TABLE     = (int32_t)0xBF, //!< User defined table 064
    FAPEX_TSD_USER_065_TABLE     = (int32_t)0xC0, //!< User defined table 065
    FAPEX_TSD_USER_066_TABLE     = (int32_t)0xC1, //!< User defined table 066
    FAPEX_TSD_USER_067_TABLE     = (int32_t)0xC2, //!< User defined table 067
    FAPEX_TSD_USER_068_TABLE     = (int32_t)0xC3, //!< User defined table 068
    FAPEX_TSD_USER_069_TABLE     = (int32_t)0xC4, //!< User defined table 069
    FAPEX_TSD_USER_070_TABLE     = (int32_t)0xC5, //!< User defined table 070
    FAPEX_TSD_USER_071_TABLE     = (int32_t)0xC6, //!< User defined table 071
    FAPEX_TSD_USER_072_TABLE     = (int32_t)0xC7, //!< User defined table 072
    FAPEX_TSD_USER_073_TABLE     = (int32_t)0xC8, //!< User defined table 073
    FAPEX_TSD_USER_074_TABLE     = (int32_t)0xC9, //!< User defined table 074
    FAPEX_TSD_USER_075_TABLE     = (int32_t)0xCA, //!< User defined table 075
    FAPEX_TSD_USER_076_TABLE     = (int32_t)0xCB, //!< User defined table 076
    FAPEX_TSD_USER_077_TABLE     = (int32_t)0xCC, //!< User defined table 077
    FAPEX_TSD_USER_078_TABLE     = (int32_t)0xCD, //!< User defined table 078
    FAPEX_TSD_USER_079_TABLE     = (int32_t)0xCE, //!< User defined table 079
    FAPEX_TSD_USER_080_TABLE     = (int32_t)0xCF, //!< User defined table 080
    FAPEX_TSD_USER_081_TABLE     = (int32_t)0xD0, //!< User defined table 081
    FAPEX_TSD_USER_082_TABLE     = (int32_t)0xD1, //!< User defined table 082
    FAPEX_TSD_USER_083_TABLE     = (int32_t)0xD2, //!< User defined table 083
    FAPEX_TSD_USER_084_TABLE     = (int32_t)0xD3, //!< User defined table 084
    FAPEX_TSD_USER_085_TABLE     = (int32_t)0xD4, //!< User defined table 085
    FAPEX_TSD_USER_086_TABLE     = (int32_t)0xD5, //!< User defined table 086
    FAPEX_TSD_USER_087_TABLE     = (int32_t)0xD6, //!< User defined table 087
    FAPEX_TSD_USER_088_TABLE     = (int32_t)0xD7, //!< User defined table 088
    FAPEX_TSD_USER_089_TABLE     = (int32_t)0xD8, //!< User defined table 089
    FAPEX_TSD_USER_090_TABLE     = (int32_t)0xD9, //!< User defined table 090
    FAPEX_TSD_USER_091_TABLE     = (int32_t)0xDA, //!< User defined table 091
    FAPEX_TSD_USER_092_TABLE     = (int32_t)0xDB, //!< User defined table 092
    FAPEX_TSD_USER_093_TABLE     = (int32_t)0xDC, //!< User defined table 093
    FAPEX_TSD_USER_094_TABLE     = (int32_t)0xDD, //!< User defined table 094
    FAPEX_TSD_USER_095_TABLE     = (int32_t)0xDE, //!< User defined table 095
    FAPEX_TSD_USER_096_TABLE     = (int32_t)0xDF, //!< User defined table 096
    FAPEX_TSD_USER_097_TABLE     = (int32_t)0xE0, //!< User defined table 097
    FAPEX_TSD_USER_098_TABLE     = (int32_t)0xE1, //!< User defined table 098
    FAPEX_TSD_USER_099_TABLE     = (int32_t)0xE2, //!< User defined table 099
    FAPEX_TSD_USER_100_TABLE     = (int32_t)0xE3, //!< User defined table 100
    FAPEX_TSD_USER_101_TABLE     = (int32_t)0xE4, //!< User defined table 101
    FAPEX_TSD_USER_102_TABLE     = (int32_t)0xE5, //!< User defined table 102
    FAPEX_TSD_USER_103_TABLE     = (int32_t)0xE6, //!< User defined table 103
    FAPEX_TSD_USER_104_TABLE     = (int32_t)0xE7, //!< User defined table 104
    FAPEX_TSD_USER_105_TABLE     = (int32_t)0xE8, //!< User defined table 105
    FAPEX_TSD_USER_106_TABLE     = (int32_t)0xE9, //!< User defined table 106
    FAPEX_TSD_USER_107_TABLE     = (int32_t)0xEA, //!< User defined table 107
    FAPEX_TSD_USER_108_TABLE     = (int32_t)0xEB, //!< User defined table 108
    FAPEX_TSD_USER_109_TABLE     = (int32_t)0xEC, //!< User defined table 109
    FAPEX_TSD_USER_110_TABLE     = (int32_t)0xED, //!< User defined table 110
    FAPEX_TSD_USER_111_TABLE     = (int32_t)0xEE, //!< User defined table 111
    FAPEX_TSD_USER_112_TABLE     = (int32_t)0xEF, //!< User defined table 112
    FAPEX_TSD_USER_113_TABLE     = (int32_t)0xF0, //!< User defined table 113
    FAPEX_TSD_USER_114_TABLE     = (int32_t)0xF1, //!< User defined table 114
    FAPEX_TSD_USER_115_TABLE     = (int32_t)0xF2, //!< User defined table 115
    FAPEX_TSD_USER_116_TABLE     = (int32_t)0xF3, //!< User defined table 116
    FAPEX_TSD_USER_117_TABLE     = (int32_t)0xF4, //!< User defined table 117
    FAPEX_TSD_USER_118_TABLE     = (int32_t)0xF5, //!< User defined table 118
    FAPEX_TSD_USER_119_TABLE     = (int32_t)0xF6, //!< User defined table 119
    FAPEX_TSD_USER_120_TABLE     = (int32_t)0xF7, //!< User defined table 120
    FAPEX_TSD_USER_121_TABLE     = (int32_t)0xF8, //!< User defined table 121
    FAPEX_TSD_USER_122_TABLE     = (int32_t)0xF9, //!< User defined table 122
    FAPEX_TSD_USER_123_TABLE     = (int32_t)0xFA, //!< User defined table 123
    FAPEX_TSD_USER_124_TABLE     = (int32_t)0xFB, //!< User defined table 124
    FAPEX_TSD_USER_125_TABLE     = (int32_t)0xFC, //!< User defined table 125
    FAPEX_TSD_USER_126_TABLE     = (int32_t)0xFD, //!< User defined table 126
    FAPEX_TSD_USER_127_TABLE     = (int32_t)0xFE  //!< User defined table 127

} FAPEX_TSD_TableIdT;


/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/

/*!
*************************************************************************
** \brief open parameters for the TSD driver
**
** \sa FAPEX_TSD_Open
*************************************************************************/
typedef struct {
    /*!
        STC using flag (0:disable, 1:enable)
    */
    uint32_t    useStc;
    /*!
        TS input select<br>
        \param #FAPEX_TSD_INPUT_CH_A              TS A input channel
        \param #FAPEX_TSD_INPUT_CH_B              TS B input channel
        \param #FAPEX_TSD_INPUT_CH_C              TS C input channel
        \param #FAPEX_TSD_INPUT_LINEAR_AUTO_MODE  TS input via packet RAM (Linear write mode with automatical start)
    */
    uint32_t    tsSelect;
    /*!
        the distance between SYNC bytes (eg. 188, 204)
    */
    uint32_t    syncByteDistance;
    /*!
        TS input flags<br>
        The value of iflags is the bitwise-inclusive OR of values from the following list.
        \param #FAPEX_TSD_IFLAGS_AM_SYNC       Enable automatical synchronization
        \param #FAPEX_TSD_IFLAGS_PACKET_START  TSx_PACKET_START low active
        \param #FAPEX_TSD_IFLAGS_EN            TSx_EN low active
        \param #FAPEX_TSD_IFLAGS_INV_CLK       TSx_CLK invert
        \param #FAPEX_TSD_IFLAGS_SP_MODE1      serial   with    TSx_PACKET_START
        \param #FAPEX_TSD_IFLAGS_SP_MODE2      parallel without TSx_PACKET_START
        \param #FAPEX_TSD_IFLAGS_SP_MODE3      parallel with    TSx_PACKET_START
    */
    uint32_t    iflags;
    /*!
        TS output flags<br>
        The value of oflags is the bitwise-inclusive OR of values from the following list.
        \param #FAPEX_TSD_OFLAGS_OUT_INV_CLK TS_OUT_CLK invert
        \param #FAPEX_TSD_OFLAGS_OUT_EN      generate TS_OUT_EN, TS_OUT_START
    */
    uint32_t    oflags;

} FAPEX_TSD_OpenParamStrT;

#define FAPEX_TSD_INPUT_CH_A                    0x00     //!< TS A input channel
#define FAPEX_TSD_INPUT_CH_B                    0x01     //!< TS B input channel
#define FAPEX_TSD_INPUT_CH_C                    0x02     //!< TS C input channel
#define FAPEX_TSD_INPUT_DIRECT_MODE             0x09     //!< not supported. TS input via packet RAM (Direct Write Mode)
#define FAPEX_TSD_INPUT_LINEAR_MODE             0x19     //!< not supported. TS input via packet RAM (Linear Write mode)
#define FAPEX_TSD_INPUT_LINEAR_AUTO_MODE        0x29     //!< TS input via packet RAM (Linear write mode with automatical start)

#define FAPEX_TSD_IFLAGS_AM_SYNC                (0x0020) //!< Enable automatical synchronization
#define FAPEX_TSD_IFLAGS_PACKET_START           (0x0010) //!< TSx_PACKET_START low active
#define FAPEX_TSD_IFLAGS_EN                     (0x0008) //!< TSx_EN low active
#define FAPEX_TSD_IFLAGS_INV_CLK                (0x0004) //!< TSx_CLK invert
#define FAPEX_TSD_IFLAGS_SP_MODE1               (0x0001) //!< serial   with    TSx_PACKET_START
#define FAPEX_TSD_IFLAGS_SP_MODE2               (0x0002) //!< parallel without TSx_PACKET_START
#define FAPEX_TSD_IFLAGS_SP_MODE3               (0x0003) //!< parallel with    TSx_PACKET_START
#define FAPEX_TSD_IFLAGS_SP_MODE_MSK            (0x0003) //!< mask for sirial/parallel mode

#define FAPEX_TSD_OFLAGS_OUT_INV_CLK            (0x0200) //!< TS_OUT_CLK invert
#define FAPEX_TSD_OFLAGS_OUT_EN                 (0x0100) //!< generate TS_OUT_EN, TS_OUT_START
#define FAPEX_TSD_OFLAGS_OUT_CLK_PRESCALER_MSK  (0x00ff) //!< mask for prescaler to generate TS_OUT_CLK

/*!
*************************************************************************
** \brief TSD block input parameters for the TSD driver<br>
**        This structure contents are the same as FAPEX_TSD_OpenParamStrT.
**
** The related ioctl command type :
** \ref tsd_cmd_set_input "FAPEX_IOCCOM_TSD_SET_INPUT"
**
** \sa FAPEX_TSD_Ioctl
*************************************************************************/
typedef FAPEX_TSD_OpenParamStrT  FAPEX_TSD_InputParamStrT;

/*!
*************************************************************************
** \brief open sub_id parameters for the TSD driver
**
** The related ioctl command type :
** \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID"
**
** \sa FAPEX_TSD_Ioctl
*************************************************************************/
typedef struct {
    /*!
        sub id<br>
        \param[in]  sub_id  Device minor_no (#FAPEXDEV_TSD_SUB_MINOR_FIRST .. #FAPEXDEV_TSD_SUB_MINOR_LAST).
        \param[out] sub_id  Device identifier for TSD channel.
    */
    int32_t     sub_id;
    /*!
        channel number (0..#FAPEXDEV_TSD_CH_SUB_DESCRIPTOR_NUM-1)
    */
    int32_t     channelNo;

} FAPEX_TSD_OpenSubIdParamStrT;

/*!
*************************************************************************
** \brief pid parameters for the TSD driver
**
** The related ioctl command type :
** \ref tsd_cmd_set_pid_filter "FAPEX_IOCCOM_TSD_SET_PID_FILTER",
** \ref tsd_cmd_set_pid_value "FAPEX_IOCCOM_TSD_SET_PID_VALUE"
**
** \note
** For using #FAPEX_IOCCOM_TSD_SET_PID_VALUE, the TSD channel should be already effective
** by setup of #FAPEX_IOCCOM_TSD_SET_PID_FILTER, #FAPEX_IOCCOM_TSD_SET_SECTION_FILTER or
** #FAPEX_IOCCOM_TSD_SET_PES_FILTER.
**
** \sa FAPEX_TSD_Ioctl
*************************************************************************/
typedef struct {
    /*!
        PID value to be filtered. Since a PID shall only be associated once within
        a TSD block, in case the PID is already in use(except for PCR PID), the 
        command "FAPEX_IOCCOM_TSD_SET_PID_FILTER" will return with error.
        If the PID value is 0x1fff, it will be ignore.
        If the PID is the PCR PID which has the same value with video PID or audio PID,
        it's OK to set the PID value with command "FAPEX_IOCCOM_TSD_SET_PID_FILTER".
    */
    uint32_t    pid;
    /*!
        pid filtering flags<br>
        The value of flags is the bitwise-inclusive OR of values from the following list.<br>
        \param #FAPEX_TSD_FLTFLAGS_CHECK_CC        check continuity counter,
        \param #FAPEX_TSD_FLTFLAGS_RM_TS_HEADER    remove TS header,
        \param #FAPEX_TSD_FLTFLAGS_PCR             PCR detection,
        \param #FAPEX_TSD_FLTFLAGS_PACKET_END      packet_end detection
        \param #FAPEX_TSD_FLTFLAGS_SWAPWR_SWAP     enable write swap
        \param #FAPEX_TSD_FLTFLAGS_SWAPWR_NO_SWAP  disable write swap
        \param #FAPEX_TSD_FLTFLAGS_SWAPRD_SWAP     enable read swap
        \param #FAPEX_TSD_FLTFLAGS_SWAPRD_NO_SWAP  disable read swap
        \param #FAPEX_TSD_FLTFLAGS_SWAPMODE_2BYTE  2byte swap mode
        \param #FAPEX_TSD_FLTFLAGS_SWAPMODE_4BYTE  4byte swap mode

        If command type is #FAPEX_IOCCOM_TSD_SET_PID_VALUE, only #FAPEX_TSD_FLTFLAGS_PCR is
        effective to flags.
    */
    uint32_t    flags;
    /*!
        buffer size<br>
        If command type is #FAPEX_IOCCOM_TSD_SET_PID_VALUE, the bufferSize is not effective.
    */
    uint32_t    bufferSize;
    /*!
        buffer threshold heigh level<br>
        If this value is bufferSize, the notification by bufferThresholdH is not effective.<br>
        If command type is #FAPEX_IOCCOM_TSD_SET_PID_VALUE, the bufferThresholdH is not effective.
    */
    uint32_t    bufferThresholdH;
    /*!
        buffer threshold low level<br>
        If this value is 0, the notification by bufferThresholdL is not effective.<br>
        If command type is #FAPEX_IOCCOM_TSD_SET_PID_VALUE, the bufferThresholdL is not effective.
    */
    uint32_t    bufferThresholdL;

} FAPEX_TSD_PidParamStrT;

/*!
*************************************************************************
** \brief section filter parameters for the TSD driver
**
** The related ioctl command type :
** \ref tsd_cmd_set_section_filter "FAPEX_IOCCOM_TSD_SET_SECTION_FILTER"
**
** \sa FAPEX_TSD_Ioctl
*************************************************************************/
typedef struct {
    /*!
        PID value to be filtered. The section packet PID is set to this.
        Since a PID shall only be associated once within a TSD block, 
        in case the PID is already in use, the command 
        "FAPEX_IOCCOM_TSD_SET_SECTION_FILTER" will return with error.
    */
    uint32_t    pid;
    /*!
        section filter flags<br>
        The value of flags is the bitwise-inclusive OR of values from the following list.<br>
        However, select one between FAPEX_TSD_FLTFLAGS_TABLE_NUM1 and FAPEX_TSD_FLTFLAGS_TABLE_NUM32.<br>
        \param #FAPEX_TSD_FLTFLAGS_CHECK_CC           check continuity counter,
        \param #FAPEX_TSD_FLTFLAGS_RM_TS_HEADER       remove TS header,
        \param #FAPEX_TSD_FLTFLAGS_ONESHOT_FILTERING  one-shot filtering for section packet
        \param #FAPEX_TSD_FLTFLAGS_NEW_VERSION        only new version detection
        \param #FAPEX_TSD_FLTFLAGS_PACKET_END         packet_end detection
        \param #FAPEX_TSD_FLTFLAGS_TIMEOUT            timeout detection
        \param #FAPEX_TSD_FLTFLAGS_SWAPWR_SWAP        enable write swap
        \param #FAPEX_TSD_FLTFLAGS_SWAPWR_NO_SWAP     disable write swap
        \param #FAPEX_TSD_FLTFLAGS_SWAPRD_SWAP        enable read swap
        \param #FAPEX_TSD_FLTFLAGS_SWAPRD_NO_SWAP     disable read swap
        \param #FAPEX_TSD_FLTFLAGS_SWAPMODE_2BYTE     2byte swap mode
        \param #FAPEX_TSD_FLTFLAGS_SWAPMODE_4BYTE     4byte swap mode
        \param #FAPEX_TSD_FLTFLAGS_TABLE_NUM1         Number of table_id for section filtering is 1.
        \param #FAPEX_TSD_FLTFLAGS_TABLE_NUM2         Number of table_id for section filtering is 2.
        \param #FAPEX_TSD_FLTFLAGS_TABLE_NUM4         Number of table_id for section filtering is 4.
        \param #FAPEX_TSD_FLTFLAGS_TABLE_NUM8         Number of table_id for section filtering is 8.
        \param #FAPEX_TSD_FLTFLAGS_TABLE_NUM12        Number of table_id for section filtering is 12.
        \param #FAPEX_TSD_FLTFLAGS_TABLE_NUM16        Number of table_id for section filtering is 16.
        \param #FAPEX_TSD_FLTFLAGS_TABLE_NUM20        Number of table_id for section filtering is 20.
        \param #FAPEX_TSD_FLTFLAGS_TABLE_NUM24        Number of table_id for section filtering is 24.
        \param #FAPEX_TSD_FLTFLAGS_TABLE_NUM28        Number of table_id for section filtering is 28.
        \param #FAPEX_TSD_FLTFLAGS_TABLE_NUM32        Number of table_id for section filtering is 32.
    */
    uint32_t    flags;
    /*!
        buffer size
    */
    uint32_t    bufferSize;
    /*!
        buffer threshold heigh level<br>
        If this value is bufferSize, the notification by bufferThresholdH is not effective.<br>
    */
    uint32_t    bufferThresholdH;
    /*!
        buffer threshold low level<br>
        If this value is 0, the notification by bufferThresholdL is not effective.<br>
    */
    uint32_t    bufferThresholdL;
    /*!
        bytes to match<br>
        This is the filtering pattern for section header detection.
        When (data[] & mask[]) equals filter[], section filter matches.
        The filter[0] is used for the matching with table_id.
        The filter[1] and filter[2] are not effective for the matching.
        The filter[3..] are used for the matching with data after section_length.
    */
    uint8_t     filter[FAPEX_TSD_FILTER_SIZE];
    /*!
        filter mask<br>
        When (data[] & mask[]) equals filter[], section filter matches.
        This is the masking pattern for section header detection.
        The mask[0] is used for the matching with table_id.
        The mask[1] and mask[2] are not effective for the matching.
        The mask[3..] are used for the matching with data after section_length.
    */
    uint8_t     mask[FAPEX_TSD_FILTER_SIZE];
    /*!
        timeout value [msec]<br>
        This value is effective only when flags includes #FAPEX_TSD_FLTFLAGS_TIMEOUT.
        If this value is smaller than max timeout value of same TSD block, this value
        will be exchanged to its max value.
    */
    uint32_t    timeout;

} FAPEX_TSD_SctFltParamStrT;

/*!
*************************************************************************
** \brief PES filter parameters for the TSD driver
**
** The related ioctl command type :
** \ref tsd_cmd_set_pes_filter "FAPEX_IOCCOM_TSD_SET_PES_FILTER"
**
** \sa FAPEX_TSD_Ioctl
*************************************************************************/
typedef struct {
    /*!
        PID value to be filtered. The PES packet PID is set to this.
        Since a PID shall only be associated once within
        a TSD block, in case the PID is already in use(except for PCR PID), the 
        command "FAPEX_IOCCOM_TSD_SET_PES_FILTER" will return with error.
        If the PID value is 0x1fff, it will be ignore.
        If the PID is the PCR PID which has the same value with video PID or audio PID,
        it's OK to set the PID value with command "FAPEX_IOCCOM_TSD_SET_PES_FILTER".
    */
    uint32_t    pid;
    /*!
        PES filter flags<br>
        The value of flags is the bitwise-inclusive OR of values from the following list.<br>
        \param #FAPEX_TSD_FLTFLAGS_CHECK_CC           check continuity counter,
        \param #FAPEX_TSD_FLTFLAGS_RM_TS_HEADER       remove TS header,
        \param #FAPEX_TSD_FLTFLAGS_RM_PES_HEADER      remove TS & PES header
        \param #FAPEX_TSD_FLTFLAGS_NEW_VERSION        only new version detection
        \param #FAPEX_TSD_FLTFLAGS_PCR                PCR detection
        \param #FAPEX_TSD_FLTFLAGS_PTS                PTS detection
        \param #FAPEX_TSD_FLTFLAGS_SCRAMBLED_PACKET   scrambled_packet detection
        \param #FAPEX_TSD_FLTFLAGS_ONESHOT_FILTERING  one-shot filtering for scrambled packet
        \param #FAPEX_TSD_FLTFLAGS_SWAPWR_SWAP        enable write swap
        \param #FAPEX_TSD_FLTFLAGS_SWAPWR_NO_SWAP     disable write swap
        \param #FAPEX_TSD_FLTFLAGS_SWAPRD_SWAP        enable read swap
        \param #FAPEX_TSD_FLTFLAGS_SWAPRD_NO_SWAP     disable read swap
        \param #FAPEX_TSD_FLTFLAGS_SWAPMODE_2BYTE     2byte swap mode
        \param #FAPEX_TSD_FLTFLAGS_SWAPMODE_4BYTE     4byte swap mode
    */
    uint32_t    flags;
    /*!
        buffer size
    */
    uint32_t    bufferSize;
    /*!
        buffer threshold heigh level<br>
        If this value is bufferSize, the notification by bufferThresholdH is not effective.<br>
        If streamType is #FAPEX_TSD_STREAMTYPE_TELETEXT, the notification by bufferThresholdH
        is canceled.
    */
    uint32_t    bufferThresholdH;
    /*!
        buffer threshold low level<br>
        If this value is 0, the notification by bufferThresholdL is not effective.<br>
        If streamType is #FAPEX_TSD_STREAMTYPE_TELETEXT, the notification by bufferThresholdL
        is canceled.
    */
    uint32_t    bufferThresholdL;
    /*!
        stream type<br>
        set either the following.
        \param #FAPEX_TSD_STREAMTYPE_AUDIO     Audio stream
        \param #FAPEX_TSD_STREAMTYPE_VIDEO     Video stream
        \param #FAPEX_TSD_STREAMTYPE_TELETEXT  VBI-teletext stream
        \param #FAPEX_TSD_STREAMTYPE_UNKNOWN   otherwise
    */
    uint8_t     streamType;

} FAPEX_TSD_PesParamStrT;

#define FAPEX_TSD_FLTFLAGS_CHECK_CC          (0x00002000)    //!< check continuity counter flag
#define FAPEX_TSD_FLTFLAGS_RM_TS_HEADER      (0x00001000)    //!< remove TS header flag
#define FAPEX_TSD_FLTFLAGS_RM_PES_HEADER     (0x00001800)    //!< remove TS & PES header flag
#define FAPEX_TSD_FLTFLAGS_ONESHOT_FILTERING (0x00000400)    //!< one-shot filtering enable flag
#define FAPEX_TSD_FLTFLAGS_NEW_VERSION       (0x00000200)    //!< only new version detection flag
#define FAPEX_TSD_FLTFLAGS_PCR               (0x00000100)    //!< PCR detection flag
#define FAPEX_TSD_FLTFLAGS_PTS               (0x00000080)    //!< PTS detection flag
#define FAPEX_TSD_FLTFLAGS_PACKET_END        (0x00000040)    //!< packet_end detection flag
#define FAPEX_TSD_FLTFLAGS_SCRAMBLED_PACKET  (0x00000020)    //!< scrambled_packet detection flag
#define FAPEX_TSD_FLTFLAGS_TIMEOUT           (0x00000010)    //!< timeout detection flag

#define FAPEX_TSD_FLTFLAGS_SWAPWR_MSK        (0x00000004)    //!< write swap switch
#define FAPEX_TSD_FLTFLAGS_SWAPWR_SWAP       (0x00000004)    //!< enable write swap
#define FAPEX_TSD_FLTFLAGS_SWAPWR_NO_SWAP    (0x00000000)    //!< disable write swap
#define FAPEX_TSD_FLTFLAGS_SWAPRD_MSK        (0x00000002)    //!< read swap switch
#define FAPEX_TSD_FLTFLAGS_SWAPRD_SWAP       (0x00000002)    //!< enable read swap
#define FAPEX_TSD_FLTFLAGS_SWAPRD_NO_SWAP    (0x00000000)    //!< disable read swap
#define FAPEX_TSD_FLTFLAGS_SWAPMODE_MSK      (0x00000001)    //!< swap mode
#define FAPEX_TSD_FLTFLAGS_SWAPMODE_2BYTE    (0x00000001)    //!< 2byte swap mode
#define FAPEX_TSD_FLTFLAGS_SWAPMODE_4BYTE    (0x00000000)    //!< 4byte swap mode
#define FAPEX_TSD_FLTFLAGS_SWAP_MSK          (0x00000007)

#define FAPEX_TSD_FLTFLAGS_TABLE_NUM1        (0x00000000)    //!< Number of table_id for section filtering is 1.
#define FAPEX_TSD_FLTFLAGS_TABLE_NUM2        (0x00010000)    //!< Number of table_id for section filtering is 2.
#define FAPEX_TSD_FLTFLAGS_TABLE_NUM4        (0x00030000)    //!< Number of table_id for section filtering is 4.
#define FAPEX_TSD_FLTFLAGS_TABLE_NUM8        (0x00070000)    //!< Number of table_id for section filtering is 8.
#define FAPEX_TSD_FLTFLAGS_TABLE_NUM12       (0x000B0000)    //!< Number of table_id for section filtering is 12.
#define FAPEX_TSD_FLTFLAGS_TABLE_NUM16       (0x000F0000)    //!< Number of table_id for section filtering is 16.
#define FAPEX_TSD_FLTFLAGS_TABLE_NUM20       (0x00130000)    //!< Number of table_id for section filtering is 20.
#define FAPEX_TSD_FLTFLAGS_TABLE_NUM24       (0x00170000)    //!< Number of table_id for section filtering is 24.
#define FAPEX_TSD_FLTFLAGS_TABLE_NUM28       (0x001B0000)    //!< Number of table_id for section filtering is 28.
#define FAPEX_TSD_FLTFLAGS_TABLE_NUM32       (0x001F0000)    //!< Number of table_id for section filtering is 32.
#define FAPEX_TSD_FLTFLAGS_TABLE_NUM_SHIFT   16

/*! \brief stream_type */
#define FAPEX_TSD_STREAMTYPE_UNKNOWN         (0)    //!< unknown stream
#define FAPEX_TSD_STREAMTYPE_AUDIO           (1)    //!< Audio stream
#define FAPEX_TSD_STREAMTYPE_VIDEO           (2)    //!< Video stream
#define FAPEX_TSD_STREAMTYPE_TELETEXT        (4)    //!< VBI-teletext stream
#define FAPEX_TSD_STREAMTYPE_AUDIO_SPDIF     (5)    //!< Audio stream channel 2 for SPDIF
#define FAPEX_TSD_STREAMTYPE_AUDIO_MIX0     (6)
#define FAPEX_TSD_STREAMTYPE_AUDIO_MIX1     (7)
#define FAPEX_TSD_STREAMTYPE_AUDIO_MIX2     (8)
#define FAPEX_TSD_STREAMTYPE_AUDIO_MIX3     (9)
#define FAPEX_TSD_STREAMTYPE_AUDIO_MIX4     (10)

/*! \brief descramble work type */
#define FAPEX_DESC_SINGLE_DES_ENCRYPT         0     //!< Single DES encryption mode
#define FAPEX_DESC_DVB_CSA2_DECRYPT           1     //!< DVB CSA 2.1 descrambling mode
#define FAPEX_DESC_DVB_CSA1_DECRYPT           2     //!< DVB CSA 1.0 descrambling mode
#define FAPEX_DESC_MULTI_2_DECRYPT            3     //!< MULTI2 descrambling mode
#define FAPEX_DESC_AES_CBC_DECRYPT            4     //!< AES descryption in CBC mode
#define FAPEX_DESC_SINGLE_DES_DECRYPT         5     //!< Single DES decryption mode
#define FAPEX_DESC_AES_CBC_ENCRYPT            6     //!< AES encryption in CBC mode
#define FAPEX_DESC_AES_ECB_ENCRYPT            7     //!< AES encryption in ECB mode

/*!
*************************************************************************
** \brief Descramble Keys parameters for the TSD driver
**
** The related ioctl command type :
** \ref tsd_cmd_set_keys "FAPEX_IOCCOM_TSD_SET_KEYS"
**
** \sa FAPEX_TSD_Ioctl
*************************************************************************/
typedef struct {
    /*
      work mode
    */
    uint32_t    mode;

    /*
       use 128 bits key
    */
     uint32_t  use128BitKey;

    /*!
        even keys
    */
    uint8_t     evenKey[16];
    uint32_t    evenKeyLength;
    /*!
        odd keys
    */
    uint8_t     oddKey[16];
    uint32_t    oddKeyLength;

   /*!
      initial vector keys
   */
   uint8_t   inVectorKey[32];
   uint32_t  inVectorKeyLength;
   uint32_t  inVectorIndex;

   /*!
     system keys
   */
   uint8_t  systemKey[32];
   uint32_t systemKeyLength;

} FAPEX_TSD_DescKeysParamStrT;

/*!
*************************************************************************
** \brief read information parameters for the TSD driver
**
**        Each parameter are indicated as the figure below.<br>
**        The 'mmaping address' is return value of #FAPEX_TSD_Mmap.
**
** \image html tsd_ReadInfoStrT.png
**
** The related ioctl command type :
** \ref tsd_cmd_get_buffer_readinfo "FAPEX_IOCCOM_TSD_GET_BUFFER_READINFO"
**
** \sa FAPEX_TSD_Ioctl, FAPEX_TSD_Mmap
*************************************************************************/
typedef struct {
    /*!
        buffer filling size
    */
    uint32_t    fillSize;
    /*!
        read pointer offset<br>
        This is offset value from buffer top address mapped by #FAPEX_TSD_Mmap.
    */
    uint32_t    readOffset;
    /*!
        read pointer offset of current section<br>
        This is offset value from buffer top address mapped by #FAPEX_TSD_Mmap.
    */
    uint32_t    sectionOffset;
    /*!
        The length of current section<br>
        This value includes 3 bytes field from 'table_id' to 'section_length'.
        And this is rounded up to multiples of 4.<br>
        There may not be section data for sectionLength in the buffer enough.
        So, it is necessary to compare with fillSize and the difference between readOffset and sectionOffset.
    */
    uint32_t    sectionLength;
    /*!
        section order number<br>
        If the section end is not detected, this value is '-1'.<br>
        If the section end is detected, this value is order number set by
        FAPEX_IOCCOM_TSD_SET_SECTION_FILTER command.
    */
    int32_t     sectionNo;
    /*!
        The buffer status<br>
        The value of bufferStatus is the bitwise-inclusive OR of values from the following list.<br>
        \param #FAPEX_TSD_BUFFSTAT_NONE              normal status
        \param #FAPEX_TSD_BUFFSTAT_PACKET_END        packet end
        \param #FAPEX_TSD_BUFFSTAT_SECTION_END       section end
        \param #FAPEX_TSD_BUFFSTAT_SCRAMBLED_PACKET  scrambled packet
        \param #FAPEX_TSD_BUFFSTAT_CRCERROR          CRC error
        \param #FAPEX_TSD_BUFFSTAT_THRESHUNDER       under Low level threshold
        \param #FAPEX_TSD_BUFFSTAT_THRESHOVER        over High level threshold
        \param #FAPEX_TSD_BUFFSTAT_OVERFLOW          overflow
        \param #FAPEX_TSD_BUFFSTAT_TIMEOUT           timeout
        \param #FAPEX_TSD_BUFFSTAT_STOPPED           stopped TSD channel demuxing
    */
    uint32_t    bufferStatus;

} FAPEX_TSD_ReadInfoStrT;

#define FAPEX_TSD_BUFFSTAT_NONE              (0x0000)    //!< normal status
#define FAPEX_TSD_BUFFSTAT_PACKET_END        (0x0001)    //!< packet end
#define FAPEX_TSD_BUFFSTAT_SECTION_END       (0x0002)    //!< section end
#define FAPEX_TSD_BUFFSTAT_SCRAMBLED_PACKET  (0x0004)    //!< scrambled packet
#define FAPEX_TSD_BUFFSTAT_CRCERROR          (0x0008)    //!< CRC error
#define FAPEX_TSD_BUFFSTAT_THRESHUNDER       (0x0010)    //!< under Low level threshold
#define FAPEX_TSD_BUFFSTAT_THRESHOVER        (0x0020)    //!< over High level threshold
#define FAPEX_TSD_BUFFSTAT_OVERFLOW          (0x0040)    //!< Overflow
#define FAPEX_TSD_BUFFSTAT_TIMEOUT           (0x0080)    //!< Timeout
#define FAPEX_TSD_BUFFSTAT_STOPPED           (0x0100)    //!< stopped TSD channel demuxing

/*!
*************************************************************************
** \brief write information parameters for the TSD driver
**
** The related ioctl command type :
** \ref tsd_cmd_get_buffer_writeinfo "FAPEX_IOCCOM_TSD_GET_BUFFER_WRITEINFO"
**
** \sa FAPEX_TSD_Ioctl
*************************************************************************/
typedef struct {
    /*!
        buffer filling size
    */
    uint32_t    fillSize;
    /*!
        write pointer offset
    */
    uint32_t    writeOffset;
    /*!
        The buffer status<br>
        The value of bufferStatus is the bitwise-inclusive OR of values from the following list.<br>
        \param #FAPEX_TSD_BUFFSTAT_NONE         normal status
        \param #FAPEX_TSD_BUFFSTAT_THRESHUNDER  under Low level threshold
        \param #FAPEX_TSD_BUFFSTAT_THRESHOVER   over High level threshold
        \param #FAPEX_TSD_BUFFSTAT_TIMEOUT      timeout
        \param #FAPEX_TSD_BUFFSTAT_OVERFLOW     overflow
    */
    uint32_t    bufferStatus;

} FAPEX_TSD_WriteInfoStrT;

/*!
*************************************************************************
** \brief TSD block information parameters for the TSD driver
**
** The related ioctl command type :
** \ref tsd_cmd_get_block_info "FAPEX_IOCCOM_TSD_GET_BLOCK_INFO"
**
** \sa FAPEX_TSD_Ioctl
*************************************************************************/
typedef struct {
    /*!
        number of used channel
    */
    uint32_t    usedChannelNum;
    /*!
        enable channel flag<br>
        Each channel status is shown by bit flag.
    */
    uint32_t    enableChannelFlag;
    /*!
        running channnel flag<br>
        Each channel status is shown by bit flag.
    */
    uint32_t    runningChannelFlag;
    /*!
        block setting parameters<br>
        This content is the same as the parameter which was set by FAPEX_TSD_Open()
        or \ref tsd_cmd_set_input "FAPEX_IOCCOM_TSD_SET_INPUT" command with FAPEX_TSD_Ioctl().
    */
    FAPEX_TSD_InputParamStrT    blockParams;

} FAPEX_TSD_BlockInfoStrT;

/*!
*************************************************************************
** \brief TSD channel information parameters for the TSD driver
**
** The related ioctl command type :
** \ref tsd_cmd_get_channel_info "FAPEX_IOCCOM_TSD_GET_CHANNEL_INFO"
**
** \sa FAPEX_TSD_Ioctl
*************************************************************************/
typedef struct {
    /*!
        channel number
    */
    uint32_t    channelNo;
    /*!
        pid value  
        PID=0x1FFF means we has not set PID value before. 
    */
    uint32_t    pid;
    /*!
        buffsre size
    */
    uint32_t    bufferSize;

} FAPEX_TSD_ChannelInfoStrT;

/*!
*************************************************************************
** \brief TSD block/channel status parameters for the TSD driver
**
** The related ioctl command type :
** \ref tsd_cmd_get_status "FAPEX_IOCCOM_TSD_GET_STATUS"
**
** \sa FAPEX_TSD_Ioctl
*************************************************************************/
typedef struct {
    /*!
        TSD block/channel status<br>
        \param #FAPEX_STATUS_OPENED   opened status
        \param #FAPEX_STATUS_READY    ready starts
        \param #FAPEX_STATUS_RUNNING  running status
    */
    int32_t     status;
    /*!
        STC status<br>
        \param #FAPEX_TSD_SYNC_INITIAL_STATUS  Initial driver status
        \param #FAPEX_TSD_SYNC_SYNCHRONISING   Driver is synchronising
        \param #FAPEX_TSD_SYNC_SYNCHRONISED    Driver is synchronised
        \param #FAPEX_TSD_SYNC_NO_PCR          Synchronisation failed
        \param #FAPEX_TSD_SYNC_TWO_PCRS        Synchronisation failed
        \param #FAPEX_TSD_SYNC_UNCERTAIN       Synchronisation may reset
    */
    int32_t     syncStatus;

} FAPEX_TSD_StatusStrT;

#define FAPEX_TSD_SYNC_INITIAL_STATUS   0   //!< Initial driver status
#define FAPEX_TSD_SYNC_SYNCHRONISING    1   //!< Driver is synchronising
#define FAPEX_TSD_SYNC_SYNCHRONISED     2   //!< Driver is synchronised
#define FAPEX_TSD_SYNC_NO_PCR           3   //!< Synchronisation failed
#define FAPEX_TSD_SYNC_TWO_PCRS         4   //!< Synchronisation failed
#define FAPEX_TSD_SYNC_UNCERTAIN        5   //!< Synchronisation may reset

/*!
*************************************************************************
** \brief TS packet input control mapping information parameters
**        for the TSD driver
**
**        Each parameter are indicated as the figure below.<br>
**        The 'mmaping address' is return value of #FAPEX_TSD_Mmap.
**
** \image html tsd_PacketInMapInfoStrT.png
**
** The related ioctl command type :
** \ref tsd_cmd_get_input_map_info "FAPEX_IOCCOM_TSD_GET_INPUT_MAP_INFO"
**
** \sa FAPEX_TSD_Ioctl, FAPEX_TSD_Mmap
*************************************************************************/
typedef struct {
    /*!
        The mapping size for TS packet registering [byte]<br>
        This is required value for '\e length' of #FAPEX_TSD_Mmap.
    */
    uint32_t    mapSize;
    /*!
        The mapping offset for TS packet registering [byte]<br>
        This is offset value from input buffer top address mapped by #FAPEX_TSD_Mmap.
    */
    uint32_t    mapOffset;
    /*!
        the offset size for each TS packet registering [byte]<br>
        ex. TS packet register buffer address<br>
        &nbsp;&nbsp;PACKET_0 : (mapping address) + mapOffset<br>
        &nbsp;&nbsp;PACKET_1 : (mapping address) + mapOffset + (1 x packetOffset)<br>
        &nbsp;&nbsp;PACKET_2 : (mapping address) + mapOffset + (2 x packetOffset)<br>
        &nbsp;&nbsp;PACKET_3 : (mapping address) + mapOffset + (3 x packetOffset)<br>
    */
    uint32_t    packetOffset;

} FAPEX_TSD_PacketInMapInfoStrT;

#define FAPEX_TSD_PACKET_INPUT_BYTES        188
#define FAPEX_TSD_PACKET_INPUT_BUFF_NUM     4

#define FAPEX_TSD_PACKET_INPUT_LENGTH       188
#define FAPEX_TSD_PACKET_INPUT_LENGTH_A     192
#define FAPEX_TSD_PACKET_INPUT_LENGTH_B     204

/*!
*************************************************************************
** \brief TS packet input information parameters for the TSD driver
**
** The related ioctl command type :
** \ref tsd_cmd_set_input_packetinfo "FAPEX_IOCCOM_TSD_SET_INPUT_PACKETINFO",
** \ref tsd_cmd_get_input_packetinfo "FAPEX_IOCCOM_TSD_GET_INPUT_PACKETINFO"
**
** \sa FAPEX_TSD_Ioctl
*************************************************************************/
typedef struct {
    /*!
        The input buffer status<br>
        When you read this parameter with #FAPEX_IOCCOM_TSD_GET_INPUT_PACKETINFO command,
        this value of inputStatus is the bitwise-inclusive OR of values from the following list.<br>
        \param #FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF0  buffer0 is busy
        \param #FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF1  buffer1 is busy
        \param #FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF2  buffer2 is busy
        \param #FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF3  buffer3 is busy

        When you write this parameter with #FAPEX_IOCCOM_TSD_SET_INPUT_PACKETINFO command,
        this value of inputStatus is the bitwise-inclusive OR of values from the following list.<br>
        \param #FAPEX_TSD_PACKET_INPUT_STATUS_VALID_BUFF0  buffer0 is valid
        \param #FAPEX_TSD_PACKET_INPUT_STATUS_VALID_BUFF1  buffer1 is valid
        \param #FAPEX_TSD_PACKET_INPUT_STATUS_VALID_BUFF2  buffer2 is valid
        \param #FAPEX_TSD_PACKET_INPUT_STATUS_VALID_BUFF3  buffer3 is valid
    */
    uint32_t    inputStatus;
    /*!
        The buffer status<br>
        The value of bufferStatus is the bitwise-inclusive OR of values from the following list.<br>
        \param #FAPEX_TSD_BUFFSTAT_NONE     normal status
        \param #FAPEX_TSD_BUFFSTAT_TIMEOUT  timeout

        If command type is #FAPEX_IOCCOM_TSD_SET_INPUT_PACKETINFO, the bufferSize is not effective.
    */
    uint32_t    bufferStatus;

} FAPEX_TSD_PacketInInfoStrT;


#define FAPEX_TSD_PVR_TRACK_MAX    31

/*!
*************************************************************************
** \brief  PVR track info parameters for the PVR channel
**
** \ref FAPEX_TSD_PvrChannelSetStrT
*************************************************************************/
typedef struct {
    /*!
      stream type<br>
      set either the following.
      \param #FAPEX_TSD_STREAMTYPE_AUDIO     Audio stream
      \param #FAPEX_TSD_STREAMTYPE_VIDEO     Video stream
      \param #FAPEX_TSD_STREAMTYPE_TELETEXT  VBI-teletext stream
      \param #FAPEX_TSD_STREAMTYPE_UNKNOWN   otherwise
      */
    uint32_t    track_type;
    /*!
      stream pid<br>
      */
    uint32_t    track_pid;

    /*!
      track assigned tsd channel id<br>
      for bm_wrap_tsd (track_31) this id is 31
      for other tsd this id is auto detected
      */
    int32_t     track_channel_id;
}FAPEX_TSD_PvrTrackInfoStrT;

/*!
*************************************************************************
** \brief  PVR channel parameters for the TSD driver
**
** The related ioctl command type :
** \ref tsd_cmd_pvr_open_channel_set  "FAPEX_IOCCOM_TSD_PVR_OPEN_CHANNEL_SET"
** \ref tsd_cmd_pvr_close_channel_set "FAPEX_IOCCOM_TSD_PVR_CLOSE_CHANNEL_SET"
** \ref tsd_cmd_pvr_ctrl_channel_set  "FAPEX_IOCCOM_TSD_PVR_CTRL_CHANNEL_SET"
**
** \sa FAPEX_TSD_Ioctl
*************************************************************************/
typedef struct {
    int32_t     bm_wrap_tsd_id;
    int32_t     bm_wrap_tsd_fd;         /*tsd channel who has setup BM*/
    uint32_t    tsd_block_index;        /*tsd 31 & bm31*/
    int32_t     tsd_block_fd;
    uint32_t    enable;                 /*1 enable; 0 disable*/

    FAPEX_TSD_PvrTrackInfoStrT  track_set[FAPEX_TSD_PVR_TRACK_MAX];
    FAPEX_TSD_PvrTrackInfoStrT  track_31;
    int32_t                     track_tsd_handle[FAPEX_TSD_PVR_TRACK_MAX];
    uint32_t                    track_cnt; /*track count without track_31*/
}FAPEX_TSD_PvrChannelSetStrT;


#define FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_FLAG     (0x0004)
#define FAPEX_TSD_PACKET_INPUT_STATUS_VALID_FLAG    (0x0004)
#define FAPEX_TSD_PACKET_INPUT_STATUS_BUFF_SHFT     4

#define FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF0    (FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_FLAG)
#define FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF1    (FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_FLAG <<      FAPEX_TSD_PACKET_INPUT_STATUS_BUFF_SHFT )
#define FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF2    (FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_FLAG << (2 * FAPEX_TSD_PACKET_INPUT_STATUS_BUFF_SHFT))
#define FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF3    (FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_FLAG << (3 * FAPEX_TSD_PACKET_INPUT_STATUS_BUFF_SHFT))
#define FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF_ALL \
 (FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF0 | FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF1 | \
  FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF2 | FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF3)

#define FAPEX_TSD_PACKET_INPUT_STATUS_VALID_BUFF0    (FAPEX_TSD_PACKET_INPUT_STATUS_VALID_FLAG | 0x0000)
#define FAPEX_TSD_PACKET_INPUT_STATUS_VALID_BUFF1   ((FAPEX_TSD_PACKET_INPUT_STATUS_VALID_FLAG | 0x0001) <<      FAPEX_TSD_PACKET_INPUT_STATUS_BUFF_SHFT )
#define FAPEX_TSD_PACKET_INPUT_STATUS_VALID_BUFF2   ((FAPEX_TSD_PACKET_INPUT_STATUS_VALID_FLAG | 0x0002) << (2 * FAPEX_TSD_PACKET_INPUT_STATUS_BUFF_SHFT))
#define FAPEX_TSD_PACKET_INPUT_STATUS_VALID_BUFF3   ((FAPEX_TSD_PACKET_INPUT_STATUS_VALID_FLAG | 0x0003) << (3 * FAPEX_TSD_PACKET_INPUT_STATUS_BUFF_SHFT))
#define FAPEX_TSD_PACKET_INPUT_STATUS_VALID_BUFF_ALL \
 (FAPEX_TSD_PACKET_INPUT_STATUS_VALID_BUFF0 | FAPEX_TSD_PACKET_INPUT_STATUS_VALID_BUFF1 | \
  FAPEX_TSD_PACKET_INPUT_STATUS_VALID_BUFF2 | FAPEX_TSD_PACKET_INPUT_STATUS_VALID_BUFF3)

/*!
*************************************************************************
** \brief ioctl specific command for the TSD driver
*************************************************************************/
#define FAPEX_IOCCOM_TSD_OPEN                   _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD +  0,  FAPEX_TSD_OpenParamStrT)
#define FAPEX_IOCCOM_TSD_OPEN_SUBID             _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_SUBID_TSD, FAPEX_TSD_OpenSubIdParamStrT)
#define FAPEX_IOCCOM_TSD_GET_BLOCK_INFO         _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD +  1,  FAPEX_TSD_BlockInfoStrT)
#define FAPEX_IOCCOM_TSD_GET_CHANNEL_INFO       _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD +  2,  FAPEX_TSD_ChannelInfoStrT)
#define FAPEX_IOCCOM_TSD_GET_STATUS             _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD +  3,  FAPEX_TSD_StatusStrT)
#define FAPEX_IOCCOM_TSD_SET_INPUT              _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD +  4,  FAPEX_TSD_InputParamStrT)
#define FAPEX_IOCCOM_TSD_SET_PID_FILTER         _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD +  5,  FAPEX_TSD_PidParamStrT)
#define FAPEX_IOCCOM_TSD_SET_SECTION_FILTER     _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD +  6,  FAPEX_TSD_SctFltParamStrT)
#define FAPEX_IOCCOM_TSD_SET_PES_FILTER         _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD +  7,  FAPEX_TSD_PesParamStrT)
#define FAPEX_IOCCOM_TSD_SET_PID_VALUE          _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD +  8,  FAPEX_TSD_PidParamStrT)
#define FAPEX_IOCCOM_TSD_SET_KEYS               _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD +  9,  FAPEX_TSD_DescKeysParamStrT)
#define FAPEX_IOCCOM_TSD_SET_BUFFER_READSIZE    _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD + 10,  uint32_t)
#define FAPEX_IOCCOM_TSD_SET_BUFFER_WRITEPOINTEROFFSET _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD + 11,  uint32_t)
#define FAPEX_IOCCOM_TSD_GET_BUFFER_READINFO    _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD + 12,  FAPEX_TSD_ReadInfoStrT)
#define FAPEX_IOCCOM_TSD_GET_BUFFER_WRITEINFO   _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD + 13,  FAPEX_TSD_WriteInfoStrT)
#define FAPEX_IOCCOM_TSD_GET_BUFFER_LEVEL       _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD + 14,  uint32_t)
#define FAPEX_IOCCOM_TSD_GET_INPUT_MAP_INFO     _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD + 15,  FAPEX_TSD_PacketInMapInfoStrT)
#define FAPEX_IOCCOM_TSD_SET_INPUT_PACKETINFO   _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD + 16,  FAPEX_TSD_PacketInInfoStrT)
#define FAPEX_IOCCOM_TSD_GET_INPUT_PACKETINFO   _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD + 16,  FAPEX_TSD_PacketInInfoStrT)
#define FAPEX_IOCCOM_TSD_GET_STC_VALUE          _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD + 17,  uint64_t)
#define FAPEX_IOCCOM_TSD_PVR_OPEN_CHANNEL_SET   _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD + 18,  FAPEX_TSD_PvrChannelSetStrT)
#define FAPEX_IOCCOM_TSD_PVR_CLOSE_CHANNEL_SET  _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD + 19,  FAPEX_TSD_PvrChannelSetStrT)
#define FAPEX_IOCCOM_TSD_PVR_CTRL_CHANNEL_SET   _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD + 20,  FAPEX_TSD_PvrChannelSetStrT)
#define FAPEX_IOCCOM_TSD_RESET_BUFFER           _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD + 21,  uint32_t)
#define FAPEX_IOCCOM_TSD_SET_PES_PARSER         _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD + 22,  uint32_t)
#define FAPEX_IOCCOM_TSD_RESET_PES_PARSER       _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD + 24,  uint32_t)
#define FAPEX_IOCCOM_TSD_CLEAR_SECTION_FILTER   _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TSD + 25,  uint32_t)

/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
extern "C" {
#endif

FAPEXOPENRET_T  FAPEX_TSD_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T   FAPEX_TSD_Close(FAPEXRELARG1_T id);
FAPEXREADRET_T  FAPEX_TSD_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T buf, FAPEXREADARG3_T count);
FAPEXWRTRET_T   FAPEX_TSD_Write(FAPEXWRTARG1_T id, FAPEXWRTARG2_T buf, FAPEXWRTARG3_T count);
FAPEXIOCRET_T   FAPEX_TSD_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);
FAPEXMMPRET_T   FAPEX_TSD_Mmap(FAPEXMMPARG1_T start, FAPEXMMPARG2_T length,
                               FAPEXMMPARG3_T prot,  FAPEXMMPARG4_T flags,
                               FAPEXMMPARG5_T id,    FAPEXMMPARG6_T offset);
FAPEXMUNMPRET_T FAPEX_TSD_Munmap(FAPEXMMPARG1_T start, FAPEXMMPARG2_T length);

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBTSD_H */

