/*!
*************************************************************************
**
** \file    fapex/abst_libvidec.h
**
** \brief   abstraction library header file (Video Decoder driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBVIDEC_H
#define _FAPEX_ABST_LIBVIDEC_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/
#define FAPEX_SYNC_UNKNOWN_BASE     -1    //!< SYNC base is unknown.
#define FAPEX_SYNC_AUDIO_PTS_BASED   0    //!< SYNC is Audio PTS based environment.
#define FAPEX_SYNC_VIDEO_PTS_BASED   1    //!< SYNC is Video PTS based environment.
#define FAPEX_SYNC_PCR_BASED         2    //!< SYNC PCR based environment.
#define FAPEX_SYNC_STREAMING_PCR_BASED 3 //!<streaming PCRs or external PCRs shall be synchronized to STC.

/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex VIDEC error code base value */
#define FAPEX_VIDEC_ERR_BASE            (FAPEX_VIDEC_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_VIDEC_ERR_BAD_PARAMETER   (FAPEX_VIDEC_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define FAPEX_VIDEC_ERR_OUT_OF_MEMORY   (FAPEX_VIDEC_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)

/*! Timeout occured. */
#define FAPEX_VIDEC_ERR_TIMEOUT         (FAPEX_VIDEC_ERR_BASE + FAPEX_ERR_TIMEOUT)

/*! Invalid status. */
#define FAPEX_VIDEC_ERR_INVALID_STATUS  (FAPEX_VIDEC_ERR_BASE + FAPEX_ERR_INVALID_HANDLE)


/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/

/*!
*******************************************************************************
** \brief Parameter for defining the reset type of 
** the video decoder.
*******************************************************************************
*/
#define FAPEX_VIDEC_SOFT_RESET  0   /*!< Software reset of VD.*/
#define FAPEX_VIDEC_HARD_RESET  1   /*!<  Hard reset of VD, It also causes internal rebooting.*/

/*!
*************************************************************************
** \brief open parameters for the VIDEC driver
**
** \sa FAPEX_VIDEC_Open
*************************************************************************/
typedef struct {
    /*!
        input Device minor number<br>
        (#FAPEXDEV_TSD_SUB_MINOR_FIRST..#FAPEXDEV_TSD_SUB_MINOR_LAST).
    */
    int32_t     inputId;

    /*!
        Video codec type<br>
        \param #FAPEX_VIDEC_CODEC_TYPE_MPEG2 MPEG2
        \param #FAPEX_VIDEC_CODEC_TYPE_H264  H.264
        \param #FAPEX_VIDEC_CODEC_TYPE_VC1   VC1
        \param #FAPEX_VIDEC_CODEC_TYPE_AVS   AVS
    */
    int32_t     codecType;

    /*!
        Playback mode<br>
        \param #FAPEX_VIDEC_PLAYBACK_ENVIRONMENT_LIVE Live broadcast environment.
        \param #FAPEX_VIDEC_PLAYBACK_ENVIRONMENT_PVR  recorded live PVR environment.
        \param #FAPEX_VIDEC_PLAYBACK_ENVIRONMENT_VOD  recorded live VOD environment.
    */
    int32_t     playbackEnv;

    /*!
        active format description control flag<br>
        \param #FAPEX_VIDEC_AFD_UNKNOWN     Unknown user request on how to handle AFD data if present.
        \param #FAPEX_VIDEC_AFD_ENABLE_2    User defined Afd is 2
        \param #FAPEX_VIDEC_AFD_ENABLE_3    User defined Afd is 3
        \param #FAPEX_VIDEC_AFD_ENABLE_4    User defined Afd is 4
        \param #FAPEX_VIDEC_AFD_ENABLE_8    User defined Afd is 8
        \param #FAPEX_VIDEC_AFD_ENABLE_9    User defined Afd is 9
        \param #FAPEX_VIDEC_AFD_ENABLE_10   User defined Afd is 10
        \param #FAPEX_VIDEC_AFD_ENABLE_11   User defined Afd is 11
        \param #FAPEX_VIDEC_AFD_ENABLE_13   User defined Afd is 13
        \param #FAPEX_VIDEC_AFD_ENABLE_14   User defined Afd is 14
        \param #FAPEX_VIDEC_AFD_ENABLE_15   User defined Afd is 15
        \param #FAPEX_VIDEC_AFD_ENABLE_AUTO Afd process is executed as defined in the stream.
        \param #FAPEX_VIDEC_AFD_DISABLE     The AFD data application to the decoded picture shall be
                                            discarded in case the AFD data is present.
    */
    int32_t     afdFlag;

    /*!
        Option for video decoder to decode H.264 streams which have a AUD
        delimiter or use a PES header as a delimiter. The choice is only offered
        for H.264 streams and for MPEG streams AUD delimiter is used which
        is actually a delimiter to Picture start code.<br>
        \param #FAPEX_VIDEC_DELIMITER_UNKNOWN    Unknown delimiter
        \param #FAPEX_VIDEC_DELIMITER_AUD_HEADER Delimiter is an AUD
        \param #FAPEX_VIDEC_DELIMITER_PES_HEADER Delimiter is a PES Header
    */
    int32_t     delimiterType;

} FAPEX_VIDEC_OpenParamStrT;

// codec type
#define FAPEX_VIDEC_CODEC_TYPE_UNKNOWN  0   //!< Unknown type decoder
#define FAPEX_VIDEC_CODEC_TYPE_MPEG2    1   //!< MPEG2
#define FAPEX_VIDEC_CODEC_TYPE_H264     2   //!< H.264
#define FAPEX_VIDEC_CODEC_TYPE_AVS      3   //!< AVS
#define FAPEX_VIDEC_CODEC_TYPE_VC1      4   //!< VC1

// Frame Bank Size type
#define FAPEX_VIDEC_FRAME_BANK_SIZE_UNKNOWN   (-1)  //!< Unknown Frame bank Size
#define FAPEX_VIDEC_FRAME_BANK_SIZE_1920HD      0   //!< 1920 pixels per one decoded picture line
#define FAPEX_VIDEC_FRAME_BANK_SIZE_1440HD      1   //!< 1440 pixels per one decoded picture line
#define FAPEX_VIDEC_FRAME_BANK_SIZE_1280HD      2   //!< 1280 pixels per one decoded picture line
#define FAPEX_VIDEC_FRAME_BANK_SIZE_SD          3   //!< 720 pixels per one decoded picture line

// Playback mode
#define FAPEX_VIDEC_PLAYBACK_ENVIRONMENT_LIVE   0   //!< Live broadcast environment.
#define FAPEX_VIDEC_PLAYBACK_ENVIRONMENT_PVR    1   //!< recorded live PVR environment.
#define FAPEX_VIDEC_PLAYBACK_ENVIRONMENT_VOD    2   //!< recorded live VOD environment.
#define FAPEX_VIDEC_PALYBACK_ENVIRONMENT_STREAMING 3//!< playback from a file or ethernet.

// active format description control flag
#define FAPEX_VIDEC_AFD_UNKNOWN       (-1)   /*!< Unknown user request on how to handle AFD data if present.
                                                 if the state is unknown then video decoder shall fail to open. */
#define FAPEX_VIDEC_AFD_ENABLE_2        2    /*!< User defined Afd is 2 */
#define FAPEX_VIDEC_AFD_ENABLE_3        3    /*!< User defined Afd is 3 */
#define FAPEX_VIDEC_AFD_ENABLE_4        4    /*!< User defined Afd is 4 */
#define FAPEX_VIDEC_AFD_ENABLE_8        8    /*!< User defined Afd is 8 */
#define FAPEX_VIDEC_AFD_ENABLE_9        9    /*!< User defined Afd is 9 */
#define FAPEX_VIDEC_AFD_ENABLE_10      10    /*!< User defined Afd is 10 */
#define FAPEX_VIDEC_AFD_ENABLE_11      11    /*!< User defined Afd is 11 */
#define FAPEX_VIDEC_AFD_ENABLE_13      13    /*!< User defined Afd is 13 */
#define FAPEX_VIDEC_AFD_ENABLE_14      14    /*!< User defined Afd is 14 */
#define FAPEX_VIDEC_AFD_ENABLE_15      15    /*!< User defined Afd is 15 */
#define FAPEX_VIDEC_AFD_ENABLE_AUTO    16    /*!< Afd process is executed as defined in the stream. */
#define FAPEX_VIDEC_AFD_DISABLE        17    /*!< The AFD data application to the decoded picture shall be
                                                  discarded in case the AFD data is present. */

// delimiter type
#define FAPEX_VIDEC_DELIMITER_UNKNOWN     (-1) //!< Unknown delimiter
#define FAPEX_VIDEC_DELIMITER_AUD_HEADER    0  //!< Delimiter is an AUD
#define FAPEX_VIDEC_DELIMITER_PES_HEADER    1  //!< Delimiter is a PES Header

/*!
*************************************************************************
** \brief start parameter for the VIDEC driver
**
** The related ioctl command type :
** \ref videc_cmd_start "FAPEX_IOCCOM_VIDEC_START"
**
** \sa FAPEX_VIDEC_Ioctl
*************************************************************************/
typedef struct {
    /*!
        Decoder Output mode<br>
        \param #FAPEX_VIDEC_ENABLE_FORCE_OFF                        Disables the video to the VO block on the subsequent VBI.
        \param #FAPEX_VIDEC_ENABLE_FORCE_ON_IMMEDIATE               Enables the video to the VO block on the subsequent VBI
                                                                    without defering it for predefined number of frames.
        \param #FAPEX_VIDEC_ENABLE_FORCE_ON                         Enables the video to the VO block on the VBI that occurs
                                                                    after a predefined number of frames.
        \param #FAPEX_VIDEC_ENABLE_SYNCHRONOUSLY_SUSPEND            Enables the video to the VO block in sync with the first
                                                                    decoded frame while waiting for the first frame to be decoded.
                                                                    Enables the video to the VO block on the VBI that occurs
                                                                    after a predefined number of frames.
                                                                    This parameter causes a timed blocking action.
        \param #FAPEX_VIDEC_ENABLE_SYNCHRONOUSLY_NO_SUSPEND         Enables the video to the VO block in sync with the first
                                                                    decoded frame. This causes a non-blocking call
                                                                    Enables the video to the VO block on the VBI that occurs
                                                                    after a predefined number of frames.
        \param #FAPEX_VIDEC_ENABLE_SYNCHRONOUSLY_ON_SYNC_SUSPEND    Enables the video to the VO block when the decoded
                                                                    frame is and is in sync to system time clock.
                                                                    This type of enable is permitted only under normal playback mode
                                                                    This causes a blocking call.
                                                                    Enables the video to the VO block on the VBI that occurs
                                                                    after a predefined number of frames.
        \param #FAPEX_VIDEC_ENABLE_SYNCHRONOUSLY_ON_SYNC_NO_SUSPEND Enables the video to the VO block when the decoded
                                                                    frame is and is in sync to system time clock.
                                                                    This type of enable is permitted only under normal playback mode
                                                                    Enables the video to the VO block on the VBI that occurs
                                                                    after a predefined number of frames.
    */
    int32_t     outputMode;

    /*!
        Playback mode<br>
        \param #FAPEX_VIDEC_PLAYBACK_MODE_PAUSE           Pauses the playback in PVR environment otherwise
                                                          the request is rejected.
        \param #FAPEX_VIDEC_PLAYBACK_MODE_NORMAL          Continues to play the video normally.
        \param #FAPEX_VIDEC_PLAYBACK_MODE_FIRST_I_ONLY    Displays only the first occuring I Frame.
        \param #FAPEX_VIDEC_PLAYBACK_MODE_I_ONLY          Continues to play the video in the trick mode I_ONLY.
        \param #FAPEX_VIDEC_PLAYBACK_MODE_I_AND_P_ONLY    Continues to play the video in the trick mode
                                                          skipping B frames.
        \param #FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_2X Continues to play the video in the trick mode
                                                          slow forward 2 times.
        \param #FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_4X Continues to play the video in the trick mode
                                                          slow forward 4 times.
        \param #FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_8X Continues to play the video in the trick mode
                                                          slow forward 8 times.
    */
    int32_t     playbackMode;

    /*!
        notification control flags<br>
        The value of flags is the bitwise-inclusive OR of values from the following list.<br>
        \param #FAPEX_VIDEC_NOTIFY_FLAGS_NONE           none event
        \param #FAPEX_VIDEC_NOTIFY_FLAGS_DEC_AVAILABLE  frame decoding avilable notification
        \param #FAPEX_VIDEC_NOTIFY_FLAGS_DISP_AVAILABLE frame display avilable notification
        \param #FAPEX_VIDEC_NOTIFY_FLAGS_FRAME_INFO     frame information enable notification
        \param #FAPEX_VIDEC_NOTIFY_FLAGS_PARAM_CHANGE   display parameter change notification
    */
    uint32_t    flags;

} FAPEX_VIDEC_StartParamStrT;

// Decoder Output mode
#define FAPEX_VIDEC_ENABLE_FORCE_OFF                      (-1) /*!< Disables the video to the VO block on the subsequent VBI. */
#define FAPEX_VIDEC_ENABLE_FORCE_ON_IMMEDIATE               0  /*!< Enables the video to the VO block on the subsequent VBI
                                                                    without defering it for predefined number of frames. */
#define FAPEX_VIDEC_ENABLE_FORCE_ON                         1  /*!< Enables the video to the VO block on the VBI that occurs
                                                                    after a predefined number of frames. */
#define FAPEX_VIDEC_ENABLE_SYNCHRONOUSLY_SUSPEND            2  /*!< Enables the video to the VO block in sync with the first
                                                                    decoded frame while waiting for the first frame to be decoded.
                                                                    Enables the video to the VO block on the VBI that occurs
                                                                    after a predefined number of frames.
                                                                    This parameter causes a timed blocking action. */
#define FAPEX_VIDEC_ENABLE_SYNCHRONOUSLY_NO_SUSPEND         3  /*!< Enables the video to the VO block in sync with the first
                                                                    decoded frame. This causes a non-blocking call
                                                                    Enables the video to the VO block on the VBI that occurs
                                                                    after a predefined number of frames. */
#define FAPEX_VIDEC_ENABLE_SYNCHRONOUSLY_ON_SYNC_SUSPEND    4  /*!< Enables the video to the VO block when the decoded
                                                                    frame is and is in sync to system time clock.
                                                                    This type of enable is permitted only under normal playback mode
                                                                    This causes a blocking call.
                                                                    Enables the video to the VO block on the VBI that occurs
                                                                    after a predefined number of frames. */
#define FAPEX_VIDEC_ENABLE_SYNCHRONOUSLY_ON_SYNC_NO_SUSPEND 5  /*!< Enables the video to the VO block when the decoded
                                                                    frame is and is in sync to system time clock.
                                                                    This type of enable is permitted only under normal playback mode
                                                                    Enables the video to the VO block on the VBI that occurs
                                                                    after a predefined number of frames. */

// Playback mode
#define FAPEX_VIDEC_PLAYBACK_MODE_UNKNOWN         (-1)  /*!< Unknown playback mode. */
#define FAPEX_VIDEC_PLAYBACK_MODE_PAUSE             0   /*!< Pauses the playback in PVR environment otherwise
                                                             the request is rejected. */
#define FAPEX_VIDEC_PLAYBACK_MODE_NORMAL            1   /*!< Continues to play the video normally. */
#define FAPEX_VIDEC_PLAYBACK_MODE_FIRST_I_ONLY      2   /*!< Displays only the first occuring I Frame. */
#define FAPEX_VIDEC_PLAYBACK_MODE_I_ONLY            3   /*!< Continues to play the video in the trick mode I_ONLY. */
#define FAPEX_VIDEC_PLAYBACK_MODE_I_AND_P_ONLY      4   /*!< Continues to play the video in the trick mode
                                                             skipping B frames.
                                                             This mode is available for MPEG streams only.*/
#define FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_2X   5   /*!< Continues to play the video in the trick mode
                                                             slow forward 2 times. */
#define FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_4X   6   /*!< Continues to play the video in the trick mode
                                                             slow forward 4 times. */
#define FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_8X   7   /*!< Continues to play the video in the trick mode
                                                             slow forward 8 times. */

// notification control flags
#define FAPEX_VIDEC_NOTIFY_FLAGS_NONE           (0x0000)    //!< none event
#define FAPEX_VIDEC_NOTIFY_FLAGS_DEC_AVAILABLE  (0x0001)    //!< frame decoding avilable notification
#define FAPEX_VIDEC_NOTIFY_FLAGS_DISP_AVAILABLE (0x0002)    //!< frame display avilable notification
#define FAPEX_VIDEC_NOTIFY_FLAGS_FRAME_INFO     (0x0004)    //!< frame information enable notification
#define FAPEX_VIDEC_NOTIFY_FLAGS_PARAM_CHANGE   (0x0010)    //!< display parameter change notification

/*!
*************************************************************************
** \brief status parameters for the VIDEC driver
**
** The related ioctl command type :
** \ref videc_cmd_get_status "FAPEX_IOCCOM_VIDEC_GET_STATUS"
**
** \sa FAPEX_VIDEC_Ioctl
*************************************************************************/
typedef struct {

    /*! Firmware version of the video firmware as in flash.
        The firmware version in the flash is available only if the video decoder
        is already booted.
    */
    uint32_t    firmwareVersionInFlash;
    /*!
        Firmware version of the video firmware that is supported by fapi.
    */
    uint32_t    firmwareVersionSupported;
    /*!
        Decoder type.<br>
        \param #FAPEX_VIDEC_CODEC_TYPE_UNKNOWN Unknown type decoder
        \param #FAPEX_VIDEC_CODEC_TYPE_MPEG2   MPEG2
        \param #FAPEX_VIDEC_CODEC_TYPE_H264    H.264
        \param #FAPEX_VIDEC_CODEC_TYPE_VC1     VC1
        \param #FAPEX_VIDEC_CODEC_TYPE_AVS     AVS
    */
    int32_t     decoderType;
    /*!
        Memory layout for a maximum sized frame.<br>
        \param #FAPEX_VIDEC_FRAME_BANK_SIZE_UNKNOWN Unknown Frame bank Size
        \param #FAPEX_VIDEC_FRAME_BANK_SIZE_1920HD  1920 pixels per one decoded picture line
        \param #FAPEX_VIDEC_FRAME_BANK_SIZE_1440HD  1440 pixels per one decoded picture line
        \param #FAPEX_VIDEC_FRAME_BANK_SIZE_1280HD  1280 pixels per one decoded picture line
        \param #FAPEX_VIDEC_FRAME_BANK_SIZE_SD      720 pixels per one decoded picture line
    */
    int32_t     decodeFrameBankSizeType;
    /*!
        Width of display picture in pixels
    */
    uint32_t    displayWidth;
    /*!
        Width of display picture in pixels
    */
    uint32_t    displayHeight;
    /*!
        Width of current decoded frame in pixels
    */
    uint32_t    decodedWidth;
    /*!
        Height of current decoded frame in lines
    */
    uint32_t    decodedHeight;
    /*!
        Aspect Ratio of current decoded frame<br>
        \param #FAPEX_VIDEC_AR_UNKNOWN  Unknown aspect ratio.
        \param #FAPEX_VIDEC_AR_4_3      Aspect ratio is 4:3.
        \param #FAPEX_VIDEC_AR_16_9     Aspect ratio is 16:9.
        \param #FAPEX_VIDEC_AR_14_9     Aspect ratio is 14:9.
        \param #FAPEX_VIDEC_AR_221_100  Aspect ratio is 221:100.
        \param #FAPEX_VIDEC_AR_235_100  Aspect ratio is 235:100.
    */
    int32_t     aspectRatio;
    /*!
        Average bit rate of Video elementary stream in bits per second
        It is averaged since the start of video stream
    */
    uint32_t    averageBitRate;
    /*!
        Current bit rate of Video elementary stream in bits per second
        It is averaged over the last 5 pictures.
    */
    uint32_t    currBitRate;
    /*!
        Frame rate of the input video elementary stream [100 x Hz].
    */
    uint32_t    frameRate;

    //State variables
    /*!
        Boot State<br>
        \param #FAPEX_VIDEC_BOOT_STATE_NON     Not booted state.
        \param #FAPEX_VIDEC_BOOT_STATE_REQUEST Request to bootState the video core.
        \param #FAPEX_VIDEC_BOOT_STATE_DONE    Video core is in the booted state.
    */
    int32_t     bootState;
    /*!
        Hard Reset State<br>
        \param #FAPEX_VIDEC_RESET_STATE_UNKNOWN Unknown reset state before booting.
        \param #FAPEX_VIDEC_RESET_STATE_NON     Non reset state, after reset is completed this state is the final state.
        \param #FAPEX_VIDEC_RESET_STATE_REQUEST Request to reset the video core and the driver as well.
        \param #FAPEX_VIDEC_RESET_STATE_DONE    Request to reset the video core and the driver as well is served
                                                and now will return to FAPEX_VIDEC_RESET_STATE_NON state.
    */
    int32_t     resetState;
    /*!
        Enable State<br>
        \param #FAPEX_VIDEC_ENABLE_DISABLE_STATE_UNKNOWN                 Unknown enable/disable state.
        \param #FAPEX_VIDEC_REQUEST_TO_SYNCHRONIZE_ENABLE_ON_VIDEO_FRAME Request to synchronize enable with first decoded frame.
        \param #FAPEX_VIDEC_REQUEST_TO_SYNCHRONIZE_ENABLE_ON_VIDEO_SYNC  Request to synchronize enable with first decoded frame after Video synchronization to STC.
        \param #FAPEX_VIDEC_SYNCHRONIZE_ENABLING                         Request to synchronize enable with first decoded frame is pending.
        \param #FAPEX_VIDEC_SYNCHRONIZE_ENABLED_ON_VIDEO_FRAME           Request to synchronize enable with first decoded frame is served.
        \param #FAPEX_VIDEC_SYNCHRONIZE_ENABLED_ON_VIDEO_SYNC            Request to synchronize enable with first decoded frame synchronized to STC.
        \param #FAPEX_VIDEC_FORCE_ENABLED                                Request to forcefully enable on the next occuring Frame sync is done.
        \param #FAPEX_VIDEC_REQUEST_TO_FORCE_DISABLE                     Request to forcefully disable on the next occuring Frame sync.
        \param #FAPEX_VIDEC_FORCE_DISABLED                               Request to forcefully disable on the next occuring Frame sync is done.
    */
    int32_t     enableState;
    /*!
        Start State<br>
        \param #FAPEX_VIDEC_START_STOP_STATE_UNKNOWN   Unknown start stop state.
        \param #FAPEX_VIDEC_PREPARE_TO_STOP            Prepare to stop.
        \param #FAPEX_VIDEC_REQUEST_TO_STOP            Request to stop.
        \param #FAPEX_VIDEC_STOPPED                    Request to stop is served.
        \param #FAPEX_VIDEC_STOPPED_AFTER_FIRST_IFRAME Software Stop only. Video decoding stops after first I Frame decode.
        \param #FAPEX_VIDEC_PREPARE_TO_START           Prepare to start video decoder.
        \param #FAPEX_VIDEC_REQUEST_TO_START           Request to start video decoding as well.
        \param #FAPEX_VIDEC_STARTED                    Request to start is served.
    */
    int32_t     startState;
    /*!
        Freeze State<br>
        \param #FAPEX_VIDEC_FREEZE_OFF  Continues the video to the VO block.
        \param #FAPEX_VIDEC_FREEZE_ON   Freezes the video to the VO block.
    */
    int32_t     freezeState;
    /*!
        Health State<br>
        \param #FAPEX_VIDEC_HEALTH_UNKNOWN  Specifies an unknown health of the Video decoder
        \param #FAPEX_VIDEC_HEALTH_GOOD     Specifies good health as everything is running normally.
        \param #FAPEX_VIDEC_HEALTH_BAD_L0   Specifies bad health of level 0, a reset is not required.
                                            Still VD is in recoverable state.
        \param #FAPEX_VIDEC_HEALTH_BAD_L1   Specifies bad health of level 1, a reset is not required.
                                            Still VD is in recoverable state.
        \param #FAPEX_VIDEC_HEALTH_BAD_L2   Specifies bad health of level 2, a reset is not required.
                                            Still VD is in recoverable state.
        \param #FAPEX_VIDEC_HEALTH_DEAD     Specifies dead state, a complete reboot is required.
                                            VD is non-recoverable state.
    */
    int32_t     healthState;

} FAPEX_VIDEC_StatusStrT;

// Profile type
#define FAPEX_VIDEC_PROFILE_UNKNOWN           (-1)  //!< Unknown profile
#define FAPEX_VIDEC_PROFILE_RESERVED            0   //!< reserved profile
#define FAPEX_VIDEC_PROFILE_SIMPLE              1   //!< Simple profile
#define FAPEX_VIDEC_PROFILE_BASELINE            2   //!< Baseline profile
#define FAPEX_VIDEC_PROFILE_MAIN                3   //!< Main profile
#define FAPEX_VIDEC_PROFILE_ADVANCED            4   //!< Advanced profile
#define FAPEX_VIDEC_PROFILE_SNR_SCALABLE        5   //!< SNR scaleable profile as indicated by MPEG standard
#define FAPEX_VIDEC_PROFILE_SPATIALLY_SCALABLE  6   //!< Spatially scaleable profile as indicated by MPEG standard
#define FAPEX_VIDEC_PROFILE_EXTENDED            7   //!< Extended profile
#define FAPEX_VIDEC_PROFILE_HIGH                8   //!< High profile
#define FAPEX_VIDEC_PROFILE_HIGH_10             9   //!< High 10 profile
#define FAPEX_VIDEC_PROFILE_HIGH_422            10  //!< High 4:2:2 profile
#define FAPEX_VIDEC_PROFILE_HIGH_444            11  //!< High 4:4:4 profile
#define	FAPEX_VIDEC_PROFILE_JIZHUN              12  //!< Jizhun profile  

// Level type
#define FAPEX_VIDEC_LEVEL_UNKNOWN             (-1)  //!< Specifies an unknown Level
#define FAPEX_VIDEC_LEVEL_RESERVED              0   //!< Specifies a reserved Level
#define FAPEX_VIDEC_LEVEL_LOW                   1   //!< Low Level
#define FAPEX_VIDEC_LEVEL_MEDIUM                2   //!< Medium Level
#define FAPEX_VIDEC_LEVEL_MAIN                  3   //!< Main Level
#define FAPEX_VIDEC_LEVEL_HIGH_1440             4   //!< High 1440 Level
#define FAPEX_VIDEC_LEVEL_HIGH                  5   //!< High Level
#define FAPEX_VIDEC_LEVEL_0                     6   //!< Level 0
#define FAPEX_VIDEC_LEVEL_1                     7   //!< Level 1
#define FAPEX_VIDEC_LEVEL_2                     8   //!< Level 2
#define FAPEX_VIDEC_LEVEL_3                     9   //!< Level 3
#define FAPEX_VIDEC_LEVEL_4                     10   //!< Level 4
#define FAPEX_VIDEC_LEVEL_1_0                   11   //!< Level 1.0
#define FAPEX_VIDEC_LEVEL_1_0_B                 12   //!< Level 1.0 b
#define FAPEX_VIDEC_LEVEL_1_1                   13  //!< Level 1.1
#define FAPEX_VIDEC_LEVEL_1_2                   14   //!< Level 1.2
#define FAPEX_VIDEC_LEVEL_1_3                   15
#define FAPEX_VIDEC_LEVEL_2_0                   16   //!< Level 2.0
#define FAPEX_VIDEC_LEVEL_2_1                   17   //!< Level 2.1
#define FAPEX_VIDEC_LEVEL_2_2                   18   //!< Level 2.2
#define FAPEX_VIDEC_LEVEL_3_0                   19   //!< Level 3.0
#define FAPEX_VIDEC_LEVEL_3_1                   20   //!< Level 3.1
#define FAPEX_VIDEC_LEVEL_3_2                   21   //!< Level 3.2
#define FAPEX_VIDEC_LEVEL_4_0                   22   //!< Level 4.0
#define FAPEX_VIDEC_LEVEL_4_1                   23   //!< Level 4.1
#define FAPEX_VIDEC_LEVEL_4_2                   24   //!< Level 4.2
#define FAPEX_VIDEC_LEVEL_5_0                   25   //!< Level 5.0
#define FAPEX_VIDEC_LEVEL_5_1                   26   //!< Level 5.1
#define FAPEX_VIDEC_LEVEL_20                    27   //!< Level 2.0
#define FAPEX_VIDEC_LEVEL_40                    28   //!< Level 4.0
#define FAPEX_VIDEC_LEVEL_42                    29   //!< Level 4.2
#define	FAPEX_VIDEC_LEVEL_60                    30   //!< Level 6.0
#define FAPEX_VIDEC_LEVEL_62                    31   //!< Level 6.2

// Aspect Ratio type
#define FAPEX_VIDEC_AR_UNKNOWN          0   //!< Unknown aspect ratio.
#define FAPEX_VIDEC_AR_4_3              1   //!< Aspect ratio is 4:3.
#define FAPEX_VIDEC_AR_14_9             2   //!< Aspect ratio is 14:9.
#define FAPEX_VIDEC_AR_16_9             3   //!< Aspect ratio is 16:9.
#define FAPEX_VIDEC_AR_221_100          4   //!< Aspect ratio is 221:100.
#define FAPEX_VIDEC_AR_235_100          5   //!< Aspect ratio is 235:100.

// Boot state
#define FAPEX_VIDEC_BOOT_STATE_NON    (-1)  //!< Not booted state.
#define FAPEX_VIDEC_BOOT_STATE_REQUEST  0   //!< Request to bootState the video core.
#define FAPEX_VIDEC_BOOT_STATE_DONE     1   //!< Video core is in the booted state.

// Hard Reset state
#define FAPEX_VIDEC_RESET_STATE_UNKNOWN (-1)    /*!< Unknown reset state before booting. */
#define FAPEX_VIDEC_RESET_STATE_NON       0     /*!< Non reset state, after reset is completed this state is the final state. */
#define FAPEX_VIDEC_RESET_STATE_PREPARE   1     /*!< Preparation for reset the video core and the driver as well. */
#define FAPEX_VIDEC_RESET_STATE_REQUEST   2     /*!< Request to reset the video core and the driver as well. */ 
#define FAPEX_VIDEC_RESET_STATE_DONE      3     /*!< Request to reset the video core and the driver as well is served
                                                     and now will return to FAPEX_VIDEC_RESET_STATE_NON state. */

// Start State
#define FAPEX_VIDEC_START_STOP_STATE_UNKNOWN  (-1) //!< Unknown start stop state.
#define FAPEX_VIDEC_PREPARE_TO_STOP             0  //!< Prepare to stop.
#define FAPEX_VIDEC_REQUEST_TO_STOP             1  //!< Request to stop.
#define FAPEX_VIDEC_STOPPED                     2  //!< Request to stop is served.
#define FAPEX_VIDEC_STOPPED_AFTER_FIRST_IFRAME  3  //!< Software Stop only. Video decoding stops after first I Frame decode.
#define FAPEX_VIDEC_PREPARE_TO_START            4  //!< Prepare to start video decoder.
#define FAPEX_VIDEC_REQUEST_TO_START            5  //!< Request to start video decoding as well.
#define FAPEX_VIDEC_STARTED                     6  //!< Request to start is serv

// Enable/Disable state
#define FAPEX_VIDEC_ENABLE_DISABLE_STATE_UNKNOWN               (-1) //!< Unknown enable/disable state.
#define FAPEX_VIDEC_REQUEST_TO_SYNCHRONIZE_ENABLE_ON_VIDEO_FRAME 0  //!< Request to synchronize enable with first decoded frame.
#define FAPEX_VIDEC_REQUEST_TO_SYNCHRONIZE_ENABLE_ON_VIDEO_SYNC  1  //!< Request to synchronize enable with first decoded frame after Video synchronization to STC.
#define FAPEX_VIDEC_SYNCHRONIZE_ENABLING                         2  //!< Request to synchronize enable with first decoded frame is pending.
#define FAPEX_VIDEC_SYNCHRONIZE_ENABLED_ON_VIDEO_FRAME           3  //!< Request to synchronize enable with first decoded frame is served.
#define FAPEX_VIDEC_SYNCHRONIZE_ENABLED_ON_VIDEO_SYNC            4  //!< Request to synchronize enable with first decoded frame synchronized to STC.
#define FAPEX_VIDEC_REQUEST_TO_FORCE_ENABLE                      5  //!< Request to forcefully enable on the next occuring Frame sync.
#define FAPEX_VIDEC_FORCE_ENABLED                                6  //!< Request to forcefully enable on the next occuring Frame sync is done.
#define FAPEX_VIDEC_REQUEST_TO_FORCE_DISABLE                     7  //!< Request to forcefully disable on the next occuring Frame sync.
#define FAPEX_VIDEC_FORCE_DISABLED                               8  //!< Request to forcefully disable on the next occuring Frame sync is done.

// Freeze State
#define FAPEX_VIDEC_FREEZE_OFF          0   //!< Continues the video to the VO block.
#define FAPEX_VIDEC_FREEZE_ON           1   //!< Freezes the video to the VO block.

// Health State
#define FAPEX_VIDEC_HEALTH_UNKNOWN    (-1)  /*!< Specifies an unknown health of the Video decoder */
#define FAPEX_VIDEC_HEALTH_GOOD         0   /*!< Specifies good health as everything is running normally.*/
#define FAPEX_VIDEC_HEALTH_BAD_L0       1   /*!< Specifies bad health of level 0, a reset is not required.
                                                 Still VD is in recoverable state. */
#define FAPEX_VIDEC_HEALTH_BAD_L1       2   /*!< Specifies bad health of level 1, a reset is not required.
                                                 Still VD is in recoverable state. */
#define FAPEX_VIDEC_HEALTH_BAD_L2       3   /*!< Specifies bad health of level 2, a reset is not required.
                                                 Still VD is in recoverable state. */
#define FAPEX_VIDEC_HEALTH_DEAD         4   /*!< Specifies dead state, a complete reboot is required.
                                                 VD is non-recoverable state. */

/*!
*************************************************************************
** \brief Video ES parameters for the VIDEC driver
**
** The related ioctl command type :
** \ref videc_cmd_get_ves_params "FAPEX_IOCCOM_VIDEC_GET_VES_PARAMS"
**
** \sa FAPEX_VIDEC_Ioctl
*************************************************************************/
typedef struct {
    /*!
        Profile of the bitstream.<br>
        \param #FAPEX_VIDEC_PROFILE_UNKNOWN            Unknown profile
        \param #FAPEX_VIDEC_PROFILE_RESERVED           reserved profile
        \param #FAPEX_VIDEC_PROFILE_SIMPLE             Simple profile
        \param #FAPEX_VIDEC_PROFILE_BASELINE           Baseline profile
        \param #FAPEX_VIDEC_PROFILE_MAIN               Main profile
        \param #FAPEX_VIDEC_PROFILE_SNR_SCALABLE       SNR scaleable profile as indicated by MPEG standard
        \param #FAPEX_VIDEC_PROFILE_SPATIALLY_SCALABLE Spatially scaleable profile as indicated by MPEG standard
        \param #FAPEX_VIDEC_PROFILE_EXTENDED           Extended profile
        \param #FAPEX_VIDEC_PROFILE_HIGH               High profile
        \param #FAPEX_VIDEC_PROFILE_HIGH_10            High 10 profile
        \param #FAPEX_VIDEC_PROFILE_HIGH_422           High 4:2:2 profile
        \param #FAPEX_VIDEC_PROFILE_HIGH_444           High 4:4:4 profile
    */
    int32_t     profile;
    /*!
        Level of the bitstream.<br>
        \param #FAPEX_VIDEC_LEVEL_UNKNOWN   Specifies an unknown Level
        \param #FAPEX_VIDEC_LEVEL_RESERVED  Specifies a reserved Level
        \param #FAPEX_VIDEC_LEVEL_LOW       Low Level
        \param #FAPEX_VIDEC_LEVEL_MAIN      Main Level
        \param #FAPEX_VIDEC_LEVEL_HIGH_1440 High 1440 Level
        \param #FAPEX_VIDEC_LEVEL_HIGH      High Level
        \param #FAPEX_VIDEC_LEVEL_1_0       Level 1.0
        \param #FAPEX_VIDEC_LEVEL_1_0_B     Level 1.0 b
        \param #FAPEX_VIDEC_LEVEL_1_1       Level 1.1
        \param #FAPEX_VIDEC_LEVEL_1_2       Level 1.2
        \param #FAPEX_VIDEC_LEVEL_2_0       Level 2.0
        \param #FAPEX_VIDEC_LEVEL_2_1       Level 2.1
        \param #FAPEX_VIDEC_LEVEL_2_2       Level 2.2
        \param #FAPEX_VIDEC_LEVEL_3_0       Level 3.0
        \param #FAPEX_VIDEC_LEVEL_3_1       Level 3.1
        \param #FAPEX_VIDEC_LEVEL_3_2       Level 3.2
        \param #FAPEX_VIDEC_LEVEL_4_0       Level 4.0
        \param #FAPEX_VIDEC_LEVEL_4_1       Level 4.1
        \param #FAPEX_VIDEC_LEVEL_4_2       Level 4.2
        \param #FAPEX_VIDEC_LEVEL_5_0       Level 5.0
        \param #FAPEX_VIDEC_LEVEL_5_1       Level 5.1
    */
    int32_t     level;
    /*!
        Average of the stream's bitrate
    */
    uint32_t    averageBitRate;
    /*!
        currentbitrate averaged over last 1 second
    */
    uint32_t    currBitRate;
    /*!
        Aspect Ratio of current decoded frame<br>
        \param #FAPEX_VIDEC_AR_UNKNOWN  Unknown aspect ratio.
        \param #FAPEX_VIDEC_AR_4_3      Aspect ratio is 4:3.
        \param #FAPEX_VIDEC_AR_16_9     Aspect ratio is 16:9.
        \param #FAPEX_VIDEC_AR_14_9     Aspect ratio is 14:9.
        \param #FAPEX_VIDEC_AR_221_100  Aspect ratio is 221:100.
        \param #FAPEX_VIDEC_AR_235_100  Aspect ratio is 235:100.
    */
    int32_t     aspectRatio;
    /*!
        Frame rate of the input video elementary stream [100 x Hz].
    */
    uint32_t    frameRate;
    /*!
        Decoder Latency of video decoding pipeline.
    */
    int64_t     decoderLatency;
    /*!
        This mode parameter is used to define interlace or progressive modes.<br>
        0 : interlace mode, 1 : progressive mode, 0xFFFFFFFF : unknown
    */
    uint32_t    frameField;
    /*!
        Width of display picture in pixels
    */
    uint32_t    displayWidth;
    /*!
        Width of display picture in pixels
    */
    uint32_t    displayHeight;
    /*!
        Width of current decoded frame in pixels
    */
    uint32_t    decodedWidth;
    /*!
        Height of current decoded frame in lines
    */
    uint32_t    decodedHeight;
    /*!
        Sample aspect ratio width.
    */
    uint32_t    sarWidth;
    /*!
        Sample aspect ratio heght.
    */
    uint32_t    sarHeight;

} FAPEX_VIDEC_VESParamStrT;

/*!
*************************************************************************
** \brief Video changed decoded frame information
**
**        Each parameter are indicated as the figure below.<br>
**        The 'mmaping address' is return value of #FAPEX_VIDEC_Mmap.
**        The 'Frame_Bank_Size' is got with command \ref videc_cmd_get_frame_bank_size
**        "FAPEX_IOCCOM_VIDEC_GET_FRAME_BANK_SIZE" of #FAPEX_VIDEC_Ioctl.
**
** \image html videc_DecFrameInfoStrT.png
**
** The related ioctl command type :
** \ref videc_cmd_get_dec_frame_info "FAPEX_IOCCOM_VIDEC_GET_DEC_FRAME_INFO"
**
** \sa FAPEX_VIDEC_Ioctl, FAPEX_VIDEC_Mmap
*************************************************************************/
typedef struct {
    /*!
        video decoded frame information flag<br>
        The value of flags is the bitwise-inclusive OR of values from the following list.<br>
        \param #FAPEX_VIDEC_INFO_NONE                   no decoded frame information
        \param #FAPEX_VIDEC_INFO_DEC_AVAILABLE          available flag for first video decode frame
        \param #FAPEX_VIDEC_INFO_DISP_AVAILABLE         available flag for first video display frame
        \param #FAPEX_VIDEC_INFO_DISP_ENABLE            enable flag for video frame information
        \param #FAPEX_VIDEC_INFO_DISP_CHANGED_AR        aspect ratio changed
        \param #FAPEX_VIDEC_INFO_DISP_CHANGED_FRM_FLD   frame/field type changed
        \param #FAPEX_VIDEC_INFO_DISP_CHANGED_FLD_INV   field inversion changed
        \param #FAPEX_VIDEC_INFO_DISP_CHANGED_DEC_SIZE  decoded frame size changed
        \param #FAPEX_VIDEC_INFO_STOPPED                stopped video frame decoding
    */
    uint32_t    enableFlag;
    /*!
        Aspect Ratio of current decoded frame<br>
        This value is enable only when 
        \param #FAPEX_VIDEC_AR_UNKNOWN  Unknown aspect ratio.
        \param #FAPEX_VIDEC_AR_4_3      Aspect ratio is 4:3.
        \param #FAPEX_VIDEC_AR_16_9     Aspect ratio is 16:9.
        \param #FAPEX_VIDEC_AR_14_9     Aspect ratio is 14:9.
        \param #FAPEX_VIDEC_AR_221_100  Aspect ratio is 221:100.
        \param #FAPEX_VIDEC_AR_235_100  Aspect ratio is 235:100.
    */
    int32_t     aspectRatio;
    /*!
        This mode parameter is used to define progressive or interlace modes.<br>
        1 : progressive mode, 0 : interlace mode
    */
    uint32_t    frameField;
    /*!
        field inversion flag.<br>
        0 : top-bottom field, 1 : bottom-top field
    */
    uint32_t    fieldInversion;
    /*!
        decoded width
    */
    uint32_t    decodedWidth;
    /*!
        decoded height
    */
    uint32_t    decodedHeight;
    /*!
        display width
    */
    uint32_t    displayWidth;
    /*!
        display height
    */
    uint32_t    displayHeight;
    /*!
        Y offset of video frame buffer<br>
        This is offset value from the decoded frame buffer top address mapped by #FAPEX_VIDEC_Mmap
        for luminance component of current image data.
    */
    uint32_t    offset_y;
    /*!
        C offset of video frame buffer<br>
        This is offset value from the decoded frame buffer top address mapped by #FAPEX_VIDEC_Mmap
        for chrominance component of current image data.
    */
    uint32_t    offset_c;

   uint32_t    frameBank;

} FAPEX_VIDEC_DecFrameInfoStrT;

#define FAPEX_VIDEC_INFO_NONE                   (0x0000)    //!< no decoded frame information
#define FAPEX_VIDEC_INFO_DEC_AVAILABLE          (0x0001)    //!< available flag for first video decode frame
#define FAPEX_VIDEC_INFO_DISP_AVAILABLE         (0x0002)    //!< available flag for first video display frame
#define FAPEX_VIDEC_INFO_DISP_ENABLE            (0x0004)    //!< enable flag for video frame information
#define FAPEX_VIDEC_INFO_DISP_CHANGED_AR        (0x0010)    //!< aspect ratio changed
#define FAPEX_VIDEC_INFO_DISP_CHANGED_FRM_FLD   (0x0020)    //!< frame/field type changed
#define FAPEX_VIDEC_INFO_DISP_CHANGED_FLD_INV   (0x0040)    //!< field inversion changed
#define FAPEX_VIDEC_INFO_DISP_CHANGED_DEC_SIZE  (0x0080)    //!< decoded frame size changed
#define FAPEX_VIDEC_INFO_STOPPED                (0x8000)    //!< stopped video frame decoding
#define FAPEX_VIDEC_INFO_DISP_CHANGED           \
 (FAPEX_VIDEC_INFO_DISP_CHANGED_AR | FAPEX_VIDEC_INFO_DISP_CHANGED_FRM_FLD | \
  FAPEX_VIDEC_INFO_DISP_CHANGED_FLD_INV | FAPEX_VIDEC_INFO_DISP_CHANGED_DEC_SIZE)

/*!
*************************************************************************
** \brief Video Elementary Stream data parameter for the VIDEC driver
**
** The related ioctl command type :
** \ref videc_cmd_dec_single_iframe   "FAPEX_IOCCOM_VIDEC_DEC_SINGLE_I_FRAME",
** \ref videc_cmd_dec_sequence_iframe "FAPEX_IOCCOM_VIDEC_DEC_SEQUENCE_I_FRAME"
**
** \sa FAPEX_VIDEC_Ioctl
*************************************************************************/
typedef struct {
    /*!
        Data pointer to the Video Elementary Stream.
    */
    uint32_t    *data;

    /*!
        Size of the Video Elementary Stream in Bytes.
    */
    uint32_t    size;

} FAPEX_VIDEC_ESdataParamStrT;

typedef struct {
	uint32_t searchPattern;
	uint32_t foundPtr; 
	uint64_t pts;
}
FAPEX_VIDEC_AudFeedStrT;
/*!
**************************************************************************
** select the Stream segment mode 
*************************************************************************/
#define FAPEX_VIDEC_UNKNOWN_STREAM_SEGMENT_MODE      -1 
#define FAPEX_VIDEC_STREAM_SEGMENT_MODE_FULL          0 //!<stream segment is normal.
#define FAPEX_VIDEC_STREAM_SEGMENT_MODE_1             1 //!<stream segment is of 1seg type as in ISDB-T

/*!
*************************************************************************
** \brief ioctl specific command for the VIDEC driver
*************************************************************************/
#define FAPEX_IOCCOM_VIDEC_OPEN                   _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC +  0, FAPEX_VIDEC_OpenParamStrT)
#define FAPEX_IOCCOM_VIDEC_START                  _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC +  1, FAPEX_VIDEC_StartParamStrT)
#define FAPEX_IOCCOM_VIDEC_GET_STATUS             _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC +  2, FAPEX_VIDEC_StatusStrT)
#define FAPEX_IOCCOM_VIDEC_SET_OUTPUT             _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC +  3, int32_t)
#define FAPEX_IOCCOM_VIDEC_SET_DISPON             _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC +  4, int32_t)
#define FAPEX_IOCCOM_VIDEC_SET_DISPOFF            _IO (FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC +  5)
#define FAPEX_IOCCOM_VIDEC_GET_VES_PARAMS         _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC +  6, FAPEX_VIDEC_VESParamStrT)
#define FAPEX_IOCCOM_VIDEC_GET_FRAME_BANK_SIZE    _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC +  7, FAPEX_VIDEC_DecFrameInfoStrT)
#define FAPEX_IOCCOM_VIDEC_GET_DEC_FRAME_INFO     _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC +  8, FAPEX_VIDEC_DecFrameInfoStrT)
#define FAPEX_IOCCOM_VIDEC_SET_PLAYBACK_ENV       _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC +  9, int32_t)
#define FAPEX_IOCCOM_VIDEC_SET_PLAYBACK_MODE      _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 10, int32_t)
#define FAPEX_IOCCOM_VIDEC_SET_FREEZE_MODE        _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 11, int32_t)
#define FAPEX_IOCCOM_VIDEC_RECOVER                _IO (FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 12)
#define FAPEX_IOCCOM_VIDEC_DEC_SINGLE_I_FRAME     _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 13, FAPEX_VIDEC_ESdataParamStrT)
#define FAPEX_IOCCOM_VIDEC_DISP_SINGLE_I_FRAME    _IO (FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 14)
#define FAPEX_IOCCOM_VIDEC_DEC_SEQUENCE_I_FRAME   _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 15, FAPEX_VIDEC_ESdataParamStrT)
#define FAPEX_IOCCOM_VIDEC_DISP_SEQUENCE_I_FRAME  _IO (FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 16)
#define FAPEX_IOCCOM_VIDEC_SET_OUTPUT_DELAY       _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 17, int32_t)
#define FAPEX_IOCCOM_VIDEC_SET_SYNC               _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 18, uint32_t)
#define FAPEX_IOCCOM_SYNC_SET_STC_REGULATION      _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 19, uint32_t)
#define FAPEX_IOCCOM_SYNC_SET_ENVIRONMENT         _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 20, uint32_t)
#define FAPEX_IOCCOM_VIDEC_SET_INFO_CALLBACK      _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 21, uint32_t)
#define	FAPEX_IOCCOM_VIDEC_SET_STREAM_SEGMENT_MODE  _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 22, int32_t)
#define FAPEX_IOCCOM_VIDEC_GET_STREAM_SEGMENT_MODE  _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 22, int32_t)
#define FAPEX_IOCCOM_VIDEC_SET_FRAME_RATE         _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 23, uint32_t)
#define FAPEX_IOCCOM_VIDEC_CLEAR_FRAME_RATE       _IO (FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 24)
#define FAPEX_IOCCOM_VIDEC_CLEAR_STREAM_SEGMENT_MODE _IO (FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 25)
#define FAPEX_IOCCOM_VIDEC_ENABLE_DISABLE_OUTPUT  _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 26, int32_t)
#define FAPEX_IOCCOM_VIDEC_AUD_FEED_EX		  _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIDEC + 27, uint32_t)

/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
extern "C" {
#endif

FAPEXOPENRET_T  FAPEX_VIDEC_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T   FAPEX_VIDEC_Close(FAPEXRELARG1_T id);
FAPEXIOCRET_T   FAPEX_VIDEC_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);
FAPEXMMPRET_T   FAPEX_VIDEC_Mmap(FAPEXMMPARG1_T start, FAPEXMMPARG2_T length,
                                 FAPEXMMPARG3_T prot,  FAPEXMMPARG4_T flags,
                                 FAPEXMMPARG5_T id,    FAPEXMMPARG6_T offset);
FAPEXMUNMPRET_T FAPEX_VIDEC_Munmap(FAPEXMMPARG1_T start, FAPEXMMPARG2_T length);

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBVIDEC_H */

