/*!
*************************************************************************
**
** \file    fapex/abst_libvrflash.h
**
** \brief   abstraction library header file (FLASH driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2009-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBVRFLASH_H
#define _FAPEX_ABST_LIBVRFLASH_H

#include "fapex/abst_lib.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/
#if 0
typedef struct
{
    /*!
    ** The number of logical blocks used
    */
    uint32_t lbaCount;
    /*!
    ** The size of logical blocks in bytes
    */
    uint32_t lbaBytes;
    /*!
    ** The first sector to be used, taken from FAPEX_VRFLASH_OpenParamsT
    */
    uint32_t firstSector;
    /*!
    ** The number of sectors to be used, taken from FAPEX_VRFLASH_OpenParamsT
    ** but recalculated to the number of available sectors.
    */
    uint32_t sectorCount;
} FAPEX_VRFLASH_VolumeT;
#endif

#define FAPEX_VRFLASH_DEVICE_PFLASH    0   //!< the parallel flash device
#define FAPEX_VRFLASH_DEVICE_SFLASH    1   //!< the serial flash device

#define FAPEX_VRFLASH_MODULE_BASE (0x1000)

#define FAPEX_SYS_ERR_BAD_PARAMETER    (1)

#define FAPEX_VRFLASH_ERR_BAD_PARAMETER \
        (int32_t)(FAPEX_VRFLASH_MODULE_BASE+FAPEX_SYS_ERR_BAD_PARAMETER)

typedef enum
{
    FAPEX_VRFLASH_VERSION = (int32_t)0x00000001   //!< The current driver version
} FAPEX_VRFLASH_VersionEnumT;

/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/


/*!
*************************************************************************
** \brief Open parameters for the FLASH driver.
**
** This data structure covers all parameters which need to be specified
** when an instance of the driver is opened.
*************************************************************************/
typedef struct _FAPEX_VRFLASH_OpenParamsT
{
    /*!
    ** The version of the driver.
    ** \ref FAPEX_VRFLASH_VersionEnumT "FAPEX_VRFLASH_VERSION".
    */
    FAPEX_VRFLASH_VersionEnumT version;
    /*!
    ** The device type to open, either the parallel or the serial flash
    */
    uint32_t deviceType;
    /*!
    ** The first sector to be used
    */
    uint32_t firstSector;
    /*!
    ** The number of sectors to be used, '0' means use all available sectors
    */
    uint32_t sectorCount;
    /*!
    ** A flag which decides whether the flash is used by a file system
    ** or not, within a file system an erased sector contains zeros instead
    ** of 0xFF for every byte.
    */
    uint32_t fsysFlag;
    /*!
    ** A flag which decides whether to force flushing the current sector
    ** buffer data to flash memory while closing a handle.
    */
    uint32_t forceFlush;

    int flag;
} FAPEX_VRFLASH_OpenParamsT;

/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
FAPEX_SYS_HandleT* FAPEX_VRFLASH_Open( FAPEX_VRFLASH_OpenParamsT *paramsPtr,
                                       int32_t* errorCodePtr );
int32_t            FAPEX_VRFLASH_Close( FAPEX_SYS_HandleT *flash_handle_ptr );
const char_t*      FAPEX_VRFLASH_GetDeviceName( FAPEX_SYS_HandleT handle );
int32_t            FAPEX_VRFLASH_GetSectorIndex( FAPEX_SYS_HandleT handle,
                                                 uint32_t address );
int32_t            FAPEX_VRFLASH_GetSectorStart( FAPEX_SYS_HandleT handle,
                                                 uint32_t sectorIndex );
int32_t            FAPEX_VRFLASH_GetSectorBytes( FAPEX_SYS_HandleT handle,
                                                 uint32_t sectorIndex );
int32_t            FAPEX_VRFLASH_ReadBytes( FAPEX_SYS_HandleT handle,
                                            uint32_t address,
                                            uint32_t byteCount,
                                            void* bufferPtr );
int32_t            FAPEX_VRFLASH_WriteBytes( FAPEX_SYS_HandleT handle,
                                             uint32_t address,
                                             uint32_t byteCount,
                                             const void* bufferPtr );
int32_t            FAPEX_VRFLASH_WriteBytesDirect( FAPEX_SYS_HandleT handle,
                                             uint32_t address,
                                             uint32_t byteCount,
                                             const void* bufferPtr );
int32_t            FAPEX_VRFLASH_EraseSector( FAPEX_SYS_HandleT handle,
                                              uint32_t sectorIndex );
int32_t            FAPEX_VRFLASH_GetDeviceSize( FAPEX_SYS_HandleT handle );
int32_t            FAPEX_VRFLASH_GetSectorCount( FAPEX_SYS_HandleT handle );
int32_t            FAPEX_VRFLASH_Flush( FAPEX_SYS_HandleT handle );

#endif /* _FAPEX_ABST_LIBVRFLASH_H */
