/*!
*************************************************************************
**
** \file    abst_libblte.c
**
** \brief   abstraction library source file (BLTE driver).
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#else
#include <linux/ioctl.h>
#endif
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "abst_common_priv.h"
#include "fapex/abst_libblte.h"

/*!
*******************************************************************************
**
** \brief  open method.
**         This function is called with the systemcall "open".
**
** \param  id     The device identifier.
** \param  flags  This parameter indicate a open mode (read only/write only/read-write).<br>
**                This implementation supports only read-write mode.
** \param  argptr argument pointer (NULL)
**
** \retval >=0 process OK, new file descriptor is returned.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXOPENRET_T FAPEX_BLTE_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr)
{
    return fapex_COMMON_Open(id, flags, argptr, FAPEX_IOCCOM_BLTE_OPEN);
}

/*!
*******************************************************************************
**
** \brief  release method.
**         This function is called with the systemcall "close".
**
** \param  id     The identifier for this descriptor.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXRELRET_T FAPEX_BLTE_Close(FAPEXRELARG1_T id)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Release(id);
#else /* !_LINUX_FAPI */
    return close(id);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  ioctl method.
**         This function is called with the systemcall "ioctl".
**
** \param  id     The identifier for this descriptor.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
**<hr>
** ioctl command(\e cmd) type:
** \anchor blte_cmd_open
** \li #FAPEX_IOCCOM_BLTE_OPEN<br>
**      Passs FAPEX_BLTE_OpenParamsT pointer to pArg
**
** \anchor blte_cmd_color_key
** \li #FAPEX_IOCCOM_BLTE_COLOR_KEY<br>
**      Pass FAPEX_BLTE_ColorKeyParamT pointer to pArg
**
** \anchor blte_cmd_shading_param
** \li #FAPEX_IOCCOM_BLTE_SHADING_PARAM<br>
**      Pass FAPEX_BLTE_ShadingParamsT pointer to pArg
**
** \anchor blte_cmd_blending_param
** \li #FAPEX_IOCCOM_BLTE_BLENDING_PARAM<br>
**      Pass FAPEX_BLTE_BlendingParamsT pointer to pArg
**
** \anchor blte_cmd_mtx_param
** \li #FAPEX_IOCCOM_BLTE_MTX_PARAM<br>
**      Pass FAPEX_BLTE_MtxParamT pointer to pArg
**
** \anchor blte_cmd_clut_param
** \li #FAPEX_IOCCOM_BLTE_CLUT_PARAM<br>
**      Pass FAPEX_BLTE_ClutParamT pointer to pArg
**
** \anchor blte_cmd_clip_param
** \li #FAPEX_IOCCOM_BLTE_CLIP_PARAM<br>
**      Pass FAPEX_BLTE_ClipParamT pointer to pArg
**
** \anchor blte_cmd_source_format
** \li #FAPEX_IOCCOM_BLTE_SOURCE_FORMAT<br>
**      Pass FAPEX_BLTE_FormatParamT pointer to pArg
**
** \anchor blte_cmd_draw_rect
** \li #FAPEX_IOCCOM_BLTE_DRAW_RECT<br>
**      Pass FAPEX_BLTE_DrawRectParamT pointer to pArg
**
** \anchor blte_cmd_draw_font
** \li #FAPEX_IOCCOM_BLTE_DRAW_FONT<br>
**      Pass FAPEX_BLTE_DrawFontParamT pointer to pArg
**
** \anchor blte_cmd_draw_uline
** \li #FAPEX_IOCCOM_BLTE_DRAW_ULINE<br>
**      Pass FAPEX_BLTE_DrawUlineParamT pointer to pArg
**
** \anchor blte_cmd_draw_copy
** \li #FAPEX_IOCCOM_BLTE_DRAW_COPY<br>
**      Pass FAPEX_BLTE_CopyParamT pointer to pArg
**
** \anchor blte_cmd_draw_bitmap
** \li #FAPEX_IOCCOM_BLTE_DRAW_BITMAP<br>
**      Pass FAPEX_BLTE_DrawBitmapParamT pointer to pArg
**
** \anchor blte_cmd_set_frame
** \li #FAPEX_IOCCOM_BLTE_SET_FRAME<br>
**      Pass FAPEX_BLTE_SetFrameBaseParamsT pointer to pArg
**
******************************************************************************/
FAPEXIOCRET_T FAPEX_BLTE_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Ioctl(id, cmd, pArg);
#else /* !_LINUX_FAPI */
    return ioctl(id, (int)cmd, pArg);
#endif /* !_LINUX_FAPI */
}

