/*!
*************************************************************************
**
** \file    abst_libclkpwr.c
**
** \brief   abstraction library source file (CLKPWR driver).
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#else
#include <linux/ioctl.h>
#endif
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "abst_common_priv.h"

/*!
*******************************************************************************
**
** \brief  open method.
**         This function is called with the systemcall "open".
**
** \param  id     The device identifier.
** \param  flags  This parameter indicate a open mode (read only/write only/read-write).<br>
**                This implementation supports only read-write mode.
** \param  argptr argument pointer (NULL)
**
** \retval >=0 process OK, new file descriptor is returned.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXOPENRET_T FAPEX_CLKPWR_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr)
{
    return fapex_COMMON_Open(id, flags, argptr, FAPEX_IOCCOM_OPEN);
}

/*!
*******************************************************************************
**
** \brief  release method.
**         This function is called with the systemcall "close".
**
** \param  id     The identifier for this descriptor.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXRELRET_T FAPEX_CLKPWR_Close(FAPEXRELARG1_T id)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Release(id);
#else /* !_LINUX_FAPI */
    return close(id);
#endif /* !_LINUX_FAPI */
}

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  read method.
**         This function is called with the systemcall "read".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the destination buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually read.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXREADRET_T FAPEX_CLKPWR_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T buf, FAPEXREADARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Read(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)read(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  write method.
**         This function is called with the systemcall "write".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the source buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually write.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXWRTRET_T FAPEX_CLKPWR_Write(FAPEXWRTARG1_T id, FAPEXWRTARG2_T buf, FAPEXWRTARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Write(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)write(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}
#endif

/*!
*******************************************************************************
**
** \brief  ioctl method.
**         This function is called with the systemcall "ioctl".
**
** \param  id     The identifier for this descriptor.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
** <hr>
** ioctl command(\e cmd) type:
** \anchor clkpwr_cmd_start
** \li #FAPEX_IOCCOM_CLKPWR_START<br>
**      wakeup CLKPWR.<br>
**      Set the mask(uint32_t) which contains the power down modules to \e pArg.<br>
**      The value of mask is the bitwise-inclusive OR of values from the following list.
**      - #FAPEX_CLKPWR_ARM
**      - #FAPEX_CLKPWR_AUDIO
**      - #FAPEX_CLKPWR_AUDIO_PROCESSOR
**      - #FAPEX_CLKPWR_AXI
**      - #FAPEX_CLKPWR_DDR2_SDRAM
**      - #FAPEX_CLKPWR_DMA
**      - #FAPEX_CLKPWR_ETH
**      - #FAPEX_CLKPWR_I2C
**      - #FAPEX_CLKPWR_ICC_APB_EFUSE
**      - #FAPEX_CLKPWR_IRQ_CONTROL
**      - #FAPEX_CLKPWR_GPIO
**      - #FAPEX_CLKPWR_SFLASH
**      - #FAPEX_CLKPWR_SSP
**      - #FAPEX_CLKPWR_TSD
**      - #FAPEX_CLKPWR_UART_FPC
**      - #FAPEX_CLKPWR_UPI
**      - #FAPEX_CLKPWR_USB
**      - #FAPEX_CLKPWR_VIDEO
**      .
**      ex. The following are in the above bit set.
**      - #FAPEX_CLKPWR_BASIC_POWER_DOWN<br>
**        Disables audio (also processor), tsd, video and reduces ARM speed to 162 MHz.
**      - #FAPEX_CLKPWR_GENERIC_POWER_DOWN<br>
**        Additionally disables...
**      - #FAPEX_CLKPWR_UNIVERSAL_POWER_DOWN<br>
**        Disables all allowed modules.
**      - #FAPEX_CLKPWR_BASIC_WAKE_UP
**      - #FAPEX_CLKPWR_GENERIC_WAKE_UP
**      - #FAPEX_CLKPWR_UNIVERSAL_WAKE_UP
**
** \anchor clkpwr_cmd_stop
** \li #FAPEX_IOCCOM_CLKPWR_STOP<br>
**      standby CLKPWR.<br>
**      Set the mask(uint32_t) which contains the power down modules to \e pArg.<br>
**      The value of mask is the bitwise-inclusive OR of values from the following list.
**      - #FAPEX_CLKPWR_ARM
**      - #FAPEX_CLKPWR_AUDIO
**      - #FAPEX_CLKPWR_AUDIO_PROCESSOR
**      - #FAPEX_CLKPWR_AXI
**      - #FAPEX_CLKPWR_DDR2_SDRAM
**      - #FAPEX_CLKPWR_DMA
**      - #FAPEX_CLKPWR_ETH
**      - #FAPEX_CLKPWR_I2C
**      - #FAPEX_CLKPWR_ICC_APB_EFUSE
**      - #FAPEX_CLKPWR_IRQ_CONTROL
**      - #FAPEX_CLKPWR_GPIO
**      - #FAPEX_CLKPWR_SFLASH
**      - #FAPEX_CLKPWR_SSP
**      - #FAPEX_CLKPWR_TSD
**      - #FAPEX_CLKPWR_UART_FPC
**      - #FAPEX_CLKPWR_UPI
**      - #FAPEX_CLKPWR_USB
**      - #FAPEX_CLKPWR_VIDEO
**      .
**      ex. The following are in the above bit set.
**      - #FAPEX_CLKPWR_BASIC_POWER_DOWN<br>
**        Disables audio (also processor), tsd, video and reduces ARM speed to 162 MHz.
**      - #FAPEX_CLKPWR_GENERIC_POWER_DOWN<br>
**        Additionally disables...
**      - #FAPEX_CLKPWR_UNIVERSAL_POWER_DOWN<br>
**        Disables all allowed modules.
**      - #FAPEX_CLKPWR_BASIC_WAKE_UP
**      - #FAPEX_CLKPWR_GENERIC_WAKE_UP
**      - #FAPEX_CLKPWR_UNIVERSAL_WAKE_UP
**
** \anchor clkpwr_cmd_reset
** \li #FAPEX_IOCCOM_RESET<br>
**      reset CLKPWR.<br>
**      Set the mask(uint32_t) which contains the power down modules to \e pArg.<br>
**      The value of mask is the bitwise-inclusive OR of values from the following list.
**      - #FAPEX_CLKPWR_ARM
**      - #FAPEX_CLKPWR_AUDIO
**      - #FAPEX_CLKPWR_AUDIO_PROCESSOR
**      - #FAPEX_CLKPWR_AXI
**      - #FAPEX_CLKPWR_DDR2_SDRAM
**      - #FAPEX_CLKPWR_DMA
**      - #FAPEX_CLKPWR_ETH
**      - #FAPEX_CLKPWR_I2C
**      - #FAPEX_CLKPWR_ICC_APB_EFUSE
**      - #FAPEX_CLKPWR_IRQ_CONTROL
**      - #FAPEX_CLKPWR_GPIO
**      - #FAPEX_CLKPWR_SFLASH
**      - #FAPEX_CLKPWR_SSP
**      - #FAPEX_CLKPWR_TSD
**      - #FAPEX_CLKPWR_UART_FPC
**      - #FAPEX_CLKPWR_UPI
**      - #FAPEX_CLKPWR_USB
**      - #FAPEX_CLKPWR_VIDEO
**      .
**      ex. The following are in the above bit set.
**      - #FAPEX_CLKPWR_BASIC_POWER_DOWN<br>
**        Disables audio (also processor), tsd, video and reduces ARM speed to 162 MHz.
**      - #FAPEX_CLKPWR_GENERIC_POWER_DOWN<br>
**        Additionally disables...
**      - #FAPEX_CLKPWR_UNIVERSAL_POWER_DOWN<br>
**        Disables all allowed modules.
**      - #FAPEX_CLKPWR_BASIC_WAKE_UP
**      - #FAPEX_CLKPWR_GENERIC_WAKE_UP
**      - #FAPEX_CLKPWR_UNIVERSAL_WAKE_UP
**
** \anchor clkpwr_cmd_get_status
** \li #FAPEX_IOCCOM_CLKPWR_GET_STATUS<br>
**      get CLKPWR status.<br>
**      Set pointer of status data(#FAPEX_CLKPWR_RUNNING, #FAPEX_CLKPWR_STAND_BY) field
**      (int32_t) to \e pArg.<br>
**      The status value will be set to this field.
**
** \anchor clkpwr_cmd_watchdog_start
** \li #FAPEX_IOCCOM_CLKPWR_WATCHDOG_START<br>
**      start Watchdog.<br>
**      Set the watchdog start value(#FAPEX_CLKPWR_WATCHDOG_MIN_VALUE
**      .. #FAPEX_CLKPWR_WATCHDOG_MAX_VALUE) to \e pArg.
**
** \anchor clkpwr_cmd_watchdog_stop
** \li #FAPEX_IOCCOM_CLKPWR_WATCHDOG_STOP<br>
**      stop Watchdog. (The \e pArg is ignored)
**
** \anchor clkpwr_cmd_watchdog_reset
** \li #FAPEX_IOCCOM_CLKPWR_WATCHDOG_RESET<br>
**      reset Watchdog (The \e pArg is ignored)
**
** \anchor clkpwr_cmd_set_pwm
** \li #FAPEX_IOCCOM_CLKPWR_SET_PWM<br>
**      Sets the pulse width modulation (PWM).<br>
**      Set the PWM counter value (max 10bit) to \e pArg.
**
** \anchor clkpwr_cmd_get_pwm
** \li #FAPEX_IOCCOM_CLKPWR_GET_PWM<br>
**      Gets the pulse width modulation (PWM).<br>
**      Set pointer of int32_t data field to \e pArg.<br>
**      The PWM counter value (max 10bit) will be set to this field.
**
** \anchor clkpwr_cmd_set_deepmode
** \li #FAPEX_IOCCOM_CLKPWR_SET_DEEPMODE<br>
**     enter 'deep power down' power saving mode 
**
** \anchor clkpwr_cmd_get_wakeupsrc
** \li #FAPEX_IOCCOM_CLKPWR_GET_WAKEUPSRC<br>
**    Gets the the cause of an wakeup from the 'deep power down' power saving mode.<br> 
** 
******************************************************************************/
FAPEXIOCRET_T FAPEX_CLKPWR_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Ioctl(id, cmd, pArg);
#else /* !_LINUX_FAPI */
    return ioctl(id, (int)cmd, pArg);
#endif /* !_LINUX_FAPI */
}
