/*!
*************************************************************************
**
** \file    abst_libflash.c
**
** \brief   abstraction library source file (FLASH driver).
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#else
#include <linux/ioctl.h>
#endif
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "abst_common_priv.h"
#include "fapex/abst_libflash.h"

/*!
*******************************************************************************
**
** \brief  open method.
**         This function is called with the systemcall "open".
**
** \param  id     The device identifier.
** \param  flags  This parameter indicate a open mode (read only/write only/read-write).<br>
**                This implementation supports only read-write mode.
** \param  argptr argument pointer (NULL)
**
** \retval >=0 process OK, new file descriptor is returned.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXOPENRET_T FAPEX_FLASH_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr)
{
    return fapex_COMMON_Open(id, flags, argptr, FAPEX_IOCCOM_FLASH_OPEN);
}

/*!
*******************************************************************************
**
** \brief  release method.
**         This function is called with the systemcall "close".
**
** \param  id     The identifier for this descriptor.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXRELRET_T FAPEX_FLASH_Close(FAPEXRELARG1_T id)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Release(id);
#else /* !_LINUX_FAPI */
    return close(id);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  ioctl method.
**         This function is called with the systemcall "ioctl".
**
** \param  id     The identifier for this descriptor.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
** <hr>
** ioctl command(\e cmd) type:
** \anchor flash_cmd_open
** \li #FAPEX_IOCCOM_FLASH_OPEN<br>
**      open the flash for operation.<br>
**
** \anchor flash_cmd_erase
** \li #FAPEX_IOCCOM_FLASH_ERASE_CHIP<br>
**      erase the total flash.<br>
**
** \anchor flash_cmd_erase_sector
** \li #FAPEX_IOCCOM_FLASH_ERASE_SECTOR<br>
**      erase the flash sector.<br>
**
** \anchor flash_cmd_write_bytes
** \li #FAPEX_IOCCOM_FLASH_WRITE_BYTES<br>
**      write to the flash.<br>
**      write the bytes from \e pArg to flash.
**
** \anchor flash_cmd_read_bytes
** \li #FAPEX_IOCCOM_FLASH_READ_BYTES<br>
**      get bytes from flash.<br>
**      get bytes from the flash to \e pArg.<br>
**
** \anchor flash_cmd_get_device_size
** \li #FAPEX_IOCCOM_FLASH_GET_DEVICE_SIZE<br>
**      get the deivce size.<br>
**      get the flash size to \e pArg.
**
** \anchor flash_cmd_get_device_id
** \li #FAPEX_IOCCOM_FLASH_GET_DEVICE_ID<br>
**      get flash device id.<br>
**      get flash device id to \e pArg.
**
** \anchor flash_cmd_get_sector_count
** \li #FAPEX_IOCCOM_FLASH_GET_SECTOR_COUNT<br>
**      get flash sector count.<br>
**      get flash sector count to \e pArg.<br>
**
** \anchor flash_cmd_get_sector_bytes
** \li #FAPEX_IOCCOM_FLASH_GET_SECTOR_BYTES<br>
**      get the sector size.<br>
**      get the sector size to \e pArg.
**
** \anchor flash_cmd_get_sector_start
** \li #FAPEX_IOCCOM_FLASH_GET_SECTOR_START<br>
**      get the sector start address.<br>
**      get the sector start address to \e pArg.
**
******************************************************************************/
FAPEXIOCRET_T FAPEX_FLASH_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Ioctl(id, cmd, pArg);
#else /* !_LINUX_FAPI */
    return ioctl(id, (int)cmd, pArg);
#endif /* !_LINUX_FAPI */
}

