/*!
*************************************************************************
**
** \file    abst_libfpc.c
**
** \brief   abstraction library source file (FPC driver).
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#else
#include <linux/ioctl.h>
#endif
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_libfpc.h"
#include "abst_common_priv.h"

/*!
*******************************************************************************
**
** \brief  open method.
**         This function is called with the systemcall "open".
**
** \param  id     The device identifier.
** \param  flags  This parameter indicate a open mode (read only/write only/read-write).<br>
**                This implementation supports only read-write mode.
** \param  argptr argument pointer (NULL)
**
** \retval >=0 process OK, new file descriptor is returned.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXOPENRET_T FAPEX_FPC_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr)
{
    return fapex_COMMON_Open(id, flags, argptr, FAPEX_IOCCOM_OPEN);
}

/*!
*******************************************************************************
**
** \brief  release method.
**         This function is called with the systemcall "close".
**
** \param  id     The identifier for this descriptor.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXRELRET_T FAPEX_FPC_Close(FAPEXRELARG1_T id)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Release(id);
#else /* !_LINUX_FAPI */
    return close(id);
#endif /* !_LINUX_FAPI */
}

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  read method.
**         This function is called with the systemcall "read".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the destination buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually read.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXREADRET_T FAPEX_FPC_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T buf, FAPEXREADARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Read(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)read(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  write method.
**         This function is called with the systemcall "write".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the source buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually write.
** \retval <0  process fails, its value is set to errno.
**

******************************************************************************/
FAPEXWRTRET_T FAPEX_FPC_Write(FAPEXWRTARG1_T id, FAPEXWRTARG2_T buf, FAPEXWRTARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Write(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)write(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}
#endif

/*!
*******************************************************************************
**
** \brief  ioctl method.
**         This function is called with the systemcall "ioctl".
**
** \param  id     The identifier for this descriptor.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
** <hr>
** ioctl command(\e cmd) type:
** \anchor fpc_cmd_set_brightness
** \li #FAPEX_IOCCOM_FPC_SET_BRIGHTNESS<br>
**      set display's brightness in percent.<br>
**      Set brightness value (uint32_t) to \e pArg.
**
** \anchor fpc_cmd_set_change_brightness
** \li #FAPEX_IOCCOM_FPC_SET_CAHNGE_BRIGHTNESS<br>
**      set display's brightness steps.<br>
**      Set step value (int32_t) to \e pArg.
**
** \anchor fpc_cmd_set_display_segment
** \li #FAPEX_IOCCOM_FPC_SET_DISPLAY_SEGMENT<br>
**      set 4 display segments.<br>
**      Set pointer of #FAPEX_FPC_DispSegParamStrT data field to \e pArg.
**
** \anchor fpc_cmd_set_digit
** \li #FAPEX_IOCCOM_FPC_SET_DIGIT<br>
**      set digit.<br>
**      Set pointer of #FAPEX_FPC_DigitParamStrT data field to \e pArg.
**
** \anchor fpc_cmd_get_digit
** \li #FAPEX_IOCCOM_FPC_GET_DIGIT<br>
**      get digit.<br>
**      Set pointer of #FAPEX_FPC_DigitParamStrT data field to \e pArg.<br>
**      The digit parameter value will be set to this field.
**
** \anchor fpc_cmd_set_increment_digit
** \li #FAPEX_IOCCOM_FPC_SET_INCREMENT_DIGIT<br>
**      set increment for one of the 4 segments.<br>
**      Set pointer of #FAPEX_FPC_DigitParamStrT data field to \e pArg.
**
** \anchor fpc_cmd_set_decrement_digit
** \li #FAPEX_IOCCOM_FPC_SET_DECREMENT_DIGIT<br>
**      set decrement for one of the 4 segments.<br>
**      Set pointer of #FAPEX_FPC_DigitParamStrT data field to \e pArg.
**
** \anchor fpc_cmd_get_dot
** \li #FAPEX_IOCCOM_FPC_GET_DOT<br>
**      get dot status.<br>
**      Set pointer of #FAPEX_FPC_DotParamStrT data field to \e pArg.<br>
**      The dot status value will be set to this field.
**
** \anchor fpc_cmd_set_dot_toggle
** \li #FAPEX_IOCCOM_FPC_SET_DOT_TOGGLE<br>
**      set toggle dot position.<br>
**      Set position value (uint32_t) to \e pArg.
**
** \anchor fpc_cmd_set_integer_value
** \li #FAPEX_IOCCOM_FPC_SET_INTEGER_VALUE<br>
**      set the value on the 4-segment display.<br>
**      Set integer value (int32_t) to \e pArg.
**
** \anchor fpc_cmd_get_integer_value
** \li #FAPEX_IOCCOM_FPC_GET_INTEGER_VALUE<br>
**      get the value on the 4-segment display.<br>
**      Set pointer of integer data field (int32_t) to \e pArg.<br>
**      The displayed value will be set to this field.
**
** \anchor fpc_cmd_set_increment_interger_value
** \li #FAPEX_IOCCOM_FPC_SET_INCREMENT_INTEGER_VALUE<br>
**      set increment value shown on the 4-segment display. (The \e pArg is ignored)
**
** \anchor fpc_cmd_set_decrement_interger_value
** \li #FAPEX_IOCCOM_FPC_SET_DECREMENT_INTEGER_VALUE<br>
**      set decrement value shown on the 4-segment display. (The \e pArg is ignored)
**
** \anchor fpc_cmd_set_scroll_text
** \li #FAPEX_IOCCOM_FPC_SET_SCROLL_TEXT<br>
**      set csroll test.<br>
**      Set pointer of #FAPEX_FPC_MessageParamStrT data field to \e pArg.
**
** \anchor fpc_cmd_get_button_notify
** \li #FAPEX_IOCCOM_FPC_GET_BUTTON_NOTIFY<br>
**      wait for button event.<br>
**      Set pointer of waiting button event flag (int32_t) to \e pArg.<br>
**      The button event flag will be set to this field.<br>
**      This event button flag should be the bitwise-inclusive OR of values from the following list.
**      - #FAPEX_FPC_EVENT_NONE<br>
**        none event
**      - #FAPEX_FPC_EVENT_BUTTON1<br>
**        button1 event
**      - #FAPEX_FPC_EVENT_BUTTON2<br>
**        button2 event
**      - #FAPEX_FPC_EVENT_BUTTON3<br>
**        button3 event
**      - #FAPEX_FPC_EVENT_BUTTON4<br>
**        button4 event
**      - #FAPEX_FPC_EVENT_BUTTON5<br>
**        button5 event
**      - #FAPEX_FPC_EVENT_BUTTON6<br>
**        button6 event
**      - #FAPEX_FPC_EVENT_BUTTON7<br>
**        button7 event
**      - #FAPEX_FPC_EVENT_BUTTON8<br>
**        button8 event
**      .
**      This command is waited for until the button event is generated.
**
******************************************************************************/
FAPEXIOCRET_T FAPEX_FPC_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Ioctl(id, cmd, pArg);
#else /* !_LINUX_FAPI */
    return ioctl(id, (int)cmd, pArg);
#endif /* !_LINUX_FAPI */
}
