/*!
*************************************************************************
**
** \file    abst_libhdmi.c
**
** \brief   abstraction library source file (HDMI driver).
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#else
#include <linux/ioctl.h>
#endif
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_libhdmi.h"
#include "abst_common_priv.h"

/*!
*******************************************************************************
**
** \brief  open method.
**         This function is called with the systemcall "open".
**
** \param  id     The device identifier.
** \param  flags  This parameter indicate a open mode (read only/write only/read-write).<br>
**                This implementation supports only read-write mode.
** \param  argptr argument pointer (NULL)
**
** \retval >=0 process OK, new file descriptor is returned.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXOPENRET_T FAPEX_HDMI_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr)
{
    return fapex_COMMON_Open(id, flags, argptr, FAPEX_IOCCOM_OPEN);
}

/*!
*******************************************************************************
**
** \brief  release method.
**         This function is called with the systemcall "close".
**
** \param  id     The identifier for this descriptor.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXRELRET_T FAPEX_HDMI_Close(FAPEXRELARG1_T id)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Release(id);
#else /* !_LINUX_FAPI */
    return close(id);
#endif /* !_LINUX_FAPI */
}

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  read method.
**         This function is called with the systemcall "read".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the destination buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually read.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXREADRET_T FAPEX_HDMI_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T buf, FAPEXREADARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Read(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)read(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  write method.
**         This function is called with the systemcall "write".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the source buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually write.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXWRTRET_T FAPEX_HDMI_Write(FAPEXWRTARG1_T id, FAPEXWRTARG2_T buf, FAPEXWRTARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Write(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)write(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}
#endif

/*!
*******************************************************************************
**
** \brief  ioctl method.
**         This function is called with the systemcall "ioctl".
**
** \param  id     The identifier for this descriptor.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
** <hr>
** ioctl command(\e cmd) type:
** \anchor hdmi_cmd_check_hpd
** \li #FAPEX_IOCCOM_HDMI_CHECK_HPD<br>
**      check HPD status.<br>
**      Set pointer of status data field (int32_t) to \e pArg.<br>
**      The status value (bit6: HPD_int bit5: RSEN_int bit2: HPD_status bit1: RSEN_status)
**      will be set to this field.
**
** \anchor hdmi_cmd_read_edid
** \li #FAPEX_IOCCOM_HDMI_READ_EDID<br>
**      read the EDID status.<br>
**      Set pointer of status data field (int32_t) to \e pArg.<br>
**      The status value will be one of following.
**      - #FAPEX_HDMI_STATUS_OK<br>
**        successful
**      - #FAPEX_HDMI_STATUS_ERR_EDID<br>
**        HDMI EDID read error
**      - #FAPEX_HDMI_STATUS_ERR_DDC_TIMEOUT<br>
**        HDMI DDC timeout error
**      - #FAPEX_ERR_INVALID_HANDLE<br>
**        Invalid handle was passed
**
** \anchor hdmi_cmd_get_edid_info
** \li #FAPEX_IOCCOM_HDMI_GET_EDID_INFO<br>
**      get the EDID information data.<br>
**      Set pointer of #FAPEX_HDMI_EdidInfoStrT data field to \e pArg.<br>
**      The EDID information value will be set to this field.
**
** \anchor hdmi_cmd_get_edid_physaddr
** \li #FAPEX_IOCCOM_HDMI_GET_EDID_PHYSADDR<br>
**      get the physical address in EDID.<br>
**      Set pointer of address data field (uint32_t) to \e pArg.<br>
**      The physical address value will be set to this field.
**
** \anchor hdmi_cmd_link_hdcp
** \li #FAPEX_IOCCOM_HDMI_LINK_HDCP<br>
**      Linkage HDCP.<br>
**      Set pointer of status data field (int32_t) to \e pArg.<br>
**      The linkage status value will be one of following.
**      - #FAPEX_HDMI_STATUS_OK<br>
**        successful
**      - #FAPEX_HDMI_STATUS_ERR_HDCP_TIMEOUT<br>
**        HDMI HDCP timeout occured
**      - #FAPEX_HDMI_STATUS_ERR_HDCP_MAX_DEVS_EXCEED<br>
**        HDMI HDCP exceed device count
**      - #FAPEX_HDMI_STATUS_ERR_HDCP_MAX_CASCADE_EXCEEDED<br>
**        HDMI HDCP cascade exceeded
**      - #FAPEX_HDMI_STATUS_ERR_HDCP_SHA_1_MISMATCH<br>
**        HDMI HDCP SHA-1 mismatch
**      - #FAPEX_HDMI_STATUS_ERR_HDCP_AN_DDC<br>
**        HDMI HDCP AN DDC Error
**      - #FAPEX_HDMI_STATUS_ERR_HDCP_AKSV_DDC<br>
**        HDMI HDCP AKSV DDC Error
**      - #FAPEX_HDMI_STATUS_ERR_HDCP_BKSV<br>
**        HDMI HDCP BKSV Error
**      - #FAPEX_HDMI_STATUS_ERR_HDCP_RI_FAILURE<br>
**        HDMI HDCP RI failure
**      - #FAPEX_HDMI_STATUS_ERR_HDCP_RI_NOT_ABAILABLE<br>
**        HDMI HDCP RI failure
**      - #FAPEX_HDMI_STATUS_ERR_HPD_NOT_DETECT<br>
**        HDMI HPD not detect
**      - #FAPEX_HDMI_STATUS_ERR_EXIST_IN_REVOCATION_LISTS<br>
**        HDMI HDCP BKSV exist in Revocation Lists
**      - #FAPEX_HDMI_STATUS_ERR_HDCP_NOT_SUPPORT<br>
**        HDCP feature is not available
**      - #FAPEX_HDMI_STATUS_ERR_HDCP_AKSV<br>
**        HDMI HDCP AKSV Error
**
** \anchor hdmi_cmd_check_hdcp_state
** \li #FAPEX_IOCCOM_HDMI_CHECK_HDCP_STATE<br>
**      check HDCP status.<br>
**      Set pointer of status data field (int32_t) to \e pArg.<br>
**      The linkage status value will be one of following.
**      - #FAPEX_HDMI_HDCP_STATUS_FIRST_AUTHENTICATION
**      - #FAPEX_HDMI_HDCP_STATUS_WAIT_FOR_RXRDY
**      - #FAPEX_HDMI_HDCP_STATUS_SECOND_AUTHENTICATION
**      - #FAPEX_HDMI_HDCP_STATUS_WAIT_FIFORDY
**      - #FAPEX_HDMI_HDCP_STATUS_FINISH_REPEATER_CHECK
**      - #FAPEX_HDMI_HDCP_STATUS_THIRD_AUTHENTICATION
**      - #FAPEX_HDMI_HDCP_STATUS_REQ_NOHDCP
**      - #FAPEX_HDMI_HDCP_STATUS_NO_ENCRYPTION
**
** \anchor hdmi_cmd_set_dynamic_hdcp
** \li #FAPEX_IOCCOM_HDMI_SET_DYNAMIC_HDCP<br>
**      set dynamic on/off HDCP.<br>
**      Set on/off flag (int32_t) to \e pArg.<br>
**      ex.
**      - #FAPEX_HDMI_HDCP_DYNAMIC_ON
**      - #FAPEX_HDMI_HDCP_DYNAMIC_OFF
**
******************************************************************************/
FAPEXIOCRET_T FAPEX_HDMI_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Ioctl(id, cmd, pArg);
#else /* !_LINUX_FAPI */
    return ioctl(id, (int)cmd, pArg);
#endif /* !_LINUX_FAPI */
}
