/*!
*************************************************************************
**
** \file    abst_libhdmicec.c
**
** \brief   abstraction library source file (HDMI-CEC driver).
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#else
#include <linux/ioctl.h>
#endif
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_libhdmicec.h"
#include "abst_common_priv.h"

/*!
*******************************************************************************
**
** \brief  open method.
**         This function is called with the systemcall "open".
**
** \param  id     The device identifier.
** \param  flags  This parameter indicate a open mode (read only/write only/read-write).<br>
**                This implementation supports only read-write mode.
** \param  argptr argument pointer (NULL)
**
** \retval >=0 process OK, new file descriptor is returned.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXOPENRET_T FAPEX_HDMICEC_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr)
{
    return fapex_COMMON_Open(id, flags, argptr, FAPEX_IOCCOM_OPEN);
}

/*!
*******************************************************************************
**
** \brief  release method.
**         This function is called with the systemcall "close".
**
** \param  id     The identifier for this descriptor.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXRELRET_T FAPEX_HDMICEC_Close(FAPEXRELARG1_T id)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Release(id);
#else /* !_LINUX_FAPI */
    return close(id);
#endif /* !_LINUX_FAPI */
}

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  read method.
**         This function is called with the systemcall "read".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the destination buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually read.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXREADRET_T FAPEX_HDMICEC_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T buf, FAPEXREADARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Read(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)read(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  write method.
**         This function is called with the systemcall "write".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the source buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually write.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXWRTRET_T FAPEX_HDMICEC_Write(FAPEXWRTARG1_T id, FAPEXWRTARG2_T buf, FAPEXWRTARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Write(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)write(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}
#endif

/*!
*******************************************************************************
**
** \brief  ioctl method.
**         This function is called with the systemcall "ioctl".
**
** \param  id     The identifier for this descriptor.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
** <hr>
** ioctl command(\e cmd) type:
** \anchor hdmicec_cmd_get_send_status
** \li #FAPEX_IOCCOM_HDMICEC_GET_SEND_STATUS<br>
**      get send status for sending.<br>
**      Set pointer of status data field (uint32_t) to \e pArg.<br>
**      The status value will be one of following.
**      - #FAPEX_HDMICEC_AWAITING_SENDING<br>
**        state of awaiting sending
**      - #FAPEX_HDMICEC_DURING_SENDING<br>
**        state of during sending
**      - #FAPEX_HDMICEC_SENDING_COMPLETE<br>
**        state of sending complete
**      - #FAPEX_HDMICEC_SENDING_ERROR<br>
**        state of sending error
**      .
**      This command is waited for until the event is generated.
**
** \anchor hdmicec_cmd_get_receive_status
** \li #FAPEX_IOCCOM_HDMICEC_GET_RECEIVE_STATUS<br>
**      get receive status in receiving.<br>
**      Set pointer of status data field (uint32_t) to \e pArg.<br>
**      The status value will be one of following.
**      - #FAPEX_HDMICEC_AWAITING_RECEIVING<br>
**        state of awaiting receiving
**      - #FAPEX_HDMICEC_DURING_RECEIVING<br>
**        state of during receving
**      - #FAPEX_HDMICEC_RECEPTION_COMPLETE<br>
**        state of reception complete
**      - #FAPEX_HDMICEC_RECEPTION_ERROR<br>
**        state of reception error
**      .
**      This command is waited for until the event is generated.
**
** \anchor hdmicec_cmd_check_send_status
** \li #FAPEX_IOCCOM_HDMICEC_CHECK_SEND_STATUS<br>
**      check CEC communication status for sending.<br>
**      Set pointer of status data field (uint32_t) to \e pArg.<br>
**      The status value will be one of following.
**      - #FAPEX_HDMICEC_AWAITING_SENDING<br>
**        state of awaiting sending
**      - #FAPEX_HDMICEC_DURING_SENDING<br>
**        state of during sending
**      - #FAPEX_HDMICEC_SENDING_COMPLETE<br>
**        state of sending complete
**      - #FAPEX_HDMICEC_SENDING_ERROR<br>
**        state of sending error
**
** \anchor hdmicec_cmd_check_receive_status
** \li #FAPEX_IOCCOM_HDMICEC_CHECK_RECEIVE_STATUS<br>
**      check CEC communication status in receiving.<br>
**      Set pointer of status data field (uint32_t) to \e pArg.<br>
**      The status value will be one of following.
**      - #FAPEX_HDMICEC_AWAITING_RECEIVING<br>
**        state of awaiting receiving
**      - #FAPEX_HDMICEC_DURING_RECEIVING<br>
**        state of during receving
**      - #FAPEX_HDMICEC_RECEPTION_COMPLETE<br>
**        state of reception complete
**      - #FAPEX_HDMICEC_RECEPTION_ERROR<br>
**        state of reception error
**
** \anchor hdmicec_cmd_set_message
** \li #FAPEX_IOCCOM_HDMICEC_SET_MESSAGE<br>
**      set CEC message parameter to send follower.<br>
**      Set pointer of #FAPEX_HDMICEC_MessageParamStrT data field to \e pArg.
**
** \anchor hdmicec_cmd_get_message
** \li #FAPEX_IOCCOM_HDMICEC_GET_MESSAGE<br>
**      get CEC message parameter from initiator.<br>
**      Set pointer of #FAPEX_HDMICEC_MessageParamStrT data field to \e pArg.<br>
**      The message parameter will be set to this field.
**
** \anchor hdmicec_cmd_send_message
** \li #FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE<br>
**      send CEC message.<br>
**      Set pointer of status data field (int32_t) to \e pArg.<br>
**      The status value will be one of following.
**      - #FAPEX_HDMI_STATUS_OK<br>
**        successful
**      - #FAPEX_HDMICEC_STATUS_ERR_BUSY<br>
**        HDMI CEC is sending the message
**      - #FAPEX_HDMICEC_STATUS_ERR_MESSAGE_BUFFER_EMPTY<br>
**        HDMI CEC error of message buffer empty
**      - #FAPEX_HDMICEC_STATUS_ERR_SEND_PARAMETER<br>
**        HDMI CEC error of sending parameter
**
** \anchor hdmicec_cmd_receive_message
** \li #FAPEX_IOCCOM_HDMICEC_RECEIVE_MESSAGE<br>
**      receive CEC message.<br>
**      Set pointer of status data field (int32_t) to \e pArg.<br>
**      The status value will be one of following.
**      - #FAPEX_HDMI_STATUS_OK<br>
**        successful
**      - #FAPEX_HDMICEC_STATUS_ERR_BUSY<br>
**        HDMI CEC is receiving the message
**      - #FAPEX_HDMICEC_STATUS_ERR_RECEIVE_PARAMETER<br>
**        HDMI CEC error of receiving parameter
**
** \anchor hdmicec_cmd_set_logical_address
** \li #FAPEX_IOCCOM_HDMICEC_SET_LOGICAL_ADDRESS<br>
**      set logical address for CEC.<br>
**      Set address data (uint32_t) to \e pArg.
**
** \anchor hdmicec_cmd_get_logical_address
** \li #FAPEX_IOCCOM_HDMICEC_GET_LOGICAL_ADDRESS<br>
**      get logical address for CEC.<br>
**      Set pointer of address data field (uint32_t) to \e pArg.<br>
**      The address value will be set to this field.
**
******************************************************************************/
FAPEXIOCRET_T FAPEX_HDMICEC_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Ioctl(id, cmd, pArg);
#else /* !_LINUX_FAPI */
    return ioctl(id, (int)cmd, pArg);
#endif /* !_LINUX_FAPI */
}
