/*!
*************************************************************************
**
** \file    abst_libssp.c
**
** \brief   abstraction library source file (SSP driver).
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#else
#include <linux/ioctl.h>
#endif
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_libssp.h"
#include "abst_common_priv.h"

/*!
*******************************************************************************
**
** \brief  open method.
**         This function is called with the systemcall "open".
**
** \param  id     The device identifier.
** \param  flags  This parameter indicate a open mode (read only/write only/read-write).<br>
**                This implementation supports only read-write mode.
** \param  argptr argument pointer(#FAPEX_SSP_OpenParamStrT)<br>
**                If descriptor for this '\e id' has already been opened,
**                this '\e argptr' will be invalid.
**
** \retval >=0 process OK, new file descriptor is returned.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXOPENRET_T FAPEX_SSP_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr)
{
    return fapex_COMMON_Open(id, flags, argptr, FAPEX_IOCCOM_SSP_OPEN);
}

/*!
*******************************************************************************
**
** \brief  release method.
**         This function is called with the systemcall "close".
**
** \param  id     The identifier for this descriptor.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXRELRET_T FAPEX_SSP_Close(FAPEXRELARG1_T id)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Release(id);
#else /* !_LINUX_FAPI */
    return close(id);
#endif /* !_LINUX_FAPI */
}

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  read method.
**         This function is called with the systemcall "read".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the destination buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually read.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXREADRET_T FAPEX_SSP_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T buf, FAPEXREADARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Read(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)read(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  write method.
**         This function is called with the systemcall "write".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the source buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually write.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXWRTRET_T FAPEX_SSP_Write(FAPEXWRTARG1_T id, FAPEXWRTARG2_T buf, FAPEXWRTARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Write(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)write(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}
#endif /* not supported */

/*!
*******************************************************************************
**
** \brief  ioctl method.
**         This function is called with the systemcall "ioctl".
**
** \param  id     The identifier for this descriptor.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
**<hr>
** ioctl command(\e cmd) type:

**
** \anchor ssp_cmd_set_frameformat
** \li #FAPEX_IOCCOM_SSP_SET_FRAMEFORMAT<br>
**      set frameformat SSP<br>
**      Set pointer of #FAPEX_SSP_FrameformatlT data field to \e pArg.<br>
**      Please note the specification of the connected device.
**
**
** \anchor ssp_cmd_set_speed
** \li #FAPEX_IOCCOM_SSP_SET_SPEED<br>
**      set frameformat SSP<br>
**      Set pointer of #FAPEX_SSP_SpeedT data field to \e pArg.<br>
**      Please note the specification of the connected device.
**

** \anchor ssp_cmd_read
** \li #FAPEX_IOCCOM_SSP_READ<br>
**      read SSP<br>
**      Set pointer of #FAPEX_SSP_RParamStrT data field to \e pArg.<br>
**      Please follow the specification of the connected device about
**      the method of setting slaveaddress and subaddress(in txData), etc.
**
** \anchor ssp_cmd_write
** \li #FAPEX_IOCCOM_SSP_WRITE<br>
**      write SSP<br>
**      Set pointer of #FAPEX_SSP_WParamStrT data field to \e pArg.<br>
**      Please follow the specification of the connected device about
**      the method of setting slaveaddress and subaddress(in txData), etc.
**
** \anchor ssp_cmd_getdata
** \li #FAPEX_IOCCOM_SSP_GETDATA<br>
**      write SSP<br>
**      Set pointer of #FAPEX_SSP_GetDataParamStrT data field to \e pArg.<br>
**      Please follow the specification of the connected device about
**      the method of setting slaveaddress and subaddress(in txData), etc.
**
**

******************************************************************************/
FAPEXIOCRET_T FAPEX_SSP_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Ioctl(id, cmd, pArg);
#else /* !_LINUX_FAPI */
    return ioctl(id, (int)cmd, pArg);
#endif /* !_LINUX_FAPI */
}
