/*!
*************************************************************************
**
** \file    abst_libtsd.c
**
** \brief   abstraction library source file (TSD driver).
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#else
#include <linux/ioctl.h>
#endif
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_libtsd.h"
#include "abst_common_priv.h"

/*!
*******************************************************************************
**
** \brief  open method.
**         This function is called with the systemcall "open".
**
** \param  id     The identifier for TSD block descriptor (\b common descriptor).
** \param  flags  This parameter indicate a open mode (read only/write only/read-write).<br>
**                This implementation supports only read-write mode.
** \param  argptr argument pointer (#FAPEX_TSD_OpenParamStrT)<br>
**                If descriptor for this '\e id' has already been opened,
**                this '\e argptr' will be invalid.
**
** \retval >=0 process OK, new file descriptor is returned.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXOPENRET_T FAPEX_TSD_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr)
{
    return fapex_COMMON_Open(id, flags, argptr, FAPEX_IOCCOM_TSD_OPEN);
}

/*!
*******************************************************************************
**
** \brief  release method.
**         This function is called with the systemcall "close".
**
** \param  id     The identifier for TSD block descriptor (\b common descriptor).
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXRELRET_T FAPEX_TSD_Close(FAPEXRELARG1_T id)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Release(id);
#else /* !_LINUX_FAPI */
    return close(id);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  read method.
**         This function is called with the systemcall "read".<br>
**         This function is waited for until the stream data is prepared.
**
** \param  id     The identifier for TSD channel descriptor (\b sub descriptor).
** \param  buf    The pointer to the destination buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually read.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXREADRET_T FAPEX_TSD_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T buf, FAPEXREADARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Read(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)read(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  write method.
**         This function is called with the systemcall "write".<br>
**         This function is waited for until the empty area is available in buffer.<br>
**         This function is supported only when the tsSelect value of FAPEX_TSD_OpenParamStrT /
**         #FAPEX_TSD_InputParamStrT structure is #FAPEX_TSD_INPUT_LINEAR_AUTO_MODE
**         at #FAPEX_TSD_Open() or \ref tsd_cmd_set_input "FAPEX_IOCCOM_TSD_SET_INPUT"
**         command with FAPEX_TSD_Ioctl().<br>
**
** \param  id     The identifier for TSD block descriptor (\b common descriptor).
** \param  buf    The pointer to the source buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually write.
** \retval <0  process fails, its value is set to errno.
**
** \note There are limitation for using this function as follows.
**         - The first data in \e 'buf' area should be the sync word of TS packet.
**         - The size of TS packet should be "188" byte.
**         - TS packet data should be registered with big endian format.
**
******************************************************************************/
FAPEXWRTRET_T FAPEX_TSD_Write(FAPEXWRTARG1_T id, FAPEXWRTARG2_T buf, FAPEXWRTARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Write(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)write(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  ioctl method.
**         This function is called with the systemcall "ioctl".
**
** \param  id     The identifier for this descriptor.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
** \retval 0  process OK
** \retval <0 process fails, its value is set to errno.
**
** <hr>
** ioctl command(\e cmd) type:
** \anchor tsd_cmd_start
** \li #FAPEX_IOCCOM_START<br>
**      start TSD channel. (\e pArg is ignored)<br>
**      When \e id is \b common descriptor for TSD block, all TSD channels on the ready
**      state in this block will start.<br>
**      When \e id is \b sub descriptor for a TSD channel on the ready state, only this
**      TSD channel will start.
**
** \anchor tsd_cmd_stop
** \li #FAPEX_IOCCOM_STOP<br>
**      stop TSD channel. (\e pArg is ignored)<br>
**      When \e id is \b common descriptor for TSD block, all TSD channels on the running
**      state in this block will stop.<br>
**      When \e id is \b sub descriptor for a TSD channel on the running state, only this
**      TSD channel will start.
**
** \anchor tsd_cmd_reset
** \li #FAPEX_IOCCOM_RESET<br>
**      reset TSD block STC regulation. (\e pArg is ignored)<br>
**      The \e id should be \b common descriptor for TSD block.<br>
**
** \anchor tsd_cmd_open_subid
** \li #FAPEX_IOCCOM_TSD_OPEN_SUBID<br>
**      open TSD channel.<br>
**      Set pointer of #FAPEX_TSD_OpenSubIdParamStrT data field to \e pArg.<br>
**      The \e id should be \b common descriptor for TSD block.<br>
**      The \b sub descriptor will be returned in ((FAPEX_TSD_OpenSubIdParamStrT *)\e pArg)->sub_id.<br>
**      If \b sub descriptor for this '\e sub_id' has already been opened,
**      parameter except '\e sub_id' of FAPEX_TSD_OpenSubIdParamStrT will be invalid.
**
** \anchor tsd_cmd_close_subid
** \li #FAPEX_IOCCOM_CLOSE_SUBID<br>
**      close TSD channel. (\e pArg is ignored)<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".
**
** \anchor tsd_cmd_get_block_info
** \li #FAPEX_IOCCOM_TSD_GET_BLOCK_INFO<br>
**      get TSD block information.<br>
**      Set pointer of #FAPEX_TSD_BlockInfoStrT data field to \e pArg.<br>
**      The \e id should be \b common descriptor opened in #FAPEX_TSD_Open().
**
** \anchor tsd_cmd_get_channel_info
** \li #FAPEX_IOCCOM_TSD_GET_CHANNEL_INFO<br>
**      get channel information.<br>
**      Set pointer of #FAPEX_TSD_ChannelInfoStrT data field to \e pArg.<br>
**      Return PID=0x1FFF means  not setting a valid PID  before.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".
**
** \anchor tsd_cmd_get_status
** \li #FAPEX_IOCCOM_TSD_GET_STATUS<br>
**      get TSD block/channel status.<br>
**      Set pointer of #FAPEX_TSD_StatusStrT data field to \e pArg.<br>
**      The status value will be set to this field.<br>
**      When \e id is \b common descriptor, it is possible to get TSD block status.<br>
**      When \e id is \b sub descriptor, it is possible to get channel status.
**
** \anchor tsd_cmd_set_input
** \li #FAPEX_IOCCOM_TSD_SET_INPUT<br>
**      set TSD input/output parameters.<br>
**      Set pointer of #FAPEX_TSD_InputParamStrT data field to \e pArg.<br>
**      The \e id should be \b common descriptor opened in #FAPEX_TSD_Open().
**
** \anchor tsd_cmd_get_input_map_info
** \li #FAPEX_IOCCOM_TSD_GET_INPUT_MAP_INFO<br>
**      get TS packet input control mapping information parameters.<br>
**      Set pointer of #FAPEX_TSD_PacketInMapInfoStrT data field to \e pArg.<br>
**      The \e id should be \b common descriptor opened in #FAPEX_TSD_Open().<br>
**      This #FAPEX_TSD_PacketInMapInfoStrT parameter will be referred by FAPEX_TSD_Mmap().
**
** \anchor tsd_cmd_set_input_packetinfo
** \li #FAPEX_IOCCOM_TSD_SET_INPUT_PACKETINFO<br>
**      set TS packet input information parameters.<br>
**      Set pointer of FAPEX_TSD_PacketInInfoStrT data field to \e pArg.<br>
**      The \e id should be \b common descriptor opened in #FAPEX_TSD_Open().
**      \note this command is not supported in current version.
**
** \anchor tsd_cmd_get_input_packetinfo
** \li #FAPEX_IOCCOM_TSD_GET_INPUT_PACKETINFO<br>
**      get TS packet input information parameters.<br>
**      Set pointer of FAPEX_TSD_PacketInInfoStrT data field to \e pArg.<br>
**      The \e id should be \b common descriptor opened in #FAPEX_TSD_Open().
**
** \anchor tsd_cmd_set_pid_filter
** \li #FAPEX_IOCCOM_TSD_SET_PID_FILTER<br>
**      set PID parameters.<br>
**      Set pointer of #FAPEX_TSD_PidParamStrT data field to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".<br>
**      When #FAPEX_TSD_FLTFLAGS_PACKET_END is set to the flags of #FAPEX_TSD_PesParamStrT structure,<br>
**      the detection of packet-end can be confirmed by FAPEX_SYS_SelectCall().
**
** \anchor tsd_cmd_set_section_filter
** \li #FAPEX_IOCCOM_TSD_SET_SECTION_FILTER<br>
**      set section parameters.<br>
**      Set pointer of #FAPEX_TSD_SctFltParamStrT data field to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".<br>
**      When you register two or more table_id filter setting with same pid,<br>
**      you should set FAPEX_TSD_FLTFLAGS_TABLE_NUMx(choose one between FAPEX_TSD_FLTFLAGS_TABLE_NUM2
**      and FAPEX_TSD_FLTFLAGS_TABLE_NUM32) to flags of #FAPEX_TSD_SctFltParamStrT structure.<br>
**      And execute this command only of the amounts of the set number(x).<br>
**      The detection of section-end can be confirmed by FAPEX_SYS_SelectCall().<br>
**      When #FAPEX_TSD_FLTFLAGS_PACKET_END is set to the flags of #FAPEX_TSD_PesParamStrT structure,<br>
**      the detection of packet-end can be confirmed by FAPEX_SYS_SelectCall().<br>
**      When non zero value is set to timeout of #FAPEX_TSD_PesParamStrT structure,<br>
**      the timeout detection can be confirmed by FAPEX_SYS_SelectCall().
**
** \anchor tsd_cmd_set_pes_filter
** \li #FAPEX_IOCCOM_TSD_SET_PES_FILTER<br>
**      set PES parameters.<br>
**      Set pointer of #FAPEX_TSD_PesParamStrT data field to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".<br>
**      When #FAPEX_TSD_FLTFLAGS_SCRAMBLED_PACKET is set to the flags of #FAPEX_TSD_PesParamStrT structure,<br>
**      the detection of scrambled-packet can be confirmed by FAPEX_SYS_SelectCall().
**
** \anchor tsd_cmd_set_pid_value
** \li #FAPEX_IOCCOM_TSD_SET_PID_VALUE<br>
**      set PID value.<br>
**      Set pointer of #FAPEX_TSD_PidParamStrT data field to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".
**
** \anchor tsd_cmd_set_keys
** \li #FAPEX_IOCCOM_TSD_SET_KEYS<br>
**      set desclamble keys.<br>
**      Set pointer of #FAPEX_TSD_DescKeysParamStrT data field to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".<br>
**      \note This command is not supported in current version.
**
** \anchor tsd_cmd_set_buffer_readsize
** \li #FAPEX_IOCCOM_TSD_SET_BUFFER_READSIZE<br>
**      set read size.<br>
**      Set size data field (uint32_t) to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".
**
** \anchor tsd_cmd_get_buffer_readinfo
** \li #FAPEX_IOCCOM_TSD_GET_BUFFER_READINFO<br>
**      get read information.<br>
**      Set pointer of #FAPEX_TSD_ReadInfoStrT data field to \e pArg.
**      The read information value will be set to this field.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".<br>
**      This command is waited for until the stream data is prepared.
**
** \anchor tsd_cmd_get_buffer_writeinfo
** \li #FAPEX_IOCCOM_TSD_GET_BUFFER_WRITEINFO<br>
**      get write information.<br>
**      Set pointer of #FAPEX_TSD_WriteInfoStrT data field to \e pArg.<br>
**      The write information value will be set to this field.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".
**
** \anchor tsd_cmd_get_buffer_level
** \li #FAPEX_IOCCOM_TSD_GET_BUFFER_LEVEL<br>
**      get buffer filling level.<br>
**      Set pointer of filling level data field (uint32_t) to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".<br>
**
** \anchor tsd_cmd_get_stc_value
** \li #FAPEX_IOCCOM_TSD_GET_STC_VALUE<br>
**      get STC value (unit of 90kHz).<br>
**      Set pointer of uint64_t data to \e pArg.<br>
**      If the appointed TSD block doesn't use STC, the returned value will be
**      0xFFFFFFFFFFFFFFFF.
**
** \anchor tsd_cmd_set_write_pointer
** \li #FAPEX_IOCCOM_TSD_SET_BUFFER_WRITEPOINTEROFFSET<br>
**      set write pointer offset.<br>
**      Set write pointer offset (uint32_t) to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".
**
** \anchor tsd_cmd_pvr_open_channel_set
** \li #FAPEX_IOCCOM_TSD_PVR_OPEN_CHANNEL_SET<br>
**      open pvr channels.<br>
**      get the pvr channel info #FAPEX_TSD_PvrChannelSetStrT data from \e pArg and open the pvr channel.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".
**
** \anchor tsd_cmd_pvr_close_channel_set
** \li #FAPEX_IOCCOM_TSD_PVR_CLOSE_CHANNEL_SET<br>
**      closes pvr channels.<br>
**      get the pvr channel info #FAPEX_TSD_PvrChannelSetStrT data from \e pArg and close the pvr channel.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".
**
** \anchor tsd_cmd_pvr_ctrl_channel_set
** \li #FAPEX_IOCCOM_TSD_PVR_CTRL_CHANNEL_SET<br>
**      enable or diable pvr channels.<br>
**      get the pvr channel info #FAPEX_TSD_PvrChannelSetStrT data from \e pArg.<br>
**      enable or disable the pvr channels following the \b enable field of #FAPEX_TSD_PvrChannelSetStrT.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".
**
** \anchor tsd_cmd_reset_buffer
** \li #FAPEX_IOCCOM_TSD_RESET_BUFFER<br>
**      reset the \b BM buffer with the given channel.<br>
**      set both the buffer write pointer and buffer read pointer to zero.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".
**
** \anchor tsd_cmd_set_pes_parser
** \li #FAPEX_IOCCOM_TSD_SET_PES_PARSER<br>
**      enable the PES parser on the given TSD channel.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".
**
** \anchor tsd_cmd_reset_pes_parser
** \li #FAPEX_IOCCOM_TSD_RESET_PES_PARSER<br>
**      disable the PES parser on the given TSD channel.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".
**
** \anchor tsd_cmd_clear_section_filter
** \li #FAPEX_IOCCOM_TSD_CLEAR_SECTION_FILTER<br>
**      Clear a section filter of a given channel.<br>
**      The \e id should be \b sub descriptor opened in \ref tsd_cmd_open_subid "FAPEX_IOCCOM_TSD_OPEN_SUBID".
******************************************************************************/
FAPEXIOCRET_T FAPEX_TSD_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg)
{
    return fapex_COMMON_Ioctl(id, cmd, pArg);
}

/*!
*******************************************************************************
**
** \brief  mmap method.
**         This function is called with the systemcall "mmap".<br>
**
**       - <b>Input Buffer Mapping</b><br>
**         When '\e id' is TSD block descriptor (\b common descriptor),<br>
**         this function supports accessing of the mapped area for input of TS packet.<br>
**         In this case, the tsSelect of FAPEX_TSD_OpenParamStrT structure in FAPEX_TSD_Open() or
**         the tsSelect of #FAPEX_TSD_InputParamStrT structure in \ref tsd_cmd_set_input
**         "FAPEX_IOCCOM_TSD_SET_INPUT" command with FAPEX_TSD_Ioctl() should be FAPEX_TSD_INPUT_LINEAR_AUTO_MODE.<br>
**         The '\e length' should be mapSize of FAPEX_TSD_PacketInMapInfoStrT structure.<br>
**         This information of FAPEX_TSD_PacketInMapInfoStrT is got by using
**         \ref tsd_cmd_get_input_map_info "FAPEX_IOCCOM_TSD_GET_INPUT_MAP_INFO" command with
**         FAPEX_TSD_Ioctl().<br>
**         The writable area is only (packetOffset x FAPEX_TSD_PACKET_INPUT_BUFF_NUM) bytes
**         from the mapped address + mapOffset.<br>
**         These packetOffset and mapOffset are also parameter of FAPEX_TSD_PacketInMapInfoStrT
**         structure.<br>
**         If the access of area excluding this is executed, the operation will not be guaranteed.<br>
**
**         \image html tsd_PacketInMapInfoStrT.png
**
**       - <b>Demuxed Buffer Mapping</b><br>
**         When '\e id' is TSD channel descriptor (\b sub descriptor),<br>
**         this function supports accessing of the mapped area forTS-Demuxed payload data.<br>
**         The readable area is only bufferSize bytes from mapped address.<br>
**         This bufferSize is parameter of FAPEX_TSD_PidParamStrT or the FAPEX_TSD_SctFltParamStrT structure.<br>
**         If the access of area excluding this is executed, the operation will not be guaranteed.<br>
**         After reading of this payload data, the \ref tsd_cmd_set_buffer_readsize
**         "FAPEX_IOCCOM_TSD_SET_BUFFER_READSIZE" command with FAPEX_TSD_Ioctl() should be executed.
**
**         \image html tsd_ReadInfoStrT.png
**
** \param  start  The memory mapping start pointer.
** \param  length The memory mapping size.
** \param  prot   The desired memory protection.
** \param  flags  The mapping option.
** \param  id     The identifier for TSD block descriptor (\b common descriptor)
**                or TSD channel descriptor (\b sub descriptor).
** \param  offset The memory mapping offset value.
**
** \retval >0  process OK, its value is the mapping address.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXMMPRET_T FAPEX_TSD_Mmap(FAPEXMMPARG1_T start, FAPEXMMPARG2_T length,
                             FAPEXMMPARG3_T prot,  FAPEXMMPARG4_T flags,
                             FAPEXMMPARG5_T id,    FAPEXMMPARG6_T offset)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Mmap(start, length, prot, flags, id, offset);
#else /* !_LINUX_FAPI */
    return mmap(start, length, prot, flags, id, offset);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  munmap method.
**         This function is called with the systemcall "munmap".
**
** \param  start  The memory mapping start pointer.
** \param  length The memory mapping size.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXMUNMPRET_T FAPEX_TSD_Munmap(FAPEXMMPARG1_T start, FAPEXMMPARG2_T length)
{
#ifndef _LINUX_FAPI
    return 0;
#else /* !_LINUX_FAPI */
    return munmap(start, length);
#endif /* !_LINUX_FAPI */
}
