/*!
*************************************************************************
**
** \file    abst_libvidec.c
**
** \brief   abstraction library source file (Video Decoder driver).
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#else
#include <linux/ioctl.h>
#endif
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_libvidec.h"
#include "abst_common_priv.h"

/*!
*******************************************************************************
**
** \brief  open method.
**         This function is called with the systemcall "open".
**
** \param  id     The device identifier.
** \param  flags  This parameter indicate a open mode (read only/write only/read-write).<br>
**                This implementation supports only read-write mode.
** \param  argptr argument pointer (#FAPEX_VIDEC_OpenParamStrT)<br>
**                If descriptor for this '\e id' has already been opened,
**                this '\e argptr' will be invalid.
**
** \retval >=0 process OK, new file descriptor is returned.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXOPENRET_T FAPEX_VIDEC_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr)
{
    return fapex_COMMON_Open(id, flags, argptr, FAPEX_IOCCOM_VIDEC_OPEN);
}

/*!
*******************************************************************************
**
** \brief  release method.
**         This function is called with the systemcall "close".
**
** \param  id     The identifier for this descriptor.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXRELRET_T FAPEX_VIDEC_Close(FAPEXRELARG1_T id)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Release(id);
#else /* !_LINUX_FAPI */
    return close(id);
#endif /* !_LINUX_FAPI */
}

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  read method.
**         This function is called with the systemcall "read".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the destination buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually read.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXREADRET_T FAPEX_VIDEC_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T buf, FAPEXREADARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Read(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)read(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  write method.
**         This function is called with the systemcall "write".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the source buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually write.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXWRTRET_T FAPEX_VIDEC_Write(FAPEXWRTARG1_T id, FAPEXWRTARG2_T buf, FAPEXWRTARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Write(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)write(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}
#endif

/*!
*******************************************************************************
**
** \brief  ioctl method.
**         This function is called with the systemcall "ioctl".
**
** \param  id     The identifier for this descriptor.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
** <hr>
** ioctl command(\e cmd) type:
** \anchor videc_cmd_start
** \li #FAPEX_IOCCOM_START<br>
**      start Video Decoder.<br>
**      Set pointer of #FAPEX_VIDEC_StartParamStrT data field to \e pArg.<br>
**      If you set the value except of FAPEX_VIDEC_NOTIFY_FLAGS_NONE to 'flags'
**      of #FAPEX_VIDEC_StartParamStrT structure,<br>
**      you will be able to get the decoded video frame information by
**      \ref videc_cmd_get_dec_frame_info "FAPEX_IOCCOM_VIDEC_GET_DEC_FRAME_INFO"
**      command.
**
** \anchor videc_cmd_stop
** \li #FAPEX_IOCCOM_STOP<br>
**      stop Video Decoder. (\e pArg is ignored)
**
** \anchor videc_cmd_pause
** \li #FAPEX_IOCCOM_PAUSE<br>
**      pause Video Decoder. (\e pArg is ignored)
**
** \anchor videc_cmd_reset
** \li #FAPEX_IOCCOM_RESET<br>
**      reset Video Decoder.
**      Set the reset type (#FAPEX_VIDEC_SOFT_RESET or #FAPEX_VIDEC_HARD_RESET) to \e pArg
**
** \anchor videc_cmd_get_status
** \li #FAPEX_IOCCOM_VIDEC_GET_STATUS<br>
**      get status.<br>
**      Set pointer of #FAPEX_VIDEC_StatusStrT data field to \e pArg.<br>
**      The status value will be set to this field.
**
** \anchor videc_cmd_set_output
** \li #FAPEX_IOCCOM_VIDEC_SET_OUTPUT<br>
**      set link to Video Output scaler.<br>
**      Set pointer of Video Output device minor number field (int32_t) to \e pArg.
**
** \anchor videc_cmd_set_sync
** \li #FAPEX_IOCCOM_VIDEC_SET_SYNC<br>
**      enable or disable the Video Synchronization.<br>
**      Set enable(!=0) or disable(=0) value (uint32_t) to \e pArg.
**
** \anchor videc_cmd_set_dispon
** \li #FAPEX_IOCCOM_VIDEC_SET_DISPON<br>
**      enable video.<br>
**      Set the enable type (int32_t) to \e pArg.<br>
**      The enable type is as follows.<br>
**      - #FAPEX_VIDEC_ENABLE_SYNCHRONOUSLY_SUSPEND<br>
**        Enables the video to the VO block in sync with the first decoded frame
**        while waiting for the first frame to be decoded.<br>
**        This parameter causes a timed blocking action.
**      - #FAPEX_VIDEC_ENABLE_SYNCHRONOUSLY_NO_SUSPEND<br>
**        Enables the video to the VO block in sync with the first decoded frame.<br>
**        This causes a non-blocking call.
**      - #FAPEX_VIDEC_ENABLE_FORCE_ON<br>
**        Enables the video to the VO block on the subsequent VBI.
**
** \anchor videc_cmd_set_dispoff
** \li #FAPEX_IOCCOM_VIDEC_SET_DISPOFF<br>
**      disable video. (\e pArg is ignored)
**
** \anchor videc_cmd_get_ves_params
** \li #FAPEX_IOCCOM_VIDEC_GET_VES_PARAMS<br>
**      get Video ES parameters.<br>
**      Set pointer of #FAPEX_VIDEC_VESParamStrT data field to \e pArg.<br>
**      The VES parameter value will be set to this field.
**
** \anchor videc_cmd_get_frame_bank_size
** \li #FAPEX_IOCCOM_VIDEC_GET_FRAME_BANK_SIZE<br>
**      get Video frame bank size.<br>
**      Set pointer of bank size data field (uint32_t) to \e pArg.<br>
**      The bank size value will be set to this field.
**
** \anchor videc_cmd_get_dec_frame_info
** \li #FAPEX_IOCCOM_VIDEC_GET_DEC_FRAME_INFO<br>
**      check the decoded video frame information<br>
**      Set pointer of #FAPEX_VIDEC_DecFrameInfoStrT data field to \e pArg.<br>
**      If you want to get #FAPEX_VIDEC_DecFrameInfoStrT information by this command,<br>
**      you should set the value except of FAPEX_VIDEC_NOTIFY_FLAGS_NONE to 'flags' of
**      #FAPEX_VIDEC_StartParamStrT structure of \ref videc_cmd_start "FAPEX_IOCCOM_START" command.
**
** \anchor videc_cmd_set_playback_env
** \li #FAPEX_IOCCOM_VIDEC_SET_PLAYBACK_ENV<br>
**      set playback environment for Video Decoder.<br>
**      Set value of playback environment type to \e pArg.<br>
**      - #FAPEX_VIDEC_PLAYBACK_ENVIRONMENT_LIVE
**      - #FAPEX_VIDEC_PLAYBACK_ENVIRONMENT_PVR
**      - #FAPEX_VIDEC_PLAYBACK_ENVIRONMENT_VOD
**
** \anchor videc_cmd_set_playback_mode
** \li #FAPEX_IOCCOM_VIDEC_SET_PLAYBACK_MODE<br>
**      set playback mode for Video Decoder.<br>
**      Set value of playback mode to \e pArg.<br>
**      ex.
**      - #FAPEX_VIDEC_PLAYBACK_MODE_PAUSE<br>
**        Pauses the playback in PVR environment otherwise the request is rejected.
**      - #FAPEX_VIDEC_PLAYBACK_MODE_NORMAL<br>
**        Continues to play the video normally.
**      - #FAPEX_VIDEC_PLAYBACK_MODE_FIRST_I_ONLY<br>
**        Displays only the first occuring I Frame.
**      - #FAPEX_VIDEC_PLAYBACK_MODE_I_ONLY<br>
**        Continues to play the video in the trick mode I_ONLY.
**      - #FAPEX_VIDEC_PLAYBACK_MODE_I_AND_P_ONLY<br>
**        Continues to play the video in the trick mode skipping B frames.
**      - #FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_2X<br>
**        Continues to play the video in the trick mode slow forward 2 times.
**      - #FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_4X<br>
**        Continues to play the video in the trick mode slow forward 4 times.
**      - #FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_8X<br>
**        Continues to play the video in the trick mode slow forward 8 times.
**
** \anchor videc_cmd_set_freeze_mode
** \li #FAPEX_IOCCOM_VIDEC_SET_FREEZE_MODE<br>
**      set freeze mode for Video Decoder.<br>
**      Set #FAPEX_VIDEC_FREEZE_OFF or #FAPEX_VIDEC_FREEZE_ON to \e pArg.
**
** \anchor videc_cmd_recover
** \li #FAPEX_IOCCOM_VIDEC_RECOVER<br>
**      recover the Video Decoder. (\e pArg is ignored)
**
** \anchor videc_cmd_dec_single_iframe
** \li #FAPEX_IOCCOM_VIDEC_DEC_SINGLE_I_FRAME<br>
**      decode Single I-frame(MPEG).<br>
**      Set pointer of #FAPEX_VIDEC_ESdataParamStrT data field to \e pArg.
**
** \anchor videc_cmd_dec_sequence_iframe
** \li #FAPEX_IOCCOM_VIDEC_DEC_SEQUENCE_I_FRAME<br>
**      decode Single I-frame(MPEG) for trick mode as well.<br>
**      Set pointer of #FAPEX_VIDEC_ESdataParamStrT data field to \e pArg.
**
** \anchor videc_cmd_disp_single_iframe
** \li #FAPEX_IOCCOM_VIDEC_DISP_SINGLE_I_FRAME<br>
**      display Single I-frame(MPEG). (\e pArg is ignored)
**
** \anchor videc_cmd_set_output_delay
** \li #FAPEX_IOCCOM_VIDEC_SET_OUTPUT_DELAY<br>
**      set the delay for video display.<br>
**      Set value of delay in milliseconds to \e pArg.<br>
**      For Live stream the output delay can be set dynamically.
**      For PVR playback the output delay should be set before starting the video
**      and only positive delay is allowed under PVR environment.
**      The delay is not applicable in the trick modes.
**
** \anchor videc_cmd_set_stream_segment_mode
** \li #FAPEX_IOCCOM_VIDEC_SET_STREAM_SEGMENT_MODE<br>
**      set the video stream segment mode.<br>
**      Set video stream segment mode from \e pArg.<br>
**      #FAPEX_VIDEC_STREAM_SEGMENT_MODE_FULL specifies the stream segment is normal.
**      #FAPEX_VIDEC_STREAM_SEGMENT_MODE_1 specifies the stream segment is of 1seg type
**      as in ISDB-T in japan and Brazil.
**
** \anchor videc_cmd_get_stream_segment_mode
** \li #FAPEX_IOCCOM_VIDEC_GET_STREAM_SEGMENT_MODE<br>
**      get the video stream segment mode.<br>
**      Set video stream segment mode to \e pArg.<br>
**      #FAPEX_VIDEC_STREAM_SEGMENT_MODE_FULL specifies the stream segment is normal.
**      #FAPEX_VIDEC_STREAM_SEGMENT_MODE_1 specifies the stream segment is of 1seg type
**      as in ISDB-T in japan and Brazil.
**
** \anchor videc_cmd_set_stc_regulation
** \li #FAPEX_IOCCOM_SYNC_SET_STC_REGULATION<br>
**      set the stc regulation mode.<br>
**      Set the stc regulation mode from \e pArg.<br>
**
** \anchor videc_cmd_set_environment
** \li #FAPEX_IOCCOM_SYNC_SET_ENVIRONMENT<br>
**      set the video play environment.<br>
**      Set the video play environment from \e pArg.<br>
**      ex.<br>
**      - #FAPEX_SYNC_UNKNOWN_BASE<br>
**        Unknown sync mode
**      - #FAPEX_SYNC_AUDIO_PTS_BASED<br>
**        Do the sync based on audio PTS
**      - #FAPEX_SYNC_VIDEO_PTS_BASED<br>
**        Do the sync based on video PTS
**      - #FAPEX_SYNC_PCR_BASED<br>
**        Do the sync based on PCR
**      - #FAPEX_SYNC_STREAMING_PCR_BASED<br>
**        Do the sync based on streaming PCR
**
** \anchor videc_cmd_set_info_callback
** \li #FAPEX_IOCCOM_VIDEC_SET_INFO_CALLBACK<br>
**      change the callback setting of videc.<br>
**      Set the videc callback setting from \e pArg.<br>
**      ex.<br>
**      - #FAPEX_VIDEC_NOTIFY_FLAGS_NONE<br>
**        Clear all the videc callback
**      - #FAPEX_VIDEC_NOTIFY_FLAGS_DEC_AVAILABLE<br>
**        Set the frame decoded callback
**      - #FAPEX_VIDEC_NOTIFY_FLAGS_DISP_AVAILABLE<br>
**        Set the frame available callback      
**
** \anchor videc_cmd_set_frame_rate
** \li #FAPEX_IOCCOM_VIDEC_SET_FRAME_RATE<br>
**      set the frame rate of video decoder.<br>
**      Set the rame rate of video decoder from \e pArg.<br>   
**
** \anchor videc_cmd_clear_frame_rate
** \li #FAPEX_IOCCOM_VIDEC_CLEAR_FRAME_RATE<br>
**      clear the frame rate of video decoder.<br>
**
** \anchor videc_cmd_clear_stream_segment_mode
** \li #FAPEX_IOCCOM_VIDEC_CLEAR_STREAM_SEGMENT_MODE<br>
**      clear the stream segment mode of video decoder.<br>
**
** \anchor videc_cmd_enable_disable_output
** \li #FAPEX_IOCCOM_VIDEC_ENABLE_DISABLE_OUTPUT<br>
**      set the enable disable state of video decoder.<br>
**      set the enable disable state of video decoder from \e pArg.<br>
**      ex.<br>
**      - #FAPEX_VIDEC_ENABLE_FORCE_OFF<br>
**      - #FAPEX_VIDEC_ENABLE_FORCE_ON_IMMEDIATE<br>
**      - #FAPEX_VIDEC_ENABLE_FORCE_ON<br>
**      - #FAPEX_VIDEC_ENABLE_SYNCHRONOUSLY_SUSPEND<br>
**      - #FAPEX_VIDEC_ENABLE_SYNCHRONOUSLY_NO_SUSPEND<br>
**      - #FAPEX_VIDEC_ENABLE_SYNCHRONOUSLY_ON_SYNC_SUSPEND<br>
**      - #FAPEX_VIDEC_ENABLE_SYNCHRONOUSLY_ON_SYNC_NO_SUSPEND<br>
**
** \anchor videc_cmd_aud_feed_ex
** \li #FAPEX_IOCCOM_VIDEC_AUD_FEED_EX<br>
**      feed the AUD data externally.<br>
**      feed the AUD data #FAPEX_VIDEC_AudFeedStrT from \e pArg.<br>
**
******************************************************************************/
FAPEXIOCRET_T FAPEX_VIDEC_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Ioctl(id, cmd, pArg);
#else /* !_LINUX_FAPI */
    return ioctl(id, (int)cmd, pArg);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  mmap method.
**         This function is called with the systemcall "mmap".
**
**         This function supports accessing of the mapped area for decoded frame buffer.<br>
**         The '\e length' should be value returned by \ref videc_cmd_get_frame_bank_size
**         "FAPEX_IOCCOM_VIDEC_GET_FRAME_BANK_SIZE" command with FAPEX_VIDEC_Ioctl().<br>
**         For accessing to frame buffer, you should use decodedWidth, decodedHeight,
**         offset_y and offset_c of FAPEX_VIDEC_DecFrameInfoStrT.<br>
**         This information of FAPEX_VIDEC_DecFrameInfoStrT is got by using
**         \ref videc_cmd_get_dec_frame_info "FAPEX_IOCCOM_VIDEC_GET_DEC_FRAME_INFO"
**         command with FAPEX_VIDEC_Ioctl().
**
**         \image html videc_DecFrameInfoStrT.png
**
** \param  start  The memory mapping start pointer.
** \param  length The memory mapping size.
** \param  prot   The desired memory protection.
** \param  flags  The mapping option.
** \param  id     The identifier for this descriptor.
** \param  offset The memory mapping offset value.
**
** \retval >0  process OK, its value is the mapping address.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXMMPRET_T FAPEX_VIDEC_Mmap(FAPEXMMPARG1_T start, FAPEXMMPARG2_T length,
                               FAPEXMMPARG3_T prot,  FAPEXMMPARG4_T flags,
                               FAPEXMMPARG5_T id,    FAPEXMMPARG6_T offset)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Mmap(start, length, prot, flags, id, offset);
#else /* !_LINUX_FAPI */
    return mmap(start, length, prot, flags, id, offset);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  munmap method.
**         This function is called with the systemcall "munmap".
**
** \param  start  The memory mapping start pointer.
** \param  length The memory mapping size.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXMUNMPRET_T FAPEX_VIDEC_Munmap(FAPEXMMPARG1_T start, FAPEXMMPARG2_T length)
{
#ifndef _LINUX_FAPI
    return 0;
#else /* !_LINUX_FAPI */
    return munmap(start, length);
#endif /* !_LINUX_FAPI */
}
