/*!
*************************************************************************
**
** \file    abst_libviout.c
**
** \brief   abstraction library source file (Video Output driver).
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#else
#include <linux/ioctl.h>
#endif
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_libviout.h"
#include "abst_common_priv.h"

/*!
*******************************************************************************
**
** \brief  open method.
**         This function is called with the systemcall "open".
**
** \param  id     The identifier for all layer (\b common descriptor).
** \param  flags  This parameter indicate a open mode (read only/write only/read-write).<br>
**                This implementation supports only read-write mode.
** \param  argptr argument pointer (#FAPEX_VIOUT_OpenCmnParamStrT)<br>
**                If descriptor for this '\e id' has already been opened,
**                this '\e argptr' will be invalid.
**
** \retval >=0 process OK, new file descriptor is returned.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXOPENRET_T FAPEX_VIOUT_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr)
{
    return fapex_COMMON_Open(id, flags, argptr, FAPEX_IOCCOM_VIOUT_OPEN);
}

/*!
*******************************************************************************
**
** \brief  release method.
**         This function is called with the systemcall "close".
**
** \param  id     The identifier for all layer (\b common descriptor).
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXRELRET_T FAPEX_VIOUT_Close(FAPEXRELARG1_T id)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Release(id);
#else /* !_LINUX_FAPI */
    return close(id);
#endif /* !_LINUX_FAPI */
}

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  read method.
**         This function is called with the systemcall "read".
**
** \param  id     The identifier for VIDEO/OSD1/OSD2/CursorPlane layer descriptor (\b sub descriptor).
** \param  buf    The pointer to the destination buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually read.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXREADRET_T FAPEX_VIOUT_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T buf, FAPEXREADARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Read(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)read(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  write method.
**         This function is called with the systemcall "write".
**
** \param  id     The identifier for VIDEO/OSD1/OSD2/CursorPlane layer descriptor (\b sub descriptor).
** \param  buf    The pointer to the source buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually write.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXWRTRET_T FAPEX_VIOUT_Write(FAPEXWRTARG1_T id, FAPEXWRTARG2_T buf, FAPEXWRTARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Write(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)write(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}
#endif

/*!
*******************************************************************************
**
** \brief  ioctl method.
**         This function is called with the systemcall "ioctl".
**
** \param  id     The identifier for this descriptor.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
** <hr>
** ioctl command(\e cmd) type:
** \anchor viout_cmd_set_planeorder
** \li #FAPEX_IOCCOM_VIOUT_SET_PLANEORDER<br>
**      set Plane Order.<br>
**      Set pointer of order array data field (int32_t order[6]) to \e pArg.<br>
**      (order[0]:video, order[1]:osd1, order[2]:osd2, order[3]:osd3, order[4]:osd4, order[5]:cp)<br>
**      The \e id should be \b common descriptor opened in #FAPEX_VIOUT_Open().
**
** \anchor viout_cmd_get_planeorder
** \li #FAPEX_IOCCOM_VIOUT_GET_PLANEORDER<br>
**      get Plane Order.<br>
**      Set pointer of order array data field (int32_t order[4]) to \e pArg.<br>
**      (order[0]:video, order[1]:osd1, order[2]:osd2, order[3]:cp)
**      The order number value will be set to these field.<br>
**      If it is negative value, its plane is not shown.<br>
**      The \e id should be \b common descriptor opened in #FAPEX_VIOUT_Open().
**
** \anchor viout_cmd_set_bgcolour
** \li #FAPEX_IOCCOM_VIOUT_SET_BGCOLOUR<br>
**      set back ground color.<br>
**      Set colour data (bit23-16:Y bit15-8:Cr bit7-0:Cb) to \e pArg.<br>
**      The \e id should be \b common descriptor opened in #FAPEX_VIOUT_Open().
**
** \anchor viout_cmd_get_bgcolour
** \li #FAPEX_IOCCOM_VIOUT_GET_BGCOLOUR<br>
**      get back ground color.<br>
**      Set pointer of colour data field (uint32_t) to \e pArg.<br>
**      The colour value (bit23-16:Y bit15-8:Cr bit7-0:Cb) will be set to this field.<br>
**      The \e id should be \b common descriptor opened in #FAPEX_VIOUT_Open().
**
** \anchor viout_cmd_get_system_mode
** \li #FAPEX_IOCCOM_VIOUT_GET_SYSTEM_MODE<br>
**      get system mode.<br>
**      Set pointer of system mode data field (int32_t) to \e pArg.<br>
**      The system mode value will be set to this field.<br>
**      ex.
**      - #FAPEX_VIOUT_SYSTEM_MODE_UNKNOWN
**      - #FAPEX_VIOUT_SYSTEM_MODE_1920x1080x2397P
**      - #FAPEX_VIOUT_SYSTEM_MODE_1920x1080x24P
**      - #FAPEX_VIOUT_SYSTEM_MODE_1920x1080x25P
**      - #FAPEX_VIOUT_SYSTEM_MODE_1920x1080x2997P
**      - #FAPEX_VIOUT_SYSTEM_MODE_1920x1080x30P
**      - #FAPEX_VIOUT_SYSTEM_MODE_1920x1080x50I
**      - #FAPEX_VIOUT_SYSTEM_MODE_1920x1080x5994I
**      - #FAPEX_VIOUT_SYSTEM_MODE_1920x1080x60I
**      - #FAPEX_VIOUT_SYSTEM_MODE_1280x720x2397P
**      - #FAPEX_VIOUT_SYSTEM_MODE_1280x720x24P
**      - #FAPEX_VIOUT_SYSTEM_MODE_1280x720x25P
**      - #FAPEX_VIOUT_SYSTEM_MODE_1280x720x2997P
**      - #FAPEX_VIOUT_SYSTEM_MODE_1280x720x30P
**      - #FAPEX_VIOUT_SYSTEM_MODE_1280x720x50P
**      - #FAPEX_VIOUT_SYSTEM_MODE_1280x720x5994P
**      - #FAPEX_VIOUT_SYSTEM_MODE_1280x720x60P
**      - #FAPEX_VIOUT_SYSTEM_MODE_720x576x50I
**      - #FAPEX_VIOUT_SYSTEM_MODE_720x576x50P
**      - #FAPEX_VIOUT_SYSTEM_MODE_720x480x5994I
**      - #FAPEX_VIOUT_SYSTEM_MODE_720x480x5994P
**      - #FAPEX_VIOUT_SYSTEM_MODE_LCD_640x480x50P
**      - #FAPEX_VIOUT_SYSTEM_MODE_LCD_1024x768x50P
**      .
**      The \e id should be \b common descriptor opened in #FAPEX_VIOUT_Open().
**
** \anchor viout_cmd_start
** \li #FAPEX_IOCCOM_START<br>
**      start Video output layer. (\e pArg is ignored)<br>
**      When \e id is \b common descriptor, all Video output layers on the ready state
**      will start.<br>
**      When \e id is \b sub descriptor on the ready state, only this layer will start.
**
** \anchor viout_cmd_stop
** \li #FAPEX_IOCCOM_STOP<br>
**      stop Video Output. (\e pArg is ignored)<br>
**      When \e id is \b common descriptor, all Video output layers on the running state
**      will stop.<br>
**      When \e id is \b sub descriptor on the running state, only this layer will stop.
**
** \anchor viout_cmd_open_subid
** \li #FAPEX_IOCCOM_VIOUT_OPEN_SUBID<br>
**      open Video output layer.<br>
**      Set pointer of #FAPEX_VIOUT_OpenSubParamStrT data field to \e pArg.<br>
**      The \e id should be \b common descriptor.<br>
**      The \b sub descriptor will be returned in ((FAPEX_VIOUT_OpenSubParamStrT *)\e pArg)->sub_id.<br>
**      If \b sub descriptor for this '\e sub_id' has already been opened,
**      parameter except '\e sub_id' of FAPEX_VIOUT_OpenSubParamStrT will be invalid.
**
** \anchor viout_cmd_close_subid
** \li #FAPEX_IOCCOM_CLOSE_SUBID<br>
**      close Video output layer. (\e pArg is ignored)<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_enable
** \li #FAPEX_IOCCOM_VIOUT_ENABLE<br>
**      set enable layer. (\e pArg is ignored)<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_disable
** \li #FAPEX_IOCCOM_VIOUT_DISABLE<br>
**      set disable layer. (\e pArg is ignored)<br><br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_set_scaler_params
** \li #FAPEX_IOCCOM_VIOUT_SET_SCALER_PARAMS
**      set scaler parameter<br>
**      Set pointer of #FAPEX_VIOUT_ScalerParamStrT data field to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_get_scaler_params
** \li #FAPEX_IOCCOM_VIOUT_GET_SCALER_PARAMS
**      get scaler parameter<br>
**      Set pointer of #FAPEX_VIOUT_ScalerParamStrT data field to \e pArg.<br>
**      The scaler parameter value will be set to this field.<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_set_aspect_ratio
** \li FAPEX_IOCCOM_VIOUT_SET_ASPECT_RATIO
**      set display aspect ratio<br>
**      Set aspect ratio flag to \e pArg.<br>
**      ex.
**      - #FAPEX_VIOUT_DISPLAY_AR_NONE
**      - #FAPEX_VIOUT_DISPLAY_AR_4_3
**      - #FAPEX_VIOUT_DISPLAY_AR_14_9
**      - #FAPEX_VIOUT_DISPLAY_AR_16_9
**      - #FAPEX_VIOUT_DISPLAY_AR_221_100
**      - #FAPEX_VIOUT_DISPLAY_AR_235_100
**      .
**      When \e id is \b common descriptor, \e pArg value will be set to all Video output
**      layers except OSD1/OSD2/CursorPlane.<br>
**      When \e id is \b sub descriptor, \e pArg value will be set to one Video output layers.
**
** \anchor viout_cmd_set_alpha
** \li #FAPEX_IOCCOM_VIOUT_SET_ALPHA<br>
**      set layer's alpha.<br>
**      Set pointer of #FAPEX_VIOUT_AlphaParamStrT data field to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_get_alpha
** \li #FAPEX_IOCCOM_VIOUT_GET_ALPHA<br>
**      get layer's alpha.<br>
**      Set pointer of alpha data field (uint32_t) to \e pArg.<br>
**      The alpha parameter value will be set to this field.<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_set_fading
** \li #FAPEX_IOCCOM_VIOUT_SET_FADING<br>
**      set fade parameter.<br>
**      Set pointer of #FAPEX_VIOUT_FadingParamStrT data field to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_get_fading
** \li #FAPEX_IOCCOM_VIOUT_GET_FADING<br>
**      get fade parameter.<br>
**      Set pointer of #FAPEX_VIOUT_FadingParamStrT data field to \e pArg.<br>
**      The fade parameter value will be set to this field.<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_set_resize
** \li #FAPEX_IOCCOM_VIOUT_SET_RESIZE<br>
**      set resize parameter.<br>
**      Set pointer of #FAPEX_VIOUT_ResizeParamStrT data field to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_set_position
** \li #FAPEX_IOCCOM_VIOUT_SET_POSITION<br>
**      set Plane position.<br>
**      Set pointer of #FAPEX_VIOUT_PosParamStrT data field to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_get_position
** \li #FAPEX_IOCCOM_VIOUT_GET_POSITION<br>
**      set Plane position.<br>
**      Set pointer of #FAPEX_VIOUT_PosParamStrT data field to \e pArg.<br>
**      The position parameter value will be set to this field.<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_set_fillcolour
** \li #FAPEX_IOCCOM_VIOUT_SET_FILLCOLOUR<br>
**      fill a plane with a given colour.<br>
**      Set fill colour data to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_get_fillcolour
** \li #FAPEX_IOCCOM_VIOUT_GET_FILLCOLOUR<br>
**      get fill colour.<br>
**      Set pointer of colour data field (uint32_t) to \e pArg.<br>
**      The colour value will be set to this field.<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_set_clut
** \li #FAPEX_IOCCOM_VIOUT_SET_CLUT<br>
**      set Colour Look Up Table.<br>
**      Set pointer of Look Up Table array data field (uint32_t LUT[256]) to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_get_clut
** \li #FAPEX_IOCCOM_VIOUT_GET_CLUT<br>
**      get Colour Look Up Table.<br>
**      Set pointer of Look Up Table array data field (uint32_t LUT[256]) to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_set_hilight
** \li #FAPEX_IOCCOM_VIOUT_SET_HILIGHT<br>
**      set hilight mode.<br>
**      Set mode (uint32_t 0:hilight off, other:hilight on) to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_fill_rectangle
** \li #FAPEX_IOCCOM_VIOUT_FILL_RECTANGLE<br>
**      Fill rectangle.<br>
**      Set pointer of #FAPEX_VIOUT_FillRectParamStrT data field to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".<br>
**
** \anchor viout_cmd_flush_plane
** \li #FAPEX_IOCCOM_VIOUT_FLUSH_PLANE<br>
**      Flush the data cache for the Plane. (\e pArg is ignored)<br>
**      The \e id should be \b sub descriptor opened in \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID".
**
** \anchor viout_cmd_update_avi_info
** \li #FAPEX_IOCCOM_VIOUT_UPDATE_AVI_INFO<br>
**      update HDMI AVI InfoFrame.<br>
**      Set pointer of #FAPEX_VIOUT_HdmiParamStrT data field to \e pArg.<br>
**      The \e id should be \b common descriptor opened in #FAPEX_VIOUT_Open().
**
** \anchor viout_cmd_hdmi_restart
** \li #FAPEX_IOCCOM_VIOUT_HDMI_RESTART<br>
**      restart HDMI Video. (\e pArg is ignored)<br>
**      The \e id should be \b common descriptor opened in #FAPEX_VIOUT_Open().
**
** \anchor viout_cmd_set_brightness
** \li #FAPEX_IOCCOM_VIOUT_SET_BRIGHTNESS<br>
**      set the brightness. <br>
**      Set the brightness from the \e arg.
**      The recommended brightness values run from -400 to +400
**
** \anchor viout_cmd_set_contrast
** \li #FAPEX_IOCCOM_VIOUT_SET_CONTRAST<br>
**      set the contrast. <br>
**      Set the contrast from the \e arg.
**      The recommended contrast values run from 0 (0%) to +400 (400%)
**
** \anchor viout_cmd_set_saturation
** \li #FAPEX_IOCCOM_VIOUT_SET_SATURATION<br>
**      set the saturation. <br>
**      Set the saturation from the \e arg.
**      The recommended saturation values run from 0 (0%) to +400 (400%).
**      0% saturation stands for complete Black & white video.
**      100% saturation stands for coded chrominance levels.
**      400% saturation stands for over saturated chrominance levels.
**
** \anchor viout_cmd_scaler_set_wssstate
** \li #FAPEX_IOCCOM_VIOUT_SCALER_SET_WSSSTATE<br>
**      set the Wss signal handling strategy<br>
**      set the Wss signal handling strategy from \e pArg.<br>
**      ex.
**      - #FAPEX_VIOUT_DISPLAY_AR_NONE<br>
**      - #FAPEX_VISCALE_UNKNOWN_WSS_STATE<br>
**      - #FAPEX_VISCALE_WSS_ENABLE_4_3_FULLFORMAT<br>
**      - #FAPEX_VISCALE_WSS_ENABLE_14_9_CENTRE<br>
**      - #FAPEX_VISCALE_WSS_ENABLE_16_9_BOX_CENTRE<br>
**      - #FAPEX_VISCALE_WSS_ENABLE_16_9_BOX_GT_CENTRE<br>
**      - #FAPEX_VISCALE_WSS_ENABLE_14_9_FULLFORMAT<br>
**      - #FAPEX_VISCALE_WSS_ENABLE_16_9_ANAMORPHIC<br>
**      - #FAPEX_VISCALE_WSS_ENABLE_14_9_TOP<br>
**      - #FAPEX_VISCALE_WSS_ENABLE_16_9_BOX_TOP<br>
**      - #FAPEX_VISCALE_WSS_DISABLE<br>

******************************************************************************/
FAPEXIOCRET_T FAPEX_VIOUT_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg)
{
    return fapex_COMMON_Ioctl(id, cmd, pArg);
}

/*!
*******************************************************************************
**
** \brief  mmap method.
**         This function is called with the systemcall "mmap".<br>
**
**         This function supports accessing of the mapped area for frame buffer of OSD1/OSD2/CursorPlane.<br>
**         The '\e length' should be computed by ((inputWindowWidth x inputWindowHeight x bpp) >> 3)
**         of FAPEX_VIOUT_ScalerParamStrT structure.<br>
**         This information of FAPEX_VIOUT_ScalerParamStrT is got by using
**         \ref viout_cmd_get_scaler_params "FAPEX_IOCCOM_VIOUT_GET_SCALER_PARAMS" command
**         with FAPEX_VIOUT_Ioctl().
**
** \param  start  The memory mapping start pointer.
** \param  length The memory mapping size.
** \param  prot   The desired memory protection.
** \param  flags  The mapping option.
** \param  id     The identifier for OSD1/OSD2/CursorPlane layer descriptor (\b sub descriptor).
** \param  offset The memory mapping offset value.
**
** \retval >0  process OK, its value is the mapping address.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXMMPRET_T FAPEX_VIOUT_Mmap(FAPEXMMPARG1_T start, FAPEXMMPARG2_T length,
                               FAPEXMMPARG3_T prot,  FAPEXMMPARG4_T flags,
                               FAPEXMMPARG5_T id,    FAPEXMMPARG6_T offset)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Mmap(start, length, prot, flags, id, offset);
#else /* !_LINUX_FAPI */
    return mmap(start, length, prot, flags, id, offset);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  munmap method.
**         This function is called with the systemcall "munmap".
**
** \param  start  The memory mapping start pointer.
** \param  length The memory mapping size.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXMUNMPRET_T FAPEX_VIOUT_Munmap(FAPEXMMPARG1_T start, FAPEXMMPARG2_T length)
{
#ifndef _LINUX_FAPI
    return 0;
#else /* !_LINUX_FAPI */
    return munmap(start, length);
#endif /* !_LINUX_FAPI */
}
