/*
********************************************************************************
********************************************************************************
**
** \file        ./sample_application/fapex_bd_demo/src/main.c
**
** \version     $Id$
**
** \brief       Demo Application for boardifc.
**
** This application implements a example for how to get the board and the chip 
** information.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited
** (C) Copyright 2011 by Fujitsu Semiconductor Limited FSDC
**
********************************************************************************
********************************************************************************
*/

#include <string.h>
#include <ctype.h>
#include <fapex/abst_types.h>
#include <fapex/abst_libboardifc.h>
#include <fapex/abst_libclkpwr.h>

#ifndef _LINUX_FAPI
#include <fapi/sys_shell.h>
#else
#include "shell.h"
#endif /* !_LINUX_FAPI */
#include "demo_app.h"

//******************************************************************************
//******************************************************************************
//** Defines and Macros
//******************************************************************************
//******************************************************************************

#if defined(__arm__)
    #define APP_CPU      "arm1176"
#elif defined(_ARC)
    #define APP_CPU      "arc725d"
#else
    #define APP_CPU      "unknown"
#endif

#if defined(MB86H60)
    #define APP_TARGET   "MB86H60"
#elif defined(MB86H61)
    #define APP_TARGET   "MB86H61"
#else
    #define APP_TARGET   "unknown"
#endif


#define APP_VERSION      "2.0.6"
#define APP_PREFIX       "[av]"
#define APP_SHELL_PROMPT "[av] ==> "

#define APP_CLEARSCREEN "" // "\033[2J\033[0;0H"
#define APP_SEPARATOR   "################################################################"
#define APP_DESCRIPTION "A/V Decoder Demo Application (FAPex version)"
#define APP_COPYRIGHT   "(C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH"
#define APP_COPYRIGHT2  "(C) Copyright 2008-2010 by Fujitsu Semiconductor Limited"

#define APP_REMOTE_MODE     FAPEX_IR_MODE_NEC   // FAPEX_IR_MODE_NEC or FAPEX_IR_MODE_RC5

//******************************************************************************
//******************************************************************************
//** thread related defines and macros
//******************************************************************************
//******************************************************************************

#define APP_START_THREAD_STACKSIZE         256
#define APP_START_THREAD_PRIORITY          FAPEX_SYS_PRIORITY_HIGHEST
#define APP_START_THREAD_NAME              "app:start"
#define APP_SHELL_THREAD_STACKSIZE         2048
#define APP_SHELL_THREAD_PRIORITY          FAPEX_SYS_PRIORITY_MEDIUM
#define APP_SHELL_THREAD_NAME              "app:shell"

#define APP_WATCHDOG_THREAD_STACKSIZE      256
#define APP_WATCHDOG_THREAD_PRIORITY       (FAPEX_SYS_PRIORITY_HIGHEST - 1)
#define APP_WATCHDOG_THREAD_NAME           "app:watchdog"


//******************************************************************************
//******************************************************************************
//** Local Types
//******************************************************************************
//******************************************************************************
#define DRV_ID_IR0      (FAPEXDEV_IR_MINOR_FIRST)
#define DRV_ID_FPC0     (FAPEXDEV_FPC_MINOR_FIRST)
#define DRV_ID_CLKPWR   (FAPEXDEV_CLKPWR_MINOR_FIRST)

#define PCR_TSD_CHANNEL     7
#define DRV_ID_TSD0_PCR_CH  (FAPEXDEV_TSD0_SUB_MINOR_FIRST + PCR_TSD_CHANNEL)

//******************************************************************************
//******************************************************************************
//** Local data
//******************************************************************************
//******************************************************************************

static int32_t fd_CLKPWR   = -1;


#ifndef _LINUX_FAPI
static FAPI_SYS_ShellInputHandler* appShellInputHandler;
#endif

static FAPEX_SYS_ThreadT    appStartThreadHandle;
static FAPEX_SYS_ThreadT    appShellThreadHandle;

#ifdef USE_WATCHDOG
static FAPEX_SYS_ThreadT    appWatchdogThreadHandle;
#endif

#ifdef _LINUX_FAPI
#define APP_ColorFgBlack     "\033[30m"
#define APP_ColorFgGreen     "\033[32m"
#define APP_ColorFgBlue      "\033[34m"
#define APP_ColorBgWhite     "\033[47m"
#define APP_CommandTableSize  64
#define APP_CommandStringSize 80
#define APP_HistoryTableSize  16
#define APP_ArgvSize          16

typedef struct APP_CommandStruct {
    char_t*    command;
    char_t*    brief;
    char_t*    description;
    uint32_t   (*function)(int32_t, char_t**);
} APP_CommandStr;

APP_CommandStr  commandTable[APP_CommandTableSize];
char_t          inputCommand[APP_CommandStringSize];
char_t          historyTable[APP_HistoryTableSize][APP_CommandStringSize];
int32_t         historyIndex;
int32_t         updownIndex;

#endif /* !_LINUX_FAPI */

//******************************************************************************
//******************************************************************************
//** Global Data
//******************************************************************************
//******************************************************************************
extern int32_t fd_TSD[];

/*
*****************************************************************************
** \brief
*****************************************************************************
*/
void APP_ThreadStatus_Init(APP_ThreadStatus *status)
{
    status->value = APP_THREAD_STATUS_DISABLE;
    status->sem   = FAPEX_SYS_CreateSemaphore(1);
}

/*
*****************************************************************************
** \brief
*****************************************************************************
*/
void APP_ThreadStatus_Exit(APP_ThreadStatus *status)
{
    FAPEX_SYS_DestroySemaphore(status->sem);
}

/*
*****************************************************************************
** \brief
*****************************************************************************
*/
void APP_ThreadStatus_Lock(APP_ThreadStatus *status)
{
    FAPEX_SYS_GetSemaphore(status->sem, FAPEX_SYS_SUSPEND);
}

/*
*****************************************************************************
** \brief
*****************************************************************************
*/
void APP_ThreadStatus_Unlock(APP_ThreadStatus *status)
{
    FAPEX_SYS_SetSemaphore(status->sem, FAPEX_SYS_NO_SUSPEND);
}

/*
*****************************************************************************
** \brief
*****************************************************************************
*/
void APP_ThreadStatus_SetValue(APP_ThreadStatus *status, int32_t value)
{
    // lock
    FAPEX_SYS_GetSemaphore(status->sem, FAPEX_SYS_SUSPEND);

    status->value = value;

    // unlock
    FAPEX_SYS_SetSemaphore(status->sem, FAPEX_SYS_SUSPEND);
}


/*!
*******************************************************************************
**
** \brief Callback function to handle the "halt" command
**
** This function will be called by the shell thread if the "halt" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via APP_Printf.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via APP_Printf, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_HaltCommand( int32_t argc, char_t* argv[] )
{
    if( argc == 1 ) {
#ifdef USE_WATCHDOG
        {
            int32_t  result;

            result = FAPEX_CLKPWR_Ioctl( fd_CLKPWR,
                                         FAPEX_IOCCOM_CLKPWR_WATCHDOG_STOP,
                                         (FAPEXIOCARG3_T)NULL );
            if(result < 0) {
                FAPI_SYS_PRINT_MSG("Watchdon Stop Error %d\n", result);
            }
        }
#endif

        FAPI_SYS_PRINT_MSG( "%s Halting - Bye bye!!!\n\n", APP_PREFIX );
        exit( 1 );
    }
    return( 0 );
}
static uint32_t APP_BDetect( int32_t argc, char_t* argv[] )
{
       
    argc = argc;
    argv = argv;
    FAPI_SYS_PRINT_MSG("BDetect:\n");
    /*Open devices*/

    BOARDS_INSTALL_PLATFORM_SUPPORT_EXT();
    FAPI_BOARD_INIT_DETECT_FUNCTION(0,0);
    FAPI_BOARD_GETBOARDINFO();

    FAPI_SYS_PRINT_MSG("MagicAndVersion:     0x%x\n",hwInfo.MagicAndVersion);
    FAPI_SYS_PRINT_MSG("flagUsbHostOnly:     0x%x\n",hwInfo.flagUsbHostOnly);
    FAPI_SYS_PRINT_MSG("typeOfFlashAvailable:0x%x\n",hwInfo.typeOfFlashAvailable);
    FAPI_SYS_PRINT_MSG("pHwManufactString:   %s\n",  hwInfo.pHwManufactString);
    FAPI_SYS_PRINT_MSG("pHwModelString:      %s\n",  hwInfo.pHwModelString);
    return 0;
}




#ifdef _LINUX_FAPI
/**
*******************************************************************************
**
** \brief Print the shell prompt.
**
** This function prints the shell prompt in green color on white background
** and resets the color to blue which is used for echoing the command input.
**
*******************************************************************************
*/
static void APP_PrintPrompt(void)
{
    FAPI_SYS_PRINT_MSG( "\r%s%s%s%s%s",
                        APP_ColorBgWhite, APP_ColorFgGreen,
                        APP_SHELL_PROMPT,
                        APP_ColorBgWhite, APP_ColorFgBlack );
}
#endif /* _LINUX_FAPI */

/**
*******************************************************************************
**
** \brief Register a new command.
**
** This function registers the given command to the internal command table.
** An already existing command will be unregistered first and replaced by the
** given parameters. The buffers for the brief and the description texts will
** be dynamically allocated to avoid buffer overflows.
**
** \param command     The command (string) to register.
** \param brief       A short (one line) help message.
** \param description The full command description, maybe NULL.
** \param function    The function to be called when the user enters the
**                    "command" (with optional parameters), this functions
**                    must accept the argc, argv argument pair for additional
**                    optional parameters [strings].
**
*******************************************************************************
*/
int32_t appRegisterCommand( char_t*  command,
                            uint32_t (*function)(int32_t, char_t**),
                            char_t*  brief,
                            char_t*  description )
{
#ifndef _LINUX_FAPI
    return FAPI_SYS_ShellRegisterCommand(appShellInputHandler,
                                         command, function, brief, description);
#else
    return SHELL_RegisterCommand(command, function, brief, description);
#endif
}


/*
********************************************************************************
** The application's initialisation thread.
**
** This function is required to initialise the entire system.
** Within a non-RTOS environment is is called a a generic function,
** within an RTOS environment it is called within thread context.
********************************************************************************
*/
static void appInitialize( void )
{

#ifdef _LINUX_FAPI
    int32_t  index;
#endif

    FAPI_SYS_PRINT_MSG( "%s\n", APP_CLEARSCREEN );
    FAPI_SYS_PRINT_MSG( "##%s\n", APP_SEPARATOR );
    FAPI_SYS_PRINT_MSG( "#\n" );
    FAPI_SYS_PRINT_MSG( "# %s\n", APP_DESCRIPTION );
    FAPI_SYS_PRINT_MSG( "#\n" );
    FAPI_SYS_PRINT_MSG( "# Target....: %s (%s)\n", APP_TARGET, APP_CPU );
    FAPI_SYS_PRINT_MSG( "# Version...: v%s\n", APP_VERSION );
    FAPI_SYS_PRINT_MSG( "#\n" );
    FAPI_SYS_PRINT_MSG( "# %s\n", APP_COPYRIGHT );
    FAPI_SYS_PRINT_MSG( "# %s\n", APP_COPYRIGHT2 );
    FAPI_SYS_PRINT_MSG( "#\n" );
    FAPI_SYS_PRINT_MSG( "##%s\n", APP_SEPARATOR );
    FAPI_SYS_PRINT_MSG( "\n" );

    FAPI_SYS_PRINT_MSG( "%s creating KBD/RC input handler\n", APP_PREFIX );
#ifndef _LINUX_FAPI
    appShellInputHandler = FAPI_SYS_ShellCreateInputHandler( APP_SHELL_PROMPT,
                                                             50,
                                                             NULL );
    if( !appShellInputHandler ) {
        exit(1);
    }
#else
    for( index=0; index < APP_CommandTableSize; index++ )
    {
        commandTable[index].command     = NULL;
        commandTable[index].brief       = NULL;
        commandTable[index].description = NULL;
        commandTable[index].function    = NULL;
    }
    memset( inputCommand, '\0', APP_CommandStringSize );
    for( index=0; index < APP_HistoryTableSize; index++ )
        memset( historyTable[index], '\0', APP_CommandStringSize );
    historyIndex = 0;
    updownIndex  = 0;

    (void)appRegisterCommand(
        "?",
        SHELL_HelpCommand,
        "display all available commands",
        "To get more help about a specific command simply pass the command\n" \
        "as optional argument to ?\n"                                         \
        "Example: ?       displays all available commands, one per line\n"    \
        "Example: ? !     displays a more detailed help for the ! command\n"
    );

    (void)appRegisterCommand(
        "h",
        SHELL_HistoryCommand,
        "display the command history",
        NULL
    );

    (void)appRegisterCommand(
        "!",
        SHELL_RedoCommand,
        "redo a previous command",
        "This functions re-calls a previous command identified by number.\n" \
        "To find out the index of a previous command use the \"history\".\n" \
        "Example: !4     re-calls the command listed as #4 in the history output\n" \
        "Example: !!     re-calls the last command\n"
    );

    (void)appRegisterCommand(
        "kbd:cursor-up",
        SHELL_CursorUpDownCommand,
        "loops backward through the command history",
        NULL
    );

    (void)appRegisterCommand(
        "kbd:cursor-down",
        SHELL_CursorUpDownCommand,
        "loops foreward through the command history",
        NULL
    );

    (void)appRegisterCommand(
        "kbd:backspace",
        SHELL_CursorUpDownCommand,
        "deletes the character left of cursor",
        NULL
    );
#endif /* !_LINUX_FAPI */


    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "exit" );
    (void)appRegisterCommand(
        "exit",
        APP_HaltCommand,
        "Exit application.",
        "Usage: exit\n"
    );
    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "bdetect" );
    (void)appRegisterCommand(
        "bdetect",
        APP_BDetect,
        "bdetect application.",
        "Usage: bdetect\n"
    );
    FAPI_SYS_PRINT_MSG( "%s starting command shell\n", APP_PREFIX );

#ifndef _LINUX_FAPI
    FAPI_SYS_ShellStartInputHandler( appShellInputHandler );
#endif
}

/*
*****************************************************************************
**
** \brief Shell thread
**
** This function monitors the input command.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appShellThreadFunction(void* optArg)
{
#ifndef _LINUX_FAPI
    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appShellThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));
    appInitialize();

#else
    int32_t   i, c;
    char_t    args_buff[APP_CommandStringSize];

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appShellThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));
    appInitialize();

    FAPI_SYS_PRINT_MSG( "%s%s", APP_ColorBgWhite, APP_ColorFgBlack );
    strcpy( args_buff, "?" );
    SHELL_DoCommand( args_buff );
    strcpy( args_buff, "? ?" );
    SHELL_DoCommand( args_buff );
    inputCommand[0] = '\0';
    FAPI_SYS_PRINT_MSG( "\n" );

    while(1) {
        i = 0;
        APP_PrintPrompt();

        /* get command string */
        while( !feof(stdin) ) {
            c = getchar();
            args_buff[i] = c;
            if( (c == '\n') || (i >= (APP_CommandStringSize-1)) ) {
                args_buff[i] = '\0';
                break;
            }
            else if( (c == '\b') && (i > 0) ) {
                i --;
            }
            else {
                i++;
            }
        }

        SHELL_DoCommand( args_buff );

        FAPI_SYS_SLEEP(100);
    }
#endif /* !_LINUX_FAPI */


}

#ifdef USE_WATCHDOG
/*
*****************************************************************************
**
** \brief Watchdog thread
**
** This function will call the watchdog reset to prevent a system restart.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appWatchdogThreadFunction(void* optArg)
{
    int32_t  result;

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appWatchdogThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    for(;;)
    {
        result = FAPEX_CLKPWR_Ioctl( fd_CLKPWR,
                                     FAPEX_IOCCOM_CLKPWR_WATCHDOG_RESET,
                                     (FAPEXIOCARG3_T)NULL );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR:[main.c] Watchdog Reset %d\n", result);
            break;
        }

        FAPI_SYS_SLEEP( 1000 );
    }
}
#endif /* USE_WATCHDOG */

/*
*****************************************************************************
**
** \brief Start thread
**
** This function implements the very first thread which is used to create
** all other required threads and test cases.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appStartThreadFunction(void* optArg)
{
    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appStartThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    appShellThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_SHELL_THREAD_STACKSIZE,
            APP_SHELL_THREAD_PRIORITY,
            appShellThreadFunction,
            0, 0, APP_SHELL_THREAD_NAME );
    if( appShellThreadHandle == NULL )
    {
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                            APP_SHELL_THREAD_NAME );
        exit(1);
    }

    // Open CLKPWR
    fd_CLKPWR = FAPEX_CLKPWR_Open( DRV_ID_CLKPWR,
                                   O_RDWR,
                                   (FAPEXOPENARG3_T)NULL );
    if(fd_CLKPWR < 0) {
        FAPI_SYS_PRINT_MSG("ERROR:[main.c] Open CLKPWR driver.\n");
    }
#ifdef USE_WATCHDOG
    else {
        int32_t  result;

        result = FAPEX_CLKPWR_Ioctl( fd_CLKPWR,
                                     FAPEX_IOCCOM_CLKPWR_WATCHDOG_START,
                                     (FAPEXIOCARG3_T)FAPEX_CLKPWR_WATCHDOG_MIN_VALUE );
        if(result < 0) {
            FAPI_SYS_PRINT_MSG( "ERROR:[main.c] Watchdog Start %d\n", result );
        }
        else {
            appWatchdogThreadHandle = FAPEX_SYS_CreateThread(
                    NULL,
                    APP_WATCHDOG_THREAD_STACKSIZE,
                    APP_WATCHDOG_THREAD_PRIORITY,
                    appWatchdogThreadFunction,
                    0, 0, APP_WATCHDOG_THREAD_NAME );
            if( appWatchdogThreadHandle == NULL )
            {
                FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                                    APP_WATCHDOG_THREAD_NAME );
                FAPEX_CLKPWR_Close( fd_CLKPWR );
                exit(1);
            }
        }
    }
    FAPEX_SYS_WaitEndThread( appWatchdogThreadHandle );
#endif /* USE_WATCHDOG */

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appStartThreadFunction() end\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    FAPEX_SYS_WaitEndThread( appShellThreadHandle );
}

//***********************************************************************
//***********************************************************************
//** External Functions
//***********************************************************************
//***********************************************************************

#ifdef _LINUX_FAPI
//***************************************************************************
static void print_usage(char *name)
{
    FAPI_SYS_PRINT_MSG("usage: %s [-s hd_mode (sd_mode)]\n", name);
}
#endif

/*
*****************************************************************************
** \brief Testprogram main function
*****************************************************************************
*/
int32_t main( int32_t argc, char_t* argv[] )
{
    int32_t  result = 0;

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, main() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    int32_t fd_boardifc;
    fd_boardifc =  FAPEX_BOARDIFC_Open(FAPEXDEV_BOARDIFC_MINOR_FIRST,O_RDWR,NULL );
    if(fd_boardifc < 0)
    {
        FAPI_SYS_PRINT_MSG("open board error\n");
    }
    else
    {
        FAPEX_BOARD_HwPlatformInfosStrT  data;
        FAPEX_BOARDIFC_ChipIDParamStrT chipData;
        char pzHwManufactString[32];
        char pzHwModelString[32];

        data.pHwManufactString = pzHwManufactString;
        data.pHwModelString = pzHwModelString;

        FAPEX_BOARDIFC_Ioctl(fd_boardifc,FAPEX_IOCCOM_BOARDIFC_GETBOARDINFO,(FAPEXIOCARG3_T)&data);
        FAPEX_BOARDIFC_Ioctl(fd_boardifc,FAPEX_IOCCOM_CHIP_ID,(FAPEXIOCARG3_T)&chipData);

        FAPI_SYS_PRINT_MSG("------------------------------------------\n");
        FAPI_SYS_PRINT_MSG("---Chip Info---\n");
        FAPI_SYS_PRINT_MSG("factory_code:0x%x\n",chipData.factory_code);
        FAPI_SYS_PRINT_MSG("LOT_No:0x%x\n",chipData.LOT_No);
        FAPI_SYS_PRINT_MSG("wafer_No:0x%x\n",chipData.wafer_No);
        FAPI_SYS_PRINT_MSG("wafer_Xaxis:0x%x\n",chipData.wafer_Xaxis);
        FAPI_SYS_PRINT_MSG("wafer_Yaxis:0x%x\n",chipData.wafer_Yaxis);
        FAPI_SYS_PRINT_MSG("------------------------------------------\n");
        FAPI_SYS_PRINT_MSG("---Board Info---\n");
        FAPI_SYS_PRINT_MSG("MagicAndVersion:0x%x\n",data.MagicAndVersion);
        FAPI_SYS_PRINT_MSG("HwManufactString:%s\n",data.pHwManufactString);
        FAPI_SYS_PRINT_MSG("HwModelString:%s\n",data.pHwModelString);
        FAPI_SYS_PRINT_MSG("flagUsbHostOnly:0x%x\n",data.flagUsbHostOnly);
        FAPI_SYS_PRINT_MSG("typeOfFlashAvailable:0x%x\n",data.typeOfFlashAvailable);
        FAPI_SYS_PRINT_MSG("AudioConfigurationInfoA:0x%x\n",data.AudioConfigurationInfoA);
        FAPI_SYS_PRINT_MSG("AudioConfigurationInfoB:0x%x\n",data.AudioConfigurationInfoB);
        FAPI_SYS_PRINT_MSG("numberOfAvlFrontEnds:0x%x\n",data.numberOfAvlFrontEnds);
        FAPI_SYS_PRINT_MSG("numberOfAvlCiSlots:0x%x\n",data.numberOfAvlCiSlots);
        FAPI_SYS_PRINT_MSG("flagHardDiskInterface:0x%x\n",data.flagHardDiskInterface);
        FAPI_SYS_PRINT_MSG("flgDdr2MemoryInfos:0x%x\n",data.flgDdr2MemoryInfos);
        FAPI_SYS_PRINT_MSG("videoAmpCharacteristic:0x%x\n",data.videoAmpCharacteristic);
        FAPI_SYS_PRINT_MSG("------------------------------------------\n");

    }

    while(1);

#ifndef _LINUX_FAPI
    ///////////////////////////////////////////////////////////////////
    // This ugly delay loop is required to get the entire system
    // also running from flash, becuase the ROM boot loader first
    // starts an ARM application, but "av_demo" expects the ARC based
    // audio firmware already running.
    // During "make run" the audio firmware will be executed first,
    // and the "av_demo" will be started after a 1 second delay!
    ///////////////////////////////////////////////////////////////////

    #if defined(__arm__) && !defined(__GNUC__)
        #pragma O0
    #endif
    {
        uint32_t  delay;

        for( delay=0; delay < 0x00100000; delay++ );
    }
    #if defined(__arm__) && !defined(__GNUC__)
        #pragma O2
    #endif
    //BOARDS_INSTALL_PLATFORM_SUPPORT();
    result = FAPEX_common_init();
    if ( result != FAPI_OK ) {
        exit( result );
    }
#else
    /* initial input settings */
    if(argc > 1) {
        int32_t  i;

        for(i=1; i<argc; i++) {
            if(strcmp("-s", argv[i]) == 0) {

            }
            else if(*argv[i] == '-') {
                print_usage(argv[0]);
                exit(1);
            }
        }
    }
#endif

    appStartThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_START_THREAD_STACKSIZE,
            APP_START_THREAD_PRIORITY,
            appStartThreadFunction,
            0, 0, APP_START_THREAD_NAME );
    if( appStartThreadHandle == NULL )
    {
        FAPI_SYS_PRINT_MSG( "ERROR: creating thread \"%s\" failed\n",
                            APP_START_THREAD_NAME );
        exit(1);
    }

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, thread start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    FAPEX_SYS_StopPrintThread();
    FAPEX_SYS_StartMultitasking();

    FAPEX_SYS_WaitEndThread( appStartThreadHandle );

    exit( result );
}
