/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
//#include <fapi/sys_services.h>
//#include <fapi/drv_timer.h>
//#include <fe/fe.h>

#include "frontend.h"


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
//#define DEBUG 2
#define SCAN_DEMO_TRANSPONDERS

/* MISRA: only preprocessor statements and comments before '#include' */
/*lint -save -e961 */
static FE_TUNE_SAT_S defaultTS[] = { 0 };
/*lint -restore */

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/
FAPI_SYS_HandleT    feHandle = NULL;
FE_SYSTEM_TYPE_E    delivery_type = FE_SYSTEM_TYPE_DIRECT;


/*---------------------------------------------------------------------------*/
/* external data                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* functions accessible from extern                                          */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* init function                                                             */
/*---------------------------------------------------------------------------*/
void FRONTEND_Init(void)
{
    int32_t err_code = FE_Init();

    if( err_code != FAPEX_OK)
        FAPI_SYS_PRINT_MSG("[frontend.c] FE_Init(): (%d)\n",err_code);
}


/*---------------------------------------------------------------------------*/
/* return delivery type (sat, ter, cab, ...                                  */
/*---------------------------------------------------------------------------*/
FE_SYSTEM_TYPE_E FRONTEND_GetDeliveryType(void)
{
    return(delivery_type);
}

/*---------------------------------------------------------------------------*/
/* fill up tune parameters with some default values                          */
/*---------------------------------------------------------------------------*/
int32_t FRONTEND_GetNumberOfTransportStreams(void)
{
    return(sizeof(defaultTS)/sizeof(defaultTS[0]));
}

/*---------------------------------------------------------------------------*/
/* Open FE function                                                          */
/*---------------------------------------------------------------------------*/
int32_t FRONTEND_Open(uint32_t slotNumber)
{
    int32_t            error_code = 0;
    FE_SYSTEM_PARAMS_S sys_params;
    FE_OpenParamsT     open_params;

    memset(&sys_params, 0, sizeof(sys_params));
    memset(&open_params, 0, sizeof(open_params));

    /* in case it was opened before */
    (void)FE_Close(feHandle);
    feHandle = NULL;

    open_params.version = FE_VERSION;
    open_params.slotNum = (uint8_t)slotNumber;

    if(FE_DetectSlot(open_params.slotNum) != FAPEX_OK)
        return -1;

    feHandle = FE_Open(&open_params, &error_code);

    if(!feHandle)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[frontend.c] FE_Open(): (%d))\n", error_code);
        return(error_code);
    }

    /* set system parameters for FE depending on delivery type */
    error_code = FE_GetSystemType(feHandle, &delivery_type);
    if(error_code != FAPEX_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[frontend.c] FE_GetSystemType(): (%d))\n", error_code);
        return(error_code);
    }

    switch(delivery_type)
    {
        case FE_SYSTEM_TYPE_TERRESTRIAL:
            FAPI_SYS_PRINT_MSG("[frontend.c] detected type: TERRESTRIAL\n");
            sys_params.terr.autoBandwidth     = 1;
            sys_params.terr.autoFEC           = 1;
            sys_params.terr.autoModulation    = 1;
            sys_params.terr.autoTransMode     = 1;
            sys_params.terr.autoGuardInterval = 1;
            sys_params.terr.autoHierarchy     = 1;
            sys_params.terr.defaultIQ         = FE_IQ_AUTO;
            sys_params.terr.custom            = NULL;
            break;

        case FE_SYSTEM_TYPE_SATELLITE:
            FAPI_SYS_PRINT_MSG("[frontend.c] detected type: SATELLITE\n");
            sys_params.sat.classic       = 1;
            sys_params.sat.posBy22kHz    = 0;
            sys_params.sat.toneburst     = 0;
            sys_params.sat.diseqc        = FE_DISEQC_1_0;
            sys_params.sat.diseqc10Rep   = 0;
            sys_params.sat.diseqc11Rep   = 0;
            sys_params.sat.lowFrequency  = 9750000;
            sys_params.sat.highFrequency = 10600000;
            sys_params.sat.autoFEC       = 1;
            sys_params.sat.defaultIQ     = FE_IQ_AUTO;
            break;

        case FE_SYSTEM_TYPE_CABLE:
            FAPI_SYS_PRINT_MSG("[frontend.c] detected type: CABLE\n");
            sys_params.cable.autoModulation = 1;
            sys_params.cable.autoFEC        = 1;
            sys_params.cable.defaultIQ      = FE_IQ_AUTO;
            break;

        case FE_SYSTEM_TYPE_DIRECT:
            return(-1);

        default:
            return(-1);
    }

    error_code = FE_SetSystemParams(feHandle, &sys_params);
    if(error_code != FAPEX_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[frontend.c] FE_SetSystemParams(): (%d))\n", error_code );
        return(error_code);
    }

    return(FAPEX_OK);
}


/*---------------------------------------------------------------------------*/
/* fill up satellite system parameters                                       */
/*---------------------------------------------------------------------------*/
int32_t FRONTEND_SetSatParmeters(FE_SYSTEM_PARAMS_S *sys_params)
{
    int32_t             error_code = 0;

    if(!feHandle)
    {
        FAPI_SYS_PRINT_MSG("[frontend.c] ERROR: open FE driver before using it.\n");
        return(-1);
    }

    (void)FE_Reset(feHandle);

    error_code = FE_SetSystemParams(feHandle, sys_params);

    if(error_code != FAPEX_OK)
    {
        FAPI_SYS_PRINT_MSG("[frontend.c] FE_SetSystemParams(): (%d)\n", error_code);
        return(error_code);
    }

    return(FAPEX_OK);
}


/*---------------------------------------------------------------------------*/
/* fill up tune parameters with some default values                          */
/*---------------------------------------------------------------------------*/
int32_t FRONTEND_SetDefaultTuneParmeters(int32_t index, FE_TUNE_PARAMS_S *tune_params)
{
    if(!feHandle)
    {
        FAPI_SYS_PRINT_MSG("[frontend.c] ERROR: open FE driver before using it.\n");
        return(-1);
    }

    if((uint32_t)index >= (sizeof(defaultTS)/sizeof(defaultTS[0])))
    {
        FAPI_SYS_PRINT_MSG("[frontend.c] ERROR: index out of range (max.=0-%i).\n",
                                (sizeof(defaultTS)/sizeof(defaultTS[0]))-1);
        return(-1);
    }

    memset(tune_params, 0, sizeof(FE_TUNE_PARAMS_S));

    switch(delivery_type)
    {
        case FE_SYSTEM_TYPE_TERRESTRIAL:
            /* DVB-T frequencies for Rhein-Main region (channel 22 ARD) */
            tune_params->terr.frequency     = 482000000;
            tune_params->terr.bandwidth     = FE_BANDWIDTH_8_MHZ;

            tune_params->terr.codeRateHP    = FE_FEC_AUTO;
            tune_params->terr.codeRateLP    = FE_FEC_AUTO;
            tune_params->terr.modulation    = FE_MOD_AUTO;
            tune_params->terr.transmission  = FE_TRANS_MODE_AUTO;
            tune_params->terr.guardInterval = FE_GUARD_INTERVAL_AUTO;
            tune_params->terr.hierarchy     = FE_HIERARCHY_AUTO;
            tune_params->terr.iq            = FE_IQ_AUTO;
            break;

        case FE_SYSTEM_TYPE_SATELLITE:
            /* ASTRA ARD transponder */
            memcpy(tune_params, &defaultTS[index], sizeof(FE_TUNE_PARAMS_S));
            break;

        case FE_SYSTEM_TYPE_CABLE:
            /* Fujitsu lab test transponder */
            tune_params->cable.frequency       = 650000000;
            tune_params->cable.symbolRate      = 6671;
            tune_params->cable.modulation      = FE_MOD_QAM_256;

            tune_params->cable.fec             = FE_FEC_AUTO;
            tune_params->cable.iq              = FE_IQ_AUTO;
            break;

        case FE_SYSTEM_TYPE_DIRECT:
            return(-1);

        default:
            return(-1);
    }

    return(FAPEX_OK);
}

/*---------------------------------------------------------------------------*/
/* tune function                                                             */
/*---------------------------------------------------------------------------*/
int32_t FRONTEND_Tune(FE_TUNE_PARAMS_S *tune_params)
{
    int32_t            error_code    = 0;
    int32_t            timeout       = 5;
//#if RTOS_USE_FAMOS == 0
#if 0
    int32_t            time_out      = 1000;
#endif
    FE_STATUS_E        status        = FE_STATUS_UNKNOWN;


    if((!feHandle) && (delivery_type != FE_SYSTEM_TYPE_DIRECT) )
    {
        FAPI_SYS_PRINT_MSG("[frontend.c] ERROR: open FE driver before using it.\n");
        return(-1);
    }

    switch(delivery_type)
    {
        case FE_SYSTEM_TYPE_TERRESTRIAL:
            FAPI_SYS_PRINT_MSG("[frontend.c] freq:%i bandwidth:%i\n",
                tune_params->terr.frequency,
                tune_params->terr.bandwidth);
            break;

        case FE_SYSTEM_TYPE_SATELLITE:
            FAPI_SYS_PRINT_MSG("[frontend.c] freq:%i symbolrate:%i ",
                tune_params->sat.frequency,
                tune_params->sat.symbolRate);
            if(tune_params->sat.modulation == FE_MOD_QPSK_S2)
                FAPI_SYS_PRINT_MSG("QPSK_S2 ");
            if(tune_params->sat.modulation == FE_MOD_8PSK)
                FAPI_SYS_PRINT_MSG("8PSK ");
            if(tune_params->sat.pol == FE_POL_HOR)
            {
                FAPI_SYS_PRINT_MSG("horizontal\n");
            }
            else
            {
                FAPI_SYS_PRINT_MSG("vertical\n");
            }
            break;

        case FE_SYSTEM_TYPE_CABLE:
            FAPI_SYS_PRINT_MSG("[frontend.c] freq:%i symbolrate:%i mod:",
                tune_params->cable.frequency,
                tune_params->cable.symbolRate);
                if( tune_params->cable.modulation == FE_MOD_QPSK)
                {
                    FAPI_SYS_PRINT_MSG("QPSK\n");
                }
                else if( tune_params->cable.modulation == FE_MOD_QPSK_S2)
                {
                    FAPI_SYS_PRINT_MSG("QPSK_S2\n");
                }
                else if( tune_params->cable.modulation == FE_MOD_8PSK )
                {
                    FAPI_SYS_PRINT_MSG("8PSK\n");
                }
                else if( tune_params->cable.modulation == FE_MOD_QAM_16)
                {
                    FAPI_SYS_PRINT_MSG("QAM16\n");
                }
                else if( tune_params->cable.modulation == FE_MOD_QAM_32)
                {
                    FAPI_SYS_PRINT_MSG("QAM32\n");
                }
                else if( tune_params->cable.modulation == FE_MOD_QAM_64)
                {
                    FAPI_SYS_PRINT_MSG("QAM64\n");
                }
                else if( tune_params->cable.modulation == FE_MOD_QAM_128)
                {
                    FAPI_SYS_PRINT_MSG("QAM128\n");
                }
                else if( tune_params->cable.modulation == FE_MOD_QAM_256)
                {
                    FAPI_SYS_PRINT_MSG("QAM256\n");
                }
                else
                {
                    FAPI_SYS_PRINT_MSG("AUTO\n");
                }
            break;

        case FE_SYSTEM_TYPE_DIRECT:
            return(FAPEX_OK);

        default:
            return(-1);
    }

    FAPI_SYS_PRINT_MSG("[frontend.c] Frontend tuning. Please wait...\n");
    error_code = FE_Tune(feHandle, tune_params);
    if(error_code != FAPEX_OK)
    {
        FAPI_SYS_PRINT_MSG("[frontend.c] FE_Tune(): (%d)\n", error_code);
        return(error_code);
    }

    for(;;)
    {
        error_code = FE_GetStatus(feHandle, &status);
        if(error_code != FAPEX_OK)
        {
            FAPI_SYS_PRINT_MSG("[frontend.c] FE_GetStatus(): (%d)\n", error_code);
            return(FAPI_ERR_TIMEOUT);
        }

        if(status == FE_STATUS_LOCKED)
            break;

        timeout--;
        if(timeout == 0)
        {
            FAPI_SYS_PRINT_MSG("[frontend.c] ERROR: timeout\n");
            FAPI_SYS_PRINT_MSG("[frontend.c] Check antenna input and start 'tune' again.\n");
            return(-1);
        }

        /*
        ***************************************************************
        **
        ** The polling of FAPI_TIMER_Bsr1() function is needed because
        ** in none RTOS enviroment the callbacks of software timer was
        ** never called if we don't do this.
        ** This callbacks are used by tuning the frontend.
        **
        ***************************************************************
        */
    //#if RTOS_USE_FAMOS == 0
    #if 0
        time_out = 500;
        for(;;)
        {
            FAPI_TIMER_Bsr1(0);
            FAPI_SYS_SLEEP(2);
            if( !time_out--  )
                break;
        }
    #else
        FAPI_SYS_SLEEP(1000);
    #endif

    }

    return(FAPEX_OK);
}
