

#include "pvr_m.h"
#include "pvr/pvr.h"
/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/* NOTE: If PVRSYS_ENCRYPTION_ENABLED is defined, encryption for recordings and
         decryption of playback will be enabled. This will apply on all
         playbacks and recordings. Currently no mechanism is available to
         detect if a PVR recording is encrypted or not. Playing a clear file
         with decryption enabled or playing a crypted file without decryption
         enabled will fail. */
#undef PVRSYS_ENCRYPTION_ENABLED

#define PVRSYS_BUFALIGN_SIZE        0x00000400
#define PVRSYS_BUFALIGN_MASK        0xFFFFFC00

#define PVRSYS_BUFREC_SIZE          (PVR_CHUNK_SIZE * 10)
#define PVRSYS_BUFPLAY_SIZE         (PVR_CHUNK_SIZE * 8)

#define PVRSYS_RECFILE_EXTENSION    "ts"

/* define application-specific track types */
#define PVR_TRACK_TYPE_ECM          (PVR_TRACK_TYPE_NUMS + 0)
#define PVR_TRACK_TYPE_EMM          (PVR_TRACK_TYPE_NUMS + 1)
#define PVR_TRACK_TYPE_TELETEXT     (PVR_TRACK_TYPE_NUMS + 2)
#define PVR_TRACK_TYPE_SUBTITLES    (PVR_TRACK_TYPE_NUMS + 3)

#define PVR_REC_ADD_SDT              //!< If defined, add SDT to CA recordings.

#define PVR_LIB_TIMEOUT        10000 //!< Timeout when waiting for state
                                     //!< changes in pvr_lib.
#define PVR_TIMEOUT_RETRIES      100 //!< Number of retries in polling loops.
#define PVR_TIMEOUT_SLEEPTIME     50 //!< Timeout in ms is: sleeptime * retries.
#define PVR_PERFLIMIT_TIMEOUT   6000 //!< Timeout in ms.

#define PVR_FATFS_FILESIZE_MAX   0x7FFFFFFF   // 2 GB - 1 Byte

// FIXME: track handling still needed?
#define PVR_REC_LOWER_TRACKS      6  //!< Tracks assigned in HW-controlled
                                     //!< channel area.
#define PVR_REC_UPPER_TRACKS      19 //!< Tracks assigned in upper channel area.
#if 0
#define PVR_REC_LOWER_START        1
#define PVR_REC_UPPER_START       10
#endif

#define PVR_REC_TRACKS_MAX        (PVR_REC_LOWER_TRACKS + PVR_REC_UPPER_TRACKS)

#define PVR_REC_VIDEO_MAX          1
#define PVR_REC_AUDIO_MAX          5
#define PVR_REC_TTX_MAX            2
#define PVR_REC_SUBTT_MAX          4
#define PVR_REC_EMM_MAX            5
#define PVR_REC_ECM_MAX            6

#define PVRSYS_PLAYMODE_FF_NUMS      8  //!< Number of fast forward modes
                                        //!< (index 0...).
#define PVRSYS_PLAYMODE_FR_NUMS      8  //!< Number of fast forward modes
                                        //!< (index 0...).
#define PVRSYS_PLAYMODE_SF_NUMS      3  //!< Number of slow forward modes
                                        //!< (index 0...).

// FIXME: maybe not necessary anymore
/* NOTE: (Sum of PVR_REC_XXX_MAX values + 2) <= PVR_REC_TRACKS_MAX
         The additional 2 tracks are needed for EIT/CAT filtering. CAT is only
         recorded for scrambled services that are to be descrambled by CI
         during playback.

         TSD Channel assigment for recording TSDs 2/3 is as follows:

          0      Used by fd_pip (TSD-3 only)
          1...6  PVR_REC_LOWER_START: Passed to PVR lib for recording.
          7...9  Used by SmartGo-HD for PAT/PMT/CAT
         10...28 PVR_REC_UPPER_START: Passed to PVR lib for recording.
         29      Used by SmartGo-HD for PIP.
         30      Used by PVR lib for playback.
         31      "buffer 31" used by PVR lib for generation of TS to be
                 recorded.
*/

#define PVR_PID_NONE          0xFFFF    //!< Marks track as unused.

#if defined(APPL_CI_ENABLED) && defined(APPL_DESCRREC_ENABLED)

/* Constants for TS playback feature */
#define PVR_LW_PER_PKT              47  //!< TS packet size in longwords.
#define PVR_TSPKTS_PER_LOOP         200 //!< Number of packets in one transfer
                                        //!< loop

#endif /* defined(APPL_CI_ENABLED) && defined(APPL_DESCRREC_ENABLED) */


#define PVR_CASYS_IDS_MAX           128

#if (PVR_REC_TRACKS_MAX > PVR_MAX_TRACKS )
    #error PVR configuration error!
#endif

#define PVRSYS_FLUSH_CHUNKS  256    //!< Number of recorded chunks after which
                                    //!< recording data will be periodically
                                    //!< flushed.

#define PVRSYS_PLAYMODE_FAST_NUMS   (PVRSYS_PLAYMODE_FF_NUMS + \
                                     PVRSYS_PLAYMODE_FR_NUMS)

enum
{
    PVRSYS_FASTCFG_MPEG = 0,    //!< Fast mode config. for MPEG video streams.
    PVRSYS_FASTCFG_H264,        //!< Fast mode config. for H.264 video streams.
    PVRSYS_FASTCFG_NOVIDEO,     //!< Fast mode config. for audio-only streams.
    PVRSYS_FASTCFG_NUMS
};

static PVRSYS_FastModeCfgT fastCfgMPEG[PVRSYS_PLAYMODE_FAST_NUMS] =
{
    { VAL_PLAYMODE_FAST_LIB_IONLY,       200, -256, 256 },   // FR
    { VAL_PLAYMODE_FAST_LIB_IONLY,       200, -128, 128 },   // FR
    { VAL_PLAYMODE_FAST_LIB_IONLY,       200,  -64,  64 },   // FR
    { VAL_PLAYMODE_FAST_LIB_IONLY,       200,  -32,  32 },   // FR
    { VAL_PLAYMODE_FAST_LIB_IONLY,       200,  -16,  16 },   // FR
    { VAL_PLAYMODE_FAST_LIB_IONLY,       200,   -8,   8 },   // FR
    { VAL_PLAYMODE_FAST_LIB_IONLY,       200,   -4,   4 },   // FR
    { VAL_PLAYMODE_FAST_LIB_IONLY,       200,   -2,   2 },   // FR
    #if 0
    { VAL_PLAYMODE_FAST_DEC_IP,            0,    0,   2 },   // FF
    { VAL_PLAYMODE_FAST_DEC_IONLY,         0,    0,   4 },   // FF
    #else
    { VAL_PLAYMODE_FAST_LIB_IONLY,       200,    2,   2 },   // FF
    { VAL_PLAYMODE_FAST_LIB_IONLY,       200,    4,   4 },   // FF
    #endif
    { VAL_PLAYMODE_FAST_LIB_IONLY,       200,    8,   8 },   // FF
    { VAL_PLAYMODE_FAST_LIB_IONLY,       200,   16,  16 },   // FF
    { VAL_PLAYMODE_FAST_LIB_IONLY,       200,   32,  32 },   // FF
    { VAL_PLAYMODE_FAST_LIB_IONLY,       200,   64,  64 },   // FF
    { VAL_PLAYMODE_FAST_LIB_IONLY,       200,  128, 128 },   // FF
    { VAL_PLAYMODE_FAST_LIB_IONLY,       200,  256, 256 }    // FF
};

static PVRSYS_FastModeCfgT fastCfgH264[PVRSYS_PLAYMODE_FAST_NUMS] =
{
    { VAL_PLAYMODE_FAST_CONTROLLEDFEED,  200, -256, 256 },   // FR
    { VAL_PLAYMODE_FAST_CONTROLLEDFEED,  200, -128, 128 },   // FR
    { VAL_PLAYMODE_FAST_CONTROLLEDFEED,  200,  -64,  64 },   // FR
    { VAL_PLAYMODE_FAST_CONTROLLEDFEED,  200,  -32,  32 },   // FR
    { VAL_PLAYMODE_FAST_CONTROLLEDFEED,  200,  -16,  16 },   // FR
    { VAL_PLAYMODE_FAST_CONTROLLEDFEED,  200,   -8,   8 },   // FR
    { VAL_PLAYMODE_FAST_CONTROLLEDFEED,  200,   -4,   4 },   // FR
    { VAL_PLAYMODE_FAST_CONTROLLEDFEED,  200,   -2,   2 },   // FR
    #if 0
    { VAL_PLAYMODE_FAST_DEC_IONLY,         0,    0,   2 },   // FF
    #else
    { VAL_PLAYMODE_FAST_CONTROLLEDFEED,       200,    2,   2 },   // FF
    #endif
    { VAL_PLAYMODE_FAST_CONTROLLEDFEED,  200,    4,   4 },   // FF
    { VAL_PLAYMODE_FAST_CONTROLLEDFEED,  200,    8,   8 },   // FF
    { VAL_PLAYMODE_FAST_CONTROLLEDFEED,  200,   16,  16 },   // FF
    { VAL_PLAYMODE_FAST_CONTROLLEDFEED,  200,   32,  32 },   // FF
    { VAL_PLAYMODE_FAST_CONTROLLEDFEED,  200,   64,  64 },   // FF
    { VAL_PLAYMODE_FAST_CONTROLLEDFEED,  200,  128, 128 },   // FF
    { VAL_PLAYMODE_FAST_CONTROLLEDFEED,  200,  256, 256 }    // FF
};

static PVRSYS_FastModeCfgT fastCfgAudio[PVRSYS_PLAYMODE_FAST_NUMS] =
{
    { VAL_PLAYMODE_FAST_NOFEED,            0, -256, 256 },   // FR
    { VAL_PLAYMODE_FAST_NOFEED,            0, -128, 128 },   // FR
    { VAL_PLAYMODE_FAST_NOFEED,            0,  -64,  64 },   // FR
    { VAL_PLAYMODE_FAST_NOFEED,            0,  -32,  32 },   // FR
    { VAL_PLAYMODE_FAST_NOFEED,            0,  -16,  16 },   // FR
    { VAL_PLAYMODE_FAST_NOFEED,            0,   -8,   8 },   // FR
    { VAL_PLAYMODE_FAST_NOFEED,            0,   -4,   4 },   // FR
    { VAL_PLAYMODE_FAST_NOFEED,            0,   -2,   2 },   // FR
    { VAL_PLAYMODE_FAST_NOFEED,            0,    2,   2 },   // FF
    { VAL_PLAYMODE_FAST_NOFEED,            0,    4,   4 },   // FF
    { VAL_PLAYMODE_FAST_NOFEED,            0,    8,   8 },   // FF
    { VAL_PLAYMODE_FAST_NOFEED,            0,   16,  16 },   // FF
    { VAL_PLAYMODE_FAST_NOFEED,            0,   32,  32 },   // FF
    { VAL_PLAYMODE_FAST_NOFEED,            0,   64,  64 },   // FF
    { VAL_PLAYMODE_FAST_NOFEED,            0,  128, 128 },   // FF
    { VAL_PLAYMODE_FAST_NOFEED,            0,  256, 256 }    // FF
};

static PVRSYS_FastModeCfgT* pFastCfg[PVRSYS_FASTCFG_NUMS] = { fastCfgMPEG,
                                                              fastCfgH264,
                                                              fastCfgAudio };

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

static int32_t 
PVRSYS_SetFastModes (const PVRSYS_FastModeCfgT* pCfg,
                                     uint16_t* index)
{
    PVR_FastModeConfigT fmConfig;
    int32_t             retVal = FAPI_OK;
    uint16_t            cnt;

    memset (&fmConfig, 0, sizeof(fmConfig));

    for ( cnt = 0; cnt < PVRSYS_PLAYMODE_FAST_NUMS; cnt++ )
    {
        switch ( pCfg[cnt].videoMode )
        {
            case VAL_PLAYMODE_FAST_DEC_IP:
            case VAL_PLAYMODE_FAST_DEC_IONLY:
                fmConfig.type = PVR_FASTMODETYPE_FEED_ALL;
                break;

            case VAL_PLAYMODE_FAST_NOFEED:
                fmConfig.type                   = PVR_FASTMODETYPE_FEED_NONE;
                fmConfig.params.feedNone.factor = pCfg[cnt].factor;
                break;

            case VAL_PLAYMODE_FAST_CONTROLLEDFEED:
                fmConfig.type = PVR_FASTMODETYPE_FEED_CONTROLLED;
                fmConfig.params.feedCtrl.factor      = pCfg[cnt].factor;
                fmConfig.params.feedCtrl.displayTime = pCfg[cnt].
                                                       iFrameDisplayTime;
                break;

            default:
            case VAL_PLAYMODE_FAST_LIB_IONLY:
                fmConfig.type                       = PVR_FASTMODETYPE_IFRAMES;
                fmConfig.params.iFrames.factor      = pCfg[cnt].factor;
                fmConfig.params.iFrames.displayTime = pCfg[cnt].
                                                      iFrameDisplayTime;
                break;
        }

        retVal = PVR_SetFastModeConfig(*index + cnt, &fmConfig);
        if ( retVal != FAPI_OK ) break;
    }

    *index += cnt;

    return retVal;
}

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/
int32_t 
PVRSYS_PlaySpeedToFastModeIdx (FAPEX_SYS_HandleT playHandle,
                                               int16_t speedIndex,
                                               uint8_t* pModeIndex)
{
    int32_t         retVal;
    PVR_TrackInfoT  trackInfo;
    uint8_t         track   = 0xFF;
    int32_t         fastCfg = PVRSYS_FASTCFG_MPEG;

    if ( (speedIndex < (-PVRSYS_PLAYMODE_FR_NUMS))
      || (speedIndex >=  PVRSYS_PLAYMODE_FF_NUMS) )
    {
        retVal = -1;
    }
    else
    {
        retVal = PVR_GetPlaybackTrack (playHandle, PVR_DECTYPE_VIDEO,
                                       &track);

        if ( retVal == FAPI_OK )
        {
            retVal = PVR_GetPlaybackTrackInfo (playHandle, &trackInfo);

            if ( retVal == FAPI_OK )
            {
                switch ( trackInfo.trackFormat[track] )
                {
                    default:
                    case PVR_TRACK_FORMAT_VIDEO_MPEG1:
                    case PVR_TRACK_FORMAT_VIDEO_MPEG2:
                        fastCfg = PVRSYS_FASTCFG_MPEG;
                        break;

                    case PVR_TRACK_FORMAT_VIDEO_H264:
                        fastCfg = PVRSYS_FASTCFG_H264;
                        break;
                }
            }
        }
        else if ( retVal == SUBSYS_PVR_ERR_NO_TRACK_ASSIGNED )
        {
            fastCfg = PVRSYS_FASTCFG_NOVIDEO;
            retVal  = FAPI_OK;
        }
    }

    if ( retVal == FAPI_OK )
    {
        *pModeIndex = (uint8_t)((fastCfg * PVRSYS_PLAYMODE_FAST_NUMS)
                                + speedIndex + PVRSYS_PLAYMODE_FR_NUMS);
    }

    return retVal;
}

int32_t 
PVRSYS_PlayFastModeIdxToSpeed (uint8_t modeIndex, int16_t* pSpeed)
{
    int32_t retVal;

    if ( modeIndex >= (PVRSYS_FASTCFG_NUMS * PVRSYS_PLAYMODE_FAST_NUMS) )
    {
        retVal = -1;
    }
    else
    {
        *pSpeed = (modeIndex % PVRSYS_PLAYMODE_FAST_NUMS)
                  -  PVRSYS_PLAYMODE_FR_NUMS;

        retVal = FAPI_OK;
    }

    return retVal;
}

uint16_t 
PVRSYS_PlayGetFastSpeedFactor (uint8_t modeIndex)
{
    uint16_t speedFactor = 0;

    if ( modeIndex < (PVRSYS_FASTCFG_NUMS * PVRSYS_PLAYMODE_FAST_NUMS) )
    {
        speedFactor = pFastCfg[modeIndex / PVRSYS_PLAYMODE_FAST_NUMS]
                              [modeIndex % PVRSYS_PLAYMODE_FAST_NUMS].
                              speedFactor;
    }

    return speedFactor;
}

uint16_t 
PVRSYS_PlayGetSlowSpeedFactor (uint8_t modeIndex)
{
    PVRSYS_SlowModeCfgT modeCfg;

    modeCfg.speedFactor = 0;

    (void)PVRSYS_GetSlowModeConfig (modeIndex, &modeCfg);

    return modeCfg.speedFactor;
}

int32_t
PVRSYS_Init(void)
{
    uint16_t cnt;
    uint16_t index;
    int32_t  retVal;

    /* ----- configure fast modes -------------------------------------- */

    /*

    NOTE: Fast modes are configured as follows:
    -  0 ...  7     FF modes MPEG
    -  8 ... 15     FR modes MPEG
    - 16 ... 23     FF modes H.264
    - 24 ... 31     FR modes H.264
    - 32 ... 39     FF modes audio
    - 40 ... 47     FR modes audio

    Speed is approximately doubled with each fast mode index, starting with
    2x normal speed.
    */

    index = 0;

    for ( cnt = 0; cnt < PVRSYS_FASTCFG_NUMS; cnt++ )
    {
        retVal = PVRSYS_SetFastModes(pFastCfg[cnt], &index);
        if ( retVal != FAPI_OK ) break;
    }

    return FAPI_OK;
}




int32_t 
PVRSYS_GetFastModeConfig (uint16_t index, PVRSYS_FastModeCfgT* cfgPtr)
{
    int32_t result;

    if ( index >= (PVRSYS_FASTCFG_NUMS * PVRSYS_PLAYMODE_FAST_NUMS) )
    {
        result = -1;
    }
    else
    {
        *cfgPtr = pFastCfg[index / PVRSYS_PLAYMODE_FAST_NUMS]
                          [index % PVRSYS_PLAYMODE_FAST_NUMS];

        result = FAPI_OK;
    }

    return result;
}

int32_t 
PVRSYS_GetSlowModeConfig (uint16_t index, PVRSYS_SlowModeCfgT* cfgPtr)
{
    memset(cfgPtr, 0, sizeof(PVRSYS_SlowModeCfgT));

    switch ( index )
    {
        case 0:
            cfgPtr->videoMode   = VAL_PLAYMODE_SLOW_2X;
            cfgPtr->speedFactor = 2;
            break;

        case 1:
            cfgPtr->videoMode   = VAL_PLAYMODE_SLOW_4X;
            cfgPtr->speedFactor = 4;
            break;

        default:
            cfgPtr->videoMode   = VAL_PLAYMODE_SLOW_8X;
            cfgPtr->speedFactor = 8;
            break;
    }

    return FAPI_OK;
}
