/*
********************************************************************************
**
** \file        ./sample_application/fapex_av_demo/src/scan.c
**
** \version     $Id$
**
** \brief       transport stream scan.
**
** This file contains the declaration of the scan driver API. It initializes
** and configures TSD including all related buffers. Furthermore it provides
** a simple API to start/stop transport stream scan.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited
**
********************************************************************************
*/

#include <fapex/abst_types.h>
#include <fapex/abst_libtsd.h>
#include "tsd.h"
#include "scan.h"
#include "servicelist.h"

#if DEBUG == 2
    #include "sidebug.h"
#endif

//******************************************************************************
//******************************************************************************
//** Local Defines and Macros
//******************************************************************************
//******************************************************************************
#define SCAN_TIMEOUT                 1500 /* [msec] */

#define SCAN_SKIP_FILTER_RESULT      0x00000010
#define SCAN_SKIP_PAT_HEADER         0x00000008
#define SCAN_SKIP_PAT_ENTRY_LENGTH   0x00000004
#define SCAN_SKIP_SDT_HEADER         0x0000000B
#define SCAN_SKIP_PROVIDER_NAME      0x00000004
#define SCAN_SKIP_SERVICE_LENGTH     0x00000001
#define SCAN_SKIP_LOOP_INFO          0x00000005

/* DEFINES USED FOR DVB SI-PARSING OF DVB DESCRIPTORS ETS 300468 */
#define SCAN_SERVICE_DESCRIPTOR                         0x48
#define SCAN_TELETEXT_DESCRIPTOR                        0x56
#define SCAN_SUBTITLE_DESCRIPTOR                        0x59
#define SCAN_AC_3_DESCRIPTOR                            0x6a
#define SCAN_EAC_3_DESCRIPTOR                           0x7a
#define SCAN_CA_DESCRIPTOR                              0x09

#define SCAN_ALIGN(ptr, n_kbytes)\
        ((((uint32_t)(ptr)) + (n_kbytes << 10)) & ~((n_kbytes << 10) - 1))

//******************************************************************************
//******************************************************************************
//** Local Data
//******************************************************************************
//******************************************************************************

typedef struct {
    /* \brief TSD channel id */
    int32_t             id;
    uint32_t            pid;
    uint32_t            bufferLength;
    uint32_t            crcError;
    /* filter status */
    uint32_t            foundTable;
    uint32_t            filtering;
    uint32_t            stopRequest;
    void*               bufferPtr;
    uint8_t*            sectionDataPtr;
    uint32_t            bufferPtrPtr;
} SCAN_TableInfoT;

static SCAN_TableInfoT  scanPatTable;
static SCAN_TableInfoT  scanSdtTable;
static SCAN_TableInfoT  scanPmtTable;
static int32_t          scanProcess             = 0;
static uint32_t         scanPmtAmount           = 0;
static uint32_t         scanCurrentPmtIndex     = 0;
static uint32_t         scanStarted             = 0;
static uint32_t         scanSdtSections         = 0;
static uint32_t         scanSdtSectionsFiltered = 0;
static int32_t          scanTransportIndex      = 0;

FAPEX_SYS_SelectParamStrT  select_table;
void                       *select_work;

//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************
extern int32_t fd_TSD[];


//******************************************************************************
//******************************************************************************
//** Local Functions Declaration
//******************************************************************************
//******************************************************************************

static int32_t scanOpen(void);
static void    scanClose(SCAN_TableInfoT *table);
static void    scanSearchNextPmtPidInPat(void);
static int32_t scanProcessPmtTable(uint8_t* dataPtr);
static void    scanProcessSdtTable(uint8_t* dataPtr);
static int32_t scanGetTagInDescriptor(uint8_t* descriptionDataPtr,
                                      uint32_t descriptionLength,
                                      uint8_t tagToFind);
static void    scanGetServiceNameFromDescriptors(uint8_t* descriptionDataPtr,
                                                 uint16_t descriptionLength,
                                                 uint8_t* serviceName,
                                                 uint32_t* serviceType);

//******************************************************************************
//******************************************************************************
//** Local Functions
//******************************************************************************
//******************************************************************************

/*!
********************************************************************************
** Opens TSD/BM according to table and allocates memory.
********************************************************************************
*/
static int32_t scanOpen(void)
{
    int32_t                       size, result = 0;
    FAPEX_TSD_OpenSubIdParamStrT  pidChParam;
    FAPEX_TSD_SctFltParamStrT     sctFltParam;

    memset(&(sctFltParam.mask), 0, FAPEX_TSD_FILTER_SIZE);

    /* 1. open TSD channel */
    pidChParam.channelNo = PAT_CHANNEL_ID;
    pidChParam.sub_id    = FAPEXDEV_TSD0_SUB_MINOR_FIRST + PAT_CHANNEL_ID;
    result = FAPEX_TSD_Ioctl( fd_TSD[0], FAPEX_IOCCOM_TSD_OPEN_SUBID,
                              (FAPEXIOCARG3_T)&pidChParam );
    scanPatTable.id = pidChParam.sub_id;

    pidChParam.channelNo = PMT_CHANNEL_ID;
    pidChParam.sub_id    = FAPEXDEV_TSD0_SUB_MINOR_FIRST + PMT_CHANNEL_ID;
    result = FAPEX_TSD_Ioctl( fd_TSD[0], FAPEX_IOCCOM_TSD_OPEN_SUBID,
                              (FAPEXIOCARG3_T)&pidChParam );
    scanPmtTable.id = pidChParam.sub_id;

    pidChParam.channelNo = SDT_CHANNEL_ID;
    pidChParam.sub_id    = FAPEXDEV_TSD0_SUB_MINOR_FIRST + SDT_CHANNEL_ID;
    result = FAPEX_TSD_Ioctl( fd_TSD[0], FAPEX_IOCCOM_TSD_OPEN_SUBID,
                              (FAPEXIOCARG3_T)&pidChParam );
    scanSdtTable.id = pidChParam.sub_id;

    /* 2. Set table ID and section byte filter */
    /* PAT */
    scanPatTable.pid             = FAPEX_TSD_PAT_SECTION_PID;
    sctFltParam.pid              = scanPatTable.pid;
    sctFltParam.flags            = FAPEX_TSD_FLTFLAGS_RM_TS_HEADER | FAPEX_TSD_FLTFLAGS_TIMEOUT |
                                   FAPEX_TSD_FLTFLAGS_SWAPWR_SWAP | FAPEX_TSD_FLTFLAGS_SWAPMODE_4BYTE;
    sctFltParam.bufferSize       = PAT_BUFFER_SIZE;
    sctFltParam.bufferThresholdH = PAT_BUFFER_SIZE;
    sctFltParam.bufferThresholdL = 0;
    sctFltParam.filter[0]        = FAPEX_TSD_PAT_TABLE;
    sctFltParam.mask[0]          = 0xff;
    sctFltParam.timeout          = SCAN_TIMEOUT;

    result = FAPEX_TSD_Ioctl( scanPatTable.id, FAPEX_IOCCOM_TSD_SET_SECTION_FILTER,
                              (FAPEXIOCARG3_T)&sctFltParam );
    if(result < 0)
    {
        FAPI_SYS_PRINT_ERR("[SCAN] Error: PAT setup failed (code: %d).\n", result);
        return( result );
    }
    scanPatTable.filtering  = 0;
    scanPatTable.foundTable = 0;

    /* PMT */
    scanPmtTable.pid             = FAPEX_TSD_PMT_MIN_SECTION_PID;  /* dummy set */
    sctFltParam.pid              = scanPmtTable.pid;
    sctFltParam.bufferSize       = PMT_BUFFER_SIZE;
    sctFltParam.bufferThresholdH = PMT_BUFFER_SIZE;
    sctFltParam.filter[0]        = FAPEX_TSD_PMT_TABLE;
    result = FAPEX_TSD_Ioctl( scanPmtTable.id, FAPEX_IOCCOM_TSD_SET_SECTION_FILTER,
                              (FAPEXIOCARG3_T)&sctFltParam );
    if(result < 0)
    {
        FAPI_SYS_PRINT_ERR("[SCAN] Error: PMT setup failed (code: %d).\n", result);
        return( result );
    }
    scanPmtTable.filtering  = 0;
    scanPmtTable.foundTable = 0;

    /* SDT */
    scanSdtTable.pid             = FAPEX_TSD_SDT_SECTION_PID;
    sctFltParam.pid              = scanSdtTable.pid;
    sctFltParam.bufferSize       = SDT_BUFFER_SIZE;
    sctFltParam.bufferThresholdH = SDT_BUFFER_SIZE;
    sctFltParam.filter[0]        = FAPEX_TSD_SDT_ATS_TABLE;
    result = FAPEX_TSD_Ioctl( scanSdtTable.id, FAPEX_IOCCOM_TSD_SET_SECTION_FILTER,
                              (FAPEXIOCARG3_T)&sctFltParam );
    if(result < 0)
    {
        FAPI_SYS_PRINT_ERR("[SCAN] Error: SDT setup failed (code: %d).\n", result);
        return( result );
    }
    scanSdtTable.filtering  = 0;
    scanSdtTable.foundTable = 0;

    /* setup select */
    size = FAPEX_SYS_SelectGetSize((FAPEX_SELECT_MODE_READ | FAPEX_SELECT_MODE_EXPT));
    select_work = (void *)FAPI_SYS_MALLOC(size);
    result = FAPEX_SYS_SelectInit(&select_table, select_work,
                                 (FAPEX_SELECT_MODE_READ | FAPEX_SELECT_MODE_EXPT));
    if(result < 0)
    {
        FAPI_SYS_PRINT_ERR("FAPEX_SYS_SelectInit: Error %d\n", result);
    }
    return( result );
}

/*!
********************************************************************************
** Closes TSD/BM according to table and frees allocated memory.
********************************************************************************
*/
static void scanClose(SCAN_TableInfoT *table)
{
    int32_t result = 0;

    /******************************/
    /* close all TSDs and buffers */
    /******************************/
    FAPI_SYS_PRINT_DEBUG(0, "[SCAN] Close PID:%i\n", table->pid);

    FAPEX_SYS_SelectClearID( &select_table, table->id,
                             (FAPEX_SELECT_MODE_READ | FAPEX_SELECT_MODE_EXPT) );
    if(table->id >= 0) {
        result = FAPEX_TSD_Ioctl( table->id,
                                  FAPEX_IOCCOM_CLOSE_SUBID,
                                  (FAPEXIOCARG3_T)NULL );
        if(result != FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(1, "[SCAN] TSD Close failed (code: %d).\n", result);
        }
    }

    /* free buffers */
    if(table->sectionDataPtr)
    {
        FAPI_SYS_FREE(table->sectionDataPtr);
        memset(table, 0, sizeof(SCAN_TableInfoT));
        scanStarted--;
     }
     table->id = -1;
}

/*!
********************************************************************************
** Searches for PMTs listed in PAT
********************************************************************************
*/
static void scanSearchNextPmtPidInPat(void)
{
    int32_t  result         = 0;
    uint32_t pid            = 0;
    uint32_t service_number = 0;
    uint32_t service_index0 = 0;
    uint32_t service_index1 = 0;
    uint32_t pid_index0     = 0;
    uint32_t pid_index1     = 0;
    FAPEX_TSD_PidParamStrT pidChParam;

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, scanSearchNextPmtPidInPat() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    result = FAPEX_TSD_Ioctl( scanPmtTable.id, FAPEX_IOCCOM_STOP, (FAPEXIOCARG3_T)NULL );
    if( result < 0 ) {
        FAPI_SYS_PRINT_ERR("[SCAN] Error: could not stop PMT PID (code: %d)\n", result);
    }

    while((service_number == 0) && (scanCurrentPmtIndex <= scanPmtAmount))
    {
        service_index0 = (SCAN_SKIP_FILTER_RESULT +
                          SCAN_SKIP_PAT_HEADER +
                          (scanCurrentPmtIndex * SCAN_SKIP_PAT_ENTRY_LENGTH));
        service_index1 = (SCAN_SKIP_FILTER_RESULT +
                          SCAN_SKIP_PAT_HEADER +
                          (scanCurrentPmtIndex * SCAN_SKIP_PAT_ENTRY_LENGTH) +
                          1);
        service_number = scanPatTable.sectionDataPtr[service_index1];
        service_number |= (scanPatTable.sectionDataPtr[service_index0] << 8);

        pid_index0 = (SCAN_SKIP_FILTER_RESULT +
                      SCAN_SKIP_PAT_HEADER +
                      (scanCurrentPmtIndex * SCAN_SKIP_PAT_ENTRY_LENGTH) +
                      2);
        pid_index1 = (SCAN_SKIP_FILTER_RESULT +
                      SCAN_SKIP_PAT_HEADER +
                      (scanCurrentPmtIndex * SCAN_SKIP_PAT_ENTRY_LENGTH) +
                      3);
        pid = scanPatTable.sectionDataPtr[pid_index1];
        pid |= ((scanPatTable.sectionDataPtr[pid_index0] & 0x1F) << 8);

        scanCurrentPmtIndex++;
    }

    if( (scanCurrentPmtIndex - 1) >= scanPmtAmount )
    {
        /* all PMT's filtered so stop scanning */
        FAPI_SYS_PRINT_DEBUG(1, "[SCAN] Last PMT reached.\n");
        /* PMT aquired */
        scanPmtTable.filtering = 2;

        if(scanSdtTable.filtering != 1) {
            /* scan for SDT */
            scanSdtTable.filtering = 1;
            result = FAPEX_TSD_Ioctl( scanSdtTable.id, FAPEX_IOCCOM_START, (FAPEXIOCARG3_T)NULL );
            if(result < 0) {
                FAPI_SYS_PRINT_ERR("[SCAN] Error: could not start SDT PID (code: %d).\n", result);
            }
        }
        return;
    }

    scanPmtTable.pid        = pid;
    scanPmtTable.filtering  = 1;
    scanPmtTable.foundTable = 0;

    /* search for next PMT */
    pidChParam.pid   = pid;
    pidChParam.flags = 0;
    result = FAPEX_TSD_Ioctl( scanPmtTable.id, FAPEX_IOCCOM_TSD_SET_PID_VALUE,
                              (FAPEXIOCARG3_T)&pidChParam );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("[SCAN] Error: could not set PMT PID (code: %d).\n", result);
    }

    result = FAPEX_TSD_Ioctl( scanPmtTable.id, FAPEX_IOCCOM_START, (FAPEXIOCARG3_T)NULL );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("[SCAN] Error: could not start PMT PID (code: %d).\n", result);
    }
    scanPmtTable.stopRequest = 0;
}

/*!
********************************************************************************
** Evaluates the PMT data to register services
********************************************************************************
*/
static int32_t scanProcessPmtTable(uint8_t *dataPtr)
{
    SERVICELIST_ServiceInfo_T   service;
    uint32_t                    section_length     = 0;
    uint32_t                    description_length = 0;
    uint32_t                    stream_offset      = 0;
    uint32_t                    pid                = 0;
    uint32_t                    stream_type        = 0;
    uint32_t                    es_info_length     = 0;
    int32_t                     service_no         = -1;
    uint32_t                    iso_639            = 0;
    int32_t                     get_tag            = 0;

    /* clean service struct */
    memset(&service, 0, sizeof(SERVICELIST_ServiceInfo_T));

    /*************/
    /* check PMT */
    /*************/

    /* retrieve program number which is equal to service ID */
    service.serviceId   = (dataPtr[3] << 8) + dataPtr[4];

    /* get PCR PID */
    service.pcrPid      = ((dataPtr[8] << 8) + dataPtr[9]) & 0x1FFF;

    /* calculate offset to stream descriptor loop */
    section_length       = ((dataPtr[1] << 8) + dataPtr[2]) & 0x0FFF;
    description_length   = ((dataPtr[10] << 8) + dataPtr[11]) & 0x0FFF;
    if( !section_length )
    {
        FAPI_SYS_PRINT_MSG( "[SCAN] Error: PMT section length missing.\n" );
        return -1;
    }

    /* check first decriptor loop for CA descriptor */
    if(scanGetTagInDescriptor(&dataPtr[12], description_length, SCAN_CA_DESCRIPTOR) != -1)
        service.caFlag = 1;

    /* check stream for audio and video */
    stream_offset = 12 + description_length;
    while(stream_offset < (section_length - 1))
    {
        /* loop until stream offset is on first CRC32 byte */
        stream_type    = dataPtr[stream_offset];
        pid            = ((dataPtr[stream_offset+1]<<8) + dataPtr[stream_offset+2]) & 0x1FFF;
        es_info_length = ((dataPtr[stream_offset+3]<<8) + dataPtr[stream_offset+4]) & 0x0FFF;

        /* we parse all streams but as soon as one A/V is found we don't overwrite */
        switch(stream_type)
        {
            case VID_MPEG2:
            case VID_MPEG1:
            case VID_H264:
                if(service.videoPid == 0)
                {
                    service.videoPid  = pid;
                    service.videoType = stream_type;
                }
                break;

            case AUD_MPEG1:
            case AUD_MPEG2:
            case AUD_AAC_MPEG4:
            case AUD_AAC_MPEG2:
            case AUD_AC3_ATSC_A52:
                get_tag = scanGetTagInDescriptor(&dataPtr[stream_offset+5], es_info_length, 0x0a);
                if(get_tag != -1)
                {
                    iso_639 = (dataPtr[(int32_t)(stream_offset+7)+get_tag] << 24) +
                              (dataPtr[(int32_t)(stream_offset+8)+get_tag] << 16) +
                              (dataPtr[(int32_t)(stream_offset+9)+get_tag] << 8) +
                              dataPtr[(int32_t)(stream_offset+10)+get_tag];
                    if( dataPtr[(int32_t)(stream_offset+10)+get_tag]==0x3)
                    {
                        service.audioDescriptionLanguage = iso_639;
                        service.audioDescriptionPid = pid;
                    }
                    else
                    {
                        if(service.audioNumberOfTracks<SERVICELIST_MAX_AUDIO_TRACK_NUMBER)
                            service.audioNumberOfTracks++;
                        service.audioPid[service.audioNumberOfTracks-1]  = pid;
                        service.audioType[service.audioNumberOfTracks-1] = stream_type;
                        service.audioLanguage[service.audioNumberOfTracks-1] = iso_639;
                    }
                }
                else
                {
                    if(service.audioNumberOfTracks<SERVICELIST_MAX_AUDIO_TRACK_NUMBER)
                        service.audioNumberOfTracks++;
                    service.audioPid[service.audioNumberOfTracks-1]  = pid;
                    service.audioType[service.audioNumberOfTracks-1] = stream_type;
                }
                break;

            case PRIV_PES:
                if(scanGetTagInDescriptor(&dataPtr[stream_offset+5], es_info_length, SCAN_AC_3_DESCRIPTOR) != -1)
                {
                    if(service.audioNumberOfTracks<SERVICELIST_MAX_AUDIO_TRACK_NUMBER)
                        service.audioNumberOfTracks++;
                    service.audioPid[service.audioNumberOfTracks-1]  = pid;
                    service.audioType[service.audioNumberOfTracks-1] = USER_DEF_AC_3;
                }
                if(scanGetTagInDescriptor(&dataPtr[stream_offset+5], es_info_length, SCAN_EAC_3_DESCRIPTOR) != -1)
                {
                    if(service.audioNumberOfTracks<SERVICELIST_MAX_AUDIO_TRACK_NUMBER)
                        service.audioNumberOfTracks++;
                    service.audioPid[service.audioNumberOfTracks-1]  = pid;
                    service.audioType[service.audioNumberOfTracks-1] = USER_DEF_EAC_3;
                }
                if(scanGetTagInDescriptor(&dataPtr[stream_offset+5], es_info_length, SCAN_TELETEXT_DESCRIPTOR) != -1)
                {
                    service.teletextPid  = pid;
                }
                if(scanGetTagInDescriptor(&dataPtr[stream_offset+5], es_info_length, SCAN_SUBTITLE_DESCRIPTOR) != -1)
                {
                    service.dvbSubtitlePid  = pid;
                }
                break;
            default:
                break;
        }

        /* check second decriptor loop for CA descriptor */
        if(scanGetTagInDescriptor(&dataPtr[stream_offset+5], es_info_length, SCAN_CA_DESCRIPTOR) != -1)
            service.caFlag = 1;

        /* calculate offset for next descriptor */
        stream_offset += 5 + es_info_length;
    }

    /* Add service to list - in this example we add only video or audio services */
    if ((service.videoPid > 0) || (service.audioPid[0] > 0))
    {
        sprintf((char_t*)service.name, "SVC ID:%ul", service.serviceId);
        /* get transport stream list index to associate NIM parameters */
        service.trpListIndex = scanTransportIndex;
        service_no = SERVICELIST_AddService(&service);
    }
    return service_no;
}

/*!
********************************************************************************
** Extracts given tag from a descriptor. Return value is position of tag or -1
** if not found.
********************************************************************************
*/
static int32_t scanGetTagInDescriptor(uint8_t* descriptionDataPtr,
                                      uint32_t descriptionLength,
                                      uint8_t tagToFind)
{
    uint8_t  tag      = 0;
    uint8_t* data_ptr = 0;
    uint32_t count    = 0;

    count = 0;

    while( count < descriptionLength )
    {
        data_ptr = &descriptionDataPtr[count];
        tag = data_ptr[0];
        if (tag == tagToFind)
            return((int32_t)count);
        count += data_ptr[1] + 2; /* desc[1] = length */
    }

    return -1;
}

/*!
********************************************************************************
** Evaluates the data within an SDT table
********************************************************************************
*/
static void scanProcessSdtTable(uint8_t* dataPtr)
{
    SERVICELIST_ServiceInfo_T service;
    uint32_t                  section_length          = 0;
    uint32_t                  section_count           = 0;
    uint32_t                  description_loop_length = 0;
    uint32_t                  stream_offset           = 0;

    /* clean service struct */
    memset(&service, 0, sizeof(SERVICELIST_ServiceInfo_T));

    /*************/
    /* Check SDT */
    /*************/

    /* evaluate SDT sections */
    for(section_count = 0; section_count <= dataPtr[7]; section_count++)
        scanSdtSections |= 0x1 << section_count;

    /* SDT section has been received */
    scanSdtSectionsFiltered |= 0x1 << dataPtr[6];

    /* calculate offset to descriptor loop */
    section_length = ((dataPtr[1]<<8) + dataPtr[2]) & 0x0FFF;
    stream_offset = SCAN_SKIP_SDT_HEADER;

    if( !section_length )
    {
        FAPI_SYS_PRINT_MSG( "[SCAN] Error: SDT section length missing.\n" );
        return;
    }

    /* check descriptor for service name */
    while( stream_offset < (section_length - 1) )
    {
        /* loop until stream offset is on first CRC32 byte */
        service.serviceId = (dataPtr[stream_offset]<<8) + dataPtr[stream_offset+1];
        description_loop_length = ((dataPtr[(stream_offset + 3)] << 8)
                                  + dataPtr[(stream_offset + 4)]) & 0x0FFF;
        /* get service name from decriptor */
        scanGetServiceNameFromDescriptors(&dataPtr[(stream_offset + 5)], description_loop_length,
                                          service.name, &service.serviceType);
        //FAPI_SYS_PRINT_MSG("SVCID:%d name'%s'\n", service.serviceId, service.name);
        /* update service list */
        (void)SERVICELIST_UpdateService(&service);
        /* calculate offset for next descriptor */
        stream_offset += (SCAN_SKIP_LOOP_INFO + description_loop_length);
    }
}

/*!
********************************************************************************
** Extracts the service's name from a descriptor
********************************************************************************
*/
static void scanGetServiceNameFromDescriptors(uint8_t* descriptionDataPtr,
                                                 uint16_t descriptionLength,
                                                 uint8_t* serviceName,
                                                 uint32_t* serviceType)
{
    uint8_t     descriptor_tag     = 0;
    uint8_t     character          = 0;
    uint32_t    descriptor_counter = 0;
    uint32_t    character_counter  = 0;
    uint32_t    service_counter    = 0;
    uint32_t    service_offset     = 0;
    uint8_t*    data_ptr           = NULL;

    /* loop through each descriptor */
    while( descriptor_counter < descriptionLength )
    {
        /* point to the first field of each descriptor */
        data_ptr = &descriptionDataPtr[descriptor_counter];

        /* retrieve the descriptor's tag */
        descriptor_tag = data_ptr[0];
        /* retrieve descriptor length */
        //descriptor_counter += data_ptr[1] + 2;
        descriptor_counter += data_ptr[1] + 1;

        /* descroptor is a service descriptor */
        if(descriptor_tag == SCAN_SERVICE_DESCRIPTOR)
        {
            /* retrieve the service type */
            serviceType[0] = data_ptr[2];
            /* retrieve the service name */
            service_counter = 0;
            service_offset = (data_ptr[3] + SCAN_SKIP_PROVIDER_NAME + SCAN_SKIP_SERVICE_LENGTH);
            for(character_counter = 0;
                character_counter < data_ptr[(data_ptr[3] + SCAN_SKIP_PROVIDER_NAME)];
                character_counter++)
            {
                //character = data_ptr[data_ptr[3]+5+character_counter];
                character = data_ptr[service_offset + character_counter];
                if( (character > 0x10) &&
                    (character != 0x86) &&
                    (character != 0x87) &&
                    (character != 0x8A))
                {
                    if(service_counter < (SERVICELIST_MAX_SERVICE_NAME_LEN - 1))
                    {
                        serviceName[service_counter] = character;
                        service_counter++;
                    }
                }
            }

            /* end the service's name */
            serviceName[service_counter] = 0;
        }
    }
}

//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

/*!
********************************************************************************
**
** Starts transport stream scan.
**
** This function opens the TSD/BM handles for PAT, PMT and SDT.
********************************************************************************
*/
void SCAN_Start(int32_t trpListIndex)
{
    int32_t result = 0;

    if(scanStarted)
        SCAN_Stop();

    if(scanStarted != 0)
    {
        FAPI_SYS_PRINT_MSG("[SCAN] Error: scan could not be stopped.\n");
        return;
    }
    scanTransportIndex = trpListIndex;
    scanStarted = 3;
    scanSdtSectionsFiltered = 0;
    scanSdtSections = 0;
    FAPI_SYS_PRINT_MSG("\n[SCAN] Transport stream scan mode started! Please wait...\n");

    /* clear structures */
    memset(&scanPatTable, 0, sizeof(scanPatTable));
    memset(&scanPmtTable, 0, sizeof(scanPmtTable));
    memset(&scanSdtTable, 0, sizeof(scanSdtTable));

    /* allocate section data buffers */
    scanPatTable.sectionDataPtr = FAPI_SYS_MALLOC(PAT_BUFFER_SIZE);
    scanPmtTable.sectionDataPtr = FAPI_SYS_MALLOC(PMT_BUFFER_SIZE);
    scanSdtTable.sectionDataPtr = FAPI_SYS_MALLOC(SDT_BUFFER_SIZE);

    /* open TSD channels, BM buffers and set section filters */
    result = scanOpen();
    if(result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[SCAN] Error: aborting scan.\n");
        return;
    }

    /* reset PMT settings */
    scanPmtAmount = 0;
    scanCurrentPmtIndex = 0;

    /* start scanning for PAT */
    scanPatTable.filtering = 1;

    result = FAPEX_TSD_Ioctl( scanPatTable.id, FAPEX_IOCCOM_START, (FAPEXIOCARG3_T)NULL );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("[PAT] TSD Error %d (FAPEX_TSD_StartChannel/SCAN_Start)\n", result);
    }

    /* start scanning for SDT */
    scanSdtTable.filtering = 1;

    result = FAPEX_TSD_Ioctl( scanSdtTable.id, FAPEX_IOCCOM_START, (FAPEXIOCARG3_T)NULL );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("[SCAN] Error: could not start SDT PID (code: %d).\n", result);
    }

    scanProcess = 1;
}

/*!
********************************************************************************
**
** Stops transport stream scan.
**
** This function closes the TSD/BM handles for PAT, PMT and SDT.
********************************************************************************
*/
void SCAN_Stop(void)
{
    int32_t result = 0;

    if(scanProcess == 1)
    {
        scanProcess = 0;

        /******************************/
        /* close all TSDs and buffers */
        /******************************/

        /* PMT */
        if(scanPmtTable.id >= 0) {
            result = FAPEX_TSD_Ioctl( scanPmtTable.id, FAPEX_IOCCOM_STOP, (FAPEXIOCARG3_T)NULL );
            if(result < 0) {
                FAPI_SYS_PRINT_ERR("[SCAN] Error: could not stop PMT PID (code: %d)\n", result);
            }
        }
        scanClose(&scanPmtTable);

        /* PAT */
        if(scanPatTable.id >= 0) {
            result = FAPEX_TSD_Ioctl( scanPatTable.id, FAPEX_IOCCOM_STOP, (FAPEXIOCARG3_T)NULL );
            if(result < 0) {
                FAPI_SYS_PRINT_ERR("[SCAN] Error: could not stop PAT PID (code: %d)\n", result);
            }
        }
        scanClose(&scanPatTable);

        /* SDT */
        if(scanSdtTable.id >= 0) {
            result = FAPEX_TSD_Ioctl( scanSdtTable.id, FAPEX_IOCCOM_STOP, (FAPEXIOCARG3_T)NULL );
            if(result < 0) {
                FAPI_SYS_PRINT_ERR("[SCAN] Error: could not stop SDT PID (code: %d)\n", result);
            }
        }
        scanClose(&scanSdtTable);

        FAPI_SYS_FREE(select_work);

        FAPI_SYS_PRINT_MSG("[SCAN] Current scan stopped.\n");
    }

    scanStarted = 0;
}

/*!
********************************************************************************
**
** Task of the transport stream scan.
**
** Called periodically too update tables and monitor scanProcess.
**
** \retval  0 if not scanProcess
** \retval  1 if still scanProcess
** \retval  2 if first program is found
** \retval -1 when scan finished
********************************************************************************
*/
int32_t SCAN_Task(void)
{
    FAPEX_TSD_ReadInfoStrT  read_info;
    FAPEX_SYS_TIMEVAL_T     timeout;
    uint32_t section_length = 0;
    int32_t  service_no = -1;
    int32_t  max_id = -1;
    int32_t  result = 0;

    if(scanProcess == 0) {
        /* scan disable */
        return 0;
    }
    else if(scanProcess == -1) {
        /* scan completed */
        FAPI_SYS_PRINT_MSG("[SCAN] Scan mode finished.\n");

        /* any open tables shall be closed */
        if(scanStarted)
        {
            scanClose(&scanPatTable);
            scanClose(&scanPmtTable);
            scanClose(&scanSdtTable);
        }

        scanProcess = 0;

        return(-1);
    }

    if(scanPatTable.id >= 0) {
        result = FAPEX_SYS_SelectSetID( &select_table, scanPatTable.id,
                                        (FAPEX_SELECT_MODE_READ | FAPEX_SELECT_MODE_EXPT) );
        if(result == 0) {
            max_id = scanPatTable.id;
        }
    }
    if(scanSdtTable.id >= 0) {
        result = FAPEX_SYS_SelectSetID( &select_table, scanSdtTable.id,
                                        (FAPEX_SELECT_MODE_READ | FAPEX_SELECT_MODE_EXPT) );
        if(result == 0) {
            if(max_id < scanSdtTable.id) {
                max_id = scanSdtTable.id;
            }
        }
    }
    if(scanPmtTable.id >= 0) {
        result = FAPEX_SYS_SelectSetID( &select_table, scanPmtTable.id,
                                        (FAPEX_SELECT_MODE_READ | FAPEX_SELECT_MODE_EXPT) );
        if(result == 0) {
            if(max_id < scanPmtTable.id) {
                max_id = scanPmtTable.id;
            }
        }
    }

    timeout.tv_sec  = 5;
    timeout.tv_usec = 0;
    result = FAPEX_SYS_SelectSetParam( &select_table, max_id, &timeout );

    result = FAPEX_SYS_SelectCall( &select_table );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("Select Error %d (FAPEX_SYS_SelectCall/SCAN_Task)\n", result);
        goto SCAN_Task_table_close;
    }
    else if(result == 0) {
        goto SCAN_Task_timeout;
    }

    /************************/
    /* process found tables */
    /************************/

    /* PAT */
    if(scanPatTable.filtering == 1) {
        result = FAPEX_SYS_SelectCheck( &select_table, scanPatTable.id,
                                        (FAPEX_SELECT_MODE_READ | FAPEX_SELECT_MODE_EXPT) );
        if(result != 0) {
            FAPI_SYS_PRINT_DEBUG(1, "[SCAN] Read PAT.\n");

            /* read PAT data */
            result = FAPEX_TSD_Ioctl( scanPatTable.id, FAPEX_IOCCOM_TSD_GET_BUFFER_READINFO,
                                      (FAPEXIOCARG3_T)&read_info );
            if(result < 0) {
                FAPI_SYS_PRINT_ERR("\n[SCAN] Error: could not set PAT read pointer (code: %d).\n", result);
            }

            if(read_info.bufferStatus & FAPEX_TSD_BUFFSTAT_TIMEOUT) {
                goto SCAN_Task_table_close;
            }
            scanPatTable.bufferLength = read_info.fillSize;
            scanPatTable.crcError     = read_info.bufferStatus & FAPEX_TSD_BUFFSTAT_CRCERROR;
            scanPatTable.foundTable   = 1;

            result = FAPEX_TSD_Ioctl( scanPatTable.id, FAPEX_IOCCOM_STOP, (FAPEXIOCARG3_T)NULL );
            if(result < 0) {
                FAPI_SYS_PRINT_ERR("\n[SCAN] Error: could not stop PAT (code: %d).\n", result);
            }
            scanPatTable.stopRequest = 2;

            result = FAPEX_TSD_Read(scanPatTable.id, scanPatTable.sectionDataPtr, scanPatTable.bufferLength);
            if(result != (int32_t)scanPatTable.bufferLength) {
                FAPI_SYS_PRINT_ERR("\n[SCAN] Error: could not read PAT data (code: %d).\n", result);
            }

            /*********************/
            /* evaluate PAT data */
            /*********************/

            /* lower byte */
            section_length = scanPatTable.sectionDataPtr[2 + SCAN_SKIP_FILTER_RESULT];
            /* higher byte without the first 4 bits */
            section_length |= (( scanPatTable.sectionDataPtr[1 + SCAN_SKIP_FILTER_RESULT] & 0x0F ) << 8 );
            if( !section_length ) {
                FAPI_SYS_PRINT_MSG( "[SCAN] Error: PAT section length missing.\n" );
            }
            scanPmtAmount = (section_length - 9) >> 2; /* 9bytes for header and CRC, one prg consist of 4bytes */;
            FAPI_SYS_PRINT_MSG("\n[SCAN] Found PAT including %d PMTs\n[SCAN] Searching for services...\n", scanPmtAmount);

            /* PAT aquired -> search PMTs */
            scanSearchNextPmtPidInPat();
            /* stop filtering */
            scanPatTable.filtering = 2;
        }
    }

    /* PMT */
    if(scanPmtTable.filtering == 1) {
        result = FAPEX_SYS_SelectCheck( &select_table, scanPmtTable.id,
                                        (FAPEX_SELECT_MODE_READ | FAPEX_SELECT_MODE_EXPT) );
        if(result != 0) {
            FAPI_SYS_PRINT_DEBUG(1, "[SCAN] Read PMT.\n");

            /* read PMT data */
            result = FAPEX_TSD_Ioctl( scanPmtTable.id, FAPEX_IOCCOM_TSD_GET_BUFFER_READINFO,
                                      (FAPEXIOCARG3_T)&read_info );
            if(result < 0) {
                FAPI_SYS_PRINT_ERR("\n[SCAN] Error: could not set PMT read pointer (code: %d).\n", result);
            }
            if(read_info.bufferStatus & FAPEX_TSD_BUFFSTAT_TIMEOUT) {
                goto SCAN_Task_table_close;
            }
            scanPmtTable.bufferLength = read_info.fillSize;
            scanPmtTable.crcError     = read_info.bufferStatus & FAPEX_TSD_BUFFSTAT_CRCERROR;
            scanPmtTable.foundTable   = 1;

            result = FAPEX_TSD_Ioctl( scanPmtTable.id, FAPEX_IOCCOM_STOP, (FAPEXIOCARG3_T)NULL );
            if(result < 0) {
                FAPI_SYS_PRINT_ERR("\n[SCAN] Error: could not stop PMT (code: %d).\n", result);
            }
            scanPmtTable.stopRequest = 2;

            result = FAPEX_TSD_Read(scanPmtTable.id, scanPmtTable.sectionDataPtr, scanPmtTable.bufferLength);
            if(result != (int32_t)scanPmtTable.bufferLength) {
                FAPI_SYS_PRINT_MSG("\n[SCAN] Error: could not read PMT data (code: %d).\n", result);
            }

            /* process PMT data (skip filter result) */
            service_no = scanProcessPmtTable(&scanPmtTable.sectionDataPtr[SCAN_SKIP_FILTER_RESULT]);
            scanSearchNextPmtPidInPat();
        }
    }

    /* SDT */
    if(scanSdtTable.filtering == 1) {
        result = FAPEX_SYS_SelectCheck( &select_table, scanSdtTable.id,
                                        (FAPEX_SELECT_MODE_READ | FAPEX_SELECT_MODE_EXPT) );
        if(result != 0) {
            FAPI_SYS_PRINT_DEBUG(1, "[SCAN] Read SDT.\n");

            /* read SDT data */
            result = FAPEX_TSD_Ioctl( scanSdtTable.id, FAPEX_IOCCOM_TSD_GET_BUFFER_READINFO,
                                      (FAPEXIOCARG3_T)&read_info );
            if(result < 0) {
                FAPI_SYS_PRINT_ERR("\n[SCAN] Error: could not set SDT read pointer (code: %d).\n", result);
            }

            if(read_info.bufferStatus & FAPEX_TSD_BUFFSTAT_TIMEOUT) {
                goto SCAN_Task_table_close;
            }
            scanSdtTable.bufferLength = read_info.fillSize;
            scanSdtTable.crcError     = read_info.bufferStatus & FAPEX_TSD_BUFFSTAT_CRCERROR;
            scanSdtTable.foundTable   = 1;

            result = FAPEX_TSD_Ioctl( scanSdtTable.id, FAPEX_IOCCOM_STOP, (FAPEXIOCARG3_T)NULL );
            if(result < 0) {
                FAPI_SYS_PRINT_ERR("\n[SCAN] Error: could not stop SDT (code: %d).\n", result);
            }
            scanSdtTable.stopRequest = 2;

            scanSdtTable.filtering  = 0;
            result = FAPEX_TSD_Read(scanSdtTable.id, scanSdtTable.sectionDataPtr, scanSdtTable.bufferLength);
            if(result != (int32_t)scanSdtTable.bufferLength) {
                FAPI_SYS_PRINT_ERR("\n[SCAN] Error: could not read SDT data (code: %d).\n", result);
            }

            if(scanSdtSectionsFiltered == 0) {
                FAPI_SYS_PRINT_MSG("\n[SCAN] Updating service names.\n");
            }
            scanProcessSdtTable(&scanSdtTable.sectionDataPtr[SCAN_SKIP_FILTER_RESULT]);

            /* keep updating service names */
            if(scanSdtSectionsFiltered != scanSdtSections) {
                /* scan for SDT */
                scanSdtTable.stopRequest = 0;
                scanSdtTable.filtering   = 1;
                result = FAPEX_TSD_Ioctl( scanSdtTable.id, FAPEX_IOCCOM_START, (FAPEXIOCARG3_T)NULL );
                if(result != FAPI_OK) {
                    FAPI_SYS_PRINT_ERR("[SCAN] Error: could not start SDT PID (code: %d).\n", result);
                }
                scanSdtTable.foundTable = 0;
            }
            /* SDT aquired */
            else {
                scanSdtTable.filtering = 2;
            }
        }
    }

    if(scanProcess == 1) {
        /*****************************/
        /* evaluate scanning process */
        /*****************************/

SCAN_Task_table_close:
        /* finsh scan mode */
        if( (scanPatTable.filtering == 2)
         && (scanPmtTable.filtering == 2)
         && (scanSdtTable.filtering == 2) )
        {
            scanClose(&scanPatTable);
            scanClose(&scanPmtTable);
            scanClose(&scanSdtTable);

            scanProcess = -1;
        }
        if(service_no == 0) {
            return(2);
        }
        return(1);
    }
    return(0);

SCAN_Task_timeout:
    /********************/
    /* timeout handling */
    /********************/

    /* PAT */
    if((scanPatTable.filtering == 1) && (scanPatTable.foundTable == 0)) {
        FAPI_SYS_PRINT_MSG("[SCAN] Error: PAT acquisition timeout!\n");
        FAPI_SYS_PRINT_MSG("[SCAN] Check TS input and start 'scan' again.\n");
        /* no PAT -> stop scan */
        scanClose(&scanPatTable);
        scanClose(&scanPmtTable);
        scanClose(&scanSdtTable);
        scanProcess = -1;
    }

    /* SDT */
    if((scanSdtTable.filtering == 1) && (scanSdtTable.foundTable == 0)) {
        /* some streams don't contain an SDT */
        FAPI_SYS_PRINT_MSG("[SCAN] Warning: SDT acquisition timeout!\n");
        /* no SDT -> stop scan */
        scanClose(&scanPatTable);
        scanClose(&scanPmtTable);
        scanClose(&scanSdtTable);
        scanProcess = -1; /* complete nevertheless */
    }

    /* PMT */
    if((scanPmtTable.filtering == 1) && (scanPmtTable.foundTable == 0)) {
        FAPI_SYS_PRINT_MSG("[SCAN] Error: PMT(%i of %i) acquisition timeout!\n",
                           scanCurrentPmtIndex, scanPmtAmount);
        if(scanCurrentPmtIndex > scanPmtAmount) {
            /* no PMT -> stop scan */
            scanClose(&scanPatTable);
            scanClose(&scanPmtTable);
            scanClose(&scanSdtTable);
            scanProcess = -1;
        }
        else {
            /* PMT is missing or timeout too short - try next one */
            scanSearchNextPmtPidInPat();
        }
    }
    return(1);
}

