/*
*****************************************************************************
**
** \file        ./sample_application/fapex_av_demo/src/servicelist.c
**
** \version     $Id$
**
** \brief       service list with tuning informaton.
**
** This file contains the declaration of the service list API. It initializes
** and configures the service list. Furthermore it provides a simple API to
** fill and retrieve the service list.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited
**
*****************************************************************************
*/
#include "servicelist.h"

//***********************************************************************
//***********************************************************************
//** Local Defines and Macros
//***********************************************************************
//***********************************************************************


//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************
static SERVICELIST_ServiceInfo_T list[SERVICELIST_MAX_NUM_SERVICES];


//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************


/*!
*******************************************************************************
**
** Initialize service list.
**
** This function inititializes the service list. Here we only set all parameters
** to 0.
**
*******************************************************************************
*/
void SERVICELIST_Init(void)
{
    SERVICELIST_ClearList();
}


/*!
*******************************************************************************
**
** Delete all CA services.
**
** This function deletes all CA services.
**
** \return  number of removed services in list.
*******************************************************************************
*/
int32_t SERVICELIST_DeleteAllCaServices(void)
{
    uint32_t cnt     = 0;
    uint32_t size    = 0;
    int32_t  removed = 0;

    size = SERVICELIST_GetSize();
    while ((list[cnt].name!=0)&&(list[cnt].name[0]!=0))
    {
        if( list[cnt].caFlag == 1) {
            /* delete service */
            memmove( &list[cnt], &list[cnt+1], (size-(cnt+1))*sizeof(SERVICELIST_ServiceInfo_T) );
            memset(  &list[size-1], 0, sizeof(SERVICELIST_ServiceInfo_T));
            size--;
            removed++;
        }
        else
            cnt++;
    }
    return(removed);
}


/*!
*******************************************************************************
** Get size.
**
** Returns number of services in the list.
**
** \return Number of services in list.
*******************************************************************************
*/
uint32_t SERVICELIST_GetSize(void)
{
    uint32_t size=0;
    while ((size < SERVICELIST_MAX_NUM_SERVICES)&&(list[size].name!=0)&&(list[size].name[0]!=0)) size++;
    return size;
}

/*!
*******************************************************************************
** Clear list.
**
** Clears the service list.
*******************************************************************************
*/
void SERVICELIST_ClearList(void)
{
    memset( list, 0, SERVICELIST_MAX_NUM_SERVICES*sizeof(SERVICELIST_ServiceInfo_T));
}

/*!
*******************************************************************************
** Delete Service.
**
** Deletes an existing service identified by ID with all parameters.
**
** \param id Service ID (according DVB) of the service.
**
** \return pointer to requested service or NULL if index out of range.
*******************************************************************************
*/
int32_t SERVICELIST_DeleteServiceById(uint32_t id)
{
    uint32_t cnt, size;

    size = SERVICELIST_GetSize();
    for(cnt=0; cnt<size; cnt++) {
        if( list[cnt].serviceId == id) {
            /* move items list up */
            memmove( &list[cnt], &list[cnt+1], (size-(cnt+1))*sizeof(SERVICELIST_ServiceInfo_T) );
            memset(  &list[size-1], 0, sizeof(SERVICELIST_ServiceInfo_T));
            return(FAPI_OK);
        }
    }
    return(-1); /* ID not found */
}


/*!
*******************************************************************************
** Add Service.
**
** Adds service to list.
**
** \param service Pointer to service that should be added.
**
** \retval >=0 Number of services in list.
** \retval <0  otherwise error code.
*******************************************************************************
*/
int32_t SERVICELIST_AddService(SERVICELIST_ServiceInfo_T* service)
{
    uint32_t size = SERVICELIST_GetSize();

    if (size >= (SERVICELIST_MAX_NUM_SERVICES-1))
        return(-1);

    memcpy( &list[size], service, sizeof( SERVICELIST_ServiceInfo_T ) );
    return size;
}

/*!
*******************************************************************************
**
** Update service.
**
** Updates an existing service identified by ID with all parameters..
**
** \param service Pointer to service that should be added.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t SERVICELIST_UpdateService(SERVICELIST_ServiceInfo_T* service)
{
    uint32_t cnt = 0;

    for(cnt=0; cnt<SERVICELIST_GetSize(); cnt++)
    {
        if( list[cnt].serviceId == service->serviceId)
        {
            /* Update all items different from 0 */
            if(service->name[0] != 0)
                memcpy( list[cnt].name, service->name, SERVICELIST_MAX_SERVICE_NAME_LEN );
            if(service->videoPid != 0)
                list[cnt].videoPid = service->videoPid;
            if(service->videoType != 0)
                list[cnt].videoType = service->videoType;

            if(service->audioPid[0] != 0)
                list[cnt].audioPid[0] = service->audioPid[0];
            if(service->audioType[0] != 0)
                list[cnt].audioType[0] = service->audioType[0];

            if(service->teletextPid != 0)
                list[cnt].teletextPid = service->teletextPid;
            if(service->pcrPid != 0)
                list[cnt].pcrPid = service->pcrPid;
            if(service->serviceType != 0)
                list[cnt].serviceType = service->serviceType;
            if(service->caFlag != 0)
                list[cnt].caFlag = service->caFlag;
            return(FAPI_OK);
        }
    }
    return(-1); /* ID not found */
}

/*!
*******************************************************************************
** Get Service.
**
** Returns the service by a special index.
**
** \param index of the service.
**
** \return pointer to requested service or NULL if index out of range
*******************************************************************************
*/
SERVICELIST_ServiceInfo_T*  SERVICELIST_GetService(uint32_t index)
{
    if (index>=SERVICELIST_GetSize())
        return NULL;
    return(&list[index]);
}

/*!
*******************************************************************************
** Load list default test services.
**
** Loads service list with test services ARD transponder.
*******************************************************************************
*/
void SERVICELIST_LoadTestServices(void)
{
    SERVICELIST_ClearList();

    /* pre-defined service lists for test */
    sprintf((char_t*)list[0].name, "Das Erste");
    list[0].videoPid       = 101;
    list[0].videoType      = VID_MPEG2;
    list[0].audioPid[0]    = 102;
    list[0].audioType[0]   = AUD_MPEG1;
    list[0].teletextPid    = 104;
    list[0].pcrPid         = 101;
    list[0].serviceId      = 28106;
    list[0].caFlag         = 0;

    sprintf((char_t*)list[1].name, "Bayerisches FS");
    list[1].videoPid       = 201;
    list[1].videoType      = VID_MPEG2;
    list[1].audioPid[0]    = 202;
    list[1].audioType[0]   = AUD_MPEG1;
    list[1].teletextPid    = 204;
    list[1].pcrPid         = 201;
    list[1].serviceId      = 28107;
    list[1].caFlag         = 0;

    sprintf((char_t*)list[2].name, "hr-fernsehen");
    list[2].videoPid       = 301;
    list[2].videoType      = VID_MPEG2;
    list[2].audioPid[0]    = 302;
    list[2].audioType[0]   = AUD_MPEG1;
    list[2].teletextPid    = 304;
    list[2].pcrPid         = 301;
    list[2].serviceId      = 28108;
    list[2].caFlag         = 0;
}
