/*
*****************************************************************************
**
** \file        ./subsystems/closedCaption/src/cc.h
**
** \brief       Closed Caption PES Decord driver.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2010 by Fujitsu Semiconductor Limited.
*****************************************************************************
*/
#ifndef CC_H
#define CC_H

#include <stdint.h>
#include <fapi/sys_error.h>

/*
********************************************************************************
Data types
********************************************************************************/

typedef struct {
    uint32_t PTS;
    uint8_t  fNotPTS;
    uint8_t  data_group_id;
    uint8_t  is_management;
    uint8_t  data_group_version;
    uint8_t  data_group_link_number;
    uint8_t  last_data_group_link_number;
    uint8_t  TMD;
    uint8_t  num_languages;
    uint8_t  language_tag;
    uint8_t  DMF;
    uint8_t  ISO_639_language_code[3];
    uint8_t  Format;
    uint8_t  TCS;
    uint8_t  rollup_mode;
    uint8_t  data_unit_parameter;
    uint32_t data_unit_size;
} CC_HeaderInfo;

typedef struct {
    /* Active position */
    uint8_t fAPB;
    uint8_t fAPF;
    uint8_t fAPD;
    uint8_t fAPU;
    uint8_t fAPR;
    uint8_t fPAPF;
    uint8_t fAPS;
    uint8_t paramPAPF[1];
    uint8_t paramAPS[2];

    /* Colour Control */
    uint8_t fBKF;
    uint8_t fRDF;
    uint8_t fGRF;
    uint8_t fYLF;
    uint8_t fBLF;
    uint8_t fMGF;
    uint8_t fCNF;
    uint8_t fWHF;
    uint8_t fCOL;
    uint8_t charColor[1];
    uint8_t paramCOL[2];

    /* Character Size */
    uint8_t fSSZ;
    uint8_t fMSZ;
    uint8_t fNSZ;
    uint8_t charSize[1];
    uint8_t fSZX;
    uint8_t paramSZX[1];

    /* Flashing */
    uint8_t fFLC;
    uint8_t paramFLC[1];

    /* Pattern Polarity */
    uint8_t fPOL;
    uint8_t paramPOL[1];

    /* HIGHLIGHTING CHARACTER BLOCK */
    uint8_t fHLC;
    uint8_t paramHLC[1];

    /* Repeat Character */
    uint8_t fRPC;
    uint8_t paramRPC[1];

    /* Lining */
    uint8_t fSPL;
    uint8_t fSTL;

    /* Time */
    uint8_t fTIME;
    uint8_t paramTIME[2];

    /* Clear screen */
    uint8_t fCS;

    /* Control Sequence Introducer */
    uint8_t fCSI;
    uint8_t fSWF;
    uint8_t fRCS;
    uint8_t fACPS;
    uint8_t fSDF;
    uint8_t fSDP;
    uint8_t fSSM;
    uint8_t fSHS;
    uint8_t fSVS;
    uint8_t fORN;
    uint8_t fPRA;
    uint8_t paramSWF[3];        /* Set Writing Format                             */
    uint8_t paramRCS[3];        /* Raster Colour command                          */
    uint8_t paramACPS_H[3];     /* Active Coordinate Position Set(Horizontal)     */
    uint8_t paramACPS_V[3];     /* Active Coordinate Position Set(Vertical)       */
    uint8_t paramSDF_H[3];      /* SET DISPLAY FORMAT(Horizontal)                 */
    uint8_t paramSDF_V[3];      /* SET DISPLAY FORMAT(Vertical)                   */
    uint8_t paramSDP_H[3];      /* Set Display Position(Horizontal)               */
    uint8_t paramSDP_V[3];      /* Set Display Position(Vertical)                 */
    uint8_t paramSSM_H[3];      /* Character composition dot designation(width)   */
    uint8_t paramSSM_V[3];      /* Character composition dot designation(height)  */
    uint8_t paramSHS[3];        /* Set Horizontal Spacing                         */
    uint8_t paramSVS[3];        /* Set Vertical Spacing                           */
    uint8_t paramORN[1];        /* Ornament Control                               */
    uint8_t paramORN_C[8];      /* Ornament Colour Control                        */
    uint8_t paramPRA[3];        /* Built-in sound replay                          */

    uint32_t APB_num;
    uint32_t APF_num;
    uint32_t APD_num;
    uint32_t APU_num;
    uint32_t APR_num;

    uint32_t num1_SWF;
    uint32_t num1_RCS;
    uint32_t num1_ACPS;
    uint32_t num2_ACPS;
    uint32_t num1_SDF;
    uint32_t num2_SDF;
    uint32_t num1_SDP;
    uint32_t num2_SDP;
    uint32_t num1_SSM;
    uint32_t num2_SSM;
    uint32_t num1_SHS;
    uint32_t num1_SVS;
    uint32_t num1_ORN;
    uint32_t num2_ORN;
    uint32_t num1_PRA;

    char_t decordedData[360];  /* Max character num at one line */
    uint32_t charcount;

} CC_DataInfo;

/*
********************************************************************************
Standard error codes for the CC driver
********************************************************************************/

/* Bad parameter passed. */
#define CC_ERR_BAD_PARAMETER \
        (int32_t)(SUBSYS_CC_MODULE_BASE + FAPI_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define CC_ERR_OUT_OF_MEMORY \
        (int32_t)(SUBSYS_CC_MODULE_BASE + FAPI_ERR_OUT_OF_MEMORY)

/*! Device already initialised. */
#define CC_ERR_ALREADY_INITIALIZED \
        (int32_t)(SUBSYS_CC_MODULE_BASE + FAPI_ERR_ALREADY_INITIALIZED)

/*! Device not initialised. */
#define CC_ERR_NOT_INITIALIZED \
        (int32_t)(SUBSYS_CC_MODULE_BASE + FAPI_ERR_NOT_INITIALIZED)


/*
********************************************************************************
API Functions
********************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif
int32_t CC_Init (void);
int32_t CC_Exit (void);
int32_t CC_AnalyzePes(char_t* pesPacket,               /* [IN]     */
                      uint32_t pesPacketSize,          /* [IN]     */
                      CC_HeaderInfo* headerInfo,       /* [OUT]    */
                      char_t* dataUnit);
int32_t CC_DecordDataUnit(char_t* dataUnit,            /* [IN]     */
                          CC_DataInfo* dataInfo,       /* [OUT]    */
                          uint32_t *leftDataUnitSize); /* [IN/OUT] */
#ifdef __cplusplus
}
#endif
#endif
