/******************************************************************************
**
** \file      fd_fe_M88DC2800.c
**
** \brief     Front-end device driver for Fujitsu MXDX320 DVB-T frontend.
**
**            (C) Fujitsu Microelectronics Shanghai 2008 - 2009
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU MICROELECTRONICS
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: fd_fe_M88DC2800.c,v 1.0 2008-5-28 12:01 Rick Lu $
**
******************************************************************************/
#include "fapex/abst_types.h"
#include "fe.h"
#include "fe_priv.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
static feHandleT *pData[FE_MAX_NUM];
#define FFALSE 0
#define FTRUE  1

/*---------------------------------------------------------------------------*/
/* Local Definitions                                                         */
/*---------------------------------------------------------------------------*/
#define HIWORD(x)  (((x) & 0xFFFF0000) >> 16)

#define MIN_FREQ 4.9e+7              /* Min frequency*/
#define MAX_FREQ 8.58e+8             /* Max frequency*/

#define MIN_SYM  3.6e+3              /* Min symbol K*/
#define MAX_SYM  6.952e+3            /* Max symbol K*/

#define TUNER_I2C_ADDR          0xC0     /* The tuner address maybe changed */
#define M88DC2800_I2C_ADDR      0x38     /* M88DC2800 address       */
#define M88DC2800_CRYSTAL_CLK   28.8

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

static FE_MOD_E        	qamMode[FE_MAX_NUM]    = { FE_MOD_NONE };
static uint8_t       	lockStatus[FE_MAX_NUM] = { 0 };
static uint32_t       	lastTuneSymbolRate = 0;
static uint32_t       	lastTuneModulation = 0;

/*---------------------------------------------------------------------------*/
/* M88DC2800 local functions                                                 */
/*---------------------------------------------------------------------------*/
static int32_t m88dc2800RegInitial(uint8_t index);
static int32_t m88dc2800SetFrequency( uint8_t index, uint32_t frq );
static int32_t m88dc2800SetIQ(uint8_t index, uint8_t j83, FE_IQ_E iq);
static int32_t m88dc2800SetSym(uint8_t index, uint32_t sym);
static int32_t m88dc2800SetQAM(uint8_t index, FE_MOD_E Qam);
static int32_t m88dc2800SoftReset(uint8_t index);
static int32_t m88dc2800SetTsType(uint8_t index, uint8_t type);
static uint8_t m88dc2800GetAgcLock(uint8_t index);
static int32_t m88dc2800I2Cread( int32_t id, uint8_t msSlaveAddr, uint8_t reg ,uint8_t *pdata);
static int32_t m88dc2800I2Cwrite(int32_t id, uint8_t msSlaveAddr, uint8_t reg, uint8_t data );
/*---------------------------------------------------------------------------*/
/* callback functions                                                        */
/*---------------------------------------------------------------------------*/
int32_t M88DC2800_Open(uint8_t index, FE_HANDLE* pDeviceData);
int32_t M88DC2800_Close(uint8_t index);
int32_t M88DC2800_Reset(uint8_t index);
int32_t M88DC2800_SetSystemParams(uint8_t index, FE_SYSTEM_PARAMS_S* pSystemParams);
int32_t M88DC2800_GetSystemParams(uint8_t index,FE_SYSTEM_PARAMS_S* pSystemParams);
int32_t M88DC2800_Tune(uint8_t index, FE_TUNE_PARAMS_S* pTuneParams);
int32_t M88DC2800_GetStatus(uint8_t index, FE_STATUS_E* pStatus);
int32_t M88DC2800_GetBER(uint8_t index, uint32_t* pBER);
int32_t M88DC2800_GetSignalQuality(uint8_t index, uint16_t* pQuality);
int32_t M88DC2800_GetSignalStrength(uint8_t index, uint16_t* pStrength);
int32_t M88DC2800_GetTuneParams(uint8_t index, FE_TUNE_PARAMS_S *pTuneParams);
void m88dc2800Sleep(uint32_t milliseconds);

#define U_Sleep m88dc2800Sleep
/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

void m88dc2800Sleep(uint32_t milliseconds)
{
      uint32_t t1, t2;

      t1 = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
      do
      {
        t2 = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
        if (t2<t1) /* in case of timer overflow */
        {
            t1 = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
            t2 = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
        }
      } while ((t2-t1) < (uint32_t)milliseconds);
}
/*!
*******************************************************************************
**
** \brief Set callback function for the current FE handle.
** \param hp - FE handle
**
*******************************************************************************
*/
void m88dc2800SetFunctions(feHandleT *hp)
{
    hp->systemType             = FE_SYSTEM_TYPE_CABLE;
    hp->feOpenFct              = M88DC2800_Open;
    hp->feCloseFct             = M88DC2800_Close;
    hp->feGetSystemParamsFct   = M88DC2800_GetSystemParams;
    hp->feSetSystemParamsFct   = M88DC2800_SetSystemParams;
    hp->feResetFct             = M88DC2800_Reset;
    hp->feTuneFct              = M88DC2800_Tune;
    hp->feScanFct              = NULL;
    hp->feSendDiseqcCommandFct = NULL;
    hp->feGetStatusFct         = M88DC2800_GetStatus;
    hp->feGetFECFct            = NULL;
    hp->feGetAFCFct            = NULL;
    hp->feGetBERFct            = M88DC2800_GetBER;
    hp->feGetSignalQualityFct  = M88DC2800_GetSignalQuality;
    hp->feGetSignalStrengthFct = M88DC2800_GetSignalStrength;
    hp->feGetTuneParamsFct     = M88DC2800_GetTuneParams;
    hp->feSetAntennaPowerFct   = NULL;
    hp->isPresent              = 1;
           
}
/*!
*******************************************************************************
**
** \brief Check chip ID.
** \param hp - FE handle for current FE front-end.
** \param i2cAddress  - i2c address for demod.
**
*******************************************************************************
*/
int32_t m88dc2800CheckChipId(feHandleT *hp, uint8_t i2cAddress)
{
    uint8_t i2cBuffer[2];
    int32_t ferr;
    
    i2cBuffer[0] = 0x80;
    i2cBuffer[1] = 0xFF;
    ferr = FE_I2C_Read(hp->i2cId, i2cAddress, &(i2cBuffer[0]), 1,
                                                  &(i2cBuffer[1]), 1);
    FAPI_SYS_PRINT_MSG("[M88DC2800] Checking for M88DC2800 on address %02x ferr %d chipID(0x00) %x\n", i2cAddress, ferr, i2cBuffer[1] &0xf0);
    if(ferr != FE_OK)
        return ferr;

    return FE_OK;
}
/******************************************************************************
**
** \brief  Tries to detect and open the M88DC2800 device.
**
** \param  index        Internal index of front-end.
** \param   #FD_FE_OPEN_PARAMS_S* pOpenParams   pointer to open parameters
**                                              the function returns also the device data in this structure
** \param   slot                                selects if front end is expected in slot 1 or 2
**
** \return Possible return codes:
**         return codes from #FD_FE_Open
**
******************************************************************************/
int32_t M88DC2800_Detect( uint8_t slot, feHandleT *hp)
{
    if (slot == 0)
        hp->i2cAddrDemod  = 0x38;
    else if (slot == 1)
        hp->i2cAddrDemod  = 0x3a;
    else
        return (FE_ERR_BAD_PARAMETER);
        
	if(slot ==0)M88DC2800_Reset(slot);
  	
    if(m88dc2800CheckChipId(hp, hp->i2cAddrDemod) != FE_OK)
        return FE_ERR_FE_UNKNOWN_DEVICE;
    m88dc2800SetFunctions(hp);
    hp->i2cAddrTuner = 0xC0;      
    hp->slotNum  = slot;
    
    return FE_OK;
}



/******************************************************************************

Name:        M88DC2800_Open

Description: Opens the front-end device and fills the device data structure
             with call-back functions.

Parameters:  1) pointer to front-end device structure

Returns:     FE_OK
             FE_ERR_BAD_PARAMETER
             FE_ERR_FE_OPEN
             FE_ERR_FE_OPEN_I2C

******************************************************************************/
int32_t M88DC2800_Open(uint8_t index, FE_HANDLE* pDeviceData)
{
    int32_t ferr;

    pData[index]=&feHandleArray[index];
    *pDeviceData=(FE_HANDLE)(&feHandleArray[index]);
    
	lockStatus[index] = FFALSE;

	ferr = m88dc2800RegInitial( index );
	if( ferr != FE_OK )
		return ferr;
            
	ferr = m88dc2800SoftReset( index );
	if( ferr != FE_OK )
		return ferr;
	m88dc2800GetAgcLock(0);
	return FE_OK;
}

int32_t M88DC2800_Close(uint8_t index)
{
    if( index >= FE_MAX_NUM )
        return FE_ERR_BAD_PARAMETER;

    /* do nothing now */

    return FE_OK;
}
int32_t M88DC2800_GetSystemParams(uint8_t index,
                                 FE_SYSTEM_PARAMS_S* pSystemParams)
{
    pSystemParams = pSystemParams;
    pSystemParams = &(pData[index]->pSystemParams);  
    //memcpy(pSystemParams,&(pData[index]->pSystemParams),sizeof(FE_SYSTEM_PARAMS_S));
    return FE_OK;                           	
}
/******************************************************************************

Name:        M88DC2800_SetSystemParams

Description: Changes the system parameters.

Parameters:  1) index of front-end instance
             2) pointer to system parameters

Returns:     FE_OK
             FE_ERR_BAD_PARAMETER

******************************************************************************/
int32_t M88DC2800_SetSystemParams(uint8_t index,
                                    FE_SYSTEM_PARAMS_S* pSystemParams)
{
    pSystemParams = pSystemParams;
    memcpy(&(pData[index]->pSystemParams),pSystemParams,sizeof(FE_SYSTEM_PARAMS_S));
    return FE_OK;
}
/***********************************************
   Config tuner via M88DC2800 I2C repeater
   frq:     RF frequency, unit: MHz
   sym:     symbol rate, unit: KBaud
   Qam:     QAM mode, 16, 32, 64, 128, 256
   Inverted:  Spectrum inverted(1) or not(0)
   xtal:    in KHz
************************************************/
int32_t M88DC2800_Tune(uint8_t index, FE_TUNE_PARAMS_S* pTuneParams)
{
    int32_t ferr;
    int32_t waiting_time;
    uint8_t value1, value2;
    FE_STATUS_E feStatus;
	
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pTuneParams == NULL) return FE_ERR_BAD_PARAMETER;

    if( pTuneParams->cable.frequency < MIN_FREQ || pTuneParams->cable.frequency > MAX_FREQ )
        return FE_ERR_BAD_PARAMETER;

    if( pTuneParams->cable.symbolRate < MIN_SYM || pTuneParams->cable.symbolRate > MAX_SYM )
        return FE_ERR_BAD_PARAMETER;

    if( pTuneParams->cable.modulation == FE_MOD_NONE ||
        pTuneParams->cable.modulation == FE_MOD_QPSK ||
        pTuneParams->cable.modulation == FE_MOD_AUTO )
        return FE_ERR_BAD_PARAMETER;

    qamMode[index]      = pTuneParams->cable.modulation;
    lockStatus[index]   = FFALSE;
    
    FE_DPrintf("[m88dc2800] freq = %d hz, sym =%d kpbs, mod = %d, iq = %d\n",
    						pTuneParams->cable.frequency,
    						pTuneParams->cable.symbolRate,
    						pTuneParams->cable.modulation,
    						pTuneParams->cable.iq);

    ferr = m88dc2800SetFrequency( index, pTuneParams->cable.frequency );
    if( ferr != FE_OK )
        return ferr;
    
    m88dc2800RegInitial(index);
    
	ferr = m88dc2800SetSym( index, pTuneParams->cable.symbolRate );
	FE_DPrintf("[m88dc2800] m88dc2800SetSym ret = %d\n",ferr);
	if( ferr != FE_OK )
		return ferr;
	
	ferr = m88dc2800SetQAM( index, pTuneParams->cable.modulation );
	FE_DPrintf("[m88dc2800] m88dc2800SetQAM ret = %d\n",ferr);
	if( ferr != FE_OK )
		return ferr;
    
    ferr = m88dc2800SetIQ( index, 0, pTuneParams->cable.iq ); /*  J83A          */
    FE_DPrintf("[m88dc2800] m88dc2800SetIQ ret = %d\n",ferr);
    if( ferr != FE_OK )
        return ferr;
    
    m88dc2800SetTsType( index, 0 ); //parallel ts output
       
    m88dc2800SoftReset(index);    
        
    m88dc2800Sleep(20);
    
#if defined (TARGET_DL_HUALU_PUSHVOD_V100)
        m88dc2800SetTsType( index, 0 );
#endif
    
    m88dc2800I2Cread(pData[index]->i2cId,pData[index]->i2cAddrDemod, 0xE3, &value1);
    m88dc2800I2Cread(pData[index]->i2cId,pData[index]->i2cAddrDemod, 0xE4, &value2);

    if ( ((value1 & 0x80) == 0x80) && ((value2 & 0x80) == 0x80) )
    {
        waiting_time = 800;
    }
    else
    {
        waiting_time = 500;
    }
    lastTuneSymbolRate = lastTuneSymbolRate;
    lastTuneModulation = lastTuneModulation;
    lastTuneSymbolRate = pTuneParams->cable.symbolRate;
    lastTuneModulation = pTuneParams->cable.modulation;

    while (waiting_time > 0)
    {
        U_Sleep(50);
        waiting_time -= 50;

        ferr = M88DC2800_GetStatus(index, &feStatus);
        if( ferr != FE_OK )
            return FE_ERR_BAD_PARAMETER;

        if( feStatus == FE_STATUS_LOCKED )
        {
            return FE_OK;
        }
    }

#if 0

    if (inverted != 0)
    {
        inverted = 0;
    }
    else
    {
        inverted = 1;
    }

    m88dc2800SetIQ(index, 0, pTuneParams->cable.iq);  /*  J83A          */
    m88dc2800SoftReset(index);
    
    m88dc2800I2Cread(&(pData[index]->i2cHandle),pData[index]->i2cAddrDemod, 0xE3, &value1);
    m88dc2800I2Cread(&(pData[index]->i2cHandle),pData[index]->i2cAddrDemod, 0xE4, &value2);

    if (((value1 & 0x80) == 0x80) && ((value2 & 0x80) == 0x80))
    {
        waiting_time = 800;
    }
    else
    {
        waiting_time = 500;
    }

    while (waiting_time > 0)
    {
        Sleep(50);
        waiting_time -= 50;

        ferr = M88DC2800_GetStatus(index, &feStatus);
        if( ferr != FE_OK )
            return FE_ERR_BAD_PARAMETER;

        if( feStatus == FE_STATUS_LOCKED )
        {
            return FE_OK;
        }
    }

#endif

    return FE_OK;

}
/***********************************************
   Get lock status
   returned 1 when locked;0 when unlocked
************************************************/
int32_t M88DC2800_GetStatus(uint8_t index, FE_STATUS_E* pStatus)
{
	uint8_t  i, value, value1, value2, value3;

    if( index >= FE_MAX_NUM )
        return FE_ERR_BAD_PARAMETER;

    m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x80, &value);

    if ( value < 0x06 )
    {
        m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xdf, &value1);
        m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x91, &value2);
        m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x43, &value3);

        if ( ((value1 & 0x80) == 0x80) &&
             ((value2 & 0x23) == 0x03) &&
             ((value3 & 0x08) == 0x08) )
        {
            if( !(lockStatus[index]) )
            {
                for( i = 0; i < 20; i++ )
                {
                    m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xdf ,&value1);
                    if( !((( value1 & 0x80 ) == 0x80) && ( ( value2 & 0x23 ) == 0x03 ) && ( ( value3 & 0x08 ) == 0x08 ) ) )
                    {
                        lockStatus[index] = FFALSE;
                        *pStatus = FE_STATUS_NOT_LOCKED;
                        break;
                    }
                }
                if(((value1 & 0x80 ) == 0x80) && ( ( value2 & 0x23 ) == 0x03 ) && ( ( value3 & 0x08 ) == 0x08 ) )
                {
                    lockStatus[index] = FTRUE;
                    *pStatus = FE_STATUS_LOCKED;
                }
            }
            else
                *pStatus = FE_STATUS_LOCKED;
        }
        else
        {
            if( lockStatus[index] )
                lockStatus[index] = FFALSE;

            *pStatus = FE_STATUS_NOT_LOCKED;
        }
    }
    else
    {
        m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x85, &value1);

        if (( value1 & 0x08 ) == 0x08 )
        {
            lockStatus[index] = FTRUE;
            *pStatus = FE_STATUS_LOCKED;
        }
        else
        {
             lockStatus[index] = FFALSE;
             *pStatus = FE_STATUS_NOT_LOCKED;
        }
    }
    return FE_OK;
}
/***********************************************
   Get BER (bit error rate)
************************************************/

int32_t M88DC2800_GetBER(uint8_t index, uint32_t* pBER)
{
    static double  ber = 0;
    uint16_t            tmp;
    uint8_t             value, value1, value2;
    int32_t           ferr;
    FE_STATUS_E feStatus;

    if( index >= FE_MAX_NUM )
        return FE_ERR_BAD_PARAMETER;

    ferr = M88DC2800_GetStatus( index, &feStatus );
    if( ferr != FE_OK )
        return FE_ERR_BAD_PARAMETER;

    if( feStatus != FE_STATUS_LOCKED )
    {
        pBER[0] = 1e7;
        pBER[1] = 1e7;
        pBER[2] = 1e7;

        return FE_OK;
    }
    
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xA0,0x05);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xA0,0x85);

    m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xA0, &value);
    do
    {
        ferr = M88DC2800_GetStatus(index, &feStatus);
        if( ferr != FE_OK )
            return FE_ERR_BAD_PARAMETER;

        if( feStatus != FE_STATUS_LOCKED )
        {
            return FE_OK;
        }

        U_Sleep(20);

    } while ((value & 0x80) == 0x80);

    m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xA2, &value1);
    m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xA1, &value2);

    tmp  = value1 << 8;
    tmp += value2;
    ber  = tmp / 33554432.0;      /*  (2^(2*5+12))*8    */

    pBER[0] = (uint32_t)(ber*1e9);
    pBER[1] = tmp;
    pBER[2] = 1e7;
    
    return FE_OK;
}
/***********************************************
   Get SNR (signal noise ratio)
************************************************/

int32_t M88DC2800_GetSignalQuality(uint8_t index, uint16_t* pQuality)
{
	const uint32_t mes_log[] =
    {
        0,      3010,   4771,   6021,   6990,   7781,   8451,   9031,   9542,   10000,
        10414,  10792,  11139,  11461,  11761,  12041,  12304,  12553,  12788,  13010,
        13222,  13424,  13617,  13802,  13979,  14150,  14314,  14472,  14624,  14771,
        14914,  15052,  15185,  15315,  15441,  15563,  15682,  15798,  15911,  16021,
        16128,  16232,  16335,  16435,  16532,  16628,  16721,  16812,  16902,  16990,
        17076,  17160,  17243,  17324,  17404,  17482,  17559,  17634,  17709,  17782,
        17853,  17924,  17993,  18062,  18129,  18195,  18261,  18325,  18388,  18451,
        18513,  18573,  18633,  18692,  18751,  18808,  18865,  18921,  18976,  19031
    };
    uint32_t   snr;
    uint8_t    i, value, value1, value2;
    uint32_t   mse = 0, msesub = 0;

    m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x91, &value);

    if ( (value & 0x23) != 0x03 )
    {
        *pQuality = 0;
    }

    for (i=0; i<30; i++)
    {
        m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x08, &value1);
        m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x07, &value2);
        mse = (value1 << 8) + value2;
        msesub += mse;
    }
    mse = msesub / 30;
    if (mse > 80)
    {
        mse = 80;
    }

    switch (qamMode[index])
    {
        case FE_MOD_QAM_16:  snr = 34080;    break;
        case FE_MOD_QAM_32:  snr = 37600;    break;
        case FE_MOD_QAM_64:  snr = 40310;    break;
        case FE_MOD_QAM_128: snr = 43720;    break;
        case FE_MOD_QAM_256: snr = 46390;    break;
        default:                snr = 40310;    break;
    }

    snr -= mes_log[mse-1];          /*  C - 10*log10(MSE) */
    snr /= 1000;
    if (snr > 0xff)
    {
        snr = 0xff;
    }
    snr = snr*256;
    *pQuality = (uint16_t)(snr);

    return FE_OK;
}
/***********************************************
   Get signal strength
************************************************/
int32_t M88DC2800_GetSignalStrength(uint8_t index, uint16_t* pStrength)
{
    uint8_t  value, value1, value2, value3, value4;
    FE_STATUS_E feStatus;
    int32_t ferr;

    ferr = M88DC2800_GetStatus( index, &feStatus );
    if( ferr != FE_OK )
        return FE_ERR_BAD_PARAMETER;

    if( feStatus != FE_STATUS_LOCKED )
    {
        *pStrength = 0;
        return FE_OK;
    }

    m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x43, &value);

    if ( (value & 0x08) == 0x08 )
    {
        m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xE3, &value1);
        m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xE4, &value2);

        if ( ((value1 & 0xC0) == 0x00) && ((value2 & 0xC0) == 0x00) )
        {
            m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x55, &value3);
            m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x56, &value4);

            *pStrength = 255 - ((value3 >> 1) + (value4 >> 1));    
        }
        else
        {
            m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x3B, &value3);
            m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x3C, &value4);

            *pStrength = 255 - ((value3 >> 1) + (value4 >> 1));
        }
        
        /* change it to percents */
       // *pStrength = *pStrength * 100 / 0xFF;
       *pStrength = *pStrength * 256;
    }
    else
      *pStrength = 0;

    return FE_OK;
}

int32_t M88DC2800_GetTuneParams(uint8_t index, FE_TUNE_PARAMS_S *pTuneParams)
{

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pTuneParams == NULL) return FE_ERR_BAD_PARAMETER;


    return FE_OK;
}

int32_t M88DC2800_Reset(uint8_t index)
{
	 #if 0
    int32_t err;
    FAPI_SYS_HandleT gpioHandle;
    FAPI_GPIO_OpenParamsT paramsGPIO;
    
	  paramsGPIO.version = FAPI_GPIO_VERSION;
    paramsGPIO.pin = 12; //29
    paramsGPIO.function = FAPI_GPIO_OUT_0;//0;
    paramsGPIO.callback = NULL;
    
    gpioHandle = FAPI_GPIO_Open(&paramsGPIO,&err);
    
    FAPI_GPIO_WriteBit(gpioHandle,1);
    FAPI_SYS_SLEEP(100);
    FAPI_GPIO_WriteBit(gpioHandle,0);
    FAPI_SYS_SLEEP(100);
    FAPI_GPIO_WriteBit(gpioHandle,1);
    FAPI_SYS_SLEEP(100); 
    #endif
    
    int32_t error;

    if (index >= FE_MAX_NUM)
    {
        return FE_ERR_BAD_PARAMETER;
    }

    error = FE_ControlFeReset(index, FE_RESET_PULSE);
    if (error != FE_OK)
    {
        return error;
    }
    FE_DPrintf("[M88DC2800] reset \n");
    FAPI_SYS_PRINT_MSG("###################\n");

    return FE_OK;
}




/******************************************************************************

Name:        m88dc2800I2Cwrite

Description: I2cWriteReg.

Parameters:

Returns:     FE_OK
             FE_ERR_BAD_PARAMETER

******************************************************************************/
static int32_t m88dc2800I2Cwrite(int32_t i2cId, uint8_t msSlaveAddr, uint8_t reg, uint8_t data )
{
    int32_t ferr;
    uint8_t cmds[2];

    cmds[0] = reg;
    cmds[1] = data;


    ferr = FE_I2C_Write( i2cId, msSlaveAddr, cmds, 2 );

    if( ferr != FE_OK )
    {
        FE_DPrintf("\n[m88dc2800] i2c write error = %d\n",ferr);
        return ferr;
    }
    
    return FE_OK;
}

/******************************************************************************

Name:        m88dc2800I2Cread

Description: I2cWriteReg.

Parameters:

Returns:     FE_OK
             FE_ERR_BAD_PARAMETER

******************************************************************************/
static int32_t m88dc2800I2Cread( int32_t i2cId, uint8_t msSlaveAddr, uint8_t reg ,uint8_t *pdata)
{
    int32_t ferr;
    uint8_t cmds[1];

    cmds[0] = reg;

    ferr = FE_I2C_Read( i2cId, msSlaveAddr, cmds, 1, pdata, 1 );

    if( ferr != FE_OK )
    {
        FE_DPrintf("\n[m88dc2800] i2c read error = %d\n",ferr);
        return ferr;
    }

    return FE_OK;
}

/***********************************************
 Initialize the internal registers in M88DC2800
************************************************/
static int32_t m88dc2800RegInitial(uint8_t index)
{
    uint8_t RegE3H, RegE4H;

    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x00,0x48);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x01,0x09);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xFB,0x0A);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xFC,0x0B);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x02,0x0B);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x03,0x18);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x05,0x0D);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x36,0x80);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x43,0x40);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x55,0x7A);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x56,0xD9);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x57,0xDF);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x58,0x39);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x5A,0x00);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x5C,0x71);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x5D,0x23);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x86,0x40);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF9,0x08);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x61,0x40);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x62,0x0A);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x4A,0xCD);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x4B,0xFF);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x90,0x06);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xDE,0x00);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xA0,0x03);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xDF,0x81);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xFA,0x40);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x37,0x10);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF0,0x40);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF2,0x9C);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF3,0x40);

    m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xE3, &RegE3H);
    m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xE4, &RegE4H);

    if (((RegE3H & 0xC0) == 0x00) && ((RegE4H & 0xC0) == 0x00))
    {
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x30,0xFF);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x31,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x32,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x33,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x35,0x32);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x40,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x41,0x10);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF1,0x02);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF4,0x04);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF5,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x42,0x14);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xE1,0x25);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x8B,0x5B);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x8E,0x9D);
    }
    else if (((RegE3H & 0xC0) == 0x80) && ((RegE4H & 0xC0) == 0x40))
    {
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x30,0xFF);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x31,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x32,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x33,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x35,0x32);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x39,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x3A,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x40,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x41,0x10);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF1,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF4,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF5,0x40);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x42,0x14);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xE1,0x25);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x8B,0x5B);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x8E,0x9D);

    }
    else if ((RegE3H == 0x80 || RegE3H == 0x81) && (RegE4H == 0x80 || RegE4H == 0x81))
    {
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x30,0xFF);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x31,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x32,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x33,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x35,0x32);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x39,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x3A,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF1,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF4,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF5,0x40);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x42,0x24);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xE1,0x25);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x8B,0x5A);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x8E,0xBD);

        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x92,0x7F);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x93,0x91);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x95,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x2B,0x33);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x2A,0x2A);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x2E,0x80);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x25,0x25);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x2D,0xFF);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x26,0xFF);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x27,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x24,0x25);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xA4,0xFF);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xA3,0x0D);

    }
    else
    {
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x30,0xFF);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x31,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x32,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x33,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x35,0x32);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x39,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x3A,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF1,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF4,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF5,0x40);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x42,0x24);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xE1,0x27);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x8B,0x5A);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x8E,0xBD);

        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x92,0x7F);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x93,0x91);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x95,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x2B,0x33);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x2A,0x2A);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x2E,0x80);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x25,0x25);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x2D,0xFF);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x26,0xFF);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x27,0x00);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x24,0x25);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xA4,0xFF);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xA3,0x10);
    }
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF6,0x4E);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xF7,0x20);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x89,0x02);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x14,0x08);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x6F,0x0D);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x10,0xFF);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x11,0x00);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x12,0x30);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x13,0x23);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x60,0x00);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x69,0x00);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x6A,0x03);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x4C,0x0C);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x4D,0xF7);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xE0,0x75);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xE2,0x06);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x8D,0x29);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x4E,0xD8);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x84,0x6C);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xC0,0x43);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x88,0x80);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x52,0x79);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x53,0x03);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x59,0x30);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x5E,0x02);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x5F,0x0F);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x71,0x03);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x72,0x12);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x73,0x12);

    return FE_OK;
}

/***********************************************
   Config tuner via M88DC2800 I2C repeater
   frq: RF frequency, unit: KHz
   Now works only for Alps TDAE2
   TODO: Add more front-end support
************************************************/
static int32_t m88dc2800SetFrequency( uint8_t index, uint32_t frq )
{
    uint8_t   value;
    uint8_t   tuner_reg[5];
    uint16_t  N;
    int32_t ferr;

    frq /= 1000;
    N = (uint16_t)((frq + 36125) * 10 / 625);	/*	step:	62.5 KHz	*/
    tuner_reg[0] = (uint8_t)((N >> 8) & 0x7f);
    tuner_reg[1] = (uint8_t)(N &0xff);
    tuner_reg[2] = 0x9B;

    if (frq < 125000)
        tuner_reg[3] = 0xA0;
    else if (frq < 366000)
        tuner_reg[3] = 0xA2;
    else if (frq < 622000)
        tuner_reg[3] = 0x68;
    else if (frq < 726000)
        tuner_reg[3] = 0xA8;
    else
        tuner_reg[3] = 0xE8;

   	tuner_reg[4] = 0xC6;
   	    
    /* enable I2C repeater and config tuner via I2C write */
    m88dc2800I2Cread(pData[index]->i2cId,pData[index]->i2cAddrDemod, 0x86, &value);

    value |= 0x80;

    m88dc2800I2Cwrite(pData[index]->i2cId,pData[index]->i2cAddrDemod, 0x86, value);

    /* write to RF */
    ferr = FE_I2C_Write((pData[index]->i2cId), pData[index]->i2cAddrTuner, tuner_reg, 5);
    //ferr = FD_SW_I2C_Write(&(pData[index]->i2cHandle), pData[index]->i2cAddrTuner, tuner_reg, 5);
    if(ferr != FE_OK)
        return FE_ERR_BAD_PARAMETER;

    m88dc2800Sleep(1);
    return FE_OK;
}

/***********************************************
   Set spectrum inversion and J83
   Inverted: 1, inverted; 0, not inverted
   J83: 0, J83A; 1, J83C
 ***********************************************/
static int32_t m88dc2800SetIQ(uint8_t index, uint8_t j83, FE_IQ_E iq)
{
    uint8_t value = 0;

    if (j83)
    {
        value |= 0x01;
    }

    switch( iq )
    {
        case  FE_IQ_NONE:
        case  FE_IQ_NORMAL:
            m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x83, value);
            break;

        case  FE_IQ_INVERTED:
            value |= 0x08;  //spectrum inverted
            m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x83, value);
            break;

        default:
            return FE_ERR_BAD_PARAMETER;
    }
    return FE_OK;
}

/***********************************************
   Set    symbol rate
   sym:   symbol rate, unit: KBaud
   xtal:  unit, KHz
************************************************/
static int32_t m88dc2800SetSym(uint8_t index, uint32_t sym)
{
    uint8_t  value, value1, value2;
    uint8_t  reg6FH, reg12H;
    double  fValue;
    uint32_t dwValue;

    fValue = (double)( ( (sym /1000.000) + 0.01 ) * 4294967296.0 /( M88DC2800_CRYSTAL_CLK ) );

    dwValue = (uint32_t)fValue;
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x58, (dwValue >> 24) & 0xff);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x57, (dwValue >> 16) & 0xff);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x56, (dwValue >>  8) & 0xff);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x55, (dwValue >>  0) & 0xff);

    fValue = 2048.0 * (M88DC2800_CRYSTAL_CLK)*1000.0 / sym;
    dwValue = (uint32_t)fValue;
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x5D, (dwValue >> 8) & 0xff);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x5C, (dwValue >> 0) & 0xff);

    m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x5A, &value);
    if ( (HIWORD(dwValue) & 0x0001) == 0 )
        value &= 0x7F;
    else
        value |= 0x80;
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x5A, value);

    m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x89, &value);
    if ( sym <= 1800 )
        value |= 0x01;
    else
        value &= 0xFE;
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x89, value);

    if ( sym >= 6700 )
    {
        reg6FH = 0x0D;
        reg12H = 0x30;
    }
    else if ( sym >= 4000 )
    {
        fValue = 22 * 4096 / sym;
        reg6FH = (uint8_t)fValue;
        reg12H = 0x30;
    }
    else if ( sym >= 2000 )
    {
        fValue = 14 * 4096 / sym;
        reg6FH = (uint8_t)fValue;
        reg12H = 0x20;
    }
    else
    {
        fValue = 7 * 4096 / sym;
        reg6FH = (uint8_t)fValue;
        reg12H = 0x10;
    }
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x6F, reg6FH);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x12, reg12H);

    m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xE3, &value1);
    m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xE4, &value2);

    if ( ((value1 & 0x80) == 0x80) && ((value2 & 0x80) == 0x80) )
    {
        if( sym < 3000 )
        {
            m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x6C,0x16);
            m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x6D,0x10);
            m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x6E,0x18);
        }
        else
        {
            m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x6C,0x14);
            m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x6D,0x0E);
            m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x6E,0x36);
        }
    }
    else
    {
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x6C,0x16);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x6D,0x10);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x6E,0x18);
    }
    return FE_OK;
}

/***********************************************
   Set QAM mode
   Qam: QAM mode, 16, 32, 64, 128, 256
************************************************/
static int32_t m88dc2800SetQAM(uint8_t index, FE_MOD_E Qam)
{
    uint8_t  reg00H, reg4AH, reg84H, regC2H, reg44H, reg4CH,reg4DH,reg74H,reg8BH,reg8EH;
    uint8_t  value, value1,value2;

    qamMode[index] = Qam;
    
    m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xE3, &value1);
    m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xE4, &value2);
    
    switch( Qam )
    {
        case FE_MOD_QAM_16:  // 16 QAM
          reg00H = 0x08;
          reg4AH = 0x0F;
          reg84H = 0x6C;
          regC2H = 0x02;
          reg44H = 0xAA;
          reg4CH = 0x0C;
          reg4DH = 0xF7;
          reg74H = 0x0E;
          if(((value1 & 0x80) == 0x80) && ((value2 & 0x80) == 0x80) )
          {
              reg8BH = 0x5A;
              reg8EH = 0xBD;      
          }
          else
          {
              reg8BH = 0x5B;
              reg8EH = 0x9D;   
          }
        break;
        case FE_MOD_QAM_32:  // 32 QAM
          reg00H = 0x18;
          reg4AH = 0xFB;
          reg84H = 0x6C;
          regC2H = 0x02;
          reg44H = 0xAA;
          reg4CH = 0x0C;
          reg4DH = 0xF7;
          reg74H = 0x0E;
          if(((value1 & 0x80) == 0x80) && ((value2 & 0x80) == 0x80) )
          {
              reg8BH = 0x5A;
              reg8EH = 0xBD;      
          }
          else
          {
              reg8BH = 0x5B;
              reg8EH = 0x9D;   
          }
        break;
        case FE_MOD_QAM_64:  // 64 QAM
          reg00H = 0x48;
          reg4AH = 0xCD;
          reg84H = 0x6C;
          regC2H = 0x02;
          reg44H = 0xAA;
          reg4CH = 0x0C;
          reg4DH = 0xF7;
          reg74H = 0x0E;
           if(((value1 & 0x80) == 0x80) && ((value2 & 0x80) == 0x80) )
          {
              reg8BH = 0x5A;
              reg8EH = 0xBD;      
          }
          else
          {
              reg8BH = 0x5B;
              reg8EH = 0x9D;   
          }
        break;
        case FE_MOD_QAM_128: // 128 QAM
          reg00H = 0x28;
          reg4AH = 0xFF;
          reg84H = 0x6C;
          regC2H = 0x02;
          reg44H = 0xA9;
          reg4CH = 0x08;
          reg4DH = 0xF5;
          reg74H = 0x0E;
          reg8BH = 0x5B;
          reg8EH = 0x9D;
        break;
        case FE_MOD_QAM_256: // 256 QAM
          reg00H = 0x38;
          reg4AH = 0xCD;
          reg84H = 0x6C;
          regC2H = 0x01;
          reg44H = 0xA9;
          reg4CH = 0x08;
          reg4DH = 0xF5;
          reg74H = 0x0E;
          reg8BH = 0x5B;
          reg8EH = 0x9D;
        break;
        default:  // 64 QAM
          reg00H = 0x48;
          reg4AH = 0xCD;
          reg84H = 0x6C;
          regC2H = 0x02;
          reg44H = 0xAA;
          reg4CH = 0x0C;
          reg4DH = 0xF7;
          reg74H = 0x0E;
          if(((value1 & 0x80) == 0x80) && ((value2 & 0x80) == 0x80) )
          {
              reg8BH = 0x5A;
              reg8EH = 0xBD;      
          }
          else
          {
              reg8BH = 0x5B;
              reg8EH = 0x9D;   
          }
        break;
    }
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x00,reg00H);

    m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x88, &value);
    value |= 0x08;
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x88, value);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x4B, 0xFF);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x4A, reg4AH);
    value &= 0xF7;
    m88dc2800I2Cwrite(pData[index]->i2cId,pData[index]->i2cAddrDemod, 0x88, value);

    m88dc2800I2Cwrite(pData[index]->i2cId,pData[index]->i2cAddrDemod, 0x84, reg84H);
    m88dc2800I2Cwrite(pData[index]->i2cId,pData[index]->i2cAddrDemod, 0xC2, regC2H);
    m88dc2800I2Cwrite(pData[index]->i2cId,pData[index]->i2cAddrDemod, 0x44, reg44H);
    m88dc2800I2Cwrite(pData[index]->i2cId,pData[index]->i2cAddrDemod, 0x4C, reg4CH);
    m88dc2800I2Cwrite(pData[index]->i2cId,pData[index]->i2cAddrDemod, 0x4D, reg4DH);
    m88dc2800I2Cwrite(pData[index]->i2cId,pData[index]->i2cAddrDemod, 0x74, reg74H);
    m88dc2800I2Cwrite(pData[index]->i2cId,pData[index]->i2cAddrDemod, 0x8B, reg8BH);
    m88dc2800I2Cwrite(pData[index]->i2cId,pData[index]->i2cAddrDemod, 0x8E, reg8EH);

    return FE_OK;
}

/***********************************************
 Soft reset M88DC2800
 Reset the internal status of each funtion block,
 not reset the registers.
************************************************/
static int32_t m88dc2800SoftReset(uint8_t index)
{
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x80, 0x01);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x82, 0x00);
    m88dc2800Sleep(1);
    m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x80, 0x00);

    return FE_OK;
}

/***********************************************
   Set the type of MPEG/TS interface
   type: 1, serial format; 2, parallel format
************************************************/
static int32_t m88dc2800SetTsType(uint8_t index, uint8_t type)
{
    if( type == 0 )     // parallel TS
    {
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x84, 0x6C);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xC0, 0x43);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xE2, 0x06);
    }
    else                // serial TS
    {
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x84, 0x6A);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xC0, 0x47);
        m88dc2800I2Cwrite((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0xE2, 0x02);
    }
    
    return FE_OK;
}

/***********************************************
   Get AGC lock status
   returned 1 when locked;0 when unlocked
************************************************/
static uint8_t m88dc2800GetAgcLock(uint8_t index)
{
    uint8_t  value;

    m88dc2800I2Cread((pData[index]->i2cId),pData[index]->i2cAddrDemod, 0x43, &value);

    if ( (value & 0x08) == 0x08 )
    {
        return 1;
    }
    else
    {
        return 0;
    }
}
