/******************************************************************************
**
** \file      ./subsystems/fe/mb86667/src/MB86667_Api.c
**
** \brief     public functions of MB86667 driver
**
**            (C) Fujitsu Semiconductor Europe 2006 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: MB86667_Api.c 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/
// #define MB86667_LNB_MANUAL // needed for PC Monitor software, set in C++ project options 

/* ------- required includes --------------------------------------- */
#include <stdlib.h>

#include "MB86667_Api.h"
#include "MB86667_Customise.h"
#include "MB86667_Priv.h"
#include "MB86667_Regs.h"

#include "MB86667_Demod.h"
#include "fapex/abst_types.h"


#ifdef  __cplusplus
extern "C" {
#endif

/*** defines for BER counter ***/
  /* 0 = short monitoring time for pre viterbi,
     2 = long monitoring time */
  #define BERTIM 0x00
  /* 4 = BER monitor enabled */
  #define BER_EN 0x04
  /* sampling time for post viterbi BER monitor
     0x00 = 12500 ksamples, 0x08 = 25000 ksamples,
     0x10 = 50000 ksamples, 0x18 = 100000 ksamples */
  #define BERRST 0x00


/***********************************
  MB86667_Init
    Configures the parameters describing the dish installation.
    Resets the MB86667. A hardware reset of the MB86667 must be done
    after power up and before calling this function.
  Input:
    pDownlink: pointer to Downlink structure, defined in MB86667_API.h
  Output:
    none
  Returns:
    FE_OK
    Errors from I2C routines
************************************/
int32_t MB86667_Init(Downlink_Type *pDownlink)
{
  uint8_t     ucReg;
  int32_t   error;

  /* hard reset chip */
  error  = MB86667_HardReset();
  if (error != FE_OK)
    return(error);
  /* check chip version */
  error = MB86667_I2C_Read(MB86667_REG_VER, 1, &ucReg);
  if (error != FE_OK)
    return(error);
  if (ucReg != 0xFD)
    return(FE_ERR_FE_UNKNOWN_DEVICE);

  /* switch LNB power off */
  ucReg = MB86667_BIT_GPIO | MB86667_BIT_LNB13V | MB86667_BIT_LNBOFF;
  error = MB86667_I2C_Write(MB86667_REG_PORT0, 1, &ucReg);
  if (error != FE_OK)
    return(error);

  /* switch pin to DiSeqC output */
  ucReg = MB86667_BIT_TONE2_TMD_DISEQ;
  error  = MB86667_I2C_Write(MB86667_REG_TONE2, 1, &ucReg);
  if (error != FE_OK)
    return(error);

  /* set TS output */
  ucReg = MB86667_DIG_OUTPUTS_ENABLE_BY_I2C | MB86667_TS_SERIAL | MB86667_TS_ORDER | MB86667_TS_ERROR | MB86667_TS_POL;
  error  = MB86667_I2C_Write(MB86667_REG_TSOUT, 1, &ucReg);
  if (error != FE_OK)
    return(error);
  /* set frame synchonisation thresholds */
  ucReg = MB86667_FR_LOCK | (MB86667_FR_UNLOCK << 4);
  error  = MB86667_I2C_Write(MB86667_REG_SYNC, 1, &ucReg);
  if (error != FE_OK)
    return(error);

  /* preset clock factor */
  mb86667_clock_fac(SET, 3);

  /* preset LNB control flags */
  mb86667_flags(SET, pDownlink->flags);
  /* preset repetitions of committed switches */
  mb86667_com_sw(SET, pDownlink->RepComSw);
  /* preset repetitions of uncommitted switches */
  mb86667_unc_sw(SET, pDownlink->RepUncSw);

  return(FE_OK);
} /* MB86667_Init */

/***********************************
  MB86667_RegReset
    (resets MB86667 block + registers)
    Note: This does not replace a hardware reset. A hardware
    reset must be done after power up to put also the analogue parts
    of the MB86667 in a known state.
  Input:
    none
  Output:
    none
  Returns:
    FE_OK
    Errors from I2C routines
************************************/
int32_t MB86667_RegReset(void)
{
  uint8_t     ucReg;
  int32_t   error;

  ucReg = MB86667_BIT_RESET_LSI;
  error = MB86667_I2C_Write(MB86667_REG_RESET, 1, &ucReg);
  if (error != FE_OK)
    return(error);

  /* set BER monitor to post Viterbi and a short monitor time, but do not enable it */
  error = MB86667_DisBer(1);
  if (error != FE_OK)
    return(error);

  return(FE_OK);

} /* MB86667_RegReset */

/***********************************
  MB86667_SendDiseqcCommand
   Sends a 3 to 6 byte long DiSEqC Command.
   Receive service is not implemented.
  Input:
    length        command length
    pCommand      Pointer to Command
    pReturnLength received length 
    pReturnBuffer receive Buffer
  Returns:
    FE_OK
    Errors from I2C routines
    
************************************/
int32_t MB86667_SendDiseqcCommand( uint8_t  length,
                                uint8_t* pCommands,
                                uint8_t* pReturnLength,
                                uint8_t* pReturnBuffer)
{
    uint8_t    ucDIST, ucDICR, ucDICR_tmp, ucDIEN, ucDIEN_tmp,cnt;
    int32_t  int32_t;

    /* read DiSEqC registers: DIST */
    int32_t = MB86667_I2C_Read(MB86667_REG_DIST, 1, &ucDIST);
    if( int32_t != FE_OK) return int32_t;

    if ( (ucDIST & 0xFE) == 0)  /* check if DiSEqC transmission is busy */
    {
        /* read DiSEqC registers: DIEN; DICR */
        int32_t = MB86667_I2C_Read(MB86667_REG_DIEN, 1, &ucDIEN);
        if( int32_t != FE_OK) return int32_t;

        int32_t = MB86667_I2C_Read(MB86667_REG_DICR, 1, &ucDICR);
        if( int32_t != FE_OK) return int32_t;

        if ( (ucDIEN&0xFE) != 1)
        {
            /* enable DiSEqC ouput */
            ucDIEN_tmp = ((ucDIEN&0xFE)+1);
            int32_t = MB86667_I2C_Write(MB86667_REG_DIEN, 1, &ucDIEN_tmp);
            if( int32_t != FE_OK) return int32_t;
        }        
        /* Switch off 22kHz and wait >10ms */
        if ( (ucDICR & 0x20) == 0x20)
        {
            ucDICR_tmp = (ucDICR & 0xDF);
            int32_t = MB86667_I2C_Write(MB86667_REG_DICR, 1, &ucDICR_tmp);
            if( int32_t != FE_OK) return int32_t;           
        }
        MB86667_User_Sleep(10);
      
        /* write DiSEqC command bytes */
        cnt = 0;
        do{
          int32_t = MB86667_I2C_Write(MB86667_REG_DIB0+cnt, 1, pCommands+cnt);
          if( int32_t != FE_OK) return int32_t;
          cnt++;
        }while ( cnt < length);
    
        /* Write DiSEqC Configuration register and wait */
        ucDICR_tmp = length;  /* set command length */
        ucDICR_tmp |= 0x80;   /* set start bit */
        int32_t = MB86667_I2C_Write(MB86667_REG_DICR, 1, &ucDICR_tmp);
        if( int32_t != FE_OK) return int32_t;

        MB86667_User_Sleep((length*12)+15);

        /* restore Tonburst settings */
        ucDICR_tmp |= (ucDICR & 0x18);
        int32_t = MB86667_I2C_Write(MB86667_REG_DICR, 1, &ucDICR_tmp);
        if( int32_t != FE_OK) return int32_t;

        MB86667_User_Sleep(13+15);

        /* restore DiSEqC Config register */
        int32_t = MB86667_I2C_Write(MB86667_REG_DICR, 1, &ucDICR);
        if( int32_t != FE_OK) return int32_t;
                         
        /* if DiSEqC was not enabled, disable DiSEqC output pin */
        if ( (ucDIEN&0xFE) != 1)
        {
            int32_t = MB86667_I2C_Write(MB86667_REG_DIEN, 1, &ucDIEN);
            if( int32_t != FE_OK) return int32_t;
        }
        return FE_OK;
    }
    else
        return FE_ERR_FE_BUSY;
}

/***********************************
  MB86667_SetAntennaPower
    Immediately turns on/off the LNB power.
    It also sets the internal power flag which is checked
    in the next tune call.
  Input:
    power: bool that indicates if power should be on or off
  Output:
    none
  Returns:
    FE_OK
    FD_ERR_BAD_PARAMETER
    Errors from I2C routines
************************************/
int32_t MB86667_SetAntennaPower(FBOOL power)
{
    uint8_t     ucReg, ucFlags;
    int32_t   error;

    /* change the power flag in the demods flags */
    ucFlags = (uint8_t)mb86667_flags(GET, NO_DIS_FLAG);
    if ( power )
        ucFlags |= LNB_POWER;
    else
        ucFlags &= (~LNB_POWER);
    mb86667_flags(SET, (FAPI_MB86667_DIS_FLAGS_E)ucFlags);

    if ( power )
    {
        /* switch LNB supply on */
        ucReg = MB86667_BIT_GPIO | MB86667_BIT_LNB13V | MB86667_BIT_LNBON;
        /* we switch to 13V by default, other settings are done in lnb_control
           at the next tune command */
    }
    else
    {
        /* switch LNB supply off */
        ucReg = MB86667_BIT_GPIO | MB86667_BIT_LNB13V | MB86667_BIT_LNBOFF;
    }
    error = MB86667_I2C_Write(MB86667_REG_PORT0, 1, &ucReg);
    return(error);
}

/***********************************
  MB86667_Tune
    Outputs LNB control (22KHz + DiSEqC + ...) and
    starts the acquisition process
  Input:
    pTransponder: structure with all parameters describing the transponder
    pSatellite: structure with all parameters decribing the satellite
    Refer to MB86667_API.h for definition of these structures
    quicklock: FTRUE = uses hardware routines to aquire lock
               FFALSE = uses software routines to aquire lock, better for difficult reception conditions
  Output:
    none
  Returns:
    FE_OK
    FD_ERR_BAD_PARAMETER
    Errors from I2C routines
************************************/
int32_t MB86667_Tune(MB86667_TRANSPONDER_T *pTransponder, MB86667_SATELLITE_T *pSatellite, FBOOL quicklock)
{
  #ifndef MB86667_LNB_MANUAL
  FAPI_MB86667_BAND_E bFreqBand;
  #endif
  int32_t              error;
  uint32_t               ulIFFreq, ulLockTime;
  MB86667_SET_PARAM tp_param;

FAPI_SYS_PRINT_MSG("\nwe tune to freq: %d (kHz)%d (kSymb) highLNB: %d (kHz) lowLNB: %d (kHz)\n",
                   pTransponder->frequency,
                   pTransponder->symbolrate,
                   pSatellite->LNBhighfreq,
                   pSatellite->LNBlowfreq);
  /* check parameters */
  #ifdef FAPI_FE_MB86667_DEBUG01
    sprintf(PrintHelp, "\n-----------------------------");
    MB86667_User_Print(PrintHelp);
    sprintf(PrintHelp, "\nMB86667_Tune got \nFrequency: %d KHz", (int)pTransponder->frequency);
    MB86667_User_Print(PrintHelp);
    sprintf(PrintHelp, "\nSymbolrate: %d KS/s", pTransponder->symbolrate);
    MB86667_User_Print(PrintHelp);
    if (pTransponder->fec == FEC_1_2)
      sprintf(PrintHelp, "\nFEC: 1/2");
    else if (pTransponder->fec == FEC_2_3)
      sprintf(PrintHelp, "\nFEC: 2/3");
    else if (pTransponder->fec == FEC_3_4)
      sprintf(PrintHelp, "\nFEC: 3/4");
    else if (pTransponder->fec == FEC_5_6)
      sprintf(PrintHelp, "\nFEC: 5/6");
    else if (pTransponder->fec == FEC_7_8)
      sprintf(PrintHelp, "\nFEC: 7/8");
    else if (pTransponder->fec == FEC_AUTO)
      sprintf(PrintHelp, "\nFEC: auto");
    else
      sprintf(PrintHelp, "\nFEC: unknown 0x%x", pTransponder->fec);
    MB86667_User_Print(PrintHelp);
    if ((pTransponder->params & MB86667_IQ_INV) == MB86667_IQ_INV)
      sprintf(PrintHelp, "\nIQ: inverted");
    else if ((pTransponder->params & MB86667_AUTO_IQ) == MB86667_AUTO_IQ)
      sprintf(PrintHelp, "\nIQ: auto");
    else
      sprintf(PrintHelp, "\nIQ: normal");
    MB86667_User_Print(PrintHelp);
    if ((pTransponder->params & 0x03) == POL_HOR)
      sprintf(PrintHelp, "\nPolarisation: horizontal");
    else if ((pTransponder->params & 0x03) == POL_VER)
      sprintf(PrintHelp, "\nPolarisation: vertical");
    else
      sprintf(PrintHelp, "\nPolarisation: unknown 0x%x", (pTransponder->params & 0x03));
    MB86667_User_Print(PrintHelp);
    sprintf(PrintHelp, "\n-----------------------------\n");
    MB86667_User_Print(PrintHelp);
  #endif

  /* preset flags to "unknown" state */
  mb86667_tune_states(SET, TF_UNKNOWN);

  /*  With the commonly used LO frequencies of 9.75GHz and 10.6GHz the
      Ku band is divided at 11.7 GHz resulting in IF frequencies of
      950 - 1950 and 1100 - 2150 MHz */
  if (labs(pTransponder->frequency - pSatellite->LNBlowfreq) <= MB86667_BAND_SW)
  {                 /* low band ok */
    ulIFFreq = labs(pTransponder->frequency - pSatellite->LNBlowfreq);
    #ifndef MB86667_LNB_MANUAL
    bFreqBand = LOW;
    #endif
  }
  else
  {
    /* using high band */
    ulIFFreq = labs(pTransponder->frequency - pSatellite->LNBhighfreq);
    #ifndef MB86667_LNB_MANUAL
    bFreqBand = HIGH;
    #endif
  }
  if ((ulIFFreq < MB86667_MIN_FREQ) || (ulIFFreq > MB86667_MAX_FREQ))
    return(FE_ERR_BAD_PARAMETER);
  if ((pTransponder->symbolrate < MB86667_MIN_SYM) || (pTransponder->symbolrate > MB86667_MAX_SYM))
    return(FE_ERR_BAD_PARAMETER);
  if (pTransponder->fec > FEC_AUTO)
    return(FE_ERR_BAD_PARAMETER);

  /* disable bit error rate counter before tuning */
  /* BER counter is re-enabled again during next call of MB86667_GetBer */
  error = MB86667_DisBer(1);
  if (error != FE_OK)
    return(error);

  /* set LNB control */
  #ifndef MB86667_LNB_MANUAL
    error = mb86667_lnb_control(pTransponder->frequency, pTransponder->params, pSatellite->satID, bFreqBand);
    if (error != FE_OK)
      return(error);
  #endif

  tp_param.frequency  = ulIFFreq;
  tp_param.symbolrate = pTransponder->symbolrate;
  /* the following 3 are returned from the tune function */
  tp_param.fec        = 0;
  tp_param.freqerr    = 0;
  error = MB86667_DoTune(&tp_param, 0, quicklock);
  if (error != FE_OK)
    return(error);

  /* Set time to lock */
  ulLockTime = 8000000 / pTransponder->symbolrate; /* rough estimation */
  ulLockTime *= 2;
  ulLockTime += TIM_RESOLUTION;

  #ifdef FAPI_FE_MB86667_DEBUG01
    sprintf(PrintHelp, "\nCalculated max. lock time is %d ms", (int)ulLockTime);
    MB86667_User_Print(PrintHelp);
  #endif

  mb86667_lock_time(SET, ulLockTime);

  /* Reset lock timer */
  error = MB86667_User_Timers(TIM_RES_TIM, TIM_LOCK, &ulLockTime);
  if (error != FE_OK)
    return(error);

  mb86667_tune_states(SET, TF_ACQ_START);
  return(FE_OK);
} /* MB86667_Tune */

/***********************************
  MB86667_GetBer
    read out BER value after Viterbi
  Input:
    bersel: 1 = post Viterbi, 0 = pre Viterbi
  Output:
    Errors based on 1e-6
    e.g. 200 = 2*10e4
  Returns:
    FE_OK
    Errors from I2C routines
************************************/
int32_t MB86667_GetBer(uint32_t *pulErrors, uint8_t bersel)
{
  uint32_t    ulErr;
  uint8_t     ucBER0, ucBER1, ucBER2, ucVal;
  int32_t   error;


  *pulErrors = 500000;
  error = MB86667_I2C_Read(MB86667_REG_STAT, 1, &ucVal);
  if (error != FE_OK)
    return(error);
  if ((0x01 & ucVal) != 0x01) /* check frame sync */
    return(FE_OK); /* if not in sync return here */

  /* enable BER monitoring */
  ucVal = (uint8_t)(bersel | BER_EN | BERRST);
  error = MB86667_I2C_Write(MB86667_REG_BER, 1, &ucVal);
  if (error != FE_OK)
    return(error);

  /* read the BER values */
  error = MB86667_I2C_Read(MB86667_REG_BERLS, 1, &ucBER0);
  if (error != FE_OK)
    return(error);
  error = MB86667_I2C_Read(MB86667_REG_BERMI, 1, &ucBER1);
  if (error != FE_OK)
    return(error);
  error = MB86667_I2C_Read(MB86667_REG_BERMS, 1, &ucBER2);
  if (error != FE_OK)
    return(error);

  ulErr = ucBER2;
  ulErr = (ulErr << 8) + ucBER1;
  ulErr = (ulErr << 8) + ucBER0;
  if (bersel == 0)
  { /* pre-Viterbi */
    /* original equation is:
       *pulErrors = (ulErr >> 16 (resp. 24)) * 1000000
       But this has to modified to work with int to: */
    *pulErrors = (ulErr * (1000000 >> 12)) >> 4;
    #if BERTIM > 0
      *pulErrors >>= 8;
    #endif
  }
  else
  { /* post-Viterbi */
    /* 2 / 25 = 1 / 12.5, 12.5 comes from the number of samples / 1000000 */
    /* original equation is:
       *pulErrors = ulErr / 12500000 * 1000000
                  = ulErr / 125      * 10
                  = ulErr / 25      * 2  */
    *pulErrors = (2 * ulErr) / (25 << (BERRST >> 3)) ;
  }
  return(FE_OK);
} /* MB86667_GetBer */

/***********************************
  MB86667_DisBer
    disable BER counter
  Input:
    bersel: 1 = post Viterbi, 0 = pre Viterbi
  Output:
    none
  Returns:
    FE_OK
    Errors from I2C routines
************************************/
int32_t MB86667_DisBer(uint8_t bersel)
{
  uint8_t     ucVal;
  int32_t   error;

  /* disable BER monitoring */
  ucVal = (uint8_t)(bersel | BERRST);
  error = MB86667_I2C_Write(MB86667_REG_BER, 1, &ucVal);
  return(error);
} /* MB86667_DisBer */

/***********************************
  MB86667_GetLock
    Check, if frame sync was achieved
  Input:
    none
  Output:
    scStatus:
      -3 = unknown (returned in case of an error)
      -2 = acquisition pending, too soon after tune command
      -1 = acquisition pending, carrier found, but not yet locked
       0 = acquisition time over, not locked
       1 = completely locked
  Returns:
    FE_OK
    Errors from I2C routines
************************************/
int32_t MB86667_GetLock(int8_t *scStatus)
{
  uint8_t   ucVal ;
  uint32_t  ulDiffTime;
  int32_t   error;
  FAPI_MB86667_TUNE_STATE_E tsState;

  *scStatus = -3;
  tsState = mb86667_tune_states(GET, TF_ACQ_END);
  #ifdef FAPI_FE_MB86667_DEBUG01
    sprintf(PrintHelp, "\nmb86667_tune_states=%d\n",tsState);
    MB86667_User_Print(PrintHelp);
  #endif
  #ifdef FAPI_FE_MB86667_DEBUG01
  {
     uint8_t val;
     MB86667_GetSigLevel(&val);
     sprintf(PrintHelp, "MB86667_GetSigLevel=%d\n",val);
     MB86667_User_Print(PrintHelp);
     MB86667_GetSigQual(&val);
     sprintf(PrintHelp, "MB86667_GetSigQual=%d\n",val);
     MB86667_User_Print(PrintHelp);
  }
  #endif
  /* lock information not available in case of failed tuning procedure */
  if (tsState == TF_UNKNOWN)
  {
    return(FE_OK);
  }

  if (tsState == TF_ACQ_START)
  { /* check the expired time after a tune process */
    error = MB86667_User_Timers(TIM_GET_TIM, TIM_LOCK, &ulDiffTime);
    if (error != FE_OK)
      return(error);
    if (ulDiffTime < 50)
    { /* keep tune state */
      *scStatus = -2;
      return(FE_OK); /* there was no time to unlock */
    }
    else
    {
      tsState = TF_CS_START;
      mb86667_tune_states(SET, TF_CS_START);
      /* no return here -> continue to check lock status */
    }
  }

  /* here it can already be checked, if the MB86667 is in lock */
  error = MB86667_I2C_Read(MB86667_REG_STAT, 1, &ucVal);
  #ifdef FAPI_FE_MB86667_DEBUG01
    sprintf(PrintHelp, "\nwe check lock=%d\n",(ucVal & 0x03));
    MB86667_User_Print(PrintHelp);
    {
      switch((ucVal & 0x1c)>>2)
      {
        case 0:
          sprintf(PrintHelp, "we are unlock!\n");
        break;
        case 1:
          sprintf(PrintHelp, "we are lock! found FEC=1/2\n");
        break;
        case 2:
          sprintf(PrintHelp, "we are lock! found FEC=2/3\n");
        break;
        case 3:
          sprintf(PrintHelp, "we are lock! found FEC=3/4\n");
        break;
        case 4:
          sprintf(PrintHelp, "we are lock! found FEC=5/6\n");
        break;
        case 5:
          sprintf(PrintHelp, "we are lock! found FEC=7/8\n");
        break;
        default:
          sprintf(PrintHelp, "we are lock! FEC=???\n");
        break;
     }
  }
    MB86667_User_Print(PrintHelp);
  #endif
  if (error != FE_OK)
    return(error);
  /* check frame and viterbi sync */
  if ((0x03 & ucVal) == 0x03)
  { /* if MB86667 is locked then ignore all further states */
    *scStatus = 1;
    mb86667_tune_states(SET, TF_ACQ_END);
    return (FE_OK);
  }
  else if (tsState == TF_ACQ_END)
  { /* not locked but end of acquisition reached -> was set in previous call */
    *scStatus = 0;
    return (FE_OK);
  }
  /* else ... */
  else
  {
    error = MB86667_User_Timers(TIM_GET_TIM, TIM_LOCK, &ulDiffTime);
    #ifdef FAPI_FE_MB86667_DEBUG01
        sprintf(PrintHelp, "we waited: %lumsec\n",ulDiffTime);
        MB86667_User_Print(PrintHelp);
    #endif
    if (error != FE_OK)
      return(error);
    if(ulDiffTime>=mb86667_lock_time(GET, 0))
    {
       mb86667_tune_states(SET, TF_ACQ_END);
       *scStatus = 0;
       return(FE_OK);
    } 
  }
  /* check AGC value */
  error = MB86667_I2C_Read(MB86667_REG_AGCI, 1, &ucVal);
  if (error != FE_OK)
    return (error);
  if( ucVal == 0xff )
  { /* AGC still indicates: no transponder */
    *scStatus = 0; /* => this means: not locked */
    mb86667_tune_states(SET, TF_ACQ_END);
    return(FE_OK);
  }

  *scStatus = -2;
  return(FE_OK); /* there was not enough time to find carrier */

} /* MB86667_GetLock */


/***********************************
  MB86667_GetFEC
    Check, if the demodulator is in lock
  Input:
    none
  Output:
    scFEC: <0   = returned in case of an error
           0x00 = not in lock
           0x01 = FEC rate 1/2
           0x02 = FEC rate 2/3
           0x03 = FEC rate 3/4
           0x04 = FEC rate 5/6
           0x05 = FEC rate 7/8
  Returns:
    FE_OK
    Errors from I2C routines
************************************/
int32_t MB86667_GetFEC(int8_t *scFEC)
{
  uint8_t   ucVal;
  int32_t error;

  *scFEC = -1;
  /* check lock staus */
  error = MB86667_I2C_Read(MB86667_REG_STAT, 1, &ucVal);
  if (error != FE_OK)
    return(error);

  /* preset to unlocked */
  *scFEC = 0;
  /* check lock */
  if ((0x03 & ucVal) == 0x03)
  { /* MB86667 is in lock */
    *scFEC = (uint8_t)((ucVal >> 2) & 0x07); /* return FEC value */
  }
  /* else .. not in lock */
  return (FE_OK);
} /* MB86667_GetFEC */

/***********************************
  MB86667_GetSigLevel
    read out signal level
  Inputs:
    none
  Outputs:
    ucLevel: ranges from 0 (no signal) to 255 (strong signal)
  Returns:
    FE_OK
    Errors from I2C routines
************************************/
int32_t MB86667_GetSigLevel(uint8_t *ucLevel)
{
  uint8_t   ucVal ;
  int32_t error;

  *ucLevel = 0;
  error = MB86667_I2C_Read(MB86667_REG_AGC_MONITOR, 1, &ucVal);
  if (error != FE_OK)
    return(error);

  if (ucVal < 46)
    ucVal = 0;
  else if (ucVal >= 174)
    ucVal = 127;
  else
    ucVal = (uint8_t)(ucVal - 46);
  *ucLevel = (uint8_t)(2*(127 - ucVal));
  return(FE_OK);
} /* MB86667_GetSigLevel */

/***********************************
  MB86667_GetSigQual
    read out signal quality based on the C/N value
  Inputs:
    none
  Outputs:
    ucQuality: ranges from 0 (bad) to 255 (very good)
  Returns:
    FE_OK
    Errors from I2C routines
************************************/
int32_t MB86667_GetSigQual(uint8_t *ucQuality)
{
  uint8_t   ucVal ;
  int32_t error;

  *ucQuality = 0;
  error = MB86667_I2C_Read(MB86667_REG_CN_MONITOR, 1, &ucVal);
  if (error != FE_OK)
    return(error);

  *ucQuality = ucVal;
  return(FE_OK);
} /* MB86667_GetSigQual */

/***********************************
  MB86667_GetSRErr
    works only in normal lock mode
  Input:
  Output:
    symbol rate error in KS/s
  Returns:
    FE_OK
    FE_ERR_FE_I2C
************************************/
int32_t MB86667_GetSRErr(int32_t *iSRE)
{
   int32_t clkmst;
   int32_t strm ;
   uint8_t  csel, rsel, deci;
   uint8_t  ucVal;
   uint16_t usSymRate;

   /* get clockmaster and symbolrate */
   usSymRate = mb86667_sym_rate(GET, 0);
   MB86667_Smrt_Info_Get(usSymRate, &deci, &csel, &rsel, &clkmst );
   if( MB86667_I2C_Read(MB86667_REG_STRM, 1, &ucVal) != 0 )
       return(FE_ERR_FE_I2C);
   if( ucVal>127 )
       strm = (int32_t)(ucVal) -256 ;
   else
       strm = (int32_t)(ucVal) ;
   *iSRE = (strm * clkmst) / (1<<deci)/65536;

   return(FE_OK);
}


/***********************************
  MB86667_GetAFC
    Returns the frequency offset in KHz.
    The MB86667 needs to be in lock to get valid result
    and it works only in normal lock mode.
  Input:
    none
    uses the clock factor that was set in "mb86667_clock_fac"
  Output:
    frequency offset in KHz
  Returns:
    FE_OK
    Errors from I2C routines
************************************/
int32_t MB86667_GetAFC(int32_t *iAFC)
{
    int32_t err=MB86667_Freqerr_Chk(0, 0, mb86667_sym_rate(GET, 0), 1, iAFC);
    FAPI_SYS_PRINT_MSG("\nFreq-Offset=%d (kHz)\n\n",*iAFC);
    return err;
} /* MB86667_GetAFC */

#ifdef  __cplusplus
}
#endif

/* Version History */
/*****************************************************************************

$Log: MB86667_Api.c,v $
Revision 1.2  2008/08/28 15:08:29  sgehre
use FE_ERR* codes consistently, removing
occurrences of FAPI_ERR* in the code

Revision 1.1  2008/07/11 13:36:32  sgehre
- DSF89000 activated by default
- added support for MB86667 DVB-S frontend
- use SUBSYS_FE_MODULE_BASE for errors

Revision 1.3  2008/02/24 21:16:12  kraft
version Fuj 4.3 implemented + debug on freq added

Revision 1.2  2007/11/19 09:11:32  kraft
fuj sat fe with blind scan

Revision 1.8  2007/09/03 09:16:30  skau
SendDiseqcCommand added.

Revision 1.7  2007/07/04 10:38:28  skau
Additional parameters for MB86667_Tune and MB86667_GetBer added.

Revision 1.6  2007/06/25 12:26:49  skau
New file structure.

Revision 1.3  2007/01/24 10:08:00  skau
Compiles without warnings, all CVS headers updated and
ready for first customer release.

Revision 1.2  2006/11/30 09:50:19  skau
First working version.

Revision 1.1  2006/11/03 15:11:02  skau
Driver for MB86667.This is the first (non-working) version.


*****************************************************************************/
