/******************************************************************************
**
** \file      ./subsystems/fe/mb86667/src/MB86667_Priv.c
**
** \brief     private functions for MB86667 driver
**
**            (C) Fujitsu Semiconductor Europe 2006 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: MB86667_Priv.c 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/

#include "MB86667_Api.h"
#include "MB86667_Customise.h"
#include "MB86667_Regs.h"
#include "MB86667_Priv.h"

#ifdef  __cplusplus
extern "C" {
#endif

int power_logic=0;

/***********************************
  mb86667_clock_fac
  Input:
    dir: defines, if the clock factor is set or returned
    ucClockFac: clock factor in case of "SET"
  Output:
    none
  Returns:
    the clock factor in case of "GET" AND "SET"
************************************/
uint8_t mb86667_clock_fac(FAPI_MB86667_GETSET_E dir,  uint8_t ucClockFac)
{
  static uint8_t factor = 0;

  if(dir == SET)
  {
     factor = ucClockFac;
  }
  return factor;
}


/***********************************
  mb86667_com_sw
  Input:
    dir: defines if the flags are set or returned
    ucRep: number of repetitions for committed switches
  Output:
    none
  Returns:
    the number of repetitions in case of "GET" AND "SET"
************************************/
uint8_t mb86667_com_sw(FAPI_MB86667_GETSET_E dir, uint8_t ucRep)
{
  static uint8_t No = 0;

  if(dir == SET)
  {
     No = ucRep;
  }
  return No;
}

/***********************************
  mb86667_cs_time
    stores the time that needs to be waited until carrier search is finished
  Input:
    dir: defines, if the carrier search time is set or returned
    ulTime: wait time in ms
  Output:
    none
  Returns:
    the clock factor in case of "GET" and "SET"
************************************/
uint32_t mb86667_cs_time(FAPI_MB86667_GETSET_E dir,  uint32_t ulTime)
{
  static uint32_t time = 0;

  if(dir == SET)
  {
     time = ulTime;
  }
  return time;
}

/***********************************
  mb86667_flags
  Input:
    dir: defines if the flags are set or returned
    DisFlags: flags in case of "SET"
  Output:
    none
  Returns:
    the flags in case of "GET" AND "SET"
************************************/
FAPI_MB86667_DIS_FLAGS_E mb86667_flags(FAPI_MB86667_GETSET_E dir, FAPI_MB86667_DIS_FLAGS_E DisFlags)
{
  static FAPI_MB86667_DIS_FLAGS_E flags = NO_DIS_FLAG;

  if(dir == SET)
  {
     flags = DisFlags;
  }
  return flags;
}

/***********************************
  mb86667_lock_time
    stores the time that needs to be waited until lock can be achieved
  Input:
    dir: defines, if the lock time is set or returned
    ulTime: wait time in ms
  Output:
    none
  Returns:
    the clock factor in case of "GET" and "SET"
************************************/
uint32_t mb86667_lock_time(FAPI_MB86667_GETSET_E dir,  uint32_t ulTime)
{
  static uint32_t time = 0;

  if(dir == SET)
  {
     time = ulTime;
  }
  return time;
}

/***********************************
  mb86667_sym_rate
  Input:
    dir: defines, if the clock factor is set or returned
    uwSymRate: symbol rate in case of "SET"
  Output:
    none
  Returns:
    the symbol rate in case of "GET" AND "SET"
************************************/
uint16_t mb86667_sym_rate(FAPI_MB86667_GETSET_E dir,  uint16_t uwSymRate)
{
  static uint16_t rate = 0;

  if(dir == SET)
  {
     rate = uwSymRate;
  }
  return rate;
}

/***********************************
  mb86667_tune_states
    Holds the flags to indicate the timing after a tune command.
    This is needed to avoid continuous read out of the timer once the
    acquisition time has expired.
    4 = last tune call failed, MB86667 has unknown settings
    3 = acquisition just started, maybe still in lock with old settings
    2 = new settings in place, carrier search started
    1 = carrier search ended
    0 = acquisition time ended
  Input:
    dir: defines, if the lock flags are set or returned
    ucTuneState: lock flags in case of "SET"
  Output:
    none
  Returns:
    the tuning status in case of "GET" AND "SET"
************************************/
FAPI_MB86667_TUNE_STATE_E mb86667_tune_states(FAPI_MB86667_GETSET_E dir, FAPI_MB86667_TUNE_STATE_E tuneState)
{
  static FAPI_MB86667_TUNE_STATE_E flags = TF_ACQ_END;

  if(dir == SET)
  {
     flags = tuneState;
  }
  return (flags);
}

/***********************************
  mb86667_unc_sw
  Input:
    dir: defines if the flags are set or returned
    ucRep: number of repetitions for uncommitted switches
  Output:
    none
  Returns:
    the number of repetitions in case of "GET" AND "SET"
************************************/
uint8_t mb86667_unc_sw(FAPI_MB86667_GETSET_E dir, uint8_t ucRep)
{
  static uint8_t No = 0;

  if(dir == SET)
  {
     No = ucRep;
  }
  return No;
}

/***********************************
  mb86667_lnb_control
    makes 22KHz, 13/18V, tone burst and DiSEqC settings.
    If only tone burst or DiSEqC are selected (and no classical control)
    then the voltage is switched to 13V to save power in the LNB.
    (The recommendation of EUTELSAT is to switch to 12V, but this would mean
    another voltage control pin)
    DiSEqC 1.2 support is limited here to sending the position command to the positioner.
    To adjust the positioner to the positions and to store the postition is not part
    of the tuning procedure. It is very much depending on the look and feel (menu design)
    of the STBs and therefore has to be implemented by the STB manufacturer directly.
  Input:
    ulFreq: Transponder centre frequency in KHz
    usTrPar: Transponder parameters as defined in structure "Transponder_Type"
    usSatID: satellite ID
    bFreqB:  high- or low-band
  Output:
    none
  Returns:
    FE_OK
    Errors from I2C routines
************************************/
int32_t mb86667_lnb_control(uint32_t ulFreq, uint16_t usTrPar, uint16_t usSatID, FAPI_MB86667_BAND_E bFreqB)
{
  uint8_t     ucOldPorts, ucReg, ucFlags;
  uint8_t     ucBytes[2][6], DCCNUM[2], ucTmpSw[2];
  uint32_t    ulDisTime[2];
  uint16_t    i, j;
  int32_t   error;

  /* get current setting */
  error = MB86667_I2C_Read(MB86667_REG_PORT0, 1, &ucOldPorts);
  if (error != FE_OK)
    return(error);

  /* check first if LNB power should be on */
  ucFlags = (uint8_t)mb86667_flags(GET, NO_DIS_FLAG);

  if ((ucFlags & LNB_POWER) == 0)
  {
    /* switch continuous tone off */
    ucReg = 0x80;
    error = MB86667_I2C_Write(MB86667_REG_DICR, 1, &ucReg);
    if (error != FE_OK)
      return(error);
    /* Disable DiSEqC output */
    ucReg = 0;
    error = MB86667_I2C_Write(MB86667_REG_DIEN, 1, &ucReg);
    if (error != FE_OK)
      return(error);
    /* switch LNB supply off */
    if(power_logic==0)     //org
    {
       ucReg = MB86667_BIT_GPIO | MB86667_BIT_LNB13V | MB86667_BIT_LNBOFF;
    }
    else     // AVB
    {
       ucReg = MB86667_BIT_GPIO | MB86667_BIT_LNB13V | MB86667_BIT_LNBON; 
    }
    error = MB86667_I2C_Write(MB86667_REG_PORT0, 1, &ucReg);
    if (error != FE_OK)
      return(error);

    /* LNB supply is off now, no further action required */
    return(FE_OK);
  }
  /* else LNB supply is on */

  /*************************************************
   DiSEqC command settings
   *************************************************/
  /* check, which DiSEqC is selected */
  if ((ucFlags & DISEQC1_0) != 0)
  { /* send committed switches */
    ucTmpSw[0] = 0;
    ucTmpSw[1] = (uint8_t)(mb86667_com_sw(GET, 0) + 1);
    /* set transmission time for the DiSEqC commands */
    ulDisTime[0] =    0;
    ulDisTime[1] =    (30 + 54);

    DCCNUM[0] = 0;
    DCCNUM[1] = 4;
  }
  else if ((ucFlags & DISEQC1_1DTH) != 0)
  { /* send uncommitted and committed switches */
    ucTmpSw[0] = (uint8_t)(mb86667_unc_sw(GET, 0) + 1);
    ucTmpSw[1] = (uint8_t)(mb86667_com_sw(GET, 0) + 1);
    /* set transmission time for the DiSEqC commands */
    ulDisTime[0] =    (30 + 54);
    ulDisTime[1] =    (30 + 54);
    DCCNUM[0] = 4;
    DCCNUM[1] = 4;
    ucBytes[0][0] = 0xE0;
    ucBytes[0][1] = 0x10;
    ucBytes[0][2] = 0x39;
    ucBytes[0][3] = 0xF0;
    if ((usSatID & 0x04) == 0x04) /* check lowest bit */
      ucBytes[0][3] |= 0x01;
    if ((usSatID & 0x08) == 0x08) /* check next bit */
      ucBytes[0][3] |= 0x02;
    if ((usSatID & 0x10) == 0x10) /* check next bit */
      ucBytes[0][3] |= 0x04;
    if ((usSatID & 0x20) == 0x20) /* check next bit */
      ucBytes[0][3] |= 0x08;
  }
  else if ((ucFlags & DISEQC1_1SMA) != 0)
  { /* send frequency and committed switches */
    ucTmpSw[0] = (uint8_t)(mb86667_unc_sw(GET, 0) + 1);
    ucTmpSw[1] = (uint8_t)(mb86667_com_sw(GET, 0) + 1);
    /* set frequency first */
    /* set transmission time for the DiSEqC commands */
    ulDisTime[0] =    (30 + 54);
    ulDisTime[1] =    (30 + 54);
    DCCNUM[0] = 4;
    DCCNUM[1] = 4;
    ucBytes[0][0] = 0xE0;
    ucBytes[0][1] = 0x71;
    ucBytes[0][2] = 0x58;
    /* first two nibbles */
    ucBytes[0][3]  = (uint8_t)((ulFreq / 10000000) << 4);
    ulFreq -= (ulFreq / 10000000) * 10000000;
    ucBytes[0][3] += (uint8_t)(ulFreq /  1000000);
    ulFreq -= (ulFreq / 1000000) * 1000000;

    if(ulFreq != 0)
    {                 /* one more byte to send */
      DCCNUM[0]++;
      /* add another 14 ms wait time */
      ulDisTime[0] +=  14;
      ucBytes[0][4]  = (uint8_t)((ulFreq / 100000) << 4);
      ulFreq     -= (ulFreq / 100000) * 100000;
      ucBytes[0][4] += (uint8_t)(ulFreq /  10000);
      ulFreq     -= (ulFreq /  10000) *  10000;
    }
    if(ulFreq != 0)
    {                 /* one more byte to send */
      DCCNUM[0]++;
      /* add abother 14 ms wait time */
      ulDisTime[0] +=  14;
      ucBytes[0][5]  = (uint8_t)((ulFreq / 1000) << 4);
      ulFreq     -= (ulFreq / 1000) * 1000;
      ucBytes[0][5] += (uint8_t)(ulFreq /  100);
    }
  }
  else if ((ucFlags & DISEQC1_2) != 0)
  { /* send position and committed switches */
    ucTmpSw[0] = (uint8_t)(mb86667_unc_sw(GET, 0) + 1);
    ucTmpSw[1] = (uint8_t)(mb86667_com_sw(GET, 0) + 1);
    /* set positioner command first */
    /* set transmission time for the DiSEqC commands */
    ulDisTime[0] =    (30 + 54);
    ulDisTime[1] =    (30 + 54);
    DCCNUM[0] = 4;
    DCCNUM[1] = 4;
    ucBytes[0][0] = 0xE0;
    ucBytes[0][1] = 0x30;
    ucBytes[0][2] = 0x6B;
    ucBytes[0][3] = usSatID;
  }
  else
  {
    DCCNUM[0] = 0;
    DCCNUM[1] = 0;
    ucTmpSw[0] = 0;
    ucTmpSw[1] = 0;
  }

  if (DCCNUM[1] > 0)
  { /* execute only if DiSEqC is wanted */
    /* set the committed switches anyhow */
    ucBytes[1][0] = 0xE0;
    ucBytes[1][1] = 0x10;
    ucBytes[1][2] = 0x38;
    ucBytes[1][3] = 0xF0;
    /* check hi/lo band */
    if (bFreqB == HIGH)
      ucBytes[1][3] |= 0x01; /* switch to high band */
    /* check polarity */
    if ((usTrPar & (POL_HOR | POL_VER)) == POL_HOR)
      ucBytes[1][3] |= 0x02; /* switch on for horizontal */

    /* check sat ID, position */
    if ((usSatID & 0x01) != 0)
      ucBytes[1][3] |= 0x04; /* position 1 or 3 */
    /* check sat ID, option */
    if ((usSatID & 0x02) != 0)
      ucBytes[1][3] |= 0x08; /* position 2 or 3 */
  }
  /*************************************************
   end of DiSEqC command settings
   *************************************************/

  /*************************************************
   LNB supply settings
   *************************************************/
  /* check for change in LNB supply */
  if( (ucFlags & (MINI_DISEQC |  DISEQC1_0 | DISEQC1_1DTH | DISEQC1_1SMA | DISEQC1_2)) != 0)
  { /* switch continuous tone off only if there is a DiSEqC command or Tone Burst */
    ucReg = 0x80;
    error = MB86667_I2C_Write(MB86667_REG_DICR, 1, &ucReg);
    if (error != FE_OK)
      return(error);
  }

  /* check 22KHz / voltage control */
  if(power_logic==0)     //org
  {
     ucReg = MB86667_BIT_GPIO | MB86667_BIT_LNBON;
  }
  else    // AVB
  {
     ucReg = MB86667_BIT_GPIO | MB86667_BIT_LNBOFF;
  }
  if ((ucFlags & CLASSIC_CTR) != 0)
  {
    /* check polarity for LNB power */
    if ((usTrPar & (POL_HOR | POL_VER)) == POL_HOR)
      /* switch 18V on */
      ucReg |= MB86667_BIT_LNB18V;
    else
      /* switch 13V on */
      ucReg |= MB86667_BIT_LNB13V;
  }
  else
    /* switch on 18V to reduce power dissipation in regulator and give more power to motors */
    ucReg |= MB86667_BIT_LNB18V;
  error = MB86667_I2C_Write(MB86667_REG_PORT0, 1, &ucReg);
  if (error != FE_OK)
    return(error);

  /* wait 200ms if LNB was switched off before */
//  if((ucOldPorts & MB86667_BIT_LNB) != (ucReg & MB86667_BIT_LNB)) //MB86667_BIT_LNBOFF
    MB86667_User_Sleep(185); /* + 15ms which come from MB86667 itself */
  /*************************************************
   end of LNB supply settings
   *************************************************/

  /*************************************************
   Output DiSEqC bytes
   *************************************************/
  /* Enable DiSEqC output */
  ucReg = 0x01;
  error = MB86667_I2C_Write(MB86667_REG_DIEN, 1, &ucReg);
  if (error != FE_OK)
    return(error);

  for (j = 0; j <= 1; j++)
  {
    while (ucTmpSw[j] > 0)
    { /* fill DiSEqC registers with pre-entered bytes */
      for ( i = 0; i < DCCNUM[j]; i++)
      {
        error = MB86667_I2C_Write((uint8_t)(MB86667_REG_DIB0 + i), 1, &ucBytes[j][i]);
        if (error != FE_OK)
          return(error);
      }
      /* Send commmand */
      /* output DiSEqC and/or tone burst */
      ucReg = DCCNUM[j] | (uint8_t)0x80; /* set start bit */
      error = MB86667_I2C_Write(MB86667_REG_DICR, 1, &ucReg);
      if (error != FE_OK)
        return(error);
      /* decrement repetition counter */
      ucTmpSw[j]--;
      /* wait for transmission of DiSEqC commands */
      MB86667_User_Sleep(ulDisTime[j]);
    } /* while (ucTmpSw[j] > 0) */
  } /* for (j = 0... */


  /* set tone burst flags */
  if ((ucFlags & MINI_DISEQC) != 0)
  {
    /* the 15 ms pause that is needed here comes from the chip itself */
    ucReg = 0x10; /* enable tone burst */
    if ((usSatID & 0x01) != 0) /* set tone burst 1 for odd sat IDs */
      ucReg |= 0x08;
    /* output DiSEqC and/or tone burst */
    ucReg |= 0x80; /* set start bit */
    error = MB86667_I2C_Write(MB86667_REG_DICR, 1, &ucReg);
    if (error != FE_OK)
      return(error);
    /* set wait time for tone burst (pre-pause + tone burst duration) */
    MB86667_User_Sleep(15 + 13);
  }

  if( (ucFlags & (MINI_DISEQC |  DISEQC1_0 | DISEQC1_1DTH | DISEQC1_1SMA | DISEQC1_2)) != 0)
  { /* execute only if there was a DiSEqC command or Tone Burst before */
    /* this is always executed (eve when no continuous tone follows)
       to avoid overlapping in case mb86667_lnb_control is called immediately again */
    MB86667_User_Sleep(15); /* wait 15 ms in case another command follows */
  }

  /* finally output continuous tone */
  if ((ucFlags & CLASSIC_CTR) != 0)
  {
    /* preset to off */
    ucReg = 0x80;
    /* set continuous tone only if it is not SMATV */
    if( ((ucFlags & DISEQC1_1SMA) == 0) && (bFreqB == HIGH) )
      ucReg |= 0x20;

    error = MB86667_I2C_Write(MB86667_REG_DICR, 1, &ucReg);
    if (error != FE_OK)
      return(error);
  }
  return (FE_OK);
} /* mb86667_lnb_control */

#ifdef  __cplusplus
}
#endif

/* Version History */
/*****************************************************************************

$Log: MB86667_Priv.c,v $
Revision 1.1  2008/07/11 13:36:32  sgehre
- DSF89000 activated by default
- added support for MB86667 DVB-S frontend
- use SUBSYS_FE_MODULE_BASE for errors

Revision 1.2  2007/11/19 09:11:32  kraft
fuj sat fe with blind scan

Revision 1.8  2007/07/04 10:40:45  skau
Finally made compatible to the windows monitor software.

Revision 1.7  2007/06/25 12:26:49  skau
New file structure.

Revision 1.4  2007/01/24 12:11:32  skau
Static variables removed to support more than one instance.

Revision 1.3  2007/01/24 10:08:00  skau
Compiles without warnings, all CVS headers updated and
ready for first customer release.

Revision 1.2  2006/11/30 09:50:19  skau
First working version.

Revision 1.1  2006/11/03 15:11:02  skau
Driver for MB86667.This is the first (non-working) version.

*****************************************************************************/



