/******************************************************************************
**
** \file      ./subsystems/fe/mb86668/src/mb86668_demod.c
**
** \brief     Demodulator driver functions for Fujitsu MB86668 cable frontend.
**
**            (C) Fujitsu Semiconductor Europe 2002 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: mb86668_demod.c 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/
/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include  <stdlib.h>

#include "../src/fe_priv.h"
#include "mb86668_fe_types.h"
#include "mb86668_fe_tools.h"
#include "mb86668_fe_c.h"
#include "mb86668_demod_reg.h"

/*****************************************************
**FUNCTION   ::    DEM_i2cRead
**ACTION     ::    does i2c read access to demod
**PARAMS IN  ::    FEC_device * handle,uint8_t subreg
**PARAMS OUT ::    uint8_t *data   1 Byte
**RETURN     ::    FeErr
*****************************************************/
static FeErr DEM_i2cRead(FEC_device * handle ,uint8_t subreg,uint8_t *data)
{
   handle->Demod.WrBuffer[0]=subreg;
   if(TOOL_i2cWrite(handle->Demod.I2cAddress,handle->Demod.WrBuffer,1)!=FeOk)
      return DEM_I2C_ERR;

   if(TOOL_i2cRead(handle->Demod.I2cAddress,data,1)!=FeOk)
      return DEM_I2C_ERR;
    //FE_DPrintf("DEM_i2cRead(0x%02x)=0x%02x\n",subreg,*data);
   return FeOk;
}


/*****************************************************
**FUNCTION   ::    DEM_i2cWrite
**ACTION     ::    does i2c write access to demod
**PARAMS IN  ::    FEC_device * handle,uint8_t subreg,uint8_t *data   1 Byte
**PARAMS OUT ::    NONE
**RETURN     ::    FeErr
*****************************************************/
static FeErr DEM_i2cWrite(FEC_device * handle,uint8_t subreg,uint8_t data)
{
   handle->Demod.WrBuffer[0]=subreg;
   handle->Demod.WrBuffer[1]=data;
   if(TOOL_i2cWrite(handle->Demod.I2cAddress,handle->Demod.WrBuffer,2)!=FeOk)
      return DEM_I2C_ERR;
    //FE_DPrintf("DEM_i2cWrite(0x%02x):0x%02x\n",subreg,data);
   return FeOk;
}


/*****************************************************
--FUNCTION   ::    DEM_86668_TunerSet
--ACTION     ::    set tuner to frequency
--PARAMS IN  ::    Handle    ==>    Front End Handle
--PARAMS OUT ::    NONE
--RETURN     ::    FeErr (if any)
--***************************************************/
static FeErr DEM_86668_TunerSet(FEC_device * handle)
{
   uint32_t newFrequency;
   FeErr err;
   FE_86668_InternalParams_t* pParams;
   pParams=&(handle->Demod.params);

   if((err=TUN_C_setFrequency(handle,pParams->Frequency,&newFrequency))!=FeOk)
   {
      FE_DPrintf("TUN_setFrequency()=%i\n",err);
      if(err==TUN_I2C_ERR)
         return DEM_I2C_ERR;
      else
         return DEM_TUNING_ERROR;
   }
   /* *Freq=returned_freq; */
   (pParams->Frequency)=newFrequency;
   return FeOk;
}


/*****************************************************
--FUNCTION   ::  DEM_86668_SymbolRateSet
--ACTION     ::  sets symbolrate to demod
--PARAMS IN  ::  Handle  ==>  Front End Handle
--PARAMS OUT ::  NONE
--RETURN     ::  FeErr (if any)
--***************************************************/
static FeErr DEM_86668_SymbolRateSet(FEC_device * handle)
{
   uint8_t deci_val;
   uint32_t d_decimation,s_intpo_nco,d_interpofs1,d_interpofs2;
   uint32_t sr;
   FE_86668_InternalParams_t* pParams;
   pParams=&(handle->Demod.params);
   sr= 10*pParams->SymbolRate;

   if(22000 <= sr && sr < 25000)
   {
      deci_val=2;
      d_decimation=1;
      s_intpo_nco=1000000;
   }
   else if(25000 <= sr && sr <= 26000)
   {
      deci_val=0;
      d_decimation=0;
      s_intpo_nco=600000;
   }
   else if(26000 < sr && sr < 29500)
   {
      deci_val=2;
      d_decimation=1;
      s_intpo_nco=1000000;
   }
   else if(29500 <= sr && sr < 34500)
   {
      deci_val=0;
      d_decimation=0;
      s_intpo_nco=600000;
   }
   else if(34500 <= sr && sr < 35964)
   {
      deci_val=2;
      d_decimation=1;
      s_intpo_nco=1000000;
   }
   else if(35964 <= sr && sr <= 38500)
   {
      deci_val=0;
      d_decimation=0;
      s_intpo_nco=666667;
   }
   else if(38500 < sr && sr < 41000)
   {
      deci_val=0;
      d_decimation=0;
      s_intpo_nco=714286;
   }
   else if(41000 <= sr && sr < 42000)
   {
      deci_val=0;
      d_decimation=0;
      s_intpo_nco=777778;
   }
   else if(42000 <= sr && sr < 45000)
   {
      deci_val=0;
      d_decimation=0;
      s_intpo_nco=787234;
   }
   else if(45000 <= sr && sr <= 47000)
   {
      deci_val=0;
      d_decimation=0;
      s_intpo_nco=818182;
   }
   else if(47000 < sr && sr <= 48000)
   {
      deci_val=0;
      d_decimation=0;
      s_intpo_nco=846154;
   }
   else if(48000 < sr && sr <= 51000)
   {
      deci_val=0;
      d_decimation=0;
      s_intpo_nco=905660;
   }
   else if(51000 < sr && sr <= 53000)
   {
      deci_val=0;
      d_decimation=0;
      s_intpo_nco=943182;
   }
   else if(53000 < sr && sr <= 55000)
   {
      deci_val=0;
      d_decimation=0;
      s_intpo_nco=777778;
   }
   else if(55000 < sr && sr <= 59000)
   {
      deci_val=0;
      d_decimation=0;
      s_intpo_nco=1000000;
   }
   else if(59000 < sr && sr < 67000)
   {
      deci_val=0;
      d_decimation=0;
      s_intpo_nco=943182;
   }
   else if(67000 <= sr && sr <= 69570)
   {
      deci_val=0;
      d_decimation=0;
      s_intpo_nco=1000000;
   }
   else
   {
      FE_DPrintf("DEM_WRONG_PARAMETER: sr=%d\n",sr);
      return DEM_WRONG_PARAMETER;
   }
   pParams->D_decimation=d_decimation;
   pParams->S_intpo_nco=s_intpo_nco;

   d_interpofs1= (uint32_t) ((double)(0xffffff)*(1.0 - (((double)s_intpo_nco)/
                        1000000.0))+0.5);
   FE_DPrintf(" over double: d_interpofs1= 0x%08x\n",d_interpofs1);
   d_interpofs2=  (uint32_t) ((double)(0xffffff *(1.0-(4.0*((double)sr)/10.0*
                         ((double)(1<<d_decimation))/(((double)pParams->Clock)*
                         (((double)s_intpo_nco)/1000000.0)))))+0.5);
   FE_DPrintf(" over double: d_interpofs2= 0x%08x\n",d_interpofs2);

   FE_DPrintf("R_DECI (0x%02x): 0x%02x\n",R_DECI,deci_val);
   if(DEM_i2cWrite(handle,R_DECI,deci_val)!=FeOk)
       return DEM_I2C_ERR;
   FE_DPrintf("R_INTERPOFS11 (0x%02x): 0x%02x\n",R_INTERPOFS11,(uint8_t)(d_interpofs1&0x000000FF));
   if(DEM_i2cWrite(handle,R_INTERPOFS11,(uint8_t)(d_interpofs1&0x000000FF))!=FeOk)
       return DEM_I2C_ERR;
   FE_DPrintf("R_INTERPOFS12 (0x%02x): 0x%02x\n",R_INTERPOFS12,(uint8_t)((d_interpofs1&0x0000FF00)>>8));
   if(DEM_i2cWrite(handle,R_INTERPOFS12,(uint8_t)((d_interpofs1&0x0000FF00)>>8))!=FeOk)
       return DEM_I2C_ERR;
   FE_DPrintf("R_INTERPOFS13 (0x%02x): 0x%02x\n",R_INTERPOFS13,(uint8_t)((d_interpofs1&0x00FF0000)>>16));
   if(DEM_i2cWrite(handle,R_INTERPOFS13,(uint8_t)((d_interpofs1&0x00FF0000)>>16))!=FeOk)
       return DEM_I2C_ERR;
   FE_DPrintf("R_INTERPOFS21 (0x%02x): 0x%02x\n",R_INTERPOFS21,(uint8_t)(d_interpofs2&0x000000FF));
   if(DEM_i2cWrite(handle,R_INTERPOFS21,(uint8_t)(d_interpofs2&0x000000FF))!=FeOk)
       return DEM_I2C_ERR;
   FE_DPrintf("R_INTERPOFS22 (0x%02x): 0x%02x\n",R_INTERPOFS22,(uint8_t)((d_interpofs2&0x0000FF00)>>8));
   if(DEM_i2cWrite(handle,R_INTERPOFS22,(uint8_t)((d_interpofs2&0x0000FF00)>>8))!=FeOk)
       return DEM_I2C_ERR;
   FE_DPrintf("R_INTERPOFS23 (0x%02x): 0x%02x\n",R_INTERPOFS23,(uint8_t)((d_interpofs2&0x00FF0000)>>16));
   if(DEM_i2cWrite(handle,R_INTERPOFS23,(uint8_t)((d_interpofs2&0x00FF0000)>>16))!=FeOk)
       return DEM_I2C_ERR;

   return FeOk;
}


/*****************************************************
--FUNCTION   ::  DEM_86668_QamSet
--ACTION     ::  sets Qam to demod
--PARAMS IN  ::  Handle  ==>  Front End Handle
--PARAMS OUT ::  NONE
--RETURN     ::  FeErr (if any)
--***************************************************/
static FeErr DEM_86668_QamSet(FEC_device * handle)
{
    uint8_t cordeci1_val,cordeci2_val;
    uint8_t leva_val,leva2_val;
    uint8_t devle0_val,qam_val;

    FE_86668_InternalParams_t* pParams;
    pParams=&(handle->Demod.params);

    switch(pParams->Qam)
    {
        case QAM16:
           FE_DPrintf("QAM=QAM16\n");
           cordeci1_val=0xA0;
           cordeci2_val=0xA0;
           leva_val=0x0F;
           leva2_val=0x17;
           devle0_val=0x00;
           qam_val=0x01;
           //REGISTER R_IF_REF_L   0xb2  bb     default
           if(DEM_i2cWrite(handle,R_IF_REF_L,0xbb)!=FeOk)
               return DEM_I2C_ERR;
           break;
        case QAM32:
           FE_DPrintf("QAM=QAM32\n");
           cordeci1_val=0xA0;
           cordeci2_val=0xA0;
           leva_val=0x0F;
           leva2_val=0x13;
           devle0_val=0x00;
           qam_val=0x02;
           //REGISTER R_IF_REF_L   0xb2  bb     default
           if(DEM_i2cWrite(handle,R_IF_REF_L,0xbb)!=FeOk)
               return DEM_I2C_ERR;
           break;
        case QAM64:
           FE_DPrintf("QAM=QAM64\n");
           cordeci1_val=0xA0;
           cordeci2_val=0xA0;
           leva_val=0x17;
           leva2_val=0x1B;
           devle0_val=0x00;
           qam_val=0x03;
           //REGISTER R_IF_REF_L   0xb2  bb     default
           if(DEM_i2cWrite(handle,R_IF_REF_L,0xbb)!=FeOk)
               return DEM_I2C_ERR;
           break;
        case QAM128:
           FE_DPrintf("QAM=QAM128\n");
           cordeci1_val=0xAA;
           cordeci2_val=0xAA;
           leva_val=0x13;
           leva2_val=0x15;
           devle0_val=0x01;
           qam_val=0x04;
           //REGISTER R_IF_REF_L   0xb2  bb     default
           if(DEM_i2cWrite(handle,R_IF_REF_L,0xbb)!=FeOk)
               return DEM_I2C_ERR;
           break;
        case QAM256:
           FE_DPrintf("QAM=QAM256\n");
           cordeci1_val=0xA0;
           cordeci2_val=0xA0;
           leva_val=0x1B;
           leva2_val=0x1D;
           devle0_val=0x01;
           qam_val=0x05;
           //REGISTER R_IF_REF_L   0xb2  bb/69     default
           if(DEM_i2cWrite(handle,R_IF_REF_L,0xbb)!=FeOk)
               return DEM_I2C_ERR;
           break;
        default:
           return DEM_WRONG_PARAMETER;
    }
    FE_DPrintf("R_CORDECI1 (0x%02x): 0x%02x\n",(unsigned char)R_CORDECI1,cordeci1_val);
    if(DEM_i2cWrite(handle,R_CORDECI1,cordeci1_val)!=FeOk)
       return DEM_I2C_ERR;
    FE_DPrintf("R_CORDECI2 (0x%02x): 0x%02x\n",(unsigned char)R_CORDECI2,cordeci2_val);
    if(DEM_i2cWrite(handle,R_CORDECI2,cordeci2_val)!=FeOk)
       return DEM_I2C_ERR;
    FE_DPrintf("R_LEVA (0x%02x): 0x%02x\n",(unsigned char)R_LEVA,leva_val);
    if(DEM_i2cWrite(handle,R_LEVA,leva_val)!=FeOk)
       return DEM_I2C_ERR;
    FE_DPrintf("R_LEVA2 (0x%02x): 0x%02x\n",(unsigned char)R_LEVA2,leva2_val);
    if(DEM_i2cWrite(handle,R_LEVA2,leva2_val)!=FeOk)
       return DEM_I2C_ERR;
    FE_DPrintf("R_DEVLE0 (0x%02x): 0x%02x\n",(unsigned char)R_DEVLE0,devle0_val);
    if(DEM_i2cWrite(handle,R_DEVLE0,devle0_val)!=FeOk)
       return DEM_I2C_ERR;
    FE_DPrintf("R_QAM (0x%02x): 0x%02x\n",(unsigned char)R_QAM,qam_val);
    if(DEM_i2cWrite(handle,R_QAM,qam_val)!=FeOk)
       return DEM_I2C_ERR;
    return FeOk;
}

/*****************************************************
--FUNCTION   ::  DEM_86668_LimiterSet
--ACTION     ::  sets limiters to demod
--PARAMS IN  ::  Handle  ==>  Front End Handle
--PARAMS OUT ::  NONE
--RETURN     ::  FeErr (if any)
--***************************************************/
static FeErr DEM_86668_LimiterSet(FEC_device * handle)
{
    uint8_t cracclim;
    uint16_t stracclim;
    uint32_t sr,deci,clk,s_int;
    FE_86668_InternalParams_t* pParams;
    pParams=&(handle->Demod.params);
    sr=pParams->SymbolRate;
    deci=pParams->D_decimation;
    clk=pParams->Clock;
    s_int=pParams->S_intpo_nco;

    cracclim=(uint8_t)((((double)CARRIER_RECOVERY_LIMITER_FREQ)*
                   ((double)((1<<24)-1)))/(((double)(1<<14))*((double)sr))+0.5);
    FE_DPrintf("R_CRACCLIM (0x%02x): 0x%02x\n",R_CRACCLIM,cracclim);
    if(DEM_i2cWrite(handle,R_CRACCLIM,cracclim)!=FeOk)
       return DEM_I2C_ERR;

    stracclim=(uint16_t)(( ( ((double)SYMBOL_TIMING_RECOVERY_LIMITER_FREQ) * 4.0 * ((double)sr) *
                     ((double)(1<<deci)) * ((double)((1<<24)-1)) ) /
                     ( 16.0* ((double)clk) * ((double)s_int) ))+0.5);
    FE_DPrintf("R_STRACCLIM_L (0x%02x): 0x%02x\n",R_STRACCLIM_L,stracclim);
    if(DEM_i2cWrite(handle,R_STRACCLIM_L,(uint8_t)stracclim)!=FeOk)
       return DEM_I2C_ERR;
    FE_DPrintf("R_STRACCLIM_H (0x%02x): 0x%02x\n",R_STRACCLIM_H,(((uint8_t)(stracclim>>8))& 0x0F));
    if(DEM_i2cWrite(handle,R_STRACCLIM_H,(((uint8_t)(stracclim>>8))& 0x0F))!=FeOk)
       return DEM_I2C_ERR;

    return FeOk;

}

/*****************************************************
--FUNCTION   ::  DEM_86668_softReset
--ACTION     ::  does a soft reset to demod
--PARAMS IN  ::  Handle  ==>  Front End Handle
--PARAMS OUT ::  NONE
--RETURN     ::  FeErr (if any)
--***************************************************/
static FeErr DEM_86668_softReset(FEC_device * handle)
{
    uint16_t val;

    //REGISTER R_QAM_RST   0x95  04
    if(DEM_i2cWrite(handle,R_QAM_RST,0x04)!=FeOk)
       return DEM_I2C_ERR;
    //REGISTER R_STRSW2   0x4F  00 / 03
    if(DEM_i2cWrite(handle,R_STRSW2,0x00)!=FeOk)
       return DEM_I2C_ERR;
    if(DEM_i2cWrite(handle,R_STRSW2,0x03)!=FeOk)
       return DEM_I2C_ERR;

    /* AGC settings  */
    //REGISTER R_AGC_GAIN   0xc5  00
    if(DEM_i2cWrite(handle,R_AGC_GAIN,0x00)!=FeOk)
       return DEM_I2C_ERR;
    //REGISTER R_SINGLE   0xc6  01     2 point control
    if(DEM_i2cWrite(handle,R_SINGLE,0x01)!=FeOk)
       return DEM_I2C_ERR;
    //REGISTER R_AGC_AUTO   0xb1  07     2 point control
    if(DEM_i2cWrite(handle,R_AGC_AUTO,0x07)!=FeOk)
       return DEM_I2C_ERR;

    //REGISTER R_IF_REF_L   0xb2  bb     default
    val=handle->Tuner.IfAgcRef;
    if(DEM_i2cWrite(handle,R_IF_REF_L,(uint8_t)val)!=FeOk)
       return DEM_I2C_ERR;
    //REGISTER R_IF_REF_M   0xb3  00     default
    if(DEM_i2cWrite(handle,R_IF_REF_M,(uint8_t)(val>>8))!=FeOk)
       return DEM_I2C_ERR;

    //REGISTER R_RF_REF_L   0xb4  00     default
    if(DEM_i2cWrite(handle,R_RF_REF_L,0x00)!=FeOk)
       return DEM_I2C_ERR;
    //REGISTER R_RF_REF_M   0xb5  00     default
    if(DEM_i2cWrite(handle,R_RF_REF_M,0x00)!=FeOk)
       return DEM_I2C_ERR;


    // LPF settings
    //REGISTER R_ALPHA2   0xbf  08
    if(DEM_i2cWrite(handle,R_ALPHA2,0x08)!=FeOk)
        return DEM_I2C_ERR;
    //REGISTER R_ALPHA3   0xc0  08
    if(DEM_i2cWrite(handle,R_ALPHA3,0x08)!=FeOk)
        return DEM_I2C_ERR;
    //REGISTER R_FIL23_SFT   0xc2  88
    if(DEM_i2cWrite(handle,R_FIL23_SFT,0x88)!=FeOk)
        return DEM_I2C_ERR;
    // AGC reset
    //REGISTER R_AGCENA   0xb0  00
    if(DEM_i2cWrite(handle,R_AGCENA,0x00)!=FeOk)
        return DEM_I2C_ERR;
    //REGISTER R_AGCENA   0xb0  01
    if(DEM_i2cWrite(handle,R_AGCENA,0x01)!=FeOk)
        return DEM_I2C_ERR;

    return FeOk;
}



/*****************************************************
--FUNCTION   ::  DEM_86668_openRepeater
--ACTION     ::  opens repeater for tuner access
--PARAMS IN  ::  Handle  ==>  Front End Handle
--PARAMS OUT ::  NONE
--RETURN     ::  FeErr (if any)
--***************************************************/
FeErr DEM_86668_openRepeater(FEC_device * handle)
{
    //REGISTER R_TUNERI2C   0x9b  00
    if(DEM_i2cWrite(handle,R_TUNERI2C,0x00)!=FeOk)
       return DEM_I2C_ERR;
    return FeOk;
}

/*****************************************************
--FUNCTION   ::  DEM_86668_closeRepeater
--ACTION     ::  closes i2c repeater to tuner
--PARAMS IN  ::  Handle  ==>  Front End Handle
--PARAMS OUT ::  NONE
--RETURN     ::  FeErr (if any)
--***************************************************/
FeErr DEM_86668_closeRepeater(FEC_device * handle)
{
   // MB86668 detects automatically the stop condition so noting is necessary to do
   return FeOk;
}

/*****************************************************
--FUNCTION   ::  calcAgcVals
--ACTION     ::  calculates AGC limiter vals
--PARAMS IN  ::  short voltage
--PARAMS OUT ::  NONE
--RETURN     ::  result formated
--***************************************************/
static unsigned short calcAgcVals(short voltage)
{
   return ((unsigned short)(((((long)voltage)*1023)/330 -512) & 0x03ff));
}

static void debugAgcVals(char * txt,short in_val)
{
   double erg;
   long val=(long)in_val;
   if((val&0x0200)!=0) // negative result
      val |= 0xfffffc00;
   val += 512;
   val *=330;
   val +=512;
   val /=1023;
   erg= ((double)val) /100.0;
   FE_DPrintf("%s: val: 0x%04x -> Voltage: %f\n",txt,in_val,erg );
}


/*****************************************************
--FUNCTION   ::  DEM_86668_TestDevice
--ACTION     ::  tests if device is available
--PARAMS IN  ::  inside handle
--PARAMS OUT ::  NONE
--RETURN     ::  FeErr
--***************************************************/
FeErr DEM_86668_TestDevice(FEC_device * handle)
{
   uint8_t data;
   if(DEM_i2cRead(handle,R_VERSION,&data)!=FeOk)
      return DEM_NOT_FOUND;
   return FeOk;
}

/*****************************************************
--FUNCTION   ::  DEM_86668_Init
--ACTION     ::  Initialisation of the MB86668 chip
--PARAMS IN  ::  inside handle
--PARAMS OUT ::  NONE
--RETURN     ::  FeErr
--***************************************************/
// presetting of registers after HW reset
// setting of output mode (parallel) and clock (rising/falling edge)

FeErr DEM_86668_Init(FEC_device * handle)
{
    uint32_t if_setting;
    unsigned short val;
    FE_DPrintf("DEM_86668_Init\n");

    handle->Demod.params.Clock=USED_DEMOD_CLOCK;
    /* DEM_86668_Init is called after TUN_C_init.
       In TUN_C_Init Tuner.IF will be set according to the tuner type */
    handle->Demod.params.If_frequency=handle->Tuner.IF;

    // final hw
    handle->Demod.params.OutputSettingsVal=0x01;

    /* ADC init */
    //REGISTER R_ADCINI   0x9f  90
    if(DEM_i2cWrite(handle,R_ADCINI,0x90)!=FeOk)
       return DEM_I2C_ERR;
    //REGISTER R_ADCINI   0x9f  00
    if(DEM_i2cWrite(handle,R_ADCINI,0x00)!=FeOk)
       return DEM_I2C_ERR;

    /* AGC settings  */
    //REGISTER R_SINGLE   0xc6  01     2 point control
    if(DEM_i2cWrite(handle,R_SINGLE,0x01)!=FeOk)
       return DEM_I2C_ERR;
    //REGISTER R_AGC_AUTO   0xb1  07     2 point control
    if(DEM_i2cWrite(handle,R_AGC_AUTO,0x07)!=FeOk)
       return DEM_I2C_ERR;

    //REGISTER R_IF_REF_L   0xb2  bb     default
    val=handle->Tuner.IfAgcRef;
    if(DEM_i2cWrite(handle,R_IF_REF_L,(uint8_t)val)!=FeOk)
       return DEM_I2C_ERR;
    //REGISTER R_IF_REF_M   0xb3  00     default
    if(DEM_i2cWrite(handle,R_IF_REF_M,(uint8_t)(val>>8))!=FeOk)
       return DEM_I2C_ERR;


    //REGISTER R_RF_REF_L   0xb4  00     default
    if(DEM_i2cWrite(handle,R_RF_REF_L,0x00)!=FeOk)
       return DEM_I2C_ERR;
    //REGISTER R_RF_REF_M   0xb5  00     default
    if(DEM_i2cWrite(handle,R_RF_REF_M,0x00)!=FeOk)
       return DEM_I2C_ERR;

    // voltage adaptations: value =(Uin/3,3V)*1023-512
    // calculate R_IF_LIM_L
    val=calcAgcVals(handle->Tuner.IfAgcLimiterLow); // Units= 100 * voltage
    debugAgcVals("R_IF_LIM_L",val);
    //REGISTER R_IF_LIM_L1   0xb6  26     0 V =-512
//debugAgcVals("IF_LIM_L",0x326);
    if(DEM_i2cWrite(handle,R_IF_LIM_L1,(uint8_t)val)!=FeOk)
       return DEM_I2C_ERR;
    //REGISTER R_IF_LIM_L2   0xb7  02     0 V =-512
    if(DEM_i2cWrite(handle,R_IF_LIM_L2,(uint8_t)(val>>8))!=FeOk)
       return DEM_I2C_ERR;

    // calculate R_IF_LIM_H
    val=calcAgcVals(handle->Tuner.IfAgcLimiterHigh); // Units= 100 * voltage
    debugAgcVals("R_IF_LIM_H",val);
    //REGISTER R_IF_LIM_H1   0xb8  A2     3,5V
//debugAgcVals("IF_LIM_H",0x1c2);
    if(DEM_i2cWrite(handle,R_IF_LIM_H1,(uint8_t)val)!=FeOk)
       return DEM_I2C_ERR;
    //REGISTER R_IF_LIM_H2   0xb9  02     3,5V
    if(DEM_i2cWrite(handle,R_IF_LIM_H2,(uint8_t)(val>>8))!=FeOk)
        return DEM_I2C_ERR;

    // calculate R_RF_LIM_L
    val=calcAgcVals(handle->Tuner.RfAgcLimiterLow); // Units= 100 * voltage
    debugAgcVals("R_RF_LIM_L",val);
    //REGISTER R_RF_LIM_L1   0xba  26     0 V =-512
//debugAgcVals("RF_LIM_L",0x302);
    if(DEM_i2cWrite(handle,R_RF_LIM_L1,(uint8_t)val)!=FeOk)
       return DEM_I2C_ERR;
    //REGISTER R_RF_LIM_L2   0xbb  02     0 V =-512
    if(DEM_i2cWrite(handle,R_RF_LIM_L2,(uint8_t)(val>>8))!=FeOk)
       return DEM_I2C_ERR;

    // calculate R_RF_LIM_H
    val=calcAgcVals(handle->Tuner.RfAgcLimiterHigh); // Units= 100 * voltage
    debugAgcVals("R_RF_LIM_H",val);
    //REGISTER R_RF_LIM_H1   0xbc  a2     3,5V
//debugAgcVals("RF_LIM_H",0x1a2);
    if(DEM_i2cWrite(handle,R_RF_LIM_H1,(uint8_t)val)!=FeOk)
       return DEM_I2C_ERR;
    //REGISTER R_RF_LIM_H2   0xbd  02     3,5V
    if(DEM_i2cWrite(handle,R_RF_LIM_H2,(uint8_t)(val>>8))!=FeOk)
        return DEM_I2C_ERR;

    // LPF settings
    //REGISTER R_ALPHA2   0xbf  08
    if(DEM_i2cWrite(handle,R_ALPHA2,0x08)!=FeOk)
        return DEM_I2C_ERR;
    //REGISTER R_ALPHA3   0xc0  08
    if(DEM_i2cWrite(handle,R_ALPHA3,0x08)!=FeOk)
        return DEM_I2C_ERR;
    //REGISTER R_FIL23_SFT   0xc2  88
    if(DEM_i2cWrite(handle,R_FIL23_SFT,0x88)!=FeOk)
        return DEM_I2C_ERR;

    // AGC reset
    //REGISTER R_AGCENA   0xb0  00
    if(DEM_i2cWrite(handle,R_AGCENA,0x00)!=FeOk)
        return DEM_I2C_ERR;
    //REGISTER R_AGCENA   0xb0  01
    if(DEM_i2cWrite(handle,R_AGCENA,0x01)!=FeOk)
        return DEM_I2C_ERR;

    // outout settings
    //REGISTER R_TSOUT   0x88  00 to std parallel mode
    FE_DPrintf("R_TSOUT: 0x%02x\n",handle->Demod.params.OutputSettingsVal);
    if(DEM_i2cWrite(handle,R_TSOUT,handle->Demod.params.OutputSettingsVal)!=FeOk)
        return DEM_I2C_ERR;

    // frame lock settings
    //REGISTER R_FRAMESET   0x8a  F2 to 15x failed=unlock 2x=locked
    if(DEM_i2cWrite(handle,R_FRAMESET,0xF2)!=FeOk)
        return DEM_I2C_ERR;

    // BER settings
    //REGISTER R_BERMSET   0x90  07 in errors/byte   = 1E8 Bit intevall
    //REGISTER R_BERMSET   0x90  01 in errors/byte   = (1E8/8) Bit intevall
    if(DEM_i2cWrite(handle,R_BERMSET,0x07)!=FeOk)
        return DEM_I2C_ERR;

    // IF frequency settings
    FE_DPrintf("If_frequency: %d - Clock: %d\n",handle->Demod.params.If_frequency,handle->Demod.params.Clock);
    if_setting=((32767* (handle->Demod.params.If_frequency - handle->Demod.params.Clock))+
                 (handle->Demod.params.Clock/2))/handle->Demod.params.Clock;

    //REGISTER R_FNCO1 + R_FNCO2
    FE_DPrintf("R_FNCO1 (0x%02x): 0x%02x\n",R_FNCO1,(uint8_t)if_setting);
    if(DEM_i2cWrite(handle,R_FNCO1,(uint8_t)if_setting)!=FeOk)
        return DEM_I2C_ERR;
    FE_DPrintf("R_FNCO2 (0x%02x): 0x%02x\n",R_FNCO2,(uint8_t)(if_setting>>8));
    if(DEM_i2cWrite(handle,R_FNCO2,(uint8_t)(if_setting>>8))!=FeOk)
        return DEM_I2C_ERR;

    return FeOk;
}

/*****************************************************
--FUNCTION   ::  DEM_86668_SearchTransponder
--ACTION     ::  Search for a valid transponder
--PARAMS IN  ::  Handle  ==>  Front End Handle
                 FE_86668_SearchParams_t *pSearch
--PARAMS OUT ::  NONE
--RETURN     ::  FeErr (if any)
--***************************************************/
FeErr DEM_86668_SearchTransponder(FEC_device * handle, FE_86668_SearchParams_t *pSearch)
{
   FeErr ret;
   FE_86668_InternalParams_t* pParams;
   pParams=&(handle->Demod.params);

   pParams->Frequency = pSearch->Frequency;
   pParams->Qam = pSearch->Qam;
   pParams->SymbolRate = pSearch->SymbolRate;

   FE_DPrintf("DEM_86668_TunerSet\n");
   if((ret=DEM_86668_TunerSet(handle))!=FeOk)
      return ret;
   FE_DPrintf("DEM_86668_QamSet\n");
   if((ret=DEM_86668_QamSet(handle))!=FeOk)
      return ret;
   FE_DPrintf("DEM_86668_SymbolRateSet: %d\n",pParams->SymbolRate);
   if((ret=DEM_86668_SymbolRateSet(handle))!=FeOk)
      return ret;
   FE_DPrintf("DEM_86668_LimiterSet\n");
   if((ret=DEM_86668_LimiterSet(handle))!=FeOk)
      return ret;
   FE_DPrintf("DEM_86668_softReset\n");
   if((ret=DEM_86668_softReset(handle))!=FeOk)
      return ret;

   return FeOk;
}


/*****************************************************
**FUNCTION   ::  DEM_8666_GetCarrierOffset
**ACTION     ::  reads and recalculates carrier offset
**PARAMS IN  ::  FEC_device * handle
**PARAMS OUT ::  uint32_t *pOffset in Hz
**RETURN     ::  FeErr
*****************************************************/
FeErr DEM_86668_GetCarrierOffset(FEC_device * handle, uint32_t *pOffset)
{
  uint32_t crm_val,sr;
        uint8_t dat;
        FE_86668_InternalParams_t* pParams;
        pParams=&(handle->Demod.params);
        sr=pParams->SymbolRate;

        crm_val=0;
  if(DEM_i2cRead(handle,R_CRM1,&dat)!=FeOk)
                return DEM_I2C_ERR;
        crm_val |= (uint32_t)dat;
  if(DEM_i2cRead(handle,R_CRM2,&dat)!=FeOk)
                return DEM_I2C_ERR;
        crm_val |= (((uint32_t)dat)<<8);
  if(DEM_i2cRead(handle,R_CRM3,&dat)!=FeOk)
                return DEM_I2C_ERR;
        crm_val |= (((uint32_t)dat)<<16);
        *pOffset=(uint32_t)((((double)crm_val) * ((double)sr))/((double)(1<<24)));
        return FeOk;
}



/*****************************************************
**FUNCTION   ::  DEM_8666_GetBer
**ACTION     ::  calc bit error rate independent of demod settings
                 result in reality: *pBer x 10E-8
**PARAMS IN  ::  FEC_device * handle
**PARAMS OUT ::  uint32_t *pBer
**RETURN     ::  FeErr
*****************************************************/
FeErr DEM_86668_GetBer(FEC_device * handle, uint32_t *pBer)
{
  uint8_t dat;

        *pBer=0;
  if(DEM_i2cRead(handle,R_BER_LSB,&dat)!=FeOk)
                return DEM_I2C_ERR;
        *pBer |= (uint32_t)dat;
  if(DEM_i2cRead(handle,R_BER_MID,&dat)!=FeOk)
                return DEM_I2C_ERR;
        *pBer |= (((uint32_t)dat)<<8);
  if(DEM_i2cRead(handle,R_BER_MSB,&dat)!=FeOk)
                return DEM_I2C_ERR;
        *pBer |= (((uint32_t)(dat & 0xF))<<16);
    FE_DPrintf("DEM_86668_GetBer %d\n",*pBer);
        return FeOk;
}


/*****************************************************
**FUNCTION   ::  DEM_8666_GetCN
**ACTION     ::  gets C/N value
**PARAMS IN  ::  FEC_device * handle
**PARAMS OUT ::  uint32_t *pNoise
**RETURN     ::  FeErr
*****************************************************/
FeErr DEM_86668_GetCN(FEC_device * handle,uint32_t *pNoise)
{
  uint8_t dat;

   *pNoise=0;
   if(DEM_i2cRead(handle,R_CNMON1,&dat)!=FeOk)
      return DEM_I2C_ERR;
   *pNoise |= (uint32_t)dat;
   if(DEM_i2cRead(handle,R_CNMON2,&dat)!=FeOk)
      return DEM_I2C_ERR;
   *pNoise |= (((uint32_t)(dat & 0x3F))<<8);
   #ifdef FD_FE_MB86668_DEBUG02
   TOOL_Printf("DEM_86668_GetCN %d\n",*pNoise);
   #endif
   return FeOk;
}



/*****************************************************
**FUNCTION   ::  DEM_86668_GetLock
**ACTION     ::  checks for lock condition
**PARAMS IN  ::  FEC_device * handle
**PARAMS OUT ::  uint8_t *status 1=locked / 0=unlocked   1 Byte
**RETURN     ::  FeErr
*****************************************************/
FeErr DEM_86668_GetLock(FEC_device * handle,uint8_t *status)
{
    uint8_t dat;

    if(DEM_i2cRead(handle,R_FSYNC,&dat)!=FeOk)
        return DEM_I2C_ERR;

    if((dat & BIT0)==BIT0) //locked
    {
        *status=1;
    }
    else
    {
        FeErr ret;
        *status=0;
        if((ret=DEM_86668_softReset(handle))!=FeOk)
            return ret;
    }
    FE_DPrintf("DEM_86668_GetLock %d\n",*status);
    return FeOk;
}



/*****************************************************
**FUNCTION   ::  DEM_86668_GetAgc
**ACTION     ::  gets AGC value
**PARAMS IN  ::  FEC_device * handle
**PARAMS OUT ::  uint16_t *pAgc   2 Bytes
**RETURN     ::  FeErr
*****************************************************/
FeErr DEM_86668_GetAgc(FEC_device * handle,uint16_t *pAgc)
{
    uint8_t dat;

    *pAgc = 0;
    // AGC low value
    if(DEM_i2cRead(handle,R_RF_AGC_MON_L,&dat)!=FeOk)
        return DEM_I2C_ERR;
    *pAgc |= (uint16_t) dat;

    // AGC high value
    if(DEM_i2cRead(handle,R_RF_AGC_MON_H,&dat)!=FeOk)
        return DEM_I2C_ERR;
    *pAgc |= (((uint16_t) dat)<<8);
    *pAgc &= 0x3ff;
    #ifdef FD_FE_MB86668_DEBUG02
    TOOL_Printf("DEM_86668_GetAgc %d\n",*pAgc);
    #endif
    return FeOk;
}

/* Version History */
/******************************************************************************

$Log: mb86668_demod.c,v $
Revision 1.1  2008/05/07 11:10:19  wlaris
Initial creation.

Revision 1.1  2008/05/07 08:39:07  wlaris
Initial creation.

Revision 1.7  2007/11/21 11:35:42  skau
Supporting tuners with different IF frequencies now.

Revision 1.6  2007/10/12 09:26:57  skau
DEBUGOUT() prints encapsulated by #define MB86668_DEBUG_PRINTS.

Revision 1.5  2007/07/13 09:12:27  skau
Get AGC function optimised for level bar graph display.
DCT70701 tuner support added.

Revision 1.4  2007/07/11 14:26:10  skau
FD_GHW support added and some namings changed, further clean up.

Revision 1.3  2007/01/31 06:43:23  shelle
Copyright updated

Revision 1.2  2007/01/25 13:48:51  skau
Removed a few test routines.

Revision 1.1  2007/01/25 13:44:27  skau
First version of MB86668 driver in CVS.

Revision 1.0  2007/1/24 21:15:29  skau
First version in CVS

******************************************************************************/
