/******************************************************************************
**
** \file      ./subsystems/fe/mb86668/src/mb86668_tuner_c.c
**
** \brief     Tuner driver for Philips tuner.
**
**            (C) Fujitsu Semiconductor Europe 2002 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: mb86668_tuner_c.c 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/  
#include "../src/fe_priv.h"
#include "mb86668_fe_types.h"
#include "mb86668_fe_c.h"
#include "mb86668_fe_tools.h"

#define MAX_TRIES 10

extern FeErr DEM_86668_openRepeater(FEC_device * handle);
extern FeErr DEM_86668_closeRepeater(FEC_device * handle);

/*-----------------------------------------------------------------------------
   function:    FeErr TUN_C_i2cWrite(FEC_device * handle)
   description: encapsulated i2c write access
   input:       FEC_device * handle
   output:      nothing
   return:      FeErr
-----------------------------------------------------------------------------*/
static FeErr TUN_C_i2cWrite(FEC_device * handle)
{
   FeErr err;

   //debug
   {
      uint8_t i;
      for(i=0;i<handle->Tuner.WrSize;i++)
      {
         FE_DPrintf("0x%02x ",handle->Tuner.WrBuffer[i]);
      }
      FE_DPrintf("\n");
   }

   // open repeater
   if((err=DEM_86668_openRepeater(handle))!=FeOk)
   {
      return TUN_I2C_ERR;
   }
   // write tuner
   if((err=TOOL_i2cWrite(handle->Tuner.I2cAddress,handle->Tuner.WrBuffer,handle->Tuner.WrSize))!=FeOk)
   {
      return TUN_I2C_ERR;
   }
   // close repeater
   err=DEM_86668_closeRepeater(handle);
   return err;
}

/*-----------------------------------------------------------------------------
   function:    FeErr TUN_C_i2cRead(FEC_device * handle)
   description: encapsulated i2c read access
   input:       FEC_device * handle
   output:      nothing
   return:      FeErr
-----------------------------------------------------------------------------*/
static FeErr TUN_C_i2cRead(FEC_device * handle)
{
   FeErr err;

   // open repeater
   if((err=DEM_86668_openRepeater(handle))!=FeOk)
   {
      FE_DPrintf("DEM_86668_openRepeater() failed\n");
      return TUN_I2C_ERR;
   }
   // read from tuner
   if((err=TOOL_i2cRead(handle->Tuner.I2cAddress,handle->Tuner.RdBuffer,handle->Tuner.RdSize))!=FeOk)
   {
      FE_DPrintf("TOOL_i2cRead() failed\n");
      return TUN_I2C_ERR;
   }
   // close repeater
   err=DEM_86668_closeRepeater(handle);
   return err;
}

/*****************************************************
--FUNCTION   ::  TUN_C_TestDevice
--ACTION     ::  tests if device is available
--PARAMS IN  ::  inside handle
--PARAMS OUT ::  NONE
--RETURN     ::  FeErr
--***************************************************/
FeErr TUN_C_TestDevice(FEC_device * handle)
{
   handle->Tuner.RdSize=1;
   if(TUN_C_i2cRead(handle)!=FeOk)
      return TUN_NOT_FOUND;
   return FeOk;
}

/*-----------------------------------------------------------------------------
   function:    FeErr TUN_C_init(FEC_device * handle)
   description: initialize tuner
   input:       FEC_device * handle
   output:      nothing
   return:      FeErr
-----------------------------------------------------------------------------*/
FeErr TUN_C_init(FEC_device * handle)
{
   FeErr err=FeOk;
   switch(handle->Tuner.Type)
   {
      case CD1316L_3:
      case CD1600L_3:
         FE_DPrintf("Philips CD1316/CD1600 selected\n");
         handle->Tuner.I_Q = 1;
         handle->Tuner.StepSize = 62500L;
         handle->Tuner.IF = 36125;
         handle->Tuner.BandWidth =  8000L;  // in kHz

         handle->Tuner.IfAgcLimiterLow=80;
         handle->Tuner.IfAgcLimiterHigh=320;
         handle->Tuner.RfAgcLimiterLow=70;
         handle->Tuner.RfAgcLimiterHigh=320;
         handle->Tuner.IfAgcRef=0x00bb;

         err=FeOk;
         break;
      case DCT70602:
         FE_DPrintf("Thomson DCT70602 selected\n");
         handle->Tuner.I_Q = 1;
         handle->Tuner.StepSize = 62500L;
         handle->Tuner.IF = 36000;
         handle->Tuner.BandWidth =  8000L;  // in kHz

         handle->Tuner.IfAgcLimiterLow=170;
         handle->Tuner.IfAgcLimiterHigh=320;
         handle->Tuner.RfAgcLimiterLow=70;
         handle->Tuner.RfAgcLimiterHigh=320;
         handle->Tuner.IfAgcRef=0x00bb;

         err=FeOk;
         break;
      case DCT70701:
         FE_DPrintf("Thomson DCT70701 selected\n");
         handle->Tuner.I_Q = 1;
         handle->Tuner.StepSize = 62500L;
         handle->Tuner.IF = 36000;
         handle->Tuner.BandWidth =  8000L;  // in kHz

         handle->Tuner.IfAgcLimiterLow=170;
         handle->Tuner.IfAgcLimiterHigh=320;
         handle->Tuner.RfAgcLimiterLow=70;
         handle->Tuner.RfAgcLimiterHigh=320;
         handle->Tuner.IfAgcRef=0x00bb;

         err=FeOk;
         break;
      default:
         FE_DPrintf("No tuner selected\n");
         err=TUN_WRONG_PARAMETER;
         break;
   }
   return err;
}


/*-----------------------------------------------------------------------------
   function:    FeErr TUN_C_setFrequency(FEC_device * handle,uint32_t frequency)
   description: sets tuner to frequency
   input:       FEC_device * handle
                uint32_t frequency in kHz!
   output:      uint32_t *newFrequency  in kHz!
   return:      FeErr
-----------------------------------------------------------------------------*/
FeErr TUN_C_setFrequency(FEC_device *handle, uint32_t frequency,uint32_t *newFrequency)
{
   FeErr err;
   uint8_t bandflag,rflag;
   int32_t nbsteps;

   // stepwidth
   switch(handle->Tuner.StepSize)
   {
      default:
      case 62500L:
         rflag=STEPWIDTH62500;
         break;
   }

   // calc osz. frequency
   frequency += handle->Tuner.IF;

   switch(handle->Tuner.Type)
   {
      case CD1316L_3:
      case CD1600L_3:
         // bandwidth: here all times 8MHz

         if (frequency < 49000+handle->Tuner.IF)
            return TUN_WRONG_PARAMETER;

         else if (frequency < 167000+handle->Tuner.IF)
         {
            bandflag = 0x01;              // low band osz.
         }
         else if (frequency < 454000+handle->Tuner.IF)
         {
            bandflag = 0x02;            // mid band osz.
         }
         else if (frequency <= 860000+handle->Tuner.IF)
         {
            bandflag = 0x04;           // high band osz.
         }
         else
            return TUN_WRONG_PARAMETER;

        // calc divider
        nbsteps = (frequency * 2000 + handle->Tuner.StepSize)/(handle->Tuner.StepSize*2);
        handle->Tuner.WrBuffer[0] = (uint8_t)((nbsteps >> 8) & 0x7f) ;
        handle->Tuner.WrBuffer[1] = (uint8_t)((nbsteps >> 0) & 0xff);

        // set control
        handle->Tuner.WrBuffer[2] = BIT7|BIT6|rflag;
        // set switchport
        handle->Tuner.WrBuffer[3] = bandflag| AGC_MEDIUM_2; // RF-AGC TOP medium2

        // send divider + control + switchport
        handle->Tuner.WrSize=4;
        err = TUN_C_i2cWrite(handle);
        if (err != FeOk)
           return err;

        *newFrequency = ((nbsteps*(handle->Tuner.StepSize/2)) - 500*handle->Tuner.IF)/500;
      break;

      case DCT70602:
         // bandwidth: here all times 8MHz

         if (frequency < 49000+handle->Tuner.IF)
            return TUN_WRONG_PARAMETER;

         else if (frequency < 167000+handle->Tuner.IF)
         {
            bandflag = 0x01;            // low band osz.
         }
         else if (frequency < 454000+handle->Tuner.IF)
         {
            bandflag = 0x02;            // mid band osz.
         }
         else if (frequency <= 860000+handle->Tuner.IF)
         {
            bandflag = 0x08;      // high band osz.
         }
                 else
            return TUN_WRONG_PARAMETER;

        // calc divider
        nbsteps = (frequency * 2000 + handle->Tuner.StepSize)/(handle->Tuner.StepSize*2);
        handle->Tuner.WrBuffer[0] = (uint8_t)((nbsteps >> 8) & 0x7f) ;
        handle->Tuner.WrBuffer[1] = (uint8_t)((nbsteps >> 0) & 0xff);

        // set control
        handle->Tuner.WrBuffer[2] = BIT7|rflag;
        // set switchport
          handle->Tuner.WrBuffer[3] = bandflag;

        // send divider + control + switchport
        handle->Tuner.WrSize=4;
        err = TUN_C_i2cWrite(handle);
        if (err != FeOk)
           return err;

        *newFrequency = ((nbsteps*(handle->Tuner.StepSize/2)) - 500*handle->Tuner.IF)/500;
      break;
      case DCT70701:
         // bandwidth: here all times 8MHz

         if (frequency < 49000+handle->Tuner.IF)
            return TUN_WRONG_PARAMETER;

         else if (frequency < 149000+handle->Tuner.IF)
         {
            bandflag = 0x01;            // low band osz.
         }
         else if (frequency < 429000+handle->Tuner.IF)
         {
            bandflag = 0x06;            // mid band osz.
         }
         else if (frequency <= 860000+handle->Tuner.IF)
         {
            bandflag = 0x0C;            // high band osz.
         }
         else
            return TUN_WRONG_PARAMETER;

        // calc divider
        nbsteps = (frequency * 2000 + handle->Tuner.StepSize)/(handle->Tuner.StepSize*2);
        handle->Tuner.WrBuffer[0] = (uint8_t)((nbsteps >> 8) & 0x7f) ;
        handle->Tuner.WrBuffer[1] = (uint8_t)((nbsteps >> 0) & 0xff);

        // set control
        handle->Tuner.WrBuffer[2] = BIT7|BIT5|BIT4|BIT3| (BIT1|BIT0);
        // set switchport
        handle->Tuner.WrBuffer[3] = bandflag;

        handle->Tuner.WrBuffer[4] = 0xC3;

        // send divider + control + switchport
        handle->Tuner.WrSize = 5;
        err = TUN_C_i2cWrite(handle);
        if (err != FeOk)
           return err;

        *newFrequency = ((nbsteps*(handle->Tuner.StepSize/2)) - 500*handle->Tuner.IF)/500;
      break;

      default:
        err=TUN_WRONG_PARAMETER;
   }
   return err;
}

/* Version History */
/******************************************************************************

$Log: mb86668_tuner_c.c,v $
Revision 1.1  2008/05/07 11:10:19  wlaris
Initial creation.

Revision 1.1  2008/05/07 08:39:07  wlaris
Initial creation.


******************************************************************************/
