/*==============================================================================
    FILE NAME   : FC2581_driver_v1411.c
    
    VERSION     : 1.411
    
    UPDATE      : October 11. 2010

    Application  : ISDB-T

==============================================================================*/ 

/*==============================================================================

  Chip ID of fc2581 is 0x56 or 0xAC(including I2C write bit)

==============================================================================*/
#include "../MB86A22_user_define.h"

#include "FC2581_driver_v1411.h"

static fc2581_band_type curr_band = NO_BAND;
static unsigned char data_0x02 = (USE_EXT_CLK<<5)|0x0E;

/*==============================================================================

           fc2581 i2c write

  This function is a generic function which write a byte into fc2581's
 specific address.

  <input parameter>
  tunerAddr
    type: unsigned char
    tuner I2C address
    
  addr
	fc2581's memory address
	type : byte

  data
	target data
	type : byte

==============================================================================*/
void fc2581_i2c_write(unsigned char tunerAddr, unsigned char addr, unsigned char data)
{
    MB86A22_silicon_tuner_write_in_bytes(tunerAddr, addr, 1, &data);
}

/*==============================================================================

           fc2581 i2c read

  This function is a generic function which gets called to read data from
 fc2581's target memory address.

  <input parameter>
  tunerAddr
    type: unsigned char
    tuner I2C address
    
  addr
	fc2581's memory address
	type : byte


  <return value>
  data
	a byte of data read out of target address 'addr'
	type : byte

==============================================================================*/
unsigned char fc2581_i2c_read(unsigned char tunerAddr, unsigned char addr)
{
    unsigned char data;
    
    MB86A22_silicon_tuner_read_in_bytes(tunerAddr, addr, 1, &data);
    
    return data;
}

/*==============================================================================
       fc2581 I2C Test

  This function is a generic function which tests I2C interface's availability

  by reading out it's I2C id data from reg. address '0x01'.

  <input parameter>

  tunerAddr
    type: unsigned char
    tuner I2C address
  
  <return value>
  int
	1 : success - communication is avilable
	0 : fail - communication is unavailable
  

==============================================================================*/
int fc2581_i2c_test(unsigned char tunerAddr)
{
	return (fc2581_i2c_read(tunerAddr, 0x01) == 0x56)? 0x01 : 0x00;
}

/*==============================================================================
       fc2581 initial setting

  This function is a generic function which gets called to initialize

  fc2581 in DVB-H mode or L-Band TDMB mode

  <input parameter>
  tunerAddr
    type: unsigned char
    tuner I2C address
    
  ifagc_mode
    type : integer
	1 : Internal AGC
	2 : Voltage Control Mode

==============================================================================*/
void fc2581_set_init(unsigned char tunerAddr, int ifagc_mode)
{
	fc2581_i2c_write(tunerAddr, 0x00, 0x00);	/*** Confidential ***/
	fc2581_i2c_write(tunerAddr, 0x05, 0xF8);	//BS STDBY dependency
	fc2581_i2c_write(tunerAddr, 0x08, 0x03);	//VCO BOOT time
	fc2581_i2c_write(tunerAddr, 0x12, 0x86);
	fc2581_i2c_write(tunerAddr, 0x14, 0x5C);
	fc2581_i2c_write(tunerAddr, 0x15, 0x9F);	//BS Nclose loop
	fc2581_i2c_write(tunerAddr, 0x16, 0x3C);
	fc2581_i2c_write(tunerAddr, 0x1F, 0xD2);
	fc2581_i2c_write(tunerAddr, 0x09, 0xD7);
	fc2581_i2c_write(tunerAddr, 0x0B, 0xD5);
	fc2581_i2c_write(tunerAddr, 0x0C, 0x32);
	fc2581_i2c_write(tunerAddr, 0x0E, 0x43);
	fc2581_i2c_write(tunerAddr, 0x21, 0x0A);
	fc2581_i2c_write(tunerAddr, 0x22, 0x82);
	fc2581_i2c_write(tunerAddr, 0x23, 0xFF);	//STDBY bank hold time
	if( ifagc_mode == 1 )
	{
		fc2581_i2c_write(tunerAddr, 0x45, 0x10);	//internal AGC
		fc2581_i2c_write(tunerAddr, 0x4C, 0x00);	//HOLD_AGC polarity
	}
	else if( ifagc_mode == 2 )
	{
		fc2581_i2c_write(tunerAddr, 0x45, 0x20);	//Voltage Control Mode
		fc2581_i2c_write(tunerAddr, 0x4C, 0x00);	//HOLD_AGC polarity
	}
	else if( ifagc_mode == 3 )
	{
		fc2581_i2c_write(tunerAddr, 0x45, 0x30);	//Up/Down Control ( Digital AGC )
		fc2581_i2c_write(tunerAddr, 0x4C, 0x02);	//HOLD_AGC polarity
	}
	fc2581_i2c_write(tunerAddr, 0x3F, 0x88);
	fc2581_i2c_write(tunerAddr, 0x02, 0x0E);
	fc2581_i2c_write(tunerAddr, 0x58, 0x14);
	fc2581_set_filter(tunerAddr, 10);	//ISDB-T

}

/*==============================================================================
       fc2581 frequency setting

  This function is a generic function which gets called to change LO Frequency

  of fc2581 in DVB-H mode or L-Band TDMB mode

  <input parameter>
  tunerAddr
    type: unsigned char
    tuner I2C address
    
  f_lo
	Value of target LO Frequency in 'kHz' unit
	ex) 2.6GHz = 2600000

==============================================================================*/
void fc2581_set_freq(unsigned char tunerAddr, unsigned int f_lo)
{
	unsigned int f_diff, f_diff_shifted, n_val, k_val;
	unsigned int f_vco, r_val, f_comp;
	unsigned char pre_shift_bits = 4;// number of preshift to prevent overflow in shifting f_diff to f_diff_shifted
	unsigned char data_0x18;

	fc2581_band_type band = ( f_lo > 1000000 )? L_BAND : ( f_lo > 400000 )? UHF_BAND : VHF_BAND;

	f_vco = ( band == UHF_BAND )? f_lo * 4 : (( band == L_BAND )? f_lo * 2 : f_lo * 12);
	r_val = ( f_vco >= 2*76*FREQ_XTAL )? 1 : ( f_vco >= 76*FREQ_XTAL )? 2 : 4;
	f_comp = FREQ_XTAL/r_val;
	n_val =	( f_vco / 2 ) / f_comp;
	
	f_diff = f_vco - 2* f_comp * n_val;
	f_diff_shifted = f_diff << ( 20 - pre_shift_bits );
	k_val = f_diff_shifted / ( ( 2* f_comp ) >> pre_shift_bits );
	
	if( f_diff_shifted - k_val * ( ( 2* f_comp ) >> pre_shift_bits ) >= ( f_comp >> pre_shift_bits ) )
	k_val = k_val + 1;
	
	if( f_vco >= BORDER_FREQ )	//Select VCO Band
		data_0x02 = data_0x02 | 0x08;	//0x02[3] = 1;
	else
		data_0x02 = data_0x02 & 0xF7;	//0x02[3] = 0;
	
	if( band != curr_band ) {
		switch(band)
		{
			case UHF_BAND:
				data_0x02 = (data_0x02 & 0x3F);

				fc2581_i2c_write(tunerAddr, 0x25, 0xF0);
				fc2581_i2c_write(tunerAddr, 0x27, 0x77);
				fc2581_i2c_write(tunerAddr, 0x28, 0x53);
				fc2581_i2c_write(tunerAddr, 0x29, 0x60);
				fc2581_i2c_write(tunerAddr, 0x30, 0x09);
				fc2581_i2c_write(tunerAddr, 0x50, 0x8C);
				fc2581_i2c_write(tunerAddr, 0x53, 0x50);
				
				fc2581_i2c_write(tunerAddr, 0x67, 0x03); // for ISDB-T fullseg ACS
				fc2581_i2c_write(tunerAddr, 0x68, 0x05); // for ISDB-T fullseg ACS


				if( f_lo < 538000 )
					fc2581_i2c_write(tunerAddr, 0x5F, 0x13);
				else					
					fc2581_i2c_write(tunerAddr, 0x5F, 0x15);

				if( f_lo < 538000 )
				{
					fc2581_i2c_write(tunerAddr, 0x61, 0x07);
					fc2581_i2c_write(tunerAddr, 0x62, 0x06);
					//fc2581_i2c_write(0x67, 0x06);
					//fc2581_i2c_write(0x68, 0x08);
					fc2581_i2c_write(tunerAddr, 0x69, 0x10);
					fc2581_i2c_write(tunerAddr, 0x6A, 0x12);
				}
				else if( f_lo < 794000 )
				{
					fc2581_i2c_write(tunerAddr, 0x61, 0x03);
					fc2581_i2c_write(tunerAddr, 0x62, 0x03);
					//fc2581_i2c_write(0x67, 0x07);
					//fc2581_i2c_write(0x68, 0x08);
					fc2581_i2c_write(tunerAddr, 0x69, 0x0C);
					fc2581_i2c_write(tunerAddr, 0x6A, 0x0E);
				}
				else
				{
					fc2581_i2c_write(tunerAddr, 0x61, 0x07);
					fc2581_i2c_write(tunerAddr, 0x62, 0x06);
					//fc2581_i2c_write(0x67, 0x07);
					//fc2581_i2c_write(0x68, 0x09);
					fc2581_i2c_write(tunerAddr, 0x69, 0x10);
					fc2581_i2c_write(tunerAddr, 0x6A, 0x12);
				}

				fc2581_i2c_write(tunerAddr, 0x63, 0x15);

				fc2581_i2c_write(tunerAddr, 0x6B, 0x0B);
				fc2581_i2c_write(tunerAddr, 0x6C, 0x0C);
				fc2581_i2c_write(tunerAddr, 0x6D, 0x78);
				fc2581_i2c_write(tunerAddr, 0x6E, 0x32);
				fc2581_i2c_write(tunerAddr, 0x6F, 0x14);
				fc2581_set_filter(tunerAddr, 10);	//BW = ISDB-T full seg
				break;
			case VHF_BAND:
				data_0x02 = (data_0x02 & 0x3F) | 0x80;
				fc2581_i2c_write(tunerAddr, 0x27, 0x77);
				fc2581_i2c_write(tunerAddr, 0x28, 0x33);
				fc2581_i2c_write(tunerAddr, 0x29, 0x40);
				fc2581_i2c_write(tunerAddr, 0x30, 0x09);
				fc2581_i2c_write(tunerAddr, 0x50, 0x8C);
				fc2581_i2c_write(tunerAddr, 0x53, 0x50);
				fc2581_i2c_write(tunerAddr, 0x5F, 0x0F);
				fc2581_i2c_write(tunerAddr, 0x61, 0x07);
				fc2581_i2c_write(tunerAddr, 0x62, 0x00);
				fc2581_i2c_write(tunerAddr, 0x63, 0x15);
				fc2581_i2c_write(tunerAddr, 0x67, 0x03);
				fc2581_i2c_write(tunerAddr, 0x68, 0x05);
				fc2581_i2c_write(tunerAddr, 0x69, 0x10);
				fc2581_i2c_write(tunerAddr, 0x6A, 0x12);
				fc2581_i2c_write(tunerAddr, 0x6B, 0x08);
				fc2581_i2c_write(tunerAddr, 0x6C, 0x0A);
				fc2581_i2c_write(tunerAddr, 0x6D, 0x78);
				fc2581_i2c_write(tunerAddr, 0x6E, 0x32);
				fc2581_i2c_write(tunerAddr, 0x6F, 0x54);
				fc2581_set_filter(tunerAddr, 10);	//BW = ISDB-T full seg
				break;
			case L_BAND:
				data_0x02 = (data_0x02 & 0x3F) | 0x40;
				fc2581_i2c_write(tunerAddr, 0x2B, 0x70);
				fc2581_i2c_write(tunerAddr, 0x2C, 0x37);
				fc2581_i2c_write(tunerAddr, 0x2D, 0xE7);
				fc2581_i2c_write(tunerAddr, 0x30, 0x09);
				fc2581_i2c_write(tunerAddr, 0x44, 0x20);
				fc2581_i2c_write(tunerAddr, 0x50, 0x8C);
				fc2581_i2c_write(tunerAddr, 0x53, 0x50);
				fc2581_i2c_write(tunerAddr, 0x5F, 0x0F);
				fc2581_i2c_write(tunerAddr, 0x61, 0x0F);
				fc2581_i2c_write(tunerAddr, 0x62, 0x00);
				fc2581_i2c_write(tunerAddr, 0x63, 0x13);
				fc2581_i2c_write(tunerAddr, 0x67, 0x00);
				fc2581_i2c_write(tunerAddr, 0x68, 0x02);
				fc2581_i2c_write(tunerAddr, 0x69, 0x0C);
				fc2581_i2c_write(tunerAddr, 0x6A, 0x0E);
				fc2581_i2c_write(tunerAddr, 0x6B, 0x08);
				fc2581_i2c_write(tunerAddr, 0x6C, 0x0A);
				fc2581_i2c_write(tunerAddr, 0x6D, 0xA0);
				fc2581_i2c_write(tunerAddr, 0x6E, 0x50);
				fc2581_i2c_write(tunerAddr, 0x6F, 0x14);
				fc2581_set_filter(tunerAddr, 1);	//BW = 1.53MHz
				break;
			default:
				break;
		}
		curr_band = band;
	}

	//A command about AGC clock's pre-divide ratio
	if( FREQ_XTAL >= 28000 )
		fc2581_i2c_write(tunerAddr, 0x4B, 0x22);

	//Commands about VCO Band and PLL setting.
	fc2581_i2c_write(tunerAddr, 0x02, data_0x02);
	data_0x18 = ( ( r_val == 1 )? 0x00 : ( ( r_val == 2 )? 0x10 : 0x20 ) ) + (unsigned char)(k_val >> 16);
	fc2581_i2c_write(tunerAddr, 0x18, data_0x18);						//Load 'R' value and high part of 'K' values
	fc2581_i2c_write(tunerAddr, 0x1A, (unsigned char)( k_val >> 8 ) );	//Load middle part of 'K' value
	fc2581_i2c_write(tunerAddr, 0x1B, (unsigned char)( k_val ) );		//Load lower part of 'K' value
	fc2581_i2c_write(tunerAddr, 0x1C, (unsigned char)( n_val ) );		//Load 'N' value

	//A command about UHF LNA Load Cap
	if( band == UHF_BAND )
		fc2581_i2c_write(tunerAddr, 0x2D, ( f_lo <= (unsigned int)794000 )? 0x9F : 0x8F );	//LNA_OUT_CAP


}


/*==============================================================================
       fc2581 filter BW setting

  This function is a generic function which gets called to change Bandwidth

  frequency of fc2581's channel selection filter

  <input parameter>
  tunerAddr
    type: unsigned char
    tuner I2C address
    
  filter_bw
    1 : 1.53MHz(TDMB)
	5 : 5MHz
	6 : 6MHz
	7 : 6.8MHz
	8 : 7.8MHz
	9 : 8.0MHz
	10 : ISDB-T, full-seg
	11 : ISDB-T, 1-seg
	13 : ISDB-T, 3-seg

==============================================================================*/
void fc2581_set_filter(unsigned char tunerAddr, unsigned char filter_bw)
{
	unsigned char	cal_mon, i;

	if(filter_bw == 1)
	{
		fc2581_i2c_write(tunerAddr, 0x36, 0x1C);
		fc2581_i2c_write(tunerAddr, 0x37, (unsigned char)(4.151*FREQ_XTAL/1000) );
		fc2581_i2c_write(tunerAddr, 0x39, 0x00);
		fc2581_i2c_write(tunerAddr, 0x2E, 0x09);
	}
	if(filter_bw == 5)
	{
		fc2581_i2c_write(tunerAddr, 0x36, 0x18);
		fc2581_i2c_write(tunerAddr, 0x37, (unsigned char)(5.32*FREQ_XTAL/1000) );
		fc2581_i2c_write(tunerAddr, 0x39, 0x00);
		fc2581_i2c_write(tunerAddr, 0x2E, 0x09);
	}
	else if(filter_bw == 6)
	{
		fc2581_i2c_write(tunerAddr, 0x36, 0x18);
		fc2581_i2c_write(tunerAddr, 0x37, (unsigned char)(4.4*FREQ_XTAL/1000) );
		fc2581_i2c_write(tunerAddr, 0x39, 0x00);
		fc2581_i2c_write(tunerAddr, 0x2E, 0x09);
	}
	else if(filter_bw == 7)
	{
		fc2581_i2c_write(tunerAddr, 0x36, 0x18);
		fc2581_i2c_write(tunerAddr, 0x37, (unsigned char)(3.91*FREQ_XTAL/1000) );
		fc2581_i2c_write(tunerAddr, 0x39, 0x80);
		fc2581_i2c_write(tunerAddr, 0x2E, 0x09);
	}
	else if(filter_bw == 8)
	{
		fc2581_i2c_write(tunerAddr, 0x36, 0x18);
		fc2581_i2c_write(tunerAddr, 0x37, (unsigned char)(3.3*FREQ_XTAL/1000) );
		fc2581_i2c_write(tunerAddr, 0x39, 0x80);
		fc2581_i2c_write(tunerAddr, 0x2E, 0x09);
	}
	else if(filter_bw == 9)
	{
		fc2581_i2c_write(tunerAddr, 0x36, 0x18);
		fc2581_i2c_write(tunerAddr, 0x37, (unsigned char)(2.45*FREQ_XTAL/1000) );
		fc2581_i2c_write(tunerAddr, 0x39, 0x80);
		fc2581_i2c_write(tunerAddr, 0x2E, 0x09);
	}

	else if(filter_bw == 10) // for ISDB-T
	{
		fc2581_i2c_write(tunerAddr, 0x36, 0x18);
		fc2581_i2c_write(tunerAddr, 0x37, (unsigned char)(4.69*FREQ_XTAL/1000) );
		fc2581_i2c_write(tunerAddr, 0x39, 0x80);
		fc2581_i2c_write(tunerAddr, 0x2E, 0x09);
	}
	else if(filter_bw == 11) // for ISDB-T
	{
		fc2581_i2c_write(tunerAddr, 0x36, 0x1C);
		fc2581_i2c_write(tunerAddr, 0x37, (unsigned char)(5.5*FREQ_XTAL/1000) );
		fc2581_i2c_write(tunerAddr, 0x39, 0x00);
		fc2581_i2c_write(tunerAddr, 0x2E, 0x09);
	}
	else if(filter_bw == 13) // for ISDB-T
	{
		fc2581_i2c_write(tunerAddr, 0x36, 0x1C);
		fc2581_i2c_write(tunerAddr, 0x37, (unsigned char)(5.8*FREQ_XTAL/1000) );
		fc2581_i2c_write(tunerAddr, 0x39, 0x80);
		fc2581_i2c_write(tunerAddr, 0x2E, 0x09);
	}
	
	
	for(i=0; i<5; i++)
	{
		MB86A22_wait(5);//wait 5ms
		cal_mon = fc2581_i2c_read(tunerAddr, 0x2F);
		if( (cal_mon & 0xC0) != 0xC0)
		{
			fc2581_i2c_write(tunerAddr, 0x2E, 0x01);
			fc2581_i2c_write(tunerAddr, 0x2E, 0x09);
		}
		else
			break;
	}

	fc2581_i2c_write(tunerAddr, 0x2E, 0x01);
}

/*==============================================================================
       fc2581 RSSI function

  This function is a generic function which returns fc2581's
  
  current RSSI value.

  <input parameter>
  tunerAddr
    type: unsigned char
    tuner I2C address

  <return value>
  int
  	rssi : estimated input power.

==============================================================================*/
int fc2581_get_rssi(unsigned char tunerAddr)
{
	int ofs_lna, ofs_rfvga, ofs_csf, ofs_ifvga, rssi;
	
	int s_lna = fc2581_i2c_read(tunerAddr, 0x71);
	int s_rfvga = fc2581_i2c_read(tunerAddr, 0x72);
	int s_cfs = fc2581_i2c_read(tunerAddr, 0x73);
	int s_ifvga = fc2581_i2c_read(tunerAddr, 0x74);
	
	ofs_lna = 
			(curr_band==UHF_BAND)?
				(s_lna==0)? 0 :
				(s_lna==1)? -6 :
				(s_lna==2)? -17 :
				(s_lna==3)? -22 : -30 :
			(curr_band==VHF_BAND)?
				(s_lna==0)? 0 :
				(s_lna==1)? -6 :
				(s_lna==2)? -19 :
				(s_lna==3)? -24 : -32 :
			(curr_band==L_BAND)?
				(s_lna==0)? 0 :
				(s_lna==1)? -6 :
				(s_lna==2)? -11 :
				(s_lna==3)? -16 : -34 :
			0;//NO_BAND
	ofs_rfvga = -s_rfvga+((s_rfvga>=11)? 1 : 0) + ((s_rfvga>=18)? 1 : 0);
	ofs_csf = -6*s_cfs;
	ofs_ifvga = s_ifvga/4;

	 rssi = ofs_lna+ofs_rfvga+ofs_csf+ofs_ifvga+OFS_RSSI;
   return rssi;
				
}
