/*
 MxL136RF Source Code : V1.1.0
 
 Copyright, Maxlinear, Inc.
 All Rights Reserved
 
 File Name:      MxL136RF.c

 Description: The source code is for MxL136RF user to quickly integrate MxL136RF into their software.
    There are two functions the user can call to generate a array of I2C command that's require to
    program the MxL136RF tuner. The user should pass an array pointer and an integer pointer in to the 
    function. The funciton will fill up the array with format like follow:
    
        addr1
        data1
        addr2
        data2
        ...
    
    The user can then pass this array to their I2C function to perform progromming the tuner. 
*/
//#include "StdAfx.h"
#include "MxL136RF_Common.h"
#include "MxL136RF.h"
#include "stdio.h"



unsigned int MxL136RF_Init(unsigned char* pArray,               // a array pointer that store the addr and data pairs for I2C write
                    unsigned int* Array_Size,           // a integer pointer that store the number of element in above array
                    unsigned char Mode,             
                    signed int IF_Diff_Out_Level,                           
                    unsigned int Xtal_Freq_Hz,          
                    unsigned int IF_Freq_Hz,                
                    unsigned char Invert_IF,                                            
                    unsigned char Clk_Out_Enable,    
                    unsigned char Clk_Out_Amp                                                   
                    )
{
    
    unsigned int Reg_Index=0;
    unsigned int Array_Index=0;

    IRVType IRV_Init[]=
    {
        //{ Addr, Data} 
        /* IF Output Settings
        [7] [6] [5] | [4] |        | [3] [2] [1] [0]
         0   0   0    DIG_IFINVERT   DIG_IFSELECT */
        { 0x02, 0x06}, 

        /* XTAL Frequency Selection and Clock Out Settings
        [7] [6] [5] [4] | [3]           | [2] [1] [0]
        DIG_XTALSELECT    XTAL_EN_CLKOUT  XTAL_CLKOUT_AMP */
        { 0x03, 0x48},  

        /* Reference Frequency Selection
        [7] [6] [5] [4] | [3] [2] [1] [0]
         0   0   0   0    DIG_RFREFSELECT */
        { 0x05, 0x04}, 
    
            
        { 0x06, 0x10}, 
        { 0x2E, 0x15},  //Override
        { 0x30, 0x10},  //Override
        { 0x45, 0x58},  //Override
        { 0x48, 0x19},  //Override
        { 0x52, 0x03},  //Override
        { 0x53, 0x44},  //Override
        { 0x6A, 0x4B},  //Override
        { 0x76, 0x00},  //Override
        { 0x78, 0x18},  //Override
        { 0x7A, 0x17},  //Override
        { 0x85, 0x06},  //Override  

        /* Reference Frequency Selection
        [7] [6] [5] [4] [3] [2] [1] | [0]
         0   0   0   0   0   0   0    TOP_MASTER_ENABLE */
        { 0x01, 0x01}, //TOP_MASTER_ENABLE=1
        { 0, 0}
    };


    IRVType IRV_Init_Cable[]=
    {
        //{ Addr, Data} 
        { 0x02, 0x06}, 
        { 0x03, 0x48},
        { 0x05, 0x04},  
        { 0x06, 0x10},  
        { 0x09, 0x3F},  
        { 0x0A, 0x3F},  
        { 0x0B, 0x3F},  
        { 0x2E, 0x15},  //Override
        { 0x30, 0x10},  //Override
        { 0x45, 0x58},  //Override
        { 0x48, 0x19},  //Override
        { 0x52, 0x03},  //Override
        { 0x53, 0x44},  //Override
        { 0x6A, 0x4B},  //Override
        { 0x76, 0x00},  //Override
        { 0x78, 0x18},  //Override
        { 0x7A, 0x17},  //Override
        { 0x85, 0x06},  //Override  
        { 0x01, 0x01}, //TOP_MASTER_ENABLE=1
        { 0, 0}
    };
    //edit Init setting here

    PIRVType myIRV;

    switch(Mode)
    {
    case MxL_MODE_ISDBT: //ISDB-T Mode  
        myIRV = IRV_Init;
        MxL136RF_SetIRVBit(myIRV, 0x06, 0x1F, 0x10);  
        break;
    case MxL_MODE_DVBT: //DVBT Mode         
        myIRV = IRV_Init;
        MxL136RF_SetIRVBit(myIRV, 0x06, 0x1F, 0x11);  
        break;
    case MxL_MODE_ATSC: //ATSC Mode         
        myIRV = IRV_Init;
        MxL136RF_SetIRVBit(myIRV, 0x06, 0x1F, 0x12);  
        break;  
    case MxL_MODE_CABLE:                        
        myIRV = IRV_Init_Cable;
        MxL136RF_SetIRVBit(myIRV, 0x09, 0xFF, 0xC1); 
        MxL136RF_SetIRVBit(myIRV, 0x0A, 0xFF, (unsigned char)(8-IF_Diff_Out_Level)); //Add (UINT8)() by Fujitsu to delete warning
        MxL136RF_SetIRVBit(myIRV, 0x0B, 0xFF, 0x17);                         
        break;
    default:
        return MxL_ERR_INIT;
    }

    switch(IF_Freq_Hz)
    {
    case MxL_IF_4_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x02, 0x0F, 0x00);         
        break;
    case MxL_IF_4_5_MHZ: //4.5MHz
        MxL136RF_SetIRVBit(myIRV, 0x02, 0x0F, 0x02);         
        break;
    case MxL_IF_4_57_MHZ: //4.57MHz
        MxL136RF_SetIRVBit(myIRV, 0x02, 0x0F, 0x03); 
        break;
    case MxL_IF_5_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x02, 0x0F, 0x04); 
        break;
    case MxL_IF_5_38_MHZ: //5.38MHz
        MxL136RF_SetIRVBit(myIRV, 0x02, 0x0F, 0x05); 
        break;
    case MxL_IF_6_MHZ: 
        MxL136RF_SetIRVBit(myIRV, 0x02, 0x0F, 0x06); 
        break;
    case MxL_IF_6_28_MHZ: //6.28MHz
        MxL136RF_SetIRVBit(myIRV, 0x02, 0x0F, 0x07); 
        break;
    case MxL_IF_9_1915_MHZ: //9.1915MHz
        MxL136RF_SetIRVBit(myIRV, 0x02, 0x0F, 0x08); 
        break;
    case MxL_IF_35_25_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x02, 0x0F, 0x09); 
        break;
    case MxL_IF_36_15_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x02, 0x0F, 0x0a); 
        break;
    case MxL_IF_44_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x02, 0x0F, 0x0B); 
        break;
    default:
        return MxL_ERR_INIT;
    }

    if(Invert_IF) 
        MxL136RF_SetIRVBit(myIRV, 0x02, 0x10, 0x10);   //Invert IF
    else
        MxL136RF_SetIRVBit(myIRV, 0x02, 0x10, 0x00);   //Normal IF


    switch(Xtal_Freq_Hz)
    {
    case MxL_XTAL_16_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x03, 0xF0, 0x00);  //select xtal freq & Ref Freq
        MxL136RF_SetIRVBit(myIRV, 0x05, 0x0F, 0x00);
        break;
    case MxL_XTAL_20_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x03, 0xF0, 0x10);
        MxL136RF_SetIRVBit(myIRV, 0x05, 0x0F, 0x01);
        break;
    case MxL_XTAL_20_25_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x03, 0xF0, 0x20);
        MxL136RF_SetIRVBit(myIRV, 0x05, 0x0F, 0x02);
        break;
    case MxL_XTAL_20_48_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x03, 0xF0, 0x30);
        MxL136RF_SetIRVBit(myIRV, 0x05, 0x0F, 0x03);
        break;
    case MxL_XTAL_24_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x03, 0xF0, 0x40);
        MxL136RF_SetIRVBit(myIRV, 0x05, 0x0F, 0x04);
        break;
    case MxL_XTAL_25_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x03, 0xF0, 0x50);
        MxL136RF_SetIRVBit(myIRV, 0x05, 0x0F, 0x05);
        break;
    case MxL_XTAL_25_14_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x03, 0xF0, 0x60);
        MxL136RF_SetIRVBit(myIRV, 0x05, 0x0F, 0x06);
        break;
    case MxL_XTAL_27_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x03, 0xF0, 0x70);
        MxL136RF_SetIRVBit(myIRV, 0x05, 0x0F, 0x07);
        break;
    case MxL_XTAL_28_8_MHZ: 
        MxL136RF_SetIRVBit(myIRV, 0x03, 0xF0, 0x80);
        MxL136RF_SetIRVBit(myIRV, 0x05, 0x0F, 0x08);
        break;
    case MxL_XTAL_32_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x03, 0xF0, 0x90);
        MxL136RF_SetIRVBit(myIRV, 0x05, 0x0F, 0x09);
        break;
    case MxL_XTAL_40_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x03, 0xF0, 0xA0);
        MxL136RF_SetIRVBit(myIRV, 0x05, 0x0F, 0x0A);
        break;
    case MxL_XTAL_44_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x03, 0xF0, 0xB0);
        MxL136RF_SetIRVBit(myIRV, 0x05, 0x0F, 0x0B);
        break;
    case MxL_XTAL_48_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x03, 0xF0, 0xC0);
        MxL136RF_SetIRVBit(myIRV, 0x05, 0x0F, 0x0C);
        break;
    case MxL_XTAL_49_3811_MHZ:
        MxL136RF_SetIRVBit(myIRV, 0x03, 0xF0, 0xD0);
        MxL136RF_SetIRVBit(myIRV, 0x05, 0x0F, 0x0D);
        break;  
    }

    if(!Clk_Out_Enable) //default is enable 
        MxL136RF_SetIRVBit(myIRV, 0x03, 0x08, 0x00);   

    //Clk_Out_Amp
    MxL136RF_SetIRVBit(myIRV, 0x03, 0x07, Clk_Out_Amp);   

    //Generate one Array that Contain Data, Address
    while (myIRV[Reg_Index].Num || myIRV[Reg_Index].Val)
    {
        pArray[Array_Index++] = myIRV[Reg_Index].Num;
        pArray[Array_Index++] = myIRV[Reg_Index].Val;
        Reg_Index++;
    }
        
    *Array_Size=Array_Index;
    return 0;
}


unsigned int MxL136RF_RFTune(unsigned char* pArray, unsigned int* Array_Size, unsigned int RF_Freq, unsigned char BWMHz)
{
    IRVType IRV_RFTune[]=
    {
        //{ Addr, Data}
        /* Sequencer Setting
        [7] [6] [5] [4] [3] [2] [1] | [0]
         0   0   0   0   0   0   0    START_TUNE */
        { 0x0F, 0x00},  //abort tune
        { 0x0C, 0x15},
        { 0x0D, 0x40},
        { 0x0E, 0x0E},  
        { 0x1F, 0x87},  //Override
        { 0x20, 0x1F},  //Override
        { 0x21, 0x87},  //Override
        { 0x22, 0x1F},  //Override      
        { 0x80, 0x01},  //Freq Dependent Setting
        { 0x0F, 0x01},  //start tune
        { 0, 0}
    };

    unsigned int dig_rf_freq=0;
    unsigned int temp;
    unsigned int Reg_Index=0;
    unsigned int Array_Index=0;
    unsigned int i;
    unsigned int frac_divider = 1000000;

    switch(BWMHz)
    {
    case MxL_BW_6MHz: //6MHz
            MxL136RF_SetIRVBit(IRV_RFTune, 0x0C, 0x3F, 0x15);  
        break;
    case MxL_BW_7MHz: //7MHz
            MxL136RF_SetIRVBit(IRV_RFTune, 0x0C, 0x3F, 0x2A);
        break;
    case MxL_BW_8MHz: //8MHz
            MxL136RF_SetIRVBit(IRV_RFTune, 0x0C, 0x3F, 0x3F);
        break;
    default:
        return MxL_ERR_RFTUNE;
    }


    //Convert RF frequency into 16 bits => 10 bit integer (MHz) + 6 bit fraction
    dig_rf_freq = RF_Freq / MHz;
    temp = RF_Freq % MHz;
    for(i=0; i<6; i++)
    {
        dig_rf_freq <<= 1;
        frac_divider /=2;
        if(temp > frac_divider)
        {
            temp -= frac_divider;
            dig_rf_freq++;
        }
    }

    //add to have shift center point by 7.8124 kHz
    if(temp > 7812)
        dig_rf_freq ++;
        
    MxL136RF_SetIRVBit(IRV_RFTune, 0x0D, 0xFF, (unsigned char)dig_rf_freq);
    MxL136RF_SetIRVBit(IRV_RFTune, 0x0E, 0xFF, (unsigned char)(dig_rf_freq>>8));

    if (RF_Freq >=333*MHz)
        MxL136RF_SetIRVBit(IRV_RFTune, 0x80, 0x40, 0x40);

    //Generate one Array that Contain Data, Address 
    while (IRV_RFTune[Reg_Index].Num || IRV_RFTune[Reg_Index].Val)
    {
        pArray[Array_Index++] = IRV_RFTune[Reg_Index].Num;
        pArray[Array_Index++] = IRV_RFTune[Reg_Index].Val;
        Reg_Index++;
    }
    
    *Array_Size=Array_Index;
    
    return 0;
}

//local functions called by Init and RFTune
unsigned int MxL136RF_SetIRVBit(PIRVType pIRV, unsigned char Num, unsigned char Mask, unsigned char Val)
{
    while (pIRV->Num || pIRV->Val)
    {
        if (pIRV->Num==Num)
        {
            pIRV->Val&=~Mask;
            pIRV->Val|=Val;
        }
        pIRV++;

    }   
    return 0;
}

