/*

 Driver APIs for MxL136RF Tuner

 Copyright, Maxlinear, Inc.
 All Rights Reserved

 File Name:      MxL136RF_API.h

 */
#ifndef __MxL136RF_API_H
#define __MxL136RF_API_H

#include "MxL136RF_Common.h"
#include "MxL136RF.h"


/******************************************************************************
**
**  Name: MxL_Set_Register
**
**  Description:    Write one register to MxL136RF
**
**  Parameters:     myTuner             - Pointer to MxL136RF_TunerConfigS
**                  RegAddr             - Register address to be written
**                  RegData             - Data to be written
**
**  Returns:        MxL_ERR_MSG         - MxL_OK if success 
**                                      - MxL_ERR_SET_REG if fail
**
******************************************************************************/
MxL_ERR_MSG MxL_Set_Register(MxL136RF_TunerConfigS* myTuner, unsigned char RegAddr, unsigned char RegData);

/******************************************************************************
**
**  Name: MxL_Get_Register
**
**  Description:    Read one register from MxL136RF
**
**  Parameters:     myTuner             - Pointer to MxL136RF_TunerConfigS
**                  RegAddr             - Register address to be read
**                  RegData             - Pointer to register read
**
**  Returns:        MxL_ERR_MSG         - MxL_OK if success 
**                                      - MxL_ERR_GET_REG if fail
**
******************************************************************************/
MxL_ERR_MSG MxL_Get_Register(MxL136RF_TunerConfigS* myTuner, unsigned char RegAddr, unsigned char *RegData);

/******************************************************************************
**
**  Name: MxL_Tuner_Init
**
**  Description:    MxL136RF Initialization
**
**  Parameters:     myTuner             - Pointer to MxL136RF_TunerConfigS
**
**  Returns:        MxL_ERR_MSG         - MxL_OK if success 
**                                      - MxL_ERR_INIT if fail
**
******************************************************************************/
MxL_ERR_MSG MxL_Tuner_Init(MxL136RF_TunerConfigS* );

/******************************************************************************
**
**  Name: MxL_Tuner_RFTune
**
**  Description:    Frequency tunning for channel
**
**  Parameters:     myTuner             - Pointer to MxL136RF_TunerConfigS
**                  RF_Freq_Hz          - RF Frequency in Hz
**                  BWMHz               - Bandwidth 6, 7 or 8 MHz
**
**  Returns:        MxL_ERR_MSG         - MxL_OK if success 
**                                      - MxL_ERR_RFTUNE if fail
**
******************************************************************************/
MxL_ERR_MSG MxL_Tuner_RFTune(MxL136RF_TunerConfigS*, unsigned int RF_Freq_Hz, MxL136RF_BW_MHz BWMHz);       

/******************************************************************************
**
**  Name: MxL_Soft_Reset
**
**  Description:    Software Reset the MxL136RF Tuner
**
**  Parameters:     myTuner             - Pointer to MxL136RF_TunerConfigS
**
**  Returns:        MxL_ERR_MSG         - MxL_OK if success 
**                                      - MxL_ERR_OTHERS if fail
**
******************************************************************************/
MxL_ERR_MSG MxL_Soft_Reset(MxL136RF_TunerConfigS*);

/******************************************************************************
**
**  Name: MxL_Loop_Through_On
**
**  Description:    Turn On/Off on-chip Loop-through
**
**  Parameters:     myTuner             - Pointer to MxL136RF_TunerConfigS
**                  isOn                - True to turn On Loop Through
**                                      - False to turn off Loop Through
**
**  Returns:        MxL_ERR_MSG         - MxL_OK if success 
**                                      - MxL_ERR_OTHERS if fail
**
******************************************************************************/
MxL_ERR_MSG MxL_Loop_Through_On(MxL136RF_TunerConfigS*, MxL136RF_LoopThru);

/******************************************************************************
**
**  Name: MxL_Standby
**
**  Description:    Enter Standby Mode
**
**  Parameters:     myTuner             - Pointer to MxL136RF_TunerConfigS
**
**  Returns:        MxL_ERR_MSG         - MxL_OK if success 
**                                      - MxL_ERR_OTHERS if fail
**
******************************************************************************/
MxL_ERR_MSG MxL_Stand_By(MxL136RF_TunerConfigS*);

/******************************************************************************
**
**  Name: MxL_Wakeup
**
**  Description:    Wakeup from Standby Mode (Note: after wake up, please call RF_Tune again)
**
**  Parameters:     myTuner             - Pointer to MxL136RF_TunerConfigS
**
**  Returns:        MxL_ERR_MSG         - MxL_OK if success 
**                                      - MxL_ERR_OTHERS if fail
**
******************************************************************************/
MxL_ERR_MSG MxL_Wake_Up(MxL136RF_TunerConfigS*);

/******************************************************************************
**
**  Name: MxL_Check_ChipVersion
**
**  Description:    Return the MxL136RF Chip ID
**
**  Parameters:     myTuner             - Pointer to MxL136RF_TunerConfigS
**          
**  Returns:        MxL_ChipVersion         
**
******************************************************************************/
MxL136RF_ChipVersion MxL_Check_ChipVersion(MxL136RF_TunerConfigS*);

/******************************************************************************
**
**  Name: MxL_RFSynth_Lock_Status
**
**  Description:    RF synthesizer lock status of MxL136RF
**
**  Parameters:     myTuner             - Pointer to MxL136RF_TunerConfigS
**                  isLock              - Pointer to Lock Status
**
**  Returns:        MxL_ERR_MSG         - MxL_OK if success 
**                                      - MxL_ERR_OTHERS if fail
**
******************************************************************************/
MxL_ERR_MSG MxL_RFSynth_Lock_Status(MxL136RF_TunerConfigS* , BOOL* isLock);

/******************************************************************************
**
**  Name: MxL_REFSynth_Lock_Status
**
**  Description:    REF synthesizer lock status of MxL136RF
**
**  Parameters:     myTuner             - Pointer to MxL136RF_TunerConfigS
**                  isLock              - Pointer to Lock Status
**
**  Returns:        MxL_ERR_MSG         - MxL_OK if success 
**                                      - MxL_ERR_OTHERS if fail    
**
******************************************************************************/
MxL_ERR_MSG MxL_REFSynth_Lock_Status(MxL136RF_TunerConfigS* , BOOL* isLock);

/******************************************************************************
**
**  Name: MxL_Set_IF_Output_Gain
**
**  Description:    Sets the MxL135 IF output gain level
**
**  Parameters:     myTuner             - Pointer to MxL136RF_TunerConfigS
**                  IF_Gain             - IF gain (56->min, 63->max)
**
**  Returns:        MxL_ERR_MSG         - MxL_OK if success 
**                                      - MxL_ERR_OTHERS if fail
**
******************************************************************************/
MxL_ERR_MSG MxL_Set_IF_Output_Gain(MxL136RF_TunerConfigS*, unsigned int IF_Gain);

//void MxL_OFMD_adr_init(unsigned char OFDM_adr);

#endif //__MxL136RF_API_H
