/*
*****************************************************************************
**
** \file        ./subsystems/fe/src/fe.c
**
** \version     $Id: fe.c 3733 2009-06-03 09:03:44Z jmorha $
**
** \brief       FE Driver.
**
** This files contains the implementation of the FE driver's core API.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2009 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2009 by Fujitsu Semiconductor Limited
**
*****************************************************************************
*/
/*lint -save -e19 -e526 -e553 -e757 -e961 */

#include "fapex/abst_types.h"
#include "fapex/abst_libi2c.h"
#include "fe_priv.h"

#if SUPPORT_CE6353 == 1
#include "ce6353/src/ce6353.h"
#endif

#if SUPPORT_DSF89000 == 1
#include "dsf89000/src/dsf89000.h"
#endif

#if SUPPORT_MB86668 == 1
#include "mb86668/src/mb86668.h"
#endif

#if SUPPORT_MB86667 == 1
#include "mb86667/src/mb86667.h"
#endif

#if SUPPORT_M88DC2800_DCT70701 == 1
#include "M88DC2800.h"
#endif

#if SUPPORT_FEDIRECT == 1
#include "fedirect/src/fedirect.h"
#endif

#if SUPPORT_M88DS3K == 1
#include "m88ds3k/src/m88ds3k.h"
#endif

#if SUPPORT_M88DC2800 == 1
#include "m88dc2800.h"
#endif

#if SUPPORT_MB86A21 == 1
#include "mb86a21/src/mb86a21.h"
#endif

#if SUPPORT_MB86A22 == 1
#include "mb86a22/src/mb86a22.h"
#endif
//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************

#define FE_HANDLE_ID                (uint32_t)0x20206665 /* '  fe' */

#define FE_LOCK   FAPI_SYS_GET_SEMAPHORE( feSemaphore, FAPI_SYS_SUSPEND )
#define FE_UNLOCK FAPI_SYS_SET_SEMAPHORE( feSemaphore, FAPI_SYS_NO_SUSPEND )

#define DO_DETECT(X) if( X ##_Detect(slotNum, &feHandleArray[slotNum]) == FAPI_OK){\
    FE_DPrintf("[FE] Detected frontend " #X "\n");\
    (void)FAPEX_I2C_Close(hp->i2cId);\
    return FAPI_OK; }


//***************************************************************************
//***************************************************************************
//** Local Structures
//***************************************************************************
//***************************************************************************

//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************

                           /* FE semaphore for secured operations */
static FAPI_SYS_SemaphoreT feSemaphore    = (FAPI_SYS_SemaphoreT)0;
                           /* to set if FE driver is initialized */
static uint32_t            feInitialized  = 0;

static FAPEX_I2C_OpenParamStrT i2cDefaultParams = {
    FAPEX_I2C0,                 // blockIndex
#if ((SUPPORT_MB86A22 == 1) || (SUPPORT_MB86A21 == 1))
    FAPEX_I2C_100KBPS,          // speed
    FAPEX_I2C_RESTART_PROTOCOL, // protocol
#else
    FAPEX_I2C_400KBPS,          // speed
    FAPEX_I2C_COMMON_PROTOCOL, // protocol
#endif
    0,                          // slaveAddress
    FAPEX_I2C_HARDWARE,         // type
    200                         // timeOut
};

//***************************************************************************
//***************************************************************************
//** Local Functions Declaration
//***************************************************************************
//***************************************************************************
                    /* allocate a FE handle */
static feHandleT*   feAllocateHandle(uint8_t index);
                    /* release a FE handle */
static void         feReleaseHandle(feHandleT* handlePtr);
                    /* check if a handle is a FE handle */
static uint32_t     feCheckHandle(const feHandleT* const handlePtr);


//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************

              /* array of FE handles */
feHandleT     feHandleArray[FE_MAX_NUM];

/*!
*****************************************************************************
** \brief FE Driver Interface implementation.
**
** This datastucture represents the FE driver specific implementation
** of the system driver interface.
**
** \sa FAPI_SYS_DriverT
****************************************************************************
*/
FAPI_SYS_DriverT FE_Driver = {
    "FE",                         //!< Driver register name.
    8,                            //!< The handled IRQ.
    FAPI_SYS_BSR_EXEC_UNDEFINED,  //!< The BSR execution directive.
    FE_Init,                      //!< The drivers init function.
    FE_Exit,                      //!< The drivers exit function.
    NULL,                         //!< The drivers ISR.
    NULL,                         //!< no BSR implemented.
    0,                            //!< BSR stack size.
    0                             //!< BSR fifo depth.
};

//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

/*
********************************************************************************
** \brief Allocates a new handle.
**
** This function allocates a new FE handle from the FE handle array.
**
** \return
** - (>0) If handle can be allocated
** - (0)  In case of error, no more handles available
********************************************************************************
*/
static feHandleT* feAllocateHandle(uint8_t index)
{
    feHandleT* fe_handle_ptr = NULL;

    if( !feHandleArray[index].inUse )
    {
        /* set in use */
        feHandleArray[index].inUse = 1;
        feHandleArray[index].id = FE_HANDLE_ID;
        /* set handle pointer */
        fe_handle_ptr = &(feHandleArray[index]);
    }
    return(fe_handle_ptr);
}

/*
********************************************************************************
** \brief Releases a given handle.
**
** This function de-allocates a given FE handle.
********************************************************************************
*/
static void feReleaseHandle(feHandleT* handlePtr)
{
    /* clear handle context */
    memset( handlePtr, 0, sizeof(feHandleT) );
    return;
}

/*
********************************************************************************
** \brief Checks a given handle.
**
** This function checks a given FE handle.
**
** \return
** - (1) If handle is OK
** - (0) If handle is invalid
********************************************************************************
*/
static uint32_t feCheckHandle(const feHandleT* handlePtr)
{
    return( feInitialized && handlePtr && handlePtr->inUse
            && ( handlePtr->id == FE_HANDLE_ID ) );
}

//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

/*!
*****************************************************************************
** \brief Initialize the FE driver.
**
** This function initializes the driver. Initialization is performed with
** default values of the driver parameters if they have not been overwritten.
**
** \attention It is assumed the hardware register layer has been initialized
**            before. This must be ensured on application level either by
**            calling FAPI_SYS_Init or by directly calling the related
**            FREG_XXXX_Init function(s).
**
** \note This function implements FAPI_SYS_DriverT::initDriverFunc().
**
** \return
**        - #FAPI_OK if initialization was successful
**        - #FE_ERR_ALREADY_INITIALIZED if the function has already been
**          called.
**        - #FE_ERR_SEMAPHORE_CREATE if semaphore creation failed.
**        - .... add further error code here
**
** \sa FE_Exit
*****************************************************************************
*/
int32_t FE_Init(void)
{
    uint8_t ii;

    if (feInitialized) return( FE_ERR_ALREADY_INITIALIZED );

    for(ii=0;ii<FE_MAX_NUM;ii++)
    {
        memset(&feHandleArray[ii], 0, sizeof(feHandleT));
        feHandleArray[ii].slotNum = ii;
        feHandleArray[ii].i2cDev  = DRV_ID_I2S0 + ii;
    }

    feSemaphore = FAPI_SYS_CREATE_SEMAPHORE(1);
    if (!feSemaphore ) return( FE_ERR_SEMAPHORE_CREATE );

    feInitialized = 1;

    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief Exit the FE driver.
**
** This function shuts down the driver and safely disables the underlying
** hardware block. Pending accesses will be flushed and all allocated
** resources will be released.
**
** \note This function implements FAPI_SYS_DriverT::exitDriverFunc().
**
** \sa FE_Init
*****************************************************************************
*/
void FE_Exit(void)
{
    if ( !feInitialized ) return;

    feInitialized = 0;
    FAPI_SYS_DESTROY_SEMAPHORE( feSemaphore );
    feSemaphore = 0;
}

/*!
*****************************************************************************
** \brief Interrupt service routine of the FE driver.
**
** \note This function implements FAPI_SYS_DriverT::isrFunc().
**
** \return (0) since no data for background processing must be returned.
*****************************************************************************
*/
uint32_t FE_Isr(void)
{
    return(0);
}

/*!
*****************************************************************************
** \brief Open the FE driver.
**
** This function opens an instance of the FE driver.
**
** \param paramsPtr Pointer to the open parameters.
** \param errorCodePtr TO BE DONE
**
** \return
**         - (>0) valid driver instance handle on success.
**         - (0) if the driver couldn't be opened.
**
** \sa FE_Close
*****************************************************************************
*/
FAPI_SYS_HandleT FE_Open(const FE_OpenParamsT* paramsPtr, int32_t* errorCodePtr)
{
    feHandleT* fe_handle_ptr = NULL;
    int32_t    error_code    = FAPI_OK;
    uint8_t    slotNum;

    /* error if driver not initialized */
    if(feInitialized == 0)
    {
        error_code = FE_ERR_NOT_INITIALIZED;
        if( errorCodePtr )
            *errorCodePtr = error_code;
        return(0);
    }
    /* check parameters */
    if(paramsPtr == NULL)
    {
        error_code = FE_ERR_BAD_PARAMETER;
        if( errorCodePtr )
            *errorCodePtr = error_code;
        return(0);
    }
    /* check version */
    if(!FAPI_SYS_CHECK_VERSION(paramsPtr->version, FE_VERSION))
    {
        error_code = FE_ERR_UNSUPPORTED_VERSION;
        if( errorCodePtr )
            *errorCodePtr = error_code;
        return(0);
    }

    ///////////////
    FE_LOCK;
    ///////////////

    /* allocate handle */
    slotNum = paramsPtr->slotNum;
    if(slotNum >= FE_MAX_NUM)
    {
        ////////////////
        FE_UNLOCK;
        ////////////////
        error_code = FE_ERR_BAD_PARAMETER;
        if( errorCodePtr )
            *errorCodePtr = error_code;
        return(0);
    }
    fe_handle_ptr = feAllocateHandle(slotNum);
    if( !feCheckHandle( fe_handle_ptr ) )
    {
        ////////////////
        FE_UNLOCK;
        ////////////////
        error_code = FE_ERR_INVALID_HANDLE;
        if( errorCodePtr )
            *errorCodePtr = error_code;
        return(0);
    }

    /* copy open params to handle */
    memcpy( &(fe_handle_ptr->openParams),
            paramsPtr, sizeof(FE_OpenParamsT) );
    fe_handle_ptr->slotNum = slotNum;

    if((!feHandleArray[slotNum].isPresent) ||
       (!feHandleArray[slotNum].feOpenFct))
    {
        ////////////////
        FE_UNLOCK;
        ////////////////
        error_code = FE_ERR_INVALID_HANDLE;
        if( errorCodePtr )
            *errorCodePtr = error_code;
        return(0);
    }
    fe_handle_ptr->i2cId = FAPEX_I2C_Open(fe_handle_ptr->i2cDev, O_RDWR, &i2cDefaultParams);

    if(fe_handle_ptr->i2cId < 0)
    {
        ////////////////
        FE_UNLOCK;
        ////////////////
        if( errorCodePtr )
            *errorCodePtr = error_code;
        return(0);
    }
    if(feHandleArray[slotNum].feOpenFct(slotNum, (FAPI_SYS_HandleT *)(&fe_handle_ptr)) != FAPI_OK)
    {
        ////////////////
        FE_UNLOCK;
        ////////////////
        error_code = FE_ERR_INVALID_HANDLE;
        if( errorCodePtr )
            *errorCodePtr = error_code;
        return(0);
    }

    ////////////////
    FE_UNLOCK;
    ////////////////

    return((FAPI_SYS_HandleT)fe_handle_ptr);
}

/*!
*****************************************************************************
** \brief Closes the FE driver.
**
** This function closes an instance of the FE driver. All resources
** associated with the given driver handle will be released.
**
** \attention After the driver is closed the given handle is invalid and
**            must not be further used.
**
** \param handle Valid driver instance handle previously returned by
**               FE_Open.
**
** \sa FE_Open
*****************************************************************************
*/
int32_t FE_Close ( FAPI_SYS_HandleT handle )
{
    feHandleT* fe_handle_ptr  = (feHandleT*)handle;
    int32_t    error_code     = FAPI_OK;

    ///////////////
    FE_LOCK;
    ///////////////

    /* check handle */
    if( feCheckHandle( fe_handle_ptr ) )
    {
        if(!fe_handle_ptr->feCloseFct)
        {
            error_code = FE_ERR_INVALID_HANDLE;
        }
        else
        {
            /* frontend-specific close function */
            error_code = fe_handle_ptr->feCloseFct(fe_handle_ptr->slotNum);
        }
        (void)FAPEX_I2C_Close(fe_handle_ptr->i2cId);
        /* release handle */
        feReleaseHandle( fe_handle_ptr );
    }
    else
    {
        error_code = FE_ERR_INVALID_HANDLE;
    }

    ////////////////
    FE_UNLOCK;
    ////////////////

    return error_code;
}

/*!
*****************************************************************************
** \brief Detects type (S/C/T) and hardware of connected frontend
**
** This function uses I2C probing to detect the connected frontend
**
** \attention This must be called before an FE_Open call for the same slot
**
** \param slotNum The hardware slot at which the frontend is located
**
** \sa FE_Open
*****************************************************************************
*/
int32_t FE_DetectSlot(uint8_t slotNum)
{
    feHandleT *hp = &feHandleArray[slotNum];
    int32_t retVal = FE_ControlFeReset(slotNum, FE_RESET_PULSE);

    if(retVal != FAPI_OK)
        return retVal;
    FE_DPrintf("FE_DetectSlot %d\n", slotNum);

    if(slotNum >= FE_MAX_NUM)
        return FE_ERR_BAD_PARAMETER;
    hp->i2cId = FAPEX_I2C_Open(hp->i2cDev, O_RDWR, &i2cDefaultParams);

    if(hp->i2cId < 0)
        return retVal;
        
    #if(SUPPORT_M88DS3K == 1)
    DO_DETECT(M88DS3K_HE6306);
    #endif    

    #if(SUPPORT_CE6353_DTT75404 == 1)
    DO_DETECT(CE6353_DTT75404);
    #endif

    #if(SUPPORT_CE6353_DTT75404_IDTV == 1)
    DO_DETECT(CE6353_DTT75404_IDTV);
    #endif

    #if(SUPPORT_CE6353_TD911 == 1)
    DO_DETECT(CE6353_TD911);
    #endif

    #if(SUPPORT_CE6353_TD1316_IDTV == 1)
    DO_DETECT(CE6353_TD1316_IDTV);
    #endif

    #if(SUPPORT_DSF89000 == 1)
    DO_DETECT(DSF89000);
    #endif

    #if(SUPPORT_MB86668_CD1316 == 1)
    DO_DETECT(MB86668_CD1316);
    #endif

    #if(SUPPORT_MB86668_CD1600 == 1)
    DO_DETECT(MB86668_CD1600);
    #endif

    #if(SUPPORT_MB86668_DCT70602 == 1)
    DO_DETECT(MB86668_DCT70602);
    #endif

    #if(SUPPORT_MB86668_DCT70701 == 1)
    DO_DETECT(MB86668_DCT70701);
    #endif

    #if(SUPPORT_M88DC2800_DCT70701 == 1)
        DO_DETECT(MB88DC2800_DCT70701);
    #endif

    #if(SUPPORT_MB86667_S7HZ1402 == 1)
    DO_DETECT(MB86667_S7HZ1402);
    #endif

   #if(SUPPORT_M88DC2800 == 1)
    DO_DETECT(M88DC2800);
   #endif

    #if(SUPPORT_MB86A22 == 1)
    DO_DETECT(MB86A22);
    #endif

    #if(SUPPORT_MB86A21 == 1)
    DO_DETECT(MB86A21_MXL135RF);
    DO_DETECT(MB86A21_MAX3580);
    #endif
    #if(SUPPORT_FEDIRECT == 1)
    DO_DETECT(FE_DIRECT);
    #endif

    (void)FAPEX_I2C_Close(hp->i2cId);
    return FE_ERR_FE_UNKNOWN_DEVICE;
}

/*!
*****************************************************************************
** \brief Detects type (S/C/T) and hardware of all connected frontends
**
** This function uses I2C probing to detect the connected frontends
**
** \attention This must be called before any FE_Open call
**
** \sa FE_DetectSlot
*****************************************************************************
*/
int32_t FE_DetectAll()
{
    uint8_t ii;

    for(ii=0;ii<FE_MAX_NUM;ii++)
        (void)FE_DetectSlot(ii);
    return FAPI_OK;
}

/*!
*****************************************************************************
** \brief Get the delivery system type (S/C/T) of a frontend
**
** This function writes the type of frontend associated with the handle
** to the location given by the pointer argument
**
** \param handle Valid driver instance handle previously returned by
**               FE_Open
** \param pType location where type will be stored
**
** \sa FE_DetectSlot
*****************************************************************************
*/
int32_t FE_GetSystemType(FAPI_SYS_HandleT handle, FE_SYSTEM_TYPE_E *pType)
{
    feHandleT *hp = handle;

    if(!feCheckHandle(hp))
        return FE_ERR_INVALID_HANDLE;

    *pType = hp->systemType;
    return FAPI_OK;
}

/*!
*****************************************************************************
** \brief Get the number of open frontend instances
**
** This function writes the number of open frontend instances to the pointer
** argument
**
** \param pOpenNumber location where number of instances will be stored
**
** \sa FE_DetectSlot
*****************************************************************************
*/
int32_t FE_GetOpenNumber(uint8_t* pOpenNumber)
{
    uint32_t ii;
    uint8_t cnt = 0;

    for(ii=0;ii<FE_MAX_NUM;ii++)
        if(feHandleArray[ii].inUse)
            cnt++;
    *pOpenNumber = cnt;
    return FAPI_OK;
}

/******************************************************************************
**
** \brief  Resets the front-end.
**
** \param handle Valid driver instance handle previously returned by
**               FE_Open
**
** \return Possible return codes:
**         - #FAPI_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_FE_I2C
**         - #FE_ERR_FEATURE_NOT_SUPPORTED
**
******************************************************************************/
int32_t FE_Reset(FAPI_SYS_HandleT handle)
{
    feHandleT *hp = handle;

    if(!feCheckHandle(hp))
        return FE_ERR_BAD_PARAMETER;

    if(hp->feResetFct)
        return hp->feResetFct(hp->slotNum);
    else
        return FE_ERR_FEATURE_NOT_SUPPORTED;
}

/*-**************************************************************************/
/*  map API functions to internal function pointers                         */
/*-**************************************************************************/
#define INTFUNCVAR(fname, args...) {feHandleT *hp = handle;\
    if(!feCheckHandle(hp))\
        return FE_ERR_BAD_PARAMETER;\
    if(hp->fname)\
        return hp->fname(hp->slotNum, args);\
    else\
    return FE_ERR_FEATURE_NOT_SUPPORTED;}

/******************************************************************************
**
** \brief  Returns current AFC.
**
** \param  handle Valid driver instance handle previously returned by
**               FE_Open
** \param  pAFC   Pointer to AFC.
**
** \return Possible return codes:
**         - #FAPI_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_NOT_INITIALIZED
**         - #FE_ERR_FE_NOT_LOCKED
**         - #FE_ERR_FE_I2C
**
******************************************************************************/
int32_t   FE_GetAFC(FAPI_SYS_HandleT handle, int32_t *pAFC)
INTFUNCVAR(feGetAFCFct,                               pAFC);

/******************************************************************************
**
** \brief  Returns current BER.
**
** \param  handle Valid driver instance handle previously returned by
**               FE_Open
** \param  pBER   Pointer to BER.
**
** \return Possible return codes:
**         - #FAPI_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_NOT_INITIALIZED
**         - #FE_ERR_FE_NOT_LOCKED
**         - #FE_ERR_FE_I2C
**
******************************************************************************/
int32_t   FE_GetBER(FAPI_SYS_HandleT handle, uint32_t *pBER)
INTFUNCVAR(feGetBERFct,                                pBER);

/******************************************************************************
**
** \brief  Returns current FEC.
**
** \param  handle Valid driver instance handle previously returned by
**               FE_Open
** \param  pFEC   Pointer to FEC.
**
** \return Possible return codes:
**         - #FAPI_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_NOT_INITIALIZED
**         - #FE_ERR_FE_NOT_LOCKED
**         - #FE_ERR_FE_I2C
**
******************************************************************************/
int32_t   FE_GetFEC(FAPI_SYS_HandleT handle, FE_FEC_E *pFEC)
INTFUNCVAR(feGetFECFct,                                pFEC);

/******************************************************************************
**
** \brief  Returns the signal quality.
**
** \param  handle Valid driver instance handle previously returned by
**               FE_Open
** \param  pQuality  Pointer to quality. (range 0 to 30 dB)
**
** \return Possible return codes:
**         - #FAPI_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_NOT_INITIALIZED
**         - #FE_ERR_FE_I2C
**
******************************************************************************/
int32_t   FE_GetSignalQuality(FAPI_SYS_HandleT handle, uint16_t *pQuality)
INTFUNCVAR(feGetSignalQualityFct,                                pQuality);

/******************************************************************************
**
** \brief  Returns the signal strength.
**
** \param  handle Valid driver instance handle previously returned by
**               FE_Open
** \param  pStrength  Pointer to strength.
**
** \return Possible return codes:
**         - #FAPI_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_NOT_INITIALIZED
**         - #FE_ERR_FE_I2C
**
******************************************************************************/
int32_t   FE_GetSignalStrength(FAPI_SYS_HandleT handle, uint16_t *pStrength)
INTFUNCVAR(feGetSignalStrengthFct,                                pStrength);

/******************************************************************************
**
** \brief  Returns the current status.
**
** \param  handle Valid driver instance handle previously returned by
**               FE_Open
** \param  pStatus  Pointer to status.
**
** \return Possible return codes:
**         - #FAPI_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_NOT_INITIALIZED
**         - #FE_ERR_FE_I2C
**
******************************************************************************/
int32_t   FE_GetStatus(FAPI_SYS_HandleT handle, FE_STATUS_E *pStatus)
INTFUNCVAR(feGetStatusFct,                                   pStatus);

/*!
*******************************************************************************
**
** \brief  Get current delivery system parameters
**
** \param  handle Valid driver instance handle previously returned by
**               FE_Open
** \param   pSystemParams pointer to store system parameters
**
** \return Possible return codes:
**         - #FAPI_OK
**         - #FE_ERR_BAD_PARAMETER
**         - failure codes returned from #FAPI_I2C_Read(), #FAPI_I2C_Write()
**
** \sa
**
*******************************************************************************
*/
int32_t   FE_GetSystemParams(FAPI_SYS_HandleT handle, FE_SYSTEM_PARAMS_S *pSystemParams)
INTFUNCVAR(feGetSystemParamsFct,                                          pSystemParams);

/*!
*******************************************************************************
**
** \brief  Get current received TPS parameters
**
** \param  handle Valid driver instance handle previously returned by
**               FE_Open
** \param   pTuneParams   pointer to store tune parameters
**
** \return Possible return codes:
**         - #FAPI_OK
**         - #FE_ERR_BAD_PARAMETER
**         - failure codes returned from #FAPI_I2C_Read(), #FAPI_I2C_Write()
**
** \sa
**
*******************************************************************************
*/
int32_t   FE_GetTuneParams(FAPI_SYS_HandleT handle, FE_TUNE_PARAMS_S *pTuneParams)
INTFUNCVAR(feGetTuneParamsFct,                                        pTuneParams);

/*!
*******************************************************************************
**
** \brief  Scan the frontend according to frequency range passed
** The scan function uses on internal loop to optimize the frequency settings
** since the demodulator provides some function to obtain the offset.
** The "custom" parameter is a pointer used to Tune Parameter settings.
**
** \attention Currently this function should not be used because it is still
**            under construction
**
** \return Possible return codes:
**         - #FAPI_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_FE_OTHER
**         - #FE_ERR_FE_TIMER
**         - failure codes returned from #FAPI_I2C_Read(), #FAPI_I2C_Write()
**
** \sa     CE6353_Tune()
**
*******************************************************************************
*/
int32_t   FE_Scan(FAPI_SYS_HandleT handle, FE_SCAN_PARAMS_S *pScanParams)
INTFUNCVAR(feScanFct,                                        pScanParams);

/******************************************************************************
**
** \brief  Sends a Diseqc command sequence.
**
** \param  handle Valid driver instance handle previously returned by
**               FE_Open
** \param  length         Length of command array.
** \param  pCommands      Pointer to array of commands.
** \param  pReturnLength  Number of retunered bytes which will be filled by
**                        the function.
** \param  pReturnBuffer  Pointer to buffer containing the returned values.
**
** \return Possible return codes:
**         - #FAPI_OK
**         - #FE_ERR_FE_I2C
**         - #FE_ERR_BAD_PARAMETER
**
******************************************************************************/
int32_t   FE_SendDiseqcCommand(FAPI_SYS_HandleT handle, uint8_t length,
            uint8_t* pCommands, uint8_t* pReturnLength, uint8_t* pReturnBufer)
INTFUNCVAR(feSendDiseqcCommandFct,                              length,
                     pCommands,          pReturnLength,          pReturnBufer);


/******************************************************************************
**
** \brief  Changes the system parameters.
**
** \param  handle Valid driver instance handle previously returned by
**               FE_Open
** \param  pSystemParams  Pointer to system parameters to set.
**
** \return Possible return codes:
**         - #FAPI_OK
**         - #FE_ERR_BAD_PARAMETER
**
******************************************************************************/
int32_t   FE_SetSystemParams(FAPI_SYS_HandleT handle,
                            FE_SYSTEM_PARAMS_S *pSystemParams)
INTFUNCVAR(feSetSystemParamsFct,                pSystemParams);

/******************************************************************************
**
** \brief  Tunes the front-end.
**
** \param  handle Valid driver instance handle previously returned by
**               FE_Open
** \param  pTuneParams  Pointer to tuning parameters.
**
** \return Possible return codes:
**         - #FAPI_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_FE_BUSY
**         - #FE_ERR_FE_I2C
**         - #FE_ERR_FE_TUNE
**
******************************************************************************/
int32_t   FE_Tune(FAPI_SYS_HandleT handle, FE_TUNE_PARAMS_S *pTuneParams)
INTFUNCVAR(feTuneFct,                                        pTuneParams);

/*!
*******************************************************************************
**
** \brief  Enable/Disable antenna power output flag
** Antenna power output however will be set first when tune function is called.
**
** \param   handle Valid driver instance handle previously returned by
**               FE_Open
** \param   isPowered     type 1 = power on, 0 = power off,
**
** \return Possible return codes:
**         - #FAPI_OK
**         - #FE_ERR_BAD_PARAMETER
**
** \sa
**
*******************************************************************************
*/
int32_t   FE_SetAntennaPower(FAPI_SYS_HandleT handle, uint8_t isPowered)
INTFUNCVAR(feSetAntennaPowerFct,                              isPowered);

/*!
*******************************************************************************
**
** \brief  Stops the Diseqc12 motor
**
** \param[in]  handle       Handle of frontend.
**
** \return Possible return codes:
**         - #return values from FE_SendDiseqcCommand()
**
******************************************************************************/
int32_t FE_DISEQC12_Halt(FAPI_SYS_HandleT  handle)
{
    feHandleT *hdl = (feHandleT*)handle;
    uint8_t cmd[3] = { 0XE0, 0x00, 0x60 };

    FE_DPrintf("FE_DISEQC12_Halt\n");
    if(!feCheckHandle(hdl))
        return FE_ERR_BAD_PARAMETER;
    cmd[1] = hdl->pSystemParams.sat.motorAddress;

        /* Halt is send 2 time to be sure that motot is stopped */
    (void) FE_SendDiseqcCommand (handle, 3, cmd, NULL, NULL);
    return FE_SendDiseqcCommand (handle, 3, cmd, NULL, NULL);
}

/*!
*******************************************************************************
**
** \brief  Recalculation of Stored Sat positions.
**
** \param[in]  handle       Handle of frontend.
**
** \return Possible return codes:
**         - #return values from FE_SendDiseqcCommand()
**
******************************************************************************/
int32_t FE_DISEQC12_ReCalcPos(FAPI_SYS_HandleT  handle)
{
    feHandleT *hdl = (feHandleT*)handle;
    uint8_t cmd[4] = { 0XE0, 0x00, 0x6f, 0x00 };

    FE_DPrintf("FE_DISEQC12_ReCalcPos\n");
    if(!feCheckHandle(hdl))
        return FE_ERR_BAD_PARAMETER;
    cmd[1] = hdl->pSystemParams.sat.motorAddress;
    return FE_SendDiseqcCommand (handle, 4, cmd, NULL, NULL);
}

/*!
*******************************************************************************
**
** \brief  Switch off set rotation limits.
**
** \param[in]  handle     Handle of frontend.
**
** \return Possible return codes:
**         - #return values from FE_SendDiseqcCommand()
**
******************************************************************************/
int32_t FE_DISEQC12_LimitsOff(FAPI_SYS_HandleT  handle)
{
    feHandleT *hdl = (feHandleT*)handle;
    uint8_t cmd[3] = { 0XE0, 0x00, 0x63 };

    FE_DPrintf("FE_DISEQC12_LimitsOff\n");
    if(!feCheckHandle(hdl))
        return FE_ERR_BAD_PARAMETER;
    cmd[1] = hdl->pSystemParams.sat.motorAddress;
    return FE_SendDiseqcCommand (handle, 3, cmd, NULL, NULL);
}

/*!
*******************************************************************************
**
** \brief  Set actual position as Eastern limit.
**
** \param[in]  handle     Handle of frontend.
**
** \return Possible return codes:
**         - #return values from FE_SendDiseqcCommand()
**
******************************************************************************/
int32_t FE_DISEQC12_SetLimitEast(FAPI_SYS_HandleT  handle)
{
    feHandleT *hdl = (feHandleT*)handle;
    uint8_t cmd[3] = { 0XE0, 0x00, 0x66 };

    FE_DPrintf("FE_DISEQC12_SetLimitEast\n");
    if(!feCheckHandle(hdl))
        return FE_ERR_BAD_PARAMETER;
    cmd[1] = hdl->pSystemParams.sat.motorAddress;
    return FE_SendDiseqcCommand (handle, 3, cmd, NULL, NULL);
}

/*!
*******************************************************************************
**
** \brief  Set actual position as Western limit.
**
** \param[in]  handle     Handle of frontend.
**
** \return Possible return codes:
**         - #return values from FE_SendDiseqcCommand()
**
******************************************************************************/
int32_t FE_DISEQC12_SetLimitWest(FAPI_SYS_HandleT  handle)
{
    feHandleT *hdl = (feHandleT*)handle;
    uint8_t cmd[3] = { 0XE0, 0x00, 0x67 };

    FE_DPrintf("FE_DISEQC12_SetLimitWest\n");
    if(!feCheckHandle(hdl))
        return FE_ERR_BAD_PARAMETER;
    cmd[1] = hdl->pSystemParams.sat.motorAddress;
    return FE_SendDiseqcCommand (handle, 3, cmd, NULL, NULL);
}

/*!
*******************************************************************************
**
** \brief  Drives the motor East
**
** \param[in]  handle       Handle of frontend.
** \param[in]  steps        Number of steps the Diseqc12 motor should move.
**
** \return Possible return codes:
**         - #return values from FE_SendDiseqcCommand()
**
******************************************************************************/
int32_t FE_DISEQC12_DriveMotorEast(FAPI_SYS_HandleT  handle, uint8_t steps)
{
    feHandleT *hdl = (feHandleT*)handle;
    uint8_t cmd[4] = { 0XE0, 0x00, 0x68, 0x00 };

    FE_DPrintf("FE_DISEQC12_DriveMotorEast\n");
    if(!feCheckHandle(hdl))
        return FE_ERR_BAD_PARAMETER;
    cmd[1] = hdl->pSystemParams.sat.motorAddress;
    cmd[3] = steps;
    return FE_SendDiseqcCommand (handle, 4, cmd, NULL, NULL);
}

/*!
*******************************************************************************
**
** \brief  Drives the motor West
**
** \param[in]  handle       Handle of frontend.
** \param[in]  steps        Number of steps the Diseqc12 motor should move.
**
** \return Possible return codes:
**         - #return values from FE_SendDiseqcCommand()
**
******************************************************************************/
int32_t FE_DISEQC12_DriveMotorWest(FAPI_SYS_HandleT  handle, uint8_t steps)
{
    feHandleT *hdl = (feHandleT*)handle;
    uint8_t cmd[4] = { 0XE0, 0x00, 0x68, 0x00 };

    FE_DPrintf("FE_DISEQC12_DriveMotorEast\n");
    if(!feCheckHandle(hdl))
        return FE_ERR_BAD_PARAMETER;
    cmd[1] = hdl->pSystemParams.sat.motorAddress;
    cmd[3] = steps;
    return FE_SendDiseqcCommand (handle, 4, cmd, NULL, NULL);
}

/*!
*******************************************************************************
**
** \brief  Store the actual Position
**
** \param[in]  handle       Handle of frontend.
** \param[in]  satNum       Number of satellite.
**
** \return Possible return codes:
**         - #return values from FE_SendDiseqcCommand()
**
******************************************************************************/
int32_t FE_DISEQC12_StoreSatPos(FAPI_SYS_HandleT  handle, uint8_t satNum)
{
    feHandleT *hdl = (feHandleT*)handle;
    uint8_t cmd[4] = { 0XE0, 0x00, 0x6A, 0x00 };

    FE_DPrintf("FE_DISEQC12_StoreSatPos as %d\n", satNum);
    if(!feCheckHandle(hdl))
        return FE_ERR_BAD_PARAMETER;
    cmd[1] = hdl->pSystemParams.sat.motorAddress;
    cmd[3] = satNum;
    return FE_SendDiseqcCommand (handle, 4, cmd, NULL, NULL);
}

/*!
*******************************************************************************
**
** \brief  Move the motor to the requested Sat.
**
** \param[in]  handle       Handle of the frontend.
** \param[in]  satNum       Number of satellite.
**
** \return Possible return codes:
**         - #return values from FE_SendDiseqcCommand()
**
******************************************************************************/
int32_t FE_DISEQC12_GotoSat(FAPI_SYS_HandleT  handle, uint8_t satNum)
{
    feHandleT *hdl = (feHandleT*)handle;
    uint8_t cmd[4] = { 0XE0, 0x00, 0x6B, 0x00 };

    FE_DPrintf("FE_DISEQC12_GotoSat no. %d\n", satNum);
    if(!feCheckHandle(hdl))
        return FE_ERR_BAD_PARAMETER;
    cmd[1] = hdl->pSystemParams.sat.motorAddress;
    cmd[3] = satNum;
    return FE_SendDiseqcCommand (handle, 4, cmd, NULL, NULL);
}

/*!
*******************************************************************************
**
** \brief  Move the motor to reference position
**
** \param[in]  handle     Handle of the frontend.
**
** \return Possible return codes:
**         - #return values from FE_SendDiseqcCommand()
**
******************************************************************************/
int32_t FE_DISEQC12_Goto00(FAPI_SYS_HandleT  handle)
{
    feHandleT *hdl = (feHandleT*)handle;
    uint8_t cmd[3] = { 0XE0, 0x00, 0x6B };

    FE_DPrintf("FE_DISEQC12_Goto00\n");
    if(!feCheckHandle(hdl))
        return FE_ERR_BAD_PARAMETER;
    cmd[1] = hdl->pSystemParams.sat.motorAddress;
    return FE_SendDiseqcCommand (handle, 3, cmd, NULL, NULL);
}

