/*!
*****************************************************************************
**
** \file        ./subsystems/fe/src/fe.h
**
** \version     $Id: fe.h 3545 2009-05-14 09:40:32Z sgehre $
**
** \brief       FE Driver.
**
** This file contains the declaration of the FE driver API.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2009 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2009 by Fujitsu Semiconductor Limited
**
*****************************************************************************
*/

#ifndef FAPI_FE_H
#define FAPI_FE_H

#include "fapex/abst_lib.h"

//***********************************************************************
//***********************************************************************
//** Defines and Macros
//***********************************************************************
//***********************************************************************
/*! \brief The maximum number of front-end instances. */
#define FE_MAX_NUM                  4

#define FE_ERR_BASE                SUBSYS_FE_MODULE_BASE

#define DRV_ID_I2S0                FAPEXDEV_I2C_MINOR_FIRST

/*!
********************************************************************************
** Standard error codes for the driver
********************************************************************************
*/

/*! Bad parameter passed. */
#define FE_ERR_BAD_PARAMETER         (int32_t)(FE_ERR_BASE +\
                                                    FAPI_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define FE_ERR_OUT_OF_MEMORY         (int32_t)(FE_ERR_BASE +\
                                                    FAPI_ERR_OUT_OF_MEMORY)

/*! Device already initialised. */
#define FE_ERR_ALREADY_INITIALIZED   (int32_t)(FE_ERR_BASE +\
                                              FAPI_ERR_ALREADY_INITIALIZED)

/*! Device not initialised. */
#define FE_ERR_NOT_INITIALIZED       (int32_t)(FE_ERR_BASE +\
                                                  FAPI_ERR_NOT_INITIALIZED)

/*! Feature or function is not available. */
#define FE_ERR_FEATURE_NOT_SUPPORTED (int32_t)(FE_ERR_BASE +\
                                            FAPI_ERR_FEATURE_NOT_SUPPORTED)

/*! Timeout occured. */
#define FE_ERR_TIMEOUT               (int32_t)(FE_ERR_BASE +\
                                                          FAPI_ERR_TIMEOUT)

/*! The device is busy, try again later. */
#define FE_ERR_DEVICE_BUSY           (int32_t)(FE_ERR_BASE +\
                                                      FAPI_ERR_DEVICE_BUSY)

/*! Invalid handle was passed. */
#define FE_ERR_INVALID_HANDLE        (int32_t)(FE_ERR_BASE +\
                                                   FAPI_ERR_INVALID_HANDLE)

/*! Semaphore could not be created. */
#define FE_ERR_SEMAPHORE_CREATE      (int32_t)(FE_ERR_BASE +\
                                                 FAPI_ERR_SEMAPHORE_CREATE)

/*! The driver's used version is not supported. */
#define FE_ERR_UNSUPPORTED_VERSION   (int32_t)(FE_ERR_BASE +\
                                              FAPI_ERR_UNSUPPORTED_VERSION)

/*!
********************************************************************************
** Specific error codes for the driver
********************************************************************************
*/
/*! No more front-end instances available (see also #FE_MAX_NUM). */
#define FE_ERR_FE_ALL_OPEN          FE_ERR_BASE - 101
/*! Opening a front-end instance failed. */
#define FE_ERR_FE_OPEN              FE_ERR_BASE - 102
/*! Attempt to tune. */
#define FE_ERR_FE_TUNE              FE_ERR_BASE - 103
/*! The front-end is busy, e.g. with tuning. */
#define FE_ERR_FE_BUSY              FE_ERR_BASE - 104
/*! GPIO error, e.g. for reseting the front-end. */
#define FE_ERR_FE_GPIO              FE_ERR_BASE - 105
/*! I2C communication error. */
#define FE_ERR_FE_I2C               FE_ERR_BASE - 106
/*! Timeout, i.e. the front-end did not respond in time. */
#define FE_ERR_FE_TIMER             FE_ERR_BASE - 107
/*! The front-end is not locked. */
#define FE_ERR_FE_NOT_LOCKED        FE_ERR_BASE - 108
/*! Other error, not further specified. */
#define FE_ERR_FE_OTHER             FE_ERR_BASE - 109
/*! IC could not be identified. */
#define FE_ERR_FE_UNKNOWN_DEVICE    FE_ERR_BASE - 110
/*! UPI could not be configured */
#define FE_ERR_FE_UPI_CONFIGURATION FE_ERR_BASE - 111

#define FE_HANDLE FAPI_SYS_HandleT
//***********************************************************************
//***********************************************************************
//** Data types
//***********************************************************************
//***********************************************************************

/*!
*******************************************************************************
**
** \brief Delivery system types.
**
******************************************************************************/
typedef enum
{
    FE_SYSTEM_TYPE_DIRECT,      //!< Direct input e.g. parallel.
    FE_SYSTEM_TYPE_SATELLITE,   //!< Satellite system.
    FE_SYSTEM_TYPE_TERRESTRIAL, //!< Terrestrial system.
    FE_SYSTEM_TYPE_CABLE        //!< Cable system.
} FE_SYSTEM_TYPE_E;

/*!
*******************************************************************************
**
** \brief DiSEqC modes for satellite systems.
**
******************************************************************************/
typedef enum
{
    FE_DISEQC_NONE,      //!< No DiSEqC support.
    FE_DISEQC_1_0,       //!< DiSEqC version 1.0.
    FE_DISEQC_1_1_DTH,   //!< DiSEqC version 1.1 DTH.
    FE_DISEQC_1_1_SMATV, //!< DiSEqC version 1.1 SMATV.
    FE_DISEQC_1_2,       //!< DiSEqC version 1.2.
    FE_DISEQC_2_0        //!< DiSEqC version 2.0.
} FE_DISEQC_E;

/*!
*******************************************************************************
**
** \brief Polarisation types for satellite systems.
**
******************************************************************************/
typedef enum
{
    FE_POL_HOR,  //!< Horizontal.
    FE_POL_VER,  //!< Vertical.
    FE_POL_LEFT, //!< Circular left.
    FE_POL_RIGHT //!< Circular right.
} FE_POL_E;

/*!
*******************************************************************************
**
** \brief Forward error correction (FEC) types.
**
******************************************************************************/
typedef enum
{
    FE_FEC_NONE, //!< Not defined.
    FE_FEC_1_4,  //!< FEC 1/4.
    FE_FEC_1_3,  //!< FEC 1/3.
    FE_FEC_1_2,  //!< FEC 1/2.
    FE_FEC_3_5,  //!< FEC 3/5.
    FE_FEC_2_3,  //!< FEC 2/3.
    FE_FEC_3_4,  //!< FEC 3/4.
    FE_FEC_4_5,  //!< FEC 4/5.
    FE_FEC_5_6,  //!< FEC 5/6.
    FE_FEC_7_8,  //!< FEC 7/8.
    FE_FEC_8_9,  //!< FEC 8/9.
    FE_FEC_9_10, //!< FEC 9/10.
    FE_FEC_AUTO  //!< Automatic detection.
} FE_FEC_E;

/*!
*******************************************************************************
**
** \brief Roll-off factor for satellite systems.
**
******************************************************************************/
typedef enum
{
    FE_ROLL_OFF_NONE, //!< Not defined.
    FE_ROLL_OFF_0_20, //!< Roll-off alpha of 0.20
    FE_ROLL_OFF_0_25, //!< Roll-off alpha of 0.25
    FE_ROLL_OFF_0_35, //!< Roll-off alpha of 0.35
    FE_ROLL_OFF_AUTO  //!< Automatic detection.
} FE_ROLL_OFF_E;

/*!
*******************************************************************************
**
** \brief IQ types.
**
******************************************************************************/
typedef enum
{
    FE_IQ_NONE,     //!< Not defined.
    FE_IQ_NORMAL,   //!< Normal, not inverted.
    FE_IQ_INVERTED, //!< Inverted.
    FE_IQ_AUTO      //!< Automatic detection.
} FE_IQ_E;

/*!
*******************************************************************************
**
** \brief West/east flags for satellite systems.
**
******************************************************************************/
typedef enum
{
    FE_WE_FLAG_NONE, //!< Not defined.
    FE_WE_FLAG_WEST, //!< West.
    FE_WE_FLAG_EAST  //!< East.
} FE_WE_FLAG_E;

/*!
*******************************************************************************
**
** \brief Modulation types.
**
******************************************************************************/
typedef enum
{
    FE_MOD_NONE,    //!< Not defined.
    FE_MOD_QPSK,    //!< QPSK.
    FE_MOD_QPSK_S2, //!< QPSK S2.
    FE_MOD_8PSK,    //!< 8PSK.
    FE_MOD_QAM_16,  //!< QAM 16.
    FE_MOD_QAM_32,  //!< QAM 32.
    FE_MOD_QAM_64,  //!< QAM 64.
    FE_MOD_QAM_128, //!< QAM 129.
    FE_MOD_QAM_256, //!< QAM 256.
    FE_MOD_AUTO     //!< Automatic detection.
} FE_MOD_E;

/*!
*******************************************************************************
**
** \brief Channel bandwidth types for terrestrial systems.
**
******************************************************************************/
typedef enum
{
    FE_BANDWIDTH_NONE,       //!< Not defined.
	//FE_BANDWIDTH_5_MHZ,      //!< Bandwidth 5 MHz.
    FE_BANDWIDTH_6_MHZ,      //!< Bandwidth 6 MHz.
    FE_BANDWIDTH_7_MHZ,      //!< Bandwidth 7 MHz.
	FE_BANDWIDTH_8_MHZ,      //!< Bandwidth 8 MHz.
	FE_BANDWIDTH_1_7_MHZ,    //!< Bandwidth 1.7 MHz(DVB-T2 only).
    FE_BANDWIDTH_10_MHZ,     //!< Bandwidth 10 MHz(DVB-T2 only).
	FE_BANDWIDTH_ISDB_6_MHZ, //!< Bandwidth 6 MHz(ISDB-T only).
	FE_BANDWIDTH_ISDB_7_MHZ, //!< Bandwidth 7 MHz(ISDB-T only).  Reserved.
	FE_BANDWIDTH_ISDB_8_MHZ, //!< Bandwidth 8 MHz(ISDB-T only). Reserved.
	FE_BANDWIDTH_AUTO        //!< Automatic detection.
} FE_BANDWIDTH_E;

/*!
*******************************************************************************
**
** \brief OFDM transmission (carrier) modes for terrestrial systems.
**
******************************************************************************/
typedef enum
{
    FE_TRANS_MODE_NONE, //!< Not defined.
    FE_TRANS_MODE_2K,   //!< Transmission mode 2K.
    FE_TRANS_MODE_8K,   //!< Transmission mode 8K.
    FE_TRANS_MODE_AUTO  //!< Automatic detection.
} FE_TRANS_MODE_E;

/*!
*******************************************************************************
**
** \brief Guard interval types for terrestrial systems.
**
******************************************************************************/
typedef enum
{
    FE_GUARD_INTERVAL_NONE, //!< Not defined.
    FE_GUARD_INTERVAL_1_4,  //!< Guard interval 1/4.
    FE_GUARD_INTERVAL_1_8,  //!< Guard interval 1/8.
    FE_GUARD_INTERVAL_1_16, //!< Guard interval 1/16.
    FE_GUARD_INTERVAL_1_32, //!< Guard interval 1/32.
    FE_GUARD_INTERVAL_AUTO  //!< Automatic detection.
} FE_GUARD_INTERVAL_E;

/*!
*******************************************************************************
**
** \brief Hierarchy types for terrestrial systems.
**
******************************************************************************/
typedef enum
{
    FE_HIERARCHY_NONE, //!< No hierarchy.
    FE_HIERARCHY_1,    //!< Hierarchy 1.
    FE_HIERARCHY_2,    //!< Hierarchy 2.
    FE_HIERARCHY_4,    //!< Hierarchy 4.
    FE_HIERARCHY_AUTO, //!< Automatic detection.
} FE_HIERARCHY_E;

/*!
*******************************************************************************
**
** \brief Front-end status types.
**
******************************************************************************/
typedef enum
{
    FE_STATUS_UNKNOWN,    //!< Current status is unknown.
    FE_STATUS_LOCKED,     //!< Front-end is locked.
    FE_STATUS_CARRIER,    //!< A carrier is available, but the FE is not locked.
    FE_STATUS_SIGNAL,     //!< A signal is available.
    FE_STATUS_NOT_LOCKED, //!< Front-end is not locked.
    FE_STATUS_SCANNING,   //!< Scanning is in progress.
    FE_STATUS_TUNING,     //!< Tuning is in progress.
    FE_STATUS_BUSY        //!< Front-end is currently busy.
} FE_STATUS_E;

/*!
*******************************************************************************
**
** \brief Tuning parameters for satellite systems.
**
** \sa    FE_TUNE_PARAMS_S
**
******************************************************************************/
typedef struct
{
    uint32_t        frequency;       //!< Transponder frequency [kHz].
    uint32_t        ifFrequency;     //!< IF frequency [MHz] DiSEqC 1.1 SMATV.
    uint16_t        symbolRate;      //!< Symbol rate [kSamples/s], 1000-50000.
    FE_FEC_E        fec;             //!< Inner FEC.
    FE_POL_E        pol;             //!< Polarisation.
    uint16_t        satID;           //!< Satellite ID.
    uint16_t        orbitalPosition; //!< Orbital position [1/10 deg], 0-700.
    FE_WE_FLAG_E    westEastFlag;    //!< West east flag.
    FE_IQ_E         iq;              //!< Inversion mode.
    FE_MOD_E        modulation;      //!< Modulation type.
    FE_ROLL_OFF_E   rollOff;         //!< Roll-off factor.
} FE_TUNE_SAT_S;

/*!
*******************************************************************************
**
** \brief Tuning parameters for cable systems.
**
** \sa    FE_TUNE_PARAMS_S
**
******************************************************************************/
typedef struct
{
    uint32_t     frequency;  //!< Frequency [Hz].
    uint16_t     symbolRate; //!< Symbol rate [kSamples/s], 0-7000.
    FE_MOD_E     modulation; //!< Modulation type.
    FE_FEC_E     fec;        //!< Inner FEC.
    FE_IQ_E      iq;         //!< Inversion mode.
} FE_TUNE_CABLE_S;

/*!
*******************************************************************************
**
** \brief Tuning parameters for terrestrial systems.
**
** \sa    FE_TUNE_PARAMS_S
**
******************************************************************************/
typedef struct
{
    uint32_t            frequency;     //!< Frequency [Hz].
    FE_BANDWIDTH_E      bandwidth;     //!< Bandwidth.
    FE_FEC_E            codeRateHP;    //!< Code rate HP stream.
    FE_FEC_E            codeRateLP;    //!< Code rate LP stream.
    FE_MOD_E            modulation;    //!< Constellation.
    FE_TRANS_MODE_E     transmission;  //!< Transmission mode.
    FE_GUARD_INTERVAL_E guardInterval; //!< Guard interval.
    FE_HIERARCHY_E      hierarchy;     //!< Hierarchy information.
    FE_IQ_E             iq;            //!< Inversion mode.
} FE_TUNE_TERR_S;

/*!
*******************************************************************************
**
** \brief Tuning parameters.
**
**        This union structure summarizes the tuning parameters for all
**        supported delivery systems.
**
** \sa    FE_DEVICE_DATA_S   <br>
**        FE_SCAN_PARAMS_S   <br>
**        FE_GetTuneParams() <br>
**        FE_Tune()
**
******************************************************************************/
typedef union
{
    FE_TUNE_SAT_S   sat;   //!< Parameters for satellite systems.
    FE_TUNE_CABLE_S cable; //!< Parameters for cable systems.
    FE_TUNE_TERR_S  terr;  //!< Parameters for terrestrial systems.
} FE_TUNE_PARAMS_S;

/*!
*******************************************************************************
**
** \brief Scanning parameters.
**
** \sa    FE_DEVICE_DATA_S <br>
**        FE_Scan()
**
******************************************************************************/
typedef struct
{
    uint32_t         startFrequency;  //!< Start frequency [Hz].
    uint32_t         stopFrequency;   //!< Stop frequency [Hz].
    uint16_t         startSymbolRate; //!< Start symbol rate.
    uint16_t         stopSymbolRate;  //!< Stop symbol rate.
    FE_POL_E         pol;             //!< Polarisation.
    uint16_t         satID;           //!< Satellite ID.
    uint32_t*        custom;          //!< Custom pointer to additional data.

    /*!
    **  Callback function which is called frequently while scanning
    **  to check if the scan shall to be aborted.
    */
    uint8_t    (*feScanAbortFct)();

    /*!
    **  Callback function which is called frequently while scanning
    **  to inform the application about the progress.
    */
    void (*feScanProgressFct)(uint16_t progress);

    /*!
    **  Callback function which is called every time after finding a valid
    **  frequency or transponder.
    */
    void (*feScanNotifyFct)(FE_TUNE_PARAMS_S* pFoundTuneParams);
} FE_SCAN_PARAMS_S;

/*!
*******************************************************************************
**
** \brief System parameters.
**
** \sa    FE_OPEN_PARAMS_S     <br>
**        FE_DEVICE_DATA_S     <br>
**        FE_GetSystemParams() <br>
**        FE_SetSystemParams() <br>
**
******************************************************************************/
typedef union
{
    #if 0
    struct /* satellite systems: */
    {
        uint8_t             classic;       //!< Enable classic H/V control with 22KHz and 14/18V.
        uint8_t             toneburst;     //!< Enable toneburst control for 2 satellites (sat ID 0/1).
        FE_DISEQC_E         diseqc;        //!< DiSEqC type.
        uint8_t             diseqc10Rep;   //!< Command repetition for DiSEqC 1.0 (0 = command sent once).
        uint8_t             diseqc11Rep;   //!< Command repetition for DiSEqC 1.1 DTH.
        uint32_t            lowFrequency;  //!< LNB low oscillator frequency [KHz].
        uint32_t            highFrequency; //!< LNB high oscillator frequency [KHz].
        uint8_t             autoFEC;       //!< Enable automatic FEC.
        FE_IQ_E             defaultIQ;     //!< Default inversion mode.
        uint32_t*           custom;        //!< For custom use.
        uint8_t             motorAddress;  //!< DiSEqC address for motor control.
    } sat;
    #endif
    struct /* satellite systems: */
    {
        uint8_t             classic;       //!< Enable classic H/V control with 22KHz and 14/18V.
        uint8_t             posBy22kHz;    //!< Enable Sat selection with 22KHz and H/V control 14/18V.
        uint8_t             toneburst;     //!< Enable toneburst control for 2 satellites (sat ID 0/1).
        FE_DISEQC_E         diseqc;        //!< DiSEqC type.
        uint8_t             diseqc10Rep;   //!< Command repetition for DiSEqC 1.0 (0 = command sent once).
        uint8_t             diseqc11Rep;   //!< Command repetition for DiSEqC 1.1 DTH.
        uint8_t             ifFrequency;   //!< IF Frequency DiSEqC 1.1 SMATV.
        uint32_t            lowFrequency;  //!< LNB low oscillator frequency [KHz].
        uint32_t            highFrequency; //!< LNB high oscillator frequency [KHz].
        uint8_t             autoFEC;       //!< Enable automatic FEC.
        FE_IQ_E             defaultIQ;     //!< Default inversion mode.
        uint32_t*           custom;        //!< For custom use.
        uint8_t             motorAddress;  //!< DiSEqC address for motor control.
    } sat;

    struct /* cable systems: */
    {
        uint8_t         autoModulation; //!< Enable automatic moduation detection.
        uint8_t         autoFEC;        //!< Enable automatic FEC.
        FE_IQ_E         defaultIQ;      //!< Default inversion mode.
        uint32_t*       custom;         //!< For custom use.
    } cable;

    struct /* terrestrial systems: */
    {
        uint8_t         autoBandwidth;     //!< Enable automatic bandwidth handling.
        uint8_t         autoFEC;           //!< Enable automatic FEC handling.
        uint8_t         autoModulation;    //!< Enable automatic modulation handling.
        uint8_t         autoTransMode;     //!< enable automatic transmission mode handling.
        uint8_t         autoGuardInterval; //!< Enable automatic guard interval handling.
        uint8_t         autoHierarchy;     //!< Enable automatic hierarchy handling.
        FE_IQ_E         defaultIQ;         //!< Default inversion mode.
        uint32_t*       custom;            //!< For custom use.
    } terr;
} FE_SYSTEM_PARAMS_S;


/*!
*************************************************************************
** \brief Version of the FE driver.
**
** This number consists of a \b major version number stored in bits
** 16 to 31 (upper word) and a \b minor version number stored in bits
** 0 to 15 (lower word).
** - The \b major number has to be increased when a parameter change
**   occurs for the existing driver's API after its first release.
** - The \b minor number has to be increased when functions are added to
**   the existing driver's API after its first release.
**
** \note This value has to be assigend to the \b version field of the
**       FAPI_FE_OpenParamsT structure.
**
*************************************************************************
*/

typedef enum {
    FE_VERSION = 0x00010000   //!< The current driver version
} FE_VersionEnumT;

/*!
*************************************************************************
** \brief Open parameters for the FE driver.
**
** This data structure covers all parameters which need to be specified
** when an instance of the driver is opened.
**
*************************************************************************
*/
typedef struct {
    /*!
       The version of the driver.
       \ref FE_VersionEnumT "FAPI_FE_VERSION".
    */
    FE_VersionEnumT version;
    uint8_t         slotNum;
} FE_OpenParamsT;

//***********************************************************************
//***********************************************************************
//** Global Data
//***********************************************************************
//***********************************************************************
extern FAPI_SYS_DriverT FE_Driver;

//***********************************************************************
//***********************************************************************
//** API Functions
//***********************************************************************
//***********************************************************************

#ifdef __cplusplus
extern "C" {
#endif
int32_t          FE_Init(void);
void             FE_Exit(void);
uint32_t         FE_Isr(void);
FAPI_SYS_HandleT FE_Open(const FE_OpenParamsT* paramsPtr, int32_t* errorCodePtr);
int32_t          FE_Close(FAPI_SYS_HandleT handle);
int32_t          FE_DetectSlot(uint8_t slotNum);
int32_t          FE_DetectAll(void);
int32_t          FE_GetSystemType(FE_HANDLE handle, FE_SYSTEM_TYPE_E *pType);
int32_t          FE_GetAFC(FE_HANDLE handle, int32_t* pAFC);
int32_t          FE_GetBER(FE_HANDLE handle, uint32_t* pBER);
int32_t          FE_GetFEC(FE_HANDLE handle, FE_FEC_E* pFEC);
int32_t          FE_GetOpenNumber(uint8_t* pOpenNumber);
int32_t          FE_GetSignalQuality(FE_HANDLE handle, uint16_t* pQuality);
int32_t          FE_GetSignalStrength(FE_HANDLE handle, uint16_t* pStrength);
int32_t          FE_GetStatus(FE_HANDLE handle, FE_STATUS_E* pStatus);
int32_t          FE_GetSystemParams(FE_HANDLE handle,
                                    FE_SYSTEM_PARAMS_S* pSystemParams);
int32_t          FE_GetTuneParams(FE_HANDLE handle,
                                  FE_TUNE_PARAMS_S *pTuneParams);
int32_t          FE_Reset(FE_HANDLE handle);
int32_t          FE_Scan(FE_HANDLE handle, FE_SCAN_PARAMS_S *pScanParams);
int32_t          FE_SendDiseqcCommand(FE_HANDLE handle, uint8_t length,
                     uint8_t* pCommands, uint8_t* pReturnLength,
                     uint8_t* pReturnBufer);
int32_t          FE_SetSystemParams(FE_HANDLE handle,
                                    FE_SYSTEM_PARAMS_S* pSystemParams);
int32_t          FE_Tune(FE_HANDLE handle, FE_TUNE_PARAMS_S *pTuneParams);
int32_t          FE_SetAntennaPower(FE_HANDLE handle, uint8_t    antPow);

/* DiSEqC 1.2 motor control */
int32_t          FE_DISEQC12_Halt(FAPI_SYS_HandleT  handle);
int32_t          FE_DISEQC12_ReCalcPos(FAPI_SYS_HandleT  handle);
int32_t          FE_DISEQC12_LimitsOff(FAPI_SYS_HandleT  handle);
int32_t          FE_DISEQC12_SetLimitEast(FAPI_SYS_HandleT  handle);
int32_t          FE_DISEQC12_SetLimitWest(FAPI_SYS_HandleT  handle);
int32_t          FE_DISEQC12_DriveMotorEast(FAPI_SYS_HandleT  handle, uint8_t steps);
int32_t          FE_DISEQC12_DriveMotorWest(FAPI_SYS_HandleT  handle, uint8_t steps);
int32_t          FE_DISEQC12_StoreSatPos(FAPI_SYS_HandleT  handle, uint8_t satNum);
int32_t          FE_DISEQC12_GotoSat(FAPI_SYS_HandleT  handle, uint8_t satNum);
int32_t          FE_DISEQC12_Goto00(FAPI_SYS_HandleT  handle);

#ifdef __cplusplus
}
#endif
#endif
