#ifndef _FE_PRIV_H
#define _FE_PRIV_H

#include "fapex/abst_types.h"
#include "fapex/abst_libi2c.h"
#include "fe.h"

#define  FE_DEBUG

#if(0)
#include "fapi/sys_services.h"
#define FE_DPrintf(ARGS...) FAPI_SYS_PRINT_MSG(ARGS)
#else
#ifdef  FE_DEBUG
#define FE_DPrintf(ARGS...) FAPI_SYS_PRINT_MSG(ARGS)
#else
#define FE_DPrintf(ARGS...)
#endif
#endif

#define FE_OK                   FAPEX_OK
#define FE_I2C_ERR               1
#define FE_TUNER_ERR             2
#define FE_DEMOD_ERR             3
#define FE_LNB_ERR               4
#define FE_WRONG_PARAM           5


#define FE_I2C_COMMON_PROTOCOL  FAPEX_I2C_COMMON_PROTOCOL
#define FE_I2C_RESTART_PROTOCOL FAPEX_I2C_RESTART_PROTOCOL

/*!
 * Min LNB frequency
 */
#define FE_MIN_FREQ                950000

/*!
 * Max LNB frequency
 */
#define FE_MAX_FREQ               2240000

/*!
 *  switching frequency from low- to high-band
 */
#define FE_BAND_SW                1950000

/*!
 *  Min symbol rate
 */
#define FE_MIN_SYM                   1000

/*!
 *  Max symbol rate
 */
#define FE_MAX_SYM                  45000


#define BLINDSCAN_MIN_SYMB     2000 // kHz
#define BLINDSCAN_MAX_SYMB    45000 // kHz
#define BLINDSCAN_MIN_FREQ   950000 // kHz
#define BLINDSCAN_MAX_FREQ  2150000 // kHz


typedef enum
{
    FE_RESET_PULSE,
    FE_RESET_HIGH,

} FE_RESET_E;

typedef struct
{
    uint8_t      txData[8];
    uint8_t      txSize;

    uint8_t      rxEnabled;
    uint8_t      rxData[8];
    uint8_t      rxSize;

} FE_DISEQC_CMD;

typedef enum
{
    FE_22KHZ_OFF = 0,
    FE_22KHZ_ON,
    FE_22KHZ_TONE_0,
    FE_22KHZ_TONE_1

} FE_22KHZ_E;

typedef enum
{
    FE_LNB_13V = 0,
    FE_LNB_18V,
    FE_LNB_OFF

} FE_LNB_VOLTAGE_E;


/*!
*******************************************************************************
**
** \brief Front-end device data parameters.
**
** \sa    FE_OPEN_PARAMS_S
**
******************************************************************************/
typedef struct
{
    uint32_t          id;                /* handle ID */
    uint32_t          inUse;             /* specifies if handle is in use */
    FE_OpenParamsT    openParams;        /* open params of the handle */

    /* Delivery system type of front-end. */
    FE_SYSTEM_TYPE_E systemType;
    /* System parameters. */
    FE_SYSTEM_PARAMS_S pSystemParams;
    /*! I2C device address for tuner. */
    uint8_t i2cAddrTuner;
    /*! I2C device address for demodulator (0 = same as tuner). */
    uint8_t i2cAddrDemod;
    /*! Device Identifier of I2C channel for communication. */
    int32_t i2cDev;
    /*! Identifier of I2C channel for communication. */
    int32_t i2cId;
    /*! Device specific function for opening the front-end. */
    int32_t (*feOpenFct)(uint8_t, FE_HANDLE *);
    /*! Device specific function for closing the front-end. */
    int32_t (*feCloseFct)(uint8_t);
    /*! Device specific function for setting the system parameters. */
    int32_t (*feSetSystemParamsFct)(uint8_t, FE_SYSTEM_PARAMS_S*);
    /*! Device specific function for resetting the front-end. */
    int32_t (*feResetFct)(uint8_t);
    /*! Device specific function for tuning the front-end. */
    int32_t (*feTuneFct)(uint8_t, FE_TUNE_PARAMS_S*);
    /*! Device specific function for scanning the front-end. */
    int32_t (*feScanFct)(uint8_t, FE_SCAN_PARAMS_S*);
    /*! Device specific function for sending DiSEqC commands. */
    int32_t (*feSendDiseqcCommandFct)(uint8_t, uint8_t, uint8_t*, uint8_t*, uint8_t*);
    /*! Device specific function for getting the current status. */
    int32_t (*feGetStatusFct)(uint8_t, FE_STATUS_E*);
    /*! Device specific function for getting the current FEC value. */
    int32_t (*feGetFECFct)(uint8_t, FE_FEC_E*);
    /*! Device specific function for getting the current AFC value. */
    int32_t (*feGetAFCFct)(uint8_t, int32_t*);
    /*! Device specific function for getting the current BER value. */
    int32_t (*feGetBERFct)(uint8_t, uint32_t*);
    /*! Device specific function for getting the current signal quality. */
    int32_t (*feGetSignalQualityFct)(uint8_t, uint16_t*);
    /*! Device specific function for getting the current signal strength. */
    int32_t (*feGetSignalStrengthFct)(uint8_t, uint16_t*);
    /*! Device specific function for getting the system parameters. */
    int32_t (*feGetSystemParamsFct)(uint8_t, FE_SYSTEM_PARAMS_S*);
    /*! Device specific function for getting the current tuning parameters. */
    int32_t (*feGetTuneParamsFct)(uint8_t, FE_TUNE_PARAMS_S*);
    /*! Device specific function to enable/disable antenna power. */
    int32_t (*feSetAntennaPowerFct)(uint8_t, uint8_t);
    /*! Device specific function to control 22kHz. */
    int32_t (*feControl22kHz)(uint8_t, FE_22KHZ_E);
    /*! Device specific function to control LNB-Voltage. */
    int32_t (*feControlLnbVoltage)(uint8_t, FE_LNB_VOLTAGE_E);
    uint8_t  isPresent;
    uint8_t  slotNum;
    uint8_t  tunerType;
} feHandleT;

extern feHandleT feHandleArray[FE_MAX_NUM];

void     FE_Wait(uint32_t millis);
int32_t  FE_ControlFeReset(uint8_t slotNum, FE_RESET_E type);
int32_t  FE_ControlAntPow(uint8_t slotNum, uint8_t doPower);
uint32_t FE_GetMillis(void);
int32_t  FE_I2C_SetProtocol(int32_t hp, FAPEX_I2C_ProtocolT protocol);
void*    FE_CreateSemaphore(const uint8_t* name);
void     FE_GetSemaphore(void* id);
void     FE_SetSemaphore(void* id);
void     FE_DestroySemaphore(void* id);
int32_t  FE_I2C_Read(const int32_t id, const uint8_t msSlaveAddress,
                     uint8_t* txDataPtr, const uint32_t txDataLength,
                     uint8_t* rxDataPtr, uint32_t rxDataLength);
int32_t  FE_I2C_Write(const int32_t id,
                      const uint8_t msSlaveAddress,
                      uint8_t* txDataPtr,
                      const uint32_t txDataLength);
#ifdef _LINUX_FAPI
//int32_t FAPI_I2C_Write(const FAPI_SYS_HandleT handle, const uint8_t msSlaveAddress,
int32_t FAPI_I2C_Write(const int32_t handle, const uint8_t msSlaveAddress,

                       uint8_t* txDataPtr, const uint32_t txDataLength);
//int32_t FAPI_I2C_Read(const FAPI_SYS_HandleT handle, const uint8_t msSlaveAddress,
int32_t FAPI_I2C_Read(const int32_t handle, const uint8_t msSlaveAddress,
                      uint8_t* txDataPtr, const uint32_t txDataLength,
                      uint8_t* rxDataPtr, uint32_t rxDataLength);
#endif

#if 0
void     FE_PrepareDiseqcCmd(FE_DISEQC_E diseqcType,FE_TUNE_SAT_S *tuneParams,
                           uint8_t isHighBand, uint8_t *cmd, uint8_t *lenCmd);
#endif
int32_t  FE_SetSignalRoute(feHandleT *hp, FE_TUNE_PARAMS_S* pTuneParams);
void     FE_PrepareDiseqcCmd(FE_SYSTEM_PARAMS_S* h,
                             FE_DISEQC_E diseqcType,
                             FE_TUNE_SAT_S *tuneParams,
                             FE_DISEQC_CMD *msg);
void    FE_SetTuneTimeout(uint8_t index, uint32_t symbolRate);
uint8_t FE_IsTuneTimeout(uint8_t index);


#endif
