/*
 * jdapimin.c
 *
 * Copyright (C) 1994-1998, Thomas G. Lane.
 * This file is part of the Independent JPEG Group's software.
 * For conditions of distribution and use, see the accompanying README file.
 *
 * This file contains application interface code for the decompression half
 * of the JPEG library.  These are the "minimum" API routines that may be
 * needed in either the normal full-decompression case or the
 * transcoding-only case.
 *
 * Most of the routines intended to be called directly by an application
 * are in this file or in jdapistd.c.  But also see jcomapi.c for routines
 * shared by compression and decompression, and jdtrans.c for the transcoding
 * case.
 */

#define JPEG_INTERNALS
#include "jinclude.h"
#include <jpeg/jpeglib.h>


/*
 * Initialization of a JPEG decompression object.
 * The error manager must already be set up (in case memory manager fails).
 */

void jpeg_CreateDecompress(j_decompress_ptr cinfo)
{
  int        i;

  cinfo->mem= NULL;        /* so jpeg_destroy knows mem mgr not called */

  /* Initialize a memory manager instance for this object */
  jinit_memory_mgr(cinfo);

  /* Zero out pointers to permanent structures. */
  cinfo->src = NULL;

  for(i= 0; i < NUM_QUANT_TBLS; i++)
    cinfo->quant_tbl_ptrs[i] = NULL;

  for(i= 0; i < NUM_HUFF_TBLS; i++)
  {
    cinfo->dc_huff_tbl_ptrs[i] = NULL;
    cinfo->ac_huff_tbl_ptrs[i] = NULL;
  }

  /* Initialize marker processor so application can override methods
   * for COM, APPn markers before calling jpeg_read_header.
   */
  jinit_marker_reader(cinfo);

  /* And initialize the overall input controller. */
  jinit_input_controller(cinfo);

  /* OK, I'm ready */
  cinfo->global_state= DSTATE_START;
}



/*
 * Set default decompression parameters.
 */

void default_decompress_parms(j_decompress_ptr cinfo)
{
  /* Guess the input colorspace, and set output colorspace accordingly. */
  /* (Wish JPEG committee had provided a real way to specify this...) */
  /* Note application may override our guesses. */
  switch (cinfo->num_components)
  { case 1:
      cinfo->jpeg_color_space= JCS_GRAYSCALE;
      break;
    
    case 3:
      if(cinfo->saw_JFIF_marker)
        cinfo->jpeg_color_space= JCS_YCbCr; /* JFIF implies YCbCr */
      else if (cinfo->saw_Adobe_marker)
      { switch (cinfo->Adobe_transform)
        { case 0:
            cinfo->jpeg_color_space= JCS_RGB;
            break;
          case 1:
            cinfo->jpeg_color_space = JCS_YCbCr;
            break;
          default:
            cinfo->jpeg_color_space= JCS_YCbCr; /* assume it's YCbCr */
            break;
         }
      }
      else
      { int cid0= cinfo->comp_info[0].component_id;
        int cid1= cinfo->comp_info[1].component_id;
        int cid2= cinfo->comp_info[2].component_id;

        if(cid0 == 1 && cid1 == 2 && cid2 == 3)
          cinfo->jpeg_color_space = JCS_YCbCr; /* assume JFIF w/out marker */
        else if(cid0 == 82 && cid1 == 71 && cid2 == 66)
          cinfo->jpeg_color_space = JCS_RGB; /* ASCII 'R', 'G', 'B' */
        else
          cinfo->jpeg_color_space = JCS_YCbCr; /* assume it's YCbCr */
      }
      break;
    
    case 4:
      if(cinfo->saw_Adobe_marker)
      { switch(cinfo->Adobe_transform)
        { case 0:
            cinfo->jpeg_color_space= JCS_CMYK;
            break;
          case 2:
            cinfo->jpeg_color_space = JCS_YCCK;
            break;
          default:
            cinfo->jpeg_color_space = JCS_YCCK; /* assume it's YCCK */
            break;
        }
      }
      else
        cinfo->jpeg_color_space = JCS_CMYK;
      break;
    
    default:
      cinfo->jpeg_color_space= JCS_UNKNOWN;
      break;
  }

  /* Set defaults for other decompression parameters. */
  cinfo->scale_num= 1;        
  cinfo->scale_denom= 1;
  cinfo->buffered_image= FALSE;
  cinfo->dct_method= JDCT_ISLOW;
  cinfo->do_fancy_upsampling= TRUE;
  cinfo->do_block_smoothing= TRUE;
}


/*
 * Decompression startup: read start of JPEG datastream to see what's there.
 * Need only initialize JPEG object and supply a data source before calling.
 *
 * This routine will read as far as the first SOS marker (ie, actual start of
 * compressed data), and will save all tables and parameters in the JPEG
 * object.  It will also initialize the decompression parameters to default
 * values, and finally return JPEG_HEADER_OK.  On return, the application may
 * adjust the decompression parameters and then call jpeg_start_decompress.
 * (Or, if the application only wanted to determine the image parameters,
 * the data need not be decompressed.  In that case, call jpeg_abort or
 * jpeg_destroy to release any temporary space.)
 * If an abbreviated (tables only) datastream is presented, the routine will
 * return JPEG_HEADER_TABLES_ONLY upon reaching EOI.  The application may then
 * re-use the JPEG object to read the abbreviated image datastream(s).
 * It is unnecessary (but OK) to call jpeg_abort in this case.
 * The JPEG_SUSPENDED return code only occurs if the data source module
 * requests suspension of the decompressor.  In this case the application
 * should load more source data and then re-call jpeg_read_header to resume
 * processing.
 * If a non-suspending data source is used and require_image is TRUE, then the
 * return code need not be inspected since only JPEG_HEADER_OK is possible.
 *
 * This routine is now just a front end to jpeg_consume_input, with some
 * extra error checking.
 */

int jpeg_read_header(j_decompress_ptr cinfo)
{
  int retcode;

  retcode= jpeg_consume_input(cinfo);

  switch(retcode)
  { case JPEG_REACHED_SOS:
      retcode= JPEG_HEADER_OK;
      break;
    case JPEG_REACHED_EOI:
      retcode= JPEG_HEADER_TABLES_ONLY;
      break;
    case JPEG_SUSPENDED:
      break;
  }
  return retcode;
}


/*
 * Consume data in advance of what the decompressor requires.
 * This can be called at any time once the decompressor object has
 * been created and a data source has been set up.
 *
 * This routine is essentially a state machine that handles a couple
 * of critical state-transition actions, namely initial setup and
 * transition from header scanning to ready-for-start_decompress.
 * All the actual input is done via the input controller's consume_input
 * method.
 */

int jpeg_consume_input(j_decompress_ptr cinfo)
{
  int        retcode= JPEG_SUSPENDED;

  switch(cinfo->global_state)
  { case DSTATE_START:
      (*cinfo->inputctl->reset_input_controller) (cinfo);
      /* Initialize application's data source module */
      (*cinfo->src->init_source) (cinfo);
      cinfo->global_state= DSTATE_INHEADER;
      /*FALLTHROUGH*/
    case DSTATE_INHEADER:
      retcode= (*cinfo->inputctl->consume_input)(cinfo);
      if(retcode == JPEG_REACHED_SOS)
      { /* Found SOS, prepare to decompress */
        default_decompress_parms(cinfo);
        /* Set global state: ready for start_decompress */
        cinfo->global_state= DSTATE_READY;
      }
      break;
    case DSTATE_READY:
      /* Can't advance past first SOS until start_decompress is called */
      retcode= JPEG_REACHED_SOS;
      break;
    case DSTATE_PRELOAD:
    case DSTATE_PRESCAN:
    case DSTATE_SCANNING:
    case DSTATE_RAW_OK:
    case DSTATE_BUFIMAGE:
    case DSTATE_BUFPOST:
    case DSTATE_STOPPING:
      retcode= (*cinfo->inputctl->consume_input)(cinfo);
      break;
  }
  return retcode;
}


/*
 * Finish JPEG decompression.
 *
 * This will normally just verify the file trailer and release temp storage.
 *
 * Returns FALSE if suspended.  The return value need be inspected only if
 * a suspending data source is used.
 */

boolean jpeg_finish_decompress(j_decompress_ptr cinfo)
{
  if((cinfo->global_state == DSTATE_SCANNING || cinfo->global_state == DSTATE_RAW_OK) && ! cinfo->buffered_image)
  { /* Terminate final pass of non-buffered mode */
    (*cinfo->master->finish_output_pass)(cinfo);
    cinfo->global_state= DSTATE_STOPPING;
  }
  else if(cinfo->global_state == DSTATE_BUFIMAGE)
  { /* Finishing after a buffered-image operation */
    cinfo->global_state = DSTATE_STOPPING;
  }
  return TRUE;
}
