/*
 * jdapistd.c
 *
 * Copyright (C) 1994-1996, Thomas G. Lane.
 * This file is part of the Independent JPEG Group's software.
 * For conditions of distribution and use, see the accompanying README file.
 *
 * This file contains application interface code for the decompression half
 * of the JPEG library.  These are the "standard" API routines that are
 * used in the normal full-decompression case.  They are not used by a
 * transcoding-only application.  Note that if an application links in
 * jpeg_start_decompress, it will end up linking in the entire decompressor.
 * We thus must separate this file from jdapimin.c to avoid linking the
 * whole decompression library into a transcoder.
 */

#define JPEG_INTERNALS
#include "jinclude.h"
#include <jpeg/jpeglib.h>


/* Forward declarations */
void output_pass_setup JPP((j_decompress_ptr cinfo));


/*
 * Decompression initialization.
 * jpeg_read_header must be completed before calling this.
 *
 * If a multipass operating mode was selected, this will do all but the
 * last pass, and thus may take a great deal of time.
 *
 */

short jpeg_start_decompress (j_decompress_ptr cinfo)
{ int    retcode;
    
  if(cinfo->global_state == DSTATE_READY)
  { /* First call: initialize master control, select active modules */
    retcode= jinit_master_decompress(cinfo);
    if(retcode)
        return retcode;
      
    if(cinfo->buffered_image)
    { /* No more work here; expecting jpeg_start_output next */
      cinfo->global_state= DSTATE_BUFIMAGE;
      return 0;
    }
    cinfo->global_state= DSTATE_PRELOAD;
  }
  if(cinfo->global_state == DSTATE_PRELOAD)
  { /* If file has multiple scans, absorb them all into the coef buffer */
    if(cinfo->inputctl->has_multiple_scans)
    { for(;;)
      { /* Absorb some more input */
        retcode= (*cinfo->inputctl->consume_input) (cinfo);
        if(retcode == JPEG_REACHED_EOI)
          break;
      }
    }
    cinfo->output_scan_number= cinfo->input_scan_number;
  }
  else if(cinfo->global_state != DSTATE_PRESCAN)
    return JERR_BAD_STATE;
  /* Perform any dummy output passes, and set up for the final pass */
  output_pass_setup(cinfo);
  return 0;
}


/*
 * Set up for an output pass, and perform any dummy pass(es) needed.
 * Common subroutine for jpeg_start_decompress and jpeg_start_output.
 * Entry: global_state = DSTATE_PRESCAN only if previously suspended.
 * Exit: If done, returns TRUE and sets global_state for proper output mode.
 *       If suspended, returns FALSE and sets global_state = DSTATE_PRESCAN.
 */

void output_pass_setup(j_decompress_ptr cinfo)
{ if(cinfo->global_state != DSTATE_PRESCAN)
  { /* First call: do pass setup */
    (*cinfo->master->prepare_for_output_pass)(cinfo);
    cinfo->output_scanline = 0;
    cinfo->global_state = DSTATE_PRESCAN;
  }
  /* Ready for application to drive output pass through
   * jpeg_read_scanlines or jpeg_read_raw_data.
   */
  cinfo->global_state= DSTATE_SCANNING;
}


/*
 * Read some scanlines of data from the JPEG decompressor.
 *
 * The return value will be the number of lines actually read.
 * This may be less than the number requested in several cases,
 * including bottom of image, data source suspension, and operating
 * modes that emit multiple scanlines at a time.
 *
 * Note: we warn about excess calls to jpeg_read_scanlines() since
 * this likely signals an application programmer error.  However,
 * an oversize buffer (max_lines > scanlines remaining) is not an error.
 */

char jpeg_read_scanlines (j_decompress_ptr cinfo, JPEG_scaler_t * scalerP)
{
  if(cinfo->output_scanline >= cinfo->output_height)
    return 0;

  /* Process some data */

  (*cinfo->main->process_data) (cinfo, scalerP);
  cinfo->output_scanline += 1;
  return 1;
}


