/*
 * jdmarker.c
 *
 * Copyright (C) 1991-1998, Thomas G. Lane.
 * This file is part of the Independent JPEG Group's software.
 * For conditions of distribution and use, see the accompanying README file.
 *
 * This file contains routines to decode JPEG datastream markers.
 * Most of the complexity arises from our desire to support input
 * suspension: if not all of the data for a marker is available,
 * we must exit back to the application.  On resumption, we reprocess
 * the marker.
 */

#define JPEG_INTERNALS
#include "jinclude.h"
#include <jpeg/jpeglib.h>


typedef enum {            /* JPEG marker codes */
  M_SOF0  = 0xc0,
  M_SOF1  = 0xc1,
  M_SOF2  = 0xc2,
  M_SOF3  = 0xc3,
  
  M_SOF5  = 0xc5,
  M_SOF6  = 0xc6,
  M_SOF7  = 0xc7,
  
  M_JPG   = 0xc8,
  M_SOF9  = 0xc9,
  M_SOF10 = 0xca,
  M_SOF11 = 0xcb,
  
  M_SOF13 = 0xcd,
  M_SOF14 = 0xce,
  M_SOF15 = 0xcf,
  
  M_DHT   = 0xc4,
  
  M_DAC   = 0xcc,
  
  M_RST0  = 0xd0,
  M_RST1  = 0xd1,
  M_RST2  = 0xd2,
  M_RST3  = 0xd3,
  M_RST4  = 0xd4,
  M_RST5  = 0xd5,
  M_RST6  = 0xd6,
  M_RST7  = 0xd7,
  
  M_SOI   = 0xd8,
  M_EOI   = 0xd9,
  M_SOS   = 0xda,
  M_DQT   = 0xdb,
  M_DNL   = 0xdc,
  M_DRI   = 0xdd,
  M_DHP   = 0xde,
  M_EXP   = 0xdf,
  
  M_APP0  = 0xe0,
  M_APP1  = 0xe1,
  M_APP2  = 0xe2,
  M_APP3  = 0xe3,
  M_APP4  = 0xe4,
  M_APP5  = 0xe5,
  M_APP6  = 0xe6,
  M_APP7  = 0xe7,
  M_APP8  = 0xe8,
  M_APP9  = 0xe9,
  M_APP10 = 0xea,
  M_APP11 = 0xeb,
  M_APP12 = 0xec,
  M_APP13 = 0xed,
  M_APP14 = 0xee,
  M_APP15 = 0xef,
  
  M_JPG0  = 0xf0,
  M_JPG13 = 0xfd,
  M_COM   = 0xfe,
  
  M_TEM   = 0x01,
  
  M_ERROR = 0x100
} JPEG_MARKER;


/* Private state */

typedef struct {
  struct jpeg_marker_reader pub; /* public fields */
} my_marker_reader;

typedef my_marker_reader * my_marker_ptr;


/*
 * Macros for fetching data from the data source module.
 *
 * At all times, cinfo->src->next_input_byte and ->bytes_in_buffer reflect
 * the current restart point; we update them only when we have reached a
 * suitable place to restart if a suspension occurs.
 */

/* Declare and initialize local copies of input pointer/count */
#define INPUT_VARS(cinfo)  \
    struct jpeg_source_mgr * datasrc = (cinfo)->src;  \
    const JOCTET * next_input_byte = datasrc->next_input_byte;  \
    size_t bytes_in_buffer = datasrc->bytes_in_buffer

/* Unload the local copies --- do this only at a restart boundary */
#define INPUT_SYNC(cinfo)  \
    ( datasrc->next_input_byte = next_input_byte,  \
      datasrc->bytes_in_buffer = bytes_in_buffer )

/* Reload the local copies --- used only in MAKE_BYTE_AVAIL */
#define INPUT_RELOAD(cinfo)  \
    ( next_input_byte = datasrc->next_input_byte,  \
      bytes_in_buffer = datasrc->bytes_in_buffer )

/* Internal macro for INPUT_BYTE and INPUT_2BYTES: make a byte available.
 * Note we do *not* do INPUT_SYNC before calling fill_input_buffer,
 * but we must reload the local copies after a successful fill.
 */
#define MAKE_BYTE_AVAIL(cinfo)  \
    if (bytes_in_buffer == 0) {  \
      (*datasrc->fill_input_buffer) (cinfo);  \
      INPUT_RELOAD(cinfo);  \
    }

/* Read a byte into variable V.
 */    
#define INPUT_BYTE(cinfo,V)  \
        { MAKE_BYTE_AVAIL(cinfo); \
          bytes_in_buffer--; \
          V = GETJOCTET(*next_input_byte++); \
        }    

/* As above, but read two bytes interpreted as an unsigned 16-bit integer.
 * V should be declared unsigned int or perhaps INT32.
 */                
#define INPUT_2BYTES(cinfo,V)  \
         { MAKE_BYTE_AVAIL(cinfo); \
          bytes_in_buffer--; \
          V = ((unsigned int) GETJOCTET(*next_input_byte++)) << 8; \
          MAKE_BYTE_AVAIL(cinfo); \
          bytes_in_buffer--; \
          V += GETJOCTET(*next_input_byte++); \
        }
          
          
/*
 * Routines to process JPEG markers.
 *
 * Entry condition: JPEG marker itself has been read and its code saved
 *   in cinfo->unread_marker; input restart point is just after the marker.
 *
 * Exit: if return TRUE, have read and processed any parameters, and have
 *   updated the restart point to point after the parameters.
 *   If return FALSE, was forced to suspend before reaching end of
 *   marker parameters; restart point has not been moved.  Same routine
 *   will be called again after application supplies more input data.
 *
 * This approach to suspension assumes that all of a marker's parameters
 * can fit into a single input bufferload.  This should hold for "normal"
 * markers.  Some COM/APPn markers might have large parameter segments
 * that might not fit.  If we are simply dropping such a marker, we use
 * skip_input_data to get past it, and thereby put the problem on the
 * source manager's shoulders.  If we are saving the marker's contents
 * into memory, we use a slightly different convention: when forced to
 * suspend, the marker processor updates the restart point to the end of
 * what it's consumed (ie, the end of the buffer) before returning FALSE.
 * On resumption, cinfo->unread_marker still contains the marker code,
 * but the data source will point to the next chunk of marker data.
 * The marker processor must retain internal state to deal with this.
 *
 * Note that we don't bother to avoid duplicate trace messages if a
 * suspension occurs within marker parameters.  Other side effects
 * require more care.
 */


boolean get_soi (j_decompress_ptr cinfo)
/* Process an SOI marker */
{
  /* Reset all parameters that are defined to be reset by SOI */

  cinfo->restart_interval= 0;

  /* Set initial assumptions for colorspace etc */

  cinfo->jpeg_color_space= JCS_UNKNOWN;
  
  cinfo->saw_JFIF_marker= FALSE;
  cinfo->density_unit= 0;
  cinfo->X_density= 1;
  cinfo->Y_density= 1;
  cinfo->saw_Adobe_marker= FALSE;
  cinfo->Adobe_transform= 0;
  cinfo->marker->saw_SOI= TRUE;

  return TRUE;
}


boolean get_sof(j_decompress_ptr cinfo, boolean is_prog, boolean is_arith)
/* Process a SOFn marker */
{
  INT32                    length;
  int                    c, ci;
  jpeg_component_info * compptr;
  INPUT_VARS(cinfo);

  cinfo->progressive_mode= is_prog;
  cinfo->arith_code= is_arith;

  INPUT_2BYTES(cinfo, length);

  INPUT_BYTE(cinfo, cinfo->data_precision);
  INPUT_2BYTES(cinfo, cinfo->image_height);
  INPUT_2BYTES(cinfo, cinfo->image_width);
  INPUT_BYTE(cinfo, cinfo->num_components);

  length -= 8;

  if(cinfo->comp_info == NULL)    /* do only once, even if suspend */
    cinfo->comp_info= (jpeg_component_info *)(*cinfo->mem->alloc_small)(cinfo, JPOOL_PERMANENT, cinfo->num_components * SIZEOF(jpeg_component_info));
  
  for(ci= 0, compptr = cinfo->comp_info; ci < cinfo->num_components; ci++, compptr++)
  { compptr->component_index= ci;
    INPUT_BYTE(cinfo, compptr->component_id);
    INPUT_BYTE(cinfo, c);
    compptr->h_samp_factor = (c >> 4) & 15;
    compptr->v_samp_factor = (c     ) & 15;
    INPUT_BYTE(cinfo, compptr->quant_tbl_no);
  }

  cinfo->marker->saw_SOF= TRUE;

  INPUT_SYNC(cinfo);
  return TRUE;
}


boolean get_sos (j_decompress_ptr cinfo)
/* Process a SOS marker */
{
  INT32                    length;
  int                    i, ci, n, c, cc;
  jpeg_component_info * compptr;
  INPUT_VARS(cinfo);

  INPUT_2BYTES(cinfo, length);
  INPUT_BYTE(cinfo, n); /* Number of components */

   
  cinfo->comps_in_scan= n;

  /* Collect the component-spec parameters */

  for(i= 0; i < n; i++)
  { INPUT_BYTE(cinfo, cc);
    INPUT_BYTE(cinfo, c);
    
    if( cinfo->comp_info != NULL)
    {
        for(ci= 0, compptr= cinfo->comp_info; ci < cinfo->num_components; ci++, compptr++)
        { if(cc == compptr->component_id)
            break;
        }
        
        cinfo->cur_comp_info[i]= compptr;
        compptr->dc_tbl_no= (c >> 4) & 15;
        compptr->ac_tbl_no= (c     ) & 15;
    }
  }

  /* Collect the additional scan parameters Ss, Se, Ah/Al. */
  INPUT_BYTE(cinfo, c);
  cinfo->Ss= c;
  INPUT_BYTE(cinfo, c);
  cinfo->Se= c;
  INPUT_BYTE(cinfo, c);
  cinfo->Ah= (c >> 4) & 15;
  cinfo->Al= (c     ) & 15;

  /* Prepare to scan data & restart markers */
  cinfo->marker->next_restart_num= 0;

  /* Count another SOS marker */
  cinfo->input_scan_number++;

  INPUT_SYNC(cinfo);
  return TRUE;
}


boolean get_dht (j_decompress_ptr cinfo)
/* Process a DHT marker */
{
  INT32            length;
  UINT8            bits[17];
  UINT8            huffval[256];
  int            i, index, count;
  JHUFF_TBL        **htblptr;
  INPUT_VARS(cinfo);

  INPUT_2BYTES(cinfo, length);
  length -= 2;
  
  while(length > 16)
  {
    INPUT_BYTE(cinfo, index);

    bits[0]= 0;
    count= 0;
    for(i= 1; i <= 16; i++)
    { INPUT_BYTE(cinfo, bits[i]);
      count += bits[i];
    }

    length -= 1 + 16;

    for(i= 0; i < count; i++)
      INPUT_BYTE(cinfo, huffval[i]);

    length -= count;

    if(index & 0x10)
    {        /* AC table definition */
      index -= 0x10;
      htblptr= &cinfo->ac_huff_tbl_ptrs[index];
    }
    else
    {            /* DC table definition */
      htblptr= &cinfo->dc_huff_tbl_ptrs[index];
    }

    if(*htblptr == NULL)
      *htblptr = jpeg_alloc_huff_table(cinfo);
  
    MEMCOPY((*htblptr)->bits, bits, SIZEOF((*htblptr)->bits));
    MEMCOPY((*htblptr)->huffval, huffval, SIZEOF((*htblptr)->huffval));
  }

  INPUT_SYNC(cinfo);
  return TRUE;
}


boolean get_dqt (j_decompress_ptr cinfo)
/* Process a DQT marker */
{
  INT32            length;
  int            n, i, prec;
  unsigned int    tmp;
  JQUANT_TBL    *quant_ptr;
  INPUT_VARS(cinfo);

  INPUT_2BYTES(cinfo, length);
  length -= 2;

  while(length > 0)
  {
    INPUT_BYTE(cinfo, n);
    prec= n >> 4;
    n &= 0x0F;
      
    if(cinfo->quant_tbl_ptrs[n] == NULL)
      cinfo->quant_tbl_ptrs[n]= jpeg_alloc_quant_table(cinfo);
    quant_ptr= cinfo->quant_tbl_ptrs[n];

    for(i= 0; i < DCTSIZE2; i++)
    { if(prec)
      {  INPUT_2BYTES(cinfo, tmp); }
      else
      {  INPUT_BYTE(cinfo, tmp); }
      /* We convert the zigzag-order table to natural array order. */
      quant_ptr->quantval[jpeg_natural_order[i]] = (UINT16) tmp;
    }

    length -= DCTSIZE2+1;
    if(prec)
      length -= DCTSIZE2;
  }

  INPUT_SYNC(cinfo);
  return TRUE;
}


boolean get_dri (j_decompress_ptr cinfo)
/* Process a DRI marker */
{
  INT32 length;
  unsigned int tmp;
  INPUT_VARS(cinfo);

  INPUT_2BYTES(cinfo, length);
  
  INPUT_2BYTES(cinfo, tmp);

  cinfo->restart_interval = tmp;

  INPUT_SYNC(cinfo);
  return TRUE;
}


/*
 * Routines for processing APPn and COM markers.
 * These are either saved in memory or discarded, per application request.
 * APP0 and APP14 are specially checked to see if they are
 * JFIF and Adobe markers, respectively.
 */

#define APP0_DATA_LEN    14    /* Length of interesting data in APP0 */
#define APP14_DATA_LEN    12    /* Length of interesting data in APP14 */
#define APPN_DATA_LEN    14    /* Must be the largest of the above!! */


void examine_app0 (j_decompress_ptr cinfo, JOCTET * data, unsigned int datalen, INT32 remaining)
/* Examine first few bytes from an APP0.
 * Take appropriate action if it is a JFIF marker.
 * datalen is # of bytes at data[], remaining is length of rest of marker data.
 */
{
  INT32            totallen= (INT32) datalen + remaining;

  if(datalen >= APP0_DATA_LEN && GETJOCTET(data[0]) == 0x4A && GETJOCTET(data[1]) == 0x46 && GETJOCTET(data[2]) == 0x49 && GETJOCTET(data[3]) == 0x46 && GETJOCTET(data[4]) == 0)
  { /* Found JFIF APP0 marker: save info */
    cinfo->saw_JFIF_marker= TRUE;
    cinfo->density_unit= GETJOCTET(data[7]);
    cinfo->X_density= (GETJOCTET(data[8]) << 8) + GETJOCTET(data[9]);
    cinfo->Y_density= (GETJOCTET(data[10]) << 8) + GETJOCTET(data[11]);
    
    totallen -= APP0_DATA_LEN;
    
  }
}


void examine_app14(j_decompress_ptr cinfo, JOCTET * data, unsigned int datalen, INT32 remaining)
/* Examine first few bytes from an APP14.
 * Take appropriate action if it is an Adobe marker.
 * datalen is # of bytes at data[], remaining is length of rest of marker data.
 */
{
  unsigned int        version, flags0, flags1, transform;

  if(datalen >= APP14_DATA_LEN && GETJOCTET(data[0]) == 0x41 && GETJOCTET(data[1]) == 0x64 && GETJOCTET(data[2]) == 0x6F && GETJOCTET(data[3]) == 0x62 && GETJOCTET(data[4]) == 0x65)
  { /* Found Adobe APP14 marker */
    version= (GETJOCTET(data[5]) << 8) + GETJOCTET(data[6]);
    flags0= (GETJOCTET(data[7]) << 8) + GETJOCTET(data[8]);
    flags1= (GETJOCTET(data[9]) << 8) + GETJOCTET(data[10]);
    transform= GETJOCTET(data[11]);
    cinfo->saw_Adobe_marker= TRUE;
    cinfo->Adobe_transform= (UINT8) transform;
  }
}


boolean get_interesting_appn (j_decompress_ptr cinfo)
/* Process an APP0 or APP14 marker without saving it */
{
  INT32            length;
  JOCTET        b[APPN_DATA_LEN];
  unsigned int    i, numtoread;
  INPUT_VARS(cinfo);

  INPUT_2BYTES(cinfo, length);
  length -= 2;

  /* get the interesting part of the marker data */
  if(length >= APPN_DATA_LEN)
    numtoread = APPN_DATA_LEN;
  else if (length > 0)
    numtoread = (unsigned int) length;
  else
    numtoread = 0;
 
  for(i = 0; i < numtoread; i++)
    INPUT_BYTE(cinfo, b[i]);
  length -= numtoread;

  /* process it */
  switch (cinfo->unread_marker)
  { case M_APP0:
      examine_app0(cinfo, (JOCTET *) b, numtoread, length);
      break;
    case M_APP14:
      examine_app14(cinfo, (JOCTET *) b, numtoread, length);
      break;
  }

  /* skip any remaining data -- could be lots */
  INPUT_SYNC(cinfo);
  if(length > 0)
    (*cinfo->src->skip_input_data) (cinfo, (long) length);

  return TRUE;
}


boolean skip_variable (j_decompress_ptr cinfo)
/* Skip over an unknown or uninteresting variable-length marker */
{
  INT32 length;
  INPUT_VARS(cinfo);

  INPUT_2BYTES(cinfo, length);
  length -= 2;
  
  INPUT_SYNC(cinfo);        /* do before skip_input_data */
  if(length > 0)
    (*cinfo->src->skip_input_data) (cinfo, (long) length);

  return TRUE;
}


/*
 * Find the next JPEG marker, save it in cinfo->unread_marker.
 * Returns FALSE if had to suspend before reaching a marker;
 * in that case cinfo->unread_marker is unchanged.
 *
 * Note that the result might not be a valid marker code,
 * but it will never be 0 or FF.
 */

boolean next_marker(j_decompress_ptr cinfo)
{
  int        c;
  INPUT_VARS(cinfo);

  for (;;)
  { INPUT_BYTE(cinfo, c);
    /* Skip any non-FF bytes.
     * This may look a bit inefficient, but it will not occur in a valid file.
     * We sync after each discarded byte so that a suspending data source
     * can discard the byte from its buffer.
     */
    while(c != 0xFF)
    { cinfo->marker->discarded_bytes++;
      INPUT_SYNC(cinfo);
      INPUT_BYTE(cinfo, c);
    }
    /* This loop swallows any duplicate FF bytes.  Extra FFs are legal as
     * pad bytes, so don't count them in discarded_bytes.  We assume there
     * will not be so many consecutive FF bytes as to overflow a suspending
     * data source's input buffer.
     */
    do
    { INPUT_BYTE(cinfo, c);
    }while(c == 0xFF);
    
    if(c != 0)
      break;            /* found a valid marker, exit loop */
    /* Reach here if we found a stuffed-zero data sequence (FF/00).
     * Discard it and loop back to try again.
     */
    cinfo->marker->discarded_bytes += 2;
    INPUT_SYNC(cinfo);
  }

  if(cinfo->marker->discarded_bytes != 0)
    cinfo->marker->discarded_bytes= 0;

  cinfo->unread_marker= c;

  INPUT_SYNC(cinfo);
  return TRUE;
}


boolean first_marker(j_decompress_ptr cinfo)
/* Like next_marker, but used to obtain the initial SOI marker. */
/* For this marker, we do not allow preceding garbage or fill; otherwise,
 * we might well scan an entire input file before realizing it ain't JPEG.
 * If an application wants to process non-JFIF files, it must seek to the
 * SOI before calling the JPEG library.
 */
{
  int c, c2;
  INPUT_VARS(cinfo);

  INPUT_BYTE(cinfo, c);
  INPUT_BYTE(cinfo, c2);

  cinfo->unread_marker = c2;

  INPUT_SYNC(cinfo);
  return TRUE;
}


/*
 * Read markers until SOS or EOI.
 *
 * Returns same codes as are defined for jpeg_consume_input:
 * JPEG_SUSPENDED, JPEG_REACHED_SOS, or JPEG_REACHED_EOI.
 */

int read_markers(j_decompress_ptr cinfo)
{
  /* Outer loop repeats once for each marker. */
  for(;;)
  { /* Collect the marker proper, unless we already did. */
    /* NB: first_marker() enforces the requirement that SOI appear first. */
    if(cinfo->unread_marker == 0)
    { if(! cinfo->marker->saw_SOI)
      { if(! first_marker(cinfo))
          return JPEG_SUSPENDED;
      }
      else
      { if(! next_marker(cinfo))
          return JPEG_SUSPENDED;
      }
    }
    /* At this point cinfo->unread_marker contains the marker code and the
     * input point is just past the marker proper, but before any parameters.
     * A suspension will cause us to return with this state still true.
     */
    switch (cinfo->unread_marker)
    { case M_SOI:
        if(! get_soi(cinfo))
          return JPEG_SUSPENDED;
        break;

      case M_SOF0:        /* Baseline */
      case M_SOF1:        /* Extended sequential, Huffman */
        if(! get_sof(cinfo, FALSE, FALSE))
          return JPEG_SUSPENDED;
        break;

      case M_SOF2:        /* Progressive, Huffman */
        if(! get_sof(cinfo, TRUE, FALSE))
          return JPEG_SUSPENDED;
        break;

      case M_SOF9:        /* Extended sequential, arithmetic */
        if(! get_sof(cinfo, FALSE, TRUE))
          return JPEG_SUSPENDED;
        break;

      case M_SOF10:        /* Progressive, arithmetic */
        if(! get_sof(cinfo, TRUE, TRUE))
          return JPEG_SUSPENDED;
        break;

      case M_SOS:
        if(! get_sos(cinfo))
          return JPEG_SUSPENDED;
        cinfo->unread_marker = 0;    /* processed the marker */
        return JPEG_REACHED_SOS;
    
      case M_EOI:
        cinfo->unread_marker = 0;    /* processed the marker */
        return JPEG_REACHED_EOI;
      
      case M_DHT:
        if(! get_dht(cinfo))
          return JPEG_SUSPENDED;
        break;
      
      case M_DQT:
        if(! get_dqt(cinfo))
          return JPEG_SUSPENDED;
        break;
      
      case M_DRI:
        if(! get_dri(cinfo))
          return JPEG_SUSPENDED;
        break;
      
      case M_APP0:
        if(! get_interesting_appn(cinfo))
          return JPEG_SUSPENDED;
          break;
          
      case M_APP14:
          if(! get_interesting_appn(cinfo))
              return JPEG_SUSPENDED;
          break;
   
      default:
        if(! skip_variable(cinfo))
          return JPEG_SUSPENDED;
        break;
    }
    /* Successfully processed marker, so reset state variable */
    cinfo->unread_marker = 0;
  } /* end loop */
}


/*
 * Read a restart marker, which is expected to appear next in the datastream;
 * if the marker is not there, take appropriate recovery action.
 * Returns FALSE if suspension is required.
 *
 * This is called by the entropy decoder after it has read an appropriate
 * number of MCUs.  cinfo->unread_marker may be nonzero if the entropy decoder
 * has already read a marker from the data source.  Under normal conditions
 * cinfo->unread_marker will be reset to 0 before returning; if not reset,
 * it holds a marker which the decoder will be unable to read past.
 */

boolean read_restart_marker (j_decompress_ptr cinfo)
{
  /* Obtain a marker unless we already did. */
  /* Note that next_marker will complain if it skips any data. */
  if(cinfo->unread_marker == 0)
  { if(! next_marker(cinfo))
      return FALSE;
  }

  if(cinfo->unread_marker == ((int) M_RST0 + cinfo->marker->next_restart_num))
  { /* Normal case --- swallow the marker and let entropy decoder continue */
    cinfo->unread_marker = 0;
  }
  else
  { /* Uh-oh, the restart markers have been messed up. */
    /* Let the data source manager determine how to resync. */
    if(! (*cinfo->src->resync_to_restart) (cinfo, cinfo->marker->next_restart_num))
      return FALSE;
  }

  /* Update next-restart state */
  cinfo->marker->next_restart_num = (cinfo->marker->next_restart_num + 1) & 7;

  return TRUE;
}


/*
 * This is the default resync_to_restart method for data source managers
 * to use if they don't have any better approach.  Some data source managers
 * may be able to back up, or may have additional knowledge about the data
 * which permits a more intelligent recovery strategy; such managers would
 * presumably supply their own resync method.
 *
 * read_restart_marker calls resync_to_restart if it finds a marker other than
 * the restart marker it was expecting.  (This code is *not* used unless
 * a nonzero restart interval has been declared.)  cinfo->unread_marker is
 * the marker code actually found (might be anything, except 0 or FF).
 * The desired restart marker number (0..7) is passed as a parameter.
 * This routine is supposed to apply whatever error recovery strategy seems
 * appropriate in order to position the input stream to the next data segment.
 * Note that cinfo->unread_marker is treated as a marker appearing before
 * the current data-source input point; usually it should be reset to zero
 * before returning.
 * Returns FALSE if suspension is required.
 *
 * This implementation is substantially constrained by wanting to treat the
 * input as a data stream; this means we can't back up.  Therefore, we have
 * only the following actions to work with:
 *   1. Simply discard the marker and let the entropy decoder resume at next
 *      byte of file.
 *   2. Read forward until we find another marker, discarding intervening
 *      data.  (In theory we could look ahead within the current bufferload,
 *      without having to discard data if we don't find the desired marker.
 *      This idea is not implemented here, in part because it makes behavior
 *      dependent on buffer size and chance buffer-boundary positions.)
 *   3. Leave the marker unread (by failing to zero cinfo->unread_marker).
 *      This will cause the entropy decoder to process an empty data segment,
 *      inserting dummy zeroes, and then we will reprocess the marker.
 *
 * #2 is appropriate if we think the desired marker lies ahead, while #3 is
 * appropriate if the found marker is a future restart marker (indicating
 * that we have missed the desired restart marker, probably because it got
 * corrupted).
 * We apply #2 or #3 if the found marker is a restart marker no more than
 * two counts behind or ahead of the expected one.  We also apply #2 if the
 * found marker is not a legal JPEG marker code (it's certainly bogus data).
 * If the found marker is a restart marker more than 2 counts away, we do #1
 * (too much risk that the marker is erroneous; with luck we will be able to
 * resync at some future point).
 * For any valid non-restart JPEG marker, we apply #3.  This keeps us from
 * overrunning the end of a scan.  An implementation limited to single-scan
 * files might find it better to apply #2 for markers other than EOI, since
 * any other marker would have to be bogus data in that case.
 */

boolean jpeg_resync_to_restart(j_decompress_ptr cinfo, int desired)
{
  int    marker= cinfo->unread_marker;
  int    action= 1;
  
  /* Outer loop handles repeated decision after scanning forward. */
  for (;;)
  { if(marker < (int) M_SOF0)
      action= 2;        /* invalid marker */
    else if(marker < (int) M_RST0 || marker > (int) M_RST7)
      action= 3;        /* valid non-restart marker */
    else
    { if(marker == ((int) M_RST0 + ((desired+1) & 7)) || marker == ((int) M_RST0 + ((desired+2) & 7)))
        action= 3;        /* one of the next two expected restarts */
      else if(marker == ((int) M_RST0 + ((desired-1) & 7)) || marker == ((int) M_RST0 + ((desired-2) & 7)))
        action= 2;        /* a prior restart, so advance */
      else
        action= 1;        /* desired restart or too far away */
    }
    switch(action)
    { case 1:
        /* Discard marker and let entropy decoder resume processing. */
        cinfo->unread_marker = 0;
        return TRUE;
      case 2:
        /* Scan to the next marker, and repeat the decision loop. */
        if(! next_marker(cinfo))
          return FALSE;
        marker = cinfo->unread_marker;
        break;
      case 3:
        /* Return without advancing past this marker. */
        /* Entropy decoder will be forced to process an empty segment. */
        return TRUE;
    }
  } /* end loop */
}


/*
 * Reset marker processing state to begin a fresh datastream.
 */

void reset_marker_reader (j_decompress_ptr cinfo)
{
  my_marker_ptr        marker= (my_marker_ptr) cinfo->marker;

  cinfo->comp_info= NULL;        /* until allocated by get_sof */
  cinfo->input_scan_number= 0;        /* no SOS seen yet */
  cinfo->unread_marker= 0;        /* no pending marker */
  marker->pub.saw_SOI= FALSE;        /* set internal state too */
  marker->pub.saw_SOF= FALSE;
  marker->pub.discarded_bytes= 0;
}


/*
 * Initialize the marker reader module.
 * This is called only once, when the decompression object is created.
 */

void jinit_marker_reader(j_decompress_ptr cinfo)
{
  my_marker_ptr            marker;

  /* Create subobject in permanent pool */
  marker= (my_marker_ptr)(*cinfo->mem->alloc_small)(cinfo, JPOOL_PERMANENT, SIZEOF(my_marker_reader));
  cinfo->marker= (struct jpeg_marker_reader *) marker;
  /* Initialize public method pointers */
  marker->pub.reset_marker_reader= reset_marker_reader;
  marker->pub.read_markers= read_markers;
  marker->pub.read_restart_marker= read_restart_marker;
 
  /* Reset marker processing state */
  reset_marker_reader(cinfo);
}

