/*
 * jmemmgr.c
 *
 * Copyright (C) 1991-1997, Thomas G. Lane.
 * This file is part of the Independent JPEG Group's software.
 * For conditions of distribution and use, see the accompanying README file.
 *
 * This file contains the JPEG system-independent memory management
 * routines.  This code is usable across a wide variety of machines; most
 * of the system dependencies have been isolated in a separate file.
 * The major functions provided here are:
 *   * pool-based allocation and freeing of memory;
 *   * policy decisions about how to divide available memory among the
 *     virtual arrays;
 *   * control logic for swapping virtual arrays between main memory and
 *     backing storage.
 * The separate system-dependent file provides the actual backing-storage
 * access code, and it contains the policy decision about how much total
 * main memory to use.
 * This file is system-dependent in the sense that some of its functions
 * are unnecessary in some systems.  For example, if there is enough virtual
 * memory so that backing storage will never be used, much of the virtual
 * array control logic could be removed.  (Of course, if you have that much
 * memory then you shouldn't care about a little bit of unused code...)
 */

#define JPEG_INTERNALS
#define AM_MEMORY_MANAGER    /* we define jvirt_Xarray_control structs */
#include "jinclude.h"
#include <jpeg/jpeglib.h>
#include "jmemsys.h"        /* import the system-dependent declarations */



/*
 * Many machines require storage alignment: longs must start on 4-byte
 * boundaries, doubles on 8-byte boundaries, etc.  On such machines, malloc()
 * always returns pointers that are multiples of the worst-case alignment
 * requirement, and we had better do so too.
 * There isn't any really portable way to determine the worst-case alignment
 * requirement.  This module assumes that the alignment requirement is
 * multiples of sizeof(ALIGN_TYPE).
 * By default, we define ALIGN_TYPE as double.  This is necessary on some
 * workstations (where doubles really do need 8-byte alignment) and will work
 * fine on nearly everything.  If your machine has lesser alignment needs,
 * you can save a few bytes by making ALIGN_TYPE smaller.
 * The only place I know of where this will NOT work is certain Macintosh
 * 680x0 compilers that define double as a 10-byte IEEE extended float.
 * Doing 10-byte alignment is counterproductive because longwords won't be
 * aligned well.  Put "#define ALIGN_TYPE long" in jconfig.h if you have
 * such a compiler.
 */

#define ALIGN_TYPE  long


/*
 * Here is the full definition of a memory manager object.
 */

typedef struct {
  struct jpeg_memory_mgr pub;    /* public fields */

  /* Since we only have one lifetime class of virtual arrays, only one
   * linked list is necessary (for each datatype).  Note that the virtual
   * array control blocks being linked together are actually stored somewhere
   * in the small-pool list.
   */
  jvirt_sarray_ptr virt_sarray_list;
  jvirt_barray_ptr virt_barray_list;
  
  /* alloc_sarray and alloc_barray set this value for use by virtual
   * array routines.
   */
  JDIMENSION last_rowsperchunk;    /* from most recent alloc_sarray/barray */
} my_memory_mgr;

typedef my_memory_mgr * my_mem_ptr;


/*
 * The control blocks for virtual arrays.
 * Note that these blocks are allocated in the "small" pool area.
 * System-dependent info for the associated backing store (if any) is hidden
 * inside the backing_store_info struct.
 */

struct jvirt_sarray_control {
  JSAMPARRAY mem_buffer;    /* => the in-memory buffer */
  JDIMENSION rows_in_array;    /* total virtual array height */
  JDIMENSION samplesperrow;    /* width of array (and of memory buffer) */
  JDIMENSION maxaccess;        /* max rows accessed by access_virt_sarray */
  JDIMENSION rows_in_mem;    /* height of memory buffer */
  JDIMENSION rowsperchunk;    /* allocation chunk size in mem_buffer */
  JDIMENSION cur_start_row;    /* first logical row # in the buffer */
  JDIMENSION first_undef_row;    /* row # of first uninitialized row */
  boolean pre_zero;        /* pre-zero mode requested? */
  boolean dirty;        /* do current buffer contents need written? */
  jvirt_sarray_ptr next;    /* link to next virtual sarray control block */
};

struct jvirt_barray_control {
  JBLOCKARRAY mem_buffer;    /* => the in-memory buffer */
  JDIMENSION rows_in_array;    /* total virtual array height */
  JDIMENSION blocksperrow;    /* width of array (and of memory buffer) */
  JDIMENSION maxaccess;        /* max rows accessed by access_virt_barray */
  JDIMENSION rows_in_mem;    /* height of memory buffer */
  JDIMENSION rowsperchunk;    /* allocation chunk size in mem_buffer */
  JDIMENSION cur_start_row;    /* first logical row # in the buffer */
  JDIMENSION first_undef_row;    /* row # of first uninitialized row */
  boolean pre_zero;        /* pre-zero mode requested? */
  boolean dirty;        /* do current buffer contents need written? */
  jvirt_barray_ptr next;    /* link to next virtual barray control block */
};



/*
 * Allocation of "small" objects.
 *
 * For these, we use pooled storage.  When a new pool must be created,
 * we try to get enough space for the current request plus a "slop" factor,
 * where the slop will be the amount of leftover space in the new pool.
 * The speed vs. space tradeoff is largely determined by the slop values.
 * A different slop value is provided for each pool class (lifetime),
 * and we also distinguish the first pool of a class from later ones.
 * NOTE: the values given work fairly well on both 16- and 32-bit-int
 * machines, but may be too small if longs are 64 bits or more.
 */

void * alloc_small(j_decompress_ptr cinfo, int pool_id, size_t sizeofobject)
{ void *    resultP= &cinfo->pool[pool_id][cinfo->used[pool_id]];
  
  sizeofobject= (sizeofobject + 3) & 0xFFFFFFFC;
  cinfo->used[pool_id]= cinfo->used[pool_id] + sizeofobject;
    
  return resultP;
}


/*
 * Creation of 2-D sample arrays.
 * The pointers are in near heap, the samples themselves in FAR heap.
 *
 * To minimize allocation overhead and to allow I/O of large contiguous
 * blocks, we allocate the sample rows in groups of as many rows as possible
 * without exceeding MAX_ALLOC_CHUNK total bytes per allocation request.
 * NB: the virtual array control routines, later in this file, know about
 * this chunking of rows.  The rowsperchunk value is left in the mem manager
 * object so that it can be saved away if this sarray is the workspace for
 * a virtual array.
 */

JSAMPARRAY alloc_sarray(j_decompress_ptr cinfo, int pool_id, JDIMENSION samplesperrow, JDIMENSION numrows)
/* Allocate a 2-D sample array */
{ my_mem_ptr        mem= (my_mem_ptr) cinfo->mem;
  JSAMPARRAY        result;
  JSAMPROW            workspace;
  JDIMENSION        rowsperchunk, currow, i;

  rowsperchunk= numrows;
  mem->last_rowsperchunk= rowsperchunk;

  /* Get space for row pointers (small object) */
  result= (JSAMPARRAY) alloc_small(cinfo, pool_id, (size_t) (numrows * SIZEOF(JSAMPROW)));

  /* Get the rows themselves (large objects) */
  currow = 0;
  while (currow < numrows)
  { rowsperchunk= MIN(rowsperchunk, numrows - currow);
    workspace= (JSAMPROW) alloc_small(cinfo, pool_id, (size_t) ((size_t) rowsperchunk * (size_t) samplesperrow * SIZEOF(JSAMPLE)));
    for (i = rowsperchunk; i > 0; i--)
    { result[currow++]= workspace;
      workspace += samplesperrow;
    }
  }
  return result;
}


/*
 * Creation of 2-D coefficient-block arrays.
 * This is essentially the same as the code for sample arrays, above.
 */

JBLOCKARRAY alloc_barray (j_decompress_ptr cinfo, int pool_id, JDIMENSION blocksperrow, JDIMENSION numrows)
/* Allocate a 2-D coefficient-block array */
{ my_mem_ptr        mem= (my_mem_ptr) cinfo->mem;
  JBLOCKARRAY        result;
  JBLOCKROW            workspace;
  JDIMENSION        rowsperchunk, currow, i;

  rowsperchunk= numrows;
  mem->last_rowsperchunk= rowsperchunk;

  /* Get space for row pointers (small object) */
  result= (JBLOCKARRAY) alloc_small(cinfo, pool_id, (size_t) (numrows * SIZEOF(JBLOCKROW)));

  /* Get the rows themselves (large objects) */
  currow = 0;
  while (currow < numrows)
  { rowsperchunk= MIN(rowsperchunk, numrows - currow);
    workspace= (JBLOCKROW) alloc_small(cinfo, pool_id, (size_t) ((size_t) rowsperchunk * (size_t) blocksperrow * SIZEOF(JBLOCK)));
    for (i = rowsperchunk; i > 0; i--)
    { result[currow++] = workspace;
      workspace += blocksperrow;
    }
  }

  return result;
}


/*
 * About virtual array management:
 *
 * The above "normal" array routines are only used to allocate strip buffers
 * (as wide as the image, but just a few rows high).  Full-image-sized buffers
 * are handled as "virtual" arrays.  The array is still accessed a strip at a
 * time, but the memory manager must save the whole array for repeated
 * accesses.  The intended implementation is that there is a strip buffer in
 * memory (as high as is possible given the desired memory limit), plus a
 * backing file that holds the rest of the array.
 *
 * The request_virt_array routines are told the total size of the image and
 * the maximum number of rows that will be accessed at once.  The in-memory
 * buffer must be at least as large as the maxaccess value.
 *
 * The request routines create control blocks but not the in-memory buffers.
 * That is postponed until realize_virt_arrays is called.  At that time the
 * total amount of space needed is known (approximately, anyway), so free
 * memory can be divided up fairly.
 *
 * The access_virt_array routines are responsible for making a specific strip
 * area accessible (after reading or writing the backing file, if necessary).
 * Note that the access routines are told whether the caller intends to modify
 * the accessed strip; during a read-only pass this saves having to rewrite
 * data to disk.  The access routines are also responsible for pre-zeroing
 * any newly accessed rows, if pre-zeroing was requested.
 *
 * In current usage, the access requests are usually for nonoverlapping
 * strips; that is, successive access start_row numbers differ by exactly
 * num_rows = maxaccess.  This means we can get good performance with simple
 * buffer dump/reload logic, by making the in-memory buffer be a multiple
 * of the access height; then there will never be accesses across bufferload
 * boundaries.  The code will still work with overlapping access requests,
 * but it doesn't handle bufferload overlaps very efficiently.
 */


jvirt_barray_ptr request_virt_barray(j_decompress_ptr cinfo, int pool_id, boolean pre_zero,  JDIMENSION blocksperrow, JDIMENSION numrows, JDIMENSION maxaccess)
/* Request a virtual 2-D coefficient-block array */
{ my_mem_ptr        mem= (my_mem_ptr) cinfo->mem;
  jvirt_barray_ptr    result;

  /* get control block */
  result= (jvirt_barray_ptr) alloc_small(cinfo, pool_id, SIZEOF(struct jvirt_barray_control));

  result->mem_buffer= NULL;    /* marks array not yet realized */
  result->rows_in_array= numrows;
  result->blocksperrow= blocksperrow;
  result->maxaccess= maxaccess;
  result->pre_zero= pre_zero;
  result->next= mem->virt_barray_list; /* add to list of virtual arrays */
  mem->virt_barray_list= result;

  return result;
}


void realize_virt_arrays (j_decompress_ptr cinfo)
/* Allocate the in-memory buffers for any unrealized virtual arrays */
{ my_mem_ptr            mem= (my_mem_ptr) cinfo->mem;
  long                    space_per_minheight, maximum_space;
  long                    minheights;
  jvirt_sarray_ptr        sptr;
  jvirt_barray_ptr        bptr;

  /* Compute the minimum space needed (maxaccess rows in each buffer)
   * and the maximum space needed (full image height in each buffer).
   */
  space_per_minheight= 0;
  maximum_space= 0;
  for(sptr= mem->virt_sarray_list; sptr != NULL; sptr= sptr->next)
  { if(sptr->mem_buffer == NULL)
    { /* if not realized yet */
      space_per_minheight += (long) sptr->maxaccess * (long) sptr->samplesperrow * SIZEOF(JSAMPLE);
      maximum_space += (long) sptr->rows_in_array * (long) sptr->samplesperrow * SIZEOF(JSAMPLE);
    }
  }
  for(bptr= mem->virt_barray_list; bptr != NULL; bptr= bptr->next)
  { if(bptr->mem_buffer == NULL)
    { /* if not realized yet */
      space_per_minheight += (long) bptr->maxaccess * (long) bptr->blocksperrow * SIZEOF(JBLOCK);
      maximum_space += (long) bptr->rows_in_array * (long) bptr->blocksperrow * SIZEOF(JBLOCK);
    }
  }

  if(space_per_minheight <= 0)
    return;            /* no unrealized arrays, no work */

  /* Allocate the in-memory buffers and initialize backing store as needed. */

  for(sptr= mem->virt_sarray_list; sptr != NULL; sptr= sptr->next)
  { if(sptr->mem_buffer == NULL)
    { /* if not realized yet */
      minheights= ((long) sptr->rows_in_array - 1L) / sptr->maxaccess + 1L;
      sptr->rows_in_mem= sptr->rows_in_array;
      sptr->mem_buffer= alloc_sarray(cinfo, JPOOL_IMAGE,  sptr->samplesperrow, sptr->rows_in_mem);
      sptr->rowsperchunk= mem->last_rowsperchunk;
      sptr->cur_start_row= 0;
      sptr->first_undef_row= 0;
      sptr->dirty= FALSE;
    }
  }

  for(bptr= mem->virt_barray_list; bptr != NULL; bptr= bptr->next)
  { if(bptr->mem_buffer == NULL)
    { /* if not realized yet */
      minheights= ((long) bptr->rows_in_array - 1L) / bptr->maxaccess + 1L;
      bptr->rows_in_mem= bptr->rows_in_array;
      bptr->mem_buffer= alloc_barray(cinfo, JPOOL_IMAGE, bptr->blocksperrow, bptr->rows_in_mem);
      bptr->rowsperchunk= mem->last_rowsperchunk;
      bptr->cur_start_row= 0;
      bptr->first_undef_row= 0;
      bptr->dirty = FALSE;
    }
  }
}



JBLOCKARRAY access_virt_barray (j_decompress_ptr cinfo, jvirt_barray_ptr ptr, JDIMENSION start_row, JDIMENSION num_rows, boolean writable)
/* Access the part of a virtual block array starting at start_row */
/* and extending for num_rows rows.  writable is true if  */
/* caller intends to modify the accessed area. */
{ JDIMENSION        end_row= start_row + num_rows;
  JDIMENSION        undef_row;

  /* Ensure the accessed part of the array is defined; prezero if needed.
   * To improve locality of access, we only prezero the part of the array
   * that the caller is about to access, not the entire in-memory array.
   */
  if(ptr->first_undef_row < end_row)
  { if(ptr->first_undef_row < start_row)
    { undef_row= start_row;    /* but reader is allowed to read ahead */
    }
    else
    { undef_row = ptr->first_undef_row;
    }
    if(writable)
      ptr->first_undef_row= end_row;
    if(ptr->pre_zero)
    { size_t bytesperrow= (size_t) ptr->blocksperrow * SIZEOF(JBLOCK);
      undef_row -= ptr->cur_start_row; /* make indexes relative to buffer */
      end_row -= ptr->cur_start_row;
      while (undef_row < end_row)
      { jzero((void *) ptr->mem_buffer[undef_row], bytesperrow);
        undef_row++;
      }
    }
  }
  /* Flag the buffer dirty if caller will write in it */
  if (writable)
    ptr->dirty= TRUE;
  /* Return address of proper part of the buffer */
  return ptr->mem_buffer + (start_row - ptr->cur_start_row);
}



/*
 * Memory manager initialization.
 * When this is called, only the error manager pointer is valid in cinfo!
 */

void jinit_memory_mgr (j_decompress_ptr cinfo)
{
  my_mem_ptr        mem;
    
  mem= (my_mem_ptr)alloc_small(cinfo, JPOOL_PERMANENT, SIZEOF(my_memory_mgr));
  mem->pub.alloc_small= alloc_small;
  mem->pub.alloc_sarray= alloc_sarray;
  mem->pub.alloc_barray= alloc_barray;
  mem->pub.request_virt_barray= request_virt_barray;
  mem->pub.realize_virt_arrays= realize_virt_arrays;
  mem->pub.access_virt_barray= access_virt_barray;
  mem->virt_sarray_list = NULL;
  mem->virt_barray_list = NULL;

  cinfo->mem= &mem->pub;
}
