/*!
*****************************************************************************
**
** \file        ./subsystems/paint/osdhandler/src/osdhandler.h
**
** \version     $Id: osdhandler.h 5280 2011-02-21 16:39:28Z wlaris $
**
** \brief       OSD handler for the paint middleware
**
** This file contains the declaration of the OSDHANDLER API.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2007 - 2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/


#ifndef OSDHANDLER_H
    #define OSDHANDLER_H

    #include <stdio.h>
    #include <stdint.h>
    #include <fapex/abst_types.h>
    #include <fapex/abst_libviout.h>

    //***********************************************************************
    //***********************************************************************
    //** Defines and Macros
    //***********************************************************************
    //***********************************************************************

    //! Identifier for OSD1 (scalable)
    #define OSDHANDLER_OSD1                         1

    //! Identifier for OSD2 (not scalable)
    #define OSDHANDLER_OSD2                         2

    //! Identifier for OSD3 (not scalable)
    #define OSDHANDLER_OSD3                         3

    //! Identifier for OSD4 (not scalable)
    #define OSDHANDLER_OSD4                         4

    //! Identifier for Cursor Plane (not scalable)
    #define OSDHANDLER_CURSOR                       5
    //! maximum horizontal resolution for OSDs
    #define OSDHANDLER_MAX_HRES               1920

    //! maximum vertical resolution for OSDs
    #define OSDHANDLER_MAX_VRES                     1088

    //! maximum number of hardware layers
    #define OSDHANDLER_MAX_NUM_HW_LAYER             5

    // only for internal debug purposes used
    #define OSDHANDLER_DEBUG                        0

    // enable/disable support for virtual layers
    #ifndef OSDHANDLER_USE_VIRTUAL_LAYERS
    #define OSDHANDLER_USE_VIRTUAL_LAYERS           0
    #endif

    /*!
    ********************************************************************************
    ** Standard error codes for the OSDHANDLER driver
    ********************************************************************************
    */

    /*! Bad parameter passed. */
    #define OSDHANDLER_ERR_BAD_PARAMETER \
            (int32_t)(SUBSYS_PAINT_OSDHANDLER_MODULE_BASE + FAPI_ERR_BAD_PARAMETER)

    /*! Memory allocation failed. */
    #define OSDHANDLER_ERR_OUT_OF_MEMORY \
            (int32_t)(SUBSYS_PAINT_OSDHANDLER_MODULE_BASE + FAPI_ERR_OUT_OF_MEMORY)

    /*! Device already initialised. */
    #define OSDHANDLER_ERR_ALREADY_INITIALIZED \
            (int32_t)(SUBSYS_PAINT_OSDHANDLER_MODULE_BASE + FAPI_ERR_ALREADY_INITIALIZED)

    /*! Device not initialised. */
    #define OSDHANDLER_ERR_NOT_INITIALIZED \
            (int32_t)(SUBSYS_PAINT_OSDHANDLER_MODULE_BASE + FAPI_ERR_NOT_INITIALIZED)

    /*! Feature or function is not available. */
    #define OSDHANDLER_ERR_FEATURE_NOT_SUPPORTED \
            (int32_t)(SUBSYS_PAINT_OSDHANDLER_MODULE_BASE + FAPI_ERR_FEATURE_NOT_SUPPORTED)

    /*! Timeout occured. */
    #define OSDHANDLER_ERR_TIMEOUT \
            (int32_t)(SUBSYS_PAINT_OSDHANDLER_MODULE_BASE + FAPI_ERR_TIMEOUT)

    /*! The device is busy, try again later. */
    #define OSDHANDLER_ERR_DEVICE_BUSY \
            (int32_t)(SUBSYS_PAINT_OSDHANDLER_MODULE_BASE + FAPI_ERR_DEVICE_BUSY)

    /*! Invalid handle was passed. */
    #define OSDHANDLER_ERR_INVALID_HANDLE \
            (int32_t)(SUBSYS_PAINT_OSDHANDLER_MODULE_BASE + FAPI_ERR_INVALID_HANDLE)

    /*! Semaphore could not be created. */
    #define OSDHANDLER_ERR_SEMAPHORE_CREATE \
            (int32_t)(SUBSYS_PAINT_OSDHANDLER_MODULE_BASE + FAPI_ERR_SEMAPHORE_CREATE)

    /*! The driver's used version is not supported. */
    #define OSDHANDLER_ERR_UNSUPPORTED_VERSION \
            (int32_t)(SUBSYS_PAINT_OSDHANDLER_MODULE_BASE + FAPI_ERR_UNSUPPORTED_VERSION)


    /*!
    ****************************************************************************
    ** Additional OSDHANDLER driver error codes
    ** that may occur for both, procedure calls for hardware and
    ** virtual layers. (reserved from -100 to -199)
    **
    ** Each error code shall be decreased in steps of 1.
    ****************************************************************************
    */
    /* \brief OSDHANDLER specific error code offset */
    #define OSDHANDLER_HLVL_ERR_BASE \
            (int32_t) (SUBSYS_PAINT_OSDHANDLER_MODULE_BASE - 100)

    //! an invalid layerId has been passed
    #define OSDHANDLER_ERR_BAD_LAYER_ID      (int32_t) (OSDHANDLER_HLVL_ERR_BASE - 0)

    //! There was no handle allocated for the requested OSD/CP at initialization.
    #define OSDHANDLER_ERR_NO_OSD_HANDLE     (int32_t) (OSDHANDLER_HLVL_ERR_BASE - 1)

    //! Returned if attempted to operate an OSD/CP in a bad colour mode.
    #define OSDHANDLER_ERR_BAD_COLOUR_MODE   (int32_t) (OSDHANDLER_HLVL_ERR_BASE - 2)

    //! The passed dimensioning for an OSD/CP would cause it to exceed its maximum size.
    #define OSDHANDLER_ERR_OUT_OF_SIZE       (int32_t) (OSDHANDLER_HLVL_ERR_BASE - 3)

    //! The passed coordinates are out of bounds of the OSD picture.
    #define OSDHANDLER_ERR_OUT_OF_BOUNDS     (int32_t) (OSDHANDLER_HLVL_ERR_BASE - 4)

    //! The layer has not been started yet.
    #define OSDHANDLER_ERR_LAYER_NOT_STARTED (int32_t) (OSDHANDLER_HLVL_ERR_BASE - 5)

    //! The limit of 4 KiB for the Cursor Plane for H61
    #define OSDHANDLER_ERR_CP_4KB_EXCEEDED   (int32_t) (OSDHANDLER_HLVL_ERR_BASE - 6)
    /*!
    ****************************************************************************
    ** Additional OSDHANDLER error codes that may be returned through an
    ** error code pointer parameter. Each error code must allocate a single bit
    ** within a 32-bit word. If the error occured, the bit must be set to 1,
    ** otherwise 0.
    ** This scheme allows multiple error codes to be contained in a single word.
    ****************************************************************************
    */
    /*

    OSD1 0x10000000;
    OSD2 0x20000000;
    OSD3 0x30000000;
    OSD4 0x40000000;
    CP   0x50000000;

    open         ---> 1
    close        ---> 2
    start        ---> 3
    stop         ---> 4
    resize       ---> 5
    enable       ---> 6
    clut         ---> 7
    fade         ---> 8

    */
    #define OSD1 (uint32_t) 0x10000000
    #define OSD2 (uint32_t) 0x20000000
    #define OSD3 (uint32_t) 0x30000000
    #define OSD4 (uint32_t) 0x40000000
    #define CP   (uint32_t) 0x50000000

    #define OPEN      (uint32_t) 0x1
    #define CLOSE      (uint32_t) 0x2
    #define START      (uint32_t) 0x3
    #define STOP      (uint32_t) 0x4
    #define RESIZE  (uint32_t) 0x5
    #define ENABLE  (uint32_t) 0x6
    #define CLUT      (uint32_t) 0x7
    #define FADE      (uint32_t) 0x8

    /*! OSD1 could not be opened at initialization. */
    #define OSDHANDLER_ERR_CODE_OSD1_OPEN   (OSD1+OPEN)

    /*! OSD2 could not be opened at initialization. */
    #define OSDHANDLER_ERR_CODE_OSD2_OPEN   (OSD2+OPEN)

        /*! OSD3 could not be opened at initialization. */
    #define OSDHANDLER_ERR_CODE_OSD3_OPEN   (OSD3+OPEN)

    /*! OSD4 could not be opened at initialization. */
    #define OSDHANDLER_ERR_CODE_OSD4_OPEN   (OSD4+OPEN)

    /*! Cursor Plane could not be opened at initialization. */
    #define OSDHANDLER_ERR_CODE_CP_OPEN     (CP+OPEN)

    /*! OSD1 could not be stopped at exit. */
    #define OSDHANDLER_ERR_CODE_OSD1_CLOSE  (OSD1+CLOSE)

    /*! OSD2 could not be stopped at exit. */
    #define OSDHANDLER_ERR_CODE_OSD2_CLOSE  (OSD2+CLOSE)

        /*! OSD3 could not be stopped at exit. */
    #define OSDHANDLER_ERR_CODE_OSD3_CLOSE  (OSD3+CLOSE)

    /*! OSD4 could not be stopped at exit. */
    #define OSDHANDLER_ERR_CODE_OSD4_CLOSE  (OSD4+CLOSE)

    /*! Cursor Plane could not be stopped at exit. */
    #define OSDHANDLER_ERR_CODE_CP_CLOSE    (CP+CLOSE)

    /*! OSD1 could not be started by VISCALE. */
    #define OSDHANDLER_ERR_CODE_OSD1_START  (OSD1+START)

    /*! OSD2 could not be started by VISCALE. */
    #define OSDHANDLER_ERR_CODE_OSD2_START  (OSD2+START)

        /*! OSD3 could not be started by VISCALE. */
    #define OSDHANDLER_ERR_CODE_OSD3_START  (OSD3+START)

    /*! OSD4 could not be started by VISCALE. */
    #define OSDHANDLER_ERR_CODE_OSD4_START  (OSD4+START)

    /*! Cursor Plane could not be started by VISCALE. */
    #define OSDHANDLER_ERR_CODE_CP_START    (CP+START)

    /*! OSD1 could not be stopped by VISCALE. */
    #define OSDHANDLER_ERR_CODE_OSD1_STOP   (OSD1+STOP)

    /*! OSD2 could not be stopped by VISCALE. */
    #define OSDHANDLER_ERR_CODE_OSD2_STOP   (OSD2+STOP)

        /*! OSD3 could not be stopped by VISCALE. */
    #define OSDHANDLER_ERR_CODE_OSD3_STOP   (OSD3+STOP)

    /*! OSD4 could not be stopped by VISCALE. */
    #define OSDHANDLER_ERR_CODE_OSD4_STOP   (OSD4+STOP)

    /*! Cursor Plane could not be stopped by VISCALE. */
    #define OSDHANDLER_ERR_CODE_CP_STOP     (CP+STOP)

    /*! OSD1 resize failed. */
    #define OSDHANDLER_ERR_CODE_OSD1_RESIZE (OSD1+RESIZE)

    /*! OSD2 resize failed. */
    #define OSDHANDLER_ERR_CODE_OSD2_RESIZE (OSD2+RESIZE)

        /*! OSD3 resize failed. */
    #define OSDHANDLER_ERR_CODE_OSD3_RESIZE (OSD3+RESIZE)

    /*! OSD4 resize failed. */
    #define OSDHANDLER_ERR_CODE_OSD4_RESIZE (OSD4+RESIZE)

    /*! Cursor Plane resize failed. */
    #define OSDHANDLER_ERR_CODE_CP_RESIZE   (CP+RESIZE)

    /*! OSD1 enable/disable failed. */
    #define OSDHANDLER_ERR_CODE_OSD1_ENABLE (OSD1+ENABLE)

    /*! OSD2 enable/disable failed. */
    #define OSDHANDLER_ERR_CODE_OSD2_ENABLE (OSD2+ENABLE)

        /*! OSD3 enable/disable failed. */
    #define OSDHANDLER_ERR_CODE_OSD3_ENABLE (OSD3+ENABLE)

    /*! OSD4 enable/disable failed. */
    #define OSDHANDLER_ERR_CODE_OSD4_ENABLE (OSD4+ENABLE)

    /*! Cursor Plane enable/disable failed.      */
    #define OSDHANDLER_ERR_CODE_CP_ENABLE   (CP+ENABLE)

    /*! OSD1 CLUT update failed. */
    #define OSDHANDLER_ERR_CODE_OSD1_CLUT   (OSD1+CLUT)

    /*! OSD2 CLUT update failed. */
    #define OSDHANDLER_ERR_CODE_OSD2_CLUT   (OSD2+CLUT)

        /*! OSD3 CLUT update failed. */
    #define OSDHANDLER_ERR_CODE_OSD3_CLUT   (OSD3+CLUT)

    /*! OSD4 CLUT update failed. */
    #define OSDHANDLER_ERR_CODE_OSD4_CLUT   (OSD4+CLUT)

    /*! Cursor Plane CLUT update failed. */
    #define OSDHANDLER_ERR_CODE_CP_CLUT     (CP+CLUT)

    /*! OSD1 plane fading failed. */
    #define OSDHANDLER_ERR_CODE_OSD1_FADE   (OSD1+FADE)

    /*! OSD2 plane fading failed. */
    #define OSDHANDLER_ERR_CODE_OSD2_FADE   (OSD2+FADE)

        /*! OSD3 plane fading failed. */
    #define OSDHANDLER_ERR_CODE_OSD3_FADE   (OSD3+FADE)

    /*! OSD4 plane fading failed. */
    #define OSDHANDLER_ERR_CODE_OSD4_FADE   (OSD4+FADE)

    /*! Cursor Plane plane fading failed. */
    #define OSDHANDLER_ERR_CODE_CP_FADE     (CP+FADE)

    /*! Hilight function for the Cursor Plane failed. */
    #define OSDHANDLER_ERR_CODE_CP_HILIGHT  (uint32_t) 0x01000000

    /*! Setting up plane order failed. */
    #define OSDHANDLER_ERR_CODE_PLANE_ORDER (uint32_t) 0x02000000

    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************

    /*!
    *************************************************************************
    ** \brief OSD and CP related colour modes
    *************************************************************************
    */
    typedef enum
    {
        //! unknown OSD mode
        OSDHANDLER_OSD_MODE_UNKNOWN = (int32_t)0,

        //! 4-bit colour lookup table mode for OSD
        OSDHANDLER_OSD_MODE_CLUT_4BIT = (int32_t)1,

        //! 8-bit colour lookup table mode for OSD
        OSDHANDLER_OSD_MODE_CLUT_8BIT = (int32_t)2,

        //! 16-bit CCIR mode for OSD
        OSDHANDLER_OSD_MODE_16BIT = (int32_t)3,

        //! 16-bit mode for OSD with 4 bits for each component
        OSDHANDLER_OSD_MODE_4444 = (int32_t)4,

        //! 16-bit mode for OSD with 5 bits for each component and a single alpha switch
        OSDHANDLER_OSD_MODE_5551 = (int32_t)5,

        //! 32-bit mode for OSD with 8 bits for each colour component
        OSDHANDLER_OSD_MODE_32_24BIT = (int32_t)6,

        //! unknown mode for the Cursor Plane
        OSDHANDLER_CURSOR_MODE_UNKNOWN = (int32_t)7,

        //! 1-bit colour lookup table mode for Cursor Plane
        OSDHANDLER_CURSOR_MODE_1BIT = (int32_t)8,

        //! 2-bit colour lookup table mode for Cursor Plane
        OSDHANDLER_CURSOR_MODE_2BIT = (int32_t)9,

        //! 4-bit colour lookup table mode for Cursor Plane
        OSDHANDLER_CURSOR_MODE_4BIT = (int32_t)10,

        //! 8-bit colour lookup table mode for Cursor Plane
        OSDHANDLER_CURSOR_MODE_8BIT = (int32_t)11
    } OSDHANDLER_ColourModeT;

    /*!
    ****************************************************************************
    ** \brief bit organisation of pixel data in the OSD memory
    ****************************************************************************
    */
    typedef enum {
        //! Unknown layout format of OSD in memory
        OSDHANDLER_UNKNOWN_OSD_LAYOUT = (int32_t)0,
        //! RGB mode with alpha as most significant byte
        OSDHANDLER_OSD_LAYOUT_ARGB = 1,
        //! RGB mode with alpha as least significant byte
        OSDHANDLER_OSD_LAYOUT_RGBA = 2,
        //! CbYCr mode with alpha as most significant byte
        OSDHANDLER_OSD_LAYOUT_ACbYCr = 3,
        //! CbYCr mode with alpha as least significant byte
        OSDHANDLER_OSD_LAYOUT_CbYCrA = 4
    }OSDHANDLER_OsdLayoutEnumT;

    /*!
    *************************************************************************
    ** \brief   Specifies an encoder type. Needed by the Viscale driver in
    **          order to perform rescale operations.
    *************************************************************************
    */
    typedef enum
    {
        //! Unknown encoder
        OSDHANDLER_UNKNOWN_ENCODER = (int32_t)0,
        //! Scaling for PAL encoder
        OSDHANDLER_ENCODER_PAL,
        //! Scaling for NTSC encoder
        OSDHANDLER_ENCODER_NTSC,
        //! Scaling for SECAM encoder
        OSDHANDLER_ENCODER_SECAM
    } OSDHANDLER_EncoderEnumT;

    /*!
    ****************************************************************************
    ** \brief Enumeration type for user selection of plane order.
    ****************************************************************************
    */
    typedef enum {
        /*! Identifies the Video layer.*/
        OSDHANDLER_VIDEO_PLANE = 0,
        /*! Identifies the OSD1 layer.*/
        OSDHANDLER_OSD1_PLANE = 1,
        /*! Identifies the OSD2 layer.*/
        OSDHANDLER_OSD2_PLANE = 2,
        /*! Identifies the OSD3 layer.*/
        OSDHANDLER_OSD3_PLANE = 3,
        /*! Identifies the OSD4 layer.*/
        OSDHANDLER_OSD4_PLANE = 4,
        /*! Identifies the Cursor Plane layer.*/
        OSDHANDLER_CURSOR_PLANE = 5
    }OSDHANDLER_PlaneDataSelectorEnumT;

    /*!
    *************************************************************************
    ** \brief OSD and CP related initialization parameters
    *************************************************************************
    */
    typedef struct
    {
        /*! Specifies whether the driver should try to open the corresponding
        **  OSD layer and allocate the handle. Choose any value but 0 to
        **  open the layer.
        */
        uint32_t                    inUse;

        /*! Specifies the Colour Mode in which the OSD is going to be started up.
        **  See OSDHANDLER_ColourModeT for the available modes.
        */
        OSDHANDLER_ColourModeT      colourMode;

        /*! Specifies the bit organisation layout of the OSD.
        **  See FAPI_VISCALE_OsdLayoutEnumT for the available modes.
        */
        OSDHANDLER_OsdLayoutEnumT   osdLayout;

        /*! Points to a field of 256 colours (4 bytes each). Format: CbYCrA */
        uint32_t*                   defaultClutPtr;

        /*! Decides whether the alpha may be set globally for \b all pixels
        **  or if the global alpha value for each pixel is read from the
        **  colour information of the pixel itself.
        */
        uint32_t                    useGlobalAlpha;

        /*! Initial global alpha value that is used if global alpha is enabled.
        */
        uint32_t                    globalAlpha;
    } OSDHANDLER_OsdInitParamsT;

    /*!
    *************************************************************************
    ** \brief Init parameters for the OSDHANDLER
    *************************************************************************
    */
    typedef struct
    {
        OSDHANDLER_OsdInitParamsT   osd1InitParams; //!< OSD1 related init params
        OSDHANDLER_OsdInitParamsT   osd2InitParams; //!< OSD2 related init params
        OSDHANDLER_OsdInitParamsT   osd3InitParams; //!< OSD3 related init params
        OSDHANDLER_OsdInitParamsT   osd4InitParams; //!< OSD4 related init params
        OSDHANDLER_OsdInitParamsT   cpInitParams;   //!< CP related init params
    } OSDHANDLER_InitParamsT;

    /*!
    *************************************************************************
    ** \brief   OSD and CursorPlane (CP) related start parameters.
    **
    ** \note    The actual resolution of the OSD/CP may differ from
    **          that you defined by the width and height parameters
    **          if scaling is set to 0.
    *************************************************************************
    */
    typedef struct
    {
        /*! The x position of the top left corner of the OSD/CP.      */
        uint32_t                    xPos;

        /*! The y position of the top left corner of the OSD/CP.      */
        uint32_t                    yPos;

        /*! The internal horizontal resolution of the OSD.            */
        uint32_t                    inputWidth;

        /*! The internal vertical resolution of the OSD.              */
        uint32_t                    inputHeight;

        /*! The horizontal resolution of the OSD on the video output.
        **  If this parameter is 0, no horizontal rescaling will be
        **  performed and the output width will be set to the internal
        **  horizontal resolution.
        */
        uint32_t                    outputWidth;

        /*! The vertical resolution of the OSD on the video output.
        **  If this parameter is 0, no vertical rescaling will be performed
        **  and the output height will be set to the internal vertical
        **  resolution for frame based pictures and to half of
        **  the internal vertical resolution for field based pictures.
        **  In order to avoid rescaling, the internal vertical resolution
        **  ('inputHeight') must be a multiple of two in the
        **  field based mode.
        */
        uint32_t                    outputHeight;

        /*!
        **  Indicates whether an OSD/CP should be interpreted as
        **  field based or frame based.
        **  If you provide this parameter with a 0, the OSD/CP
        **  memory will be interpreted as a field based picture,
        **  otherwise as a frame based one.
        */
        uint32_t                    frameBased;

        /*! Defines the memory base address that will be used for the OSD
        **  that is to be started. If you specify this parameter,
        **  you will have to take care that there is enough memory
        **  available for the chosen resolution at the current colour
        **  mode by calling #OSDHANDLER_GetOsdMemConsumption().
        **  If you set this parameter to NULL, the needed memory will be
        **  allocated from the heap automatically.
        */
        uint8_t*                   memoryBasePtr;
    } OSDHANDLER_StartParamsT;

    /*!
    ***************************************************************************
    ** \brief   Specifies how an OSDHANDLER function may behave and
    **          handle the other passed arguments.
    ***************************************************************************
    */
    typedef enum
    {

        OSDHANDLER_PARAMETER_GET = (int32_t)0,  //!< get the value
        OSDHANDLER_PARAMETER_SET,               //!< set the value
        OSDHANDLER_PARAMETER_INCREASE,          //!< increase the value
        OSDHANDLER_PARAMETER_DECREASE,          //!< decrease the value
        OSDHANDLER_PARAMETER_ON,                //!< switch something on
        OSDHANDLER_PARAMETER_OFF                //!< switch something off
    } OSDHANDLER_ModeParamT;

    /*!
    *************************************************************************
    ** \brief Layer parameters structure that is passed to other middleware
    **        from the paint library in order to draw pixels on the
    **        OSDs or the Cursor Plane.
    *************************************************************************
    */
    typedef struct
    {
        /*!
        ** the base address of the content of an OSD/CP is NULL if the OSD
        ** is not started. Therefore there is no memory allocated for the
        ** usage of this OSD.
        */
        uint32_t    pictureBase;
        /*! The internal horizontal resolution of the OSD/CP.       */
        uint32_t    xRes;
        /*! The internal vertical resolution of the OSD/CP.         */
        uint32_t    yRes;
        /*! The count of bits per pixel in the current colour mode. */
        uint32_t    bpp;
        /*! The colour mode that is set for the OSD/CP.             */
        OSDHANDLER_ColourModeT colourMode;
        /*! The format of the bit organization for one pixel.       */
        OSDHANDLER_OsdLayoutEnumT layoutMode;
    } OSDHANDLER_LayerParamsT;

    /*!
    *************************************************************************
    ** \brief   Union that provides access to an CbYCrA coded colour
    **          component wise and as a whole.
    *************************************************************************
    */
    typedef union
    {
        //! Element that contains the whole CbYCrA coded colour.
        uint32_t all;
        //! Element that contains the CbYCrA coded colour component wise.
        struct
        {
            //! Component that contains the alpha amount only.
            uint32_t alpha : 8;
            //! Component that contains the Cr amount only.
            uint32_t cr    : 8;
            //! Component that contains the Y amount only.
            uint32_t y     : 8;
            //! Component that contains the Cb amount only.
            uint32_t cb    : 8;
        } comp;
    } OSDHANDLER_CbYCrA__T;

    /*!
    *************************************************************************
    ** \brief   Union that provides access to an ARGB coded colour
    **          component wise and as a whole.
    *************************************************************************
    */
    typedef union
    {
        //! Element that contains the whole ARGB coded colour.
        uint32_t all;
        //! Element that contains the ARGB coded colour component wise.
        struct
        {
            //! Component that contains the blue amount only.
            uint32_t b     : 8;
            //! Component that contains the green amount only.
            uint32_t g     : 8;
            //! Component that contains the red amount only.
            uint32_t r     : 8;
            //! Component that contains the alpha amount only.
            uint32_t alpha : 8;
        } comp;
    } OSDHANDLER_ARGB__T;

    /*!
    *************************************************************************
    ** \brief   Union that provides access to an RGBA coded colour
    **          component wise and as a whole.
    *************************************************************************
    */
    typedef union
    {
        //! Element that contains the whole RGBA coded colour.
        uint32_t all;
        //! Element that contains the RGBA coded colour component wise.
        struct
        {
            //! Component that contains the alpha amount only.
            uint32_t alpha : 8;
            //! Component that contains the blue amount only.
            uint32_t b     : 8;
            //! Component that contains the green amount only.
            uint32_t g     : 8;
            //! Component that contains the red amount only.
            uint32_t r     : 8;
        } comp;
    } OSDHANDLER_RGBA__T;

    /*!
    *************************************************************************
    ** \brief   Structure used by OSDHANLDER_SetUpdateArea() function.
    **
    *************************************************************************
    */
    typedef struct
    {
        //! x position
        uint32_t x;
        //! y position
        uint32_t y;
        //! width
        uint32_t width;
        //! height
        uint32_t height;
    } OSDHANDLER_PositionT;

    //***********************************************************************
    //***********************************************************************
    //** Global Data
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** API Functions
    //***********************************************************************
    //***********************************************************************

    #ifdef __cplusplus
    extern "C" {
    #endif
    int32_t  OSDHANDLER_Init(const OSDHANDLER_InitParamsT* initParamsPtr,
                             uint32_t* errorCodePtr);
    uint32_t  OSDHANDLER_Exit(void);
    uint32_t OSDHANDLER_IsInitialized(void);
    int32_t  OSDHANDLER_StartLayer(const uint32_t layerId,
                                   const OSDHANDLER_StartParamsT* startParamsPtr,
                                   uint32_t* errorCodePtr);
    int32_t  OSDHANDLER_StopLayer(const uint32_t layerId,
                                  uint32_t* errorCodePtr);
    int32_t  OSDHANDLER_ShowLayer(const uint32_t layerId,
                                  uint32_t* errorCodePtr);
    int32_t  OSDHANDLER_HideLayer(const uint32_t layerId,
                                  uint32_t* errorCodePtr);
    int32_t  OSDHANDLER_EnableDoubleBuffering(const uint32_t layerId,
                                              uint8_t* memoryPtr);
    int32_t  OSDHANDLER_MarkLayerDirty(const uint32_t layerId);
    int32_t  OSDHANDLER_BlitDirtyDoubleBuffer(const uint32_t layerId);
    void     OSDHANDLER_BlitDirtyDoubleBuffers(void);
    int32_t  OSDHANDLER_SetColourMode(const uint32_t layerId,
                                      const OSDHANDLER_ColourModeT colourMode,
                                      uint32_t* errorCodePtr);
    int32_t  OSDHANDLER_SetLayoutMode(const uint32_t layerId,
                                      const OSDHANDLER_OsdLayoutEnumT layoutMode,
                                      uint32_t* errorCodePtr);
    int32_t  OSDHANDLER_SetFillColour(const uint32_t layerId, const uint32_t colour);
    int32_t  OSDHANDLER_Clear(const uint32_t layerId);
    int32_t  OSDHANDLER_BitBlitFill(const uint32_t layerId,
                                    const uint32_t xx,
                                    const uint32_t yy,
                                    const uint32_t width,
                                    const uint32_t height,
                                    const uint32_t colour);
    int32_t  OSDHANDLER_SetClut(const uint32_t layerId,
                                uint32_t* newClutPtr,
                                uint32_t* errorCodePtr);
    int32_t  OSDHANDLER_SetClutPartially(const uint32_t layerId,
                                         uint32_t*      newClutPtr,
                                         const uint32_t startIndex,
                                         const uint32_t entryCount,
                                         uint32_t* errorCodePtr);
    int32_t  OSDHANDLER_GetClut(const uint32_t layerId, uint32_t* destClutPtr);
    int32_t  OSDHANDLER_Reposition(const uint32_t layerId,
                                   const uint32_t xx,
                                   const uint32_t yy,
                                   uint32_t* errorCodePtr);
    int32_t  OSDHANDLER_Rescale(const uint32_t layerId,
                                const uint32_t outputWidth,
                                const uint32_t outputHeight,
                                uint32_t* errorCodePtr);
    int32_t  OSDHANDLER_GetLayerParameters(const uint32_t layerId,
                                           OSDHANDLER_LayerParamsT* layerParamsPtr);
    int32_t  OSDHANDLER_GetPixelAddress(const uint32_t layerId,
                                        const uint32_t xx,
                                        const uint32_t yy,
                                        uint32_t* pixelBaseAddrPtr,
                                        uint32_t* pixelBitOffsetPtr);
    int32_t  OSDHANDLER_GlobalAlpha(const uint32_t layerId,
                                    const OSDHANDLER_ModeParamT mode,
                                    const uint32_t level);
    int32_t  OSDHANDLER_PlaneFading(const uint32_t layerId,
                                    const OSDHANDLER_ModeParamT mode,
                                    uint32_t* errorCodePtr);
    int32_t  OSDHANDLER_CursorHilight(const OSDHANDLER_ModeParamT onOff,
                                      uint32_t* errorCodePtr);
    int32_t  OSDHANDLER_FlushPicture(const uint32_t layerId);
    int32_t  OSDHANDLER_PlaneOrder(OSDHANDLER_ModeParamT mode,
                                   uint32_t plane5,
                                   uint32_t plane4,
                                   uint32_t plane3,
                                   uint32_t plane2,
                                   uint32_t plane1,
                                   uint32_t plane0,
                                   uint32_t* errorCodePtr);
    uint32_t OSDHANDLER_GetOsdMemConsumption(const uint32_t inputWidth,
                                             const uint32_t inputHeight,
                                             const OSDHANDLER_ColourModeT colourMode);
    uint32_t OSDHANDLER_ConvertARGB2CbYCrA_union(const OSDHANDLER_ARGB__T argbUnion);
    uint32_t OSDHANDLER_ConvertARGB2CbYCrA_word(const uint32_t argbWord);
    uint16_t OSDHANDLER_ConvertARGB2CbYCrA_5551(const uint16_t argb1555);
    uint32_t OSDHANDLER_ConvertRGBA2CbYCrA_union(const OSDHANDLER_RGBA__T rgbaUnion);
    uint32_t OSDHANDLER_ConvertRGBA2CbYCrA_word(const uint32_t rgbaWord);
    uint32_t OSDHANDLER_ConvertCbYCrA2RGBA_union(const OSDHANDLER_CbYCrA__T cbycraUnion);
    uint32_t OSDHANDLER_ConvertCbYCrA2RGBA_word(const uint32_t cbycraWord);
    uint16_t OSDHANDLER_ConvertCbYCrA2RGBA_5551(const uint16_t cbycra5551);
    uint32_t OSDHANDLER_ConvertARGB2CCIR(const uint32_t leftARGB, const uint32_t rightARGB);
    uint32_t OSDHANDLER_ConvertRGBA2CCIR(const uint32_t leftRGBA, const uint32_t rightRGBA);
    void     OSDHANDLER_SetLeftRGBA2CCIR(uint32_t* ccirPtr, uint32_t leftRGBA);
    void     OSDHANDLER_SetRightRGBA2CCIR(uint32_t* ccirPtr, uint32_t rightRGBA);

    // DEBUG functions:
    void     OSDHANDLERDEBUG_DumpLayerParams(uint32_t layer);

    #ifdef __cplusplus
    }
    #endif

#endif // OSDHANDLER_H
