/*
*****************************************************************************
**
** \file        ./subsystems/paint/osdhandler/src/osdhandler_fb.c
**
** \version     $Id: osdhandler_fb.c 5868 2011-03-25 16:53:01Z jrende $
**
** \brief       OSD frame buffer handler for the paint subsystem.
**
** This file contains the implementation of the OSDHANDLER FRAME BUFFER API.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2007 - 2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/


/*! Enables/disables assembler optimized CPU routines. */
#define OSDHANDLER_FB_USE_ASM               0

#include <stdint.h>
#include <fapex/abst_types.h>
#include "osdhandler.h"
#include "osdhandler_priv.h"
#include "osdhandler_fb.h"

#if OSDHANDLER_FB_USE_ASM
#include "osdhandler_asm.h"
#endif

//***************************************************************************
//***************************************************************************
//** Local Defines and Macros
//***************************************************************************
//***************************************************************************

#define OSDHANDLER_FB_STATE_VALID           0
#define OSDHANDLER_FB_STATE_DYNAMIC_MEM     1
#define OSDHANDLER_FB_STATE_CLUT_ALLOCATED  2

#if 0 /*FMC */
#define OSDHANDLER_FB_MALLOC(dst,segment,size)                                  \
                            if(segment)                                         \
                            {                                                   \
                                dst = FAPI_SYS_MALLOC_SEGMENT(segment, size);   \
                            }                                                   \
                            else                                                \
                            {                                                   \
                                dst = FAPI_SYS_MALLOC(size);                    \
                            }
#else
#define OSDHANDLER_FB_MALLOC(dst,segment,size)                                  \
                                dst = FAPI_SYS_MALLOC(size);

#endif
//***************************************************************************
//***************************************************************************
//** Local Structures
//***************************************************************************
//***************************************************************************

typedef struct
{
    /*! Properties of the frame buffer structure. */
    OSDHANDLER_FB_SetupT setup;
    /*! CLUT lock */
    OSDHANDLER_FB_ClutLockT clutLock;
    /*! The physical horizontal resolution of the frame buffer. */
    uint32_t    xResPhy;
    /*! The physical vertical resolution of the frame buffer. */
    uint32_t    yResPhy;
    /*! The internal state of the frame buffer (alloc information). */
    uint32_t    state;
    /*! Base address for the frame buffer. */
    uint32_t*   picBasePtr;
    /*! Allocated memory for frame buffer as returned by allocation function. */
    uint32_t*   picBaseAllocatedPtr;
    /*! colour lookup table */
    uint32_t*   clutPtr;
    /*! bits per pixel      */
    uint32_t    bpp;
} osdhandlerFbHandleT;


//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Local Functions Declaration
//***************************************************************************
//***************************************************************************

static int32_t  osdhandlerFbCheckHandle(OSDHANDLER_FB_HandleT handle);
static uint32_t osdhandlerFbGetBpp(OSDHANDLER_ColourModeT colourMode);

//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Private Functions
//***************************************************************************
//***************************************************************************

/*!
*****************************************************************************
** \brief   Checks if a frame buffer handle is valid.
**
** \param   handle
**              The handle to be checked.
**
** \return
**          0 if the handle is not valid, a value unequal 0 otherwise.
**
*****************************************************************************
*/
static int32_t osdhandlerFbCheckHandle(OSDHANDLER_FB_HandleT handle)
{
    osdhandlerFbHandleT* struct_ptr = (osdhandlerFbHandleT*)handle;

    if(struct_ptr == NULL)
    {
        return(0);
    }
    return((struct_ptr->picBasePtr) && \
           (struct_ptr->setup.colourMode != OSDHANDLER_OSD_MODE_UNKNOWN) && \
           (struct_ptr->setup.colourMode != OSDHANDLER_CURSOR_MODE_UNKNOWN) && \
           (struct_ptr->setup.layoutMode != OSDHANDLER_UNKNOWN_OSD_LAYOUT) && \
           (struct_ptr->state & (1 << OSDHANDLER_FB_STATE_VALID)));
}

/*!
*****************************************************************************
** \brief   Get the number of bits per pixel for a specific colour mode.
**
** \param   colourMode
**              The colour mode to inspect.
**
** \return
**          - the count of bits per pixel on success
**          - 0 if the colour mode is unknown
**
*****************************************************************************
*/
static uint32_t osdhandlerFbGetBpp(OSDHANDLER_ColourModeT colourMode)
{
    switch(colourMode)
    {
        case(OSDHANDLER_OSD_MODE_UNKNOWN):
        case(OSDHANDLER_CURSOR_MODE_UNKNOWN):
        return(0);

        case(OSDHANDLER_CURSOR_MODE_1BIT):
        return(1);

        case(OSDHANDLER_CURSOR_MODE_2BIT):
        return(2);

        case(OSDHANDLER_OSD_MODE_CLUT_4BIT):
        case(OSDHANDLER_CURSOR_MODE_4BIT):
        return(4);

        case(OSDHANDLER_OSD_MODE_CLUT_8BIT):
        case(OSDHANDLER_CURSOR_MODE_8BIT):
        return(8);

        case(OSDHANDLER_OSD_MODE_16BIT):
        case(OSDHANDLER_OSD_MODE_4444):
        case(OSDHANDLER_OSD_MODE_5551):
        return(16);

        case(OSDHANDLER_OSD_MODE_32_24BIT):
        return(32);

        default:
        break;
    }
    return(0);
}


//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

/*!
*****************************************************************************
** \brief   Calculate the memory consumption for a frame buffer.
**
** \param   setup
**              The properties of the frame buffer.
** \param   sizePtr
**              Will receive the size in bytes that is consumed by the
**              frame buffer.
**
** \return
**          - #FAPI_OK
**              if calculation was successful. The result is stored in sizePtr.
**          - #OSDHANDLER_ERR_NOT_INITIALIZED
**              if the OSDHANDLER is not initialized.
**          - #OSDHANDLER_ERR_BAD_PARAMETER
**              if the sizePtr is NULL.
**          - #OSDHANDLER_ERR_OUT_OF_SIZE
**              if the dimensions of the frame buffer properties are invalid.
**          - #OSDHANDLER_ERR_BAD_COLOUR_MODE
**              if the colour mode of the frame buffer is invalid.
**
*****************************************************************************
*/
int32_t OSDHANDLER_FB_CalcMemConsumption (OSDHANDLER_FB_SetupT setup,
                                          uint32_t* sizePtr)
{
    uint32_t bpp        = 0;
    uint32_t h_res_phy  = 0;
    uint32_t v_res_phy  = 0;

    if (!osdhandlerInitialized)
    {
        return(OSDHANDLER_ERR_NOT_INITIALIZED);
    }
    if(sizePtr == NULL)
    {
        return(OSDHANDLER_ERR_BAD_PARAMETER);
    }
    *sizePtr = 0;

    if((setup.xRes > OSDHANDLER_MAX_HRES)
     ||(setup.yRes > OSDHANDLER_MAX_VRES))
    {
        return(OSDHANDLER_ERR_OUT_OF_SIZE);
    }

    bpp = osdhandlerFbGetBpp(setup.colourMode);

    if(bpp == 0)
    {
        return(OSDHANDLER_ERR_BAD_COLOUR_MODE);
    }

    h_res_phy = setup.xRes;
    v_res_phy = setup.yRes;

    if((setup.colourMode == OSDHANDLER_OSD_MODE_CLUT_4BIT)
    || (setup.colourMode == OSDHANDLER_OSD_MODE_CLUT_8BIT)
    || (setup.colourMode == OSDHANDLER_OSD_MODE_16BIT)
    || (setup.colourMode == OSDHANDLER_OSD_MODE_4444)
    || (setup.colourMode == OSDHANDLER_OSD_MODE_5551)
    || (setup.colourMode == OSDHANDLER_OSD_MODE_32_24BIT))
    { // processing for OSD requirements
        // one line must allocate a multiple of OSDHANDLER_OSD_BURST_LENGTH bytes:
        if (h_res_phy % ((OSDHANDLER_OSD_BURST_LENGTH << 3) / bpp))
        {
            h_res_phy += ((OSDHANDLER_OSD_BURST_LENGTH << 3) / bpp) - \
                         (h_res_phy % ((OSDHANDLER_OSD_BURST_LENGTH << 3) / bpp));
        }
        // v_res must be a multiple of 2
        if (v_res_phy % 2)
        {
            v_res_phy++;
        }
        *sizePtr = ((h_res_phy * v_res_phy * bpp) >> 3) + \
                   OSDHANDLER_OSD_BASE_ADDR_ALIGN;
    }
    else // // processing for CP requirements
    {
        // one line must allocate a multiple of OSDHANDLER_CP_BURST_LENGTH bytes:
        if (h_res_phy % ((OSDHANDLER_CP_BURST_LENGTH << 3) / bpp))
        {
            h_res_phy += ((OSDHANDLER_CP_BURST_LENGTH << 3) / bpp) - \
                         (h_res_phy % ((OSDHANDLER_CP_BURST_LENGTH << 3) / bpp));
        }
        // v_res must be a multiple of 2
        if (v_res_phy % 2)
        {
            v_res_phy++;
        }
        *sizePtr = ((h_res_phy * v_res_phy * bpp) >> 3) + \
                   OSDHANDLER_CP_BASE_ADDR_ALIGN;
    }
    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Allocate a frame buffer handle.
**
** \param   setup
**              Properties of the requested frame buffer.
**              This structure must be set up before calling this function.
** \param   picBasePtr
**              Optional argument that specifies the frame buffer address.
**              Make sure that the address is aligned according to the hardware
**              requirements of the used Fujitsu decoder. Also make sure to
**              provide enough memory (see OSDHANDLER_FB_CalcMemConsumption()).
**              If the argument is NULL, memory will be allocated automatically
**              from the segment specified by the 'segment' parameter.
** \param   memorySegment
**              Memory will be allocated from this segment for the frame buffer
**              handle and the frame buffer memory (if the 'picBasePtr'
**              parameter is NULL). If the 'segment' parameter is NULL,
**              memory will be allocated from the default segment.
** \param   handlePtr
**              Will receive the frame buffer handle that will be set up.
**
** \return
**          - #FAPI_OK
**              if a handle was successfully set up.
**          - #OSDHANDLER_ERR_NOT_INITIALIZED
**              if the OSDHANDLER is not initialized.
**          - #OSDHANDLER_ERR_BAD_PARAMETER
**              if the handlePtr is NULL.
**          - #OSDHANDLER_ERR_OUT_OF_SIZE
**              if the dimensions of the frame buffer properties are invalid.
**          - #OSDHANDLER_ERR_BAD_COLOUR_MODE
**              if the colour mode of the frame buffer properties is invalid.
**          - #OSDHANDLER_ERR_OUT_OF_MEMORY
**              if there is not enough memory available for allocation.
**
*****************************************************************************
*/
int32_t OSDHANDLER_FB_AllocateHandle
                (OSDHANDLER_FB_SetupT setup,
                 uint32_t* picBasePtr,
                 FAPI_SYS_HandleT memorySegment,
                 OSDHANDLER_FB_HandleT* handlePtr )
{
    uint32_t h_res_phy  = 0;
    uint32_t v_res_phy  = 0;
    uint32_t bpp = 0;
    osdhandlerFbHandleT* handle_ptr = NULL;

    if (!osdhandlerInitialized)
    {
        return(OSDHANDLER_ERR_NOT_INITIALIZED);
    }
    if(handlePtr == NULL)
    {
        return(OSDHANDLER_ERR_BAD_PARAMETER);
    }
    if((setup.xRes > OSDHANDLER_MAX_HRES)
     ||(setup.yRes > OSDHANDLER_MAX_VRES))
    {
        return(OSDHANDLER_ERR_OUT_OF_SIZE);
    }
    switch (setup.colourMode)
    {
        case(OSDHANDLER_OSD_MODE_UNKNOWN):
            bpp = 0;
            break;
        case(OSDHANDLER_OSD_MODE_CLUT_4BIT):
            bpp = 4;
            break;
        case(OSDHANDLER_OSD_MODE_CLUT_8BIT):
            bpp = 8;
            break;
        case(OSDHANDLER_OSD_MODE_16BIT):
        case(OSDHANDLER_OSD_MODE_4444):
        case(OSDHANDLER_OSD_MODE_5551):
            bpp = 16;
            break;
        case(OSDHANDLER_OSD_MODE_32_24BIT):
            bpp = 32;
            break;
        case(OSDHANDLER_CURSOR_MODE_UNKNOWN):
            bpp = 0;
            break;
        case(OSDHANDLER_CURSOR_MODE_1BIT):
            bpp = 1;
            break;
        case(OSDHANDLER_CURSOR_MODE_2BIT):
            bpp = 2;
            break;
        case(OSDHANDLER_CURSOR_MODE_4BIT):
            bpp = 4;
            break;
        case(OSDHANDLER_CURSOR_MODE_8BIT):
            bpp = 8;
            break;
        default:
            bpp = 0;
            break;
    }
    if(bpp == 0)
    {
        return(OSDHANDLER_ERR_BAD_COLOUR_MODE);
    }

    h_res_phy = setup.xRes;
    v_res_phy = setup.yRes;

    if((setup.colourMode == OSDHANDLER_OSD_MODE_CLUT_4BIT)
    || (setup.colourMode == OSDHANDLER_OSD_MODE_CLUT_8BIT)
    || (setup.colourMode == OSDHANDLER_OSD_MODE_16BIT)
    || (setup.colourMode == OSDHANDLER_OSD_MODE_4444)
    || (setup.colourMode == OSDHANDLER_OSD_MODE_5551)
    || (setup.colourMode == OSDHANDLER_OSD_MODE_32_24BIT))
    {
        // set up for OSD requirements
        // one line must allocate a multiple of OSDHANDLER_OSD_BURST_LENGTH bytes:
        if (h_res_phy % ((OSDHANDLER_OSD_BURST_LENGTH << 3) / bpp))
        {
            h_res_phy += ((OSDHANDLER_OSD_BURST_LENGTH << 3) / bpp) - \
                         (h_res_phy % ((OSDHANDLER_OSD_BURST_LENGTH << 3) / bpp));
        }
        // v_res must be a multiple of 2
        if (v_res_phy % 2)
        {
            v_res_phy++;
        }

        // prepare the handle:
        OSDHANDLER_FB_MALLOC(handle_ptr, memorySegment, sizeof(osdhandlerFbHandleT));
        if(handle_ptr == NULL)
        {
            return(OSDHANDLER_ERR_OUT_OF_MEMORY);
        }
        memset(handle_ptr, 0, sizeof(osdhandlerFbHandleT));
        memcpy(&handle_ptr->setup, &setup, sizeof(OSDHANDLER_FB_SetupT));

        if(picBasePtr)
        {
            handle_ptr->picBaseAllocatedPtr = picBasePtr;
        }
        else
        {
            // allocate automatically and note down allocation in the state variable
            OSDHANDLER_FB_MALLOC (handle_ptr->picBaseAllocatedPtr, memorySegment, \
                (((h_res_phy * v_res_phy * bpp) >> 3) + OSDHANDLER_OSD_BASE_ADDR_ALIGN));
            if(handle_ptr->picBaseAllocatedPtr == NULL)
            {
                memset(handle_ptr, 0, sizeof(osdhandlerFbHandleT));
                FAPI_SYS_FREE(handle_ptr);
                return(OSDHANDLER_ERR_OUT_OF_MEMORY);
            }
            handle_ptr->state |= (1 << OSDHANDLER_FB_STATE_DYNAMIC_MEM);
        }
        /*lint -save -e923 */
        if(((uint32_t)handle_ptr->picBaseAllocatedPtr) % \
            OSDHANDLER_OSD_BASE_ADDR_ALIGN)
        {
            handle_ptr->picBasePtr = handle_ptr->picBaseAllocatedPtr + \
                                     ((OSDHANDLER_OSD_BASE_ADDR_ALIGN - \
                                     (((uint32_t)handle_ptr->picBaseAllocatedPtr) % \
                                     OSDHANDLER_OSD_BASE_ADDR_ALIGN)) / 4);
        }
        /*lint -restore */
        else
        {
            handle_ptr->picBasePtr = handle_ptr->picBaseAllocatedPtr;
        }
    }
    else // set up for Cursor Plane requirements
    {
        // one line must allocate a multiple of OSDHANDLER_CP_BURST_LENGTH bytes:
        if (h_res_phy % ((OSDHANDLER_CP_BURST_LENGTH << 3) / bpp))
        {
            h_res_phy += ((OSDHANDLER_CP_BURST_LENGTH << 3) / bpp) - \
                         (h_res_phy % ((OSDHANDLER_CP_BURST_LENGTH << 3) / bpp));
        }
        // v_res must be a multiple of 2
        if (v_res_phy % 2)
        {
            v_res_phy++;
        }

        // the Cursor Plane must NOT exceed 16 KiB of memory:
        if(((h_res_phy * v_res_phy * bpp) >> 3) > OSDHANDLER_CP_MAX_SIZE)
        {
            return(OSDHANDLER_ERR_CP_4KB_EXCEEDED);
        }

        // prepare the handle:
        OSDHANDLER_FB_MALLOC(handle_ptr, memorySegment, sizeof(osdhandlerFbHandleT));
        if(handle_ptr == NULL)
        {
            return(OSDHANDLER_ERR_OUT_OF_MEMORY);
        }
        memset(handle_ptr, 0, sizeof(osdhandlerFbHandleT));
        memcpy(&handle_ptr->setup, &setup, sizeof(OSDHANDLER_FB_SetupT));

        if(picBasePtr)
        {
            handle_ptr->picBaseAllocatedPtr = picBasePtr;
        }
        else
        {
            // allocate automatically and note down allocation in the state variable
            OSDHANDLER_FB_MALLOC (handle_ptr->picBaseAllocatedPtr, memorySegment, \
                (((h_res_phy * v_res_phy * bpp) >> 3) + OSDHANDLER_CP_BASE_ADDR_ALIGN));
            if(handle_ptr->picBaseAllocatedPtr == NULL)
            {
                memset(handle_ptr, 0, sizeof(osdhandlerFbHandleT));
                FAPI_SYS_FREE(handle_ptr);
                return(OSDHANDLER_ERR_OUT_OF_MEMORY);
            }
            handle_ptr->state |= (1 << OSDHANDLER_FB_STATE_DYNAMIC_MEM);
        }
        /*lint -save -e923 */
        if(((uint32_t)handle_ptr->picBaseAllocatedPtr) % \
            OSDHANDLER_CP_BASE_ADDR_ALIGN)
        {
            handle_ptr->picBasePtr = handle_ptr->picBaseAllocatedPtr + \
                                     ((OSDHANDLER_CP_BASE_ADDR_ALIGN - \
                                     (((uint32_t)handle_ptr->picBaseAllocatedPtr) % \
                                     OSDHANDLER_CP_BASE_ADDR_ALIGN)) / 4);
        }
        /*lint -restore */
        else
        {
            handle_ptr->picBasePtr = handle_ptr->picBaseAllocatedPtr;
        }
    }

    handle_ptr->xResPhy = h_res_phy;
    handle_ptr->yResPhy = v_res_phy;
    handle_ptr->bpp     = bpp;

    // allocate a CLUT for indexed colour modes:
    if((setup.colourMode == OSDHANDLER_OSD_MODE_CLUT_4BIT)
    || (setup.colourMode == OSDHANDLER_OSD_MODE_CLUT_8BIT)
    || (setup.colourMode == OSDHANDLER_CURSOR_MODE_1BIT)
    || (setup.colourMode == OSDHANDLER_CURSOR_MODE_2BIT)
    || (setup.colourMode == OSDHANDLER_CURSOR_MODE_4BIT)
    || (setup.colourMode == OSDHANDLER_CURSOR_MODE_8BIT))
    {
        OSDHANDLER_FB_MALLOC(handle_ptr->clutPtr, memorySegment, \
                                ((1UL << bpp) * sizeof(uint32_t)));
        if(handle_ptr->clutPtr == NULL)
        {
            if((handle_ptr->state & (1 << OSDHANDLER_FB_STATE_DYNAMIC_MEM)) && \
                handle_ptr->picBaseAllocatedPtr)
            {
                FAPI_SYS_FREE(handle_ptr->picBaseAllocatedPtr);
            }
            memset(handle_ptr, 0, sizeof(osdhandlerFbHandleT));
            FAPI_SYS_FREE(handle_ptr);
            return(OSDHANDLER_ERR_OUT_OF_MEMORY);
        }
        memset(handle_ptr->clutPtr, 0, ((1UL << bpp) * sizeof(uint32_t)));
        handle_ptr->state |= (1 << OSDHANDLER_FB_STATE_CLUT_ALLOCATED);
    }

    handle_ptr->state |= (1 << OSDHANDLER_FB_STATE_VALID);

    *handlePtr = (OSDHANDLER_FB_HandleT)handle_ptr;
    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Destroys a frame buffer handle.
**
** \param   handle
**              The frame buffer handle to be destroyed.
**
** \return
**          - #FAPI_OK
**              if the frame buffer handle was successfully destroyed.
**          - #OSDHANDLER_ERR_NOT_INITIALIZED
**              if the OSDHANDLER is not initialized.
**          - #OSDHANDLER_FB_ERR_BAD_HANDLE
**              if the frame buffer handle is not valid.
**
*****************************************************************************
*/
int32_t OSDHANDLER_FB_DestroyHandle(OSDHANDLER_FB_HandleT handle)
{
    osdhandlerFbHandleT* handle_ptr = NULL;

    if (!osdhandlerInitialized)
    {
        return(OSDHANDLER_ERR_NOT_INITIALIZED);
    }
    if(!osdhandlerFbCheckHandle(handle))
    {
        return(OSDHANDLER_FB_ERR_BAD_HANDLE);
    }
    handle_ptr = (osdhandlerFbHandleT*)handle;

    if((handle_ptr->state & (1 << OSDHANDLER_FB_STATE_DYNAMIC_MEM)) && \
       (handle_ptr->picBaseAllocatedPtr))
    {
        FAPI_SYS_FREE(handle_ptr->picBaseAllocatedPtr);
    }
    if((handle_ptr->state & (1 << OSDHANDLER_FB_STATE_CLUT_ALLOCATED)) && \
       (handle_ptr->clutPtr))
    {
        FAPI_SYS_FREE(handle_ptr->clutPtr);
    }

    memset(handle_ptr, 0, sizeof(osdhandlerFbHandleT));
    FAPI_SYS_FREE(handle_ptr);

    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Set up the CLUT of a framebuffer.
**
** \param   handle
**              The frame buffer handle to be set up.
**
** \param   clutPtr
**              References the CLUT that will be copied to the frame buffer.
**              This pointer can be safely deallocated after setup of the CLUT.
**
** \return
**          - #FAPI_OK
**              on success.
**          - #OSDHANDLER_ERR_NOT_INITIALIZED
**              if the OSDHANDLER is not initialized.
**          - #OSDHANDLER_FB_ERR_BAD_HANDLE
**              if the frame buffer handle is not valid.
**          - #OSDHANDLER_ERR_BAD_PARAMETER
**              if the clutPtr is NULL.
**          - #OSDHANDLER_FB_ERR_BAD_COLOUR_MODE
**              if the framebuffer is driven a colour mode with no CLUT.
**
*****************************************************************************
*/
int32_t OSDHANDLER_FB_SetClut(OSDHANDLER_FB_HandleT handle, \
                              uint32_t* clutPtr)
{
    osdhandlerFbHandleT* handle_ptr = NULL;

    if(!osdhandlerInitialized)
    {
        return(OSDHANDLER_ERR_NOT_INITIALIZED);
    }
    if(!osdhandlerFbCheckHandle(handle))
    {
        return(OSDHANDLER_FB_ERR_BAD_HANDLE);
    }
    if(clutPtr == NULL)
    {
        return(OSDHANDLER_ERR_BAD_PARAMETER);
    }
    handle_ptr = (osdhandlerFbHandleT*)handle;
    if(handle_ptr->bpp == 0)
    {
        return(OSDHANDLER_FB_ERR_BAD_COLOUR_MODE);
    }
    memcpy(handle_ptr->clutPtr, clutPtr, ((1UL << handle_ptr->bpp) * sizeof(uint32_t)));
    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Set up the CLUT lock of a framebuffer.
**
**          OSDHANDLER_FB_CLUT_LOCK_OFF: [default] Causes libraries that modify
**          the frame buffer content to set up the CLUT of the frame buffer,
**          e.g., when decoding picture files that contain a CLUT.
**          OSDHANDLER_FB_CLUT_LOCK_ON: Causes libraries that modify the
**          frame buffer content not to change the CLUT of the frame buffer.
**
** \param   handle
**              The frame buffer handle to be set up.
**
** \param   clutLock
**              Specifies whether to enable or disable the CLUT lock.
**
** \return
**          - #FAPI_OK
**              on success.
**          - #OSDHANDLER_ERR_NOT_INITIALIZED
**              if the OSDHANDLER is not initialized.
**          - #OSDHANDLER_FB_ERR_BAD_HANDLE
**              if the frame buffer handle is not valid.
**
*****************************************************************************
*/
int32_t OSDHANDLER_FB_SetClutLock(OSDHANDLER_FB_HandleT handle, \
                                  OSDHANDLER_FB_ClutLockT clutLock)
{
    osdhandlerFbHandleT* handle_ptr = NULL;

    if(!osdhandlerInitialized)
    {
        return(OSDHANDLER_ERR_NOT_INITIALIZED);
    }
    if(!osdhandlerFbCheckHandle(handle))
    {
        return(OSDHANDLER_FB_ERR_BAD_HANDLE);
    }
    handle_ptr = (osdhandlerFbHandleT*)handle;
    handle_ptr->clutLock = clutLock;
    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Fill a frame buffer with a colour value.
**
** \param   handle
**              The frame buffer handle.
**
** \param   colour
**              The frame buffer will be filled with this colour value.
**
** \return
**          - #FAPI_OK
**              on success.
**          - #OSDHANDLER_ERR_NOT_INITIALIZED
**              if the OSDHANDLER is not initialized.
**          - #OSDHANDLER_FB_ERR_BAD_HANDLE
**              if the frame buffer handle is not valid.
**          - #OSDHANDLER_ERR_BAD_PARAMETER
**              if the colour code has too many bits for the colour mode
**              of the frame buffer.
**
*****************************************************************************
*/
int32_t OSDHANDLER_FB_Fill(OSDHANDLER_FB_HandleT handle, uint32_t colour)
{
    uint32_t  bpp        = 0;
    uint32_t  ii         = 0;
    uint32_t  count      = 0;
    uint32_t  colour_32  = 0UL;
    uint32_t* dst_32_ptr = NULL;
    int32_t result;

    osdhandlerFbHandleT* handle_ptr = NULL;

// assember optimized routine variables:
#if OSDHANDLER_FB_USE_ASM       \
&& defined(__arm__)             \
&& !defined(__GNUC__)
    OSDHANDLER_RectT dst_rect;
    uint32_t layer_stride = 0;
#endif // OSDHANDLER_FB_USE_ASM

    if (!osdhandlerInitialized)
    {
        return(OSDHANDLER_ERR_NOT_INITIALIZED);
    }
    if(!osdhandlerFbCheckHandle(handle))
    {
        return(OSDHANDLER_FB_ERR_BAD_HANDLE);
    }
    handle_ptr = (osdhandlerFbHandleT*)handle;
    bpp = osdhandlerFbGetBpp(handle_ptr->setup.colourMode);

    // check if the colour has too many bits for the colour mode of the FB:
    if(colour & ~((1UL << bpp) - 1UL))
    {
        return(OSDHANDLER_ERR_BAD_PARAMETER);
    }

    dst_32_ptr = handle_ptr->picBasePtr;

    // =========== Assembler optimized CPU routine =============
#if OSDHANDLER_FB_USE_ASM       \
&& defined(__arm__)             \
&& !defined(__GNUC__)

    if(bpp >= 8)
    {
        layer_stride = (handle_ptr->xResPhy * bpp) >> 3;

        memset(&dst_rect, 0, sizeof(dst_rect));

        dst_rect.x = 0;
        dst_rect.y = 0;
        dst_rect.w = handle_ptr->xResPhy;
        dst_rect.h = handle_ptr->yResPhy;
        dst_rect.c = colour;

        switch(bpp)
        {
        case 8:
            OSD_Fill8bpp ((uint8_t*)dst_32_ptr, &dst_rect, layer_stride);
            return(FAPI_OK);
        case 16:
            OSD_Fill16bpp((uint16_t*)dst_32_ptr, &dst_rect, layer_stride);
            return(FAPI_OK);
        case 32:
            OSD_Fill32bpp(dst_32_ptr, &dst_rect, layer_stride);
            return(FAPI_OK);
        default:
            break;
        }
    }
#endif // OSDHANDLER_FB_USE_ASM
    // ======== end of Assembler optimized CPU routine =========

    // generate 32-bit colour code
    colour_32 = colour;
    for (ii = ((32 / bpp) - 1); ii != 0; ii--)
    {
        colour_32 = ((colour_32 << bpp) | colour);
    }
    count = (handle_ptr->yResPhy * handle_ptr->xResPhy * bpp) >> 5;

    // ============= CPU only =============
    ii = count;
    while(ii--)
    {
        *dst_32_ptr++ = colour_32;
    }
    /*lint -save -e923 */
//    FAPI_SYS_FlushDataCache((uint32_t)handle_ptr->picBasePtr, count << 2);
//    result = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)osd_cp_params_ptr->fd,
//                                FAPEX_IOCCOM_VIOUT_FLUSH_PLANE,
//                                (FAPEXIOCARG3_T)NULL );
    /*lint -restore */
    // ========= end of CPU only ==========
    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Get verbose information about a frame buffer structure.
**
** \param   handle
**              The frame buffer handle to be investigated.
**
** \param   frameBufferInfoPtr
**              This structure will receive the verbose information about
**              the frame buffer.
**
** \return
**          - #FAPI_OK
**              on success.
**          - #OSDHANDLER_ERR_NOT_INITIALIZED
**              if the OSDHANDLER is not initialized.
**          - #OSDHANDLER_FB_ERR_BAD_HANDLE
**              if the frame buffer handle is not valid.
**          - #OSDHANDLER_ERR_BAD_PARAMETER
**              if the frameBufferInfoPtr is NULL.
**
*****************************************************************************
*/
int32_t OSDHANDLER_FB_GetInfo(OSDHANDLER_FB_HandleT handle, \
                              OSDHANDLER_FB_FrameBufferT* frameBufferInfoPtr)
{
    osdhandlerFbHandleT* handle_ptr = NULL;

    if (!osdhandlerInitialized)
    {
        return(OSDHANDLER_ERR_NOT_INITIALIZED);
    }
    if(!osdhandlerFbCheckHandle(handle))
    {
        return(OSDHANDLER_FB_ERR_BAD_HANDLE);
    }
    if(frameBufferInfoPtr == NULL)
    {
        return(OSDHANDLER_ERR_BAD_PARAMETER);
    }

    handle_ptr = (osdhandlerFbHandleT*)handle;

    frameBufferInfoPtr->picBasePtr = handle_ptr->picBasePtr;
    frameBufferInfoPtr->colourMode = handle_ptr->setup.colourMode;
    frameBufferInfoPtr->layoutMode = handle_ptr->setup.layoutMode;
    frameBufferInfoPtr->clutLock   = handle_ptr->clutLock;
    frameBufferInfoPtr->xResApp    = handle_ptr->setup.xRes;
    frameBufferInfoPtr->yResApp    = handle_ptr->setup.yRes;
    frameBufferInfoPtr->xResPhy    = handle_ptr->xResPhy;
    frameBufferInfoPtr->yResPhy    = handle_ptr->yResPhy;
    frameBufferInfoPtr->clutPtr    = handle_ptr->clutPtr;
    frameBufferInfoPtr->bpp        = handle_ptr->bpp;
    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Get the address of a pixel in the frame buffer memory.
**
** \param   handle
**              The frame buffer handle to be investigated.
**
** \param   xx
**              Coordinates of the demanded pixel.
**
** \param   yy
**              Coordinates of the demanded pixel.
**
** \param   pixelBaseAddrPtrPtr
**              Will receive a 32-bit aligned address that contains the
**              desired pixel.
**
** \param   pixelBitOffsetPtr
**              Will receive the offset in terms of pixels from the MSB side
**              in the 32-bit word pointed to by *pixelBaseAddrPtrPtr.
**
** \return
**          - #FAPI_OK
**              on success.
**          - #OSDHANDLER_ERR_NOT_INITIALIZED
**              if the OSDHANDLER is not initialized.
**          - #OSDHANDLER_FB_ERR_BAD_HANDLE
**              if the frame buffer handle is not valid.
**          - #OSDHANDLER_ERR_BAD_PARAMETER
**              if pixelBaseAddrPtrPtr or pixelBitOffsetPtr is NULL.
**
*****************************************************************************
*/
int32_t OSDHANDLER_FB_GetPixelAddress(OSDHANDLER_FB_HandleT handle, \
                                      uint32_t xx, \
                                      uint32_t yy, \
                                      uint32_t** pixelBaseAddrPtrPtr, \
                                      uint32_t* pixelBitOffsetPtr)
{
    osdhandlerFbHandleT* handle_ptr = NULL;
    //uint32_t addr = 0;

    *pixelBaseAddrPtrPtr = NULL;
    *pixelBitOffsetPtr = 0;

    if (!osdhandlerInitialized)
    {
        return(OSDHANDLER_ERR_NOT_INITIALIZED);
    }
    if((pixelBaseAddrPtrPtr == NULL)
    || (pixelBitOffsetPtr == NULL))
    {
        return(OSDHANDLER_ERR_BAD_PARAMETER);
    }
    if(!osdhandlerFbCheckHandle(handle))
    {
        return(OSDHANDLER_FB_ERR_BAD_HANDLE);
    }
    handle_ptr = (osdhandlerFbHandleT*)handle;

    ///*lint -save -e923 */
    //addr = ((uint32_t)handle_ptr->picBasePtr) +
    //        ((yy * handle_ptr->xResPhy * handle_ptr->bpp) >> 3) +
    //        ((xx * handle_ptr->bpp) >> 3); // /8 ==> byte address
    //addr -= addr % 4; //4-byte aligned address
    //*pixelBaseAddrPtrPtr = (uint32_t*)addr;
    //*pixelBitOffsetPtr = (xx * handle_ptr->bpp) % 32;
    ///*lint -restore */

    *pixelBaseAddrPtrPtr = handle_ptr->picBasePtr +
            ((yy * handle_ptr->xResPhy * handle_ptr->bpp) >> 5) +
            ((xx * handle_ptr->bpp) >> 5); // ==> 32-bit word address
    *pixelBitOffsetPtr = (xx * handle_ptr->bpp) % 32; // ==> offset from left side
    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Display a frame buffer on a hardware OSD.
**
** \param   handle
**              The handle of the frame buffer to be displayed.
**
** \param   startParamsPtr
**              Start parameters specifying a layerId, OSD position and size.
**              Note that the scaledWidth and scaledHeight parameters are
**              only applied for hardware OSDs with a built-in hardware scaler.
**
** \return
**          - #FAPI_OK
**              on success.
**          - #OSDHANDLER_ERR_NOT_INITIALIZED
**              if the OSDHANDLER is not initialized.
**          - #OSDHANDLER_ERR_BAD_PARAMETER
**              if the startParamsPtr is NULL.
**          - #OSDHANDLER_FB_ERR_BAD_HANDLE
**              if the frame buffer handle is not valid.
**          - #OSDHANDLER_ERR_BAD_LAYER_ID
**              if the layerId is invalid.
**          - #OSDHANDLER_ERR_NO_OSD_HANDLE
**              if the OSDHANDLER has not allocated the handle of
**              the requested OSD/CP at initialization.
**          - #OSDHANDLER_ERR_BAD_COLOUR_MODE
**              if the colour mode of the frame buffer can not be
**              applied to the requested hardware OSD.
**          - some VISCALE specific error code on driver failure.
**
** \sa      - #OSDHANDLER_FB_Stop
**
*****************************************************************************
*/
int32_t OSDHANDLER_FB_Display(OSDHANDLER_FB_HandleT handle,
                              OSDHANDLER_FB_StartParamsT* startParamsPtr)
{
    osdhandlerFbHandleT* handle_ptr = NULL;
    OSDHANDLER_FB_StartParamsT start;
    OSDHANDLER_OsdCpParamsT* osd_cp_params_ptr = NULL;
    FAPEX_VIOUT_ResizeParamStrT params;
    int32_t  ii     = 0;
    int32_t  result = 0;

    if (!osdhandlerInitialized)
    {
        return(OSDHANDLER_ERR_NOT_INITIALIZED);
    }
    if(startParamsPtr == NULL)
    {
        return(OSDHANDLER_ERR_BAD_PARAMETER);
    }
    if(!osdhandlerFbCheckHandle(handle))
    {
        return(OSDHANDLER_FB_ERR_BAD_HANDLE);
    }
    handle_ptr = (osdhandlerFbHandleT*)handle;
    osd_cp_params_ptr = osdhandlerGetOsdCpParams(startParamsPtr->layerId);
    if(osd_cp_params_ptr == NULL)
    {
        return(OSDHANDLER_ERR_BAD_LAYER_ID);
    }
//    if (osd_cp_params_ptr->handle == NULL)
//    {
//        return(OSDHANDLER_ERR_NO_OSD_HANDLE);
//    }
    if((startParamsPtr->layerId == OSDHANDLER_OSD1)
    || (startParamsPtr->layerId == OSDHANDLER_OSD2)
    || (startParamsPtr->layerId == OSDHANDLER_OSD3)
    || (startParamsPtr->layerId == OSDHANDLER_OSD4))
    {
        if((handle_ptr->setup.colourMode == OSDHANDLER_OSD_MODE_UNKNOWN)
        || (handle_ptr->setup.colourMode == OSDHANDLER_CURSOR_MODE_UNKNOWN)
        || (handle_ptr->setup.colourMode == OSDHANDLER_CURSOR_MODE_1BIT)
        || (handle_ptr->setup.colourMode == OSDHANDLER_CURSOR_MODE_2BIT)
        || (handle_ptr->setup.colourMode == OSDHANDLER_CURSOR_MODE_4BIT)
        || (handle_ptr->setup.colourMode == OSDHANDLER_CURSOR_MODE_8BIT))
        {
            return(OSDHANDLER_ERR_BAD_COLOUR_MODE);
        }
    }
    if(startParamsPtr->layerId == OSDHANDLER_CURSOR)
    {
        if((handle_ptr->setup.colourMode == OSDHANDLER_OSD_MODE_UNKNOWN)
        || (handle_ptr->setup.colourMode == OSDHANDLER_OSD_MODE_CLUT_4BIT)
        || (handle_ptr->setup.colourMode == OSDHANDLER_OSD_MODE_CLUT_8BIT)
        || (handle_ptr->setup.colourMode == OSDHANDLER_OSD_MODE_16BIT)
        || (handle_ptr->setup.colourMode == OSDHANDLER_OSD_MODE_4444)
        || (handle_ptr->setup.colourMode == OSDHANDLER_OSD_MODE_5551)
        || (handle_ptr->setup.colourMode == OSDHANDLER_OSD_MODE_32_24BIT)
        || (handle_ptr->setup.colourMode == OSDHANDLER_CURSOR_MODE_UNKNOWN))
        {
            return(OSDHANDLER_ERR_BAD_COLOUR_MODE);
        }
    }

    // update hardware OSD CLUT:
    if(handle_ptr->bpp <= 8)
    {
        for(ii = (int32_t)((1UL << handle_ptr->bpp) - 1); ii >= 0; ii--)
        {
//            osd_cp_params_ptr->scalerParams.colourLut[ii] = *(handle_ptr->clutPtr + ii);
            osd_cp_params_ptr->clut[ii] = *(handle_ptr->clutPtr + ii);
        }
    }

    if((osd_cp_params_ptr == &osdhandlerOsd1Params)
    || (osd_cp_params_ptr == &osdhandlerOsd2Params)
    || (osd_cp_params_ptr == &osdhandlerOsd3Params)
    || (osd_cp_params_ptr == &osdhandlerOsd4Params))
    {
        // ************** start the OSD plane  **********************
        memcpy(&start, startParamsPtr, sizeof(OSDHANDLER_FB_StartParamsT));

        if(osd_cp_params_ptr == &osdhandlerOsd1Params)
        {
            start.scaledWidth = (startParamsPtr->scaledWidth > 0) ?         \
                                      startParamsPtr->scaledWidth :         \
                                      handle_ptr->setup.xRes;
            start.scaledHeight = (startParamsPtr->scaledHeight > 0) ?       \
                                      startParamsPtr->scaledHeight :        \
                                      (startParamsPtr->frameBased ?         \
                                          handle_ptr->setup.yRes :     \
                                         (handle_ptr->setup.yRes / 2));

            // apply system limitations:
            if(start.scaledWidth > osdhandlerMaxHRES)
            {
                start.scaledWidth = osdhandlerMaxHRES;
            }
            if(start.scaledHeight > osdhandlerMaxVRES)
            {
                start.scaledHeight = osdhandlerMaxVRES;
            }
        }
        else
        {
            start.scaledWidth  = handle_ptr->setup.xRes;
            start.scaledHeight = start.frameBased ?
                                 handle_ptr->setup.yRes :
                                (handle_ptr->setup.yRes / 2);

            if((start.scaledWidth  > osdhandlerMaxHRES) ||
               (start.scaledHeight > osdhandlerMaxVRES))
            {
                return(OSDHANDLER_ERR_OUT_OF_SIZE);
            }
        }
        // avoid the picture to be started outside of the active display area:
        if((start.xPos + start.scaledWidth) > osdhandlerMaxHRES)
        {
            start.xPos = osdhandlerMaxHRES - start.scaledWidth;
        }
        if((start.yPos + start.scaledHeight) > osdhandlerMaxVRES)
        {
            start.yPos = osdhandlerMaxVRES - start.scaledHeight;
        }

        osd_cp_params_ptr->scalerParams.outputWindowX0 = start.xPos;
        osd_cp_params_ptr->scalerParams.outputWindowY0 = start.yPos;
        osd_cp_params_ptr->scalerParams.inputWindowWidth = handle_ptr->setup.xRes;
        osd_cp_params_ptr->scalerParams.inputWindowHeight = handle_ptr->setup.yRes;
//        osd_cp_params_ptr->scalerParams.inputPicWidth = handle_ptr->xResPhy;
//        osd_cp_params_ptr->scalerParams.inputPicHeight = handle_ptr->yResPhy;
        osd_cp_params_ptr->scalerParams.outputWindowWidth = start.scaledWidth;
        osd_cp_params_ptr->scalerParams.outputWindowHeight = start.scaledHeight;
        osd_cp_params_ptr->scalerParams.flagFrameField = (start.frameBased) ? 1 : 0;

        osd_cp_params_ptr->colourMode = handle_ptr->setup.colourMode;
        osd_cp_params_ptr->scalerParams.mode = osdhandlerGetOsdColourMode(handle_ptr->setup.colourMode);
        osd_cp_params_ptr->osdLayout  = handle_ptr->setup.layoutMode;
        osd_cp_params_ptr->scalerParams.osdByteOrder = osdhandlerGetOsdLayout(handle_ptr->setup.layoutMode);

        if((osd_cp_params_ptr->isMemoryDynamic)
         &&(osd_cp_params_ptr->allocatedMemPtr != NULL))
        {
            FAPI_SYS_FREE(osd_cp_params_ptr->allocatedMemPtr);
            osd_cp_params_ptr->allocatedMemPtr = NULL;
        }
        osd_cp_params_ptr->isMemoryDynamic = 0;
        osd_cp_params_ptr->allocatedMemPtr = (uint8_t*)handle_ptr->picBasePtr;


        /*lint -save -e923 */
//        osd_cp_params_ptr->scalerParams.picBase = (uint32_t)handle_ptr->picBasePtr;
        /*lint -restore */

//        result = FAPI_VISCALE_Start(osd_cp_params_ptr->handle,
//                                    &osd_cp_params_ptr->scalerParams);
//        // TODO: an operation timeout occurs for OSD2..4?
//        if((result != FAPI_OK) && (result != FAPI_VISCALE_ERR_OPERATION_TIMEOUT))
//        {
//            return(result);
//        }
        result = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)osd_cp_params_ptr->fd,
                                    FAPEX_IOCCOM_START,
                                    (FAPEXIOCARG3_T)NULL );
        if((result != 0) )
        {
            FAPI_SYS_PRINT_MSG("OSDHANDLER_FB_Display--->IOCTL.start,error=%d\n",result);
        }
        if(osd_cp_params_ptr == &osdhandlerOsd1Params)
        {
//            result = FAPI_VISCALE_Resize(osd_cp_params_ptr->handle,
//                                        &osd_cp_params_ptr->scalerParams);
            params.inputWindowX0     = osd_cp_params_ptr->scalerParams.inputWindowX0;
            params.inputWindowY0     = osd_cp_params_ptr->scalerParams.inputWindowY0;
            params.inputWindowWidth  = osd_cp_params_ptr->scalerParams.inputWindowWidth;
            params.inputWindowHeight = osd_cp_params_ptr->scalerParams.inputWindowHeight;
            params.outputWindowX0    = osd_cp_params_ptr->scalerParams.outputWindowX0;
            params.outputWindowY0    = osd_cp_params_ptr->scalerParams.outputWindowY0;
            result = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)osd_cp_params_ptr->fd,
                                FAPEX_IOCCOM_VIOUT_SET_RESIZE,
                                (FAPEXIOCARG3_T)&params );
            if(result != FAPI_OK)
            {
                return(result);
            }
        }
        if(handle_ptr->bpp <= 8)
        {
//            result = FAPI_VISCALE_UpdateClut(osd_cp_params_ptr->handle,
//                                             osd_cp_params_ptr->scalerParams.colourLut);
            result = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)osd_cp_params_ptr->fd,
                                    FAPEX_IOCCOM_VIOUT_SET_CLUT,
                                    (FAPEXIOCARG3_T)osd_cp_params_ptr->clut );
            if(result != FAPI_OK)
            {
                return(result);
            }
        }
    }
    else if (osd_cp_params_ptr == &osdhandlerCpParams)
    {
        // ************** start the CursorPlane *********************
        memcpy(&start, startParamsPtr, sizeof(OSDHANDLER_FB_StartParamsT));

        // Ignore the output width since the CP can not be rescaled:
        start.scaledWidth  = handle_ptr->setup.xRes;
        start.scaledHeight = start.frameBased ?
                             handle_ptr->setup.yRes :
                            (handle_ptr->setup.yRes / 2);

        if((start.scaledWidth  > osdhandlerMaxHRES) ||
           (start.scaledHeight > osdhandlerMaxVRES))
        {
            return(OSDHANDLER_ERR_OUT_OF_SIZE);
        }
        // avoid the picture to be started outside of the active display area:
        if((start.xPos + start.scaledWidth)  > osdhandlerMaxHRES)
        {
            start.xPos = osdhandlerMaxHRES - start.scaledWidth;
        }
        if((start.yPos + start.scaledHeight) > osdhandlerMaxVRES)
        {
            start.yPos = osdhandlerMaxVRES - start.scaledHeight;
        }

        osd_cp_params_ptr->colourMode = handle_ptr->setup.colourMode;
//        osd_cp_params_ptr->scalerParams.cpMode = osdhandlerGetCpColourMode(handle_ptr->setup.colourMode);
        osd_cp_params_ptr->osdLayout = handle_ptr->setup.layoutMode;
        osd_cp_params_ptr->scalerParams.osdByteOrder = osdhandlerGetOsdLayout(handle_ptr->setup.layoutMode);

        osd_cp_params_ptr->scalerParams.outputWindowX0 = start.xPos;
        osd_cp_params_ptr->scalerParams.outputWindowY0 = start.yPos;
        osd_cp_params_ptr->scalerParams.inputWindowWidth  = handle_ptr->setup.xRes; // start.scaledWidth;
        osd_cp_params_ptr->scalerParams.inputWindowHeight = handle_ptr->setup.yRes;
//        osd_cp_params_ptr->scalerParams.inputPicWidth = handle_ptr->xResPhy;
//        osd_cp_params_ptr->scalerParams.inputPicHeight = handle_ptr->yResPhy;
        osd_cp_params_ptr->scalerParams.outputWindowWidth = start.scaledWidth;
        osd_cp_params_ptr->scalerParams.outputWindowHeight = start.scaledHeight;
        osd_cp_params_ptr->scalerParams.flagFrameField = (start.frameBased) ? 1 : 0;

        if((osd_cp_params_ptr->isMemoryDynamic)
         &&(osd_cp_params_ptr->allocatedMemPtr != NULL))
        {
            FAPI_SYS_FREE(osd_cp_params_ptr->allocatedMemPtr);
            osd_cp_params_ptr->allocatedMemPtr = NULL;
        }
        osd_cp_params_ptr->isMemoryDynamic = 0;
        osd_cp_params_ptr->allocatedMemPtr = (uint8_t*)handle_ptr->picBasePtr;
        /*lint -save -e923 */
//        osd_cp_params_ptr->scalerParams.picBase = (uint32_t)handle_ptr->picBasePtr;
        /*lint -restore */

//        FAPI_SYS_FlushDataCache(osdhandlerCpParams.scalerParams.picBase,
//                        (osdhandlerCpParams.scalerParams.inputPicWidth *
//                         osdhandlerCpParams.scalerParams.inputPicHeight *
//                         osdhandlerGetBitsPerPixel(&osdhandlerCpParams)) >> 3);

        result = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)osd_cp_params_ptr->fd,
                                FAPEX_IOCCOM_VIOUT_FLUSH_PLANE,
                                (FAPEXIOCARG3_T)NULL );
//        result = FAPI_VISCALE_Start(osdhandlerCpParams.handle,
//                                   &osdhandlerCpParams.scalerParams);
        result = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)osd_cp_params_ptr->fd,
                                    FAPEX_IOCCOM_START,
                                    (FAPEXIOCARG3_T)NULL );
        if(result != FAPI_OK)
        {
            return(result);
        }
        if(handle_ptr->bpp <= 8)
        {
//            result = FAPI_VISCALE_UpdateClut(osdhandlerCpParams.handle,
//                                             osdhandlerCpParams.clut);
            result = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)osd_cp_params_ptr->fd,
                                    FAPEX_IOCCOM_VIOUT_SET_CLUT,
                                    (FAPEXIOCARG3_T)osd_cp_params_ptr->clut );
            if(result != FAPI_OK)
            {
                return(result);
            }
        }
    }
    else
    {
        // unknown osd_cp_params_ptr referenced (should be unreachable)
        return(OSDHANDLER_ERR_BAD_PARAMETER);
    }

    // set the hardware OSD visible:
//    result = FAPI_VISCALE_EnableDisable(osd_cp_params_ptr->handle,
//                                        FAPI_VISCALE_PARAMETER_ON);
    result = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)osd_cp_params_ptr->fd,
                                FAPEX_IOCCOM_VIOUT_ENABLE,
                                (FAPEXIOCARG3_T)NULL );
    if(result != FAPI_OK)
    {
        return(result);
    }
//    osd_cp_params_ptr->scalerParams.layerEnable = 1;
    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Stop hardware layers that display a specific frame buffer.
**
**          If the frame buffer is displayed on more than one hardware OSD,
**          all OSDs that display the frame buffer will be stopped.
**
** \param   handle
**              The handle of the frame buffer to be stopped.
**
** \return
**          - #FAPI_OK
**              on success.
**          - #OSDHANDLER_ERR_NOT_INITIALIZED
**              if the OSDHANDLER is not initialized.
**          - #OSDHANDLER_FB_ERR_BAD_HANDLE
**              if the frame buffer handle is not valid.
**          - some VISCALE specific error code on driver failure.
**
** \sa      - #OSDHANDLER_FB_Display
**
*****************************************************************************
*/
int32_t OSDHANDLER_FB_Stop(OSDHANDLER_FB_HandleT handle)
{
    int32_t  result     = 0;
    uint32_t ii         = 0;
    osdhandlerFbHandleT* handle_ptr = NULL;
    OSDHANDLER_OsdCpParamsT* osd_cp_params_ptr = NULL;

    if (!osdhandlerInitialized)
    {
        return(OSDHANDLER_ERR_NOT_INITIALIZED);
    }
    if(!osdhandlerFbCheckHandle(handle))
    {
        return(OSDHANDLER_FB_ERR_BAD_HANDLE);
    }
    handle_ptr = (osdhandlerFbHandleT*)handle;
    for(ii = 1; ii < OSDHANDLER_MAX_NUM_HW_LAYER; ii++)
    {
        osd_cp_params_ptr = osdhandlerGetOsdCpParams(ii);

        // stop if handle is allocated and layer is started:
        /*lint -save -e923 */
        if((osd_cp_params_ptr) \
/*       && (osd_cp_params_ptr->handle) \ */
        && (osd_cp_params_ptr->allocatedMemPtr) \
/*       && (osd_cp_params_ptr->scalerParams.picBase == (uint32_t)handle_ptr->picBasePtr) */
        )
        /*lint -restore */
        {
//            result = FAPI_VISCALE_Stop(osd_cp_params_ptr->handle);
            result = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)osd_cp_params_ptr->fd,
                                    FAPEX_IOCCOM_STOP,
                                    (FAPEXIOCARG3_T)NULL );
            if(result)
            {
                return(result);
            }
            if((osd_cp_params_ptr->isMemoryDynamic)
             &&(osd_cp_params_ptr->allocatedMemPtr))
            {
                FAPI_SYS_FREE(osd_cp_params_ptr->allocatedMemPtr);
            }
            osd_cp_params_ptr->isMemoryDynamic = 0;
            osd_cp_params_ptr->allocatedMemPtr = NULL;
        }
    }
    return(FAPI_OK);
}
