/*!
*****************************************************************************
**
** \file        ./subsystems/paint/osdhandler/src/osdhandler_fb.h
**
** \version     $Id: osdhandler_fb.h 5868 2011-03-25 16:53:01Z jrende $
**
** \brief       OSD framebuffer handler for the paint middleware
**
** This file contains the declaration of the OSDHANDLER FRAME BUFFER API.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2007 - 2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/


#ifndef OSDHANDLER_FB_H
    #define OSDHANDLER_FB_H

    #include <stdio.h>
    #include <stdint.h>
    #include <fapex/abst_types.h>
    #include <fapex/abst_libviout.h>
    #include <osdhandler/osdhandler.h>

    //***********************************************************************
    //***********************************************************************
    //** Defines and Macros
    //***********************************************************************
    //***********************************************************************

    /*!
    ****************************************************************************
    ** Additional OSDHANDLER driver error codes
    ** that may occur for the frame buffer system. (reserved from -300 to -399)
    **
    ** Each error code shall be decreased in steps of 1.
    ****************************************************************************
    */

    /*! \brief OSDHANDLER specific error code offset */
    #define OSDHANDLER_FB_ERR_BASE \
            (int32_t) (SUBSYS_PAINT_OSDHANDLER_MODULE_BASE - 300)

    //! the handle is invalid
    #define OSDHANDLER_FB_ERR_BAD_HANDLE      (int32_t) (OSDHANDLER_FB_ERR_BASE - 0)

    //! the hardware requirement for memory alignment is not met
    #define OSDHANDLER_FB_ERR_MEMORY_ALIGN    (int32_t) (OSDHANDLER_FB_ERR_BASE - 1)

    //! an invalid colour mode is set up with respect to the desired function
    #define OSDHANDLER_FB_ERR_BAD_COLOUR_MODE (int32_t) (OSDHANDLER_FB_ERR_BASE - 2)

    //! the crop parameter is out of bounds
    #define OSDHANDLER_FB_ERR_BAD_CROP        (int32_t) (OSDHANDLER_FB_ERR_BASE - 3)

    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************

    /*!
    ****************************************************************************
    ** \brief   Frame buffer properties structure.
    **
    ****************************************************************************
    */
    typedef struct
    {
        /*! The horizontal resolution of the frame buffer. */
        uint32_t    xRes;
        /*! The vertical resolution of the frame buffer. */
        uint32_t    yRes;
        /*! The colour mode of the frame buffer. */
        OSDHANDLER_ColourModeT colourMode;
        /*! The colour layout format of the frame buffer. */
        OSDHANDLER_OsdLayoutEnumT layoutMode;
    } OSDHANDLER_FB_SetupT;

    /*!
    ****************************************************************************
    ** \brief   Frame buffer handle.
    **
    ****************************************************************************
    */
    typedef void* OSDHANDLER_FB_HandleT;

    /*!
    ****************************************************************************
    ** \brief   CLUT lock settings.
    **
    ****************************************************************************
    */
    typedef enum
    {
        //! The CLUT is updated by libraries modifying a frame buffer
        OSDHANDLER_FB_CLUT_LOCK_OFF =(int32_t) 0,

        //! The CLUT will be constant/unchanged when the framebuffer is modified
        OSDHANDLER_FB_CLUT_LOCK_ON
    }
    OSDHANDLER_FB_ClutLockT;

    /*!
    *************************************************************************
    ** \brief   Verbose information for a frame buffer.
    **
    **          The origin of a frame buffer is in the top left corner.
    **          The x axis points to the right while the y axis points
    **          downwards.
    **          The applicable and the physical resolution of the can
    **          differ since there are hardware constraints that say
    **          that a horizontal line must be a multiple of 8 bytes
    **          for the MB86H61.
    *************************************************************************
    */
    typedef struct
    {
        /*! Base address for the frame buffer. */
        uint32_t*   picBasePtr;
        /*! The colour mode of the frame buffer. */
        OSDHANDLER_ColourModeT colourMode;
        /*! The colour layout format of the frame buffer. */
        OSDHANDLER_OsdLayoutEnumT layoutMode;
        /*! The CLUT lock setting of the framebuffer. */
        OSDHANDLER_FB_ClutLockT clutLock;
        /*! The applicable horizontal resolution of the frame buffer. */
        uint32_t    xResApp;
        /*! The applicable vertical resolution of the frame buffer. */
        uint32_t    yResApp;
        /*! The physical horizontal resolution of the frame buffer. */
        uint32_t    xResPhy;
        /*! The physical vertical resolution of the frame buffer. */
        uint32_t    yResPhy;
        /*! Points to the Colour LookUp Table (CLUT) for indexed colour modes */
        uint32_t*   clutPtr;
        /*! Number of average bits per pixel */
        uint32_t    bpp;
    } OSDHANDLER_FB_FrameBufferT;

    /*!
    *************************************************************************
    ** \brief   Start parameters related to the frame buffer system.
    **
    **          This structure defines how the OSD will be shown on the
    **          display.
    *************************************************************************
    */
    typedef struct
    {
        /*! The x position of the top left corner of the OSD/CP on the display.      */
        uint32_t                    xPos;

        /*! The y position of the top left corner of the OSD/CP on the display.      */
        uint32_t                    yPos;

        /*! The horizontal resolution of the OSD on the display.
        **  If this parameter is 0, no horizontal rescaling will be
        **  performed and the output width will be set to the internal
        **  horizontal resolution of the framebuffer.
        **  For non-scalable OSDs, set this parameter to 0!
        */
        uint32_t                    scaledWidth;

        /*! The vertical resolution of the OSD on the display.
        **  If this parameter is 0, no vertical rescaling will be performed
        **  and the output height will be set to the internal vertical
        **  resolution for frame based pictures and to half of
        **  the internal vertical resolution for field based pictures.
        **  In order to avoid rescaling, the internal vertical resolution
        **  ('inputHeight') must be a multiple of two in the
        **  field based mode.
        **  For non-scalable OSDs, set this parameter to 0!
        */
        uint32_t                    scaledHeight;

        /*! Allows to specify a selection of the framebuffer to be displayed.
        **  This parameter specifies the x coordinate of the top left corner
        **  of the rectangular selection within the frame buffer area.
        **  Set this parameter to 0 in order to display the whole frame buffer!
        **  Set this parameter to 0 for the Cursor Plane on H61!
        */
        uint32_t                    sourceCropX;

        /*! Allows to specify a selection of the framebuffer to be displayed.
        **  This parameter specifies the y coordinate of the top left corner
        **  of the rectangular selection within the frame buffer area.
        **  Set this parameter to 0 in order to display the whole frame buffer!
        **  Set this parameter to 0 for the Cursor Plane on H61!
        */
        uint32_t                    sourceCropY;

        /*! Allows to specify a selection of the framebuffer to be displayed.
        **  This parameter specifies the width of the rectangular selection
        **  within the frame buffer area.
        **  Set this parameter to 0 in order to display the whole frame buffer!
        **  Set this parameter to 0 for the Cursor Plane on H61!
        */
        uint32_t                    sourceCropWidth;

        /*! Allows to specify a selection of the framebuffer to be displayed.
        **  This parameter specifies the height of the rectangular selection
        **  within the frame buffer area.
        **  Set this parameter to 0 in order to display the whole frame buffer!
        **  Set this parameter to 0 for the Cursor Plane on H61!
        */
        uint32_t                    sourceCropHeight;

        /*!
        **  Indicates whether an OSD/CP should be read as
        **  field based or frame based picture.
        **  0: field based OSD
        **  1: frame based OSD
        */
        uint32_t                    frameBased;

        /*!
        **  Identifies a hardware OSD.
        */
        uint32_t                    layerId;
    } OSDHANDLER_FB_StartParamsT;

    /*!
    *************************************************************************
    ** \brief   Transition effect identifiers.
    *************************************************************************
    */
    typedef enum
    {
        OSDHANDLER_FB_WIPE_LEFT_RIGHT = (int32_t) 0,
        OSDHANDLER_FB_WIPE_RIGHT_LEFT,
        OSDHANDLER_FB_WIPE_TOP_BOTTOM,
        OSDHANDLER_FB_WIPE_BOTTOM_TOP
    } OSDHANDLER_FB_TransitionT;

    /*!
    *************************************************************************
    ** \brief   Rotation options.
    *************************************************************************
    */
    typedef enum
    {
        OSDHANDLER_FB_ROTATION_0 = (int32_t) 0,
        OSDHANDLER_FB_ROTATION_90_LEFT,
        OSDHANDLER_FB_ROTATION_90_RIGHT,
        OSDHANDLER_FB_ROTATION_180
    } OSDHANDLER_FB_RotationT;


    //***********************************************************************
    //***********************************************************************
    //** Global Data
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** API Functions
    //***********************************************************************
    //***********************************************************************

    #ifdef __cplusplus
    extern "C" {
    #endif

    int32_t OSDHANDLER_FB_CalcMemConsumption \
                    (OSDHANDLER_FB_SetupT setup, uint32_t* sizePtr);
    int32_t OSDHANDLER_FB_AllocateHandle \
                    (OSDHANDLER_FB_SetupT setup, \
                     uint32_t* picBasePtr, \
                     FAPI_SYS_HandleT memorySegment, \
                     OSDHANDLER_FB_HandleT* handlePtr );
    int32_t OSDHANDLER_FB_DestroyHandle(OSDHANDLER_FB_HandleT handle);
    int32_t OSDHANDLER_FB_SetClut(OSDHANDLER_FB_HandleT handle, \
                                  uint32_t* clutPtr);
    int32_t OSDHANDLER_FB_SetClutLock(OSDHANDLER_FB_HandleT handle, \
                                      OSDHANDLER_FB_ClutLockT clutLock);
    int32_t OSDHANDLER_FB_Fill(OSDHANDLER_FB_HandleT handle, uint32_t colour);
    int32_t OSDHANDLER_FB_GetInfo(OSDHANDLER_FB_HandleT handle, \
                                  OSDHANDLER_FB_FrameBufferT* frameBufferInfoPtr);
    int32_t OSDHANDLER_FB_GetPixelAddress(OSDHANDLER_FB_HandleT handle, \
                                          uint32_t xx, \
                                          uint32_t yy, \
                                          uint32_t** pixelBaseAddrPtrPtr, \
                                          uint32_t* pixelBitOffsetPtr);
    int32_t OSDHANDLER_FB_Display(OSDHANDLER_FB_HandleT handle, \
                                  OSDHANDLER_FB_StartParamsT* startParamsPtr);
    int32_t OSDHANDLER_FB_Stop(OSDHANDLER_FB_HandleT handle);
    #ifdef __cplusplus
    }
    #endif

#endif // OSDHANDLER_FB_H
