/*
*****************************************************************************
**
** \file        ./subsystems/paint/picture/codecs/bmp/src/decbmp.c
**
** \version     $Id: $
**
** \brief       picture decoding library for BMP
**
**              This file contains the implementation of the DECBMP picture
**              decoder API for BMP.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

#include <stdint.h>
#include <fapex/abst_types.h>
#include <osdhandler/osdhandler.h>
#include <osdhandler/osdhandler_fb.h>
#include <picture/picdec_interface.h>
#include <picture/picscaler.h>
#include "decbmp.h"


//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Local Structures
//***************************************************************************
//***************************************************************************

/*!
*****************************************************************************
** \brief       Data structure that contains all members that are stored
**              within the header of a bitmap file.
*****************************************************************************
*/
typedef struct
{
    /*! identifies the bitmap type: 0x4D42 stands for windows 3.1x, 95, nt
    */
    uint32_t  identifier;

    /*! complete file size in bytes */
    uint32_t  fileSize;

    /*! reserved for later use */
    uint32_t  reserved;

    /*! offset from beginning of file to the beginning of the bitmap data */
    uint32_t  bitmapDataOffset;

    /*! length of the bitmap info header used to describe the bitmap colours,
    **  compression, the following sizes are possible:
    **  - 28h - windows 3.1x, 95, nt
    **  - 0ch - os/2 1.x
    **  - f0h - os/2 2.x
    */
    uint32_t  bitmapHeaderSize;

    /*! horizontal width of bitmap in pixels */
    uint32_t  width;

    /*! vertical height of bitmap in pixels */
    uint32_t  height;

    /*! number of planes in this bitmap */
    uint32_t  planes;

    /*! bits per pixel used to store palette entry information.
    **  this also identifies in an indirect way the number of possible
    **  colours. possible values are:
    **  - 1 - monochrome bitmap (CLUT)
    **  - 4 - 16 colour bitmap   (CLUT)
    **  - 8 - 256 colour bitmap  (CLUT)
    **  - 16 - 16bit (high colour) bitmap    (5551 mode)
    **  - 24 - 24bit (true colour) bitmap    (RGBA mode)
    **  - 32 - 32bit (true colour) bitmap    (RGBA mode, high byte not used)
    */
    uint32_t  bitsPerPixel;

    /*! compression specifications. the following values are possible:
    **  - 0 - none (also identified by bi_rgb)
    **  - 1 - rle 8-bit / pixel (also identified by bi_rle4)
    **  - 2 - rle 4-bit / pixel (also identified by bi_rle8)
    **  - 3 - bitfields (also identified by bi_bitfields)
    */
    uint32_t  compression;

    /*! Size of the bitmap data in bytes.
    **  This number must be rounded to the next 4 byte boundary.
    */
    uint32_t  bitmapDataSize;

    /*! horizontal resolution expressed in pixel per meter */
    uint32_t  hResolution;

    /*! vertical resolution expressed in pixels per meter */
    uint32_t  vResolution;

    /*! Number of colours used by this bitmap.
    **  For a 8-bit / pixel bitmap this will be 0x100 or 256.
    */
    uint32_t  colours;

    /*! Number of important colours. This number will be equal
    **  to the number of colours when every colour is important.
    */
    uint32_t  importantColours;

    /*! Palette size. This parameter must be calculated in accordance
    **  to the palette position in the header and the bitmapDataOffset.
    */
    uint32_t  paletteSize;

    /*! The palette specification. For every entry in the palette
    **  four bytes are used to describe the rgb values of the colour:
    **  - 1 byte for the blue component
    **  - 1 byte for the green component
    **  - 1 byte for the red component
    **  - 1 byte serves as a filler
    */
    uint8_t* palettePtr;

} decbmpFileHeadT;

/*!
*****************************************************************************
** \brief       Data structure containing information to decode a bitmap
**              linewise suitable for the scaler subsystem.
*****************************************************************************
*/
typedef struct
{
    uint32_t                stride;     // width of bmp line in bytes (incl. stuffing)
    uint32_t                lineIdx;    // index of line to decode next
    uint8_t*                bmpLinePtr; // Buffer for a raw BMP line
    decbmpFileHeadT*        headPtr;    // bmp file header
    uint32_t*               clutPtr;    // appropriately converted CLUT
    PICDEC_ReadFileChunkT   readFptr;   // reads from memory source
    PICDEC_FileSeekT        seekFptr;   // seeks the memory source
    void*                   filePtr;    // picture file handle
    int32_t                 fileOffset; // picture data offset within file
} decbmpReadFuncArgT;


//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Local Functions Declaration
//***************************************************************************
//***************************************************************************

static int32_t  decbmpParseHeader(decbmpFileHeadT* headPtr,
                                  void* filePtr,
                                  PICDEC_ReadFileChunkT readFptr);
static void     decbmpDestroyHeader (decbmpFileHeadT* headPtr);
static uint32_t decbmpScalerReadLine(void* lineBufPtr, \
                                     void* optArgPtr,  \
                                     uint32_t lineCnt);
static uint32_t decbmpScalerSkipLine(void* lineBufPtr, \
                                     void* optArgPtr,  \
                                     uint32_t lineCnt);

//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************

/*!
*****************************************************************************
** \brief BMP Driver Interface implementation.
**
** This datastucture represents the BMP driver specific implementation
** of the picture library driver interface.
**
** \sa PICDEC_INTERFACE_DriverT
****************************************************************************
*/
PICDEC_INTERFACE_DriverT DECBMP_Driver =
{
    "BMP",              // formatNamePtr
    DECBMP_Init,        // initFptr
    DECBMP_Exit,        // exitFptr
    DECBMP_CheckFile,   // checkFptr
    DECBMP_GetInfo,     // getInfoFptr
    DECBMP_Decode       // decodeFptr
};

const PICDEC_INTERFACE_DriverT* DECBMP_DRIVER_PTR = &DECBMP_Driver;


//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

/*!
*****************************************************************************
** \brief   Parse a bitmap file into a bitmap header file structure.
**          This is useful if you need to access properties of a bitmap
**          file such as resolution, bits per pixel, dpi, etc.
**
** \param   filePtr
**              Points to picture data.
** \param   readFptr
**              This function reads data chunks from filePtr.
** \param   headPtr
**              Will get all parameters that are specified within a
**              bitmap file head.
**
** \return  - #FAPI_OK on success
**          - #DECBMP_ERR_BAD_FILE_FORMAT if the file is not a supported bitmap
**          - #DECBMP_ERR_OUT_OF_MEMORY if no CLUT memory was allocated
**
*****************************************************************************
*/
static int32_t decbmpParseHeader(decbmpFileHeadT* headPtr,
                                 void* filePtr, PICDEC_ReadFileChunkT readFptr)
{
    uint32_t ii = 0;
    uint32_t palette_size = 0;

    ii = readFptr ((uint8_t*)&headPtr->identifier, filePtr, 2); if(ii != 2) return(DECBMP_ERR_DATA_SOURCE);
    if(headPtr->identifier != 0x00004D42UL)
    {
        return(DECBMP_ERR_BAD_FILE_FORMAT);
    }
    ii = readFptr ((uint8_t*)&headPtr->fileSize,          filePtr, 4); if(ii != 4) return(DECBMP_ERR_DATA_SOURCE);
    ii = readFptr ((uint8_t*)&headPtr->reserved,          filePtr, 4); if(ii != 4) return(DECBMP_ERR_DATA_SOURCE);
    ii = readFptr ((uint8_t*)&headPtr->bitmapDataOffset,  filePtr, 4); if(ii != 4) return(DECBMP_ERR_DATA_SOURCE);
    ii = readFptr ((uint8_t*)&headPtr->bitmapHeaderSize,  filePtr, 4); if(ii != 4) return(DECBMP_ERR_DATA_SOURCE);
    ii = readFptr ((uint8_t*)&headPtr->width,             filePtr, 4); if(ii != 4) return(DECBMP_ERR_DATA_SOURCE);
    ii = readFptr ((uint8_t*)&headPtr->height,            filePtr, 4); if(ii != 4) return(DECBMP_ERR_DATA_SOURCE);
    ii = readFptr ((uint8_t*)&headPtr->planes,            filePtr, 2); if(ii != 2) return(DECBMP_ERR_DATA_SOURCE);
    ii = readFptr ((uint8_t*)&headPtr->bitsPerPixel,      filePtr, 2); if(ii != 2) return(DECBMP_ERR_DATA_SOURCE);
    ii = readFptr ((uint8_t*)&headPtr->compression,       filePtr, 4); if(ii != 4) return(DECBMP_ERR_DATA_SOURCE);
    ii = readFptr ((uint8_t*)&headPtr->bitmapDataSize,    filePtr, 4); if(ii != 4) return(DECBMP_ERR_DATA_SOURCE);
    ii = readFptr ((uint8_t*)&headPtr->hResolution,       filePtr, 4); if(ii != 4) return(DECBMP_ERR_DATA_SOURCE);
    ii = readFptr ((uint8_t*)&headPtr->vResolution,       filePtr, 4); if(ii != 4) return(DECBMP_ERR_DATA_SOURCE);
    ii = readFptr ((uint8_t*)&headPtr->colours,           filePtr, 4); if(ii != 4) return(DECBMP_ERR_DATA_SOURCE);
    ii = readFptr ((uint8_t*)&headPtr->importantColours,  filePtr, 4); if(ii != 4) return(DECBMP_ERR_DATA_SOURCE);

    // Determine whether there is a CLUT present in the bitmap and if so, get it:
    palette_size = (headPtr->bitmapDataOffset > 0x36) ? \
                   (headPtr->bitmapDataOffset - 0x36) : 0;
    // data offset minus header length
    headPtr->paletteSize = palette_size;
    if(palette_size)
    {
        headPtr->palettePtr = FAPI_SYS_MALLOC_CACHED(palette_size);
        if(headPtr->palettePtr == NULL) return (DECBMP_ERR_OUT_OF_MEMORY);
        (void) readFptr (headPtr->palettePtr,  filePtr, palette_size);
    }
    return (FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Free a bitmap header structure.
**
** \param   headPtr
**              Points to a header structure parsed by decbmpParseHeader().
**
** \sa      - decbmpParseHeader
**
*****************************************************************************
*/
static void decbmpDestroyHeader (decbmpFileHeadT* headPtr)
{
    if(headPtr->paletteSize)
    {
        FAPI_SYS_FREE(headPtr->palettePtr);
    }
    return;
}

/*!
*****************************************************************************
** \brief   Callback function for decoding bitmap lines to the scaler's buffer.
**
** \param   lineBufPtr
**              The scaler's line buffer that will be written.
**              It will be filled with unscaled bitmap data.
** \param   optArgPtr
**              Actual decbmpReadFuncArgT structure.
**              It is used to gain proper access to the bitmap data source.
** \param   lineCnt
**              The number of lines to decode requested by the scaler.
**
** \return  The number of actually decoded lines.
**
*****************************************************************************
*/
static uint32_t decbmpScalerReadLine(void* lineBufPtr, \
                                     void* optArgPtr,  \
                                     uint32_t lineCnt)
{
    int32_t   ret_code   = 0;
    uint32_t  xcnt       = 0;
    uint32_t  ycnt       = 0;
    uint32_t  tmp        = 0;
    int32_t   offset     = 0;
    uint8_t*  src_8_ptr  = NULL;
    uint32_t* dst_32_ptr = NULL;
    decbmpReadFuncArgT* read_arg_ptr = NULL;

    read_arg_ptr = (decbmpReadFuncArgT*)optArgPtr;
    dst_32_ptr = (uint32_t*)lineBufPtr;
    ycnt = lineCnt;

    while(ycnt--)
    {
        // Read one BMP line into a temporary buffer:
        offset = read_arg_ptr->fileOffset + \
                 (int32_t) (read_arg_ptr->headPtr->bitmapDataOffset + \
                               (     (read_arg_ptr->headPtr->height -
                                     (read_arg_ptr->lineIdx + 1)) * \
                                      read_arg_ptr->stride) );
        ret_code = read_arg_ptr->seekFptr(read_arg_ptr->filePtr,
                                          offset, PICDEC_SEEK_SET);
        if (ret_code != 0) return (0); // data source error

        tmp = read_arg_ptr->readFptr(read_arg_ptr->bmpLinePtr,
                                     read_arg_ptr->filePtr,
                                     read_arg_ptr->stride);
        if(tmp != read_arg_ptr->stride) return(0); // signal read error to scaler
        src_8_ptr = read_arg_ptr->bmpLinePtr;

        // horizontal counter:
        xcnt = read_arg_ptr->headPtr->width;

        // read as 32-bit RGBA and write to lineBufPtr
        // all bmp colour modes have to be converted to 32-bit RGBA here
        switch(read_arg_ptr->headPtr->bitsPerPixel)
        {
            case(32):
            while(xcnt--)
            {
                /*lint -save -e701 */
                *dst_32_ptr++ = (*(src_8_ptr + 2) << 24) | // R
                                (*(src_8_ptr + 1) << 16) | // G
                                (*(src_8_ptr + 0) <<  8) | // B
                                (*(src_8_ptr + 3) <<  0);  // A
                /*lint -restore */
                src_8_ptr += 4;
            }
            break;

            case(24):
            while(xcnt--)
            {
                /*lint -save -e701 */
                *dst_32_ptr++ = (*(src_8_ptr + 2) << 24) | // R
                                (*(src_8_ptr + 1) << 16) | // G
                                (*(src_8_ptr + 0) <<  8) | // B
                                            0x000000FFUL;  // A
                /*lint -restore */
                src_8_ptr += 3;
            }
            break;

            case(16):
            while(xcnt--)
            {
                tmp = (*(src_8_ptr + 1) << 8) | *src_8_ptr;
                src_8_ptr += 2;
                *dst_32_ptr++ = ((tmp & 0x001F) << 11)
                              | ((tmp & 0x03E0) << 14)
                              | ((tmp & 0x7C00) << 17)
                              | (0x000000FF);
            }
            break;

            case(8):
            while(xcnt--)
            {
                // get colour index from RGBA formatted CLUT:
                *dst_32_ptr++ = *(read_arg_ptr->clutPtr + *src_8_ptr++);
            }
            break;

            case(4):
            tmp = 1; // toggle
            while(xcnt--)
            {
                *dst_32_ptr++ = *(read_arg_ptr->clutPtr +  \
                    (tmp ? ((*src_8_ptr & 0xF0UL) >> 4) : (*src_8_ptr++ & 0x0FUL)));
                tmp = !tmp;
            }
            break;

            case(1):
            tmp = 7;
            while(xcnt--)
            {
                *dst_32_ptr++ = *(read_arg_ptr->clutPtr +  \
                                 ((*src_8_ptr & (0x01UL << tmp)) ? 1 : 0));
                if (tmp == 0)
                {
                    tmp = 7;
                    src_8_ptr++;
                }
                else
                {
                    tmp--;
                }
            }
            break;

            default:
            break;
        }
        read_arg_ptr->lineIdx++;
    }
    return (lineCnt);
}

/*!
*****************************************************************************
** \brief   Callback function skipping a line in a bitmap.
**
** \param   lineBufPtr
**              Not used here.
** \param   optArgPtr
**              Actual decbmpReadFuncArgT structure.
** \param   lineCnt
**              The number of requested lines to skip.
**
** \return  The number of actually skipped lines.
**
*****************************************************************************
*/
static uint32_t decbmpScalerSkipLine(void* lineBufPtr, \
                                     void* optArgPtr,  \
                                     uint32_t lineCnt)
{
    decbmpReadFuncArgT* read_arg_ptr = NULL;

    lineBufPtr = lineBufPtr;
    read_arg_ptr = (decbmpReadFuncArgT*)optArgPtr;
    read_arg_ptr->lineIdx += lineCnt;
    return (lineCnt);
}


//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

/*!
*******************************************************************************
**
** \brief   Initialization of BMP decoder library.
**
** \return
**          - #FAPI_OK
**
** \sa      - DECBMP_Exit()
*******************************************************************************
**/
int32_t DECBMP_Init(void)
{
    return(FAPI_OK);
}

/*!
*******************************************************************************
**
** \brief   Exit the BMP decoder library.
**
** \return
**          - #FAPI_OK
**
** \sa      - DECBMP_Init()
*******************************************************************************
**/
void DECBMP_Exit(void)
{
    return;
}

/*!
*******************************************************************************
**
** \brief   Checks whether a file can be processed by the DECBMP library.
**
** \param   filePtr
**              Descriptor of file containing the picture data.
** \param   fileOffset
**              Picture data offset within the file.
** \param   readFptr
**              This function reads data chunks from filePtr.
** \param   seekFptr
**              This function sets the file descriptor position
**              of the memory source.
**
** \return
**          - #FAPI_OK if the file is a BMP.
**          - #DECBMP_ERR_BAD_PARAMETER if a NULL pointer is passed.
**          - #DECBMP_ERR_DATA_SOURCE if the data source does not read.
**          - #DECBMP_ERR_BAD_FILE_FORMAT if the file is no supported bitmap.
**
** \sa      See #PICDEC_INTERFACE_DriverT specifications.
*******************************************************************************
**/
int32_t DECBMP_CheckFile (void* filePtr,
                          int32_t fileOffset,
                          PICDEC_ReadFileChunkT readFptr,
                          PICDEC_FileSeekT seekFptr)
{
    int32_t  ret_code   = 0;
    uint32_t ii         = 0;
    uint8_t  sig_arr[2];

    memset(sig_arr, 0, sizeof(sig_arr));

    if((filePtr == NULL)
    || (readFptr == NULL)
    || (seekFptr == NULL))
    {
        return(DECBMP_ERR_BAD_PARAMETER);
    }

   
    // rewind memory source:
    ret_code = seekFptr(filePtr, fileOffset, PICDEC_SEEK_SET);
    if(ret_code != 0) return(DECBMP_ERR_DATA_SOURCE);

    // read in signature bytes:
    ii = readFptr(sig_arr, filePtr, 2);
    if(ii == 0)
    {
        return(DECBMP_ERR_DATA_SOURCE);
    }
    // 2 bytes BMP signature
    if((*sig_arr == 0x42) && (*(sig_arr + 1) == 0x4D))
    {
        return(FAPI_OK);
    }
    return(DECBMP_ERR_BAD_FILE_FORMAT);
}

/*!
*******************************************************************************
** \brief   Gets information about a bitmap picture.
**
** \param   filePtr
**              Descriptor of file containing the picture data.
** \param   fileOffset
**              Picture data offset within the file.
** \param   readFptr
**              This function reads data chunks from filePtr.
** \param   seekFptr
**              This function sets the file descriptor position
**              of the memory source.
** \param   infoPtr
**              Will receive the requested information about the file.
**
** \return
**          - #FAPI_OK on success.
**          - #DECBMP_ERR_BAD_PARAMETER if a NULL pointer is passed.
**          - #DECBMP_ERR_DATA_SOURCE if the data source does not read.
**          - #DECBMP_ERR_BAD_FILE_FORMAT if the file is no supported bitmap.
**          - #DECBMP_ERR_OUT_OF_MEMORY if memory could not be allocated.
**
** \sa      See #PICDEC_INTERFACE_DriverT specifications.
*******************************************************************************
*/
int32_t DECBMP_GetInfo (void* filePtr,
                        int32_t fileOffset,
                        PICDEC_ReadFileChunkT readFptr,
                        PICDEC_FileSeekT seekFptr,
                        PICDEC_RotationT rotation,
                        PICDEC_InfoT* infoPtr)
{
    int32_t  result     = 0;
    uint32_t ii         = 0;
    uint8_t* src_8_ptr  = NULL;

    decbmpFileHeadT bmp_head;

    memset(&bmp_head, 0, sizeof(decbmpFileHeadT));

    if((filePtr == NULL)
    || (readFptr == NULL)
    || (seekFptr == NULL)
    || (infoPtr == NULL))
    {
        return(DECBMP_ERR_BAD_PARAMETER);
    }

     FAPI_SYS_PRINT_DEBUG(1,"BMP get info...\n" );
    // rewind memory source:
    result = seekFptr(filePtr, fileOffset, PICDEC_SEEK_SET);
    if (result != 0) return(DECBMP_ERR_DATA_SOURCE);

    // Parse file header and check the file signature:
    result = decbmpParseHeader(&bmp_head, filePtr, readFptr);
    if(result != FAPI_OK) return(result);

    // Fill info structure:
    infoPtr->format = PICDEC_FORMAT_BMP;
    infoPtr->bpp = bmp_head.bitsPerPixel;

    switch(rotation)
    {
        case(PICDEC_ROTATION_90_LEFT):
        case(PICDEC_ROTATION_90_RIGHT):
        infoPtr->xRes = bmp_head.height;
        infoPtr->yRes = bmp_head.width;
        break;

        case(PICDEC_ROTATION_0):
        case(PICDEC_ROTATION_180):
        default:
        infoPtr->xRes = bmp_head.width;
        infoPtr->yRes = bmp_head.height;
        break;
    }
    memset(infoPtr->clutArray, 0, sizeof(infoPtr->clutArray));
    if((bmp_head.bitsPerPixel <= 8)
    && (bmp_head.paletteSize > 0))
    {
        // copy the CLUT
        src_8_ptr = bmp_head.palettePtr;
        if((bmp_head.colours == 0) \
        || (bmp_head.colours > (1UL << bmp_head.bitsPerPixel)))
        {
            bmp_head.colours = 1UL << bmp_head.bitsPerPixel;
        }
        for(ii = 0; ii < bmp_head.colours; ii++)
        {
            /*lint -save -e701 */
            infoPtr->clutArray[ii] = (*(src_8_ptr + 2) << 24) | // R
                                     (*(src_8_ptr + 1) << 16) | // G
                                     (*(src_8_ptr + 0) <<  8) | // B
                                     (*(src_8_ptr + 3) <<  0);  // A
            /*lint -restore */
            src_8_ptr += 4;
        }
    }
    decbmpDestroyHeader(&bmp_head);
    return(FAPI_OK);
}

/*!
*******************************************************************************
** \brief   Decodes a BMP picture into a frame buffer.
**
** \param   filePtr
**              Descriptor of file containing the picture data.
** \param   fileOffset
**              Picture data offset within the file.
** \param   readFptr
**              This function reads data chunks from filePtr.
** \param   seekFptr
**              This function sets the file descriptor position
**              of the memory source.
** \param   fbHandle
**              Handle for the frame buffer that will be modified.
** \param   xx
**              Horizontal coordinate of the picture in the frame buffer.
**              This coordinate is relative to the frame buffer, i.e., if
**              the picture is rotated, the xx and yy coordinates denote the
**              top left corner of the rotated picture.
** \param   yy
**              Vertical coordinate of the picture in the frame buffer.
**              This coordinate is relative to the frame buffer, i.e., if
**              the picture is rotated, the xx and yy coordinates denote the
**              top left corner of the rotated picture.
** \param   xRes
**              Desired horizontal resolution of the picture.
**              The resolution is achieved by software scaling.
**              If xRes is 0, the native horizontal resolution
**              of the picture file will be used.
** \param   yRes
**              Desired vertical resolution of the picture.
**              The resolution is achieved by software scaling.
**              If yRes is 0, the native vertical resolution
**              of the picture file will be used.
** \param   rotation
**              Defines a rotation of the picture in steps of
**              90 degrees.
** \param   scaler
**              Specifies the prefereed scaler algorithm to be
**              used while decoding. This option only applies
**              if the xRes or yRes parameter differs from the
**              native picture size.
** \param   progressCb
**              Optional progress and abort callback.
** \param   progressCbArg
**              Optional argument passed with progressCb.
**
** \return
**          - #FAPI_OK on success.
**          - #DECBMP_ERR_BAD_PARAMETER
**              if filePtr or readFptr is NULL.
**          - #DECBMP_ERR_DATA_SOURCE
**              if the data source does not read.
**          - #DECBMP_ERR_BAD_FILE_FORMAT
**              if the file is no supported bitmap.
**          - #DECBMP_ERR_FORMAT_NOT_SUPPORTED
**              if the particular file format is not supported (e.g.,  due to
**              compression in the bitmap).
**          - #DECBMP_ERR_OUT_OF_MEMORY if memory could not be allocated.
**          - some OSDHANDLER_FB specific error code if the fbHandle is bad
**            or when an error occurs while setting up the FB CLUT.
**          - #DECBMP_ERR_COLOUR_MODE_MISMATCH
**              if the FB colour mode is not supported for the colour mode
**              of the BMP. Generally all BMPs can be decoded to frame buffers
**              running OSDHANDLER_OSD_MODE_32_24BIT RGBA or CbYCrA.
**              Additionally, most BMPs can be decoded into a frame buffer
**              that has the same amount of bits per pixel as the BMP file.
**          - some PICDEC_SCALER specific error code if scaling fails.
**          - #DECBMP_ERR_OUT_OF_FB_BOUNDS
**              if the picture would decoded out of bounds of the frame buffer
**              plane.
**
*******************************************************************************
*/
int32_t DECBMP_Decode (void* filePtr,
                       int32_t fileOffset,
                       PICDEC_ReadFileChunkT readFptr,
                       PICDEC_FileSeekT seekFptr,
                       OSDHANDLER_FB_HandleT fbHandle,
                       uint32_t xx,
                       uint32_t yy,
                       uint32_t xRes,
                       uint32_t yRes,
                       PICDEC_RotationT rotation,
                       PICDEC_ScalerT scaler,
                       PICDEC_ProgressCbT progressCb,
                       void* progressCbArg)
{
    OSDHANDLER_FB_FrameBufferT fb_info;
    decbmpFileHeadT bmp_head;
    decbmpReadFuncArgT read_arg;
    PICSCALER_DecodeParamsT scaler_params;
    PICDEC_DecodeInfoT decode_info;
    int32_t   result     = 0;    // error codes
    uint32_t  ii         = 0;
    uint32_t  tmp_val    = 0;
    uint32_t  line       = 0;
    uint32_t  stride     = 0;
    uint32_t  data_mask  = 0;
    uint32_t  copy_steps = 0;
    uint32_t  copy_rest  = 0;
    uint32_t* fb_32_ptr  = NULL; // framebuffer 32-bit address
    uint32_t  fb_offset  = 0;    // offset in 32-bit word (0...31)
    uint8_t*  src_8_ptr  = NULL;
    uint32_t* dst_32_ptr = NULL;
    uint32_t* clut_ptr   = NULL;
    uint8_t*  bmp_line_ptr = NULL;

    memset(&fb_info,       0, sizeof(OSDHANDLER_FB_FrameBufferT));
    memset(&scaler_params, 0, sizeof(PICSCALER_DecodeParamsT));
    memset(&bmp_head,      0, sizeof(decbmpFileHeadT));
    memset(&read_arg,      0, sizeof(decbmpReadFuncArgT));

    if((filePtr == NULL)
    || (readFptr == NULL)
    || (seekFptr == NULL))
    {
        return(DECBMP_ERR_BAD_PARAMETER);
    }

    // rewind memory source:
    result = seekFptr(filePtr, fileOffset, PICDEC_SEEK_SET);
    if (result != 0) return(DECBMP_ERR_DATA_SOURCE);

    // Parse file header and check the file signature:
    result = decbmpParseHeader(&bmp_head, filePtr, readFptr);
    if(result != FAPI_OK) return(result);

    if (bmp_head.compression)
    {
        decbmpDestroyHeader(&bmp_head);
        return(DECBMP_ERR_FORMAT_NOT_SUPPORTED);
    }
    result = OSDHANDLER_FB_GetInfo(fbHandle, &fb_info);
    if(result != FAPI_OK)
    {
        decbmpDestroyHeader(&bmp_head);
        return(result);
    }

    // Internally, resolution is oriented according to picture orientation:
    if((rotation == PICDEC_ROTATION_90_LEFT)
    || (rotation == PICDEC_ROTATION_90_RIGHT))
    {
        tmp_val = xRes;
        xRes = yRes;
        yRes = tmp_val;
    }
    if(xRes == 0)
    {
        xRes = bmp_head.width;
    }
    if(yRes == 0)
    {
        yRes = bmp_head.height;
    }

    //  ************************************************************
    //  ******************** CLUT SETUP ****************************
    //  ************************************************************

    if ((bmp_head.bitsPerPixel == 1)
      ||(bmp_head.bitsPerPixel == 4)
      ||(bmp_head.bitsPerPixel == 8))
    {
        // src_8_ptr is used for traversing the colour palette here:
        src_8_ptr = bmp_head.palettePtr;

        if(bmp_head.colours == 0)
        {
            bmp_head.colours = 1UL << bmp_head.bitsPerPixel;
        }
        else if(bmp_head.colours > (1UL << bmp_head.bitsPerPixel))
        {
            bmp_head.colours = 1UL << bmp_head.bitsPerPixel;
        }

        clut_ptr = FAPI_SYS_MALLOC_CACHED(bmp_head.colours * sizeof(uint32_t));
        if(clut_ptr == NULL)
        {
            decbmpDestroyHeader(&bmp_head);
            return(DECBMP_ERR_OUT_OF_MEMORY);
        }

        // If the scaler is invoked, get an RGBA CLUT:
        if ((xRes != bmp_head.width)
        ||  (yRes != bmp_head.height)
        ||  (rotation != PICDEC_ROTATION_0)
        ||  (fb_info.colourMode == OSDHANDLER_OSD_MODE_16BIT))
        {
            for(line = 0; line < bmp_head.colours; line++)
            {
                /*lint -save -e701 */
                clut_ptr[line] = (*(src_8_ptr + 2) << 24) | // R
                                 (*(src_8_ptr + 1) << 16) | // G
                                 (*(src_8_ptr + 0) <<  8) | // B
                                 (*(src_8_ptr + 3) <<  0);  // A
                /*lint -restore */
                src_8_ptr += 4;
            }
        }
        // Otherwise the CLUT layout depends on the framebuffer:
        else
        {
            switch(fb_info.layoutMode)
            {
                case(OSDHANDLER_OSD_LAYOUT_ARGB):
                for(line = 0; line < bmp_head.colours; line++)
                {
                    /*lint -save -e701 */
                    clut_ptr[line] = (*(src_8_ptr + 3) << 24) | // A
                                     (*(src_8_ptr + 2) << 16) | // R
                                     (*(src_8_ptr + 1) <<  8) | // G
                                     (*(src_8_ptr + 0) <<  0);  // B
                    /*lint -restore */
                    src_8_ptr += 4;
                }
                break;

                case(OSDHANDLER_OSD_LAYOUT_RGBA):
                for(line = 0; line < bmp_head.colours; line++)
                {
                    /*lint -save -e701 */
                    clut_ptr[line] = (*(src_8_ptr + 2) << 24) | // R
                                     (*(src_8_ptr + 1) << 16) | // G
                                     (*(src_8_ptr + 0) <<  8) | // B
                                     (*(src_8_ptr + 3) <<  0);  // A
                    /*lint -restore */
                    src_8_ptr += 4;
                }
                break;

                case(OSDHANDLER_OSD_LAYOUT_ACbYCr):
                for(line = 0; line < bmp_head.colours; line++)
                {
                    /*lint -save -e701 */
                    tmp_val = OSDHANDLER_ConvertARGB2CbYCrA_word \
                                        ((*(src_8_ptr + 3) << 24) | // A
                                         (*(src_8_ptr + 2) << 16) | // R
                                         (*(src_8_ptr + 1) <<  8) | // G
                                         (*(src_8_ptr + 0) <<  0)); // B
                    /*lint -restore */
                    src_8_ptr += 4;
                    clut_ptr[line] = ((tmp_val & 0xFFUL) << 24) | (tmp_val >> 8);
                }
                break;

                case(OSDHANDLER_OSD_LAYOUT_CbYCrA):
                for(line = 0; line < bmp_head.colours; line++)
                {
                    /*lint -save -e701 */
                    clut_ptr[line] = OSDHANDLER_ConvertARGB2CbYCrA_word \
                                        ((*(src_8_ptr + 3) << 24) | // A
                                         (*(src_8_ptr + 2) << 16) | // R
                                         (*(src_8_ptr + 1) <<  8) | // G
                                         (*(src_8_ptr + 0) <<  0)); // B
                    /*lint -restore */
                    src_8_ptr += 4;
                }
                break;

                case(OSDHANDLER_UNKNOWN_OSD_LAYOUT):
                default:
                decbmpDestroyHeader(&bmp_head);
                FAPI_SYS_FREE(clut_ptr);
                return(DECBMP_ERR_COLOUR_MODE_MISMATCH);
            }

            if((fb_info.clutLock == OSDHANDLER_FB_CLUT_LOCK_OFF)
            && ((fb_info.colourMode == OSDHANDLER_OSD_MODE_CLUT_4BIT)
              ||(fb_info.colourMode == OSDHANDLER_OSD_MODE_CLUT_8BIT)
              ||(fb_info.colourMode == OSDHANDLER_CURSOR_MODE_1BIT)
              ||(fb_info.colourMode == OSDHANDLER_CURSOR_MODE_2BIT)
              ||(fb_info.colourMode == OSDHANDLER_CURSOR_MODE_4BIT)
              ||(fb_info.colourMode == OSDHANDLER_CURSOR_MODE_8BIT)))
            {
                // possible errors already checked
                result = OSDHANDLER_FB_SetClut(fbHandle, clut_ptr);
                if(result)
                {
                    decbmpDestroyHeader(&bmp_head);
                    FAPI_SYS_FREE(clut_ptr);
                    return (result);
                }
            }
        }
    }

    // lower limit line width of a bitmap in bytes:
    stride = (bmp_head.width * bmp_head.bitsPerPixel) / 8;
    // consider bpp < 8:
    if((bmp_head.width * bmp_head.bitsPerPixel) % 8) stride++;
    // apply internal alignment of bitmap format:
    stride += (4 - (stride % 4)) % 4; // align to n*4

    // allocate a buffer for one line of unscaled BMP data:
    bmp_line_ptr = FAPI_SYS_MALLOC_CACHED(stride * bmp_head.bitsPerPixel);
    if(bmp_line_ptr == NULL)
    {
        decbmpDestroyHeader(&bmp_head);
        if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
        return(DECBMP_ERR_OUT_OF_MEMORY);
    }

    // Initialize decoder info structure
    memset (&decode_info, 0, sizeof(decode_info));
    decode_info.totalPasses = 1;
    decode_info.totalLines = bmp_head.height;


    //  ************************************************************
    //  ************ SCALED/ROTATED BMP DECODING *******************
    //  ************************************************************
    // for CCIR colour mode also go through scaler backend:
    if ((xRes != bmp_head.width)
    ||  (yRes != bmp_head.height)
    ||  (rotation != PICDEC_ROTATION_0)
    ||  (fb_info.colourMode == OSDHANDLER_OSD_MODE_16BIT))
    {
        // select BRESENHAM if nothing to scale (fastest):
        if((xRes == bmp_head.width) && (yRes == bmp_head.height))
        {
            scaler = PICDEC_SCALER_BRESENHAM;
        }

        // Set up structure used to extract BMP data from memory source:
        read_arg.stride     = stride;
        read_arg.lineIdx    = 0;
        read_arg.bmpLinePtr = bmp_line_ptr;
        read_arg.headPtr    = &bmp_head;
        read_arg.clutPtr    = clut_ptr; // converted RGBA CLUT
        read_arg.readFptr   = readFptr;
        read_arg.seekFptr   = seekFptr;
        read_arg.filePtr    = filePtr;
        read_arg.fileOffset = fileOffset;

        // Set up the image scaler:
        scaler_params.fbHandle      = fbHandle;
        scaler_params.srcWidth      = bmp_head.width;
        scaler_params.srcHeight     = bmp_head.height;
        scaler_params.dstPosX       = xx;
        scaler_params.dstPosY       = yy;
        scaler_params.dstWidth      = xRes;
        scaler_params.dstHeight     = yRes;
        scaler_params.bufHeight     = 1;
        scaler_params.rotation      = rotation;
        scaler_params.scaler        = scaler;
        scaler_params.readLineFptr  = decbmpScalerReadLine;
        scaler_params.skipLineFptr  = decbmpScalerSkipLine;
        scaler_params.optArgPtr     = (void*)&read_arg;
        scaler_params.progressCb    = progressCb;
        scaler_params.progressCbArg = progressCbArg;
        scaler_params.decodeInfoPtr = &decode_info;

        // Decode picture, clean up and return:
        result = PICSCALER_Decode(&scaler_params);
        decbmpDestroyHeader(&bmp_head);
        if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
        FAPI_SYS_FREE(bmp_line_ptr);
        return(result);
    }

    //  ************************************************************
    //  *************** STANDARD BMP DECODING **********************
    //  ************************************************************

    // The code below will directly decode the bitmap into the framebuffer
    // without scaling. The scaler backend is not involved at all.

    if(((xx + bmp_head.width) > fb_info.xResApp)
    || ((yy + bmp_head.height) > fb_info.yResApp))
    {
        decbmpDestroyHeader(&bmp_head);
        if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
        FAPI_SYS_FREE(bmp_line_ptr);
        return(DECBMP_ERR_OUT_OF_FB_BOUNDS);
    }

    // Initialize line counter:
    line = 0;

    // Switch the bitmap's colour mode:
    switch(bmp_head.bitsPerPixel)
    {
        // ***************************
        // 32-bit bitmap:
        // ***************************
        case(32):
        if(fb_info.colourMode != OSDHANDLER_OSD_MODE_32_24BIT)
        {
            decbmpDestroyHeader(&bmp_head);
            if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
            FAPI_SYS_FREE(bmp_line_ptr);
            return(DECBMP_ERR_COLOUR_MODE_MISMATCH);
        }
        while(line < bmp_head.height)
        {
            if ( progressCb != NULL )
            {
                decode_info.decodedLines = line;

                /* Give progress and test for abort condition. */
                if ( progressCb (&decode_info, progressCbArg) != 0 )
                {
                    decbmpDestroyHeader(&bmp_head);
                    if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                    FAPI_SYS_FREE(bmp_line_ptr);
                    return(DECBMP_ERR_DECODE_ABORT);
                }
            }

            // Decode an unscaled bitmap line into a buffer first.
            // When decoding a whole stride, the buffer may contain
            // a few unused data at the end but this supersedes the seek.
            ii = readFptr(bmp_line_ptr, filePtr, stride);
            if (ii != stride)
            {
                decbmpDestroyHeader(&bmp_head);
                if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                FAPI_SYS_FREE(bmp_line_ptr);
                return(DECBMP_ERR_DATA_SOURCE);
            }
            src_8_ptr = bmp_line_ptr;

            (void) OSDHANDLER_FB_GetPixelAddress \
                                (fbHandle,
                                 xx,
                                 (yy + bmp_head.height) - (line + 1),
                                 &fb_32_ptr,
                                 &fb_offset);
            dst_32_ptr = fb_32_ptr;
            copy_steps = bmp_head.width;

            switch(fb_info.layoutMode)
            {
                case(OSDHANDLER_OSD_LAYOUT_ARGB):
                while(copy_steps--)
                {
                    /*lint -save -e701 */
                    *dst_32_ptr++ = (*(src_8_ptr + 3) << 24) | // A
                                    (*(src_8_ptr + 2) << 16) | // R
                                    (*(src_8_ptr + 1) <<  8) | // G
                                    (*(src_8_ptr + 0) <<  0);  // B
                    /*lint -restore */
                    src_8_ptr += 4;
                }
                break;

                case(OSDHANDLER_OSD_LAYOUT_RGBA):
                while(copy_steps--)
                {
                    /*lint -save -e701 */
                    *dst_32_ptr++ = (*(src_8_ptr + 2) << 24) | // R
                                    (*(src_8_ptr + 1) << 16) | // G
                                    (*(src_8_ptr + 0) <<  8) | // B
                                    (*(src_8_ptr + 3) <<  0);  // A
                    /*lint -restore */
                    src_8_ptr += 4;
                }
                break;

                case(OSDHANDLER_OSD_LAYOUT_ACbYCr):
                while(copy_steps--)
                {
                    /*lint -save -e701 */
                    tmp_val = OSDHANDLER_ConvertARGB2CbYCrA_word \
                                                ((*(src_8_ptr + 3) << 24) | // A
                                                 (*(src_8_ptr + 2) << 16) | // R
                                                 (*(src_8_ptr + 1) <<  8) | // G
                                                 (*(src_8_ptr + 0) <<  0)); // B
                    /*lint -restore */
                    src_8_ptr += 4;
                    *dst_32_ptr++ = ((tmp_val & 0xFFUL) << 24) | (tmp_val >> 8);
                }
                break;

                case(OSDHANDLER_OSD_LAYOUT_CbYCrA):
                while(copy_steps--)
                {
                    /*lint -save -e701 */
                    *dst_32_ptr++ = OSDHANDLER_ConvertARGB2CbYCrA_word \
                                    ((*(src_8_ptr + 3) << 24) | // A
                                     (*(src_8_ptr + 2) << 16) | // R
                                     (*(src_8_ptr + 1) <<  8) | // G
                                     (*(src_8_ptr + 0) <<  0)); // B
                    /*lint -restore */
                    src_8_ptr += 4;
                }
                break;

                case(OSDHANDLER_UNKNOWN_OSD_LAYOUT):
                default:
                decbmpDestroyHeader(&bmp_head);
                if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                FAPI_SYS_FREE(bmp_line_ptr);
                return(DECBMP_ERR_COLOUR_MODE_MISMATCH);
            }
            line++;
        }
        break;

        // ***************************
        // 24-bit bitmap:
        // ***************************
        case(24):
        if(fb_info.colourMode != OSDHANDLER_OSD_MODE_32_24BIT)
        {
            decbmpDestroyHeader(&bmp_head);
            if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
            FAPI_SYS_FREE(bmp_line_ptr);
            return(DECBMP_ERR_COLOUR_MODE_MISMATCH);
        }
        while(line < bmp_head.height)
        {
            if ( progressCb != NULL )
            {
                decode_info.decodedLines = line;

                /* Give progress and test for abort condition. */
                if ( progressCb (&decode_info, progressCbArg) != 0 )
                {
                    decbmpDestroyHeader(&bmp_head);
                    if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                    FAPI_SYS_FREE(bmp_line_ptr);
                    return(DECBMP_ERR_DECODE_ABORT);
                }
            }

            // Decode an unaltered bitmap line into a buffer:
            ii = readFptr(bmp_line_ptr, filePtr, stride);
            if (ii != stride)
            {
                decbmpDestroyHeader(&bmp_head);
                if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                FAPI_SYS_FREE(bmp_line_ptr);
                return(DECBMP_ERR_DATA_SOURCE);
            }
            src_8_ptr = bmp_line_ptr;

            (void) OSDHANDLER_FB_GetPixelAddress \
                                (fbHandle,
                                 xx,
                                 (yy + bmp_head.height) - (line + 1),
                                 &fb_32_ptr,
                                 &fb_offset);
            dst_32_ptr = fb_32_ptr;
            copy_steps = bmp_head.width;

            switch(fb_info.layoutMode)
            {
                case(OSDHANDLER_OSD_LAYOUT_ARGB):
                while(copy_steps--)
                {
                    *dst_32_ptr++ =             0xFF000000UL | // A
                                    (*(src_8_ptr + 2) << 16) | // R
                                    (*(src_8_ptr + 1) <<  8) | // G
                                    (*(src_8_ptr + 0) <<  0);  // B
                    src_8_ptr += 3;
                }
                break;

                case(OSDHANDLER_OSD_LAYOUT_RGBA):
                while(copy_steps--)
                {
                    /*lint -save -e701 */
                    *dst_32_ptr++ = (*(src_8_ptr + 2) << 24) | // R
                                    (*(src_8_ptr + 1) << 16) | // G
                                    (*(src_8_ptr + 0) <<  8) | // B
                                                0x000000FFUL;  // A
                    /*lint -restore */
                    src_8_ptr += 3;
                }
                break;

                case(OSDHANDLER_OSD_LAYOUT_ACbYCr):
                while(copy_steps--)
                {
                    tmp_val = OSDHANDLER_ConvertARGB2CbYCrA_word \
                                    (            0xFF000000UL | // A
                                     (*(src_8_ptr + 2) << 16) | // R
                                     (*(src_8_ptr + 1) <<  8) | // G
                                     (*(src_8_ptr + 0) <<  0)); // B
                    src_8_ptr += 3;
                    *dst_32_ptr++ = ((tmp_val & 0xFFUL) << 24) | (tmp_val >> 8);
                }
                break;

                case(OSDHANDLER_OSD_LAYOUT_CbYCrA):
                while(copy_steps--)
                {
                    *dst_32_ptr++ = OSDHANDLER_ConvertARGB2CbYCrA_word \
                                    (            0xFF000000UL | // A
                                     (*(src_8_ptr + 2) << 16) | // R
                                     (*(src_8_ptr + 1) <<  8) | // G
                                     (*(src_8_ptr + 0) <<  0)); // B
                    src_8_ptr += 3;
                }
                break;

                case(OSDHANDLER_UNKNOWN_OSD_LAYOUT):
                default:
                decbmpDestroyHeader(&bmp_head);
                if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                FAPI_SYS_FREE(bmp_line_ptr);
                return(DECBMP_ERR_COLOUR_MODE_MISMATCH);

            }
            line++;
        }
        break;

        // ***************************
        // 16-bit bitmap:
        // ***************************
        case(16):

        while(line < bmp_head.height)
        {
            if ( progressCb != NULL )
            {
                decode_info.decodedLines = line;

                /* Give progress and test for abort condition. */
                if ( progressCb (&decode_info, progressCbArg) != 0 )
                {
                    decbmpDestroyHeader(&bmp_head);
                    if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                    FAPI_SYS_FREE(bmp_line_ptr);
                    return(DECBMP_ERR_DECODE_ABORT);
                }
            }

            // Decode an unaltered bitmap line into a buffer:
            ii = readFptr(bmp_line_ptr, filePtr, stride);
            if (ii != stride)
            {
                decbmpDestroyHeader(&bmp_head);
                if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                FAPI_SYS_FREE(bmp_line_ptr);
                return(DECBMP_ERR_DATA_SOURCE);
            }
            src_8_ptr = bmp_line_ptr;

            (void) OSDHANDLER_FB_GetPixelAddress \
                                (fbHandle,
                                 xx,
                                 (yy + bmp_head.height) - (line + 1),
                                 &fb_32_ptr,
                                 &fb_offset);
            dst_32_ptr = fb_32_ptr;
            copy_steps = bmp_head.width;

            // ***************************
            // 16-bit bitmap on 16-bit OSD
            // ***************************
            if(fb_info.colourMode == OSDHANDLER_OSD_MODE_5551)
            {
                copy_rest = copy_steps % 2;
                copy_steps >>= 1;
                switch(fb_info.layoutMode)
                {
                    case(OSDHANDLER_OSD_LAYOUT_ARGB):
                    while(copy_steps--)
                    {
                        tmp_val  = (uint32_t)*src_8_ptr++ << 16;
                        tmp_val |= (uint32_t)*src_8_ptr++ << 24;
                        tmp_val |= (uint32_t)*src_8_ptr++ <<  0;
                        tmp_val |= (uint32_t)*src_8_ptr++ <<  8;
                        *dst_32_ptr++ = tmp_val;
                    }
                    if(copy_rest)
                    {
                        tmp_val  = ((uint32_t)*src_8_ptr++) << 16;
                        tmp_val |= ((uint32_t)*src_8_ptr++) << 24;
                        *dst_32_ptr = (*dst_32_ptr & 0x0000FFFFUL) | tmp_val;
                    }
                    break;

                    case(OSDHANDLER_OSD_LAYOUT_RGBA):
                    while(copy_steps--)
                    {
                        // sets A bit to 0 by left-shift:
                        tmp_val  = (uint32_t)*src_8_ptr++ << 17;
                        tmp_val |= (uint32_t)*src_8_ptr++ << 25;
                        tmp_val |= (uint32_t)*src_8_ptr++ <<  1;
                        tmp_val |= (((uint32_t)*src_8_ptr++) & 0x7F) <<  9;
                        *dst_32_ptr++ = tmp_val;
                    }
                    if(copy_rest)
                    {
                        tmp_val  = ((uint32_t)*src_8_ptr++) << 17;
                        tmp_val |= ((uint32_t)*src_8_ptr++) << 25;
                        *dst_32_ptr = (*dst_32_ptr & 0x0000FFFFUL) | tmp_val;
                    }
                    break;

                    case(OSDHANDLER_OSD_LAYOUT_ACbYCr):
                    while(copy_steps--)
                    {
                        // sets A bit to 0 by left-shift:
                        tmp_val = ((uint32_t)(OSDHANDLER_ConvertARGB2CbYCrA_5551
                                    (((uint32_t)*src_8_ptr) |
                                    (((uint32_t)*(src_8_ptr+1)) << 8))) << 15) & 0x7FFF0000UL;
                        src_8_ptr += 2;
                        tmp_val |= (uint32_t)(OSDHANDLER_ConvertARGB2CbYCrA_5551
                                    (((uint32_t)*src_8_ptr) |
                                    (((uint32_t)*(src_8_ptr+1)) << 8))) >> 1;
                        src_8_ptr += 2;
                        *dst_32_ptr++ = tmp_val;
                    }
                    if(copy_rest)
                    {
                        tmp_val = ((uint32_t)(OSDHANDLER_ConvertARGB2CbYCrA_5551
                                    (((uint32_t)*src_8_ptr) |
                                    (((uint32_t)*(src_8_ptr+1)) << 8))) << 15) & 0x7FFF0000UL;
                        src_8_ptr += 2;
                        *dst_32_ptr = (*dst_32_ptr & 0x0000FFFFUL) | tmp_val;
                    }
                    break;

                    case(OSDHANDLER_OSD_LAYOUT_CbYCrA):
                    while(copy_steps--)
                    {
                        tmp_val = (uint32_t)(OSDHANDLER_ConvertARGB2CbYCrA_5551
                                    (((uint32_t)*src_8_ptr) |
                                    (((uint32_t)*(src_8_ptr+1)) << 8))) << 16;
                        src_8_ptr += 2;
                        tmp_val |= (uint32_t)OSDHANDLER_ConvertARGB2CbYCrA_5551
                                    (((uint32_t)*src_8_ptr) |
                                    (((uint32_t)*(src_8_ptr+1)) << 8));
                        src_8_ptr += 2;
                        *dst_32_ptr++ = tmp_val;
                    }
                    if(copy_rest)
                    {
                        tmp_val = (uint32_t)OSDHANDLER_ConvertARGB2CbYCrA_5551
                                    (((uint32_t)*src_8_ptr) |
                                    (((uint32_t)*(src_8_ptr+1)) << 8)) << 16;
                        src_8_ptr += 2;
                        *dst_32_ptr = (*dst_32_ptr & 0x0000FFFFUL) | tmp_val;
                    }
                    break;

                    case(OSDHANDLER_UNKNOWN_OSD_LAYOUT):
                    default:
                    decbmpDestroyHeader(&bmp_head);
                    if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                    FAPI_SYS_FREE(bmp_line_ptr);
                    return(DECBMP_ERR_COLOUR_MODE_MISMATCH);
                }
            }
            // ***************************
            // 16-bit bitmap on 32-bit OSD
            // ***************************
            else if (fb_info.colourMode == OSDHANDLER_OSD_MODE_32_24BIT)
            {
                switch(fb_info.layoutMode)
                {
                    case(OSDHANDLER_OSD_LAYOUT_ARGB):
                    while(copy_steps--)
                    {
                        tmp_val = (*(src_8_ptr + 1) << 8) | *src_8_ptr;
                        src_8_ptr += 2;
                        *dst_32_ptr++ = (0xFF000000UL)
                                      | ((tmp_val & 0x001F) << 3)
                                      | ((tmp_val & 0x03E0) << 6)
                                      | ((tmp_val & 0x7C00) << 9);
                    }
                    break;

                    case(OSDHANDLER_OSD_LAYOUT_RGBA):
                    while(copy_steps--)
                    {
                        tmp_val = (*(src_8_ptr + 1) << 8) | *src_8_ptr;
                        src_8_ptr += 2;
                        *dst_32_ptr++ = ((tmp_val & 0x001F) << 11)
                                      | ((tmp_val & 0x03E0) << 14)
                                      | ((tmp_val & 0x7C00) << 17)
                                      | (0x000000FFUL);
                    }
                    break;

                    case(OSDHANDLER_OSD_LAYOUT_ACbYCr):
                    while(copy_steps--)
                    {
                        tmp_val = (*(src_8_ptr + 1) << 8) | *src_8_ptr;
                        src_8_ptr += 2;
                        tmp_val = OSDHANDLER_ConvertARGB2CbYCrA_5551(tmp_val);
                        *dst_32_ptr++ = (0xFF000000UL)
                                      | ((tmp_val & 0x003E) << 2)
                                      | ((tmp_val & 0x07C0) << 5)
                                      | ((tmp_val & 0xF800) << 8);
                    }
                    break;

                    case(OSDHANDLER_OSD_LAYOUT_CbYCrA):
                    while(copy_steps--)
                    {
                        tmp_val = (*(src_8_ptr + 1) << 8) | *src_8_ptr;
                        src_8_ptr += 2;
                        tmp_val = OSDHANDLER_ConvertARGB2CbYCrA_5551(tmp_val);
                        *dst_32_ptr++ = ((tmp_val & 0x003E) << 10)
                                      | ((tmp_val & 0x07C0) << 13)
                                      | ((tmp_val & 0xF800) << 16)
                                      | (0x000000FFUL);
                    }
                    break;

                    case(OSDHANDLER_UNKNOWN_OSD_LAYOUT):
                    default:
                    decbmpDestroyHeader(&bmp_head);
                    if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                    FAPI_SYS_FREE(bmp_line_ptr);
                    return(DECBMP_ERR_COLOUR_MODE_MISMATCH);
                }
            }
            else
            {
                decbmpDestroyHeader(&bmp_head);
                if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                FAPI_SYS_FREE(bmp_line_ptr);
                return(DECBMP_ERR_COLOUR_MODE_MISMATCH);
            }
            line++;
        }
        break;

        // ***************************
        // 8-bit bitmap:
        // ***************************
        case(8):
        // clut setup already done above but MISRA wants me to check here:
        if(clut_ptr == NULL)
        {
            decbmpDestroyHeader(&bmp_head);
            FAPI_SYS_FREE(bmp_line_ptr);
            return(DECBMP_ERR_BMP_DECODE);
        }

        while(line < bmp_head.height)
        {
            if ( progressCb != NULL )
            {
                decode_info.decodedLines = line;

                /* Give progress and test for abort condition. */
                if ( progressCb (&decode_info, progressCbArg) != 0 )
                {
                    decbmpDestroyHeader(&bmp_head);
                    if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                    FAPI_SYS_FREE(bmp_line_ptr);
                    return(DECBMP_ERR_DECODE_ABORT);
                }
            }

            // Decode an unaltered bitmap line into a buffer:
            ii = readFptr(bmp_line_ptr, filePtr, stride);
            if (ii != stride)
            {
                decbmpDestroyHeader(&bmp_head);
                if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                FAPI_SYS_FREE(bmp_line_ptr);
                return(DECBMP_ERR_DATA_SOURCE);
            }
            src_8_ptr = bmp_line_ptr;

            (void) OSDHANDLER_FB_GetPixelAddress \
                                (fbHandle,
                                 xx,
                                 (yy + bmp_head.height) - (line + 1),
                                 &fb_32_ptr,
                                 &fb_offset);
            dst_32_ptr = fb_32_ptr;
            copy_steps = bmp_head.width;

            // ***************************
            // 8-bit bitmap on 32-bit OSD
            // ***************************
            if(fb_info.colourMode == OSDHANDLER_OSD_MODE_32_24BIT)
            {
                while(copy_steps--)
                {
                    *dst_32_ptr++ = clut_ptr[*src_8_ptr++];
                }
            }
            // ***************************
            // 8-bit bitmap on 8-bit OSD
            // ***************************
            else if(fb_info.colourMode == OSDHANDLER_OSD_MODE_CLUT_8BIT)
            {
                if(fb_offset)
                {
                    for(ii = (0x20UL - fb_offset) >> 3; ii > 0; ii--)
                    {
                        data_mask = 0x000000FFUL << (0x0018UL - fb_offset);
                        *dst_32_ptr = (*dst_32_ptr & (~data_mask)) |
                                        ((uint32_t)*src_8_ptr++ <<
                                        (0x0018UL - fb_offset));
                        fb_offset += 8;
                        copy_steps--;
                    }
                    dst_32_ptr++;
                    fb_offset = 0;
                }

                // now copy 4byte-wise
                copy_rest = copy_steps % 4;
                copy_steps >>= 2;
                while(copy_steps--)
                {
                    ii  = (uint32_t)*src_8_ptr++ << 24;
                    ii |= (uint32_t)*src_8_ptr++ << 16;
                    ii |= (uint32_t)*src_8_ptr++ <<  8;
                    ii |= (uint32_t)*src_8_ptr++;
                    *dst_32_ptr++ = ii;
                }

                if(copy_rest)
                {
                    while(copy_rest--)
                    {
                        data_mask = 0x000000FFUL << (0x0018UL - fb_offset);
                        *dst_32_ptr = (*dst_32_ptr & (~data_mask)) |
                                        ((uint32_t)*src_8_ptr++ <<
                                            (0x0018UL - fb_offset));
                        fb_offset += 8;
                    }
                }
            }
            else
            {
                decbmpDestroyHeader(&bmp_head);
                if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                FAPI_SYS_FREE(bmp_line_ptr);
                return(DECBMP_ERR_COLOUR_MODE_MISMATCH);
            }
            line++;
        }
        break;

        // ***************************
        // 4-bit bitmap:
        // ***************************
        case(4):
        // clut setup already done above but MISRA wants me to check here:
        if(clut_ptr == NULL)
        {
            decbmpDestroyHeader(&bmp_head);
            FAPI_SYS_FREE(bmp_line_ptr);
            return(DECBMP_ERR_BMP_DECODE);
        }

        while(line < bmp_head.height)
        {
            if ( progressCb != NULL )
            {
                decode_info.decodedLines = line;

                /* Give progress and test for abort condition. */
                if ( progressCb (&decode_info, progressCbArg) != 0 )
                {
                    decbmpDestroyHeader(&bmp_head);
                    if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                    FAPI_SYS_FREE(bmp_line_ptr);
                    return(DECBMP_ERR_DECODE_ABORT);
                }
            }

            // Decode an unaltered bitmap line into a buffer:
            ii = readFptr(bmp_line_ptr, filePtr, stride);
            if (ii != stride)
            {
                decbmpDestroyHeader(&bmp_head);
                if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                FAPI_SYS_FREE(bmp_line_ptr);
                return(DECBMP_ERR_DATA_SOURCE);
            }
            src_8_ptr = bmp_line_ptr;

            (void) OSDHANDLER_FB_GetPixelAddress \
                                (fbHandle,
                                 xx,
                                 (yy + bmp_head.height) - (line + 1),
                                 &fb_32_ptr,
                                 &fb_offset);
            dst_32_ptr = (uint32_t*) fb_32_ptr;
            copy_steps = bmp_head.width;

            // ***************************
            // 4-bit bitmap on 32-bit OSD
            // ***************************
            if(fb_info.colourMode == OSDHANDLER_OSD_MODE_32_24BIT)
            {
                // ii serves as a flag that decides whether to read the upper or
                // the lower 4 bits of a byte from the bitmap file:
                ii = 1;
                while(copy_steps--)
                {
                    *dst_32_ptr++ = clut_ptr \
                        [ii ? ((*src_8_ptr & 0xF0UL) >> 4): (*src_8_ptr++ & 0x0FUL)];
                    ii = !ii;
                }
                if(!ii)
                {
                    src_8_ptr++;
                }
            }
            // ***************************
            // 4-bit bitmap on 8-bit OSD
            // ***************************
            else if((fb_info.colourMode == OSDHANDLER_OSD_MODE_CLUT_8BIT)
            ||      (fb_info.colourMode == OSDHANDLER_CURSOR_MODE_8BIT))
            {
                // ii serves as a flag that decides whether to read the upper or
                // the lower 4 bits of a byte from the bitmap file:
                ii = 1;
                while(copy_steps--)
                {
                    data_mask = 0x000000FFUL << (0x0018UL - fb_offset);
                    *dst_32_ptr = (*dst_32_ptr & (~data_mask)) |
                                    ((ii ?
                                     ((*src_8_ptr & 0xF0UL) >> 4) :
                                     (*src_8_ptr++ & 0x0FUL)) <<
                                        (0x0018UL - fb_offset));
                    ii = !ii;
                    fb_offset += 8;
                    if(fb_offset >> 5)
                    {
                        fb_offset = 0;
                        dst_32_ptr++;
                    }
                }
                if(!ii)
                {
                    src_8_ptr++;
                }
            }
            // ***************************
            // 4-bit bitmap on 4-bit OSD
            // ***************************
            else if((fb_info.colourMode == OSDHANDLER_OSD_MODE_CLUT_4BIT)
            ||      (fb_info.colourMode == OSDHANDLER_CURSOR_MODE_4BIT))
            {
                if(fb_offset % 8)
                {
                    // this mode is entered if the x position of the bitmap is odd:
                    // Attention: pixel accurate algorithm but low performance

                    // ii serves as a flag that decides whether to read the upper or
                    // the lower 4 bits of a byte from the bitmap file:
                    ii = 1;
                    while(copy_steps)
                    {
                        data_mask = 0x0000000FUL << (0x001CUL - fb_offset);
                        *dst_32_ptr = (*dst_32_ptr & (~data_mask)) |
                                        ((ii ?
                                         ((*src_8_ptr & 0xF0UL) >> 4):
                                         (*src_8_ptr++ & 0x0FUL)) <<
                                         (0x001CUL - fb_offset));
                        ii = !ii;
                        fb_offset += 4;
                        if(fb_offset >> 5)
                        {
                            fb_offset = 0;
                            dst_32_ptr++;
                        }
                        copy_steps--;
                    }
                    if(!ii)
                    {
                        src_8_ptr++;
                    }
                }
                else
                {
                    // higher performance algorithm for even x positions of the bmp:

                    // write the first bytes:
                    while((fb_offset) && (copy_steps >> 1))
                    {
                        data_mask = 0x000000FFUL << (0x0018UL - fb_offset);
                        *dst_32_ptr = (*dst_32_ptr & (~data_mask)) |
                                        ((uint32_t)*src_8_ptr++ <<
                                            (0x0018UL - fb_offset));
                        fb_offset += 8;
                        copy_steps -= 2;
                        if(fb_offset >> 5)
                        {
                            fb_offset = 0;
                            dst_32_ptr++;
                        }
                    }

                    // now copy 4-byte-wise:
                    copy_rest = copy_steps % 8;
                    copy_steps >>= 3;
                    while(copy_steps--)
                    {
                        ii  = (uint32_t)*src_8_ptr++ << 24;
                        ii |= (uint32_t)*src_8_ptr++ << 16;
                        ii |= (uint32_t)*src_8_ptr++ <<  8;
                        ii |= (uint32_t)*src_8_ptr++;
                        *dst_32_ptr++ = ii;
                    }

                    copy_steps = copy_rest >> 1;
                    copy_rest %= 2;

                    // copy the trailing bytes:
                    while(copy_steps)
                    {
                        data_mask = 0x000000FFUL << (0x0018UL - fb_offset);
                        *dst_32_ptr = (*dst_32_ptr & (~data_mask)) |
                                        ((uint32_t)*src_8_ptr++ <<
                                        (0x0018UL - fb_offset));
                        fb_offset += 8;
                        copy_steps--;
                    }

                    // if the horizontal resolution is uneven, write the last pixel:
                    if(copy_rest)
                    {
                        data_mask = 0x0000000FUL << (0x001CUL - fb_offset);
                        *dst_32_ptr = (*dst_32_ptr & (~data_mask)) |
                                        (((*src_8_ptr++ & 0x00F0UL) >> 4) <<
                                        (0x001CUL - fb_offset));
                    }
                }
            }
            else
            {
                decbmpDestroyHeader(&bmp_head);
                if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                FAPI_SYS_FREE(bmp_line_ptr);
                return(DECBMP_ERR_COLOUR_MODE_MISMATCH);
            }
            line++;
        }
        break;

        // ***************************
        // 1-bit bitmap:
        // ***************************
        case(1):
        // clut setup already done above but MISRA wants me to check here:
        if(clut_ptr == NULL)
        {
            decbmpDestroyHeader(&bmp_head);
            FAPI_SYS_FREE(bmp_line_ptr);
            return(DECBMP_ERR_BMP_DECODE);
        }

        while(line < bmp_head.height)
        {
            if ( progressCb != NULL )
            {
                decode_info.decodedLines = line;

                /* Give progress and test for abort condition. */
                if ( progressCb (&decode_info, progressCbArg) != 0 )
                {
                    decbmpDestroyHeader(&bmp_head);
                    if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                    FAPI_SYS_FREE(bmp_line_ptr);
                    return(DECBMP_ERR_DECODE_ABORT);
                }
            }

            // Decode an unaltered bitmap line into a buffer:
            ii = readFptr(bmp_line_ptr, filePtr, stride);
            if (ii != stride)
            {
                decbmpDestroyHeader(&bmp_head);
                if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                FAPI_SYS_FREE(bmp_line_ptr);
                return(DECBMP_ERR_DATA_SOURCE);
            }
            src_8_ptr = bmp_line_ptr;

            (void) OSDHANDLER_FB_GetPixelAddress \
                                (fbHandle,
                                 xx,
                                 (yy + bmp_head.height) - (line + 1),
                                 &fb_32_ptr,
                                 &fb_offset);
            dst_32_ptr = fb_32_ptr;
            copy_steps = bmp_head.width;
            // ***************************
            // 1-bit bitmap on 32-bit OSD
            // ***************************
            if(fb_info.colourMode == OSDHANDLER_OSD_MODE_32_24BIT)
            {
                ii = 7;
                while(copy_steps--)
                {
                    *dst_32_ptr++ = clut_ptr \
                        [(*src_8_ptr & (0x01UL << ii)) ? 1 : 0];
                    if (ii == 0)
                    {
                        ii = 7;
                        src_8_ptr++;
                    }
                    else
                    {
                        ii--;
                    }
                }
                if(ii != 7)
                {
                    src_8_ptr++;
                }
            }
            // ***************************
            // 1-bit bitmap on 8-bit OSD
            // ***************************
            else if((fb_info.colourMode == OSDHANDLER_OSD_MODE_CLUT_8BIT)
            ||      (fb_info.colourMode == OSDHANDLER_CURSOR_MODE_8BIT))
            {
                ii = 7;
                while(copy_steps--)
                {
                    data_mask = 0x000000FFUL << (0x0018UL - fb_offset);
                    *dst_32_ptr = (*dst_32_ptr & (~data_mask)) |
                                (((*src_8_ptr & (0x01UL << ii)) ? 1UL : 0UL) <<
                                    (0x0018UL - fb_offset));
                    if (ii == 0)
                    {
                        ii = 7;
                        src_8_ptr++;
                    }
                    else
                    {
                        ii--;
                    }
                    fb_offset += 8;
                    if(fb_offset >> 5)
                    {
                        fb_offset = 0;
                        dst_32_ptr++;
                    }
                }
                if(ii != 7)
                {
                    src_8_ptr++;
                }
            }
            // ***************************
            // 1-bit bitmap on 1-bit OSD
            // ***************************
            else if(fb_info.colourMode == OSDHANDLER_CURSOR_MODE_1BIT)
            {
                if(fb_offset % 8)
                {
                    /* this mode is entered if the x position of the bitmap is
                    ** not a multiple of 8. Therefore there is a misalignment
                    ** of the data in the bitmap file with respect to the frame
                    ** buffer and the single pixels need to be shifted.
                    ** Attention: low performance
                    */

                    ii = 7;
                    data_mask = 0x0001UL << (0x001FUL - fb_offset);
                    while(copy_steps)
                    {
                        if(*src_8_ptr & (0x01UL << ii))
                        {
                            *dst_32_ptr |= data_mask;
                        }
                        else
                        {
                            *dst_32_ptr &= (~data_mask);
                        }
                        if (ii == 0)
                        {
                            ii = 7;
                            src_8_ptr++;
                        }
                        else
                        {
                            ii--;
                        }
                        fb_offset++;
                        data_mask >>= 1;
                        if(fb_offset >> 5)
                        {
                            fb_offset = 0;
                            data_mask = 0x80000000UL;
                            dst_32_ptr++;
                        }
                        copy_steps--;
                    }
                    if(ii != 7)
                    {
                        src_8_ptr++;
                    }
                }
                else
                {
                    // higher performance algorithm for xx = a multiple of 8

                    // write the first bytes:
                    while((fb_offset) && (copy_steps >> 3))
                    {
                        data_mask = 0x000000FFUL << (0x0018UL - fb_offset);
                        *dst_32_ptr = (*dst_32_ptr & (~data_mask)) |
                                        ((uint32_t)*src_8_ptr++ <<
                                            (0x0018UL - fb_offset));
                        fb_offset += 8;
                        copy_steps -= 8;
                        if(fb_offset >> 5)
                        {
                            fb_offset = 0;
                            dst_32_ptr++;
                        }
                    }

                    // now copy 4-byte-wise:
                    copy_rest = copy_steps % 32;
                    copy_steps >>= 5;
                    while(copy_steps--)
                    {
                        ii  = (uint32_t)*src_8_ptr++ << 24;
                        ii |= (uint32_t)*src_8_ptr++ << 16;
                        ii |= (uint32_t)*src_8_ptr++ <<  8;
                        ii |= (uint32_t)*src_8_ptr++;
                        *dst_32_ptr++ = ii;
                    }

                    copy_steps = copy_rest >> 3;
                    copy_rest %= 8;

                    // copy the remaining bytes:

                    while(copy_steps)
                    {
                        data_mask = 0x000000FFUL << (0x0018UL - fb_offset);
                        *dst_32_ptr = (*dst_32_ptr & (~data_mask)) |
                                        ((uint32_t)*src_8_ptr++ <<
                                        (0x0018UL - fb_offset));
                        fb_offset += 8;
                        copy_steps--;
                    }

                    // copy the last pixels if any
                    if(copy_rest)
                    {
                        data_mask = ((0x0001UL << copy_rest) - 0x0001UL) <<
                                    (0x0020UL - (fb_offset + copy_rest));
                        *dst_32_ptr = (*dst_32_ptr & (~data_mask)) |
                                        (
                                         ((uint32_t)*src_8_ptr++ >>
                                                    (8 - copy_rest)) <<
                                         ((0x0020UL - (fb_offset + copy_rest)))
                                        );
                    }
                }
            }
            else
            {
                decbmpDestroyHeader(&bmp_head);
                if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
                FAPI_SYS_FREE(bmp_line_ptr);
                return(DECBMP_ERR_COLOUR_MODE_MISMATCH);
            }
            line++;
        }
        break;

        // unknown bits per pixel value of bitmap:
        default:
        decbmpDestroyHeader(&bmp_head);
        if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
        FAPI_SYS_FREE(bmp_line_ptr);
        return(DECBMP_ERR_FORMAT_NOT_SUPPORTED);
    }

    // bitmap successfully decoded:
    decbmpDestroyHeader(&bmp_head);
    if(clut_ptr) FAPI_SYS_FREE(clut_ptr);
    FAPI_SYS_FREE(bmp_line_ptr);
    return(FAPI_OK);
}
