/*!
*****************************************************************************
**
** \file        ./subsystems/paint/picture/codecs/bmp/src/decbmp.h
**
** \version     $Id: $
**
** \brief       picture decoding library for BMP
**
**              This file contains the declaration of the DECBMP picture
**              decoder API for BMP.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

#ifndef DECBMP_H
    #define DECBMP_H

    #include <stdint.h>
    #include <fapi/sys_error.h>
    #include <picture/picdec_interface.h>


    //***********************************************************************
    //***********************************************************************
    //** Defines and Macros
    //***********************************************************************
    //***********************************************************************

    /*!
    *************************************************************************
    ** Standard error codes for the DECBMP library
    *************************************************************************
    */

    #define SUBSYS_PAINT_DECBMP_MODULE_BASE \
            (int32_t)(SUBSYS_PAINT_PICTURE_MODULE_BASE - 12000)

    /*! Bad parameter passed. */
    #define DECBMP_ERR_BAD_PARAMETER \
            (int32_t)(SUBSYS_PAINT_DECBMP_MODULE_BASE + FAPI_ERR_BAD_PARAMETER)

    /*! Memory allocation failed. */
    #define DECBMP_ERR_OUT_OF_MEMORY \
            (int32_t)(SUBSYS_PAINT_DECBMP_MODULE_BASE + FAPI_ERR_OUT_OF_MEMORY)

    /*! Device already initialised. */
    #define DECBMP_ERR_ALREADY_INITIALIZED \
            (int32_t)(SUBSYS_PAINT_DECBMP_MODULE_BASE + FAPI_ERR_ALREADY_INITIALIZED)

    /*! Device not initialised. */
    #define DECBMP_ERR_NOT_INITIALIZED \
            (int32_t)(SUBSYS_PAINT_DECBMP_MODULE_BASE + FAPI_ERR_NOT_INITIALIZED)

    /*! Feature or function is not available. */
    #define DECBMP_ERR_FEATURE_NOT_SUPPORTED \
            (int32_t)(SUBSYS_PAINT_DECBMP_MODULE_BASE + FAPI_ERR_FEATURE_NOT_SUPPORTED)

    /*! Timeout occured. */
    #define DECBMP_ERR_TIMEOUT \
            (int32_t)(SUBSYS_PAINT_DECBMP_MODULE_BASE + FAPI_ERR_TIMEOUT)

    /*! The device is busy, try again later. */
    #define DECBMP_ERR_DEVICE_BUSY \
            (int32_t)(SUBSYS_PAINT_DECBMP_MODULE_BASE + FAPI_ERR_DEVICE_BUSY)

    /*! Invalid handle was passed. */
    #define DECBMP_ERR_INVALID_HANDLE \
            (int32_t)(SUBSYS_PAINT_DECBMP_MODULE_BASE + FAPI_ERR_INVALID_HANDLE)

    /*! Semaphore could not be created. */
    #define DECBMP_ERR_SEMAPHORE_CREATE \
            (int32_t)(SUBSYS_PAINT_DECBMP_MODULE_BASE + FAPI_ERR_SEMAPHORE_CREATE)

    /*! The driver's used version is not supported. */
    #define DECBMP_ERR_UNSUPPORTED_VERSION \
            (int32_t)(SUBSYS_PAINT_DECBMP_MODULE_BASE + FAPI_ERR_UNSUPPORTED_VERSION)

    /*!
    *************************************************************************
    ** Additional DECBMP error codes.
    *************************************************************************
    */

    /*! \brief DECBMP specific error code offset */
    #define DECBMP_ERR_BASE \
            (int32_t) (SUBSYS_PAINT_DECBMP_MODULE_BASE - 100)

    /*! The specified file is no BMP file or not supported. */
    #define DECBMP_ERR_BAD_FILE_FORMAT          (int32_t) (DECBMP_ERR_BASE - 0)

    /*! An error occured while decoding the BMP file. It may be corrupted. */
    #define DECBMP_ERR_BMP_DECODE               (int32_t) (DECBMP_ERR_BASE - 1)

    /*! The picture would be decoded out of bounds of the frame buffer. */
    #define DECBMP_ERR_OUT_OF_FB_BOUNDS         (int32_t) (DECBMP_ERR_BASE - 2)

    /*! The particular file format of the bitmap is not supported. */
    #define DECBMP_ERR_FORMAT_NOT_SUPPORTED     (int32_t) (DECBMP_ERR_BASE - 3)

    /*! The colour/layout mode combination of BMP and FB is not supported. */
    #define DECBMP_ERR_COLOUR_MODE_MISMATCH     (int32_t) (DECBMP_ERR_BASE - 4)

    /*! The data source could not be read or it did not seek correctly. */
    #define DECBMP_ERR_DATA_SOURCE              (int32_t) (DECBMP_ERR_BASE - 5)

    /*! The decoding process was aborted. */
    #define DECBMP_ERR_DECODE_ABORT             (int32_t) (DECBMP_ERR_BASE - 6)

    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** Global Data
    //***********************************************************************
    //***********************************************************************

    /*! \brief  Interface of the PNG decoder driver. */
    extern PICDEC_INTERFACE_DriverT DECBMP_Driver;

    /*! \brief  Points to the interface of the PNG decoder driver. */
    extern const PICDEC_INTERFACE_DriverT* DECBMP_DRIVER_PTR;


    //***********************************************************************
    //***********************************************************************
    //** API Functions
    //***********************************************************************
    //***********************************************************************

    #ifdef __cplusplus
    extern "C" {
    #endif

    int32_t DECBMP_Init(void);
    void    DECBMP_Exit(void);
    int32_t DECBMP_CheckFile (void* filePtr,
                              int32_t fileOffset,
                              PICDEC_ReadFileChunkT readFptr,
                              PICDEC_FileSeekT seekFptr);
    int32_t DECBMP_GetInfo   (void* filePtr,
                              int32_t fileOffset,
                              PICDEC_ReadFileChunkT readFptr,
                              PICDEC_FileSeekT seekFptr,
                              PICDEC_RotationT rotation,
                              PICDEC_InfoT* infoPtr);
    int32_t DECBMP_Decode    (void* filePtr,
                              int32_t fileOffset,
                              PICDEC_ReadFileChunkT readFptr,
                              PICDEC_FileSeekT seekFptr,
                              OSDHANDLER_FB_HandleT fbHandle,
                              uint32_t xx,
                              uint32_t yy,
                              uint32_t xRes,
                              uint32_t yRes,
                              PICDEC_RotationT rotation,
                              PICDEC_ScalerT scaler,
                              PICDEC_ProgressCbT progressCb,
                              void* progressCbArg);

    #ifdef __cplusplus
    }
    #endif
#endif
