/*
*****************************************************************************
**
** \file        ./subsystems/paint/picture/codecs/jpeg/src/decjpeg.c
**
** \version     $Id: $
**
** \brief       picture decoding library for JPEG
**
**              This file contains the implementation of the DECJPEG picture
**              decoder API for JPEG.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

#include <stdint.h>
#include <setjmp.h>
#include <fapi/sys_services.h>
#include <osdhandler/osdhandler.h>
#include <osdhandler/osdhandler_fb.h>
#include <picture/picdec_interface.h>
#include <picture/picscaler.h>
#include <jpeg/jpeglib.h>
#include <jpeg/jerror.h>
#include <jpeg/jdatasrcfapi.h>
#include "decjpeg.h"

//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Local Structures
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Local Functions Declaration
//***************************************************************************
//***************************************************************************

static void decjpegErrorExit(j_common_ptr cinfoPtr);
static void decjpegOutputMessage (j_common_ptr cinfoPtr);
static uint32_t decjpegScalerReadLine (void* lineBufPtr, \
                                       void* optArgPtr,  \
                                       uint32_t lineCnt);

//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************

/*!
*****************************************************************************
** \brief JPEG Driver Interface implementation.
**
** This datastucture represents the JPEG driver specific implementation
** of the picture library driver interface.
**
** \sa PICDEC_INTERFACE_DriverT
****************************************************************************
*/
PICDEC_INTERFACE_DriverT DECJPEG_Driver =
{
    "JPEG",             // formatNamePtr
    DECJPEG_Init,       // initFptr
    DECJPEG_Exit,       // exitFptr
    DECJPEG_CheckFile,  // checkFptr
    DECJPEG_GetInfo,    // getInfoFptr
    DECJPEG_Decode      // decodeFptr
};

const PICDEC_INTERFACE_DriverT* DECJPEG_DRIVER_PTR = &DECJPEG_Driver;


//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

/*!
*******************************************************************************
**
** \brief Error handler for fatal errors while decoding a JPEG image.
**
** \param cinfoPtr  JPEG decoding structure.
**
*******************************************************************************
*/
static void decjpegErrorExit(j_common_ptr cinfoPtr)
{
    jmp_buf* jmp_buf_ptr = (jmp_buf*)cinfoPtr->client_data;

    // Display the message.
    (*cinfoPtr->err->output_message) (cinfoPtr);

    // Return control to the setjmp point
    longjmp(*jmp_buf_ptr, DECJPEG_ERR_JPEG_DECODE);
}

/*!
*******************************************************************************
**
** \brief Error handler for printing error messages from inside the JPEG library.
**
** \param cinfoPtr  JPEG decoding structure.
**
*******************************************************************************
*/
static void decjpegOutputMessage (j_common_ptr cinfoPtr)
{
    char_t buffer[JMSG_LENGTH_MAX];

    // Message selection:
    if((cinfoPtr->err->msg_code != (int32_t)JMSG_NOMESSAGE)
    && (cinfoPtr->err->msg_code != (int32_t)JERR_NO_SOI))    // Not a JPEG
    {
        /* Create the message */
        (*cinfoPtr->err->format_message) (cinfoPtr, buffer);

        /* Send it via FAPI_SYS_PRINT_DEBUG, adding a newline */
        FAPI_SYS_PRINT_DEBUG(1, "\n[JPEG] %s\n", buffer);
    }
    return;
}

/*!
*******************************************************************************
**
** \brief   Callback function for scaler that reads JPEG lines.
**
**          This function is used as callback by the software scaler in
**          order to read one or several JPEG lines to a buffer.
**
** \param lineBufPtr    Buffer to which the line will be written.
** \param optArgPtr     Optional arguments. Here: jpeg_decompress_struct
** \param   lineCnt     The number of requested lines to decode.
**
** \return              The number of actually decoded lines.
**
*******************************************************************************
*/
#define DECJPEG_MAX_LINE_COUNT_AT_DECODE_BURST  8
static uint32_t decjpegScalerReadLine (void* lineBufPtr, \
                                       void* optArgPtr,  \
                                       uint32_t lineCnt)
{
    uint32_t    result = 0;
    uint32_t    kk = 0;
    uint32_t*   lineBufPtrArr[DECJPEG_MAX_LINE_COUNT_AT_DECODE_BURST];
    struct jpeg_decompress_struct* cinfo_ptr = (struct jpeg_decompress_struct*)optArgPtr;

    if(lineCnt > DECJPEG_MAX_LINE_COUNT_AT_DECODE_BURST)
    {
        FAPI_SYS_PRINT_DEBUG(1, "\n[decjpeg] warning: " \
                "More than DECJPEG_MAX_LINE_COUNT_AT_DECODE_BURST requested!\n");
        lineCnt = DECJPEG_MAX_LINE_COUNT_AT_DECODE_BURST;
    }
    for(kk = 0; kk < lineCnt; kk++)
    {
        // rows are organized linearly in the scaler
        lineBufPtrArr[kk] = ((uint32_t*)lineBufPtr) + (kk * cinfo_ptr->output_width);
    }
    /*lint -save -e64 -e740 -e918 */
    result = jpeg_read_scanlines(cinfo_ptr, (JSAMPARRAY)lineBufPtrArr, lineCnt);
    /*lint -restore */
    return (result);
}


//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

/*!
*******************************************************************************
**
** \brief   Initialization of JPEG decoder library.
**
** \return
**          - #FAPI_OK
**
** \sa      - DECJPEG_Exit()
*******************************************************************************
**/
int32_t DECJPEG_Init(void)
{
    return(FAPI_OK);
}

/*!
*******************************************************************************
**
** \brief   Exit the JPEG decoder library.
**
** \return
**          - #FAPI_OK
**
** \sa      - DECJPEG_Init()
*******************************************************************************
**/
void DECJPEG_Exit(void)
{
    return;
}

/*!
*******************************************************************************
**
** \brief Checks whether a file can be processed by the DECJPEG library.
**
** \param   filePtr
**              Descriptor of file containing the picture data.
** \param   fileOffset
**              Picture data offset within the file.
** \param   readFptr
**              This function reads data chunks from filePtr.
** \param   seekFptr
**              This function sets the file descriptor position
**              of the memory source.
**
** \return
**          - #FAPI_OK if the file is a JPEG.
**          - #DECJPEG_ERR_BAD_PARAMETER if a pointer argument is NULL.
**          - #PICDEC_ERR_FORMAT_NOT_SUPPORTED if the JPEG library signaled an
**              error while processing the file.
**
** \sa      See #PICDEC_INTERFACE_DriverT specifications.
*******************************************************************************
**/
int32_t DECJPEG_CheckFile (void* filePtr,
                           int32_t fileOffset,
                           PICDEC_ReadFileChunkT readFptr,
                           PICDEC_FileSeekT seekFptr)
{
    struct jpeg_decompress_struct cinfo;
    struct jpeg_error_mgr err_mgr;
    jmp_buf jump_buf;
    int32_t result = 0;

    memset(&cinfo,   0, sizeof(struct jpeg_decompress_struct));
    memset(&err_mgr, 0, sizeof(struct jpeg_error_mgr));

    if((filePtr == NULL)
    || (readFptr == NULL)
    || (seekFptr == NULL))
    {
        return(DECJPEG_ERR_BAD_PARAMETER);
    }

    // rewind memory source:
    result = seekFptr(filePtr, fileOffset, PICDEC_SEEK_SET);
    if (result != 0) return(DECJPEG_ERR_DATA_SOURCE);

    // Set up the error handler of JPEG decoder lib:
    cinfo.err = jpeg_std_error(&err_mgr);
    err_mgr.error_exit = decjpegErrorExit;
    err_mgr.output_message = decjpegOutputMessage;

    // make jump buffer accessible through cinfo:
    /*lint -save -e545 */
    cinfo.client_data = &jump_buf;
    /*lint -restore */

    result = setjmp(jump_buf);
    if (result)
    {
        // If we get here, the IJG JPEG library has signaled an error.
        jpeg_destroy_decompress(&cinfo);
        return (PICDEC_ERR_FORMAT_NOT_SUPPORTED);
    }

    // Initialize JPEG decompression object:
    jpeg_create_decompress(&cinfo);

    // Specify data source:
    // See "I/O suspension" in libjpeg.txt for data sources.
    fapi_ijg_jpeg_flexible_src(&cinfo, filePtr, readFptr);

    // Read file parameters, will longjmp to jump_buf if it's no JPEG:
    (void) jpeg_read_header(&cinfo, TRUE);

    // Abort decoding and destroy decompression object:
    jpeg_destroy_decompress(&cinfo);

    // If we got this far, it is a JPEG:
    return(FAPI_OK);
}

/*!
*******************************************************************************
** \brief   Gets information about a JPEG picture.
**
** \param   filePtr
**              Descriptor of file containing the picture data.
** \param   fileOffset
**              Picture data offset within the file.
** \param   readFptr
**              This function reads data chunks from filePtr.
** \param   seekFptr
**              This function sets the file descriptor position
**              of the memory source.
** \param   infoPtr
**              Will receive the requested information about the file.
**
** \return
**          - #FAPI_OK on success.
**          - #DECJPEG_ERR_BAD_PARAMETER if a pointer argument is NULL.
**          - #DECJPEG_ERR_JPEG_DECODE if the JPEG library signaled an error
**              while processing the file.
**
** \sa      See #PICDEC_INTERFACE_DriverT specifications.
*******************************************************************************
*/
int32_t DECJPEG_GetInfo (void* filePtr,
                         int32_t fileOffset,
                         PICDEC_ReadFileChunkT readFptr,
                         PICDEC_FileSeekT seekFptr,
                         PICDEC_RotationT rotation,
                         PICDEC_InfoT* infoPtr)
{
    struct jpeg_decompress_struct cinfo;
    struct jpeg_error_mgr err_mgr;
    jmp_buf jump_buf;
    int32_t result = 0;

    memset(&cinfo,   0, sizeof(struct jpeg_decompress_struct));
    memset(&err_mgr, 0, sizeof(struct jpeg_error_mgr));

    if((filePtr == NULL)
    || (readFptr == NULL)
    || (seekFptr == NULL)
    || (infoPtr == NULL))
    {
        return(DECJPEG_ERR_BAD_PARAMETER);
    }

    memset(infoPtr, 0, sizeof(PICDEC_InfoT));

    // rewind memory source:
    result = seekFptr(filePtr, fileOffset, PICDEC_SEEK_SET);
    if (result != 0) return(DECJPEG_ERR_DATA_SOURCE);

    // Set up the error handler of JPEG decoder lib:
    cinfo.err = jpeg_std_error(&err_mgr);
    err_mgr.error_exit = decjpegErrorExit;
    err_mgr.output_message = decjpegOutputMessage;

    // make jump buffer accessible through cinfo:
    /*lint -save -e545 */
    cinfo.client_data = &jump_buf;
    /*lint -restore */

    result = setjmp(jump_buf);
    if (result)
    {
        // If we get here, the JPEG code has signaled an error.
        jpeg_destroy_decompress(&cinfo);
        return (result);
    }

    // Initialize JPEG decompression object:
    jpeg_create_decompress(&cinfo);

    // Specify data source:
    // See "I/O suspension" in libjpeg.txt for data sources
    fapi_ijg_jpeg_flexible_src(&cinfo, filePtr, readFptr);

    // Read file parameters:
    (void) jpeg_read_header(&cinfo, TRUE);

    // Fill the info structure:
    // JPEG is stored either as colour (3 components) or grayscale (1 component)
    infoPtr->format = PICDEC_FORMAT_JPEG;
    infoPtr->bpp    = (uint32_t)cinfo.num_components * 8UL;
    switch(rotation)
    {
        case(PICDEC_ROTATION_90_LEFT):
        case(PICDEC_ROTATION_90_RIGHT):
        infoPtr->xRes   = cinfo.image_height;
        infoPtr->yRes   = cinfo.image_width;
        break;

        case(PICDEC_ROTATION_0):
        case(PICDEC_ROTATION_180):
        default:
        infoPtr->xRes   = cinfo.image_width;
        infoPtr->yRes   = cinfo.image_height;
        break;
    }
    memset(infoPtr->clutArray, 0, sizeof(infoPtr->clutArray));

    // Abort decoding and destroy decompression object:
    jpeg_destroy_decompress(&cinfo);

    return(FAPI_OK);

}

/*!
*******************************************************************************
** \brief   Decodes a JPEG picture into a frame buffer.
**
** \param   filePtr
**              Descriptor of file containing the picture data.
** \param   fileOffset
**              Picture data offset within the file.
** \param   readFptr
**              This function reads data chunks from filePtr.
** \param   seekFptr
**              This function sets the file descriptor position
**              of the memory source.
** \param   fbHandle
**              Handle for the frame buffer that will be modified.
** \param   xx
**              Horizontal coordinate of the picture in the frame buffer.
**              This coordinate is relative to the frame buffer, i.e., if
**              the picture is rotated, the xx and yy coordinates denote the
**              top left corner of the rotated picture.
** \param   yy
**              Vertical coordinate of the picture in the frame buffer.
**              This coordinate is relative to the frame buffer, i.e., if
**              the picture is rotated, the xx and yy coordinates denote the
**              top left corner of the rotated picture.
** \param   xRes
**              Desired horizontal resolution of the picture.
**              The resolution is achieved by software scaling.
**              If xRes is 0, the native horizontal resolution
**              of the file will be used.
** \param   yRes
**              Desired vertical resolution of the picture.
**              The resolution is achieved by software scaling.
**              If yRes is 0, the native vertical resolution
**              of the file will be used.
** \param   rotation
**              Defines a rotation of the picture in steps of
**              90 degrees.
** \param   scaler
**              Specifies the prefereed scaler algorithm to be
**              used while decoding. This option only applies
**              if the xRes or yRes parameter differs from the
**              native picture size.
** \param   progressCb
**              Optional progress and abort callback.
** \param   progressCbArg
**              Optional argument passed with progressCb.
**
** \return
**          - #FAPI_OK on success.
**          - #DECJPEG_ERR_BAD_PARAMETER if a pointer argument is NULL.
**          - #DECJPEG_ERR_JPEG_DECODE if the JPEG library signaled an error
**              while processing the file.
**          - #PICDEC_ERR_UNSUPPORTED_OSD_COLOUR_LAYOUT_MODE
**              if the combination of OSD colour and OSD layout mode is
**              not supported by the scaler lib.
**              Common combinations for scalers are 32-bit in RGBA and CbYCrA
**              layouts as well as the 16-bit CCIR mode.
**          - #PICDEC_ERR_OUT_OF_FB_BOUNDS
**              if the scaled picture is out of bounds of the frame buffer.
**          - #PICDEC_ERR_OUT_OF_MEMORY
**              if there was not enough heap memory available for the scaler.
**          - some OSDHANDLER related error code if the frame buffer handle
**            is bad.
**          - some scaler function specific error code if the scaler failed.
**
*******************************************************************************
*/
int32_t DECJPEG_Decode(void* filePtr,
                       int32_t fileOffset,
                       PICDEC_ReadFileChunkT readFptr,
                       PICDEC_FileSeekT seekFptr,
                       OSDHANDLER_FB_HandleT fbHandle,
                       uint32_t xx,
                       uint32_t yy,
                       uint32_t xRes,
                       uint32_t yRes,
                       PICDEC_RotationT rotation,
                       PICDEC_ScalerT scaler,
                       PICDEC_ProgressCbT progressCb,
                       void* progressCbArg)
{
    OSDHANDLER_FB_FrameBufferT fb_info;
    PICSCALER_DecodeParamsT scaler_params;
    PICDEC_DecodeInfoT decode_info;
    struct jpeg_decompress_struct cinfo;
    struct jpeg_error_mgr err_mgr;
    jmp_buf jump_buf;

    int32_t     result              = 0;    // error codes
    uint32_t    jpeg_width          = 0;    // JPEG dimensions
    uint32_t    jpeg_height         = 0;    // JPEG dimensions
    uint32_t    scale_h             = 0;
    uint32_t    scale_v             = 0;
    uint32_t    swap                = 0;

    memset(&fb_info,        0, sizeof(OSDHANDLER_FB_FrameBufferT));
    memset(&scaler_params,  0, sizeof(PICSCALER_DecodeParamsT));
    memset(&cinfo,          0, sizeof(struct jpeg_decompress_struct));
    memset(&err_mgr,        0, sizeof(struct jpeg_error_mgr));

    if((filePtr == NULL)
    || (readFptr == NULL)
    || (seekFptr == NULL))
    {
        return(DECJPEG_ERR_BAD_PARAMETER);
    }

    // rewind memory source:
    result = seekFptr(filePtr, fileOffset, PICDEC_SEEK_SET);
    if (result != 0) return(DECJPEG_ERR_DATA_SOURCE);

    // Set up the error handler of JPEG decoder lib:
    cinfo.err = jpeg_std_error(&err_mgr);
    err_mgr.error_exit = decjpegErrorExit;
    err_mgr.output_message = decjpegOutputMessage;

    // make jump buffer accessible through cinfo:
    /*lint -save -e545 */
    cinfo.client_data = &jump_buf;
    /*lint -restore */

    result = setjmp(jump_buf);
    if (result)
    {
        // If we get here, the JPEG code has signaled an error.
        jpeg_destroy_decompress(&cinfo);
        return (result);
    }

    // Initialize JPEG decompression object:
    jpeg_create_decompress(&cinfo);

    // Specify data source:
    // See "I/O suspension" in libjpeg.txt for data sources
    fapi_ijg_jpeg_flexible_src(&cinfo, filePtr, readFptr);

    // Read file parameters:
    (void) jpeg_read_header(&cinfo, TRUE);

    // Get image dimensions:
    jpeg_width  = cinfo.image_width;
    jpeg_height = cinfo.image_height;

    // Internally, resolution is oriented according to picture orientation:
    if((rotation == PICDEC_ROTATION_90_LEFT)
    || (rotation == PICDEC_ROTATION_90_RIGHT))
    {
        swap = xRes;
        xRes = yRes;
        yRes = swap;
    }

    if(xRes == 0)
    {
        xRes = jpeg_width;
    }
    if(yRes == 0)
    {
        yRes = jpeg_height;
    }

    // get frame buffer parameters and check dimensions:
    result = OSDHANDLER_FB_GetInfo(fbHandle, &fb_info);
    if(result)
    {
        jpeg_destroy_decompress(&cinfo);
        return(result);
    }


    //  ************************************************************
    //  ************ SCALED/ROTATED JPEG DECODING ******************
    //  ************************************************************

    // use simple BRESENHAM if nothing to scale:
    if((xRes == jpeg_width) && (yRes == jpeg_height))
    {
        scaler = PICDEC_SCALER_BRESENHAM;
    }

    // Set up the JPEG library to decode in RGBA 32-bit format:
    cinfo.out_color_space = JCS_RGBA; // Fujitsu specific 32-bit RGBA mode

    // select DCT method: JDCT_ISLOW delivers better quality but is slightly slower
    cinfo.dct_method = JDCT_IFAST; // JDCT_ISLOW

    // No speedup was found by changing these settings: --RK
    //cinfo.do_fancy_upsampling = 1;
    //cinfo.do_block_smoothing = 1;

    // prescale picture by integrated DCT scaler:
    cinfo.scale_denom = 8;

    // get the scaling ratios for both directions:
    scale_h = (xRes * cinfo.scale_denom) / jpeg_width;
    scale_v = (yRes * cinfo.scale_denom) / jpeg_height;

    // clip to the top in order to get the next higher resoluted level:
    if ((xRes * cinfo.scale_denom) % jpeg_width ) scale_h++;
    if ((yRes * cinfo.scale_denom) % jpeg_height) scale_v++;

    // clip at a factor of 2 which is a IJG decoder limitation:
    if (scale_h > 16) scale_h = 16;
    if (scale_v > 16) scale_v = 16;

    // take the higher resoluted side into account:
    cinfo.scale_num = (scale_h > scale_v) ? scale_h : scale_v;

    // This strategy proved to result in too fragmented looking pictures. --RK
    /*
    if ((scale_h > cinfo.scale_denom) && (scale_v > cinfo.scale_denom))
    {
        // on upscaling, prescale with smaller ratio and do the remaining
        // upscaling by the picture lib scaler:
        cinfo.scale_num = (scale_h < scale_v) ? scale_h : scale_v;
    }
    else if (((scale_h > cinfo.scale_denom) && (scale_v < cinfo.scale_denom))
    ||       ((scale_h < cinfo.scale_denom) && (scale_v > cinfo.scale_denom)))
    {
        // when scaling in both directions, do not prescale at all:
        cinfo.scale_num = cinfo.scale_denom;
    }
    else
    {
        // when downscaling, prescale with higher ratio in order
        // to preserve quality for the less downsccaled dimension:
        cinfo.scale_num = (scale_h > scale_v) ? scale_h : scale_v;
    }
    // */

    // update output parameters: get dimension of dct prescaled image
    jpeg_calc_output_dimensions(&cinfo);

    // begin decompression:
    result = jpeg_start_decompress(&cinfo);
    if(!result)
    {
        jpeg_destroy_decompress(&cinfo);
        return(DECJPEG_ERR_JPEG_DECODE);
    }

    //FAPI_SYS_PRINT_DEBUG(2, "\n[jpeg/debug] output resolution of jpeg: %dx%d\n",
    //                     cinfo.output_width, cinfo.output_height);

    // Initialize decoder info structure after prescaler:
    memset (&decode_info, 0, sizeof(decode_info));
    decode_info.totalPasses = 1;
    decode_info.totalLines = cinfo.output_height;

    scaler_params.fbHandle      = fbHandle;
    scaler_params.srcWidth      = cinfo.output_width;
    scaler_params.srcHeight     = cinfo.output_height;
    scaler_params.dstPosX       = xx;
    scaler_params.dstPosY       = yy;
    scaler_params.dstWidth      = xRes;
    scaler_params.dstHeight     = yRes;
    scaler_params.bufHeight     = (uint32_t)cinfo.rec_outbuf_height;
    scaler_params.rotation      = rotation;
    scaler_params.scaler        = scaler;
    scaler_params.readLineFptr  = decjpegScalerReadLine;
    scaler_params.skipLineFptr  = decjpegScalerReadLine;
    scaler_params.optArgPtr     = (void*)&cinfo;
    scaler_params.progressCb    = progressCb;
    scaler_params.progressCbArg = progressCbArg;
    scaler_params.decodeInfoPtr = &decode_info;

    result = PICSCALER_Decode(&scaler_params);

    // Finish decompression and release cinfo structure:
    // not needed since the cinfo structure isn't used any more
    // (void) jpeg_finish_decompress(&cinfo);

    // deallocate JPEG structure:
    jpeg_destroy_decompress(&cinfo);

    return(result);
}
