/*!
*****************************************************************************
**
** \file        ./subsystems/paint/picture/codecs/jpeg/src/decjpeg.h
**
** \version     $Id: $
**
** \brief       picture decoding library for JPEG
**
**              This file contains the declaration of the DECJPEG picture
**              decoder API for JPEG.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

#ifndef DECJPEG_H
    #define DECJPEG_H

    #include <stdint.h>
    #include <fapi/sys_error.h>
    #include <picture/picdec_interface.h>

    //***********************************************************************
    //***********************************************************************
    //** Defines and Macros
    //***********************************************************************
    //***********************************************************************

    /*!
    *************************************************************************
    ** Standard error codes for the DECJPEG library
    *************************************************************************
    */

    #define SUBSYS_PAINT_DECJPEG_MODULE_BASE \
            (int32_t)(SUBSYS_PAINT_PICTURE_MODULE_BASE - 14000)

    /*! Bad parameter passed. */
    #define DECJPEG_ERR_BAD_PARAMETER \
            (int32_t)(SUBSYS_PAINT_DECJPEG_MODULE_BASE + FAPI_ERR_BAD_PARAMETER)

    /*! Memory allocation failed. */
    #define DECJPEG_ERR_OUT_OF_MEMORY \
            (int32_t)(SUBSYS_PAINT_DECJPEG_MODULE_BASE + FAPI_ERR_OUT_OF_MEMORY)

    /*! Device already initialised. */
    #define DECJPEG_ERR_ALREADY_INITIALIZED \
            (int32_t)(SUBSYS_PAINT_DECJPEG_MODULE_BASE + FAPI_ERR_ALREADY_INITIALIZED)

    /*! Device not initialised. */
    #define DECJPEG_ERR_NOT_INITIALIZED \
            (int32_t)(SUBSYS_PAINT_DECJPEG_MODULE_BASE + FAPI_ERR_NOT_INITIALIZED)

    /*! Feature or function is not available. */
    #define DECJPEG_ERR_FEATURE_NOT_SUPPORTED \
            (int32_t)(SUBSYS_PAINT_DECJPEG_MODULE_BASE + FAPI_ERR_FEATURE_NOT_SUPPORTED)

    /*! Timeout occured. */
    #define DECJPEG_ERR_TIMEOUT \
            (int32_t)(SUBSYS_PAINT_DECJPEG_MODULE_BASE + FAPI_ERR_TIMEOUT)

    /*! The device is busy, try again later. */
    #define DECJPEG_ERR_DEVICE_BUSY \
            (int32_t)(SUBSYS_PAINT_DECJPEG_MODULE_BASE + FAPI_ERR_DEVICE_BUSY)

    /*! Invalid handle was passed. */
    #define DECJPEG_ERR_INVALID_HANDLE \
            (int32_t)(SUBSYS_PAINT_DECJPEG_MODULE_BASE + FAPI_ERR_INVALID_HANDLE)

    /*! Semaphore could not be created. */
    #define DECJPEG_ERR_SEMAPHORE_CREATE \
            (int32_t)(SUBSYS_PAINT_DECJPEG_MODULE_BASE + FAPI_ERR_SEMAPHORE_CREATE)

    /*! The driver's used version is not supported. */
    #define DECJPEG_ERR_UNSUPPORTED_VERSION \
            (int32_t)(SUBSYS_PAINT_DECJPEG_MODULE_BASE + FAPI_ERR_UNSUPPORTED_VERSION)

    /*!
    *************************************************************************
    ** Additional DECJPEG error codes.
    *************************************************************************
    */

    /*! \brief DECJPEG specific error code offset */
    #define DECJPEG_ERR_BASE \
            (int32_t) (SUBSYS_PAINT_DECJPEG_MODULE_BASE - 100)

    /*! The specified file is no JPEG file or not supported. */
    #define DECJPEG_ERR_BAD_FILE_FORMAT         (int32_t) (DECJPEG_ERR_BASE - 0)

    /*! An error occured while setting up the JPEG library. */
    #define DECJPEG_ERR_JPEG_INIT               (int32_t) (DECJPEG_ERR_BASE - 1)

    /*! An error occured while decoding the JPEG file. It may be corrupted. */
    #define DECJPEG_ERR_JPEG_DECODE             (int32_t) (DECJPEG_ERR_BASE - 2)

    /*! The picture would be decoded out of bounds of the frame buffer. */
    #define DECJPEG_ERR_OUT_OF_FB_BOUNDS        (int32_t) (DECJPEG_ERR_BASE - 3)

    /*! The colour/layout mode combination of JPEG and FB is not supported. */
    #define DECJPEG_ERR_COLOUR_MODE_MISMATCH    (int32_t) (DECJPEG_ERR_BASE - 4)

    /*! The layout mode is not correct for a CCIR OSD. */
    #define DECJPEG_ERR_CCIR_LAYOUT             (int32_t) (DECJPEG_ERR_BASE - 5)

    /*! The data source could not be read or it did not seek correctly. */
    #define DECJPEG_ERR_DATA_SOURCE             (int32_t) (DECJPEG_ERR_BASE - 6)

    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** Global Data
    //***********************************************************************
    //***********************************************************************

    /*! \brief  Interface of the JPEG decoder driver. */
    extern PICDEC_INTERFACE_DriverT DECJPEG_Driver;

    /*! \brief  Points to the interface of the JPEG decoder driver. */
    extern const PICDEC_INTERFACE_DriverT* DECJPEG_DRIVER_PTR;


    //***********************************************************************
    //***********************************************************************
    //** API Functions
    //***********************************************************************
    //***********************************************************************

    #ifdef __cplusplus
    extern "C" {
    #endif

    int32_t DECJPEG_Init(void);
    void    DECJPEG_Exit(void);
    int32_t DECJPEG_CheckFile (void* filePtr,
                               int32_t fileOffset,
                               PICDEC_ReadFileChunkT readFptr,
                               PICDEC_FileSeekT seekFptr);
    int32_t DECJPEG_GetInfo   (void* filePtr,
                               int32_t fileOffset,
                               PICDEC_ReadFileChunkT readFptr,
                               PICDEC_FileSeekT seekFptr,
                               PICDEC_RotationT rotation,
                               PICDEC_InfoT* infoPtr);
    int32_t DECJPEG_Decode    (void* filePtr,
                               int32_t fileOffset,
                               PICDEC_ReadFileChunkT readFptr,
                               PICDEC_FileSeekT seekFptr,
                               OSDHANDLER_FB_HandleT fbHandle,
                               uint32_t xx,
                               uint32_t yy,
                               uint32_t xRes,
                               uint32_t yRes,
                               PICDEC_RotationT rotation,
                               PICDEC_ScalerT scaler,
                               PICDEC_ProgressCbT progressCb,
                               void* progressCbArg);

    #ifdef __cplusplus
    }
    #endif
#endif // DECJPEG_H
