/*
*****************************************************************************
**
** \file        ./subsystems/paint/picture/codecs/png/src/decpng.c
**
** \version     $Id: $
**
** \brief       picture decoding library for PNG
**
**              This file contains the implementation of the DECPNG picture
**              decoder API for PNG.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

#include <stdint.h>
#include <fapex/abst_types.h>
#include <osdhandler/osdhandler.h>
#include <osdhandler/osdhandler_fb.h>
#include "png.h"
#include <picture/picdec.h>
#include <picture/picdec_interface.h>
#include <picture/picscaler.h>
#include "decpng.h"

//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************

/*! count of bytes used to check the PNG signature */
#define DECPNG_SIGNATURE_BYTES    8


//***************************************************************************
//***************************************************************************
//** Local Structures
//***************************************************************************
//***************************************************************************

typedef struct
{
    PICDEC_ReadFileChunkT readFptr;
    void* fileHandlePtr;
} decpngMemorySourceT;

typedef struct
{
    uint32_t* bufPtr;
    uint32_t  width;
    uint32_t  height;
    uint32_t  currentLine;
} decpngPicbufT;


//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Local Functions Declaration
//***************************************************************************
//***************************************************************************

static void decpngWarning(png_structp pngPtr, png_const_charp msgPtr);
static void decpngError(png_structp pngPtr, png_const_charp msgPtr);
static png_voidp decpngMalloc(png_structp pngPtr, png_size_t bytes);
static void decpngFree(png_structp pngPtr, png_voidp memPtr);
static void decpngTransformRGBA2CbYCrA(png_structp pngPtr, \
                                       png_row_infop rowInfoPtr, \
                                       png_bytep rowDataPtr);
static void decpngTransformARGB2ACbYCr(png_structp pngPtr, \
                                       png_row_infop rowInfoPtr, \
                                       png_bytep rowDataPtr);

static uint32_t decpngScalerReadLine  (void* lineBufPtr, \
                                       void* optArgPtr,  \
                                       uint32_t lineCnt);
static uint32_t decpngScalerSkipLine  (void* lineBufPtr, \
                                       void* optArgPtr,  \
                                       uint32_t lineCnt);
static uint32_t decpngScalerReadPicbuf(void* lineBufPtr, \
                                       void* optArgPtr,  \
                                       uint32_t lineCnt);
static uint32_t decpngScalerSkipPicbuf(void* lineBufPtr, \
                                       void* optArgPtr,  \
                                       uint32_t lineCnt);

//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************

/*!
*****************************************************************************
** \brief PNG Driver Interface implementation.
**
** This datastucture represents the PNG driver specific implementation
** of the picture library driver interface.
**
** \sa PICDEC_INTERFACE_DriverT
****************************************************************************
*/
PICDEC_INTERFACE_DriverT DECPNG_Driver =
{
    "PNG",              // formatNamePtr
    DECPNG_Init,        // initFptr
    DECPNG_Exit,        // exitFptr
    DECPNG_CheckFile,   // checkFptr
    DECPNG_GetInfo,     // getInfoFptr
    DECPNG_Decode       // decodeFptr
};

const PICDEC_INTERFACE_DriverT* DECPNG_DRIVER_PTR = &DECPNG_Driver;


//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

/*!
*******************************************************************************
**
** \brief Print a PNG warning message.
**
** This function prints the given PNG warning message using
** FAPI_SYS_PRINT_MSG.
**
** \note This function must be set via "png_set_error_fn()" in order to be used
**       instead of the default built-in libc based function.
**
** \param pngPtr Pointer to the png structure to access, currently unused.
** \param msgPtr The warning message to be printed.
**
*******************************************************************************
**/
static void decpngWarning(png_structp pngPtr, png_const_charp msgPtr)
{
    FAPI_SYS_PRINT_MSG("\n[decpng] Warning: %s\n", msgPtr);
}

/*!
*******************************************************************************
**
** \brief Print a PNG error message.
**
** This function prints the given PNG error message using
** FAPI_SYS_PRINT_MSG.
**
** \note This function must be set via "png_set_error_fn()" in order to be used
**       instead of the default built-in libc based function.
**
** \param pngPtr Pointer to the png structure to access, currently unused.
** \param msgPtr The error message to be printed.
**
*******************************************************************************
**/
static void decpngError(png_structp pngPtr, png_const_charp msgPtr)
{
    FAPI_SYS_PRINT_MSG("\n[decpng] Error: %s\n", msgPtr);
    longjmp(png_jmpbuf(pngPtr), 1);
}

/*!
*******************************************************************************
**
** \brief Allocate dynamic memory
**
** This function allocated the given number of bytes from the dynamic heap
** memory space by FAPI_SYS_MALLOC_CACHED.
**
** \note This function must be set via "png_set_mem_fn()" in order to be used
**       instead of the default built-in libc based function.
**
** \param pngPtr  Pointer to the png structure to access, currently unused.
** \param bytes The number of bytes to allocate.
**
** \return
** - NULL in case of error, no more memory available
** - non-NULL, a pointer to the allocated memory block
**
*******************************************************************************
**/
static png_voidp decpngMalloc(png_structp pngPtr, png_size_t bytes)
{
    png_voidp memPtr;
    memPtr = FAPI_SYS_MALLOC_CACHED(bytes);
    return(memPtr);
}

/*!
*******************************************************************************
**
** \brief Release dynamic memory.
**
** This function releases a previously allocated dynamic memory block from
** the heap memory space by FAPI_SYS_FREE.
**
** \note This function must be set via "png_set_mem_fn()" in order to be used
**       instead of the default built-in libc based function.
**
** \param pngPtr    Pointer to the png structure to access, currently unused.
** \param memPtr    The address of the memory block to release.
**
*******************************************************************************
**/
static void decpngFree(png_structp pngPtr, png_voidp memPtr)
{
    FAPI_SYS_FREE(memPtr);
    return;
}

/*!
*******************************************************************************
**
** \brief Read PNG data from file
**
** This function reads data chunks from a data source.
**
** \note This function must be set via "png_set_read_fn()" in order to be used
**       instead of the default built-in libc based function.
**
** \param pngPtr  The PNG data structure to access.
** \param dataPtr A pointer to a data buffer that will be filled with PNG data.
** \param length  The number of bytes to be written.
**
*******************************************************************************
**/
static void decpngReadData(png_structp pngPtr, png_bytep dataPtr, png_size_t length)
{
    uint32_t result;
    decpngMemorySourceT* mem_src_ptr;

    mem_src_ptr = png_get_io_ptr(pngPtr);
    result = mem_src_ptr->readFptr(dataPtr, mem_src_ptr->fileHandlePtr, length);
    if(! result)
    {
        png_error(pngPtr, "Error on decpngReadData\n");
    }
    return;
}

/*!
*******************************************************************************
**
** \brief Color transformation function: RGBA to CbYCrA
**
** This callback function transforms the color values from RGBA into CbYCrA
** format as expected by the OSD engine in OSDHANDLER_OSD_LAYOUT_CbYCrA mode.
**
** \param pngPtr     The PNG data structure to access.
** \param rowInfoPtr The PNG row information data to image information from.
** \param rowDataPtr The PNG data buffer to modify.
**
*******************************************************************************
*/
static void decpngTransformRGBA2CbYCrA(png_structp pngPtr, \
                                       png_row_infop rowInfoPtr, \
                                       png_bytep rowDataPtr)
{
    OSDHANDLER_ARGB__T a_r_g_b_word;
    OSDHANDLER_CbYCrA__T cb_y_cr_a_word;
    uint32_t ii;

    ii = rowInfoPtr->rowbytes / 4;
    while(ii--)
    {
        a_r_g_b_word.comp.alpha  = * rowDataPtr;
        a_r_g_b_word.comp.b      = *(rowDataPtr + 1);
        a_r_g_b_word.comp.g      = *(rowDataPtr + 2);
        a_r_g_b_word.comp.r      = *(rowDataPtr + 3);
        cb_y_cr_a_word.all  = OSDHANDLER_ConvertARGB2CbYCrA_union(a_r_g_b_word);
        * rowDataPtr        = cb_y_cr_a_word.comp.alpha;
        *(rowDataPtr + 1)   = cb_y_cr_a_word.comp.cr;
        *(rowDataPtr + 2)   = cb_y_cr_a_word.comp.y;
        *(rowDataPtr + 3)   = cb_y_cr_a_word.comp.cb;
        rowDataPtr += 4;
    }
    return;
}

/*!
*******************************************************************************
**
** \brief Color transformation function: ARGB to ACbYCr
**
** This callback function transforms the color values from ARGB into ACbYCr
** format as expected by the OSD engine in OSDHANDLER_OSD_LAYOUT_ACbYCr mode.
**
** \param pngPtr     The PNG data structure to access.
** \param rowInfoPtr The PNG row information data to image information from.
** \param rowDataPtr The PNG data buffer to modify.
**
*******************************************************************************
*/
static void decpngTransformARGB2ACbYCr(png_structp pngPtr, \
                                       png_row_infop rowInfoPtr, \
                                       png_bytep rowDataPtr)
{
    uint32_t a_r_g_b_word;
    uint32_t cb_y_cr_a_word;
    uint32_t ii;

    ii = rowInfoPtr->rowbytes / 4;
    while(ii--)
    {
        a_r_g_b_word =   (* rowDataPtr)              | // B
                         (*(rowDataPtr + 1)) <<  8   | // G
                        ((*(rowDataPtr + 2)) << 16)  | // R
                        ((*(rowDataPtr + 3)) << 24);   // A
        cb_y_cr_a_word = OSDHANDLER_ConvertARGB2CbYCrA_word(a_r_g_b_word);

        *rowDataPtr       = (cb_y_cr_a_word & 0x0000FF00UL) >>  8; // Cr
        *(rowDataPtr + 1) = (cb_y_cr_a_word & 0x00FF0000UL) >> 16; // Y
        *(rowDataPtr + 2) = (cb_y_cr_a_word & 0xFF000000UL) >> 24; // Cb
        *(rowDataPtr + 3) = (cb_y_cr_a_word & 0x000000FFUL);       // A
        rowDataPtr += 4;
    }
    return;
}

/*!
*******************************************************************************
**
** \brief   Callback function for scaler that reads a PNG line.
**
**          This function is used as callback by the software scaler in
**          order to read the PNG line wise.
**
** \param lineBufPtr    Buffer to which the line will be written.
** \param optArgPtr     Optional arguments. Here: png_structp of the resepective
**                      PNG to decode.
** \param lineCnt       The number of requested lines to decode.
**
** \return              The number of actually decoded lines.
**
*******************************************************************************
*/
static uint32_t decpngScalerReadLine(void* lineBufPtr, \
                                     void* optArgPtr,  \
                                     uint32_t lineCnt)
{
    png_structp png_ptr = (png_structp)optArgPtr;

    lineCnt = lineCnt; // ignore here, that's ok for the scaler
    png_read_row(png_ptr, (png_bytep) lineBufPtr, NULL);
    return(1);
}

/*!
*******************************************************************************
**
** \brief   Callback function for scaler that skips a PNG line.
**
**          This function is used as callback by the software scaler in
**          order to skip PNG lines. However, this function just reads
**          the data into a buffer instead of skipping since the PNG library
**          does not directly support such a feature.
**
** \param lineBufPtr    Buffer to which the line will be written.
** \param optArgPtr     Optional arguments. Here: png_structp of the resepective
**                      PNG to decode.
** \param lineCnt       The number of requested lines to skip.
**
** \return              The number of actually skipped lines.
**
*******************************************************************************
*/
static uint32_t decpngScalerSkipLine(void* lineBufPtr, \
                                     void* optArgPtr,  \
                                     uint32_t lineCnt)
{
    png_structp png_ptr = (png_structp)optArgPtr;

    lineCnt = lineCnt; // ignore here, that's ok for the scaler
    png_read_row(png_ptr, (png_bytep) lineBufPtr, NULL);
    return(1);
}


/*!
*******************************************************************************
**
** \brief   Callback function for scaler that reads lines from a 'picbuf'
**          picture buffer structure. This is itended to be used for the
**          display of interlaced PNGs while decoding.
**
**          This function is used as callback by the software scaler in
**          order to read the picbuf line wise.
**
** \param lineBufPtr    Buffer to which the line will be written.
** \param optArgPtr     Optional arguments. Here: pointer to a picbuf structure
**                      of the resepective PNG to decode.
** \param lineCnt       The number of requested lines to decode.
**
** \return              The number of actually decoded lines.
**
*******************************************************************************
*/
static uint32_t decpngScalerReadPicbuf(void* lineBufPtr, \
                                       void* optArgPtr,  \
                                       uint32_t lineCnt)
{
    decpngPicbufT* picbuf_ptr = (decpngPicbufT*)optArgPtr;

    memcpy(lineBufPtr,
           picbuf_ptr->bufPtr + (picbuf_ptr->currentLine * picbuf_ptr->width),
           lineCnt * picbuf_ptr->width * sizeof(uint32_t));

    picbuf_ptr->currentLine += lineCnt;

    return(lineCnt);
}

/*!
*******************************************************************************
**
** \brief   Callback function for scaler that skips lines of a 'picbuf'
**          picture buffer structure. This is itended to be used for the
**          display of interlaced PNGs while decoding.
**
**          This function is used as callback by the software scaler in
**          order to skip PNG lines. However, this function just reads
**          the data into a buffer instead of skipping since the PNG library
**          does not directly support such a feature.
**
** \param lineBufPtr    Buffer to which the line will be written.
** \param optArgPtr     Optional arguments. Here: pointer to a picbuf structure
**                      of the resepective PNG to decode.
** \param lineCnt       The number of requested lines to skip.
**
** \return              The number of actually skipped lines.
**
*******************************************************************************
*/
static uint32_t decpngScalerSkipPicbuf(void* lineBufPtr, \
                                       void* optArgPtr,  \
                                       uint32_t lineCnt)
{
    decpngPicbufT* picbuf_ptr = (decpngPicbufT*)optArgPtr;

    picbuf_ptr->currentLine += lineCnt;

    return(lineCnt);
}


//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

/*!
*******************************************************************************
**
** \brief   Initialization of PNG decoder library.
**
** \return
**          - #FAPI_OK
**
** \sa      - DECPNG_Exit()
*******************************************************************************
**/
int32_t DECPNG_Init(void)
{
    return(FAPI_OK);
}

/*!
*******************************************************************************
**
** \brief   Exit the PNG decoder library.
**
** \return
**          - #FAPI_OK
**
** \sa      - DECPNG_Init()
*******************************************************************************
**/
void DECPNG_Exit(void)
{
    return;
}

/*!
*******************************************************************************
**
** \brief Checks whether a file can be processed by the DECPNG library.
**
** \param   filePtr
**              Descriptor of file containing the picture data.
** \param   fileOffset
**              Picture data offset within the file.
** \param   readFptr
**              This function reads data chunks from filePtr.
** \param   seekFptr
**              This function sets the file descriptor position
**              of the memory source.
**
** \return
**          - #FAPI_OK if the file is a PNG.
**          - #DECPNG_ERR_BAD_PARAMETER if a NULL pointer is passed.
**          - #DECPNG_ERR_DATA_SOURCE if the read function did not return
**              any data on request.
**          - #PICDEC_ERR_FORMAT_NOT_SUPPORTED otherwise.
**
** \sa      See #PICDEC_INTERFACE_DriverT specifications.
*******************************************************************************
**/
int32_t DECPNG_CheckFile (void* filePtr,
                          int32_t fileOffset,
                          PICDEC_ReadFileChunkT readFptr,
                          PICDEC_FileSeekT seekFptr)
{
    int32_t  result = 0;
    uint32_t n_read = 0;
    uint8_t sig_arr[DECPNG_SIGNATURE_BYTES];

    memset(sig_arr, 0, sizeof(sig_arr));

    if((filePtr == NULL)
    || (readFptr == NULL)
    || (seekFptr == NULL))
    {
        return(DECPNG_ERR_BAD_PARAMETER);
    }

    // rewind memory source:
    result = seekFptr(filePtr, fileOffset, PICDEC_SEEK_SET);
    if (result != 0) return(DECPNG_ERR_DATA_SOURCE);

    // read in signature bytes:
    n_read = readFptr(sig_arr, filePtr, DECPNG_SIGNATURE_BYTES);
    if(n_read == 0)
    {
        return(DECPNG_ERR_DATA_SOURCE);
    }
    if(png_sig_cmp(sig_arr, 0, n_read) != 0)
    {
        return(PICDEC_ERR_FORMAT_NOT_SUPPORTED);
    }
    return(FAPI_OK);
}

/*!
*******************************************************************************
** \brief   Gets information about a PNG picture.
**
** \param   filePtr
**              Descriptor of file containing the picture data.
** \param   fileOffset
**              Picture data offset within the file.
** \param   readFptr
**              This function reads data chunks from filePtr.
** \param   seekFptr
**              This function sets the file descriptor position
**              of the memory source.
** \param   infoPtr
**              Will receive the requested information about the file.
**
** \return
**          - #FAPI_OK on success.
**          - #DECPNG_ERR_BAD_PARAMETER if a NULL pointer is passed.
**          - #DECPNG_ERR_DATA_SOURCE if the read function did not return
**              any data on request.
**          - #PICDEC_ERR_FORMAT_NOT_SUPPORTED if the PNG signature does not match.
**          - #DECPNG_ERR_PNG_INIT
**              if one of the internal PNG structures could not be created.
**
**
** \sa      See #PICDEC_INTERFACE_DriverT specifications.
*******************************************************************************
*/
int32_t DECPNG_GetInfo (void* filePtr,
                        int32_t fileOffset,
                        PICDEC_ReadFileChunkT readFptr,
                        PICDEC_FileSeekT seekFptr,
                        PICDEC_RotationT rotation,
                        PICDEC_InfoT* infoPtr)
{
    decpngMemorySourceT mem_src;
    uint8_t     sig_arr[DECPNG_SIGNATURE_BYTES];
    png_structp png_ptr         = NULL; // internal PNG structure
    png_infop   info_ptr        = NULL; // PNG information
    png_infop   end_ptr         = NULL;
    uint32_t    channels        = 0;
    uint32_t    bit_depth       = 0;
    png_colorp  png_palette_ptr = NULL;
    int32_t     palette_size    = 0;
    png_bytep   trans_alpha     = 0;
    int         num_trans       = 0;
    png_color_16p trans_color   = 0;
    int32_t     ii              = 0;
    int32_t     result          = 0;

    memset(&mem_src, 0, sizeof(decpngMemorySourceT));
    memset(sig_arr, 0, sizeof(sig_arr));

    if((filePtr == NULL)
    || (readFptr == NULL)
    || (seekFptr == NULL)
    || (infoPtr == NULL))
    {
        return(DECPNG_ERR_BAD_PARAMETER);
    }
    FAPI_SYS_PRINT_DEBUG(1,"PNG get info...\n" );
    // rewind memory source:
    result = seekFptr(filePtr, fileOffset, PICDEC_SEEK_SET);
    if (result != 0) return(DECPNG_ERR_DATA_SOURCE);

    // read in signature bytes:
    ii = readFptr(sig_arr, filePtr, DECPNG_SIGNATURE_BYTES);
    if(ii == 0)
    {
        return(DECPNG_ERR_DATA_SOURCE);
    }
    if(png_sig_cmp(sig_arr, 0, ii) != 0)
    {
        return(PICDEC_ERR_FORMAT_NOT_SUPPORTED);
    }

    // create internal png structure and assign FAPI specific
    // heap memory management and error and warning reporting functions
    png_ptr = png_create_read_struct_2
        (PNG_LIBPNG_VER_STRING, // version string
         NULL,                  // user_error_ptr
         decpngError,           // user_error_fn
         decpngWarning,         // user_warning_fn
         NULL,                  // user_mem_ptr
         decpngMalloc,          // user_malloc_fn
         decpngFree);           // user_free_fn
    if (!png_ptr)
    {
        return (DECPNG_ERR_PNG_INIT);
    }

    // create info structure:
    info_ptr = png_create_info_struct(png_ptr);
    if (!info_ptr)
    {
        png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
        return (DECPNG_ERR_PNG_INIT);
    }

    end_ptr = png_create_info_struct(png_ptr);
    if (!end_ptr)
    {
        png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
        return (DECPNG_ERR_PNG_INIT);
    }

    if (setjmp(png_jmpbuf(png_ptr)))
    {
        png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
        return (DECPNG_ERR_PNG_DECODE);
    }

    png_set_sig_bytes(png_ptr, ii);

    mem_src.readFptr = readFptr;
    mem_src.fileHandlePtr = filePtr;
    png_set_read_fn(png_ptr, &mem_src, decpngReadData);
    png_read_info(png_ptr, info_ptr);

    // write picture format info:
    infoPtr->format = PICDEC_FORMAT_PNG;

    // get the bpp (bits per pixel) info:
    bit_depth = png_get_bit_depth(png_ptr, info_ptr);
    if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_PALETTE)
    {
        channels = 1;
    }
    else if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB)
    {
        channels = 3;
    }
    else if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB_ALPHA)
    {
        channels = 4;
    }
    else if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY)
    {
        channels = 1;
    }
    else if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY_ALPHA)
    {
        channels = 2;
    }
    else
    {
        channels = 0;
    }
    infoPtr->bpp = bit_depth * channels;

    // get the dimensions:
    switch(rotation)
    {
        case(PICDEC_ROTATION_90_LEFT):
        case(PICDEC_ROTATION_90_RIGHT):
        infoPtr->xRes = png_get_image_height(png_ptr, info_ptr);
        infoPtr->yRes = png_get_image_width(png_ptr, info_ptr);
        break;

        case(PICDEC_ROTATION_0):
        case(PICDEC_ROTATION_180):
        default:
        infoPtr->xRes = png_get_image_width(png_ptr, info_ptr);
        infoPtr->yRes = png_get_image_height(png_ptr, info_ptr);
        break;
    }

    // get the CLUT:
    if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_PALETTE)
    {
        png_get_PLTE(png_ptr, info_ptr, &png_palette_ptr, (int*)(void*)&palette_size);
        if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
        {
            png_get_tRNS(png_ptr, info_ptr, &trans_alpha,
                         &num_trans, &trans_color);
            for(ii = 0; ii < palette_size; ii++)
            {
                infoPtr->clutArray[ii] =
                        (((png_palette_ptr + ii)->red)   << 24) |
                        (((png_palette_ptr + ii)->green) << 16) |
                        (((png_palette_ptr + ii)->blue)  <<  8) |
                        ((ii < num_trans) ? *(trans_alpha + ii) : 0xFFUL);
            }
        }
        else
        {
            for(ii = 0; ii < palette_size; ii++)
            {
                infoPtr->clutArray[ii] =
                        (((png_palette_ptr + ii)->red)   << 24) |
                        (((png_palette_ptr + ii)->green) << 16) |
                        (((png_palette_ptr + ii)->blue)  <<  8) |
                        0x000000FFUL;
            }
        }
    }
    else
    {
        memset(infoPtr->clutArray, 0, sizeof(infoPtr->clutArray));
    }

    // cleanup:
    png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);

    return(FAPI_OK);
}

/*!
*******************************************************************************
** \brief   Decodes a PNG picture into a frame buffer.
**
** \param   filePtr
**              Descriptor of file containing the picture data.
** \param   fileOffset
**              Picture data offset within the file.
** \param   readFptr
**              This function reads data chunks from filePtr.
** \param   seekFptr
**              This function sets the file descriptor position
**              of the memory source.
** \param   fbHandle
**              Handle for the frame buffer that will be modified.
** \param   xx
**              Horizontal coordinate of the picture in the frame buffer.
**              This coordinate is relative to the frame buffer, i.e., if
**              the picture is rotated, the xx and yy coordinates denote the
**              top left corner of the rotated picture.
** \param   yy
**              Vertical coordinate of the picture in the frame buffer.
**              This coordinate is relative to the frame buffer, i.e., if
**              the picture is rotated, the xx and yy coordinates denote the
**              top left corner of the rotated picture.
** \param   xRes
**              Desired horizontal resolution of the picture.
**              The resolution is achieved by software scaling.
**              If xRes is 0, the native horizontal resolution
**              of the file will be used.
** \param   yRes
**              Desired vertical resolution of the picture.
**              The resolution is achieved by software scaling.
**              If yRes is 0, the native vertical resolution
**              of the file will be used.
** \param   rotation
**              Defines a rotation of the picture in steps of
**              90 degrees.
** \param   scaler
**              Specifies the prefereed scaler algorithm to be
**              used while decoding. This option only applies
**              if the xRes or yRes parameter differs from the
**              native picture size.
** \param   progressCb
**              Optional progress and abort callback.
** \param   progressCbArg
**              Optional argument passed with progressCb.
**
** \return
**          - #FAPI_OK on success.
**          - #DECPNG_ERR_BAD_PARAMETER if a NULL pointer is passed.
**          - #DECPNG_ERR_DATA_SOURCE if the read function did not return
**              any data on request.
**          - #PICDEC_ERR_FORMAT_NOT_SUPPORTED if the PNG signature does not match.
**          - #DECPNG_ERR_PNG_INIT
**              if one of the internal PNG structures could not be created.
**          - #DECPNG_ERR_PNG_DECODE
**              if an error occured while decoding the PNG.
**          - #DECPNG_ERR_OUT_OF_FB_BOUNDS
**              if the picture would be decoded out of bounds of the
**              frame buffer.
**          - #DECPNG_ERR_OUT_OF_MEMORY
**              if there is not enough heap memory available for setting up
**              the standard PNG decoding.
**          - #DECPNG_ERR_CCIR_LAYOUT
**              if the layout mode does noes not match for the colour
**              mode when decoding for CCIR OSD.
**          - #DECPNG_ERR_COLOUR_MODE_MISMATCH
**              if the frame buffer is driven in a CLUT colour mode but the
**              PNG is not an indexed colour mode or if the indexed colour mode
**              of the PNG has more bits per pixel than the OSD colour mode.
**          - some OSDHANDLER specific error code if the fbHandle is bad.
**
*******************************************************************************
*/
int32_t DECPNG_Decode(void* filePtr,
                      int32_t fileOffset,
                      PICDEC_ReadFileChunkT readFptr,
                      PICDEC_FileSeekT seekFptr,
                      OSDHANDLER_FB_HandleT fbHandle,
                      uint32_t xx,
                      uint32_t yy,
                      uint32_t xRes,
                      uint32_t yRes,
                      PICDEC_RotationT rotation,
                      PICDEC_ScalerT scaler,
                      PICDEC_ProgressCbT progressCb,
                      void* progressCbArg)
{
    decpngMemorySourceT mem_src;
    decpngPicbufT       picbuf;
    uint8_t     sig_arr[DECPNG_SIGNATURE_BYTES];
    OSDHANDLER_FB_FrameBufferT fb_info;
    PICSCALER_DecodeParamsT scaler_params;
    PICDEC_DecodeInfoT decode_info;
    int32_t     result              = 0;    // error codes
    png_structp png_ptr             = NULL; // internal PNG structure
    png_infop   info_ptr            = NULL; // PNG information
    png_infop   end_ptr             = NULL; // PNG information
    uint32_t    png_width           = 0;    // PNG dimensions
    uint32_t    png_height          = 0;    // PNG dimensions
    uint32_t*   fb_32_ptr           = NULL; // framebuffer base address
    uint32_t    fb_offset           = 0;    // framebuffer pixel offset
    uint32_t    ii                  = 0;    // iterator
    uint32_t    kk                  = 0;    // iterator
    uint32_t*   png_rowbuf_ptr      = NULL; // temporary buffer for PNG data row 1
    uint32_t*   png_buf_read_32_ptr = NULL; // pointer used for reading the buffer
    uint8_t*    png_buf_read_8_ptr  = NULL; // pointer used for reading the buffer
    uint32_t    colour_mask         = 0;
    uint32_t    data_mask           = 0;
    uint32_t    tmp1                = 0; // temporary variable
    uint32_t    tmp2                = 0; // temporary variable
    png_colorp  png_palette_ptr     = NULL; // palette in PNG format
    int32_t     palette_size        = 0;    // size of PNG palette
    uint32_t*   fb_palette_ptr      = NULL; // palette in frame buffer format
    uint32_t    number_of_passes    = 0;    // number of passes for interlaced PNG
    uint32_t    current_pass        = 0;    // current pass for interlaced PNG


    memset(&mem_src, 0, sizeof(decpngMemorySourceT));
    memset(&picbuf, 0, sizeof(decpngPicbufT));
    memset(sig_arr, 0, sizeof(sig_arr));
    memset(&fb_info, 0, sizeof(OSDHANDLER_FB_FrameBufferT));
    memset(&scaler_params, 0, sizeof(PICSCALER_DecodeParamsT));

    if((filePtr == NULL)
    || (readFptr == NULL)
    || (seekFptr == NULL))
    {
        return(DECPNG_ERR_BAD_PARAMETER);
    }

    // rewind memory source:
    result = seekFptr(filePtr, fileOffset, PICDEC_SEEK_SET);
    if (result != 0) return(DECPNG_ERR_DATA_SOURCE);

    // read in signature bytes:
    ii = readFptr(sig_arr, filePtr, DECPNG_SIGNATURE_BYTES);
    if(ii == 0)
    {
        return(DECPNG_ERR_DATA_SOURCE);
    }
    if(png_sig_cmp(sig_arr, 0, ii) != 0)
    {
        return(PICDEC_ERR_FORMAT_NOT_SUPPORTED);
    }

    // create internal png structure and assign FAPI specific
    // heap memory management and error and warning reporting functions
    png_ptr = png_create_read_struct_2
        (PNG_LIBPNG_VER_STRING, // version string
         NULL,                  // user_error_ptr
         decpngError,           // user_error_fn
         decpngWarning,         // user_warning_fn
         NULL,                  // user_mem_ptr
         decpngMalloc,          // user_malloc_fn
         decpngFree);           // user_free_fn
    if (!png_ptr)
    {
        return (DECPNG_ERR_PNG_INIT);
    }

    // create info structure:
    info_ptr = png_create_info_struct(png_ptr);
    if (!info_ptr)
    {
        png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
        return (DECPNG_ERR_PNG_INIT);
    }

    end_ptr = png_create_info_struct(png_ptr);
    if (!end_ptr)
    {
        png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
        return (DECPNG_ERR_PNG_INIT);
    }

    if (setjmp(png_jmpbuf(png_ptr)))
    {
        png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
        return (DECPNG_ERR_PNG_DECODE);
    }

    png_set_sig_bytes(png_ptr, ii);

    mem_src.readFptr = readFptr;
    mem_src.fileHandlePtr = filePtr;
    png_set_read_fn(png_ptr, &mem_src, decpngReadData);
    png_read_info(png_ptr, info_ptr);

    // let libpng handle expansion of interlaced image:
    number_of_passes = png_set_interlace_handling(png_ptr);

    // This may occasionally cause an BGET error on
    // png_destroy_read_struct() calls. --RK
    // png_read_update_info(png_ptr, info_ptr);

    // get dimensions:
    png_width  = png_get_image_width(png_ptr, info_ptr);
    png_height = png_get_image_height(png_ptr, info_ptr);

    // Internally, resolution is oriented according to picture orientation:
    if((rotation == PICDEC_ROTATION_90_LEFT)
    || (rotation == PICDEC_ROTATION_90_RIGHT))
    {
        tmp1 = xRes;
        xRes = yRes;
        yRes = tmp1;
    }
    if(xRes == 0)
    {
        xRes = png_width;
    }
    if(yRes == 0)
    {
        yRes = png_height;
    }

    // get frame buffer parameters and check dimensions:
    result = OSDHANDLER_FB_GetInfo(fbHandle, &fb_info);
    if(result)
    {
        png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
        return(result);
    }

    // Initialize decoder info structure
    memset (&decode_info, 0, sizeof(decode_info));
    decode_info.totalPasses = number_of_passes;
    decode_info.totalLines = png_height;

    //  ************************************************************
    //  ************ SCALED/ROTATED PNG DECODING *******************
    //  ************************************************************

    if ((xRes != png_width)
    ||  (yRes != png_height)
    ||  (rotation != PICDEC_ROTATION_0)
    ||  (number_of_passes > 1))
    {
        // use simple BRESENHAM if nothing to scale:
        if((xRes == png_width) && (yRes == png_height))
        {
            scaler = PICDEC_SCALER_BRESENHAM;
        }

        // Set up the PNG library to decode in RGBA 32-bit format:
        if (png_get_bit_depth(png_ptr, info_ptr) == 16)
        {
            png_set_strip_16(png_ptr); // limit to 8 bit per channel
        }
        if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_PALETTE)
        {
            png_set_packing(png_ptr);
            png_set_palette_to_rgb(png_ptr);
            png_set_bgr(png_ptr);
            if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
            {
                png_set_tRNS_to_alpha(png_ptr);
                png_set_swap_alpha(png_ptr);
            }
            else
            {
                png_set_filler(png_ptr, 0xFF, PNG_FILLER_BEFORE);
            }
        }
        if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB)
        {
            png_set_packing(png_ptr);
            png_set_bgr(png_ptr);
            if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
            {
                png_set_tRNS_to_alpha(png_ptr);
                png_set_swap_alpha(png_ptr);
            }
            else
            {
                png_set_filler(png_ptr, 0xFF, PNG_FILLER_BEFORE);
            }
        }
        if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB_ALPHA)
        {
            png_set_packing(png_ptr);
            png_set_bgr(png_ptr);
            png_set_swap_alpha(png_ptr);
        }
        // expand gray scale images to RGB
        if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY)
        {
            png_set_expand_gray_1_2_4_to_8(png_ptr);
            png_set_gray_to_rgb(png_ptr);
            png_set_bgr(png_ptr);
            if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
            {
                png_set_tRNS_to_alpha(png_ptr);
                png_set_swap_alpha(png_ptr);
            }
            else
            {
                png_set_filler(png_ptr, 0xFF, PNG_FILLER_BEFORE);
            }
        }
        if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY_ALPHA)
        {
            png_set_expand_gray_1_2_4_to_8(png_ptr);
            png_set_gray_to_rgb(png_ptr);
            png_set_bgr(png_ptr);
            png_set_swap_alpha(png_ptr);
        }

        // Multi-pass decoding of interlaced PNG:
        if(number_of_passes > 1)
        {
            // each line is processed twice: once by the decoder and once by the scaler
            decode_info.totalLines = 2 * png_height;

            // allocate temporary buffer for image
            picbuf.bufPtr = FAPI_SYS_MALLOC_CACHED(png_width * png_height * sizeof(uint32_t));
            if(picbuf.bufPtr == NULL)
            {
                png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
                return(DECPNG_ERR_OUT_OF_MEMORY);
            }
            for(current_pass = 0; current_pass < number_of_passes; current_pass++)
            {
                decode_info.finishedPasses = current_pass;
                for(ii = 0; ii < png_height; ii++)
                {
                    if ( progressCb != NULL )
                    {
                        decode_info.decodedLines = ii;

                        /* Give progress and test for abort condition. */
                        if ( progressCb (&decode_info, progressCbArg) != 0 )
                        {
                            FAPI_SYS_FREE(picbuf.bufPtr);
                            png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
                            return(PICDEC_ERR_ABORT_REQUEST);
                        }
                    }

                    fb_32_ptr = picbuf.bufPtr + (ii * png_width);

                    // "sparkle" effect:
                    //png_read_row(png_ptr, (png_bytep)fb_32_ptr, NULL);

                    // "rectangle" effect:
                    png_read_row(png_ptr, NULL, (png_bytep)fb_32_ptr);
                }

                picbuf.width = png_width;
                picbuf.height = png_height;
                picbuf.currentLine = 0;

                // Call scaler for each pass:
                scaler_params.fbHandle      = fbHandle;
                scaler_params.srcWidth      = png_width;
                scaler_params.srcHeight     = png_height;
                scaler_params.dstPosX       = xx;
                scaler_params.dstPosY       = yy;
                scaler_params.dstWidth      = xRes;
                scaler_params.dstHeight     = yRes;
                scaler_params.bufHeight     = 1;
                scaler_params.rotation      = rotation;
                scaler_params.scaler        = scaler;
                scaler_params.readLineFptr  = decpngScalerReadPicbuf;
                scaler_params.skipLineFptr  = decpngScalerSkipPicbuf;
                scaler_params.optArgPtr     = (void*)&picbuf;
                scaler_params.progressCb    = progressCb;
                scaler_params.progressCbArg = progressCbArg;
                scaler_params.decodeInfoPtr = &decode_info;

                result = PICSCALER_Decode(&scaler_params);
                if(result != FAPI_OK)
                {
                    FAPI_SYS_FREE(picbuf.bufPtr);
                    png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
                    return(result);
                }
            }
            FAPI_SYS_FREE(picbuf.bufPtr);
            png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
            return(result);
        }
        // Single-pass decoding of progressive PNG:
        else
        {
            scaler_params.fbHandle      = fbHandle;
            scaler_params.srcWidth      = png_width;
            scaler_params.srcHeight     = png_height;
            scaler_params.dstPosX       = xx;
            scaler_params.dstPosY       = yy;
            scaler_params.dstWidth      = xRes;
            scaler_params.dstHeight     = yRes;
            scaler_params.bufHeight     = 1;
            scaler_params.rotation      = rotation;
            scaler_params.scaler        = scaler;
            scaler_params.readLineFptr  = decpngScalerReadLine;
            scaler_params.skipLineFptr  = decpngScalerSkipLine;
            scaler_params.optArgPtr     = (void*)png_ptr;
            scaler_params.progressCb    = progressCb;
            scaler_params.progressCbArg = progressCbArg;
            scaler_params.decodeInfoPtr = &decode_info;

            result = PICSCALER_Decode(&scaler_params);
            png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
            return(result);
        }
    }

    //  ************************************************************
    //  *************** STANDARD PNG DECODING **********************
    //  ************************************************************

    /*
    **  The code below decodes PNGs in many different formats
    **  to many different formatted frame buffers.
    **  Generally it is possible to decode any PNG to a frame
    **  buffer running in OSDHANDLER_OSD_MODE_32_24BIT,
    **  OSDHANDLER_OSD_MODE_5551 and OSDHANDLER_OSD_MODE_4444
    **  in any layout mode as well as to those frame buffers running in
    **  OSDHANDLER_OSD_MODE_16BIT with OSDHANDLER_OSD_LAYOUT_CbYCrA (CCIR).
    **  The PNGs with indexed colour modes must match the bit depth
    **  of the frame buffer, all layout modes are supported here
    **  as well.
    */

    if(((xx + png_width) > fb_info.xResApp)
     ||((yy + png_height) > fb_info.yResApp))
    {
        png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
        return(DECPNG_ERR_OUT_OF_FB_BOUNDS);
    }
    // Switch the frame buffer colour modes:
    if(fb_info.colourMode == OSDHANDLER_OSD_MODE_32_24BIT)
    {
        if((fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_ARGB)
        || (fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_ACbYCr))
        {
            if (png_get_bit_depth(png_ptr, info_ptr) == 16)
            {
                png_set_strip_16(png_ptr); // limit to 8 bit per channel
            }

            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_PALETTE)
            {
                png_set_packing(png_ptr);
                png_set_palette_to_rgb(png_ptr);
                png_set_bgr(png_ptr);
                if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
                {
                    png_set_tRNS_to_alpha(png_ptr);
                }
                else
                {
                    png_set_filler(png_ptr, 0xFF, PNG_FILLER_AFTER);
                }
            }

            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB)
            {
                png_set_packing(png_ptr);
                png_set_bgr(png_ptr);
                if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
                {
                    png_set_tRNS_to_alpha(png_ptr);
                }
                else
                {
                    png_set_filler(png_ptr, 0xFF, PNG_FILLER_AFTER);
                }
            }

            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB_ALPHA)
            {
                png_set_packing(png_ptr);
                png_set_bgr(png_ptr);
            }

            // expand gray scale images to RGB
            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY)
            {
                png_set_expand_gray_1_2_4_to_8(png_ptr);
                png_set_gray_to_rgb(png_ptr);
                png_set_bgr(png_ptr);
                if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
                {
                    png_set_tRNS_to_alpha(png_ptr);
                }
                else
                {
                    png_set_filler(png_ptr, 0xFF, PNG_FILLER_AFTER);
                }
            }

            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY_ALPHA)
            {
                png_set_expand_gray_1_2_4_to_8(png_ptr);
                png_set_gray_to_rgb(png_ptr);
                png_set_bgr(png_ptr);
            }

            if(fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_ACbYCr)
            {
                png_set_read_user_transform_fn(png_ptr, decpngTransformARGB2ACbYCr);
            }
        }
        else if((fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_RGBA)
             || (fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_CbYCrA))
        {
            if (png_get_bit_depth(png_ptr, info_ptr) == 16)
            {
                png_set_strip_16(png_ptr); // limit to 8 bit per channel
            }

            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_PALETTE)
            {
                png_set_packing(png_ptr);
                png_set_palette_to_rgb(png_ptr);
                png_set_bgr(png_ptr);
                if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
                {
                    png_set_tRNS_to_alpha(png_ptr);
                    png_set_swap_alpha(png_ptr);
                }
                else
                {
                    png_set_filler(png_ptr, 0xFF, PNG_FILLER_BEFORE);
                }
            }

            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB)
            {
                png_set_packing(png_ptr);
                png_set_bgr(png_ptr);
                if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
                {
                    png_set_tRNS_to_alpha(png_ptr);
                    png_set_swap_alpha(png_ptr);
                }
                else
                {
                    png_set_filler(png_ptr, 0xFF, PNG_FILLER_BEFORE);
                }
            }

            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB_ALPHA)
            {
                png_set_packing(png_ptr);
                png_set_bgr(png_ptr);
                png_set_swap_alpha(png_ptr);
            }

            // expand gray scale images to RGB
            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY)
            {
                png_set_expand_gray_1_2_4_to_8(png_ptr);
                png_set_gray_to_rgb(png_ptr);
                png_set_bgr(png_ptr);
                if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
                {
                    png_set_tRNS_to_alpha(png_ptr);
                    png_set_swap_alpha(png_ptr);
                }
                else
                {
                    png_set_filler(png_ptr, 0xFF, PNG_FILLER_BEFORE);
                }
            }

            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY_ALPHA)
            {
                png_set_expand_gray_1_2_4_to_8(png_ptr);
                png_set_gray_to_rgb(png_ptr);
                png_set_bgr(png_ptr);
                png_set_swap_alpha(png_ptr);
            }
            if(fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_CbYCrA)
            {
                png_set_read_user_transform_fn(png_ptr, decpngTransformRGBA2CbYCrA);
            }
        }

        // Decode picture:
        for(ii = 0; ii < png_height; ii++)
        {
            if ( progressCb != NULL )
            {
                decode_info.decodedLines = ii;

                /* Give progress and test for abort condition. */
                if ( progressCb (&decode_info, progressCbArg) != 0 )
                {
                    png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
                    return(PICDEC_ERR_ABORT_REQUEST);
                }
            }

            OSDHANDLER_FB_GetPixelAddress(fbHandle, xx, yy + ii, &fb_32_ptr, &fb_offset);
            png_read_row(png_ptr, (png_bytep)fb_32_ptr, NULL);
        }

        // single command decode (not applicable with abort function):
        //row_ptr_ptr = FAPI_SYS_MALLOC_CACHED(png_height * sizeof(uint32_t*));
        //if(!row_ptr_ptr)
        //{
        //    png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
        //    return(DECPNG_ERR_OUT_OF_MEMORY);
        //}
        //for(ii = 0; ii < png_height; ii++)
        //{
        //    OSDHANDLER_FB_GetPixelAddress(fbHandle, xx, yy + ii, &fb_32_ptr, &fb_offset);
        //    row_ptr_ptr[ii] = (uint8_t*)fb_32_ptr;
        //}
        //png_set_rows(png_ptr, info_ptr, row_ptr_ptr);
        //png_read_update_info(png_ptr, info_ptr);
        //png_read_image(png_ptr, row_ptr_ptr);
        //FAPI_SYS_FREE(row_ptr_ptr);

    }
    else if(fb_info.colourMode == OSDHANDLER_OSD_MODE_16BIT) // CCIR mode
    {
        if(fb_info.layoutMode != OSDHANDLER_OSD_LAYOUT_CbYCrA)
        {
            // CCIR mode is only supported for CbYCrA layout which
            // actually results in a CbY1CrY2 layout that represents
            // two pixels with same Cb and Cr values in a 32-bit word.
            png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
            return(DECPNG_ERR_CCIR_LAYOUT);
        }
        // first set up the PNG library to present its data in 32-bit RGBA format
        if (png_get_bit_depth(png_ptr, info_ptr) == 16)
        {
            png_set_strip_16(png_ptr); // limit to 8 bit per channel
        }

        if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_PALETTE)
        {
            png_set_packing(png_ptr);
            png_set_palette_to_rgb(png_ptr);
            png_set_bgr(png_ptr);
            png_set_filler(png_ptr, 0xFF, PNG_FILLER_BEFORE);
        }

        if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB)
        {
            png_set_packing(png_ptr);
            png_set_bgr(png_ptr);
            png_set_filler(png_ptr, 0xFF, PNG_FILLER_BEFORE);
        }

        if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB_ALPHA)
        {
            png_set_packing(png_ptr);
            png_set_bgr(png_ptr);
            png_set_swap_alpha(png_ptr);
        }

        // expand gray scale images to RGB
        if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY)
        {
            png_set_expand_gray_1_2_4_to_8(png_ptr);
            png_set_gray_to_rgb(png_ptr);
            png_set_bgr(png_ptr);
            png_set_filler(png_ptr, 0xFF, PNG_FILLER_BEFORE);
        }

        if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY_ALPHA)
        {
            png_set_expand_gray_1_2_4_to_8(png_ptr);
            png_set_gray_to_rgb(png_ptr);
            png_set_bgr(png_ptr);
            png_set_swap_alpha(png_ptr);
        }

        // set up a buffer that holds a single PNG row
        png_rowbuf_ptr = FAPI_SYS_MALLOC_CACHED(png_width * sizeof(uint32_t));
        if(png_rowbuf_ptr == NULL)
        {
            png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
            return(DECPNG_ERR_OUT_OF_MEMORY);
        }

        for(ii = 0; ii < png_height; ii++)
        {
            if ( progressCb != NULL )
            {
                decode_info.decodedLines = ii;

                /* Give progress and test for abort condition. */
                if ( progressCb (&decode_info, progressCbArg) != 0 )
                {
                    FAPI_SYS_FREE(png_rowbuf_ptr);
                    png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
                    return(PICDEC_ERR_ABORT_REQUEST);
                }
            }

            kk = png_width;
            // for each row copy PNG data into the buffer in ARGB format
            png_read_row(png_ptr, (png_bytep)png_rowbuf_ptr, NULL);
            png_buf_read_32_ptr = png_rowbuf_ptr;

            // use fb_32_ptr pointer to iterate the 32-bit CCIR values:
            OSDHANDLER_FB_GetPixelAddress(fbHandle, xx, yy + ii, &fb_32_ptr, &fb_offset);

            // first write a single pixel if xx is odd:
            if(xx % 2)
            {
                OSDHANDLER_SetRightRGBA2CCIR(fb_32_ptr, *png_buf_read_32_ptr++);
                fb_32_ptr++;
                kk--;
            }
            kk >>= 1;
            while (kk--)
            {
                *fb_32_ptr++ =
                    OSDHANDLER_ConvertRGBA2CCIR(*png_buf_read_32_ptr, *(png_buf_read_32_ptr+1));
                png_buf_read_32_ptr += 2;
            }
            // write out the last pixel individually if necessary:
            if((((xx % 2) == 0) && (png_width % 2))
            || ((xx % 2) && ((png_width % 2) == 0)))
            {
                OSDHANDLER_SetLeftRGBA2CCIR(fb_32_ptr, *png_buf_read_32_ptr);
            }
        }
        FAPI_SYS_FREE(png_rowbuf_ptr);
    }
    else if((fb_info.colourMode == OSDHANDLER_OSD_MODE_5551)
    ||      (fb_info.colourMode == OSDHANDLER_OSD_MODE_4444))
    {
        if((fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_ARGB)
        || (fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_ACbYCr))
        {
            if (png_get_bit_depth(png_ptr, info_ptr) == 16)
            {
                png_set_strip_16(png_ptr); // limit to 8 bit per channel
            }

            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_PALETTE)
            {
                png_set_packing(png_ptr);
                png_set_palette_to_rgb(png_ptr);
                png_set_bgr(png_ptr);
                if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
                {
                    png_set_tRNS_to_alpha(png_ptr);
                }
                else
                {
                    png_set_filler(png_ptr, 0xFF, PNG_FILLER_AFTER);
                }
            }

            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB)
            {
                png_set_packing(png_ptr);
                png_set_bgr(png_ptr);
                if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
                {
                    png_set_tRNS_to_alpha(png_ptr);
                }
                else
                {
                    png_set_filler(png_ptr, 0xFF, PNG_FILLER_AFTER);
                }
            }

            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB_ALPHA)
            {
                png_set_packing(png_ptr);
                png_set_bgr(png_ptr);
            }

            // expand gray scale images to RGB
            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY)
            {
                png_set_expand_gray_1_2_4_to_8(png_ptr);
                png_set_gray_to_rgb(png_ptr);
                png_set_bgr(png_ptr);
                if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
                {
                    png_set_tRNS_to_alpha(png_ptr);
                }
                else
                {
                    png_set_filler(png_ptr, 0xFF, PNG_FILLER_AFTER);
                }
            }

            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY_ALPHA)
            {
                png_set_expand_gray_1_2_4_to_8(png_ptr);
                png_set_gray_to_rgb(png_ptr);
                png_set_bgr(png_ptr);
            }
            if(fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_ACbYCr)
            {
                png_set_read_user_transform_fn(png_ptr, decpngTransformARGB2ACbYCr);
            }
        }
        else if((fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_RGBA)
             || (fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_CbYCrA))
        {
            if (png_get_bit_depth(png_ptr, info_ptr) == 16)
            {
                png_set_strip_16(png_ptr); // limit to 8 bit per channel
            }

            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_PALETTE)
            {
                png_set_packing(png_ptr);
                png_set_palette_to_rgb(png_ptr);
                png_set_bgr(png_ptr);
                if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
                {
                    png_set_tRNS_to_alpha(png_ptr);
                    png_set_swap_alpha(png_ptr);
                }
                else
                {
                    png_set_filler(png_ptr, 0xFF, PNG_FILLER_BEFORE);
                }
            }

            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB)
            {
                png_set_packing(png_ptr);
                png_set_bgr(png_ptr);
                if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
                {
                    png_set_tRNS_to_alpha(png_ptr);
                    png_set_swap_alpha(png_ptr);
                }
                else
                {
                    png_set_filler(png_ptr, 0xFF, PNG_FILLER_BEFORE);
                }
            }

            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB_ALPHA)
            {
                png_set_packing(png_ptr);
                png_set_bgr(png_ptr);
                png_set_swap_alpha(png_ptr);
            }

            // expand gray scale images to RGB
            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY)
            {
                png_set_expand_gray_1_2_4_to_8(png_ptr);
                png_set_gray_to_rgb(png_ptr);
                png_set_bgr(png_ptr);
                if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
                {
                    png_set_tRNS_to_alpha(png_ptr);
                    png_set_swap_alpha(png_ptr);
                }
                else
                {
                    png_set_filler(png_ptr, 0xFF, PNG_FILLER_BEFORE);
                }
            }

            if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY_ALPHA)
            {
                png_set_expand_gray_1_2_4_to_8(png_ptr);
                png_set_gray_to_rgb(png_ptr);
                png_set_bgr(png_ptr);
                png_set_swap_alpha(png_ptr);
            }
            if(fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_CbYCrA)
            {
                png_set_read_user_transform_fn(png_ptr, decpngTransformRGBA2CbYCrA);
            }
        }
        // set up a buffer that holds a single PNG row
        png_rowbuf_ptr = FAPI_SYS_MALLOC_CACHED(png_width * sizeof(uint32_t));
        if(png_rowbuf_ptr == NULL)
        {
            png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
            return(DECPNG_ERR_OUT_OF_MEMORY);
        }
        for(ii = 0; ii < png_height; ii++)
        {
            if ( progressCb != NULL )
            {
                decode_info.decodedLines = ii;

                /* Give progress and test for abort condition. */
                if ( progressCb (&decode_info, progressCbArg) != 0 )
                {
                    FAPI_SYS_FREE(png_rowbuf_ptr);
                    png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
                    return(PICDEC_ERR_ABORT_REQUEST);
                }
            }

            kk = png_width;
            // for each row copy PNG data into the buffer in ARGB format
            png_read_row(png_ptr, (png_bytep)png_rowbuf_ptr, NULL);
            png_buf_read_32_ptr = png_rowbuf_ptr;
            OSDHANDLER_FB_GetPixelAddress(fbHandle, xx, yy + ii, &fb_32_ptr, &fb_offset);
            if(fb_info.colourMode == OSDHANDLER_OSD_MODE_5551)
            {
                if((fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_ARGB)
                || (fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_ACbYCr))
                {
                    if (xx % 2)
                    {
                        *fb_32_ptr &= 0xFFFF0000UL;
                        *fb_32_ptr |=
                            ((*png_buf_read_32_ptr & 0xFF000000UL) ? (1UL << 15) : 0) |
                            ((*png_buf_read_32_ptr & 0x00F80000UL) >> 9) |
                            ((*png_buf_read_32_ptr & 0x0000F800UL) >> 6) |
                            ((*png_buf_read_32_ptr & 0x000000F8UL) >> 3);
                        kk--;
                        png_buf_read_32_ptr += 1;
                        fb_32_ptr++;
                    }
                    kk >>= 1;
                    while(kk--)
                    {
                        *fb_32_ptr =
                            ((*png_buf_read_32_ptr & 0xFF000000UL) ? (1UL << 31) : 0) |
                            ((*png_buf_read_32_ptr & 0x00F80000UL) <<  7) |
                            ((*png_buf_read_32_ptr & 0x0000F800UL) << 10) |
                            ((*png_buf_read_32_ptr & 0x000000F8UL) << 13) |
                            ((*(png_buf_read_32_ptr + 1) & 0xFF000000UL) ? (1UL << 15) : 0) |
                            ((*(png_buf_read_32_ptr + 1) & 0x00F80000UL) >> 9) |
                            ((*(png_buf_read_32_ptr + 1) & 0x0000F800UL) >> 6) |
                            ((*(png_buf_read_32_ptr + 1) & 0x000000F8UL) >> 3);
                        png_buf_read_32_ptr += 2;
                        fb_32_ptr++;
                    }
                    if(((xx % 2) && (!(png_width % 2)))
                    || ((!(xx % 2)) && (png_width % 2)))
                    {
                        *fb_32_ptr &= 0x0000FFFFUL;
                        *fb_32_ptr |=
                            ((*png_buf_read_32_ptr & 0xFF000000UL) ? (1UL << 31) : 0) |
                            ((*png_buf_read_32_ptr & 0x00F80000UL) <<  7) |
                            ((*png_buf_read_32_ptr & 0x0000F800UL) << 10) |
                            ((*png_buf_read_32_ptr & 0x000000F8UL) << 13);
                    }
                }
                else if((fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_RGBA)
                ||      (fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_CbYCrA))
                {
                    if (xx % 2)
                    {
                        *fb_32_ptr &= 0xFFFF0000UL;
                        *fb_32_ptr |=
                            ((*png_buf_read_32_ptr & 0xF8000000UL) >> 16) |
                            ((*png_buf_read_32_ptr & 0x00F80000UL) >> 13) |
                            ((*png_buf_read_32_ptr & 0x0000F800UL) >> 10) |
                            ((*png_buf_read_32_ptr & 0x000000FFUL) ? 1UL : 0UL);
                        kk--;
                        png_buf_read_32_ptr += 1;
                        fb_32_ptr++;
                    }
                    kk >>= 1;
                    while(kk--)
                    {
                        *fb_32_ptr =
                            ((*png_buf_read_32_ptr & 0xF8000000UL)      ) |
                            ((*png_buf_read_32_ptr & 0x00F80000UL) <<  3) |
                            ((*png_buf_read_32_ptr & 0x0000F800UL) <<  6) |
                            ((*png_buf_read_32_ptr & 0x000000FFUL) ? (1UL << 16) : 0UL) |
                            ((*(png_buf_read_32_ptr + 1) & 0xF8000000UL) >> 16) |
                            ((*(png_buf_read_32_ptr + 1) & 0x00F80000UL) >> 13) |
                            ((*(png_buf_read_32_ptr + 1) & 0x0000F800UL) >> 10) |
                            ((*(png_buf_read_32_ptr + 1) & 0x000000FFUL) ? 1UL : 0UL);
                        png_buf_read_32_ptr += 2;
                        fb_32_ptr++;
                    }
                    if(((xx % 2) && (!(png_width % 2)))
                    || ((!(xx % 2)) && (png_width % 2)))
                    {
                        *fb_32_ptr &= 0x0000FFFFUL;
                        *fb_32_ptr |=
                            ((*png_buf_read_32_ptr & 0xF8000000UL)      ) |
                            ((*png_buf_read_32_ptr & 0x00F80000UL) <<  3) |
                            ((*png_buf_read_32_ptr & 0x0000F800UL) <<  6) |
                            ((*png_buf_read_32_ptr & 0x000000FFUL) ? (1UL << 16) : 0UL);
                    }
                }
            }
            else if(fb_info.colourMode == OSDHANDLER_OSD_MODE_4444)
            {
                if (xx % 2)
                {
                    *fb_32_ptr &= 0xFFFF0000UL;
                    *fb_32_ptr |=
                        ((*png_buf_read_32_ptr & 0xF0000000UL) >> 16) |
                        ((*png_buf_read_32_ptr & 0x00F00000UL) >> 12) |
                        ((*png_buf_read_32_ptr & 0x0000F000UL) >>  8) |
                        ((*png_buf_read_32_ptr & 0x000000F0UL) >>  4);
                    kk--;
                    png_buf_read_32_ptr += 1;
                    fb_32_ptr++;
                }
                kk >>= 1;
                while(kk--)
                {
                    *fb_32_ptr =
                        ((*png_buf_read_32_ptr & 0xF0000000UL)      ) |
                        ((*png_buf_read_32_ptr & 0x00F00000UL) <<  4) |
                        ((*png_buf_read_32_ptr & 0x0000F000UL) <<  8) |
                        ((*png_buf_read_32_ptr & 0x000000F0UL) << 12) |
                        ((*(png_buf_read_32_ptr + 1) & 0xF0000000UL) >> 16) |
                        ((*(png_buf_read_32_ptr + 1) & 0x00F00000UL) >> 12) |
                        ((*(png_buf_read_32_ptr + 1) & 0x0000F000UL) >>  8) |
                        ((*(png_buf_read_32_ptr + 1) & 0x000000F0UL) >>  4);
                    png_buf_read_32_ptr += 2;
                    fb_32_ptr++;
                }
                if(((xx % 2) && (!(png_width % 2)))
                || ((!(xx % 2)) && (png_width % 2)))
                {
                    *fb_32_ptr &= 0x0000FFFFUL;
                    *fb_32_ptr |=
                        ((*png_buf_read_32_ptr & 0xF0000000UL)      ) |
                        ((*png_buf_read_32_ptr & 0x00F00000UL) <<  4) |
                        ((*png_buf_read_32_ptr & 0x0000F000UL) <<  8) |
                        ((*png_buf_read_32_ptr & 0x000000F0UL) << 12);
                }
            }
        }
        FAPI_SYS_FREE(png_rowbuf_ptr);
    }
    else if((fb_info.colourMode == OSDHANDLER_OSD_MODE_CLUT_8BIT)
    ||      (fb_info.colourMode == OSDHANDLER_OSD_MODE_CLUT_4BIT))
    {
        if(png_get_color_type(png_ptr, info_ptr) != PNG_COLOR_TYPE_PALETTE)
        {
            png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
            return(DECPNG_ERR_COLOUR_MODE_MISMATCH);
        }
        if(fb_info.colourMode == OSDHANDLER_OSD_MODE_CLUT_8BIT)
        {
            if(png_get_bit_depth(png_ptr, info_ptr) != 8)
            {
                png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
                return(DECPNG_ERR_COLOUR_MODE_MISMATCH);
            }
            png_rowbuf_ptr = FAPI_SYS_MALLOC_CACHED(png_width * sizeof(uint8_t));
            if(png_rowbuf_ptr == NULL)
            {
                png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
                return(DECPNG_ERR_OUT_OF_MEMORY);
            }
            for(ii = 0; ii < png_height; ii++)
            {
                if ( progressCb != NULL )
                {
                    decode_info.decodedLines = ii;

                    /* Give progress and test for abort condition. */
                    if ( progressCb (&decode_info, progressCbArg) != 0 )
                    {
                        FAPI_SYS_FREE(png_rowbuf_ptr);
                        png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
                        return(PICDEC_ERR_ABORT_REQUEST);
                    }
                }

                kk = png_width;
                // for each row copy PNG data into the buffer in ARGB format
                png_read_row(png_ptr, (png_bytep)png_rowbuf_ptr, NULL);
                png_buf_read_8_ptr = (uint8_t*)png_rowbuf_ptr;
                OSDHANDLER_FB_GetPixelAddress(fbHandle, xx, yy + ii, &fb_32_ptr, &fb_offset);

                // handle leading pixels:
                if(fb_offset)
                {
                    while(fb_offset)
                    {
                        colour_mask = 0x000000FFUL << (24 - fb_offset);
                        *fb_32_ptr &= ~colour_mask;
                        *fb_32_ptr |= *png_buf_read_8_ptr++ << (24 - fb_offset);
                        fb_offset = (fb_offset + 8) % 32;
                    }
                    fb_32_ptr++;
                }
                kk >>= 2;
                while(kk--)
                {
                    *fb_32_ptr =
                        (*png_buf_read_8_ptr     << 24) |
                        (*(png_buf_read_8_ptr+1) << 16) |
                        (*(png_buf_read_8_ptr+2) <<  8) |
                        (*(png_buf_read_8_ptr+3)      );
                        png_buf_read_8_ptr += 4;
                        fb_32_ptr++;
                }
                // handle trailing pixels:
                kk = png_width - ((4 - (xx % 4)) % 4); // strip leading pixel count
                kk %= 4; // count of remaining pixels
                while(kk--)
                {
                    colour_mask = 0x000000FFUL << (24 - fb_offset);
                    *fb_32_ptr &= ~colour_mask;
                    *fb_32_ptr |= *png_buf_read_8_ptr++ << (24 - fb_offset);
                    fb_offset = (fb_offset + 8) % 32;
                }
                fb_32_ptr++;
            }
        }
        else if(fb_info.colourMode == OSDHANDLER_OSD_MODE_CLUT_4BIT)
        {
            if(png_get_bit_depth(png_ptr, info_ptr) != 4)
            {
                png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
                return(DECPNG_ERR_COLOUR_MODE_MISMATCH);
            }
            //png_rowbuf_ptr = FAPI_SYS_MALLOC_CACHED(((png_width + 1) * fb_info.bpp) / 8);
            png_rowbuf_ptr = FAPI_SYS_MALLOC_CACHED(png_width);
            if(png_rowbuf_ptr == NULL)
            {
                png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
                return(DECPNG_ERR_OUT_OF_MEMORY);
            }
            for(ii = 0; ii < png_height; ii++)
            {
                if ( progressCb != NULL )
                {
                    decode_info.decodedLines = ii;

                    /* Give progress and test for abort condition. */
                    if ( progressCb (&decode_info, progressCbArg) != 0 )
                    {
                        FAPI_SYS_FREE(png_rowbuf_ptr);
                        png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
                        return(PICDEC_ERR_ABORT_REQUEST);
                    }
                }

                kk = png_width;
                // for each row copy PNG data into the buffer in ARGB format
                png_read_row(png_ptr, (png_bytep)png_rowbuf_ptr, NULL);
                png_buf_read_8_ptr = (uint8_t*)png_rowbuf_ptr;
                OSDHANDLER_FB_GetPixelAddress(fbHandle, xx, yy + ii, &fb_32_ptr, &fb_offset);

                // if xx is odd, the 4-bit values will have to be swapped:
                if(fb_offset % 8)
                {
                    tmp1 = 1;
                    colour_mask = 0x0000000FUL;
                    while(kk)
                    {
                        // tmp1 serves as a flag that decides whether to read the upper
                        // or the lower 4 bits of a byte from the bitmap file:
                        data_mask = colour_mask <<
                                    ((0x001CUL - fb_offset) & 0x001FUL);
                        *fb_32_ptr = (*fb_32_ptr & (~data_mask)) |
                                     ((tmp1 ?
                                      ((*png_buf_read_8_ptr & 0xF0UL) >> 4):
                                      (*png_buf_read_8_ptr++ & 0x0FUL)) <<
                                      ((0x001CUL - fb_offset) & 0x001FUL));
                        tmp1 = !tmp1;
                        fb_offset += 4;
                        if(fb_offset >> 5)
                        {
                            fb_offset = 0;
                            fb_32_ptr++;
                        }
                        kk--;
                    }
                    if(!tmp1)
                    {
                        png_buf_read_8_ptr++;
                    }
                }
                else
                {
                    colour_mask = 0x000000FFUL;
                    // write the first bytes:
                    while((fb_offset) && (kk >> 1))
                    {
                        data_mask = colour_mask << (0x0018UL - fb_offset);
                        *fb_32_ptr =   (*fb_32_ptr & (~data_mask)) |
                                       ((uint32_t)*png_buf_read_8_ptr++ <<
                                           (0x0018UL - fb_offset));
                        fb_offset += 8;
                        kk -= 2;
                        if(fb_offset >> 5)
                        {
                            fb_offset = 0;
                            fb_32_ptr++;
                        }
                    }

                    // now copy 4-byte-wise:
                    // tmp1 serves as a counter for remaining copy steps here
                    tmp1 = kk % 8;
                    kk >>= 3;
                    while(kk--)
                    {
                        tmp2  = (uint32_t)*png_buf_read_8_ptr++ << 24;
                        tmp2 |= (uint32_t)*png_buf_read_8_ptr++ << 16;
                        tmp2 |= (uint32_t)*png_buf_read_8_ptr++ <<  8;
                        tmp2 |= (uint32_t)*png_buf_read_8_ptr++;
                        *fb_32_ptr++ = tmp2;
                    }

                    kk = tmp1 >> 1;
                    tmp1 %= 2;

                    // copy the remaining bytes:

                    while(kk)
                    {
                        data_mask = colour_mask <<
                                        ((0x0018UL - fb_offset) & 0x001FUL);
                        *fb_32_ptr =   (*fb_32_ptr & (~data_mask)) |
                                       ((uint32_t)*png_buf_read_8_ptr++ <<
                                       ((0x0018UL - fb_offset) & 0x001FUL));
                        fb_offset += 8;
                        kk--;
                    }

                    // if the horizontal resolution is uneven, write the last pixel:
                    if(tmp1)
                    {
                        colour_mask = 0x0000000FUL;
                        data_mask = colour_mask <<
                                        ((0x001CUL - fb_offset) & 0x001FUL);
                        *fb_32_ptr =   (*fb_32_ptr & (~data_mask)) |
                                       (((*png_buf_read_8_ptr++ & 0x00F0UL) >> 4) <<
                                       ((0x001CUL - fb_offset) & 0x001FUL));
                    }
                }
            }
        }
        FAPI_SYS_FREE(png_rowbuf_ptr);

        // set up CLUT of the FB:
        if(fb_info.clutLock == OSDHANDLER_FB_CLUT_LOCK_OFF)
        {
            png_get_PLTE(png_ptr, info_ptr, &png_palette_ptr, (int*)(void*)&palette_size);

            fb_palette_ptr = FAPI_SYS_MALLOC_CACHED(256 * sizeof(uint32_t));
            if(fb_palette_ptr == NULL)
            {
                png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
                return(DECPNG_ERR_OUT_OF_MEMORY);
            }

            // copy the palette into an OSD capable CLUT:
            if(fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_ARGB)
            {
                for(ii = 0; ii < palette_size; ii++)
                {
                    fb_palette_ptr[ii] =
                            0xFF000000UL |
                            (((png_palette_ptr + ii)->red)  << 16) |
                            (((png_palette_ptr + ii)->green) << 8) |
                            (((png_palette_ptr + ii)->blue));
                }
            }
            else if(fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_RGBA)
            {
                for(ii = 0; ii < palette_size; ii++)
                {
                    fb_palette_ptr[ii] =
                            (((png_palette_ptr + ii)->red)   << 24) |
                            (((png_palette_ptr + ii)->green) << 16) |
                            (((png_palette_ptr + ii)->blue)  <<  8) |
                            0x000000FFUL;
                }
            }
            else if(fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_ACbYCr)
            {
                for(ii = 0; ii < palette_size; ii++)
                {
                    fb_palette_ptr[ii] = (OSDHANDLER_ConvertARGB2CbYCrA_word
                                (
                                0xFF000000UL |
                                (((png_palette_ptr + ii)->red)  << 16) |
                                (((png_palette_ptr + ii)->green) << 8) |
                                (((png_palette_ptr + ii)->blue))
                                ) >> 8) | 0xFF000000UL;
                }
            }
            else if(fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_CbYCrA)
            {
                for(ii = 0; ii < palette_size; ii++)
                {
                    fb_palette_ptr[ii] = OSDHANDLER_ConvertARGB2CbYCrA_word
                                (
                                0xFF000000UL |
                                (((png_palette_ptr + ii)->red)  << 16) |
                                (((png_palette_ptr + ii)->green) << 8) |
                                (((png_palette_ptr + ii)->blue))
                                );
                }
            }
            //(void)OSDHANDLER_SetClutPartially(layerId, fb_palette_ptr, 0, palette_size, NULL);
            (void)OSDHANDLER_FB_SetClut(fbHandle, fb_palette_ptr);
            FAPI_SYS_FREE(fb_palette_ptr);
        }
    }
    else // colourMode
    {
        png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
        return(DECPNG_ERR_COLOUR_MODE_MISMATCH);
    }
    // cleanup:
    png_read_end(png_ptr, end_ptr);
    png_destroy_read_struct(&png_ptr, &info_ptr, &end_ptr);
    // */
    return(FAPI_OK);
}
