/*!
*****************************************************************************
**
** \file        ./subsystems/paint/picture/codecs/png/src/decpng.h
**
** \version     $Id: $
**
** \brief       picture decoding library for PNG
**
**              This file contains the declaration of the DECPNG picture
**              decoder API for PNG.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

#ifndef DECPNG_H
    #define DECPNG_H

    #include <stdint.h>
    #include <fapi/sys_error.h>
    #include <picture/picdec_interface.h>

    //***********************************************************************
    //***********************************************************************
    //** Defines and Macros
    //***********************************************************************
    //***********************************************************************

    /*!
    *************************************************************************
    ** Standard error codes for the DECPNG library
    *************************************************************************
    */

    #define SUBSYS_PAINT_DECPNG_MODULE_BASE \
            (int32_t)(SUBSYS_PAINT_PICTURE_MODULE_BASE - 10000)

    /*! Bad parameter passed. */
    #define DECPNG_ERR_BAD_PARAMETER \
            (int32_t)(SUBSYS_PAINT_DECPNG_MODULE_BASE + FAPI_ERR_BAD_PARAMETER)

    /*! Memory allocation failed. */
    #define DECPNG_ERR_OUT_OF_MEMORY \
            (int32_t)(SUBSYS_PAINT_DECPNG_MODULE_BASE + FAPI_ERR_OUT_OF_MEMORY)

    /*! Device already initialised. */
    #define DECPNG_ERR_ALREADY_INITIALIZED \
            (int32_t)(SUBSYS_PAINT_DECPNG_MODULE_BASE + FAPI_ERR_ALREADY_INITIALIZED)

    /*! Device not initialised. */
    #define DECPNG_ERR_NOT_INITIALIZED \
            (int32_t)(SUBSYS_PAINT_DECPNG_MODULE_BASE + FAPI_ERR_NOT_INITIALIZED)

    /*! Feature or function is not available. */
    #define DECPNG_ERR_FEATURE_NOT_SUPPORTED \
            (int32_t)(SUBSYS_PAINT_DECPNG_MODULE_BASE + FAPI_ERR_FEATURE_NOT_SUPPORTED)

    /*! Timeout occured. */
    #define DECPNG_ERR_TIMEOUT \
            (int32_t)(SUBSYS_PAINT_DECPNG_MODULE_BASE + FAPI_ERR_TIMEOUT)

    /*! The device is busy, try again later. */
    #define DECPNG_ERR_DEVICE_BUSY \
            (int32_t)(SUBSYS_PAINT_DECPNG_MODULE_BASE + FAPI_ERR_DEVICE_BUSY)

    /*! Invalid handle was passed. */
    #define DECPNG_ERR_INVALID_HANDLE \
            (int32_t)(SUBSYS_PAINT_DECPNG_MODULE_BASE + FAPI_ERR_INVALID_HANDLE)

    /*! Semaphore could not be created. */
    #define DECPNG_ERR_SEMAPHORE_CREATE \
            (int32_t)(SUBSYS_PAINT_DECPNG_MODULE_BASE + FAPI_ERR_SEMAPHORE_CREATE)

    /*! The driver's used version is not supported. */
    #define DECPNG_ERR_UNSUPPORTED_VERSION \
            (int32_t)(SUBSYS_PAINT_DECPNG_MODULE_BASE + FAPI_ERR_UNSUPPORTED_VERSION)

    /*!
    *************************************************************************
    ** Additional DECPNG error codes.
    *************************************************************************
    */

    /*! \brief DECPNG specific error code offset */
    #define DECPNG_ERR_BASE \
            (int32_t) (SUBSYS_PAINT_DECPNG_MODULE_BASE - 100)

    /*! An error occured while setting up the PNG library. */
    #define DECPNG_ERR_PNG_INIT                 (int32_t) (DECPNG_ERR_BASE - 0)

    /*! An error occured while decoding the PNG file. It may be corrupted. */
    #define DECPNG_ERR_PNG_DECODE               (int32_t) (DECPNG_ERR_BASE - 1)

    /*! The picture would be decoded out of bounds of the frame buffer. */
    #define DECPNG_ERR_OUT_OF_FB_BOUNDS         (int32_t) (DECPNG_ERR_BASE - 2)

    /*! The colour/layout mode combination of PNG and FB is not supported. */
    #define DECPNG_ERR_COLOUR_MODE_MISMATCH     (int32_t) (DECPNG_ERR_BASE - 3)

    /*! The layout mode is not correct for a CCIR OSD. */
    #define DECPNG_ERR_CCIR_LAYOUT              (int32_t) (DECPNG_ERR_BASE - 4)

    /*! The file read function  did not return any data from the data source. */
    #define DECPNG_ERR_DATA_SOURCE              (int32_t) (DECPNG_ERR_BASE - 5)

    /*! The decoding process was aborted. */
    #define DECPNG_ERR_DECODE_ABORT             (int32_t) (DECPNG_ERR_BASE - 6)



    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** Global Data
    //***********************************************************************
    //***********************************************************************

    /*! \brief  Interface of the PNG decoder driver. */
    extern PICDEC_INTERFACE_DriverT DECPNG_Driver;

    /*! \brief  Points to the interface of the PNG decoder driver. */
    extern const PICDEC_INTERFACE_DriverT* DECPNG_DRIVER_PTR;


    //***********************************************************************
    //***********************************************************************
    //** API Functions
    //***********************************************************************
    //***********************************************************************

    #ifdef __cplusplus
    extern "C" {
    #endif

    int32_t DECPNG_Init(void);
    void    DECPNG_Exit(void);
    int32_t DECPNG_CheckFile (void* filePtr,
                              int32_t fileOffset,
                              PICDEC_ReadFileChunkT readFptr,
                              PICDEC_FileSeekT seekFptr);
    int32_t DECPNG_GetInfo   (void* filePtr,
                              int32_t fileOffset,
                              PICDEC_ReadFileChunkT readFptr,
                              PICDEC_FileSeekT seekFptr,
                              PICDEC_RotationT rotation,
                              PICDEC_InfoT* infoPtr);
    int32_t DECPNG_Decode    (void* filePtr,
                              int32_t fileOffset,
                              PICDEC_ReadFileChunkT readFptr,
                              PICDEC_FileSeekT seekFptr,
                              OSDHANDLER_FB_HandleT fbHandle,
                              uint32_t xx,
                              uint32_t yy,
                              uint32_t xRes,
                              uint32_t yRes,
                              PICDEC_RotationT rotation,
                              PICDEC_ScalerT scaler,
                              PICDEC_ProgressCbT progressCb,
                              void* progressCbArg);

    #ifdef __cplusplus
    }
    #endif
#endif // DECPNG_H
