/*
********************************************************************************
**
** \file        ./subsystems/paint/picture/src/picdec.c
**
** \version     $Id: $
**
** \brief       picture decoding library
**
**              This file contains the implementation of the picture library.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
********************************************************************************
*/

#include <stdint.h>
#include <fapex/abst_types.h>
#include "picdec.h"
#include "picdec_interface.h"


//******************************************************************************
//******************************************************************************
//** Local Defines
//******************************************************************************
//******************************************************************************

#define PICDEC_MAX_DECODERS    4
#define PICDEC_HANDLE_ID       (uint32_t)0x70696374 // 'pict'

#define PICDEC_LOCK   (void)FAPI_SYS_GET_SEMAPHORE( pictureSemaphore, FAPI_SYS_SUSPEND )
#define PICDEC_UNLOCK       FAPI_SYS_SET_SEMAPHORE( pictureSemaphore, FAPI_SYS_NO_SUSPEND )

#define HANDLE_LOCK    (void)FAPI_SYS_GET_SEMAPHORE( handle_ptr->sem, FAPI_SYS_SUSPEND )
#define HANDLE_UNLOCK        FAPI_SYS_SET_SEMAPHORE( handle_ptr->sem, FAPI_SYS_NO_SUSPEND )


//******************************************************************************
//******************************************************************************
//** Local Structures
//******************************************************************************
//******************************************************************************

/*!
********************************************************************************
** \brief Internal representation of the picture handle.
********************************************************************************
*/
typedef struct
{
    uint32_t                    id;             // should match PICDEC_HANDLE_ID
    uint32_t                    inUse : 1;      // handle in use
    int32_t                     decoderId;      // decoder suitable for current picture
    int32_t                     fileOffset;     // pic data offset within file
    FAPI_SYS_SemaphoreT         sem;            // handle semaphore
    PICDEC_ReadFileChunkT       readFptr;       // reads file chunks
    PICDEC_FileSeekT            seekFptr;       // file handle seek
    PICDEC_ScalerT              scaler;         // scaler method
    OSDHANDLER_FB_HandleT       fbHandle;       // associated framebuffer object
    PICDEC_RotationT            rotation;       // rotation setting
    void*                       fileHandlePtr;  // file handle
    PICDEC_ProgressCbT          progressCb;     // optional progress callback.
    void*                       progressCbArg;  // argument passed with progress
                                                // callback.


} picdecHandleT;


//******************************************************************************
//******************************************************************************
//** Local Data
//******************************************************************************
//******************************************************************************

static struct
{
    uint32_t initialized : 1;
    uint32_t numDecoders : 5;
} pictureState;

static FAPI_SYS_SemaphoreT pictureSemaphore = (FAPI_SYS_SemaphoreT)0;
static picdecHandleT       picdecHandleArray[PICDEC_TOTAL_HANDLE_COUNT];

PICDEC_INTERFACE_DriverT pictureDriverArray[PICDEC_MAX_DECODERS];


//******************************************************************************
//******************************************************************************
//** Local Functions Declaration
//******************************************************************************
//******************************************************************************

static picdecHandleT* pictureAllocateHandle(void);
static uint32_t pictureCheckHandle(const picdecHandleT* handlePtr);

//******************************************************************************
//******************************************************************************
//** Global Data
//******************************************************************************
//******************************************************************************


//******************************************************************************
//******************************************************************************
//** Local Functions
//******************************************************************************
//******************************************************************************

/*!
********************************************************************************
** \brief Allocate a new PICTURE handle.
**
** \return
** - (>0) If handle can be allocated
** - (0)  In case of error (no more handles available)
********************************************************************************
*/
static picdecHandleT* pictureAllocateHandle(void)
{
    picdecHandleT* handle_ptr = NULL;
    uint32_t index = 0;

    for(index = 0; index < PICDEC_TOTAL_HANDLE_COUNT; index++)
    {
        /* search for handle not in use */
        if( !picdecHandleArray[index].inUse )
        {
            memset(&(picdecHandleArray[index]), 0, sizeof(picdecHandleT));
            /* set in use */
            picdecHandleArray[index].inUse = 1;
            picdecHandleArray[index].id = PICDEC_HANDLE_ID;
            picdecHandleArray[index].sem = FAPI_SYS_CREATE_SEMAPHORE(1);

            /* set handle pointer */
            handle_ptr = &(picdecHandleArray[index]);
            break;
        }
    }
    return(handle_ptr);
}

/*!
********************************************************************************
** \brief   Checks a given handle.
**
** \param   handlePtr   Points to the picture handle to check.
**
** \return
** - (1) If handle is OK
** - (0) If handle is invalid
********************************************************************************
*/
static uint32_t pictureCheckHandle(const picdecHandleT* handlePtr)
{
    return(pictureState.initialized && handlePtr && handlePtr->inUse
            && (handlePtr->id == PICDEC_HANDLE_ID));
}

/*!
********************************************************************************
** \brief   Releases a given handle.
**
** \param   handlePtr   Points to the picture handle to close.
**
** This function de-allocates a given PICTURE handle.
********************************************************************************
*/
static void pictureReleaseHandle (picdecHandleT* handlePtr)
{
    FAPI_SYS_DESTROY_SEMAPHORE(handlePtr->sem);
    /* clear handle context */
    memset(handlePtr, 0, sizeof(picdecHandleT));
    return;
}


//******************************************************************************
//******************************************************************************
//** API Functions
//******************************************************************************
//******************************************************************************

/*!
********************************************************************************
** \brief   Initialize the PICTURE library.
**
** This function initializes the PICTURE subsystem with a given set of
** decoder libraries.
**
** \param   initParamsPtr
**              Points to the PICTURE initialization parameters.
**
** \return
**          - #FAPI_OK on success.
**          - #PICDEC_ERR_ALREADY_INITIALIZED
**              if the PICTURE library has already been initialized.
**          - #PICDEC_ERR_BAD_PARAMETER
**              if a bad parameter has been passed. Possible reasons:
**              - The passed initParamsPtr is NULL.
**              - It is attempted to initialize with 0 decoders.
**              - There are more decoders passed than PICDEC_MAX_DECODERS.
**              - The referenced init params structure is bad.
**
** \sa
**          - PICDEC_Exit()
**
********************************************************************************
*/
int32_t PICDEC_Init(const PICDEC_InitParamsT* initParamsPtr)
{
    uint32_t ii = 0;
    uint32_t* address_ptr = NULL;
    int32_t result = 0;

    if(pictureState.initialized)
    {
        return(PICDEC_ERR_ALREADY_INITIALIZED);
    }
    if ((initParamsPtr == NULL)
        || (initParamsPtr->numDecoders == 0)
        || (initParamsPtr->numDecoders > PICDEC_MAX_DECODERS)
        || (initParamsPtr->decodersPtr == NULL))
    {
        return (PICDEC_ERR_BAD_PARAMETER);
    }

    pictureSemaphore = FAPI_SYS_CREATE_SEMAPHORE(1);
    if (!pictureSemaphore)
    {
        return (PICDEC_ERR_SEMAPHORE_CREATE);
    }

    memset(&pictureState, 0, sizeof(pictureState));
    memset(pictureDriverArray, 0, sizeof(pictureDriverArray));
    memset(picdecHandleArray, 0, sizeof(picdecHandleArray));

    pictureState.numDecoders = initParamsPtr->numDecoders;

    address_ptr = initParamsPtr->decodersPtr;
    for (ii = 0; ii < pictureState.numDecoders; ii++)
    {
        if((*address_ptr) == 0)
        {
            PICDEC_Exit();
            return(PICDEC_ERR_BAD_PARAMETER);
        }
        /*lint -save -e923 */
        memcpy(&pictureDriverArray[ii], (void*)*address_ptr++,
               sizeof(PICDEC_INTERFACE_DriverT));
        /*lint -restore */
        result = pictureDriverArray[ii].initFptr();
        if(result)
        {
            PICDEC_Exit();
            return(result);
        }
    }
    pictureState.initialized = 1;
    return(FAPI_OK);
}

/*!
********************************************************************************
** \brief   Exit the PICTURE library.
**
** This function closes all handles and deallocates the registered picture
** decoders.
**
** \sa
**          - PICDEC_Init()
**
********************************************************************************
*/
void PICDEC_Exit(void)
{
    uint32_t ii = 0;

    for(ii = 0; ii < PICDEC_TOTAL_HANDLE_COUNT; ii++)
    {
        // handle check will be performed within PICDEC_Close()
        (void)PICDEC_Close((FAPI_SYS_HandleT)&picdecHandleArray[ii]);
    }

    ///////////////
    PICDEC_LOCK;
    ///////////////

    for(ii = 0; ii < pictureState.numDecoders; ii++)
    {
        if(pictureDriverArray[ii].exitFptr) pictureDriverArray[ii].exitFptr();
    }
    memset(&pictureState, 0, sizeof(pictureState));
    memset(picdecHandleArray, 0, sizeof(picdecHandleArray));
    memset(pictureDriverArray, 0, sizeof(pictureDriverArray));

    FAPI_SYS_DESTROY_SEMAPHORE(pictureSemaphore);
    pictureSemaphore = 0;

    ///////////////
    PICDEC_UNLOCK;
    ///////////////

    return;
}

/*!
*****************************************************************************
** \brief   Open a picture driver instance.
**
** \param   openParamsPtr
**              Pointer to the open parameters.
** \param   errorCodePtr
**              Optional pointer that will receive error codes.
**              Possible values:
**              - #PICDEC_ERR_NOT_INITIALIZED if PICTURE is not initialized.
**              - #PICDEC_ERR_BAD_PARAMETER if openParamsPtr is invalid.
**
** \return
**          - (>0) valid driver handle on success.
**          - (0) if the driver could not be opened.
**
** \sa
**          - PICDEC_Close
*****************************************************************************
*/
FAPI_SYS_HandleT PICDEC_Open(const PICDEC_OpenParamsT* openParamsPtr,
                              int32_t* errorCodePtr)
{
    picdecHandleT* handle_ptr;

    if(errorCodePtr) *errorCodePtr = FAPI_OK;

    if(pictureState.initialized == 0)
    {
        if(errorCodePtr) *errorCodePtr = PICDEC_ERR_NOT_INITIALIZED;
        return(NULL);
    }
    if((openParamsPtr == NULL)
    || (openParamsPtr->readFptr == NULL)
    || (openParamsPtr->seekFptr == NULL))
    {
        if(errorCodePtr) *errorCodePtr = PICDEC_ERR_BAD_PARAMETER;
        return(NULL);
    }

    ///////////////
    PICDEC_LOCK;
    ///////////////

    /* allocate handle */
    handle_ptr = pictureAllocateHandle();

    if( ! pictureCheckHandle(handle_ptr) )
    {
        ///////////////
        PICDEC_UNLOCK;
        ///////////////
        if(errorCodePtr) *errorCodePtr = PICDEC_ERR_MAX_HANDLE_COUNT;
        return(NULL);
    }

    handle_ptr->readFptr        = openParamsPtr->readFptr; // constant
    handle_ptr->seekFptr        = openParamsPtr->seekFptr; // constant
    handle_ptr->scaler          = openParamsPtr->scaler;   // mutable
    handle_ptr->fbHandle        = openParamsPtr->fbHandle; // mutable
    handle_ptr->rotation        = PICDEC_ROTATION_0;      // mutable
    handle_ptr->fileHandlePtr   = NULL;                    // mutable

    ///////////////
    PICDEC_UNLOCK;
    ///////////////

    if(errorCodePtr) *errorCodePtr = FAPI_OK;
    return(handle_ptr);
}

/*!
*****************************************************************************
** \brief   Close a PICTURE driver handle instance.
**
** All resources associated with the given driver handle will be released.
**
** \param   picdecHandle
**              Driver handle previously returned by PICDEC_Open().
**
** \return
**          - #FAPI_OK on success.
**          - #PICDEC_ERR_INVALID_HANDLE if the picture handle is invalid.
**
** \sa
**          - PICDEC_Open()
**
*****************************************************************************
*/
int32_t PICDEC_Close (FAPI_SYS_HandleT picdecHandle)
{
    picdecHandleT* handle_ptr = (picdecHandleT*)picdecHandle;

    ///////////////
    PICDEC_LOCK;
    ///////////////

    /* check handle */
    if( ! pictureCheckHandle(handle_ptr))
    {
        ///////////////
        PICDEC_UNLOCK;
        ///////////////
        return (PICDEC_ERR_INVALID_HANDLE);
    }

    // wait until operations of this handle are finished:

    ///////////////
    HANDLE_LOCK;
    ///////////////

    // release handle:
    pictureReleaseHandle(handle_ptr);

    ///////////////
    PICDEC_UNLOCK;
    ///////////////

    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Select a picture file to be processed by the PICTURE library.
**
** \param   picdecHandle
**              Driver handle previously returned by PICDEC_Open().
** \param   fileHandlePtr
**              Specifies the file handle for the picture to be processed.
**              Pass NULL in order to unregister previously opened picture
**              files.
** \param   fileOffset
**              Gives the absolute offset of the picture data within the file.
**              This feature may be used for embedded picture data, e.g., album
**              covers in audio files like mp3. This offset must be a
**              non-negative value.
**
** \return
**          - #FAPI_OK on success.
**          - #PICDEC_ERR_INVALID_HANDLE if the picture handle is invalid.
**          - #PICDEC_ERR_FORMAT_NOT_SUPPORTED if the picture file format
**              is not supported by the registered decoder drivers.
**          - #PICDEC_ERR_NO_FILE_SELECTED if no picture file is given.
**
*****************************************************************************
*/
int32_t PICDEC_SelectPicture(FAPI_SYS_HandleT picdecHandle, \
                             void* fileHandlePtr,           \
                             int32_t fileOffset)
{
    int32_t         ii = 0;
    int32_t         result = PICDEC_ERR_NO_FILE_SELECTED;
    picdecHandleT* handle_ptr = NULL;

    handle_ptr = (picdecHandleT*)picdecHandle;

    if( ! pictureCheckHandle(handle_ptr))
    {
        return (PICDEC_ERR_INVALID_HANDLE);
    }
    if(fileOffset < 0)
    {
        return (PICDEC_ERR_BAD_PARAMETER);
    }

    ///////////////
    HANDLE_LOCK;
    ///////////////

    // TODO: special treatment for unregistering the previous file?

    handle_ptr->fileHandlePtr = fileHandlePtr;
    handle_ptr->fileOffset = fileOffset;

    if(fileHandlePtr)
    {
        for(ii = 0; ii < (int32_t)pictureState.numDecoders; ii++)
        {
            if(pictureDriverArray[ii].checkFptr)
            {
                result = pictureDriverArray[ii].checkFptr
                                (fileHandlePtr,
                                 fileOffset,
                                 handle_ptr->readFptr,
                                 handle_ptr->seekFptr);
            }
            if(result == FAPI_OK) break;
        }
    }
    handle_ptr->decoderId = (result == FAPI_OK) ? ii : -1;

    //////////////
    HANDLE_UNLOCK;
    //////////////

    return(result);
}

/*!
*****************************************************************************
** \brief   Select a scaler algorithm.
**
** \param   picdecHandle
**              Driver handle previously returned by PICDEC_Open().
** \param   scaler
**              Specifies a scaler algorithm used for up- and downscaling
**              picture dimensions when decoding.
**
** \return
**          - #FAPI_OK on success.
**          - #PICDEC_ERR_INVALID_HANDLE if the picture handle is invalid.
**
*****************************************************************************
*/
int32_t PICDEC_SelectScaler(FAPI_SYS_HandleT picdecHandle, \
                             PICDEC_ScalerT scaler)
{
    picdecHandleT* handle_ptr = NULL;

    handle_ptr = (picdecHandleT*)picdecHandle;

    if( ! pictureCheckHandle(handle_ptr))
    {
        return (PICDEC_ERR_INVALID_HANDLE);
    }

    ///////////////
    HANDLE_LOCK;
    ///////////////

    handle_ptr->scaler = scaler;

    //////////////
    HANDLE_UNLOCK;
    //////////////

    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Select a framebuffer.
**
** \param   picdecHandle
**              Driver handle previously returned by PICDEC_Open().
** \param   fbHandle
**              Specifies the framebuffer that will be written when decoding
**              pictures.
**
** \return
**          - #FAPI_OK on success.
**          - #PICDEC_ERR_INVALID_HANDLE if the picture handle is invalid.
**
*****************************************************************************
*/
int32_t PICDEC_SelectFramebuffer(FAPI_SYS_HandleT picdecHandle, \
                                  OSDHANDLER_FB_HandleT fbHandle)
{
    picdecHandleT* handle_ptr = NULL;

    handle_ptr = (picdecHandleT*)picdecHandle;

    if( ! pictureCheckHandle(handle_ptr))
    {
        return (PICDEC_ERR_INVALID_HANDLE);
    }

    ///////////////
    HANDLE_LOCK;
    ///////////////

    handle_ptr->fbHandle = fbHandle;

    //////////////
    HANDLE_UNLOCK;
    //////////////

    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Select picture rotation.
**
** \param   picdecHandle
**              Driver handle previously returned by PICDEC_Open().
** \param   rotation
**              Specifies the rotation that will be applied when decoding
**              a picture to a framebuffer.
**
** \return
**          - #FAPI_OK on success.
**          - #PICDEC_ERR_INVALID_HANDLE if the picture handle is invalid.
**
*****************************************************************************
*/
int32_t PICDEC_SelectRotation(FAPI_SYS_HandleT picdecHandle, \
                               PICDEC_RotationT rotation)
{
    picdecHandleT* handle_ptr = NULL;

    handle_ptr = (picdecHandleT*)picdecHandle;

    if( ! pictureCheckHandle(handle_ptr))
    {
        return (PICDEC_ERR_INVALID_HANDLE);
    }

    ///////////////
    HANDLE_LOCK;
    ///////////////

    handle_ptr->rotation = rotation;

    //////////////
    HANDLE_UNLOCK;
    //////////////

    return(FAPI_OK);
}

/*!
********************************************************************************
** \brief   Get information about a picture file.
**
**          This function determines file format, bpp, resolution,
**          colour lookup table of a picture file.
**
** \param   picdecHandle
**              Driver handle previously returned by PICDEC_Open().
** \param   infoPtr
**              The structure where the picture file information will be stored.
**
** \return
**          - #FAPI_OK on success.
**          - #PICDEC_ERR_INVALID_HANDLE if the picture handle is invalid.
**          - #PICDEC_ERR_BAD_PARAMETER if a NULL pointer has been passed.
**          - #PICDEC_ERR_NO_FILE_SELECTED if no picture file was selected.
**          - #PICDEC_ERR_FORMAT_NOT_SUPPORTED if the picture cannot
**              be processed by the registered decoder drivers.
**          - some decoder library specific error code.
**
********************************************************************************
*/
int32_t PICDEC_GetInfo (FAPI_SYS_HandleT picdecHandle,
                         PICDEC_InfoT* infoPtr)
{
    int32_t         result = 0;
    picdecHandleT* handle_ptr = NULL;

    if (infoPtr == NULL)
    {
        return(PICDEC_ERR_BAD_PARAMETER);
    }

    handle_ptr = (picdecHandleT*)picdecHandle;
    if( ! pictureCheckHandle(handle_ptr))
    {
        return (PICDEC_ERR_INVALID_HANDLE);
    }

    if(handle_ptr->fileHandlePtr == NULL)
    {
        return (PICDEC_ERR_NO_FILE_SELECTED);
    }

    if(handle_ptr->decoderId < 0)
    {
        return (PICDEC_ERR_FORMAT_NOT_SUPPORTED);
    }

    ///////////////
    HANDLE_LOCK;
    ///////////////

    result = pictureDriverArray[handle_ptr->decoderId].getInfoFptr
                                        (handle_ptr->fileHandlePtr,
                                         handle_ptr->fileOffset,
                                         handle_ptr->readFptr,
                                         handle_ptr->seekFptr,
                                         handle_ptr->rotation,
                                         infoPtr);

    //////////////
    HANDLE_UNLOCK;
    //////////////

    return(result);
}

/*!
********************************************************************************
** \brief   Decode a picture into a framebuffer.
**
**          The picture format is determined automatically and the picture
**          will be decoded by the corresponding decoding library.
**
** \param   picdecHandle
**                      Driver handle previously returned by PICDEC_Open().
** \param   xx          The x coordinate where the picture will be decoded.
** \param   yy          The y coordinate where the picture will be decoded.
** \param   xRes        The target horizontal resolution on the framebuffer.
** \param   yRes        The target vertical resolution on the framebuffer.
**
** \return
**          - #FAPI_OK on success.
**          - #PICDEC_ERR_INVALID_HANDLE if the picture handle is invalid.
**          - #PICDEC_ERR_NO_FILE_SELECTED if no picture file was selected.
**          - #PICDEC_ERR_FORMAT_NOT_SUPPORTED if the picture cannot
**              be processed by the registered decoder drivers.
**          - some decoder library specific error code.
**
********************************************************************************
*/
int32_t PICDEC_Decode (FAPI_SYS_HandleT picdecHandle,   \
                        uint32_t xx,                    \
                        uint32_t yy,                    \
                        uint32_t xRes,                  \
                        uint32_t yRes)
{
    int32_t         result = 0;
    picdecHandleT* handle_ptr = NULL;

    handle_ptr = (picdecHandleT*)picdecHandle;
    if( ! pictureCheckHandle(handle_ptr))
    {
        return (PICDEC_ERR_INVALID_HANDLE);
    }

    if(handle_ptr->fileHandlePtr == NULL)
    {
        return (PICDEC_ERR_NO_FILE_SELECTED);
    }

    if(handle_ptr->decoderId < 0)
    {
        return (PICDEC_ERR_FORMAT_NOT_SUPPORTED);
    }

    ///////////////
    HANDLE_LOCK;
    ///////////////

    result = pictureDriverArray[handle_ptr->decoderId].decodeFptr
                        (handle_ptr->fileHandlePtr,
                         handle_ptr->fileOffset,
                         handle_ptr->readFptr,
                         handle_ptr->seekFptr,
                         handle_ptr->fbHandle,
                         xx,
                         yy,
                         xRes,
                         yRes,
                         handle_ptr->rotation,
                         handle_ptr->scaler,
                         handle_ptr->progressCb,
                         handle_ptr->progressCbArg);

    //////////////
    HANDLE_UNLOCK;
    //////////////

    return(result);
}

/*!
********************************************************************************
** \brief   Register a callback to for decoding progress and abortion of
**          decoding process.
**
** \param   picdecHandle
**                      Driver handle previously returned by PICDEC_Open().
** \param   progressCb  Pointer to callback function to be registered.
** \param   optPtr      Optional argument that will be passed as an argument of
**                      the progress callback function.
**
** \return
**          - #FAPI_OK on success.
**          - #PICDEC_ERR_INVALID_HANDLE if the picture handle is invalid.
**
********************************************************************************
*/
int32_t PICDEC_SetProgressCallback (FAPI_SYS_HandleT picdecHandle,
                                    PICDEC_ProgressCbT progressCb,
                                    void* optPtr)
{

// TODO:
// Use semaphores to synchronize execution:
// This would make sure that the next decode isn't scheduled
// before the abort request has finished executing for all decoding threads.
// Mechanism:
// 1.1: Semaphore down before setting the decodeAbort flag in the handle below.
// 1.2: Set the decodeAbort flag.
// 1.3: While an atomic counter that counts the number of decode processes
//      in the handle is greater than zero, spin in a loop.
// 1.4: When the atomic counter becomes zero, do a semaphore up.
// Also make sure that the program doesn't run into the decode function for new decode calls:
// 2.1: If the decodeAbort flag is set, do a semaphore down. This will make the process yield.
// 2.2: Do a semaphore up directly afterwards.
// 2.3: Increment the atomic counter when the decode process starts.
// 2.4: Decrement the atomic counter when the decode process ends or when it is aborted.

// Replace the term "atomic counter" by "volatile counter" since we're running on a single core.

// There may still be synchronization problems since 1.1 and 1.2 are not atomic in one op!

    picdecHandleT* handle_ptr = NULL;

    handle_ptr = (picdecHandleT*)picdecHandle;
    if( ! pictureCheckHandle(handle_ptr))
    {
        return (PICDEC_ERR_INVALID_HANDLE);
    }

    ///////////////
    HANDLE_LOCK;
    ///////////////

    handle_ptr->progressCb    = progressCb;
    handle_ptr->progressCbArg = optPtr;

    //////////////
    HANDLE_UNLOCK;
    //////////////

    return(FAPI_OK);
}
