/*!
*****************************************************************************
**
** \file        ./subsystems/paint/picture/src/picdec.h
**
** \version     $Id: $
**
** \brief       Application interface declaration for picture decoders.
**
**              This header declares the public API of the picture decoding
**              library. It provides functions for displaying pictures of
**              any format in a scaled and rotated fashion.
**
** This file contains the declaration of the picture decoding API.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

#ifndef PICDEC_H
    #define PICDEC_H

    #include <stdio.h>
    #include <stdint.h>
    #include <fapi/sys_error.h>
    #include <osdhandler/osdhandler_fb.h>

    //***********************************************************************
    //***********************************************************************
    //** Defines and Macros
    //***********************************************************************
    //***********************************************************************

    #define PICDEC_TOTAL_HANDLE_COUNT      4


    #define PICDEC_SEEK_SET                0
    #define PICDEC_SEEK_CUR                1
    #define PICDEC_SEEK_END                2

    /*!
    ****************************************************************************
    ** Standard error codes for the PICTURE driver
    ****************************************************************************
    */

    #define SUBSYS_PAINT_PICDEC_MODULE_BASE     SUBSYS_PAINT_PICTURE_MODULE_BASE

    /*! Bad parameter passed. */
    #define PICDEC_ERR_BAD_PARAMETER \
            (int32_t)(SUBSYS_PAINT_PICDEC_MODULE_BASE + FAPI_ERR_BAD_PARAMETER)

    /*! Memory allocation failed. */
    #define PICDEC_ERR_OUT_OF_MEMORY \
            (int32_t)(SUBSYS_PAINT_PICDEC_MODULE_BASE + FAPI_ERR_OUT_OF_MEMORY)

    /*! Device already initialised. */
    #define PICDEC_ERR_ALREADY_INITIALIZED \
            (int32_t)(SUBSYS_PAINT_PICDEC_MODULE_BASE + FAPI_ERR_ALREADY_INITIALIZED)

    /*! Device not initialised. */
    #define PICDEC_ERR_NOT_INITIALIZED \
            (int32_t)(SUBSYS_PAINT_PICDEC_MODULE_BASE + FAPI_ERR_NOT_INITIALIZED)

    /*! Feature or function is not available. */
    #define PICDEC_ERR_FEATURE_NOT_SUPPORTED \
            (int32_t)(SUBSYS_PAINT_PICDEC_MODULE_BASE + FAPI_ERR_FEATURE_NOT_SUPPORTED)

    /*! Timeout occured. */
    #define PICDEC_ERR_TIMEOUT \
            (int32_t)(SUBSYS_PAINT_PICDEC_MODULE_BASE + FAPI_ERR_TIMEOUT)

    /*! The device is busy, try again later. */
    #define PICDEC_ERR_DEVICE_BUSY \
            (int32_t)(SUBSYS_PAINT_PICDEC_MODULE_BASE + FAPI_ERR_DEVICE_BUSY)

    /*! Invalid handle was passed. */
    #define PICDEC_ERR_INVALID_HANDLE \
            (int32_t)(SUBSYS_PAINT_PICDEC_MODULE_BASE + FAPI_ERR_INVALID_HANDLE)

    /*! Semaphore could not be created. */
    #define PICDEC_ERR_SEMAPHORE_CREATE \
            (int32_t)(SUBSYS_PAINT_PICDEC_MODULE_BASE + FAPI_ERR_SEMAPHORE_CREATE)

    /*! The driver's used version is not supported. */
    #define PICDEC_ERR_UNSUPPORTED_VERSION \
            (int32_t)(SUBSYS_PAINT_PICDEC_MODULE_BASE + FAPI_ERR_UNSUPPORTED_VERSION)

    /*!
    ****************************************************************************
    ** Additional PICTURE driver error codes.
    **
    ** Each error code shall be decreased in steps of 1.
    ****************************************************************************
    */

    /* \brief PICTURE specific error code offset */
    #define PICDEC_ERR_BASE \
            (int32_t) (SUBSYS_PAINT_PICDEC_MODULE_BASE - 100)

    /*! The maximum count of handles has been allocated. */
    #define PICDEC_ERR_MAX_HANDLE_COUNT        (int32_t) (PICDEC_ERR_BASE - 0)

    /*! There is no picture file selected for the handle. */
    #define PICDEC_ERR_NO_FILE_SELECTED        (int32_t) (PICDEC_ERR_BASE - 1)

    /*! There is no decoder registered that can process the picture file. */
    #define PICDEC_ERR_FORMAT_NOT_SUPPORTED    (int32_t) (PICDEC_ERR_BASE - 2)

    /*! Returned when attemped to write at coordinates that are out of bounds
    **  of the frame buffer.
    */
    #define PICDEC_ERR_OUT_OF_FB_BOUNDS        (int32_t) (PICDEC_ERR_BASE - 3)

    /*! The framebuffer is driven in a colour/layout mode combination that
    **  is not supported (e.g., by the scaler library).
    */
    #define PICDEC_ERR_UNSUPPORTED_OSD_COLOUR_LAYOUT_MODE \
                                               (int32_t) (PICDEC_ERR_BASE - 4)

    /*! The source picture has to be at least 2x2 pixels in size in order to
    **  be processable by the selected scaler.
    */
    #define PICDEC_ERR_SCALER_MIN_RESOLUTION_2x2 \
                                               (int32_t) (PICDEC_ERR_BASE - 5)

    /*! The picture decoder library unexpectedly aborted decoding, e.g. due
    **  to a corrupted file. */
    #define PICDEC_ERR_DECODE_ABORT            (int32_t) (PICDEC_ERR_BASE - 6)

    /*! The application requested to abort the decoding process. See
        PICDEC_SetProgressCallback(). */
    #define PICDEC_ERR_ABORT_REQUEST           (int32_t) (PICDEC_ERR_BASE - 7)

    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************

    /*!
    ****************************************************************************
    ** \brief   Initialization parameters for PICTURE library.
    **
    **          Contains the count of decoders to use and an array of pointers
    **          to decoder driver structures (e.g., PICPNG_DRIVER_PTR).
    ****************************************************************************
    */
    typedef struct
    {
        uint32_t  numDecoders; //!< number of decoder libraries to register
        uint32_t* decodersPtr; //!< field of pointers to picture decoder structs
    } PICDEC_InitParamsT;

    /*!
    ****************************************************************************
    ** \brief   Decoding progress information structure.
    **
    **          Contains parameters representing the current status of the
    **          decoding process.
    **
    ** \sa    PICDEC_SetProgressCallback
    **
    ****************************************************************************
    */
    typedef struct
    {
        /*! Number of finished decoding passes. */
        uint32_t  finishedPasses;

        /*! Number of total decoding passes. */
        uint32_t  totalPasses;

        /*! Number of processed picture lines in current pass. */
        uint32_t  decodedLines;

        /*! Number of total picture lines to process. Please note that this
        **  number does not necessarily relate to the image height since under
        **  some conditions, a picture may be prescaled or a line may processed
        **  more than once by the decoder and scaler software. */
        uint32_t  totalLines;
    } PICDEC_DecodeInfoT;


    /*!
    ****************************************************************************
    ** \brief   Picture format specifier.
    ****************************************************************************
    */
    typedef enum
    {
        PICDEC_FORMAT_PNG = (int32_t) 0,
        PICDEC_FORMAT_BMP,
        PICDEC_FORMAT_JPEG,
        PICDEC_FORMAT_GIF
    } PICDEC_FormatT;

    /*!
    ****************************************************************************
    ** \brief   Rotation options.
    ****************************************************************************
    */
    typedef enum
    {
        PICDEC_ROTATION_0 = (int32_t) 0,
        PICDEC_ROTATION_90_LEFT,
        PICDEC_ROTATION_90_RIGHT,
        PICDEC_ROTATION_180
    } PICDEC_RotationT;

    /*!
    ****************************************************************************
    ** \brief   Set of scaling algorithms supported by the PICTURE decoding lib.
    ****************************************************************************
    */
    typedef enum
    {
        /*! automatic scaler selection */
        PICDEC_SCALER_AUTO      = (int32_t) 0,

        /*! nearest neighbor scaling (fast, low quality) */
        PICDEC_SCALER_BRESENHAM = (int32_t) 1,

        /*! bilinear interpolation (good at upscaling, slow) */
        PICDEC_SCALER_BILINEAR  = (int32_t) 2
    } PICDEC_ScalerT;

    /*!
    ****************************************************************************
    **
    ** \brief Callback function called by decoding engine for providing current
    **        progress of decoding and testing of abort condition.
    **
    ** \param decodeInfoPtr Pointer to decoding information structure provided
    **                      by PICDEC library.
    ** \param optPtr        Optional user argument that has been specified at
    **                      registration of callback function
    **
    ** \return              != 0 if decoding shall be aborted,
    **                      0 if decoding shall not be aborted.
    **
    ** \sa    PICDEC_SetProgressCallback
    **
    ***************************************************************************/
    typedef uint8_t (*PICDEC_ProgressCbT)(const PICDEC_DecodeInfoT*
                                          decodeInfoPtr, void* optPtr);

    /*!
    ****************************************************************************
    ** \brief   General information about picture files.
    ****************************************************************************
    */
    typedef struct
    {
        /*!
        ************************************************************************
        ** \brief Picture format.
        ************************************************************************
        */
        PICDEC_FormatT format;

        /*!
        ************************************************************************
        ** \brief Bits per pixel.
        ************************************************************************
        */
        uint32_t bpp;

        /*!
        ************************************************************************
        ** \brief Horizontal resolution after rotation.
        ************************************************************************
        */
        uint32_t xRes;

        /*!
        ************************************************************************
        ** \brief Vertical resolution after rotation.
        ************************************************************************
        */
        uint32_t yRes;

        /*!
        ************************************************************************
        ** \brief Colour lookup table in RGBA format.
        ************************************************************************
        */
        uint32_t clutArray[256];

    } PICDEC_InfoT;

    /*!
    ****************************************************************************
    ** \brief   Callback function invoked by picture decoder libraries to
    **          read data chunks from a file. This design allows to read files
    **          from different locations, e.g. FILE streams, memory locations,
    **          USB, FATFS, etc.
    ****************************************************************************
    */
    typedef uint32_t (*PICDEC_ReadFileChunkT)
                     (uint8_t* dstPtr, void* filePtr, uint32_t num);

    /*!
    ****************************************************************************
    ** \brief   Callback function invoked by picture decoder libraries to
    **          set the file handle's read position.
    **          The offset parameter gives the offset relative to the whence
    **          parameter. The whence parameter should either be
    **          PICDEC_SEEK_SET, PICDEC_SEEK_CUR or PICDEC_SEEK_END.
    **          The registered function must at least implement the SEEK_SET and
    **          SEEK_CUR functions. It should return 0 on success and an
    **          arbitrary error code otherwise.
    ****************************************************************************
    */
    typedef int32_t  (*PICDEC_FileSeekT)
                     (void* filePtr, int32_t offset, int32_t whence);

    /*!
    *************************************************************************
    ** \brief Open parameters for the PICTURE decoder library.
    **
    ** This data structure covers all parameters which need to be specified
    ** when an instance of the picture decoder is opened.
    ** The file read mechanisms (readFptr and seekFptr) will be immutable
    ** for an open handle.
    **
    *************************************************************************
    */
    typedef struct
    {
        PICDEC_ReadFileChunkT readFptr;    // constant
        PICDEC_FileSeekT seekFptr;         // constant
        PICDEC_ScalerT scaler;             // mutable
        OSDHANDLER_FB_HandleT fbHandle;     // mutable
    } PICDEC_OpenParamsT;


    //***********************************************************************
    //***********************************************************************
    //** Global Data
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** API Functions
    //***********************************************************************
    //***********************************************************************

    #ifdef __cplusplus
    extern "C" {
    #endif

    int32_t             PICDEC_Init(const PICDEC_InitParamsT* initParamsPtr);
    void                PICDEC_Exit(void);
    FAPI_SYS_HandleT    PICDEC_Open
                                (const PICDEC_OpenParamsT* openParamsPtr,
                                 int32_t* errorCodePtr);
    int32_t             PICDEC_Close (FAPI_SYS_HandleT picdecHandle);
    int32_t             PICDEC_SelectPicture
                                (FAPI_SYS_HandleT picdecHandle,
                                 void* fileHandlePtr,
                                 int32_t fileOffset);
    int32_t             PICDEC_SelectScaler
                                (FAPI_SYS_HandleT picdecHandle,
                                 PICDEC_ScalerT scaler);
    int32_t             PICDEC_SelectFramebuffer
                                (FAPI_SYS_HandleT picdecHandle,
                                 OSDHANDLER_FB_HandleT fbHandle);
    int32_t             PICDEC_SelectRotation
                                (FAPI_SYS_HandleT picdecHandle,
                                 PICDEC_RotationT rotation);
    int32_t             PICDEC_GetInfo (FAPI_SYS_HandleT picdecHandle,
                                         PICDEC_InfoT* infoPtr);
    int32_t             PICDEC_Decode (FAPI_SYS_HandleT picdecHandle,
                                        uint32_t xx,
                                        uint32_t yy,
                                        uint32_t xRes,
                                        uint32_t yRes);
    int32_t             PICDEC_SetProgressCallback
                                              (FAPI_SYS_HandleT picdecHandle,
                                               PICDEC_ProgressCbT progressCb,
                                               void* optPtr);

    #ifdef __cplusplus
    }
    #endif

#endif // PICDEC_H
