/*!
*****************************************************************************
**
** \file        ./subsystems/paint/picture/src/picdec_interface.h
**
** \version     $Id: $
**
** \brief       Interface declaration for picture decoding libraries.
**
**              This header declares the interface of the internal decoder
**              libraries of the picture library. Each decoder has to
**              implement this interface. This header is not intended
**              to be used from the application side.
**
** This file contains the declaration of the picture decoding API.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

#ifndef PICDEC_INTERFACE_H
    #define PICDEC_INTERFACE_H

    #include <stdio.h>
    #include <stdint.h>
    #include <fapi/sys_error.h>
    #include <osdhandler/osdhandler.h>
    #include <osdhandler/osdhandler_fb.h>
    #include <picture/picdec.h>

    //***********************************************************************
    //***********************************************************************
    //** Defines and Macros
    //***********************************************************************
    //***********************************************************************

    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************

    /*!
    ****************************************************************************
    ** \brief Interface specification for picture decoding driver library.
    **
    **        This structure has to be provided by each decoding library.
    **
    ****************************************************************************
    */
    typedef struct
    {
        /*!
        ************************************************************************
        ** \brief Pointer to the name of the picture format.
        **
        ** Each driver should register with a name (e.g. "BMP", "PNG", ...)
        ** which the operating environment may use for logging or debugging
        ** purpose.
        ************************************************************************
        */
        const char* formatNamePtr;

        /*!
        ************************************************************************
        ** \brief Initialize the picture decoder.
        ************************************************************************
        */
        int32_t (*initFptr) (void);

        /*!
        ************************************************************************
        ** \brief Exit the picture decoder.
        ************************************************************************
        */
        void (*exitFptr) (void);

        /*!
        ************************************************************************
        ** \brief   Checks whether a file can be processed by the respective
        **          library.
        **
        **          This function returns FAPI_OK if the picture file
        **          can be processed, otherwise some error code.
        **
        ** \param   filePtr
        **              Descriptor of file containing the picture data.
        ** \param   fileOffset
        **              Picture data offset within the file.
        ** \param   readFptr
        **              This function reads data chunks from picPtr.
        ** \param   seekFptr
        **              This function sets the file descriptor position
        **              of the memory source.
        **
        ************************************************************************
        */
        int32_t (*checkFptr)   (void* filePtr,
                                int32_t fileOffset,
                                PICDEC_ReadFileChunkT readFptr,
                                PICDEC_FileSeekT      seekFptr);

        /*!
        ************************************************************************
        ** \brief   Gets information about a picture file.
        **
        ** \param   filePtr
        **              Descriptor of file containing the picture data.
        ** \param   fileOffset
        **              Picture data offset within the file.
        ** \param   readFptr
        **              This function reads data chunks from picPtr.
        ** \param   seekFptr
        **              This function sets the file descriptor position
        **              of the memory source.
        ** \param  infoPtr
        **              Will receive the requested information about the file.
        */
        int32_t (*getInfoFptr) (void* filePtr,
                                int32_t fileOffset,
                                PICDEC_ReadFileChunkT readFptr,
                                PICDEC_FileSeekT      seekFptr,
                                PICDEC_RotationT      rotation,
                                PICDEC_InfoT* infoPtr);

        /*!
        ************************************************************************
        ** \brief   Decodes a picture into a frame buffer.
        **
        ** \param   filePtr
        **              Descriptor of file containing the picture data.
        ** \param   fileOffset
        **              Picture data offset within the file.
        ** \param   readFptr
        **              This function reads data chunks from picPtr.
        ** \param   seekFptr
        **              This function sets the file descriptor position
        **              of the memory source.
        ** \param   fbHandle
        **              Handle for the frame buffer that will be modified.
        ** \param   xx
        **              Horizontal coordinate in the frame buffer.
        ** \param   yy
        **              Vertical coordinate in the frame buffer.
        ** \param   xRes
        **              Desired horizontal resolution of the picture.
        **              The resolution is achieved by software scaling.
        **              If xRes is 0, the native resolution of the file
        **              will be used.
        ** \param   yRes
        **              Desired vertical resolution of the picture.
        **              The resolution is achieved by software scaling.
        **              If xRes is 0, the native resolution of the file
        **              will be used.
        ** \param   rotation
        **              Defines a rotation of the picture in steps of
        **              90 degrees.
        ** \param   scaler
        **              Specifies the prefereed scaler algorithm to be
        **              used while decoding. This option only applies
        **              if the xRes or yRes parameter differs from the
        **              native picture size.
        ** \param   progressCb
        **              Optional progress and abort callback.
        ** \param   progressCbArg
        **              Optional argument passed with progressCb.
        **
        ************************************************************************
        */
        int32_t (*decodeFptr) (void* filePtr,
                               int32_t fileOffset,
                               PICDEC_ReadFileChunkT readFptr,
                               PICDEC_FileSeekT      seekFptr,
                               OSDHANDLER_FB_HandleT fbHandle,
                               uint32_t xx,
                               uint32_t yy,
                               uint32_t xRes,
                               uint32_t yRes,
                               PICDEC_RotationT rotation,
                               PICDEC_ScalerT scaler,
                               PICDEC_ProgressCbT progressCb,
                               void* progressCbArg);
    } PICDEC_INTERFACE_DriverT;


    //***********************************************************************
    //***********************************************************************
    //** Global Data
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** API Functions
    //***********************************************************************
    //***********************************************************************

    #ifdef __cplusplus
    extern "C" {
    #endif


    #ifdef __cplusplus
    }
    #endif

#endif // PICDEC_INTERFACE_H
