/*
*****************************************************************************
**
** \file        ./subsystems/paint/picture/src/picscaler.c
**
** \version     $Id: $
**
** \brief       Scaler dispatcher of the PICTURE decoding library.
**
**              This file dispatches calls for scaling pictures to a set of
**              scaling algorithms.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

#include <stdint.h>
//#include <stdfloat.h>
#include <fapex/abst_types.h>
#include <osdhandler/osdhandler_fb.h>
#include "picdec.h"
#include "picscaler.h"

//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************

/*! order of row buffers */
#define PICSCALER_ROWBUF_ORDER_1_2   0x1
#define PICSCALER_ROWBUF_ORDER_2_1   0x2
#define PICSCALER_ROWBUF_REDUNDANT   0x4


//***************************************************************************
//***************************************************************************
//** Local Structures
//***************************************************************************
//***************************************************************************

/*!
************************************************************************
** \brief   Local structure that keeps unscaled decoded picture data.
**          It may hold several lines. This may be required by some
**          picture decoders for performance reasons.
************************************************************************
*/
typedef struct
{
    uint32_t*   decodePtr;  // base address for decode buffer
    uint32_t    blocks;     // number of row blocks
    uint32_t    blockIdx;   // current block index
    uint32_t    rdIdx;      // row read index in current block
    uint32_t    wrIdx;      // row write index in current block
} picscalerDecodeBufT;

typedef float  float32_t;
typedef double float64_t;


//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Local Functions Declaration
//***************************************************************************
//***************************************************************************
static int32_t picscalerAllocateDecodeBuffer
                        (const PICSCALER_DecodeParamsT* scalerParamsPtr,
                         picscalerDecodeBufT* decodeBufPtr,
                         uint32_t blocks);
static int32_t picscalerGetNextLine (uint32_t** linePtrPtr,
                                     PICSCALER_DecodeParamsT* scalerParamsPtr,
                                     picscalerDecodeBufT* decodeBufPtr);
static int32_t picscalerSkipLines
                        (PICSCALER_DecodeParamsT* scalerParamsPtr,
                         picscalerDecodeBufT* decodeBufPtr,
                         uint32_t skipCnt);
static void picscalerFreeDecodeBuffer
                        (picscalerDecodeBufT* decodeBufPtr);
static int32_t picscalerBresenham(PICSCALER_DecodeParamsT* scalerParamsPtr);
static int32_t picscalerBilinear (PICSCALER_DecodeParamsT* scalerParamsPtr);
static uint32_t picscalerFilterBilinear (float32_t x_diff,
                                         float32_t y_diff,
                                         uint32_t tl,
                                         uint32_t tr,
                                         uint32_t bl,
                                         uint32_t br);

//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

/*!
*****************************************************************************
** \brief   Allocates a source picture decode buffer.
**
** \attention
**          Deallocate with picscalerFreeDecodeBuffer() after processing!
**
** \param   scalerParamsPtr
**              Encapsulates all relevant information for the scaler.
** \param   decodeBufPtr
**              Buffer for unscaled decoded picture data.
** \param   blocks
**              Number of blocks of size srcWidth*bufHeight. If one needs
**              to access rows after having decoded the next burst, this
**              should be set to 2, otherwise 1.
**
** \return
**          - FAPI_OK on success
**          - PICDEC_ERR_OUT_OF_MEMORY if the system is out of memory.
**
*****************************************************************************
*/
static int32_t picscalerAllocateDecodeBuffer
                        (const PICSCALER_DecodeParamsT* scalerParamsPtr,
                         picscalerDecodeBufT* decodeBufPtr,
                         uint32_t blocks)
{
    memset(decodeBufPtr, 0, sizeof(picscalerDecodeBufT));
    decodeBufPtr->decodePtr = FAPI_SYS_MALLOC_CACHED \
        (scalerParamsPtr->srcWidth * scalerParamsPtr->bufHeight * \
         blocks * sizeof(uint32_t));
    if(decodeBufPtr->decodePtr == NULL)
    {
        return(PICDEC_ERR_OUT_OF_MEMORY);
    }
    decodeBufPtr->blocks = blocks;
    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Gets the next line of the source picture.
**
**          The source picture is decoded into a decode buffer that
**          allows to decode multiple lines at once.
**          If data is present in the buffer, a pointer to the next line will
**          be stored to linePtrPtr.
**          Otherwise the buffer will be filled with new data first.
**
** \param   linePtrPtr
**              Will receive a pointer to the next decoded picture line.
** \param   scalerParamsPtr
**              Encapsulates all relevant information for the scaler.
** \param   decodeBufPtr
**              Buffer for unscaled decoded picture data.
**
** \return
**          - #FAPI_OK on success
**          - #PICDEC_ERR_DECODE_ABORT if the decode process was unexpectedly
**              aborted, e.g. due to a corrupted file.
**          - #PICDEC_ERR_ABORT_REQUEST if the decode process was aborted
**              by the application. See PICDEC_SetProgressCallback().
**
*****************************************************************************
*/
static int32_t picscalerGetNextLine (uint32_t** linePtrPtr,
                                     PICSCALER_DecodeParamsT* scalerParamsPtr,
                                     picscalerDecodeBufT* decodeBufPtr)
{
    uint32_t  result = 0;
    uint32_t* buf_ptr = NULL;

    if ( scalerParamsPtr->progressCb != NULL )
    {
        scalerParamsPtr->decodeInfoPtr->decodedLines++;

        // Give progress and test for abort condition.
        if ( scalerParamsPtr->progressCb (scalerParamsPtr->decodeInfoPtr,
                                          scalerParamsPtr->progressCbArg) != 0 )
        {
            *linePtrPtr = NULL;
            return(PICDEC_ERR_ABORT_REQUEST);
        }
    }

    // Check if the decodeBufPtr is empty:
    // new lines have to be decoded by the decoder library:
    if(decodeBufPtr->rdIdx >= decodeBufPtr->wrIdx)
    {
        decodeBufPtr->blockIdx = (decodeBufPtr->blockIdx + 1) % decodeBufPtr->blocks;
        buf_ptr = decodeBufPtr->decodePtr +             \
                        (decodeBufPtr->blockIdx *       \
                         scalerParamsPtr->bufHeight *   \
                         scalerParamsPtr->srcWidth);
        // Fill the next block of the decode buffer:
        result = scalerParamsPtr->readLineFptr(buf_ptr,                     \
                                               scalerParamsPtr->optArgPtr,  \
                                               scalerParamsPtr->bufHeight);
        if(result == 0)
        {
            *linePtrPtr = NULL;
            return (PICDEC_ERR_DECODE_ABORT);
        }
        decodeBufPtr->wrIdx = result;
        decodeBufPtr->rdIdx = 1;
    }
    // The next lines are already decoded in the decodeBufPtr:
    else
    {
        buf_ptr = decodeBufPtr->decodePtr +             \
                        (decodeBufPtr->blockIdx *       \
                         scalerParamsPtr->bufHeight *   \
                         scalerParamsPtr->srcWidth) +   \
                        (decodeBufPtr->rdIdx *          \
                         scalerParamsPtr->srcWidth);
        decodeBufPtr->rdIdx++;
    }
    *linePtrPtr = buf_ptr;
    return (FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Skip some lines.
**
**          If one or more lines do not need to be decoded, these lines can be
**          skipped with this function. Depending on decoder library
**          capabilities, it may be more efficient than decoding them into
**          the decode buffer.
**
** \param   scalerParamsPtr
**              Encapsulates all relevant information for the scaler.
** \param   decodeBufPtr
**              Buffer for unscaled decoded picture data.
** \param   skipCnt
**              Number of lines to skip.
**
** \return
**          - #FAPI_OK on success
**          - #PICDEC_ERR_DECODE_ABORT if the decode process was unexpectedly
**              aborted, e.g. due to a corrupted file.
**          - #PICDEC_ERR_ABORT_REQUEST if the decode process was aborted
**              by the application. See PICDEC_SetProgressCallback().
**
*****************************************************************************
*/
static int32_t picscalerSkipLines
                        (PICSCALER_DecodeParamsT* scalerParamsPtr,
                         picscalerDecodeBufT* decodeBufPtr,
                         uint32_t skipCnt)
{
    // TODO: This function may potentially overwrite a buffer block that is still in use
    // after the call to this function.
    // A viable solution would be to provide an additional block that will be written
    // with dummy data (which may be generated by skipLineFptr) only.
    uint32_t  result = 0;
    uint32_t* buf_ptr = NULL;

    if ( scalerParamsPtr->progressCb != NULL )
    {
        scalerParamsPtr->decodeInfoPtr->decodedLines += skipCnt;

        // Give progress and test for abort condition.
        if ( scalerParamsPtr->progressCb (scalerParamsPtr->decodeInfoPtr,
                                          scalerParamsPtr->progressCbArg) != 0 )
        {
            return(PICDEC_ERR_ABORT_REQUEST);
        }
    }

    while(skipCnt > 0)
    {
        if(decodeBufPtr->rdIdx >= decodeBufPtr->wrIdx)
        {
            decodeBufPtr->blockIdx = (decodeBufPtr->blockIdx + 1) % decodeBufPtr->blocks;
            buf_ptr = decodeBufPtr->decodePtr +             \
                            (decodeBufPtr->blockIdx *       \
                             scalerParamsPtr->bufHeight *   \
                             scalerParamsPtr->srcWidth);
            if(skipCnt >= scalerParamsPtr->bufHeight)
            {   // skip decoding step if possible:
                result = scalerParamsPtr->skipLineFptr      \
                              (buf_ptr,                     \
                               scalerParamsPtr->optArgPtr,  \
                               scalerParamsPtr->bufHeight);
                if(result == 0) return(PICDEC_ERR_DECODE_ABORT);
                decodeBufPtr->rdIdx = result;
                decodeBufPtr->wrIdx = result;
                skipCnt -= result;
            }
            else
            {   // fill buffer with valid data:
                result = scalerParamsPtr->readLineFptr      \
                              (buf_ptr,                     \
                               scalerParamsPtr->optArgPtr,  \
                               scalerParamsPtr->bufHeight);
                if(result == 0) return(PICDEC_ERR_DECODE_ABORT);
                decodeBufPtr->rdIdx = 1;
                decodeBufPtr->wrIdx = result;
                skipCnt--;
            }
        }
        else
        {
            decodeBufPtr->rdIdx++;
            skipCnt--;
        }
    }
    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Free a source picture decode buffer.
**
** \param   scalerParamsPtr
**              Encapsulates all relevant information for the scaler.
** \param   decodeBufPtr
**              Buffer for unscaled decoded picture data.
**
*****************************************************************************
*/
static void picscalerFreeDecodeBuffer
                        (picscalerDecodeBufT* decodeBufPtr)
{
    FAPI_SYS_FREE(decodeBufPtr->decodePtr);
    memset(decodeBufPtr, 0, sizeof(picscalerDecodeBufT));
    return;
}

/*!
*****************************************************************************
** \brief   Apply the bresenham algorithm for scaling.
**
**          This algorithm uses the "nearest neighbor" method. It applies
**          pixel replication when upscaling and pixel dropping when
**          downscaling.
**          This method provides poor quality at low cost (= high speed).
**
** \param   scalerParamsPtr
**              Encapsulates all relevant information for the scaler.
**
** \return
**          - #FAPI_OK on success.
**          - #PICDEC_ERR_BAD_PARAMETER
**              if the scalerParamsPtr is NULL.
**          - #PICDEC_ERR_OUT_OF_MEMORY
**              if there is not enough heap memory available.
**          - some OSDHANDLER related error code if the frame buffer handle
**            is bad.
**
*****************************************************************************
*/
static int32_t picscalerBresenham(PICSCALER_DecodeParamsT* scalerParamsPtr)
{
    int32_t     result           = 0; // result of subroutines
    uint32_t    ii               = 0; // iterator
    uint32_t    kk               = 0; // iterator
    uint32_t    tmp1             = 0; // temporary variable
    uint32_t    tmp2             = 0; // temporary variable
    uint32_t    toggle           = 0; // toggle variable
    uint32_t    tmp_first        = 0; // temporary first pixel in line (for CCIR)
    uint32_t    tmp_last         = 0; // temporary last pixel in line (for CCIR)
    uint32_t    src_w            = 0; // source width
    uint32_t    src_h            = 0; // source height
    uint32_t    dst_x            = 0; // destination x position
    uint32_t    dst_y            = 0; // destination y position
    uint32_t    dst_w            = 0; // destination width
    uint32_t    dst_h            = 0; // destination height
    uint32_t    v_int_part       = 0;
    uint32_t    v_fract_part     = 0;
    uint32_t    v_grid_error     = 0;
    uint32_t    h_int_part       = 0;
    uint32_t    h_fract_part     = 0;
    uint32_t    h_grid_error     = 0;
    uint32_t    src_line_cnt     = 0;
    uint32_t*   src_rowbuf_ptr   = NULL; // temporary buffer for source data
    uint32_t*   dst_rowbuf1_ptr  = NULL; // temporary dst line buffer for ccir
    uint32_t*   dst_rowbuf2_ptr  = NULL; // temporary dst line buffer for ccir
    uint32_t    dst_rowbuf_order = PICSCALER_ROWBUF_ORDER_1_2;
    uint32_t*   src_32_ptr       = NULL;
    uint32_t*   src_2_32_ptr     = NULL;
    uint32_t*   dst_32_ptr       = NULL;
    uint32_t*   dst_2_32_ptr     = NULL;
    uint32_t*   fb_32_ptr        = NULL; // framebuffer base address
    uint32_t    fb_offset        = 0;    // framebuffer pixel offset
    picscalerDecodeBufT decode_buffer;
    PICDEC_RotationT rotation;
    OSDHANDLER_FB_FrameBufferT fb_info;

    if(scalerParamsPtr == NULL)
    {
        return(PICDEC_ERR_BAD_PARAMETER);
    }

    memset(&fb_info, 0, sizeof(OSDHANDLER_FB_FrameBufferT));
    result = OSDHANDLER_FB_GetInfo(scalerParamsPtr->fbHandle, &fb_info);
    if(result)
    {
        return(result);
    }

    src_w    = scalerParamsPtr->srcWidth;
    src_h    = scalerParamsPtr->srcHeight;
    dst_x    = scalerParamsPtr->dstPosX;
    dst_y    = scalerParamsPtr->dstPosY;
    dst_w    = scalerParamsPtr->dstWidth;
    dst_h    = scalerParamsPtr->dstHeight;
    rotation = scalerParamsPtr->rotation;

    // Set up a buffer for the source picture
    result = picscalerAllocateDecodeBuffer (scalerParamsPtr, &decode_buffer, 1);
    if(result != FAPI_OK)
    {
        return(result); // PICDEC_ERR_OUT_OF_MEMORY
    }

    // Get the picture linewise from src_rowbuf_ptr and apply scaling:
    v_int_part = src_h / dst_h;
    v_fract_part = src_h % dst_h;
    v_grid_error = 0;
    src_line_cnt = 1; // counter for lines to be decoded
    h_int_part = src_w / dst_w;
    h_fract_part = src_w % dst_w;

    // Get the address of the pixel in the OSD frame buffer
    // that will be written first (which corresponds to the
    // top left pixel in the source image):
    switch(rotation)
    {
        case(PICDEC_ROTATION_90_LEFT):
            (void)OSDHANDLER_FB_GetPixelAddress(scalerParamsPtr->fbHandle,
                                                dst_x,
                                                (dst_y + dst_w) - 1,
                                                &fb_32_ptr,
                                                &fb_offset);
            break;
        case(PICDEC_ROTATION_90_RIGHT):
            (void)OSDHANDLER_FB_GetPixelAddress(scalerParamsPtr->fbHandle,
                                                (dst_x + dst_h) - 1,
                                                dst_y,
                                                &fb_32_ptr,
                                                &fb_offset);
            break;
        case(PICDEC_ROTATION_180):
            (void)OSDHANDLER_FB_GetPixelAddress(scalerParamsPtr->fbHandle,
                                                (dst_x + dst_w) - 1,
                                                (dst_y + dst_h) - 1,
                                                &fb_32_ptr,
                                                &fb_offset);
            break;
        case(PICDEC_ROTATION_0):
        default:
            (void)OSDHANDLER_FB_GetPixelAddress(scalerParamsPtr->fbHandle,
                                                dst_x,
                                                dst_y,
                                                &fb_32_ptr,
                                                &fb_offset);
            break;
    }

    // /*
    if(fb_info.colourMode == OSDHANDLER_OSD_MODE_16BIT)
    {
        // ********************** CCIR DECODING **********************
        switch(rotation)
        {
            case(PICDEC_ROTATION_90_LEFT):
            case(PICDEC_ROTATION_90_RIGHT):
            // /*
            // allocate two rowbuffers that hold horizontally scaled lines in RGBA format
            // for CCIR chroma interpolation:
            dst_rowbuf1_ptr =  FAPI_SYS_MALLOC_CACHED(dst_w * sizeof(uint32_t));
            if(dst_rowbuf1_ptr == NULL)
            {
                picscalerFreeDecodeBuffer(&decode_buffer);
                return(PICDEC_ERR_OUT_OF_MEMORY);
            }
            dst_rowbuf2_ptr =  FAPI_SYS_MALLOC_CACHED(dst_w * sizeof(uint32_t));
            if(dst_rowbuf2_ptr == NULL)
            {
                FAPI_SYS_FREE(dst_rowbuf1_ptr);
                picscalerFreeDecodeBuffer(&decode_buffer);
                return(PICDEC_ERR_OUT_OF_MEMORY);
            }

            // the last rowbuffer denotes the latest decoded line:
            dst_rowbuf_order = PICSCALER_ROWBUF_ORDER_1_2;

            // number of lines to write:
            ii = dst_h;

            // **************** FIRST LINE OVERLAP ****************
            toggle = 0; // toggle for first line overlap
            switch(rotation)
            {
                case(PICDEC_ROTATION_90_LEFT):
                if( fb_offset ) toggle = 1;
                break;

                case(PICDEC_ROTATION_90_RIGHT):
                default:
                if( ! fb_offset ) toggle = 1;
                break;
            }
            if(toggle)
            {
                result = picscalerGetNextLine(&src_rowbuf_ptr,
                                              scalerParamsPtr,
                                              &decode_buffer);
                if(result)
                {
                    FAPI_SYS_FREE(dst_rowbuf1_ptr);
                    FAPI_SYS_FREE(dst_rowbuf2_ptr);
                    picscalerFreeDecodeBuffer(&decode_buffer);
                    return(result);
                }
                // apply horizontal scaling:
                dst_32_ptr = fb_32_ptr;
                src_32_ptr = src_rowbuf_ptr;
                dst_rowbuf_order = PICSCALER_ROWBUF_ORDER_1_2;
                dst_2_32_ptr = dst_rowbuf2_ptr;
                kk = dst_w;
                switch(rotation)
                {
                    case(PICDEC_ROTATION_90_LEFT):
                    while(kk--)
                    {
                        OSDHANDLER_SetRightRGBA2CCIR(dst_32_ptr, *src_32_ptr);
                        dst_32_ptr -= (fb_info.xResPhy / 2);
                        *dst_2_32_ptr++ = *src_32_ptr;
                        src_32_ptr += h_int_part;
                        h_grid_error += h_fract_part;
                        if(h_grid_error >= dst_w)
                        {
                            h_grid_error -= dst_w;
                            src_32_ptr++;
                        }
                    }
                    fb_32_ptr++;
                    break;

                    case(PICDEC_ROTATION_90_RIGHT):
                    default:
                    while(kk--)
                    {
                        OSDHANDLER_SetLeftRGBA2CCIR(dst_32_ptr, *src_32_ptr);
                        dst_32_ptr += (fb_info.xResPhy / 2);
                        *dst_2_32_ptr++ = *src_32_ptr;
                        src_32_ptr += h_int_part;
                        h_grid_error += h_fract_part;
                        if(h_grid_error >= dst_w)
                        {
                            h_grid_error -= dst_w;
                            src_32_ptr++;
                        }
                    }
                    fb_32_ptr--;
                    break;
                }
                ii--;
                src_line_cnt = v_int_part;
                v_grid_error += v_fract_part;
                // v_grid_error won't get larger than dst_h yet,
                // so there's no need to check.
            }

            tmp2 = ii % 2; // indicates whether the last line overlaps in chroma

            // ******************* PIXEL PAIRS ********************
            // two pixels are decoded into one 32-bit CCIR word with
            // chroma subsampling
            while(ii-- > (tmp2 ? 1 : 0))
            {
                if((ii % 2) == tmp2) // write out pixel pair
                {
                    if(src_line_cnt == 0) // no new line to decode from source
                    {
                        // optimization by checking whether the pixels have been
                        // the same in the previous iteration
                        if(dst_rowbuf_order & PICSCALER_ROWBUF_REDUNDANT)
                        {
                            // just copy the last line:
                            dst_32_ptr = fb_32_ptr;
                            switch(rotation)
                            {
                                case(PICDEC_ROTATION_90_LEFT):
                                for(kk = dst_w; kk > 0; kk--)
                                {
                                    *dst_32_ptr = *(dst_32_ptr - 1);
                                    dst_32_ptr -= (fb_info.xResPhy / 2);
                                }
                                break;

                                case(PICDEC_ROTATION_90_RIGHT):
                                default:
                                for(kk = dst_w; kk > 0; kk--)
                                {
                                    *dst_32_ptr = *(dst_32_ptr + 1);
                                    dst_32_ptr += (fb_info.xResPhy / 2);
                                }
                                break;
                            }
                        }
                        else
                        {
                            // both pixels are the same:
                            src_32_ptr = (dst_rowbuf_order & PICSCALER_ROWBUF_ORDER_1_2) ? \
                                          dst_rowbuf2_ptr : dst_rowbuf1_ptr;
                            dst_32_ptr = fb_32_ptr;
                            for(kk = dst_w; kk > 0; kk--)
                            {
                                *dst_32_ptr = OSDHANDLER_ConvertRGBA2CCIR(*src_32_ptr, *src_32_ptr);
                                src_32_ptr++;
                                switch(rotation)
                                {
                                    case(PICDEC_ROTATION_90_LEFT):
                                    dst_32_ptr -= (fb_info.xResPhy / 2);
                                    break;

                                    case(PICDEC_ROTATION_90_RIGHT):
                                    default:
                                    dst_32_ptr += (fb_info.xResPhy / 2);
                                    break;
                                }
                            }
                            // set a flag that indicates that a line has been written
                            // in which two adjacent pixels are the same:
                            dst_rowbuf_order |= PICSCALER_ROWBUF_REDUNDANT;
                        }
                    }
                    else
                    {
                        // decode next line and interpolate chroma with previous line,
                        // write to both the frame buffer and the rowbuf line:
                        result = picscalerSkipLines(scalerParamsPtr,
                                                        &decode_buffer,
                                                        src_line_cnt - 1);
                        if(result != FAPI_OK)
                        {
                            FAPI_SYS_FREE(dst_rowbuf1_ptr);
                            FAPI_SYS_FREE(dst_rowbuf2_ptr);
                            picscalerFreeDecodeBuffer(&decode_buffer);
                            return(result);
                        }
                        result = picscalerGetNextLine(&src_rowbuf_ptr,
                                                      scalerParamsPtr,
                                                      &decode_buffer);
                        if(result)
                        {
                            FAPI_SYS_FREE(dst_rowbuf1_ptr);
                            FAPI_SYS_FREE(dst_rowbuf2_ptr);
                            picscalerFreeDecodeBuffer(&decode_buffer);
                            return(result);
                        }
                        h_grid_error = 0;
                        src_32_ptr = src_rowbuf_ptr;
                        dst_32_ptr = fb_32_ptr;
                        dst_rowbuf_order = (dst_rowbuf_order & PICSCALER_ROWBUF_ORDER_1_2) ? \
                                            PICSCALER_ROWBUF_ORDER_2_1 : PICSCALER_ROWBUF_ORDER_1_2;
                        dst_2_32_ptr = (dst_rowbuf_order & PICSCALER_ROWBUF_ORDER_1_2) ? \
                                        dst_rowbuf2_ptr : dst_rowbuf1_ptr;
                        src_2_32_ptr = (dst_rowbuf_order & PICSCALER_ROWBUF_ORDER_1_2) ? \
                                        dst_rowbuf1_ptr : dst_rowbuf2_ptr;
                        for(kk = dst_w; kk > 0; kk--)
                        {
                            switch(rotation)
                            {
                                case(PICDEC_ROTATION_90_LEFT):
                                *dst_32_ptr = OSDHANDLER_ConvertRGBA2CCIR(*src_2_32_ptr++, *src_32_ptr);
                                dst_32_ptr -= (fb_info.xResPhy / 2);
                                break;

                                case(PICDEC_ROTATION_90_RIGHT):
                                default:
                                *dst_32_ptr = OSDHANDLER_ConvertRGBA2CCIR(*src_32_ptr, *src_2_32_ptr++);
                                dst_32_ptr += (fb_info.xResPhy / 2);
                                break;
                            }
                            *dst_2_32_ptr++ = *src_32_ptr;
                            src_32_ptr += h_int_part;
                            h_grid_error += h_fract_part;
                            if(h_grid_error >= dst_w)
                            {
                                h_grid_error -= dst_w;
                                src_32_ptr++;
                            }
                        }
                    }
                    // move fb_32_ptr for line stepping:
                    switch(rotation)
                    {
                        case(PICDEC_ROTATION_90_LEFT):
                        fb_32_ptr++;
                        break;

                        case(PICDEC_ROTATION_90_RIGHT):
                        default:
                        fb_32_ptr--;
                        break;
                    }
                }
                else // intermediate pixel: write to rowbuf only
                     // (can't write to fb yet due to chroma subsampling)
                {
                    if(src_line_cnt) // if necessary decode+hscale next line into buffer:
                    {
                        // decode the next needed line:
                        result = picscalerSkipLines(scalerParamsPtr,
                                                        &decode_buffer,
                                                        src_line_cnt - 1);
                        if(result != FAPI_OK)
                        {
                            FAPI_SYS_FREE(dst_rowbuf1_ptr);
                            FAPI_SYS_FREE(dst_rowbuf2_ptr);
                            picscalerFreeDecodeBuffer(&decode_buffer);
                            return(result);
                        }
                        result = picscalerGetNextLine(&src_rowbuf_ptr,
                                                      scalerParamsPtr,
                                                      &decode_buffer);
                        if(result)
                        {
                            FAPI_SYS_FREE(dst_rowbuf1_ptr);
                            FAPI_SYS_FREE(dst_rowbuf2_ptr);
                            picscalerFreeDecodeBuffer(&decode_buffer);
                            return(result);
                        }
                        // apply horizontal scaling:
                        h_grid_error = 0;
                        src_32_ptr = src_rowbuf_ptr;
                        dst_rowbuf_order = (dst_rowbuf_order & PICSCALER_ROWBUF_ORDER_1_2) ? \
                                            PICSCALER_ROWBUF_ORDER_2_1 : PICSCALER_ROWBUF_ORDER_1_2;
                        dst_32_ptr = (dst_rowbuf_order & PICSCALER_ROWBUF_ORDER_1_2) ? \
                                      dst_rowbuf2_ptr : dst_rowbuf1_ptr;
                        for(kk = dst_w; kk > 0; kk--)
                        {
                            *dst_32_ptr++ = *src_32_ptr;
                            src_32_ptr += h_int_part;
                            h_grid_error += h_fract_part;
                            if(h_grid_error >= dst_w)
                            {
                                h_grid_error -= dst_w;
                                src_32_ptr++;
                            }
                        }
                    }
                }
                src_line_cnt = v_int_part;
                v_grid_error += v_fract_part;
                if(v_grid_error >= dst_h)
                {
                    v_grid_error -= dst_h;
                    src_line_cnt++;
                }
            }

            // **************** LAST LINE OVERLAP *****************
            if(tmp2)
            {
                if(src_line_cnt) // if necessary decode+hscale next line into fb:
                {
                    // decode the next needed line:
                    result = picscalerSkipLines(scalerParamsPtr,
                                                    &decode_buffer,
                                                    src_line_cnt - 1);
                    if(result != FAPI_OK)
                    {
                        FAPI_SYS_FREE(dst_rowbuf1_ptr);
                        FAPI_SYS_FREE(dst_rowbuf2_ptr);
                        picscalerFreeDecodeBuffer(&decode_buffer);
                        return(result);
                    }
                    result = picscalerGetNextLine(&src_rowbuf_ptr,
                                                  scalerParamsPtr,
                                                  &decode_buffer);
                    if(result)
                    {
                        FAPI_SYS_FREE(dst_rowbuf1_ptr);
                        FAPI_SYS_FREE(dst_rowbuf2_ptr);
                        picscalerFreeDecodeBuffer(&decode_buffer);
                        return(result);
                    }
                    // apply horizontal scaling:
                    dst_32_ptr = fb_32_ptr;
                    src_32_ptr = src_rowbuf_ptr;
                    kk = dst_w;
                    switch(rotation)
                    {
                        case(PICDEC_ROTATION_90_LEFT):
                        while(kk--)
                        {
                            OSDHANDLER_SetLeftRGBA2CCIR(dst_32_ptr, *src_32_ptr);
                            dst_32_ptr -= (fb_info.xResPhy / 2);
                            src_32_ptr += h_int_part;
                            h_grid_error += h_fract_part;
                            if(h_grid_error >= dst_w)
                            {
                                h_grid_error -= dst_w;
                                src_32_ptr++;
                            }
                        }
                        break;

                        case(PICDEC_ROTATION_90_RIGHT):
                        default:
                        while(kk--)
                        {
                            OSDHANDLER_SetRightRGBA2CCIR(dst_32_ptr, *src_32_ptr);
                            dst_32_ptr += (fb_info.xResPhy / 2);
                            src_32_ptr += h_int_part;
                            h_grid_error += h_fract_part;
                            if(h_grid_error >= dst_w)
                            {
                                h_grid_error -= dst_w;
                                src_32_ptr++;
                            }
                        }
                        break;
                    }
                }
                else // write the latest decoded+hscaled+buffered line into fb:
                {
                    src_32_ptr = (dst_rowbuf_order & PICSCALER_ROWBUF_ORDER_1_2) ? \
                                  dst_rowbuf2_ptr : dst_rowbuf1_ptr;
                    dst_32_ptr = fb_32_ptr;
                    for(kk = dst_w; kk > 0; kk--)
                    {
                        switch(rotation)
                        {
                            case(PICDEC_ROTATION_90_LEFT):
                            OSDHANDLER_SetLeftRGBA2CCIR(dst_32_ptr, *src_32_ptr);
                            dst_32_ptr -= (fb_info.xResPhy / 2);
                            break;

                            case(PICDEC_ROTATION_90_RIGHT):
                            default:
                            OSDHANDLER_SetRightRGBA2CCIR(dst_32_ptr, *src_32_ptr);
                            dst_32_ptr += (fb_info.xResPhy / 2);
                            break;
                        }
                        src_32_ptr++;
                    }
                }
            }
            FAPI_SYS_FREE(dst_rowbuf1_ptr);
            FAPI_SYS_FREE(dst_rowbuf2_ptr);
            break;

            case(PICDEC_ROTATION_180):
            case(PICDEC_ROTATION_0):
            default:
            // ******** CCIR DECODING with 0 or 180 degree rotation ********
            for(ii = dst_h; ii > 0; ii--) // for each destination line
            {
                if(src_line_cnt == 0)
                {
                    // copy the last decoded line:
                    kk = dst_w;
                    dst_32_ptr = fb_32_ptr;
                    switch(rotation)
                    {
                        case(PICDEC_ROTATION_180):
                        if( ! fb_offset )
                        {
                            // interpolate first pixel:
                            OSDHANDLER_SetLeftRGBA2CCIR(dst_32_ptr--, tmp_first);
                            kk--;
                        }
                        tmp2 = kk % 2;
                        kk /= 2;
                        while(kk--)
                        {
                            *dst_32_ptr = *(dst_32_ptr + (fb_info.xResPhy / 2));
                            dst_32_ptr--;
                        }
                        if(tmp2)
                        {
                            // interpolate last pixel:
                            OSDHANDLER_SetRightRGBA2CCIR(dst_32_ptr, tmp_last);
                        }
                        break;

                        case(PICDEC_ROTATION_0):
                        default:
                        if( fb_offset )
                        {
                            // interpolate first pixel:
                            OSDHANDLER_SetRightRGBA2CCIR(dst_32_ptr++, tmp_first);
                            kk--;
                        }
                        memcpy(dst_32_ptr, dst_32_ptr - (fb_info.xResPhy / 2), (kk / 2)*sizeof(uint32_t));
                        dst_32_ptr += kk / 2;
                        if(kk % 2)
                        {
                            // interpolate last pixel:
                            OSDHANDLER_SetLeftRGBA2CCIR(dst_32_ptr, tmp_last);
                        }
                        break;
                    }
                }
                else
                {
                    // decode the next needed line:
                    result = picscalerSkipLines(scalerParamsPtr,
                                                    &decode_buffer,
                                                    src_line_cnt - 1);
                    if(result != FAPI_OK)
                    {
                        picscalerFreeDecodeBuffer(&decode_buffer);
                        return(result);
                    }
                    result = picscalerGetNextLine(&src_rowbuf_ptr,
                                                  scalerParamsPtr,
                                                  &decode_buffer);
                    if(result)
                    {
                        picscalerFreeDecodeBuffer(&decode_buffer);
                        return(result);
                    }
                    // apply horizontal scaling:
                    h_grid_error = 0;
                    kk = dst_w;
                    dst_32_ptr = fb_32_ptr;
                    src_32_ptr = src_rowbuf_ptr;

                    toggle = 0; // toggle for odd pixel processing
                    switch(rotation)
                    {
                        case(PICDEC_ROTATION_180):
                        // If the first pixel (on the right side of the line) has no offset,
                        // it is the left pixel of a pixel pair.
                        if( ! fb_offset )
                        {
                            OSDHANDLER_SetLeftRGBA2CCIR(dst_32_ptr--, *src_32_ptr);
                            toggle = 1;
                        }
                        break;

                        case(PICDEC_ROTATION_0):
                        default:
                        // If t he first pixel (on the left side of the line) has an offset,
                        // it is the right pixel of a pixel pair.
                        if( fb_offset )
                        {
                            OSDHANDLER_SetRightRGBA2CCIR(dst_32_ptr++, *src_32_ptr);
                            toggle = 1;
                        }
                        break;
                    }
                    if(toggle)
                    {
                        tmp_first = *src_32_ptr; // first pixel in line
                        src_32_ptr += h_int_part;
                        h_grid_error += h_fract_part;
                        kk--;
                    }

                    toggle = 0; // toggle for CCIR pixels
                    while(kk-- > 0)
                    {
                        if(toggle)
                        {
                            switch(rotation)
                            {
                                case(PICDEC_ROTATION_180):
                                *dst_32_ptr-- = OSDHANDLER_ConvertRGBA2CCIR (*src_32_ptr, tmp1);
                                break;

                                case(PICDEC_ROTATION_0):
                                default:
                                *dst_32_ptr++ = OSDHANDLER_ConvertRGBA2CCIR (tmp1, *src_32_ptr);
                                break;
                            }
                        }
                        else
                        {
                            tmp1 = *src_32_ptr;
                        }
                        toggle = !toggle;
                        src_32_ptr += h_int_part;
                        h_grid_error += h_fract_part;
                        if(h_grid_error >= dst_w)
                        {
                            h_grid_error -= dst_w;
                            src_32_ptr++;
                        }
                    }
                    if(toggle) // write the last pixel at the border
                    {
                        switch(rotation)
                        {
                            case(PICDEC_ROTATION_180):
                            OSDHANDLER_SetRightRGBA2CCIR(dst_32_ptr, tmp1);
                            break;

                            case(PICDEC_ROTATION_0):
                            default:
                            OSDHANDLER_SetLeftRGBA2CCIR(dst_32_ptr, tmp1);
                            break;
                        }
                        tmp_last = tmp1; // last pixel in line
                    }
                }
                if(rotation == PICDEC_ROTATION_0)
                {
                    fb_32_ptr += (fb_info.xResPhy / 2);
                }
                else if (rotation == PICDEC_ROTATION_180)
                {
                    fb_32_ptr -= (fb_info.xResPhy / 2);
                }
                src_line_cnt = v_int_part;
                v_grid_error += v_fract_part;
                if(v_grid_error >= dst_h)
                {
                    v_grid_error -= dst_h;
                    src_line_cnt++;
                }
            }
            break;
        }
    }
    // */
    else
    {
        // ********************* 32-bit DECODING *********************
        for(ii = dst_h; ii > 0; ii--) // for each target line
        {
            if(src_line_cnt == 0)
            {
                // copy the last decoded line:
                dst_32_ptr = fb_32_ptr;
                switch(rotation)
                {
                    case(PICDEC_ROTATION_90_LEFT):
                        kk = dst_w;
                        while(kk--)
                        {
                            *dst_32_ptr = *(dst_32_ptr - 1);
                            dst_32_ptr -= fb_info.xResPhy;
                        }
                        break;
                    case(PICDEC_ROTATION_90_RIGHT):
                        kk = dst_w;
                        while(kk--)
                        {
                            *dst_32_ptr = *(dst_32_ptr + 1);
                            dst_32_ptr += fb_info.xResPhy;
                        }
                        break;
                    case(PICDEC_ROTATION_180):
                        memcpy((dst_32_ptr - dst_w) + 1,
                               (dst_32_ptr - dst_w) + 1 + fb_info.xResPhy,
                               dst_w*sizeof(uint32_t));
                        break;
                    case(PICDEC_ROTATION_0):
                    default:
                        memcpy(dst_32_ptr, dst_32_ptr - fb_info.xResPhy, dst_w*sizeof(uint32_t));
                        break;
                }
            }
            else
            {
                // decode the next needed line:
                result = picscalerSkipLines(scalerParamsPtr,
                                                &decode_buffer,
                                                src_line_cnt - 1);
                if(result != FAPI_OK)
                {
                    picscalerFreeDecodeBuffer(&decode_buffer);
                    return(result);
                }
                result = picscalerGetNextLine(&src_rowbuf_ptr,
                                              scalerParamsPtr,
                                              &decode_buffer);
                if(result)
                {
                    picscalerFreeDecodeBuffer(&decode_buffer);
                    return(result);
                }
                // apply horizontal scaling:
                h_grid_error = 0;
                dst_32_ptr = fb_32_ptr;
                src_32_ptr = src_rowbuf_ptr;
                for(kk = dst_w; kk > 0; kk--)
                {
                    // switch RGBA / CbYCrA
                    if(fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_RGBA)
                    {
                        *dst_32_ptr = *src_32_ptr;
                    }
                    else // if(fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_CbYCrA)
                    {
                        *dst_32_ptr = OSDHANDLER_ConvertRGBA2CbYCrA_word (*src_32_ptr);
                    }
                    src_32_ptr += h_int_part;
                    h_grid_error += h_fract_part;
                    if(h_grid_error >= dst_w)
                    {
                        h_grid_error -= dst_w;
                        src_32_ptr++;
                    }
                    // move dst_32_ptr:
                    switch(rotation)
                    {
                        case(PICDEC_ROTATION_0):
                            dst_32_ptr++;
                            break;
                        case(PICDEC_ROTATION_90_LEFT):
                            dst_32_ptr -= fb_info.xResPhy;
                            break;
                        case(PICDEC_ROTATION_90_RIGHT):
                            dst_32_ptr += fb_info.xResPhy;
                            break;
                        case(PICDEC_ROTATION_180):
                            dst_32_ptr--;
                            break;
                        default:
                            dst_32_ptr++;
                            break;
                    }
                }
            }
            switch(rotation)
            {
                case(PICDEC_ROTATION_0):
                    fb_32_ptr += fb_info.xResPhy;
                    break;
                case(PICDEC_ROTATION_90_LEFT):
                    fb_32_ptr++;
                    break;
                case(PICDEC_ROTATION_90_RIGHT):
                    fb_32_ptr--;
                    break;
                case(PICDEC_ROTATION_180):
                    fb_32_ptr -= fb_info.xResPhy;
                    break;
                default:
                    fb_32_ptr += fb_info.xResPhy;
                    break;
            }
            src_line_cnt = v_int_part;
            v_grid_error += v_fract_part;
            if(v_grid_error >= dst_h)
            {
                v_grid_error -= dst_h;
                src_line_cnt++;
            }
        }
    }
    picscalerFreeDecodeBuffer(&decode_buffer);
    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Apply the bilinear interpolation for scaling.
**
**          The bilinear interpolation interpolates a pixel in the destination
**          picture by the four closest pixels in the source picture grid while
**          closer pixels are stronger weighted.
**          This allows smooth scaling and delivers better results than the
**          nearest neighbor method. However, it is with less performance.
**          The pixel at x_pos and y_pos indicates always the top left of the
**          four adjacent pixels in the source picture that are used for
**          interpolating a pixel in the destination picture. Therefore it must
**          be guaranteed that x_pos+1 and y_pos+1 is always in range of the
**          source picture coordinates.
**
** \param   scalerParamsPtr
**              Encapsulates all relevant information for the scaler.
**
** \return
**          - #FAPI_OK on success.
**          - #PICDEC_ERR_BAD_PARAMETER
**              if the scalerParamsPtr is NULL.
**          - #PICDEC_ERR_SCALER_MIN_RESOLUTION_2x2
**              if the source picture does not have the minimum resolution of
**              2x2 pixels.
**          - #PICDEC_ERR_OUT_OF_MEMORY
**              if there is not enough heap memory available.
**          - some OSDHANDLER related error code if the frame buffer handle
**            is bad.
**
*****************************************************************************
*/
static int32_t picscalerBilinear(PICSCALER_DecodeParamsT* scalerParamsPtr)
{
    int32_t     result           = 0; // result of subroutines
    uint32_t    ii               = 0; // iterator
    uint32_t    kk               = 0; // iterator
    uint32_t    tmp1             = 0; // temporary variable
    uint32_t    tmp2             = 0; // temporary variable
    uint32_t    toggle           = 0; // toggle variable
    uint32_t    src_w            = 0; // source width
    uint32_t    src_h            = 0; // source height
    uint32_t    dst_x            = 0; // destination x position
    uint32_t    dst_y            = 0; // destination y position
    uint32_t    dst_w            = 0; // destination width
    uint32_t    dst_h            = 0; // destination height
    uint32_t*   src_rowbuf1_ptr  = NULL; // buffer for source lines
    uint32_t*   src_rowbuf2_ptr  = NULL; // buffer for source lines
    uint32_t    src_rowbuf1_idx  = 0;
    uint32_t    src_rowbuf2_idx  = 0;
    uint32_t    src_rowbuf_order = PICSCALER_ROWBUF_ORDER_1_2;
    uint32_t    src_line_cnt     = 0;
    uint32_t*   dst_rowbuf_ptr   = NULL; // buffer for horizontally scaled lines
    uint32_t*   src_32_ptr       = NULL;
    uint32_t*   dst_32_ptr       = NULL;
    float32_t   h_ratio          = 0.0f;
    float32_t   v_ratio          = 0.0f;
    float32_t   x_pos_f          = 0.0f;
    uint32_t    x_pos            = 0;
    float32_t   x_diff           = 0.0f;
    float32_t   y_pos_f          = 0.0f;
    uint32_t    y_pos            = 0;
    float32_t   y_diff           = 0.0f;
    uint32_t*   fb_32_ptr        = NULL; // framebuffer base address
    uint32_t    fb_offset        = 0;    // framebuffer pixel offset
    picscalerDecodeBufT decode_buffer;
    PICDEC_RotationT rotation;
    OSDHANDLER_FB_FrameBufferT fb_info;

    if(scalerParamsPtr == NULL)
    {
        return(PICDEC_ERR_BAD_PARAMETER);
    }

    memset(&fb_info, 0, sizeof(OSDHANDLER_FB_FrameBufferT));
    result = OSDHANDLER_FB_GetInfo(scalerParamsPtr->fbHandle, &fb_info);
    if(result)
    {
        return(result);
    }

    src_w    = scalerParamsPtr->srcWidth;
    src_h    = scalerParamsPtr->srcHeight;
    dst_x    = scalerParamsPtr->dstPosX;
    dst_y    = scalerParamsPtr->dstPosY;
    dst_w    = scalerParamsPtr->dstWidth;
    dst_h    = scalerParamsPtr->dstHeight;
    rotation = scalerParamsPtr->rotation;

    // for bilinear filtering, the resolution must be at least 2x2 pixels:
    if((src_w < 2) || (src_h < 2))
    {
        return(PICDEC_ERR_SCALER_MIN_RESOLUTION_2x2);
    }

    // Set up a buffer for the source picture
    result = picscalerAllocateDecodeBuffer (scalerParamsPtr, &decode_buffer, 2);
    if(result != FAPI_OK)
    {
        return(result); // PICDEC_ERR_OUT_OF_MEMORY
    }

    // Choose the ratio such that the last pixel is mapped to the last two
    // pixels from the source picture while the last pixel is much stronger
    // weighted than the pixel next to the last one:
    h_ratio = (((float32_t)src_w)  - 1.0001f) / (dst_w - 1);
    v_ratio = (((float32_t)src_h) - 1.0001f) / (dst_h - 1);

    // decode the first two source picture lines in advance:
    result = picscalerGetNextLine(&src_rowbuf1_ptr,
                                  scalerParamsPtr,
                                  &decode_buffer);
    if(result)
    {
        picscalerFreeDecodeBuffer(&decode_buffer);
        return(result);
    }
    result = picscalerGetNextLine(&src_rowbuf2_ptr,
                                  scalerParamsPtr,
                                  &decode_buffer);
    if(result)
    {
        picscalerFreeDecodeBuffer(&decode_buffer);
        return(result);
    }
    src_rowbuf1_idx = 0;
    src_rowbuf2_idx = 1;

    // Get the address of the pixel in the OSD frame buffer
    // that will be written first (which corresponds to the
    // top left pixel in the source image):
    switch(rotation)
    {
        case(PICDEC_ROTATION_90_LEFT):
            (void)OSDHANDLER_FB_GetPixelAddress(scalerParamsPtr->fbHandle,
                                                dst_x,
                                                (dst_y + dst_w) - 1,
                                                &fb_32_ptr,
                                                &fb_offset);
            break;
        case(PICDEC_ROTATION_90_RIGHT):
            (void)OSDHANDLER_FB_GetPixelAddress(scalerParamsPtr->fbHandle,
                                                (dst_x + dst_h) - 1,
                                                dst_y,
                                                &fb_32_ptr,
                                                &fb_offset);
            break;
        case(PICDEC_ROTATION_180):
            (void)OSDHANDLER_FB_GetPixelAddress(scalerParamsPtr->fbHandle,
                                                (dst_x + dst_w) - 1,
                                                (dst_y + dst_h) - 1,
                                                &fb_32_ptr,
                                                &fb_offset);
            break;
        case(PICDEC_ROTATION_0):
        default:
            (void)OSDHANDLER_FB_GetPixelAddress(scalerParamsPtr->fbHandle,
                                                dst_x,
                                                dst_y,
                                                &fb_32_ptr,
                                                &fb_offset);
            break;
    }

    if(fb_info.colourMode == OSDHANDLER_OSD_MODE_16BIT)
    {
        // ********************** CCIR DECODING **********************
        switch(rotation)
        {
            case(PICDEC_ROTATION_90_LEFT):
            case(PICDEC_ROTATION_90_RIGHT):

            // allocate a rowbuffer that holds a horizontally scaled line
            //  in RGBA format (needed for CCIR chroma interpolation):
            dst_rowbuf_ptr = FAPI_SYS_MALLOC_CACHED(dst_w * sizeof(uint32_t));
            if(dst_rowbuf_ptr == NULL)
            {
                picscalerFreeDecodeBuffer(&decode_buffer);
                return(PICDEC_ERR_OUT_OF_MEMORY);
            }

            // line counter:
            ii = dst_h;

            // **************** FIRST LINE OVERLAP ****************
            toggle = 0; // toggle for first line overlap
            switch(rotation)
            {
                case(PICDEC_ROTATION_90_LEFT):
                if( fb_offset ) toggle = 1;
                break;

                case(PICDEC_ROTATION_90_RIGHT):
                default:
                if( ! fb_offset ) toggle = 1;
                break;
            }
            if(toggle)
            {
                y_pos = 0; // for the first line
                y_diff = 0; // for the first line
                dst_32_ptr = fb_32_ptr;
                for(kk = 0; kk < dst_w; kk++) // for each target pixel in the current line
                {
                    x_pos_f = kk * h_ratio;
                    x_pos = (uint32_t)x_pos_f; // position in source line
                    x_diff = x_pos_f - (float32_t)x_pos;
                    tmp1 = picscalerFilterBilinear \
                             (x_diff,
                              y_diff,
                              *(src_rowbuf1_ptr + x_pos),
                              *(src_rowbuf1_ptr + x_pos + 1),
                              *(src_rowbuf2_ptr + x_pos),
                              *(src_rowbuf2_ptr + x_pos + 1));
                    switch(rotation)
                    {
                        case(PICDEC_ROTATION_90_LEFT):
                            OSDHANDLER_SetRightRGBA2CCIR(dst_32_ptr, tmp1);
                            dst_32_ptr -= (fb_info.xResPhy / 2);
                        break;

                        case(PICDEC_ROTATION_90_RIGHT):
                        default:
                            OSDHANDLER_SetLeftRGBA2CCIR(dst_32_ptr, tmp1);
                            dst_32_ptr += (fb_info.xResPhy / 2);
                        break;
                    }
                }
                // move fb_32_ptr (line indicator):
                switch(rotation)
                {
                    case(PICDEC_ROTATION_90_LEFT):
                        fb_32_ptr++;
                        break;
                    case(PICDEC_ROTATION_90_RIGHT):
                    default:
                        fb_32_ptr--;
                        break;
                }
                ii--;
            }

            tmp2 = ii % 2; // indicates whether the last line overlaps in chroma

            // ******************* PIXEL PAIRS ********************
            // two pixels are decoded into one 32-bit CCIR word with
            // chroma subsampling
            while(ii > (tmp2 ? 1 : 0))
            {
                y_pos_f = (dst_h - ii) * v_ratio;
                y_pos = (uint32_t)y_pos_f; // y position in source picture
                y_diff = y_pos_f - (float32_t)y_pos;
                src_line_cnt = y_pos - ((src_rowbuf_order == PICSCALER_ROWBUF_ORDER_1_2) ? \
                                        (src_rowbuf1_idx) : (src_rowbuf2_idx));
                ii--;

                if(src_line_cnt == 1)
                {
                    // The next line is already decoded. Decode the line after
                    // the next line and toggle the src_rowbuf_order switch.
                    if(src_rowbuf_order == PICSCALER_ROWBUF_ORDER_1_2)
                    {
                        src_rowbuf_order = PICSCALER_ROWBUF_ORDER_2_1;
                        result = picscalerGetNextLine(&src_rowbuf1_ptr,
                                                      scalerParamsPtr,
                                                      &decode_buffer);
                        if(result)
                        {
                            FAPI_SYS_FREE(dst_rowbuf_ptr);
                            picscalerFreeDecodeBuffer(&decode_buffer);
                            return(result);
                        }
                        src_rowbuf2_idx = y_pos;
                        src_rowbuf1_idx = y_pos + 1;
                    }
                    else
                    {
                        src_rowbuf_order = PICSCALER_ROWBUF_ORDER_1_2;
                        result = picscalerGetNextLine(&src_rowbuf2_ptr,
                                                      scalerParamsPtr,
                                                      &decode_buffer);
                        if(result)
                        {
                            FAPI_SYS_FREE(dst_rowbuf_ptr);
                            picscalerFreeDecodeBuffer(&decode_buffer);
                            return(result);
                        }
                        src_rowbuf1_idx = y_pos;
                        src_rowbuf2_idx = y_pos + 1;
                    }
                }
                else if (src_line_cnt > 1)
                {
                    // first, decode the next needed source picture line, then decode
                    // the line after the next source line into the second row buffer.

                    src_rowbuf_order = PICSCALER_ROWBUF_ORDER_1_2;
                    src_line_cnt--; // one line has already been decoded in advance

                    result = picscalerSkipLines(scalerParamsPtr,
                                                    &decode_buffer,
                                                    src_line_cnt - 1);
                    if(result != FAPI_OK)
                    {
                        FAPI_SYS_FREE(dst_rowbuf_ptr);
                        picscalerFreeDecodeBuffer(&decode_buffer);
                        return(result);
                    }
                    // decode next line and the line after the next source picture line:
                    result = picscalerGetNextLine(&src_rowbuf1_ptr,
                                                  scalerParamsPtr,
                                                  &decode_buffer);
                    if(result)
                    {
                        FAPI_SYS_FREE(dst_rowbuf_ptr);
                        picscalerFreeDecodeBuffer(&decode_buffer);
                        return(result);
                    }
                    result = picscalerGetNextLine(&src_rowbuf2_ptr,
                                                  scalerParamsPtr,
                                                  &decode_buffer);
                    if(result)
                    {
                        FAPI_SYS_FREE(dst_rowbuf_ptr);
                        picscalerFreeDecodeBuffer(&decode_buffer);
                        return(result);
                    }
                    src_rowbuf1_idx = y_pos;
                    src_rowbuf2_idx = y_pos + 1;
                }
                // else go on with the two decoded lines in the source rowbuffers

                if((ii % 2) == tmp2) // write out pixel pair to fb
                {
                    dst_32_ptr = fb_32_ptr;
                    src_32_ptr = dst_rowbuf_ptr;
                    for(kk = 0; kk < dst_w; kk++) // for each target pixel in the current line
                    {
                        x_pos_f = kk * h_ratio;
                        x_pos = (uint32_t)x_pos_f; // position in source line
                        x_diff = x_pos_f - (float32_t)x_pos;
                        if(src_rowbuf_order == PICSCALER_ROWBUF_ORDER_1_2)
                        {
                            tmp1 = picscalerFilterBilinear \
                                     (x_diff,
                                      y_diff,
                                      *(src_rowbuf1_ptr + x_pos),
                                      *(src_rowbuf1_ptr + x_pos + 1),
                                      *(src_rowbuf2_ptr + x_pos),
                                      *(src_rowbuf2_ptr + x_pos + 1));
                        }
                        else
                        {
                            tmp1 = picscalerFilterBilinear \
                                     (x_diff,
                                      y_diff,
                                      *(src_rowbuf2_ptr + x_pos),
                                      *(src_rowbuf2_ptr + x_pos + 1),
                                      *(src_rowbuf1_ptr + x_pos),
                                      *(src_rowbuf1_ptr + x_pos + 1));
                        }
                        switch(rotation)
                        {
                            case(PICDEC_ROTATION_90_LEFT):
                                *dst_32_ptr = OSDHANDLER_ConvertRGBA2CCIR \
                                                (*src_32_ptr++, tmp1);
                                dst_32_ptr -= (fb_info.xResPhy / 2);
                            break;

                            case(PICDEC_ROTATION_90_RIGHT):
                            default:
                                *dst_32_ptr = OSDHANDLER_ConvertRGBA2CCIR \
                                                (tmp1, *src_32_ptr++);
                                dst_32_ptr += (fb_info.xResPhy / 2);
                            break;
                        }
                    }
                    // move fb_32_ptr (line indicator):
                    switch(rotation)
                    {
                        case(PICDEC_ROTATION_90_LEFT):
                            fb_32_ptr++;
                            break;
                        case(PICDEC_ROTATION_90_RIGHT):
                        default:
                            fb_32_ptr--;
                            break;
                    }
                }
                else // intermediate pixel: write to dst rowbuf only
                     // (can't write to fb yet due to chroma subsampling)
                {
                    dst_32_ptr = dst_rowbuf_ptr;
                    for(kk = 0; kk < dst_w; kk++) // for each target pixel in the current line
                    {
                        x_pos_f = kk * h_ratio;
                        x_pos = (uint32_t)x_pos_f; // position in source line
                        x_diff = x_pos_f - (float32_t)x_pos;
                        if(src_rowbuf_order == PICSCALER_ROWBUF_ORDER_1_2)
                        {
                            *dst_32_ptr++ = picscalerFilterBilinear \
                                            (x_diff,
                                             y_diff,
                                             *(src_rowbuf1_ptr + x_pos),
                                             *(src_rowbuf1_ptr + x_pos + 1),
                                             *(src_rowbuf2_ptr + x_pos),
                                             *(src_rowbuf2_ptr + x_pos + 1));
                        }
                        else
                        {
                            *dst_32_ptr++ = picscalerFilterBilinear \
                                            (x_diff,
                                             y_diff,
                                             *(src_rowbuf2_ptr + x_pos),
                                             *(src_rowbuf2_ptr + x_pos + 1),
                                             *(src_rowbuf1_ptr + x_pos),
                                             *(src_rowbuf1_ptr + x_pos + 1));
                        }
                    }
                }
            }

            // **************** LAST LINE OVERLAP *****************
            if(tmp2)
            {
                y_pos_f = ((float32_t)src_h) - 1;
                y_pos = (uint32_t)(y_pos_f - 1); // y position in source picture
                y_diff = 1.0;
                // y_pos_f = (dst_h - 1) * v_ratio;
                // y_pos = (uint32_t)y_pos_f; // y position in source picture
                // y_diff = y_pos_f - (float32_t)y_pos;
                src_line_cnt = y_pos - ((src_rowbuf_order == PICSCALER_ROWBUF_ORDER_1_2) ? \
                                        (src_rowbuf1_idx) : (src_rowbuf2_idx));

                if(src_line_cnt == 1)
                {
                    // The next line is already decoded. Decode the line after
                    // the next line and toggle the src_rowbuf_order switch.
                    if(src_rowbuf_order == PICSCALER_ROWBUF_ORDER_1_2)
                    {
                        src_rowbuf_order = PICSCALER_ROWBUF_ORDER_2_1;
                        result = picscalerGetNextLine(&src_rowbuf1_ptr,
                                                      scalerParamsPtr,
                                                      &decode_buffer);
                        if(result)
                        {
                            FAPI_SYS_FREE(dst_rowbuf_ptr);
                            picscalerFreeDecodeBuffer(&decode_buffer);
                            return(result);
                        }
                        src_rowbuf2_idx = y_pos;
                        src_rowbuf1_idx = y_pos + 1;
                    }
                    else
                    {
                        src_rowbuf_order = PICSCALER_ROWBUF_ORDER_1_2;
                        result = picscalerGetNextLine(&src_rowbuf2_ptr,
                                                      scalerParamsPtr,
                                                      &decode_buffer);
                        if(result)
                        {
                            FAPI_SYS_FREE(dst_rowbuf_ptr);
                            picscalerFreeDecodeBuffer(&decode_buffer);
                            return(result);
                        }
                        src_rowbuf1_idx = y_pos;
                        src_rowbuf2_idx = y_pos + 1;
                    }
                }
                else if (src_line_cnt > 1)
                {
                    // first, decode the next needed source picture line, then decode
                    // the line after the next source line into the second row buffer.

                    src_rowbuf_order = PICSCALER_ROWBUF_ORDER_1_2;
                    src_line_cnt--; // one line has already been decoded in advance

                    result = picscalerSkipLines(scalerParamsPtr,
                                                    &decode_buffer,
                                                    src_line_cnt - 1);
                    if(result != FAPI_OK)
                    {
                        FAPI_SYS_FREE(dst_rowbuf_ptr);
                        picscalerFreeDecodeBuffer(&decode_buffer);
                        return(result);
                    }
                    // decode next line and the line after the next source picture line:
                    result = picscalerGetNextLine(&src_rowbuf1_ptr,
                                                  scalerParamsPtr,
                                                  &decode_buffer);
                    if(result)
                    {
                        FAPI_SYS_FREE(dst_rowbuf_ptr);
                        picscalerFreeDecodeBuffer(&decode_buffer);
                        return(result);
                    }
                    result = picscalerGetNextLine(&src_rowbuf2_ptr,
                                                  scalerParamsPtr,
                                                  &decode_buffer);
                    if(result)
                    {
                        FAPI_SYS_FREE(dst_rowbuf_ptr);
                        picscalerFreeDecodeBuffer(&decode_buffer);
                        return(result);
                    }
                    src_rowbuf1_idx = y_pos;
                    src_rowbuf2_idx = y_pos + 1;
                }
                // else go on with the two decoded lines in the source rowbuffers
                dst_32_ptr = fb_32_ptr;
                for(kk = 0; kk < dst_w; kk++) // for each target pixel in the current line
                {
                    x_pos_f = kk * h_ratio;
                    x_pos = (uint32_t)x_pos_f; // position in source line
                    x_diff = x_pos_f - (float32_t)x_pos;
                    if(src_rowbuf_order == PICSCALER_ROWBUF_ORDER_1_2)
                    {
                        tmp1 = picscalerFilterBilinear \
                                (x_diff,
                                 y_diff,
                                 *(src_rowbuf1_ptr + x_pos),
                                 *(src_rowbuf1_ptr + x_pos + 1),
                                 *(src_rowbuf2_ptr + x_pos),
                                 *(src_rowbuf2_ptr + x_pos + 1));
                    }
                    else
                    {
                        tmp1 = picscalerFilterBilinear \
                                (x_diff,
                                 y_diff,
                                 *(src_rowbuf2_ptr + x_pos),
                                 *(src_rowbuf2_ptr + x_pos + 1),
                                 *(src_rowbuf1_ptr + x_pos),
                                 *(src_rowbuf1_ptr + x_pos + 1));
                    }
                    switch(rotation)
                    {
                        case(PICDEC_ROTATION_90_LEFT):
                            OSDHANDLER_SetLeftRGBA2CCIR(dst_32_ptr, tmp1);
                            dst_32_ptr -= (fb_info.xResPhy / 2);
                        break;

                        case(PICDEC_ROTATION_90_RIGHT):
                        default:
                            OSDHANDLER_SetRightRGBA2CCIR(dst_32_ptr, tmp1);
                            dst_32_ptr += (fb_info.xResPhy / 2);
                        break;
                    }
                }
            }
            FAPI_SYS_FREE(dst_rowbuf_ptr);
            break;

            case(PICDEC_ROTATION_180):
            case(PICDEC_ROTATION_0):
            default:
            for(ii = 0; ii < dst_h; ii++) // for each destination line
            {
                y_pos_f = ii * v_ratio;
                y_pos = (uint32_t)y_pos_f; // y position in source picture
                y_diff = y_pos_f - (float32_t)y_pos;
                src_line_cnt = y_pos - ((src_rowbuf_order == PICSCALER_ROWBUF_ORDER_1_2) ? \
                                        (src_rowbuf1_idx) : (src_rowbuf2_idx));

                if(src_line_cnt == 1)
                {
                    // The next line is already decoded. Decode the line after
                    // the next line and toggle the src_rowbuf_order switch.
                    if(src_rowbuf_order == PICSCALER_ROWBUF_ORDER_1_2)
                    {
                        src_rowbuf_order = PICSCALER_ROWBUF_ORDER_2_1;
                        result = picscalerGetNextLine(&src_rowbuf1_ptr,
                                                      scalerParamsPtr,
                                                      &decode_buffer);
                        if(result)
                        {
                            picscalerFreeDecodeBuffer(&decode_buffer);
                            return(result);
                        }
                        src_rowbuf2_idx = y_pos;
                        src_rowbuf1_idx = y_pos + 1;
                    }
                    else
                    {
                        src_rowbuf_order = PICSCALER_ROWBUF_ORDER_1_2;
                        result = picscalerGetNextLine(&src_rowbuf2_ptr,
                                                      scalerParamsPtr,
                                                      &decode_buffer);
                        if(result)
                        {
                            picscalerFreeDecodeBuffer(&decode_buffer);
                            return(result);
                        }
                        src_rowbuf1_idx = y_pos;
                        src_rowbuf2_idx = y_pos + 1;
                    }
                }
                else if (src_line_cnt > 1)
                {
                    // first, decode the next needed source picture line, then decode
                    // the line after the next source line into the second row buffer.

                    src_rowbuf_order = PICSCALER_ROWBUF_ORDER_1_2;
                    src_line_cnt--; // one line has already been decoded in advance

                    result = picscalerSkipLines(scalerParamsPtr,
                                                    &decode_buffer,
                                                    src_line_cnt - 1);
                    if(result != FAPI_OK)
                    {
                        picscalerFreeDecodeBuffer(&decode_buffer);
                        return(result);
                    }
                    // decode next line and the line after the next source picture line:
                    result = picscalerGetNextLine(&src_rowbuf1_ptr,
                                                  scalerParamsPtr,
                                                  &decode_buffer);
                    if(result)
                    {
                        picscalerFreeDecodeBuffer(&decode_buffer);
                        return(result);
                    }
                    result = picscalerGetNextLine(&src_rowbuf2_ptr,
                                                  scalerParamsPtr,
                                                  &decode_buffer);
                    if(result)
                    {
                        picscalerFreeDecodeBuffer(&decode_buffer);
                        return(result);
                    }
                    src_rowbuf1_idx = y_pos;
                    src_rowbuf2_idx = y_pos + 1;
                }
                // else go on with the two decoded lines in the source rowbuffers

                kk = dst_w;
                dst_32_ptr = fb_32_ptr;

                // The toggle indicates whether the first pixel will be
                // written separately:
                toggle = 0;
                switch(rotation)
                {
                    case(PICDEC_ROTATION_180):
                    if( ! fb_offset ) toggle = 1;
                    break;

                    case(PICDEC_ROTATION_0):
                    default:
                    if( fb_offset ) toggle = 1;
                    break;
                }
                if(toggle)
                {
                    x_pos_f = 0.0f;
                    x_pos = 0;
                    x_diff = 0.0f;

                    if(src_rowbuf_order == PICSCALER_ROWBUF_ORDER_1_2)
                    {
                        tmp1 = picscalerFilterBilinear \
                                 (x_diff,
                                  y_diff,
                                  *(src_rowbuf1_ptr + x_pos),
                                  *(src_rowbuf1_ptr + x_pos + 1),
                                  *(src_rowbuf2_ptr + x_pos),
                                  *(src_rowbuf2_ptr + x_pos + 1));
                    }
                    else
                    {
                        tmp1 = picscalerFilterBilinear \
                                 (x_diff,
                                  y_diff,
                                  *(src_rowbuf2_ptr + x_pos),
                                  *(src_rowbuf2_ptr + x_pos + 1),
                                  *(src_rowbuf1_ptr + x_pos),
                                  *(src_rowbuf1_ptr + x_pos + 1));
                    }
                    switch(rotation)
                    {
                        case(PICDEC_ROTATION_180):
                        OSDHANDLER_SetLeftRGBA2CCIR(dst_32_ptr--, tmp1);
                        break;

                        case(PICDEC_ROTATION_0):
                        default:
                        OSDHANDLER_SetRightRGBA2CCIR(dst_32_ptr++, tmp1);
                        break;
                    }
                    kk--;
                }
                toggle = 0; // toggle for CCIR pixels
                while(kk > 0)
                {
                    x_pos_f = (dst_w - kk) * h_ratio;
                    x_pos = (uint32_t)x_pos_f; // position in source line
                    x_diff = x_pos_f - (float32_t)x_pos;
                    kk--;

                    if(toggle)
                    {
                        if(src_rowbuf_order == PICSCALER_ROWBUF_ORDER_1_2)
                        {
                            tmp2 = picscalerFilterBilinear \
                                     (x_diff,
                                      y_diff,
                                      *(src_rowbuf1_ptr + x_pos),
                                      *(src_rowbuf1_ptr + x_pos + 1),
                                      *(src_rowbuf2_ptr + x_pos),
                                      *(src_rowbuf2_ptr + x_pos + 1));
                        }
                        else
                        {
                            tmp2 = picscalerFilterBilinear \
                                     (x_diff,
                                      y_diff,
                                      *(src_rowbuf2_ptr + x_pos),
                                      *(src_rowbuf2_ptr + x_pos + 1),
                                      *(src_rowbuf1_ptr + x_pos),
                                      *(src_rowbuf1_ptr + x_pos + 1));
                        }
                        switch(rotation)
                        {
                            case(PICDEC_ROTATION_180):
                            *dst_32_ptr-- = OSDHANDLER_ConvertRGBA2CCIR (tmp2, tmp1);
                            break;

                            case(PICDEC_ROTATION_0):
                            default:
                            *dst_32_ptr++ = OSDHANDLER_ConvertRGBA2CCIR (tmp1, tmp2);
                            break;
                        }
                    }
                    else
                    {
                        if(src_rowbuf_order == PICSCALER_ROWBUF_ORDER_1_2)
                        {
                            tmp1 = picscalerFilterBilinear \
                                     (x_diff,
                                      y_diff,
                                      *(src_rowbuf1_ptr + x_pos),
                                      *(src_rowbuf1_ptr + x_pos + 1),
                                      *(src_rowbuf2_ptr + x_pos),
                                      *(src_rowbuf2_ptr + x_pos + 1));
                        }
                        else
                        {
                            tmp1 = picscalerFilterBilinear \
                                     (x_diff,
                                      y_diff,
                                      *(src_rowbuf2_ptr + x_pos),
                                      *(src_rowbuf2_ptr + x_pos + 1),
                                      *(src_rowbuf1_ptr + x_pos),
                                      *(src_rowbuf1_ptr + x_pos + 1));
                        }
                    }
                    toggle = !toggle;
                }
                if(toggle) // write the last pixel at the border
                {
                    switch(rotation)
                    {
                        case(PICDEC_ROTATION_180):
                        OSDHANDLER_SetRightRGBA2CCIR(dst_32_ptr, tmp1);
                        break;

                        case(PICDEC_ROTATION_0):
                        default:
                        OSDHANDLER_SetLeftRGBA2CCIR(dst_32_ptr, tmp1);
                        break;
                    }
                }
                switch(rotation)
                {
                    case(PICDEC_ROTATION_180):
                    fb_32_ptr -= (fb_info.xResPhy / 2);
                    break;

                    case(PICDEC_ROTATION_0):
                    default:
                    fb_32_ptr += (fb_info.xResPhy / 2);
                    break;
                }
            }
            break;
        }
    }
    else
    {
        // ********************* 32-bit DECODING *********************
        for(ii = 0; ii <dst_h; ii++) // for each target line
        {
            y_pos_f = ii * v_ratio;
            y_pos = (uint32_t)y_pos_f; // y position in source picture
            y_diff = y_pos_f - (float32_t)y_pos;
            src_line_cnt = y_pos - ((src_rowbuf_order == PICSCALER_ROWBUF_ORDER_1_2) ? \
                                    (src_rowbuf1_idx) : (src_rowbuf2_idx));

            if(src_line_cnt == 1)
            {
                // The next line is already decoded. Decode the line after
                // the next line and toggle the src_rowbuf_order switch.
                if(src_rowbuf_order == PICSCALER_ROWBUF_ORDER_1_2)
                    {
                        src_rowbuf_order = PICSCALER_ROWBUF_ORDER_2_1;
                        result = picscalerGetNextLine(&src_rowbuf1_ptr,
                                                      scalerParamsPtr,
                                                      &decode_buffer);
                        if(result)
                        {
                            picscalerFreeDecodeBuffer(&decode_buffer);
                            return(result);
                        }
                        src_rowbuf2_idx = y_pos;
                        src_rowbuf1_idx = y_pos + 1;
                    }
                    else
                    {
                        src_rowbuf_order = PICSCALER_ROWBUF_ORDER_1_2;
                        result = picscalerGetNextLine(&src_rowbuf2_ptr,
                                                      scalerParamsPtr,
                                                      &decode_buffer);
                        if(result)
                        {
                            picscalerFreeDecodeBuffer(&decode_buffer);
                            return(result);
                        }
                        src_rowbuf1_idx = y_pos;
                        src_rowbuf2_idx = y_pos + 1;
                    }
            }
            else if (src_line_cnt > 1)
            {
                // first, decode the next needed source picture line, then decode
                // the line after the next source line into the second row buffer.

                src_rowbuf_order = PICSCALER_ROWBUF_ORDER_1_2;
                src_line_cnt--; // one line has already been decoded in advance

                result = picscalerSkipLines(scalerParamsPtr,
                                                &decode_buffer,
                                                src_line_cnt - 1);
                if(result != FAPI_OK)
                {
                    picscalerFreeDecodeBuffer(&decode_buffer);
                    return(result);
                }
                // decode next line and the line after the next source picture line:
                result = picscalerGetNextLine(&src_rowbuf1_ptr,
                                              scalerParamsPtr,
                                              &decode_buffer);
                if(result)
                {
                    picscalerFreeDecodeBuffer(&decode_buffer);
                    return(result);
                }
                result = picscalerGetNextLine(&src_rowbuf2_ptr,
                                              scalerParamsPtr,
                                              &decode_buffer);
                if(result)
                {
                    picscalerFreeDecodeBuffer(&decode_buffer);
                    return(result);
                }
                src_rowbuf1_idx = y_pos;
                src_rowbuf2_idx = y_pos + 1;
            }
            // else go on with the two decoded lines in the source rowbuffers

            dst_32_ptr = fb_32_ptr;
            if(src_rowbuf_order == PICSCALER_ROWBUF_ORDER_1_2)
            {
                for(kk = 0; kk < dst_w; kk++) // for each target pixel in the current line
                {
                    x_pos_f = kk * h_ratio;
                    x_pos = (uint32_t)x_pos_f; // position in source line
                    x_diff = x_pos_f - (float32_t)x_pos;
                    if(fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_RGBA)
                    {
                        *dst_32_ptr = picscalerFilterBilinear \
                                        (x_diff,
                                         y_diff,
                                         *(src_rowbuf1_ptr + x_pos),
                                         *(src_rowbuf1_ptr + x_pos + 1),
                                         *(src_rowbuf2_ptr + x_pos),
                                         *(src_rowbuf2_ptr + x_pos + 1));
                    }
                    else
                    {
                        *dst_32_ptr = OSDHANDLER_ConvertRGBA2CbYCrA_word \
                                        (picscalerFilterBilinear \
                                            (x_diff,
                                             y_diff,
                                             *(src_rowbuf1_ptr + x_pos),
                                             *(src_rowbuf1_ptr + x_pos + 1),
                                             *(src_rowbuf2_ptr + x_pos),
                                             *(src_rowbuf2_ptr + x_pos + 1)));
                    }
                    // move dst_32_ptr:
                    switch(rotation)
                    {
                        case(PICDEC_ROTATION_0):
                            dst_32_ptr++;
                            break;
                        case(PICDEC_ROTATION_90_LEFT):
                            dst_32_ptr -= fb_info.xResPhy;
                            break;
                        case(PICDEC_ROTATION_90_RIGHT):
                            dst_32_ptr += fb_info.xResPhy;
                            break;
                        case(PICDEC_ROTATION_180):
                            dst_32_ptr--;
                            break;
                        default:
                            dst_32_ptr++;
                            break;
                    }
                }
            }
            else // if(src_rowbuf_order == PICSCALER_ROWBUF_ORDER_2_1)
            {
                for(kk = 0; kk < dst_w; kk++) // for each target pixel in the current line
                {
                    x_pos_f = kk * h_ratio;
                    x_pos = (uint32_t)x_pos_f; // position in source line
                    x_diff = x_pos_f - (float32_t)x_pos;
                    if(fb_info.layoutMode == OSDHANDLER_OSD_LAYOUT_RGBA)
                    {
                        *dst_32_ptr = picscalerFilterBilinear \
                                        (x_diff,
                                         y_diff,
                                         *(src_rowbuf2_ptr + x_pos),
                                         *(src_rowbuf2_ptr + x_pos + 1),
                                         *(src_rowbuf1_ptr + x_pos),
                                         *(src_rowbuf1_ptr + x_pos + 1));
                    }
                    else
                    {
                        *dst_32_ptr = OSDHANDLER_ConvertRGBA2CbYCrA_word \
                                        (picscalerFilterBilinear \
                                            (x_diff,
                                             y_diff,
                                             *(src_rowbuf2_ptr + x_pos),
                                             *(src_rowbuf2_ptr + x_pos + 1),
                                             *(src_rowbuf1_ptr + x_pos),
                                             *(src_rowbuf1_ptr + x_pos + 1)));
                    }
                    // move dst_32_ptr:
                    switch(rotation)
                    {
                        case(PICDEC_ROTATION_0):
                            dst_32_ptr++;
                            break;
                        case(PICDEC_ROTATION_90_LEFT):
                            dst_32_ptr -= fb_info.xResPhy;
                            break;
                        case(PICDEC_ROTATION_90_RIGHT):
                            dst_32_ptr += fb_info.xResPhy;
                            break;
                        case(PICDEC_ROTATION_180):
                            dst_32_ptr--;
                            break;
                        default:
                            dst_32_ptr++;
                            break;
                    }
                }
            }

            // move fb_32_ptr (line indicator):
            switch(rotation)
            {
                case(PICDEC_ROTATION_0):
                    fb_32_ptr += fb_info.xResPhy;
                    break;
                case(PICDEC_ROTATION_90_LEFT):
                    fb_32_ptr++;
                    break;
                case(PICDEC_ROTATION_90_RIGHT):
                    fb_32_ptr--;
                    break;
                case(PICDEC_ROTATION_180):
                    fb_32_ptr -= fb_info.xResPhy;
                    break;
                default:
                    fb_32_ptr += fb_info.xResPhy;
                    break;
            }
        }
    }
    picscalerFreeDecodeBuffer(&decode_buffer);
    return(FAPI_OK);
}

/*!
*******************************************************************************
**
** \brief Bilinear interpolation between 4 pixels.
**
** Calculation of a weighted bilinear interpolation between 4 pixels.
**
** \param x_diff    The weight in x-direction (0 <= x_diff <= 1).
** \param y_diff    The weight in y-direction (0 <= y_diff <= 1).
** \param tl        Top left pixel value in RGBA 32-bit format.
** \param tr        Top right pixel value in RGBA 32-bit format.
** \param bl        Bottom left pixel value in RGBA 32-bit format.
** \param br        Bottom right pixel value in RGBA 32-bit format.
**
** \return          Interpolated pixel value in RGBA 32-bit format.
**
*******************************************************************************
*/
static uint32_t picscalerFilterBilinear (float32_t x_diff,
                                             float32_t y_diff,
                                             uint32_t tl,
                                             uint32_t tr,
                                             uint32_t bl,
                                             uint32_t br)
{
    uint32_t RR = 0;
    uint32_t GG = 0;
    uint32_t BB = 0;
    uint32_t AA = 0;

    float32_t RR_f  = 0.0;
    float32_t GG_f  = 0.0;
    float32_t BB_f  = 0.0;
    float32_t AA_f  = 0.0;

    float32_t RR_tl = (tl >> 24) & 0xFF;
    float32_t GG_tl = (tl >> 16) & 0xFF;
    float32_t BB_tl = (tl >>  8) & 0xFF;
    float32_t AA_tl = (tl >>  0) & 0xFF;

    float32_t RR_tr = (tr >> 24) & 0xFF;
    float32_t GG_tr = (tr >> 16) & 0xFF;
    float32_t BB_tr = (tr >>  8) & 0xFF;
    float32_t AA_tr = (tr >>  0) & 0xFF;

    float32_t RR_bl = (bl >> 24) & 0xFF;
    float32_t GG_bl = (bl >> 16) & 0xFF;
    float32_t BB_bl = (bl >>  8) & 0xFF;
    float32_t AA_bl = (bl >>  0) & 0xFF;

    float32_t RR_br = (br >> 24) & 0xFF;
    float32_t GG_br = (br >> 16) & 0xFF;
    float32_t BB_br = (br >>  8) & 0xFF;
    float32_t AA_br = (br >>  0) & 0xFF;

    RR_f = (RR_tl * (1.0f - x_diff) * (1.0f - y_diff)) + \
           (RR_tr * (x_diff)        * (1.0f - y_diff)) + \
           (RR_bl * (1.0f - x_diff) * (y_diff)       ) + \
           (RR_br * (x_diff)        * (y_diff)       );

    GG_f = (GG_tl * (1.0f - x_diff) * (1.0f - y_diff)) + \
           (GG_tr * (x_diff)        * (1.0f - y_diff)) + \
           (GG_bl * (1.0f - x_diff) * (y_diff)       ) + \
           (GG_br * (x_diff)        * (y_diff)       );

    BB_f = (BB_tl * (1.0f - x_diff) * (1.0f - y_diff)) + \
           (BB_tr * (x_diff)        * (1.0f - y_diff)) + \
           (BB_bl * (1.0f - x_diff) * (y_diff)       ) + \
           (BB_br * (x_diff)        * (y_diff)       );

    AA_f = (AA_tl * (1.0f - x_diff) * (1.0f - y_diff)) + \
           (AA_tr * (x_diff)        * (1.0f - y_diff)) + \
           (AA_bl * (1.0f - x_diff) * (y_diff)       ) + \
           (AA_br * (x_diff)        * (y_diff)       );

    RR = (uint32_t) (RR_f);
    GG = (uint32_t) (GG_f);
    BB = (uint32_t) (BB_f);
    AA = (uint32_t) (AA_f);

    return((RR << 24) | (GG << 16) | (BB << 8) | AA);
}


//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

/*!
*****************************************************************************
** \brief   Start to scale and decode a picture into a frame buffer.
**
**          This function will decode a picture into an OSD frame buffer.
**          It is recommended to apply this function in a picture decoder
**          library unless the respective picture format does not imply some
**          specific properties that would allow to optimize speed on scaling
**          due to the encoding of the picture.
**
** \param   scalerParamsPtr
**              Encapsulates all relevant information for the scaler.
**
** \return
**          - #FAPI_OK on success.
**          - #PICDEC_ERR_UNSUPPORTED_OSD_COLOUR_LAYOUT_MODE
**              if the combination of OSD colour and OSD layout mode is
**              not supported by the scaler lib.
**              Common combinations for scalers are 32-bit in RGBA and CbYCrA
**              layouts as well as the 16-bit CCIR mode.
**          - #PICDEC_ERR_OUT_OF_FB_BOUNDS
**              if the scaled picture is out of bounds of the frame buffer.
**          - #PICDEC_ERR_OUT_OF_MEMORY
**              if there was not enough heap memory available for the scaler.
**          - some OSDHANDLER related error code if the frame buffer handle
**            is bad.
**          - some scaler function specific error code if the scaler failed.
**
*****************************************************************************
*/
int32_t PICSCALER_Decode(PICSCALER_DecodeParamsT* scalerParamsPtr)
{
    OSDHANDLER_FB_FrameBufferT fb_info;
    int32_t result = 0;

    // destination coordinates and resolution on framebuffer:
    uint32_t dst_x = scalerParamsPtr->dstPosX;
    uint32_t dst_y = scalerParamsPtr->dstPosY;
    uint32_t dst_w = scalerParamsPtr->dstWidth;
    uint32_t dst_h = scalerParamsPtr->dstHeight;

    memset(&fb_info, 0, sizeof(OSDHANDLER_FB_FrameBufferT));

    result = OSDHANDLER_FB_GetInfo(scalerParamsPtr->fbHandle, &fb_info);
    if(result)
    {
        return(result);
    }

    if(fb_info.colourMode == OSDHANDLER_OSD_MODE_32_24BIT)
    {
        if((fb_info.layoutMode != OSDHANDLER_OSD_LAYOUT_RGBA)
        && (fb_info.layoutMode != OSDHANDLER_OSD_LAYOUT_CbYCrA))
        {
            return(PICDEC_ERR_UNSUPPORTED_OSD_COLOUR_LAYOUT_MODE);
        }
    }
    else if ((fb_info.colourMode != OSDHANDLER_OSD_MODE_16BIT)
          || (fb_info.layoutMode != OSDHANDLER_OSD_LAYOUT_CbYCrA))
    {
        return(PICDEC_ERR_UNSUPPORTED_OSD_COLOUR_LAYOUT_MODE);
    }

    switch(scalerParamsPtr->rotation)
    {
        case(PICDEC_ROTATION_90_LEFT):
        case(PICDEC_ROTATION_90_RIGHT):
            if(((dst_x + dst_h) > fb_info.xResApp)
            || ((dst_y + dst_w) > fb_info.yResApp))
            {
                return(PICDEC_ERR_OUT_OF_FB_BOUNDS);
            }
            break;
        case(PICDEC_ROTATION_0):
        case(PICDEC_ROTATION_180):
        default:
            if(((dst_x + dst_w) > fb_info.xResApp)
            || ((dst_y + dst_h) > fb_info.yResApp))
            {
                return(PICDEC_ERR_OUT_OF_FB_BOUNDS);
            }
            break;
    }

    switch(scalerParamsPtr->scaler)
    {
        case(PICDEC_SCALER_BRESENHAM):
        result = picscalerBresenham(scalerParamsPtr);
        break;

        case(PICDEC_SCALER_BILINEAR):
        result = picscalerBilinear(scalerParamsPtr);
        break;

        case(PICDEC_SCALER_AUTO):
        default:
        if((dst_w > scalerParamsPtr->srcWidth)
        || (dst_h > scalerParamsPtr->srcHeight))
        {
            result = picscalerBilinear(scalerParamsPtr);
        }
        else
        {
            result = picscalerBresenham(scalerParamsPtr);
        }
        break;
    }
    return(result);
}
