/*!
*****************************************************************************
**
** \file        ./subsystems/paint/picture/src/picscaler.h
**
** \version     $Id: $
**
** \brief       Interface declaration for the scaler of the picture library.
**
**              This header declares the interface of the internal scaler
**              of the picture library. A decoder that is able to decode
**              a picture in a line-wise fashion may use this scaler
**              instead of implementing its own scaler.
**              However, if the picture format implies some properties that
**              may be used to speed up rescaling processes, it is recommended
**              to apply scaling manually in the respective decoder library.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

#ifndef PICSCALER_H
    #define PICSCALER_H

    #include <stdio.h>
    #include <stdint.h>
    #include <fapi/sys_error.h>
    #include <osdhandler/osdhandler_fb.h>
    #include <picture/picdec.h>


    //***********************************************************************
    //***********************************************************************
    //** Defines and Macros
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************

    /*!
    ************************************************************************
    ** \brief   Reads one or several lines from the source image into a
    **          buffer in 32bit RGBA format.
    **          Returns the number of decoded lines.
    ************************************************************************
    */
    typedef uint32_t (*PICSCALER_readLineT) (void* lineBufPtr,
                                             void* optArgPtr,
                                             uint32_t lineCnt);

    /*!
    ************************************************************************
    ** \brief   Skips one or several lines from the source image. The decoder
    **          lib does not have to calculate the image data and does not
    **          have to write to lineBufPtr.
    **          Returns the number of skipped lines.
    ************************************************************************
    */
    typedef uint32_t (*PICSCALER_skipLineT) (void* lineBufPtr,
                                             void* optArgPtr,
                                             uint32_t lineCnt);

    /*!
    ************************************************************************
    ** \brief   All arguments that are needed by a scaling algorithm to
    **          process the relevant image data are packed into this
    **          structure for efficiency reasons when passing arguments
    **          along the scaler libs.
    ************************************************************************
    */
    typedef struct
    {
        OSDHANDLER_FB_HandleT fbHandle;
        uint32_t srcWidth;
        uint32_t srcHeight;
        uint32_t dstPosX;
        uint32_t dstPosY;
        uint32_t dstWidth;
        uint32_t dstHeight;
        uint32_t bufHeight;
        PICDEC_RotationT rotation;
        PICDEC_ScalerT scaler;
        PICSCALER_readLineT readLineFptr;
        PICSCALER_skipLineT skipLineFptr;
        void* optArgPtr;
        PICDEC_ProgressCbT progressCb;
        void* progressCbArg;
        PICDEC_DecodeInfoT* decodeInfoPtr;
    } PICSCALER_DecodeParamsT;


    //***********************************************************************
    //***********************************************************************
    //** Global Data
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** API Functions
    //***********************************************************************
    //***********************************************************************

    #ifdef __cplusplus
    extern "C" {
    #endif

    int32_t PICSCALER_Decode(PICSCALER_DecodeParamsT* scalerParamsPtr);

    #ifdef __cplusplus
    }
    #endif

#endif // PICSCALER_H
