/*!
*****************************************************************************
**
** \file        ./middleware/pvr/src/pvr_play.h
** 
** \version     $Id: pvr_play.h 5874 2010-03-02 14:42:55Z wlaris $
**
** \brief       PVR middleware playback engine.
**
** This file contains the declaration of the PVR playback API.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

#ifndef PVR_PLAY_H
#define PVR_PLAY_H

//#include <fapi/sys_driver.h>
//#include <fapi/drv_dma.h>
//#include <fapi/drv_tsd.h>

#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_libdma.h"

#include "pvr_priv.h"
#include "pvr.h"

#include "api_f.h"
//***********************************************************************
//***********************************************************************
//** Defines and Macros
//***********************************************************************
//***********************************************************************

#define PVR_PLAY_CHUNKTIME_AVERAGE_NUMS 8

#define PVR_PLAY_CHUNKBUFFER_CHUNKS     4          //!< Buffer size in chunks.
#define PVR_PLAY_CHUNKBUFFER_PKTS       ((PVR_PLAY_CHUNKBUFFER_CHUNKS * \
                                          PVR_CHUNK_SIZE) / PVR_TSPKT_SIZE)
                                                    //!< In packets.

#define PVR_FILE_RW_LOCK       (void)FAPI_SYS_GET_SEMAPHORE(inst->fileOpSem, FAPI_SYS_SUSPEND);
#define PVR_FILE_RW_UNLOCK           FAPI_SYS_SET_SEMAPHORE(inst->fileOpSem, FAPI_SYS_NO_SUSPEND);

//***********************************************************************
//***********************************************************************
//** Data types
//***********************************************************************
//***********************************************************************

typedef enum
{
    PLAY_STATE_NONE = -1,
    
    /* main playback modes */
    PLAY_STATE_STOPPED = 0,
    PLAY_STATE_NORMAL,
    PLAY_STATE_FAST_ALL,
    PLAY_STATE_FAST_IFRAMES,
    PLAY_STATE_FAST_NONE,
    PLAY_STATE_FAST_CONTROLLED,
    PLAY_STATE_SLOW,
    PLAY_STATE_PAUSE,
    
    /* transitional modes */
    PLAY_STATE_TO_STOP,
    PLAY_STATE_TO_NORMAL,
    PLAY_STATE_TO_FAST_ALL,
    PLAY_STATE_TO_FAST_IFRAMES,
    PLAY_STATE_TO_FAST_NONE,
    PLAY_STATE_TO_FAST_CONTROLLED,
    PLAY_STATE_TO_SLOW,
    PLAY_STATE_TO_PAUSE,
    PLAY_STATE_SEEK,
    
    PLAY_STATE_NUMS,
    
} PVR_PlayStateT;

typedef enum /* Feed thread operation states */
{
    FEED_STATE_STOPPED = 0,     //!< Feed thread has been stopped.
    FEED_STATE_ACTIVE,          //!< Feed thread is active (will feed any data
                                //!< from chunk buffer).
    FEED_STATE_TERMINATED       //!< Feed thread has been prepared for 
                                //!< termination.
    
} PVR_PlayFeedStateT;

typedef enum
{
    PLAY_STOPMODE_BLANK,
    PLAY_STOPMODE_FREEZE,
    PLAY_STOPMODE_NONE

} PVR_PlayStopMode;

typedef enum /* Feed thread commands. */
{
    FEED_CMD_NONE = 0,          //!< No command.
    FEED_CMD_START,             //!< Enable feeding operation.
    FEED_CMD_STOP,              //!< Disable feeding operation.
    FEED_CMD_TERMINATE,         //!< Prepare for destruction.
    FEED_CMD_DISCARDDATA        //!< Discard remaining data in chunk buffer,
                                //!< does not affect operational state.
    
} PVR_PlayFeedCmdT;

typedef struct
{
    PVR_PlayStateT  state;
    PVR_PlayStateT  lastState;
    uint8_t         stateIdx;
    uint8_t         reqStateIdx;
    
} PVR_PlayStateInfoT;

typedef struct
{
    uint8_t                 trackIndex;     /* Track index -> caller thread */
    uint32_t                trackData;      /* optional track data (default: 0)
                                               -> caller thread */
    uint16_t                pid;            /* PID -> playback thread */
    PVR_TrackTypeT          trackType;      /* Type -> playback thread */
    uint8_t                 format;         /* Format -> playback thread */
    void*                   decHandle;      /* Target decoder handle */
    uint8_t                 started;        /* decoder started status -> playback thread */
    int32_t                 startResult;    /* return value of decoder start
                                               request, valid if started == 1 */
    
} pvrPlayStreamT;

typedef void (*PVR_PLAY_STATEHANDLER_CB)(void* dataPtr,
                                         const PVR_PlayStateInfoT*
                                         stateInfoPtr);

typedef struct
{
    PVR_FastModeConfigT      fastModeConfig[PVR_MAX_FAST_MODES];
    PVR_DecoderTypeT         decoderStartOrder[PVR_DECTYPE_NUMS];
    
    PVR_PLAY_STATEHANDLER_CB stateHandler[PLAY_STATE_NUMS];
    
    #if (DEBUG >= 4)
    const char_t*            stateString[PLAY_STATE_NUMS];
    #endif /* (DEBUG >= 4) */
    
} pvrPlayDataT;

/* PVR playback specific handle */
typedef struct
{
    pvrPlayDataT*           playDataPtr;                       /* pointer to global play data */
    FAPEX_SYS_ThreadT       readThread;                        /* playback reading thread */
    FAPEX_SYS_ThreadT       feedThread;                        /* playback feeding thread */
    FAPEX_SYS_SemaphoreT    playSemaphore;                     /* sempahore for protecting data  */
    FAPEX_SYS_HandleT       recHandle;                         /* related recording engine inst. */
    F_FILE*                 filePtr;                           /* pointer to playback file       */
    //FAPEX_SYS_SemaphoreT    fileOpSem;                         /* semaphore for control atomtic read & write:Martin:20100703*/
    char_t                  filePath[PVR_FULL_FILEPATH_LENGTH + 1];/* temp. storage of file path */
    PVR_PlayOpenT           openParams;                        /* open params of the handle */
    PVR_FileInfoT           fileInfo;                          /* file info structure            */
    PVR_PlayStateT          state;                             /* state of playback engine       */
    PVR_PlayStateT          lastState;                         /* Last state of playback engine  */
    uint8_t                 stateIndex;                        /* state-internal index                    */
    uint8_t                 reqStateIdx;                       /* requested state index          */
    uint32_t                size;                              /* physical size                  */
    uint32_t                offset;                            /* physical offset to begin       */
    uint32_t                position;                          /* physical position              */
    uint8_t                 internalMode;                      /* Indicates internal playback    */
    PVR_PlaybackFeedOutput  feedOutput;                        /* Pointer to feed-data function  */
    PVR_HeaderPacketT       header;                            /* current header                 */
    void*                   tmpHeader;                         /* temporal header storage        */
    uint8_t                 slowFeed;
    
    /* parameters related to chunk buffer */
    void*                   chunkBuffer;                       /* buffer to store data chunks */
    uint32_t                chunkBufferRp;                     /* chunk buffer read pointer in packets */
    uint32_t                chunkBufferWp;                     /* chunk buffer read pointer in packets */
    
    /* parameters related to data decryption option */
    void*                   decryptBuffer;                     /* buffer to store data chunks before 
                                                                  they are being decrypted */
    FAPEX_DMA_CryptoKeysT   decryptKeys;
    
    /* parameters related to feed thread */
    FAPEX_SYS_SemaphoreT    feedAckSemaphore;                  /* sempahore for feeding cmd ack. */
    FAPEX_SYS_MailboxT      feedAbortMailbox;
    uint8_t                 feedAborted;
    PVR_PlayFeedStateT      feedState;
    PVR_PlayFeedCmdT        feedCmd;
    uint32_t                feedMinSpace;
    uint32_t                feedXferPkts;
    uint64_t                feedXferTimestamp;                 /* in units of us */
    
    /* parameters for seek modes */
    uint32_t                seekPosition;                      /* new position for seek mode     */
    int32_t                 seekChunks;                        /* chunks for seek mode           */
    uint16_t                newNameCounter;                    /* for name seek                  */
    uint32_t                seekTime;                          /* req. time for time seek        */

    pvrPlayStreamT          stream[PVR_DECTYPE_NUMS];

    FAPEX_SYS_HandleT       iframeBmHandle;
    FAPEX_SYS_HandleT       iframeTsdHandle;
    int32_t                 iframeTsdFd;
    uint32_t                iframeBufferMMapBaseAddress;       /*The driver layer bm mmap buffer */
    uint8_t                 iframeBufferActive;

    uint8_t*                iframeTmpBuffer;

    uint16_t                lastNoDataCounter;                 /* last no data counter           */
    uint16_t                lastTagCounter;                    /* last tag changed counter       */
    uint16_t                lastNameCounter;                   /* last name changed counter      */
    uint32_t                lastFilling;                       /* last filling of control buffer */
    uint32_t                fillingCounter;                    /* counter for check filling      */
    uint32_t                sleepCounter;                      /* counter for extra sleep        */
    uint64_t                readHddTime;                       /* time to read a data chunk      */
    uint64_t                feedTsdTime;                       /* time to feed a data chunk TSD  */
    uint64_t                iFrameStamp;                       /* timestamp used for I-frame
                                                                  display time calculation */
    uint32_t                iFrameTime;                        /* time to parse and copy I-Frame */
    uint8_t                 enableFrameView;                   /* enables display of I-Frames    */
    uint32_t                splitSize;                         /* split size [chunks]            */
    uint16_t                numberOfSplits;                    /* number of splits               */
    uint16_t                splitNumber;                       /* number of current file split   */

    #if 0
    /* playback markers not supported yet! */
    FAPI_SYS_SemaphoreT     markerSemaphore;                   /* sempahore for protecting data  */
    uint8_t                 enableMarker;                      /* flag to enable marker          */
    uint8_t                 nextCutFlag;                       /* flag to indicate a valid cut   */
    uint32_t                nextCutPosIn;                      /* in point of next cut           */
    uint32_t                nextCutPosOut;                     /* out point of next cut          */
    uint8_t                 abRepeatFlag;                      /* flag to indicate a valid AB repeat area */
    uint32_t                abRepeatPosIn;                     /* in point of next AB repeat area */
    uint32_t                abRepeatPosOut;                    /* out point of next AB repeat area */
    uint32_t*               numMarkerPtr;                      /* number of valid markers        */
    PVR_PlayMarkerT*        markerPtr;                         /* pointer to marker entries      */
    #endif
    
    uint8_t                 normEod;                           /* tells if end of playback data has been reached */
    uint32_t                normEndFilling[PVR_DECTYPE_NUMS];
    uint64_t                normEodTimeStamp;
    
    uint32_t                chunkTime[PVR_PLAY_CHUNKTIME_AVERAGE_NUMS];
    uint16_t                chunkTimeNums;
    uint16_t                chunkTimeOffset;
    uint32_t                chunkTimeLastStamp;
    uint8_t                 chunkTimeFirst;

    /* parameters for mode PLAY_STATE_NORMAL */
    uint32_t                read_data_ts;
    uint32_t                read_data_header_time;

    /* parameters for mode PLAY_STATE_FAST_NONE */
    int32_t                 fastNoneCurrFactor;
    uint32_t                fastNoneLastPlayTime;               /* in ms */
    uint64_t                fastNoneLastStamp;                  /* in ms */
    uint64_t                fastNoneNextStamp;                  /* in ms */
    int32_t                 fastNoneChunkSeek;
    
    /* parameters for mode PLAY_STATE_FAST_CONTROLLED */
    uint8_t                 fastCtrlRestart;
    uint32_t                fastCtrlChunksFed;
    uint64_t                fastCtrlLastStamp;
    uint32_t                fastCtrlLastPlayTime;               /* in ms */
    int32_t                 fastCtrlCurrFactor;
    int32_t                 fastCtrlChunkSeek;

    /*Martin added for record the init chunkSeek*/
    int32_t                 fastCtrlChunkSeekInit;              
    
    /* parameters for mode PLAY_STATE_FAST_IFRAMES */
    uint64_t                fastIFramesLastStamp;
    uint32_t                fastIFramesLastPlayTime;            /* in ms */
    int32_t                 fastIFramesCurrFactor;
    int32_t                 fastIFramesChunkSeek;
    
    // TODO: share parameters among multiple fast modes.
    
    uint8_t                 decSyncActive;
    
    #ifdef PVR_PLAY_PERIODIC_FEED
    
    uint32_t                feedStuffingPkt[PVR_TSPKT_SIZE_LW];
    uint64_t                feedLastFeedTime;
    
    #endif /* PVR_PLAY_PERIODIC_FEED */

    /* parameters for mmap */
    uint8_t*                input_port_base;
    uint8_t*                input_map;
    uint32_t                map_size;
    
    
} pvrPlayHandleT;



//***********************************************************************
//***********************************************************************
//** Global Data
//***********************************************************************
//***********************************************************************


//***********************************************************************
//***********************************************************************
//** API Functions
//***********************************************************************
//***********************************************************************

#ifdef __cplusplus
extern "C" {
#endif

int32_t          pvrPlayInit    (void);
int32_t          pvrPlayExit    (void);

int32_t          pvrPlayGetFastModeConfig(uint16_t index,
                                          PVR_FastModeConfigT* configPtr);
int32_t          pvrPlaySetFastModeConfig(uint16_t index,
                                          const PVR_FastModeConfigT* configPtr);

int32_t          pvrPlayOpen    (const PVR_PlayOpenT* paramsPtr,
                                 FAPEX_SYS_HandleT recHandle,
                                 FAPEX_SYS_HandleT* handlePtr);
int32_t          pvrPlayClose   (FAPEX_SYS_HandleT handle);
int32_t          pvrPlayNotifyRecordingClose (FAPEX_SYS_HandleT handle,
                                              FAPEX_SYS_HandleT recHandle);
int32_t          pvrPlaySetMode (FAPEX_SYS_HandleT handle, PVR_PlayModeT mode,
                                 uint8_t modeIndex);
int32_t          pvrPlaySeek    (FAPEX_SYS_HandleT handle, PVR_SeekModeT mode,
                                 uint32_t value);
int32_t          pvrPlaySetTrack(FAPEX_SYS_HandleT handle,
                                 PVR_DecoderTypeT decoderType, uint8_t track,
                                 uint32_t trackData);
int32_t          pvrPlayGetTrack(FAPEX_SYS_HandleT handle,
                                 PVR_DecoderTypeT decoderType,
                                 uint8_t* trackPtr);
int32_t          pvrPlayGetFileInfo(FAPEX_SYS_HandleT handle,
                                    PVR_FileInfoT* fileInfoPtr);
int32_t          pvrPlayGetPlayInfo(FAPEX_SYS_HandleT handle,
                                    PVR_PlayInfoT* playInfoPtr);
int32_t          pvrPlayGetTrackInfo(FAPEX_SYS_HandleT handle,
                                     PVR_TrackInfoT* trackInfoPtr);
uint32_t         pvrPlayGetChunkDuration(FAPEX_SYS_HandleT handle);
int32_t          pvrPlayEnableFrameView(FAPEX_SYS_HandleT handle,
                                        uint8_t enable);
int32_t          pvrPlayGetState(FAPEX_SYS_HandleT handle,
                                 PVR_PlayStateInfoT* stateInfoPtr);
uint8_t          pvrPlayGetTsd          (FAPEX_SYS_HandleT handle);
int32_t          pvrPlaySetPlayFunction(FAPEX_SYS_HandleT handle,
                                        PVR_PlaybackFeedOutput function);
int32_t          pvrPlaySetInternalMode(FAPEX_SYS_HandleT handle);
int32_t          pvrPlayEnableDecryption(FAPEX_SYS_HandleT handle,
                                         const PVR_CryptoParamsT* paramsPtr);

int32_t          pvrPlayFeedOpen       (pvrPlayHandleT* inst);
int32_t          pvrPlayFeedClose      (pvrPlayHandleT* inst);
int32_t          pvrPlayFeedSetState   (pvrPlayHandleT* inst,
                                        PVR_PlayFeedStateT state,
                                        uint32_t minTargetBufSpace);
int32_t          pvrPlayFeedSendCmd    (pvrPlayHandleT* inst,
                                        PVR_PlayFeedCmdT cmd);

uint8_t          pvrTrackIsActive      (pvrPlayHandleT* inst,
                                        PVR_DecoderTypeT type);
uint32_t         pvrDecoderGetBufLevel (pvrPlayHandleT* inst,
                                        PVR_DecoderTypeT type);
uint32_t         pvrDecoderGetBufSize  (pvrPlayHandleT* inst,
                                        PVR_DecoderTypeT type);
 void     pvrTrackStart         (pvrPlayHandleT* inst,
                                       PVR_DecoderTypeT type,
                                       PVR_DecoderModeT mode,
                                       uint8_t modeIndex);
 void     pvrTrackStop          (pvrPlayHandleT* inst,
                                       PVR_DecoderTypeT type,
                                       PVR_PlayStopMode stopMode);


#ifdef __cplusplus
}
#endif

#endif /* PVR_PLAY_H */
