/*
*****************************************************************************
**
** \file        ./middleware/pvr/src/pvr_play.c
**
** \version     $Id: pvr_play_feed.c 5874 2010-03-02 14:42:55Z wlaris $
**
** \brief       PVR middleware playback engine - feeding implementation.
**
** This files contains the implementation of the PVR playback API.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

//#define DEBUG 1

#include <assert.h>

//#include <fapi/sys_services.h>
//#include <fapi/drv_timer.h>

#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"

#include "pvr.h"
#include "pvr_priv.h"
#include "pvr_play.h"

/*lint -save -e716 -e717 -e788 */

/* NOTE: the following LINT statements have been suppressed:
- Info 716: while(1) ...
- Info 717: do ... while(0);
- Info 788: enum constant 'anyConst' not used within defaulted switch
*/

//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************

#define PVR_PLAY_FEEDTHREAD_STACKSIZE   1024        //!< In 32-bit longwords.
#define PVR_PLAY_FEEDTHREAD_NAME        "pvrPlay_Feed"

/* The following define is required for recorded audio channels.
   In that case the max number of data to feed must be limited to
   the audio buffer size. 64 means around 12KBytes and should be small enough. */
//Martin: change from 64 to 64*4 for the audio buffer size is 64KBytes
#define PVR_PLAY_FEEDPKTS_MAX               64

#define PVR_PLAY_DST_BUF_FULL_SLEEPTIME     20          /* in ms */
#define PVR_PLAY_SRC_BUF_EMPTY_SLEEPTIME    5           /* in ms */

/* maximum feeding datarate in slow feeding mode (startup only) */
#define PVR_PLAY_DATARATE_MAX_SLOWFEED      2000        /* in bytes/ms */
#define PVR_PLAY_DATARATE_SLEEP_SLOWFEED      30        /* in ms */

#define PVR_PLAY_DATARATE_CHECK_PKTS         384        /* in ms */
#define PVR_PLAY_DATARATE_MAX_NORMAL        7000        /* in bytes/ms */
/* Martin changed 5 -> 10 at 20100628*/
#define PVR_PLAY_DATARATE_SLEEP_NORMAL         10/*5*/      /* in ms */

#ifdef PVR_PLAY_PERIODIC_FEED

#define PVR_PLAY_FEED_INTERVAL_MAX         100          /* in ms */

#endif /* PVR_PLAY_PERIODIC_FEED */

//***************************************************************************
//***************************************************************************
//** Local Structures
//***************************************************************************
//***************************************************************************

//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************

//***************************************************************************
//***************************************************************************
//** Local Functions Declaration
//***************************************************************************
//***************************************************************************

static void     pvrPlayFeedThreadFunction (void* optArgPtr);

static uint8_t  pvrPlayFeedCheckAbortion  (pvrPlayHandleT* inst,
                                           uint32_t timeout);
static uint32_t pvrPlayFeedData           (pvrPlayHandleT* inst);

static uint8_t  pvrCheckBufferFilling     (pvrPlayHandleT* inst,
                                           uint32_t freeSpace,
                                           uint8_t enableVideoRestart);

#ifdef PVR_PLAY_PERIODIC_FEED

static uint32_t pvrPlayFeedStuffing       (pvrPlayHandleT* inst);

#endif /* PVR_PLAY_PERIODIC_FEED */

//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************

//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

int32_t pvrPlayFeedOpen (pvrPlayHandleT* inst)
{
    int32_t error_code = FAPI_OK;

    inst->feedAckSemaphore     = NULL;
    inst->feedAbortMailbox     = NULL;
    inst->feedThread           = NULL;

    inst->feedAborted           = 0;
    inst->feedState             = FEED_STATE_STOPPED;
    inst->feedCmd               = FEED_CMD_NONE;
    inst->feedMinSpace          = 0;

    inst->feedXferPkts          = 0;
    inst->feedXferTimestamp     = 0;

    #ifdef PVR_PLAY_PERIODIC_FEED

    inst->feedLastFeedTime      = 0;

    /* Generate stuffing packet that will be regularly fed to TSD */
    memset(inst->feedStuffingPkt, 0xFF, PVR_TSPKT_SIZE);

    /* Choose big endian format, does not need to be swapped by feeding
       function. */
    inst->feedStuffingPkt[0] = 0x471FFF10;

    #endif /* PVR_PLAY_PERIODIC_FEED */

    do
    {
        /* create semaphore for acknowledging of commands to feeding thread */
        inst->feedAckSemaphore = FAPEX_SYS_CreateSemaphore(0);
        if ( inst->feedAckSemaphore == 0 )
        {
            error_code = SUBSYS_PVR_ERR_SEMAPHORE_CREATE;
            break;
        }

        /* create mailbox for feeding thread process communication. */
        inst->feedAbortMailbox = FAPEX_SYS_CreateMailbox(1);
        if ( inst->feedAbortMailbox == 0 )
        {
            // TODO: modify error code?
            error_code = SUBSYS_PVR_ERR_SEMAPHORE_CREATE;
            break;
        }

        inst->feedThread = FAPEX_SYS_CreateThread(0,
                                            PVR_PLAY_FEEDTHREAD_STACKSIZE,
                                            inst->openParams.feedThreadPriority,
                                            pvrPlayFeedThreadFunction,
                                            inst, 0, PVR_PLAY_FEEDTHREAD_NAME);

        if ( inst->feedThread == NULL )
        {
            error_code = SUBSYS_PVR_ERR_THREAD_ERROR;
            break;
        }
    }
    while ( 0 );

    if ( error_code != FAPI_OK )
    {
        (void)pvrPlayFeedClose(inst);
    }

    return error_code;
}

int32_t pvrPlayFeedClose(pvrPlayHandleT* inst)
{
    if ( inst->feedThread != NULL )
    {
        (void)pvrPlayFeedSetState(inst, FEED_STATE_TERMINATED, 0);
        (void)FAPEX_SYS_WaitEndThread(inst->feedThread);
        (void)FAPEX_SYS_DestroyThread(inst->feedThread);
    }

    if ( inst->feedAckSemaphore != 0 )
    {
        FAPI_SYS_DESTROY_SEMAPHORE(inst->feedAckSemaphore);
    }

    if ( inst->feedAbortMailbox != 0 )
    {
        (void)FAPEX_SYS_DestroyMailbox(inst->feedAbortMailbox);
    }

    return FAPI_OK;
}

int32_t pvrPlayFeedSetState (pvrPlayHandleT* inst, PVR_PlayFeedStateT state,
                             uint32_t minTargetBufSpace)
{
    int32_t          error_code = FAPI_OK;
    PVR_PlayFeedCmdT cmd;

    if ( state != inst->feedState )
    {
        switch ( state )
        {
            default:
            case FEED_STATE_STOPPED:
                cmd = FEED_CMD_STOP;
                break;

            case FEED_STATE_ACTIVE:
                cmd = FEED_CMD_START;
                break;

            case FEED_STATE_TERMINATED:
                cmd = FEED_CMD_TERMINATE;
                break;
        }

        error_code = pvrPlayFeedSendCmd(inst, cmd);
    }

    inst->feedMinSpace = minTargetBufSpace;

    return error_code;
}

int32_t pvrPlayFeedSendCmd (pvrPlayHandleT* inst, PVR_PlayFeedCmdT cmd)
{
    uint8_t mailBoxMsg = 0;
    int32_t error_code;

    inst->feedCmd = cmd;

    //Martin changed "==" -> "!=" at 20100619
    if ( FAPEX_SYS_SetMailbox(inst->feedAbortMailbox, &mailBoxMsg, FAPEX_SYS_NO_SUSPEND)
                != FAPI_OK )
    {
        //error_code = RTOS_GetLastError();
        error_code = FAPEX_SYS_GetLastError();
        FAPI_SYS_PRINT_MSG("[FEED] pvrPlayFeedSendCmd: send cmd (%d) failed (error_code %d)\n", cmd, error_code);
    }
    else
    {
        /* wait for feed thread to process command */
        FAPI_SYS_PRINT_DEBUG(2, "[FEED] pvrPlayFeedSendCmd: send cmd (%d) success, wait feed thread ack\n", cmd);
        FAPI_SYS_GET_SEMAPHORE(inst->feedAckSemaphore, FAPI_SYS_SUSPEND);
        FAPI_SYS_PRINT_DEBUG(2, "[FEED] pvrPlayFeedSendCmd: get ack from feed thread success\n");
        error_code = FAPI_OK;
    }

    return error_code;
}

static void pvrPlayFeedThreadFunction(void* optArgPtr)
{
    pvrPlayHandleT* inst = (pvrPlayHandleT*)optArgPtr;
    uint32_t        sleepTime;

    #ifdef PVR_PLAY_PERIODIC_FEED
    uint32_t        tmpSleepTime;
    #endif /* PVR_PLAY_PERIODIC_FEED */

    FAPI_SYS_PRINT_DEBUG(1, "[FEED] pvrPlayFeedThreadFunction: In\n");

    while ( inst->feedState != FEED_STATE_TERMINATED )
    {
        while ( !(pvrPlayFeedCheckAbortion(inst, FAPEX_SYS_NO_SUSPEND)) )
        {
            if ( inst->feedState == FEED_STATE_ACTIVE )
            {
                sleepTime = pvrPlayFeedData(inst);
            }
            else
            {
                sleepTime = 1000; /* default sleeptime */
            }

            #ifdef PVR_PLAY_PERIODIC_FEED

            tmpSleepTime = pvrPlayFeedStuffing(inst);

            if ( tmpSleepTime < sleepTime )
            {
                sleepTime = tmpSleepTime;
            }

            #endif /* PVR_PLAY_PERIODIC_FEED */

            if ( sleepTime > 0 )
            {
                if ( pvrPlayFeedCheckAbortion(inst, sleepTime) )
                {
                    FAPI_SYS_PRINT_DEBUG(2, "[Feed] pvrPlayFeedThreadFunction:abortion detected\n");
                    break;
                }
            }
        }

        /* ----- perform command processing -------------------------------- */
        switch ( inst->feedCmd )
        {
            default:
            case FEED_CMD_NONE:
                /* do nothing */
                break;

            case FEED_CMD_START:
                inst->feedState = FEED_STATE_ACTIVE;
                break;

            case FEED_CMD_STOP:
                inst->feedState = FEED_STATE_STOPPED;
                break;

            case FEED_CMD_TERMINATE:
                inst->feedState = FEED_STATE_TERMINATED;
                FAPI_SYS_PRINT_DEBUG(1, "[FEED] pvrPlayFeedThreadFunction: FEED_STATE_TERMINATED Got\n");
                break;

            case FEED_CMD_DISCARDDATA:
                /* Empty buffer by moving read pointer to write pointer. */
                FAPI_SYS_PRINT_DEBUG(2, "[FEED] pvrPlayFeedThreadFunction: make chunk buffer empty\n");
                inst->chunkBufferRp = inst->chunkBufferWp;
                break;
        }

        inst->feedAborted = 0;

        /* acknowledge command after processing */
        FAPI_SYS_SET_SEMAPHORE(inst->feedAckSemaphore, FAPI_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(1, "[FEED] pvrPlayFeedThreadFunction: Out\n");
}

static uint8_t pvrPlayFeedCheckAbortion (pvrPlayHandleT* inst, uint32_t timeout)
{
    uint8_t mailBoxMsg;

    if ( inst->feedAborted == 0 )
    {
        if ( FAPEX_SYS_GetMailbox(inst->feedAbortMailbox, &mailBoxMsg, timeout)
             == FAPI_OK )
        {
            inst->feedAborted = 1;
        }
    }
    return inst->feedAborted;
}
/*lint -save -e826 */
static uint32_t pvrPlayFeedData (pvrPlayHandleT* inst)
{
    uint32_t    numPkts;
    uint32_t    feedPkts;
    uint32_t    bufRp;
    uint32_t    bufWp;
    uint32_t*   dataPtr;
    uint32_t    sleepTime;
    uint32_t    targetBufSpace;
    uint8_t     res;
    uint64_t    currTime;
    uint32_t    interval;       //!< in us.
    uint32_t    maxRate;

    uint32_t    curRate;

    //DEBUG//

    /* test if data for feeding is available */
    bufWp = inst->chunkBufferWp;
    bufRp = inst->chunkBufferRp;

    if ( bufWp >= bufRp )
    {
        numPkts = bufWp - bufRp;
    }
    else
    {
        numPkts = PVR_PLAY_CHUNKBUFFER_PKTS - (bufRp - bufWp);
    }
    FAPI_SYS_PRINT_DEBUG(2, "[FEED] pvrPlayFeedData: In : position[%04d], bfRp[%04d], bfWp[%04d], offset[%04d], size[%04d], numPkts[%04d]\n",\
            inst->position, inst->chunkBufferRp,inst->chunkBufferWp,inst->offset, inst->size, numPkts);
    if ( numPkts == 0 )
    {
        sleepTime = PVR_PLAY_SRC_BUF_EMPTY_SLEEPTIME * 2;
        FAPI_SYS_PRINT_DEBUG(2, "[Feed][pvrPlayFeedData] src buf empty\n");
    }
    else
    {
        sleepTime = 0;

        while ( numPkts > 0 )
        {
            if ( inst->feedXferTimestamp > 0 )
            {
                if ( (inst->slowFeed)
                  && (inst->feedXferPkts > 0) )
                {
                    maxRate = PVR_PLAY_DATARATE_MAX_SLOWFEED;
                }
                else if ( (!(inst->slowFeed))
                       && (inst->feedXferPkts >= PVR_PLAY_DATARATE_CHECK_PKTS) )
                {
                    //Martin change to /2 for test
                    maxRate = PVR_PLAY_DATARATE_MAX_NORMAL + 4000;
                }
                else
                {
                    maxRate = 0;
                }

                if ( maxRate > 0 )
                {
                    currTime = FAPEX_TIMER_GetTimeStamp(
                                                  FAPEX_TIMER_RESOLUTION_1_USEC);

                    interval = (uint32_t)(currTime - inst->feedXferTimestamp);
                    /*The linux timer accuracy depends on jiffies which is 10 ms*/
                    if (interval == 0)
                    {
                        if ( inst->slowFeed )
                        {
                            sleepTime = PVR_PLAY_DATARATE_SLEEP_SLOWFEED;
                        }
                        else
                        {
                            sleepTime = PVR_PLAY_DATARATE_SLEEP_NORMAL;
                        }
                        break;
                    }

                    curRate = inst->feedXferPkts *PVR_TSPKT_SIZE * 1000 / interval;
                    if (curRate > maxRate)
                    {
                        if ( inst->slowFeed )
                        {
                            sleepTime = PVR_PLAY_DATARATE_SLEEP_SLOWFEED;
                        }
                        else
                        {
                            sleepTime = PVR_PLAY_DATARATE_SLEEP_NORMAL / 2;
                        }
                        FAPI_SYS_PRINT_MSG("[FEED] pvrPlayFeedData: break: curRate [%d] > maxRate[%d]\n",\
                                                curRate, maxRate);
                        break;
                    }
                    else
                    {
                        inst->feedXferPkts      = 0;
                        inst->feedXferTimestamp = currTime;
                        FAPI_SYS_PRINT_DEBUG(2, "[Feed] pvrPlayFeedData: curRate[%d], maxRate[%d]\n", curRate, maxRate);
                    }
                }
            }

            if ( pvrPlayFeedCheckAbortion(inst, FAPEX_SYS_NO_SUSPEND) )
            {
                FAPI_SYS_PRINT_DEBUG(2, "[FEED] pvrPlayFeedData: break: abortion detected\n");
                break;
            }

            feedPkts = (numPkts > PVR_PLAY_FEEDPKTS_MAX) ? PVR_PLAY_FEEDPKTS_MAX
                                                         : numPkts;

            /* Check for buffer wraparound. */
            if ( (inst->chunkBufferRp + feedPkts) > PVR_PLAY_CHUNKBUFFER_PKTS )
            {
                feedPkts = PVR_PLAY_CHUNKBUFFER_PKTS - inst->chunkBufferRp;
            }

            /* minimum space required */
            targetBufSpace = feedPkts * PVR_TSPKT_SIZE;

            /* Take also special feed limitations into account. */
            if ( inst->feedMinSpace > targetBufSpace )
            {
                targetBufSpace = inst->feedMinSpace;
            }

            if ( !(pvrCheckBufferFilling(inst, targetBufSpace, 0)) )
            {
                sleepTime = PVR_PLAY_DST_BUF_FULL_SLEEPTIME;
                FAPI_SYS_PRINT_DEBUG(2,"[FEED] pvrPlayFeedData: break: dst buff full, targetBufSapce[%d]\n",\
                                                                 targetBufSpace);
                break;
            }
            else
            {
                if ( inst->feedXferTimestamp == 0 )
                {
                    #ifndef _LINUX_FAPI
                    inst->feedXferTimestamp = FAPI_TIMER_GetTimeStamp(
                                              FAPEX_TIMER_RESOLUTION_1_USEC);
                    #else
                    inst->feedXferTimestamp = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_USEC);
                    #endif
                }

                dataPtr = (uint32_t*)(((uint8_t*)(inst->chunkBuffer))
                                    + (inst->chunkBufferRp * PVR_TSPKT_SIZE));

                //FAPI_SYS_PRINT_MSG("## FEED %i pkts\n", feedPkts);

                res = inst->feedOutput(inst->openParams.tsdBlockIndex,
                                       dataPtr, feedPkts, 1, NULL, inst->openParams.tsdBlockFd,\
                                       inst->input_port_base);
                if ( res )
                {
                    #ifdef PVR_PLAY_PERIODIC_FEED
                    inst->feedLastFeedTime = pvrGetTimeStamp();
                    #endif /* PVR_PLAY_PERIODIC_FEED */
                }

                /* update buffer read pointer */
                FAPI_SYS_PRINT_DEBUG(2, "[FEED] pvrPlayFeedData: update Rp from << [%d]\n", inst->chunkBufferRp);
                if ( (inst->chunkBufferRp + feedPkts) <
                      PVR_PLAY_CHUNKBUFFER_PKTS )
                {
                    inst->chunkBufferRp += feedPkts;
                }
                else
                {
                    /* NOTE: This assumes that wraparound has been taken into
                             account when calculating feedPkts. */
                    inst->chunkBufferRp = 0;
                }
                FAPI_SYS_PRINT_DEBUG(2, "[FEED] pvrPlayFeedData: update Rp to   >> [%d]\n", inst->chunkBufferRp);
                numPkts -= feedPkts;

                inst->feedXferPkts += feedPkts;
            }
        }
    }

    {
        #if 1
        FAPI_SYS_PRINT_DEBUG(3, "[FEED] pvrPlayFeedData: Out: position[%04d], bfRp[%04d], bfWp[%04d], offset[%04d], size[%04d], sleep[%04d]\n",\
            inst->position, inst->chunkBufferRp,inst->chunkBufferWp,inst->offset, inst->size, sleepTime);
        #endif
    }
    return sleepTime;
}
/*lint -restore */


#ifdef PVR_PLAY_PERIODIC_FEED

static uint32_t pvrPlayFeedStuffing (pvrPlayHandleT* inst)
{
    uint64_t currTime     = pvrGetTimeStamp();
    uint64_t nextFeedTime = inst->feedLastFeedTime + PVR_PLAY_FEED_INTERVAL_MAX;
    uint32_t sleepTime;

    if ( currTime >= nextFeedTime )
    {
        (void)inst->feedOutput(inst->openParams.tsdBlockIndex,
                               inst->feedStuffingPkt, 1, 0, NULL, inst->openParams.tsdBlockFd,\
                               inst->input_port_base);

        inst->feedLastFeedTime = currTime;
        sleepTime = PVR_PLAY_FEED_INTERVAL_MAX;
    }
    else if ( currTime >= inst->feedLastFeedTime )
    {
        sleepTime = (uint32_t)(nextFeedTime - currTime);
    }
    else
    {
        sleepTime = PVR_PLAY_FEED_INTERVAL_MAX;
    }

    return sleepTime;
}

#endif /* PVR_PLAY_PERIODIC_FEED */

/* This function checks the available memory in video (VBV) and other buffers.
   If the video buffer allows to feed in "freeVideo" bytes AND all the other buffers
   allow to feed in "freeOthers" bytes, then the return value is FTRUE.
   In case of inst->playOpen.bnst->playOpen.b0, the freeVideo parameter will be used as reference
   and freeOthers can be set to 0.
   In case of video trick modes, only the freeVideo buffer is of interest and freeOthers
   can be set to 0.
*/
static uint8_t pvrCheckBufferFilling(pvrPlayHandleT* inst, uint32_t freeSpace,
                                     uint8_t enableVideoRestart)
{
    uint32_t size;
    uint32_t filling;

    #if 0
    // TODO: not supported yet
    /* When bufferToCheck > 0, then the application has choosen this buffer
       for filling check. This could be used for radio recordings. */
    if ( inst->openParams.bufferToCheck > 0 )
    {
        filling = FD_TSD_GetBufferAmount(inst->playOpen.bufferToCheck, inst->playOpen.tsd);
        size = FH_BM_get_BufferSize(inst->playOpen.tsd,
                                    inst->playOpen.bufferToCheck);

        return ((size - filling) >= freeVideo);
    }
    #endif

    /* The following code is used when bufferToCheck is set to 0 which means
       automatic selection of buffer to check. */

    if ( pvrTrackIsActive(inst, PVR_DECTYPE_VIDEO) )
    {
        /* use video buffer for checking */
        filling = pvrDecoderGetBufLevel(inst, PVR_DECTYPE_VIDEO);
        size    = pvrDecoderGetBufSize (inst, PVR_DECTYPE_VIDEO);


        if (inst->stream[PVR_DECTYPE_VIDEO].format == PVR_TRACK_FORMAT_VIDEO_H264)
        {
            size /= 2;
        }

        /* check for the get buffer level error, this depends on
               * the pratical realization
               */

        FAPI_SYS_PRINT_DEBUG(2, "[FEED] pvrCheckBufferFilling: Video: get filling(%d) size(%d) free(%d)\n", \
                                                      filling, size, freeSpace);
        if (filling == 0x1FFFFFFF)
        {
			if (inst->state == PLAY_STATE_FAST_IFRAMES ||
			    (inst->state == PLAY_STATE_TO_NORMAL && inst->lastState == PLAY_STATE_FAST_IFRAMES) ||
			    (inst->state == PLAY_STATE_TO_STOP && inst->lastState == PLAY_STATE_FAST_IFRAMES)   ||
			    (inst->state == PLAY_STATE_NORMAL && inst->lastState == PLAY_STATE_TO_NORMAL)   )
			{

			}
			else
			{
				return 0;
			}
        	#if 0
            if (inst->state != PLAY_STATE_FAST_IFRAMES &&
                (inst->state != PLAY_STATE_TO_NORMAL && inst->lastState != PLAY_STATE_FAST_IFRAMES))
            {
            	PVR_PlayStateInfoT play_state;
            	(void)pvrPlayGetState((FAPEX_SYS_HandleT)inst, &play_state);
            	FAPI_SYS_PRINT_MSG("[FEED] pvrCheckBufferFilling: Now:%s Last: %s\n",
            	                   inst->playDataPtr->stateString[play_state.state],
            	                   inst->playDataPtr->stateString[play_state.lastState]);
                return 0;
        }
            else
            {
            	FAPI_SYS_PRINT_MSG("[FEED] pvrCheckBufferFilling: FAST IFRAMES\n");
            	return 1;
            }
            #endif
        }
        if ( enableVideoRestart )
        {
            /* restart video if buffer filling does not change for x times */
            if ( filling == inst->lastFilling )
            {
                inst->fillingCounter++;
                //FM_PrStr(" ++ Counting up for filling\n");
                /*Martin: changed from >50 to >25 for debug*/
                if ( inst->fillingCounter > 50 ) /* is this a good value? */
                {
                    // TODO: debug messages
                    //FAPI_SYS_PRINT_MSG("<<< RESTART >>>");

                    // FIXME: temporarily disabled


                    // FIXME: works for normal playback only!
                    // FIXDONE: Martin:20100704
                        PVR_PlayStateInfoT play_state_info;

                        (void)pvrPlayGetState(inst, &play_state_info);
                        if (play_state_info.state == PLAY_STATE_NORMAL)
                        {
                            #if 0
                            FAPI_SYS_PRINT_MSG("[FEED] pvrCheckBufferFilling: restart:filling [%d]\n", filling);
                            #if 1
                            pvrTrackStop (inst, PVR_DECTYPE_VIDEO,
                                          PLAY_STOPMODE_FREEZE);
                            //pvrResetFeed (inst);
                            // TODO: restart in current mode!
                            pvrTrackStart(inst, PVR_DECTYPE_VIDEO,
                                          PVR_DECMODE_NORMAL, 0);

                            #endif

                            inst->lastFilling    = 0;
                            inst->fillingCounter = 0;

                            // FIXME: temporarily disabled
                            #if 1

                            if ( pvrTrackIsActive(inst, PVR_DECTYPE_AUDIO) )
                            {
                                // FIXME: works for normal playback only!
                                pvrTrackStop (inst, PVR_DECTYPE_AUDIO,
                                              PLAY_STOPMODE_NONE);
                                pvrTrackStart(inst, PVR_DECTYPE_AUDIO,
                                              PVR_DECMODE_NORMAL, 0);
                            }

                            // FIXME: reinit other decoders as well?
                            #endif
                            #else
                            if (inst->openParams.targetControl.decoder[PVR_DECTYPE_VIDEO].resetBuffer != NULL)
                            {
                                inst->openParams.targetControl.decoder[PVR_DECTYPE_VIDEO].resetBuffer(
                                PVR_DECTYPE_VIDEO,
                                inst->openParams.targetControl.argPtr,
                                inst->stream[PVR_DECTYPE_VIDEO].decHandle);
                            }
                            if (inst->openParams.targetControl.decoder[PVR_DECTYPE_AUDIO].resetBuffer != NULL)
                            {
                                inst->openParams.targetControl.decoder[PVR_DECTYPE_AUDIO].resetBuffer(
                                PVR_DECTYPE_VIDEO,
                                inst->openParams.targetControl.argPtr,
                                inst->stream[PVR_DECTYPE_VIDEO].decHandle);
                            }
                            if (inst->openParams.targetControl.decoder[PVR_DECTYPE_VIDEO].recover != NULL)
                            {
                                inst->openParams.targetControl.decoder[PVR_DECTYPE_VIDEO].recover(
                                PVR_DECTYPE_VIDEO,
                                inst->openParams.targetControl.argPtr,
                                inst->stream[PVR_DECTYPE_VIDEO].decHandle);
                            }
                            #endif
                        }

                }
            }
            else
            {
                inst->lastFilling    = filling;
                inst->fillingCounter = 0;
            }
        }

        if ( (size - filling) < freeSpace )
        {
        	FAPI_SYS_PRINT_DEBUG(2, "[FEED] pvrCheckBufferFilling: size[%d] filling[%d] left[%d] freeSpace[%d]\n",\
        						size, filling, (size - filling), freeSpace);
            return 0;
        }
    }

    if ( pvrTrackIsActive(inst, PVR_DECTYPE_AUDIO) )
    {
        filling = pvrDecoderGetBufLevel(inst, PVR_DECTYPE_AUDIO);
        size    = pvrDecoderGetBufSize (inst, PVR_DECTYPE_AUDIO);

        FAPI_SYS_PRINT_DEBUG(2, "[Feed] Audio(1): get filling(0x%08x) size(0x%08x) free(0x%08x)\n", \
                                                      filling, size, freeSpace);
        if (filling == 0x1FFFFFFF)
        {
            return 0;
        }
        if ( (size - filling) < freeSpace )
        {
            return 0;
        }
    }

    // FIXME: temporarily disabled
    #if 0
    if ( pvrTrackIsActive(inst, PVR_DECTYPE_AUDIO_2ND) )
    {
        filling = pvrDecoderGetBufLevel(inst, PVR_DECTYPE_AUDIO_2ND);
        size    = pvrDecoderGetBufSize (inst, PVR_DECTYPE_AUDIO_2ND);

        printf("[Feed] Audio(2): get filling(0x%08x) size(0x%08x) free(0x%08x)\n", filling, size, freeSpace);
        if ( (size - filling) < freeSpace )
        {
            return 0;
        }
    }
    #endif

    return 1;
}

/*lint -restore */
