/*
*****************************************************************************
**
** \file        ./middleware/pvr/src/pvr_priv.c
**
** \version     $Id: pvr_priv.c 5952 2010-03-08 16:31:41Z rseng $
**
** \brief       PVR middleware private functions.
**
** This files contains the implementation of the PVR private API.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

//#define DEBUG 1

#include <math.h>

//#include <fapi/sys_services.h>
//#include <fapi/drv_timer.h>
//#include <fapi/drv_tsd.h>

#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_libtsd.h"
#ifdef PVR_CYRPTO_SUPPORT
#include "fapex/abst_libdma.h"
#endif
#include "pvr.h"
#include "pvr_priv.h"

#include "api_f.h"

/*lint -save -e717 -e788 */

/* NOTE: the following LINT statements have been suppressed:
- Info 717: do ... while(0);
- Info 788: enum constant 'anyConst' not used within defaulted switch
*/

//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************

#define PVR_CRYPT_LOCK      (void)FAPEX_SYS_GetSemaphore(pvrPrivDataPtr->cryptoMutex, FAPEX_SYS_SUSPEND);
#define PVR_CRYPT_UNLOCK          FAPEX_SYS_SetSemaphore(pvrPrivDataPtr->cryptoMutex, FAPEX_SYS_NO_SUSPEND);

#define PVR_CRYPT_SYNC_LOCK (void)FAPI_SYS_GET_SEMAPHORE (pvrPrivDataPtr->cryptoSyncSema, FAPEX_SYS_SUSPEND);
#define PVR_CRYPT_SYNC_UNLOCK     FAPI_SYS_SET_SEMAPHORE (pvrPrivDataPtr->cryptoSyncSema, FAPEX_SYS_NO_SUSPEND);
//***************************************************************************
//***************************************************************************
//** Local Structures
//***************************************************************************
//***************************************************************************

typedef struct
{
    char_t                  mountPath[PVR_PATH_NAME_LENGTH + 1];
    uint8_t                 fileNameExtUsed;
    char_t                  fileNameExt[PVR_FILE_EXT_LENGTH + 1];
    char_t                  splitExtFormat[5];
    uint16_t                splitNumberMax;

    FAPEX_SYS_SemaphoreT    cryptoMutex;
    FAPEX_SYS_SemaphoreT    cryptoSyncSema;
    FAPEX_DMA_CryptoParamsT  cryptoParams;

} pvrPrivDataT;

//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************

static pvrPrivDataT* pvrPrivDataPtr = NULL;

//***************************************************************************
//***************************************************************************
//** Local Functions Declaration
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

static int32_t  pvrGetHeader               (F_FILE* filePtr,
                                            uint32_t chunkIndex,
                                            PVR_HeaderPacketT* headerPtr);
static void     pvrCryptoOperationFinished (int32_t status,uint32_t bytes);

//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

int32_t pvrPrivInit (void)
{
    int32_t result = FAPI_OK;

    if ( pvrPrivDataPtr == NULL )
    {
        do
        {
            /* allocate memory for global recording data */
            pvrPrivDataPtr = FAPI_SYS_MALLOC_CACHED(sizeof(pvrPrivDataT));
            if ( pvrPrivDataPtr == NULL )
            {
                result = SUBSYS_PVR_ERR_OUT_OF_MEMORY;
                break;
            }

            memset (pvrPrivDataPtr, 0, sizeof(pvrPrivDataT));

            pvrPrivDataPtr->splitExtFormat[0] = '%';
            pvrPrivDataPtr->splitExtFormat[1] = '0';
            pvrPrivDataPtr->splitExtFormat[4] = 0;

            result = pvrSetFileSplitParams(PVR_DEFAULT_SPLIT_DIGITS,
                                           PVR_DEFAULT_SPLIT_USE_DEC);
            if ( result != FAPI_OK )
            {
                break;
            }

            result = pvrSetFileExtension(PVR_DEFAULT_FILE_EXTENSION);
            if ( result != FAPI_OK )
            {
                break;
            }

            /* set mount path */
            result = pvrSetMountPath(PVR_DEFAULT_MOUNT_PATH);
            if ( result != FAPI_OK )
            {
                break;
            }

            /* ----- allocate resources for optional crypto feature -------- */

            pvrPrivDataPtr->cryptoSyncSema = FAPEX_SYS_CreateSemaphore(0);
            if ( pvrPrivDataPtr->cryptoSyncSema == 0 )
            {
                result = SUBSYS_PVR_ERR_SEMAPHORE_CREATE;
                break;
            }

            pvrPrivDataPtr->cryptoMutex = FAPEX_SYS_CreateSemaphore(1);
            if ( pvrPrivDataPtr->cryptoMutex == 0 )
            {
                result = SUBSYS_PVR_ERR_SEMAPHORE_CREATE;
                break;
            }

            pvrPrivDataPtr->cryptoParams.genSessionKey = 0;
            pvrPrivDataPtr->cryptoParams.keySelection  = FAPEX_DMA_CRYPTO_K_KEY;
            pvrPrivDataPtr->cryptoParams.littleEndian  = 1;
            pvrPrivDataPtr->cryptoParams.cbc           = 0; //!< EBC mode.
        }
        while ( 0 );
    }

    return result;
}

int32_t pvrSetMountPath (const char_t* pathPtr)
{
    int32_t result;

    if ( pathPtr == NULL )
    {
        result = SUBSYS_PVR_ERR_BAD_PARAMETER;
    }
    else if ( strlen(pathPtr) > PVR_PATH_NAME_LENGTH )
    {
        result = SUBSYS_PVR_ERR_BAD_PARAMETER;
    }
    else
    {
        (void)strcpy(pvrPrivDataPtr->mountPath, pathPtr);
        result = FAPI_OK;
    }

    return result;
}

const char_t* pvrGetMountPath (void)
{
    return pvrPrivDataPtr->mountPath;
}

int32_t pvrSetFileExtension (const char_t* extPtr)
{
    int32_t result = FAPI_OK;

    if ( extPtr == NULL )
    {
        pvrPrivDataPtr->fileNameExt[0]  = 0;
        pvrPrivDataPtr->fileNameExtUsed = 0;
    }
    else if ( strlen(extPtr) > PVR_FILE_EXT_LENGTH )
    {
        result = SUBSYS_PVR_ERR_BAD_PARAMETER;
    }
    else
    {
        (void)strcpy(pvrPrivDataPtr->fileNameExt, extPtr);
        pvrPrivDataPtr->fileNameExtUsed = 1;
    }

    return result;
}

const char_t* pvrGetFileExtension (void)
{
    if ( pvrPrivDataPtr->fileNameExtUsed )
    {
        return pvrPrivDataPtr->fileNameExt;
    }
    else
    {
        return 0;
    }
}

int32_t pvrSetFileSplitParams(uint8_t numDigits, uint8_t decimalNotation)
{
    int32_t  result;
    uint32_t splitNumMax;
    uint8_t  cnt;

    if ( (numDigits < 1)
      || (numDigits > PVR_FILE_EXT_LENGTH) )
    {
        result = SUBSYS_PVR_ERR_BAD_PARAMETER;
    }
    else
    {
        pvrPrivDataPtr->splitExtFormat[2] = (char_t)('0' + numDigits);

        if ( decimalNotation )
        {
            pvrPrivDataPtr->splitExtFormat[3] = 'd';
            splitNumMax = 10;
        }
        else /* use hexadecimal notation */
        {
            pvrPrivDataPtr->splitExtFormat[3] = 'x';
            splitNumMax = 16;
        }

        /* Calculate pow(splitNumMax, numDigits) */
        for ( cnt = 1; cnt < numDigits; cnt++ )
        {
            splitNumMax *= splitNumMax;
        }

        pvrPrivDataPtr->splitNumberMax = (uint16_t)(splitNumMax - 1);

        result = FAPI_OK;
    }

    return result;
}

uint16_t pvrGetMaxSplitNumber (void)
{
    return pvrPrivDataPtr->splitNumberMax;
}

uint64_t pvrGetTimeStamp (void)
{
    return FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
}

int32_t pvrGetFileInfo(const char_t* fileNamePtr,
                       PVR_FileInfoT* fileInfoPtr,
                       PVR_HeaderPacketT* headerPtr)
{
    char_t              file_path[PVR_FULL_FILEPATH_LENGTH + 1];
    int32_t             result = FAPI_OK;
    uint32_t            index  = 0;
    PVR_HeaderPacketT   header;
    PVR_HeaderPacketT   header2;
    F_FILE*             firstFilePtr;
    F_FILE*             filePtr;
    int32_t             fileRes;
#ifndef _LINUX_FAPI
    F_FIND              find;
#else
    FAPEX_SYS_FFindStrT find;
#endif
    uint16_t            splitIdx  = 0;
    uint32_t            fileSize  = 0;
    uint8_t             fileFound;
    uint32_t            chunkIdx;
    uint8_t             needRepair = 0;

    FAPI_SYS_PRINT_DEBUG(1, "[PVR ] pvrGetFileInfo: file '%s'\n", fileNamePtr);

    pvrGetFilename(file_path, fileNamePtr, 0);

    firstFilePtr = fopen(file_path, "r");
    if ( firstFilePtr == NULL )
    {
        FAPI_SYS_PRINT_DEBUG(1, "[PVR ] pvrGetFileInfo: failed to open file!\n");

        return SUBSYS_PVR_ERR_FILE_HANDLING;
    }

    do
    {
        /* get first header and check validity */
        result = pvrGetHeader(firstFilePtr, 0, &header);

        if ( result != FAPI_OK )
        {
            break;
        }

        // TODO: non-finalized circular files are not yet detected!

        if ( (header.size     == 0) ||
             (header.duration == 0) )
        {
            /* Consider recording as not being terminated gracefully. Search for
               last header of recording. */
            FAPI_SYS_PRINT_DEBUG(1, "[PVR ] pvrGetFileInfo: header size(%d) header duration(%d)\n",\
                                    header.size, header.duration);
            FAPI_SYS_PRINT_DEBUG(1, "[PVR ] pvrGetFileInfo: found non-finalized recording\n");

            fileFound = 0;

            while ( splitIdx <= pvrPrivDataPtr->splitNumberMax )
            {
                pvrGetFilename(file_path, fileNamePtr, splitIdx);

                fileRes = f_findfirst(file_path, &find);
                if ( (fileRes       == F_NO_ERROR)
                  && (find.filesize >= PVR_CHUNK_SIZE) )
                {
                    fileSize  = find.filesize;
                    fileFound = 1;
                }
                else
                {
                    break;
                }

                splitIdx++;
            }

            if ( !fileFound )
            {
                result = SUBSYS_PVR_ERR_FILE_HANDLING;
                break;
            }
            else
            {
                splitIdx--;
            }

            FAPI_SYS_PRINT_DEBUG(1, "[PVR ] pvrGetFileInfo: found %i split files\n",
                                 splitIdx);

            /* open last file and read last header */
            if ( splitIdx > 0 )
            {
                pvrGetFilename(file_path, fileNamePtr, splitIdx);

                filePtr = fopen(file_path, "r");
                if ( filePtr == NULL )
                {
                    FAPI_SYS_PRINT_DEBUG(1, "[PVR ] pvrGetFileInfo: opening of last split file failed!\n");
                    result = SUBSYS_PVR_ERR_FILE_HANDLING;
                    break;
                }
            }
            else
            {
                /* first file is alrady open */
                filePtr = firstFilePtr;
            }

            /* NOTE: fileSize is >= PVR_CHUNK_SIZE */
            chunkIdx = (fileSize / PVR_CHUNK_SIZE) - 1;

            /* get  header and check validity */
            result = pvrGetHeader(filePtr, chunkIdx, &header2);

            if ( filePtr != firstFilePtr )
            {
                (void)f_close(filePtr);
            }

            if ( result != FAPI_OK )
            {
                FAPI_SYS_PRINT_DEBUG(1, "[PVR ] pvrGetFileInfo: reading header at chunk %i failed\n",
                                     chunkIdx);

                break;
            }

            /* ----- add missing header information ---------------------------- */

            header.size           = header2.size;
            header.duration       = header2.time;
            header.offset         = 0;
            header.numberOfSplits = splitIdx + 1;

            /* still no valid information? */
            if ( (header.size     == 0) ||
                 (header.duration == 0) )
            {
                FAPI_SYS_PRINT_DEBUG(1, "[PVR ] pvrGetFileInfo: give up!\n");

                result = SUBSYS_PVR_ERR_BAD_FILE_FORMAT;
                break;
            }

            needRepair = 1;

            FAPI_SYS_PRINT_DEBUG(1, "[PVR ] pvrGetFileInfo: header successfully repaired!\n");
        }

        /* ----- read first logical packet in case of circular recorded file --- */

        if ( header.offset > 0 )
        {
            FAPI_SYS_PRINT_DEBUG(1, "[PVR ] pvrGetFileInfo: circular file detected!n");

            result = pvrGetHeader(firstFilePtr, header.offset, &header2);

            if ( result != FAPI_OK )
            {
                break;
            }

            header.tag = header2.tag;

            for ( index = 0; index < 64; index++ )
            {
                header.name[index] = header2.name[index];
            }
        }

        /* ----- fill file info structure and header structure ----------------- */

        if ( fileInfoPtr != NULL )
        {
            memset(fileInfoPtr, 0,sizeof(PVR_FileInfoT));

            fileInfoPtr->size     = header.size;
            fileInfoPtr->duration = header.duration;
            fileInfoPtr->offset   = header.offset;

            if ( header.numberOfTracks > PVR_MAX_TRACKS )
            {
                header.numberOfTracks = PVR_MAX_TRACKS;
            }

            fileInfoPtr->numberOfTracks = (uint8_t)header.numberOfTracks;

            for ( index = 0; index < header.numberOfTracks; index++ )
            {
                fileInfoPtr->trackType[index] = (PVR_TrackTypeT)
                                                (header.trackType[index]);
                fileInfoPtr->trackInfo[index] = header.trackInfo[index];
            }

            fileInfoPtr->tag = header.tag;

            for ( index = 0; index < 63; index++ )
            {
                /* NOTE: Ensure string termination by not copying last byte. */

                fileInfoPtr->name[index] = header.name[index];
            }
        }

        if ( headerPtr != NULL )
        {
            *headerPtr = header;
        }
    }
    while ( 0 );

    (void)f_close(firstFilePtr);

    if ( needRepair )
    {
        /* Attempt to write repaired header back to file */

        pvrGetFilename(file_path, fileNamePtr, 0);

        firstFilePtr = fopen(file_path, "a");
        if ( firstFilePtr != NULL )
        {
            if ( fseek(firstFilePtr, 0, SEEK_SET) == F_NO_ERROR )
            {
                FAPI_SYS_PRINT_DEBUG(8, "[RECD] [%s] %s: %ld\n", __FUNCTION__, "repair write before", ftell(firstFilePtr));
                (void)fwrite(&header, 1, sizeof(header), firstFilePtr);
                FAPI_SYS_PRINT_DEBUG(8, "[RECD] [%s] %s: %ld\n", __FUNCTION__, "repair write done", ftell(firstFilePtr));
            }

            (void)fclose (firstFilePtr);
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(1, "[PVR ] pvrGetFileInfo: writing of repaired header failed!\n");
        }
    }

    FAPI_SYS_PRINT_DEBUG(1, "[PVR ] pvrGetFileInfo: result: %i\n", result);

    return result;
}

static int32_t pvrGetHeader (F_FILE* filePtr, uint32_t chunkIndex,
                             PVR_HeaderPacketT* headerPtr)
{
    PVR_HeaderPacketT tmpHeader;
    int32_t           res;
    int32_t           result;
    uint32_t          maxChunksAtOnce = (2147483648UL / PVR_CHUNK_SIZE) - 1;

    /* ----- seek to target position --------------------------------------- */

    /* respect 2 GByte limit of f_Seek */

    /* first set position absolute */
    if ( chunkIndex <= maxChunksAtOnce )
    {
        (void)fseek(filePtr, (int32_t)(chunkIndex * PVR_CHUNK_SIZE),
                     SEEK_SET);
        chunkIndex = 0;
    }
    else
    {
        (void)fseek(filePtr, (int32_t)(maxChunksAtOnce * PVR_CHUNK_SIZE),
                     SEEK_SET);
        chunkIndex -= maxChunksAtOnce;
    }

    /* then set seek relative */
    while ( chunkIndex > 0 )
    {
        if ( chunkIndex <= maxChunksAtOnce )
        {
            (void)fseek(filePtr, (int32_t)(chunkIndex * PVR_CHUNK_SIZE),
                         SEEK_CUR);
            chunkIndex = 0;
        }
        else
        {
            (void)fseek(filePtr, (int32_t)(maxChunksAtOnce * PVR_CHUNK_SIZE),
                         SEEK_CUR);
            chunkIndex -= maxChunksAtOnce;
        }
    }

    /* ----- import header from file --------------------------------------- */

    res = fread((void*)(&tmpHeader), 1, sizeof(PVR_HeaderPacketT), filePtr);

    FAPI_SYS_PRINT_DEBUG(1, "[PVR ] pvrGetHeader: read header res = %d\n", res);
    FAPI_SYS_PRINT_MSG("[PRIV] pvrGetHeader: duration = %d\n", tmpHeader.duration);
    if ( res != (int32_t)(sizeof(PVR_HeaderPacketT)) )
    {
        result = SUBSYS_PVR_ERR_FILE_HANDLING;
    }
    else
    {
        (void)fseek(filePtr, -((int32_t)(sizeof(PVR_HeaderPacketT))),
                     SEEK_CUR);

        /* check file format */
        if ( (tmpHeader.header != 0x471FFF10)
          && (tmpHeader.header != 0x10FF1F47) )
        {
            FAPI_SYS_PRINT_DEBUG(1, "[PVR] pvrGetHeader: bad header (0x%08x)\n", 
                                                              tmpHeader.header);
            result = SUBSYS_PVR_ERR_BAD_FILE_FORMAT;
        }
        else if ( tmpHeader.magic != PVR_HEADER_MAGIC )
        {
            FAPI_SYS_PRINT_DEBUG(1, "[PVR] pvrGetHeader: bad magic (0x%08x)\n", 
                                                               tmpHeader.magic);
            result = SUBSYS_PVR_ERR_BAD_FILE_FORMAT;
        }
        else if ( (tmpHeader.version < PVR_HEADER_V2) ||
                  (tmpHeader.version > PVR_HEADER_VERSION) )
        {
            FAPI_SYS_PRINT_DEBUG(1, "[PVR] pvrGetHeader: bad version (0x%08x)\n", 
                                                             tmpHeader.version);
            result = SUBSYS_PVR_ERR_BAD_FILE_VERSION;
        }
        else
        {
            result = pvrImportHeader (headerPtr, &tmpHeader,
                                      sizeof(PVR_HeaderPacketT));
        }
    }

    return result;
}

uint8_t pvrFeedTsd(uint8_t tsd, const uint32_t* dataPtr, uint32_t packets,
                   uint8_t discardStuffing, uint64_t* millisNeededPtr, int32_t tsd_block_fd, uint8_t* input_port_base)
{
    uint8_t             result = 1;
    uint32_t            i;
    uint32_t            k;
    uint32_t            tt = 0;
    uint64_t            ts = 0;
    const uint32_t*     p32;
    #ifndef _LINUX_FAPI
    volatile uint32_t*  pPacketRam0;
    volatile uint32_t*  pPacketControl;
    #endif
    uint32_t            swapData;
    volatile uint32_t*  pDst;

    FAPEX_TSD_PacketInInfoStrT     packet_info;

    ts  = pvrGetTimeStamp();
    p32 = dataPtr;

    /*lint -save -e923 */
    #ifndef _LINUX_FAPI
    if ( tsd == FAPEX_TSD0)
    {
        pPacketRam0    = (volatile uint32_t*)0xC820E800; /* = REG_TSD0_PACKET0 */
        //pPacketRam1    = (volatile uint32_t *)0xC820E8BC; /* = REG_TSD0_PACKET1 */
        //pPacketRam2    = (volatile uint32_t *)0xC820E978; /* = REG_TSD0_PACKET1 */
        //pPacketRam3    = (volatile uint32_t *)0xC820EA34; /* = REG_TSD0_PACKET1 */
        pPacketControl = (volatile uint32_t*)0xC8208000; /* = REG_TSD0_PACKETCONTROL */
    }
    else if ( tsd == FAPEX_TSD1 )
    {
        pPacketRam0    = (volatile uint32_t*)0xC820E2F0; /* = REG_TSD1_PACKET0 */
        //pPacketRam1    = (volatile uint32_t *)0xC820E3AC; /* = REG_TSD1_PACKET1 */
        //pPacketRam2    = (volatile uint32_t *)0xC820E468; /* = REG_TSD1_PACKET1 */
        //pPacketRam3    = (volatile uint32_t *)0xC820E524; /* = REG_TSD1_PACKET1 */
        pPacketControl = (volatile uint32_t*)0xC8208800; /* = REG_TSD1_PACKETCONTROL */
    }
    else if ( tsd == FAPEX_TSD2 )
    {
        pPacketRam0    = (volatile uint32_t*)0xC8A0E800; /* = REG_TSD2_PACKET0 */
        //pPacketRam1    = (volatile uint32_t *)0xC8A0E8BC; /* = REG_TSD2_PACKET1 */
        //pPacketRam2    = (volatile uint32_t *)0xC8A0E978; /* = REG_TSD2_PACKET1 */
        //pPacketRam3    = (volatile uint32_t *)0xC8A0EA34; /* = REG_TSD2_PACKET1 */
        pPacketControl = (volatile uint32_t*)0xC8A08000; /* = REG_TSD2_PACKETCONTROL */
    }
    else if ( tsd == FAPEX_TSD3 )
    {
        pPacketRam0    = (volatile uint32_t*)0xC8A0E2F0; /* = REG_TSD2_PACKET0 */
        //pPacketRam1    = (volatile uint32_t *)0xC8A0E3AC; /* = REG_TSD2_PACKET1 */
        //pPacketRam2    = (volatile uint32_t *)0xC8A0E468; /* = REG_TSD2_PACKET1 */
        //pPacketRam3    = (volatile uint32_t *)0xC8A0E524; /* = REG_TSD2_PACKET1 */
        pPacketControl = (volatile uint32_t*)0xC8A08800; /* = REG_TSD2_PACKETCONTROL */
    }
    else
    {
        return 0;
    }
    #else
	/* mmap is done in play open*/
    
    #endif

    /*lint -restore */
    if ( ((*p32) & 0xFF000000) == 0x47000000 ) /* big endian data */
    {
    	//printf("be\n");
        /* insert packets into packet RAM */
        for ( i = 0; i < packets; i++ )
        {
            /* filter stuffing packet if filter is enabled */
            if ( !(discardStuffing && (((*p32) & 0x001FFF00) == 0x001FFF00)) )
            {
                #ifndef _LINUX_FAPI
                memcpy((void*)pPacketRam0, (void*)p32, PVR_TSPKT_SIZE);

                /* check if packet RAM is empty before starting packet RAM */
                tt = 20000;
                while ( (*pPacketControl > 0) && (tt > 0) )
                {
                    tt--;
                }
                #else
                memcpy((void*)input_port_base, (void*)p32, PVR_TSPKT_SIZE);
                tt = 20000;

                do{
                    FAPEX_TSD_Ioctl(tsd_block_fd,\
                                     FAPEX_IOCCOM_TSD_GET_INPUT_PACKETINFO,\
                                     (FAPEXIOCARG3_T)&packet_info);
                    tt--;
                }while (packet_info.inputStatus > 0 && (tt > 0));
                #endif 
                /* _LINUX_FAPI */

                if ( tt == 0 ) /* timeout */
                {
                    FAPI_SYS_PRINT_MSG("[PLAY] pvrFeedTsd: Timeout!\n");

                    result = 0;
                    break;
                }
            }

            p32 += PVR_TSPKT_SIZE_LW;
        }
    }
    else if ( ((*p32) & 0x000000FF) == 0x00000047 ) /* little endian data */
    {   
    	//printf("le\n");
        /* insert packets into packet RAM */
        for ( i = 0; i < packets; i++ )
        {
            /* filter stuffing packet if filter is enabled */
            if ( !(discardStuffing && (((*p32) & 0x00FF1F00) == 0x00FF1F00)) )
            {
                #ifndef _LINUX_FAPI
                pDst = pPacketRam0;

                for ( k = 0; k < PVR_TSPKT_SIZE_LW; k++ )
                {
                    swapData = *p32;

                    *pDst =    ( swapData               << 24)  |
                               ((swapData & 0x0000FF00) <<  8)  |
                               ((swapData & 0x00FF0000) >>  8)  |
                               ( swapData               >> 24);
                    p32++;
                    pDst++;
                }

                /* check if packet RAM is empty before starting packet RAM */
                tt = 20000;
                while ( (*pPacketControl > 0) && (tt > 0) )
                {
                    tt--;
                }
                #else


	                pDst = (uint32_t*)input_port_base;
	                for ( k = 0; k < PVR_TSPKT_SIZE_LW; k++ )
	                {
	                    swapData = *p32;

	                    *pDst =    ( swapData               << 24)  |
	                               ((swapData & 0x0000FF00) <<  8)  |
	                               ((swapData & 0x00FF0000) >>  8)  |
	                               ( swapData               >> 24);
	                    p32++;
	                    pDst++;
	                }

                /* check if packet RAM is empty before starting packet RAM */
                tt = 20000;

                do{
                    FAPEX_TSD_Ioctl(tsd_block_fd,\
                                     FAPEX_IOCCOM_TSD_GET_INPUT_PACKETINFO,\
                                     (FAPEXIOCARG3_T)&packet_info);
                    tt--;
                    if (packet_info.inputStatus & FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF1 ||
                    	packet_info.inputStatus & FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF2 ||
                    	packet_info.inputStatus & FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF3)
                	{
                		
                	}
                }while (packet_info.inputStatus > 0 && (tt > 0));
                #endif

                if ( tt == 0 ) /* timeout */
                {
                    FAPI_SYS_PRINT_MSG("[FEED] pvrFeedTsd: Timeout!\n");

                    result = 0;
                    break;
                }
            }
            else
            {
                p32 += PVR_TSPKT_SIZE_LW;
            }
        }
    }
    else /* invalid data */
    {
        FAPI_SYS_PRINT_DEBUG(1, "[FEED] FEED TSD: Invalid data!\n");
    }

    FAPI_SYS_PRINT_DEBUG(8, "[FEED] [%d]-[%lld]\n", packets,\
    							pvrGetTimeStamp() - ts);

    if ( millisNeededPtr != NULL )
    {
        *millisNeededPtr = pvrGetTimeStamp() - ts;
    }

    return result;
}

void pvrGetFilename (char_t* subFileNamePtr, const char_t* fileNamePtr,
                     uint16_t split)
{
    char_t fileNameExt[PVR_FILE_EXT_LENGTH + 1];
    char_t fileNameSep[2];

    if ( pvrPrivDataPtr->fileNameExtUsed )
    {
        fileNameSep[0] = '.';
        fileNameSep[1] = 0;
    }
    else
    {
        fileNameSep[0] = 0;
    }

    /* split file requested */
    if ( (split >= 1) && (split <= pvrPrivDataPtr->splitNumberMax) )
    {
        (void)snprintf(fileNameExt, PVR_FILE_EXT_LENGTH + 1,
                       pvrPrivDataPtr->splitExtFormat, split);
    }
    else /* first file requested */
    {
        if ( pvrPrivDataPtr->fileNameExtUsed )
        {
            strcpy(fileNameExt, pvrPrivDataPtr->fileNameExt);
        }
        else
        {
            fileNameExt[0] = 0;
        }
    }

    (void)snprintf(subFileNamePtr, PVR_FULL_FILEPATH_LENGTH + 1, "%s%s%s%s",
                   pvrPrivDataPtr->mountPath, fileNamePtr,
                   fileNameSep, fileNameExt);

}

void pvrDeleteFiles (const char_t* fileNamePtr)
{
    char_t  hstr[PVR_FULL_FILEPATH_LENGTH + 1];
    uint8_t split;
    int32_t result;
    FAPEX_SYS_FFindStrT find;

    for ( split = 0; split <= pvrPrivDataPtr->splitNumberMax; split++ )
    {
        pvrGetFilename(hstr, fileNamePtr, split);
        result = FAPEX_SYS_FFindFirst(hstr, &find);

        if ( result != 0 )
        {
            break;
        }

        //(void)f_delete(hstr);
        (void)unlink(hstr);

        FAPI_SYS_PRINT_DEBUG(1, "file '%s' deleted\n", hstr);
    }
}

int32_t pvrImportHeader(PVR_HeaderPacketT* dstPtr, void* srcDataPtr,
                        uint32_t dataBytes)
{
    PVR_HeaderPacketT*  srcPtr = (PVR_HeaderPacketT*)srcDataPtr;
    int32_t             result = FAPI_OK;

    if ( (srcDataPtr == NULL)
      || (dstPtr     == NULL)
      || (dataBytes  <  sizeof(PVR_HeaderPacketT)) )
    {
        result = SUBSYS_PVR_ERR_BAD_PARAMETER;
    }
    else if ( srcPtr->version == PVR_HEADER_VERSION ) /* current version */
    {
        *dstPtr = *srcPtr;
    }
    else if ( srcPtr->version == PVR_HEADER_V2 ) /* supported version V2 */
    {
        *dstPtr = *srcPtr;

        /* Set all track formats to PVR_TRACK_FORMAT_OTHER. */
        memset(dstPtr->trackFormat, 0, PVR_MAX_TRACKS / 2);
    }
    else
    {
        result = SUBSYS_PVR_ERR_BAD_FILE_VERSION;
    }

    return result;
}

int32_t pvrSetTrackFormat (PVR_HeaderPacketT* headerPtr, uint8_t trackIndex,
                           uint8_t format)
{
    int32_t result;
    uint8_t data;

    if ( trackIndex >= PVR_MAX_TRACKS )
    {
        result = SUBSYS_PVR_ERR_BAD_PARAMETER;
    }
    else
    {
        data = headerPtr->trackFormat[trackIndex >> 1];

        if ( (trackIndex & 0x1) == 0 ) /* even index */
        {
            data = (data & 0xF0) | (format & 0x0F);
        }
        else                           /* odd index */
        {
            data = (data & 0x0F) | ((format & 0x0F) << 4);
        }

        headerPtr->trackFormat[trackIndex >> 1] = data;

        result = FAPI_OK;
    }

    return result;
}

uint8_t pvrGetTrackFormat (const PVR_HeaderPacketT* headerPtr,
                           uint8_t trackIndex)
{
    uint8_t format = PVR_TRACK_FORMAT_OTHER;

    if ( trackIndex < PVR_MAX_TRACKS )
    {
        format = headerPtr->trackFormat[trackIndex >> 1];

        if ( (trackIndex & 0x1) == 0 ) /* even index */
        {
            format &= 0x0F;
        }
        else                           /* odd index */
        {
            format >>= 4;
        }
    }

    return format;
}

void pvrSetCryptoKeys (const PVR_CryptoParamsT* paramsPtr,
                       FAPEX_DMA_CryptoKeysT* keysPtr)
{
    uint32_t keyMSB = paramsPtr->keyMSB;
    uint32_t keyLSB = paramsPtr->keyLSB;

    memset (keysPtr, 0, sizeof(FAPEX_DMA_CryptoKeysT));

    /* NOTE: 112 bit mode is used where key1 == key3.
       Input key width is 64 bit, so upper 48 bit of key space are unused. */

    keysPtr->key1Msb = 0;

    keysPtr->key1Lsb = ((keyMSB & 0x80000000) >> 22)        //!< MSB 63...63
                     | ((keyMSB & 0x7F000000) >> 23);       //!< MSB 62...56

    keysPtr->key2Msb = ((keyMSB & 0x00FE0000) <<  8)        //!< MSB 55...49
                     | ((keyMSB & 0x0001FC00) <<  7)        //!< MSB 48...42
                     | ((keyMSB & 0x000003F8) <<  6)        //!< MSB 41...35
                     | ((keyMSB & 0x00000007) <<  5)        //!< MSB 34...32
                     | ((keyLSB & 0xF0000000) >> 27);       //!< LSB 31...28

    keysPtr->key2Lsb = ((keyLSB & 0x0FE00000) <<  4)        //!< LSB 27...21
                     | ((keyLSB & 0x001FC000) <<  3)        //!< LSB 20...14
                     | ((keyLSB & 0x00003F80) <<  2)        //!< LSB 13... 7
                     | ((keyLSB & 0x0000007F) <<  1);       //!< LSB  6... 0

    keysPtr->key3Msb = keysPtr->key1Msb;
    keysPtr->key3Lsb = keysPtr->key1Lsb;
}

/* NOTE: 32-bit-alignment of source and destination buffer pointers is
         assumed! */

int32_t pvrPerformCryptoOperation (void* dstBufPtr, const void* srcBufPtr,
                                   const FAPEX_DMA_CryptoKeysT* keysPtr,
                                   uint8_t decrypt)
{
    FAPEX_DMA_RequestT*  cryptoReq;
    int32_t             result;

    pvrPrivDataPtr->cryptoParams.decrypt = decrypt;

    /* copy header area separately -> remains unencrypted */
    memcpy (dstBufPtr, srcBufPtr, PVR_HEADERAREA_SIZE);

    //////////////
    PVR_CRYPT_LOCK;
    //////////////

    /*lint -save -e826 */
    cryptoReq = FAPEX_DMA_StartCrypto ((uint32_t*)(((uint8_t*)srcBufPtr)
                                      + PVR_HEADERAREA_SIZE),
                                      (uint32_t*)(((uint8_t*)dstBufPtr)
                                      + PVR_HEADERAREA_SIZE),
                                      PVR_CHUNK_SIZE - PVR_HEADERAREA_SIZE,
                                      &(pvrPrivDataPtr->cryptoParams),
                                      (FAPEX_DMA_CryptoKeysT*)keysPtr,
                                      pvrCryptoOperationFinished);
    /*lint -restore */

    if ( cryptoReq == NULL )
    {
        FAPI_SYS_PRINT_DEBUG(1, "[PVR] Failure in starting of crypto operation!\n");

        result = SUBSYS_PVR_ERR_CRYPTO_OP_FAILED;
    }
    else
    {
        /* Sleep until end of operation. */
        ////////////////////
        PVR_CRYPT_SYNC_LOCK;
        ////////////////////

        result = FAPI_OK;
    }

    /////////////////
    PVR_CRYPT_UNLOCK;
    /////////////////

    return result;
}

static void pvrCryptoOperationFinished (int32_t status, uint32_t bytes)
{
    status = status;
    bytes = bytes;

    //////////////////////
    PVR_CRYPT_SYNC_UNLOCK;
    //////////////////////
}

/*dummy*/
FAPEX_DMA_RequestT* FAPEX_DMA_StartCrypto(
                           volatile uint32_t*      sourceBuffer,
                           volatile uint32_t*      targetBuffer,
                           uint32_t                bufferBytes,
                           FAPEX_DMA_CryptoParamsT* cryptoParams,
                           FAPEX_DMA_CryptoKeysT*   cryptoKeys,
                           FAPEX_DMA_OptCallbackT   finishCallback )
                           {
                            return 0;
                           }

int32_t  pvrGetFileSize(const char* fpath)
{
    FILE*   fp = NULL;
    int32_t flen=0;

    if (NULL == fpath)
    {
        return -1;
    }
    
    fp = fopen(fpath, "r");
    if (NULL == fp)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[PRIV] open file[%s] failed\n", fpath);
        return -1;
    }

    fseek(fp, 0, SEEK_SET);
    fseek(fp, 0, SEEK_END);

    flen = ftell(fp);
    if (fp != NULL)
    {
        fclose(fp);
        fp = NULL;
    }
    FAPI_SYS_PRINT_MSG("[PRIV] pvrGetFileSize: [%s] size [%d]\n", fpath, flen);
    return flen;
}



/*lint -restore */
