/*!
*****************************************************************************
**
** \file        ./subsystems/pvr/src/pvr_record.h
** 
** \version     $Id: pvr_record.h 3455 2010-09-29 12:28:37Z jrende $
**
** \brief       PVR subsystem recording engine
**
** This file contains the declaration of the PVR recording API.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

#ifndef PVR_RECORD_H
#define PVR_RECORD_H

//#include <fapi/sys_driver.h>

#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"

#include "pvr_priv.h"
#include "api_f.h"

//***********************************************************************
//***********************************************************************
//** Defines and Macros
//***********************************************************************
//***********************************************************************

//***********************************************************************
//***********************************************************************
//** Data types
//***********************************************************************
//***********************************************************************
#define PVR_REC_PMT_SECTION_SIZE        1024

typedef enum
{
    REC_STATE_OPENED,
    REC_STATE_REC,
    REC_STATE_REC_NO_DATA,
    REC_STATE_REC_TO_STOP,
    REC_STATE_STOPPED,
    REC_STATE_REC_TO_PAUSE,
    REC_STATE_PAUSE,
    REC_STATE_PAUSE_TO_REC,
    
} PVR_RecStateT;

/* PVR recording specific handle */
typedef struct
{
    FAPEX_SYS_ThreadT       recThread;                            /* recording thread */
    FAPEX_SYS_SemaphoreT    recSemaphore;                         /* sempahore for protecting data  */
    FAPEX_SYS_SemaphoreT    startSemaphore;                       /* sempahore for starting thead   */
    void*                   chunkBuffer;                          /* buffer for one data cluster */
    uint8_t                 sourceIsPlayback;                     /* related playback engine inst.  */
    F_FILE*                 filePtr;                              /* pointer to recording file */
    char_t                  filePath[PVR_FULL_FILEPATH_LENGTH + 1];/* temp. storage of file path */
    PVR_RecordOpenT         openParams;                           /* open params of the handle */
    PVR_RecStateT           state;                                /* state of recording engine */
    PVR_RecordInfoT         info;                                 /* current recording info */
    uint8_t                 paramChanged;                         /* flag to indicate a PID or format change */
    uint8_t                 pidChanged;                           /* flag to indicate a PID change */
    uint32_t                tag;                                  /* last tag set */
    uint8_t                 tagChanged;                           /* flag to indicate a set tag */
    char_t                  name[PVR_NAME_LENGTH+1];              /* last name set */
    uint8_t                 nameChanged;                          /* flag to indicate a set name */
    uint64_t                firstTimeStamp;                       /* for calculating total duration */
    uint64_t                dataRateLastTs;                       /* for calculating the data rate */
    uint32_t                lastDataRate[8];                      /* for smoothing the data rate */
    uint32_t                lastFilling[3];                       /* for detecting no-data */
    uint64_t                pauseTimeStamp;                       /* TS entring pause state */
    uint64_t                writeHddTime;                         /* time to write a data chunk */
    uint32_t                hwReadPointer31;                      /* read pointer of buffer */
    PVR_HeaderPacketT       header;                               /* current header packet */
    PVR_HeaderPacketT       firstHeader;                          /* header at beginning of file */
    uint32_t                chunksPerFlush;
    uint8_t                 patCC;                                /* PAT continous counter */
    uint8_t                 pmtCC;                                /* PMT continous counter */
    uint8_t                 usePesBuffer;                         /* use at least one PES buffer */
    uint32_t                bufferFilling;                        /* current filling of buffer */
    uint8_t                 trackChannel[PVR_MAX_TRACKS];         /* TSD channel assignment */
    uint32_t                trackReadPointer[PVR_MAX_TRACKS];     /* read pointer */
    uint8_t                 trackCC[PVR_MAX_TRACKS];              /* cont. counter */
    uint32_t                stopPosition;                         /* stop position 0xFFFFFFFF = off */
    uint16_t                splitNumber;                          /* number of current file split */
    uint8_t                 pmtSection[PVR_REC_PMT_SECTION_SIZE]; /* PMT to insert */
    uint32_t                pmtSectionSize;                       /* size of PMT section */
    uint16_t                pmtPid;                               /* PMT PID */
    uint32_t                perfLimitTimeout;                     /* Performance limit timeout in ms */
    uint64_t                bufEmptyTs;                           /* Performance monitoring timest. */

    FAPEX_SYS_HandleT        handleTsBuf31;                        /* Handle of TS "buffer 31" */
    FAPEX_SYS_HandleT        handleChannel[PVR_MAX_TRACKS];        /* TSD handles for each PID */
    FAPEX_SYS_HandleT        handleBmPes[PVR_MAX_TRACKS];          /* BM handles for PES recording */

    /*FAPEX porting*/
    FAPEX_TSD_PvrChannelSetStrT  recTsdChnSet;                     /* TSD channel set for recording */
                            
    PVR_RecordDescrCb       descrCb;                              /* Optional callback for external handling of DVB descramblers */
    uint8_t                 buf31Overflow;                        /* Overflow indication */
    uint32_t                overflowCnt;                          /* Overflow counter used for
                                                                     performance measuring */
    /* parameters related to data encryption option */
    void*                   cryptBuffer;                          /* buffer to store crypted data chunks
                                                                     before being written to device */
    FAPEX_DMA_CryptoKeysT    cryptKeys;
    PVR_CryptoParamsT       cryptoParams;
} pvrRecHandleT;



//***********************************************************************
//***********************************************************************
//** Global Data
//***********************************************************************
//***********************************************************************

//***********************************************************************
//***********************************************************************
//** API Functions
//***********************************************************************
//***********************************************************************

#ifdef __cplusplus
extern "C" {
#endif

int32_t       pvrRecInit                (void);
int32_t       pvrRecExit                (void);
int32_t       pvrRecOpen                (const PVR_RecordOpenT* paramsPtr,
                                         FAPEX_SYS_HandleT* handlePtr);
int32_t       pvrRecClose               (FAPEX_SYS_HandleT handle);
int32_t       pvrRecStart               (FAPEX_SYS_HandleT handle);
int32_t       pvrRecStop                (FAPEX_SYS_HandleT handle);
int32_t       pvrRecPause               (FAPEX_SYS_HandleT handle);
PVR_RecStateT pvrRecGetInternalState    (FAPEX_SYS_HandleT handle);
const char_t* pvrRecGetFilename         (FAPEX_SYS_HandleT handle);
int32_t       pvrRecGetOpen             (FAPEX_SYS_HandleT handle,
                                         PVR_RecordOpenT* paramsPtr);
int32_t       pvrRecGetInfo             (FAPEX_SYS_HandleT handle,
                                         PVR_RecordInfoT* infoPtr);
int32_t       pvrRecSetTag              (FAPEX_SYS_HandleT handle,
                                         uint32_t tag);
int32_t       pvrRecSetName             (FAPEX_SYS_HandleT handle,
                                         const char_t* name);
int32_t       pvrRecSetPids             (FAPEX_SYS_HandleT handle,
                                         const uint16_t* pidsPtr,
                                         const uint8_t* trackFormatPtr);
int32_t       pvrRecSetPmt              (FAPEX_SYS_HandleT handle,
                                         uint16_t pmtPid, const uint8_t* pmtPtr,
                                         uint32_t pmtSize,
                                         const uint16_t* pidsPtr,
                                         const uint8_t* trackFormatPtr);
int32_t       pvrRecSetStopPosition     (FAPEX_SYS_HandleT handle,
                                         uint32_t position);
int32_t       pvrRecGetStopPosition     (FAPEX_SYS_HandleT handle,
                                         uint32_t* positionPtr);
uint32_t      pvrRecGetSplitSize        (FAPEX_SYS_HandleT handle);
uint32_t      pvrRecGetNumberOfSplits   (FAPEX_SYS_HandleT handle);
int32_t       pvrRecSetPerfLimitTimeout (FAPEX_SYS_HandleT handle,
                                         uint32_t timeout);
int32_t       pvrRecGetPerfLimitTimeout (FAPEX_SYS_HandleT handle,
                                         uint32_t* timeoutPtr);
int32_t       pvrRecGetFileInfo         (FAPEX_SYS_HandleT handle,
                                         PVR_FileInfoT* fileInfoPtr);
int32_t       pvrRecFlushRecordingData  (FAPEX_SYS_HandleT handle,
                                         PVR_RecordInfoT* recInfoPtr);
int32_t       pvrRecSetFlushingFrequency(FAPEX_SYS_HandleT handle,
                                         uint32_t numberOfChunks);
int32_t       pvrRecSetChannels         (const uint8_t* channelsPtr);
int32_t       pvrRecSetSourcePlayback   (FAPEX_SYS_HandleT handle);
int32_t       pvrRecEnableEncryption    (FAPEX_SYS_HandleT handle,
                                         const PVR_CryptoParamsT* paramsPtr);
int32_t       pvrRecSetDescramblerCb    (FAPI_SYS_HandleT handle,
                                         PVR_RecordDescrCb cb);
#ifdef __cplusplus
}
#endif

#endif /* PVR_RECORD_H */
