/*!
*****************************************************************************
**
** \file        ./subsystems/sfal/src/sfal.c
**
**              $Id: sfal.c 6046 2011-04-06 08:18:28Z spurke $
**
** \brief       Section Filter Abstraction Layer.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010  by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

//#define DEBUG 2

/*
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - e715 'handle' not used in sfalNotifier
** - e946 hardware buffer direct access via pointer in sfalNotifier.
** - e923 hardware buffer direct access via pointer in sfalNotifier.
**
*****************************************************************************
*/
/*lint -save -e715 -e946 -e923 */

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_libtsd.h"

#include "sfal.h"

/*---------------------------------------------------------------------------*/
/* Global Data                                                               */
/*---------------------------------------------------------------------------*/
extern int32_t fd_TSD[FAPEX_TSD_MAX];

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define SFAL_TOTAL_HANDLE_COUNT    128

#define SFAL_HANDLE_ID             0x7366616c   // Handle id 'sfal'

#define SFAL_INVALID_FILTER_ID     0xFFFFFFFF

#define SFAL_INVALID_CHANNEL_PID   0xFFFFFFFF

#if DEBUG > 0
#define SFAL_PREFIX                "[sfal.c]"
#endif

#define SFAL_MAX_FILTERS           (uint32_t)32   /*FAPEX TSD 32_TABLES*/

#define SFAL_MAX_CHANNELS          SFAL_MAX_FILTERS

/* Number of channels */
#define SFAL_MAX_TSD_CHANNELS      4

/*! \brief Size of internal section buffer 4kb */
#define SFAL_BUFFER_SIZE           4096

/*! \brief timeout BSR stack size */
#define SFAL_BSR_STACK_SIZE        2048

/*! \brief timeout BSR name */
#define SFAL_BSR_NAME      	   "task:SFAL"

/* Skip filter bytes (section filter added some identification bytes which not needed)*/
#define SFAL_SKIP_FILTER_BYTES     12

#define MAX_SECTION_BYTES          24   /* A maximum of 24 section bytes may be applied to a filter */
#define TABLE_ID_ELEMENT           1    /* The table ID requires an additional filter field */

#define SFAL_ALIGN(ptr, n_kbytes)\
        ((((uint32_t)(ptr)) + (n_kbytes << 10)) & ~((n_kbytes << 10) - 1))

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef struct
{
    uint32_t                	inUse;
    uint32_t                	active;
    uint32_t                	filterId;
    uint32_t                	filterDepth;
    FAPEX_TSD_SctFltParamStrT   filterData;
}sfalFilterT;

typedef struct
{
    uint32_t                inUse;
    uint32_t                pid;
    uint32_t                handleCount;
    uint32_t                filterCount;
    uint32_t                running;
    uint32_t                resetting;
    uint32_t                tsdBmReady;
    uint32_t                bmAddress;
    int32_t                 tsdHandle;
    uint8_t		            tsdBlockId;
    uint8_t		            channelNo;
    sfalFilterT             filterSet[SFAL_MAX_FILTERS];
    uint32_t  	 	        sfalHandleId[SFAL_MAX_FILTERS];		    
}sfalChannelT;

/*!
*******************************************************************************
**
**  \brief  Filter handle structure.
**
*******************************************************************************
*/
typedef struct
{
    /* handle ID */
    uint32_t                    id;
    /* specifies if handle is in use */
    uint32_t                    inUse;
    /* open params of the handle */
    SFAL_OpenParamsT            openParams;

    sfalChannelT*               channelPtr;

    sfalFilterT*                filterPtr;

    uint32_t                    pidChannelFilterId;

    /* filter array as passed by user, including first three bytes */
    uint32_t                    filterDepth;

    /* filter array as passed by user, including first three bytes */
    uint8_t                     matchBytes[SFAL_FILTERDEPTH_MAX];

    /* filter array as passed by user, including first three bytes */
    uint8_t                     equalBytes[SFAL_FILTERDEPTH_MAX];

    /* filter array as passed by user, including first three bytes */
    uint8_t                     noneqBytes[SFAL_FILTERDEPTH_MAX];

    /* timeout value for filter */
    uint32_t                    timeout;

    /* current timeout Time */
    uint64_t                    timeoutTime;

    /* pid to filter requested table on. */
    uint32_t                    pid;

    /* tsd to filter requested table on. */
    uint32_t                    tsdBlock;

    /* filter active flag possible values 1=active 0=stopped */
    uint8_t                     active;

    /* filter CRC Mode
     *  */
    uint32_t                    ignoreCrc;

    /* filter one shot Mode.
     */
    uint32_t                    oneShot;

    /* Notify function for informing about a received section.
     *  \sa     SFAL_SectionReceivedNotifyT
     */
    SFAL_SectionReceivedNotifyT secReceivedNotify;

    /* Optional data pointer for secReceivedNotify */
    void*                       secReceivedNotifyOptDataPtr;

    /* Notify function for requesting a buffer.
     *  \sa     SFAL_QueryBufferNotifyT
     */
    SFAL_QueryBufferNotifyT     bufferNotify;

    /* Optional data pointer for bufferNotify */
    void*                       bufferNotifyOptDataPtr;

    /* Notify function for informing about a filtered section.
     *  \sa     SFAL_SECTION_FILTERED_F
     *  */
    SFAL_SectionFilteredNotifyT sectionNotify;

    /* Optional data pointer for sectionNotify */
    void*                       sectionNotifyOptDataPtr;

} sfalHandleT;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
static FAPEX_SYS_SelectParamStrT    select_table;
static void                         *select_work = NULL;
static int32_t                      select_max_id = -1;
static uint32_t                     channelNoInUse[FAPEX_TSD_MAX] = {0,};

static uint32_t             sfalInitialized     = 0;

static uint8_t*             sfalSectionBuffer   = NULL;
static FAPEX_SYS_ThreadT    sfalThreadHandle    = 0;
/*
** Thread sleep time if timeout and reset is handled in own SFAL thread.
*/
static uint16_t             sfalThreadSleep     = 0;
/*
** Number of filters which are currently running with a timeout.
*/
static uint32_t             sfalNumOfTimeouts   = 0;

static sfalHandleT          sfalHandleArray[SFAL_TOTAL_HANDLE_COUNT];

static sfalChannelT         sfalChannelArray[SFAL_MAX_TSD_CHANNELS][SFAL_MAX_FILTERS];

static FAPEX_TSD_SctFltParamStrT sfalFilterDataTable[SFAL_MAX_FILTERS];

static uint32_t             sfalChannelBufferSize = 0;
static uint32_t             sfalChannelCount      = 0;
                                       // filter table:   0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32
static uint32_t             sfalNmberOfDummyFilters[] = { 0, 0, 0, 1, 0, 3, 2, 1, 0, 3, 2, 1, 0, 3, 2, 1, 0, 3, 2, 1, 0, 3, 2, 1, 0, 3, 2, 1, 0, 3, 2, 1, 0 };

                                /* section bytes of a ghost target filter */
static uint32_t             sfalEmptyFilter[FAPEX_TSD_FILTER_SIZE] =
{
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
static uint32_t             sfalFilterAll[FAPEX_TSD_FILTER_SIZE] =
{
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
};

static FAPEX_TSD_SctFltParamStrT sfalDummyFilterData;

/*---------------------------------------------------------------------------*/
/* local function declaration                                                */
/*---------------------------------------------------------------------------*/
static sfalHandleT*     sfalAllocateHandle   ( const uint32_t pid         );
static void             sfalReleaseHandle    ( sfalHandleT* sfalHandlePtr );
static uint32_t         sfalCheckHandle      ( sfalHandleT* sfalHandlePtr );

static sfalFilterT*     sfalAllocateFilter   ( sfalHandleT* sfalHandlePtr );
static void             sfalReleaseFilter    ( sfalHandleT* sfalHandlePtr );

static sfalChannelT*    sfalAllocateChannel  ( sfalHandleT* sfalHandlePtr );
static void             sfalReleaseChannel   ( sfalHandleT* sfalHandlePtr );
static uint32_t         sfalCheckChannel     ( sfalHandleT* sfalHandlePtr );

static void  		    sfalNotifier	     ( sfalChannelT* channel_ptr,
					                           FAPEX_TSD_ReadInfoStrT *secinfo);

static void             sfalBmOverflowHandler( uint32_t blockIndex,
                                               uint32_t channelIndex,
                                               void* optData            );

static int32_t          sfalOpenChannel      ( sfalChannelT* channelPtr,
                                               uint32_t      tsdIndex   );

static void             sfalCloseChannel     ( sfalChannelT* channelPtr );
static int32_t          sfalResetChannel     ( sfalChannelT* channelPtr );
static void             sfalTimeoutThread    ( void* arg                );
static int32_t          sfalSetFilter        ( sfalHandleT*  sfalHandlePtr,
                                               uint32_t  filterDepth,
                                               const uint8_t* matchPtr,
                                               const uint8_t* equalMaskPtr,
                                               const uint8_t* notEqualMaskPtr );

static FAPEX_TSD_SctFltParamStrT* sfalGetFilterTable( sfalHandleT* sfalHandlePtr,
                                               uint32_t*    filerTablePtr );

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

/*
********************************************************************************
** \brief Allocates a new handle.
**
** This function allocates a new SFAL handle from the SFAL handle array.
**
** \return
** - (>0) If handle can be allocated
** - (0)  In case of error, no more handles available
**
********************************************************************************
*/
static sfalHandleT* sfalAllocateHandle( const uint32_t pid )
{
    sfalHandleT* sfal_handle_ptr = NULL;
    uint32_t     index           = 0;

    for(index = 0; index < SFAL_TOTAL_HANDLE_COUNT; index++)
    {
        /* search for handle not in use */
        if( !sfalHandleArray[index].inUse )
        {
            /* set in use */
            sfalHandleArray[index].inUse    = 1;
            /* set pid */
            sfalHandleArray[index].pid      = pid;
            /* set handle id to identifiy filter */
            sfalHandleArray[index].pidChannelFilterId = index;
            /* set handle pointer */
            sfal_handle_ptr = &(sfalHandleArray[index]);
            break;
        }
    }

    return(sfal_handle_ptr);
}

/*
********************************************************************************
** \brief Allocates a new handle.
**
** This function allocates a new SFAL handle from the SFAL handle array.
**
** \return
** - (>0) If handle can be allocated
** - (0)  In case of error, no more handles available
**
********************************************************************************
*/
static sfalChannelT* sfalAllocateChannel( sfalHandleT* sfalHandlePtr )
{
    sfalChannelT* sfal_channel_ptr = NULL;
    uint32_t         blockIndex           = sfalHandlePtr->tsdBlock;
    uint32_t         pid                  = sfalHandlePtr->pid;
    uint32_t         index                = 0;

    if( sfalChannelCount >= SFAL_MAX_CHANNELS )
        return(0);

    for(index = 0; index < SFAL_MAX_CHANNELS; index++)
    {
        /* search for used channel with same pid */
        if( (sfalChannelArray[blockIndex][index].inUse) &&
            (sfalChannelArray[blockIndex][index].pid == pid) )
        {
            /* set pid channel pointer */
            sfalChannelArray[blockIndex][index].handleCount++;
            sfal_channel_ptr = &(sfalChannelArray[blockIndex][index]);
            break;
        }
    }
    if( sfal_channel_ptr == NULL )
    {
        for(index = 0; index < SFAL_MAX_CHANNELS; index++)
        {
            /* search for not used channels */
            if( !sfalChannelArray[blockIndex][index].inUse )
            {
                sfalChannelArray[blockIndex][index].inUse = 1;
                sfalChannelArray[blockIndex][index].pid   = pid;
                sfalChannelArray[blockIndex][index].handleCount++;
                /* set pid channel pointer */
                sfal_channel_ptr = &(sfalChannelArray[blockIndex][index]);
                sfalChannelCount++;
                break;
            }
        }
    }

    sfalHandlePtr->channelPtr = sfal_channel_ptr;

    return( sfal_channel_ptr );
}

/*
********************************************************************************
** \brief Release a handle.
**
** This function release a SFAL handle.
**
** \return
** - none
**
********************************************************************************
*/
static void sfalReleaseHandle(sfalHandleT* handlePtr)
{
    memset( handlePtr, 0, sizeof(sfalHandleT) );
    handlePtr->inUse = 0;
    handlePtr->id    = SFAL_HANDLE_ID;
    handlePtr->pid   = SFAL_INVALID_PID;
    return ;
}

/*
********************************************************************************
** \brief Checks a given handle.
**
** This function checks a given SFAL handle.
**
** \return
** - (1) If handle is OK
** - (0) If handle is invalid
**
********************************************************************************
*/
static uint32_t sfalCheckHandle(sfalHandleT* sfalHandlePtr)
{
    return( sfalInitialized && sfalHandlePtr && sfalHandlePtr->inUse
            && ( sfalHandlePtr->id == SFAL_HANDLE_ID ) );
}

/*
********************************************************************************
** \brief Checks a given pid channel handle.
**
** This function checks a given pid channel.
**
** \return
** - (1) If channel is OK
** - (0) If channel is invalid
**
********************************************************************************
*/
static uint32_t sfalCheckChannel(sfalHandleT* sfalHandlePtr)
{
    return( sfalInitialized && sfalHandlePtr &&
            sfalHandlePtr->channelPtr &&
            sfalHandlePtr->channelPtr->inUse );
}

static void sfalNotifier( sfalChannelT* channel_ptr, FAPEX_TSD_ReadInfoStrT *secinfo )
{
    sfalHandleT*       sfal_handle_ptr  = NULL;
    uint32_t           crcError         = secinfo->bufferStatus & FAPEX_TSD_BUFFSTAT_CRCERROR;
    uint32_t           count            = 0;
    uint8_t*           bufend           = 0;
    uint8_t*           bufstart         = 0;
    uint8_t            byte[SFAL_FILTERDEPTH_MAX] = {0};
    uint16_t           section_length   = 0;
    uint8_t*           out_buffer_ptr   = sfalSectionBuffer;
    uint16_t           out_buffer_size  = SFAL_BUFFER_SIZE;
    uint8_t*           read_ptr         = NULL;
    uint32_t           bufferAddress    = channel_ptr->bmAddress + secinfo->sectionOffset;
    uint32_t 	       bufferLength     = secinfo->sectionLength;
    int32_t            return_value     = FAPEX_OK;
    uint8_t            discard_section  = 1;
    uint32_t           num_of_bytes     = 0;
    uint8_t*           data_ptr         = NULL;
    int32_t	           cnt		        = 0;
    int32_t            diff             = 0;

    if( secinfo->sectionLength == 0 )
    {
        FAPI_SYS_PRINT_MSG("\n[%s] Warning:zero section length~~!\n", __FUNCTION__);
        return;
    }

    if( (secinfo->sectionNo >= 0) && (secinfo->sectionNo <= SFAL_MAX_FILTERS) )
    {
    	sfal_handle_ptr = &(sfalHandleArray[channel_ptr->sfalHandleId[secinfo->sectionNo]]);

	if( (sfal_handle_ptr->pidChannelFilterId != channel_ptr->sfalHandleId[secinfo->sectionNo])
	    || (channel_ptr->sfalHandleId[secinfo->sectionNo] == SFAL_INVALID_FILTER_ID) )
	    return;
    }
    else
    {
	    FAPI_SYS_SLEEP(5);
	    return;
    }

#if DEBUG > 2
	uint32_t i;
	uint32_t pSecBuff = channel_ptr->bmAddress + secinfo->sectionOffset;
    //for (i=0; i<secinfo->sectionLength; i++)
    for (i=0; i<32; i++)
    {
        if(i % 16 == 0)
        {
            printf("\n");
            printf("sfal:%3d(0x%x)", i, i);
        }

        if(i % 8 == 0) printf("\t");

        if( (pSecBuff + i) > (channel_ptr->bmAddress + sfalChannelBufferSize - 1) )
            pSecBuff -= sfalChannelBufferSize;

        printf("%2x ", *((uint8_t*)pSecBuff + i));
    }
    printf("\n");
#endif

    /* move reader pointer to sectionOffset */
    cnt = secinfo->sectionOffset - secinfo->readOffset;
    if((int32_t)cnt < 0)
    {
        cnt += sfalChannelBufferSize;
    }
    if(cnt > 0)
    {
        /* update read pointer */
        return_value = FAPEX_TSD_Ioctl( channel_ptr->tsdHandle, FAPEX_IOCCOM_TSD_SET_BUFFER_READSIZE,
				  	(FAPEXIOCARG3_T)cnt );
        if( return_value != FAPI_OK )
            FAPI_SYS_PRINT_MSG("\n[%s] move buffer read pointer failed (%d)~~!",
				 __FUNCTION__, return_value);
    }

    if( (sfal_handle_ptr->active) &&
        ( (  sfal_handle_ptr->ignoreCrc == SFAL_CRC_IGNORE ) ||
          ( (sfal_handle_ptr->ignoreCrc == SFAL_CRC_CHECK ) && (crcError == 0) )
        )
      )
    {
        for(;;)
        {
	    bufstart = (uint8_t*)channel_ptr->bmAddress;
	    bufend   = (uint8_t*)(channel_ptr->bmAddress + sfalChannelBufferSize - 1);

            data_ptr = (uint8_t*)bufferAddress + SFAL_SKIP_FILTER_BYTES + 1;

            if(data_ptr > bufend)
            {
                data_ptr -= sfalChannelBufferSize;
            }

            /* get necessary bytes from section */
            read_ptr = (uint8_t*)(bufferAddress + SFAL_SKIP_FILTER_BYTES);

            if (read_ptr > bufend)
            {
                read_ptr -= sfalChannelBufferSize;
            }

            num_of_bytes = sfal_handle_ptr->filterDepth;

            if ( num_of_bytes < SFAL_FILTERDEPTH_MIN )
            {
                num_of_bytes = SFAL_FILTERDEPTH_MIN;
            }

            for ( count = 0; count < num_of_bytes; count++ )
            {
                byte[count] = *read_ptr;
                if( ++read_ptr > bufend )
                    read_ptr = bufstart;
            }

            // TODO: filter equal bytes (again) as well?

            for ( count = 0; count < sfal_handle_ptr->filterDepth; count++)
            {
                if ( (sfal_handle_ptr->noneqBytes[count] != 0) &&
                      ((byte[count]                  & sfal_handle_ptr->noneqBytes[count])
                      == (sfal_handle_ptr->matchBytes[count] & sfal_handle_ptr->noneqBytes[count])) )
                {
                    break;
                }
            }

            /* if non-equal-bytes test passed */
            if ( count == sfal_handle_ptr->filterDepth )
            {
                /* get section length */
                section_length  = *(byte+1) << 8;
                section_length |= *(byte+2);
                section_length &= 0x0FFF;
                section_length += 3;
                /* check if whole section is available in buffer */
                read_ptr = (uint8_t*)(bufferAddress + SFAL_SKIP_FILTER_BYTES);
                if( read_ptr > bufend )
                {
                    read_ptr -= sfalChannelBufferSize;
                }

                if ( (sfal_handle_ptr->sectionNotify != NULL) &&
                     (sfal_handle_ptr->sectionNotify( sfal_handle_ptr, bufstart, bufend, read_ptr, section_length,
                                                       sfal_handle_ptr->sectionNotifyOptDataPtr) == 0) )
                {
                    /* if 0 returned discard section */
                }
                else
                {
                    /* get external buffer pointer if buffer request function exist */
                    if( sfal_handle_ptr->bufferNotify )
                    {
                        /* request buffer */
                        sfal_handle_ptr->bufferNotify( sfal_handle_ptr, &out_buffer_ptr, &out_buffer_size,
                                                       sfal_handle_ptr->bufferNotifyOptDataPtr );

                        if ( out_buffer_ptr == NULL )
                        {
                            out_buffer_ptr  = sfalSectionBuffer;
                            out_buffer_size = SFAL_BUFFER_SIZE;
                        }
                    }

                    /* copy section data from circular buffer */
                    uint8_t *src = (uint8_t *)bufferAddress + SFAL_SKIP_FILTER_BYTES;
                    uint8_t *dest = out_buffer_ptr;
                    uint32_t n = secinfo->sectionLength;

                    if( out_buffer_size < n )
                    {
                        n = out_buffer_size;
                    }

                    diff = (bufend - src) + 1;
                    if((int32_t) n > diff)
                    {
                        memcpy(dest, src, (size_t) diff);
                        memcpy(dest+diff, bufstart, (size_t) ((int32_t) n - diff));
                    }
                    else
                    {
                        memcpy(dest,src,n);
                    }

                    discard_section = 0;

                    if ( sfal_handle_ptr->oneShot == SFAL_LOOP_ONE_SHOT)
                    {
                        (void)SFAL_Stop((FAPI_SYS_HandleT)sfal_handle_ptr);
                    }
                    else if ( sfal_handle_ptr->timeout != SFAL_NO_TIMEOUT)
                    {
                        /* reset timeout value */
                        sfal_handle_ptr->timeoutTime  = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
                        sfal_handle_ptr->timeoutTime += sfal_handle_ptr->timeout;
                    }

                    if ( sfal_handle_ptr->secReceivedNotify )
                    {
                        sfal_handle_ptr->secReceivedNotify( sfal_handle_ptr, FAPEX_OK,
                                out_buffer_ptr, section_length,
                                sfal_handle_ptr->secReceivedNotifyOptDataPtr );
                    }

                    return_value = FAPEX_TSD_Ioctl( channel_ptr->tsdHandle,
                            FAPEX_IOCCOM_TSD_SET_BUFFER_READSIZE,
                            bufferLength + SFAL_SKIP_FILTER_BYTES);
                }
            }

            if( discard_section )
            {
                /* move read pointer */
                return_value = FAPEX_TSD_Ioctl( channel_ptr->tsdHandle,
                        FAPEX_IOCCOM_TSD_SET_BUFFER_READSIZE,
                        bufferLength + SFAL_SKIP_FILTER_BYTES);
                if(return_value != FAPEX_OK)
                {
                    FAPI_SYS_PRINT_DEBUG(1, SFAL_PREFIX "FAPEX_IOCCOM_TSD_SET_BUFFER_READSIZE (%d)\n",
                            return_value);
                    break;
                }
            }
            break; //end of for(;;)
        }
    }
    else //if( crcError)
    {
        if ( sfal_handle_ptr->secReceivedNotify )
        {
            sfal_handle_ptr->secReceivedNotify(sfal_handle_ptr,
                    SUBSYS_SFAL_ERR_SECTION_CRC,
                    0,
                    0,
                    sfal_handle_ptr->secReceivedNotifyOptDataPtr);
        }
        channel_ptr->resetting = 1;

        sfalResetChannel( channel_ptr );

        /* wake up thread to handle reset */
        if(sfalThreadHandle != NULL)
        {
            (void)FAPEX_SYS_WakeupThread(sfalThreadHandle);
        }
    }
}

static void sfalBmOverflowHandler(uint32_t blockIndex,
                               	  uint32_t channelIndex,
                               	  void* optData)
{
    sfalChannelT* channelPtr   = (sfalChannelT*) optData;
    int32_t return_value = FAPEX_OK;

    if( channelPtr == NULL )
        return;

    channelPtr->resetting = 1;

    /* wake up thread to handle reset */
    if(sfalThreadHandle != NULL)
    {
        (void)FAPEX_SYS_WakeupThread(sfalThreadHandle);
    }

    FAPI_SYS_PRINT_MSG( "[%s] Buffer OVERFLOW !!! blockIdx %d channelIdx %d\n",
                         __FUNCTION__, blockIndex, channelIndex);

    return_value = FAPEX_TSD_Ioctl(channelPtr->tsdHandle, FAPEX_IOCCOM_STOP, (FAPEXIOCARG3_T)NULL);
    if(return_value != FAPEX_OK)
    {
        FAPI_SYS_PRINT_MSG( "[%s] Stop TSD Channel failed (%d)\n", __FUNCTION__, return_value);
    }

    sfalResetChannel( channelPtr );
}

static int32_t sfalOpenChannel( sfalChannelT* channelPtr, uint32_t tsdIndex )
{
    FAPEX_TSD_OpenSubIdParamStrT    pidChParam;
    FAPEX_TSD_BlockInfoStrT         blockInfo;
    //FAPEX_TSD_PidParamStrT	    pidParam;
    int32_t                   	    return_value;
    uint32_t                        i;

    for(;;)
    {
        /* set up the TSD */

        /* go thru all the channel to find an unopened one and open it */
        for (i = 0; i < FAPEXDEV_TSD_CH_SUB_DESCRIPTOR_NUM; i++)
        {
            if( ! (channelNoInUse[tsdIndex] & ((uint32_t)1 << i)) )
            {
                /* 
                 * Check the TSD channel status with the FAPEX TSD driver.
                 * Because there are other subsystem modules(like val, aal)
                 * and Applicaton can open TSD channel outside of SFAL, the
                 * SFAL has to double check whether the selected channel has
                 * been used.
                 */
                return_value = FAPEX_TSD_Ioctl(fd_TSD[tsdIndex], FAPEX_IOCCOM_TSD_GET_BLOCK_INFO,
                                                (FAPEXIOCARG3_T)&blockInfo);
                if( ! (blockInfo.enableChannelFlag & ((uint32_t)1 << i)) )
                    break;
            }
        }

        if (i < FAPEXDEV_TSD_CH_SUB_DESCRIPTOR_NUM)
        {
            pidChParam.channelNo = i;
            pidChParam.sub_id   = (FAPEXDEV_TSD_SUB_MINOR_FIRST +
                    tsdIndex * FAPEXDEV_TSD_CH_SUB_DESCRIPTOR_NUM ) + i;
        }

        return_value = FAPEX_TSD_Ioctl(fd_TSD[tsdIndex], FAPEX_IOCCOM_TSD_OPEN_SUBID,
                (FAPEXIOCARG3_T)&pidChParam );
        if(return_value != FAPEX_OK)
        {
            FAPI_SYS_PRINT_DEBUG(1,SFAL_PREFIX " FAPEX TSD Open() (%d)\n", return_value);
            break;
        }
        channelPtr->tsdHandle = pidChParam.sub_id;
        channelNoInUse[tsdIndex] |= ((uint32_t)1 << i);
        channelPtr->tsdBlockId = tsdIndex;
        channelPtr->channelNo = i;
#if 0
        pidParam.pid   = channelPtr->pid;
        pidParam.flags = FAPEX_TSD_FLTFLAGS_CHECK_CC | FAPEX_TSD_FLTFLAGS_RM_TS_HEADER;
        return_value = FAPEX_TSD_Ioctl( channelPtr->tsdHandle, FAPEX_IOCCOM_TSD_SET_PID_VALUE,
                (FAPEXIOCARG3_T)&pidParam); 
        if(return_value != FAPEX_OK)
        {
            FAPI_SYS_PRINT_DEBUG( 1,SFAL_PREFIX " FAPEX TSD SetChannelPid() (%d)\n",return_value);
            (void)FAPEX_TSD_Ioctl( channelPtr->tsdHandle, FAPEX_IOCCOM_CLOSE_SUBID,
                    (FAPEXIOCARG3_T)NULL );
            FAPEX_TSD_Munmap( (void*)(channelPtr->bmAddress), sfalChannelBufferSize );
            channelPtr->bmAddress = 0;
            break;
        }
#endif	
        break;
    }

    if( return_value == FAPEX_OK )
        channelPtr->tsdBmReady = 1;

    return( return_value );
}

static int32_t sfalSetFilter( sfalHandleT* sfalHandlePtr,
                              uint32_t     filterDepth,
                         const uint8_t*    matchPtr,
                         const uint8_t*    equalMaskPtr,
                         const uint8_t*    notEqualMaskPtr )
{
    sfalFilterT*    filter_ptr      = sfalHandlePtr->filterPtr;
    uint32_t        index           = 0;

    if( ( filterDepth     == 0                   ) ||
        ( filterDepth     < SFAL_FILTERDEPTH_MIN ) ||
        ( matchPtr        == NULL                ) ||
        ( equalMaskPtr    == NULL                ) ||
        ( notEqualMaskPtr == NULL                ) )
    {
        return( SUBSYS_SFAL_ERR_BAD_PARAMETER );
    }

    if( filterDepth >= SFAL_FILTERDEPTH_MAX )
    {
        return( SUBSYS_SFAL_ERR_FILTER_DEPTH );
    }

    for ( index = 0; index < filterDepth; index++ )
    {
        sfalHandlePtr->matchBytes[index] = matchPtr[index];
        sfalHandlePtr->equalBytes[index] = equalMaskPtr[index];
        sfalHandlePtr->noneqBytes[index] = notEqualMaskPtr[index];
    }

    sfalHandlePtr->filterDepth = filterDepth;

    if( filter_ptr )
    {
        filter_ptr->filterDepth 	 = sfalHandlePtr->filterDepth;
	memset(filter_ptr->filterData.filter, 0, FAPEX_TSD_FILTER_SIZE);
	memcpy(filter_ptr->filterData.filter, sfalHandlePtr->matchBytes, filter_ptr->filterDepth);
	memset(filter_ptr->filterData.mask, 0, FAPEX_TSD_FILTER_SIZE);
	memcpy(filter_ptr->filterData.mask, sfalHandlePtr->equalBytes, filter_ptr->filterDepth);
	
        return( FAPEX_OK ) ;
    }
    return( SUBSYS_SFAL_ERR_NO_FREE_FILTER ) ;
}

static sfalFilterT* sfalAllocateFilter(sfalHandleT* sfalHandlePtr)
{
    sfalChannelT*    channel_ptr = sfalHandlePtr->channelPtr;
    uint32_t         filter_id   = sfalHandlePtr->pidChannelFilterId;
    sfalFilterT*     filter_ptr  = 0;
    uint32_t         index       = 0;

    for(index = 0; index < SFAL_MAX_FILTERS; index++)
    {
        /* search for used filter with same handle id */
        if( (channel_ptr->filterSet[index].inUse) &&
            (channel_ptr->filterSet[index].filterId == filter_id) )
        {
            /* set pid channel pointer */
            channel_ptr->filterSet[index].active    = 0;
            filter_ptr = &(channel_ptr->filterSet[index]);
            break;
        }
    }
    if( filter_ptr == 0 )
    {
        for(index = 0; index < SFAL_MAX_FILTERS; index++)
        {
            /* search for not used filter */
            if( !channel_ptr->filterSet[index].inUse )
            {
                channel_ptr->filterSet[index].inUse     = 1;
                channel_ptr->filterSet[index].active    = 0;
                channel_ptr->filterSet[index].filterId  = filter_id;
                channel_ptr->filterCount++;
                /* set pid channel pointer */
                filter_ptr = &(channel_ptr->filterSet[index]);
                break;
            }
        }
    }

    sfalHandlePtr->filterPtr = filter_ptr;

    return( filter_ptr );
}

static void sfalReleaseFilter(sfalHandleT* sfalHandlePtr)
{
    sfalChannelT*     channel_ptr = sfalHandlePtr->channelPtr;
    sfalFilterT*      filter_ptr  = sfalHandlePtr->filterPtr;

    memset( filter_ptr, 0, sizeof(sfalFilterT) );
    filter_ptr->filterId = SFAL_INVALID_FILTER_ID;
    channel_ptr->filterCount--;
    sfalHandlePtr->filterPtr = 0;
}

static void sfalReleaseChannel(sfalHandleT* sfalHandlePtr)
{
    sfalChannelT*  channel_ptr = sfalHandlePtr->channelPtr;

    channel_ptr->handleCount--;

    // If all handles for this channel are closed free this channel
    if( !channel_ptr->handleCount )
    {
        // Close TSD driver
        sfalCloseChannel(channel_ptr);
        // Reset channel structure
        memset( channel_ptr, 0, sizeof(sfalChannelT) );
        channel_ptr->pid = SFAL_INVALID_CHANNEL_PID;
        sfalChannelCount--;
    }

    sfalHandlePtr->channelPtr = 0;
}

static FAPEX_TSD_SctFltParamStrT* sfalGetFilterTable( sfalHandleT* sfalHandlePtr,
                                                uint32_t*    numOfTablesPtr )
{
    sfalChannelT* channel_ptr  = sfalHandlePtr->channelPtr;
    uint32_t      index        = 0;
    uint32_t      count        = 0;
    uint32_t      dummy_filter = 0;

    for( index=0; index<SFAL_MAX_FILTERS; index++ )
    {
        if( (channel_ptr->filterSet[index].inUse) &&
            (channel_ptr->filterSet[index].filterId != SFAL_INVALID_FILTER_ID) &&
            (channel_ptr->filterSet[index].active) &&
            (channel_ptr->filterSet[index].filterDepth > 0) )
        {
            memcpy( &sfalFilterDataTable[count],
                    &channel_ptr->filterSet[index].filterData,
                    sizeof(FAPEX_TSD_SctFltParamStrT) );

            channel_ptr->sfalHandleId[count] = channel_ptr->filterSet[index].filterId;

            count++;
        }
    }

    dummy_filter = sfalNmberOfDummyFilters[count];
    // Added dummy filter if needed
    for( index=0; index<dummy_filter; index++ )
    {
        memcpy( &sfalFilterDataTable[count],
                &sfalDummyFilterData,
                sizeof(FAPEX_TSD_SctFltParamStrT) );
	
        channel_ptr->sfalHandleId[count] = SFAL_INVALID_FILTER_ID;

        count++;
    }

    *numOfTablesPtr = count;

    return( &sfalFilterDataTable[0] );
}

static void sfalCloseChannel(sfalChannelT* channelPtr)
{
    int32_t return_value = FAPEX_OK;

    if( channelPtr->tsdBmReady == 0)
    {
        return;
    }

    return_value = FAPEX_TSD_Ioctl(channelPtr->tsdHandle, FAPEX_IOCCOM_STOP, (FAPEXIOCARG3_T)NULL);
    if( return_value != FAPEX_OK )
    {
        FAPI_SYS_PRINT_MSG("[%s] Error: could not stop channel (code: %d)\n",
			     __FUNCTION__, return_value);
    }

    (void)FAPEX_TSD_Ioctl(channelPtr->tsdHandle, FAPEX_IOCCOM_TSD_CLEAR_SECTION_FILTER,
			  (FAPEXIOCARG3_T)NULL);

    return_value = FAPEX_TSD_Ioctl(channelPtr->tsdHandle,
		    FAPEX_IOCCOM_TSD_RESET_BUFFER, (FAPEXIOCARG3_T)NULL );
    if(return_value != FAPEX_OK)
    {
        FAPI_SYS_PRINT_MSG("[%s] Error: could not reset channel buffer (code: %d)\n",
			    __FUNCTION__, return_value);
    }

    if ( channelPtr->bmAddress != 0)
    {
        FAPEX_TSD_Munmap((void*)(channelPtr->bmAddress), sfalChannelBufferSize);
        channelPtr->bmAddress = 0;
    }

    /* clear select ID */
    return_value = FAPEX_SYS_SelectClearID( &select_table, channelPtr->tsdHandle,
            FAPEX_SELECT_MODE_READ | FAPEX_SELECT_MODE_EXPT );
    if(return_value != FAPEX_OK)
    {
        FAPI_SYS_PRINT_DEBUG( 2, SFAL_PREFIX" FAPEX_SYS_SelectClearID() failed (%d)\n",
                return_value );
    }

    return_value = FAPEX_TSD_Ioctl(channelPtr->tsdHandle, FAPEX_IOCCOM_CLOSE_SUBID,
				   (FAPEXIOCARG3_T)NULL );
    if(return_value != FAPEX_OK)
    {
        FAPI_SYS_PRINT_MSG("[%s] channel Close failed (%d).\n", __FUNCTION__, return_value);
    }
    channelNoInUse[channelPtr->tsdBlockId] &= ~((uint32_t)1 << channelPtr->channelNo);

    channelPtr->tsdHandle   = 0;
    channelPtr->tsdBmReady  = 0;
}


static int32_t sfalResetChannel(sfalChannelT* channelPtr)
{
    int32_t             	return_value  = FAPEX_OK;

    if( channelPtr )
    {
        for(;;)
        {
            /* Stop the channel, reset buffer, start again... */
	    return_value = FAPEX_TSD_Ioctl(channelPtr->tsdHandle, 
					FAPEX_IOCCOM_STOP, (FAPEXIOCARG3_T)NULL);
            if(return_value != FAPEX_OK)
            {
		FAPI_SYS_PRINT_DEBUG( 2, "\n[%s] fail to stop channel(%d).\n",
				       __FUNCTION__, return_value);
                break;
            }

	    return_value = FAPEX_TSD_Ioctl(channelPtr->tsdHandle,
				 FAPEX_IOCCOM_TSD_RESET_BUFFER, (FAPEXIOCARG3_T)NULL );
	    if(return_value != FAPEX_OK)
            {
                FAPI_SYS_PRINT_DEBUG( 2, "\n[%s] fail to reset channel buffer(%d).\n",
                                       __FUNCTION__, return_value);
                break;
            }

	    return_value = FAPEX_TSD_Ioctl(channelPtr->tsdHandle,
				 FAPEX_IOCCOM_START, (FAPEXIOCARG3_T)NULL);
            if(return_value != FAPEX_OK)
            {
                FAPI_SYS_PRINT_DEBUG( 2, "\n[%s] fail to start channel(%d).\n",
                                       __FUNCTION__, return_value);
                break;
            }

            break; /* end of for(;;) */
        }
        channelPtr->resetting = 0;
    }

    return( return_value );
}

static void sfalTimeoutThread(void* arg)
{
    uint32_t        handle_count    = 0;
    sfalChannelT*   channel_ptr;
    sfalHandleT*    sfal_handle_ptr;

    for(;;)
    {
        if( sfalInitialized )
        {
            for( handle_count=0; handle_count<SFAL_TOTAL_HANDLE_COUNT; handle_count++ )
            {
                sfal_handle_ptr = &sfalHandleArray[handle_count];
                if( sfal_handle_ptr->inUse )
                {
                    channel_ptr = sfal_handle_ptr->channelPtr;
                    if( channel_ptr &&
                        (channel_ptr->resetting == 1) )
                    {
                        (void)sfalResetChannel(channel_ptr);
                    }
                    else if( (sfal_handle_ptr->timeout != SFAL_NO_TIMEOUT) &&
                        (sfal_handle_ptr->timeoutTime < FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC)) )
                    {
                        (void)SFAL_Stop((FAPI_SYS_HandleT)sfal_handle_ptr);
                        if(sfal_handle_ptr->secReceivedNotify)
                        {
                            sfal_handle_ptr->secReceivedNotify((FAPI_SYS_HandleT)sfal_handle_ptr,
                                                                SUBSYS_SFAL_ERR_SECTION_TIMEOUT,
                                                                0,
                                                                0,
                                                                (void *)sfal_handle_ptr->secReceivedNotifyOptDataPtr);
                        }
                    }
                }
            }

            if( ( sfalNumOfTimeouts == 0) &&
                ( sfalThreadHandle != NULL) )
            {
                (void)FAPEX_SYS_SuspendThread(sfalThreadHandle);
            }
            else
            {
                (void)usleep(sfalThreadSleep);
            }
        }
        (void)FAPI_SYS_SLEEP(1);
    }
}

static int32_t sfalSetTsdSectionFilter( sfalChannelT* channelPtr, uint32_t num_tables,
                                        FAPEX_TSD_SctFltParamStrT* filter_param_ptr,
					uint32_t timeout )
{
    int32_t return_value = FAPEX_OK;
    uint32_t table_num_flags;
    uint32_t i;
    
    switch (num_tables)
    {
	case 1:
	    table_num_flags = FAPEX_TSD_FLTFLAGS_TABLE_NUM1;
	    break;
        case 2:
            table_num_flags = FAPEX_TSD_FLTFLAGS_TABLE_NUM2;
            break;
        case 4:
            table_num_flags = FAPEX_TSD_FLTFLAGS_TABLE_NUM4;
            break;
        case 8:
            table_num_flags = FAPEX_TSD_FLTFLAGS_TABLE_NUM8;
            break;
        case 12:
            table_num_flags = FAPEX_TSD_FLTFLAGS_TABLE_NUM12;
            break;
        case 16:
            table_num_flags = FAPEX_TSD_FLTFLAGS_TABLE_NUM16;
            break;
        case 20:
            table_num_flags = FAPEX_TSD_FLTFLAGS_TABLE_NUM20;
            break;
        case 24:
            table_num_flags = FAPEX_TSD_FLTFLAGS_TABLE_NUM24;
            break;
        case 32:
            table_num_flags = FAPEX_TSD_FLTFLAGS_TABLE_NUM32;
            break;
        default:	/* table number is zero */
	    return_value = -1;
	    return return_value;
    }

    #if DEBUG
    FAPI_SYS_PRINT_DEBUG( 2, "table_num_flags:0x%x, num_tables:%d\n", table_num_flags, num_tables);
    #endif

    for( i = 0; i < num_tables; i++ )
    {
	filter_param_ptr->pid              	= channelPtr->pid;
        filter_param_ptr->bufferSize 		= sfalChannelBufferSize;
        filter_param_ptr->bufferThresholdH 	= sfalChannelBufferSize;
        filter_param_ptr->bufferThresholdL 	= 0;
	filter_param_ptr->timeout 		= timeout;
	filter_param_ptr->flags 		= FAPEX_TSD_FLTFLAGS_CHECK_CC
				  		  | FAPEX_TSD_FLTFLAGS_RM_TS_HEADER
				  		  | FAPEX_TSD_FLTFLAGS_SWAPWR_SWAP
				  		  | FAPEX_TSD_FLTFLAGS_SWAPRD_NO_SWAP
				  		  | FAPEX_TSD_FLTFLAGS_SWAPMODE_4BYTE
				  		  | table_num_flags;

	return_value = FAPEX_TSD_Ioctl( channelPtr->tsdHandle, FAPEX_IOCCOM_TSD_SET_SECTION_FILTER,
					(FAPEXIOCARG3_T)filter_param_ptr );	
	if( return_value != FAPEX_OK )
	{
	    return return_value;
	} 
	
	filter_param_ptr++;
    }

    return return_value;
}

/*!
********************************************************************************
** \brief Initialise the Section Filtering.
**
** This function initializes the subsystem. Initialization is performed with
** default values of the subsystem parameters if they have not been overwritten.
**
** \param   initParamsPtr  Pointer to init parameters.
**
** \return
**  - #FAPEX_OK
**  - #SUBSYS_SFAL_ERR_ALREADY_INITIALIZED
**  - #SUBSYS_SFAL_ERR_BAD_PARAMETER
**  - #SUBSYS_SFAL_ERR_OUT_OF_MEMORY
**  - #SUBSYS_SFAL_ERR_THREAD_CREATE
**
** \sa #SFAL_Exit()
**
********************************************************************************
*/
int32_t SFAL_Init(const SFAL_InitT* initParamsPtr)
{
    int32_t return_value = FAPEX_OK;
    uint32_t    index    = 0;
    int32_t     size	 = 0;

    for(;;)
    {
        if( sfalInitialized )
        {
            return_value = SUBSYS_SFAL_ERR_ALREADY_INITIALIZED;
            break;
        }

        if( !initParamsPtr )
        {
            return_value = SUBSYS_SFAL_ERR_BAD_PARAMETER;
            break;
        }

        if( !initParamsPtr->threadPriority )
        {
            return_value = SUBSYS_SFAL_ERR_BAD_PARAMETER;
            break;
        }

        if( initParamsPtr->kByteBufferSize < (SFAL_BUFFER_SIZE/1024) )
        {
            return_value = SUBSYS_SFAL_ERR_BUFFER_SIZE_TO_SMALL;
            break;
        }

        sfalChannelBufferSize = initParamsPtr->kByteBufferSize*1024;

        sfalSectionBuffer = FAPI_SYS_MALLOC(SFAL_BUFFER_SIZE);
        if(sfalSectionBuffer == NULL)
        {
            return_value = SUBSYS_SFAL_ERR_OUT_OF_MEMORY;
            break;
        }

        memset( sfalHandleArray, 0, sizeof(sfalHandleArray) );
        for(index=0; index<SFAL_TOTAL_HANDLE_COUNT; index++)
        {
           sfalHandleArray[index].inUse = 0;
           sfalHandleArray[index].id    = SFAL_HANDLE_ID;
           sfalHandleArray[index].pid   = SFAL_INVALID_PID;
        }

        sfalThreadSleep = initParamsPtr->threadSleep;

        sfalThreadHandle =  FAPEX_SYS_CreateThread(NULL,
                SFAL_BSR_STACK_SIZE,
                initParamsPtr->threadPriority,
                sfalTimeoutThread,
                NULL,
                NULL,
                SFAL_BSR_NAME);
        if(sfalThreadHandle == NULL)
        {
            return_value = SUBSYS_SFAL_ERR_THREAD_CREATE;
            FAPI_SYS_FREE(sfalSectionBuffer);
            break;
        }

        memset(&sfalDummyFilterData, 0, sizeof(sfalDummyFilterData));
        memcpy(sfalDummyFilterData.filter, sfalEmptyFilter, FAPEX_TSD_FILTER_SIZE);
        memcpy(sfalDummyFilterData.mask, sfalFilterAll, FAPEX_TSD_FILTER_SIZE);

        break; // end of for(;;)
    }

    /* setup select */
    size = FAPEX_SYS_SelectGetSize((FAPEX_SELECT_MODE_READ | FAPEX_SELECT_MODE_EXPT));
    select_work = (void *)FAPI_SYS_MALLOC(size);
    return_value = FAPEX_SYS_SelectInit(&select_table, select_work,
            (FAPEX_SELECT_MODE_READ | FAPEX_SELECT_MODE_EXPT));
    if( return_value != FAPEX_OK )
    {
        FAPI_SYS_PRINT_MSG( "[%s]FAPEX_SYS_SelectInit: Error %d\n", __FUNCTION__, return_value );
    }

    if( return_value == FAPEX_OK )
        sfalInitialized = 1;

    return return_value;
}

/*!
********************************************************************************
** \brief Exits the Section Filter Subsystem.
**
** This function shuts down the subsystem and safely disables the underlying
** software handles. Pending accesses will be flushed and all allocated
** resources will be released.
**
** \sa #SFAL_Init()
**
********************************************************************************
*/
void SFAL_Exit(void)
{
    uint32_t index = 0;

    if( !sfalInitialized )
        return;

    for(index=0; index<SFAL_TOTAL_HANDLE_COUNT; index++)
    {
        /* close all previously opened filter sets */
        (void)SFAL_Close((FAPI_SYS_HandleT)&sfalHandleArray[index]);
    }

    if(sfalThreadHandle != NULL)
    {
        (void)FAPEX_SYS_DestroyThread (sfalThreadHandle);
    }

    FAPI_SYS_FREE(sfalSectionBuffer);
    sfalSectionBuffer = 0;
    sfalInitialized = 0;
}

/*!
********************************************************************************
** \brief Opens one section filter.
**
** This function opens an instance of the SFAL subsystem.
**
** \param paramsPtr    Pointer to the open parameters.
** \param errorCodePtr Pointer where error code should be stored or NULL if unused.
**
** \return
**  - #FAPEX_OK
**  - #SUBSYS_SFAL_ERR_BAD_PARAMETER
**  - #SUBSYS_SFAL_ERR_OUT_OF_MEMORY
**
** \sa #SFAL_Close()
**
********************************************************************************
*/
FAPI_SYS_HandleT SFAL_Open( SFAL_OpenParamsT* paramsPtr, int32_t* errorCodePtr )
{
    sfalHandleT*        sfal_handle_ptr = NULL;
    sfalChannelT*       channel_ptr     = NULL;
    sfalFilterT*        filter_ptr      = NULL;
    int32_t             error_code      = FAPEX_OK;
    uint32_t            pid             = 0;

    channel_ptr = channel_ptr;
    filter_ptr  = filter_ptr;

    for(;;)
    {
        if( !sfalInitialized )
        {
            error_code = SUBSYS_SFAL_ERR_NOT_INITIALIZED;
            break;
        }

        if( !paramsPtr )
        {
            error_code = SUBSYS_SFAL_ERR_BAD_PARAMETER;
            break;
        }

        /* check version */
        if( !FAPI_SYS_CHECK_VERSION(paramsPtr->version, SFAL_VERSION) )
        {
            error_code = SUBSYS_SFAL_ERR_UNSUPPORTED_VERSION;
            break;
        }

        if( (paramsPtr->tsdInput != FAPEX_TSD0) &&
            (paramsPtr->tsdInput != FAPEX_TSD1) &&
            (paramsPtr->tsdInput != FAPEX_TSD2) &&
            (paramsPtr->tsdInput != FAPEX_TSD3) )
        {
            error_code = SUBSYS_SFAL_ERR_BAD_PARAMETER;
            break;
        }

        if( paramsPtr->pid >= SFAL_INVALID_PID )
        {
            pid = SFAL_INVALID_PID;
        }
        else pid = paramsPtr->pid;

        /* allocate handle */
        sfal_handle_ptr = sfalAllocateHandle(pid);
        if( !sfalCheckHandle( sfal_handle_ptr ) )
        {
            sfal_handle_ptr = 0;
            error_code = SUBSYS_SFAL_ERR_OUT_OF_MEMORY;
            break;
        }

        /* copy open params to handle */
        memcpy( &(sfal_handle_ptr->openParams),
                paramsPtr, sizeof(SFAL_OpenParamsT) );

        sfal_handle_ptr->pid      = pid;
        sfal_handle_ptr->tsdBlock = paramsPtr->tsdInput;

        channel_ptr = sfalAllocateChannel(sfal_handle_ptr);
        if( !sfalCheckChannel( sfal_handle_ptr ) )
        {
            sfalReleaseHandle( sfal_handle_ptr );
            sfal_handle_ptr = 0;
            channel_ptr = 0;
            error_code = SUBSYS_SFAL_ERR_NO_FREE_CHANNEL;
            break;
        }

        filter_ptr = sfalAllocateFilter(sfal_handle_ptr);
        if( !filter_ptr )
        {
            sfalReleaseChannel( sfal_handle_ptr );
            sfalReleaseHandle ( sfal_handle_ptr );
            sfal_handle_ptr = 0;
            channel_ptr = 0;
            error_code = SUBSYS_SFAL_ERR_NO_FREE_FILTER;
            break;
        }

        if( paramsPtr->filterDepth )
        {
            error_code = sfalSetFilter( sfal_handle_ptr,
                                         paramsPtr->filterDepth,
                                         paramsPtr->matchPtr,
                                         paramsPtr->equalMaskPtr,
                                         paramsPtr->notEqualMaskPtr );
            if( error_code != FAPEX_OK )
            {
                sfalReleaseFilter ( sfal_handle_ptr );
                sfalReleaseChannel( sfal_handle_ptr );
                sfalReleaseHandle ( sfal_handle_ptr );
                filter_ptr      = 0;
                sfal_handle_ptr = 0;
                channel_ptr     = 0;
                break;
            }

        }
        break;
    }

    if( errorCodePtr )
        *errorCodePtr = error_code;

    FAPI_SYS_PRINT_DEBUG( 1,SFAL_PREFIX " SFAL_Open() (%d)\n", error_code);

    return( sfal_handle_ptr );
}

/*!
********************************************************************************
** \brief Close section filter handle.
**
** This function closes an instance of the SFAL subsystem. All resources
** associated with the given handle will be released.
**
** \attention After the handle is closed the given handle is invalid and
**            must not be further used.
**
** \param handle Valid subsystem instance handle previously returned by
**               #SFAL_Open().
**
** \sa #SFAL_Open()
**
********************************************************************************
*/
int32_t SFAL_Close(FAPI_SYS_HandleT handle)
{
    sfalHandleT*  sfal_handle_ptr = (sfalHandleT*)handle;

    if( sfalCheckHandle(sfal_handle_ptr) )
    {
        if(sfal_handle_ptr->active)
        {
            (void)SFAL_Stop(sfal_handle_ptr);
        }

        sfalReleaseFilter ( sfal_handle_ptr );
        sfalReleaseChannel( sfal_handle_ptr );
        sfalReleaseHandle ( sfal_handle_ptr );
    }
    return(FAPEX_OK);
}


/*!
********************************************************************************
** \brief Sets all filter masks.
**
** \param handle        Valid subsystem instance handle previously returned by
**                      SFAL_Open.
** \param filterDepth     Filter depth.
** \param matchPtr        Array of uint8_t (LSB is filtered byte).
** \param equalMaskPtr    Array of uint8_t (LSB is filtered byte).
** \param notEqualMaskPtr Array of uint8_t (LSB is filtered byte).
**
** \return
**  - #FAPEX_OK
**  - #SUBSYS_SFAL_ERR_BAD_PARAMETER
**  - #SUBSYS_SFAL_ERR_INVALID_HANDLE
**  - #SUBSYS_SFAL_ERR_NOT_INITIALIZED
**
********************************************************************************
*/
int32_t SFAL_SetFilter(FAPI_SYS_HandleT handle,
                            uint32_t    filterDepth,
                       const uint8_t*   matchPtr,
                       const uint8_t*   equalMaskPtr,
                       const uint8_t*   notEqualMaskPtr)
{
    sfalHandleT*    sfal_handle_ptr = (sfalHandleT*)handle;
    int32_t         return_value    = FAPEX_OK;

    for(;;)
    {
        if( !sfalInitialized )
        {
            return_value = SUBSYS_SFAL_ERR_NOT_INITIALIZED;
            break;
        }

        if ( !sfalCheckHandle(sfal_handle_ptr) )
        {
            return_value = SUBSYS_SFAL_ERR_INVALID_HANDLE;
            break;
        }

        return_value = sfalSetFilter(sfal_handle_ptr,
                filterDepth,
                matchPtr,
                equalMaskPtr,
                notEqualMaskPtr);

        break;
    }

    return( return_value );
}

/*!
********************************************************************************
** \brief Sets the PID were the section is filtered.
**
** \param handle        Valid subsystem instance handle previously returned by
**                      SFAL_Open.
** \param pid           PID which contains the requested section.
**
** \return
**  - #FAPEX_OK
**  - #SUBSYS_SFAL_ERR_BAD_PARAMETER
**  - #SUBSYS_SFAL_ERR_OUT_OF_MEMORY
**  - #SUBSYS_SFAL_ERR_INVALID_PID
**  - #SUBSYS_SFAL_ERR_ALREADY_ACTIVE
**  - #SUBSYS_SFAL_ERR_NO_FREE_CHANNEL
**  - #SUBSYS_SFAL_ERR_NO_FREE_FILTER
**
********************************************************************************
*/
int32_t SFAL_SetPid(FAPI_SYS_HandleT handle, uint32_t pid)
{
    sfalHandleT*   sfal_handle_ptr = (sfalHandleT*)handle;
    int32_t        return_value    = 0;
    sfalChannelT*  channel_ptr     = NULL;
    sfalFilterT*   filter_ptr      = NULL;

    channel_ptr = channel_ptr;
    filter_ptr  = filter_ptr;

    for(;;)
    {
        if( !sfalInitialized )
        {
            return_value = SUBSYS_SFAL_ERR_NOT_INITIALIZED;
            break;
        }

        if ( !sfalCheckHandle(sfal_handle_ptr) )
        {
           return_value = SUBSYS_SFAL_ERR_INVALID_HANDLE;
           break;
        }

        if( pid >= SFAL_INVALID_PID)
        {
            return_value = SUBSYS_SFAL_ERR_INVALID_PID;
            break;
        }

        if( sfal_handle_ptr->active )
        {
            return_value = SUBSYS_SFAL_ERR_ALREADY_ACTIVE;
            break;
        }

        sfalReleaseFilter ( sfal_handle_ptr );
        sfalReleaseChannel( sfal_handle_ptr );

        sfal_handle_ptr->pid = pid;

        channel_ptr = sfalAllocateChannel(sfal_handle_ptr);
        if( !sfalCheckChannel( sfal_handle_ptr ) )
        {
            return_value = SUBSYS_SFAL_ERR_NO_FREE_CHANNEL;
            sfal_handle_ptr->pid = SFAL_INVALID_PID;
            break;
        }

        filter_ptr = sfalAllocateFilter(sfal_handle_ptr);
        if( !filter_ptr )
        {
            sfalReleaseChannel( sfal_handle_ptr );
            sfal_handle_ptr->pid = SFAL_INVALID_PID;
            return_value = SUBSYS_SFAL_ERR_NO_FREE_FILTER;
            break;
        }

        if( sfal_handle_ptr->filterDepth )
        {
            (void)sfalSetFilter( sfal_handle_ptr,
                                 sfal_handle_ptr->filterDepth,
                                 sfal_handle_ptr->matchBytes,
                                 sfal_handle_ptr->equalBytes,
                                 sfal_handle_ptr->noneqBytes );
        }
        break;
    }

    FAPI_SYS_PRINT_DEBUG( 1,SFAL_PREFIX " SFAL_SetPid() (%d)\n", return_value);

    return( return_value );
}

/*!
********************************************************************************
** \brief Set's a callback for notify section filtered.
**
** \param handle        Valid subsystem instance handle previously returned by
**                      SFAL_Open.
** \param sectionNotify Notify that a section was filtered.
** \param sectionNotifyOptDataPtr Optional data pointer for the notify function.
**
** \return
**  - #FAPEX_OK
**  - #SUBSYS_SFAL_ERR_INVALID_HANDLE
**
** \sa #SFAL_SectionFilteredNotifyT
**
********************************************************************************
*/
int32_t SFAL_SetSectionNotify( FAPI_SYS_HandleT handle,
                               SFAL_SectionFilteredNotifyT sectionNotify,
                               void* sectionNotifyOptDataPtr )
{
    sfalHandleT*  sfal_handle_ptr = (sfalHandleT*)handle;

    if ( !sfalCheckHandle(sfal_handle_ptr) )
        return( SUBSYS_SFAL_ERR_INVALID_HANDLE );

    sfal_handle_ptr->sectionNotify           = sectionNotify;
    sfal_handle_ptr->sectionNotifyOptDataPtr = sectionNotifyOptDataPtr;

    return FAPEX_OK;
}

/*!
********************************************************************************
** \brief Starts filtering on a specified filter.
**
** After a one shot filtering, the filter is simply stopped and can be
** restarted at any time with #SFAL_Start().
**
** \param handle                  Valid subsystem instance handle previously returned by
**                                SFAL_Open.
** \param timeout               Timeout on the filter in [ms].
**                              If 0, the timeout is considered to be forever.
**                              In case of a timeout, the section received
**                              notification function will be called.
** \param oneShotMode             One shot flag possible values are #SFAL_LOOP_ONE_SHOT and
**                              #SFAL_LOOP_CONTINUOUS.
** \param ignoreCrc               Ignore CRC flag possible values are #SFAL_CRC_CHECK and
**                              #SFAL_CRC_IGNORE.
** \param bufferNotify     Notification to provide a buffer for the
**                              received section
** \param bufferNotifyOptDataPtr  Pointer to data passed through to the
**                                bufferNotify function.
** \param sectionNotify           Notify the reception of requested section.
** \param sectionNotifyOptDataPtr Pointer to data passed through to the
**                                sectionReceivedNotify function.
**
** \return
**  - #FAPEX_OK
**  - #SUBSYS_SFAL_ERR_BAD_PARAMETER
**  - #SUBSYS_SFAL_ERR_OUT_OF_MEMORY
**  - #SUBSYS_SFAL_ERR_NO_PID_ASSIGNED
**  - #SUBSYS_SFAL_ERR_NO_FREE_CHANNEL
**  - #SUBSYS_SFAL_ERR_NO_FREE_FILTER
**  - returned values from the TSD driver
**
** \sa  #SFAL_Stop()
** \sa  #SFAL_QueryBufferNotifyT
** \sa  #SFAL_SectionReceivedNotifyT
**
********************************************************************************
*/
int32_t SFAL_Start(FAPI_SYS_HandleT            handle,
                   uint32_t                    timeout,
                   uint32_t                    oneShotMode,
                   uint32_t                    ignoreCrc,
                   SFAL_QueryBufferNotifyT     bufferNotify,
                   void*                       bufferNotifyOptDataPtr,
                   SFAL_SectionReceivedNotifyT sectionNotify,
                   void*                       sectionNotifyOptDataPtr)
{
    sfalHandleT*              sfal_handle_ptr  = (sfalHandleT*)handle;
    sfalChannelT*             channel_ptr      = NULL;
    int32_t                   return_value     = FAPEX_OK;
    uint32_t                  num_tables       = 0;
    FAPEX_TSD_SctFltParamStrT*     filter_param_ptr = NULL;

    for(;;)
    {
        if( !sfalInitialized )
        {
            return_value = SUBSYS_SFAL_ERR_NOT_INITIALIZED;
            break;
        }

        if( (ignoreCrc != SFAL_CRC_CHECK) &&
            (ignoreCrc != SFAL_CRC_IGNORE) )
        {
             return_value = SUBSYS_SFAL_ERR_BAD_PARAMETER;
             break;
        }

        if( (oneShotMode != SFAL_LOOP_CONTINUOUS) &&
            (oneShotMode != SFAL_LOOP_ONE_SHOT) )
        {
             return_value = SUBSYS_SFAL_ERR_BAD_PARAMETER;
             break;
        }

        if ( !sfalCheckHandle(sfal_handle_ptr) )
        {
            return_value = SUBSYS_SFAL_ERR_INVALID_HANDLE;
            break;
        }

        if( sfal_handle_ptr->pid >= SFAL_INVALID_PID )
        {
            return_value = SUBSYS_SFAL_ERR_NO_PID_ASSIGNED;
            break;
        }

        if( sfal_handle_ptr->active )
        {
            return_value = SUBSYS_SFAL_ERR_ALREADY_ACTIVE;
            break;
        }

        sfal_handle_ptr->active                      = 1;
        sfal_handle_ptr->filterPtr->active           = 1;
        sfal_handle_ptr->timeout                     = timeout;
        sfal_handle_ptr->secReceivedNotify           = sectionNotify;
        sfal_handle_ptr->secReceivedNotifyOptDataPtr = sectionNotifyOptDataPtr;
        sfal_handle_ptr->bufferNotify                = bufferNotify;
        sfal_handle_ptr->bufferNotifyOptDataPtr      = bufferNotifyOptDataPtr;
        sfal_handle_ptr->oneShot                     = oneShotMode;
        sfal_handle_ptr->ignoreCrc                   = ignoreCrc;

        channel_ptr = sfal_handle_ptr->channelPtr;

        if( !channel_ptr->tsdBmReady )
        {
            return_value = sfalOpenChannel(channel_ptr, sfal_handle_ptr->tsdBlock);
            if(return_value != FAPEX_OK)
            {
                FAPI_SYS_PRINT_DEBUG( 2, SFAL_PREFIX " sfalOpenChannel() (%d)\n",return_value);
                break;
            }
        }

        if( channel_ptr->running )
        {
            /* stop the TSD channel */
	    return_value = FAPEX_TSD_Ioctl(channel_ptr->tsdHandle,
                                       FAPEX_IOCCOM_STOP, (FAPEXIOCARG3_T)NULL);
            if(return_value != FAPEX_OK)
            {
                FAPI_SYS_PRINT_DEBUG( 2, SFAL_PREFIX " FAPEX TSD StopChannel() (%d)\n",return_value);
                break;
            }
            channel_ptr->running = 0;
            (void)FAPEX_TSD_Ioctl(channel_ptr->tsdHandle, FAPEX_IOCCOM_TSD_CLEAR_SECTION_FILTER,
				  (FAPEXIOCARG3_T)NULL);
        }

        filter_param_ptr = sfalGetFilterTable( sfal_handle_ptr, &num_tables );

        /* set the section filters */
	return_value = sfalSetTsdSectionFilter( channel_ptr, num_tables, filter_param_ptr, timeout );
        if(return_value != FAPEX_OK)
        {
            FAPI_SYS_PRINT_DEBUG( 2,SFAL_PREFIX " FAPEX TSD SetSectionFilter() (%d)\n",return_value);
            break;
        }

	/* mmap BM buffer */
	if(channel_ptr->bmAddress == 0)
	{
	    return_value = (uint32_t)FAPEX_TSD_Mmap( NULL, sfalChannelBufferSize,
						     (PROT_READ | PROT_WRITE), MAP_SHARED,
						     (FAPEXMMPARG5_T)(channel_ptr->tsdHandle), 
						     (FAPEXMMPARG6_T)0);
            if(return_value < 0)
            {
            	return_value = SUBSYS_SFAL_ERR_OUT_OF_MEMORY;

            	FAPI_SYS_PRINT_MSG("[%s] FAPEX_TSD_Mmap() failed\n", __FUNCTION__);
            	break;
            }
	    channel_ptr->bmAddress = return_value;
	}

	/* clear section buffer */
	if(channel_ptr->bmAddress != 0)
	    memset((uint8_t*)channel_ptr->bmAddress, 0, sfalChannelBufferSize);

	/* add channel id to select */
	return_value = FAPEX_SYS_SelectSetID( &select_table, channel_ptr->tsdHandle,
					      (FAPEX_SELECT_MODE_READ | FAPEX_SELECT_MODE_EXPT));
	if(return_value != FAPEX_OK)
        {
            FAPI_SYS_PRINT_DEBUG( 2,SFAL_PREFIX " FAPEX_SYS_SelectSetID() failed (%d)\n",return_value);
            break;
        }
    if( channel_ptr->tsdHandle > select_max_id )
        select_max_id = channel_ptr->tsdHandle;

    /* start the TSD channel */
    return_value = FAPEX_TSD_Ioctl(channel_ptr->tsdHandle,
            FAPEX_IOCCOM_START, (FAPEXIOCARG3_T)NULL);
    if(return_value != FAPEX_OK)
    {
        FAPI_SYS_PRINT_DEBUG( 2,SFAL_PREFIX " FAPEX TSD StartChannel() (%d)\n",return_value);
            break;
        }
        channel_ptr->running = 1;

        if (sfal_handle_ptr->timeout != SFAL_NO_TIMEOUT)
        {
            sfal_handle_ptr->timeoutTime  = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);;
            sfal_handle_ptr->timeoutTime += sfal_handle_ptr->timeout;
            sfalNumOfTimeouts++;
            if(sfalThreadHandle != NULL)
            {
                (void)FAPEX_SYS_WakeupThread(sfalThreadHandle);
            }
        }

        break;
    }

    FAPI_SYS_PRINT_DEBUG( 1,SFAL_PREFIX " SFAL_Start() (%d)\n", return_value);

    return(return_value);
}

/*!
********************************************************************************
** \brief Stops the filter.
**
** \param handle                  Valid subsystem instance handle previously returned by
**                                SFAL_Open.
**
** \return
**  - #FAPEX_OK
**  - #SUBSYS_SFAL_ERR_NOT_INITIALIZED
**  - #SUBSYS_SFAL_ERR_INVALID_HANDLE
**  - #SUBSYS_SFAL_ERR_NO_PID_ASSIGNED
**  - #SUBSYS_SFAL_ERR_BAD_PARAMETER
**  - returned values from the TSD driver
**
** \sa #SFAL_Start()
**
********************************************************************************
*/
int32_t SFAL_Stop(FAPI_SYS_HandleT handle)
{
    sfalHandleT*              sfal_handle_ptr = (sfalHandleT*)handle;
    sfalChannelT*             channel_ptr     = NULL;
    int32_t                   return_value    = FAPEX_OK;
    uint32_t                  num_tables      = 0;
    FAPEX_TSD_SctFltParamStrT*     filter_param_ptr;

    for(;;)
    {
        if( !sfalInitialized )
        {
            return_value = SUBSYS_SFAL_ERR_NOT_INITIALIZED;
            break;
        }

        if ( !sfalCheckHandle(sfal_handle_ptr) )
        {
            return_value = SUBSYS_SFAL_ERR_INVALID_HANDLE;
            break;
        }

        if( sfal_handle_ptr->pid >= SFAL_INVALID_PID )
        {
            return_value = SUBSYS_SFAL_ERR_NO_PID_ASSIGNED;
            break;
        }

        if( sfal_handle_ptr->active == 0 )
        {
            return_value = FAPEX_OK;
            break;
        }

        channel_ptr = sfal_handle_ptr->channelPtr;

        if( (sfal_handle_ptr->timeout != SFAL_NO_TIMEOUT) &&
             sfalNumOfTimeouts )
        {
            /* the thread suspends itself when numTimeouts is equal 0 */
            sfalNumOfTimeouts--;
        }

        // TODO stop tsd and remove filter from PID filter list.
        // Then restart tsd with all other active filters

        if( channel_ptr->running )
        {
            /* stop the TSD channel */
            return_value = FAPEX_TSD_Ioctl(channel_ptr->tsdHandle,
                    FAPEX_IOCCOM_STOP, (FAPEXIOCARG3_T)NULL);
            if(return_value != FAPEX_OK)
            {
                FAPI_SYS_PRINT_DEBUG( 2, SFAL_PREFIX " FAPEX TSD StopChannel() (%d)\n",
                        return_value);
                break;
            }
        }

        /* set the section filters */
        (void)FAPEX_TSD_Ioctl(channel_ptr->tsdHandle, FAPEX_IOCCOM_TSD_CLEAR_SECTION_FILTER,
			      (FAPEXIOCARG3_T)NULL);

        sfal_handle_ptr->active                      = 0;
        sfal_handle_ptr->filterPtr->active           = 0;

        filter_param_ptr = sfalGetFilterTable( sfal_handle_ptr, &num_tables );

        if( !num_tables )
        {
            return_value = FAPEX_OK;
            sfalCloseChannel(channel_ptr);
            break;
        }

        return_value = sfalSetTsdSectionFilter( channel_ptr, num_tables, filter_param_ptr,
                sfal_handle_ptr->timeout );
        if(return_value != FAPEX_OK)
        {
            FAPI_SYS_PRINT_DEBUG( 2, SFAL_PREFIX" FAPEX TSD SetSectionFilter() (%d)\n",
                    return_value );
            break;
        }

        /* clear section buffer */
        if(channel_ptr->bmAddress != 0)
            memset((uint8_t*)channel_ptr->bmAddress, 0, sfalChannelBufferSize);

        if(channel_ptr->running)
        {
            /* restart the TSD channel */
            return_value = FAPEX_TSD_Ioctl(channel_ptr->tsdHandle,
                    FAPEX_IOCCOM_START, (FAPEXIOCARG3_T)NULL);
            if(return_value != FAPEX_OK)
            {
                FAPI_SYS_PRINT_DEBUG( 2, SFAL_PREFIX " FAPEX TSD StartChannel() (%d)\n",
                        return_value );
                break;
            }
        }
        break;
    }

    FAPI_SYS_PRINT_DEBUG( 1,SFAL_PREFIX " SFAL_Stop() (%d)\n", return_value);

    return( return_value );
}

/*!
********************************************************************************
**
** \brief   Thread function.
**
** \attention If RTOS usage was disabled, it must be ensured,
**            that this function is called (quite) frequently.
**
********************************************************************************
*/
void SFAL_ThreadFunction (void)
{
    uint32_t    		        tsdCnt;
    uint32_t    		        chCnt;
    FAPEX_TSD_ReadInfoStrT  	readInfo;
    FAPEX_SYS_TIMEVAL_T     	timeout;
    int32_t  			        return_value;

    if( !sfalInitialized )
    {
        return;
    }

    timeout.tv_sec  = 0;
    timeout.tv_usec = 100000;
    return_value = FAPEX_SYS_SelectSetParam(&select_table, select_max_id, &timeout);

    return_value = FAPEX_SYS_SelectCall( &select_table );
    switch( return_value )
    {
	case 0:
	    // TODO: handle timeout
	    break;
	case -1:
	    FAPI_SYS_PRINT_DEBUG( 1, "[%s]FAPEX_SYS_SelectCall %d\n", __FUNCTION__, return_value);
        perror("FAPEX_SYS_SelectCall error:");

	    return;

	    break;
	default:
	    ;
    }

    for(tsdCnt = 0; tsdCnt < SFAL_MAX_TSD_CHANNELS; tsdCnt++)
    {
        for(chCnt = 0; chCnt < SFAL_MAX_CHANNELS; chCnt++)
        {
            if( (sfalChannelArray[tsdCnt][chCnt].inUse) &&
                 (sfalChannelArray[tsdCnt][chCnt].running) &&
                 (sfalChannelArray[tsdCnt][chCnt].resetting != 1) )

                return_value = FAPEX_SYS_SelectCheck( &select_table,
                        sfalChannelArray[tsdCnt][chCnt].tsdHandle,
                        (FAPEX_SELECT_MODE_READ|FAPEX_SELECT_MODE_EXPT));

            if( return_value > 0 )
            {
                return_value = FAPEX_TSD_Ioctl( sfalChannelArray[tsdCnt][chCnt].tsdHandle,
                        FAPEX_IOCCOM_TSD_GET_BUFFER_READINFO,
                        (FAPEXIOCARG3_T)&readInfo );
                if( return_value == FAPEX_OK )
                {
                    if( readInfo.bufferStatus & FAPEX_TSD_BUFFSTAT_OVERFLOW )
                    {
                        FAPI_SYS_PRINT_DEBUG( 1, SFAL_PREFIX " buffer overflow \n" );
                        sfalBmOverflowHandler(tsdCnt, chCnt, &(sfalChannelArray[tsdCnt][chCnt]));  
                    }
                    else if( readInfo.bufferStatus & FAPEX_TSD_BUFFSTAT_TIMEOUT )
                    {
                        FAPI_SYS_PRINT_DEBUG( 1, SFAL_PREFIX " section receive timeout\n" );
                    }
                    else
                    {
                        //for debug
#if DEBUG > 1
                        FAPI_SYS_PRINT_MSG("\n[%s] fillSize:%d, sectionOffset:%d,", __FUNCTION__,
                                readInfo.fillSize, readInfo.sectionOffset);
                        FAPI_SYS_PRINT_MSG("sectionLength:%d, sectionNo:%d,",
                                readInfo.sectionLength, readInfo.sectionNo);
                        FAPI_SYS_PRINT_MSG("readOffset:%d, bufferStatus:%d\n",
                                readInfo.readOffset, readInfo.bufferStatus);
                        FAPI_SYS_PRINT_MSG("\n[%s] channel buffer size: %d, crcError: %d \n",
                                __FUNCTION__, sfalChannelBufferSize, 
                                readInfo.bufferStatus & FAPEX_TSD_BUFFSTAT_CRCERROR );
#endif

                        sfalNotifier( &(sfalChannelArray[tsdCnt][chCnt]), &readInfo );
                    }
                }
            }
        }
    }
}

/******************************************************************************/
/*lint -restore */
