/*!
*****************************************************************************
**
** \file        ./subsystems/sfal/src/sfal.h
**
**              $Id: sfal.h 5280 2011-02-21 16:39:28Z wlaris $
**
** \brief       Section Filter Abstraction Layer.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010  by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef _SFAL_H_
    #define _SFAL_H_

#include "fapex/abst_types.h"
#include "fapex/abst_libtsd.h"

    //!< Maximum depth of section bytes.
    #define SFAL_MAX_DEPTH            12

    //!< Minimum filter depth (tableId + section length).
    #define SFAL_FILTERDEPTH_MIN      3

    //!< Maximum filter depth
    #define SFAL_FILTERDEPTH_MAX      ((SFAL_MAX_DEPTH) + (SFAL_FILTERDEPTH_MIN))

    //!< Filter runs till SFAL_StopFilter() is called
    #define SFAL_NO_TIMEOUT           0

    /*!
    ********************************************************************************
    ** Standard error codes for the driver
    ********************************************************************************
    */

    /*!< Bad parameter passed. */
    #define SUBSYS_SFAL_ERR_BAD_PARAMETER         (SUBSYS_SFAL_MODULE_BASE +\
                                                     FAPI_ERR_BAD_PARAMETER)

    /*! Memory allocation failed. */
    #define SUBSYS_SFAL_ERR_OUT_OF_MEMORY         (SUBSYS_SFAL_MODULE_BASE +\
                                                      FAPI_ERR_OUT_OF_MEMORY)

    /*! Device already initialised. */
    #define SUBSYS_SFAL_ERR_ALREADY_INITIALIZED   (SUBSYS_SFAL_MODULE_BASE +\
                                                        FAPI_ERR_ALREADY_INITIALIZED)

    /*! Device not initialised. */
    #define SUBSYS_SFAL_ERR_NOT_INITIALIZED       (SUBSYS_SFAL_MODULE_BASE +\
                                                        FAPI_ERR_NOT_INITIALIZED)

    /*! Feature or function is not available. */
    #define SUBSYS_SFAL_ERR_FEATURE_NOT_SUPPORTED (SUBSYS_SFAL_MODULE_BASE +\
                                                     FAPI_ERR_FEATURE_NOT_SUPPORTED)

    /*! Timeout occurred. */
    #define SUBSYS_SFAL_ERR_TIMEOUT               (SUBSYS_SFAL_MODULE_BASE +\
                                                      FAPI_ERR_TIMEOUT)

    /*! The device is busy, try again later. */
    #define SUBSYS_SFAL_ERR_DEVICE_BUSY           (SUBSYS_SFAL_MODULE_BASE +\
                                                       FAPI_ERR_DEVICE_BUSY)

    /*! Invalid handle was passed. */
    #define SUBSYS_SFAL_ERR_INVALID_HANDLE        (SUBSYS_SFAL_MODULE_BASE +\
                                                        FAPI_ERR_INVALID_HANDLE)

    /*! Semaphore could not be created. */
    #define SUBSYS_SFAL_ERR_SEMAPHORE_CREATE      (SUBSYS_SFAL_MODULE_BASE +\
                                                      FAPI_ERR_SEMAPHORE_CREATE)

    /*! The driver's used version is not supported. */
    #define SUBSYS_SFAL_ERR_UNSUPPORTED_VERSION   (SUBSYS_SFAL_MODULE_BASE +\
                                                        FAPI_ERR_UNSUPPORTED_VERSION)

    /*! The mail queue could not be created. */
    #define SUBSYS_SFAL_ERR_MAILQUEUE_CREATE      (SUBSYS_SFAL_MODULE_BASE +\
                                                     FAPI_ERR_MAILQUEUE_CREATE)

    /*!
    ********************************************************************************
    ** Additional driver error codes (reserved from -100 to -999)
    **
    ** Each error code shall be decreased in steps of 1.
    ********************************************************************************
    */

    /* SFAL error base */
    #define SUBSYS_SFAL_ERR_BASE                       (SUBSYS_SFAL_MODULE_BASE - 100)
    /*! \brief The provided PID is invalid. */
    #define SUBSYS_SFAL_ERR_INVALID_PID                (SUBSYS_SFAL_ERR_BASE -  0)
    /*! \brief No PID is associated with the provided to filter handle. */
    #define SUBSYS_SFAL_ERR_NO_PID_ASSIGNED            (SUBSYS_SFAL_ERR_BASE -  1)
    /*! \brief The thread could not be created. */
    #define SUBSYS_SFAL_ERR_THREAD_CREATE              (SUBSYS_SFAL_ERR_BASE -  2)
    /*! \brief No free channel found to start filter. */
    #define SUBSYS_SFAL_ERR_NO_FREE_CHANNEL            (SUBSYS_SFAL_ERR_BASE -  3)
    /*! \brief No free filter found. */
    #define SUBSYS_SFAL_ERR_NO_FREE_FILTER             (SUBSYS_SFAL_ERR_BASE -  4)
    /*! \brief A timeout has occurred and the filter was stopped. */
    #define SUBSYS_SFAL_ERR_SECTION_TIMEOUT            (SUBSYS_SFAL_ERR_BASE -  5)
    /*! \brief The section was received but with an invalid CRC. */
    #define SUBSYS_SFAL_ERR_SECTION_CRC                (SUBSYS_SFAL_ERR_BASE -  6)
    /*! \brief Filter already active */
    #define SUBSYS_SFAL_ERR_ALREADY_ACTIVE             (SUBSYS_SFAL_ERR_BASE -  7)
    /*! \brief Filter depth to high */
    #define SUBSYS_SFAL_ERR_FILTER_DEPTH               (SUBSYS_SFAL_ERR_BASE -  8)
    /*! \brief Channel buffer to small */
    #define SUBSYS_SFAL_ERR_BUFFER_SIZE_TO_SMALL       (SUBSYS_SFAL_ERR_BASE -  9)

        /*!
        ** \brief Continuous filtering mode.
        **  The filter works continuously.
        **  Only a set timeout could stop it automatically.
        */
    #define SFAL_LOOP_CONTINUOUS     0

        /*!
        ** \brief One shot filtering mode.
        **  After a matching section arrives the
        **  filter will be stopped automatically.
        */
    #define SFAL_LOOP_ONE_SHOT       1

        /*!
        ** \brief The checksum (CRC) of an arrived section is verified.
        **  If the checksums do not match, the section will be notified.
        */
    #define SFAL_CRC_CHECK      0

        /*!
        ** \brief The CRC of an arrived section is ignored.
        */
    #define SFAL_CRC_IGNORE     1

        /*!
        ** \brief Invalid PID
        */
    #define SFAL_INVALID_PID        0x1FFF

    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************

    /*!
    *************************************************************************
    ** \brief Version of the SFAL subsystem.
    **
    ** This number consists of a \b major version number stored in bits
    ** 16 to 31 (upper word) and a \b minor version number stored in bits
    ** 0 to 15 (lower word).
    ** - The \b major number has to be increased when a parameter change
    **   occurs for the existing driver's API after its first release.
    ** - The \b minor number has to be increased when functions are added to
    **   the existing driver's API after its first release.
    **
    ** \note This value has to be assigend to the \b version field of the
    **       FAPI_TSD_OpenParamsT structure.
    **
    *************************************************************************
    */

    typedef enum {
        SFAL_VERSION = (int32_t)0x00010001   //!< The current subsystem version
    } SFAL_VersionEnumT;

    /*!
    *******************************************************************************
    **
    ** \brief  Notify function for requesting a buffer.
    **
    ** \param  handle          Handle of filter requesting a buffer.
    ** \param  ppBuffer        Pointer to buffer pointer to be set.
    ** \param  pBufferSize     Pointer to buffer size to be set.
    ** \param  optDataPtr    Private data pointer set with SFAL_FilterStart().
    **
    ** \sa     SFAL_FilterStart
    **
    ******************************************************************************/
    typedef void (*SFAL_QueryBufferNotifyT)( FAPI_SYS_HandleT handle,
                                         uint8_t**        ppBuffer,
                                         uint16_t*        pBufferSize,
                                         void*            optDataPtr);

    /*!
    *******************************************************************************
    **
    ** \brief  Notify function for informing about a filtered section.
    **
    ** \code
    **  pSectionBufferStart ---> --------------------------------
    **                           | ############################ |
    **                           | ############################ |
    **                           |                              |
    **                           |                              |
    **                           |                              |
    **  pSectionBuffer      ---> | ############################ |
    **                           | ############################<=== section data
    **                           | ############################ |
    **  pSectionBufferEnd   ---> --------------------------------
    **
    ** \endcode
    **
    ** \param  handle               Handle of filter.
    ** \param  pSectionBufferStart  Pointer to beginning of the BM-buffer.
    ** \param  pSectionBufferEnd    Pointer to end of the BM-buffer.
    ** \param  pSectionBuffer       Pointer to buffer containing the section.
    ** \param  sectionSize          Section size in bytes.
    ** \param  optDataPtr         Private data pointer set with SFAL_FilterStart().
    **
    ** \return 1 to keep section, 0 to discard it.
    **
    ** \sa     SFAL_FilterStart
    **
    ******************************************************************************/
    typedef uint8_t (*SFAL_SectionFilteredNotifyT)(FAPI_SYS_HandleT    handle,
                                               const volatile uint8_t* pSectionBufferStart,
                                               const volatile uint8_t* pSectionBufferEnd,
                                               const volatile uint8_t* pSectionBuffer,
                                               uint16_t                sectionSize,
                                               void*                   optDataPtr);

    /*!
    *******************************************************************************
    **
    ** \brief  Notify function for informing about a received section.
    **
    ** \param  handle           Handle of filter requesting a buffer.
    ** \param  sectionStatus    Status of the provided section.
    ** \param  sectionBufferPtr Pointer to buffer containing the section.
    ** \param  sectionSize      Section size in bytes.
    ** \param  optDataPtr       Private data pointer set with SFAL_FilterStart().
    **
    ** \sa     SFAL_FilterStart
    **
    ******************************************************************************/
    typedef void (*SFAL_SectionReceivedNotifyT)(FAPI_SYS_HandleT  handle,
                                            int32_t               sectionStatus,
                                            const uint8_t*        sectionBufferPtr,
                                            uint16_t              sectionSize,
                                            void*                 optDataPtr);

    /*!
    *******************************************************************************
    **
    ** \brief  Filter set configuration data
    **
    ** \sa     SFAL_OpenFilterset
    ** \sa     SFAL_CloseFilterset
    **
    *******************************************************************************
    */
    typedef struct
    {
        /*!
        ** \brief Priority of thread.
        */
        uint32_t threadPriority;

        /*!
        ** \brief Milliseconds to sleep.
        **
        **  If a filter is assigned with a time out, the driver thread will sleep
        **  a bit before checking the timeout again. If no timeout is set, the
        **  thread will be suspended.
        **  A reasonable value is between 70ms and 150ms.
        */
        uint32_t threadSleep;

        /*!
        ** \brief Size of the internal channel buffer.
        **
        ** The size of the buffer must be a minimum value of 4kbyte.
        ** kByteBufferSize = 4 => 4096Bytes internal buffer used by one channel.
        */
        uint32_t kByteBufferSize;

    } SFAL_InitT;

    /*!
    *************************************************************************
    ** \brief Open parameters for the SFAL subsystem.
    **
    ** This data structure covers all parameters which need to be specified
    ** when an instance of the subsystem is opened.
    **
    *************************************************************************
    */
    typedef struct
    {
        /*! \brief The version of the subsystem.
        **         \ref SFAL_VersionEnumT "SFAL_VERSION".
        */
        SFAL_VersionEnumT       version;

        /*!
        **  \brief TSD input to be used.
        **
        **  Possible values are FAPI_TSD0, FAPI_TSD1, FAPI_TSD2 or FAPI_TSD3.
        */
        uint32_t                tsdInput;

        uint32_t                pid;

        uint32_t                filterDepth;

        uint8_t*                matchPtr;

        uint8_t*                equalMaskPtr;

        uint8_t*                notEqualMaskPtr;

    } SFAL_OpenParamsT;

/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/
#ifdef __cplusplus
    extern "C" {
    #endif

    int32_t          SFAL_Init              (const SFAL_InitT* initParamsPtr);
    void             SFAL_Exit              (void);
    FAPI_SYS_HandleT SFAL_Open              (SFAL_OpenParamsT* paramsPtr, int32_t* errorCodePtr);
    int32_t          SFAL_Close             (FAPI_SYS_HandleT handle);
    int32_t          SFAL_SetFilter         (FAPI_SYS_HandleT handle,
                                                 uint32_t    filterDepth,
                                            const uint8_t*   matchPtr,
                                            const uint8_t*   equalMaskPtr,
                                            const uint8_t*   notEqualMaskPtr);
    int32_t          SFAL_SetPid            (FAPI_SYS_HandleT handle, uint32_t pid);
    int32_t          SFAL_SetSectionNotify  (FAPI_SYS_HandleT            handle,
                                             SFAL_SectionFilteredNotifyT sectionNotify,
                                             void*                       sectionNotifyOptDataPtr );
    int32_t          SFAL_Start             (FAPI_SYS_HandleT            handle,
                                             uint32_t                    timeout,
                                             uint32_t                    oneShotMode,
                                             uint32_t                    ignoreCrc,
                                             SFAL_QueryBufferNotifyT     bufferNotify,
                                             void*                       bufferNotifyOptDataPtr,
                                             SFAL_SectionReceivedNotifyT sectionNotify,
                                             void*                       sectionNotifyOptDataPtr);
    int32_t          SFAL_Stop              (FAPI_SYS_HandleT handle);

    void 	     SFAL_ThreadFunction    (void);
    #ifdef __cplusplus
    }
#endif

#endif /* _SFAL_H_ */
