/***********************************************************************
 * MODULE     : DsubTClut
 * FILE       : DsubTClut.c
 * COPYRIGHT  : TARA Systems GmbH
 * DESCRIPTION:
 *    Representation of the CLUT_definition_segment. All CLUTs are 
 *    8-bit CLUTs. 4-bit and 2-bit must be mapped on 8-bit. 
 *
 *    A CLUT can be built-up of RGBA values or of YCbCrT values for 
 *    SDTV or of YCbCrT values for HDTV. 
 *    For details please refer to the various GET and SET defines.
 ***********************************************************************/

#ifndef DSUBTCLUT_H
#define DSUBTCLUT_H


#include <subtitle/DsubStddef.h>


/*****************************************************************************
 * TYPE: DsubTColorType
 *
 * DESCRIPTION:
 *   Defines the different color mappings.
 *
 * COMPONENTS:
 *   DSUB_COLOR_TYPE_YCBCRT_SDTV - Color format as defined in ITU-R BT.601-5
 *                                   plus a transparency value. 
 *   DSUB_COLOR_TYPE_YCBCRT_HDTV - Color format as defined in ITU-R BT.709
 *                                    plus a transparency value.
 *   DSUB_COLOR_TYPE_RGBA8888  - The color table contains 256 colors. Each color
 *                               is stored in RGBA8888 format. Use the
 *                               DSUB_CLUT_GET_XXX macros to access the
 *                               color values.
 *   DSUB_COLOR_TYPE_MAPPING   - A mapping table used to map the pixels to a user
 *                               defined CLUT, which was passed to the DsubDecoder
 *                               using the function DsubDecoderSetDisplayDriver().
 *                               It consists of 256 entries.
 ***************************************************************************/
 
typedef enum
{
  DSUB_COLOR_TYPE_YCBCRT_SDTV  = 1,
  DSUB_COLOR_TYPE_YCBCRT_HDTV,
  DSUB_COLOR_TYPE_RGBA8888,
  DSUB_COLOR_TYPE_MAPPING
    
} DsubTColorType;


/***************************************************************************
 * TYPE: DsubTClutAlpha
 *
 * DESCRIPTION:
 *   These values describe the level transparency.
 *
 * COMPONENTS:
 *   DSUB_CLUT_NO_ALPHA   - The color is full opaque. 
 *   DSUB_CLUT_FULL_ALPHA - The color is full transparent. 
 ***************************************************************************/

typedef enum
{
  DSUB_CLUT_NO_ALPHA   = 0x00,
  DSUB_CLUT_FULL_ALPHA = 0xFF
  
} DsubTClutAlpha;




/***************************************************************************
 * SECTION: #defines for setting/getting values from an RGBA CLUT
 ***************************************************************************/


/***************************************************************************
 * DEFINE: DSUB_CLUT_SET_RED
 *
 * DESCRIPTION:
 *   Sets the RED value of the CLUT entry as unsigned character. 
 ***************************************************************************/

#define DSUB_CLUT_SET_RED( clutEntry, value ) \
  {                                 \
    (clutEntry) &=  0x00FFFFFFUL;     \
    (clutEntry) |= ((((unsigned long) value ) & 0xFFUL) << 24); \
  }


/***************************************************************************
 * DEFINE: DSUB_CLUT_SET_GREEN
 *
 * DESCRIPTION:
 *   Sets the GREEN value of the CLUT entry as unsigned character. 
 ***************************************************************************/

#define DSUB_CLUT_SET_GREEN(  clutEntry, value ) \
  {                                 \
    (clutEntry) &=  0xFF00FFFFUL;          \
    (clutEntry) |= ((((unsigned long) value ) & 0xFFUL) << 16); \
  }

/***************************************************************************
 * DEFINE: DSUB_CLUT_SET_BLUE
 *
 * DESCRIPTION:
 *   Sets the BLUE value of the CLUT entry as unsigned character. 
 ***************************************************************************/

#define DSUB_CLUT_SET_BLUE( clutEntry, value )  \
  {                                \
    (clutEntry) &=  0xFFFF00FFUL;    \
    (clutEntry) |= ((((unsigned long) value ) & 0xFFUL ) << 8); \
  }


/***************************************************************************
 * DEFINE: DSUB_CLUT_SET_ALPHA
 *
 * DESCRIPTION:
 *   Sets the ALPHA value of the CLUT entry as unsigned character. 
 *
 *   0x00:
 *     A transparency value of 0x00 means 'no transparency'.
 *
 *   0xFF:
 *     A transparency value of 0xFF means 'full transparency'.
 ***************************************************************************/

#define DSUB_CLUT_SET_ALPHA( clutEntry, value ) \
  {                                 \
    (clutEntry) &=  0xFFFFFF00UL;     \
    (clutEntry) |= (((unsigned long) (value)) & 0xFFUL ); \
  }


/***************************************************************************
 * DEFINE: DSUB_CLUT_GET_RED
 *
 * DESCRIPTION:
 *   Returns the RED value of the CLUT entry as unsigned character. 
 ***************************************************************************/

#define DSUB_CLUT_GET_RED( clutEntry )  \
  ( (unsigned char) ((( clutEntry )  >> 24 ) & 0xFFUL ))


/***************************************************************************
 * DEFINE: DSUB_CLUT_GET_GREEN
 *
 * DESCRIPTION:
 *   Returns the GREEN value of the CLUT entry as unsigned character. 
 ***************************************************************************/

#define DSUB_CLUT_GET_GREEN( clutEntry ) \
  ( (unsigned char) ((( clutEntry )  >> 16 ) & 0xFFUL ))


/***************************************************************************
 * DEFINE: DSUB_CLUT_GET_BLUE
 *
 * DESCRIPTION:
 *   Returns the BLUE value of the CLUT entry as unsigned character. 
 ***************************************************************************/

#define DSUB_CLUT_GET_BLUE( clutEntry )  \
  ( (unsigned char) ((( clutEntry )  >>  8 ) & 0xFFUL ))


/***************************************************************************
 * DEFINE: DSUB_CLUT_GET_ALPHA
 *
 * DESCRIPTION:
 *   Returns the ALPHA value of the CLUT entry as unsigned character. 
 *
 *   0x00:
 *     A transparency value of 0x00 means 'no transparency'.
 *
 *   0xFF:
 *     A transparency value of 0xFF means 'full transparency'.
 ***************************************************************************/

#define DSUB_CLUT_GET_ALPHA( clutEntry ) \
  ( (unsigned char) (( clutEntry ) & 0xFFUL ))





/***************************************************************************
 * SECTION: #defines for setting/getting values from an YCrCbT SDTV CLUT
 ***************************************************************************/

/***************************************************************************
 * DEFINE: DSUB_CLUT_YCBCRT_SDTV_SET_Y
 *
 * DESCRIPTION:
 *   Sets the Y value of the CLUT entry as unsigned character. 
 *   
 *   NOTE: According to ITU-R BT.601.5 only Y values in the range [16;235]
 *   are allowed. This define does not check whether the value is within
 *   this range or not. If the passed value is not within the range it 
 *   cannot be guaranteed that the CLUT mapping of the DVB Subtitle 
 *   Decoder works correctly.
 ***************************************************************************/

#define DSUB_CLUT_YCBCRT_SDTV_SET_Y( clutEntry, value ) \
  {                                 \
    (clutEntry) &=  0x00FFFFFFUL;     \
    (clutEntry) |= ((((unsigned long) value ) & 0xFFUL) << 24); \
  }


/***************************************************************************
 * DEFINE: DSUB_CLUT_YCBCRT_SDTV_SET_CB
 *
 * DESCRIPTION:
 *   Sets the CB value of the CLUT entry as unsigned character. 
 *
 *   NOTE: According to ITU-R BT.601.5 only CB values in the range [16;240]
 *   are allowed, with 128 corresponding to 0. This define does not check
 *   whether the value is within this range or not. 
 *   If the passed value is not within the range it cannot be guaranteed that 
 *   the CLUT mapping of the DVB Subtitle Decoder works correctly.
 ***************************************************************************/

#define DSUB_CLUT_YCBCRT_SDTV_SET_CB(  clutEntry, value ) \
  {                                 \
    (clutEntry) &=  0xFF00FFFFUL;          \
    (clutEntry) |= ((((unsigned long) value ) & 0xFFUL) << 16); \
  }


/***************************************************************************
 * DEFINE: DSUB_CLUT_YCBCRT_SDTV_SET_CR
 *
 * DESCRIPTION:
 *   Sets the CB value of the CLUT entry as unsigned character. 
 *
 *   NOTE: According to ITU-R BT.601.5 only CR values in the range [16;240]
 *   are allowed, with 128 corresponding to 0. This define does not check
 *   whether the value is within this range or not. 
 *   If the passed value is not within the range it cannot be guaranteed that 
 *   the CLUT mapping of the DVB Subtitle Decoder works correctly.
 ***************************************************************************/

#define DSUB_CLUT_YCBCRT_SDTV_SET_CR( clutEntry, value )  \
  {                                \
    (clutEntry) &=  0xFFFF00FFUL;    \
    (clutEntry) |= ((((unsigned long) value ) & 0xFFUL ) << 8); \
  }


/***************************************************************************
 * DEFINE: DSUB_CLUT_YCBCRT_SDTV_SET_T
 *
 * DESCRIPTION:
 *   Sets the T (Transparency) value of the CLUT entry as unsigned character. 
 *
 *   0x00:
 *     A transparency value of 0x00 means 'no transparency'.
 *
 *   0xFF:
 *     A transparency value of 0xFF means 'full transparency'.
 ***************************************************************************/

#define DSUB_CLUT_YCBCRT_SDTV_SET_T( clutEntry, value ) \
  {                                 \
    (clutEntry) &=  0xFFFFFF00UL;     \
    (clutEntry) |= (((unsigned long) (value)) & 0xFFUL ); \
  }


/***************************************************************************
 * DEFINE: DSUB_CLUT_YCBCRT_SDTV_GET_Y
 *
 * DESCRIPTION:
 *   Returns the Y value of the CLUT entry as unsigned character. 
 *   If a Y value of 0 is broadcast, it is set to Y=16d and T is set to 
 *   T=0xFF. Set ETSI EN 300 743 for details. 
 *   All other broadcast values for Y are returned here as they are: 
 *   It is not checked whether they are valid according to ITU-R BT601 or not.
 ***************************************************************************/

#define DSUB_CLUT_YCBCRT_SDTV_GET_Y( clutEntry )  \
  ( (unsigned char) ((( clutEntry )  >> 24 ) & 0xFFUL ))


/***************************************************************************
 * DEFINE: DSUB_CLUT_YCBCRT_SDTV_GET_CB
 *
 * DESCRIPTION:
 *   Returns the CB value of the CLUT entry as unsigned character. 
 *   The broadcast value is returned here as it is: It is not checked 
 *   whether it is valid according to ITU-R BT601 or not.
 ***************************************************************************/

#define DSUB_CLUT_YCBCRT_SDTV_GET_CB( clutEntry ) \
  ( (unsigned char) ((( clutEntry )  >> 16 ) & 0xFFUL ))


/***************************************************************************
 * DEFINE: DSUB_CLUT_YCBCRT_SDTV_GET_CR
 *
 * DESCRIPTION:
 *   Returns the CR value of the CLUT entry as unsigned character. 
 *   The broadcast value is returned here as it is: It is not checked 
 *   whether it is valid according to ITU-R BT601 or not. 
 ***************************************************************************/

#define DSUB_CLUT_YCBCRT_SDTV_GET_CR( clutEntry )  \
  ( (unsigned char) ((( clutEntry )  >>  8 ) & 0xFFUL ))


/***************************************************************************
 * DEFINE: DSUB_CLUT_YCBCRT_SDTV_GET_T
 *
 * DESCRIPTION:
 *   Returns the T (Transparency) value of the CLUT entry as 
 *   unsigned character. 
 *
 *   0x00:
 *     A transparency value of 0x00 means 'no transparency'.
 *
 *   0xFF:
 *     A transparency value of 0xFF means 'full transparency'.
 ***************************************************************************/

#define DSUB_CLUT_YCBCRT_SDTV_GET_T( clutEntry ) \
  ( (unsigned char) (( clutEntry ) & 0xFFUL ))



/***************************************************************************
 * SECTION: #defines for setting/getting values from an YCrCbT HDTV CLUT
 ***************************************************************************/

/***************************************************************************
 * DEFINE: DSUB_CLUT_YCBCRT_HDTV_SET_Y
 *
 * DESCRIPTION:
 *   Sets the Y value of the CLUT entry as unsigned character. 
 *   
 *   NOTE: According to ITU-R BT.709 only Y values in the range [16;235]
 *   are allowed. This define does not check whether the value is within
 *   this range or not. If the passed value is not within the range it 
 *   cannot be guaranteed that the CLUT mapping of the DVB Subtitle 
 *   Decoder works correctly.
 ***************************************************************************/

#define DSUB_CLUT_YCBCRT_HDTV_SET_Y( clutEntry, value ) \
  {                                 \
    (clutEntry) &=  0x00FFFFFFUL;     \
    (clutEntry) |= ((((unsigned long) value ) & 0xFFUL) << 24); \
  }


/***************************************************************************
 * DEFINE: DSUB_CLUT_YCBCRT_HDTV_SET_CB
 *
 * DESCRIPTION:
 *   Sets the CB value of the CLUT entry as unsigned character. 
 *
 *   NOTE: According to ITU-R BT.709 only CB values in the range [16;240]
 *   are allowed, with 128 corresponding to 0. This define does not check
 *   whether the value is within this range or not. 
 *   If the passed value is not within the range it cannot be guaranteed that 
 *   the CLUT mapping of the DVB Subtitle Decoder works correctly.
 ***************************************************************************/

#define DSUB_CLUT_YCBCRT_HDTV_SET_CB(  clutEntry, value ) \
  {                                 \
    (clutEntry) &=  0xFF00FFFFUL;          \
    (clutEntry) |= ((((unsigned long) value ) & 0xFFUL) << 16); \
  }


/***************************************************************************
 * DEFINE: DSUB_CLUT_YCBCRT_HDTV_SET_CR
 *
 * DESCRIPTION:
 *   Sets the CR value of the CLUT entry as unsigned character. 
 *
 *   NOTE: According to ITU-R BT.709 only CR values in the range [16;240]
 *   are allowed, with 128 corresponding to 0. This define does not check
 *   whether the value is within this range or not. 
 *   If the passed value is not within the range it cannot be guaranteed that 
 *   the CLUT mapping of the DVB Subtitle Decoder works correctly.
 ***************************************************************************/

#define DSUB_CLUT_YCBCRT_HDTV_SET_CR( clutEntry, value )  \
  {                                \
    (clutEntry) &=  0xFFFF00FFUL;    \
    (clutEntry) |= ((((unsigned long) value ) & 0xFFUL ) << 8); \
  }


/***************************************************************************
 * DEFINE: DSUB_CLUT_YCBCRT_HDTV_SET_T
 *
 * DESCRIPTION:
 *   Sets the T (Transparency) value of the CLUT entry as unsigned character. 
 *
 *   0x00:
 *     A transparency value of 0x00 means 'no transparency'.
 *
 *   0xFF:
 *     A transparency value of 0xFF means 'full transparency'.
 ***************************************************************************/

#define DSUB_CLUT_YCBCRT_HDTV_SET_T( clutEntry, value ) \
  {                                 \
    (clutEntry) &=  0xFFFFFF00UL;     \
    (clutEntry) |= (((unsigned long) (value)) & 0xFFUL ); \
  }


/***************************************************************************
 * DEFINE: DSUB_CLUT_YCBCRT_HDTV_GET_Y
 *
 * DESCRIPTION:
 *   Returns the Y value of the CLUT entry as unsigned character. 
 *   If a Y value of 0 is broadcast, it is set to Y=16d and T is set to 
 *   T=0xFF. Set ETSI EN 300 743 for details. 
 *   All other broadcast values for Y are returned here as they are: 
 *   It is not checked whether they are valid according to  ITU-R BT.709 or not.
 ***************************************************************************/

#define DSUB_CLUT_YCBCRT_HDTV_GET_Y( clutEntry )  \
  ( (unsigned char) ((( clutEntry )  >> 24 ) & 0xFFUL ))


/***************************************************************************
 * DEFINE: DSUB_CLUT_YCBCRT_HDTV_GET_CB
 *
 * DESCRIPTION:
 *   Returns the CB value of the CLUT entry as unsigned character. 
 *   The broadcast value is returned here as it is: It is not checked 
 *   whether it is valid according to  ITU-R BT.709 or not.
 ***************************************************************************/

#define DSUB_CLUT_YCBCRT_HDTV_GET_CB( clutEntry ) \
  ( (unsigned char) ((( clutEntry )  >> 16 ) & 0xFFUL ))


/***************************************************************************
 * DEFINE: DSUB_CLUT_YCBCRT_HDTV_GET_CR
 *
 * DESCRIPTION:
 *   Returns the CR value of the CLUT entry as unsigned character. 
 *   The broadcast value is returned here as it is: It is not checked 
 *   whether it is valid according to  ITU-R BT.709 or not. 
 ***************************************************************************/

#define DSUB_CLUT_YCBCRT_HDTV_GET_CR( clutEntry )  \
  ( (unsigned char) ((( clutEntry )  >>  8 ) & 0xFFUL ))


/***************************************************************************
 * DEFINE: DSUB_CLUT_YCBCRT_HDTV_GET_T
 *
 * DESCRIPTION:
 *   Returns the T (Transparency) value of the CLUT entry as 
 *   unsigned character. 
 *
 *   0x00:
 *     A transparency value of 0x00 means 'no transparency'.
 *
 *   0xFF:
 *     A transparency value of 0xFF means 'full transparency'.
 ***************************************************************************/

#define DSUB_CLUT_YCBCRT_HDTV_GET_T( clutEntry ) \
  ( (unsigned char) (( clutEntry ) & 0xFFUL ))


/***************************************************************************
 * TYPE: DsubTClutEntry
 *
 * DESCRIPTION:
 *   This type represents a CLUT entry. It defines a color which is used
 *   to display the subtitles. The CLUT contains colors in the color format
 *   RGBA - 8888. 
 ***************************************************************************/

typedef unsigned long  DsubTClutEntry;



#endif /* DSUBTCLUT_H */

/* This Headerfile was generated with C2H.exe ( C2H ) */
/* (c) 2006 by TARA Systems GmbH Munich */
