/****************************************************************************
 * MODULE     : DsubTDecoder
 * FILE       : DsubTDecoder.h
 * COPYRIGHT  : TARA Systems GmbH
 * DESCRIPTION:
 *   This module contains the functions needed to access the DVB Subtitle
 *   Decoder.
 *
 *   The Subtitle Decoder decodes DVB subtitles according to ETSI EN 300 743
 *   V1.2.1 (2002-06). DVB Subtitles are run-length encoded bitmaps which are
 *   transmitted in PES packets.
 *
 *   To decode subtitles a DsubDecoder must be created. A DisplayDriver which
 *   copies the decoded subtitles to the platform's pixelbuffer must be
 *   created and passed to the decoder by a call to
 *   DsubDecoderSetDisplayDriver(). To ensure the correct timing of subtitle
 *   processing a DsubTGetStcTime function must be provided to the decoder
 *   with a call to DsubDecoderSetStcCallback().
 *
 *   Before starting decoding of packets the composition_page_id and the
 *   ancillary_page_id for the subtitle service must be set using the
 *   function DsubDecoderSetPageIds(). A call to DsubDecoderStart() starts
 *   the decoding process. PES packets containing subtitle data are added to
 *   the decoder's PesQueue using the DsubDecoderAddPesPacket() function.
 *   When DsubDecoderProcess() is called a packet is removed from the queue
 *   and is decoded. A call to DsubDecoderDraw() checks if the right point in
 *   time for diplaying has reached. If yes, the decoder sends the decoded
 *   bitmaps to the DsubDisplayDriver which brings the subtitles to the OSD.
 *
 *   The decoder is partially thread-safe. Only two tasks are allowed to call
 *   the DsubDecoder functions: All DsubDecoder...() functions must be called
 *   from within the same task. Except DsubDecoderAddPesPacket() may be
 *   called from another task. On destroying the DsubDecoder, please make
 *   sure that the task calling DsubDecoderAddPesPacket was stopped.
 ****************************************************************************/

#ifndef DSUBTDECODER_H
#define DSUBTDECODER_H


typedef struct DsubTDecoderStruct *DsubTDecoder;

#include <subtitle/DsubTDecoderTypes.h>
#include <subtitle/DsubTDisplayDriver.h>



#ifdef __cplusplus
  extern "C" {
#endif


/****************************************************************************
 * FUNCTION: DsubDecoderCreate
 *
 * DESCRIPTION:
 *   This function creates a new DsubDecoder.
 *
 *   The decoder never calls malloc: instead a block of memory is passed to
 *   the decoder which manages the allocation and deallocation of the needed
 *   memory by itself.
 *
 * PARAMETERS:
 *   aMemory         - A block of memory which is managed by the decoder.
 *   aMemorySize     - The size of the aMemory block.
 *   aNormVersion    - The decoder will behave according to this norm.
 *   aGetCurrentTime - This callback function is used to retrieve the current
 *                     time of the platform's system clock in seconds. It is
 *                     needed to check the timeout of displayed subtitles.
 *                     Must not be DSUB_NULL.
 *   aUserData       - This pointer is passed to the aGetCurrentTime callback
 *                     everytime it is called. DSUB_NULL is allowed.
 *
 * RETURNS:
 *   A new DsubDecoder or DSUB_NULL if an error occurred.
 *
 * SEE ALSO:
 *   DsubDecoderDestroy
 *   DsubTGetTime
 *   DsubDecoderSetStcCallback
 *   DsubDecoderSetDisplayDriver
 ****************************************************************************/

PUBLIC DsubTDecoder
DsubDecoderCreate (
  void *                  aMemory,
  unsigned long           aMemorySize,
  DsubTNormVersion        aNormVersion,  
  DsubTGetTime            aGetCurrentTime,
  void *                  aUserData );


/****************************************************************************
 * FUNCTION: DsubDecoderDestroy
 *
 * DESCRIPTION:
 *   This function destroys a DsubDecoder.
 *
 * PARAMETERS:
 *   This - The decoder to be destroyed.
 *
 * SEE ALSO:
 *   DsubDecoderCreate
 ****************************************************************************/

PUBLIC void
DsubDecoderDestroy (
  DsubTDecoder This );


/****************************************************************************
 * FUNCTION: DsubDecoderSetDisplayDriver
 *
 * DESCRIPTION:
 *   This function sets the DisplayDriver-callback functions. The previously
 *   registered functions are overwritten and the decoder is reset.
 *
 *   The aDisplayDriverDraw function is called everytime a subtitle bitmap
 *   must be copied to the platform's pixelbuffer and then be displayed on
 *   the screen. The aDisplayDriverClear function is always called when a
 *   subtitle must be removed from the screen.
 *
 *   To unregister the DisplayDriver call this function with
 *   aDisplayDriverDraw = DSUB_NULL and aDisplayDriverClear = DSUB_NULL.
 *
 *   The parameter aClut passes a user-defined CLUT to the decoder. The
 *   decoder creates mapping tables which are used in the DisplyDriver for
 *   setting the colors (see DisplayDriver for details). If DSUB_NULL is
 *   passed no mapping-table but a CLUT is passed to the DisplayDriver which
 *   must be used to write the pixels into the OSD buffer.
 *
 *   The decoder positions the subtitles. For this the decoder needs to know
 *   the actual screen bounds. The parameter aClip defines what to do with
 *   subtitles that do not fit onto the screen.
 *
 * PARAMETERS:
 *   This -
 *     The decoder.
 *
 *   aDisplayDriverDraw -
 *     The draw function of the DisplayDriver.
 *
 *   aDisplayDriverClear -
 *     The clear function of the DisplayDriver.
 *
 *   aDisplayDriverSetDisplayBounds -
 *     The setDisplayBounds function of the DisplayDriver.
 *
 *   aColorType -
 *     The color type of the passed aClut. If no CLUT is passed, this type
 *     defines the Color type of the CLUT which is passed to the
 *     DisplayDriverDraw function. Allowed types are:
 *     DSUB_COLOR_TYPE_RGBA8888, DSUB_COLOR_TYPE_YCBCRT_SDTV and
 *     DSUB_COLOR_TYPE_YCBCRT_HDTV.
 *
 *   aClut -
 *     A user-defined CLUT: An array of 256 DsubTClutEntry values. DSUB_NULL
 *     is allowed for this parameter.
 *
 *   aScreenBounds -
 *     The size of the screen. aScreenBounds.X1 and Y1 must be 0. If the
 *     screen has a width of 720 pixels, aScreenBounds.X2 must be set to 720.
 *     aScreenBounds.X2 and Y2 must be > 0.
 *
 *   aClip -
 *     The clipping mode: Defines whether subtitle are clipped or discareded,
 *     if they do not fit onto the screen.
 *
 *   aUserData -
 *     This pointer is passed to the registered aDisplayDriverDraw and
 *     aDisplayDriverClear function everytime they are called. May be
 *     DSUB_NULL.
 ****************************************************************************/

PUBLIC void
DsubDecoderSetDisplayDriver (
  DsubTDecoder                       This,
  DsubTDisplayDriverDraw             aDisplayDriverDraw,
  DsubTDisplayDriverClear            aDisplayDriverClear,
  DsubTDisplayDriverSetDisplayBounds aDisplayDriverSetDisplayBounds,
  DsubTColorType                     aColorType,
  const DsubTClutEntry *             aClut,
  DsubTRect                          aScreenBounds,
  DsubTDecoderDisplayClip            aClip,
  void *                             aUserData );


/****************************************************************************
 * FUNCTION: DsubDecoderSetStcCallback
 *
 * DESCRIPTION:
 *   This function sets a new StcCallback.
 *
 *   To display the subtitles in time the aGetStcTime callback must be
 *   passed. The decoder checks at each call to DsubDecoderProcess() if the
 *   point in time to display a new subtitle has reached. The aGetStcTime
 *   function is used to do this.
 *
 *   If DSUB_NULL is passed as aGetStcTime the decoder processes an available
 *   PES packet and displays the subtitles as soon as DsubDecoderProcess is
 *   called. In this case the timing is not checked. This is the default.
 *
 *   It is recommended to call DsubDecoderStop() and DsubDecoderReset()
 *   before calling this function.
 *
 * PARAMETERS:
 *   This            - The decoder.
 *   aStcComposition - Defines which bits of the PTS will be compared to the
 *                     STC. See DsubTStcComposition for details.
 *   aGetStcTime     - The new STC callback to be set.
 *   aUserData       - This pointer is passed to the aGetStcTime callback
 *                     everytime it is called. DSUB_NULL is allowed.
 *
 * RETURN:
 *   DSUB_TRUE   if succesful
 *   DSUB_FALSE  otherwise
 *
 * SEE ALSO:
 *   DsubTStcComposition
 ****************************************************************************/

PUBLIC DsubTBoolean
DsubDecoderSetStcCallback (
  DsubTDecoder        This,
  DsubTStcComposition aStcComposition,
  DsubTGetStcTime     aGetStcTime, 
  void *              aUserData );


/****************************************************************************
 * FUNCTION: DsubDecoderSetPageIds
 *
 * DESCRIPTION:
 *   Sets the new Ids of the ancillary and the composition page.
 *
 *   The pages with the passed Ids will be decoded next. Normally different
 *   languages are transmitted on different subtitle pages. The Ids are used
 *   to identify a particular language. Which id belongs to which language
 *   can be found out from the program map table (PMT). Refer to ETSI EN 300
 *   468 for details.
 *
 *   Before calling this function DsubDecoderStop() and DsubDecoderReset()
 *   should be called.
 *
 * PARAMETERS:
 *   This        - The page ids of this decoder are set.
 *   aCompPageId - The id of the decoded composition page id.
 *   aAncPageId  - The id of the decoded ancillary page. If no ancillary page
 *                 is used, please pass the same value as for aCompPageId.
 ****************************************************************************/

PUBLIC void
DsubDecoderSetPageIds (
  DsubTDecoder   This,
  unsigned short aCompPageId,
  unsigned short aAncPageId );


/****************************************************************************
 * FUNCTION: DsubDecoderSetAlignment
 *
 * DESCRIPTION:
 *   When the transmitted subtitle display bounds are less than the bounds of
 *   the screen (set by a call to DsubDecoderSetDisplayDriver()), the
 *   subtitles will be positioned on the screen according to the passed
 *   alignment. If no alignment is set, the default alignment is used:
 *   DSUB_DECODER_ALIGN_V_BOTTOM and DSUB_DECODER_ALIGN_H_CENTER.
 *
 * PARAMETERS:
 *   This        - The decoder.
 *   aAlignmentH - The horizontal alignment.
 *   aAlignmentV - The vertical alignement.
 ****************************************************************************/

PUBLIC void
DsubDecoderSetAlignment ( 
  DsubTDecoder        This,
  DsubTDecoderAlignH  aAlignmentH,
  DsubTDecoderAlignV  aAlignmentV );


/****************************************************************************
 * FUNCTION: DsubDecoderStart
 *
 * DESCRIPTION:
 *   This function starts a DVB Subtitle Decoder. If decoding was stopped
 *   using DsubDecoderStop() the decoder continues decoding the next packet
 *   found in its PES packet queue.
 *
 * PARAMETERS:
 *   This - This decoder will be started.
 *
 * SEE ALSO:
 *   DsubDecoderStop
 ****************************************************************************/

PUBLIC void
DsubDecoderStart (
  DsubTDecoder   This );


/****************************************************************************
 * FUNCTION: DsubDecoderStop
 *
 * DESCRIPTION:
 *   This function stops a DVB Subtitle Decoder. It can be restarted using
 *   the function DsubDecoderStart().
 *
 * PARAMETERS:
 *   This - This decoder will be stopped.
 *
 * SEE ALSO:
 *   DsubDecdoerStart
 ****************************************************************************/

PUBLIC void
DsubDecoderStop (
  DsubTDecoder   This );


/****************************************************************************
 * FUNCTION: DsubDecoderReset
 *
 * DESCRIPTION:
 *   This function resets the DVB Subtitle Decoder. The internal PES packet
 *   queue is cleared. It also resets the decoder info. A call to
 *   DsubDecoderDraw removes visible subtitles from the screen.
 *
 *   Note: This function resets only the decoding. Settings like Composition
 *   Page Id, Ancillary Page Id, StcCallback, DisplayDriver and alignment
 *   will NOT be reset to a default value by calling DsubDecoderReset().
 *
 * PARAMETERS:
 *   This - This decoder will be reset.
 *
 * SEE ALSO:
 *   DsubTDecoderInfo
 ****************************************************************************/

PUBLIC void
DsubDecoderReset (
  DsubTDecoder   This );


/****************************************************************************
 * FUNCTION: DsubDecoderAddPesPacket
 *
 * DESCRIPTION:
 *   Adds a complete packetized elementary stream (PES) packet to the
 *   decoder's PES packet queue.
 *
 *   To remove a packet from the queue and decode it a call to
 *   DsubDecoderProcess is necessary.
 *
 * PARAMETERS:
 *   This        - The subtitle decoder.
 *   aPesPacket  - The packet to be decoded.
 *   aPacketSize - The packet size in bytes.
 ****************************************************************************/

PUBLIC void
DsubDecoderAddPesPacket (
  DsubTDecoder         This,
  const unsigned char* aPesPacket,
  unsigned short       aPacketSize );


/****************************************************************************
 * FUNCTION: DsubDecoderGetInfo
 *
 * DESCRIPTION:
 *   Returns information about the DVB Subtitle Decoder.
 *
 * PARAMETERS:
 *   This  - The subtitle decoder.
 *   aInfo - Pointer to a DsubTInfo struct which will be filled with data.
 ****************************************************************************/

PUBLIC void
DsubDecoderGetInfo (
  DsubTDecoder       This,
  DsubTDecoderInfo*  aInfo );


/****************************************************************************
 * FUNCTION: DsubDecoderProcess
 *
 * DESCRIPTION:
 *   If the current STC value matches the Presentation Time Stamp (PTS) of
 *   one or more PES packets in the queue, the packets are removed form the
 *   queue and decoded.
 *
 *   If no callback function for STC value retrieval was passed to the
 *   DsubDecoderSetStcCallback() function the time stamps cannot be checked.
 *   In this case one or more packets with the same time stamp are removed
 *   from the PES packet queue and decoded immediately.
 *
 *   Data stream related reset of the decoder is handled internally.
 *
 * PARAMETERS:
 *   This - The subtitle decoder.
 *
 * RETURNS:
 *   DSUB_TRUE   if data is processed.
 *   DSUB_FALSE  if no data is processed.
 *
 * SEE ALSO:
 *   DsubDecoderDraw
 *
 * EXAMPLE:
 *   // example code for processing the PES queue in time
 *   while( (DsubDecoderProcess( dsub ) == DSUB_TRUE) &&
 *          (callCount++ < threshold) );
 *
 *   // update the display (see DsubDecoderDraw)
 ****************************************************************************/

PUBLIC DsubTBoolean
DsubDecoderProcess ( 
  DsubTDecoder This );


/****************************************************************************
 * FUNCTION: DsubDecoderGetFirstUpdateRect
 *
 * DESCRIPTION:
 *   This function returns the first rectangular area to be drawn.
 *
 *   Note: When this function was called and returned a DsubTRect with not
 *   all values 0, then only the functions DsubDecoderGetNextUpdateRect() and
 *   DsubDecoderDraw() may be called. This restriction is valid until
 *   DsubDecoderGetNextUpdateRect() returned a rect with all values 0. If any
 *   other function is called the behaviour of the decoder is undefined.
 *
 * PARAMETERS:
 *   This - The subtitle decoder.
 *
 * RETURNS:
 *   A rectangular area to be drawn. If all members of the returned DsubTRect
 *   have got the value 0, no update is necessary.
 *
 * SEE ALSO:
 *   DsubDecoderGetNextUpdateRect
 *   DsubDecoderDraw
 ****************************************************************************/

PUBLIC DsubTRect
DsubDecoderGetFirstUpdateRect ( 
  DsubTDecoder This );


/****************************************************************************
 * FUNCTION: DsubDecoderGetNextUpdateRect
 *
 * DESCRIPTION:
 *   This function returns the next rectangular area to be drawn.
 *
 * PARAMETERS:
 *   This - The subtitle decoder.
 *
 * RETURNS:
 *   A rectangular area to be drawn. If all members of the returned DsubTRect
 *   have got the value 0, all areas have been updated.
 *
 * SEE ALSO:
 *   DsubDecoderGetFirstUpdateRect
 *   DsubDecoderDraw
 ****************************************************************************/

PUBLIC DsubTRect
DsubDecoderGetNextUpdateRect ( 
  DsubTDecoder This );


/****************************************************************************
 * FUNCTION: DsubDecoderDraw
 *
 * DESCRIPTION:
 *   This function calls the registered DisplayDriver functions to draw
 *   subtitle bitmaps to the screen or to make them invisible.
 *
 *   DsubDecoderDraw() shall be called everytime DsubDecoderProcess() decoded
 *   a subtitle packet successfully. Call DsubDecoderGetFirstUpdateRect() or
 *   DsubDecoderGetNextUpdateRect() to get the areas which must be redrawn.
 *   See example below.
 *
 * PARAMETERS:
 *   This       - The subtitle decoder.
 *   aRectangle - The rectangular area to be drawn. This area must be a
 *                rectangle returned by a call to
 *                DsubDecoderGetFirstUpdateRect() or
 *                DsubDecoderGetNextUpdateRect().
 *
 * EXAMPLE:
 *   DsubTRect updateRect;
 *
 *   // Lock here the background-buffer bitmap into which the
 *   // display driver will  write the pixel data.
 *
 *   updateRect = DsubDecoderGetFirstUpdateRect( This );
 *   while ( updateRect.X1 || updateRect.X2 ||
 *           updateRect.Y1 || updateRect.Y2 )
 *   {
 *     DsubDecoderDraw( This, updateRect );
 *     updateRect = DsubDecoderGetNextUpdateRect( This );
 *   }
 *
 *   // Unlock here the bitmap into which the subtitle data was
 *   // written. The background-buffer can now be drawn onto the
 *   // screen.
 *
 * SEE ALSO:
 *   DsubTGetStcTime
 *   DsubDecoderProcess
 *   DsubDecoderInvalidateRect
 *   DsubDecoderGetFirstUpdateRect
 *   DsubDecoderGetNextUpdateRect
 ****************************************************************************/

PUBLIC void
DsubDecoderDraw ( 
  DsubTDecoder This,
  DsubTRect    aRectangle );


/****************************************************************************
 * FUNCTION: DsubDecoderInvalidateRect
 *
 * DESCRIPTION:
 *   This function invalidates a rectangle of the display. All subtitle
 *   regions which lay in this rectangle will be repainted at the next call
 *   to DsubDecoderDraw. Parts of the rectangle which are not covered by a
 *   subtitle region are ignored.
 *
 *   DsubDecoderInvalidateRect() shall be called for example when an OSD,
 *   which is laying over the subtitles, is removed from the screen. Then the
 *   decoder calculates which areas must be repainted with subtitle data.
 *   After a call to DsubDecoderInvalidateRect() the function
 *   DsubDecoderDraw() shall be called to repaint the bitmaps if necessary.
 *
 *   This function shall not be called, when DsubDecoderGetFirstUpdateRect()
 *   has been called and DsubDecoderGetFirstUpdateRect() or
 *   DsubDecoderGetNextUpdateRect() has not returned a rect with all values
 *   0, yet.
 *
 * PARAMETERS:
 *   This       - The subtitle decoder.
 *   aRectangle - The rectangle of the display to invalidate.
 *
 * SEE ALSO:
 *   DsubDecoderDraw
 ****************************************************************************/

PUBLIC void
DsubDecoderInvalidateRect ( 
  DsubTDecoder This,
  DsubTRect    aRectangle );


/****************************************************************************
 * FUNCTION: DsubDecoderGetVersion
 *
 * DESCRIPTION:
 *   This function returns the version of the subtitle decoder as string.
 *
 * RETURNS:
 *   The version string of the decoder.
 ****************************************************************************/

PUBLIC const char *
DsubDecoderGetVersion ( void );


/****************************************************************************
 * FUNCTION: DsubDecoderCheckVersion
 *
 * DESCRIPTION:
 *   Checks if the given Version String is compatible with the current
 *   Version of the lib. Format: <major version>.<minor version> e.g. 1.01.01
 *
 * PARAMETERS:
 *   aVersionString - The version string required by the caller
 *   aReturnMessage - text buffer for error messages
 *   aMessageSize   - size of the text buffer
 *
 * RETURN:
 *   true if compatible false otherwise
 ****************************************************************************/

PUBLIC DsubTBoolean  
DsubDecoderCheckVersion ( 
  char*           aVersionString, 
  char*           aReturnMessage, 
  int             aMessageSize );


/****************************************************************************
 * FUNCTION: DsubDecoderSetLogHandler
 *
 * DESCRIPTION:
 *   The function DsubDecoderSetLogHandler() sets a log handler. The same log
 *   handler is used for all DsubDecoder instances. This handler is called
 *   whenever a log (error, warning, info) is generated by a DsubDecoder. A
 *   user defined pointer can be registered with the log handler. This
 *   pointer is passed to the log handler. If not used this parameter should
 *   be set to DSUB_NULL.
 *
 * PARAMETERS:
 *   aLogHandler - The log handler to be set.
 *   aUserData   - A user defined pointer.
 *
 * SEE ALSO:
 *   DsubTLogHandler
 *
 * EXAMPLE:
 *   int SubtitlingLogHandler (
 *     void *              aUserData,
 *     const char *        aFunctionName,
 *     unsigned int        aLogId,
 *     unsigned int        aLogLevel,
 *     const char *        aLogText )
 *   {
 *     switch ( aLogLevel )
 *     {
 *     case DSUB_LOG_LEVEL_ERROR:
 *       printf( "**** ERROR: %s - %s\n", aFunctionName, aLogText );
 *       break;
 *
 *     case DSUB_LOG_LEVEL_INFO:
 *       printf( "**** INFO: %s - %s\n", aFunctionName, aLogText );
 *       break;
 *
 *     default:
 *       break;
 *     }
 *     return 0;
 *   }
 *
 *   ...
 *   DsubDecoderSetLogHandler( SubtitlingLogHandler, DSUB_NULL );
 ****************************************************************************/

PUBLIC void
DsubDecoderSetLogHandler (
  DsubTLogHandler aLogHandler,
  void *          aUserData );


/****************************************************************************
 * FUNCTION: DsubDecoderEnableLog
 *
 * DESCRIPTION:
 *   The function DsubDecoderEnableLog() enables a single log or a group of
 *   logs. With DSUB_ERROR_ALL all errors, with DSUB_WARNING_ALL all warnings
 *   and with DSUB_INFO_ALL all infos are enabled. Depending on the platform
 *   only a subset of all log ids is supported. To disable a log use the
 *   function DsubDecoderDisableLog().
 *
 * PARAMETERS:
 *   aLogId - The id of the log to be enabled.
 *
 * RETURN:
 *   DSUB_TRUE   if the log is successfully enabled.
 *   DSUB_FALSE  otherwise.
 *
 * SEE ALSO:
 *   DsubDecoderDisableLog()
 *   DesubDecoderSetLogHandler()
 *   DsubTLogId
 *
 * EXAMPLE:
 *   DsubDecoderEnableLog( DSUB_ERROR_ALL );
 *   DsubDecoderEnableLog( DSUB_INFO_PES );
 ****************************************************************************/

PUBLIC DsubTBoolean
DsubDecoderEnableLog (
  DsubTLogId aLogId );


/****************************************************************************
 * FUNCTION: DsubDecoderDisableLog
 *
 * DESCRIPTION:
 *   The function DsubDecoderDisableLog() disables a single log or a group of
 *   logs. With DSUB_ERROR_ALL all errors, with DSUB_WARNING_ALL all warnings
 *   and with DSUB_INFO_ALL all infos are disabled. Depending on the platform
 *   only a subset of all log ids is supported. To enable a log use the
 *   function DsubDecoderEnableLog().
 *
 * PARAMETERS:
 *   aLogId - The id of the log to be disabled.
 *
 * RETURN:
 *   DSUB_TRUE   if the log is successfully disabled.
 *   DSUB_FALSE  otherwise.
 *
 * SEE ALSO:
 *   DsubDecoderDisableLog()
 *   DsubDecoderSetLogHandler()
 *   DsubTLogId
 ****************************************************************************/

PUBLIC DsubTBoolean
DsubDecoderDisableLog (
  DsubTLogId aLogId );


#ifdef __cplusplus
  }
#endif

#endif /* DSUBTDECODER_H */



/* This Headerfile was generated with C2H.exe ( C2H ) */
/* (c) 2006 by TARA Systems GmbH Munich */
