/****************************************************************************
 * MODULE     : DsubTDisplayDriver
 * FILE       : DsubTDisplayDriver.h
 * COPYRIGHT  : TARA Systems GmbH
 * DESCRIPTION:
 *   This module contains the data types which are necessary for implementing
 *   a DisplayDriver for the DVB Subtitle Decoder. The DisplayDriver is used
 *   to copy the decoded subtitle bitmaps to the OSD buffer of the device.
 *
 *   A DisplayDriver must provide three functions:
 *     1) one of type DsubTDisplayDriverDraw, which is used to copy a bitmap
 *        to the OSD buffer,
 *     2) one of type DsubTDisplayDriverClear, which sets a particular area
 *        of the OSD buffer to full transparency,
 *     3) one of type DsubTDisplayDriverSetDisplayBounds which sets the
 *        bounds of the display needed to draw the subtitles
 *
 *   The display shall not be updated after every call of the
 *   DisplayDriverDraw() and DisplayDriverClear() functions. The screen
 *   update shall take place when all subtitles have been drawn to the
 *   background-buffer. Please see description of DsubDecoderDraw() for
 *   details.
 *
 * EXAMPLE:
 *
 *    void
 *    DisplayDriverDraw (
 *      void *         aUserData,
 *      DsubTBitmap    aBitmap,
 *      DsubTRect      aSrcRect,
 *      DsubTRect      aDestRect )
 *    {
 *      // add code here
 *    }
 *
 *    void
 *    DisplayDriverClear (
 *      void *         aUserData,
 *      DsubTRect      aRect )
 *    {
 *      // add code here
 *    }
 *
 *    //-------------
 *    // within main:
 *    //-------------
 *    ...
 *    DsubTDecoder    subtitleDecoder;
 *
 *    subtitleDecoder = DsubDecoderCreate(
 *         Mem, MemSize, GetSeconds, DSUB_NULL );
 *    DsubDecoderSetDisplayDriver( subtitleDecoder, DisplayDriverDraw,
 *       DisplayDriverClear, DSUB_COLOR_TYPE_RGBA8888, DSUB_NULL, UserData );
 ****************************************************************************/

#ifndef DSUBTDISPLAYDRIVER_H
#define DSUBTDISPLAYDRIVER_H


#include <subtitle/DsubStddef.h>
#include <subtitle/DsubTClut.h>


/****************************************************************************
 * TYPE: DsubTBitmapType
 *
 * DESCRIPTION:
 *   Defines the different bitmap types.
 *
 * COMPONENTS:
 *   DSUB_BITMAP_TYPE_INTERNAL -
 *     The Data member of DsubTBitmap contains a pixelbuffer which is managed
 *     by the DsubTDecoder.
 *
 *   DSUB_BITMAP_TYPE_EXTERNAL -
 *     Currently not supported.
 ****************************************************************************/
 
typedef enum
{
  DSUB_BITMAP_TYPE_INTERNAL,
  DSUB_BITMAP_TYPE_EXTERNAL
  
} DsubTBitmapType;


/****************************************************************************
 * TYPE: DsubTRect
 *
 * DESCRIPTION:
 *   This type defines a rectangle.
 *
 * COMPONENTS:
 *   X1 - Left position of the rectangle (inside the rectangle).
 *   Y1 - Top position of the rectangle (inside the rectangle).
 *   X2 - Right position of the rectangle (outside the rectangle).
 *   Y2 - Bottom position of the rectangle (outside the rectangle).
 ****************************************************************************/

typedef struct 
{
  long       X1;
  long       Y1;
  long       X2;
  long       Y2;
  
} DsubTRect;



/****************************************************************************
 * TYPE: DsubTBitmap
 *
 * DESCRIPTION:
 *   This type defines a bitmap. The bitmap contains all data needed to copy
 *   a decoded subtitle region into the OSD's pixel buffer.
 *
 *   Two different color types - stored in ColorType - are supported.
 *   Depending on the ColorType, the Color pointer must be used in different
 *   ways.
 *
 *   DSUB_COLOR_TYPE_RGBA8888:
 *     If the bitmap is of this type, the Color pointer contains 256 RGBA
 *     colors. It is indexed from 0 to 255. Each value of the Data buffer is
 *     an index for the CLUT of the bitmap.
 *
 *   DSUB_COLOR_TYPE_YCBCRT_SDTV:
 *     If the bitmap is of this type, the Color pointer contains 256 YCbCrT
 *     colors. It is indexed from 0 to 255. Each value of the Data buffer is
 *     an index for the CLUT of the bitmap.
 *
 *   DSUB_COLOR_TYPE_YCBCRT_HDTV:
 *     If the bitmap is of this type, the Color pointer contains 256 YCbCrT
 *     colors. It is indexed from 0 to 255. Each value of the Data buffer is
 *     an index for the CLUT of the bitmap.
 *
 *   DSUB_COLOR_TYPE_MAPPING:
 *     Each value of the Data buffer is an index for the mapping table of the
 *     bitmap. The mapping table returns an index for a user defined CLUT,
 *     which was registered via DsubDecoderSetDisplayDriver(). The mapping
 *     table can be accessed via the Color pointer. It has 256 entries.
 *
 *   Two different bitmap types - stored in BitmapType - are supported.
 *   Depending on the BitmapType the Data pointer must be used in different
 *   ways.
 *
 *   DSUB_BITMAP_TYPE_INTERNAL:
 *     Data points to an internal bitmap. To access the pixels of the bitmap
 *     Data must be cast to an 'unsigned char *'.
 *
 *   DSUB_BITMAP_TYPE_EXTERNAL:
 *     Data points to a user defined external bitmap. This is currently not
 *     supported.
 *
 *   The first value of the buffer represents the top-left pixel of the
 *   bitmap. The second value represents the next to the right and so on for
 *   the first row of pixels, which is given by Bounds.X2 - Bounds.X1. Then
 *   the next row of pixels starts.
 *
 * COMPONENTS:
 *   Bounds     - Position and size of the bitmap.
 *   ColorType  - The color type of the bitmap.
 *   Color      - Color points either to a CLUT or to a mapping table,
 *                depending on the ColorType. It is DSUB_NULL, if the
 *                BitmapType is of type DSUB_BITMAP_TYPE_EXTERNAL.
 *   BitmapType - Type of the Bitmap.
 *   Data       - Data points either to a pixelbuffer or to a user-defined
 *                bitmap, depending on the BitmapType.
 *
 * EXAMPLE:
 *
 *   screen-origin
 *   o------------------------------+
 *   |                              |   The screen size passed to
 *   |                              |   DsubDecoderSetDisplayDriver()
 *   |                              |   is used here.
 *   |    bitmap-origin             |
 *   |    o--------------------+    |
 *   |    |       BITMAP       |    |
 *   |    +--------------------+    |
 *   |                              |
 *   +------------------------------+
 *
 * SEE ALSO:
 *   DsubTColorType
 ****************************************************************************/

typedef struct 
{
  DsubTRect          Bounds;
  DsubTColorType     ColorType;
  unsigned long *    Color;
  DsubTBitmapType    BitmapType;
  void *             Data;
  
} DsubTBitmap;


/****************************************************************************
 * CALLBACK: DsubTDisplayDriverDraw
 *
 * DESCRIPTION:
 *   A function of this type is used to draw subtitles to the screen. The
 *   function must be registered at the DsubDecoder by a call to
 *   DsubDecoderSetDisplayDriver(). It is called everytime a subtitle region
 *   or a part of a region must be painted to the screen during a call to
 *   DsubDecoderDraw().
 *
 *   Only the part of the bitmap which is defined by aSrcRect must be
 *   repainted. It is recommended to draw only the area enclosed by aSrcRect
 *   and not to paint the whole bitmap when this function is called. The
 *   DsubDecoder may call this function several times with the same aBitmap
 *   but different aSrcRect areas.
 *
 * PARAMETERS:
 *   aUserData - The user data registered at the function
 *               DsubDecoderSetDisplayDriver is passed to this callback.
 *   aBitmap   - A part of this bitmap must be drawn to the screen.
 *   aSrcRect  - The area within the bitmap which must be painted. This
 *               rectangle is relative to the bitmap-origin.
 *   aDestRect - The area, relative to the screen-origin, into which the
 *               aSrcRect content shall be copied.
 *
 * EXAMPLE:
 *   screen-origin
 *   o------------------------------+
 *   |                              |     The screen size passed to
 *   |                              |     DsubDecoderSetDisplayDriver
 *   |                              |     is used here.
 *   |    bitmap-origin             |
 *   |    o--------####--------+    |     ####   Rectangle (aSrcRect)
 *   |    |        ####        |    |     #### = to be
 *   |    +--------####--------+    |     ####   updated
 *   |                              |
 *   +------------------------------+
 *
 * SEE ALSO:
 *   DsubTDisplayDriverClear
 ****************************************************************************/

typedef 
void ( * DsubTDisplayDriverDraw ) (
  void *       aUserData,
  DsubTBitmap  aBitmap,
  DsubTRect    aSrcRect,
  DsubTRect    aDestRect );
  

/****************************************************************************
 * CALLBACK: DsubTDisplayDriverClear
 *
 * DESCRIPTION:
 *   A function of this type must be implemented by the user. It removes
 *   subtitle data from the screen. The function is registered to the
 *   DsubDecoder by a call to DsubDecoderSetDisplayDriver(). It is called
 *   everytime a subtitle must be removed from the screen during a call to
 *   DsubDecoderDraw().
 *
 * PARAMETERS:
 *   aUserData - The user data registered at the function
 *               DsubDecoderSetDisplayDriver is passed to this callback.
 *   aRect     - The rect of the display to be cleared.
 *
 * SEE ALSO:
 *   DsubTDisplayDriverDraw
 ****************************************************************************/

typedef 
void ( * DsubTDisplayDriverClear ) (
  void *     aUserData,
  DsubTRect  aRect );


/****************************************************************************
 * CALLBACK: DsubTDisplayDriverSetDisplayBounds
 *
 * DESCRIPTION:
 *   A function of this type must be implemented by the user. It informs the
 *   display driver about the display size which is necessary to display the
 *   subtitles. The function is registered to the DsubDecoder by a call to
 *   DsubDecoderSetDisplayDriver(). It is called everytime the display size
 *   changes during a call to DsubDecoderGetFirstUpdateRect().
 *
 *   If the passed display size is equal to the screen size which was passed
 *   to DsubDecoderSetDisplayDriver(), the subtitles can be copied directly
 *   to the screen. If the display size is different to the screen size, all
 *   rectangles returned by the DsubDecoder (e.g via
 *   DsubDecoderGetFirst/NextUpdateRect), or passed to the DisplayDriver,
 *   shall be scaled using the ratio (screenSize/displaySize).
 *
 * PARAMETERS:
 *   aUserData      - The user data registered at the function
 *                    DsubDecoderSetDisplayDriver is passed to this callback.
 *   aDisplayBounds - The new display bounds.
 *
 * SEE ALSO:
 *   DsubTDisplayDriverDraw
 ****************************************************************************/

typedef 
void ( * DsubTDisplayDriverSetDisplayBounds ) (
  void *     aUserData,
  DsubTRect  aDisplayBounds );

#endif /* DSUBTDISPLAYDRIVER_H */
