/****************************************************************************
 * MODULE      : TT_Acq
 * FILE        : TT_Acq.h
 * COPYRIGHT   : TARA Systems GmbH
 * DESCRIPTION :
 *   Teletext acquisition
 *
 *   This module has three purposes:
 *     1. Processing and storage of packets into the database
 *     2. Navigation system (TOP, FLOF, page trace, subpage)
 *     3. Page request system (dynamic page database)
 *
 *   The teletext acquisition processes all incoming header and data packets.
 *   The teletext protocols (serial, parallel) are analysed and the packets
 *   are stored into the database.
 *
 *   The navigation system offers access to the subpage list, TOP/FLOF links,
 *   page trace links and subtitle list.
 *
 *   The page request system is used, if the database is not big enough to
 *   store all available pages. In this case for each incoming page a
 *   priority is calculated and the page with the lowest priority is removed
 *   from the database to store a page with higher priority.
 ****************************************************************************/

#ifndef TT_ACQ_H
#define TT_ACQ_H


/****************************************************************************
 * SECTION: #define
 ****************************************************************************/

/* navigation modes for ACQ */
#define TT_ACQ_TOP_FLOF_DISA            0x00u
#define TT_ACQ_TOP_ENA                  0x01u
#define TT_ACQ_FLOF_ENA                 0x02u
#define TT_ACQ_TOP_FLOF_ENA             0x03u
#define TT_ACQ_FLOF_TOP_ENA             0x07u


#define TT_ACQ_NO_OF_MAGAZINES          0x08
#define TT_ACQ_ALL_MAGAZINES            0xFFu
#define TT_ACQ_HEADER_PACKET            0x00u

#define TT_ACQ_MAGAZINE_CLOSED             0
#define TT_ACQ_MAGAZINE_OPEN               1

#define TT_ACQ_SUBPAGE_OVERWRITE_MODE 0xFFFFu
#define TT_ACQ_NOF_PAGES               0x800u
#define TT_ACQ_NOF_PAGES_DEZ             800u
#define TT_ACQ_NOF_PAGES_HEX     (TT_ACQ_NOF_PAGES-TT_ACQ_NOF_PAGES_DEZ)

/*  
The two different trace types supported by the ACQ.
TT_ACQ_PAGETRACE contains all pages found in the transmissin cycle.
TT_ACQ_MEMTRACE contains all pages stored in the database.
*/
#define TT_ACQ_PAGETRACE                0x00u
#define TT_ACQ_MEMTRACE                 0x01u

/* number of subpages handled in subpage trace */
#define TT_ACQ_NOF_SUBPAGES             0x100


/* return values of TT_AcqExecuteProcessing() */
#define TT_ACQ_PACKET_BUFFER_EMPTY      0x00u
#define TT_ACQ_END_OF_DISPLAY_PAGE      0x01u
#define TT_ACQ_END_OF_DISPLAY_SUBPAGE   0x02u
#define TT_ACQ_END_OF_PAGE              0x03u
#define TT_ACQ_HEADER_REPETITION        0x04u
#define TT_ACQ_DATA_PACKET_PROCESSED    0x05u
#define TT_ACQ_INVALID_PACKET           0x06u
#define TT_ACQ_SUBTITLE_DATA_FOUND      0x07u

/* return values of TT_AcqGetNavigationMode() */
#define TT_ACQ_DEFAULT_MODE                0u
#define TT_ACQ_TOP_MODE                    1u
#define TT_ACQ_FLOF_LINK                   2u
#define TT_ACQ_FLOF_TEXT                   4u

/* return values of TT_AcqGetPageState() */
#define TT_ACQ_PAGE_BEING_RECEIVED         0x00u
#define TT_ACQ_PAGE_FOUND                  0x01u
#define TT_ACQ_PAGE_IN_BTT                 0x02u
#define TT_ACQ_PAGE_IN_CYCLE               0x04u


/* return values of TT_AcqGetSubtitlePage() */
#define TT_ACQ_SUBTITLE_DISPLAY_OFF        0u
#define TT_ACQ_SUBTITLE_DISPLAY_ON         1u
#define TT_ACQ_SUBTITLE_DISPLAY_KEEP       2u
#define TT_ACQ_SUBTITLE_NOT_AVAILABLE      3u

#define TEXT_MAX_NO_OF_SUBTITLE_PAGES     20u

#define TT_ACQ_MAX_USER_PRIORITY_PAGES    12
#define TT_ACQ_NO_PRIORITY_PAGE       0x5A5Au

/* return values of AciCheckAccelerator */
#define TT_ACQ_ACI_CHECK_ERROR             0u
#define TT_ACQ_ACI_ACCELERATOR_CABLE       1u
#define TT_ACQ_ACI_ACCELERATOR_SATELLITE   2u
#define TT_ACQ_ACI_ACCELERATOR_TERRESTRIAL 3u

#define TT_ACQ_PLUS_DIRECTION           0x00u
#define TT_ACQ_MINUS_DIRECTION          0x01u

#define TT_ACQ_IGNORE_HEX_PAGES         0x80u

#define IsHexPage(page) ((( (page) & 0x0F0 ) > 0x090 ) || (( (page) & 0x00F ) > 0x009 ))
#define IsDezPage(page) ((( (page) & 0x00F ) < 0x00A ) && (( (page) & 0x0F0 ) < 0x0A0 ))

#define TT_ACQ_C4_ERASE_PAGE            0x01u
#define TT_ACQ_C5_NEWSFLASH             0x02u
#define TT_ACQ_C6_SUBTITLE              0x04u
#define TT_ACQ_C7_SUPRESS_HEADER        0x08u
#define TT_ACQ_C8_UPDATE_INDICATOR      0x10u
#define TT_ACQ_C9_INTERRUPTED_SEQUENCE  0x20u
#define TT_ACQ_C10_INHIBIT_DISPLAY      0x40u
#define TT_ACQ_C11_MAGAZINE_SERIAL      0x80u

#define TT_AcqGetC4ErasePage(x)           (((x) & TT_ACQ_C4_ERASE_PAGE ))
#define TT_AcqGetC5Newsflash(x)           (((x) & TT_ACQ_C5_NEWSFLASH ))
#define TT_AcqGetC6Subtitle(x)            (((x) & TT_ACQ_C6_SUBTITLE ))
#define TT_AcqGetC7SupressHeader(x)       (((x) & TT_ACQ_C7_SUPRESS_HEADER ))
#define TT_AcqGetC8UpdateIndicator(x)     (((x) & TT_ACQ_C8_UPDATE_INDICATOR ))
#define TT_AcqGetC9InterruptedSequence(x) (((x) & TT_ACQ_C9_INTERRUPTED_SEQUENCE ))
#define TT_AcqGetC10Inhibitdisplay(x)     (((x) & TT_ACQ_C10_INHIBIT_DISPLAY ))
#define TT_AcqGetC11MagazineSerial(x)     (((x) & TT_ACQ_C11_MAGAZINE_SERIAL ))


/****************************************************************************
 * SECTION: typedef
 ****************************************************************************/

typedef enum
{
  TT_ACQ_NEXT_PAGE,
  TT_ACQ_PREV_PAGE,
  TT_ACQ_INDEX_PAGE_P830,
  TT_ACQ_INDEX_PAGE_FLOF,
  TT_ACQ_RED_KEY,
  TT_ACQ_GREEN_KEY,
  TT_ACQ_YELLOW_KEY,
  TT_ACQ_CYAN_KEY,
  TT_ACQ_WHITE_KEY,
  TT_ACQ_FIRST_SUBPAGE,
  TT_ACQ_NEXT_SUBPAGE,
  TT_ACQ_PREV_SUBPAGE,
  TT_ACQ_CURRENT_PAGE,
  TT_ACQ_ACTUAL_SUBPAGE,
  TT_ACQ_LAST_PAGE,
  TT_ACQ_PRIOR_PAGE,
  TT_ACQ_HIGHEST_SUBPAGE,
  TT_ACQ_LAST_RECEIVED_SUBPAGE

} TT_ACQ_PAGE_CODES;

/***********************************************************************
 * TYPE: TT_ACQ_PAGE
 *
 * DESCRIPTION:
 *   The TT_ACQ_PAGE defines a Teletext page received by the acquisition.
 ***********************************************************************/
typedef struct
{
  U_INT16    MagPage;
  U_INT16    SubPage; 

} TT_ACQ_PAGE; 

/****************************************************************************
 * TYPE: TT_PacketCallback
 *
 * DESCRIPTION:
 *   Callback function for external processing of packets. The packet data is
 *   delivered in the same format as put into the packet buffer. In general
 *   44 bytes with first two bytes framing code.
 ****************************************************************************/
typedef void
(*TT_PacketCallback) (
  U_INT8 *    aPacket );

#ifdef __cplusplus
  extern "C" {
#endif



/****************************************************************************
 * FUNCTION: TT_AcqSetPacketCallback
 *
 * DESCRIPTION:
 *   Sets a callback function to the ACQ to pass all packets to external
 *   before processing.
 *
 * PARAMETERS:
 *   aPacketCallback - The packet callback function
 *
 * SEE ALSO:
 *   TT_AcqUnsetPacketCallback
 ****************************************************************************/

PUBLIC void
TT_AcqSetPacketCallback(
  TT_PacketCallback  aPacketCallback );


/****************************************************************************
 * FUNCTION: TT_AcqUnsetPacketCallback
 *
 * DESCRIPTION:
 *   Unset the packet callback function.
 *
 * SEE ALSO:
 *   TT_AcqSetPacketCallback
 ****************************************************************************/

PUBLIC void
TT_AcqUnsetPacketCallback( void );


/****************************************************************************
 * FUNCTION: TT_AcqSetIdlCallback
 *
 * DESCRIPTION:
 *   Sets a callback function to the ACQ to handle IDL packets external.
 *
 * PARAMETERS:
 *   aIdlCallback - The packet callback function
 *
 * SEE ALSO:
 *   TT_AcqUnsetIdlCallback
 ****************************************************************************/

PUBLIC void
TT_AcqSetIdlCallback(
  TT_PacketCallback  aIdlCallback );


/****************************************************************************
 * FUNCTION: TT_AcqUnsetIdlCallback
 *
 * DESCRIPTION:
 *   Unset the callback function to the ACQ to handle IDL packets external.
 *
 * SEE ALSO:
 *   TT_AcqSetIdlCallback
 ****************************************************************************/

PUBLIC void
TT_AcqUnsetIdlCallback( void );


/****************************************************************************
 * FUNCTION: TT_AcqSetHeaderCallback
 *
 * DESCRIPTION:
 *   Sets a callback function to the ACQ to handle Header packets external.
 *
 * PARAMETERS:
 *   aHeaderCallback - The packet callback function
 *
 * SEE ALSO:
 *   TT_AcqUnsetHeaderCallback
 ****************************************************************************/

PUBLIC void
TT_AcqSetHeaderCallback(
  TT_PacketCallback  aHeaderCallback );


/****************************************************************************
 * FUNCTION: TT_AcqUnsetHeaderCallback
 *
 * DESCRIPTION:
 *   Unset the callback function to ACQ to handle Header packets external.
 *
 * SEE ALSO:
 *   TT_AcqSetHeaderCallback
 ****************************************************************************/

PUBLIC void
TT_AcqUnsetHeaderCallback( void );


/****************************************************************************
 * FUNCTION: TT_AcqSetP830t1Callback
 *
 * DESCRIPTION:
 *   Sets a callback function to ACQ to handle Packet 830 Type 1 packets
 *   external.
 *
 * PARAMETERS:
 *   aP830t1Callback - The packet callback function
 *
 * SEE ALSO:
 *   TT_AcqUnsetP830t1Callback
 ****************************************************************************/

PUBLIC void
TT_AcqSetP830t1Callback(
  TT_PacketCallback  aP830t1Callback );


/****************************************************************************
 * FUNCTION: TT_AcqUnsetP830t1Callback
 *
 * DESCRIPTION:
 *   Unset the callback function to ACQ to handle Packet 830 Type 1 packets
 *   external.
 *
 * SEE ALSO:
 *   TT_AcqSetP830t1Callback
 ****************************************************************************/

PUBLIC void
TT_AcqUnsetP830t1Callback( void );


/****************************************************************************
 * FUNCTION: TT_AcqSetP830t2Callback
 *
 * DESCRIPTION:
 *   Sets a callback function to ACQ to handle Packet 830 Type 2 packets
 *   external.
 *
 * PARAMETERS:
 *   aP830t2Callback - The packet callback function
 *
 * SEE ALSO:
 *   TT_AcqUnsetP830t2Callback
 ****************************************************************************/

PUBLIC void
TT_AcqSetP830t2Callback(
  TT_PacketCallback  aP830t2Callback );


/****************************************************************************
 * FUNCTION: TT_AcqUnsetP830t2Callback
 *
 * DESCRIPTION:
 *   Unset the callback function to ACQ to handle Packet 830 Type 2 packets
 *   external.
 *
 * SEE ALSO:
 *   TT_AcqSetP830t2Callback
 ****************************************************************************/

PUBLIC void
TT_AcqUnsetP830t2Callback( void );


/****************************************************************************
 * FUNCTION: TT_AcqSetTopFlofEnable
 *
 * DESCRIPTION:
 *   Enable TOP and / or FLOF navigation
 *
 * PARAMETERS:
 *   aTopFlofEnable - enable / disable TOP navigation
 ****************************************************************************/

PUBLIC void
TT_AcqSetTopFlofEnable(
  U_INT8   aTopFlofEnable
);


/****************************************************************************
 * FUNCTION: TT_AcqGetTopFlofEnable
 *
 * DESCRIPTION:
 *   Returns the TOP and / or FLOF navigation mode
 *
 * PARAMETERS:
 *   aTopFlofEnable - enable / disable TOP navigation
 *
 * RETURN:
 *   TRUE/FALSE
 ****************************************************************************/

PUBLIC U_INT8
TT_AcqGetTopFlofEnable( void );


/****************************************************************************
 * FUNCTION: TT_AcqReset
 *
 * DESCRIPTION:
 *   Resets the teletext aquisition, TOP, packetbuffer and database.
 ****************************************************************************/

PUBLIC void
TT_AcqReset( void );


/****************************************************************************
 * FUNCTION: TT_AcqExecuteProcessing
 *
 * DESCRIPTION:
 *   Fetches all packets from packet buffer and processes them. Processing is
 *   stopped if:
 *     1. packet buffer is emtpy
 *     2. any end of page condition occured
 *
 *   (remark: end of page condition means magazine is closed)
 *
 * RETURN:
 *   end condition:
 *     - TT_ACQ_DATA_PACKET_PROCESSED
 *     - TT_ACQ_END_OF_PAGE (=> header display must be updated)
 *     - TT_ACQ_END_OF_DISPLAY_PAGE (=> display page must be updated)
 *     - TT_ACQ_END_OF_DISPLAY_SUBPAGE (=> subpage bar must be updated)
 *     - TT_ACQ_INVALID_PACKET
 *     - TT_ACQ_PACKET_BUFFER_EMPTY
 ****************************************************************************/

PUBLIC U_INT8
TT_AcqExecuteProcessing( void );


/****************************************************************************
 * FUNCTION: TT_AcqGetPageState
 *
 * DESCRIPTION:
 *   Returns the page state of the current display page. Supported states:
 *     - TT_ACQ_PAGE_BEING_RECEIVED
 *     - TT_ACQ_PAGE_FOUND
 *     - TT_ACQ_PAGE_IN_BTT
 *     - TT_ACQ_PAGE_IN_CYCLE
 *
 * RETURN:
 *   Page state
 ****************************************************************************/

PUBLIC U_INT8
TT_AcqGetPageState( void );


/****************************************************************************
 * FUNCTION: TT_AcqGetPageRequestedState
 *
 * DESCRIPTION:
 *   Returns true if the requested page has been found or not.
 *
 * RETURN:
 *   TRUE/FALSE
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AcqGetPageRequestedState( void );


/****************************************************************************
 * FUNCTION: TT_AcqSetPage
 *
 * DESCRIPTION:
 *   Sets a specified teletext page to the current display page. This
 *   function must be called every time before the display page changes.
 *
 * PARAMETERS:
 *   aPage - the page and subpage number
 ****************************************************************************/

PUBLIC void
TT_AcqSetPage(
  TT_ACQ_PAGE aPage );


/****************************************************************************
 * FUNCTION: TT_AcqGetPage
 *
 * DESCRIPTION:
 *   Returns the specified page or subpage number. This function is needed
 *   for implementing the teletext navigation.
 *
 * PARAMETERS:
 *   aPageType - type of the teletext page:
 *     -          TT_ACQ_NEXT_PAGE,
 *     -          TT_ACQ_PREV_PAGE,
 *     -          TT_ACQ_INDEX_PAGE_P830,
 *     -          TT_ACQ_INDEX_PAGE_FLOF,
 *     -          TT_ACQ_RED_KEY,
 *     -          TT_ACQ_GREEN_KEY,
 *     -          TT_ACQ_YELLOW_KEY,
 *     -          TT_ACQ_CYAN_KEY,
 *     -          TT_ACQ_WHITE_KEY,
 *     -          TT_ACQ_CURRENT_PAGE,
 *     -          TT_ACQ_FIRST_SUBPAGE,
 *     -          TT_ACQ_NEXT_SUBPAGE,
 *     -          TT_ACQ_PREV_SUBPAGE,
 *     -          TT_ACQ_ACTUAL_SUBPAGE
 *     -          TT_ACQ_LAST_PAGE
 *     -          TT_ACQ_PRIOR_PAGE
 *     -          TT_ACQ_HIGHEST_SUBPAGE
 *     -          TT_ACQ_LAST_RECEIVED_SUBPAGE
 *   aPage     - the page number
 *
 * RETURN:
 *   page found / page not found ( TRUE / FALSE )
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AcqGetPage(
  U_INT8      aPageType,
  TT_ACQ_PAGE *aPage );


/****************************************************************************
 * FUNCTION: TT_AcqGetPageInfo
 *
 * DESCRIPTION:
 *   Returns page information of the current displayed page. The bits can be
 *   accessed using the macros TT_AcqGetCx...() defined in the header file.
 *
 * PARAMETERS:
 *   aPageInfo - C bits 11,10,9,8,7,6,5,4 .of the header row
 ****************************************************************************/

PUBLIC void
TT_AcqGetPageInfo(
  U_INT8* aPageInfo );


/****************************************************************************
 * FUNCTION: TT_AcqGetPageSubNewsInfo
 *
 * DESCRIPTION:
 *   Returns page information for the specified page number.
 *
 * PARAMETERS:
 *   aSubnewsPage - Input page number.
 *
 * RETURN:
 *   TRUE if aSubnewsPage is a subtitle/newsflash (C5/C6 bit) page.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AcqGetPageSubNewsInfo (
   TT_ACQ_PAGE* aSubnewsPage );


/****************************************************************************
 * FUNCTION: TT_AcqIsLastPageSubNews
 *
 * DESCRIPTION:
 *   Returns TRUE if the last displayed page was a subtitle/newsflash page.
 *
 * RETURN:
 *   TRUE if the last displayed page was a subtitle/newsflash (C5/C6 bit)
 *   page.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AcqIsLastPageSubNews( void );


/****************************************************************************
 * FUNCTION: TT_AcqGetNavigationMode
 *
 * DESCRIPTION:
 *   Returns the navigation mode of a specified page (TOP/FLOF).
 *
 * PARAMETERS:
 *   aPage - the page and subpage number
 *
 * RETURN:
 *   Navigation mode:
 *     - TT_ACQ_TOP_MODE
 *     - TT_ACQ_FLOF_LINK
 *     - TT_ACQ_FLOF_TEXT
 ****************************************************************************/

PUBLIC U_INT8
TT_AcqGetNavigationMode(
  TT_ACQ_PAGE aPage );


/****************************************************************************
 * FUNCTION: TT_AcqGetSubpageInfo
 *
 * DESCRIPTION:
 *   Returns the pointer to the subpage info array.
 *
 * PARAMETERS:
 *   aSubpageCount    - number of subpages in range of 0..99
 *   aHighestSubpage  - highest received subpage number
 *   aHexSubpages     - number of hex subpages in range of 0..99
 *   aSpecialSubpages - subpages with subcode above 0x0099, a number of 2
 *                      indicates more than 2 special subpages available
 ****************************************************************************/

PUBLIC void
TT_AcqGetSubpageInfo(
  U_INT16*      aSubpageCount,
  U_INT16*      aHighestSubpage,
  U_INT16*      aHexSubpages,
  U_INT16*      aSpecialSubpages );


/****************************************************************************
 * FUNCTION: TT_AcqIsSubpageInTrace
 *
 * DESCRIPTION:
 *   Search the given subpage in the subpage trace.
 *
 * PARAMETERS:
 *   aTraceType - type of trace (TT_ACQ_MEMTRACE/TT_ACQ_PAGETRACE)
 *   aSubPage   - subpage to check
 *
 * RETURN:
 *   TRUE if page is in trace, FALSE otherwise.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AcqIsSubpageInTrace(
  U_INT8        aTraceType,
  U_INT16       aSubPage );


/****************************************************************************
 * FUNCTION: TT_AcqGetTopText
 *
 * DESCRIPTION:
 *   Returns the TOP text of the specified page.
 *
 * PARAMETERS:
 *   aMagPage -
 *     page
 *
 *   aBlockText -
 *     block text
 *
 *   aBlockNatOpt -
 *     block text language
 *
 *   aGroupText -
 *     group text
 *
 *   aGroupNatOpt -
 *     group text language
 *
 *   aPreviousBlockText -
 *     the pointer to the previous block string
 *
 *   aPreviousBlockNatOpt -
 *     the pointer to the previous national options / block
 ****************************************************************************/

PUBLIC void
TT_AcqGetTopText(
  U_INT16       aMagPage,
  U_INT8*       aBlockText,
  U_INT8*       aBlockNatOpt,
  U_INT8*       aGroupText,
  U_INT8*       aGroupNatOpt,
  U_INT8*       aPreviousBlockText,
  U_INT8*       aPreviousBlockNatOpt);


/****************************************************************************
 * FUNCTION: TT_AcqGetMagazineAndPacketId
 *
 * DESCRIPTION:
 *   Returns the id and magazine number of a teletext packet. Hamming check
 *   is done here.
 *
 * PARAMETERS:
 *   aPacketPointer - pointer to the header packet
 *   aMagazine      - the magazine number of the given packet
 *   aPacketId      - the packet ID of the given packet
 *
 * RETURN:
 *   no hamming errors / hamming errors
 *     - TRUE
 *     - FALSE
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AcqGetMagazineAndPacketId(
  U_INT8*  aPacketPointer,
  U_INT8*  aMagazine,
  U_INT16* aPacketId );


/****************************************************************************
 * FUNCTION: TT_AcqGetSubtitlePage
 *
 * DESCRIPTION:
 *   The function TT_AcqGetSubtitlePage returns the next available subtitle
 *   page.
 *
 * PARAMETERS:
 *   aMagPage - subtitle page number
 *   aSubPage - subtitle subpage number
 *
 * RETURN:
 *   New display mode:
 *     - TT_ACQ_SUBTITLE_DISPLAY_OFF
 *     - TT_ACQ_SUBTITLE_DISPLAY_ON
 *     - TT_ACQ_SUBTITLE_DISPLAY_KEEP
 *     - TT_ACQ_SUBTITLE_NOT_AVAILABLE
 ****************************************************************************/

PUBLIC U_INT8
TT_AcqGetSubtitlePage(
  U_INT16*   aMagPage,
  U_INT16*   aSubPage );


/****************************************************************************
 * FUNCTION: TT_AcqResetSubtitleSearchMode
 *
 * DESCRIPTION:
 *   The function TT_AcqResetSubtitleSearchMode resets the subtitle search
 *   mode.
 ****************************************************************************/

PUBLIC void
TT_AcqResetSubtitleSearchMode( void );


/****************************************************************************
 * FUNCTION: TT_AcqGetSubtitlesAvailable
 *
 * DESCRIPTION:
 *   The function TT_AcqGetSubtitlesAvailable returns true if subtitle pages
 *   are available.
 *
 * RETURN:
 *   TRUE if subtitle page found, FALSE otherwise.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AcqGetSubtitlesAvailable( void );


/****************************************************************************
 * FUNCTION: TT_AcqGetAciStatus
 *
 * DESCRIPTION:
 *   The function TT_AcqGetAciStatus returns true if a ACI accelerator
 *   character has been found inside the header.
 *
 * RETURN:
 *   TRUE a ACI accelerator character has been found. FALSE otherwise.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AcqGetAciStatus( void );


/****************************************************************************
 * FUNCTION: TT_AcqGetFirstSubpage
 *
 * DESCRIPTION:
 *   The function TT_AcqGetFirstSubpage returns the first received subpage
 *   number
 *
 * RETURN:
 *   First subpage number.
 ****************************************************************************/

PUBLIC unsigned short
TT_AcqGetFirstSubpage( void );


/****************************************************************************
 * FUNCTION: TT_AcqGetHighestSubpage
 *
 * DESCRIPTION:
 *   The function TT_AcqGetHighestSubpage returns the highest received
 *   subpage number
 *
 * RETURN:
 *   Highest subpage number.
 ****************************************************************************/

PUBLIC unsigned short
TT_AcqGetHighestSubpage( void );


/****************************************************************************
 * FUNCTION: TT_AcqSetPriorityPage
 *
 * DESCRIPTION:
 *   The function TT_AcqSetPriorityPage defines one of the user definable
 *   priority pages for the page request system. The index is in range of 0
 *   to TT_ACQ_MAX_USER_PRIORITY_PAGES. Use TT_ACQ_NO_PRIORITY_PAGE as
 *   MagPage for removing an entry.
 *
 * PARAMETERS:
 *   aIndex   - priority index
 *   aMagPage - page number
 *
 * RETURN:
 *   success
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AcqSetPriorityPage(
  U_INT8    aIndex,
  U_INT16   aMagPage );


/****************************************************************************
 * FUNCTION: TT_AcqEnableHexPageCheck
 *
 * DESCRIPTION:
 *   The function TT_AcqEnableHexPageCheck enables the default hex page check
 *   (hamming). This means all hex pages are parity checked, as long a no MOT
 *   is found.
 ****************************************************************************/

PUBLIC void
TT_AcqEnableHexPageCheck( void );


/****************************************************************************
 * FUNCTION: TT_AcqCloseAllMagazines
 *
 * DESCRIPTION:
 *   The function TT_AcqCloseAllMagazines closes all open magazines. This
 *   function must be called e.g. in case of a packet buffer overflow.
 ****************************************************************************/

PUBLIC void
TT_AcqCloseAllMagazines( void );


/****************************************************************************
 * FUNCTION: TT_AcqRequestPage
 *
 * DESCRIPTION:
 *   The function TT_AcqRequestPage requests the specified page. If the page
 *   has been found the content will be copied to the page request buffer.
 *
 * PARAMETERS:
 *   aPage    - page number to request
 *   aSubpage - subpage number to request
 ****************************************************************************/

PUBLIC void
TT_AcqRequestPage(
  U_INT16 aPage,
  U_INT16 aSubpage);


/****************************************************************************
 * FUNCTION: TT_AcqSetPageRequestBuffer
 *
 * DESCRIPTION:
 *   The function TT_AcqSetPageRequestBuffer sets the internal address and
 *   size of the page request buffer (where requested pages are stored).
 *   Packets are copied as long as aPageRequestBufferSize is big enough. The
 *   page buffer format is 42 bytes for each packet:
 *     - 2 bytes packet address (already hamming encrypted)
 *     - 40 bytes data (with paritiy information
 *
 * PARAMETERS:
 *   aPageRequestBuffer -
 *     pointer to the page request buffer
 *
 *   aPageRequestBufferSize -
 *     size of the page request buffer
 ****************************************************************************/

PUBLIC void
TT_AcqSetPageRequestBuffer(
  U_INT8*  aPageRequestBuffer,
  U_INT16  aPageRequestBufferSize );


/****************************************************************************
 * FUNCTION: TT_AcqIgnoreIncomingPage
 *
 * DESCRIPTION:
 *   The function TT_AcqIgnoreIncomingPage protects the specified page
 *   number. Protect means that this page will never be overwritten or
 *   deleted from the ACQ. This function must be called until it returns
 *   TRUE, otherwise the page will not be protected.
 *
 * PARAMETERS:
 *   aIgnorePage - the page number to protect (subpage number is not used)
 *
 * RETURN:
 *   TRUE   if page is protected now,
 *   FALSE  if page is currently received
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AcqIgnoreIncomingPage(
  TT_ACQ_PAGE aIgnorePage );


/****************************************************************************
 * FUNCTION: TT_AcqGetStatusDisplay
 *
 * DESCRIPTION:
 *   The function TT_AcqGetStatusDisplay delivers the latest received status
 *   display data from packet 8/30 format 1 as string.
 *
 * PARAMETERS:
 *   aStatusDisplay - the return pointer for the status string
 *
 * RETURN:
 *   TRUE   if packet 8/30/1 data is available,
 *   FALSE  otherwise.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AcqGetStatusDisplay(
  U_INT8** aStatusDisplay );


/****************************************************************************
 * FUNCTION: TT_AcqGetProgramTitle
 *
 * DESCRIPTION:
 *   The function TT_AcqGetProgramTitle delivers the latest received program
 *   title data from packet 8/30 format 2 as string.
 *
 * PARAMETERS:
 *   aProgramTitle - the return pointer for the program title string
 *
 * RETURN:
 *   TRUE   if packet 8/30/2 data is available,
 *   FALSE  otherwise.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AcqGetProgramTitle(
  U_INT8** aProgramTitle );


/****************************************************************************
 * FUNCTION: TT_AcqGetFirstSubpageNumber
 *
 * DESCRIPTION:
 *   Returns the first subpage of the subpage list. First subpage defines the
 *   subpage with lowest subpage number.
 *
 * PARAMETERS:
 *   aSubpageList   - type of subpage list (TT_ACQ_MEMTRACE/
 *                    TT_ACQ_PAGETRACE)
 *   aSubpageNumber - the return subpage number
 *
 * RETURN:
 *   subpage found / subpage not found
 *     - TRUE
 *     - FALSE
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AcqGetFirstSubpageNumber(
  U_INT8        aSubpageList,
  U_INT16       *aSubpageNumber );


/****************************************************************************
 * FUNCTION: TT_AcqGetNextSubpageNumber
 *
 * DESCRIPTION:
 *   Select the next subpage from selected subpage list starting with subpage
 *   number given by aSubpageNumber. The value is returned within the
 *   parameter aSubpageNumber.
 *
 * PARAMETERS:
 *   aSubpageList   - type of subpage list (TT_ACQ_MEMTRACE/
 *                    TT_ACQ_PAGETRACE)
 *   aSubpageNumber - the start and return subpage number
 *
 * RETURN:
 *   subpage found / subpage not found
 *     - TRUE
 *     - FALSE
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AcqGetNextSubpageNumber(
  U_INT8        aSubpageList,
  U_INT16       *aSubpageNumber );


/****************************************************************************
 * FUNCTION: TT_AcqGetPreviousSubpageNumber
 *
 * DESCRIPTION:
 *   Select the previous subpage from selected subpage list starting with
 *   subpage number given by aSubpageNumber. The value is returned within the
 *   parameter aSubpageNumber.
 *
 * PARAMETERS:
 *   aSubpageList   - type of subpage list (TT_ACQ_MEMTRACE/
 *                    TT_ACQ_PAGETRACE)
 *   aSubpageNumber - the start and return subpage number
 *
 * RETURN:
 *   subpage found / subpage not found
 *     - TRUE
 *     - FALSE
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AcqGetPreviousSubpageNumber(
  U_INT8        aSubpageList,
  U_INT16       *aSubpageNumber );


/****************************************************************************
 * FUNCTION: TT_AcqIsPageInTrace
 *
 * DESCRIPTION:
 *   Search the given page in the page trace.
 *
 * PARAMETERS:
 *   aTraceType - type of trace (TT_ACQ_MEMTRACE/TT_ACQ_PAGETRACE)
 *   aMagPage   - page to check
 *
 * RETURN:
 *   TRUE if page is in trace, FALSE otherwise.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AcqIsPageInTrace(
  U_INT8        aTraceType,
  U_INT16       aMagPage );


/****************************************************************************
 * FUNCTION: TT_AcqCountPagesInTrace
 *
 * DESCRIPTION:
 *   Counts all pages within the page trace
 *
 * PARAMETERS:
 *   aTraceType - type of trace (TT_ACQ_MEMTRACE/TT_ACQ_PAGETRACE)
 *
 * RETURN:
 *   Number of pages in page trace.
 ****************************************************************************/

PUBLIC U_INT16
TT_AcqCountPagesInTrace(
  U_INT8        aTraceType );


/****************************************************************************
 * FUNCTION: TT_AcqSearchPageNumberInTrace
 *
 * DESCRIPTION:
 *   Searches the next/previous page within the page trace.
 *
 * PARAMETERS:
 *   aTraceType  - type of trace (TT_ACQ_MEMTRACE/TT_ACQ_PAGETRACE)
 *   aSearchType - search type (NEXT_PAGE or PREVIOUS_PAGE and flag
 *                 IGNORE_HEX_PAGES)
 *   aMagPage    - current page
 *   aDistance   - number of pages between
 *
 * RETURN:
 *   Found page number.
 *
 * EXAMPLE:
 *   pageNumber = TT_AcqSearchPageNumberInTrace( TT_ACQ_PAGETRACE,
 *     TT_ACQ_NEXT_PAGE | TT_ACQ_IGNORE_HEX_PAGES, 0x100, 1 );
 ****************************************************************************/

PUBLIC U_INT16
TT_AcqSearchPageNumberInTrace(
  U_INT8  aTraceType,
  U_INT8  aSearchType,
  U_INT16 aMagPage,
  U_INT16 aDistance );


/****************************************************************************
 * FUNCTION: TT_AcqGetNoOfProcessedPackets
 *
 * DESCRIPTION:
 *   Return the number of processed packets since last call of this function.
 *
 * RETURN:
 *   The number of processed packets since last call
 ****************************************************************************/

PUBLIC U_INT32
TT_AcqGetNoOfProcessedPackets( void );


/****************************************************************************
 * FUNCTION: TT_AcqSetProcessDisplayPageOnlyMode
 *
 * DESCRIPTION:
 *   Activates or deactivates the handle display page only mode. In this mode
 *   only the current display page is processed by the ACQ. All other pages
 *   are ignored. This mode may be used to increase the performance on slow
 *   systems for subtitle pages.
 *
 * PARAMETERS:
 *   aEnable - enabled or disable the handle display page only mode
 ****************************************************************************/

PUBLIC void
TT_AcqSetProcessDisplayPageOnlyMode(
  TT_BOOL aEnable );



#ifdef __cplusplus
  }
#endif

#endif /* TT_ACQ_H */

/* This Headerfile was generated with C2H.exe ( C2H ) */
/* (c) 2006 by TARA Systems GmbH Munich */
