/****************************************************************************
 * MODULE     : TT_Aci
 * FILE       : TT_Aci.h
 * COPYRIGHT  : TARA Systems GmbH, Mario Stefanutti, March 2005
 * DESCRIPTION:
 *   Controls the commands and status for a independent ACI (auto channel
 *   installation) application.
 ****************************************************************************/

#ifndef TT_ACI_H
#define TT_ACI_H


/****************************************************************************
 * SECTION: #include
 ****************************************************************************/

/****************************************************************************
 * SECTION: #define
 ****************************************************************************/

/* possible commands for AciProcess */
#define TT_ACI_START                0xE0
#define TT_ACI_STOP                 0xE1
#define TT_ACI_UPDATE               0xE2
#define TT_ACI_SET_LANGUAGE         0xE3
#define TT_ACI_GET_MENU_STRING      0xE4
#define TT_ACI_GET_RESULT           0xE5
#define TT_ACI_OK                   0xF0
#define TT_ACI_UP                   0xF1
#define TT_ACI_DOWN                 0xF2


/* possible status for AciProcess */
#define TT_ACI_STATUS_OFF           0x00
#define TT_ACI_STATUS_SEARCH_HEADER 0x01
#define TT_ACI_STATUS_CONFIRM_ACI   0x02     
#define TT_ACI_STATUS_SEARCH_PAGE   0x03
#define TT_ACI_STATUS_ERROR_OCURED  0x04
#define TT_ACI_STATUS_DECODE_MULTI  0x05
#define TT_ACI_STATUS_DECODE_SINGLE 0x06
#define TT_ACI_STATUS_GET_RESULT    0x07

#define TT_ACI_HEADER_SEARCH_CYCLES    5
#define TT_ACI_MENU_STRING_LENGTH     16

#define TT_ACI_STATIC_BUFFER

/****************************************************************************
 * SECTION: typedef
 ****************************************************************************/


/****************************************************************************
 * TYPE: TT_ACI
 *
 * DESCRIPTION:
 *   The structure TT_ACI defines the object for the ACI application.
 *
 * COMPONENTS:
 *   Visible       - GUI is visible;
 *   Cursor        - Current cursor position;
 *   NoOfItems     - number of menu items in a area selection menu;
 *   AreaName      - buffer for string handling of ACI area menu. Here it is
 *                   used for a test implemetation. It is only required for a
 *                   ACI menu without Teletext SPC display.
 *   SearchCounter - counts the header detection cycles;
 *   Status        - status of the internal state machine;
 *   Language      - language for decoding and sorting;
 *   ResultBuffer  - buffer for the result items (only if
 *                   TT_ACI_STATIC_BUFFER is defined)
 ****************************************************************************/
typedef struct
{
  U_INT8               Visible;
  U_INT8               Cursor;
  U_INT8               NoOfItems;
  INT8                 AreaName[TT_ACI_MENU_STRING_LENGTH];
  U_INT16              SearchCounter;
  U_INT8               Status;
  U_INT8               Language;
#ifdef TT_ACI_STATIC_BUFFER
  TT_ACI_RESULT        ResultBuffer;
#endif /* TT_ACI_STATIC_BUFFER */
} TT_ACI;


/****************************************************************************
 * TYPE: TT_ACI_HANDLE
 ****************************************************************************/
typedef TT_ACI * TT_ACI_HANDLE;




#ifdef __cplusplus
  extern "C" {
#endif



#if (TT_ACI_FEATURE == ENABLED)
/****************************************************************************
 * FUNCTION: TT_AciCreate
 *
 * DESCRIPTION:
 *   Creates the ACI application.
 *
 * RETURN:
 *   success
 *
 * SEE ALSO:
 *   TT_AciDestroy
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AciCreate(void);


/****************************************************************************
 * FUNCTION: TT_AciDestroy
 *
 * DESCRIPTION:
 *   Delete the ACI application
 *
 * SEE ALSO:
 *   TT_AciCreate
 ****************************************************************************/

PUBLIC void
TT_AciDestroy(void);


/****************************************************************************
 * FUNCTION: TT_AciProcess
 *
 * DESCRIPTION:
 *   Runs the ACI application in an internal state machine and handles all
 *   access to the decoder and the display part.
 *
 * PARAMETERS:
 *   aCommand - possible commands for AciProcess:
 *     -         TT_ACI_UPDATE
 *     -         TT_ACI_START
 *     -         TT_ACI_STOP
 *     -         TT_ACI_OK
 *     -         TT_ACI_UP
 *     -         TT_ACI_DOWN
 *   aParam   - an optional parameter
 *
 * RETURN:
 *   True in case of success, otherwise False
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AciProcess(
  U_INT16 aCommand,
  U_INT16 aParam);


/****************************************************************************
 * FUNCTION: TT_AciGetResult
 *
 * DESCRIPTION:
 *   The function TT_AciGetResult returns a single entry of the received
 *   channel setup data. If the command is called first time it returns the
 *   first channel information. On each further call, it returns always the
 *   next channel information. The maximum number of frequency information is
 *   limited by the ACI standard to
 *     99. At the end of the list the function returns False.
 *
 * PARAMETERS:
 *   aResultItem - buffer to return a single channel information.
 *   aIndex      - number of the resultitem to read from the result buffer.
 *
 * RETURN:
 *   False at the end of the list, otherwise True.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AciGetResult(
  TT_ACI_RESULT_ITEM *aResultItem,
  U_INT8 aIndex);
#endif /* (TT_ACI_FEATURE == ENABLED) */



#ifdef __cplusplus
  }
#endif

#endif /* TT_ACI_H */

/* This Headerfile was generated with C2H.exe ( C2H ) */
/* (c) 2006 by TARA Systems GmbH Munich */
