/****************************************************************************
 * MODULE     : TT_AciDecoder
 * FILE       : TT_AciDecoder.h
 * COPYRIGHT  : TARA Systems GmbH, Mario Stefanutti, March 2005
 * DESCRIPTION:
 *   The module TT_AciDecoder handles the ACI data decoding. It is using the
 *   module TT_AciAcquisition to abstract all access to the Teletext
 *   acquisition. This makes the decoder itself platform independent. The
 *   decoder reads each required packet directly from the Teletext database
 *   and the results are provided by a callback function immediatelly after
 *   each single channel info decoding (see
 *   TT_AciDecoderInstallResultCallback and typedef TT_ACI_RESULT_CALLBACK).
 ****************************************************************************/

#ifndef TT_ACIDECODER_H
#define TT_ACIDECODER_H



/****************************************************************************
 * SECTION: #define
 ****************************************************************************/
#define ACI_VERSION_STRING "ACI V1.00"
#define ACI_VERSION_NUMBER     0x0100

#define ACI_MAX_NO_OF_CHANNELS     99
#define ACI_DEFAULT_TIMEOUT        10
#define ACI_NAME_LENGTH             7
#define ACI_DATA_FALSE              0
#define ACI_DATA_TRUE               1
#define ACI_DATA_ERR                2


/****************************************************************************
 * SECTION: typedef
 ****************************************************************************/

/****************************************************************************
 * TYPE: TT_ACI_LANGUAGE
 *
 * DESCRIPTION:
 *   The structure TT_ACI_LANGUAGE defines the language used for decoding.
 ****************************************************************************/
typedef enum{
  ACI_LANGUAGE_ENGLISH,
  ACI_LANGUAGE_FRENCH,
  ACI_LANGUAGE_GERMAN,
  ACI_LANGUAGE_ITALIAN,
  ACI_LANGUAGE_SPANISH,
  ACI_LANGUAGE_DUTCH,
  ACI_LANGUAGE_DANISH,
  ACI_LANGUAGE_NORWEGIAN,
  ACI_LANGUAGE_SWEDISH,
  ACI_LANGUAGE_SUOMI,
  ACI_LANGUAGE_POLISH,
  ACI_LANGUAGE_CZECH,
  ACI_LANGUAGE_HUNGARIAN,
  ACI_LANGUAGE_ROMANIAN,
  ACI_LANGUAGE_BULGARIAN,
  ACI_LANGUAGE_RUSHIAN,
  ACI_LANGUAGE_GREEK,
  ACI_LANGUAGE_PORTUGUESE,
  ACI_LANGUAGE_TURKISH,
  ACI_LANGUAGE_DEFAULT
} TT_ACI_LANGUAGE;



/****************************************************************************
 * TYPE: TT_ACI_RESULT_ITEM
 *
 * DESCRIPTION:
 *   The structure TT_ACI_RESULT_ITEM contains the channel setup information
 *   for a single channel.
 *
 * COMPONENTS:
 *   Name      - String for channel name
 *   System    - defines the sound system:
 *'       ACI_SOUND_TV_PAL_SECAM_BG  0x01   TV PAL/SECAM BG *
 *'       ACI_SOUND_TV_SECAML        0x02   TV SECAM L      *
 *'       ACI_SOUND_TV_PAL_M         0x03   TV PAL M        *
 *'       ACI_SOUND_TV_NTSC_M        0x04   TV NTSC M       *
 *'       ACI_SOUND_TV_PAL_I         0x05   TV PAL I        *
 *'       ACI_SOUND_TV_SECAM_DK      0x06   TV SECAM D/K    *
 *'       ACI_SOUND_RADIO_FM         0x10   RADIO FM        *
 *'       ACI_SOUND_RADIO_DSR        0x11   RADIO DSR       *
 *'       ACI_SOUND_RADIO_DAB        0x12   RADIO DAB       *
 *'       ACI_SOUND_RADIO_DCR        0x13   RADIO DCR       *
 *   Frequency - Tuning frequency in kHz
 ****************************************************************************/
typedef struct
{
    U_INT8  Name[ACI_NAME_LENGTH];
    U_INT8  System;
    U_INT32 Frequency;
} TT_ACI_RESULT_ITEM;

    
/****************************************************************************
 * TYPE: TT_ACI_RESULT
 *
 * DESCRIPTION:
 *   The structure TT_ACI_RESULT is the type for a buffer of the maximum
 *   number of result items.
 ****************************************************************************/
typedef struct
{
  TT_ACI_RESULT_ITEM Item[ACI_MAX_NO_OF_CHANNELS + 1];
} TT_ACI_RESULT; /* see AciStartDecode() */


/****************************************************************************
 * TYPE: TT_ACI_RESULT_HANDLE
 *
 * DESCRIPTION:
 *   TT_ACI_RESULT_HANDLE is a pointer to TT_ACI_RESULT.
 ****************************************************************************/
typedef TT_ACI_RESULT* TT_ACI_RESULT_HANDLE;


/****************************************************************************
 * TYPE: TT_ACI_STATUS
 *
 * DESCRIPTION:
 *   The structure TT_ACI_STATUS defines.the current decoding status. Call
 *   the
 *
 * COMPONENTS:
 *   AciDetect - True: Decoding started False: Decoding not started
 *   AciExist  - True: ACI service detected from header accelerator bit
 *               False: no ACI service on this channel
 *   AciState  - 0: not decoding 1: decoding 2: error occured
 *   AciPage   - the currently required ACI page for decoding was found;
 *   AciMulti  - the found ACI page contains multi package info
 *   AciFinish - 1: decoding is finished,the results are available.
 ****************************************************************************/
typedef struct
{
  BitField AciDetect   : 1;
  BitField AciExist    : 1;
  BitField AciState    : 2;
  BitField AciPage     : 1;
  BitField AciMulti    : 1;
  BitField AciFinish   : 1;
}TT_ACI_STATUS;


/****************************************************************************
 * TYPE: TT_ACI_RESULT_CALLBACK
 *
 * DESCRIPTION:
 *   This function is called after each decoded channel information.
 *
 * PARAMETERS:
 *   aResultItem - Returns the decoded channel information
 *   aIndex      - The channel index
 ****************************************************************************/
typedef void
(*TT_ACI_RESULT_CALLBACK)( 
  TT_ACI_RESULT_ITEM *aResultItem,
  U_INT8 aIndex);


/****************************************************************************
 * TYPE: TT_ACI_TIMER_CALLBACK
 *
 * DESCRIPTION:
 *   A function that provides a current timer value in milliseconds.
 *
 * RETURN:
 *   Milliseconds.
 ****************************************************************************/
typedef U_INT32
(*TT_ACI_TIMER_CALLBACK)( 
  void);

#ifdef __cplusplus
  extern "C" {
#endif



#if (TT_ACI_FEATURE == ENABLED)
/****************************************************************************
 * FUNCTION: TT_AciDecoderCreate
 *
 * DESCRIPTION:
 *   The function TT_AciDecoderCreate initializes the decoder. It must be
 *   called before using functions of this module.
 *
 * RETURN:
 *   True if successful, otherwise False
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AciDecoderCreate( void );


/****************************************************************************
 * FUNCTION: TT_AciDecoderDelete
 *
 * DESCRIPTION:
 *   The function TT_AciDecoderDelete destroys the Aci decoder.
 ****************************************************************************/

PUBLIC void
TT_AciDecoderDelete( void );


/****************************************************************************
 * FUNCTION: TT_AciDecoderStart
 *
 * DESCRIPTION:
 *   The function TT_AciDecoderStart starts the decoding procedure. An
 *   internal state machine requires continuously to be updated by calling
 *   the function TT_AciDecoderUpdate periodically. This means that other
 *   applications can proceed in parallel and do not have to wait until ACI
 *   decoding is finished.
 *
 * PARAMETERS:
 *   aLanguage - language used for ACI decoding. If the parameter is invalid,
 *               the default language, defined by the ACI service, will be
 *               used.
 *
 * SEE ALSO:
 *   Type
 *   TT_ACI_LANGUAGE
 ****************************************************************************/

PUBLIC void
TT_AciDecoderStart(
  TT_ACI_LANGUAGE aLanguage );


/****************************************************************************
 * FUNCTION: TT_AciDecoderStop
 *
 * DESCRIPTION:
 *   The function TT_AciDecoderStop stops ACI decoding. It must be called if
 *   ACI is aborted, an ACI error occured or ACI was finished. Otherwise the
 *   ACI decoding can not be restarted.
 ****************************************************************************/

PUBLIC void
TT_AciDecoderStop( void );


/****************************************************************************
 * FUNCTION: TT_AciDecoderUpdate
 *
 * DESCRIPTION:
 *   The function TT_AciDecoderUpdate executes on each call a small decoding
 *   task and brings the internal state machine to the next state. It must be
 *   called periodically (50-200ms recommended) until the decoding procedure
 *   is finished (see function TT_AciDecoderGetStatus). While a multi package
 *   page is detected (see TT_AciDecoderGetStatus) it is not necessarry to
 *   call TT_AciDecoderUpdate periodically untill an area is selected with
 *   TT_AciDecoderOk.
 ****************************************************************************/

PUBLIC void
TT_AciDecoderUpdate( void );


/****************************************************************************
 * FUNCTION: TT_AciDecoderOk
 *
 * DESCRIPTION:
 *   The function TT_AciDecoderOk tells the ACI decoder the area that the
 *   user has selected to continue ACI decoding. Since the ACI decoder has no
 *   cursor handling and does not know about a user selection, it is
 *   necessary to inform the decoder by the given parameter "aIndex" about
 *   the user selection. The function may only be called on a user selection
 *   with OK - don't call it for each cursor navigation up/down. Afterwards
 *   the decoding can be proceeded by repetitive calls of
 *   TT_AciDecoderUpdate.
 *
 * PARAMETERS:
 *   aIndex - index of the area to continue decoding
 ****************************************************************************/

PUBLIC void
TT_AciDecoderOk(
  U_INT8 aIndex );


/****************************************************************************
 * FUNCTION: TT_AciDecoderGetStatus
 *
 * DESCRIPTION:
 *   The function TT_AciDecoderGetStatus returns the current decoder status.
 *   The function TT_AciDecoderUpdate should be called at least once, before
 *   the new status is check by this function.
 *
 * PARAMETERS:
 *   aData - Buffer for the return value.
 ****************************************************************************/

PUBLIC void
TT_AciDecoderGetStatus(
  TT_ACI_STATUS *aData );


/****************************************************************************
 * FUNCTION: TT_AciDecoderSetTimeOut
 *
 * DESCRIPTION:
 *   The function TT_AciDecoderSetTimeOut limits the time in seconds, to
 *   search an AciPage. After that time TT_AciDecoderUpdate will detect an
 *   error. The default timeout is set to 10 seconds. See function
 *   TT_AciDecoderGetStatus.
 *
 * PARAMETERS:
 *   aTimeOut - timeout in seconds.
 ****************************************************************************/

PUBLIC void
TT_AciDecoderSetTimeOut(
  U_INT16 aTimeOut );


/****************************************************************************
 * FUNCTION: TT_AciDecoderGetAciPage
 *
 * DESCRIPTION:
 *   The function TT_AciDecoderGetAciPage returns the ACI page number.
 *
 * RETURN:
 *   Returns the number of menu items.
 ****************************************************************************/

PUBLIC U_INT16
TT_AciDecoderGetAciPage( void );


/****************************************************************************
 * FUNCTION: TT_AciDecoderGetNoOfMenuItems
 *
 * DESCRIPTION:
 *   The function TT_AciDecoderGetNoOfMenuItems returns the number of menu
 *   items in a multi package page.
 *
 * RETURN:
 *   Returns the number of menu items.
 ****************************************************************************/

PUBLIC U_INT8
TT_AciDecoderGetNoOfMenuItems(void);


/****************************************************************************
 * FUNCTION: TT_AciDecoderGetMenuString
 *
 * DESCRIPTION:
 *   The function TT_AciDecoderGetMenuString must be called after
 *   TT_AciDecoderGetStatus delivers the status "AciMulti == ACI_DATA_TRUE"
 *   (the page to decode is a multi area selection page). It starts with the
 *   first found menu item and returns only one menu-string from a multi area
 *   selection menu. To get all menu items it must be called repetitive until
 *   it returns "0" at the end of the menu. After all menu items are fetched,
 *   the ACI decoding continues with the user-selected area by a call of
 *   TT_AciDecoderOk. If a further multi area page (sub menu) gets found, the
 *   procedure must be repeated.
 *
 * PARAMETERS:
 *   aBuffer - must be a buffer of atleast 20 byte
 *
 * RETURN:
 *   ACI_DATA_FALSE string invalid ACI_DATA_TRUE string valid
 ****************************************************************************/

PUBLIC U_INT8
TT_AciDecoderGetMenuString(
  INT8 *aBuffer );


/****************************************************************************
 * FUNCTION: TT_AciDecoderIsMultiPage
 *
 * DESCRIPTION:
 *   The function TT_AciDecoderIsMultiPage returns whether the given contains
 *   multi page info.
 *
 * PARAMETERS:
 *   aRow - row to search for the multi page info.
 *
 * RETURN:
 *   True if the given row contains a page link, otherwise False
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AciDecoderIsMultiPage(
  U_INT8 aRow );


/****************************************************************************
 * FUNCTION: TT_AciDecoderSetMultiPage
 *
 * DESCRIPTION:
 *   The function TT_AciDecoderSetMultiPage sets the page link of the given
 *   row to continue decoing with that page.
 *
 * PARAMETERS:
 *   aRow - row to use for the page link info.
 *
 * RETURN:
 *   True if the given row contains a valid page link.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_AciDecoderSetMultiPage(
  U_INT8 aRow );


/****************************************************************************
 * FUNCTION: TT_AciDecoderInstallResultCallback
 *
 * DESCRIPTION:
 *   The function TT_AciDecoderInstallResultCallback installs the callback
 *   function that is called on each available result. It returns a single
 *   entry of the received channel setup data. If the decoding is finished
 *   the callback returns False and the flag "AciFinished" get set in
 *   TT_AciDecoderGetStatus().
 *
 * PARAMETERS:
 *   aCallback - function pointer to the result callback function.
 ****************************************************************************/

PUBLIC void
TT_AciDecoderInstallResultCallback(
  TT_ACI_RESULT_CALLBACK    aCallback );


/****************************************************************************
 * FUNCTION: TT_AciDecoderInstallTimerCallback
 *
 * DESCRIPTION:
 *   The function TT_AciDecoderInstallTimerCallback installs the callback
 *   function that is called to get a current timer value in milliseconds.
 *
 * PARAMETERS:
 *   aCallback - function pointer to the timer callback function.
 ****************************************************************************/

PUBLIC void
TT_AciDecoderInstallTimerCallback(
  TT_ACI_TIMER_CALLBACK    aCallback );


/****************************************************************************
 * FUNCTION: TT_AciDecoderGetVersion
 *
 * DESCRIPTION:
 *   The function TT_AciDecoderGetVersion returns the version string and
 *   version number
 *
 * PARAMETERS:
 *   aVersion - returns a 16 bit version number
 *
 * RETURN:
 *   Returns a version string
 ****************************************************************************/

PUBLIC const INT8 *
TT_AciDecoderGetVersion(
  U_INT16    *aVersion );
#endif /* (TT_ACI_FEATURE == ENABLED) */



#ifdef __cplusplus
  }
#endif

#endif /* TT_ACIDECODER_H */

/* This Headerfile was generated with C2H.exe ( C2H ) */
/* (c) 2006 by TARA Systems GmbH Munich */
