/****************************************************************************
 * MODULE     : TT_Charset
 * FILE       : TT_Charset.h
 * COPYRIGHT  : TARA Systems GmbH
 * DESCRIPTION:
 *   The TT_Charset modules stores the language settings for decoding a
 *   Teletext page and translates the Teletext specific attributes into a
 *   font selector and a character code. This information is used by a
 *   Teletext display.
 ****************************************************************************/

#ifndef TT_CHARSET_H
#define TT_CHARSET_H


/*
language index
a language index is a pointer into the tables
"Default G0 / G2 Character Set Designation and National Option"
"Second Language Designation and Nation Option"
*/
typedef unsigned char TT_Charset_LG_Index;

/*
enum for primary & secondary language
*/
typedef enum
{
  TT_Charset_Primary,
  TT_Charset_Secondary

} TT_Charset_Order;

/*
language
*/
typedef enum
{
  TT_Charset_LG_Czech,      /* Czech/Slovak               */
  TT_Charset_LG_English,    /* English                    */
  TT_Charset_LG_Estonian,   /* Estonian                   */
  TT_Charset_LG_French,     /* French                     */
  TT_Charset_LG_German,     /* German                     */
  TT_Charset_LG_Italian,    /* Italian                    */
  TT_Charset_LG_Lettish,    /* Lettish/Lithuanian         */ 
  TT_Charset_LG_Polish,     /* Polish                     */
  TT_Charset_LG_Spanish,    /* Portuguese/Spanish         */
  TT_Charset_LG_Rumanian,   /* Rumanian                   */
  TT_Charset_LG_Serbian,    /* Serbian/Croatian/Slovenian */
  TT_Charset_LG_Swedish,    /* Swedish/Finnish/Hungarian  */
  TT_Charset_LG_Turkish,    /* Turkish                    */
  TT_Charset_LG_Cyrillic1,  /* Serbian/Croatian           */
  TT_Charset_LG_Cyrillic2,  /* Russian/Bulgarian          */
  TT_Charset_LG_Cyrillic3,  /* Ukrainian                  */
  TT_Charset_LG_Greek,      /* Greek                      */
  TT_Charset_LG_Hebrew,     /* Hebrew                     */
  TT_Charset_LG_Arabic,     /* Arabic                     */
  TT_Charset_LG_Farsi,      /* Farsi                      */
  TT_Charset_LG_EnglishArabic, /* EnglishArabic           */
  TT_Charset_LG_FrenchArabic,  /* FrenchArabic            */
  TT_Charset_LG_EnglishFarsi,  /* EnglishFarsi            */
  TT_Charset_LG_FrenchFarsi,   /* FrenchFarsi             */

  TT_Charset_LG_Last = TT_Charset_LG_FrenchFarsi,

  TT_Charset_NotUsed = 0xFF

} TT_Charset_LG;

/*
fonts
*/
typedef enum
{
  TT_Charset_Font_G0_Latin,
  TT_Charset_Font_G1_block_mosaic,
  TT_Charset_Font_G1_block_mosaic_separated,
  TT_Charset_Font_G2_Latin,
  TT_Charset_Font_G3_smooth_mosaic,
  TT_Charset_Font_International1,
  TT_Charset_Font_International2,
  TT_Charset_Font_International3,
  TT_Charset_Font_Symbol,
  TT_Charset_Font_G0_Arabic,
  TT_Charset_Font_G0_Cyrillic1,
  TT_Charset_Font_G0_Cyrillic2,
  TT_Charset_Font_G0_Cyrillic3,
  TT_Charset_Font_G0_Greek,
  TT_Charset_Font_G0_Hebrew,
  TT_Charset_Font_G2_Arabic,
  TT_Charset_Font_G2_Cyrillic,
  TT_Charset_Font_G2_Greek,
  TT_Charset_Font_G3_smooth_mosaic_separated,
  TT_Charset_Font_DRCS,
  TT_Charset_Font_G0_Farsi,
  TT_Charset_Font_G2_Farsi,
  TT_Charset_Font_Reserved

} TT_Charset_Font;

/*
font property
*/
typedef struct
{
  TT_Charset_Font
    /*
    primary/secondary font selector G0, G2
    */
    FontSelG0,
    FontSelG2;

  TT_Charset_LG
    /*
    primary/secondary language
    */
    Language;

} TT_Charset_Font_Property;

/*
character property
do not change series of variables, 
because ROM tables are initialized using this entry
*/
typedef struct
{
  TT_Charset_Font
    /* font */
    Font;

  U_INT8 
    /* character code */
    Code;

} TT_Charset_Char_Property;


/*
language set
*/
typedef struct
{
  TT_Charset_LG
    /* 
    primary and secondary language
    */
    Language[2];

} TT_Charset_LG_Set;


/*
language group
*/
typedef struct
{
  TT_Charset_LG_Set
    /*
    lanuage set for each national option (c12,13,14)
    */
    NatOptLanguageSet[8];
    
} TT_Charset_LG_Group;

#ifdef __cplusplus
  extern "C" {
#endif


/****************************************************************************
 * FUNCTION: TT_CharsetGetCharCodeAndFont
 *
 * DESCRIPTION:
 *   Translate the teletext specific parameters Character, Accent, CharSet
 *   and Language to CharCode and FontSelector for the DSP this function maps
 *   accented characters and languagespecific characters to CharCodes in the
 *   Extended Font Segment.
 *
 * PARAMETERS:
 *   aCharacter    - character code
 *   aAccent       - accent
 *   aCharSet      - character set
 *   aNatOpt       - national option code()
 *   aCharCode     - character code
 *   aFontSelector - font selector
 ****************************************************************************/

PUBLIC void
TT_CharsetGetCharCodeAndFont(
  U_INT8         aCharacter,
  U_INT8         aAccent,
  U_INT8         aCharSet,
  TT_Charset_LG  aNatOpt,
  U_INT8*        aCharCode,
  U_INT8*        aFontSelector );


/****************************************************************************
 * FUNCTION: TT_CharsetSetDefaultLanguages
 *
 * DESCRIPTION:
 *   Set default languages.
 *
 * PARAMETERS:
 *   aPrimary   - default language primary
 *   aSecondary - default language secondary
 ****************************************************************************/

PUBLIC void 
TT_CharsetSetDefaultLanguages(
  TT_Charset_LG       aPrimary,
  TT_Charset_LG       aSecondary);


/****************************************************************************
 * FUNCTION: TT_CharsetSetLanguageGroup
 *
 * DESCRIPTION:
 *   Set default language group.
 *
 * PARAMETERS:
 *   aGroup - language group
 ****************************************************************************/

PUBLIC void 
TT_CharsetSetLanguageGroup(
  TT_Charset_LG_Group ROM*  aGroup );





#ifdef __cplusplus
  }
#endif

#endif /* TT_CHARSET_H */

/* This Headerfile was generated with C2H.exe ( C2H ) */
/* (c) 2006 by TARA Systems GmbH Munich */
