/****************************************************************************
 * MODULE     : TT_Database
 * FILE       : TT_Database.h
 * COPYRIGHT  : TARA Systems GmbH
 * DESCRIPTION:
 *   Database to store teletext header and data packets.
 *
 *   All packets of one teletext page are linked together using a packet
 *   chain. Starting point is always the header packet. For fast access to a
 *   teletext page the link to the header packet is stored inside the
 *   database lookup table. This is the first link to the packet chain, which
 *   contains all other packets of the teletext page.
 *
 *   All subpage headers which belong to one teletext page are linked
 *   together using a subpage chain. Each header is linked together and the
 *   link to the last received subpage header is always stored inside the
 *   database lookup table.
 *
 *   For setting packets the functions TT_DatabaseCreateHeaderPacket() and
 *   TT_DatabaseSetDataPacket() are used.
 *
 *   For retrieving packets the functions TT_DatabaseGetHeaderPacket() and
 *   TT_DatabaseGetDataPacket() are used.
 *
 *   The data packet functions expect a handle to the header packet of the
 *   desired page. The handle can be obtained with
 *   TT_DatabaseGetHeaderPacket()
 ****************************************************************************/

#ifndef TT_DATABASE_H
#define TT_DATABASE_H


/****************************************************************************
 * SECTION: #include
 ****************************************************************************/


/****************************************************************************
 * SECTION: #define
 ****************************************************************************/

#define TT_DB_PACKET_DATASIZE    40
#define TT_DB_PACKET_HEADERSIZE  36

#define TT_DB_DONT_CARE          0xFFFF
#define TT_DB_DEFAULT_HEADER     0xFFFF
#define TT_DB_ID_CARE_FIRST_BYTE 0xFF00

#define TT_DB_REMOVE_PAGE        0
#define TT_DB_REMOVE_SUBPAGE     1


/****************************************************************************
 * SECTION: #define
 ****************************************************************************/

/* a packet handle is just the number of a packet 
   inside the packet buffer */
typedef U_INT32 TT_DATABASE_PACKET_HANDLE;
typedef U_INT8  TT_DB_PACKET_DATA;

/****************************************************************************
 * TYPE: TT_DATABASE_PACKET
 *
 * DESCRIPTION:
 *   Data structure for a packet.
 ****************************************************************************/
typedef struct
{
  U_INT8  Data[ TT_DB_PACKET_DATASIZE ];

} TT_DATABASE_PACKET;


#ifdef __cplusplus
  extern "C" {
#endif


/****************************************************************************
 * FUNCTION: TT_DatabaseGetNoOfFreePackets
 *
 * DESCRIPTION:
 *   Get number of free packets in database.
 *
 * RETURN:
 *   Number of free packets in database.
 ****************************************************************************/

PUBLIC U_INT32
TT_DatabaseGetNoOfFreePackets(void);


/****************************************************************************
 * FUNCTION: TT_DatabaseGetHeaderPacket
 *
 * DESCRIPTION:
 *   Retrieves header packet data from the database.
 *
 * PARAMETERS:
 *   aPacketHandle - packet handle of the returned header
 *   aPage         - page number or TT_DB_DEFAULT_HEADER
 *   aPacket       - pointer to packet data
 *
 * RETURN:
 *   TRUE   if successfull,
 *   FALSE  otherwise.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_DatabaseGetHeaderPacket (
  TT_DATABASE_PACKET_HANDLE *aPacketHandle,
  TT_ACQ_PAGE               *aPage,
  TT_DATABASE_PACKET        *aPacket );


/****************************************************************************
 * FUNCTION: TT_DatabaseGetNextHeaderId
 *
 * DESCRIPTION:
 *   Retrieves the next header packet data from the database.
 *
 * PARAMETERS:
 *   aPacketHandle - packet handle of the returned header
 *   aPage         - page number
 *   aPacket       - pointer to packet data
 *   aId           - of the next available header ( only if the return value
 *                   is TRUE )
 *
 * RETURN:
 *   TRUE   if successfull,
 *   FALSE  otherwise.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_DatabaseGetNextHeaderId (
  TT_DATABASE_PACKET_HANDLE *aPacketHandle,
  TT_ACQ_PAGE               *aPage,
  TT_DATABASE_PACKET        *aPacket,
  U_INT16                   *aId);


/****************************************************************************
 * FUNCTION: TT_DatabaseGetDataPacket
 *
 * DESCRIPTION:
 *   Retrieves packet data from the database.
 *
 * PARAMETERS:
 *   aPacketHandle - packet handle that is used for setting data packets
 *   aId           - packet id ( row number )
 *   aPacket       - pointer to packet data
 *
 * RETURN:
 *   TRUE   if successfull,
 *   FALSE  otherwise.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_DatabaseGetDataPacket (
  TT_DATABASE_PACKET_HANDLE aPacketHandle,
  U_INT16                   aId,
  TT_DATABASE_PACKET        *aPacket );


/****************************************************************************
 * FUNCTION: TT_DatabaseRemove
 *
 * DESCRIPTION:
 *   Removes all subpages / a single subpage of the specified page .
 *
 * PARAMETERS:
 *   aPage - page number
 *   aId   - subpage number or TT_DB_DONT_CARE
 *   aMode - TT_DB_REMOVE_PAGE / TT_REMOVE_SUBPAGE
 *
 * RETURN:
 *   TRUE   if successfull,
 *   FALSE  otherwise.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_DatabaseRemove (
  U_INT16   aPage,
  U_INT16   aId,
  U_INT8    aMode );


/****************************************************************************
 * FUNCTION: TT_DatabaseGetHeaderId
 *
 * DESCRIPTION:
 *   Get the packet id of the specified header.
 *
 * PARAMETERS:
 *   aPacket - page handle that refers to the header packet
 *
 * RETURN:
 *   Header packet id ( subpage number ).
 ****************************************************************************/

PUBLIC U_INT16
TT_DatabaseGetHeaderId (
  TT_DATABASE_PACKET_HANDLE  aPacket );


/****************************************************************************
 * FUNCTION: TT_DatabaseCheckDatabase
 *
 * DESCRIPTION:
 *   Checks whether the database content is correct or not. All packets
 *   inside the database and all free packets are counted. The sum of both
 *   counters must equal the number of allocated packets at database
 *   creation.
 *
 * RETURN:
 *   NULL on success, lost packet chain on error.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_DatabaseCheckDatabase(void);



#ifdef __cplusplus
  }
#endif

#endif /* TT_DATABASE_H */

/* This Headerfile was generated with C2H.exe ( C2H ) */
/* (c) 2006 by TARA Systems GmbH Munich */
