/****************************************************************************
 * MODULE     : TT_Decoder
 * FILE       : TT_Decoder.h
 * COPYRIGHT  : TARA Systems GmbH
 * DESCRIPTION:
 *   The module TT_Decoder contains functions to create and destroy the
 *   decoder as well as a couple of interface functions. The Decoder
 *   initializes all necessary modules (ACQ, SPC, Display, Packetbuffer, TOP,
 *   Database) and contains the main processing rutines of the teletext
 *   decoder TT_DecoderProcess() and TT_DecoderDisplayProcess().
 *
 *   TT_DecoderProcess() executes:
 *     1. Acquisition packet processing
 *     2. Processing of user inputs
 *     3. Update of display
 *
 *   TT_DecoderDisplayProcess() executes:
 *     1. Processing of the pending display actions
 *     2. Flash color animation
 *
 *   Additionally to interface functions in this module, the major part of
 *   commands to control the decoder is defined in module "TT_Key.h". This
 *   commands must be sent to the decoder as messages with the function
 *   "TT_QueueSet()" if called from a different task. Or this commands are
 *   sent to the decoder calling the function TT_DecoderSetKeyCommand() if
 *   called from the same task.
 *
 * REMARK:
 *   This file can be released for customization purposes.
 ****************************************************************************/

#ifndef TT_DECODER_H
#define TT_DECODER_H


/****************************************************************************
 * SECTION: #include
 ****************************************************************************/

#include "teletext/TT_Stdinc.h"

/****************************************************************************
 * SECTION: #define
 ****************************************************************************/

#define TT_DEC_MESSAGE_DRAWROW        1u
#define TT_DEC_MESSAGE_DRAWPAGE       2u
#define TT_DEC_MESSAGE_CREATEDISPLAY  3u
#define TT_DEC_MESSAGE_DESTROYDISPLAY 4u

#define TT_DEC_HOLD_MODE              0x01u
#define TT_DEC_REVEAL_MODE            0x02u
#define TT_DEC_SIZE_MODE              0x04u
#define TT_DEC_CANCEL_MODE            0x08u
#define TT_DEC_MIX_MODE               0x10u
#define TT_DEC_LIST_MODE              0x20u
#define TT_DEC_CLOCK_MODE             0x40u
#define TT_DEC_STATUS_MODE            0x80u
#define TT_DEC_ALL_MODES              0xFFu

#define TT_DEC_SUBPAGE_BAR_ON         TRUE
#define TT_DEC_SUBPAGE_BAR_OFF        FALSE

#define TT_DEC_PAGE_IN_CYCLE          1u
#define TT_DEC_PAGE_IN_BTT            2u
#define TT_DEC_PAGE_NOT_IN_BTT        3u
#define TT_DEC_PAGE_UNKOWN            4u

#define TT_DEC_MAX_NOF_SUBPAGES       TT_ACQ_NOF_SUBPAGES  

/****************************************************************************
 * SECTION: typedef
 ****************************************************************************/



/****************************************************************************
 * TYPE: TT_DEC_STATUS
 *
 * DESCRIPTION:
 *   All status information needed from the outside world.
 *
 * COMPONENTS:
 *   Mix -
 *     Mix mode is active.
 *
 *   Cancel -
 *     Cancel mode is active.
 *
 *   Reveal -
 *     Reveal mode is active.
 *
 *   Hold -
 *     Hold mode is active.
 *
 *   User -
 *     User mode display is activated.
 *
 *   Display -
 *     The Teletext display is activated.
 *
 *   Clock -
 *     Clock mode is activated.
 *
 *   StatusDisplay -
 *     Status display is activated.
 *
 *   List -
 *     List mode selection is activated.
 *
 *   Size -
 *     The actual selected size mode. ' 0 normal size ' 1 upper half ' 2
 *     lower half
 *
 *   PageCatch -
 *     Page catching is activated.
 *
 *   TopMenu -
 *     Top menu is activated.
 *
 *   Subtitle -
 *     Subtitle pages available in this Teletext cycle.
 *
 *   SubNews -
 *     The selected page is a subtitle or newsflash page.
 *
 *   PageFound -
 *     The selected page is found.
 *
 *   PageProcessed -
 *     The selected page is processed by the decoder and ready for display.
 *
 *   PageAvailable -
 *     The selected page was previously found in the Teletext cycle.
 *
 *   PageUpdateAvailable -
 *     A new version of the selected page is available.
 *
 *   RequestedPageAvailable -
 *     The page requested by the command TT_DEC_REQUEST_PAGE is available.
 *
 *   AciFound -
 *     ACI (Automatic Channel Installation) is detected in this Teletext
 *     cycle.
 *
 *   SubpageAvailable -
 *     The selected page contains subpages
 *
 *   SubpageBarVisible -
 *     The subpage bar is visible
 *
 *   TopFlofEnable -
 *     The current TOF/FLOF navigation settings. See command
 *     TT_DEC_COLOR_KEY_MODE
 *
 *   NumCount -
 *     Number of digit inputs for direct page number input.
 *
 *   NumKey -
 *     The current number input value in BCD notation. (input 100 equals
 *     0x0100)
 *
 *   ListModeSelection -
 *     The actual selected list mode entry.
 *
 *   SubpageInput -
 *     Subpage input mode is activated.
 *
 *   SubpageInputNumCount -
 *     Number of digit inputs for direct subpage number input.
 *
 *   SubpageInputNumKey -
 *     The current number input value in BCD notation. (input 0030 equals
 *     0x0030)
 *
 *   LanguageGroup -
 *     The current language group
 ****************************************************************************/
typedef struct 
{
  TT_BOOL   Mix;
  TT_BOOL   Cancel;
  TT_BOOL   Reveal;
  TT_BOOL   Hold;
  TT_BOOL   User;
  TT_BOOL   Display;
  TT_BOOL   Clock;
  TT_BOOL   StatusDisplay;
  TT_BOOL   List;
  U_INT8    Size;
  TT_BOOL   PageCatch;
  TT_BOOL   TopMenu;
  TT_BOOL   Subtitle;
  TT_BOOL   SubNews;
  TT_BOOL   PageFound;
  TT_BOOL   PageProcessed;
  U_INT8    PageAvailable;
  TT_BOOL   PageUpdateAvailable;
  TT_BOOL   RequestedPageAvailable;
  TT_BOOL   AciFound;
  TT_BOOL   SubpageAvailable;
  TT_BOOL   SubpageBarVisible;
  U_INT8    TopFlofEnable;
  U_INT8    NumCount;
  U_INT16   NumKey;
  U_INT8    ListModeSelection;
  TT_BOOL   SubpageInput;
  U_INT8    SubpageInputNumCount;
  U_INT16   SubpageInputNumKey;
  U_INT8    LanguageGroup;

} TT_DEC_STATUS;


/***********************************************************************
 * _TYPE: TT_DEC_USER_SETTINGS
 *
 * DESCRIPTION:
 *   The user settings define the internal state of the decoder selected
 *   by the user.
 ***********************************************************************/
typedef struct 
{
  TT_BOOL    Mix;
  TT_BOOL    Cancel;
  TT_BOOL    Reveal;
  TT_BOOL    Hold;
  TT_BOOL    Show;
  TT_BOOL    Display;
  TT_BOOL    Clock;
  TT_BOOL    StatusDisplay;
  TT_BOOL    DebugDisplay;
  TT_BOOL    List;
  TT_BOOL    TopMenu;
  TT_BOOL    PageCatch;
  TT_BOOL    BeyondMagazineHandlingDefault;
  TT_BOOL    SubpageInput;
  U_INT16    SubpageInputNumKey;
  U_INT8     SubpageInputNumCount;
  U_INT8     Size;
  U_INT8     NumCount;
  U_INT8     ListModeSelection;
  U_INT8     Processing_Level;
  U_INT8     LanguageGroup;
  U_INT16    NumKey;
  U_INT16    ListRed;
  U_INT16    ListGreen;
  U_INT16    ListYellow;
  U_INT16    ListCyan;
  U_INT16    ListCustom1;
  U_INT16    ListCustom2;

} TT_DEC_USER_SETTINGS;



/****************************************************************************
 * _TYPE: TT_DecoderNewsSubttFound
 *
 * DESCRIPTION:
 *    This function sould be registered at TT_DecoderCreate and is called 
 *    when news or subtitle is found.
 ****************************************************************************/
 

typedef void (*TT_DecoderNewsSubttFound_T)(void);


#ifdef __cplusplus
  extern "C" {
#endif



/****************************************************************************
 * FUNCTION: TT_DecoderCreate
 *
 * DESCRIPTION:
 *   The function creates the decoder and all its modules related to the
 *   selected configuration. The passed memory is fully used by the Teletext
 *   Core module. The decoder itself and all other decoder modules uses
 *   static memory.
 *
 * PARAMETERS:
 *   aMemoryStart - The start address of the memory to use.
 *   aMemorySize  - The passed memory size. Both parameter are used on
 *                  dynamic memory allocation only.
 *   newsSubttFoundCB
 *                - a callback function to be registered and is called when
 *                  news or subtitle is decoded
 * NOTE:
 *   The memory passed to the core module is divides in two parts. The first 
 *   part is the memory used for all internal module buffers. The remaining 
 *   amount of memory is fully used for the page storage where about 1400 
 *   bytes can be calculated for each page on a regular Teletext broadcast.
 *   Based on the default configuration the internal memory consumption is:
 *     - for Level 1.5: 70k byte
 *     - for Level 2.5: 86k byte
 *
 * RETURN:
 *   TRUE if successful, FALSE otherwise.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_DecoderCreate( 
  unsigned long                 aMemoryStart,
  unsigned long                 aMemorySize,
  TT_DecoderNewsSubttFound_T    newsSubttFoundCB);


/****************************************************************************
 * FUNCTION: TT_DecoderDelete
 *
 * DESCRIPTION:
 *   Delete decoder instance and all depending modules.
 ****************************************************************************/

PUBLIC void
TT_DecoderDelete( void );


/****************************************************************************
 * FUNCTION: TT_DecoderGetVersion
 *
 * DESCRIPTION:
 *   Returns the decoder version string
 *
 * RETURN:
 *   Version string
 ****************************************************************************/

PUBLIC const char*
TT_DecoderGetVersion( void );


/****************************************************************************
 * FUNCTION: TT_DecoderProcess
 *
 * DESCRIPTION:
 *   This is the main function of the teletext decoder. It should be called
 *   within the decoder task or within the main loop. The function executes:
 *     1. acquisition packet processing
 *     2. processing of user inputs
 *     3. update of display
 *
 * RETURN:
 *   The number of packets processed by this function call.
 ****************************************************************************/

PUBLIC U_INT32
TT_DecoderProcess( void );


/****************************************************************************
 * FUNCTION: TT_DecoderDisplayProcess
 *
 * DESCRIPTION:
 *   Process the decoders display. This function can be called within a
 *   separate task.
 ****************************************************************************/

PUBLIC void 
TT_DecoderDisplayProcess( void );


/****************************************************************************
 * FUNCTION: TT_DecoderDisplayDrawPage
 *
 * DESCRIPTION:
 *   Draw a Teletext page starting with given row until end of the page.
 *
 * PARAMETERS:
 *   aMode     - mode of drawing
 *   aStartRow - start row for drawing
 ****************************************************************************/

PUBLIC void 
TT_DecoderDisplayDrawPage(
  U_INT8   aMode,
  U_INT8   aStartRow );


/****************************************************************************
 * FUNCTION: TT_DecoderDisplayDrawRow
 *
 * DESCRIPTION:
 *   Draw a teletext row
 *
 * PARAMETERS:
 *   aRow  - row number
 *   aMode - mode of drawing
 ****************************************************************************/

PUBLIC void 
TT_DecoderDisplayDrawRow(
  U_INT8         aRow,
  U_INT8         aMode );


/****************************************************************************
 * FUNCTION: TT_DecoderGetCurrPage
 *
 * DESCRIPTION:
 *   Get the current page number from the Teletext decoder.
 *
 * PARAMETERS:
 *   aPageNumber     - content of current page number
 ****************************************************************************/
PUBLIC void
TT_DecoderGetCurrPage( 
  U_INT16* aPageNumber);

/****************************************************************************
 * FUNCTION: TT_DecoderSetListPages
 *
 * DESCRIPTION:
 *   Set the list mode data to the Teletext decoder.
 *
 * PARAMETERS:
 *   aRedPage     - Content of red link
 *   aGreenPage   - Content of yellow link
 *   aYellowPage  - Content of green link
 *   aCyanPage    - Content of cyan link
 *   aCustom1Page - Content of custom link
 *   aCustom2Page - Content of custom link
 ****************************************************************************/

PUBLIC void
TT_DecoderSetListPages(
  U_INT16 aRedPage,
  U_INT16 aGreenPage,
  U_INT16 aYellowPage,
  U_INT16 aCyanPage,
  U_INT16 aCustom1Page,
  U_INT16 aCustom2Page );


/****************************************************************************
 * FUNCTION: TT_DecoderGetListPages
 *
 * DESCRIPTION:
 *   Returns the list mode data currently used by the Teletext decoder.
 *
 * PARAMETERS:
 *   aRedPage     - Content of red link
 *   aGreenPage   - Content of green link
 *   aYellowPage  - Content of yellow link
 *   aCyanPage    - Content of cyan link
 *   aCustom1Page - Content of custom link
 *   aCustom2Page - Content of custom link
 ****************************************************************************/

PUBLIC void
TT_DecoderGetListPages(
  U_INT16* aRedPage,
  U_INT16* aGreenPage,
  U_INT16* aYellowPage,
  U_INT16* aCyanPage,
  U_INT16* aCustom1Page,
  U_INT16* aCustom2Page );


/****************************************************************************
 * FUNCTION: TT_DecoderGetStatus
 *
 * DESCRIPTION:
 *   Returns the decoder status.
 *
 * PARAMETERS:
 *   aDecStatus - decoder status information
 *
 * SEE ALSO:
 *   TT_DEC_STATUS
 ****************************************************************************/

PUBLIC void
TT_DecoderGetStatus(
  TT_DEC_STATUS* aDecStatus );


/****************************************************************************
 * FUNCTION: TT_DecoderSetPageRequestBuffer
 *
 * DESCRIPTION:
 *   The function TT_DecoderSetPageRequestBuffer sets the internal address
 *   and size of the page request buffer (where requested pages are stored).
 *   Packets are copied as long as aPageRequestBufferSize is big enough. The
 *   page buffer format is 42 bytes for each packet:
 *     - 2 bytes packet address (already hamming encrypted)
 *     - 40 bytes data (with paritiy information)
 *
 *   A page is requested by sending the command TT_DEC_REQUEST_PAGE with
 *   parameter1 = page number and parameter2 = subpage number. To get
 *   information about the reception status call the function
 *   TT_DecoderGetStatus.
 *
 * PARAMETERS:
 *   aPageRequestBuffer -
 *     pointer to the page request buffer
 *
 *   aPageRequestBufferSize -
 *     size of the page request buffer
 ****************************************************************************/

PUBLIC void
TT_DecoderSetPageRequestBuffer(
  U_INT8*  aPageRequestBuffer,
  U_INT16  aPageRequestBufferSize );


/****************************************************************************
 * FUNCTION: TT_DecoderGetNumberOfPagesInDb
 *
 * DESCRIPTION:
 *   Returns the amount of pages stored inside the database. In case
 *   TT_DEC_STAT_FILE is defined a text file is generated. This file can be
 *   used e.g. in EXCEL for further statistics.
 *
 * PARAMETERS:
 *   aNumberOfPages      - number of pages inside database
 *   aNumberOfSubPages   - number of additional stored subpages
 *   aNumberOfTotalPages - nubmer of all pages stored inside database
 ****************************************************************************/

PUBLIC void
TT_DecoderGetNumberOfPagesInDb(
  U_INT32 * aNumberOfPages,
  U_INT32 * aNumberOfSubPages,
  U_INT32 * aNumberOfTotalPages );


/****************************************************************************
 * FUNCTION: TT_DecoderSetKeyCommand
 *
 * DESCRIPTION:
 *   Calls directly the command processor with given command. This function
 *   can be used within the decoder task calling TT_DecoderProcess only.
 *
 * PARAMETERS:
 *   aCommand - the user key to process
 *   aParam1  - optional parameter1
 *   aParam2  - optional parameter2
 ****************************************************************************/

PUBLIC void
TT_DecoderSetKeyCommand(
  U_INT16   aCommand,
  U_INT16   aParam1,
  U_INT16   aParam2 );


/****************************************************************************
 * FUNCTION: TT_DecoderSetTopRow23Decoding
 *
 * DESCRIPTION:
 *   This function enables or disabled the row 23 data decoding within the
 *   basic top table. The decoding ir related to the patent 'EP 0 625 298'.
 *   By default the row 23 decoding is disabled. This decoding takes effect
 *   if TOP is enabled only.
 *
 * PARAMETERS:
 *   aRow23Decoding - the decoding setting ( TRUE or FALSE )
 ****************************************************************************/

PUBLIC void
TT_DecoderSetTopRow23Decoding(
  U_INT8       aRow23Decoding );



#ifdef __cplusplus
  }
#endif

#endif /* TT_DECODER_H */

/* This Headerfile was generated with C2H.exe ( C2H ) */
/* (c) 2006 by TARA Systems GmbH Munich */
