/*
*****************************************************************************
**
** \file      ./subsystems/voc/src/voc.c
**
** \brief     Video Output Capture driver for mosaic views and screen capture.
**
** This module grabs decoded I-Frames and performs a software scaling to copy
** them into a target rectangle in OSD. It may be used to realise a "mosaic
** preview" feature, i.e. presenting and updating pictures from multiple
** services in parallel.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU MICROELECTRONICS
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Microelectronics Europe GmbH
**
*****************************************************************************
*/

/*
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - e715
** - e946
** - e923
**
*****************************************************************************
*/
/*lint -save -e715 -e946 -e923 */

#include <stdio.h>

#include <fapex/abst_types.h>
#include <fapex/abst_libvidec.h>
#include <fapex/abst_libvienc.h>

#include <osdhandler/osdhandler.h>
#include "voc.h"
#include <fatfs/api_f.h>


//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************

#define VOC_DEBUG

#define VOC_WRITE_CHUNK_SIZE  (90*1024)

/*!
*******************************************************************************
** \brief Bank offsets on ramif_v 1920 x 1080
*******************************************************************************
*/
uint32_t VIDEC_NEXTBANK_OFFSET_1920HD[22] = {
    0x0000,0x0340,0x0680,0x09C0,0x0D00,0x1040,0x1380,0x16C0,0x1A00,
    0x2000,0x2340,0x2680,0x29C0,0x2D00,0x3040,0x3380,0x36C0,0x3A00,
    0x4000,0x4340,0x4680,0x49C0
};
/*!
*******************************************************************************
** \brief Bank offsets on ramif_v 1440 x 1080
*******************************************************************************
*/
uint32_t VIDEC_NEXTBANK_OFFSET_1440HD[22] = {
    0x0000,0x0270,0x04E0,0x0750,0x09C0,0x0C30,0x0EA0,0x1110,0x1380,
    0x15F0,0x1860,0x1AD0,0x1D40,0x2000,0x2270,0x24E0,0x2750,0x29C0,
    0x2C30,0x2EA0,0x3110,0x3380
};
/*!
*******************************************************************************
** \brief Bank offsets on ramif_v 1280 x 720
*******************************************************************************
*/
uint32_t VIDEC_NEXTBANK_OFFSET_1280HD[22] = {
    0x0000,0x0168,0x02D0,0x0438,0x05A0,0x0708,0x0870,0x09D8,0x0B40,
    0x0CA8,0x0E10,0x0F78,0x10E0,0x1248,0x13B0,0x1518,0x1680,0x17E8,
    0x1950,0x1AB8,0x1C20,0x1D88
};
/*!
*******************************************************************************
** \brief Bank offsets on ramif_v SD
*******************************************************************************
*/
uint32_t VIDEC_NEXTBANK_OFFSET_SD[22] = {
    0x0000,0x00A8,0x0150,0x01F8,0x02A0,0x0348,0x03F0,0x0498,0x0540,
    0x05E8,0x0690,0x0738,0x07E0,0x0888,0x0930,0x09D8,0x0A80,0x0B28,
    0x0BD0,0x0C78,0x0D20,0x0DC8
};

//***************************************************************************
//***************************************************************************
//** Local Structures
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************
typedef struct
{
    /*! docu */
    uint8_t*                frame_bank_map;
    uint8_t*                ccir656_map;
    uint32_t                frame_bank_size;
    uint32_t                ccir656_buf_size;
    FAPEX_SYS_SemaphoreT sem;
} VOC_DATA_S;

static VOC_DATA_S*          vocDat  = NULL;
static VOC_OpenParamsT*     vocOpenDat = NULL;
//***************************************************************************
//***************************************************************************
//** Local Functions Declaration
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Private Functions
//***************************************************************************
//***************************************************************************

#if 0
/*lint -save -e715 */
static void captureSync()
{
    FAPI_SYS_PRINT_MSG("[VOC] CCIR capture is synchronizing\n");
}
#endif

/*lint -restore */

//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************



/*!
*******************************************************************************
**
** \brief  Initialize the VOC library.
**
** \return One of the following status codes:
**       - \c #FAPI_OK if successful
**       - \c #VOC_ERR_ALREADY_INITIALIZED if the driver is already initialized
**       - \c #VOC_ERR_OUT_OF_MEMORY if the system runs out of memory
**
** \sa VOC_Exit
******************************************************************************/
int32_t VOC_Init (void)
{
    if(vocDat != NULL) return VOC_ERR_ALREADY_INITIALIZED;


    vocDat = FAPI_SYS_MALLOC_CACHED(sizeof(VOC_DATA_S));
    if(vocDat == NULL) return VOC_ERR_OUT_OF_MEMORY;

    memset(vocDat,0,sizeof(VOC_DATA_S));

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Exit the VOC library.
**
** \return One of the following status codes:
**         - \c #FAPI_OK if successful
**         - \c #VOC_ERR_NOT_INITIALIZED if driver is not initialized
**
** \sa VOC_Init
******************************************************************************/
int32_t VOC_Exit (void)
{
    if(vocDat == NULL)
    {
        return VOC_ERR_NOT_INITIALIZED;
    }
    else
    {
        if(vocOpenDat != NULL)
        {
            (void)VOC_Close((FAPI_SYS_HandleT) vocOpenDat);
            vocOpenDat = NULL;
        }
        FAPI_SYS_FREE(vocDat);
        vocDat = NULL;

        return FAPI_OK;
    }
}

/*!
*******************************************************************************
**
** \brief  Open VOC instance.
**
** This function creates an instance with a given function mode in openParams.
** Note that the driver supports only ONE instance at one time.
** VOC_CaptureScreen() can not work with  VOC_CopyIFrame() or VOC_CopyIFrameCCIR()
** at the same time.
**
** \param   openParamsPtr   Pointer to open parameters.
** \param   handleP         Pointer to a valid driver instance handle on success.
**
** \return One of the following status codes:
**         -  #FAPI_OK if successful
**         -  #VOC_ERR_ALREADY_INITIALIZED  if the DRAW driver has already been
**          initialized
**
** \sa  VOC_Close
******************************************************************************/
int32_t VOC_Open (VOC_OpenParamsT* openParamsPtr, FAPI_SYS_HandleT* handleP)
{
    int32_t      error_code = FAPI_OK;

    if(openParamsPtr == NULL)
    {
        return(VOC_ERR_BAD_PARAMETER);
    }
    
    if (vocDat->sem == NULL) {
        if ((vocDat->sem = FAPEX_SYS_CreateSemaphore(1)) == NULL) {
            return VOC_ERR_SEMAPHORE_CREATE;
        }
    }
    if (FAPEX_SYS_GetSemaphore(vocDat->sem, FAPEX_SYS_SUSPEND) != 0) {
        return VOC_ERR_TIMEOUT;
    }

    vocOpenDat = FAPI_SYS_MALLOC_CACHED(sizeof(VOC_OpenParamsT));
    if ( vocOpenDat == NULL )
    {
        error_code = VOC_ERR_OUT_OF_MEMORY;
        goto SET_SEM;
    }
    *vocOpenDat = *openParamsPtr;
    *handleP = (FAPI_SYS_HandleT) vocOpenDat;

    error_code = FAPEX_VIDEC_Ioctl( vocOpenDat->videcId, FAPEX_IOCCOM_VIDEC_GET_FRAME_BANK_SIZE,
                                     (FAPEXIOCARG3_T)&(vocDat->frame_bank_size) );
    vocDat->frame_bank_map = (uint8_t*)FAPEX_VIDEC_Mmap( NULL, vocDat->frame_bank_size,
                                                        (PROT_READ | PROT_WRITE), MAP_SHARED,
                                                        vocOpenDat->videcId, 0 );
    if ( vocDat->frame_bank_map == (void*)-1 )
    {
        FAPI_SYS_PRINT_MSG("[VOC]VOC_Open: video decoder buffer mmap failed frame_bank_size=0x%0x\n", vocDat->frame_bank_size);
        error_code = -1;
        goto SET_SEM;
    }
    if( vocOpenDat->functionMode == VOC_MODE_SCREENCAPTURE )
    {
        error_code = FAPEX_VIENC_Ioctl( vocOpenDat->viencId, FAPEX_IOCCOM_VIENC_SET_CCIR656DMA, (FAPEXIOCARG3_T)NULL );
        if ( error_code != FAPEX_OK )
        {
            FAPI_SYS_PRINT_MSG("[VOC]VOC_Open: set ccir656 dam buffer failed\n");
            goto SET_SEM;
        }
        vocDat->ccir656_buf_size = 720*580*2;
        vocDat->ccir656_map = (uint8_t*)FAPEX_VIENC_Mmap( NULL, vocDat->ccir656_buf_size,
                                                          (PROT_READ | PROT_WRITE), MAP_SHARED,
                                                          vocOpenDat->viencId, 0 );
        if ( vocDat->ccir656_map == (void*)-1 )
        {
            FAPI_SYS_PRINT_MSG("[VOC]VOC_Open: ccir656 dma buffer mmap failed \n");
            error_code = -1;
        }
    }

SET_SEM:
    FAPEX_SYS_SetSemaphore(vocDat->sem, FAPEX_SYS_SUSPEND);
    
    return error_code;
}
/*!
*******************************************************************************
**
** \brief  Close VOC instance.
**
** This function destroy the instance opened last time.
**
** \param handleP  Pointer to handle, will be set to NULL on success.
**
** \return One of the following status codes:
**         - \c #FAPI_OK if successful
**         - \c #VOC_ERR_BAD_PARAMETER if close with wrong handle
**
** \sa  VOC_Open
******************************************************************************/
int32_t VOC_Close (FAPI_SYS_HandleT handleP)
{
    int32_t      error_code = FAPI_OK;

    FAPI_SYS_PRINT_MSG("[VOC]VOC_Close come \n");

    if (vocDat->sem != NULL) {
        FAPEX_SYS_GetSemaphore(vocDat->sem, FAPEX_SYS_SUSPEND);
    }
    
    if (( handleP == NULL ) || (handleP != (FAPI_SYS_HandleT) vocOpenDat))
    {
        error_code = VOC_ERR_BAD_PARAMETER;
        goto SET_SEM;
    }

    FAPEX_VIDEC_Munmap(vocDat->frame_bank_map, vocDat->frame_bank_size);
    vocDat->frame_bank_size = 0;
    vocDat->frame_bank_map  = NULL;

    if( vocOpenDat->functionMode == VOC_MODE_SCREENCAPTURE )
    {
        FAPEX_VIENC_Munmap(vocDat->ccir656_map, vocDat->ccir656_buf_size);
        vocDat->ccir656_buf_size = 0;
        vocDat->ccir656_map      = NULL;

        error_code = FAPEX_VIENC_Ioctl( vocOpenDat->viencId, FAPEX_IOCCOM_VIENC_FREE_CCIR656DMA, (FAPEXIOCARG3_T)NULL );
        if ( error_code != FAPEX_OK )
        {
            FAPI_SYS_PRINT_MSG("[VOC]VOC_Close: free ccir656 dam buffer failed\n");
            goto SET_SEM;
        }
    }

    FAPI_SYS_FREE(vocOpenDat);

    vocOpenDat              = NULL;

    FAPI_SYS_PRINT_MSG("[VOC]VOC_Close live \n");

SET_SEM:
    if (vocDat->sem != NULL) {
        FAPEX_SYS_SetSemaphore(vocDat->sem, FAPEX_SYS_SUSPEND);
        FAPEX_SYS_DestroySemaphore(vocDat->sem);
        vocDat->sem = NULL;
    }
    
    return error_code;
}


/*!
*******************************************************************************
**
** \brief  Scales and copies the video data to OSD from CCIR output buffer.
**
** \param hh            VOC instance handle.
** \param winPos        position of a preview winodw on OSD
** \param layerParams   parameters of designated OSD
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**         - #VOC_ERR_NOT_INITIALIZED  if handle is NULL
**         - #VOC_ERR_INVALID_HANDLE   if handle does not match to the opened
**                                       instance
** \sa   FAPI_DMA_StartCcir656Capture
******************************************************************************/
/*lint -save -e923 */
#if 0
int32_t VOC_CopyIFrameCCIR (FAPI_SYS_HandleT hh, VOC_PictureWindowT* winPos,
                             OSDHANDLER_LayerParamsT* layerParams)
{
    volatile uint32_t               osdBufferAddr;
    FAPI_DMA_Ccir656CaptureParamsT  captureParams;
    uint32_t                        req;
    VOC_OpenParamsT*                inst = (VOC_OpenParamsT*)hh;

    if(inst == NULL ) return VOC_ERR_NOT_INITIALIZED;
    if ((inst != vocOpenDat) ||(inst->functionMode !=VOC_MODE_MOSAIC_CCIR))
        return VOC_ERR_INVALID_HANDLE;
    osdBufferAddr = layerParams->pictureBase;

    winPos->left /= 4;
    winPos->left *= 4;
    winPos->hsize /= 4;
    winPos->hsize *= 4;

    captureParams.useRawMode  = 0;
    captureParams.dataSource  = 0;
    captureParams.invertClock = 0;
    captureParams.useField0   = 1;
    captureParams.useField1   = 0;
    captureParams.absColumnStart = 140 + winPos->left;
    captureParams.absColumnEnd =
                         (captureParams.absColumnStart + winPos->hsize) - 2;
    captureParams.absLineStart = 24 + (winPos->top/2);
    captureParams.absLineEnd =
                         (captureParams.absLineStart + (winPos->vsize/2)) - 2;
    captureParams.enableAutoRestart = 0;


    osdBufferAddr += (winPos->top + 1) * layerParams->xRes * 2;
    osdBufferAddr += (winPos->left + 2) * 2;

    vocDat->captureFinishCounter = 1;

    req = (uint32_t)FAPI_DMA_StartCcir656Capture(
                (volatile uint32_t*)osdBufferAddr,             //targetBuffer
                layerParams->xRes * layerParams->yRes * 2,     //bufferBytes
                ((2 * layerParams->xRes) - winPos->hsize) * 2, //bufferLineInc
                &captureParams,                                //captureParams
                (FAPI_DMA_OptCallbackT) captureSync,           //syncCallback
                captureFinish);                                //finishCallback
    if (req != 0)
    {
        while (vocDat->captureFinishCounter > 0){}
    }


    osdBufferAddr += layerParams->xRes * 2;

    vocDat->captureFinishCounter = 1;
    req = (uint32_t)FAPI_DMA_StartCcir656Capture((uint32_t*)osdBufferAddr,
                                  layerParams->xRes * layerParams->yRes * 2,
                                  ((2 * layerParams->xRes) - winPos->hsize) * 2,
                                  &captureParams,
                                  (FAPI_DMA_OptCallbackT) captureSync,
                                  captureFinish);
    if (req != 0)
    {
        while (vocDat->captureFinishCounter > 0){}
    }

    #ifdef VOC_DEBUG
    FAPI_SYS_PRINT_MSG("[VOC] DMA capture done!\n");
    #endif

    return FAPI_OK;
}
#endif
/*lint -restore */
/*!
*******************************************************************************
**
** \brief  Scales and copies the video image to OSD from frame buffer.
**
** This function copies the video image from video memory and performs format
**  converting and scaling in software and outputs the processed image to a
**  given window in an OSD layer in 16-bit CbYCrA format.
**
** \param hh               handle given by VOC_Open.
** \param winPos           position of output window
** \param videcInfo        video decoding information, including: frameBank
**                            decodedHeight, displayWidth, and
**                            decodeFrameBankSizeType
** \param osdlayerParams   parameters of designated OSD
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**         - #VOC_ERR_NOT_INITIALIZED  if handle is NULL
**         - #VOC_ERR_INVALID_HANDLE   if handle does not match to the opened
**                                       instance
******************************************************************************/
int32_t VOC_CopyIFrame (FAPI_SYS_HandleT hh, VOC_PictureWindowT* winPos,
                        VOC_VidecInfoT* videcInfo,
                        OSDHANDLER_LayerParamsT* osdlayerParams )
{
    volatile uint64_t*          YBufferPtr;
    volatile uint64_t*          YBlockPtr;
    volatile uint64_t*          CBufferPtr;
    volatile uint64_t*          CBlockPtr;
    volatile uint32_t*          outputPtr;
    uint32_t                    YBufferAddr;
    uint32_t                    CBufferAddr;
    uint32_t                    sectionSize;
    uint32_t                    arraySize;
    uint32_t                    tgtWidth, srcWidth, tgtHeight, srcHeight;
    uint32_t                    hIntPart, hFractPart, vIntPart, vFractPart;
    uint64_t                    YData, CData;
    uint64_t                    byte_mask   = 0x00000000000000FF;
    uint32_t                    tmp0, tmp1, tmp32;
    uint32_t                    i, j = 0, k = 0;
    uint32_t*                   line_offset_array;
    uint32_t                    line_offset = 0;
    uint8_t                     line_num    = 0;           //for each 32 lines
    uint32_t                    line_count  = 0;
    uint8_t                     block_num   = 0;           //for each 16 blocks
    uint8_t                     byte_num    = 0;           //for each 4 pixels
    uint32_t                    byte_offset0= 0, byte_offset= 0;
    uint32_t                    hE0 = 0, hE = 0, vE = 0;
    uint32_t       block_offset_array0[16] = {0,     4*32,     8*32,     12*32,
                                              1, (4*32)+1, (8*32)+1, (12*32)+1,
                                              2, (4*32)+2, (8*32)+2, (12*32)+2,
                                              3, (4*32)+3, (8*32)+3, (12*32)+3};
    uint32_t       block_offset_array1[16] = {    8*32,     12*32, 0,     4*32,
                                              (8*32)+1, (12*32)+1, 1, (4*32)+1,
                                              (8*32)+2, (12*32)+2, 2, (4*32)+2,
                                              (8*32)+3, (12*32)+3, 3, (4*32)+3};
    VOC_OpenParamsT*            inst = (VOC_OpenParamsT*)hh;
    int32_t error_code = FAPEX_OK;

    if(inst == NULL ) return VOC_ERR_NOT_INITIALIZED;

    if (FAPEX_SYS_GetSemaphore(vocDat->sem, FAPEX_SYS_SUSPEND) != 0) {
        return VOC_ERR_TIMEOUT;
    }
    
    if ((inst != vocOpenDat) ||(inst->functionMode !=VOC_MODE_MOSAIC)) {
        error_code = VOC_ERR_INVALID_HANDLE;
        goto SET_SEM;
    }

    YBufferAddr = (uint32_t)vocDat->frame_bank_map + videcInfo->offset_y;
    CBufferAddr = (uint32_t)vocDat->frame_bank_map + videcInfo->offset_c;

    FAPI_SYS_PRINT_MSG("[VOC]VOC_CopyIFrame:YBufferAddr=0x%0x, CBufferAddr=0x%0x, frameBank = %d\n",YBufferAddr,CBufferAddr,videcInfo->frameBank);
    FAPI_SYS_PRINT_MSG("[VOC]VOC_CopyIFrame:videcInfo->frameBank=%d, videcInfo->offset_y=0x%0x, videcInfo->offset_c=0x%0x \n",videcInfo->frameBank,videcInfo->offset_y, videcInfo->offset_c);
    FAPI_SYS_PRINT_MSG("[VOC]VOC_CopyIFrame:videcInfo->decodedHeight=%d,videcInfo->displayWidth=%d\n",videcInfo->decodedHeight,videcInfo->displayWidth);


    //adjust for 1920*1080 video frame
    if (videcInfo->decodedHeight == 1080)
    {
        videcInfo->decodedHeight = 1088;
    }

    //adjust for 320*180 video frame
    if (videcInfo->decodedHeight == 180)
    {
        videcInfo->decodedHeight = 192;
    }
    /*lint -save -e923 */
    YBufferPtr = (volatile uint64_t*)YBufferAddr;
    CBufferPtr = (volatile uint64_t*)CBufferAddr;
    outputPtr  = (volatile uint32_t*)(osdlayerParams->pictureBase)
                    + ((winPos->top*osdlayerParams->xRes)/2)
                    + (winPos->left/2);

    /*lint -restore */

    tgtWidth  = winPos->hsize/2;
    tgtHeight = winPos->vsize/2;

    srcWidth  = videcInfo->displayWidth/2;
    srcHeight = videcInfo->decodedHeight/2;

    hIntPart = srcWidth/tgtWidth;
    hFractPart = srcWidth%tgtWidth;
    vIntPart = srcHeight/tgtHeight;
    vFractPart = srcHeight%tgtHeight;

    if(vIntPart>0)
    {
        arraySize = 32;            //scale down
    }
    else
    {
        arraySize  = 32*((tgtHeight/srcHeight)+1);        //scale up
    }

    line_offset_array = FAPI_SYS_MALLOC_CACHED(4*arraySize);
    if(line_offset_array == NULL) {
        error_code = VOC_ERR_OUT_OF_MEMORY;
        goto SET_SEM;
    }
    
    memset(line_offset_array,0,4*arraySize);

    for(i=0; i<(videcInfo->decodedHeight>>6);i++)
    {
        //setup an array to hold the line offset for each 32 line
        k = 0;
        while(line_offset<32)
        {
            if(k>=arraySize)        break;

            line_offset_array[k] = line_offset;

            line_offset += vIntPart;
            vE += vFractPart;
            if(vE>=tgtHeight)
            {
                vE -= tgtHeight;
                line_offset++;
            }
            k++;
        }

        line_offset -= 32;
        line_num = (uint8_t)k;

        do /* while(j<(videcInfo->displayWidth>>3)) */
        {
            block_num = j%16;
            //copy LUMA and CHROMA pixel from each micro block
            for(k=0; k<line_num; k++)
            {
                byte_num = 0;

                //LUMA color pixels
                if ((i%2) == 1)
                {
                    YBlockPtr = YBufferPtr + (block_offset_array1[block_num] +
                                                    (line_offset_array[k]*4));
                }
                else
                {
                    YBlockPtr = YBufferPtr + (block_offset_array0[block_num] +
                                                    (line_offset_array[k]*4));
                }

                YData = (*YBlockPtr);
                byte_offset0 = byte_offset;
                hE0 = hE;

                while (byte_offset0<4)
                {
                    if (byte_offset0<2)
                    {
                        tmp0 = (uint32_t)
                                (YData&(byte_mask<<(byte_offset0*2*8)));
                        tmp1 = (uint32_t)
                                (YData&(byte_mask<<(((byte_offset0*2)+1)*8)));
                    }
                    else
                    {
                        tmp0 = (uint32_t)
                            ((YData&(byte_mask<<(byte_offset0*2*8)))>>32);
                        tmp1 = (uint32_t)
                            ((YData&(byte_mask<<(((byte_offset0*2)+1)*8)))>>32);
                    }
                    if(byte_offset0%2)
                    {
                        tmp32 = (tmp0)|(tmp1>>24);
                        *outputPtr = (*outputPtr&0xFF00FF00)|tmp32;
                        outputPtr += (osdlayerParams->xRes>>1);
                        *outputPtr = (*outputPtr&0xFF00FF00)|tmp32;
                        outputPtr -= (osdlayerParams->xRes>>1) - 1;
                    }
                    else
                    {
                        tmp32 = (tmp0<<16)|(tmp1>>8);
                        *outputPtr = (*outputPtr&0xFF00FF00)|tmp32;
                        outputPtr += (osdlayerParams->xRes>>1);
                        *outputPtr = (*outputPtr&0xFF00FF00)|tmp32;
                        outputPtr -= (osdlayerParams->xRes>>1) - 1;
                    }

                    byte_offset0 += hIntPart;
                    hE0 += hFractPart;

                    if(hE0>=tgtWidth)
                    {
                        hE0 -= tgtWidth;
                        byte_offset0++;
                    }
                    byte_num++;
                }

                //prepare for chroma pixels copying
                outputPtr -= byte_num;

                //Chroma color pixels
                if(((i>>1)%2)==1)
                {
                    CBlockPtr = CBufferPtr + (block_offset_array1[block_num] +
                                ((line_offset_array[k]/2)*4));
                }
                else
                {
                    CBlockPtr = CBufferPtr + (block_offset_array0[block_num] +
                                ((line_offset_array[k]/2)*4));
                }

                CData = (*CBlockPtr);
                byte_offset0 = byte_offset;
                hE0 = hE;

                while (byte_offset0<4)
                {
                    if (byte_offset0<2)
                    {
                        tmp0 = (uint32_t)
                                (CData&(byte_mask<<(byte_offset0*2*8)));
                        tmp1 = (uint32_t)
                                (CData&(byte_mask<<(((byte_offset0*2)+1)*8)));
                    }
                    else
                    {
                        tmp0 = (uint32_t)
                            ((CData&(byte_mask<<(byte_offset0*2*8)))>>32);
                        tmp1 = (uint32_t)
                            ((CData&(byte_mask<<(((byte_offset0*2)+1)*8)))>>32);
                    }

                    if(byte_offset0%2)
                    {
                        tmp32 = (tmp0<<8)|(tmp1>>16);
                        *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                        outputPtr += (osdlayerParams->xRes>>1);
                        *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                        outputPtr -= (osdlayerParams->xRes>>1) - 1;
                    }
                    else
                    {
                        tmp32 = (tmp0<<24)|(tmp1);
                        *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                        outputPtr += (osdlayerParams->xRes>>1);
                        *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                        outputPtr -= (osdlayerParams->xRes>>1) - 1;
                    }

                    byte_offset0 += hIntPart;
                    hE0 += hFractPart;
                    if(hE0>=tgtWidth)
                    {
                        hE0 -= tgtWidth;
                        byte_offset0++;
                    }
                }
                //increase two lines
                outputPtr += ((osdlayerParams->xRes>>1)*2) - byte_num;

            }
            //prepare for next micro block
            outputPtr -= (2*line_num*(osdlayerParams->xRes>>1))-byte_num;

            do
            {
                byte_offset0 -= 4;
                j++;
                block_num++;
            }while(byte_offset0>=4) ;

            byte_offset = byte_offset0;
            hE = hE0;

            if(block_num>=16)
            {
                YBufferPtr += 4*4*32;
                CBufferPtr += 4*4*32;
            }

        }
        while(j<(videcInfo->displayWidth>>3));

        j = 0;  block_num = 0;
        hE = 0; hE0 = 0;
        byte_offset = 0; byte_offset0 = 0;
        line_count += line_num;

        switch(videcInfo->decodeFrameBankSizeType)
        {
            case FAPEX_VIDEC_FRAME_BANK_SIZE_1920HD:
                sectionSize = 0x10000;
                break;
            case FAPEX_VIDEC_FRAME_BANK_SIZE_1440HD:
                sectionSize = 0xC000;
                break;
            case FAPEX_VIDEC_FRAME_BANK_SIZE_1280HD:
                sectionSize = 0xA000;
                break;
            case FAPEX_VIDEC_FRAME_BANK_SIZE_SD:
                sectionSize = 0x6000;
                break;
            case FAPEX_VIDEC_FRAME_BANK_SIZE_UNKNOWN:
            default:
                error_code = VOC_ERR_UNKNOWN_FRAMEBANK;
                goto SET_SEM;
        }

        /*lint -save -e923 */
        YBufferPtr =((volatile uint64_t*)(YBufferAddr+((i+1)*sectionSize)));
        outputPtr  = (volatile uint32_t*)(osdlayerParams->pictureBase)
                        + (osdlayerParams->xRes*line_count)
                        + ((winPos->top*osdlayerParams->xRes)/2)
                        + (winPos->left/2);

        if((i%2) == 1)
        {
            CBufferPtr =((volatile uint64_t*)
                                    (CBufferAddr+(((i/2)+1)*sectionSize)));
        }
        else
        {
            CBufferPtr =((volatile uint64_t*)
                                   ((CBufferAddr+((i/2)*sectionSize))+(16*32)));
        }
        /*lint -restore */
    }

    FAPI_SYS_FREE(line_offset_array);

    #ifdef VOC_DEBUG
    FAPI_SYS_PRINT_MSG("[VOC] line_count after scaling: %d\n", line_count);
    #endif

SET_SEM:
    FAPEX_SYS_SetSemaphore(vocDat->sem, FAPEX_SYS_SUSPEND);
    return error_code;
}

/*!
*******************************************************************************
**
** \brief  Copies and outputs the video image to a BMP file
**
**  By selection of capture mode, this function copies the video image from
**   1) video memory or 2) CCIR output and outputs the copied image to a
**   24-bit BMP file, and save the image to a given file path in a device.
**
** Note: a chunk write is provided in saving the image. This is to avoid data
**       blocking in accessing USB device.
**
** \param hh               pointer to VOC instance.
** \param filePath         output path and file name of the BMP file
** \param captureMode      capture mode. either #VOC_CAPTURE_VIDEO_ONLY or
**                         #VOC_CAPTURE_VIDEO_OSD
** \param sdSystem         SD system type to distinguish PAL and NTSC
** \param videcInfo        video decoding information, including: frameBank
**                              decodedHeight, displayWidth, and
**                              decodeFrameBankSizeType
** \param bmpWriteNotifyCb callback function. While file accessing is time
**                          comsuming, it notifies the application that the
**                          starts to write BMP file so that application has a
**                          chance to react to this hanging-like situation.
** \param arg              Argument for callback function.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**         - #VOC_ERR_NOT_INITIALIZED  if handle is NULL
**         - #VOC_ERR_INVALID_HANDLE   if handle does not match to the opened
**                                       instance
**         - #VOC_ERR_OUT_OF_MEMORY    if driver runs out of memery
** \sa   FAPI_DMA_StartCcir656Capture
******************************************************************************/
/*lint -save -e702 -e826 -e923 */
int32_t VOC_CaptureScreen(FAPI_SYS_HandleT hh, const char_t* filePath,
                          VOC_ScreenCaptureEnumT captureMode,
                          VOC_SDTV_STANDARD sdSystem,
                          VOC_VidecInfoT* videcInfo,
                          void (*bmpWriteNotifyCb)(void* arg), void* arg)
{
    //FAPI_DMA_Ccir656CaptureParamsT  captureParams;
    volatile uint64_t*              YtBufferPtr  = NULL;    //top field
    volatile uint64_t*              YbBufferPtr  = NULL;    //bottom field
    volatile uint64_t*              CtBufferPtr  = NULL;
    volatile uint64_t*              CbBufferPtr  = NULL;
    volatile uint32_t*              outputPtr;
    uint32_t                        FrameBankMap;
    uint32_t                        YtBufferAddr = 0;
    uint32_t                        YbBufferAddr = 0;
    uint32_t                        CtBufferAddr = 0;
    uint32_t                        CbBufferAddr = 0;
    uint64_t                        YData, CData;
    uint32_t                        sectionSize  = 0;
    uint32_t                        tmp0, tmp1, tmp2, tmp3, tmp32;
    uint8_t                         i = 0, j = 0, k = 0, m = 0, n = 0;
    uint8_t*                        yuvP         = NULL;
    uint8_t*                        pAllocYuvBuff = NULL;
    uint8_t*                        bmpdata      = NULL;
    int16_t                         Y1, Y2, U, V, R, G, B;
    uint32_t                        x, y;
    uint32_t                        bmpWidth = 0, bmpHeight = 0;
    uint32_t                        dmask = 0xFF;
    uint32_t                        fileSize;
    uint32_t                        dataSize;             //size of pixel data
    F_FILE*                         fileP        = NULL;
    uint32_t                        writeCnt     = 0;
    uint32_t                        writeFactor  = 0;
    uint8_t*                        chunkP       = NULL;
    uint32_t                        lineByte     = 0;
    uint8_t                         range;
    int32_t                         writeSize;
    int32_t                         retVal = FAPI_OK;
    uint8_t   header[54]= { /* File Header */
                'B', 'M',                 /* indicating BMP file */
                0x36, 0xFC, 0x12, 0x00,   /* file size */ //0x36, 0xFC, 0x12,
                0x00, 0x00, 0x00, 0x00,   /* reserved */
                0x36, 0x00, 0x00, 0x00,   /* offset to pixel data */

                /* info block */
                0x28, 0x00, 0x00, 0x00,   /* header size */
                0xD0, 0x02, 0x00, 0x00,   /* picture width (pixel) */ //D0
                0x40, 0x02, 0x00, 0x00,   /* picture height (pixel) */
                0x01, 0x00,               /* number of pixel layers */
                0x18, 0x00,               /* bits per pixel */
                0x00, 0x00, 0x00, 0x00,   /* compression */
                0x00, 0xFC, 0x12, 0x00,   /* size of pixel data */
                0x00, 0x00, 0x00, 0x00,   /* horizontal print resolution */
                0x00, 0x00, 0x00, 0x00,   /* vertical print resolution */
                0x00, 0x00, 0x00, 0x00,   /* number of used colors */
                0x00, 0x00, 0x00, 0x00    /* number of important colors */
                };
    VOC_OpenParamsT*                inst = (VOC_OpenParamsT*)hh;

    if (FAPEX_SYS_GetSemaphore(vocDat->sem, FAPEX_SYS_SUSPEND) != 0) {
        return VOC_ERR_TIMEOUT;
    }

    FrameBankMap = (uint32_t)vocDat->frame_bank_map;
    
    for(;;)
    {
        if(inst == NULL )
        {
            retVal = VOC_ERR_NOT_INITIALIZED;
            break;
        }
        if ((inst != vocOpenDat)||
            (inst->functionMode != VOC_MODE_SCREENCAPTURE))
        {
            retVal = VOC_ERR_INVALID_HANDLE;
            break;
        }

        //reserve memory for chunk write
        chunkP = FAPI_SYS_MALLOC_CACHED(VOC_WRITE_CHUNK_SIZE);
        if (chunkP == NULL)
        {
            retVal = VOC_ERR_OUT_OF_MEMORY;
            break;
        }

        /* 1. capture video according to the given capture mode*/
        if(captureMode == VOC_CAPTURE_VIDEO_OSD)
        {
            FAPI_SYS_PRINT_MSG("[VOC]VOC_CaptureScreen: sdSystem=%d\n",sdSystem);
             /* PAL/NTSC/SECAM mode*/
            if((sdSystem == VOC_SDSYS_PAL) || (sdSystem == VOC_SDSYS_NTSC))
            {
                retVal = FAPEX_VIENC_Ioctl( vocOpenDat->viencId,
                                            FAPEX_IOCCOM_VIENC_START_CCIR656DMA,
                                            (FAPEXIOCARG3_T)sdSystem);
                if (retVal != FAPEX_OK ) {
                    retVal = VOC_ERR_OUT_OF_MEMORY;
                    goto SET_SEM;
                }

                yuvP = vocDat->ccir656_map;
            }
            else
            {
                retVal = VOC_ERR_BAD_PARAMETER;
                goto SET_SEM;
            }
        }

        else if (captureMode == VOC_CAPTURE_VIDEO_ONLY)
        {

            /*adjust for 1280*720 video, since 720 is not a multiple of 64.
            Extra data will not be copied to BMP file*/
            if(videcInfo->decodedHeight == 720)
            {
                videcInfo->decodedHeight = 768;        //32*12*2
            }
            //adjust for 1920x1080 video, since 1080 is not a multiple of 64
            else if (videcInfo->decodedHeight == 1080)
            {
                videcInfo->decodedHeight = 1088;        //32*17*2
            }
            /* adjust 720x480 video, since 480 is not a multiple of 64 */
            else if (videcInfo->decodedHeight == 480)
            {
                videcInfo->decodedHeight = 512;         //32*8*2
            }

            /*allocate memory to store the video frame*/
            yuvP = FAPI_SYS_MALLOC_CACHED(
                                videcInfo->decodedWidth*videcInfo->decodedHeight*2);
            if(!yuvP)
            {
                FAPI_SYS_PRINT_MSG("[VOC] Not enough memory for yuvP!\n");
                retVal = VOC_ERR_OUT_OF_MEMORY;
                break;
            }
            outputPtr = (uint32_t*)yuvP;
            pAllocYuvBuff = yuvP;
            memset(yuvP, 0, videcInfo->decodedWidth*videcInfo->decodedHeight*2);

            #ifdef VOC_DEBUG
            FAPI_SYS_PRINT_MSG("[VOC] video frame width :%u, height:%u\n",
                                videcInfo->decodedWidth, videcInfo->decodedHeight);
            #endif /* VOC_DEBUG */

            //Allocation of framebank addresses
            switch(videcInfo->decodeFrameBankSizeType)
            {
                case FAPEX_VIDEC_FRAME_BANK_SIZE_1920HD:
                    YtBufferAddr = FrameBankMap +
                        ((VIDEC_NEXTBANK_OFFSET_1920HD[videcInfo->frameBank]) << 12);
                    YbBufferAddr = FrameBankMap +
                        ((VIDEC_NEXTBANK_OFFSET_1920HD[videcInfo->frameBank]  + \
                                    0x0110) << 12);
                    CtBufferAddr = FrameBankMap +
                        ((VIDEC_NEXTBANK_OFFSET_1920HD[videcInfo->frameBank] + \
                                    0x0230) << 12);
                    CbBufferAddr = FrameBankMap +
                        ((VIDEC_NEXTBANK_OFFSET_1920HD[videcInfo->frameBank] + \
                                    0x02B0) << 12);
                    sectionSize = 0x10000;
                    break;
                case FAPEX_VIDEC_FRAME_BANK_SIZE_1440HD:
                    YtBufferAddr = FrameBankMap +
                        ((VIDEC_NEXTBANK_OFFSET_1440HD[videcInfo->frameBank]) << 12);
                    YbBufferAddr = FrameBankMap +
                        ((VIDEC_NEXTBANK_OFFSET_1440HD[videcInfo->frameBank]  + \
                                    0x00CC) << 12);
                    CtBufferAddr = FrameBankMap +
                        ((VIDEC_NEXTBANK_OFFSET_1440HD[videcInfo->frameBank] + \
                                    0x01A4) << 12);
                    CbBufferAddr = FrameBankMap +
                        ((VIDEC_NEXTBANK_OFFSET_1440HD[videcInfo->frameBank] + \
                                    0x0204) << 12);
                    sectionSize = 0xC000;
                    break;
                case FAPEX_VIDEC_FRAME_BANK_SIZE_1280HD:
                    YtBufferAddr = FrameBankMap +
                       ((VIDEC_NEXTBANK_OFFSET_1280HD[videcInfo->frameBank]) << 12);
                    YbBufferAddr = FrameBankMap +
                       ((VIDEC_NEXTBANK_OFFSET_1280HD[videcInfo->frameBank]  + \
                                    0x006E) << 12);
                    CtBufferAddr = FrameBankMap +
                        ((VIDEC_NEXTBANK_OFFSET_1280HD[videcInfo->frameBank] + \
                                    0x00E6) << 12);
                    CbBufferAddr = FrameBankMap +
                        ((VIDEC_NEXTBANK_OFFSET_1280HD[videcInfo->frameBank] + \
                                    0x0122) << 12);
                    sectionSize = 0xA000;
                    break;
                case FAPEX_VIDEC_FRAME_BANK_SIZE_SD:
                    YtBufferAddr = FrameBankMap +
                        ((VIDEC_NEXTBANK_OFFSET_SD[videcInfo->frameBank]) << 12);
                    YbBufferAddr = FrameBankMap +
                        ((VIDEC_NEXTBANK_OFFSET_SD[videcInfo->frameBank] + \
                                    0x0036) << 12);
                    CtBufferAddr = FrameBankMap +
                        ((VIDEC_NEXTBANK_OFFSET_SD[videcInfo->frameBank] + \
                                    0x0072) << 12);
                    CbBufferAddr = FrameBankMap +
                        ((VIDEC_NEXTBANK_OFFSET_SD[videcInfo->frameBank] + \
                                    0x008A) << 12);
                    sectionSize = 0x6000;
                    break;
                case FAPEX_VIDEC_FRAME_BANK_SIZE_UNKNOWN:
                default:
                    retVal = VOC_ERR_UNKNOWN_FRAMEBANK;
                    break;
            }
            if(retVal != FAPI_OK)
            {
                break;
            }

            YtBufferPtr = (volatile uint64_t*)(YtBufferAddr);
            YbBufferPtr = (volatile uint64_t*)(YbBufferAddr);
            CtBufferPtr = (volatile uint64_t*)(CtBufferAddr);
            CbBufferPtr = (volatile uint64_t*)(CbBufferAddr);

            if ((YtBufferPtr == NULL)||(YbBufferPtr == NULL)||
                (CtBufferPtr == NULL)||(CbBufferPtr == NULL)||
                (sectionSize == 0))
            {
                retVal = VOC_ERR_BAD_PARAMETER;
                break;
            }

            /* Progressive mode - both top filed and bottom field are used*/
            if(videcInfo->proggInter == 1)
            {
                /*TOP field*/
                for(i=0; i<(videcInfo->decodedHeight>>6);i++)
                {
                    do
                    {
                        //LUMA color pixels
                        for (m=0;m<4;m++)
                        {
                            if (i&1)
                            {
                                if(m==0)        YtBufferPtr += 2*4*32;
                                else if(m==2)   YtBufferPtr -= 4*4*32;
                            }

                            for (k=0;k<32;k++)
                            {
                                YData = (*YtBufferPtr);

                                tmp0 = YData&0x00000000000000FFULL;
                                tmp1 = YData&0x000000000000FF00ULL;
                                tmp32 = (tmp0<<16)|(tmp1>>8);
                                *outputPtr = (*outputPtr&0xFF00FF00)|tmp32;
                                outputPtr++;

                                tmp0 = YData&0x0000000000FF0000ULL;
                                tmp1 = YData&0x00000000FF000000ULL;
                                tmp32 = (tmp0)|(tmp1>>24);
                                *outputPtr = (*outputPtr&0xFF00FF00)|tmp32;
                                outputPtr++;

                                tmp0 = (YData&0x000000FF00000000ULL)>>32;
                                tmp1 = (YData&0x0000FF0000000000ULL)>>32;
                                tmp32 = (tmp0<<16)|(tmp1>>8);
                                *outputPtr = (*outputPtr&0xFF00FF00)|tmp32;
                                outputPtr++;

                                tmp0 = (YData&0x00FF000000000000ULL)>>32;
                                tmp1 = (YData&0xFF00000000000000ULL)>>32;
                                tmp32 = (tmp0)|(tmp1>>24);
                                *outputPtr = (*outputPtr&0xFF00FF00)|tmp32;
                                outputPtr++;

                                YtBufferPtr+= 4;
                                outputPtr += (2*(videcInfo->decodedWidth>>1))-4;
                            }
                            outputPtr -= (64*(videcInfo->decodedWidth>>1))-4;

                            n++;
                            if(n>=(videcInfo->decodedWidth>>3)) break;
                        }
                        if(!(n%4))
                            outputPtr -= 16; //prepare for chroma pixels copying
                        else
                            outputPtr -= 4*(n%4);

                        //Chroma color pixels
                        for (m=0;m<4;m++)
                        {
                            if (((i>>1)%2)==1)
                            {
                                if(m==0)        CtBufferPtr += 2*4*32;
                                else if(m==2)   CtBufferPtr -= 4*4*32;
                            }

                            for (k=0;k<16;k++)
                            {
                                CData = (*CtBufferPtr);

                                tmp2 = CData&0x00000000000000FFULL;
                                tmp3 = CData&0x000000000000FF00ULL;
                                tmp32 = (tmp2<<24)|(tmp3);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += 2*(videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr -= ((videcInfo->decodedWidth>>1)*2) - 1;

                                tmp2 = CData&0x0000000000FF0000ULL;
                                tmp3 = CData&0x00000000FF000000ULL;
                                tmp32 = (tmp2<<8)|(tmp3>>16);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += 2*(videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr -= ((videcInfo->decodedWidth>>1)*2) - 1;

                                tmp2 = (CData&0x000000FF00000000ULL)>>32;
                                tmp3 = (CData&0x0000FF0000000000ULL)>>32;
                                tmp32 = (tmp2<<24)|(tmp3);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += 2*(videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr -= ((videcInfo->decodedWidth>>1)*2) - 1;

                                tmp2 = (CData&0x00FF000000000000ULL)>>32;
                                tmp3 = (CData&0xFF00000000000000ULL)>>32;
                                tmp32 = (tmp2<<8)|(tmp3>>16);
                                *outputPtr  = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += 2*(videcInfo->decodedWidth>>1);
                                *outputPtr  = (*outputPtr&0x00FF00FF)|tmp32;

                                CtBufferPtr+= 4;
                                outputPtr += (2*(videcInfo->decodedWidth>>1))-3;

                            }

                            outputPtr -= (64*(videcInfo->decodedWidth>>1))-4;
                            CtBufferPtr += 4*16;  //32 line of increment

                            j++;
                            if(j>=(videcInfo->decodedWidth>>3)) break;

                        }

                        if((j%16)==0)
                        {
                            if (((i>>1)%2)==1)    CtBufferPtr+= (2*4*32)-3;
                            else                  CtBufferPtr-= 3;

                            if ((i%2) == 1)       YtBufferPtr+= (2*4*32)-3;
                            else                  YtBufferPtr-= 3;
                        }
                        else
                        {
                            if (((i>>1)%2)==1)    CtBufferPtr-= (2*4*32) - 1 ;
                            else                  CtBufferPtr-= (4*4*32) - 1 ;

                            if ((i%2) == 1)       YtBufferPtr-= (2*4*32) - 1 ;
                            else                  YtBufferPtr-= (4*4*32) - 1 ;
                        }
                    }
                    while(j<(videcInfo->decodedWidth>>3));

                    j=0; n=0;

                    YtBufferPtr =((volatile uint64_t*)(YtBufferAddr+((i+1)*sectionSize)));
                    outputPtr  = (volatile uint32_t*)(yuvP +
                                                (videcInfo->decodedWidth*2*32*2*(i+1)));

                    if(i & 1)
                        CtBufferPtr =((volatile uint64_t*)
                                                (CtBufferAddr+(((i/2)+1)*sectionSize)));
                    else
                        CtBufferPtr =((volatile uint64_t*)
                                        ((CtBufferAddr+((i/2)*sectionSize)) + (16*32)));
                }

                /* increase 1 line to add bottom field */
                outputPtr = (volatile uint32_t*)(yuvP + (videcInfo->decodedWidth*2));

                /*BOTTOM field*/
                /* LUMA color pixels */
                for(i=0; i<(videcInfo->decodedHeight>>6);i++)
                {
                    do
                    {
                        for (m=0;m<4;m++)
                        {
                            if (i&1)    //3012
                            {
                                if(m==0)    YbBufferPtr += 1*4*32;
                                if(m==3)    YbBufferPtr -= 4*4*32;
                            }
                            else        //1230
                            {
                                if(m==0)    YbBufferPtr += 3*4*32;
                                if(m==1)    YbBufferPtr -= 4*4*32;
                            }

                            //skip first 16 lines
                            if(i == 0)
                            {
                                range = 16;
                                YbBufferPtr += 4*16;
                            }
                            else
                            {
                                range = 0;
                            }

                            for (k=range;k<32;k++)
                            {
                                YData = (*YbBufferPtr);

                                tmp0 = YData&0x00000000000000FFULL;
                                tmp1 = YData&0x000000000000FF00ULL;
                                tmp32 = (tmp0<<16)|(tmp1>>8);
                                *outputPtr = (*outputPtr&0xFF00FF00)|tmp32;
                                outputPtr++;

                                tmp0 = YData&0x0000000000FF0000ULL;
                                tmp1 = YData&0x00000000FF000000ULL;
                                tmp32 = (tmp0)|(tmp1>>24);
                                *outputPtr = (*outputPtr&0xFF00FF00)|tmp32;
                                outputPtr++;

                                tmp0 = (YData&0x000000FF00000000ULL)>>32;
                                tmp1 = (YData&0x0000FF0000000000ULL)>>32;
                                tmp32 = (tmp0<<16)|(tmp1>>8);
                                *outputPtr = (*outputPtr&0xFF00FF00)|tmp32;
                                outputPtr++;

                                tmp0 = (YData&0x00FF000000000000ULL)>>32;
                                tmp1 = (YData&0xFF00000000000000ULL)>>32;
                                tmp32 = (tmp0)|(tmp1>>24);
                                *outputPtr  = (*outputPtr&0xFF00FF00)|tmp32;
                                outputPtr++;

                                YbBufferPtr+= 4;
                                outputPtr += (2*(videcInfo->decodedWidth>>1))-4;

                            }
                            if(i==0)
                            {
                                outputPtr -= (32*(videcInfo->decodedWidth>>1))-4;
                            }
                            else
                            {
                                outputPtr -= (64*(videcInfo->decodedWidth>>1))-4;
                            }

                            j++;
                            if(j>=(videcInfo->decodedWidth>>3)) break;
                        }

                        if (i&1)
                        {
                            YbBufferPtr += 3*4*32;
                        }
                        else
                        {
                            YbBufferPtr += 1*4*32;
                        }
                        if((j%16)==0)
                        {
                            YbBufferPtr-= 3;
                        }
                        else
                        {
                            YbBufferPtr-= (4*4*32) - 1 ;
                        }
                    }
                    while(j<(videcInfo->decodedWidth>>3));

                    j=0;
                    YbBufferPtr =((volatile uint64_t*)(YbBufferAddr+((i+1)*sectionSize)));

                    if(i==0)
                    {
                        outputPtr  = (volatile uint32_t*)(yuvP + (videcInfo->decodedWidth*2*16*2) +
                                     (videcInfo->decodedWidth*2));
                    }
                    else
                    {
                        outputPtr  = (volatile uint32_t*)(yuvP + (videcInfo->decodedWidth*2*16*2) +
                                     (videcInfo->decodedWidth*2*32*2*i) + (videcInfo->decodedWidth*2));
                    }

                }

                /* Chroma color pixels */
                //j = 0;
                outputPtr = (volatile uint32_t*)(yuvP + (videcInfo->decodedWidth*2));
                CbBufferPtr += 4*16;        //skip first 16 lines

                for(i=1; i<=(videcInfo->decodedHeight>>6);i++)
                {
                    do
                    {
                        for (m=0;m<4;m++)
                        {
                            if (((i>>1)%2)==1)
                            {
                                if(m==0)   CbBufferPtr += 1*4*32;
                                if(m==3)   CbBufferPtr -= 4*4*32;
                            }
                            else
                            {
                                if(m==0)   CbBufferPtr += 3*4*32;
                                if(m==1)   CbBufferPtr -= 4*4*32;
                            }

                            for (k=0;k<16;k++)
                            {
                                CData = (*CbBufferPtr);

                                tmp2 = CData&0x00000000000000FFULL;
                                tmp3 = CData&0x000000000000FF00ULL;
                                tmp32 = (tmp2<<24)|(tmp3);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += 2*(videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr -= ((videcInfo->decodedWidth>>1)*2) - 1;

                                tmp2 = CData&0x0000000000FF0000ULL;
                                tmp3 = CData&0x00000000FF000000ULL;
                                tmp32 = (tmp2<<8)|(tmp3>>16);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += 2*(videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr -= ((videcInfo->decodedWidth>>1)*2) - 1;

                                tmp2 = (CData&0x000000FF00000000ULL)>>32;
                                tmp3 = (CData&0x0000FF0000000000ULL)>>32;
                                tmp32 = (tmp2<<24)|(tmp3);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += 2*(videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr -= ((videcInfo->decodedWidth>>1)*2) - 1;

                                tmp2 = (CData&0x00FF000000000000ULL)>>32;
                                tmp3 = (CData&0xFF00000000000000ULL)>>32;
                                tmp32 = (tmp2<<8)|(tmp3>>16);
                                *outputPtr  = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += 2*(videcInfo->decodedWidth>>1);
                                *outputPtr  = (*outputPtr&0x00FF00FF)|tmp32;

                                CbBufferPtr+= 4;
                                outputPtr += (2*(videcInfo->decodedWidth>>1))-3;
                            }

                            CbBufferPtr += 4*16;  //32 line of increment
                            outputPtr   -= (64*(videcInfo->decodedWidth>>1))-4;

                            j++;
                            if(j>=(videcInfo->decodedWidth>>3)) break;
                        }

                        if (((i>>1)%2)==1)
                        {
                            CbBufferPtr += 3*4*32;
                        }
                        else
                        {
                            CbBufferPtr += 1*4*32;
                        }
                        if((j%16)==0)
                        {
                            CbBufferPtr-= 3;
                        }
                        else
                        {
                            CbBufferPtr-= (4*4*32) - 1 ;
                        }

                    }while(j<(videcInfo->decodedWidth>>3));

                    j=0;
                    outputPtr  = (volatile uint32_t*)(yuvP + (videcInfo->decodedWidth*2*32*2*i) +
                                 (videcInfo->decodedWidth*2));

                    if(i & 1)
                        CbBufferPtr =((volatile uint64_t*)
                                                (CbBufferAddr+(((i/2)+1)*sectionSize)));
                    else
                        CbBufferPtr =((volatile uint64_t*)
                                                ((CbBufferAddr+((i/2)*sectionSize)) + (16*32)));
                }
            }
            /* interlace mode - ONLY top field is used */
            else
            {
                for(i=0; i<(videcInfo->decodedHeight>>6);i++)
                {
                    do
                    {
                        //LUMA color pixels
                        for (m=0;m<4;m++)
                        {
                            if (i&1)
                            {
                                if(m==0)        YtBufferPtr += 2*4*32;
                                else if(m==2)   YtBufferPtr -= 4*4*32;
                            }

                            for (k=0;k<32;k++)
                            {
                                YData = (*YtBufferPtr);

                                tmp0 = YData&0x00000000000000FFULL;
                                tmp1 = YData&0x000000000000FF00ULL;
                                tmp32 = (tmp0<<16)|(tmp1>>8);
                                *outputPtr = (*outputPtr&0xFF00FF00)|tmp32;
                                outputPtr += (videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0xFF00FF00)|tmp32;
                                outputPtr -= (videcInfo->decodedWidth>>1) - 1;

                                tmp0 = YData&0x0000000000FF0000ULL;
                                tmp1 = YData&0x00000000FF000000ULL;
                                tmp32 = (tmp0)|(tmp1>>24);
                                *outputPtr = (*outputPtr&0xFF00FF00)|tmp32;
                                outputPtr += (videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0xFF00FF00)|tmp32;
                                outputPtr -= (videcInfo->decodedWidth>>1) - 1;

                                tmp0 = (YData&0x000000FF00000000ULL)>>32;
                                tmp1 = (YData&0x0000FF0000000000ULL)>>32;
                                tmp32 = (tmp0<<16)|(tmp1>>8);
                                *outputPtr = (*outputPtr&0xFF00FF00)|tmp32;
                                outputPtr += (videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0xFF00FF00)|tmp32;
                                outputPtr -= (videcInfo->decodedWidth>>1) - 1;

                                tmp0 = (YData&0x00FF000000000000ULL)>>32;
                                tmp1 = (YData&0xFF00000000000000ULL)>>32;
                                tmp32 = (tmp0)|(tmp1>>24);
                                *outputPtr  = (*outputPtr&0xFF00FF00)|tmp32;
                                outputPtr += (videcInfo->decodedWidth>>1);
                                *outputPtr  = (*outputPtr&0xFF00FF00)|tmp32;

                                YtBufferPtr+= 4;
                                outputPtr += (videcInfo->decodedWidth>>1)-3;
                            }
                            outputPtr -= (64*(videcInfo->decodedWidth>>1))-4;

                            n++;
                            if(n>=(videcInfo->decodedWidth>>3)) break;
                        }
                        if(!(n%4))
                            outputPtr -= 16; //prepare for chroma pixels copying
                        else
                            outputPtr -= 4*(n%4);

                        //Chroma color pixels
                        for (m=0;m<4;m++)
                        {
                            if (((i>>1)%2)==1)
                            {
                                if(m==0)        CtBufferPtr += 2*4*32;
                                else if(m==2)   CtBufferPtr -= 4*4*32;
                            }

                            for (k=0;k<16;k++)
                            {
                                CData = (*CtBufferPtr);

                                tmp2 = CData&0x00000000000000FFULL;
                                tmp3 = CData&0x000000000000FF00ULL;
                                tmp32 = (tmp2<<24)|(tmp3);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += (videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += (videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += (videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr -= ((videcInfo->decodedWidth>>1)*3) - 1;

                                tmp2 = CData&0x0000000000FF0000ULL;
                                tmp3 = CData&0x00000000FF000000ULL;
                                tmp32 = (tmp2<<8)|(tmp3>>16);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += (videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += (videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += (videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr -= ((videcInfo->decodedWidth>>1)*3) - 1;

                                tmp2 = (CData&0x000000FF00000000ULL)>>32;
                                tmp3 = (CData&0x0000FF0000000000ULL)>>32;
                                tmp32 = (tmp2<<24)|(tmp3);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += (videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += (videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += (videcInfo->decodedWidth>>1);
                                *outputPtr = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr -= ((videcInfo->decodedWidth>>1)*3) - 1;

                                tmp2 = (CData&0x00FF000000000000ULL)>>32;
                                tmp3 = (CData&0xFF00000000000000ULL)>>32;
                                tmp32 = (tmp2<<8)|(tmp3>>16);
                                *outputPtr  = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += (videcInfo->decodedWidth>>1);
                                *outputPtr  = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += (videcInfo->decodedWidth>>1);
                                *outputPtr  = (*outputPtr&0x00FF00FF)|tmp32;
                                outputPtr += (videcInfo->decodedWidth>>1);
                                *outputPtr  = (*outputPtr&0x00FF00FF)|tmp32;

                                CtBufferPtr+= 4;
                                outputPtr += (videcInfo->decodedWidth>>1)-3;
                            }

                            outputPtr -= (64*(videcInfo->decodedWidth>>1))-4;
                            CtBufferPtr += 4*16;  //32 line of increment

                            j++;
                            if(j>=(videcInfo->decodedWidth>>3)) break;

                        }

                        if((j%16)==0)
                        {
                            if (((i>>1)%2)==1)    CtBufferPtr+= (2*4*32)-3;
                            else                  CtBufferPtr-= 3;

                            if ((i%2) == 1)       YtBufferPtr+= (2*4*32)-3;
                            else                  YtBufferPtr-= 3;
                        }
                        else
                        {
                            if (((i>>1)%2)==1)    CtBufferPtr-= (2*4*32) - 1 ;
                            else                  CtBufferPtr-= (4*4*32) - 1 ;

                            if ((i%2) == 1)       YtBufferPtr-= (2*4*32) - 1 ;
                            else                  YtBufferPtr-= (4*4*32) - 1 ;
                        }
                    }
                    while(j<(videcInfo->decodedWidth>>3));

                    j=0; n=0;

                    YtBufferPtr =((volatile uint64_t*)(YtBufferAddr+((i+1)*sectionSize)));
                    outputPtr  = (volatile uint32_t*)(yuvP +
                                                (videcInfo->decodedWidth*2*32*2*(i+1)));

                    if(i & 1)
                        CtBufferPtr =((volatile uint64_t*)
                                                (CtBufferAddr+(((i/2)+1)*sectionSize)));
                    else
                        CtBufferPtr =((volatile uint64_t*)
                                        ((CtBufferAddr+((i/2)*sectionSize)) + (16*32)));
                }
            }
        }
        else
        {
            retVal = VOC_ERR_BAD_PARAMETER;
            break;
        }

        /* 2. Notify application after screen was captured and before bmp file
           is written */
        if(bmpWriteNotifyCb != NULL)
            (*bmpWriteNotifyCb)(arg);

        /* 3. write bmp files to USB/HDD device */
        fileP = f_open(filePath, "w");
        if ( fileP == NULL )
        {
            #ifndef _LINUX_FAPI
            retVal = f_getlasterror();
            #endif
            FAPI_SYS_PRINT_MSG(
                    "[VOC] File open failed! error code: %d\n",
                    retVal);
            retVal = VOC_ERR_BAD_PARAMETER;
            break;
        }
        else
        {
            if(yuvP == NULL)
            {
                FAPI_SYS_PRINT_MSG("[VOC] Out of memory!\n");
                retVal = VOC_ERR_OUT_OF_MEMORY;
                break;
            }
            #ifdef VOC_DEBUG
            FAPI_SYS_PRINT_MSG("[VOC] Writing captured image to '%s' \n", filePath);
            #endif /* VOC_DEBUG */

            if(captureMode == VOC_CAPTURE_VIDEO_OSD)
            {
                if(sdSystem == VOC_SDSYS_PAL)
                {
                    bmpWidth  = 720;
                    bmpHeight = 576;
                }
                else if(sdSystem == VOC_SDSYS_NTSC)
                {
                    bmpWidth  = 720;
                    bmpHeight = 480;
                }
                yuvP += bmpWidth*bmpHeight*2;
            }
            else if(captureMode == VOC_CAPTURE_VIDEO_ONLY)
            {
                //adjust for 1920x1080 video,
                if (videcInfo->decodedHeight == 1088)
                {
                    videcInfo->decodedHeight = 1080;        //32*17*2
                }
                /*adjust for 1280*720 video,*/
                else if(videcInfo->decodedHeight == 768)
                {
                    videcInfo->decodedHeight = 720;
                }
                /*adjust for 720*480 video,*/
                else if(videcInfo->decodedHeight == 512)
                {
                    videcInfo->decodedHeight = 480;
                }

                yuvP += videcInfo->decodedWidth*videcInfo->decodedHeight*2;

                bmpWidth  = videcInfo->decodedWidth;
                bmpHeight = videcInfo->decodedHeight;
            }

            /*adjust the header of the BMP file*/
            //size of bmp file
            fileSize = (bmpWidth*bmpHeight*3)+54;
            header[2] = (uint8_t)(fileSize  & dmask);
            header[3] = (uint8_t)((fileSize & (dmask<<8))>>8);
            header[4] = (uint8_t)((fileSize & (dmask<<16))>>16);
            header[5] = (uint8_t)((fileSize & (dmask<<24))>>24);
            //size of raw image data
            dataSize   = fileSize - 54;
            header[34] = (uint8_t)(dataSize  & dmask);
            header[35] = (uint8_t)((dataSize & (dmask<<8))>>8);
            header[36] = (uint8_t)((dataSize & (dmask<<16))>>16);
            header[37] = (uint8_t)((dataSize & (dmask<<24))>>24);
            //picture width (pixel)
            header[18] = (uint8_t)(bmpWidth & dmask);
            header[19] = (uint8_t)((bmpWidth & (dmask<<8))>>8);
            //picture height (pixel)
            header[22] = (uint8_t)(bmpHeight & dmask);
            header[23] = (uint8_t)((bmpHeight & (dmask<<8))>>8);

            writeSize = f_write((void*)header, 1, 54, fileP);

            //prepare params for chunk write
            lineByte    = (bmpWidth * 3);
            if(lineByte != 0)
            {
                writeFactor = (VOC_WRITE_CHUNK_SIZE / lineByte);
            }

            //convert and write pixel data to BMP file
            if ( writeSize == 54 )
            {
                bmpdata = chunkP;
                for(y= 0; y<bmpHeight; y++)
                {
                    writeCnt++;

                    for(x= 1; x<(bmpWidth>>1); x++)
                    {
                        V= yuvP[1+(x*4)] - 128;
                        Y2= yuvP[0+(x*4)] - 16;
                        U= yuvP[3+(x*4)] - 128;
                        Y1= yuvP[2+(x*4)] - 16;

                        Y1= (Y1 * 256) / (235-16); // expand range from 16..235 to 0..255
                        R= ((256 * Y1) + (359 * V)) >> 8;
                        G= (((256 * Y1) - (88 * U)) - (183 * V)) >> 8;
                        B= ((256 * Y1) + (454 * U)) >> 8;

                        if(R < 0) R= 0;
                        if(R > 255) R= 255;
                        if(G < 0) G= 0;
                        if(G > 255) G= 255;
                        if(B < 0) B= 0;
                        if(B > 255) B= 255;

                        bmpdata[x*6]= (uint8_t)B;
                        bmpdata[(x*6)+1]= (uint8_t)G;
                        bmpdata[(x*6)+2]= (uint8_t)R;

                        Y2= (Y2 * 256) / (235-16);
                        R= ((256 * Y2) + (359 * V)) >> 8;
                        G= (((256 * Y2) - (88 * U)) - (183 * V)) >> 8;
                        B= ((256 * Y2) + (454 * U)) >> 8;

                        if(R < 0) R= 0;
                        if(R > 255) R= 255;
                        if(G < 0) G= 0;
                        if(G > 255) G= 255;
                        if(B < 0) B= 0;
                        if(B > 255) B= 255;

                        bmpdata[(x*6)+3]= (uint8_t)B;
                        bmpdata[(x*6)+4]= (uint8_t)G;
                        bmpdata[(x*6)+5]= (uint8_t)R;

                    }/* for(x= 1; x<(bmpWidth>>1); x++) */

                    bmpdata  = (uint8_t*)(bmpdata + lineByte);

                    // task of chunk write
                    if ((writeCnt == writeFactor)||(y == (bmpHeight-1)))
                    {
                        writeSize = f_write(chunkP, 1,
                                        (int32_t) (lineByte*writeCnt), fileP);

                        if (writeSize == (int32_t)(lineByte*writeCnt))
                        {
                            writeCnt = 0;
                            memset(chunkP, 0 , VOC_WRITE_CHUNK_SIZE);
                            bmpdata = chunkP;
                        }
                        else
                        {
                            break; /* end of write process */
                        }
                    }

                    switch (captureMode)
                    {
                        case VOC_CAPTURE_VIDEO_OSD:
                            yuvP -= 720 * 2;
                            break;
                        case VOC_CAPTURE_VIDEO_ONLY:
                            yuvP -= (bmpWidth*2);
                            break;
                    }

                }/* for(y= 0; y<bmpHeight; y++) */

            }/* if ( writeSize == 54 ) */
        }

        break; /* end of for(;;) */
    }

    if(chunkP)
    {
        FAPI_SYS_FREE(chunkP);
    }

    if(pAllocYuvBuff)
    {
        FAPI_SYS_FREE(pAllocYuvBuff);
    }

    if(fileP)
    {
        (void)f_close(fileP);
    }

    FAPI_SYS_PRINT_MSG("[VOC]VOC_Open: VOC_CaptureScreen Ok\n");

SET_SEM:
    FAPEX_SYS_SetSemaphore(vocDat->sem, FAPEX_SYS_SUSPEND);
    return retVal;
}
/*lint -restore */
/* END OF FILE */



