/*
*****************************************************************************
**
** \file        ./subsystems/voc/src/voc.h
**
** \brief       Video Output Capture driver to support mosaic views and screen
**              capture.
**
** This module grabs decoded I-Frames and performs a software scaling to copy
** them into a target rectangle in OSD. It may be used to realise a "mosaic
** preview" feature, i.e. presenting and updating pictures from multiple
** services in parallel.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU MICROELECTRONICS
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Microelectronics Europe GmbH
**
*****************************************************************************
*/

#ifndef VOC_H
    #define VOC_H
    #include <osdhandler/osdhandler.h>
    #include <fapex/abst_types.h>
    #include <fapex/abst_libvidec.h>

    //***********************************************************************
    //***********************************************************************
    //** Defines and Macros
    //***********************************************************************
    //***********************************************************************

       /*!
    ********************************************************************************
    ** Standard error codes for the VOC driver
    ********************************************************************************
    */

    /*! Bad parameter passed. */
    #define VOC_ERR_BAD_PARAMETER \
            (int32_t)(SUBSYS_VOC_MODULE_BASE + FAPI_ERR_BAD_PARAMETER)

    /*! Memory allocation failed. */
    #define VOC_ERR_OUT_OF_MEMORY \
            (int32_t)(SUBSYS_VOC_MODULE_BASE + FAPI_ERR_OUT_OF_MEMORY)

    /*! Device already initialised. */
    #define VOC_ERR_ALREADY_INITIALIZED \
            (int32_t)(SUBSYS_VOC_MODULE_BASE + FAPI_ERR_ALREADY_INITIALIZED)

    /*! Device not initialised. */
    #define VOC_ERR_NOT_INITIALIZED \
            (int32_t)(SUBSYS_VOC_MODULE_BASE + FAPI_ERR_NOT_INITIALIZED)

    /*! Feature or function is not available. */
    #define VOC_ERR_FEATURE_NOT_SUPPORTED \
            (int32_t)(SUBSYS_VOC_MODULE_BASE + FAPI_ERR_FEATURE_NOT_SUPPORTED)

    /*! Timeout occured. */
    #define VOC_ERR_TIMEOUT \
            (int32_t)(SUBSYS_VOC_MODULE_BASE + FAPI_ERR_TIMEOUT)

    /*! The device is busy, try again later. */
    #define VOC_ERR_DEVICE_BUSY \
            (int32_t)(SUBSYS_VOC_MODULE_BASE + FAPI_ERR_DEVICE_BUSY)

    /*! Invalid handle was passed. */
    #define VOC_ERR_INVALID_HANDLE \
            (int32_t)(SUBSYS_VOC_MODULE_BASE + FAPI_ERR_INVALID_HANDLE)

    /*! Semaphore could not be created. */
    #define VOC_ERR_SEMAPHORE_CREATE \
            (int32_t)(SUBSYS_VOC_MODULE_BASE + FAPI_ERR_SEMAPHORE_CREATE)

    /*! The driver's used version is not supported. */
    #define VOC_ERR_UNSUPPORTED_VERSION \
            (int32_t)(SUBSYS_VOC_MODULE_BASE + FAPI_ERR_UNSUPPORTED_VERSION)

    /*! Driver has not been opened. */
    #define VOC_ERR_NOT_OPEN \
            (int32_t)(SUBSYS_VOC_MODULE_BASE + FAPI_ERR_NOT_OPEN)

    /*! Driver is already open. */
    #define VOC_ERR_ALREADY_OPEN \
            (int32_t)(SUBSYS_VOC_MODULE_BASE + FAPI_ERR_ALREADY_OPEN)

     /*!
    ****************************************************************************
    ** Additional VOC driver error codes.
    **
    ** Each error code shall be decreased in steps of 1.
    ****************************************************************************
    */
    /* \brief VOC specific error code offset */
    #define VOC_ERR_BASE \
            (int32_t) (SUBSYS_VOC_MODULE_BASE - 100)

    /*! Returned when attemped to write at coordinates that are out of bounds
    **  of the OSD or CP plane.
    */
    #define VOC_ERR_OUT_OF_OSD_SIZE            (int32_t) (VOC_ERR_BASE - 0)

    /*! The content that should be drawn does not fit for the colour mode
    **  that is currently selected for the OSD.
    */
    #define VOC_ERR_WRONG_COLOUR_MODE          (int32_t) (VOC_ERR_BASE - 1)

    /*! The video Dec returns unknown frame bank */
    #define VOC_ERR_UNKNOWN_FRAMEBANK          (int32_t) (VOC_ERR_BASE - 2)

    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************
    typedef enum {
        //!   Capture the video only.
        VOC_CAPTURE_VIDEO_ONLY,
        //!   Capture the video together with the OSD.
    	VOC_CAPTURE_VIDEO_OSD,

    } VOC_ScreenCaptureEnumT;

    typedef enum{
        VOC_SDSYS_PAL = 0,
        VOC_SDSYS_NTSC,
        VOC_SDSYS_SECAM,
        VOC_SDSYS_NUMS

    } VOC_SDTV_STANDARD;
    /*!
        *******************************************************************************
        **
        ** \brief Picture window position and size.
        **
        ******************************************************************************/
    typedef struct{
        uint16_t            left;       //!< Left position of picture.
        uint16_t            top;        //!< Top position of picture.
        uint16_t            hsize;      //!< Horizontal size (width).
        uint16_t            vsize;      //!< Vertical size (height).

    } VOC_PictureWindowT;

    //! voc function mode enum
    typedef enum
    {
        VOC_MODE_NONE = 0,
        VOC_MODE_SCREENCAPTURE,
        VOC_MODE_MOSAIC,
        VOC_MODE_MOSAIC_CCIR

    } VOC_FunctionT;

    //! voc open parameter
    typedef struct
    {
        VOC_FunctionT            functionMode;        //!< function mode
        int32_t                  videcId;
        int32_t                  viencId;

    } VOC_OpenParamsT;

    //! decoder information structure
    typedef struct{

        uint32_t                             frameBank;
        uint32_t                             offset_y;
        uint32_t                             offset_c;

        uint32_t                             displayWidth;
        uint32_t                             displayHeight;
        uint32_t                             decodedWidth;
        uint32_t                             decodedHeight;
        int32_t                              decodeFrameBankSizeType;
        uint32_t                             proggInter;
    } VOC_VidecInfoT;


    //***********************************************************************
    //***********************************************************************
    //** API Functions
    //***********************************************************************
    //***********************************************************************

    #ifdef __cplusplus
    extern "C" {
    #endif

    int32_t VOC_Init (void);
    int32_t VOC_Exit (void);
    int32_t VOC_Open (VOC_OpenParamsT* openParamsPtr, FAPI_SYS_HandleT* handleP);
    int32_t VOC_Close (FAPI_SYS_HandleT handleP);
    int32_t VOC_CopyIFrame (FAPI_SYS_HandleT hh, VOC_PictureWindowT* winPos,
                            VOC_VidecInfoT* videcInfo,
                            OSDHANDLER_LayerParamsT* osdlayerParams);
    int32_t VOC_CopyIFrameCCIR (FAPI_SYS_HandleT hh, VOC_PictureWindowT* winPos,
                             OSDHANDLER_LayerParamsT* layerParams);
    int32_t VOC_CaptureScreen(FAPI_SYS_HandleT hh, const char_t* filePath,
                          VOC_ScreenCaptureEnumT captureMode,
                          VOC_SDTV_STANDARD sdSystem,
                          VOC_VidecInfoT* videcInfo,
                          void (*bmpWriteNotifyCb)(void* arg), void* arg);

    #ifdef __cplusplus
    }
    #endif
#endif
