/*
* Copyright (c) 2010 Fujitsu Semiconductor
*
*/

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <stdbool.h>

#include <fapex/abst_types.h>
#include <fapex/abst_libclkpwr.h>

#define WATCHDOG_COUNT_OVERFLOW 0x0fffffff
#define WATCHDOG_ONE_COUNT_NS (1000 / 36)   // watchdog frequence is 36MHz, one second is 1G ns

typedef struct {
    int fd;
    struct timespec interval;
    bool isFeed;
    bool isExit;
} FeedEnv;

/**
 * get sleep timespec from watchdog value
 */
static struct timespec getSleepTime(uint32_t value) {
    uint32_t halfRemainCnt = (WATCHDOG_COUNT_OVERFLOW - value) / 2; // for feed dog quick before overflow, we only sleep half of remain count
    uint32_t ns = WATCHDOG_ONE_COUNT_NS * halfRemainCnt;
    struct timespec ts = {
        .tv_sec = ns / (1000 * 1000 * 1000),
        .tv_nsec = ns % (1000 * 1000 * 1000)
    };
    return ts;
}

static void *feedDog(void *pArg) {
    FeedEnv *pEnv = (FeedEnv *) pArg;
    while (!pEnv->isExit) {
        if (pEnv->isFeed) {
            if (FAPEX_CLKPWR_Ioctl(pEnv->fd, FAPEX_IOCCOM_CLKPWR_WATCHDOG_RESET, (uint32_t)NULL) < 0) {
                FAPI_SYS_PRINT_MSG("feed error\n");
                pEnv->isExit = true;
            }
        }
        nanosleep(&pEnv->interval, NULL);
    }
    
    return NULL;
}

int main(int argc, char **argv) {
    FAPI_SYS_PRINT_MSG("Usage:\n"
                                        "\tstart value:\tstart watchdog with value\n"
                                        "\tfeed <1|0>:\tfeed(1) or don't feed(0) watchdog\n"
                                        "\tstop:\tstop watchdog\n"
                                        "\texit:\texit test\n"
                                        "\tNOTE: because watchdog count fast, so you should type \"feed 1\" before start,\n"
                                        "\t\telse watchdog may overflow before you type feed\n");

    int err = 0;
    FeedEnv env = {
        .fd = -1,
        {.tv_sec = 0, .tv_nsec = 0},
        .isFeed = false,
        .isExit = false
    };
    env.fd = FAPEX_CLKPWR_Open(FAPEXDEV_CLKPWR_MINOR_FIRST, O_RDWR, NULL);
    if (env.fd < 0) {
        FAPI_SYS_PRINT_MSG("Open clkpwr error\n");
        return -1;
    }

    pthread_t feedTid;
    if (pthread_create(&feedTid, NULL, feedDog, &env) < 0) {
        FAPI_SYS_PRINT_MSG("create thread error\n");
        FAPEX_CLKPWR_Close(env.fd);
        return -1;
    }
    
    char aBuff[256];
    uint32_t value = 0;
    while (!env.isExit) {
        memset(aBuff, 0, sizeof(aBuff));
        if (gets(aBuff) != NULL) {
            if (strstr(aBuff, "start") != NULL) {            // start
                sscanf(aBuff, "start %i", &value);
                if ((err = FAPEX_CLKPWR_Ioctl(env.fd, FAPEX_IOCCOM_CLKPWR_WATCHDOG_START, value)) < 0) {    // NOTE: use value directly, see fapex_fdlclkpwr_main.c:201
                    FAPI_SYS_PRINT_MSG("Ioctl clkpwr err = %d\n", err);
                    break;
                }
                env.interval = getSleepTime(value);
            } else if (strstr(aBuff, "feed") != NULL) {    // feed
                sscanf(aBuff, "feed %i", &value);
                env.isFeed = (value != 0);
            } else if (strstr(aBuff, "exit") != NULL) {       // exit
                env.isExit = true;
                break;
            } else if (strstr(aBuff, "stop") != NULL) {    // stop
                FAPEX_CLKPWR_Ioctl(env.fd, FAPEX_IOCCOM_CLKPWR_WATCHDOG_STOP,(uint32_t)NULL);
            } 
        }
    }
    
    pthread_join(feedTid, NULL);
    FAPEX_CLKPWR_Ioctl(env.fd, FAPEX_IOCCOM_CLKPWR_WATCHDOG_STOP, (uint32_t)NULL);
    FAPEX_CLKPWR_Close(env.fd);
    return 0;
}
