/*
*****************************************************************************
**
** \file        ./middleware/pvr/src/pvr_play.c
**
** \version     $Id: pvr_play.c 5952 2010-03-08 16:31:41Z rseng $
**
** \brief       PVR middleware playback engine.
**
** This files contains the implementation of the PVR playback API.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

//#define DEBUG 1

//#define VIDEC_IFRAME

//#define STATIC_CHUNK_BUFFER

#include <assert.h>

#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"

#include "pvr.h"
#include "pvr_priv.h"
#include "pvr_record.h"
#include "pvr_play.h"
#include "api_f.h"

/*lint -save -e716 -e717 -e788 -e970 */

/* NOTE: the following LINT statements have been suppressed:
- Info 716: while(1) ...
- Info 717: do ... while(0);
- Info 788: Enum constant 'anyConst' not used within defaulted switch
- Note 970: Use of modifier or type 'float' outside of a typedef
*/

//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************

#define PVR_PLAY_LOCK       (void)FAPI_SYS_GET_SEMAPHORE(inst->playSemaphore, FAPI_SYS_SUSPEND);
#define PVR_PLAY_UNLOCK           FAPI_SYS_SET_SEMAPHORE(inst->playSemaphore, FAPI_SYS_NO_SUSPEND);

#define PVR_PLAY_READTHREAD_STACKSIZE   2048        //!< In 32-bit longwords.
#define PVR_PLAY_READTHREAD_NAME        "pvrPlay_Read"

#define PVR_PLAY_SEEK_MAX_CHUNKS   ((2147483648UL / PVR_CHUNK_SIZE) - 1)

#define PVR_PLAY_CHUNKBUFFULL_SLEEPTIME     5          /* in ms */
#define PVR_PLAY_ENDOFDATA_SLEEPTIME        200        /* in ms */
#define PVR_PLAY_IFRAME_SLEEPTIME           20

#define PVR_PLAY_IFRAME_MPEG_CHUNKFEED_MAX  20
#define PVR_PLAY_IFRAME_SEARCHTIMEOUT       5000        /* in ms */

//#define PVR_PLAY_IFRAME_BIGENDIAN

#define PVR_PLAY_NOFEED_CHUNKSEEK_MAX       16
#define PVR_PLAY_NOFEED_SLEEPTIME           20          /* in ms */

#define PVR_PLAY_CTRLFEED_CHUNKFEED_MAX     200
#define PVR_PLAY_CTRLFEED_SLEEPTIME         20          /* in ms */

#define PVR_PLAY_TIMESEEK_ATTEMPTS_MAX      5
#define PVR_PLAY_TIMESEEK_DATARATE_MIN      50000       /* in bits/s */
#define PVR_PLAY_TIMESEEK_DATARATE_MAX      15000000    /* in bits/s */
#define PVR_PLAY_TIMESEEK_DATARATE_DEFAULT  3000000     /* in bits/s */

#define PVR_PLAY_SEEKOFFSET_MAX             1024        /*512*/
#define PVR_PLAY_SEEKOFFSET_FACTOR_MAX      4           /*Martin changed for debug: 4 is original value*/
#define PVR_PLAY_SEEKOFFSET_ATTENUATION     2

#define PVR_PLAY_IFRAME_TMP_BUFFER_SIZE     1*1024*1024 /* in bytes*/

//***************************************************************************
//***************************************************************************
//** Local Structures
//***************************************************************************
//***************************************************************************

enum /* internal seek options */
{
    PLAY_SEEK_POS_ABS = 0,
    PLAY_SEEK_POS_REL,
    PLAY_SEEK_NAME_FORWARD,
    PLAY_SEEK_NAME_BACKWARDS,
    PLAY_SEEK_TAG_FORWARD,
    PLAY_SEEK_TAG_BACKWARDS,
    PLAY_SEEK_IFRAME_FORWARD,
    PLAY_SEEK_IFRAME_BACKWARDS,
    PLAY_SEEK_TIME_ABS
};

typedef enum /* I-frame seek modes */
{
    PLAY_SEEKMODE_NONE = 0,     //!< Leave file position at end of I-Frame.
    PLAY_SEEKMODE_BACK,         //!< Reset file position to start of search.
    PLAY_SEEKMODE_IFRAMESTART   //!< Set file position to I-Frame start.

} PVR_PlaySeekModeT;

#if 0 /* Move to .h by Martin*/
typedef enum
{
    PLAY_STOPMODE_BLANK,
    PLAY_STOPMODE_FREEZE,
    PLAY_STOPMODE_NONE

} PVR_PlayStopMode;
#endif
//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************

static pvrPlayDataT* pvrPlayDataPtr = NULL;
#ifdef STATIC_CHUNK_BUFFER
static uint8_t pvrPlayStaticChunkBuffer[PVR_PLAY_CHUNKBUFFER_CHUNKS * PVR_CHUNK_SIZE];
#endif


//***************************************************************************
//***************************************************************************
//** Local Functions Declaration
//***************************************************************************
//***************************************************************************

static int32_t  pvrPlayClearBuffer      (pvrPlayHandleT* inst);

static void     pvrPlaySetState(pvrPlayHandleT* inst, PVR_PlayStateT state,
                                uint8_t stateIndex);
static void     pvrPlayReadThreadFunction(void* optArgPtr);
static void     pvrPlayThreadStoppedTask(void* inst,
                                        const PVR_PlayStateInfoT* stateInfoPtr);
static void     pvrPlayThreadNormalTask(void* inst,
                                        const PVR_PlayStateInfoT* stateInfoPtr);
static void     pvrPlayThreadSlowTask(void* inst,
                                      const PVR_PlayStateInfoT* stateInfoPtr);
static void     pvrPlayThreadFastIFramesTask(void* inst,
                                             const PVR_PlayStateInfoT*
                                             stateInfoPtr);
static void     pvrPlayThreadFastIFramesReset(pvrPlayHandleT* inst,
                                              uint8_t fastModeIdx);
static void     pvrPlayThreadFastAllTask(void* inst,
                                        const PVR_PlayStateInfoT* stateInfoPtr);
static void     pvrPlayThreadFastNoneReset(pvrPlayHandleT* inst,
                                           uint8_t fastModeIdx);
static void     pvrPlayThreadFastNoneTask(void* inst,
                                          const PVR_PlayStateInfoT*
                                          stateInfoPtr);
static void     pvrPlayThreadFastControlledTask(void* inst,
                                        const PVR_PlayStateInfoT* stateInfoPtr);
static void     pvrPlayThreadFastControlledReset(pvrPlayHandleT* inst,
                                                 uint8_t fastModeIdx);

static void     pvrPlayThreadPauseTask(void* inst,
                                       const PVR_PlayStateInfoT* stateInfoPtr);
static void     pvrPlayThreadSeekTask(void* inst,
                                      const PVR_PlayStateInfoT* stateInfoPtr);
static void     pvrPlaySeekToTime(pvrPlayHandleT* inst, uint32_t seekTime);
static void     pvrPlaySingleIFrame(pvrPlayHandleT* inst,
                                    PVR_PlayNotifyT* notifyReasonPtr);
static uint8_t  pvrGetTrack(pvrPlayHandleT* inst, PVR_DecoderTypeT decoderType);
//static void     pvrProcessSleepCounter(pvrPlayHandleT* inst);
static uint8_t  pvrSupportTimeShifting(pvrPlayHandleT* inst);

static uint8_t  pvrCheckForParamChange(pvrPlayHandleT* inst,
                                       PVR_DecoderTypeT type);
static uint8_t  pvrCheckForTrackStop(pvrPlayHandleT* inst,
                                     PVR_DecoderTypeT type);

static int32_t  pvrReadHeader(pvrPlayHandleT* inst,
                              PVR_HeaderPacketT* headerPtr);
static int32_t  pvrReadData  (pvrPlayHandleT* inst,
                              PVR_HeaderPacketT* headerPtr);
static int32_t  pvrReadChunkFromFile(pvrPlayHandleT* inst, void* dstDataPtr);

static uint8_t  pvrIsEndOfData(pvrPlayHandleT* inst);
static uint8_t  pvrIsBeginOfData(pvrPlayHandleT* inst);

static void     pvrSeekChunksSet(pvrPlayHandleT* inst, uint32_t chunks);
static void     pvrSeekChunksCur(pvrPlayHandleT* inst, int32_t chunks);
// TODO: not supported yet
//static void     pvrSeekNameForward(pvrPlayHandleT* inst, uint8_t seekName);
//static void     pvrSeekNameBackward(pvrPlayHandleT* inst, uint8_t seekName);

static int32_t  pvrUpdateChunkTime(pvrPlayHandleT* inst, uint32_t time);
static int32_t  pvrPlayMPEGIFrame(pvrPlayHandleT* inst,
                                  PVR_PlaySeekModeT seekMode,
                                  uint32_t displayTime);
static uint8_t* pvrFindIFrameStart (pvrPlayHandleT* inst,
                                    const uint8_t* bufStartPtr,
                                    const uint8_t* bufEndPtr,
                                    uint8_t** bufRpPtr,
                                    uint8_t* statePtr);
static uint8_t* pvrFindIFrameEnd   (pvrPlayHandleT* inst,
                                    const uint8_t* bufStartPtr,
                                    const uint8_t* bufEndPtr,
                                    uint8_t** bufRpPtr,
                                    uint8_t* state1Ptr, uint8_t* state2Ptr);
static int32_t pvrPlayCtrlFeedIFrame(pvrPlayHandleT* inst,
                                     PVR_PlaySeekModeT seekMode,
                                     uint32_t displayTime);

static int32_t  pvrPlayCalcNewOffset (int32_t playTimeDiff,
                                      int32_t realTimeDiff,
                                      int32_t factor, int32_t currOffset,
                                      int32_t initOffset);

static int32_t  pvrIFrameChannelOpen (pvrPlayHandleT* inst);
static void     pvrIFrameChannelClose(pvrPlayHandleT* inst);
static void     pvrIFrameChannelStart(pvrPlayHandleT* inst);
static void     pvrIFrameChannelStop (pvrPlayHandleT* inst);

#ifdef PVR_PLAY_IFRAME_BIGENDIAN

static void pvrCopyBigEndian(void* dstPtr, const void* srcPtr, int32_t size);

#endif /* PVR_PLAY_IFRAME_BIGENDIAN */

//static void     pvrGetNextCutInOutPos(pvrPlayHandleT* inst);
//static void     pvrGetPreviousCutInOutPos(pvrPlayHandleT* inst);
//static void     pvrGetNextABInOutPos(pvrPlayHandleT* inst);

static void     pvrPlayFreeResources(pvrPlayHandleT* inst);

static void     pvrStopMedia(pvrPlayHandleT* inst);
static void     pvrStartAll(pvrPlayHandleT* inst, PVR_DecoderModeT mode,
                            uint8_t modeIndex);


static void     pvrPlayNotify(pvrPlayHandleT* inst, PVR_PlayNotifyT reason);

static uint8_t  pvrPidIsValid(pvrPlayHandleT* inst, PVR_DecoderTypeT type);

// Move to .h by Martin
#if 0
 void     pvrTrackStart         (pvrPlayHandleT* inst,
                                       PVR_DecoderTypeT type,
                                       PVR_DecoderModeT mode,
                                       uint8_t modeIndex);
 void     pvrTrackStop          (pvrPlayHandleT* inst,
                                       PVR_DecoderTypeT type,
                                       PVR_PlayStopMode stopMode);
#endif
static void     pvrTrackSetMode       (pvrPlayHandleT* inst,
                                       PVR_DecoderTypeT type,
                                       PVR_DecoderModeT mode,
                                       uint8_t modeIndex);
static void     pvrTrackResetBuffer   (pvrPlayHandleT* inst,
                                       PVR_DecoderTypeT type);
static uint8_t  pvrTrackGetFormat     (pvrPlayHandleT* inst,
                                       PVR_DecoderTypeT type);

static int32_t  pvrDecoderStart       (pvrPlayHandleT* inst,
                                       PVR_DecoderTypeT type,
                                       uint8_t trackIndex,
                                       uint32_t trackData);
static int32_t  pvrDecoderStop        (pvrPlayHandleT* inst,
                                       PVR_DecoderTypeT type,
                                       PVR_PlayStopMode stopMode);
static int32_t  pvrDecoderSetMode     (pvrPlayHandleT* inst,
                                       PVR_DecoderTypeT type,
                                       PVR_DecoderModeT mode,
                                       uint8_t modeIndex);
static int32_t  pvrDecoderFeedData    (pvrPlayHandleT* inst,
                                       PVR_DecoderTypeT type,
                                       const uint8_t* dataPtr,
                                       uint32_t dataBytes);
static int32_t  pvrDecoderResetBuffer (pvrPlayHandleT* inst,
                                       PVR_DecoderTypeT type);
static uint8_t  pvrDecoderIsDataDecoded(pvrPlayHandleT* inst,
                                        PVR_DecoderTypeT type);
static int32_t  pvrDecoderPrepareData (pvrPlayHandleT* inst,
                                       PVR_DecoderTypeT type);
static void     pvrDecoderTasks       (pvrPlayHandleT* inst);

static int32_t  pvrSyncStart          (pvrPlayHandleT* inst);
static int32_t  pvrSyncStop           (pvrPlayHandleT* inst);

static int32_t pvrDecoderGetTsdFd(pvrPlayHandleT* inst, PVR_DecoderTypeT type);

#if 0
static int32_t
pvrCacheRead(pvrPlayHandleT *inst, int32_t cacheSize, int32_t bytePerRead);
#endif


#if DEBUG > 1
static void     pvrPlayFastModeConfigDump(PVR_FastModeConfigT fastModeConfig);
#endif

//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************
//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

int32_t pvrPlayInit(void)
{
    uint16_t index;

    if ( pvrPlayDataPtr == NULL )
    {
        /* allocate memory for global playback data */
        pvrPlayDataPtr = FAPI_SYS_MALLOC_CACHED(sizeof(pvrPlayDataT));
        if ( pvrPlayDataPtr == NULL )
        {
            return SUBSYS_PVR_ERR_OUT_OF_MEMORY;
        }

        memset(pvrPlayDataPtr, 0, sizeof(pvrPlayDataT));

        /* set default fast mode configuration */
        for ( index = 0; index < (PVR_MAX_FAST_MODES / 2); index++ )
        {
            /* fast backward */
            pvrPlayDataPtr->fastModeConfig[index].type =
                                        PVR_FASTMODETYPE_IFRAMES;
            pvrPlayDataPtr->fastModeConfig[index].params.iFrames.factor =
                                        -((int32_t)index + 2);
            pvrPlayDataPtr->fastModeConfig[index].params.iFrames.displayTime =
                                        200;
        }

        for ( ; index < PVR_MAX_FAST_MODES; index++ )
        {
            /* fast forward */
            pvrPlayDataPtr->fastModeConfig[index].type =
                                PVR_FASTMODETYPE_IFRAMES;
            pvrPlayDataPtr->fastModeConfig[index].params.iFrames.factor =
                                ((int32_t)index - (PVR_MAX_FAST_MODES / 2)) + 2;
            pvrPlayDataPtr->fastModeConfig[index].params.iFrames.displayTime =
                                200;
        }

        /* set default decoder starting order */
        #if 1

        pvrPlayDataPtr->decoderStartOrder[0] = PVR_DECTYPE_VIDEO;
        pvrPlayDataPtr->decoderStartOrder[1] = PVR_DECTYPE_AUDIO;
        pvrPlayDataPtr->decoderStartOrder[2] = PVR_DECTYPE_AUDIO_2ND;
        pvrPlayDataPtr->decoderStartOrder[3] = PVR_DECTYPE_TTX;

        #else

        pvrPlayDataPtr->decoderStartOrder[0] = PVR_DECTYPE_AUDIO;
        pvrPlayDataPtr->decoderStartOrder[1] = PVR_DECTYPE_AUDIO_2ND;
        pvrPlayDataPtr->decoderStartOrder[2] = PVR_DECTYPE_TTX;
        pvrPlayDataPtr->decoderStartOrder[3] = PVR_DECTYPE_VIDEO;

        #endif

        /* assign handlers for playback state machine */

        pvrPlayDataPtr->stateHandler[PLAY_STATE_STOPPED]         = pvrPlayThreadStoppedTask;
        pvrPlayDataPtr->stateHandler[PLAY_STATE_NORMAL]          = pvrPlayThreadNormalTask;
        pvrPlayDataPtr->stateHandler[PLAY_STATE_FAST_ALL]        = pvrPlayThreadFastAllTask;
        pvrPlayDataPtr->stateHandler[PLAY_STATE_FAST_IFRAMES]    = pvrPlayThreadFastIFramesTask;
        pvrPlayDataPtr->stateHandler[PLAY_STATE_FAST_NONE]       = pvrPlayThreadFastNoneTask;
        pvrPlayDataPtr->stateHandler[PLAY_STATE_FAST_CONTROLLED] = pvrPlayThreadFastControlledTask;
        pvrPlayDataPtr->stateHandler[PLAY_STATE_SLOW]            = pvrPlayThreadSlowTask;
        pvrPlayDataPtr->stateHandler[PLAY_STATE_PAUSE]           = pvrPlayThreadPauseTask;
        pvrPlayDataPtr->stateHandler[PLAY_STATE_SEEK]            = pvrPlayThreadSeekTask;

        #if (DEBUG >= 4)

        pvrPlayDataPtr->stateString[PLAY_STATE_STOPPED]             = "STOPPED";
        pvrPlayDataPtr->stateString[PLAY_STATE_NORMAL]              = "NORMAL";
        pvrPlayDataPtr->stateString[PLAY_STATE_FAST_ALL]            = "FAST_ALL";
        pvrPlayDataPtr->stateString[PLAY_STATE_FAST_IFRAMES]        = "FAST_IFRAMES";
        pvrPlayDataPtr->stateString[PLAY_STATE_FAST_NONE]           = "FAST_NONE";
        pvrPlayDataPtr->stateString[PLAY_STATE_FAST_CONTROLLED]     = "FAST_CONTROLLED";
        pvrPlayDataPtr->stateString[PLAY_STATE_SLOW]                = "SLOW";
        pvrPlayDataPtr->stateString[PLAY_STATE_PAUSE]               = "PAUSE";
        pvrPlayDataPtr->stateString[PLAY_STATE_TO_STOP]             = "TO_STOP";
        pvrPlayDataPtr->stateString[PLAY_STATE_TO_NORMAL]           = "TO_NORMAL";
        pvrPlayDataPtr->stateString[PLAY_STATE_TO_FAST_ALL]         = "TO_FAST_ALL";
        pvrPlayDataPtr->stateString[PLAY_STATE_TO_FAST_IFRAMES]     = "TO_FAST_IFRAMES";
        pvrPlayDataPtr->stateString[PLAY_STATE_TO_FAST_NONE]        = "TO_FAST_NONE";
        pvrPlayDataPtr->stateString[PLAY_STATE_TO_FAST_CONTROLLED]  = "TO_FAST_CONTROLLED";
        pvrPlayDataPtr->stateString[PLAY_STATE_TO_SLOW]             = "TO_SLOW";
        pvrPlayDataPtr->stateString[PLAY_STATE_TO_PAUSE]            = "TO_PAUSE";
        pvrPlayDataPtr->stateString[PLAY_STATE_SEEK]                = "SEEK";

        #endif /* (DEBUG >= 4) */
    }

    return FAPI_OK;
}

int32_t pvrPlayExit(void)
{
    if ( pvrPlayDataPtr != NULL )
    {
        FAPI_SYS_FREE(pvrPlayDataPtr);
        pvrPlayDataPtr = NULL;
    }

    return FAPI_OK;
}

int32_t pvrPlayGetFastModeConfig (uint16_t index,
                                  PVR_FastModeConfigT* configPtr)
{
    // TODO: needs to be protected!

    if ( configPtr != NULL )
    {
        *configPtr = pvrPlayDataPtr->fastModeConfig[index];
    }

    return FAPI_OK;
}

int32_t pvrPlaySetFastModeConfig (uint16_t index,
                                  const PVR_FastModeConfigT* configPtr)
{
    // TODO: needs to be protected!

    pvrPlayDataPtr->fastModeConfig[index] = *configPtr;

    FAPI_SYS_PRINT_DEBUG(1,"[PLAY] pvrPlaySetFastModeConfig: index[%02d], type[%d]",\
                   index,
                   pvrPlayDataPtr->fastModeConfig[index].type);

    switch(pvrPlayDataPtr->fastModeConfig[index].type)
    {
        case PVR_FASTMODETYPE_IFRAMES:
            FAPI_SYS_PRINT_DEBUG(1," facotor[%d], sleep[%d]\n",\
               pvrPlayDataPtr->fastModeConfig[index].params.iFrames.factor,
               pvrPlayDataPtr->fastModeConfig[index].params.iFrames.displayTime);
            break;

        case PVR_FASTMODETYPE_FEED_CONTROLLED:
            FAPI_SYS_PRINT_DEBUG(1," facotor[%d], sleep[%d]\n",\
               pvrPlayDataPtr->fastModeConfig[index].params.feedCtrl.factor,
               pvrPlayDataPtr->fastModeConfig[index].params.feedCtrl.displayTime);
            break;

        case PVR_FASTMODETYPE_FEED_NONE:
            FAPI_SYS_PRINT_DEBUG(1," facotor[%d], sleep[%d]\n",\
               pvrPlayDataPtr->fastModeConfig[index].params.feedNone.factor, 0);

            break;

        case PVR_FASTMODETYPE_FEED_ALL:
            FAPI_SYS_PRINT_DEBUG(1," facotor[%d], sleep[%d]\n",\
               pvrPlayDataPtr->fastModeConfig[index].params.feedAll.unused,\
               pvrPlayDataPtr->fastModeConfig[index].params.feedAll.unused);

            break;

        default:
        	FAPI_SYS_PRINT_MSG("[PLAY] pvrPlaySetFastModeConfig: Unknown Type!\n");
        	return SUBSYS_PVR_ERR_BAD_PARAMETER;
    }


    return FAPI_OK;
}

int32_t pvrPlayOpen (const PVR_PlayOpenT* paramsPtr,
                     FAPEX_SYS_HandleT recHandle, FAPEX_SYS_HandleT* handlePtr)
{
    pvrPlayHandleT*         inst       = NULL;
    int32_t                 error_code = FAPI_OK;
    PVR_DecoderTypeT        decIdx;
    pvrPlayStreamT*         streamPtr;


    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayOpen: In\n");
    do
    {
        /* allocate memory for this instance */
        inst = FAPI_SYS_MALLOC_CACHED(sizeof(pvrPlayHandleT));
        if ( inst == NULL )
        {
            error_code = SUBSYS_PVR_ERR_OUT_OF_MEMORY;
            break;
        }
        FAPI_SYS_PRINT_DEBUG(2, "[PLAY] pvrPlayOpen: malloc mem for play inst success\n");
        memset(inst, 0, sizeof(pvrPlayHandleT));

        inst->playDataPtr       = pvrPlayDataPtr;
        inst->readThread        = NULL;
        inst->recHandle         = recHandle;
        inst->openParams        = *paramsPtr;

        inst->tmpHeader = FAPI_SYS_MALLOC_CACHED(sizeof(PVR_HeaderPacketT));
        if ( inst->tmpHeader == NULL )
        {
            error_code = SUBSYS_PVR_ERR_OUT_OF_MEMORY;
            break;
        }

        /* allocate buffer memory */
        #ifndef _LINUX_FAPI
        inst->chunkBuffer = FAPI_SYS_MALLOC_UNCACHED(PVR_PLAY_CHUNKBUFFER_CHUNKS
                                                   * PVR_CHUNK_SIZE);
        #else
        #ifndef STATIC_CHUNK_BUFFER
        inst->chunkBuffer = FAPI_SYS_MALLOC_CACHED(PVR_PLAY_CHUNKBUFFER_CHUNKS
                                                   * PVR_CHUNK_SIZE);
        #else
        memset(pvrPlayStaticChunkBuffer, 0, sizeof(pvrPlayStaticChunkBuffer));
        inst->chunkBuffer = (void*)pvrPlayStaticChunkBuffer;
        #endif
        #endif
        if ( inst->chunkBuffer == NULL )
        {
            error_code = SUBSYS_PVR_ERR_OUT_OF_MEMORY;
            break;
        }
        FAPI_SYS_PRINT_DEBUG(2, "[PLAY] pvrPlayOpen: malloc chunk buffer success\n");

        inst->chunkBufferRp = 0;
        inst->chunkBufferWp = 0;

        /* create semaphore for protection of shared data */
        inst->playSemaphore = FAPEX_SYS_CreateSemaphore(1);
        if ( inst->playSemaphore == 0 )
        {
            error_code = SUBSYS_PVR_ERR_SEMAPHORE_CREATE;
            break;
        }

        /* open TSD and buffer for I-frame capturing */
        if ( inst->openParams.iFrameBufferSize > 0 )
        {
            error_code = pvrIFrameChannelOpen(inst);
            if ( error_code != FAPI_OK )
            {
                break;
            }

            if (NULL == inst->iframeTmpBuffer)
	        {
	            inst->iframeTmpBuffer = FAPI_SYS_MALLOC_CACHED(PVR_PLAY_IFRAME_TMP_BUFFER_SIZE);
	            if (NULL == inst->iframeTmpBuffer)
	            {
	                FAPI_SYS_PRINT_MSG("[PLAY] pvrPlayOpen: malloc iframe temp buff failed\n");
	                error_code  = SUBSYS_PVR_ERR_OUT_OF_MEMORY;
	                break;
	            }
	            else
	            {
	                FAPI_SYS_PRINT_MSG("[PLAY] pvrPlayOpen: malloc iframe temp success[0x%08x]\n",\
	                    (uint32_t)inst->iframeTmpBuffer);
	            }
	        }
        }

        /* open TSD Block ram map */
        {
            FAPEX_TSD_PacketInMapInfoStrT  input_map_info;
            uint8_t   *input_map;

            error_code = FAPEX_TSD_Ioctl( paramsPtr->tsdBlockFd,
                              FAPEX_IOCCOM_TSD_GET_INPUT_MAP_INFO,
                              (FAPEXIOCARG3_T)&input_map_info );
            if(error_code < 0) {
                FAPI_SYS_PRINT_ERR("[PLAY] pvrPlayOpen: Error: get TSD input buffer mmapping info failed %d.\n",
                                   error_code);
                return error_code;
            }
            FAPI_SYS_PRINT_DEBUG(2, "[PLAY] pvrPlayOpen: mapSize = %d, tsd_block_fd = %d\n", \
                                                  input_map_info.mapSize, paramsPtr->tsdBlockFd);
            input_map = (uint8_t*)FAPEX_TSD_Mmap( NULL, input_map_info.mapSize,
                                                  (PROT_READ | PROT_WRITE), MAP_SHARED,
                                                  paramsPtr->tsdBlockFd, 0 );
            if (input_map == (void*)-1){
                error_code = -1;
                FAPI_SYS_PRINT_MSG("[PLAY] pvrPlayOpen: TSD input buffer mmap failed %d. tsd_block_index[%d], tsd_block_fd[%d]\n", \
                                                            error_code, paramsPtr->tsdBlockIndex, paramsPtr->tsdBlockFd);
                return error_code;
            }

            inst->input_port_base = input_map + input_map_info.mapOffset;
            inst->map_size = input_map_info.mapSize;
            inst->input_map = input_map;
        }

        inst->state                 = PLAY_STATE_STOPPED;
        inst->lastState             = PLAY_STATE_STOPPED;
        inst->stateIndex            = 0;
        inst->reqStateIdx           = 0;
        inst->size                  = 0;
        inst->offset                = 0;
        inst->position              = 0;
        inst->internalMode          = 0;
        inst->feedOutput            = pvrFeedTsd;

        inst->seekPosition          = 0;
        inst->seekChunks            = 0;
        inst->newNameCounter        = 0;
        inst->seekTime              = 0;

        inst->lastNoDataCounter     = 0;
        inst->lastTagCounter        = 0;
        inst->lastNameCounter       = 0;
        inst->lastFilling           = 0;
        inst->fillingCounter        = 0;
        inst->sleepCounter          = 0;
        inst->readHddTime           = 0;
        inst->feedTsdTime           = 0;
        inst->iFrameTime            = 0;
        inst->enableFrameView       = 1;

        inst->chunkTimeNums         = 0;
        inst->chunkTimeOffset       = 0;
        inst->chunkTimeLastStamp    = 0;
        inst->chunkTimeFirst        = 1;

        inst->read_data_ts          = 0;
        inst->read_data_header_time = 0;

        if ( inst->recHandle == NULL )
        {
            error_code = pvrGetFileInfo (inst->openParams.fileName,
                                         &(inst->fileInfo),
                                         &(inst->header));

            if ( error_code == FAPI_OK )
            {
                inst->splitSize             = inst->header.splitSize;
                inst->numberOfSplits        = inst->header.numberOfSplits;
            }
        }
        else
        {
            error_code = pvrRecGetFileInfo(inst->recHandle, &(inst->fileInfo));

            if ( error_code == FAPI_OK )
            {
                inst->splitSize      = pvrRecGetSplitSize     (inst->recHandle);
                inst->numberOfSplits = pvrRecGetNumberOfSplits(inst->recHandle);
            }
        }

        if ( error_code != FAPI_OK )
        {
            break;
        }

        /* empty file, cannot play */
        if ( inst->fileInfo.size < 1 )
        {
            FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayOpen: empty file, cannot play\n");
            error_code = SUBSYS_PVR_ERR_BAD_FILE_FORMAT;
            break;
        }

        inst->size          = inst->fileInfo.size;
        inst->offset        = inst->fileInfo.offset;
        inst->splitNumber   = 0;

        /* open first file */
        pvrGetFilename (inst->filePath, inst->openParams.fileName, 0);

        inst->filePtr = fopen(inst->filePath, "r");
        if ( inst->filePtr == NULL )
        {
            error_code = SUBSYS_PVR_ERR_FILE_HANDLING;
            break;
        }

        if ( inst->recHandle == NULL )
        {
            /* set read position to begin of data */
            pvrSeekChunksSet(inst, inst->offset
                                 + inst->openParams.startPosition);
        }
        else
        {
            /* reset position to (near) end of data in case of time-shifting */
            FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayOpen: support timeshifting\n");
            (void)pvrSupportTimeShifting(inst);

            if ( inst->offset == 0 )
            {
                pvrSeekChunksSet(inst, inst->size);
            }
            else
            {
                pvrSeekChunksSet(inst, inst->offset - 1);
            }

            /* MBisch: needed to be increased because of reading more initial data. */
            pvrSeekChunksCur(inst, -32);
        }

        /* read header */
        (void)pvrReadHeader(inst, &(inst->header));
        FAPI_SYS_PRINT_DEBUG(2, "[PLAY] pvrPlayOpen: read Header done, inst->position[0x%08x]\n",\
                                    inst->position);

        inst->slowFeed = 1;

        for ( decIdx = (PVR_DecoderTypeT)0; decIdx < PVR_DECTYPE_NUMS;
              decIdx++ )
        {
            streamPtr = inst->stream + (int32_t)decIdx;

            /* get default tracks for each track type */
            streamPtr->trackIndex = pvrGetTrack(inst, decIdx);
            streamPtr->trackData  = 0;

            /* remain in slow feed mode only if no tracks are selected */
            if ( streamPtr->trackIndex != 0xFF )
            {
                inst->slowFeed = 0;
            }

            /* open all target decoders */
            if ( inst->openParams.targetControl.decoder[decIdx].open != NULL )
            {
                streamPtr->decHandle = inst->openParams.targetControl.decoder
                                       [decIdx].open(decIdx,
                                       inst->openParams.targetControl.argPtr);
            }
            else
            {
                streamPtr->decHandle = NULL;
            }

        }

        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayOpen: decoder open\n");

        #if 1 /*original feed open place*/
        /* open feeding module */
        error_code = pvrPlayFeedOpen (inst);
        if ( error_code != FAPI_OK )
        {
            break;
        }

        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayOpen: open feed module success\n");
        #endif
        /* create reading thread */

        inst->readThread = FAPEX_SYS_CreateThread(NULL, /*Martin: make stack size 512K for static chunkbuffer*/
                                            PVR_PLAY_READTHREAD_STACKSIZE,
                                            inst->openParams.readThreadPriority,
                                            pvrPlayReadThreadFunction,
                                            inst, 0, PVR_PLAY_READTHREAD_NAME);

        if ( inst->readThread == NULL )
        {
            error_code = SUBSYS_PVR_ERR_THREAD_ERROR;
            break;
        }else {
            FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayOpen: open read thread success\n");
        }
    }
    while ( 0 );

    if ( error_code != FAPI_OK )
    {
        if ( inst != NULL )
        {
            (void)pvrPlayFeedClose(inst);

            pvrPlayFreeResources(inst);
            FAPI_SYS_FREE(inst);
        }
    }
    else
    {
        *handlePtr = inst;
    }

    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayOpen: quit.[%d] inst->position[0x%08x]\n", \
                                error_code, inst->position);
    return error_code;
}

int32_t pvrPlayClose (FAPEX_SYS_HandleT handle)
{
    pvrPlayHandleT*     inst = (pvrPlayHandleT*)handle;
    PVR_DecoderTypeT    decIdx;
    int32_t				result;


    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayClose: In\n");
    if ( inst->state != PLAY_STATE_STOPPED )
    {
        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayClose: Out[%d]\n", SUBSYS_PVR_ERR_BAD_MODE);
        return SUBSYS_PVR_ERR_BAD_MODE;
    }

    (void)pvrPlayFeedClose(inst);

    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayClose: feed close done\n");

    FAPEX_TSD_Munmap(inst->input_map, inst->map_size);

    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayClose: unmmap done\n");

    #ifndef _LINUX_FAPI
    (void)RTOS_DestroyThread(inst->readThread);
    #else
    result = FAPEX_SYS_DestroyThread(inst->readThread);
    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayClose: read thread detroy return[%d]\n", result);
    #endif

    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayClose: read thread detroy done\n");

    /* close all target decoders */
    for ( decIdx = (PVR_DecoderTypeT)0; decIdx < PVR_DECTYPE_NUMS; decIdx++ )
    {
        if ( (inst->openParams.targetControl.decoder[decIdx].close != NULL)
          && (inst->stream[decIdx].decHandle                       != NULL) )
        {
            (void)inst->openParams.targetControl.decoder[decIdx].close(
                    decIdx,
                    inst->openParams.targetControl.argPtr,
                    inst->stream[decIdx].decHandle);
        }
    }

    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayClose: decoders close done\n");

    pvrPlayFreeResources(inst);

    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayClose: resource free done\n");

    /*lint -save -e923 */
    #ifndef _LINUX_FAPI
    f_releaseFS((int32_t)inst->readThread);
    #endif
    /*lint -restore */

    FAPI_SYS_FREE(inst);

    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayClose: inst close done\n");

    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayClose: Out[0]\n");
    return FAPI_OK;
}

int32_t pvrPlayNotifyRecordingClose(FAPEX_SYS_HandleT handle,
                                    FAPEX_SYS_HandleT recHandle)
{
    pvrPlayHandleT* inst = (pvrPlayHandleT*)handle;

    if ( inst->recHandle == recHandle )
    {
        inst->recHandle = NULL;
    }

    return FAPI_OK;
}

int32_t pvrPlaySetMode(FAPEX_SYS_HandleT handle, PVR_PlayModeT mode,
                       uint8_t modeIndex)
{
    pvrPlayHandleT*      inst     = (pvrPlayHandleT*)handle;
    int32_t              result   = FAPI_OK;
    PVR_PlayStateT       tmpState = PLAY_STATE_NONE;
    PVR_FastModeConfigT* configPtr;

    //////////////
    PVR_PLAY_LOCK;
    //////////////

    switch ( mode )
    {
        case PVR_PLAY_MODE_STOPPED:

            modeIndex = 0;

            switch ( inst->state )
            {
                case PLAY_STATE_STOPPED:
                    /* do nothing */
                    break;

                case PLAY_STATE_NORMAL:
                case PLAY_STATE_FAST_IFRAMES:
                case PLAY_STATE_FAST_ALL:
                case PLAY_STATE_FAST_NONE:
                case PLAY_STATE_FAST_CONTROLLED:
                case PLAY_STATE_SLOW:
                case PLAY_STATE_PAUSE:
                    inst->lastState = inst->state;
                    inst->state     = PLAY_STATE_TO_STOP;
                    break;

                default:
                    result = SUBSYS_PVR_ERR_BAD_MODE;
                    break;
            }
            break;

        /* ----------------------------------------------------------------- */

        case PVR_PLAY_MODE_NORMAL:

            modeIndex = 0;

            switch ( inst->state )
            {
                case PLAY_STATE_NORMAL:
                    /* do nothing */
                    break;

                case PLAY_STATE_STOPPED:
                case PLAY_STATE_FAST_IFRAMES:
                case PLAY_STATE_FAST_ALL:
                case PLAY_STATE_FAST_NONE:
                case PLAY_STATE_FAST_CONTROLLED:
                case PLAY_STATE_SLOW:
                case PLAY_STATE_PAUSE:
                    inst->lastState = inst->state;
                    inst->state     = PLAY_STATE_TO_NORMAL;
                    break;

                default:
                    result = SUBSYS_PVR_ERR_BAD_MODE;
                    break;
            }
            break;

        /* ----------------------------------------------------------------- */

        case PVR_PLAY_MODE_PAUSE:

            modeIndex = 0;

            switch ( inst->state )
            {
                case PLAY_STATE_PAUSE:
                    /* do nothing */
                    break;

                case PLAY_STATE_STOPPED:
                case PLAY_STATE_NORMAL:
                case PLAY_STATE_FAST_IFRAMES:
                case PLAY_STATE_FAST_ALL:
                case PLAY_STATE_FAST_NONE:
                case PLAY_STATE_FAST_CONTROLLED:
                case PLAY_STATE_SLOW:
                    inst->lastState = inst->state;
                    inst->state     = PLAY_STATE_TO_PAUSE;
                    break;

                default:
                    result = SUBSYS_PVR_ERR_BAD_MODE;
                    break;
            }
            break;

        /* ----------------------------------------------------------------- */

        case PVR_PLAY_MODE_FAST:

            if ( modeIndex >= PVR_MAX_FAST_MODES )
            {
                result = SUBSYS_PVR_ERR_BAD_PARAMETER;
                break;
            }

            configPtr = inst->playDataPtr->fastModeConfig + modeIndex;
            FAPI_SYS_PRINT_MSG("[PLAY] pvrPlaySetMode: type[%d],factor[%d]\n", \
                            configPtr->type, configPtr->params.iFrames.factor);

            switch ( inst->state )
            {
                case PLAY_STATE_STOPPED:
                case PLAY_STATE_NORMAL:
                case PLAY_STATE_SLOW:
                case PLAY_STATE_PAUSE:
                case PLAY_STATE_FAST_IFRAMES:
                case PLAY_STATE_FAST_ALL:
                case PLAY_STATE_FAST_NONE:
                case PLAY_STATE_FAST_CONTROLLED:

                    if ( !(((inst->state == PLAY_STATE_FAST_IFRAMES)
                          && (configPtr->type == PVR_FASTMODETYPE_IFRAMES))
                        || ((inst->state == PLAY_STATE_FAST_ALL)
                          && (configPtr->type == PVR_FASTMODETYPE_FEED_ALL))
                        || ((inst->state == PLAY_STATE_FAST_NONE)
                          && (configPtr->type == PVR_FASTMODETYPE_FEED_NONE))
                        || ((inst->state == PLAY_STATE_FAST_CONTROLLED)
                          && (configPtr->type == PVR_FASTMODETYPE_FEED_CONTROLLED))) )
                    {
                        switch ( configPtr->type )
                        {
                            case PVR_FASTMODETYPE_IFRAMES:
                                tmpState = PLAY_STATE_TO_FAST_IFRAMES;
                                break;

                            case PVR_FASTMODETYPE_FEED_ALL:
                                tmpState = PLAY_STATE_TO_FAST_ALL;
                                break;

                            case PVR_FASTMODETYPE_FEED_NONE:
                                tmpState = PLAY_STATE_TO_FAST_NONE;
                                break;

                            case PVR_FASTMODETYPE_FEED_CONTROLLED:
                                tmpState = PLAY_STATE_TO_FAST_CONTROLLED;
                                break;

                            default:
                                result = SUBSYS_PVR_ERR_BAD_PARAMETER;
                                break;
                        }

                        if ( result == FAPI_OK )
                        {
                            inst->lastState = inst->state;
                            inst->state     = tmpState;
                        }
                    }
                    break;

                default:
                    result = SUBSYS_PVR_ERR_BAD_MODE;
                    break;
            }
            break;

        /* ----------------------------------------------------------------- */

        case PVR_PLAY_MODE_SLOW:

            switch ( inst->state )
            {
                case PLAY_STATE_SLOW:
                    /* do nothing -> only index is changed */
                    break;

                case PLAY_STATE_STOPPED:
                case PLAY_STATE_NORMAL:
                case PLAY_STATE_FAST_IFRAMES:
                case PLAY_STATE_FAST_ALL:
                case PLAY_STATE_FAST_NONE:
                case PLAY_STATE_FAST_CONTROLLED:
                case PLAY_STATE_PAUSE:
                    inst->lastState = inst->state;
                    inst->state     = PLAY_STATE_TO_SLOW;
                    break;

                default:
                    result = SUBSYS_PVR_ERR_BAD_MODE;
                    break;
            }
            break;

        default:
            result = SUBSYS_PVR_ERR_BAD_MODE;
            break;
    }

    if ( result == FAPI_OK )
    {
        inst->reqStateIdx = modeIndex;
    }

    //////////////
    PVR_PLAY_UNLOCK;
    //////////////

    return result;
}

int32_t pvrPlaySeek(FAPEX_SYS_HandleT handle, PVR_SeekModeT mode,
                    uint32_t value)
{
    pvrPlayHandleT* inst       = (pvrPlayHandleT*)handle;
    int32_t         error_code = FAPI_OK;
    uint32_t        seekPosition;

    //////////////
    PVR_PLAY_LOCK;
    //////////////

    switch ( inst->state )
    {
        case PLAY_STATE_STOPPED:

            ////////////////
            PVR_PLAY_UNLOCK;
            ////////////////

            switch ( mode )
            {
                case PVR_SEEK_MODE_CHUNKS:

                    seekPosition = inst->offset + value;

                    while ( seekPosition >= inst->size )
                    {
                        seekPosition -= inst->size;
                    }

                    pvrSeekChunksSet(inst, seekPosition);
                    break;

                case PVR_SEEK_MODE_CHUNKS_FORWARD:

                    if ( !(pvrIsEndOfData(inst)) )
                    {
                        pvrSeekChunksCur(inst, (int32_t)value);
                    }
                    break;

                case PVR_SEEK_MODE_CHUNKS_BACKWARD:

                    if ( !(pvrIsBeginOfData(inst)) )
                    {
                        pvrSeekChunksCur(inst, -((int32_t)value));
                    }
                    break;

                case PVR_SEEK_MODE_TIME:
                    pvrPlaySeekToTime(inst, value);
                    break;

                // TODO: Support also other seek operations from stop mode!

                default:
                    error_code = SUBSYS_PVR_ERR_BAD_PARAMETER;
                    break;
            }

            //////////////
            PVR_PLAY_LOCK;
            //////////////
            break;

        /* ----------------------------------------------------------------- */

        case PLAY_STATE_PAUSE:

            switch ( mode )
            {
                case PVR_SEEK_MODE_CHUNKS:
                    inst->seekPosition = inst->offset + value;

                    while ( inst->seekPosition >= inst->size )
                    {
                        inst->seekPosition -= inst->size;
                    }

                    inst->reqStateIdx = PLAY_SEEK_POS_ABS;
                    break;

                case PVR_SEEK_MODE_CHUNKS_FORWARD:
                    inst->seekChunks  = (int32_t)value;
                    inst->reqStateIdx = PLAY_SEEK_POS_REL;
                    break;

                case PVR_SEEK_MODE_CHUNKS_BACKWARD:
                    inst->seekChunks  = -((int32_t)value);
                    inst->reqStateIdx = PLAY_SEEK_POS_REL;
                    break;

                case PVR_SEEK_MODE_NAME_FORWARD:
                    inst->reqStateIdx = PLAY_SEEK_NAME_FORWARD;
                    break;

                case PVR_SEEK_MODE_NAME_BACKWARD:
                    inst->reqStateIdx = PLAY_SEEK_NAME_BACKWARDS;
                    break;

                case PVR_SEEK_MODE_TAG_FORWARD:
                    inst->reqStateIdx = PLAY_SEEK_TAG_FORWARD;
                    break;

                case PVR_SEEK_MODE_TAG_BACKWARD:
                    inst->reqStateIdx = PLAY_SEEK_TAG_BACKWARDS;
                    break;

                case PVR_SEEK_MODE_IFRAME_FORWARD:
                    inst->seekChunks  = 1;
                    inst->reqStateIdx = PLAY_SEEK_IFRAME_FORWARD;
                    break;

                case PVR_SEEK_MODE_IFRAME_BACKWARD:
                    inst->seekChunks  = -1;
                    inst->reqStateIdx = PLAY_SEEK_IFRAME_BACKWARDS;
                    break;

                case PVR_SEEK_MODE_TIME:
                    inst->seekTime    = value;
                    inst->reqStateIdx = PLAY_SEEK_TIME_ABS;
                    break;

                default:
                    error_code = SUBSYS_PVR_ERR_BAD_PARAMETER;
                    break;
            }

            if ( error_code == FAPI_OK )
            {
                inst->state = PLAY_STATE_SEEK;
            }
            break;

        default:
            error_code = SUBSYS_PVR_ERR_BAD_MODE;
            break;
    }

    ////////////////
    PVR_PLAY_UNLOCK;
    ////////////////

    return error_code;
}

int32_t pvrPlaySetTrack(FAPEX_SYS_HandleT handle, PVR_DecoderTypeT decoderType,
                        uint8_t track, uint32_t trackData)
{
    pvrPlayHandleT*     inst   = (pvrPlayHandleT*)handle;
    int32_t             result = FAPI_OK;
    PVR_DecoderTypeT    decIdx;

    if ( (track >= inst->fileInfo.numberOfTracks) && (track != 0xFF) )
    {
        result = SUBSYS_PVR_ERR_BAD_PARAMETER;
    }
    else if ( decoderType >= PVR_DECTYPE_NUMS )
    {
        result = SUBSYS_PVR_ERR_BAD_PARAMETER;
    }
    else
    {
        inst->stream[decoderType].trackIndex = track;
        inst->stream[decoderType].trackData  = trackData;

        if ( track != 0xFF )
        {
            /* exit from potentially active slow feed mode if a track
               is selected */
            inst->slowFeed = 0;
        }
        else
        {
            /* enter slow feed mode if no more tracks are selected */
            for ( decIdx = (PVR_DecoderTypeT)0; decIdx < PVR_DECTYPE_NUMS;
                  decIdx++ )
            {
                if ( inst->stream[decIdx].trackIndex != 0xFF )
                {
                    break;
                }
            }

            if ( decIdx >= PVR_DECTYPE_NUMS )
            {
                inst->slowFeed = 1;
            }
        }
    }

    return result;
}

int32_t pvrPlayGetTrack(FAPEX_SYS_HandleT handle, PVR_DecoderTypeT decoderType,
                        uint8_t* trackPtr)
{
    pvrPlayHandleT* inst   = (pvrPlayHandleT*)handle;
    int32_t         result = FAPI_OK;

    if ( decoderType >= PVR_DECTYPE_NUMS )
    {
        result = SUBSYS_PVR_ERR_BAD_PARAMETER;
    }
    else
    {
        *trackPtr = inst->stream[decoderType].trackIndex;

        if ( (*trackPtr) == 0xFF )
        {
            result = SUBSYS_PVR_ERR_NO_TRACK_ASSIGNED;
        }
    }

    return result;
}

int32_t pvrPlayGetFileInfo(FAPEX_SYS_HandleT handle,
                           PVR_FileInfoT* fileInfoPtr)
{
    pvrPlayHandleT* inst   = (pvrPlayHandleT*)handle;

    if ( fileInfoPtr == NULL )
    {
        return SUBSYS_PVR_ERR_BAD_PARAMETER;
    }

    //////////////
    PVR_PLAY_LOCK;
    //////////////

    inst->fileInfo.size   = inst->size;
    inst->fileInfo.offset = inst->offset;
    *fileInfoPtr          = inst->fileInfo;

    ////////////////
    PVR_PLAY_UNLOCK;
    ////////////////

    return FAPI_OK;
}

int32_t pvrPlayGetPlayInfo(FAPEX_SYS_HandleT handle, PVR_PlayInfoT* playInfoPtr)
{
    pvrPlayHandleT* inst   = (pvrPlayHandleT*)handle;

    if ( playInfoPtr == NULL )
    {
        return SUBSYS_PVR_ERR_BAD_PARAMETER;
    }

    //////////////
    PVR_PLAY_LOCK;
    //////////////

    playInfoPtr->isTimeShifting = (inst->recHandle != NULL);
    playInfoPtr->size           = inst->size;
    playInfoPtr->offset         = inst->offset;
    playInfoPtr->position       = inst->position;
    playInfoPtr->duration       = inst->fileInfo.duration;
    playInfoPtr->time           = inst->header.time;
    playInfoPtr->tag            = inst->header.tag;

    strcpy(playInfoPtr->name, inst->header.name);

    playInfoPtr->dataRate       = inst->header.dataRate;

    ////////////////
    PVR_PLAY_UNLOCK;
    ////////////////

    return FAPI_OK;
}

int32_t pvrPlayGetTrackInfo(FAPEX_SYS_HandleT handle,
                            PVR_TrackInfoT* trackInfoPtr)
{
    pvrPlayHandleT* inst     = (pvrPlayHandleT*)handle;
    uint8_t         trackCnt = 0;

    if ( trackInfoPtr == NULL )
    {
        return SUBSYS_PVR_ERR_BAD_PARAMETER;
    }

    //////////////
    PVR_PLAY_LOCK;
    //////////////

    trackInfoPtr->numberOfTracks = (uint8_t)(inst->header.numberOfTracks);

    for ( trackCnt = 0; trackCnt < PVR_MAX_TRACKS; trackCnt++ )
    {
        trackInfoPtr->trackType  [trackCnt] = (PVR_TrackTypeT)
                                              inst->header.trackType[trackCnt];
        trackInfoPtr->trackInfo  [trackCnt] = inst->header.trackInfo[trackCnt];
        trackInfoPtr->trackPid   [trackCnt] = inst->header.trackPid [trackCnt];
        trackInfoPtr->trackFormat[trackCnt] = pvrGetTrackFormat(&(inst->header),
                                                                trackCnt);
    }

    ////////////////
    PVR_PLAY_UNLOCK;
    ////////////////

    return FAPI_OK;
}

int32_t pvrPlayEnableFrameView(FAPEX_SYS_HandleT handle, uint8_t enable)
{
    ((pvrPlayHandleT*)handle)->enableFrameView = enable;

    return FAPI_OK;
}

uint8_t pvrPlayGetTsd(FAPEX_SYS_HandleT handle)
{
    return ((pvrPlayHandleT*)handle)->openParams.tsdBlockIndex;
}

int32_t pvrPlaySetPlayFunction(FAPEX_SYS_HandleT handle,
                               PVR_PlaybackFeedOutput function)
{
    pvrPlayHandleT* inst   = (pvrPlayHandleT*)handle;
    int32_t         result = FAPI_OK;

    //////////////
    PVR_PLAY_LOCK;
    //////////////

    if ( inst->state != PLAY_STATE_STOPPED )
    {
        result = SUBSYS_PVR_ERR_BAD_MODE;
    }
    else
    {
        if ( function == NULL )
        {
            inst->feedOutput = pvrFeedTsd;
        }
        else
        {
            inst->feedOutput = function;
        }
    }

    ////////////////
    PVR_PLAY_UNLOCK;
    ////////////////

    return result;
}

int32_t pvrPlaySetInternalMode (FAPEX_SYS_HandleT handle)
{
    ((pvrPlayHandleT*)handle)->internalMode = 1;

    return FAPI_OK;
}

int32_t pvrPlayEnableDecryption (FAPEX_SYS_HandleT handle,
                                 const PVR_CryptoParamsT* paramsPtr)
{
    pvrPlayHandleT* inst   = (pvrPlayHandleT*)handle;
    int32_t         result = FAPI_OK;

    //////////////
    PVR_PLAY_LOCK;
    //////////////

    if ( paramsPtr == NULL )
    {
        result = SUBSYS_PVR_ERR_BAD_PARAMETER;
    }
    else if ( inst->state != PLAY_STATE_STOPPED )
    {
        result = SUBSYS_PVR_ERR_BAD_MODE;
    }
    else
    {
        /* NOTE: Resources are only allocated once, but it is possible to
                 call this function multiple times in order to modify
                 decryption parameters. */
        if ( inst->decryptBuffer == NULL )
        {
            #ifndef _LINUX_FAPI
            inst->decryptBuffer = FAPI_SYS_MALLOC_UNCACHED(PVR_CHUNK_SIZE);
            #else
            inst->decryptBuffer = FAPI_SYS_MALLOC_CACHED(PVR_CHUNK_SIZE);
            #endif
            if ( inst->decryptBuffer == NULL )
            {
                result = SUBSYS_PVR_ERR_OUT_OF_MEMORY;
            }
        }
        if ( inst->decryptBuffer != NULL )
        {
            pvrSetCryptoKeys (paramsPtr, &(inst->decryptKeys));
        }
    }

    ////////////////
    PVR_PLAY_UNLOCK;
    ////////////////

    return result;
}

//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

static void pvrPlayFreeResources(pvrPlayHandleT* inst)
{
    if ( inst != NULL )
    {
        pvrIFrameChannelClose(inst);

        if (inst->iframeTmpBuffer != NULL)
	    {
	        free(inst->iframeTmpBuffer);
	        inst->iframeTmpBuffer = NULL;

			FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayFreeResources: IFrame Temp Buffer Release succes\n");
	    }

        if ( inst->playSemaphore != 0 )
        {
            FAPI_SYS_DESTROY_SEMAPHORE(inst->playSemaphore);
        }

        if ( inst->decryptBuffer != NULL )
        {
            FAPI_SYS_FREE(inst->decryptBuffer);
        }

        if ( inst->chunkBuffer != NULL )
        {
        	#ifndef STATIC_CHUNK_BUFFER
            FAPI_SYS_FREE(inst->chunkBuffer);
            #else
            inst->chunkBuffer = NULL;
            memset(pvrPlayStaticChunkBuffer, 0, sizeof(pvrPlayStaticChunkBuffer));
            #endif
        }

        if ( inst->tmpHeader != NULL )
        {
            FAPI_SYS_FREE(inst->tmpHeader);
        }

        if ( inst->filePtr != NULL )
        {
            (void)f_close(inst->filePtr);
        }
    }
}

int32_t pvrPlayGetState (FAPEX_SYS_HandleT handle,
                         PVR_PlayStateInfoT* stateInfoPtr)
{
    pvrPlayHandleT* inst = (pvrPlayHandleT*)handle;

    if ( inst == NULL ) return SUBSYS_PVR_ERR_BAD_PARAMETER;

    //////////////
    PVR_PLAY_LOCK;
    //////////////

    if ( stateInfoPtr != NULL )
    {
        stateInfoPtr->state       = inst->state;
        stateInfoPtr->lastState   = inst->lastState;
        stateInfoPtr->stateIdx    = inst->stateIndex;
        stateInfoPtr->reqStateIdx = inst->reqStateIdx;
    }

    //////////////
    PVR_PLAY_UNLOCK;
    //////////////

    return FAPI_OK;
}

static void pvrPlaySetState (pvrPlayHandleT* inst, PVR_PlayStateT state,
                             uint8_t stateIndex)
{
    //////////////
    PVR_PLAY_LOCK;
    //////////////

    inst->lastState  = inst->state;
    inst->state      = state;
    inst->stateIndex = stateIndex;

    //////////////
    PVR_PLAY_UNLOCK;
    //////////////
}

static void pvrPlayReadThreadFunction(void* optArgPtr)
{
    pvrPlayHandleT*     inst      = (pvrPlayHandleT*)optArgPtr;
    PVR_PlayStateT      prevState = PLAY_STATE_NONE;
    PVR_PlayStateInfoT  stateInfo;

    #ifndef _LINUX_FAPI
    (void)f_enterFS();
    #endif

    memset(&stateInfo, 0, sizeof(stateInfo));

    stateInfo.state     = PLAY_STATE_STOPPED;
    stateInfo.lastState = PLAY_STATE_STOPPED;


    FAPI_SYS_PRINT_DEBUG(1, "[READ] pvrPlayReadThreadFunction: In\n");
    while ( 1 )
    {
        /* support time shifting and check if current position is overwritten */
        if ( inst->recHandle != NULL )
        {
            if ( !(pvrSupportTimeShifting(inst)) )
            {
                /* current playback position overwritten
                   -> start normal playback */

                FAPI_SYS_PRINT_DEBUG(1, "[READ] pvrPlayReadThreadFunction: support time shifting 2\n");
                pvrStopMedia(inst);
                pvrTrackStop(inst, PVR_DECTYPE_VIDEO, PLAY_STOPMODE_BLANK);
                (void)pvrSyncStop (inst);

                pvrIFrameChannelStop(inst);

                /* make sure internal file pointers are updated */
                (void)pvrSupportTimeShifting(inst);

                /* seek a few clusters ahead of current write position */
                (void)pvrPlayClearBuffer(inst);
                pvrSeekChunksSet(inst, inst->offset);
                pvrSeekChunksCur(inst, 4);

                /* start normal playback */

                //////////////
                PVR_PLAY_LOCK;
                //////////////

                inst->state       = PLAY_STATE_TO_NORMAL;
                inst->lastState   = PLAY_STATE_STOPPED;
                inst->stateIndex  = 0;
                inst->reqStateIdx = 0;

                //////////////
                PVR_PLAY_UNLOCK;
                //////////////

                pvrPlayNotify(inst, PVR_PLAY_NOTIFY_BEGIN_OVERWRITTEN);
            }
        }

        (void)pvrPlayGetState((FAPEX_SYS_HandleT)inst, &stateInfo);

        if ( stateInfo.state != prevState )
        {
            #if (DEBUG >= 4)

            FAPI_SYS_PRINT_DEBUG(4, "[READ] pvrPlayReadThreadFunction: PLAY_STATE_%s (Last PLAY_STATE_%s)\n",
                                 inst->playDataPtr->stateString[stateInfo.state],
                                 inst->playDataPtr->stateString[stateInfo.lastState]);
            #endif /* (DEBUG >= 4) */
        }

        prevState = stateInfo.state;

        /*lint -save -e522 -e961 -e40 -e10 */
        assert(stateInfo.state < PLAY_STATE_NUMS);
        /*lint -restore */

        if ( inst->playDataPtr->stateHandler[stateInfo.state] != NULL )
        {
            inst->playDataPtr->stateHandler[stateInfo.state](inst, &stateInfo);
        }
        else
        {
            switch ( stateInfo.state )
            {
                case PLAY_STATE_TO_STOP:

                    pvrIFrameChannelStop(inst);

                    pvrStopMedia(inst);
                    pvrTrackStop(inst, PVR_DECTYPE_VIDEO, PLAY_STOPMODE_BLANK);
                    (void)pvrSyncStop(inst);

                    (void)pvrPlayFeedSetState(inst, FEED_STATE_STOPPED, 0);
                    (void)pvrPlayClearBuffer (inst);

                    pvrPlaySetState(inst, PLAY_STATE_STOPPED, 0);
                    break;

                case PLAY_STATE_TO_NORMAL:

                    pvrIFrameChannelStop(inst);

                    if ( stateInfo.lastState == PLAY_STATE_FAST_IFRAMES )
                    {
                        (void)pvrPlayClearBuffer(inst);
                    }

                    (void)pvrPlayFeedSetState(inst, FEED_STATE_ACTIVE, 0);

                    pvrStartAll(inst, PVR_DECMODE_NORMAL, 0);
                    (void)pvrSyncStart (inst);

                    inst->normEod = 0;

                    memset (inst->normEndFilling, 0,
                            sizeof(inst->normEndFilling));

                    pvrPlaySetState(inst, PLAY_STATE_NORMAL, 0);
                    break;

                case PLAY_STATE_TO_SLOW:

                    pvrIFrameChannelStop(inst);

                    if ( stateInfo.lastState == PLAY_STATE_FAST_IFRAMES )
                    {
                        (void)pvrPlayClearBuffer(inst);
                    }

                    (void)pvrPlayFeedSetState(inst, FEED_STATE_ACTIVE, 0);

                    pvrStopMedia (inst);
                    pvrTrackStart(inst, PVR_DECTYPE_VIDEO,
                                  PVR_DECMODE_SLOW, stateInfo.reqStateIdx);
                    (void)pvrSyncStop(inst);

                    pvrPlaySetState(inst, PLAY_STATE_SLOW, stateInfo.reqStateIdx);
                    break;

                case PLAY_STATE_TO_FAST_IFRAMES:

                    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] read thread: ===> PLAY_STATE_TO_FAST_IFRAMES\n");
                    pvrStopMedia     (inst);
                    (void)pvrSyncStop(inst);

                    (void)pvrPlayFeedSetState(inst, FEED_STATE_ACTIVE, 0);

                    (void)pvrPlayClearBuffer(inst);


                    pvrTrackStop(inst, PVR_DECTYPE_VIDEO, PLAY_STOPMODE_FREEZE);

                    pvrTrackStart    (inst, PVR_DECTYPE_VIDEO,
                                      PVR_DECMODE_FAST, stateInfo.reqStateIdx);

					/* Martin -- 20100712 -- MPEG I Frame FF*/
                    /*
                    			  * stop videc tsd channel but not stop the video decoder
                    			  */
                    #if 1
                    if (pvrDecoderGetTsdFd(inst,PVR_DECTYPE_VIDEO) != -1)
                    {
                        FAPEX_TSD_Ioctl(pvrDecoderGetTsdFd(inst,PVR_DECTYPE_VIDEO),\
                                        FAPEX_IOCCOM_STOP,
                                        (FAPEXIOCARG3_T)NULL);
                    }
                    #endif

                    pvrPlayThreadFastIFramesReset(inst, stateInfo.reqStateIdx);

                    pvrIFrameChannelStart(inst);

                    pvrPlaySetState(inst, PLAY_STATE_FAST_IFRAMES,
                                    stateInfo.reqStateIdx);
                    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] read thread: ===> PLAY_STATE_FAST_IFRAMES\n");
                    break;

                case PLAY_STATE_TO_FAST_ALL:

                    pvrIFrameChannelStop(inst);

                    if ( stateInfo.lastState == PLAY_STATE_FAST_IFRAMES )
                    {
                        (void)pvrPlayClearBuffer(inst);
                    }

                    (void)pvrPlayFeedSetState(inst, FEED_STATE_ACTIVE, 0);

                    pvrStopMedia     (inst);
                    pvrTrackStart    (inst, PVR_DECTYPE_VIDEO,
                                      PVR_DECMODE_FAST, stateInfo.reqStateIdx);
                    (void)pvrSyncStop(inst);

                    pvrPlaySetState(inst, PLAY_STATE_FAST_ALL,
                                    stateInfo.reqStateIdx);
                    break;

                case PLAY_STATE_TO_FAST_NONE:

                    pvrIFrameChannelStop(inst);

                    pvrStopMedia     (inst);
                    pvrTrackStop     (inst, PVR_DECTYPE_VIDEO,
                                      PLAY_STOPMODE_BLANK);
                    (void)pvrSyncStop(inst);

                    (void)pvrPlayFeedSetState(inst, FEED_STATE_STOPPED, 0);
                    (void)pvrPlayClearBuffer (inst);

                    pvrPlayThreadFastNoneReset(inst, stateInfo.reqStateIdx);

                    pvrPlaySetState(inst, PLAY_STATE_FAST_NONE,
                                    stateInfo.reqStateIdx);
                    break;

                case PLAY_STATE_TO_FAST_CONTROLLED:

                    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] read thread: ===> PLAY_STATE_TO_FAST_CONTROLLED\n");

                    pvrIFrameChannelStop(inst);

                    if ( stateInfo.lastState == PLAY_STATE_FAST_IFRAMES )
                    {
                        (void)pvrPlayClearBuffer(inst);
                    }

                    (void)pvrPlayFeedSetState(inst, FEED_STATE_ACTIVE, 0);

                    pvrStopMedia     (inst);
                    pvrTrackStart    (inst, PVR_DECTYPE_VIDEO,
                                      PVR_DECMODE_FAST, stateInfo.reqStateIdx);
                    (void)pvrSyncStop(inst);

                    pvrPlayThreadFastControlledReset(inst,
                                                     stateInfo.reqStateIdx);

                    pvrPlaySetState(inst, PLAY_STATE_FAST_CONTROLLED,
                                    stateInfo.reqStateIdx);
                    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] read thread: ===> PLAY_STATE_FAST_CONTROLLED\n");
                    break;

                case PLAY_STATE_TO_PAUSE:

					FAPI_SYS_PRINT_DEBUG(1, "[PLAY] read thread: ===> PLAY_STATE_TO_PAUSE\n");
                    pvrIFrameChannelStop(inst);

                    if ( (stateInfo.lastState == PLAY_STATE_FAST_IFRAMES)
                      || (stateInfo.lastState == PLAY_STATE_FAST_ALL) )
                    {
                        /* reset video buffer pointer to start with clean data */
                        // FIXME: necessary?
                        pvrTrackResetBuffer(inst, PVR_DECTYPE_VIDEO);
                    }

                    (void)pvrPlayFeedSetState(inst, FEED_STATE_ACTIVE, 0);

                    pvrStartAll       (inst, PVR_DECMODE_PAUSE, 0);
                    (void)pvrSyncStop (inst);

                    /* update playback position */
                    (void)pvrReadHeader(inst, &(inst->header));
                    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] read thread: pvrReadHeader done\n");

                    pvrPlaySetState(inst, PLAY_STATE_PAUSE, 0);
                    break;

                /* --------------------------------------------------------- */

                default:
                    pvrPlaySetState(inst, PLAY_STATE_STOPPED, 0);
                    break;
            }
        }

        /*--------------------------------*/
        /* always check and notify events */
        /*--------------------------------*/

        if ( inst->header.nodataCounter != inst->lastNoDataCounter )
        {
            inst->lastNoDataCounter = inst->header.nodataCounter;
            pvrPlayNotify(inst, PVR_PLAY_NOTIFY_GAP_IN_DATA);
        }

        if ( inst->header.tagCounter != inst->lastTagCounter )
        {
            inst->lastTagCounter = inst->header.tagCounter;
            pvrPlayNotify(inst, PVR_PLAY_NOTIFY_TAG_SET);
        }

        if ( inst->header.nameCounter != inst->lastNameCounter )
        {
            inst->lastNameCounter = inst->header.nameCounter;
            pvrPlayNotify(inst, PVR_PLAY_NOTIFY_NAME_SET);
        }

    } /* while (1) */
    FAPI_SYS_PRINT_DEBUG(1, "[READ] pvrPlayReadThreadFunction: Out\n");
}

/*lint -save -e715 */
static void pvrPlayThreadStoppedTask (void* dataPtr,
                                      const PVR_PlayStateInfoT* stateInfoPtr)
{
    FAPI_SYS_SLEEP(50);
}
/*lint -restore */

/*lint -save -e715 */
static void pvrPlayThreadNormalTask (void* dataPtr,
                                     const PVR_PlayStateInfoT* stateInfoPtr)
{
    pvrPlayHandleT*     inst = (pvrPlayHandleT*)dataPtr;
    int32_t             res;
    uint32_t            sleepTime = 0;
    uint32_t            currTime;
    PVR_DecoderTypeT    decType;
    uint32_t            filling;
    uint8_t             playEnd;



    //FAPI_SYS_PRINT_DEBUG(4, "[Play] Normal Task In, inst->pos (0x%08x)\n", inst->position);
    if ( !(inst->normEod) )
    {
        /* read data from disk */
        res = pvrReadData(inst, &(inst->header));

        if ( res == SUBSYS_PVR_ERR_FILE_READ)
        {
            pvrSeekChunksSet(inst, inst->position);

            pvrPlaySetState (inst, PLAY_STATE_TO_PAUSE, 0);
            pvrPlayNotify   (inst, PVR_PLAY_NOTIFY_FILE_ERROR);
            FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayThreadNormalTask: Out: File Error)\n");
            return;
        }
        else if ( res == SUBSYS_PVR_ERR_END_OF_DATA)
        {
            inst->normEod          = 1;
            inst->normEodTimeStamp = 0;
        }
        else if ( res == SUBSYS_PVR_ERR_NO_BUFFER_SPACE )
        {
            sleepTime = PVR_PLAY_CHUNKBUFFULL_SLEEPTIME;

            /* No data read because of full chunk buffer: Adjust playback time
               	(e.g. for radio playback where one data chunk represents more than
               	one second). */
            currTime = (uint32_t)pvrGetTimeStamp();
            if ( currTime > inst->read_data_ts )
            {
                inst->header.time = inst->read_data_header_time
                                  + (currTime - inst->read_data_ts);
            }
        }
        else
        {
            inst->read_data_header_time = inst->header.time;
            inst->read_data_ts          = (uint32_t)pvrGetTimeStamp();
        }
    }
    else
    {
        /*ensure that not in timeshift mode*/
        if (inst->recHandle != NULL)
        {
            if (inst->size > inst->position)
            {
                FAPI_SYS_PRINT_DEBUG(1, "[READ] pvrPlayThreadNormalTask: size changed detected:size[%d] position[%d]\n",\
                                                    inst->size, inst->position);
                inst->normEod = 0;
                return;
            }
        }

        /* ensure that all data has been fed from chunk buffer */
        if ( inst->chunkBufferWp == inst->chunkBufferRp )
        {
            if ( inst->normEodTimeStamp == 0 )
            {
                inst->normEodTimeStamp = pvrGetTimeStamp();
            }

            playEnd = 1;

            /* wait for decoder input buffers to empty */
            for ( decType = (PVR_DecoderTypeT)0; decType < PVR_DECTYPE_NUMS;
                  decType++ )
            {
                if ( pvrTrackIsActive(inst, decType) )
                {
                    filling = pvrDecoderGetBufLevel(inst, decType);

                    FAPI_SYS_PRINT_DEBUG(2, "[READ] pvrPlayThreadNormalTask: filling[%d] = [%d]\n", decType, filling);
                    if ( (filling > 0)
                      && (filling != inst->normEndFilling[decType]) )
                    {
                        inst->normEndFilling[decType] = filling;
                        playEnd = 0;
                        break;
                    }
                }
            }

            if ( playEnd )
            {
                pvrPlaySetState (inst, PLAY_STATE_TO_PAUSE, 0);
                pvrPlayNotify   (inst, PVR_PLAY_NOTIFY_END_OF_DATA);
                FAPI_SYS_PRINT_DEBUG(1,"pvrPlayNotify called at %s %d\n", __FILE__, __LINE__);
                FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayThreadNormalTask: Out: End of Data\n");
                return;
            }
        }

        sleepTime = PVR_PLAY_ENDOFDATA_SLEEPTIME;
    }

    if ( pvrCheckForParamChange(inst, PVR_DECTYPE_VIDEO)
      || (inst->header.nodataCounter != inst->lastNoDataCounter) )
    {
        pvrStopMedia(inst);
        pvrTrackStop(inst, PVR_DECTYPE_VIDEO, PLAY_STOPMODE_BLANK);

        pvrStartAll (inst, PVR_DECMODE_NORMAL, 0);
    }
    else
    {
        /* control of audio: PID or track changed? */
        if ( pvrCheckForParamChange(inst, PVR_DECTYPE_AUDIO) )
        {
            pvrTrackStop    (inst, PVR_DECTYPE_AUDIO,
                             PLAY_STOPMODE_NONE);
            pvrTrackStart   (inst, PVR_DECTYPE_AUDIO,
                             PVR_DECMODE_NORMAL, 0);
        }
        else if ( pvrCheckForTrackStop(inst, PVR_DECTYPE_AUDIO) )
        {
            pvrTrackStop(inst, PVR_DECTYPE_AUDIO, PLAY_STOPMODE_NONE);
        }

        /* control of second audio: PID or track changed? */
        if ( pvrCheckForParamChange(inst, PVR_DECTYPE_AUDIO_2ND) )
        {
            pvrTrackStop    (inst, PVR_DECTYPE_AUDIO_2ND,
                             PLAY_STOPMODE_NONE);
            pvrTrackStart   (inst, PVR_DECTYPE_AUDIO_2ND,
                             PVR_DECMODE_NORMAL, 0);
        }
        else if ( pvrCheckForTrackStop (inst, PVR_DECTYPE_AUDIO_2ND) )
        {
            pvrTrackStop (inst, PVR_DECTYPE_AUDIO_2ND, PLAY_STOPMODE_NONE);
        }

        /* control of Teletext: PID or track changed? */
        if ( pvrCheckForParamChange(inst, PVR_DECTYPE_TTX) )
        {
            pvrTrackStop    (inst, PVR_DECTYPE_TTX, PLAY_STOPMODE_NONE);
            pvrTrackStart   (inst, PVR_DECTYPE_TTX, PVR_DECMODE_NORMAL, 0);
        }
        else if ( pvrCheckForTrackStop(inst, PVR_DECTYPE_TTX) )
        {
            pvrTrackStop (inst, PVR_DECTYPE_TTX, PLAY_STOPMODE_NONE);
        }
    }

    pvrDecoderTasks(inst); /* perform decoder-specific tasks */

    if ( sleepTime > 0 )
    {
        FAPI_SYS_SLEEP(sleepTime);
    }

    //FAPI_SYS_PRINT_DEBUG(2, "[Play][Task] Normal Task Out\n");
}
/*lint -restore */

static void pvrPlayThreadSlowTask (void* dataPtr,
                                   const PVR_PlayStateInfoT* stateInfoPtr)
{
    pvrPlayHandleT* inst = (pvrPlayHandleT*)dataPtr;
    int32_t         res;
    uint32_t        sleepTime = 0;
    uint32_t        videoSpaceRequired;

    /* test for index changes */
    if ( stateInfoPtr->reqStateIdx != stateInfoPtr->stateIdx )
    {
        pvrPlaySetState(inst, stateInfoPtr->state, stateInfoPtr->reqStateIdx);

        pvrTrackSetMode(inst, PVR_DECTYPE_VIDEO,
                        PVR_DECMODE_SLOW, stateInfoPtr->reqStateIdx);
    }

    if ( !(pvrTrackIsActive(inst, PVR_DECTYPE_VIDEO)) )
    {
    	FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayThreadSlowTask: video track is inactive\n");
        sleepTime = 100;
    }
    else
    {
        /* NOTE: It is tried to keep video decoder buffer level as low as
                 possible. This improves resyncing with audio when going back
                 to normal mode. */

        if ( !(pvrDecoderIsDataDecoded (inst, PVR_DECTYPE_VIDEO)) )
        {
            videoSpaceRequired = pvrDecoderGetBufSize(inst, PVR_DECTYPE_VIDEO)
                                 / 2;
        }
        else
        {
            videoSpaceRequired = (pvrDecoderGetBufSize(inst,
                                  PVR_DECTYPE_VIDEO) * 6) / 7;
        }

        if ( videoSpaceRequired < PVR_CHUNK_SIZE )
        {
            videoSpaceRequired = PVR_CHUNK_SIZE;
        }

        inst->feedMinSpace = videoSpaceRequired;

        res = pvrReadData(inst, &(inst->header));

        if ( res == SUBSYS_PVR_ERR_FILE_READ )
        {
            pvrSeekChunksSet(inst, inst->position);

            pvrPlaySetState(inst, PLAY_STATE_TO_PAUSE, 0);

            pvrPlayNotify(inst, PVR_PLAY_NOTIFY_FILE_ERROR);
            return;
        }
        else if ( res == SUBSYS_PVR_ERR_END_OF_DATA )
        {
            // FIXME: changing of state to be improved!
            pvrPlaySetState(inst, PLAY_STATE_TO_PAUSE, 0);

            pvrPlayNotify(inst, PVR_PLAY_NOTIFY_END_OF_DATA);
            FAPI_SYS_PRINT_DEBUG(1,"pvrPlayNotify called at %s %d\n", __FILE__, __LINE__);
            return;
        }
        else if ( res == SUBSYS_PVR_ERR_NO_BUFFER_SPACE )
        {
        	//FAPI_SYS_PRINT_DEBUG(4, "[PLAY] pvrPlayThreadSlowTask: no buffer space\n");
            sleepTime = PVR_PLAY_CHUNKBUFFULL_SLEEPTIME * 2;
        }

        if ( pvrCheckForParamChange(inst, PVR_DECTYPE_VIDEO)
          || (inst->header.nodataCounter != inst->lastNoDataCounter) )
        {
            /* stop and close video and restore stop mode */
            pvrTrackStop (inst, PVR_DECTYPE_VIDEO, PLAY_STOPMODE_BLANK);
            pvrTrackStart(inst, PVR_DECTYPE_VIDEO, PVR_DECMODE_SLOW,
                          stateInfoPtr->reqStateIdx);
        }
    }

    pvrDecoderTasks(inst); /* perform decoder-specific tasks */

    if ( sleepTime > 0 )
    {
        FAPI_SYS_SLEEP(sleepTime);
    }
}

static void pvrPlayThreadFastIFramesReset (pvrPlayHandleT* inst,
                                           uint8_t fastModeIdx)
{
    inst->fastIFramesLastStamp    = 0;
    inst->fastIFramesCurrFactor   = inst->playDataPtr->fastModeConfig
                                          [fastModeIdx].params.iFrames.factor;
    inst->fastIFramesLastPlayTime = inst->header.time;

    /* set initial seek offset */
    if ( inst->fastIFramesCurrFactor >= 0 )
    {
        inst->fastIFramesChunkSeek = inst->fastIFramesCurrFactor;
    }
    else
    {
        inst->fastIFramesChunkSeek = -(inst->fastIFramesCurrFactor);
    }
}

static void pvrPlayThreadFastIFramesTask (void* dataPtr,
                                          const PVR_PlayStateInfoT* stateInfoPtr)
{
    pvrPlayHandleT*     inst         = (pvrPlayHandleT*)dataPtr;
    int32_t             res          = FAPI_OK;
    PVR_PlayNotifyT     notifyReason = PVR_PLAY_NOTIFY_NONE;
    PVR_FastModeConfigT currFastMode;
    uint8_t             toPause = 0;
    int32_t             playTimeDiff;
    int32_t             realTimeDiff;

    /* test for index changes */
    if ( stateInfoPtr->reqStateIdx != stateInfoPtr->stateIdx )
    {
        /* reset calculation if speed factor is different */
        if ( inst->fastIFramesCurrFactor !=
             inst->playDataPtr->fastModeConfig[stateInfoPtr->reqStateIdx].
             params.iFrames.factor )
        {
            pvrPlayThreadFastIFramesReset(inst, stateInfoPtr->reqStateIdx);
        }

        pvrPlaySetState(inst, stateInfoPtr->state,
                        stateInfoPtr->reqStateIdx);
        pvrTrackSetMode(inst, PVR_DECTYPE_VIDEO,
                        PVR_DECMODE_FAST, stateInfoPtr->reqStateIdx);
    }

    // TODO: to be protected!
    currFastMode = inst->playDataPtr->fastModeConfig[stateInfoPtr->reqStateIdx];

    if ( pvrIsEndOfData(inst)
      && (inst->fastIFramesCurrFactor >= 0) )
    {
        /* end of data reached */
        notifyReason = PVR_PLAY_NOTIFY_END_OF_DATA;

        pvrPlaySingleIFrame(inst, NULL);

        toPause = 1;
    }
    else if ( pvrIsBeginOfData(inst)
           && (inst->fastIFramesCurrFactor < 0) )
    {
        /* beginning of data reached */
        notifyReason = PVR_PLAY_NOTIFY_BEGIN_OF_DATA;

        pvrPlaySingleIFrame(inst, NULL);

        toPause = 1;
    }
    else
    {
        if ( inst->fastIFramesCurrFactor >= 0 )
        {
            pvrSeekChunksCur(inst, inst->fastIFramesChunkSeek);
        }
        else
        {
            pvrSeekChunksCur(inst, -(inst->fastIFramesChunkSeek));
        }

        if ( pvrTrackIsActive(inst, PVR_DECTYPE_VIDEO) )
        {
            if ( inst->fastIFramesCurrFactor >= 0 )
            {
                res = pvrPlayMPEGIFrame(inst, PLAY_SEEKMODE_NONE,
                                        currFastMode.params.iFrames.displayTime);
            }
            else
            {
                res = pvrPlayMPEGIFrame(inst, PLAY_SEEKMODE_BACK,
                                        currFastMode.params.iFrames.
                                        displayTime);
            }

            if ( res == FAPI_OK )
            {
                /* Adjust seek offset - suppressed for first seeking operation and
                                        also if data gap is detected. */

                if ( (inst->fastIFramesLastStamp != 0)
                  && (inst->header.nodataCounter == inst->lastNoDataCounter) )
                {
                    playTimeDiff = (int32_t)((int64_t)(inst->header.time)
                                 - (int64_t)(inst->fastIFramesLastPlayTime));

                    realTimeDiff = (int32_t)((int64_t)pvrGetTimeStamp()
                                        - (int64_t)inst->fastIFramesLastStamp);

                    inst->fastIFramesChunkSeek = pvrPlayCalcNewOffset(
                                                    playTimeDiff,
                                                    realTimeDiff,
                                                    inst->fastIFramesCurrFactor,
                                                    inst->fastIFramesChunkSeek,
                                                    inst->fastCtrlChunkSeekInit);

                    FAPI_SYS_PRINT_DEBUG(3, "[PLAY] pvrPlayThreadFastIFramesTask: play %i real %i -> %f - pos %i chunks %i \n",
                    playTimeDiff, realTimeDiff,
                    (float)((float)playTimeDiff / (float)realTimeDiff),
                    inst->position,
                    inst->fastIFramesChunkSeek);
                }

                inst->fastIFramesLastPlayTime = inst->header.time;
                inst->fastIFramesLastStamp    = pvrGetTimeStamp();
            }
        }
        else
        {
            res = pvrReadHeader(inst, &(inst->header));

            if ( res == FAPI_OK )
            {
                FAPI_SYS_SLEEP(500); // FIXME: why is this sleep so long?
            }
        }

        if ( res == SUBSYS_PVR_ERR_FILE_READ )
        {
            pvrSeekChunksSet(inst, inst->position);

            notifyReason = PVR_PLAY_NOTIFY_FILE_ERROR;

            toPause = 1;
        }
        else if ( res == SUBSYS_PVR_ERR_END_OF_DATA )
        {
            notifyReason = PVR_PLAY_NOTIFY_END_OF_DATA;

            toPause = 1;
        }
        else if ( pvrCheckForParamChange(inst, PVR_DECTYPE_VIDEO) )
        {
            FAPI_SYS_PRINT_DEBUG(1, "Video PID changed!\n");

            pvrTrackStop(inst, PVR_DECTYPE_VIDEO, PLAY_STOPMODE_BLANK);
            pvrIFrameChannelStop(inst);
            pvrTrackStart(inst, PVR_DECTYPE_VIDEO, PVR_DECMODE_FAST,
                          stateInfoPtr->reqStateIdx);
            pvrIFrameChannelStart(inst);
        }
    }

    if ( toPause )
    {
        pvrPlaySetState(inst, PLAY_STATE_TO_PAUSE, 0);

        if ( notifyReason != PVR_PLAY_NOTIFY_NONE )
        {
            pvrPlayNotify(inst, notifyReason);
            if (notifyReason == PVR_PLAY_NOTIFY_END_OF_DATA) {
            	FAPI_SYS_PRINT_DEBUG(1,"pvrPlayNotify called at %s %d\n", __FILE__, __LINE__);
            }
        }
    }
}

static void pvrPlayThreadFastAllTask (void* dataPtr,
                                      const PVR_PlayStateInfoT* stateInfoPtr)
{
    pvrPlayHandleT* inst = (pvrPlayHandleT*)dataPtr;
    int32_t         res;
    uint32_t        sleepTime = 0;

    if ( stateInfoPtr->reqStateIdx != stateInfoPtr->stateIdx )
    {
        pvrPlaySetState(inst, stateInfoPtr->state,
                        stateInfoPtr->reqStateIdx);

        pvrTrackSetMode(inst, PVR_DECTYPE_VIDEO, PVR_DECMODE_FAST,
                        stateInfoPtr->reqStateIdx);
    }

    if ( pvrIsEndOfData(inst) )
    {
        // FIXME: changing of state to be improved!

        //FDVR_Debug0("PLAY_STATE_FAST_ALL: end of data -> set speed to 0\n");
        pvrPlaySetState(inst, PLAY_STATE_TO_PAUSE, 0);

        pvrPlayNotify(inst, PVR_PLAY_NOTIFY_END_OF_DATA);
        FAPI_SYS_PRINT_DEBUG(1,"pvrPlayNotify called at %s %d\n", __FILE__, __LINE__);
        return;
    }

    // TODO: test if needed for high data rates!
    //pvrProcessSleepCounter(inst);

    res = pvrReadData(inst, &(inst->header));

    if ( res == SUBSYS_PVR_ERR_FILE_READ )
    {
        pvrSeekChunksSet(inst, inst->position);

        pvrPlaySetState(inst, PLAY_STATE_TO_PAUSE, 0);

        pvrPlayNotify(inst, PVR_PLAY_NOTIFY_FILE_ERROR);
        return;
    }
    else if ( res == SUBSYS_PVR_ERR_END_OF_DATA )
    {
        pvrPlaySetState(inst, PLAY_STATE_TO_PAUSE, 0);

        pvrPlayNotify(inst, PVR_PLAY_NOTIFY_END_OF_DATA);
        FAPI_SYS_PRINT_DEBUG(1,"pvrPlayNotify called at %s %d\n", __FILE__, __LINE__);
        return;
    }
    else if ( res == SUBSYS_PVR_ERR_NO_BUFFER_SPACE )
    {
        sleepTime = PVR_PLAY_CHUNKBUFFULL_SLEEPTIME;
    }

    if ( pvrCheckForParamChange(inst, PVR_DECTYPE_VIDEO) )
    {
        FAPI_SYS_PRINT_DEBUG(2, "Video PID changed!\n");

        pvrTrackStop(inst, PVR_DECTYPE_VIDEO, PLAY_STOPMODE_BLANK);
        pvrTrackStart(inst, PVR_DECTYPE_VIDEO, PVR_DECMODE_FAST,
                      stateInfoPtr->reqStateIdx);
    }

    pvrDecoderTasks(inst); /* perform decoder-specific tasks */

    if ( sleepTime > 0 )
    {
        FAPI_SYS_SLEEP(sleepTime);
    }
}

static void pvrPlayThreadFastControlledReset (pvrPlayHandleT* inst,
                                              uint8_t fastModeIdx)
{
    inst->fastCtrlRestart      = 1;
    inst->fastCtrlChunksFed    = 0;
    inst->fastCtrlLastStamp    = 0;
    inst->fastCtrlCurrFactor   = inst->playDataPtr->fastModeConfig[fastModeIdx].
                                                    params.feedCtrl.factor;
    inst->fastCtrlLastPlayTime = inst->header.time;

    /* set initial seek offset */

    #if 1 /* Martin: Test debug the init fastCtrlChunkSeek value*/
    if ( inst->fastCtrlCurrFactor >= 0 )
    {
        inst->fastCtrlChunkSeek = inst->fastCtrlCurrFactor;
    }
    else
    {
        inst->fastCtrlChunkSeek = -(inst->fastCtrlCurrFactor);
    }
    #else
    inst->fastCtrlChunkSeek = (int32_t)(((float)inst->fileInfo.size/ \
                                         (float)((float)inst->fileInfo.duration / 1000.00)) * \
                                         (float)inst->fastCtrlCurrFactor);


    if ( inst->fastCtrlCurrFactor >= 0 )
    {
        if (inst->fastCtrlChunkSeek == 0)
        {
            inst->fastCtrlChunkSeek = inst->fastCtrlCurrFactor;
        }

    }
    else
    {
        if (inst->fastCtrlChunkSeek == 0)
        {
            inst->fastCtrlChunkSeek = -(inst->fastCtrlCurrFactor);
        }
        else
        {
            inst->fastCtrlChunkSeek = -(inst->fastCtrlChunkSeek);
        }
    }
    #endif

    inst->fastCtrlChunkSeekInit = inst->fastCtrlChunkSeek;

	/*Martin -- 20100804 -- added for fix the issue that too much feed but no decoded event */
    (void)pvrDecoderPrepareData (inst, PVR_DECTYPE_VIDEO);

    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayThreadFastControlledReset: "\
                                    "init seek offset[%d]\n",\
                                    inst->fastCtrlChunkSeek);
}

static int32_t pvrPlayCalcNewOffset (int32_t playTimeDiff,
                                         int32_t realTimeDiff,
                                         int32_t factor,
                                         int32_t currOffset,
                                         int32_t initOffset)
{
    int8_t      sign;
    int32_t     offsetDiff;
    int32_t     currDiff;

	#if 0
    float       f_currDiff;
    int32_t     offsetBase;
    #endif

    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayCalcNewOffset:In: pDiff[%d], rDiff[%d], factor[%d], curOffset[%d]\n",\
                            playTimeDiff, realTimeDiff, factor, currOffset);

    playTimeDiff /= factor;

    if ( playTimeDiff < 0 ) playTimeDiff = -playTimeDiff;

    if ( playTimeDiff > realTimeDiff )
    {
        /* too fast, decrease seek offset */
        sign = -1;

        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayCalcNewOffset: too fast, decrease seek offset\n");

        #if 1
        currDiff = (int32_t)((float)playTimeDiff / (float)realTimeDiff);

        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayCalcNewOffset: currDiff[%d]\n", currDiff);
        if ( currDiff > PVR_PLAY_SEEKOFFSET_FACTOR_MAX )
        {
            currDiff = PVR_PLAY_SEEKOFFSET_FACTOR_MAX;
        }

        if ( currDiff >= 2 )
        {
            // FIXME: float operation needed here?
            offsetDiff = (currOffset / currDiff) * (currDiff - 1);

            offsetDiff /= PVR_PLAY_SEEKOFFSET_ATTENUATION;
        }
        else
        {
            offsetDiff = 1;
        }
        #else
        f_currDiff = (float)((float)playTimeDiff / (float)realTimeDiff);

        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayCalcNewOffset: f_currDiff[%f]\n", f_currDiff);

        offsetBase = currOffset >= 512 ? 512 : currOffset;

        if (offsetBase == 0)
            offsetBase = initOffset;

        offsetDiff = (int32_t)((float)(f_currDiff - 1) * (float)offsetBase);

        if (offsetDiff == 0)
        {
            offsetDiff = 1;
        }
        #endif

        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayCalcNewOffset: offsetDiff[%d]\n", offsetDiff);
    }
    else if ( playTimeDiff < realTimeDiff ) /* too slow */
    {
        /* too slow, increase seek offset */
        sign = 1;


        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayCalcNewOffset: too slow, increase seek offset\n");
        #if 1
        currDiff = (int32_t)((float)realTimeDiff / (float)playTimeDiff);

        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayCalcNewOffset: currDiff[%d]\n", currDiff);
        if ( currDiff > PVR_PLAY_SEEKOFFSET_FACTOR_MAX )
        {
            currDiff = PVR_PLAY_SEEKOFFSET_FACTOR_MAX;
        }

        if ( currDiff >= 2 )
        {
            offsetDiff = currOffset * (currDiff - 1);

            offsetDiff /= PVR_PLAY_SEEKOFFSET_ATTENUATION;
        }
        else
        {
            offsetDiff = 1;
        }
        #else

        f_currDiff = (float)((float)realTimeDiff / (float)playTimeDiff);

        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayCalcNewOffset: f_currDiff[%f]\n", f_currDiff);

        offsetBase = currOffset >= 512 ? 512 : currOffset;

        if (offsetBase == 0)
            offsetBase = initOffset;

        offsetDiff = (int32_t)((float)(f_currDiff - 1) * (float)offsetBase);
        //offsetDiff /= PVR_PLAY_SEEKOFFSET_ATTENUATION;

        if (offsetDiff == 0)
        {
            offsetDiff = 1;
        }

        #endif

        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayCalcNewOffset: offsetDiff[%d]\n", offsetDiff);
    }
    else
    {
        sign       = 1;
        offsetDiff = 0;
    }

    currOffset += (sign * offsetDiff);

    if ( currOffset > PVR_PLAY_SEEKOFFSET_MAX )
    {
        currOffset = PVR_PLAY_SEEKOFFSET_MAX;
    }
    else if ( currOffset < 0 )
    {
        currOffset = 0;
    }

    return currOffset;
}

static void pvrPlayThreadFastControlledTask (void* dataPtr,
                                             const PVR_PlayStateInfoT*
                                             stateInfoPtr)
{
    pvrPlayHandleT*     inst = (pvrPlayHandleT*)dataPtr;
    int32_t             res  = FAPI_OK;
    PVR_FastModeConfigT currFastMode;
    uint64_t            currTs;
    uint32_t            delaySleep;
    uint32_t            sleepTime = 0;
    int32_t             playTimeDiff;
    int32_t             realTimeDiff;

    FAPI_SYS_PRINT_DEBUG(3, "[FAST] ControlTask: In: pos[%d] timeStamp[%lld]\n",\
                        inst->position,\
                        FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    /* test for index changes */
    if ( stateInfoPtr->reqStateIdx != stateInfoPtr->stateIdx )
    {
        FAPI_SYS_PRINT_DEBUG(1, "[FAST] ControlTask: stateIdx changed\n");
        /* reset calculation if speed factor is different */
        if ( inst->fastCtrlCurrFactor !=
             inst->playDataPtr->fastModeConfig[stateInfoPtr->reqStateIdx].
             params.feedCtrl.factor )
        {
            FAPI_SYS_PRINT_DEBUG(1, "[FAST] ControlTask: curFactor[%d], reqFactor[%d]\n",
                 inst->fastCtrlCurrFactor,\
                 inst->playDataPtr->fastModeConfig[stateInfoPtr->reqStateIdx].params.feedCtrl.factor);

            pvrPlayThreadFastControlledReset(inst, stateInfoPtr->reqStateIdx);
        }

        pvrPlaySetState(inst, stateInfoPtr->state,
                        stateInfoPtr->reqStateIdx);
        pvrTrackSetMode(inst, PVR_DECTYPE_VIDEO,
                        PVR_DECMODE_FAST, stateInfoPtr->reqStateIdx);
    }

    // TODO: to be protected!
    currFastMode    = inst->playDataPtr->fastModeConfig[stateInfoPtr->reqStateIdx];

    if ( inst->fastCtrlRestart )
    {
        if ( inst->fastCtrlCurrFactor >= 0 ) /* Forward */
        {
        	FAPI_SYS_PRINT_DEBUG(1, "[FAST] ControlTask: curFactor(%d) chunkMove(%d) chunkFeed(%d)\n",\
                    inst->fastCtrlCurrFactor, inst->fastCtrlChunkSeek, inst->fastCtrlChunksFed);
            pvrSeekChunksCur(inst, inst->fastCtrlChunkSeek);

            if ( pvrIsEndOfData(inst) )
            {
                FAPI_SYS_PRINT_DEBUG(1, "[FAST] ControlTask: Out: EndOfData Meet, pos[%d],timeStamp[%lld]\n",\
                                        inst->position,\
                                        FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));
                pvrPlaySetState(inst, PLAY_STATE_TO_PAUSE, 0);
                pvrPlayNotify(inst, PVR_PLAY_NOTIFY_END_OF_DATA);
                FAPI_SYS_PRINT_DEBUG(1,"pvrPlayNotify called at %s %d\n", __FILE__, __LINE__);
                return;
            }
        }
        else /* Backward */
        {
            FAPI_SYS_PRINT_DEBUG(1, "[FAST] ControlTask: curFactor(%d) chunkMove(%d) chunkFeed(%d)\n",\
                    inst->fastCtrlCurrFactor, inst->fastCtrlChunkSeek, inst->fastCtrlChunksFed);
            pvrSeekChunksCur(inst, -(inst->fastCtrlChunkSeek
                                   + (int32_t)(2 * inst->fastCtrlChunksFed)));

            if ( pvrIsBeginOfData(inst) )
            {
                FAPI_SYS_PRINT_DEBUG(1, "[FAST] ControlTask: Out: BeginOfData Meet\n");
                pvrPlaySetState(inst, PLAY_STATE_TO_PAUSE, 0);
                pvrPlayNotify(inst, PVR_PLAY_NOTIFY_BEGIN_OF_DATA);
                return;
            }
        }

        inst->fastCtrlRestart   = 0;
        inst->fastCtrlChunksFed = 0;
    }

    if ( pvrTrackIsActive(inst, PVR_DECTYPE_VIDEO) )
    {
        // TODO: test if needed for high data rates!
        //pvrProcessSleepCounter(inst);

        uint8_t isDecoded = 0;
        uint32_t duration_seconds;
        uint32_t chunk_feed_max;
        uint32_t br;

        duration_seconds = inst->fileInfo.duration / 1000;
        duration_seconds = duration_seconds == 0 ? 1 : duration_seconds;

        br = (inst->fileInfo.size * PVR_CHUNK_SIZE) / duration_seconds;


        if (br > 1024*1024)
        {
            chunk_feed_max = 128;
        }
        else
        {
            chunk_feed_max = 16;
        }

        //isDecoded = pvrDecoderIsDataDecoded (inst, PVR_DECTYPE_VIDEO);
        isDecoded = isDecoded;

        if ( ( pvrDecoderIsDataDecoded (inst, PVR_DECTYPE_VIDEO))
          || (inst->fastCtrlChunksFed >=  /*chunk_feed_max*/PVR_PLAY_CTRLFEED_CHUNKFEED_MAX) )
        {

            FAPI_SYS_PRINT_DEBUG(2, "[PLAY] ControlTask: stream bitrate[%d]\n", br);
            FAPI_SYS_PRINT_DEBUG(2, "[PLAY] ControlTask: fastCtrlChunksFed[%d] "
                                            "isDecoded[%i]\n",
                                            inst->fastCtrlChunksFed,
                                            (uint32_t)isDecoded);
            /* discard data not yet fed to TSD */
            (void)pvrPlayClearBuffer(inst);

            currTs = pvrGetTimeStamp();

            delaySleep = PVR_PLAY_CTRLFEED_SLEEPTIME;

            if ( (inst->fastCtrlLastStamp != 0)
              && (currTs >= inst->fastCtrlLastStamp) )
            {
                if ( (inst->fastCtrlLastStamp
                      + currFastMode.params.feedCtrl.displayTime) > currTs )
                {
                    delaySleep = (uint32_t)
                                 (currFastMode.params.feedCtrl.displayTime
                                 - (currTs - inst->fastCtrlLastStamp));

                    if ( delaySleep < PVR_PLAY_CTRLFEED_SLEEPTIME )
                    {
                        delaySleep = PVR_PLAY_CTRLFEED_SLEEPTIME;
                    }
                }
            }
            FAPI_SYS_PRINT_DEBUG(1, "[FAST] ControlTask: delay sleep [%d] ms\n", delaySleep);
            FAPI_SYS_SLEEP(delaySleep);

            // FIXME: this reads header from next chunk: acceptable?
            (void)pvrReadHeader(inst, &(inst->header));

            /* Adjust seek offset - suppressed for first seeking operation and
                                    also if data gap is detected. */
            if ( (inst->fastCtrlLastStamp != 0)
              && (inst->header.nodataCounter == inst->lastNoDataCounter) )
            {
                playTimeDiff = (int32_t)((int64_t)(inst->header.time)
                             - (int64_t)(inst->fastCtrlLastPlayTime));

                realTimeDiff = (int32_t)((int64_t)pvrGetTimeStamp()
                                       - (int64_t)inst->fastCtrlLastStamp);

                inst->fastCtrlChunkSeek = pvrPlayCalcNewOffset(
                                                    playTimeDiff,
                                                    realTimeDiff,
                                                    inst->fastCtrlCurrFactor,
                                                    inst->fastCtrlChunkSeek,
                                                    inst->fastCtrlChunkSeekInit);

                FAPI_SYS_PRINT_DEBUG(2, "[FAST] ControlTask: play %i real %i -> %f - pos %i chunks %i fed %i\n",
                playTimeDiff, realTimeDiff,
                (float)((float)playTimeDiff / (float)realTimeDiff),
                inst->position,
                inst->fastCtrlChunkSeek,
                inst->fastCtrlChunksFed);

            }

            inst->fastCtrlLastPlayTime = inst->header.time;
            inst->fastCtrlLastStamp    = pvrGetTimeStamp();
            inst->fastCtrlRestart      = 1;

            (void)pvrDecoderPrepareData (inst, PVR_DECTYPE_VIDEO);
        }
        else
        {
            /*
                      *  Do not update header, this is done after data has been
                      *  decoded.
                      */
            res = pvrReadData(inst, NULL);

            inst->fastCtrlChunksFed++;

            if ( res == SUBSYS_PVR_ERR_FILE_READ )
            {
                pvrSeekChunksSet(inst, inst->position);

                pvrPlaySetState(inst, PLAY_STATE_TO_PAUSE, 0);

                pvrPlayNotify(inst, PVR_PLAY_NOTIFY_FILE_ERROR);
                return;
            }
            else if ( res == SUBSYS_PVR_ERR_END_OF_DATA )
            {
                pvrPlaySetState(inst, PLAY_STATE_TO_PAUSE, 0);

                pvrPlayNotify(inst, PVR_PLAY_NOTIFY_END_OF_DATA);
                FAPI_SYS_PRINT_DEBUG(1,"pvrPlayNotify called at %s %d\n", __FILE__, __LINE__);
                return;
            }
            else if ( res == SUBSYS_PVR_ERR_NO_BUFFER_SPACE )
            {
                sleepTime = PVR_PLAY_CHUNKBUFFULL_SLEEPTIME;
            }
        }
    }
    else
    {
        (void)pvrReadHeader(inst, &(inst->header));

        sleepTime = 500; // FIXME: why is this sleep so long?
    }

    if ( pvrCheckForParamChange(inst, PVR_DECTYPE_VIDEO) )
    {
        FAPI_SYS_PRINT_DEBUG(2, "Video PID changed!\n");

        pvrTrackStop (inst, PVR_DECTYPE_VIDEO, PLAY_STOPMODE_BLANK);
        pvrTrackStart(inst, PVR_DECTYPE_VIDEO, PVR_DECMODE_FAST,
                      stateInfoPtr->reqStateIdx);
    }

    pvrDecoderTasks(inst); /* perform decoder-specific tasks */

    if ( sleepTime > 0 )
    {
        FAPI_SYS_PRINT_DEBUG(3, "[FAST] ControlTask: sleep [%d] ms\n", sleepTime);
        FAPI_SYS_SLEEP(sleepTime);
    }

    FAPI_SYS_PRINT_DEBUG(3, "[FAST] ControlTask: Out: pos[%d] timeStamp[%lld]\n",\
                        inst->position,
                        FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));
}

static void pvrPlayThreadFastNoneReset (pvrPlayHandleT* inst,
                                        uint8_t fastModeIdx)
{
    inst->fastNoneCurrFactor   = inst->playDataPtr->fastModeConfig[fastModeIdx].
                                 params.feedNone.factor;
    inst->fastNoneLastPlayTime = inst->header.time;
    inst->fastNoneLastStamp    = pvrGetTimeStamp();
    inst->fastNoneNextStamp    = 0;
    inst->fastNoneChunkSeek    = 1;
}

static void pvrPlayThreadFastNoneTask (void* dataPtr,
                                       const PVR_PlayStateInfoT* stateInfoPtr)
{
    pvrPlayHandleT* inst = (pvrPlayHandleT*)dataPtr;
    int32_t         playTimeDiff;
    int32_t         realTimeDiff;
    int32_t         res;
    uint32_t        sleepTime = PVR_PLAY_NOFEED_SLEEPTIME;
    uint64_t        currTs;

    /* test for index changes */
    if ( stateInfoPtr->reqStateIdx != stateInfoPtr->stateIdx )
    {
        /* reset calculation if speed factor is different */
        if ( inst->fastNoneCurrFactor !=
             inst->playDataPtr->fastModeConfig[stateInfoPtr->reqStateIdx].
             params.feedNone.factor )
        {
            pvrPlayThreadFastNoneReset(inst, stateInfoPtr->reqStateIdx);
        }

        pvrPlaySetState(inst, stateInfoPtr->state,
                        stateInfoPtr->reqStateIdx);
    }

    /* currently in delay phase? */
    if ( inst->fastNoneNextStamp > 0 )
    {
        currTs = pvrGetTimeStamp();

        if ( currTs >= inst->fastNoneNextStamp )
        {
            inst->fastNoneLastPlayTime = inst->header.time;
            inst->fastNoneLastStamp    = pvrGetTimeStamp();
            inst->fastNoneNextStamp    = 0;

            if ( inst->fastNoneChunkSeek > 1 )
            {
                inst->fastNoneChunkSeek--;
            }
        }
        else
        {
            if ( (inst->fastNoneNextStamp - currTs) <
                 PVR_PLAY_NOFEED_SLEEPTIME )
            {
                sleepTime = (uint32_t)(inst->fastNoneNextStamp - currTs);
            }
            else
            {
                sleepTime = PVR_PLAY_NOFEED_SLEEPTIME;
            }

            FAPI_SYS_SLEEP(sleepTime);
            return;
        }
    }

    if ( inst->fastNoneCurrFactor < 0 )
    {
        pvrSeekChunksCur(inst, -(inst->fastNoneChunkSeek));

        if ( pvrIsBeginOfData(inst) )
        {
            pvrPlaySetState(inst, PLAY_STATE_TO_PAUSE, 0);
            pvrPlayNotify(inst, PVR_PLAY_NOTIFY_BEGIN_OF_DATA);
            return;
        }
    }
    else
    {
        pvrSeekChunksCur(inst, inst->fastNoneChunkSeek);
    }

    res = pvrReadHeader(inst, &(inst->header));

    if ( res == SUBSYS_PVR_ERR_FILE_READ )
    {
        pvrSeekChunksSet(inst, inst->position);

        pvrPlaySetState(inst, PLAY_STATE_TO_PAUSE, 0);
        pvrPlayNotify(inst, PVR_PLAY_NOTIFY_FILE_ERROR);
        return;
    }
    else if ( res == SUBSYS_PVR_ERR_END_OF_DATA )
    {
        pvrPlaySetState(inst, PLAY_STATE_TO_PAUSE, 0);
        pvrPlayNotify(inst, PVR_PLAY_NOTIFY_END_OF_DATA);
        FAPI_SYS_PRINT_DEBUG(1,"pvrPlayNotify called at %s %d\n", __FILE__, __LINE__);
        return;
    }

    if ( inst->header.nodataCounter != inst->lastNoDataCounter )
    {
        /* Gap in data detected, reset speed control */
        inst->fastNoneLastPlayTime = inst->header.time;
        inst->fastNoneLastStamp    = pvrGetTimeStamp();
        inst->fastNoneNextStamp    = 0;
    }
    else
    {
        playTimeDiff = (int32_t)((int64_t)(inst->header.time)
                     - (int64_t)(inst->fastNoneLastPlayTime));

        realTimeDiff = (int32_t)((int64_t)pvrGetTimeStamp()
                               - (int64_t)inst->fastNoneLastStamp);

        playTimeDiff /= inst->fastNoneCurrFactor;

        if ( playTimeDiff < 0 ) playTimeDiff = -playTimeDiff;

        if ( playTimeDiff > realTimeDiff )
        {
            if ( (playTimeDiff - realTimeDiff) < PVR_PLAY_NOFEED_SLEEPTIME )
            {
                FAPI_SYS_SLEEP ((uint32_t)(playTimeDiff - realTimeDiff));

                inst->fastNoneLastPlayTime = inst->header.time;
                inst->fastNoneLastStamp    = pvrGetTimeStamp();
                inst->fastNoneNextStamp    = 0;

                if ( inst->fastNoneChunkSeek > 1 )
                {
                    inst->fastNoneChunkSeek--;
                }

                sleepTime = 0;
            }
            else
            {
                inst->fastNoneNextStamp = inst->fastNoneLastStamp
                                        + (uint32_t)playTimeDiff;
            }
        }
        else if ( playTimeDiff < realTimeDiff )
        {
            if ( inst->fastNoneChunkSeek < PVR_PLAY_NOFEED_CHUNKSEEK_MAX )
            {
                inst->fastNoneChunkSeek++;
            }
        }
    }

    FAPI_SYS_SLEEP(sleepTime);
}

/*lint -save -e715 */
static void pvrPlayThreadPauseTask (void* dataPtr,
                                    const PVR_PlayStateInfoT* stateInfoPtr)
{
    FAPI_SYS_SLEEP(50);
}
/*lint -restore */

static void pvrPlayThreadSeekTask (void* dataPtr,
                                   const PVR_PlayStateInfoT* stateInfoPtr)
{
    pvrPlayHandleT* inst         = (pvrPlayHandleT*)dataPtr;
    PVR_PlayNotifyT notifyReason = PVR_PLAY_NOTIFY_NONE;
    uint8_t         playFrame    = 0;

    switch ( stateInfoPtr->reqStateIdx )
    {
        case PLAY_SEEK_POS_ABS:
            if ( inst->seekPosition != inst->position )
            {
                pvrSeekChunksSet(inst, inst->seekPosition);
                playFrame = 1;
            }
            break;

        case PLAY_SEEK_POS_REL:
            if ( !(((inst->seekChunks > 0) && pvrIsEndOfData(inst))
                || ((inst->seekChunks < 0) && pvrIsBeginOfData(inst))) )
            {
                pvrSeekChunksCur(inst, inst->seekChunks);
                playFrame = 1;
            }
            break;

        #if 0

        // TODO: not supported yet!

        case PLAY_SEEK_NAME_FORWARD:
            pvrSeekNameForward(inst, 1);
            playFrame = 1;
            break;

        case PLAY_SEEK_NAME_BACKWARDS:
            pvrSeekNameBackward(inst, 1);
            playFrame = 1;
            break;

        case PLAY_SEEK_TAG_FORWARD:
            pvrSeekNameForward(inst, 0);
            playFrame = 1;
            break;

        case PLAY_SEEK_TAG_BACKWARDS:
            pvrSeekNameBackward(inst, 0);
            playFrame = 1;
            break;

        #endif

        case PLAY_SEEK_IFRAME_FORWARD:
            // TODO: to be implemented!
            #if 0
            if ( !(pvrIsEndOfData(inst)) )
            {
                pvrSeekChunksCur(inst, inst->seekChunks);
                pvrPlayIFrameAndLock(inst, &notifyReason);
            }
            #endif
            break;

        case PLAY_SEEK_IFRAME_BACKWARDS:
            // TODO: to be implemented!
            #if 0
            if ( !(pvrIsBeginOfData(inst)) )
            {
                pvrSeekChunksCur(inst, inst->seekChunks);
                pvrPlayIFrameAndLock(inst, &notifyReason);
            }
            #endif
            break;

        case PLAY_SEEK_TIME_ABS:
            pvrPlaySeekToTime(inst, inst->seekTime);
            playFrame = 1;
            break;

        default:
            /* do nothing */
            break;
    }

    if ( playFrame )
    {
        pvrPlaySingleIFrame(inst, &notifyReason);
    }

    // TODO: use PLAY_STATE_TO_PAUSE !

    (void)pvrPlayClearBuffer(inst);

    pvrStartAll(inst, PVR_DECMODE_PAUSE, 0);

    pvrPlaySetState(inst, PLAY_STATE_PAUSE, 0);

    if ( notifyReason != PVR_PLAY_NOTIFY_NONE )
    {
        pvrPlayNotify(inst, notifyReason);
        if (notifyReason == PVR_PLAY_NOTIFY_END_OF_DATA) {
        	FAPI_SYS_PRINT_DEBUG(1,"pvrPlayNotify called at %s %d\n", __FILE__, __LINE__);
        }
    }
}

static void pvrPlaySeekToTime(pvrPlayHandleT* inst, uint32_t seekTime)
{
    PVR_HeaderPacketT   header;
    int32_t             seekTimeSeconds;
    uint8_t             firstSeek  = 1;
    uint8_t             preliminarySeek = 0;
    int32_t             currTime   = 0;     //!< in milliseconds
    int32_t             timeDiff   = 0;     //!< in milliseconds
    int32_t             seekChunks = 0;
    int64_t             seekBits;
    uint16_t            numAttempts = 0;
    uint32_t            currDataRate;       //!< in bits/second
    int32_t             lastTime;
    int32_t             lastTimeDiff;       //!< in milliseconds
    int32_t             lastTimeJump;       //!< in milliseconds
    int32_t             lastSeekChunks;
    int32_t             newPosition;

    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlaySeekToTime: Seek to time %i ms, curr pos %i\n",
                         seekTime, inst->position);

    if ( seekTime > inst->fileInfo.duration )
    {
        /* seek to file end if requested time is greater than file duration. */

        FAPI_SYS_PRINT_DEBUG(3, "[PLAY] pvrPlaySeekToTime: Time seek -> file end\n");

        if ( inst->offset == 0 )
        {
            pvrSeekChunksSet(inst, inst->size - 1);
        }
        else
        {
            pvrSeekChunksSet(inst, inst->offset - 1);
        }
    }
    else
    {
        seekTimeSeconds = (int32_t)(seekTime / 1000);

        while ( numAttempts++ < PVR_PLAY_TIMESEEK_ATTEMPTS_MAX )
        {
            if ( pvrIsEndOfData(inst) )
            {
                memset(&header, 0, sizeof(header));

                header.time = inst->fileInfo.duration;
                FAPI_SYS_PRINT_DEBUG(1, "pvrIsEndOfData called %s %d\n", __FILE__, __LINE__);
            }
            else
            {
                if ( pvrReadHeader(inst, &header) != FAPI_OK )
                {
                    break;
                }
            }

            lastTime       = currTime;
            lastSeekChunks = seekChunks;
            lastTimeDiff   = timeDiff;

            currTime     = (int32_t)header.time;
            timeDiff     = ((int32_t)seekTime) - currTime;

            /* target time shall lie within the same second than requested
               time. */

            if ( (currTime / 1000) == seekTimeSeconds )
            {
                FAPI_SYS_PRINT_DEBUG(3, "[PLAY] pvrPlaySeekToTime: Time seek -> match!\n");

                /* exact match, abort seeking */
                break;
            }
            else if ( preliminarySeek )
            {
                preliminarySeek = 0;

                if ( abs(timeDiff) > abs(lastTimeDiff) )
                {
                    /* revert last operation and abort if resulting position
                       from preliminary seek is worse than before. */

                    FAPI_SYS_PRINT_DEBUG(3, "[PLAY] pvrPlaySeekToTime: Time seek -> revert last operation!\n");

                    pvrSeekChunksCur(inst, -lastSeekChunks);
                    break;
                }
            }

            /* ----- calculate seek width ---------------------------------- */

            if ( firstSeek )
            {
                firstSeek = 0;

                /* calculate jump width using datarate */

                currDataRate = header.dataRate;

                if ( (currDataRate <= PVR_PLAY_TIMESEEK_DATARATE_MIN)
                  || (currDataRate >= PVR_PLAY_TIMESEEK_DATARATE_MAX) )
                {
                    currDataRate = PVR_PLAY_TIMESEEK_DATARATE_DEFAULT;
                }

                seekBits   = ((int64_t)timeDiff * (int64_t)currDataRate) / 1000;
                seekChunks = (int32_t)((seekBits / 8) / PVR_CHUNK_SIZE);
            }
            else
            {
                lastTimeJump = currTime - lastTime;

                if ( lastTimeJump == 0 ) lastTimeJump = 1;

                seekChunks = (int32_t)(((int64_t)lastSeekChunks *
                                        (int64_t)timeDiff) / lastTimeJump);
            }

            if ( seekChunks == 0 )
            {
                FAPI_SYS_PRINT_DEBUG(3, "[PLAY] pvrPlaySeekToTime: Attempt preliminary seek!\n");

                if ( timeDiff < 0 ) seekChunks = -1;
                else                seekChunks =  1;

                preliminarySeek = 1;
            }

            /* ----- ensure that seekChunks points to valid position ------- */

            newPosition = (int32_t)inst->position + seekChunks;

            if ( inst->offset == 0 )
            {
                if ( newPosition < 0 ) /* BOF */
                {
                    seekChunks = -((int32_t)inst->position);
                }
                else if ( ((uint32_t)newPosition) >= inst->size ) /* EOF */
                {
                    seekChunks = ((int32_t)inst->size - (int32_t)inst->position)
                                                      - 1;
                }
            }
            else
            {
                if ( inst->position < inst->offset )
                {
                    if ( seekChunks >= 0 )
                    {
                        if ( newPosition >= (int32_t)inst->offset )
                        {
                            /* logical EOF */
                            seekChunks = (((int32_t)(inst->offset)) - 1)
                                         - (int32_t)inst->position;
                        }
                    }
                    else
                    {
                        if ( seekChunks < (-(((int32_t)inst->size
                                            - (int32_t)inst->offset)
                                            + (int32_t)inst->position)) )
                        {
                            /* logical BOF */
                            seekChunks = -(((int32_t)inst->size
                                          - (int32_t)inst->offset)
                                          + (int32_t)inst->position);
                        }
                    }
                }
                else
                {
                    if ( seekChunks >= 0 )
                    {
                        if ( seekChunks >= (((int32_t)inst->size
                                           - (int32_t)inst->position)
                                           + (int32_t)inst->offset) )
                        {
                            /* logical EOF */
                            seekChunks = (((int32_t)inst->size
                                         - (int32_t)inst->position)
                                         + (int32_t)inst->offset) - 1;
                        }
                    }
                    else
                    {
                        if ( seekChunks < ((int32_t)inst->offset
                                         - (int32_t)inst->position) )
                        {
                            /* logical BOF */
                            seekChunks = (int32_t)inst->offset
                                       - (int32_t)inst->position;
                        }
                    }
                }
            }

            FAPI_SYS_PRINT_DEBUG(3, "[PLAY] pvrPlaySeekToTime: Seek #%2i: pos %i, time %i, diff %i s, chunks %i\n",\
                                 numAttempts - 1,
                                 inst->position,
                                 currTime / 1000,
                                 timeDiff / 1000,
                                 seekChunks);

            /* ------------------------------------------------------------- */

            if ( seekChunks == 0 )
            {
                break;
            }

            pvrSeekChunksCur(inst, seekChunks);
        }
    }

    /* update playback time */
    (void)pvrReadHeader(inst, &(inst->header));
}

static void pvrPlaySingleIFrame (pvrPlayHandleT* inst,
                                 PVR_PlayNotifyT* notifyReasonPtr)
{
    PVR_DecoderModeT decoderMode;
    int32_t          res;
    int32_t          (*frameFeedFct)(pvrPlayHandleT*,PVR_PlaySeekModeT,
                                     uint32_t);

    if ( notifyReasonPtr != NULL )
    {
        *notifyReasonPtr = PVR_PLAY_NOTIFY_NONE;
    }

    switch ( pvrTrackGetFormat(inst, PVR_DECTYPE_VIDEO) )
    {
        default:
            return;

        case PVR_TRACK_FORMAT_OTHER:
        case PVR_TRACK_FORMAT_VIDEO_MPEG1:
        case PVR_TRACK_FORMAT_VIDEO_MPEG2:
            frameFeedFct = pvrPlayMPEGIFrame;
            decoderMode  = PVR_DECMODE_FEEDDATA;
            break;

        case PVR_TRACK_FORMAT_VIDEO_H264:
            frameFeedFct = pvrPlayCtrlFeedIFrame;
            decoderMode  = PVR_DECMODE_FEEDCTRL;
            break;
    }

    if ( pvrTrackIsActive(inst, PVR_DECTYPE_VIDEO) && (inst->enableFrameView) )
    {
        /* set decoders */
        pvrIFrameChannelStop (inst);
        pvrStopMedia         (inst);
        (void)pvrSyncStop    (inst);

        (void)pvrPlayClearBuffer(inst);

        pvrTrackSetMode(inst, PVR_DECTYPE_VIDEO, decoderMode, 0);

        if ( frameFeedFct == pvrPlayMPEGIFrame )
        {
            pvrIFrameChannelStart(inst);
        }

        res = frameFeedFct(inst, PLAY_SEEKMODE_BACK, 200);

        if ( pvrCheckForParamChange(inst, PVR_DECTYPE_VIDEO) )
        {
            FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlaySingleIFrame: Video PID changed -> play frame again\n");

            pvrStopMedia(inst);
            pvrTrackStop(inst, PVR_DECTYPE_VIDEO, PLAY_STOPMODE_BLANK);

            (void)pvrPlayClearBuffer(inst);

            pvrStartAll (inst, decoderMode, 0);

            (void)frameFeedFct(inst, PLAY_SEEKMODE_BACK, 200);
        }

        if ( frameFeedFct == pvrPlayMPEGIFrame )
        {
            pvrIFrameChannelStop(inst);
        }
    }
    else
    {
        res = pvrReadHeader(inst, &(inst->header));
    }

    if ( res == SUBSYS_PVR_ERR_END_OF_DATA )
    {
        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlaySingleIFrame: End of data reached\n");

        if ( notifyReasonPtr != NULL )
        {
            *notifyReasonPtr = PVR_PLAY_NOTIFY_END_OF_DATA;
        }
    }
    else if ( pvrIsBeginOfData(inst) )
    {
        if ( notifyReasonPtr != NULL )
        {
            *notifyReasonPtr = PVR_PLAY_NOTIFY_BEGIN_OF_DATA;
        }
    }
}

static uint8_t pvrGetTrack(pvrPlayHandleT* inst, PVR_DecoderTypeT decoderType)
{
    uint8_t         index;
    PVR_TrackTypeT  trackType;
    uint8_t         trackIndex = 0xFF;

    for ( index = 0; index < inst->fileInfo.numberOfTracks; index++ )
    {
        trackType = inst->fileInfo.trackType[index];

        /*lint -save -e787 */
        switch ( decoderType )
        {
            case PVR_DECTYPE_VIDEO:
                if ( trackType == PVR_TRACK_TYPE_VIDEO )
                {
                    trackIndex = index;
                }
                break;

            case PVR_DECTYPE_AUDIO:
            case PVR_DECTYPE_AUDIO_2ND:
                if ( (trackType == PVR_TRACK_TYPE_AUDIO)
                  || (trackType == PVR_TRACK_TYPE_DEPRECATED) )
                {
                    /* NOTE: PVR_TRACK_TYPE_DEPRECATED is also evaluated
                             for compatibility reasons. */
                    trackIndex = index;
                }
                break;

            case PVR_DECTYPE_TTX:
                if ( trackType == PVR_TRACK_TYPE_TTX )
                {
                    trackIndex = index;
                }
                break;
            default:
                break;
        }
        /*lint -restore */

        if ( trackIndex != 0xFF ) break;
    }

    return trackIndex;
}

#if 0
/* currently unused */
static void pvrProcessSleepCounter(pvrPlayHandleT* inst)
{
    /* let other threads run as well (needed for very high data rates) */
    if ( inst->sleepCounter > 0 )
    {
        inst->sleepCounter--;
    }
    else
    {
        FAPI_SYS_SLEEP(30);
        inst->sleepCounter = 3;
    }
}
#endif

static uint8_t pvrSupportTimeShifting(pvrPlayHandleT* inst)
{
    uint32_t        last_offset = 0;
    PVR_RecordInfoT record_info;

    if ( inst->recHandle == NULL ) return 1;

    last_offset = inst->offset;

    inst->splitSize      = pvrRecGetSplitSize(inst->recHandle);
    inst->numberOfSplits = pvrRecGetNumberOfSplits(inst->recHandle);

    // FIXME: necessary?
    memset(&record_info, 0, sizeof(record_info));

    (void)pvrRecGetInfo(inst->recHandle, &record_info);

    inst->fileInfo.size     = record_info.size;
    inst->fileInfo.duration = record_info.duration;
    inst->size              = record_info.size;

    if ( record_info.hasLooped )
    {
        inst->offset = record_info.position;

        if ( (last_offset    <  inst->offset)
          && (inst->position >= last_offset)
          && (inst->position <  inst->offset) )
        {
            return 0;
        }
        else if ( (last_offset    >  inst->offset)
               && (inst->position >= last_offset) )
        {
            return 0;
        }
        else if ( (last_offset    >  inst->offset)
               && (inst->position <  inst->offset) )
        {
            return 0;
        }
    }

    return 1;
}

static uint8_t pvrCheckForParamChange(pvrPlayHandleT* inst, PVR_DecoderTypeT type)
{
    pvrPlayStreamT* streamPtr = inst->stream + (int32_t)type;

    if ( (streamPtr->trackIndex != 0xFF)
      && (streamPtr->decHandle  != NULL)
      && ((streamPtr->pid       != (inst->header.trackPid[streamPtr->trackIndex]
                                   & 0x1FFF))
       || (streamPtr->format    != pvrGetTrackFormat(&(inst->header),
                                                     streamPtr->trackIndex))) )
    {
        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrCheckForParamChange: trackIndex[%d] decHandle[0x%08x] pid[%d] trackPid[%d] format[%d] getFormat[%d]\n",\
                           streamPtr->trackIndex, (uint32_t)streamPtr->decHandle,
                           streamPtr->pid, inst->header.trackPid[streamPtr->trackIndex]&0x1FFF,
                           streamPtr->format, pvrGetTrackFormat(&(inst->header),streamPtr->trackIndex));
        return 1;
    }
    else
    {
        return 0;
    }
}

static uint8_t pvrCheckForTrackStop(pvrPlayHandleT* inst, PVR_DecoderTypeT type)
{
    if ( (inst->stream[type].trackIndex == 0xFF) && pvrPidIsValid(inst, type) )
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

/* ------------------------------------------------------------------------- */

static int32_t pvrReadHeader(pvrPlayHandleT* inst, PVR_HeaderPacketT* headerPtr)
{
    int32_t error_code = FAPI_OK;
    int32_t res;

    if ( pvrIsEndOfData(inst) )
    {
    	FAPI_SYS_PRINT_DEBUG(1, "pvrIsEndOfData called %s %d\n", __FILE__, __LINE__);
        error_code = SUBSYS_PVR_ERR_END_OF_DATA;
    }
    else
    {
        res = fread(inst->tmpHeader, 1, sizeof(PVR_HeaderPacketT),
                     inst->filePtr);

        if ( res != (int32_t)(sizeof(PVR_HeaderPacketT)) )
        {
            error_code = SUBSYS_PVR_ERR_FILE_READ;
        }
        else
        {
            (void)fseek(inst->filePtr, -((int32_t)(sizeof(PVR_HeaderPacketT))),
                         SEEK_CUR);

            if ( headerPtr != NULL )
            {
                error_code = pvrImportHeader(headerPtr, inst->tmpHeader,
                                             sizeof(PVR_HeaderPacketT));
            }
        }
    }

    return error_code;
}

/*lint -save -e826 */
/* read one chunk of data */
static int32_t pvrReadData(pvrPlayHandleT* inst, PVR_HeaderPacketT* headerPtr)
{
    uint16_t    split;
    int32_t     error_code;
    uint64_t    ts;
    F_FILE*     file;
    uint32_t    bufRp;
    uint32_t    bufWp;
    uint32_t    freePkts;
    uint32_t*   dataPtr;

    if ( pvrIsEndOfData(inst) )
    {
    	FAPI_SYS_PRINT_DEBUG(1, "pvrIsEndOfData called %s %d\n", __FILE__, __LINE__);
        FAPI_SYS_PRINT_DEBUG(1, "[READ] pvrReadData: Out: End of Data\n");
        return SUBSYS_PVR_ERR_END_OF_DATA;
    }

    /* check for space in chunk buffer */
    bufRp = inst->chunkBufferRp;
    bufWp = inst->chunkBufferWp;

    if ( bufWp < bufRp )
    {
        freePkts = bufRp - bufWp;
    }
    else
    {
        /* PVR_PLAY_CHUNKBUFFER_PKTS == 2048 */
        freePkts = PVR_PLAY_CHUNKBUFFER_PKTS - (bufWp - bufRp);
    }

    /* NOTE: This ensures that buffer is not completely filled and is crucial
             for buffer r/w pointer arithmetic. */
    if ( freePkts > 0 )
    {
        freePkts--;
    }

    if ( freePkts < PVR_PKTS_PER_CHUNK )
    {
        FAPI_SYS_PRINT_DEBUG(2, "[READ] pvrReadData: Out: No buffer space\n");
        return SUBSYS_PVR_ERR_NO_BUFFER_SPACE;
    }

    dataPtr = (uint32_t*)(((uint8_t*)(inst->chunkBuffer))
                        + (inst->chunkBufferWp * PVR_TSPKT_SIZE));

    ts = pvrGetTimeStamp();


	if (inst->recHandle)
	{
	    if (inst->position == inst->size || inst->position == inst->size - 1)
	    {
			pvrRecHandleT *rH = inst->recHandle;

	    	if (rH)
	    	{
	    		if (rH->filePtr && inst->filePtr)
	    		{
	    			fflush(rH->filePtr);
	    			fflush(inst->filePtr);
	    			FAPI_SYS_SLEEP(50);
	    		}
	    	}
	    }
    }

    error_code = pvrReadChunkFromFile (inst, dataPtr);

    if ( error_code != FAPI_OK )
    {
        FAPI_SYS_PRINT_DEBUG(1, "[READ] pvrReadData: Out: read chunk from file failed[%d]\n", error_code);
        return error_code;
    }

    if ( (inst->chunkBufferWp + PVR_PKTS_PER_CHUNK) <
          PVR_PLAY_CHUNKBUFFER_PKTS )
    {
        inst->chunkBufferWp += PVR_PKTS_PER_CHUNK;
    }
    else
    {
        inst->chunkBufferWp = 0;
    }

    if ( headerPtr != NULL ) /* update header */
    {
        if ( pvrImportHeader(headerPtr, dataPtr, PVR_CHUNK_SIZE) == FAPI_OK )
        {
            (void)pvrUpdateChunkTime(inst, headerPtr->time);
        }
    }

    /* --------------------------------------------------------------------- */

    inst->readHddTime = pvrGetTimeStamp() - ts;

    inst->position++; /* increment chunk position */

    if ( (inst->offset > 0)
      && (inst->position == inst->size) ) /* circular ? */
    {
        pvrSeekChunksSet(inst, 0);
    }

    if ( pvrIsEndOfData(inst) )
    {
    	/*Martin -- 20100718 -- Avoid playback catch up with recording in Timeshift*/
		PVR_RecordInfoT rec_info;

		if (inst->recHandle != NULL)
		{
			int32_t wait_time = 4000;
			FAPI_SYS_PRINT_DEBUG(2, "[READ] pvrReadData: End of data 2.wait %d seconds to confirm\n",\
								wait_time / 1000);
			while (wait_time >=  0)
			{
				FAPI_SYS_SLEEP(500);
				wait_time -= 500;
				memset(&rec_info, 0, sizeof(PVR_RecordInfoT));

				if (inst->recHandle != NULL)
			    {
			        pvrRecGetInfo(inst->recHandle,&rec_info);
			        if (rec_info.size <= inst->position)
			        {
			        	continue;
			        }
			        else
			        {
			        	FAPI_SYS_PRINT_DEBUG(2, "[READ] pvrReadData: rec_info increase, keep on playing\n");
			        	break;
			        }
		        }
		        else
		        {
		        	/* recording has been terminated outside */
		        	return SUBSYS_PVR_ERR_END_OF_DATA;
		        }
			}

			if (wait_time < 0)
			{
				return SUBSYS_PVR_ERR_END_OF_DATA;
			}

        }
        else
        {
        	FAPI_SYS_PRINT_DEBUG(1, "[READ] pvrReadData: Out: End of data 2. NO recording\n");
        	return SUBSYS_PVR_ERR_END_OF_DATA;
        }
    }

    /* support file splitting */
    if ( (inst->splitSize > 0)
      && (inst->numberOfSplits > 1) )
    {
        FAPI_SYS_PRINT_DEBUG(2, "[Read] pvrReadData: splitSize = %d\n", inst->splitSize);
        split = inst->position / inst->splitSize;
        if ( split >= inst->numberOfSplits ) split = inst->numberOfSplits - 1;
        if ( split != inst->splitNumber )
        {
            pvrGetFilename(inst->filePath, inst->openParams.fileName, split);

            file = fopen(inst->filePath, "r");
            if ( file == NULL )
            {
                FAPI_SYS_PRINT_DEBUG(1, "[READ] pvrReadData: Out: err file read\n");
                return SUBSYS_PVR_ERR_FILE_READ; //todo: return other error code?
            }

            (void)f_close(inst->filePtr);

            inst->filePtr     = file;
            inst->splitNumber = split;
        }
    }

    #if 1
    FAPI_SYS_PRINT_DEBUG(2, "[READ] pvrReadData: Out: position[%04d], bfRp[%04d], bfWp[%04d], offset[%04d], size[%04d]\n",\
    inst->position, inst->chunkBufferRp,inst->chunkBufferWp,inst->offset, inst->size);
    #endif
    return FAPI_OK;
}
/*lint -restore */

static int32_t pvrReadChunkFromFile (pvrPlayHandleT* inst, void* dstDataPtr)
{
    int32_t error_code = FAPI_OK;
    void*   dataPtr;
    int32_t ret;

    if ( inst->decryptBuffer != NULL )
    {
        dataPtr = inst->decryptBuffer;
    }
    else
    {
        dataPtr = dstDataPtr;
    }

    if ( (ret = fread (dataPtr, 1, PVR_CHUNK_SIZE, inst->filePtr)) != PVR_CHUNK_SIZE )
    {
        FAPI_SYS_PRINT_MSG("[READ] pvrReadChunkFromFile: return erro [%d]\n", ret);
        error_code = SUBSYS_PVR_ERR_FILE_READ;
    }
    else if ( inst->decryptBuffer != NULL )
    {
        error_code = pvrPerformCryptoOperation (dstDataPtr, inst->decryptBuffer,
                                                &(inst->decryptKeys), 1);
    }

    return error_code;
}

/*---------------------------------------------------------------------------*/

static uint8_t pvrIsEndOfData(pvrPlayHandleT* inst)
{
    if (((inst->offset == 0)&&(inst->position >= inst->size))        /* normal file */
      ||((inst->offset >  0)&&(inst->position == (inst->offset-1)))) /* circular */
    {
        FAPI_SYS_PRINT_DEBUG(2, "[READ] pvrIsEndOfData: offset(0x%08x) size(0x%08x) pos(0x%08x)\n",\
                                inst->offset, inst->size, inst->position);
        inst->header.time = inst->fileInfo.duration;
        return 1;
    }
    else
    {
        return 0;
    }
}

static uint8_t pvrIsBeginOfData(pvrPlayHandleT* inst)
{
    return (inst->position == inst->offset);
}

/*---------------------------------------------------------------------------*/

/* chunks = physical position */
static void pvrSeekChunksSet(pvrPlayHandleT* inst, uint32_t chunks)
{
    uint16_t split;
    uint32_t chunks_left;
    F_FILE* file;

    if ( chunks > inst->size ) chunks = inst->size;
    chunks_left = chunks;

    FAPI_SYS_PRINT_DEBUG(1, "[SEEK] pvrSeekChunksSet: position[%d], offset[%d], size[%d], chunks[%d]\n",\
                inst->position,inst->offset, inst->size, chunks);
    /* support file splitting */
    if ( (inst->splitSize      > 0)
      && (inst->numberOfSplits > 1) )
    {
        split = 0;
        while ( chunks_left >= inst->splitSize )
        {
            chunks_left -= inst->splitSize;
            split++;
        }

        if ( split >= inst->numberOfSplits )
        {
            split = inst->numberOfSplits - 1;
        }

        if ( split != inst->splitNumber )
        {
            pvrGetFilename(inst->filePath, inst->openParams.fileName, split);

            file = fopen(inst->filePath, "r");

            if ( file == NULL )
            {
                return;
            }

            (void)f_close(inst->filePtr);

            inst->filePtr     = file;
            inst->splitNumber = split;
        }
    }

    /* set position absolute */
    if ( chunks_left <= PVR_PLAY_SEEK_MAX_CHUNKS )
    {
        (void)fseek(inst->filePtr, (int32_t)(chunks_left * PVR_CHUNK_SIZE),
                     SEEK_SET);
        chunks_left = 0;
    }
    else
    {
        (void)fseek(inst->filePtr, (PVR_PLAY_SEEK_MAX_CHUNKS * PVR_CHUNK_SIZE),
                                     SEEK_SET);
        chunks_left -= PVR_PLAY_SEEK_MAX_CHUNKS;
    }

    /* seek relative */
    while ( chunks_left > 0 )
    {
        if(chunks_left <= PVR_PLAY_SEEK_MAX_CHUNKS)
        {
            (void)fseek(inst->filePtr, (int32_t)(chunks_left * PVR_CHUNK_SIZE),
                         SEEK_CUR);
            chunks_left = 0;
        }
        else
        {
            (void)fseek(inst->filePtr, (PVR_PLAY_SEEK_MAX_CHUNKS *
                                         PVR_CHUNK_SIZE), SEEK_CUR);
            chunks_left -= PVR_PLAY_SEEK_MAX_CHUNKS;
        }
    }

    inst->position = chunks;
    FAPI_SYS_PRINT_DEBUG(1, "[SEEK] pvrSeekChunksSet: inst->position[%d]\n", inst->position);
}

static void pvrSeekChunksCur(pvrPlayHandleT* inst, int32_t chunks)
{
    int32_t new_pos;

    if ( chunks == 0 ) return;

    new_pos = (int32_t)inst->position + chunks;
    FAPI_SYS_PRINT_DEBUG(1, "[READ] pvrSeekChunksCur: In: position[%d], offset[%d], size[%d], chunks[%d]\n",\
                inst->position,inst->offset, inst->size, chunks);
    if ( chunks > 0 ) /* forward? */
    {
        if ( (inst->offset   > 0) /* circular file? */
          && (inst->position <  inst->offset)
          && (new_pos        >= (int32_t)inst->offset) )
        {
            pvrSeekChunksSet(inst, inst->offset - 1); /* logical EOF */
        }
        else if ( new_pos >= (int32_t)(inst->size) )
        {
            if(inst->offset == 0) /* normal file? */
            {
                pvrSeekChunksSet(inst, inst->size); /* EOF */
            }
            else /* circular file */
            {
                while ( new_pos >= (int32_t)(inst->size) )
                {
                    new_pos -= (int32_t)inst->size;
                }

                pvrSeekChunksSet(inst, (uint32_t)new_pos);
            }
        }
        else
        {
            if ( (inst->splitSize > 0)
              && (inst->numberOfSplits > 1)
              && (((uint32_t)new_pos / inst->splitSize) != inst->splitNumber) )
            {
                pvrSeekChunksSet(inst, (uint32_t)new_pos);
            }
            else
            {
                while ( chunks > 0 )
                {
                    if ( chunks <= (int32_t)PVR_PLAY_SEEK_MAX_CHUNKS )
                    {
                        (void)fseek(inst->filePtr, (chunks * PVR_CHUNK_SIZE),
                                     SEEK_CUR);

                        inst->position += (uint32_t)chunks;
                        chunks          = 0;
                    }
                    else
                    {
                        (void)fseek(inst->filePtr,
                                     (PVR_PLAY_SEEK_MAX_CHUNKS * PVR_CHUNK_SIZE),
                                     SEEK_CUR);

                        inst->position +=          PVR_PLAY_SEEK_MAX_CHUNKS;
                        chunks         -= (int32_t)PVR_PLAY_SEEK_MAX_CHUNKS;
                    }
                }
            }
        }
    }
    else /* backward? -> 'chunks' is negative! */
    {
        if ( (inst->offset > 0) /* circular file? */
           &&(inst->position >= inst->offset)
           &&(new_pos <= (int32_t)inst->offset) )
        {
            pvrSeekChunksSet(inst, inst->offset); /* logical BOF */
        }
        else if(new_pos < (int32_t)0)
        {
            if(inst->offset == 0) /* normal file? */
            {
                pvrSeekChunksSet(inst, 0); /* BOF */
            }
            else /* circular file */
            {
                while (new_pos < (int32_t)0)
                    new_pos += (int32_t)inst->size;
                pvrSeekChunksSet(inst, (uint32_t)new_pos);
            }
        }
        else
        {
            if ( (inst->splitSize > 0)
              && (inst->numberOfSplits > 1)
              && (((uint32_t)new_pos / inst->splitSize) != inst->splitNumber) )
            {
                pvrSeekChunksSet(inst, (uint32_t)new_pos);
            }
            else
            {
                while ( chunks < 0 )
                {
                    if ( -chunks <= (int32_t)PVR_PLAY_SEEK_MAX_CHUNKS )
                    {
                        (void)fseek (inst->filePtr, (chunks * PVR_CHUNK_SIZE),
                                      SEEK_CUR);

                        /*lint -save -e737 */
                        inst->position += chunks;
                        /*lint -restore */
                        chunks          = 0;
                    }
                    else
                    {
                        (void)fseek (inst->filePtr,
                               (-((int32_t)
                                  (PVR_PLAY_SEEK_MAX_CHUNKS * PVR_CHUNK_SIZE))),
                               SEEK_CUR);

                        inst->position -=          PVR_PLAY_SEEK_MAX_CHUNKS;
                        chunks         += (int32_t)PVR_PLAY_SEEK_MAX_CHUNKS;
                    }
                }
            }
        }
    }
    FAPI_SYS_PRINT_DEBUG(1, "[SEEK] pvrSeekChunksCur: Out: "
                            "inst->position[%d]\n", inst->position);
}

#if 0
static void pvrSeekNameForward(pvrPlayHandleT* inst, uint8_t seekName)
{
    // TODO: not implemented yet!
}

static void pvrSeekNameBackward(pvrPlayHandleT* inst, uint8_t seekName)
{
    // TODO: not implemented yet!
}
#endif

/* updates the header and info data from the buffer content */
static int32_t pvrUpdateChunkTime(pvrPlayHandleT* inst, uint32_t time)
{
    /* store duration of last data chunk */
    if ( (!(inst->chunkTimeFirst))
      && (inst->chunkTimeLastStamp > 0)
      && (time > inst->chunkTimeLastStamp) )
    {
        //////////////
        PVR_PLAY_LOCK;
        //////////////

        inst->chunkTime[inst->chunkTimeOffset++] = time
                                                 - inst->chunkTimeLastStamp;

        if ( inst->chunkTimeOffset >= PVR_PLAY_CHUNKTIME_AVERAGE_NUMS )
        {
            inst->chunkTimeOffset = 0;
        }

        if( inst->chunkTimeNums < PVR_PLAY_CHUNKTIME_AVERAGE_NUMS )
        {
            inst->chunkTimeNums++;
        }

        ////////////////
        PVR_PLAY_UNLOCK;
        ////////////////
    }

    inst->chunkTimeLastStamp = time;
    inst->chunkTimeFirst     = 0;

    return FAPI_OK;
}

uint32_t pvrPlayGetChunkDuration(FAPEX_SYS_HandleT handle)
{
    pvrPlayHandleT* inst   = (pvrPlayHandleT*)handle;
    uint32_t        chunk_time = 0;
    uint16_t        count;

    //////////////
    PVR_PLAY_LOCK;
    //////////////

    for ( count = 0; count < inst->chunkTimeNums; count++ )
    {
        /* prevent chunk time from overflowing */
        if( (0xFFFFFFFF - chunk_time) < inst->chunkTime[count] ) break;

        chunk_time += inst->chunkTime[count];
    }

    if ( count > 0 )
    {
        chunk_time /= count;
    }

    ////////////////
    PVR_PLAY_UNLOCK;
    ////////////////

    return chunk_time;
}


static int32_t pvrIFrameChannelOpen (pvrPlayHandleT* inst)
{
#ifndef _LINUX_FAPI
    FAPI_BM_OpenParamsT     bm_open_params;
    FAPI_BM_InfoT           bm_info;
    FAPI_TSD_OpenParamsT    tsd_open_params;
    int32_t                 error_code = FAPI_OK;

    if ( (inst->iframeBmHandle != NULL) || (inst->iframeTsdHandle != NULL) )
    {
        return FAPI_OK;
    }

    do
    {
        memset(&bm_open_params, 0, sizeof(bm_open_params));

        bm_open_params.version    = FAPI_BM_VERSION;
        bm_open_params.blockIndex = inst->openParams.tsdBlockIndex;
        bm_open_params.channelId  = FAPI_BM_CHANNEL_AUTODETECT;
        bm_open_params.address    = inst->openParams.iFrameBufferAddress
                                  - FAPI_BM_GetGlobalOffset(
                                    inst->openParams.tsdBlockIndex);
        bm_open_params.size       = inst->openParams.iFrameBufferSize / 2;

        #ifdef PVR_PLAY_IFRAME_BIGENDIAN

        bm_open_params.swapWrite  = FAPI_BM_NO_SWAP;

        #else

        bm_open_params.swapWrite  = FAPI_BM_SWAP;

        #endif /* PVR_PLAY_IFRAME_BIGENDIAN */

        bm_open_params.swapRead   = FAPI_BM_NO_SWAP;
        bm_open_params.swapMode   = FAPI_BM_SWAP_4_BYTES;

        inst->iframeBmHandle = FAPI_BM_Open(&bm_open_params, &error_code);
        if ( inst->iframeBmHandle == NULL )
        {
            //FAPI_SYS_PRINT_MSG("### FAPI_BM_Open(iframebuf) failed!\n");
            break;
        }

        memset(&tsd_open_params, 0, sizeof(tsd_open_params));

        tsd_open_params.version    = FAPI_TSD_VERSION;
        tsd_open_params.blockIndex = inst->openParams.tsdBlockIndex;
        tsd_open_params.channelId  = FAPI_TSD_CHANNEL_AUTODETECT;

        inst->iframeTsdHandle = FAPI_TSD_Open(&tsd_open_params, &error_code);
        if ( inst->iframeTsdHandle == NULL )
        {
            //FAPI_SYS_PRINT_MSG("### FAPI_TSD_Open(iframebuf) failed!\n");
            break;
        }

        /* NOTE: special handling of error code necessary here (see doc) */
        error_code = FAPI_TSD_SetPesParser(inst->iframeTsdHandle, 1, 1, 1);
        if ( error_code < 0 )
        {
            break;
        }

        error_code = FAPI_BM_GetInfo(inst->iframeBmHandle, &bm_info);
        if ( error_code != FAPI_OK )
        {
            break;
        }

        error_code = FAPI_TSD_SetBuffer(inst->iframeTsdHandle, bm_info);
        if ( error_code != FAPI_OK )
        {
            break;
        }

        inst->iframeBufferActive = 0;
    }
    while ( 0 );

    return error_code;
#else

    #ifndef VIDEC_IFRAME
    FAPEX_TSD_OpenSubIdParamStrT  pidChParam;
    //FAPEX_TSD_PesParamStrT        pesFilter;
    FAPEX_TSD_PidParamStrT        pidFilter;
    int32_t                       result;

    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrIFrameChannelOpen: In: ch_no[%d], blockIndex[%d]\n",\
                            inst->openParams.iFrameTsdChannelNo,\
                            inst->openParams.tsdBlockIndex);
    do
    {
        if (inst->iframeTsdFd > 0)
        {
            FAPI_SYS_PRINT_MSG("[PLAY] pvrIFrameChannelOpen: iframe tsd channel already opened\n");
            return FAPI_OK;
        }

        pidChParam.channelNo = inst->openParams.iFrameTsdChannelNo;
        pidChParam.sub_id = FAPEXDEV_TSD0_SUB_MINOR_FIRST + \
                     inst->openParams.tsdBlockIndex * 32 + pidChParam.channelNo;

        result = FAPEX_TSD_Ioctl(inst->openParams.tsdBlockFd, \
                                         FAPEX_IOCCOM_TSD_OPEN_SUBID,\
                                                   (FAPEXIOCARG3_T)&pidChParam);
        if (result < 0)
        {
            FAPI_SYS_PRINT_MSG("[PLAY] pvrIFrameChannelOpen: open tsd channel failed[%d]\n",\
                                                                        result);
            return result;
        }
        inst->iframeTsdFd = pidChParam.sub_id;
        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrIFrameChannelOpen: open tsd channel success[%d]\n",\
                                                                        inst->iframeTsdFd);

        FAPEX_TSD_Ioctl(inst->iframeTsdFd, FAPEX_IOCCOM_TSD_SET_PES_PARSER, (FAPEXIOCARG3_T)NULL);

        #if 0
        pesFilter.bufferSize = inst->openParams.iFrameBufferSize / 2;
        /* The pid could be changed in pvrIFrameChannelStart */
        pesFilter.pid = inst->stream[PVR_DECTYPE_VIDEO].pid;
        pesFilter.bufferThresholdH = pesFilter.bufferSize;
        pesFilter.bufferThresholdL = 0;
        pesFilter.flags = FAPEX_TSD_FLTFLAGS_SWAPRD_NO_SWAP |
                          FAPEX_TSD_FLTFLAGS_PTS |
                          FAPEX_TSD_FLTFLAGS_SWAPMODE_4BYTE |
                          FAPEX_TSD_FLTFLAGS_RM_PES_HEADER;
        #ifdef PVR_PLAY_IFRAME_BIGENDIAN
        pesFilter.flags |= FAPEX_TSD_FLTFLAGS_SWAPWR_NO_SWAP;
        #else
        pesFilter.flags |= FAPEX_TSD_FLTFLAGS_SWAPWR_SWAP;
        #endif

        pesFilter.streamType = FAPEX_TSD_STREAMTYPE_VIDEO;

        result = FAPEX_TSD_Ioctl(inst->iframeTsdFd, \
                                        FAPEX_IOCCOM_TSD_SET_PES_FILTER,\
                                                    (FAPEXIOCARG3_T)&pesFilter);
        if (result < 0)
        {
            FAPI_SYS_PRINT_MSG("[PLAY] pvrIFrameChannelOpen: set pes filter failed[%d]\n",\
                                                                        result);
            return result;
        }
        #else
        pidFilter.bufferSize = inst->openParams.iFrameBufferSize / 2;
        /* The pid could be changed in pvrIFrameChannelStart */
        pidFilter.pid = inst->stream[PVR_DECTYPE_VIDEO].pid;

        pidFilter.pid = 0x1fff;
        pidFilter.bufferThresholdH = pidFilter.bufferSize;
        pidFilter.bufferThresholdL = 0;
        pidFilter.flags = FAPEX_TSD_FLTFLAGS_SWAPRD_NO_SWAP |
                          FAPEX_TSD_FLTFLAGS_SWAPMODE_4BYTE |
                          FAPEX_TSD_FLTFLAGS_RM_TS_HEADER;
        #ifdef PVR_PLAY_IFRAME_BIGENDIAN
        pidFilter.flags |= FAPEX_TSD_FLTFLAGS_SWAPWR_NO_SWAP;
        #else
        pidFilter.flags |= FAPEX_TSD_FLTFLAGS_SWAPWR_SWAP;
        #endif

        result = FAPEX_TSD_Ioctl(inst->iframeTsdFd, \
                                        FAPEX_IOCCOM_TSD_SET_PID_FILTER,\
                                                    (FAPEXIOCARG3_T)&pidFilter);
        if (result < 0)
        {
            FAPI_SYS_PRINT_MSG("[PLAY] pvrIFrameChannelOpen: set pid filter failed[%d]\n",\
                                                                        result);
            FAPEX_TSD_Ioctl(inst->iframeTsdFd, \
                                        FAPEX_IOCCOM_CLOSE_SUBID,\
                                                    (FAPEXIOCARG3_T)NULL);
            inst->iframeTsdFd = 0;
            return result;
        }
        #endif
        inst->iframeBufferMMapBaseAddress = (uint32_t)FAPEX_TSD_Mmap(NULL, \
                                            inst->openParams.iFrameBufferSize/2, \
                                            (PROT_READ | PROT_WRITE), MAP_SHARED, \
                                            inst->iframeTsdFd, 0);
        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrIFrameChannelOpen: iframeFd[%d], mmap address[0x%08x]\n",\
                            inst->iframeTsdFd,
                            inst->iframeBufferMMapBaseAddress);
        inst->iframeBufferActive = 0;




    }
    while (0);

    #else
    int32_t         videc_fd;
    uint32_t        videc_buffer_size;

    videc_fd = pvrDecoderGetTsdFd(inst,PVR_DECTYPE_VIDEO);

    videc_buffer_size = pvrDecoderGetBufSize(inst, PVR_DECTYPE_VIDEO);

    inst->iframeBufferMMapBaseAddress = (uint32_t)FAPEX_TSD_Mmap(NULL, \
                                            videc_buffer_size, \
                                            (PROT_READ | PROT_WRITE), MAP_SHARED, \
                                            videc_fd, 0);
    #endif
    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrIFrameChannelOpen: Out[%d]\n", result);
    return FAPI_OK;
#endif
}


static void pvrIFrameChannelClose (pvrPlayHandleT* inst)
{
#ifndef _LINUX_FAPI
    if ( inst->iframeTsdHandle != NULL )
    {
        (void)FAPI_TSD_Close(inst->iframeTsdHandle);

        inst->iframeTsdHandle = NULL;
    }

    if ( inst->iframeBmHandle != NULL )
    {
        (void)FAPI_BM_Close(inst->iframeBmHandle);

        inst->iframeBmHandle = NULL;
    }
#else

    #ifndef VIDEC_IFRAME
    int32_t result;
    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrIFrameChannelClose: In\n");
    if (inst->iframeTsdFd > 0)
    {
        if (inst->iframeBufferActive != 0)
        {
            pvrIFrameChannelStop(inst);
            inst->iframeBufferActive  = 0;
        }

        FAPEX_TSD_Munmap((uint8_t*)inst->iframeBufferMMapBaseAddress, \
                                             inst->openParams.iFrameBufferSize / 2);
        inst->iframeBufferMMapBaseAddress = 0;

        result = FAPEX_TSD_Ioctl(inst->iframeTsdFd, FAPEX_IOCCOM_CLOSE_SUBID,\
                                                          (FAPEXIOCARG3_T)NULL);
        if (result < 0)
        {
            FAPI_SYS_PRINT_MSG("[PLAY] pvrIFrameChannelClose: failed[%d]\n",\
                                                                        result);
        }

        inst->iframeTsdFd = -1;

    }


    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrIFrameChannelClose: Out[%d]\n", result);

    #else
    uint32_t        videc_buffer_size;
    videc_buffer_size = pvrDecoderGetBufSize(inst, PVR_DECTYPE_VIDEO);

    FAPEX_TSD_Munmap((uint8_t*)inst->iframeBufferMMapBaseAddress, \
                                             videc_buffer_size);

    inst->iframeBufferMMapBaseAddress = 0;
    #endif
#endif
}


static void pvrIFrameChannelStart (pvrPlayHandleT* inst)
{
#ifndef _LINUX_FAPI
    if ( (!(inst->iframeBufferActive))
      && pvrTrackIsActive(inst, PVR_DECTYPE_VIDEO) )
    {
        /* start video PES parsing for I-frame search */
        if ( (inst->iframeBmHandle  != NULL)
          && (inst->iframeTsdHandle != NULL) )
        {
            (void)FAPI_BM_SetBufferWritePointer (inst->iframeBmHandle, 0, 0);
            (void)FAPI_BM_SetBufferReadPointer  (inst->iframeBmHandle, 0, 0);

            (void)FAPI_TSD_SetChannelPid(inst->iframeTsdHandle,
                                         inst->stream[PVR_DECTYPE_VIDEO].pid,
                                         0, 1);
            (void)FAPI_TSD_StartChannel (inst->iframeTsdHandle);

            inst->iFrameStamp        = 0;
            inst->iframeBufferActive = 1;
        }
    }
#else

    int32_t result = FAPI_OK;
    FAPEX_TSD_PidParamStrT pidParam;

    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrIFrameChannelStart: In\n");

    if ( !(inst->iframeBufferActive) && \
                                      pvrTrackIsActive(inst, PVR_DECTYPE_VIDEO))
    {
        /* start video PES parsing for I-frame search */
        if ( inst->iframeTsdFd > 0)
        {
            result = FAPEX_TSD_Ioctl(inst->iframeTsdFd, \
                        FAPEX_IOCCOM_TSD_RESET_BUFFER, (FAPEXIOCARG3_T)NULL);
            if (result < 0)
            {
                FAPI_SYS_PRINT_MSG("[PLAY] pvrIFrameChannelStart: reset buffer failed[%d]\n",\
                                                                        result);
                return;
            }
            memset(&pidParam, 0, sizeof(FAPEX_TSD_PidParamStrT));
            pidParam.pid = inst->stream[PVR_DECTYPE_VIDEO].pid;
            result = FAPEX_TSD_Ioctl(inst->iframeTsdFd, \
                     FAPEX_IOCCOM_TSD_SET_PID_VALUE, (FAPEXIOCARG3_T)&pidParam);
            if (result < 0)
            {
                FAPI_SYS_PRINT_MSG("[PLAY] pvrIFrameChannelStart: set pid failed[%d]\n",\
                                                                        result);
            }

            result = FAPEX_TSD_Ioctl(inst->iframeTsdFd,\
                           FAPEX_IOCCOM_START, (FAPEXIOCARG3_T)NULL);
            FAPI_SYS_PRINT_MSG("[PLAY] pvrIFrameChannelStart: start[%d]\n",\
                                                                        result);
            inst->iFrameStamp        = 0;
            inst->iframeBufferActive = 1;
        }
    }
	FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrIFrameChannelStart: IFrame Tmp Buffer[0x%08x]\n", (uint32_t)inst->iframeTmpBuffer);
    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrIFrameChannelStart: Out [%d]\n", result);
#endif
}



static void pvrIFrameChannelStop (pvrPlayHandleT* inst)
{
#ifndef _LINUX_FAPI
    if ( inst->iframeBufferActive )
    {
        if ( inst->iframeTsdHandle != NULL )
        {
            /* stop video PES parsing for I-frame search */
            (void)FAPI_TSD_StopChannel(inst->iframeTsdHandle);
        }

        inst->iframeBufferActive = 0;
    }
#else

    int32_t result = FAPI_OK;

    FAPI_SYS_PRINT_DEBUG(0, "[PLAY] pvrIFrameChannelStop: In\n");
    if (inst->iframeBufferActive)
    {
        if (inst->iframeTsdFd > 0)
        {
            result = FAPEX_TSD_Ioctl(inst->iframeTsdFd, FAPEX_IOCCOM_STOP,\
                                                         (FAPEXIOCARG3_T)NULL);
            if (result < 0)
            {
                FAPI_SYS_PRINT_MSG("[PLAY] pvrIFrameChannelStop: failed[%d]\n",\
                                                                        result);
            }

        }
        inst->iframeBufferActive = 0;
    }
    FAPI_SYS_PRINT_DEBUG(0, "[PLAY] pvrIFrameChannelStop: Out [%d]\n", result);
#endif
}

static int32_t pvrPlayMPEGIFrame(pvrPlayHandleT* inst,
                                 PVR_PlaySeekModeT seekMode,
                                 uint32_t displayTime)
{
    uint8_t     state1;
    uint8_t     state2;
    uint32_t    chunksFed;
    uint32_t    searchStartPos;
    uint32_t    iFrameStartPos = 0;
    uint8_t     searchForwards;
    uint64_t    ts;
    uint32_t    sleepTime;
    uint8_t*    iFrameStart;    //!< Absolute memory pointer to I-Frame start.
    uint8_t*    iFrameEnd;      //!< Absolute memory pointer to I-Frame end.
    uint32_t    iFrameSize;     //!< I-Frame size in bytes.
    uint32_t    iFramePos;
    uint8_t*    bufStartPtr;    //!< Absolute memory pointer to buffer start.
    uint8_t*    bufEndPtr;      //!< absolute memory pointer to buffer end.
    uint8_t*    bufRp;          //!< Absolute memory pointer to curr. read pos.
    int32_t     res;
    uint64_t    currTimeStamp;
    uint8_t     endReached;

    FAPEX_TSD_ReadInfoStrT read_info;

    /*lint -save -e946 */

    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayMPEGIFrame: in: seekMode[%d]\n", seekMode);
    #ifndef _LINUX_FAPI
    if ( (inst->iframeTsdHandle == NULL)
      || (inst->iframeBmHandle  == NULL)
      || (!(inst->iframeBufferActive)) )
    {
        return FAPI_OK;
    }
    #else
        #ifndef VIDEC_IFRAME
        if ( (inst->iframeTsdFd <= 0)
          ||  (!(inst->iframeBufferActive)))
        {
            FAPI_SYS_PRINT_MSG("[PLAY] pvrPlayMPEGIFrame: Out: invalid iframe tsd[%d] or inactive iframe buffer[%d]\n",\
                               inst->iframeTsdFd, inst->iframeBufferActive);
            return FAPI_OK;
        }
        #else

        #endif
    #endif

    /* I-Frame preview disabled */
    if ( !(inst->enableFrameView) )
    {
        return pvrReadHeader(inst, &(inst->header));
    }

    /* --------------------------------------------------------------------- */

    /* no video -> just get header info and sleep */
    if ( !(pvrTrackIsActive(inst, PVR_DECTYPE_VIDEO)) )
    {
        res = pvrReadHeader(inst, &(inst->header));

        if ( res == FAPI_OK )
        {
            // FIXME: how to port this?

            #if 0

            switch ( inst->speed )
            {
                case 1: case -1: sleepTime = 1200; break;
                case 2: case -2: sleepTime = 600; break;
                case 3: case -3: sleepTime = 300; break;
                case 4: case -4: sleepTime = 200; break;
                case 5: case -5: sleepTime = 100; break;
                case 6: case -6: sleepTime = 70; break;
                default:         sleepTime = 30; break;
            }

            #else

            sleepTime = 100;

            #endif

            FAPI_SYS_SLEEP(sleepTime);
        }

        return res;
    }

    /* --------------------------------------------------------------------- */

    ts = pvrGetTimeStamp();

    /* with video: */
    res            = FAPI_OK;
    searchForwards = 1;
    chunksFed      = 0;
    iFrameStart    = NULL;
    iFrameEnd      = NULL;
    searchStartPos = inst->position;
    state1         = 0;
    endReached     = 0;

    /*FIXME: Need initialized with 0 ??: Martin:20100701*/
    state2         = 0;

    #ifndef _LINUX_FAPI
    (void)FAPI_BM_SetBufferWritePointer (inst->iframeBmHandle, 0, 0);
    (void)FAPI_BM_SetBufferReadPointer  (inst->iframeBmHandle, 0, 0);

    bufStartPtr = FAPI_BM_GetBufferReadPointer(inst->iframeBmHandle);
    #else

    FAPEX_TSD_Ioctl(inst->iframeTsdFd, FAPEX_IOCCOM_TSD_RESET_BUFFER, \
                            (FAPEXIOCARG3_T)NULL);

    FAPEX_TSD_Ioctl(inst->iframeTsdFd, FAPEX_IOCCOM_TSD_GET_BUFFER_READINFO,\
                                                    (FAPEXIOCARG3_T)&read_info);
    bufStartPtr = (uint8_t*)(inst->iframeBufferMMapBaseAddress + read_info.readOffset);
    #endif
    bufEndPtr   = bufStartPtr + (inst->openParams.iFrameBufferSize / 2);
    bufRp       = bufStartPtr;

    while ( 1 )
    {
        if ( !searchForwards )
        {
            /* Reset state in case of backwards search as data is
                    non-continuous. */
            state1 = 0;

            if ( inst->position < 2 )
            {
                FAPI_SYS_PRINT_MSG("[PLAY] pvrPlayMPEGIFrame: Out: SUBSYS_PVR_ERR_BEGIN_OF_DATA\n");
                return SUBSYS_PVR_ERR_BEGIN_OF_DATA;
            }

            pvrSeekChunksCur(inst, -2);
        }

        if ( (iFrameStart == NULL) && (state1 == 0) )
        {
            #ifndef _LINUX_FAPI
            (void)FAPI_BM_SetBufferWritePointer (inst->iframeBmHandle, 0, 0);
            (void)FAPI_BM_SetBufferReadPointer  (inst->iframeBmHandle, 0, 0);
            #else
            FAPEX_TSD_Ioctl(inst->iframeTsdFd, FAPEX_IOCCOM_TSD_RESET_BUFFER, \
                                                          (FAPEXIOCARG3_T)NULL);
            #endif
            bufRp = bufStartPtr;
        }

        /* NOTE: header is updated after complete I-Frame has been found. */
        res = pvrReadData(inst, NULL);

        /* wait until all data has been fed to TSD (and filtered by I-Frame
                    buffer). Note that also for error code
                     SUBSYS_PVR_ERR_END_OF_DATA data could have been written. */
        FAPI_SYS_PRINT_DEBUG(1,"[PLAY] pvrPlayMPEGIFrame: wait feed:chunkWp[0x%08x] chunkRp[0x%08x]\n",\
               inst->chunkBufferWp, inst->chunkBufferRp);

        while ( inst->chunkBufferWp != inst->chunkBufferRp )
        {
            FAPI_SYS_SLEEP(4);
        }

        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayMPEGIFrame: wait done:chunkWp[0x%08x] chunkRp[0x%08x]\n",\
               inst->chunkBufferWp, inst->chunkBufferRp);
        if ( res == SUBSYS_PVR_ERR_END_OF_DATA )
        {
            if ( !endReached )
            {
                /* start search again in backwards direction */
                iFrameStart    = NULL;
                state1         = 0;
                searchForwards = 0;
                chunksFed      = 0;
                endReached     = 1;
                continue;
            }
            else
            {
            	FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayMPEGIFrame: break for reach end\n");
                break;
            }
        }
        else if ( res != FAPI_OK )
        {
        	FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayMPEGIFrame: break read data error[%d]\n", res);
            break;
        }

        chunksFed++;

        /* abort if video PID changed */
        if ( pvrCheckForParamChange(inst, PVR_DECTYPE_VIDEO) )
        {
        	FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayMPEGIFrame: abort for video PID change\n");
            break;
        }

        if ( iFrameStart == NULL ) /* search for I-frame start */
        {
            iFrameStart = pvrFindIFrameStart(inst, bufStartPtr, bufEndPtr,
                                             &bufRp, &state1);

            if ( iFrameStart != NULL )
            {
                state1         = 0;
                state2         = 0;
                bufRp          = iFrameStart;
                iFrameEnd      = NULL;
                searchForwards = 1;

                if ( inst->position > 0 )
                {
                    iFrameStartPos = inst->position;
                }
                else
                {
                    iFrameStartPos = 0;
                }
            }
        }
        else /* search for I-frame end */
        {
            iFrameEnd = pvrFindIFrameEnd(inst, bufStartPtr, bufEndPtr, &bufRp,
                                         &state1, &state2);

            if ( iFrameEnd != NULL )
            {
                // FIXME: This reads header from next chunk: acceptable?
                (void)pvrReadHeader(inst, &(inst->header));
                FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayMPEGIFrame: break for ending found\n");
                break;
            }
        }

        if ( chunksFed > PVR_PLAY_IFRAME_MPEG_CHUNKFEED_MAX )
        {
            FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayMPEGIFrame: Abort I-frame search, max. chunks fed!\n");
            break;
        }

        if ( (chunksFed % 3) == 0 )
        {
            FAPI_SYS_SLEEP(30);
        }
    }

    if ( seekMode == PLAY_SEEKMODE_BACK )
    {
        /* position read pointer to search start position */
        pvrSeekChunksSet(inst, searchStartPos);
    }
    else if ( seekMode == PLAY_SEEKMODE_IFRAMESTART )
    {
        /* position read pointer to chunk of I-Frame start */

        if ( iFrameStart != NULL )
        {
            pvrSeekChunksSet(inst, iFrameStartPos);
        }
        else
        {
            pvrSeekChunksSet(inst, searchStartPos);
        }
    }

    if ( res != FAPI_OK )
    {
        return res;
    }
    else if ( endReached )
    {
        res = SUBSYS_PVR_ERR_END_OF_DATA;
    }

    /* --------------------------------------------------------------------- */
    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayMPEGIFrame: start=0x%08x, end=0x%08x\n",\
                          (uint32_t)iFrameStart, (uint32_t)iFrameEnd);
    /* copy I-Frame into VBV buffer */
    if ( (iFrameStart != NULL) && (iFrameEnd != NULL) )
    {
        /* delay picture presentation if necessary */
        if ( (inst->iFrameStamp > 0)
          && (displayTime       > PVR_PLAY_IFRAME_SLEEPTIME) )
        {
            currTimeStamp = pvrGetTimeStamp();

            if ( (currTimeStamp >= inst->iFrameStamp)
              && (currTimeStamp < (inst->iFrameStamp + displayTime)) )
            {
                sleepTime = displayTime -
                            (uint32_t)(currTimeStamp - inst->iFrameStamp);

                FAPI_SYS_PRINT_MSG(">> delay: %i ms\n", sleepTime);

                FAPI_SYS_SLEEP(sleepTime);
            }
        }

        inst->iFrameStamp = pvrGetTimeStamp();
        FAPI_SYS_PRINT_DEBUG(2, "[PLAY] pvrPlayMPEGIFrame: iFrameStamp[%lld]\n",\
                                inst->iFrameStamp);
        /* ----- pass I-Frame data to decoder ------------------------------ */

        /*lint -save -e923 */
        if ( (((uint32_t)iFrameStart) & 3) == 0 ) /* start is 32-bit-aligned */
        /*lint -restore */
        {
            if ( iFrameEnd >= iFrameStart )
            {
                iFrameSize = (((uint32_t)(iFrameEnd - iFrameStart)) + 3)
                             & 0xFFFFFFFC;
                FAPI_SYS_PRINT_DEBUG(2, "[PLAY] pvrPlayMPEGIFrame: 32-bit-align(Yes) Wrap(No ) size[%d]\n",\
                                iFrameSize);
            }
            else /* buffer wraparound */
            {
                iFrameSize = (((uint32_t)(iFrameEnd - bufStartPtr)) + 3)
                             & 0xFFFFFFFC;
                FAPI_SYS_PRINT_DEBUG(2, "[PLAY] pvrPlayMPEGIFrame: 32-bit-align(Yes) Wrap(Yes) size[%d]\n",\
                                iFrameSize);
                /* recombine I-frame by appending data at buffer end */
                //memcpy(bufEndPtr, bufStartPtr, iFrameSize);
                // FIXME: add size check to ensure memory integrity!

                iFrameSize += (uint32_t)(bufEndPtr - iFrameStart);
            }
        }
        else /* start is not 32-bit aligned */
        {
            /* copy all data, regardless of wraparound */
            if ( iFrameEnd >= iFrameStart )
            {
                iFrameSize = (((uint32_t)(iFrameEnd - iFrameStart)) + 3)
                             & 0xFFFFFFFC;

                FAPI_SYS_PRINT_DEBUG(2, "[PLAY] pvrPlayMPEGIFrame: 32-bit-align(No ) Wrap(No ) size[%d]\n",\
                                iFrameSize);

                #ifdef PVR_PLAY_IFRAME_BIGENDIAN

                    pvrCopyBigEndian(bufEndPtr, iFrameStart, iFrameSize);

                #else

                    //memcpy(bufEndPtr, iFrameStart, iFrameSize);

                #endif /* PVR_PLAY_IFRAME_BIGENDIAN */
            }
            else /* buffer wraparound */
            {
                iFrameSize = (uint32_t)(bufEndPtr - iFrameStart);

                FAPI_SYS_PRINT_DEBUG(2, "[PLAY] pvrPlayMPEGIFrame: 32-bit-align(No ) Wrap(Yes) size[%d]\n",\
                                iFrameSize);

                #ifdef PVR_PLAY_IFRAME_BIGENDIAN

                    pvrCopyBigEndian(bufEndPtr, iFrameStart, iFrameSize);

                #else

                   // memcpy(bufEndPtr, iFrameStart, iFrameSize);

                #endif /* PVR_PLAY_IFRAME_BIGENDIAN */

                iFramePos = iFrameSize;

                iFrameSize += (uint32_t)(iFrameEnd - bufStartPtr);

                #ifdef PVR_PLAY_IFRAME_BIGENDIAN

                    pvrCopyBigEndian(bufEndPtr + iFramePos, bufStartPtr,
                                     iFrameSize - iFramePos);

                #else

                    //memcpy(bufEndPtr + iFramePos, bufStartPtr,
                    //       iFrameSize - iFramePos);

                #endif /* PVR_PLAY_IFRAME_BIGENDIAN */

                iFrameSize = (iFrameSize + 3) & 0xFFFFFFFC;
            }

            //iFrameStart = bufEndPtr;
        }
        FAPI_SYS_PRINT_DEBUG(2, "[PLAY] pvrPlayMPEGIFrame: feed decoder, start[0x%08x],size[%d]\n",\
                                (uint32_t)iFrameStart, iFrameSize);
        /* feed I-Frame to target decoder */
        FAPI_SYS_PRINT_DEBUG(2, "[PLAY] I Frame: 0x%02x 0x%02x 0x%02x 0x%02x\n",\
                                iFrameStart[0],iFrameStart[1],iFrameStart[2],iFrameStart[3]);

        /* copy I-Frame data to tmp buffer*/
        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] fstart[0x%08x] fend[0x%08x] bstart[0x%08x] bend[0x%08x]\n",\
        							(uint32_t)iFrameStart, (uint32_t)iFrameEnd,\
   	     							(uint32_t)bufStartPtr, (uint32_t)bufEndPtr);
        memset(inst->iframeTmpBuffer, 0, PVR_PLAY_IFRAME_TMP_BUFFER_SIZE);
        if (iFrameEnd >= iFrameStart)
        {
        	memcpy(inst->iframeTmpBuffer, iFrameStart, iFrameSize);
        }
        else
        {
        	uint32_t tempSize1=0;
        	uint32_t tempSize2=0;

        	tempSize1 = (uint32_t)(bufEndPtr - iFrameStart);
        	memcpy(inst->iframeTmpBuffer, iFrameStart, tempSize1);

			tempSize2 = (uint32_t)(iFrameEnd - bufStartPtr);
        	memcpy( (uint8_t*)((uint32_t)inst->iframeTmpBuffer + tempSize1),\
        			bufStartPtr, tempSize2);
        }
        /* call outside I frame show function */
        (void)pvrDecoderFeedData(inst, PVR_DECTYPE_VIDEO, inst->iframeTmpBuffer,
                                 iFrameSize);
        /* ----------------------------------------------------------------- */

        FAPI_SYS_SLEEP(PVR_PLAY_IFRAME_SLEEPTIME);
    }
    else
    {
        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayMPEGIFrame: no I-frame found!\n");
    }

    inst->iFrameTime = (uint32_t)(pvrGetTimeStamp() - ts);

    /*lint -restore */
    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrPlayMPEGIFrame: iFrameTime: %d ms\n", inst->iFrameTime);
    return res;
}

static uint8_t* pvrFindIFrameStart (pvrPlayHandleT* inst,
                                    const uint8_t* bufStartPtr,
                                    const uint8_t* bufEndPtr,
                                    uint8_t** bufRpPtr,
                                    uint8_t* statePtr)
{
    uint8_t     b;
    uint8_t     state;
    uint8_t*    iFrameStart = NULL;
    uint8_t*    bufRp;
    uint8_t*    bufWp;
    FAPEX_TSD_WriteInfoStrT wr_info;
    uint32_t                rd_size = 0;

    /*lint -save -e946 */

    state = *statePtr;
    bufRp = *bufRpPtr;
    #ifndef _LINUX_FAPI
    bufWp = FAPI_BM_GetBufferWritePointer(inst->iframeBmHandle);
    #else

        #ifndef VIDEC_IFRAME
        FAPEX_TSD_Ioctl(inst->iframeTsdFd, FAPEX_IOCCOM_TSD_GET_BUFFER_WRITEINFO, \
                                                          (FAPEXIOCARG3_T)&wr_info);
        bufWp = (uint8_t*)(inst->iframeBufferMMapBaseAddress + wr_info.writeOffset);
        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrFindIFrameStart: bufWp[0x%08x], bufRp[0x%08x], writeOffset[0x%08x]\n",\
                (uint32_t)bufWp, (uint32_t)bufRp, wr_info.writeOffset);

        #else
        int32_t videc_fd;

        videc_fd = pvrDecoderGetTsdFd(inst,PVR_DECTYPE_VIDEO);

        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrFindIFrameStart: get fd[%d]\n", videc_fd);

        if (videc_fd == -1)
        {
            return NULL;
        }

        FAPEX_TSD_Ioctl(videc_fd, FAPEX_IOCCOM_TSD_GET_BUFFER_WRITEINFO, \
                                                          (FAPEXIOCARG3_T)&wr_info);
        bufWp = (uint8_t*)(inst->iframeBufferMMapBaseAddress + wr_info.writeOffset);
        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrFindIFrameStart: bufWp[0x%08x], bufRp[0x%08x], writeOffset[0x%08x]\n",\
                (uint32_t)bufWp, (uint32_t)bufRp, wr_info.writeOffset);
        #endif

    #endif
    while ( bufRp != bufWp )
    {
        #if 0
        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrFindIFrameStart:bufWp[0x%08x], bufRp[0x%08x]\n",\
            (uint32_t)bufWp, (uint32_t)bufRp);
        #endif
        #ifdef PVR_PLAY_IFRAME_BIGENDIAN

        b = *((uint8_t*)(((uint32_t)bufRp) ^ 3));

        #else

        b = *bufRp;

        #endif /* PVR_PLAY_IFRAME_BIGENDIAN */

        bufRp++;
        rd_size++; /* read size count */

        if ( bufRp >= bufEndPtr )
        {
            bufRp = (uint8_t*)bufStartPtr;
        }

        switch ( state )
        {
            case 0:
                if (b == 0x00) state = 1;
                break;

            case 1:
                if (b == 0x00) state = 2;
                else           state = 0;
                break;

            case 2:
                if      (b == 0x01) state = 3;
                else if (b == 0x00) state = 2;
                else                state = 0;
                break;

            case 3:
                /* 0x00 0x00 0x01 found */
                state = 0;

                //FAPI_SYS_PRINT_MSG("FOUND: 0x00 0x00 0x01 0x%02X \n", b);

                if ( b == 0xB3 ) /* sequence_header_code? */
                {
                    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrFindIFrameStart: FOUND: 0x00 0x00 0x01 0x%02X \n", b);

                    iFrameStart = bufRp - 4;

                    if ( iFrameStart < bufStartPtr )
                    {
                        iFrameStart += (bufEndPtr - bufStartPtr);
                    }
                }
                break;

            default:
                state = 0;
                break;
        }

        if ( iFrameStart != NULL ) break;
    }

    #ifndef _LINUX_FAPI
    (void)FAPI_BM_SetBufferReadPointer(inst->iframeBmHandle,
                                      (uint32_t)(bufRp - bufStartPtr), 0);
    #else
        #ifndef VIDEC_IFRAME
        FAPEX_TSD_Ioctl(inst->iframeTsdFd, FAPEX_IOCCOM_TSD_SET_BUFFER_READSIZE, rd_size);
        #else
        FAPEX_TSD_Ioctl(videc_fd, FAPEX_IOCCOM_TSD_SET_BUFFER_READSIZE, rd_size);
        #endif
    #endif
    *bufRpPtr = bufRp;
    *statePtr = state;

    /*lint -restore */

    return iFrameStart;
}

static uint8_t* pvrFindIFrameEnd (pvrPlayHandleT* inst,
                                  const uint8_t* bufStartPtr,
                                  const uint8_t* bufEndPtr,
                                  uint8_t** bufRpPtr,
                                  uint8_t* state1Ptr, uint8_t* state2Ptr)
{
    uint8_t     b;
    uint8_t     state1;
    uint8_t     state2;
    uint8_t*    bufRp;
    uint8_t*    bufWp;
    uint8_t*    iFrameEnd = NULL;

    FAPEX_TSD_WriteInfoStrT wr_info;
    uint32_t                rd_size = 0;

    /*lint -save -e946 */

    state1 = *state1Ptr;
    state2 = *state2Ptr;
    bufRp  = *bufRpPtr;
    #ifndef _LINUX_FAPI
    bufWp  = FAPI_BM_GetBufferWritePointer(inst->iframeBmHandle);
    #else

        #ifndef VIDEC_IFRAME
        FAPEX_TSD_Ioctl(inst->iframeTsdFd, FAPEX_IOCCOM_TSD_GET_BUFFER_WRITEINFO, \
                                                          (FAPEXIOCARG3_T)&wr_info);
        bufWp = (uint8_t*)(inst->iframeBufferMMapBaseAddress + wr_info.writeOffset);
        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrFindIFrameEnd: bufWp[0x%08x], bufRp[0x%08x], writeOffset[0x%08x]\n",\
                (uint32_t)bufWp, (uint32_t)bufRp, wr_info.writeOffset);
        #else
        int32_t videc_fd;

        videc_fd = pvrDecoderGetTsdFd(inst,PVR_DECTYPE_VIDEO);

        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrFindIFrameEnd: get fd[%d]\n", videc_fd);

        if (videc_fd == -1)
        {
            return NULL;
        }

        FAPEX_TSD_Ioctl(videc_fd, FAPEX_IOCCOM_TSD_GET_BUFFER_WRITEINFO, \
                                                          (FAPEXIOCARG3_T)&wr_info);
        bufWp = (uint8_t*)(inst->iframeBufferMMapBaseAddress + wr_info.writeOffset);
        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrFindIFrameEnd: bufWp[0x%08x], bufRp[0x%08x], writeOffset[0x%08x]\n",\
                (uint32_t)bufWp, (uint32_t)bufRp, wr_info.writeOffset);
        #endif
    #endif
    while ( bufRp != bufWp )
    {
        #ifdef PVR_PLAY_IFRAME_BIGENDIAN

        b = *((uint8_t*)(((uint32_t)bufRp) ^ 3));

        #else

        b = *bufRp;

        #endif /* PVR_PLAY_IFRAME_BIGENDIAN */

        bufRp++;
        rd_size++; /*read size count*/

        if ( bufRp >= bufEndPtr )
        {
            bufRp = (uint8_t*)bufStartPtr;
        }

        switch ( state1 )
        {
            case 0:
                if ( b == 0x00 ) state1 = 1;
                break;

            case 1:
                if ( b == 0x00 ) state1 = 2;
                else             state1 = 0;
                break;

            case 2:
                if      ( b == 0x01 ) state1 = 3;
                else if ( b == 0x00 ) state1 = 2;
                else                  state1 = 0;
                break;

            case 3:
                state1 = 0;

                if ( state2 == 0 ) /* begin of I-Frame */
                {
                    if ( b == 0xB3 )
                    {
                        state2 = 2; /* sequence_header_code */
                    }
                }
                //else if (state2 == 1)
                //{
                //    if (b == 0xb8) state2 = 2; /* group_start_code */
                //    else if (b == 0xb2) state2 = 1; /* ??? */
                //    else if (b == 0xb5) state2 = 1; /* extension_start_code */
                //    else if (b > 0xb8) state2 = 1; /* user code */
                //    else state2 = 0;
                //}
                else if ( state2 == 2 )
                {
                    /* first picture = I-Frame */
                    if ( b == 0x00 )
                    {
                        state2 = 3; /* picture_start_code */
                    }
                }
                else if ( state2 == 3 )
                {
                    /* next picture starts -> I-Frame complete */
                    if ( b == 0x00 ) /* picture_start_code */
                    {
                        state2    = 0;
                        iFrameEnd = bufRp - 4;

                        if ( iFrameEnd < bufStartPtr )
                        {
                            iFrameEnd += (bufEndPtr - bufStartPtr);
                        }

                    }
                }
                else
                {
                    state2 = 0; /* bad state -> init */
                }
                break;

            default:
                state1 = 0; /* bad state -> init */
                break;
        }

        if ( iFrameEnd != NULL ) break;
    }

    *state1Ptr = state1;
    *state2Ptr = state2;
    *bufRpPtr  = bufRp;

    #ifndef _LINUX_FAPI
    (void)FAPI_BM_SetBufferReadPointer(inst->iframeBmHandle,
                                      (uint32_t)(bufRp - bufStartPtr), 0);
    #else
        #ifndef VIDEC_IFRAME
        FAPEX_TSD_Ioctl(inst->iframeTsdFd, FAPEX_IOCCOM_TSD_SET_BUFFER_READSIZE, rd_size);
        #else
        FAPEX_TSD_Ioctl(videc_fd, FAPEX_IOCCOM_TSD_SET_BUFFER_READSIZE, rd_size);
        #endif
    #endif
    /*lint -restore */

    return iFrameEnd;
}

#ifdef PVR_PLAY_IFRAME_BIGENDIAN

static void pvrCopyBigEndian(void* dstPtr, const void* srcPtr, int32_t size)
{
    uint32_t    cnt;
    uint32_t    srcAddr = (uint32_t)srcPtr;
    uint32_t    dstAddr = (uint32_t)dstPtr;

    for ( cnt = 0; cnt < size; cnt++ )
    {
        *((uint8_t*)(dstAddr ^ 3)) = *((uint8_t*)(srcAddr ^ 3));

        srcAddr++;
        dstAddr++;
    }
}

#endif /* PVR_PLAY_IFRAME_BIGENDIAN */

/*lint -save -e715 */
static int32_t pvrPlayCtrlFeedIFrame(pvrPlayHandleT* inst,
                                     PVR_PlaySeekModeT seekMode,
                                     uint32_t displayTime)
{
    uint64_t    ts;
    uint32_t    searchStartPos;
    uint32_t    sleepTime;
    int32_t     res = FAPI_OK;
    uint32_t    chunksFed;
    uint8_t     abort;
    uint8_t     endReached;

    //FAPI_SYS_PRINT_MSG("pvrPlayCtrlFeedIFrame! \n");

    /* I-Frame preview disabled */
    if ( !(inst->enableFrameView) )
    {
        res = pvrReadHeader(inst, &(inst->header));
    }

    /* --------------------------------------------------------------------- */

    /* no video -> just get header info and sleep */
    else if ( !(pvrTrackIsActive(inst, PVR_DECTYPE_VIDEO)) )
    {
        res = pvrReadHeader(inst, &(inst->header));

        if ( res == FAPI_OK )
        {
            // FIXME: how to port this?

            #if 0

            switch ( inst->speed )
            {
                case 1: case -1: sleepTime = 1200; break;
                case 2: case -2: sleepTime = 600; break;
                case 3: case -3: sleepTime = 300; break;
                case 4: case -4: sleepTime = 200; break;
                case 5: case -5: sleepTime = 100; break;
                case 6: case -6: sleepTime = 70; break;
                default:         sleepTime = 30; break;
            }

            #else

            sleepTime = 100;

            #endif

            FAPI_SYS_SLEEP(sleepTime);
        }
    }

    /* --------------------------------------------------------------------- */

    else
    {
        ts = pvrGetTimeStamp();

        searchStartPos = inst->position;
        abort          = 0;
        endReached     = 0;

        while ( !abort )
        {
            chunksFed = 0;

            (void)pvrDecoderPrepareData (inst, PVR_DECTYPE_VIDEO);

            while ( 1 )
            {
                if ( (pvrGetTimeStamp() - ts) >= PVR_PLAY_IFRAME_SEARCHTIMEOUT )
                {
                    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] I-Frame search timeout!\n");

                    res = SUBSYS_PVR_ERR_TIMEOUT;
                    abort = 1;
                    break;
                }

                // TODO: adjust sleeptime?
                sleepTime = 0;

                /* NOTE: header is updated on first chunk only */
                res = pvrReadData(inst, (chunksFed == 0) ? (&(inst->header))
                                                         : NULL);

                if ( res == SUBSYS_PVR_ERR_END_OF_DATA )
                {
                    if ( !endReached )
                    {
                        /* seek back in file */

                        // TODO: choose sensible value
                        pvrSeekChunksCur(inst, -50);

                        /* start search again */
                        endReached = 1;
                    }
                    else
                    {
                        abort = 1;
                    }

                    break;
                }
                else if ( res == SUBSYS_PVR_ERR_NO_BUFFER_SPACE )
                {
                    sleepTime = PVR_PLAY_CHUNKBUFFULL_SLEEPTIME;
                }
                else if ( res != FAPI_OK )
                {
                    // TODO: seek back (according to seekmode!)?
                    //pvrSeekChunksSet(inst, inst->position);

                    abort = 1;
                    break;
                }
                else
                {
                    chunksFed++;

                    if ( pvrDecoderIsDataDecoded (inst, PVR_DECTYPE_VIDEO) )
                    {
                        inst->iFrameTime = (uint32_t)(pvrGetTimeStamp() - ts);

                        FAPI_SYS_PRINT_DEBUG(2, "[PLAY] I-Frame after %i chunks, %i ms!\n",
                                                 chunksFed, inst->iFrameTime);

                        abort = 1;
                        break;
                    }

                    if ( chunksFed > PVR_PLAY_CTRLFEED_CHUNKFEED_MAX )
                    {
                        FAPI_SYS_PRINT_DEBUG(1, "[PLAY] Abort I-frame search, max. chunks fed!\n");
                        abort = 1;
                        break;
                    }

                    FAPI_SYS_SLEEP(sleepTime);
                }
            }
        }

        if ( (seekMode == PLAY_SEEKMODE_BACK)
          || (seekMode == PLAY_SEEKMODE_IFRAMESTART) )
        {
            /* NOTE: I-frame start position cannot be determined in this
                     mode -> treat request as PLAY_SEEKMODE_BACK */

            /* position read pointer to search start position */
            pvrSeekChunksSet(inst, searchStartPos);
        }

        if ( (res == FAPI_OK) && endReached )
        {
            res = SUBSYS_PVR_ERR_END_OF_DATA;
        }
    }

    return res;
}
/*lint -restore */

static void pvrPlayNotify (pvrPlayHandleT* inst, PVR_PlayNotifyT reason)
{
    if ( inst->openParams.callback != NULL )
    {
        inst->openParams.callback((FAPEX_SYS_HandleT)inst, reason);
    }
}

#if 0
static void pvrGetNextCutInOutPos(pvrPlayHandleT* inst)
{
    /* markers are not implemented yet -> do nothing */
}

static void pvrGetPreviousCutInOutPos(pvrPlayHandleT* inst)
{
    /* markers are not implemented yet -> do nothing */
}

static void pvrGetNextABInOutPos(pvrPlayHandleT* inst)
{
    /* markers are not implemented yet -> do nothing */
}
#endif

static uint8_t pvrPidIsValid(pvrPlayHandleT* inst, PVR_DecoderTypeT type)
{
    return ((inst->stream[type].pid > 0) && (inst->stream[type].pid < 0x1FFF));
}

/* ------------------------------------------------------------------------- */

static int32_t pvrPlayClearBuffer (pvrPlayHandleT* inst)
{
    return pvrPlayFeedSendCmd (inst, FEED_CMD_DISCARDDATA);
}

/* ------------------------------------------------------------------------- */

static void pvrStartAll (pvrPlayHandleT* inst, PVR_DecoderModeT mode,
                         uint8_t modeIndex)
{
    PVR_DecoderTypeT    currDecoder;
    uint32_t            cnt;

    for ( cnt = 0; cnt < (uint32_t)PVR_DECTYPE_NUMS; cnt++ )
    {
        currDecoder = inst->playDataPtr->decoderStartOrder[cnt];

        pvrTrackStart(inst, currDecoder, mode, modeIndex);
    }
}

static void pvrStopMedia(pvrPlayHandleT* inst)
{
    pvrTrackStop(inst, PVR_DECTYPE_AUDIO,     PLAY_STOPMODE_NONE);
    pvrTrackStop(inst, PVR_DECTYPE_AUDIO_2ND, PLAY_STOPMODE_NONE);
    pvrTrackStop(inst, PVR_DECTYPE_TTX,       PLAY_STOPMODE_NONE);
}

/* ------------------------------------------------------------------------- */

// change to public interface by Martin
void pvrTrackStart (pvrPlayHandleT* inst, PVR_DecoderTypeT type,
                           PVR_DecoderModeT mode, uint8_t modeIndex)
{
    pvrPlayStreamT* streamPtr = inst->stream + (int32_t)type;

    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrTrackStart: In:mode[%d], modeIndex[%d], type[%d]\n",\
                         	 mode, modeIndex, type);

    if ( (streamPtr->trackIndex != 0xFF)
      && (streamPtr->decHandle  != NULL) )
    {
        if ( !(streamPtr->started) )
        {
            streamPtr->pid       = inst->header.trackPid[streamPtr->trackIndex]
                                   & 0x1FFF;
            streamPtr->trackType = inst->header.trackType[streamPtr->
                                                          trackIndex];
            streamPtr->format    = pvrGetTrackFormat(&(inst->header),
                                                     streamPtr->trackIndex);

            if ( pvrPidIsValid(inst, type) )
            {
                pvrTrackResetBuffer     (inst, type);

                (void)pvrDecoderSetMode (inst, type, mode, modeIndex);

                streamPtr->startResult = pvrDecoderStart (inst, type,
                                                          streamPtr->trackIndex,
                                                          streamPtr->trackData);

                streamPtr->started = 1;
            }
        }
        else if ( streamPtr->startResult == FAPI_OK )
        {
            /* track already started, change mode only */
            if ( pvrPidIsValid(inst, type) )
            {
                (void)pvrDecoderSetMode (inst, type, mode, modeIndex);
            }
        }
    }
    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrTrackStart: Out\n");
}

void pvrTrackStartIFrame (pvrPlayHandleT* inst, PVR_DecoderTypeT type,
                           PVR_DecoderModeT mode, uint8_t modeIndex)
{
    pvrPlayStreamT* streamPtr = inst->stream + (int32_t)type;

    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrTrackStart: mode[%d], modeIndex[%d]\n",\
                         mode, modeIndex);

    if ( (streamPtr->trackIndex != 0xFF)
      && (streamPtr->decHandle  != NULL) )
    {
        streamPtr->pid       = inst->header.trackPid[streamPtr->trackIndex]
                               & 0x1FFF;
        streamPtr->trackType = inst->header.trackType[streamPtr->
                                                      trackIndex];
        streamPtr->format    = pvrGetTrackFormat(&(inst->header),
                                                 streamPtr->trackIndex);

        if ( pvrPidIsValid(inst, type) )
        {
            pvrTrackResetBuffer     (inst, type);

            (void)pvrDecoderSetMode (inst, type, mode, modeIndex);

            streamPtr->started = 1;
        }

    }
}



void pvrTrackStop (pvrPlayHandleT* inst, PVR_DecoderTypeT type,
                          PVR_PlayStopMode stopMode)
{
	FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrTrackStop: mode[%d], type[%d]\n",\
                         stopMode, type);
    if ( pvrPidIsValid(inst, type) )
    {
        (void)pvrDecoderStop (inst, type, stopMode);

        pvrTrackResetBuffer  (inst, type);

        inst->stream[type].pid       = 0;
        inst->stream[type].trackType = PVR_TRACK_TYPE_OTHER;
        inst->stream[type].format    = PVR_TRACK_FORMAT_OTHER;
    }

    inst->stream[type].started     = 0;
    inst->stream[type].startResult = FAPI_OK;
}

static void pvrTrackSetMode (pvrPlayHandleT* inst, PVR_DecoderTypeT type,
                             PVR_DecoderModeT mode, uint8_t modeIndex)
{
	FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrTrackStop: mode[%d], type[%d]\n",\
                         mode, type);
    if ( pvrPidIsValid(inst, type) )
    {
        (void)pvrDecoderSetMode (inst, type, mode, modeIndex);
    }
}

static void pvrTrackResetBuffer (pvrPlayHandleT* inst, PVR_DecoderTypeT type)
{
    (void)pvrDecoderResetBuffer (inst, type);

    if ( type == PVR_DECTYPE_VIDEO )
    {
        /* reset filling indicator parameters */
        inst->lastFilling    = 0;
        inst->fillingCounter = 0;
    }
}

static uint8_t pvrTrackGetFormat (pvrPlayHandleT* inst, PVR_DecoderTypeT type)
{
    uint8_t format;

    if ( pvrPidIsValid(inst, type) )
    {
        format = inst->stream[type].format;
    }
    else
    {
        format = PVR_TRACK_FORMAT_OTHER;
    }

    return format;
}

uint8_t pvrTrackIsActive (pvrPlayHandleT* inst, PVR_DecoderTypeT type)
{
    pvrPlayStreamT* streamPtr = inst->stream + (int32_t)type;

    if ( (pvrPidIsValid(inst, type))
      && (streamPtr->started)
      && (streamPtr->startResult == FAPI_OK) )
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

/* ------------------------------------------------------------------------- */

static int32_t pvrDecoderStart (pvrPlayHandleT* inst, PVR_DecoderTypeT type,
                                uint8_t trackIndex, uint32_t trackData)
{
    pvrPlayStreamT* streamPtr = inst->stream + (int32_t)type;
    int32_t         result;

    if ( streamPtr->decHandle == NULL )
    {
        result = FAPI_ERR_NOT_OPEN;
    }
    else if ( inst->openParams.targetControl.decoder[type].start != NULL )
    {
        result =  inst->openParams.targetControl.decoder[type].start(
                        type, trackIndex, trackData, streamPtr->trackType,
                        streamPtr->format,
                        inst->openParams.targetControl.argPtr,
                        streamPtr->pid,
                        streamPtr->decHandle);
    }
    else
    {
        result = FAPI_OK;
    }

    return result;
}

static int32_t pvrDecoderStop (pvrPlayHandleT* inst, PVR_DecoderTypeT type,
                               PVR_PlayStopMode stopMode)
{
    if ( (inst->openParams.targetControl.decoder[type].stop != NULL)
      && (inst->stream[type].decHandle                      != NULL) )
    {
        return inst->openParams.targetControl.decoder[type].stop(
                  type,
                  inst->openParams.targetControl.argPtr,
                  (stopMode == PLAY_STOPMODE_FREEZE) ? 1 : 0,
                  inst->stream[type].decHandle);
    }
    else
    {
        return FAPI_OK;
    }
}

static int32_t pvrDecoderSetMode (pvrPlayHandleT* inst, PVR_DecoderTypeT type,
                                  PVR_DecoderModeT mode, uint8_t modeIndex)
{
    if ( (inst->openParams.targetControl.decoder[type].setMode != NULL)
      && (inst->stream[type].decHandle                         != NULL) )
    {
        return inst->openParams.targetControl.decoder[type].setMode(
                  type,
                  inst->openParams.targetControl.argPtr,
                  mode, modeIndex, inst->stream[type].decHandle);
    }
    else
    {
        return FAPI_OK;
    }
}

static int32_t pvrDecoderFeedData (pvrPlayHandleT* inst, PVR_DecoderTypeT type,
                                   const uint8_t* dataPtr, uint32_t dataBytes)
{
    if ( (inst->openParams.targetControl.decoder[type].feedData != NULL)
      && (inst->stream[type].decHandle                          != NULL) )
    {
        return inst->openParams.targetControl.decoder[type].feedData(
                  type,
                  inst->openParams.targetControl.argPtr,
                  dataPtr, dataBytes,
                  inst->stream[type].decHandle);
    }
    else
    {
        return FAPI_OK;
    }
}

static int32_t pvrDecoderResetBuffer (pvrPlayHandleT* inst,
                                      PVR_DecoderTypeT type)
{
    if ( (inst->openParams.targetControl.decoder[type].resetBuffer != NULL)
      && (inst->stream[type].decHandle                             != NULL) )
    {
        return inst->openParams.targetControl.decoder[type].resetBuffer(
                    type,
                    inst->openParams.targetControl.argPtr,
                    inst->stream[type].decHandle);
    }
    else
    {
        return 0;
    }
}

uint32_t pvrDecoderGetBufLevel (pvrPlayHandleT* inst, PVR_DecoderTypeT type)
{
    if ( (inst->openParams.targetControl.decoder[type].getBufferLevel != NULL)
      && (inst->stream[type].decHandle                                != NULL) )
    {
        return inst->openParams.targetControl.decoder[type].getBufferLevel(
                    type,
                    inst->openParams.targetControl.argPtr,
                    inst->stream[type].decHandle);
    }
    else
    {
        return 0;
    }
}

uint32_t pvrDecoderGetBufSize (pvrPlayHandleT* inst, PVR_DecoderTypeT type)
{
    if ( (inst->openParams.targetControl.decoder[type].getBufferSize != NULL)
      && (inst->stream[type].decHandle                               != NULL) )
    {
        return inst->openParams.targetControl.decoder[type].getBufferSize(
                    type,
                    inst->openParams.targetControl.argPtr,
                    inst->stream[type].decHandle);
    }
    else
    {
        return 0;
    }
}

static uint8_t pvrDecoderIsDataDecoded (pvrPlayHandleT* inst,
                                        PVR_DecoderTypeT type)
{
    if ( (inst->openParams.targetControl.decoder[type].isDataDecoded != NULL)
      && (inst->stream[type].decHandle                               != NULL) )
    {
        return inst->openParams.targetControl.decoder[type].isDataDecoded(
                    type,
                    inst->openParams.targetControl.argPtr,
                    inst->stream[type].decHandle);
    }
    else
    {
        return 0;
    }
}

static int32_t pvrDecoderPrepareData (pvrPlayHandleT* inst,
                                      PVR_DecoderTypeT type)
{
    if ( (inst->openParams.targetControl.decoder[type].prepareData != NULL)
      && (inst->stream[type].decHandle                             != NULL) )
    {
        return inst->openParams.targetControl.decoder[type].prepareData(
                    type,
                    inst->openParams.targetControl.argPtr,
                    inst->stream[type].decHandle);
    }
    else
    {
        return 0;
    }
}

static void pvrDecoderTasks (pvrPlayHandleT* inst)
{
    PVR_DecoderTypeT    decIdx;
    pvrPlayStreamT*     streamPtr;

    for ( decIdx = (PVR_DecoderTypeT)0; decIdx < PVR_DECTYPE_NUMS;
          decIdx++ )
    {
        streamPtr = inst->stream + (int32_t)decIdx;

        if ( (inst->openParams.targetControl.decoder[decIdx].task != NULL)
          && (streamPtr->decHandle                                != NULL) )
        {
            inst->openParams.targetControl.decoder[decIdx].task(decIdx,
                  inst->openParams.targetControl.argPtr,
                  streamPtr->decHandle);
        }
    }
}

static int32_t pvrSyncStart (pvrPlayHandleT* inst)
{
    PVR_DecoderTypeT syncSource = PVR_DECTYPE_VIDEO;

    if ( (inst->openParams.targetControl.avSync != NULL)
      && (!(inst->decSyncActive)) )
    {
        /* determine A/V sync reference stream */
        if (  pvrTrackIsActive(inst, PVR_DECTYPE_VIDEO) )
        {
            syncSource = PVR_DECTYPE_VIDEO;
        }
        else if ( pvrTrackIsActive(inst, PVR_DECTYPE_AUDIO) )
        {
            syncSource = PVR_DECTYPE_AUDIO;
        }
        else if ( pvrTrackIsActive(inst, PVR_DECTYPE_AUDIO_2ND) )
        {
            syncSource = PVR_DECTYPE_AUDIO_2ND;
        }

        inst->decSyncActive = 1;

        return inst->openParams.targetControl.avSync(1, syncSource,
                                                     inst->openParams.
                                                     targetControl.argPtr);
    }
    else
    {
        return FAPI_OK;
    }
}

static int32_t pvrSyncStop (pvrPlayHandleT* inst)
{
    if ( (inst->openParams.targetControl.avSync != NULL)
      && (inst->decSyncActive) )
    {
        inst->decSyncActive = 0;

        /* NOTE: 'source' argument is not used when disabling sync. */
        return inst->openParams.targetControl.avSync(0, PVR_DECTYPE_VIDEO,
                                                     inst->openParams.
                                                     targetControl.argPtr);
    }
    else
    {
        return FAPI_OK;
    }
}

static int32_t pvrDecoderGetTsdFd(pvrPlayHandleT* inst, PVR_DecoderTypeT type)
{
    if ( (inst->openParams.targetControl.decoder[type].getTsdFd!= NULL)
      && (inst->stream[type].decHandle                               != NULL) )
    {
        return inst->openParams.targetControl.decoder[type].getTsdFd(
                    type,
                    inst->openParams.targetControl.argPtr,
                    inst->stream[type].decHandle);
    }
    else
    {
        return -1;
    }
}

#if 0
/*
 * bytePerRead: can't be bigger than 1 chunk length (512 * 188)
 *
 */
static int32_t pvrCacheRead(pvrPlayHandleT *inst, int32_t cacheSize, int32_t bytePerRead)
{
    /* cache read  */
    FILE*       fp = NULL;
    int32_t     flen = 0 ;
    int32_t     i;
    uint8_t*    temp_buf = NULL;
    int32_t     cache_read_length = cacheSize;
    int32_t     error_code = FAPI_OK;
    uint64_t 	t;
    int32_t		loop;

   	t = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
    pvrGetFilename (inst->filePath, inst->openParams.fileName, 0);

	flen = pvrGetFileSize(inst->filePath);

	if (flen == -1)
    {
        error_code = SUBSYS_PVR_ERR_FILE_HANDLING;
        goto end;
	}

    fp = fopen(inst->filePath, "r");
    if ( fp == NULL )
    {
        error_code = SUBSYS_PVR_ERR_FILE_HANDLING;
        goto end;
    }

    temp_buf = (uint8_t*)FAPI_SYS_MALLOC_CACHED(bytePerRead);
    cache_read_length = cache_read_length > flen ? flen : cache_read_length;

	if (bytePerRead <= 0)
	{
		error_code = SUBSYS_PVR_ERR_BAD_PARAMETER;
		goto end;
	}
    loop = cache_read_length / bytePerRead;

    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrCacheRead: cache length[%d], bytePerRead[%d] loop[%d]\n",
    						cache_read_length, bytePerRead, loop);
  	if (cache_read_length  < cacheSize && loop > 1)
  	{
  		loop--;
  	}
    for (i = 0; loop; i++)
    {
        if (bytePerRead != fread(temp_buf, 1, bytePerRead, fp))
        {
            break;
        }
    }

end:
    if (temp_buf) {
        FAPI_SYS_FREE(temp_buf);
        temp_buf = 0;
    }

    if (fp != NULL) {
        fclose(fp);
        fp = NULL;
    }
    FAPI_SYS_PRINT_DEBUG(1, "[PLAY] pvrCacheRead: cacheRead done[%d] using[%lld]\n",\
    						error_code,
    						FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC) - t);
    return error_code;
}
#endif

#if DEBUG>1
static void pvrPlayFastModeConfigDump(PVR_FastModeConfigT fastModeConfig)
{
    printf("\n");
    switch (fastModeConfig.type)
    {
        case PVR_FASTMODETYPE_FEED_CONTROLLED:
            FAPI_SYS_PRINT_DEBUG(1, "[FAST] ControlTask: currFastMode[FEED_CONTROLLED]\n");
            FAPI_SYS_PRINT_DEBUG(1, "type...........[%d]\n",fastModeConfig.type);
            FAPI_SYS_PRINT_DEBUG(1, "factor.........[%d]\n",fastModeConfig.params.feedCtrl.factor);
            FAPI_SYS_PRINT_DEBUG(1, "displayTime....[%d]\n",fastModeConfig.params.feedCtrl.displayTime);
            break;

        case PVR_FASTMODETYPE_IFRAMES:
            FAPI_SYS_PRINT_DEBUG(1, "[FAST] ControlTask: currFastMode[IFRAMES]\n");
            FAPI_SYS_PRINT_DEBUG(1, "type...........[%d]\n",fastModeConfig.type);
            FAPI_SYS_PRINT_DEBUG(1, "factor.........[%d]\n",fastModeConfig.params.iFrames.factor);
            FAPI_SYS_PRINT_DEBUG(1, "displayTime....[%d]\n",fastModeConfig.params.iFrames.displayTime);
            break;

        case PVR_FASTMODETYPE_FEED_NONE:
            FAPI_SYS_PRINT_DEBUG(1, "[FAST] ControlTask: currFastMode[FEED_NONE]\n");
            FAPI_SYS_PRINT_DEBUG(1, "type...........[%d]\n",fastModeConfig.type);
            FAPI_SYS_PRINT_DEBUG(1, "factor.........[%d]\n",fastModeConfig.params.feedNone.factor);
            break;

        case PVR_FASTMODETYPE_FEED_ALL:
            FAPI_SYS_PRINT_DEBUG(1, "[FAST] ControlTask: currFastMode[FEED_ALL]\n");
            FAPI_SYS_PRINT_DEBUG(1, "type...........[%d]\n",fastModeConfig.type);
            FAPI_SYS_PRINT_DEBUG(1, "unused.........[%d]\n",fastModeConfig.params.feedAll.unused);
            break;

        default:
            FAPI_SYS_PRINT_DEBUG(1, "[FAST] ControlTask: currFastMode[UNKNOWN_FASTMODE]\n");
            break;
    }
    printf("\n");
}

#endif

/*lint -restore */
